# Data structure in counter:
# {
#   <item_id>: {
#     time: <unix timestmap>
#     value: <value>
#   }
# }
#
# Returned data structure
# {
#   <item_id>: <value>
# }

package Counter;

use strict;
use warnings;

use Data::Dumper;
use File::Basename;
use JSON;

sub new {
  my($self, $technology, $id) = @_;

  my $o = {};
  $o->{technology} = $technology;

  if (defined $id) {
    $o->{path} = dirname(__FILE__)."/../server-nest/microservices/".$o->{technology}."/counter_".$id.".json";
  } else {
    $o->{path} = dirname(__FILE__)."/../server-nest/microservices/".$o->{technology}."/counter.json";
  }

  unless (-e $o->{path}) {
    open(my $fh, '>', $o->{path}) or die $!;
    print $fh '{}';
    close($fh);  
  }

  bless $o, $self;

  return $o;
}

sub getPath {
  my($self) = @_;

  return $self->{path};
}

sub getData {
  my($self) = @_;

  my $data;
  my $json = '';
  if ( open( my $fh, '<', $self->{path} ) ) {
    while ( my $row = <$fh> ) {
      chomp $row;
      $json .= $row;
    }
    close($fh);
  }
  else {
    warn( localtime() . ": Cannot open the file ".$self->{path}." ($!)" );
  }

  eval { $data = decode_json($json); };
  if ($@) {
    my $error = $@;
    warn( localtime() . ": Cannot decode the file ".$self->{path}." ($error)" );
  }

  return $data;
}

sub saveData {
  my($self, $data) = @_;

  open my $fh, ">", $self->{path};
  print $fh JSON->new->pretty->encode( $data );
  close $fh;

  return 1;
}

sub saveAndReturnAveraged {
  my($self, $data) = @_;

  my %averagedData;
  my $counterData = $self->getData();
  foreach my $id (keys %{$data}) {
    if (defined $counterData->{$id}) {
      if ($counterData->{$id}{value} > $data->{$id}{value}) {
        next;
      }
      $averagedData{$id} = ($data->{$id}{value} - $counterData->{$id}{value}) / (($data->{$id}{time} - $counterData->{$id}{time}) / 60);
    }
  }

  $self->saveData($data);

  return \%averagedData;
}

1;
