package MicroService;

use strict;
use warnings;

use Server;
use Client;
use Xormon;
use Data::Dumper;


# MicroService class to manage the server and client connections

# Constructor to initialize a Microservice object
# Parameters:
#   server - An instance of the Server class
sub new {
    my ($class, %args) = @_;
    my $self = {
        server => $args{server},  # Instance of Server class
    };
    bless $self, $class;
    return $self;
}

# Method to start the microservice, which in turn starts the server
sub start {
    my ($self) = @_;
    
    $self->{server}->start();
    while (1) {         
        # Waiting to client. If not connected do next
        my $client_socket = $self->{server}->accept_client();        
        next unless $client_socket;
     
        my $pid = fork();     

        if (not defined $pid) {
            Xormon::log("Failed to fork: $!", "error");
            next;
        } elsif ($pid == 0) {            
            #instance of client            
            my $client_handler = $self->{server}->{client_handler_factory}->($client_socket);
            $client_handler->process_request();
            exit 0;
        } else {
            $client_socket->autoflush(1);
        }
    }
}

# Method to gracefully stop the microservice
sub stop {
    my ($self) = @_;
    Xormon::log("Stopping microservice...");
    $self->{server}->close();
    Xormon::log("MicroService stopped.");
}

1;
