package Test;

use strict;
use warnings;

use JSON;
use Data::Dumper;
use Time::HiRes qw(usleep nanosleep);

sub new {
  my($self, $xormon) = @_;

  my $o = {};
  $o->{sections} = ();
  if (defined $xormon) {
    $o->{xormon} = $xormon;
  }
  bless $o, $self;

  return $o;
}

sub addSection {
  my($self,$label,$success,$text,$acceptable) = @_;
  if (defined $label && defined $success) {
    my %section = (
      'label' => $label,
      'success' => $success == 1 ? \1 : \0,
      'text' => defined $text ? $text : '',
      'acceptable' => defined $acceptable ? $acceptable : \0,
    );
    
    if (defined $self->{xormon}) {
      if (defined $self->{xormon}->connected) {
        $self->{xormon}->send(encode_json(\%section));
        # Sleep for 10ms because of websocket. Avoid the JSONs to merge together.
        usleep(10000);
      } else {
        print STDERR '[Test] Socket is closed\n';
      }
    } else {
      # deprecated
      push(@{$self->{sections}}, \%section);
    }
    
  }
}

sub stop {
  my ($self) = @_;
  if (defined $self->{xormon}) {
    $self->{xormon}->shutdown(2);
  }
}

# deprecated 
sub genResult() {
  my ($self) = @_;
  return encode_json(\@{$self->{sections}});
}

#### STATIC METHODS ####

# Connection test, 1 - success, 0 - failed
sub connTest {
  my ($host, $port, $protocol) = @_;

  if (!defined $protocol) {
    $protocol = 'tcp'
  }

  my $success = 0;

  eval {
    use IO::Socket::INET;

    my $socket = new IO::Socket::INET(
        PeerHost => $host,
        PeerPort => $port,
        Proto => $protocol,
        Timeout => 5
    );

    if ($socket) {
      $success = 1;
    } else {
      $success = 0;
    }
  };

  if ($@) {
    my $error = $@;
    print STDERR '[Test] Conn test error [$host]: ' . $error;
  }

  return $success;
}

1;