package XormonLogger;

use strict;
use warnings;

use Data::Dumper;
use File::Basename;
no warnings 'utf8';


use Term::ANSIColor qw(:constants);
$Term::ANSIColor::AUTORESET = 1;

sub new {
  my ( $self, $hwtype, $devuid, $alias, $colored, $debug ) = @_;

  my $o = {};
  $o->{hwtype}      = $hwtype;
  $o->{alias}       = $alias;
  $o->{device_uuid} = $devuid;
  $o->{debug}       = $debug;

  my $logdir = dirname( $ENV{pm_err_log_path} );

  if ( !-d "$logdir/$hwtype" ) {
    mkdir( "$logdir/$hwtype", 0755 ) || Xormin::log( "Cannot mkdir $logdir/$hwtype $!" . __FILE__ . ":" . __LINE__ ) && exit;
  }

  if ( !-f "$logdir/$hwtype/${devuid}_${alias}.alias" ) {
    `touch "$logdir/$hwtype/${devuid}_${alias}.alias"`;
  }

  my $fstdout = "$logdir/$hwtype/$devuid.out.log";
  my $fstderr = "$logdir/$hwtype/$devuid.err.log";

  open( $o->{stdout}, ">>", $fstdout );
  open( $o->{stderr}, '>>', $fstderr );

  if ($colored) {
    $o->{colored} = 1;
  }

  bless $o, $self;

  return $o;
}

sub _logline {
  my ( $self, $text, $channel ) = @_;
  if ($text) {
    my $act_time = localtime();
    chomp($text);
    my $FH = $self->{$channel};
    $| = 1;
    if ($self->{colored}) {
      print $FH BOLD, ( $channel eq 'stdout' ? GREEN : RED ), "$act_time ", RESET, YELLOW, " [PID $$]", WHITE, " : $text", "\n";
    }
    else {
      print $FH ( "$act_time [PID $$] : $text\n" );
    }
    if ($channel eq 'stdout'){
      print STDOUT "[$self->{'alias'}] $text";
    } else {
      print STDERR "[$self->{'alias'}] $text";
    }
  }
}

sub info {
  my ( $self, $text ) = @_;
  $self->_logline( $text, 'stdout' );
}

sub debug {
  my ( $self, $text ) = @_;
  $self->_logline( "DEBUG : $text", 'stdout' ) if (defined $self->{'debug'} && $self->{'debug'});
}

sub error {
  my ( $self, $text ) = @_;
  $self->_logline( $text, 'stderr' );
}

sub dump {
}

1;
