"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var AppService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppService = void 0;
const common_1 = require("@nestjs/common");
const config_1 = require("@nestjs/config");
const typeorm_1 = require("@nestjs/typeorm");
const typeorm_2 = require("typeorm");
const License_1 = require("./config/License");
const dataSource_1 = require("./config/dataSource");
const backend_entity_1 = require("./entity/backend.entity");
const objectItem_entity_1 = require("./entity/objectItem.entity");
const App_1 = require("./service/configuration/App");
const PM2_1 = require("./service/configuration/PM2");
const VmWareSDK_1 = require("./service/configuration/VmWareSDK");
const Microservice_1 = require("./service/microservices/Microservice");
const Linux_1 = require("./service/menu/menu/Linux");
const class_constants_1 = require("./common/constants/class.constants");
const Architecture_1 = require("./service/architecture/Architecture");
let AppService = AppService_1 = class AppService {
    constructor(configService, dataSource, backendRepository, objectItemsRepository) {
        this.configService = configService;
        this.dataSource = dataSource;
        this.backendRepository = backendRepository;
        this.objectItemsRepository = objectItemsRepository;
        this.logger = new common_1.Logger(AppService_1.name);
    }
    async onApplicationBootstrap() {
        if ((!PM2_1.PM2.isRunning() || PM2_1.PM2.isMasterBackend()) && App_1.App.isProduction()) {
            try {
                await (0, dataSource_1.getDataSource)().query("TRUNCATE public.sessions");
            }
            catch (e) {
                console.error("An error occurred while deleting sessions");
            }
        }
    }
    async onApplicationShutdown(signal) {
        console.warn("Shutdown signal: " + signal);
        if (this.dataSource.isInitialized) {
            await this.dataSource.destroy();
        }
        const dataSource = (0, dataSource_1.getDataSource)();
        if (dataSource.isInitialized) {
            await dataSource.destroy();
        }
        console.log("Datasource closed");
    }
    async onModuleInit() {
        if (PM2_1.PM2.isRunning() && !PM2_1.PM2.isMasterBackend()) {
            console.warn("[CheckPM2]: starting as slave backend process");
            await App_1.App.sleep(3000);
        }
        else {
            if (App_1.App.isDockerVersion()) {
                await App_1.App.checkVersionInDatabase();
                await VmWareSDK_1.VmWareSDK.createDockerPathSymlink();
            }
            await VmWareSDK_1.VmWareSDK.checkSdk();
            App_1.App.registerNode();
        }
        const license = await License_1.License.data();
        if (!license.verified) {
            this.logger.log("Loaded license: free");
            process.env['APP_FREE'] = "true";
        }
        else {
            this.logger.log("Loaded license: premium");
        }
    }
    async getDevices() {
        let devices = {
            Server: {},
            Storage: {},
            SAN: {},
            LAN: {},
            Database: {},
            Container: {},
            Cloud: {}
        };
        const storageItems = await this.objectItemsRepository.find({
            relations: {
                children: {
                    _child: true
                },
                hw_type_entity: true
            },
            where: {
                subsystem: 'device',
                hw_type_entity: {
                    class: 'storage'
                },
                children: {
                    _child: {
                        subsystem: (0, typeorm_2.In)(['pool', 'volume'])
                    }
                }
            }
        });
        for (const storageItem of storageItems) {
            const pools = storageItem.children.filter(child => child._child.subsystem === 'pool').length;
            const volumes = storageItem.children.filter(child => child._child.subsystem === 'volume').length;
            if (devices.Storage[storageItem.hw_type_entity.label]) {
                devices.Storage[storageItem.hw_type_entity.label].Device += 1;
                devices.Storage[storageItem.hw_type_entity.label].Pool += pools;
                devices.Storage[storageItem.hw_type_entity.label].Volume += volumes;
            }
            else {
                devices.Storage[storageItem.hw_type_entity.label] = {
                    Device: 1,
                    Pool: pools,
                    Volume: volumes
                };
            }
        }
        const sanItems = await this.objectItemsRepository.find({
            relations: {
                children: {
                    _child: true
                },
                hw_type_entity: true
            },
            where: {
                subsystem: 'device',
                hw_type_entity: {
                    class: 'san'
                },
                children: {
                    _child: {
                        subsystem: 'sanport'
                    }
                }
            }
        });
        for (const sanItem of sanItems) {
            const ports = sanItem.children.filter(child => child._child.subsystem === 'sanport').length;
            if (devices.SAN[sanItem.hw_type_entity.label]) {
                devices.SAN[sanItem.hw_type_entity.label].Device += 1;
                devices.SAN[sanItem.hw_type_entity.label].Port += ports;
            }
            else {
                devices.SAN[sanItem.hw_type_entity.label] = {
                    Device: 1,
                    Port: ports
                };
            }
        }
        const lanItems = await this.objectItemsRepository.find({
            relations: {
                children: {
                    _child: true
                },
                hw_type_entity: true
            },
            where: {
                subsystem: 'device',
                hw_type_entity: {
                    class: 'lan'
                },
                children: {
                    _child: {
                        subsystem: 'physical'
                    }
                }
            }
        });
        for (const lanItem of lanItems) {
            const ports = lanItem.children.filter(child => child._child.subsystem === 'physical').length;
            if (devices.LAN[lanItem.hw_type_entity.label]) {
                devices.LAN[lanItem.hw_type_entity.label].Device += 1;
                devices.LAN[lanItem.hw_type_entity.label].Port += ports;
            }
            else {
                devices.LAN[lanItem.hw_type_entity.label] = {
                    Device: 1,
                    Port: ports
                };
            }
        }
        const subsystemLabels = {
            vm: 'VM',
            server: 'VM',
            container: 'Container',
            db: 'DB',
            instance: 'DB',
            cluster: 'Cluster',
            pod: 'Pod',
            node: 'Node'
        };
        const otherItems = await this.objectItemsRepository.find({
            relations: {
                hw_type_entity: {
                    _class: true
                },
            },
            where: [{
                    hw_type_entity: {
                        class: (0, typeorm_2.In)(['virtualization'])
                    },
                    subsystem: 'vm'
                }, {
                    hw_type: 'linux',
                    subsystem: 'server'
                },
                {
                    hw_type: 'as400',
                    subsystem: 'server'
                }, {
                    hw_type_entity: {
                        class: 'container'
                    },
                    subsystem: (0, typeorm_2.In)(['cluster', 'pod', 'node'])
                }, {
                    hw_type_entity: {
                        class: 'database'
                    },
                    subsystem: (0, typeorm_2.In)(['db', 'instance'])
                }]
        });
        const linuxMapped = await (0, Linux_1.getLinuxMappedIds)(true);
        for (const otherItem of otherItems) {
            if (otherItem.hw_type === 'linux' && linuxMapped.includes(otherItem.item_id)) {
                continue;
            }
            const subsystemLabel = subsystemLabels[otherItem.subsystem];
            if (devices[otherItem.hw_type_entity._class.label][otherItem.hw_type_entity.label]) {
                if (devices[otherItem.hw_type_entity._class.label][otherItem.hw_type_entity.label][subsystemLabel]) {
                    devices[otherItem.hw_type_entity._class.label][otherItem.hw_type_entity.label][subsystemLabel] += 1;
                }
                else {
                    devices[otherItem.hw_type_entity._class.label][otherItem.hw_type_entity.label][subsystemLabel] = 1;
                }
            }
            else {
                devices[otherItem.hw_type_entity._class.label][otherItem.hw_type_entity.label] = {
                    [subsystemLabel]: 1
                };
            }
        }
        const cloudItems = await this.objectItemsRepository.find({
            relations: {
                hw_type_entity: {
                    _class: true
                },
            },
            where: {
                hw_type_entity: {
                    class: 'cloud'
                },
                subsystem: (0, typeorm_2.In)(['vm', 'compute', 'ec2'])
            }
        });
        for (const cloudItem of cloudItems) {
            if (devices.Cloud[cloudItem.hw_type_entity.label]) {
                devices.Cloud[cloudItem.hw_type_entity.label].VM += 1;
            }
            else {
                devices.Cloud[cloudItem.hw_type_entity.label] = {
                    VM: 1
                };
            }
        }
        return devices;
    }
    async getServices() {
        let services = [];
        let list = await PM2_1.PM2.listServices();
        let coreServices = Microservice_1.Microservice.getCoreServices();
        for (let i = 0; i < list.length; i++) {
            const service = list[i];
            if (coreServices.includes(service.name)) {
                continue;
            }
            let microservice = new Microservice_1.Microservice();
            await microservice.init(service.name);
            services.push({
                label: microservice.label,
                microservice: microservice.microservice,
                script: microservice.script,
                active: microservice.isActive(),
                pid: microservice.pid && microservice.pid !== 0 ? microservice.pid : null,
                version: microservice.version,
                monitor: microservice.monitor,
            });
        }
        services.sort((a, b) => a.label.localeCompare(b.label));
        return services;
    }
    async getInfo() {
        const data = {
            backend: {
                free: await App_1.App.isFree(),
                version: App_1.App.getVersion(),
                node_version: process.version,
                demo: App_1.App.isDemo(),
                docker: App_1.App.isDockerVersion(),
                production: App_1.App.isProduction(),
                premium: await this.getLicenseInfo(),
                limits: await License_1.License.getLimits(),
                appliance: await App_1.App.getApplianceVersion(),
                cert: await App_1.App.getCert(),
                firstLogin: await App_1.App.isFirstLogin()
            },
            database: {
                server: this.configService.get('DB_HOST'),
                port: this.configService.get('DB_PORT'),
                database: this.configService.get('DB_DATABASE'),
                version: await App_1.App.getTimescaleVersion(),
            }
        };
        return data;
    }
    async getLicenseLimits() {
        let limits = [];
        const license = await this.getLicenseInfo();
        const hwTypes = await Architecture_1.Architecture.getHwTypes(false);
        const classes = await Architecture_1.Architecture.getClasses(false);
        const licenseTypes = License_1.License.licenseTypes;
        if (license.valid) {
            const now = new Date().getTime();
            for (const technology of license.technologies) {
                const expiration = new Date(technology.expirationDate);
                const expired = expiration.getTime() < now;
                let _class;
                let label;
                if (technology.technology === '*') {
                    label = "Trial";
                }
                else if ([class_constants_1.ClassKey.STORAGE, class_constants_1.ClassKey.SAN, class_constants_1.ClassKey.LAN].includes(technology.technology)) {
                    _class = technology.technology;
                    const _classDef = classes.find(c => c.class === _class);
                    label = _classDef ? _classDef.label : class_constants_1.classDefinitionLabel[technology.technology];
                }
                else {
                    const hwType = hwTypes.find(hwType => hwType.hw_type === technology.technology);
                    if (hwType) {
                        _class = hwType.class;
                        label = hwType.label;
                    }
                    else if (License_1.License.isSubType(technology.technology)) {
                        const split = technology.technology.split("-");
                        const hwTypeDefSubType = hwTypes.find(hwType => hwType.hw_type === split[0]);
                        if (hwTypeDefSubType) {
                            _class = hwTypeDefSubType.class;
                            label = hwTypeDefSubType.label + " " + split[1];
                        }
                    }
                }
                limits.push({
                    technology: technology.technology,
                    label: label ? label : technology.technology,
                    subsystem: _class && licenseTypes[_class]
                        ? licenseTypes[_class].subsystem
                        : '',
                    publicAPI: technology.publicAPI,
                    limit: technology.limit,
                    expiration: technology.expirationDate,
                    expired: expired
                });
            }
        }
        return limits;
    }
    async getLicenseInfo() {
        const data = await License_1.License.data();
        if (data.verified) {
            return {
                valid: data.verified,
                isTrial: data.isTrial,
                expirationDate: data.expirationDate,
                technologies: data.technologies
            };
        }
        else {
            return {
                valid: false,
                isTrial: data.isTrial,
                expirationDate: data.expirationDate ? data.expirationDate : null,
                technologies: data.technologies
            };
        }
    }
    async getTables() {
        const entityManager = (0, dataSource_1.getDataSource)();
        let tables = {
            total: 0,
            hw_types: {},
        };
        const exclude_list = ['metrics', 'metrics_5m', 'metrics_1h', 'metrics_6h', 'metrics_1d'];
        const hypertables = await entityManager.query("SELECT hypertable_name, hypertable_size(format('%I.%I', hypertable_schema, hypertable_name)::regclass) FROM timescaledb_information.hypertables;");
        for (const hypertable of hypertables) {
            if (exclude_list.includes(hypertable['hypertable_name'])) {
                continue;
            }
            const splitted = hypertable['hypertable_name'].split('_', 2);
            tables['total'] += +hypertable['hypertable_size'];
            if (tables['hw_types'][splitted[1]]) {
                if (!tables['hw_types'][splitted[1]]['subsystems'][hypertable['hypertable_name'].split(splitted[1] + '_')[1]]) {
                    tables['hw_types'][splitted[1]]['subsystems'][hypertable['hypertable_name'].split(splitted[1] + '_')[1]] = {
                        total: 0,
                        tables: {},
                    };
                }
                tables['hw_types'][splitted[1]]['subsystems'][hypertable['hypertable_name'].split(splitted[1] + '_')[1]]['total'] +=
                    +hypertable['hypertable_size'];
                tables['hw_types'][splitted[1]]['subsystems'][hypertable['hypertable_name'].split(splitted[1] + '_')[1]]['tables'][hypertable['hypertable_name']] = +hypertable['hypertable_size'];
                tables['hw_types'][splitted[1]]['total'] += +hypertable['hypertable_size'];
            }
            else {
                tables['hw_types'][splitted[1]] = {
                    total: +hypertable['hypertable_size'],
                    subsystems: {},
                };
                tables['hw_types'][splitted[1]]['subsystems'][hypertable['hypertable_name'].split(splitted[1] + '_')[1]] = {
                    total: +hypertable['hypertable_size'],
                    tables: {},
                };
                tables['hw_types'][splitted[1]]['subsystems'][hypertable['hypertable_name'].split(splitted[1] + '_')[1]]['tables'][hypertable['hypertable_name']] = +hypertable['hypertable_size'];
                tables['hw_types'][splitted[1]]['total'] += +hypertable['hypertable_size'];
            }
        }
        const views = await entityManager.query("SELECT view_name, hypertable_size(format('%I.%I', materialization_hypertable_schema , materialization_hypertable_name )::regclass) FROM timescaledb_information.continuous_aggregates;");
        for (const view of views) {
            const splitted = view['view_name'].split('_');
            tables['total'] += +view['hypertable_size'];
            const subsystem = view['view_name'].split(splitted[1] + '_')[1].replace(/_5m|_1h|_6h|_1d/g, '');
            if (tables['hw_types'][splitted[1]]) {
                tables['hw_types'][splitted[1]]['subsystems'][subsystem]['tables'][view['view_name']] = +view['hypertable_size'];
                tables['hw_types'][splitted[1]]['subsystems'][subsystem]['total'] += +view['hypertable_size'];
                tables['hw_types'][splitted[1]]['total'] += +view['hypertable_size'];
            }
        }
        return tables;
    }
};
exports.AppService = AppService;
exports.AppService = AppService = AppService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(1, (0, typeorm_1.InjectDataSource)()),
    __param(2, (0, typeorm_1.InjectRepository)(backend_entity_1.Backend)),
    __param(3, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __metadata("design:paramtypes", [config_1.ConfigService,
        typeorm_2.DataSource,
        typeorm_2.Repository,
        typeorm_2.Repository])
], AppService);
//# sourceMappingURL=app.service.js.map