"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const process_1 = require("process");
const fs = require("fs");
const path_1 = require("path");
if (!process.argv[2]) {
    console.error("The menu name is not specified. Example: GCloud");
    (0, process_1.exit)(1);
}
if (!process.argv[3]) {
    console.error("The hwType is not specified. Example: gcloud");
}
const menuName = process.argv[2];
const hwType = process.argv[3];
const optionalStuff = process.argv[4];
class MenuWriter {
    constructor(menuName, hwType) {
        this.menuName = menuName;
        this.hwType = hwType;
        this.menuPath = (0, path_1.resolve)("src/service/menu/menu");
        this.fileName = this.menuPath + "/" + this.menuName + ".ts";
        this.className = menuName.toLowerCase().replace(menuName.charAt(0).toLowerCase(), menuName.charAt(0).toUpperCase());
        fs.writeFileSync(this.fileName, "");
    }
    writeImports() {
        const importSection = `
import { METRIC_AGGREGATE } from '@xormon/common/constants/pageResponse.constants';
import { AbstractFolder, AbstractPage } from './AbstractMenu';
import { ObjectItem } from '@xormon/entity/objectItem.entity';
import { ClassKey, HwTypeKey } from '@const/class.constants';
import { HwTypeKey } from '@const/hwTypeKey,constants';
import { MetricsWrapper } from '@xormon/service/metrics/MetricsWrapper';
import { URL } from './URL';
import { ChartType, FourGraphBox, Metric, MetricAggregate, Page, Tab } from '@xormon/common/pageResponse.dto';
import { Overview } from './Overview';
import { ACLCheck } from '@xormon/service/acl/ACLCheck';
import { Table } from './Table';
import { UNIT } from '@xormon/common/constants/subsystemMetricsResponse.constants';
import { Top } from './Top';
import { Heatmap } from './Heatmap';
import { ConfigurationTable } from './ConfigurationTable';

`;
        fs.appendFileSync(this.fileName, importSection);
    }
    writeWarningSection() {
        const warningSection = `
==========================================================================
==========================================================================
==========================================================================

instructions:

After defining your root folder add the class of the root folder to the 
${this.className}Page class like: rootFolder = ${this.className}RootFolder

Define definition of MetricsWrapper in your hwType Metrics class at the 
bottom, you need to define your class and hwType

If your file got generated without heatmap and configuration pages, you 
can add third command line argument for this script (it can by anything)

The heatmap and configuration sections are incomplete and are in need of
filling inforamtion for them to work. So go through them carefully

==========================================================================
==========================================================================
==========================================================================   
 
`;
        fs.appendFileSync(this.fileName, warningSection);
    }
    writeMainMenuClasses() {
        const menuClassSection = `
export class ${this.className}Folder extends AbstractFolder{
  hwType: string = '${this.hwType}';
}

export class ${this.className}Page extends AbstractPage{
  hwType: string = '${this.hwType}';
  rootFolder = null;
}

export class ${this.className}SubsystemFolder extends ${this.className}Folder {
  subsystem: string;
  itemId: string;

  constructor(itemId?: string) {
    super();
    if (itemId !== null) {
      this.itemId = itemId;
    }
  }
}

export abstract class ${this.className}SingleItemPage extends ${this.className}Page {
  subsystem: string;
}

export class ${this.className}RootFolder extends ${this.className}Folder {
  

  async getChildren() {
    let menuItems = [];

    //fill in your RootFolder menuItems

    return menuItems;
  }

  async getChildrenAcl() {
    
  }
}

`;
        fs.appendFileSync(this.fileName, menuClassSection);
    }
    writeConfSection() {
        const confSection = `
export class ${this.className}ConfigurationPage extends ${this.className}Page {
    title: string = 'Configuration';
  
    async getPage() {
      let title = 'Configuration';
      let tabs: Tab[] = [];
  
      let yourSubsystem = await this.objectItemsRepository.findBy({ hw_type: '${this.hwType}', subsystem: null /*subsystem*/ });
      let subsystemTable = new ConfigurationTable(yourSubsystem.map(item => item.item_id), [/*your conf props*/], [/*this subsystem parent subsystems*/]);
      subsystemTable.setItemLabel("ItemLabel");
      subsystemTable.setAcl(this.request);
  
      tabs.push({
        title: 'Your Title',
        content: [await subsystemTable.generate()],
      });

      //for more subsystem tabs add the same as above with different susbystem and props
              
      this.page = new Page(title, tabs);
      return this.page;
    }
}`;
        fs.appendFileSync(this.fileName, confSection);
    }
    writeHeatmapSection() {
        const heatmapSecion = `
export class ${this.className}HeatmapPage extends ${this.className}Page {
    title: string = 'Heatmap';
  
    async getPage() {
      let title = 'Heatmap';
  
      let tabs: Tab[] = [];
  
      tabs.push({
        title: 'Your tab title',
        content: [
          //await new Heatmap('${this.hwType}', null/*subsystem*/, true)
          //  .computedHeatmapBox(ClassKey.THIS_HWTYPES_CLASS, this.request) //change heatmapbox type if needed (virtualization)!!
        ],
      });
  
      let heatmapTable = new Table()
        .addHeader({ value: "" })
      //define your heatmapTable headers

  
      let items = await this.objectItemsRepository.find({
        where: {
          hw_type: '${this.hwType}',
          subsystem: '', //subsystem
          parents: {
            _parent: {
              subsystem: ''//parent subsystem
            }
          }
        },
        relations: {
          heatmaps: true,
          parents: {
            _parent: true
          },
          hostcfg_relations: {
            _hostcfg: true
          }
        }
      });
  
      // ACL filter
      if (this.request) {
        const ids = await ACLCheck.filterItemsWithRequest(this.request, items.map(item => item.item_id));
        items = items.filter(item => ids.includes(item.item_id));
      }
  
      for (const item of items) {
        //fill your table with data
      }
  
      heatmapTable.sortByIndex(0); //sort table by column
  
      tabs.push({
        title: 'Your table title',
        content: [heatmapTable.generateBox()],
      });
  
      //add more if needed

      this.page = new Page(title, tabs);
      return this.page;
    }
}`;
        fs.appendFileSync(this.fileName, heatmapSecion);
    }
    writeMetricsTemplate() {
        const metricsSection = `
export class ${this.menuName}Metrics {
    static Metric = class extends Metric {
      constructor(subsystem: string, metric: string) {
        super();
        this.metric = metric;
        
        const definition = null;// example: MetricsWrapper.getMetric(ClassKey.CLOUD, HwTypeKey.GCLOUD, subsystem, metric);
        if (definition) {
          Object.assign(this, definition);
        }
      }
      getGraph(type: ChartType, stacked?: boolean, order?: number, color?: string) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.color = color ? color : undefined;
        return this;
      }
      getAggrGraph(type: ChartType, stacked?: boolean, order?: number, aggregate?: MetricAggregate) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.aggregate = aggregate ? aggregate : METRIC_AGGREGATE.items;
        return this;
      }
    };
  }

`;
        fs.appendFileSync(this.fileName, metricsSection);
    }
    writeOptionalClasses() {
        this.writeConfSection();
        this.writeHeatmapSection();
    }
    async createMenu() {
        this.writeImports();
        this.writeWarningSection();
        this.writeMainMenuClasses();
        if (optionalStuff) {
            this.writeOptionalClasses();
        }
        this.writeMetricsTemplate();
    }
}
new MenuWriter(menuName, hwType).createMenu().then(() => (0, process_1.exit)(0)).catch((e) => {
    console.error(e);
    (0, process_1.exit)(1);
});
//# sourceMappingURL=cli-create-menu.js.map