import { BOX_TYPE, CHART_TYPE, EXPORT_METHOD, GRAPH_INTERVAL, METRIC_AGGREGATE, MULTIPLIER, ORIENTATION, TABLE_STYLE_CLASS, TEXT_STYLE_CLASS, UNIT_PREFIX } from '@const/pageResponse.constants';
import { Status } from '@dto/healthStatus.dto';
import { BasicResponseDTO } from './basicResponse.dto';
import { TOPOLOGY_ITEM_TYPE, TOPOLOGY_TYPE } from './constants/pageResponse.constants';
import { DashboardGroup } from './dashboard.dto';
import { Unit } from './subsystemMetricsResponse.dto';
import { PredictionBase } from './timeseriesRequest.dto';
export type ChartType = typeof CHART_TYPE[keyof typeof CHART_TYPE];
export type BoxType = typeof BOX_TYPE[keyof typeof BOX_TYPE];
export type ExportMethod = typeof EXPORT_METHOD[keyof typeof EXPORT_METHOD];
export type GraphInterval = typeof GRAPH_INTERVAL[keyof typeof GRAPH_INTERVAL];
export type MetricAggregate = typeof METRIC_AGGREGATE[keyof typeof METRIC_AGGREGATE];
export type Multiplier = typeof MULTIPLIER[keyof typeof MULTIPLIER];
export type TableStyleClass = typeof TABLE_STYLE_CLASS[keyof typeof TABLE_STYLE_CLASS];
export type TextStyleClass = typeof TEXT_STYLE_CLASS[keyof typeof TEXT_STYLE_CLASS];
export type UnitPrefix = typeof UNIT_PREFIX[keyof typeof UNIT_PREFIX];
export type TopologyType = typeof TOPOLOGY_TYPE[keyof typeof TOPOLOGY_TYPE];
export type TopologyItemType = typeof TOPOLOGY_ITEM_TYPE[keyof typeof TOPOLOGY_ITEM_TYPE];
export type OrientationType = typeof ORIENTATION[keyof typeof ORIENTATION];
export interface IMetricType {
    unit: Unit;
    shortcut: string;
    defaultPrefix?: UnitPrefix;
    defaultPrecision?: number;
    metricType?: string;
    unitLabel?: string;
    minY?: number;
}
export declare class Metric implements IMetricType {
    type: ChartType;
    stacked: boolean;
    metric: string;
    order: number;
    aggregate?: MetricAggregate;
    unit: Unit;
    unitLabel?: string;
    shortcut: string;
    label?: string;
    color?: string;
    defaultPrefix?: UnitPrefix;
    defaultPrecision?: number;
    minY?: number;
    legendTop?: boolean;
    legendOnly?: boolean;
}
export declare class CapacityTableBase {
    header: CapacityTableHeader[];
    rows: CapacityTableColumn[][];
    summaryRow?: CapacityTableColumn[];
    export?: Export;
}
export declare class TableHeaderBase {
    order: number;
    value: string;
    prefix?: UnitPrefix;
    number?: boolean;
    decimal?: number;
    shortcut?: string;
}
export declare class CapacityTableHeader extends TableHeaderBase {
    thermometer?: boolean;
}
export declare class CapacityTableColumn {
    order: number;
    value: number | string;
    url?: string;
    tooltip?: string;
}
export declare class CapacityTableBox extends CapacityTableBase implements Box {
    type: "CapacityTableBox";
}
export declare class TableHeader extends TableHeaderBase {
    unit?: Unit;
    multiplier?: 1000 | 1024;
    shortcut?: string;
    styleClass?: TableStyleClass;
    sort?: 'desc' | 'asc';
    noWrap?: boolean;
    trim: boolean;
    group?: string;
    newLines?: boolean;
    linuxMpath?: boolean;
    trimLength?: number;
    disable?: {
        search?: boolean;
        sort?: boolean;
    };
}
export declare class TableGroupHeader {
    order: number;
    value: string;
    group: string;
    number?: boolean;
}
export declare class TableGroup {
    group: string;
    unit?: Unit;
    shortcut?: string;
    prefix?: UnitPrefix;
    number?: boolean;
    decimal?: number;
    styleClass?: TableStyleClass;
    multiplier?: 1000 | 1024;
    trim: boolean;
}
export declare class TableColumn {
    order: number;
    value: (string | number)[];
    styleClass?: TableStyleClass;
    bgStatus?: Status;
    bgColor?: string;
    url?: string[];
    tooltip?: string;
    modal?: Box[];
    noTrim?: boolean;
}
export interface Box {
    type: BoxType;
    help?: Help;
}
export declare class TextStyled {
    text: string;
    styleClass?: TextStyleClass;
}
export declare class TextBox extends TextStyled implements Box {
    type: "TextBox";
}
export declare class HtmlTextBox implements Box {
    type: "HtmlTextBox";
    html: string;
}
export declare class DashboardBox implements Box {
    type: "DashboardBox";
    groups: DashboardGroup[];
    width: number;
}
export declare class SunburstItem {
    item_id: string;
    label: string;
    parent_id: string;
    parent_label: string;
    value: number;
}
export declare class SunburstBox implements Box {
    type: "SunburstBox";
    items: SunburstItem[];
}
export declare class StaticMetric {
    type: ChartType;
    label: string;
    value: number;
    unit: Unit;
    shortcut: string;
}
export declare class GraphTableSort {
    metric: string;
    order: 'avg' | 'max';
}
export declare class GraphBase {
    aggregated?: boolean;
    label?: string;
    graphLabel?: string;
    parent?: GraphBaseParent;
    metrics: Metric[];
    minInterval?: number;
    peakMetric?: Metric;
    properties?: GraphTableProperty[];
    sort?: GraphTableSort;
    staticMetrics?: StaticMetric[];
    uuids: string[];
}
export declare class GraphBaseParent {
    label: string;
    subsystem: string;
}
export declare class GraphTableProperty {
    label: string;
    search?: boolean;
    values: {
        [item_id: string]: GraphTablePropertyItem;
    };
}
export declare class GraphTablePropertyItem {
    value: string;
    url?: string;
}
export declare class MultiGraphBox implements Box {
    type: "MultiGraphBox";
    graphs: GraphBaseInterval[];
}
export declare class GraphBaseInterval extends GraphBase {
    interval: GraphInterval;
}
export declare class GraphBaseTimeRange extends GraphBase {
    start: number;
    end: number;
}
export declare class SingleTimeRangeGraphBox extends GraphBaseTimeRange implements Box {
    type: "SingleTimeRangeGraphBox";
}
export declare class SingleGraphBox extends GraphBaseInterval implements Box {
    type: "SingleGraphBox";
}
export declare class PredictionMetadata {
    label: string;
    key: string;
    number: boolean;
}
export declare class PredictionGraphBox extends PredictionBase implements Box {
    type: "PredictionGraphBox";
    graphLabel: string;
    metric: Metric;
    other_metrics: Metric[];
    metadata: PredictionMetadata[];
    lowerMax?: number;
    help?: Help;
}
export declare class Help {
    text: string;
    url: string;
}
export declare class FourGraphBox extends GraphBase implements Box {
    type: BoxType;
}
export declare class RegroupGraph extends GraphBase {
    url: string;
    tab: string;
}
export declare class RegroupButtonBox implements Box {
    type: "RegroupButtonBox";
    graphs: RegroupGraph[];
}
export declare class TableBase {
    header: TableHeader[];
    rows: TableColumn[][];
    fixedCols?: number;
    export?: Export;
    headerNoWrap?: boolean;
    orientation: OrientationType;
    headerNoUnit?: boolean;
}
export declare class TableBox extends TableBase implements Box {
    type: "TableBox";
}
export declare class MirrorBox implements Box {
    type: "MirrorBox";
}
export declare class TableGroupBox implements Box {
    type: "TableGroupBox";
    tables: TableBox[] | TableHeaderGroupBox[];
}
export declare class TableHeaderGroupBox implements Box {
    type: "TableHeaderGroupBox";
    header: TableGroupHeader[];
    rows: TableColumn[][];
    groups: TableGroup[];
    orientation: OrientationType;
}
export declare class TablePivotBox extends TableBase implements Box {
    type: "TablePivotBox";
}
export declare class Export {
    fileName?: string;
    url: string;
    method: ExportMethod;
    body?: {};
}
export declare class TopBoxTable extends TableBase {
    label: string;
    export: Export;
}
export declare class TopBox implements Box {
    type: BoxType;
    tables: TopBoxTable[];
}
export declare class PortHealthStatusPort {
    label: string;
    url: string;
    operationalStatus: string;
    operationalStatusColor: string;
    physicalStatus?: string;
    physicalStatusColor?: string;
    adminStatus?: string;
    adminStatusColor?: string;
    operationalStatusReason?: string;
    operationalStatusReasonColor?: string;
}
export declare class PortHealthStatusDevice {
    label: string;
    ports: PortHealthStatusPort[];
}
export declare class PortHealthStatusLegendStatus {
    color: string;
    label: string;
    text?: string;
}
export declare const PORT_STATUS_TYPE: readonly ["operationalStatus", "physicalStatus", "adminStatus", "operationalStatusReason"];
export type PortStatusType = typeof PORT_STATUS_TYPE[number];
export declare class PortHealthStatusLegend {
    type: PortStatusType;
    label: string;
    text?: string;
    statuses: PortHealthStatusLegendStatus[];
}
export declare class PortHealthStatusHwType {
    label: string;
    devices: PortHealthStatusDevice[];
    legends: PortHealthStatusLegend[];
}
export declare class PortHealthStatusBox implements Box {
    type: "PortHealthStatusBox";
    hwTypes: PortHealthStatusHwType[];
    tabs?: Tab[];
}
export interface IHeatmap {
    label: string;
    metric: string;
    items: {
        item_id?: string;
        label: string;
        percent: number;
        url: string;
    }[];
}
export declare class HeatmapBox implements Box {
    type: "HeatmapBox";
    metrics: IHeatmap[];
}
export declare class OverviewRequest {
    item_id: string;
    start: number;
    end: number;
}
export declare class OverviewItem {
    label: string;
    item_id: string;
}
export declare class OverviewCategory {
    label: string;
    items: OverviewItem[];
}
export declare class OverviewBox implements Box {
    type: "OverviewBox";
    categories: OverviewCategory[];
}
export declare class CustomTopBox {
    type: "CustomTopBox";
    hw_type: string;
    subsystem: string;
    metrics: string[];
    parentsSubsystem?: string[];
    pageLabel?: string;
}
export declare class CustomTopRequest {
    hw_type: string;
    subsystem: string;
    type: 'avg' | 'max';
    metrics: string[];
    start: number;
    end: number;
    parentsSubsystem?: string[];
    pageLabel?: string;
}
export declare class StorageHeatmapInterval {
    color: string;
    limit: number;
}
export declare class StorageHeatmapItem {
    device: string;
    label: string;
    item_id: string;
    read?: StorageHeatmapItemMetric;
    write?: StorageHeatmapItemMetric;
    total?: StorageHeatmapItemMetric;
    url: string;
    sortValue?: number;
}
export declare class StorageHeatmapItemMetric {
    line: string;
    color: string;
    value?: number;
}
export declare class StorageHeatmapTier {
    items: StorageHeatmapItem[];
    read_intervals: StorageHeatmapInterval[];
    write_intervals: StorageHeatmapInterval[];
}
export declare class StorageHeatmapTiers {
    [tier: string]: StorageHeatmapTier;
}
export declare class StorageHeatmapBox implements Box {
    type: "StorageHeatmapBox";
    tiers: StorageHeatmapTiers;
}
export declare class TopologyConnection {
    sourceWwn: string;
    destinationWwn: string;
    sourceType: string;
    destinationType: string;
    percent: number;
    text: string;
}
export declare class TopologyItem {
    label: string;
    id?: string;
    wwn?: string;
    url?: string;
    children?: TopologyItem[];
}
export declare class TopologyGroup {
    label: string;
    type: TopologyItemType;
    children: TopologyItem[];
}
export declare class TopologyRequest {
    item_id?: string;
    type: TopologyType;
    hw_type?: string;
}
export declare class TopologyRequestItem extends TopologyRequest {
    label: string;
}
export declare class TopologyRequestBox implements Box {
    type: "TopologyRequestBox";
    topologies: TopologyRequestItem[];
}
export declare class TopologyBox implements Box {
    type: "TopologyBox";
    connections?: TopologyConnection[];
    groups: TopologyGroup[];
}
export declare class Tab<T extends Box = Box> {
    title: string;
    content?: T[];
    lazyContent?: string;
    export?: Export;
    help?: Help;
    subTabs?: SubTab[];
}
export type SubTab = Omit<Tab, "subTabs">;
export type MenuFormPageItem = {
    hwType: string;
    subsystem: string;
    class: string;
};
export declare class MenuFormPage {
    hwTypes: MenuFormPageItem[];
    title: string;
}
export declare class Page {
    title: string;
    tabs: Tab[];
    formPage?: MenuFormPage;
    constructor(title: string, tabs: Tab[], formPage?: MenuFormPage);
}
export declare class PageResponseDTO extends BasicResponseDTO {
    data: Page;
}
export declare class TabResponseDTO extends BasicResponseDTO {
    data: Tab;
}
export declare class OverviewResponseDTO extends BasicResponseDTO {
    data: Tab;
}
export declare class CustomTopResponseDTO extends BasicResponseDTO {
    data: Tab;
}
export declare class HeatmapContent {
    heatmapBox: HeatmapBox | StorageHeatmapBox;
    textBox: TextBox;
}
