"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DatabaseHelper = void 0;
const TableGenerator_1 = require("../service/metrics/TableGenerator");
const dataSource_1 = require("./dataSource");
class DatabaseHelper {
    static async getColumns(schema, table, dataSource) {
        try {
            if ((0, dataSource_1.getDataSource)() === undefined) {
                if (!dataSource) {
                    console.warn("Undefined dataSource");
                    return;
                }
            }
            else {
                dataSource = (0, dataSource_1.getDataSource)();
            }
            return (await dataSource.query(`SELECT * FROM information_schema.columns WHERE table_schema = '${schema}' AND table_name = '${table}';`))
                .map(column => column.column_name);
        }
        catch (e) {
            console.error(`[DatabaseHelper] [getColumns] schema: ${schema}, table: ${table}, error: ${e}`);
            return [];
        }
    }
    static async renamePerfColumn(table, oldColumn, newColumn, dataSource) {
        const hwType = table.split("_")[1];
        const subsystem = table.split(hwType + "_")[1];
        const maxMetrics = await TableGenerator_1.TableGenerator.getMaxMetrics();
        let incMax = false;
        if (maxMetrics[hwType] && maxMetrics[hwType][subsystem] && maxMetrics[hwType][subsystem].includes(newColumn)) {
            incMax = true;
        }
        if ((await this.getColumns("metrics", table, dataSource)).includes(oldColumn)) {
            await dataSource.renameColumn('metrics.' + table, oldColumn, newColumn);
            const intervals = ["1m", "5m", "1h", "6h", "1d"];
            for (const interval of intervals) {
                if (await dataSource.hasTable('metrics.' + table + "_" + interval)) {
                    if (incMax) {
                        await dataSource.renameColumn('metrics.' + table + "_" + interval, oldColumn + "_max", newColumn + "_max");
                    }
                    await dataSource.renameColumn('metrics.' + table + "_" + interval, oldColumn, newColumn);
                }
            }
        }
    }
    static async renamePerfTable(table, newTableName, dataSource) {
        const isTableExists = typeof dataSource.hasTable === 'function'
            ? await dataSource.hasTable('metrics.' + table)
            : await this.tableExists(table, dataSource, 'metrics');
        if (isTableExists) {
            const intervals = ["1d", "6h", "1h", "5m", "1m"];
            for (const interval of intervals) {
                const isViewExists = typeof dataSource.hasTable === 'function'
                    ? await dataSource.hasTable(`metrics.${table}_${interval}`)
                    : await this.tableExists(`${table}_${interval}`, dataSource, 'metrics');
                if (isViewExists) {
                    if (typeof dataSource.hasTable === 'function') {
                        await dataSource.query(`ALTER TABLE metrics.${table}_${interval} RENAME TO ${newTableName}_${interval};`);
                    }
                    else {
                        await dataSource.manager.connection.query(`ALTER TABLE metrics.${table}_${interval} RENAME TO ${newTableName}_${interval};`);
                    }
                }
            }
            if (typeof dataSource.hasTable === 'function') {
                await dataSource.query(`ALTER TABLE metrics.${table} RENAME TO ${newTableName};`);
            }
            else {
                await dataSource.manager.connection.query(`ALTER TABLE metrics.${table} RENAME TO ${newTableName};`);
            }
        }
    }
    static async deletePerfTable(table, dataSource) {
        const isTableExists = typeof dataSource.hasTable === 'function'
            ? await dataSource.hasTable('metrics.' + table)
            : await this.tableExists(table, dataSource, 'metrics');
        if (isTableExists) {
            const intervals = ["1d", "6h", "1h", "5m", "1m"];
            let err = 0;
            for (const interval of intervals) {
                const view = "metrics." + table + "_" + interval;
                try {
                    if (typeof dataSource.hasTable === 'function') {
                        await dataSource.query("DROP MATERIALIZED VIEW IF EXISTS " + view + " CASCADE");
                    }
                    else {
                        await dataSource.manager.connection.query("DROP MATERIALIZED VIEW IF EXISTS " + view + " CASCADE");
                    }
                }
                catch (e) {
                    err++;
                }
            }
            if (err > 2) {
                throw Error("Failed to delete materialized views");
            }
            if (typeof dataSource.hasTable === 'function') {
                await dataSource.query("DROP TABLE metrics." + table + " CASCADE");
            }
            else {
                await dataSource.manager.connection.query("DROP TABLE metrics." + table + " CASCADE");
            }
            console.log("Table " + table + " removed");
        }
        return;
    }
    static async tableExists(tableName, dataSource, schema) {
        return (await dataSource.query(`SELECT exists (
              SELECT FROM information_schema.tables
                WHERE table_name  = '${tableName}' AND table_schema = '${schema ? schema : 'public'}'
        )`))[0].exists;
    }
    static async enumExists(enumName, dataSource) {
        return (await dataSource.query(`SELECT 1 FROM pg_type WHERE typname = '${enumName}'`))[0] !== undefined;
    }
}
exports.DatabaseHelper = DatabaseHelper;
//# sourceMappingURL=DatabaseHelper.js.map