"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TimeseriesInterceptor = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const objectItem_entity_1 = require("../entity/objectItem.entity");
const user_entity_1 = require("../entity/user.entity");
const ACLCheck_1 = require("../service/acl/ACLCheck");
const Linux_1 = require("../service/menu/menu/Linux");
const URL_1 = require("../service/menu/menu/URL");
const User_1 = require("../service/users/User");
const operators_1 = require("rxjs/operators");
const typeorm_2 = require("typeorm");
let TimeseriesInterceptor = class TimeseriesInterceptor {
    constructor(objectItemsRepository, usersRepository) {
        this.objectItemsRepository = objectItemsRepository;
        this.usersRepository = usersRepository;
    }
    async intercept(context, next) {
        const request = context.switchToHttp().getRequest();
        const response = context.switchToHttp().getResponse();
        const caller = context.getHandler().name;
        const user = await User_1.User.getUserByRequest(request, true);
        const groupIds = user.groups.map(group => group.group_id);
        const url = response.req.url;
        return next
            .handle()
            .pipe((0, operators_1.map)(async (data) => {
            if (!Object.keys(data.data.metrics)[0]) {
                return data;
            }
            const linuxMapping = await (0, Linux_1.getLinuxMapping)();
            for (let metric of Object.keys(data.data.metrics)) {
                for (let item of data.data.metrics[metric]) {
                    if (item.metadata.url) {
                        const url = item.metadata.url;
                        if (url.includes('/page/linux/')) {
                            const tmpUrl = new URL('https://do.not.care' + url);
                            const itemId = tmpUrl.searchParams.get('item_id');
                            const mapping = linuxMapping.find(item => item.agent_id === itemId);
                            if (mapping) {
                                const objectItem = await this.objectItemsRepository.findOne({ where: { item_id: mapping.item_id } });
                                if (objectItem) {
                                    const newURL = URL_1.URL.generate(objectItem.hw_type, objectItem.subsystem, objectItem.item_id);
                                    item.metadata.url = newURL;
                                }
                            }
                        }
                    }
                    if (item.metadata.parent && item.metadata.parent.url) {
                        const url = item.metadata.parent.url;
                        if (url.includes('/page/linux/')) {
                            const tmpUrl = new URL('https://do.not.care' + url);
                            const itemId = tmpUrl.searchParams.get('item_id');
                            const mapping = linuxMapping.find(item => item.agent_id === itemId);
                            if (mapping) {
                                const objectItem = await this.objectItemsRepository.findOne({ where: { item_id: mapping.item_id } });
                                if (objectItem) {
                                    const newURL = URL_1.URL.generate(objectItem.hw_type, objectItem.subsystem, objectItem.item_id);
                                    item.metadata.parent.url = newURL;
                                }
                            }
                        }
                    }
                }
            }
            if (groupIds.includes(1)) {
                return data;
            }
            const items = data.data.metrics[Object.keys(data.data.metrics)[0]];
            const itemIds = items.map(item => item.metadata.uuid);
            const filteredIds = await ACLCheck_1.ACLCheck.filterItemsByRulesTopDown(groupIds, itemIds);
            for (const itemId of itemIds) {
                let uuid = itemId;
                const mapping = linuxMapping.find(item => item.agent_id === uuid);
                if (mapping) {
                    const filteredMappedId = await ACLCheck_1.ACLCheck.filterItemsByRulesTopDown(groupIds, [mapping.item_id]);
                    if (filteredMappedId.includes(mapping.item_id)) {
                        filteredIds.push(mapping.agent_id);
                    }
                }
                else {
                    if (uuid == null) {
                        filteredIds.push(uuid);
                        continue;
                    }
                    if (uuid.startsWith('Linux_') || uuid.startsWith('power_') || uuid.startsWith('as400_')) {
                        let itemWithParents = await ACLCheck_1.ACLCheck.getItemWithParents(uuid);
                        if (itemWithParents && itemWithParents.parents) {
                            for (let parent of itemWithParents.parents) {
                                const mapping = linuxMapping.find(item => item.agent_id === parent.parent);
                                if (mapping) {
                                    const filteredMappedId = await ACLCheck_1.ACLCheck.filterItemsByRulesTopDown(groupIds, [mapping.item_id]);
                                    if (filteredMappedId.includes(mapping.item_id)) {
                                        filteredIds.push(uuid);
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            for (const metric in data.data.metrics) {
                data.data.metrics[metric] = data.data.metrics[metric].filter(item => filteredIds.includes(item.metadata.uuid));
            }
            return data;
        }));
    }
};
exports.TimeseriesInterceptor = TimeseriesInterceptor;
exports.TimeseriesInterceptor = TimeseriesInterceptor = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(1, (0, typeorm_1.InjectRepository)(user_entity_1.User)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository])
], TimeseriesInterceptor);
//# sourceMappingURL=timeserie.interceptor.js.map