"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var AppService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const hwTypeKey_constants_1 = require("../common/constants/hwTypeKey.constants");
const aggregation_entity_1 = require("../entity/aggregation.entity");
const typeorm_2 = require("typeorm");
let AppService = AppService_1 = class AppService {
    constructor(dataSource) {
        this.dataSource = dataSource;
        this.logger = new common_1.Logger(AppService_1.name);
        this.compressedLimit = 2 * 24 * 60 * 60 * 1000;
        this._5minAggregationLimit = 60 * 60 * 1000;
    }
    async saveMetrics(metrics, hw_type, subsystem) {
        this.logger.log("Saving " + metrics.length + " perf rows for hw_type:" + hw_type + " and subsystem:" + subsystem);
        if (hw_type && subsystem) {
            const table = 'metrics.metrics_' + hw_type.toLowerCase() + '_' + subsystem.toLowerCase();
            const repository = this.dataSource.getRepository(table);
            const columns = repository.metadata.columns.map((column) => {
                return column.propertyName;
            });
            const now = new Date().getTime();
            let checkManualAggregation = (hwTypeKey_constants_1.HwTypeKey.LINUX, hwTypeKey_constants_1.HwTypeKey.WINDOWS).includes(hw_type);
            let manualAggregationTime = undefined;
            const experimentMetrics = metrics.filter((datapoint) => {
                if (now - new Date(datapoint.time).getTime() >= this.compressedLimit) {
                    console.warn("CompressLimitSkip [" + hw_type + " " + subsystem + " " + datapoint.item_id + "]: " + (now - new Date(datapoint.time).getTime()) + " > " + this.compressedLimit);
                    return false;
                }
                if (datapoint.item_id === "") {
                    return false;
                }
                if (checkManualAggregation && manualAggregationTime == undefined && now - new Date(datapoint.time).getTime() >= this._5minAggregationLimit) {
                    manualAggregationTime = datapoint.time;
                }
                return true;
            }).map((obj) => {
                let rObj = {
                    time: obj.time,
                    item_id: obj.item_id,
                    ...obj.data,
                };
                return rObj;
            });
            const chunk = 1000;
            let inserted = 0;
            for (let index = 0; index < experimentMetrics.length; index += chunk) {
                const toSaveChunk = experimentMetrics.slice(index, index + chunk);
                if (index === 0) {
                    let difference = Object.keys(toSaveChunk[0]).filter((firstItemEntry) => !columns.includes(firstItemEntry));
                    if (difference.length >= 1) {
                        this.logger.warn('Undefined metrics: ' + difference.toString() + ' in hw_type: ' + hw_type + ' and subsystem: ' + subsystem);
                    }
                }
                try {
                    const saved = await repository.createQueryBuilder().insert().values(toSaveChunk).orIgnore().execute();
                    inserted += saved.generatedMaps.length;
                }
                catch (e) {
                    console.error(e);
                    console.error(toSaveChunk);
                }
            }
            if (manualAggregationTime) {
                await this.dataSource.createQueryBuilder().insert().into(aggregation_entity_1.Aggregation).values({
                    table: table,
                    time: new Date(manualAggregationTime)
                }).orIgnore().execute();
            }
            return inserted;
        }
        else {
            return 0;
        }
    }
    checkValue(value) {
        if (isNaN(value) == true || value.length < 1) {
            return null;
        }
        else {
            return this.noExponents(value);
        }
    }
    noExponents(number) {
        const data = String(number).split(/[eE]/);
        if (data.length == 1)
            return number;
        let z = '', sign = number < 0 ? '-' : '', str = data[0].replace('.', ''), mag = Number(data[1]) + 1;
        if (mag < 0) {
            z = sign + '0.';
            while (mag++)
                z += '0';
            return +(z + str.replace(/^\-/, ''));
        }
        mag -= str.length;
        while (mag--)
            z += '0';
        return +(str + z);
    }
};
exports.AppService = AppService;
exports.AppService = AppService = AppService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectEntityManager)()),
    __metadata("design:paramtypes", [typeorm_2.EntityManager])
], AppService);
//# sourceMappingURL=app.service.js.map