"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigurationController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const rawbody = require("raw-body");
const architecture_dto_1 = require("../common/architecture.dto");
const itemRelation_entity_1 = require("../entity/itemRelation.entity");
const configuration_service_1 = require("./configuration.service");
const app_service_1 = require("./app.service");
const hostCfgResponse_dto_1 = require("../common/hostCfgResponse.dto");
const xormon_exception_1 = require("../exception/xormon.exception");
const metricsTable_service_1 = require("../service/metrics/metricsTable.service");
const App_1 = require("../service/configuration/App");
let ConfigurationController = class ConfigurationController {
    constructor(configurationService, metricsTableService) {
        this.configurationService = configurationService;
        this.metricsTableService = metricsTableService;
        this.logger = new common_1.Logger(app_service_1.AppService.name);
    }
    async saveConfigrationJSON(data) {
        const hash = this.configurationService.hashCode(new Date().getTime().toString());
        let items = this.configurationService.transformObjToItemProperties(data);
        this.logger.log(`[${hash}] Incoming ${items.length} configuration json entries`);
        let inserted = 0;
        let errors = [];
        const chunk = 1000;
        for (let index = 0; index < items.length; index += chunk) {
            const toSaveChunk = items.slice(index, index + chunk);
            try {
                this.logger.log(`[${hash}] Saving ${toSaveChunk.length} configuration json entries`);
                inserted += await this.configurationService.saveItemProperties(items);
                this.logger.log(`[${hash}] Saved ${toSaveChunk.length} configuration json entries`);
            }
            catch (e) {
                const itemIds = items
                    .map((item) => item.item_id)
                    .filter(function (elem, index, self) {
                    return index === self.indexOf(elem);
                });
                const existedItemIds = await this.configurationService.getItemIds();
                const missing = itemIds.filter(item => existedItemIds.indexOf(item) < 0);
                if (missing.length >= 1) {
                    errors.push(`Missing item_id in object_items [IDs: ${missing.toString()}]`);
                    console.error(`[${hash}] Missing item_id in object_items [IDs: ${missing.toString()}]: ${e}`);
                }
                else {
                    errors.push(`Configuration store error [IDs: ${missing.toString()}]`);
                    console.error(`[${hash}] Configuration store error [IDs: ${missing.toString()}]: ${e}`);
                }
            }
        }
        if (errors.length >= 1) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: errors,
                error: 'error occurred while saving configurations data',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: 'saved ' + inserted + ' rows',
        };
    }
    async saveConfigrationCSV(req) {
        if (req.readable) {
            let items = [];
            const raw = await rawbody(req);
            const csv = raw.toString().trim().split('\n');
            const legend = csv[0].split(';');
            this.logger.log("Saving " + csv.length + " configuration csv lines");
            for (let i = 1; i < csv.length; i++) {
                const line = csv[i].split(';');
                for (let x = 1; x < line.length; x++) {
                    if (legend[x].length === 0) {
                        continue;
                    }
                    let itemProperty = {
                        item_id: line[0],
                        property_name: legend[x],
                        property_value: line[x],
                    };
                    items.push(itemProperty);
                }
            }
            let inserted = 0;
            try {
                inserted = await this.configurationService.saveItemProperties(items);
            }
            catch (error) {
                const itemIds = items
                    .map((item) => item.item_id)
                    .filter(function (elem, index, self) {
                    return index === self.indexOf(elem);
                });
                console.error('Configuration store error [IDs: ' + itemIds.toString() + ']: ' + error);
                throw new common_1.HttpException({
                    statusCode: common_1.HttpStatus.BAD_REQUEST,
                    message: 'bad data',
                    error: 'bad data',
                }, common_1.HttpStatus.BAD_REQUEST);
            }
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'saved ' + inserted + ' rows',
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'bad request body',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async saveArchitecture(architecture, agent = 0) {
        if (architecture && architecture.length >= 1) {
            this.logger.log("Saving " + architecture.length + " architecture entries");
            const now = new Date();
            let objectItems = [];
            let itemRelations = [];
            let agentMapping = [];
            let hostcfgRelations = [];
            let storageMapping = [];
            for (const arch of architecture) {
                objectItems.push({
                    item_id: arch.item_id,
                    label: arch.label,
                    object_id: arch.hw_type.toLowerCase(),
                    hw_type: arch.hw_type.toLowerCase(),
                    subsystem: arch.subsystem.toLowerCase(),
                    updatedAt: now,
                    deletedAt: null
                });
                if (arch.parents && arch.parents.length >= 1) {
                    for (const parent of arch.parents) {
                        itemRelations.push({
                            parent: parent,
                            child: arch.item_id,
                            origin: itemRelation_entity_1.RelationOrigin.child,
                            updatedAt: now,
                        });
                    }
                }
                if (arch.children && arch.children.length >= 1) {
                    for (const child of arch.children) {
                        itemRelations.push({
                            parent: arch.item_id,
                            child: child,
                            origin: itemRelation_entity_1.RelationOrigin.parent,
                            updatedAt: now,
                        });
                    }
                }
                if (arch.agent_mapping) {
                    agentMapping.push({
                        agent_id: arch.item_id,
                        item_id: arch.agent_mapping,
                    });
                }
                if (arch.hostcfg_id) {
                    hostcfgRelations.push({
                        hostcfg_id: arch.hostcfg_id,
                        item_id: arch.item_id,
                        updatedAt: now,
                    });
                }
                if (arch.wwn) {
                    storageMapping.push({
                        item_id: arch.item_id,
                        wwn: arch.wwn,
                    });
                }
                if (arch.wwns) {
                    for (const wwn of arch.wwns) {
                        storageMapping.push({
                            item_id: arch.item_id,
                            wwn: wwn,
                        });
                    }
                }
                if (arch.target_wwn) {
                    storageMapping.push({
                        item_id: arch.item_id,
                        wwn: arch.target_wwn,
                        target: true
                    });
                }
                if (arch.target_wwns) {
                    for (const wwn of arch.target_wwns) {
                        storageMapping.push({
                            item_id: arch.item_id,
                            wwn: wwn,
                            target: true
                        });
                    }
                }
            }
            const isDebug = await App_1.App.isDebug();
            let inserted = 0;
            let errors = [];
            await this.configurationService.saveArchitecture(objectItems)
                .then(output => inserted = output)
                .catch(e => {
                this.logger.error("[saveArchitecture] Error: " + e);
                if (e.message.includes("PK_4433d14fb839775f030587b5279")) {
                    const itemIds = objectItems.map(objectItem => objectItem.item_id);
                    const duplicates = itemIds.filter((item, index) => itemIds.indexOf(item) !== index);
                    let error = "Duplicates in architecture: " + duplicates.toString();
                    this.logger.error(error);
                    errors.push(`[Architecture] ${error}`);
                }
                else {
                    errors.push(`[Architecture] ${e.message}`);
                }
                if (isDebug) {
                    this.logger.error("[saveArchitecture] Input: " + JSON.stringify(objectItems, null, 2));
                }
            });
            await this.configurationService.saveItemRelations(itemRelations)
                .catch(e => {
                this.logger.error("[saveItemRelations] Error: " + e);
                if (e.message.includes("FK_e469d55961845e794cc353079d8") || e.message.includes("FK_142c8dd549586c659ab97ab8c07")) {
                    this.logger.error(e.detail);
                    errors.push(`[ItemRelations] ${e.detail}`);
                }
                else {
                    errors.push(`[ItemRelations] ${e.message}`);
                }
                if (isDebug) {
                    this.logger.error("[saveItemRelations] Input: " + JSON.stringify(itemRelations, null, 2));
                }
            });
            this.configurationService.saveAgentMapping(agentMapping)
                .catch(e => {
                this.logger.error("[saveAgentMapping] Error: " + e);
                errors.push(`[AgentMapping] ${e.message}`);
                if (isDebug) {
                    this.logger.error("[saveAgentMapping] Input: " + JSON.stringify(agentMapping, null, 2));
                }
            });
            this.configurationService.saveHostcfgRelations(hostcfgRelations)
                .catch(e => {
                this.logger.error("[saveHostcfgRelations] Error: " + e);
                errors.push(`[HostcfgRelation] ${e.message}`);
                if (isDebug) {
                    this.logger.error("[saveHostcfgRelations] Input: " + JSON.stringify(hostcfgRelations, null, 2));
                }
            });
            this.configurationService.saveStorageMapping(storageMapping)
                .catch(e => {
                this.logger.error("[saveStorageMapping] Error: " + e);
                errors.push(`[StorageMapping] ${e.message}`);
                if (isDebug) {
                    this.logger.error("[saveStorageMapping] Input: " + JSON.stringify(storageMapping, null, 2));
                }
            });
            if (errors.length >= 1) {
                throw new common_1.HttpException({
                    statusCode: common_1.HttpStatus.BAD_REQUEST,
                    message: errors,
                    error: 'error occurred while saving architecture data',
                }, common_1.HttpStatus.BAD_REQUEST);
            }
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'inserted ' + inserted + ' items',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("No architecture data", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async saveHostCfg(hostcfg, request) {
        const address = request.ip ? request.ip.split("f:")[1] : "unknown";
        const data = {
            sourceAddress: address,
            ...hostcfg.data
        };
        if (hostcfg.data.node1Host) {
            data.node1Host = hostcfg.data.node1Host;
        }
        if (hostcfg.data.node2Host) {
            data.node2Host = hostcfg.data.node2Host;
        }
        if (hostcfg.data.host) {
            data.host = hostcfg.data.host;
        }
        if (hostcfg.data.hosts && hostcfg.data.hosts.length > 0) {
            data.hosts = hostcfg.data.hosts;
        }
        const saved = await this.configurationService.saveHostCfg({
            hostcfg_id: hostcfg.hostcfg_id,
            label: hostcfg.label,
            target: true,
            hw_type: hostcfg.hw_type,
            data: data
        });
        if (saved.hostcfg_id) {
            this.metricsTableService.generate();
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'successfully saved',
                data: saved,
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while saving hostcfg", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteArchitecture(architecture) {
        if (architecture.hostcfg_id) {
            let affected = this.configurationService.deleteArchitecture(architecture.hostcfg_id);
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'architecture deleted successfully',
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'missing hostcfg_id in request body',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
};
exports.ConfigurationController = ConfigurationController;
__decorate([
    openapi.ApiOperation({ description: "stores configuration data from JSON format" }),
    (0, swagger_1.ApiBody)({
        schema: {
            type: 'object',
            example: {
                'f6969d67-453d-4337-970c-ccc72f5d8c43': {
                    name: 'testItem',
                    num_cpu: 6,
                    ip: '10.0.0.2',
                },
            },
        },
    }),
    (0, common_1.Post)('v1/properties'),
    openapi.ApiResponse({ status: 201, type: require("../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveConfigrationJSON", null);
__decorate([
    openapi.ApiOperation({ description: "stores configuration data from CSV format" }),
    (0, swagger_1.ApiBody)({
        schema: {
            type: 'string',
            example: 'uuid;name;num_cpu;ip;\n\
        f6969d67-453d-4337-970c-ccc72f5d8c43;testItem;6;10.0.0.2;\n\
        4aa4351f-3b05-420f-9060-5814740fe5ac;testItem2;6;10.0.0.3;',
        },
    }),
    (0, swagger_1.ApiHeader)({ name: 'Content-Type', description: 'text/plain' }),
    (0, common_1.Post)('v2/properties'),
    openapi.ApiResponse({ status: 201, type: require("../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveConfigrationCSV", null);
__decorate([
    openapi.ApiOperation({ description: "save architecture" }),
    (0, common_1.Post)('v1/architecture'),
    openapi.ApiResponse({ status: 201, type: require("../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __param(1, (0, common_1.Query)('agent')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array, Number]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveArchitecture", null);
__decorate([
    openapi.ApiOperation({ description: "Target destination HostCfg" }),
    (0, common_1.Post)('v1/hostcfg'),
    openapi.ApiResponse({ status: 201, type: Object }),
    __param(0, (0, common_1.Body)()),
    __param(1, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [hostCfgResponse_dto_1.TargetHostCfgDTO, Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveHostCfg", null);
__decorate([
    openapi.ApiOperation({ description: "Delete architecture relations by hostcfg_id\nWIP" }),
    (0, common_1.Delete)('v1/architecture'),
    openapi.ApiResponse({ status: 200, type: require("../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [architecture_dto_1.DeleteArchitecture]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "deleteArchitecture", null);
exports.ConfigurationController = ConfigurationController = __decorate([
    (0, swagger_1.ApiTags)('Configuration'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/configuration'),
    __metadata("design:paramtypes", [configuration_service_1.ConfigurationService,
        metricsTable_service_1.MetricsTableService])
], ConfigurationController);
//# sourceMappingURL=configuration.controller.js.map