"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigurationService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const typeorm_2 = require("typeorm");
const hostcfgRelation_entity_1 = require("../entity/hostcfgRelation.entity");
const itemRelation_entity_1 = require("../entity/itemRelation.entity");
const objectItem_entity_1 = require("../entity/objectItem.entity");
const agentMapping_entity_1 = require("../entity/agentMapping.entity");
const storageMapping_entity_1 = require("../entity/storageMapping.entity");
const itemProperty_entity_1 = require("../entity/itemProperty.entity");
const healthStatus_entity_1 = require("../entity/healthStatus.entity");
const heatmap_entity_1 = require("../entity/heatmap.entity");
const hostcfg_entity_1 = require("../entity/hostcfg.entity");
const top_entity_1 = require("../entity/top.entity");
let ConfigurationService = class ConfigurationService {
    constructor(hostcfgRepository, itemPropertiesRepository, objectItemsRepository, itemRelationsRepository, agentMappingRepository, storageMappingRepository, healthStatusRepository, hostcfgRelationsRepository, heatmapRepository, topRepository, dataSource) {
        this.hostcfgRepository = hostcfgRepository;
        this.itemPropertiesRepository = itemPropertiesRepository;
        this.objectItemsRepository = objectItemsRepository;
        this.itemRelationsRepository = itemRelationsRepository;
        this.agentMappingRepository = agentMappingRepository;
        this.storageMappingRepository = storageMappingRepository;
        this.healthStatusRepository = healthStatusRepository;
        this.hostcfgRelationsRepository = hostcfgRelationsRepository;
        this.heatmapRepository = heatmapRepository;
        this.topRepository = topRepository;
        this.dataSource = dataSource;
    }
    async saveItemProperties(items) {
        const insert = await this.itemPropertiesRepository.upsert(items, ['item_id', 'property_name']);
        return insert.generatedMaps.length;
    }
    async getItemIds() {
        return (await this.objectItemsRepository.find({ select: { item_id: true } })).map(item => item.item_id);
    }
    async saveArchitecture(objectItems) {
        const insert = await this.objectItemsRepository.save(objectItems);
        return insert.length;
    }
    async saveArchitectureAgent(objectItems) {
        let toSave = [];
        const items = await this.objectItemsRepository.findByIds(objectItems.map((item) => {
            return item.item_id;
        }));
        for (const index in objectItems) {
            let item = items.find((i) => i.item_id === objectItems[index]['item_id']);
            if (item && item.hw_type !== 'LINUX') {
                item.hasAgent = true;
                toSave.push(item);
            }
            else {
                toSave.push(objectItems[index]);
            }
        }
        const insert = await this.objectItemsRepository.save(toSave);
        return insert.length;
    }
    async saveAgentMapping(agentMapping) {
        const insert = await this.agentMappingRepository.save(agentMapping);
        return insert.length;
    }
    async saveHostcfgRelations(hostcfgRelations) {
        const insert = await this.dataSource.createQueryBuilder().insert().into(hostcfgRelation_entity_1.HostcfgRelation).values(hostcfgRelations).orIgnore().execute();
        return insert.generatedMaps.length;
    }
    async saveStorageMapping(storageMapping) {
        const insert = await this.storageMappingRepository.save(storageMapping);
        return insert.length;
    }
    async saveItemRelations(itemRelations) {
        const entityManager = this.dataSource;
        let num = await entityManager.transaction(async (transactionalEntityManager) => {
            const hourAgo = new Date(new Date().getTime() - (1000 * 60 * 60));
            const relationsByParent = itemRelations.filter(itemRelation => itemRelation.origin === 'parent');
            await transactionalEntityManager.getRepository(itemRelation_entity_1.ItemRelation).delete({
                parent: (0, typeorm_2.In)(relationsByParent.map(item => item.parent)),
                origin: itemRelation_entity_1.RelationOrigin.parent,
                updatedAt: (0, typeorm_2.LessThan)(hourAgo)
            });
            const relationsByChildren = itemRelations.filter(itemRelation => itemRelation.origin === 'child');
            await transactionalEntityManager.getRepository(itemRelation_entity_1.ItemRelation).delete({
                child: (0, typeorm_2.In)(relationsByChildren.map(item => item.child)),
                origin: itemRelation_entity_1.RelationOrigin.child,
                updatedAt: (0, typeorm_2.LessThan)(hourAgo)
            });
            const insert = await transactionalEntityManager.getRepository(itemRelation_entity_1.ItemRelation).save(itemRelations);
            return insert.length;
        });
        return num;
    }
    async deleteArchitecture(id) {
        let ids = await this.hostcfgRelationsRepository.find({ select: ["item_id"], where: { hostcfg_id: id } });
        let affected = 0;
        let hostsIds = await this.getArrayFromResponse(ids, "item_id");
        hostsIds.push(id);
        let idsToRemove = await this.gatherLineage(hostsIds);
        let itemRepChild = await this.itemRelationsRepository.delete({ child: (0, typeorm_2.In)(idsToRemove) });
        affected += itemRepChild.affected;
        let itemRepParent = await this.itemRelationsRepository.delete({ parent: (0, typeorm_2.In)(idsToRemove) });
        let hlthStts = await this.healthStatusRepository.delete({ item_id: (0, typeorm_2.In)(idsToRemove) });
        let objItms = await this.objectItemsRepository.delete({ item_id: (0, typeorm_2.In)(idsToRemove) });
        let htmp = await this.heatmapRepository.delete({ item_id: (0, typeorm_2.In)(idsToRemove) });
        affected = itemRepChild.affected + itemRepParent.affected + hlthStts.affected + objItms.affected + htmp.affected;
        return affected;
    }
    async gatherLineage(ids) {
        let collectedIds = ids;
        let newIdsbefore = await this.itemRelationsRepository.find({ select: ["child"], where: { parent: (0, typeorm_2.In)(ids) } });
        let newIds = await this.getArrayFromResponse(newIdsbefore, "child");
        if (newIds.length >= 1) {
            collectedIds = collectedIds.concat(await this.gatherLineage(newIds));
        }
        return collectedIds;
    }
    async getArrayFromResponse(response, elmntType) {
        let arr = [];
        for (const elmnt of response) {
            if (elmnt !== undefined) {
                if (elmnt[elmntType]) {
                    arr.push(elmnt[elmntType]);
                }
            }
        }
        return arr;
    }
    transformObjToItemProperties(data) {
        let items = [];
        for (const [item_id, properties] of Object.entries(data)) {
            for (const [property_name, property_value] of Object.entries(properties)) {
                items.push({
                    item_id: item_id,
                    property_name: property_name,
                    property_value: property_value
                });
            }
        }
        return items;
    }
    hashCode(s) {
        return Buffer.from(s).toString('base64').slice(0, 12);
    }
    async getHostCfgs(_class, hwType) {
        let conditions = {};
        if (_class) {
            conditions['hw_type_entity'] = {
                class: _class
            };
        }
        if (hwType) {
            conditions['hw_type'] = hwType;
        }
        return this.hostcfgRepository.find({
            where: conditions,
            relations: {
                hw_type_entity: true
            },
            order: {
                createdAt: 'ASC'
            }
        });
    }
    async saveHostCfg(hostcfg) {
        return await this.hostcfgRepository.save(hostcfg);
    }
    async filterProxyItems(items) {
        const ids = items.map(it => it.item_id);
        const itemsInDb = await this.objectItemsRepository.findBy({ item_id: (0, typeorm_2.In)(ids) });
        return items.filter((it => itemsInDb.find(itDb => itDb.item_id === it.item_id)));
    }
    async addHeatmap(items) {
        return await this.heatmapRepository.upsert(await this.filterProxyItems(items), ['item_id', 'metric']);
    }
    async addTop(items) {
        return await this.topRepository.upsert(await this.filterProxyItems(items), ['item_id', 'metric', 'interval']);
    }
};
exports.ConfigurationService = ConfigurationService;
exports.ConfigurationService = ConfigurationService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(hostcfg_entity_1.HostCfg)),
    __param(1, (0, typeorm_1.InjectRepository)(itemProperty_entity_1.ItemProperty)),
    __param(2, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(3, (0, typeorm_1.InjectRepository)(itemRelation_entity_1.ItemRelation)),
    __param(4, (0, typeorm_1.InjectRepository)(agentMapping_entity_1.AgentMapping)),
    __param(5, (0, typeorm_1.InjectRepository)(storageMapping_entity_1.StorageMapping)),
    __param(6, (0, typeorm_1.InjectRepository)(healthStatus_entity_1.HealthStatus)),
    __param(7, (0, typeorm_1.InjectRepository)(hostcfgRelation_entity_1.HostcfgRelation)),
    __param(8, (0, typeorm_1.InjectRepository)(heatmap_entity_1.Heatmap)),
    __param(9, (0, typeorm_1.InjectRepository)(top_entity_1.Top)),
    __param(10, (0, typeorm_1.InjectEntityManager)()),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.EntityManager])
], ConfigurationService);
//# sourceMappingURL=configuration.service.js.map