"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HealthStatusMirrorService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const consistencyGroup_entity_1 = require("../entity/consistencyGroup.entity");
const healthStatusMirror_entity_1 = require("../entity/healthStatusMirror.entity");
const healthStatusMirrorRel_entity_1 = require("../entity/healthStatusMirrorRel.entity");
const typeorm_2 = require("typeorm");
const dataSource_1 = require("./dataSource");
let HealthStatusMirrorService = class HealthStatusMirrorService {
    constructor(healthStatusMirrorRepository, healthStatusMirrorRelRepository, consistencyGroupRepository, dataSource) {
        this.healthStatusMirrorRepository = healthStatusMirrorRepository;
        this.healthStatusMirrorRelRepository = healthStatusMirrorRelRepository;
        this.consistencyGroupRepository = consistencyGroupRepository;
        this.dataSource = dataSource;
    }
    ;
    getConsistencyGroup(id) {
        return this.consistencyGroupRepository.findOne({
            where: {
                consistency_group_id: id
            },
            relations: {
                mirrors: true
            }
        });
    }
    async getMirrorsByConsistencyGroup() {
        const cgs = await this.consistencyGroupRepository.find({
            relations: {
                mirrors: {
                    _sourceMirror: true,
                    _targetMirror: true
                }
            },
        });
        const map = {
            cgMirrors: {},
            noCgMirrors: []
        };
        for (const group of cgs) {
            if (group.mirrors && group.mirrors.length) {
                map.cgMirrors[group.consistency_group_id + group.label] = {
                    label: group.label,
                    id: group.consistency_group_id,
                    mirrors: group.mirrors.map(m => {
                        return {
                            source_vol_id: m.source_vol_id,
                            target_vol_id: m.target_vol_id,
                            source_item_id: m._sourceMirror.item_id,
                            target_item_id: m._targetMirror ? m._targetMirror.item_id : undefined,
                            type_state: m.type_state,
                            severity: m.severity,
                            system_name: m.system_name,
                            remote_system_name: m.remote_system_name,
                            freeze_time: m.freeze_time,
                            progress: m.progress
                        };
                    })
                };
            }
            else {
            }
        }
        return map;
    }
    insertConsistencyGroup(consistencyGroup) {
        return this.consistencyGroupRepository.save(consistencyGroup);
    }
    updateHealthStatusMirror(healthStatusMirror) {
        return this.healthStatusMirrorRepository.save(healthStatusMirror);
    }
    async getMirros() {
        const mirrors = await this.healthStatusMirrorRelRepository.find({
            relations: {
                _sourceMirror: true,
                _targetMirror: true,
                _consistency_group: true
            }
        });
        return mirrors.map((m) => {
            return {
                source_vol_id: m.source_vol_id,
                target_vol_id: m.target_vol_id,
                source_item_id: m._sourceMirror.item_id,
                target_item_id: m._targetMirror ? m._targetMirror.item_id : undefined,
                type_state: m.type_state,
                severity: m.severity,
                system_name: m.system_name,
                remote_system_name: m.remote_system_name,
                freeze_time: m.freeze_time,
                progress: m.progress,
                _consistency_group: m._consistency_group ? {
                    consistency_group_id: m._consistency_group.consistency_group_id,
                    label: m._consistency_group.label,
                    mode: m._consistency_group.mode,
                    status: m._consistency_group.status,
                    syncing_lvl: m._consistency_group.syncing_lvl,
                    role: m._consistency_group.role
                } : undefined
            };
        });
    }
    async insertHealthStatusMirror(healthStatusMirror, cgs) {
        const queryRunner = (0, dataSource_1.getDataSource)().createQueryRunner();
        await queryRunner.connect();
        await queryRunner.startTransaction();
        const mirrorRepo = queryRunner.manager.getRepository(healthStatusMirror_entity_1.HealthStatusMirror);
        const mirrorRelRepo = queryRunner.manager.getRepository(healthStatusMirrorRel_entity_1.HealthStatusMirrorRel);
        const consistencyGroupRepo = queryRunner.manager.getRepository(consistencyGroup_entity_1.ConsistencyGroup);
        try {
            const mirrors = [];
            const mirrorRels = [];
            const allRels = await mirrorRelRepo.find();
            const map = {};
            for (const rel of allRels) {
                const key = rel.source_vol_id + rel.target_vol_id + rel.remote_system_name;
                map[key] = rel;
            }
            for (const val of healthStatusMirror) {
                const keyTarget = val.target_vol_id + val.source_vol_id + val.system_name;
                const keySource = val.source_vol_id + val.target_vol_id + val.remote_system_name;
                if (map[keyTarget]) {
                    if (!map[keyTarget].target_hostcfg_id) {
                        map[keyTarget].target_hostcfg_id = val.source_hostcfg_id;
                        mirrors.push({
                            hostcfg_id: val.source_hostcfg_id,
                            vol_id: val.source_vol_id,
                            item_id: val.item_id
                        });
                    }
                    map[keyTarget].consistency_group_id = val.consistency_group_id;
                    map[keyTarget].consistency_group_label = val.consistency_group_label;
                }
                else if (map[keySource]) {
                    map[keySource] = {
                        ...val,
                        target_hostcfg_id: map[keySource].target_hostcfg_id ? map[keySource].target_hostcfg_id : undefined
                    };
                }
                else {
                    map[keySource] = {
                        ...val
                    };
                    mirrors.push({
                        hostcfg_id: val.source_hostcfg_id,
                        vol_id: val.source_vol_id,
                        item_id: val.item_id
                    });
                }
            }
            for (const key in map) {
                mirrorRels.push(map[key]);
            }
            cgs ? await consistencyGroupRepo.save(cgs) : null;
            await mirrorRepo.save(mirrors);
            await mirrorRelRepo.save(mirrorRels);
            await queryRunner.commitTransaction();
        }
        catch (error) {
            console.error(error);
            await queryRunner.rollbackTransaction();
            await queryRunner.release();
            throw error;
        }
        finally {
            await queryRunner.release();
        }
    }
    deleteHealthStatusMirror(mirror) {
        return this.healthStatusMirrorRepository.remove(mirror);
    }
    async deleteConsistencyGroup(id) {
        const entityManager = this.dataSource;
        return await entityManager.transaction(async (transactionalEntityManager) => {
            const updated = await transactionalEntityManager.getRepository(healthStatusMirror_entity_1.HealthStatusMirror)
                .createQueryBuilder()
                .update(healthStatusMirrorRel_entity_1.HealthStatusMirrorRel)
                .set({
                consistency_group_id: null
            })
                .where("consistency_group_id = :id", { id: id })
                .execute();
            const deleted = await transactionalEntityManager.getRepository(consistencyGroup_entity_1.ConsistencyGroup)
                .delete({ consistency_group_id: id });
            return deleted.affected ? true : false;
        });
    }
};
exports.HealthStatusMirrorService = HealthStatusMirrorService;
exports.HealthStatusMirrorService = HealthStatusMirrorService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(healthStatusMirror_entity_1.HealthStatusMirror)),
    __param(1, (0, typeorm_1.InjectRepository)(healthStatusMirrorRel_entity_1.HealthStatusMirrorRel)),
    __param(2, (0, typeorm_1.InjectRepository)(consistencyGroup_entity_1.ConsistencyGroup)),
    __param(3, (0, typeorm_1.InjectEntityManager)()),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.EntityManager])
], HealthStatusMirrorService);
//# sourceMappingURL=healtStatusMirror.service.js.map