"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var HealthStatusController_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.HealthStatusController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const healthStatus_service_1 = require("./healthStatus.service");
const swagger_1 = require("@nestjs/swagger");
const healthStatus_entity_1 = require("../entity/healthStatus.entity");
const xormon_exception_1 = require("../exception/xormon.exception");
const App_1 = require("../service/configuration/App");
const PM2_1 = require("../service/configuration/PM2");
const taskQueue_constants_1 = require("../common/constants/taskQueue.constants");
let HealthStatusController = HealthStatusController_1 = class HealthStatusController {
    constructor(healthStatusService) {
        this.healthStatusService = healthStatusService;
        this.logger = new common_1.Logger(HealthStatusController_1.name);
    }
    async saveHealthStatus(healthStatus) {
        const isDebug = await App_1.App.isDebug();
        let inserted = 0;
        try {
            inserted = await this.healthStatusService.saveHealthStatus(healthStatus.map(hs => {
                return {
                    ...hs,
                    type: hs.type ? hs.type : healthStatus_entity_1.StatusType.HEALTH,
                    data: hs.data ? hs.data : null,
                    updatedAt: new Date()
                };
            }));
        }
        catch (e) {
            this.logger.error("[saveHealthStatus] Error: " + e);
            if (e.message.includes("FK_99e3a748292ea5203b62e0b69b6")) {
                this.logger.error(e.detail);
            }
            if (isDebug) {
                this.logger.error("[saveHealthStatus] Input: " + JSON.stringify(healthStatus, null, 2));
            }
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'an error occurred while saving health status',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        PM2_1.PM2.sendMessage('tasks-server', {
            action: "insert-task",
            taskType: taskQueue_constants_1.TaskTypes.HEALT_STATUS,
            taskPayload: healthStatus.map(hs => {
                return {
                    ...hs,
                    type: hs.type ? hs.type : healthStatus_entity_1.StatusType.HEALTH,
                    data: hs.data ? hs.data : null,
                };
            }),
            number_of_items: healthStatus.length
        });
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'saved ' + inserted + ' items',
        };
    }
    async saveStatus(hostcfgStatus) {
        let healthstatus = [];
        if (hostcfgStatus.connection !== undefined) {
            if (hostcfgStatus.hwType) {
                await this.healthStatusService.propagateParents(hostcfgStatus);
            }
            healthstatus.push({
                item_id: hostcfgStatus.hostcfg_id,
                type: healthStatus_entity_1.StatusType.CONNECTION,
                status: hostcfgStatus.connection ? healthStatus_entity_1.Status.OK : healthStatus_entity_1.Status.ERROR,
                data: hostcfgStatus.message,
            });
        }
        if (hostcfgStatus.data !== undefined) {
            healthstatus.push({
                item_id: hostcfgStatus.hostcfg_id,
                type: healthStatus_entity_1.StatusType.DATA,
                status: hostcfgStatus.data ? healthStatus_entity_1.Status.OK : healthStatus_entity_1.Status.ERROR,
                data: hostcfgStatus.message,
            });
        }
        const saved = await this.saveHealthStatus(healthstatus);
        if (!saved) {
            throw new xormon_exception_1.XormonException("An error occurred while saving the hostcfg status", common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: "Successfully saved status"
        };
    }
};
exports.HealthStatusController = HealthStatusController;
__decorate([
    openapi.ApiOperation({ description: "save health status" }),
    (0, common_1.Post)('v1/store'),
    openapi.ApiResponse({ status: 201, type: require("../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", Promise)
], HealthStatusController.prototype, "saveHealthStatus", null);
__decorate([
    (0, common_1.Post)(['v1/datatest', 'v1/conntest']),
    openapi.ApiResponse({ status: 201, type: require("../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], HealthStatusController.prototype, "saveStatus", null);
exports.HealthStatusController = HealthStatusController = HealthStatusController_1 = __decorate([
    (0, swagger_1.ApiTags)('Health Status Metrics Server'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/health_status'),
    __metadata("design:paramtypes", [healthStatus_service_1.HealthStatusService])
], HealthStatusController);
//# sourceMappingURL=healthStatus.controller.js.map