"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var ACLCheck_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ACLCheck = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const aclResponse_constants_1 = require("../../common/constants/aclResponse.constants");
const menuResponse_dto_1 = require("../../common/menuResponse.dto");
const dataSource_1 = require("../../config/dataSource");
const acl_entity_1 = require("../../entity/acl.entity");
const class_entity_1 = require("../../entity/class.entity");
const itemRelation_entity_1 = require("../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const Architecture_1 = require("../architecture/Architecture");
const User_1 = require("../users/User");
const typeorm_2 = require("typeorm");
let ACLCheck = ACLCheck_1 = class ACLCheck {
    static async getUserGroupsFromRequest(request) {
        const user = request ? await User_1.User.getUserByRequest(request, true) : null;
        return user.groups.map((group) => group.group_id);
    }
    static async getItemsChildren(itemIds) {
        const itemsWithChildren = {};
        const items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                item_id: (0, typeorm_2.In)(itemIds)
            },
            relations: {
                children: true
            }
        });
        for (const item of items) {
            itemsWithChildren[item.item_id] = item.children.map(relation => relation.child);
        }
        return itemsWithChildren;
    }
    static async findParents2(item_ids) {
        let dbArray = '(\'\', ';
        for (const [i, value] of item_ids.entries()) {
            if (i === item_ids.length - 1) {
                dbArray += "'" + value + "')";
            }
            else {
                dbArray += "'" + value + "',";
            }
        }
        const parents = await (0, dataSource_1.getDataSource)().query('select ir.child as child, oi.item_id as parent, oi.label, oi.subsystem from item_relations ir inner join object_items oi on ir.parent = oi.item_id WHERE ir.child IN ' +
            dbArray);
        let parentDict = {};
        for (const parent of parents) {
            if (parentDict[parent.child]) {
                parentDict[parent.child].push({
                    item_id: parent.parent,
                    label: parent.label,
                    subsystem: parent.subsystem
                });
            }
            else {
                parentDict[parent.child] = [];
                parentDict[parent.child].push({
                    item_id: parent.parent,
                    label: parent.label,
                    subsystem: parent.subsystem
                });
            }
        }
        return parentDict;
    }
    static async getItemWithParents(item_id) {
        return await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findOne({
            where: {
                item_id: item_id
            },
            relations: {
                parents: {
                    _parent: true
                }
            },
            order: {
                label: 'ASC'
            }
        });
    }
    static async getItemWithChildren(item_id) {
        return await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findOne({
            where: {
                item_id: item_id
            },
            relations: {
                children: {
                    _child: true
                }
            },
            order: {
                label: 'ASC'
            }
        });
    }
    static async findParents(uuid, level = 0) {
        if (level >= this.maxLevel) {
            return {};
        }
        else {
            level++;
        }
        const objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
        let parents = {};
        const items = await objectItemsRepository.findBy({ child: uuid });
        for (const item of items) {
            const itemParents = await this.findParents(item.parent, level);
            if (Object.keys(itemParents).length >= 1) {
                let parent = {};
                parent[item.parent] = itemParents;
                parents = Object.assign(parents, parent);
            }
            else {
                parents[item.parent] = {};
            }
        }
        return parents;
    }
    static async findChildren(uuid, level = 0) {
        if (level >= this.maxLevel) {
            return {};
        }
        else {
            level++;
        }
        const objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
        let children = {};
        const items = await objectItemsRepository.findBy({ parent: uuid });
        for (const item of items) {
            const itemChildren = await this.findChildren(item.child, level);
            if (Object.keys(itemChildren).length >= 1) {
                console.log(itemChildren);
                let child = {};
                child[item.child] = itemChildren;
                children = Object.assign(children, child);
            }
            else {
                children[item.child] = {};
            }
        }
        console.log(children);
        return children;
    }
    static async filterItemsByEnabledSubsys(itemIds) {
        const objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        const items = await objectItemsRepository.findBy({ item_id: (0, typeorm_2.In)(itemIds) });
        let filteredIds = [];
        for (let item of items) {
            if (this.enabledSubsystems[item.hw_type]) {
                if (this.enabledSubsystems[item.hw_type].includes(item.subsystem)) {
                    filteredIds.push(item.item_id);
                }
            }
            else {
                filteredIds.push(item.item_id);
            }
        }
        return filteredIds;
    }
    static isEverythingGranted(groupIds) {
        if (groupIds.includes(1)) {
            return true;
        }
        else {
            return false;
        }
    }
    static async filterItems2(groupIds, itemIds, hwType, subsys) {
        let items = [];
        if (this.isEverythingGranted(groupIds)) {
            return itemIds;
        }
        if (hwType) {
            const hwTypeStatus = await this.getHwTypeStatus(groupIds, hwType, false);
            if (hwTypeStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                return itemIds;
            }
            else if (hwTypeStatus === aclResponse_constants_1.ITEM_STATUS.denied) {
                return [];
            }
        }
        if (hwType && subsys) {
            const subsysStatus = await this.getSubsysStatus(groupIds, hwType, subsys, [], false);
            if (subsysStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                return itemIds;
            }
        }
        const aclRepository = (0, dataSource_1.getDataSource)().getRepository(acl_entity_1.ACL);
        const itemRelationRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
        const rule = hwType
            ? await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: (0, typeorm_2.In)([aclResponse_constants_1.RULE_TYPE.SUBSYS, aclResponse_constants_1.RULE_TYPE.ID]), value0: hwType } })
            : await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: (0, typeorm_2.In)([aclResponse_constants_1.RULE_TYPE.SUBSYS, aclResponse_constants_1.RULE_TYPE.ID]) } });
        if (rule && rule.length > 0) {
            for (let r of rule) {
                if (r.type === aclResponse_constants_1.RULE_TYPE.ID && itemIds.includes(r.value1)) {
                    if (items.indexOf(r.value1) === -1) {
                        items.push(r.value1);
                    }
                }
                const parentId = (r.type === aclResponse_constants_1.RULE_TYPE.ID) ? r.value1 : r.value2;
                const children = await itemRelationRepository.findBy({ parent: parentId, child: (0, typeorm_2.In)(itemIds) });
                for (let child of children) {
                    const childId = child.child;
                    if (items.indexOf(childId) === -1) {
                        if (r.type === aclResponse_constants_1.RULE_TYPE.ID) {
                            items.push(childId);
                        }
                        else if (r.type === aclResponse_constants_1.RULE_TYPE.SUBSYS) {
                            const childInfo = await this.getItemClassification(childId);
                            if (r.value1 === childInfo.subsystem) {
                                items.push(childId);
                            }
                        }
                    }
                }
            }
        }
        return items;
    }
    static async isAnySubsysItemGranted(request, subsys, hwType, _class, strict = true) {
        const groupIds = await this.getUserGroupsFromRequest(request);
        if (this.isEverythingGranted(groupIds)) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        let hwTypes = [];
        if (hwType) {
            hwTypes.push(hwType);
        }
        else if (_class) {
            const classHwTypes = await Architecture_1.Architecture.getHwTypes(false, _class);
            hwTypes = classHwTypes.map((hwType) => hwType.hw_type);
        }
        const objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        for (const hwType of hwTypes) {
            const hwTypeStatus = await this.getHwTypeStatus(groupIds, hwType, false);
            if (hwTypeStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                return aclResponse_constants_1.ITEM_STATUS.granted;
            }
            else if (hwTypeStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
                const items = await objectItemsRepository.find({ where: { hw_type: hwType, subsystem: subsys } });
                if (items && items.length > 0) {
                    const ids = await this.filterItems(groupIds, items.map((item) => item.item_id), hwType, subsys);
                    if (ids && ids.length > 0) {
                        return aclResponse_constants_1.ITEM_STATUS.granted;
                    }
                    else if (!strict) {
                        for (const item of items) {
                            const itemStatus = await this.getItemStatus(groupIds, item.item_id, false);
                            if (itemStatus !== aclResponse_constants_1.ITEM_STATUS.denied) {
                                return itemStatus;
                            }
                        }
                    }
                }
            }
        }
        return aclResponse_constants_1.ITEM_STATUS.denied;
    }
    static async filterObjectItemsWithRequest(request, items, hwType, subsys) {
        const groupIds = await this.getUserGroupsFromRequest(request);
        const ids = await ACLCheck_1.filterItems(groupIds, items.map((item) => item.item_id), hwType, subsys);
        const filteredItems = items.filter(item => ids.includes(item.item_id));
        return filteredItems;
    }
    static async filterObjectItemsWithGroups(groupIds, items, hwType, subsys) {
        const ids = await ACLCheck_1.filterItems(groupIds, items.map((item) => item.item_id), hwType, subsys);
        const filteredItems = items.filter(item => ids.includes(item.item_id));
        return filteredItems;
    }
    static async filterItemsWithRequest(request, itemIds, hwType, subsys) {
        const groupIds = await this.getUserGroupsFromRequest(request);
        return await this.filterItems(groupIds, itemIds, hwType, subsys);
    }
    static async filterItems(groupIds, itemIds, hwType, subsys) {
        let items = [];
        if (this.isEverythingGranted(groupIds)) {
            return itemIds;
        }
        const aclRepository = (0, dataSource_1.getDataSource)().getRepository(acl_entity_1.ACL);
        const itemRelationRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
        const objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        if (hwType) {
            const hwTypeStatus = await this.getHwTypeStatus(groupIds, hwType, false);
            if (hwTypeStatus === aclResponse_constants_1.ITEM_STATUS.granted)
                return itemIds;
            if (hwTypeStatus === aclResponse_constants_1.ITEM_STATUS.denied)
                return [];
        }
        const objectItems = await objectItemsRepository.find({ relations: { hw_type_entity: true }, where: { item_id: (0, typeorm_2.In)(itemIds) } });
        let hwTypes = objectItems.map(item => item.hw_type);
        hwTypes = hwTypes.filter((n, i) => hwTypes.indexOf(n) === i);
        let classes = objectItems.map(item => item.hw_type_entity.class);
        classes = classes.filter((n, i) => classes.indexOf(n) === i);
        const ruleClass = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.CLASS, value0: (0, typeorm_2.In)(classes) } });
        const foundClasses = ruleClass.map(item => item.value0);
        const ruleHwType = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.HWTYPE, value0: (0, typeorm_2.In)(hwTypes) } });
        const foundHwTypes = ruleHwType.map(item => item.value0);
        objectItems.slice(0).forEach((item, index) => {
            if ((foundClasses.indexOf(item.hw_type_entity.class) !== -1) || (foundHwTypes.indexOf(item.hw_type) !== -1)) {
                items.push(item.item_id);
                objectItems.splice(index, 1);
            }
        });
        if (objectItems.length < 1)
            return items;
        let subsystems = objectItems.map(item => item.subsystem);
        subsystems = subsystems.filter((n, i) => subsystems.indexOf(n) === i);
        const ruleSubsys = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.SUBSYS, value0: (0, typeorm_2.In)(hwTypes), value1: (0, typeorm_2.In)(subsystems), value2: (0, typeorm_2.IsNull)() } });
        for (const rule of ruleSubsys) {
            objectItems.slice(0).forEach((item, index) => {
                if (item.hw_type === rule.value0 && item.subsystem === rule.value1) {
                    items.push(item.item_id);
                    objectItems.splice(index, 1);
                }
            });
        }
        if (objectItems.length < 1)
            return items;
        let remainingIds = objectItems.map(item => item.item_id);
        const ruleId = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.ID, value0: (0, typeorm_2.In)(hwTypes), value1: (0, typeorm_2.In)(remainingIds) } });
        const foundIds = ruleId.map(item => item.value1);
        objectItems.slice(0).forEach((item, index) => {
            if (foundIds.indexOf(item.item_id) !== -1) {
                items.push(item.item_id);
                objectItems.splice(index, 1);
            }
        });
        if (objectItems.length < 1)
            return items;
        remainingIds = objectItems.map(item => item.item_id);
        const parents = await itemRelationRepository.find({ where: { child: (0, typeorm_2.In)(remainingIds) } });
        const parentIds = parents.map(item => item.parent);
        const ruleParentId = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.ID, value0: (0, typeorm_2.In)(hwTypes), value1: (0, typeorm_2.In)(parentIds) } });
        for (const rule of ruleParentId) {
            objectItems.slice(0).forEach((item, index) => {
                if (item.hw_type === rule.value0) {
                    if (parents.find(relation => {
                        return (relation.parent === rule.value1 && relation.child === item.item_id);
                    })) {
                        items.push(item.item_id);
                        objectItems.splice(index, 1);
                    }
                }
            });
        }
        if (objectItems.length < 1)
            return items;
        const ruleParentSubsys = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.SUBSYS, value0: (0, typeorm_2.In)(hwTypes), value2: (0, typeorm_2.In)(parentIds) } });
        for (const rule of ruleParentSubsys) {
            objectItems.slice(0).forEach((item, index) => {
                if (item.hw_type === rule.value0 && item.subsystem.includes(rule.value1)) {
                    if (parents.find(relation => {
                        return (relation.parent === rule.value2 && relation.child === item.item_id);
                    })) {
                        items.push(item.item_id);
                        objectItems.splice(index, 1);
                    }
                }
            });
        }
        return items;
    }
    static async getItemClassification(itemId) {
        const item = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findOne({
            relations: {
                hw_type_entity: true
            },
            where: {
                item_id: itemId
            }
        });
        if (!item) {
            return null;
        }
        return {
            class: item.hw_type_entity.class,
            hwType: item.hw_type,
            subsystem: item.subsystem,
            itemId: itemId
        };
    }
    static async parseLinkUrl(url) {
        console.log("DEBUG PARSE URL %s", url);
        const path = url.replace(menuResponse_dto_1.MenuLink.API_URL_PAGE, '');
        const tmpPath = path.split('/');
        const hwType = tmpPath[0];
        let _class;
        if (hwType === 'san' || hwType === 'lan') {
            _class = hwType;
        }
        else {
            const classEntry = await Architecture_1.Architecture.getHwType(hwType);
            _class = classEntry.class.toString();
        }
        const subsystem = (tmpPath.length > 1) ? tmpPath[1] : null;
        const tmpUrl = new URL('https://do.not.care' + url);
        const itemId = tmpUrl.searchParams.get('id');
        const hwTypeParam = tmpUrl.searchParams.get('hw_type');
        return {
            class: _class,
            hwType: hwType,
            subsystem: subsystem,
            itemId: itemId
        };
    }
    static async getClassStatus(groupIds, _class, strict = true) {
        if (this.isEverythingGranted(groupIds)) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        const aclRepository = (0, dataSource_1.getDataSource)().getRepository(acl_entity_1.ACL);
        const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.CLASS, value0: _class } });
        if (rule && rule.length > 0) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        if (!strict) {
            const hwTypes = (await Architecture_1.Architecture.getHwTypes(false, _class)).map(hwType => hwType.hw_type);
            const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, value0: (0, typeorm_2.In)(hwTypes) } });
            if (rule && rule.length > 0) {
                return aclResponse_constants_1.ITEM_STATUS.partial;
            }
        }
        return aclResponse_constants_1.ITEM_STATUS.denied;
    }
    static async getHwTypeStatus(groupIds, hwType, strict = true) {
        if (this.isEverythingGranted(groupIds)) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        const aclRepository = (0, dataSource_1.getDataSource)().getRepository(acl_entity_1.ACL);
        const itemClassEntry = await Architecture_1.Architecture.getHwType(hwType);
        if (!itemClassEntry) {
            return aclResponse_constants_1.ITEM_STATUS.denied;
        }
        const itemClass = itemClassEntry.class.toString();
        const classStatus = await this.getClassStatus(groupIds, itemClass);
        if (classStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.HWTYPE, value0: hwType } });
        if (rule && rule.length > 0) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        if (!strict) {
            const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, value0: hwType } });
            if (rule && rule.length > 0) {
                return aclResponse_constants_1.ITEM_STATUS.partial;
            }
        }
        return aclResponse_constants_1.ITEM_STATUS.denied;
    }
    static async getSubsysStatus(groupIds, hwType, subsys, parentIds, strict) {
        if (this.isEverythingGranted(groupIds)) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        const aclRepository = (0, dataSource_1.getDataSource)().getRepository(acl_entity_1.ACL);
        const hwTypeStatus = await this.getHwTypeStatus(groupIds, hwType, true);
        if (hwTypeStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.SUBSYS, value0: hwType, value1: subsys } });
        if (rule && rule.length > 0) {
            for (let r of rule) {
                if (r.value2) {
                    if (parentIds && parentIds.length > 0) {
                        const rParentIds = r.value2.split(',');
                        const match = rParentIds.every(val => parentIds.includes(val));
                        if (match) {
                            return aclResponse_constants_1.ITEM_STATUS.granted;
                        }
                    }
                }
                else if (!parentIds || parentIds.length == 0) {
                    return aclResponse_constants_1.ITEM_STATUS.granted;
                }
            }
        }
        if (parentIds && parentIds.length > 0) {
            for (let parentId of parentIds) {
                const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.ID, value0: hwType, value1: parentId } });
                if (rule && rule.length > 0) {
                    return aclResponse_constants_1.ITEM_STATUS.granted;
                }
                const parentItem = await this.getItemClassification(parentId);
                if (parentItem) {
                    if (this.enabledSubsystems[hwType] && !this.enabledSubsystems[hwType].includes(parentItem.subsystem)) {
                        continue;
                    }
                    const itemStatus = await this.getItemStatus(groupIds, parentId, false);
                    if (itemStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                        return aclResponse_constants_1.ITEM_STATUS.granted;
                    }
                    if (parentItem.subsystem === subsys) {
                        if (itemStatus !== aclResponse_constants_1.ITEM_STATUS.partial) {
                            return itemStatus;
                        }
                    }
                    const partParentIds = parentIds.filter((id) => { return id !== parentId; });
                    const subsysStatus = await this.getSubsysStatus(groupIds, hwType, parentItem.subsystem, partParentIds, true);
                    if (subsysStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                        return aclResponse_constants_1.ITEM_STATUS.granted;
                    }
                }
            }
        }
        if (!strict) {
            if (this.enabledSubsystems[hwType] && !this.enabledSubsystems[hwType].includes(subsys)) {
                return aclResponse_constants_1.ITEM_STATUS.denied;
            }
            const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, value0: hwType } });
            if (rule && rule.length > 0) {
                for (let r of rule) {
                    let childId;
                    let childSubsys;
                    if (r.type === aclResponse_constants_1.RULE_TYPE.ID) {
                        childId = r.value1;
                    }
                    else if (r.type === aclResponse_constants_1.RULE_TYPE.SUBSYS) {
                        childSubsys = r.value1;
                        if (r.value2) {
                            childId = r.value2;
                        }
                        else {
                            if ((hwType == 'power') && (r.value1 == 'vm')) {
                                return aclResponse_constants_1.ITEM_STATUS.partial;
                            }
                        }
                    }
                    else {
                        continue;
                    }
                    if (childId) {
                        if (!parentIds || parentIds.length == 0) {
                            const childItem = await this.getItemClassification(childId);
                            if (childItem.subsystem === subsys) {
                                return aclResponse_constants_1.ITEM_STATUS.partial;
                            }
                        }
                        if (childSubsys && parentIds && parentIds.length == 1) {
                            if (childId === parentIds[0]) {
                                if (this.enabledSubsystems[hwType] && this.enabledSubsystems[hwType].includes(childSubsys)) {
                                    return aclResponse_constants_1.ITEM_STATUS.partial;
                                }
                            }
                        }
                        const childWithParents = await this.getItemWithParents(childId);
                        if (childWithParents && childWithParents.parents.length > 0) {
                            const childsParentIds = childWithParents.parents.map(item => item.parent);
                            if (parentIds && parentIds.length > 0) {
                                for (let parentId of parentIds) {
                                    if (childsParentIds.includes(parentId)) {
                                        return aclResponse_constants_1.ITEM_STATUS.partial;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return aclResponse_constants_1.ITEM_STATUS.denied;
    }
    static async getItemStatus(groupIds, itemId, strict) {
        if (this.isEverythingGranted(groupIds)) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        const aclRepository = (0, dataSource_1.getDataSource)().getRepository(acl_entity_1.ACL);
        const item = await this.getItemClassification(itemId);
        if (!item) {
            return aclResponse_constants_1.ITEM_STATUS.denied;
        }
        const hwTypeStatus = await this.getHwTypeStatus(groupIds, item.hwType, true);
        if (hwTypeStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        const ruleId = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.ID, value0: item.hwType, value1: itemId } });
        if (ruleId && ruleId.length > 0) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        const subsysStatus = await this.getSubsysStatus(groupIds, item.hwType, item.subsystem, [], true);
        if (subsysStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        const itemWithParents = await this.getItemWithParents(itemId);
        if (itemWithParents && itemWithParents.parents.length > 0) {
            const parentIds = itemWithParents.parents.map(item => item.parent);
            for (let parentId of parentIds) {
                const subsysStatus = await this.getSubsysStatus(groupIds, item.hwType, item.subsystem, [parentId], true);
                if (subsysStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                    return aclResponse_constants_1.ITEM_STATUS.granted;
                }
                const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.ID, value0: item.hwType, value1: parentId } });
                if (rule && rule.length > 0) {
                    return aclResponse_constants_1.ITEM_STATUS.granted;
                }
            }
        }
        if (!strict) {
            if (this.enabledSubsystems[item.hwType] && !this.enabledSubsystems[item.hwType].includes(item.subsystem)) {
                return aclResponse_constants_1.ITEM_STATUS.denied;
            }
            const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, value0: item.hwType } });
            if (rule && rule.length > 0) {
                for (let r of rule) {
                    let childId;
                    let childSubsys;
                    if (r.type === aclResponse_constants_1.RULE_TYPE.ID) {
                        childId = r.value1;
                    }
                    else if (r.type === aclResponse_constants_1.RULE_TYPE.SUBSYS) {
                        childSubsys = r.value1;
                        if (r.value2) {
                            childId = r.value2;
                        }
                        else {
                            if ((item.hwType == 'power') && (r.value1 == 'vm')) {
                                return aclResponse_constants_1.ITEM_STATUS.partial;
                            }
                        }
                    }
                    else {
                        continue;
                    }
                    if (childSubsys) {
                        if (childId === itemId) {
                            if (this.enabledSubsystems[item.hwType] && this.enabledSubsystems[item.hwType].includes(childSubsys)) {
                                return aclResponse_constants_1.ITEM_STATUS.partial;
                            }
                        }
                    }
                    const childWithParents = await this.getItemWithParents(childId);
                    if (childWithParents && childWithParents.parents.length > 0) {
                        const childsParentIds = childWithParents.parents.map(item => item.parent);
                        if (childsParentIds.includes(itemId)) {
                            return aclResponse_constants_1.ITEM_STATUS.partial;
                        }
                    }
                }
            }
        }
        return aclResponse_constants_1.ITEM_STATUS.denied;
    }
    static async getRules(groupIds) {
        const aclRepository = (0, dataSource_1.getDataSource)().getRepository(acl_entity_1.ACL);
        const rules = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) } } });
        return (rules && rules.length > 0) ? rules : [];
    }
    static async getObjectItems(itemIds) {
        const objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        const objectItems = await objectItemsRepository.find({ relations: { hw_type_entity: true, parents: { _parent: true } }, where: { item_id: (0, typeorm_2.In)(itemIds) } });
        return (objectItems && objectItems.length > 0) ? objectItems : [];
    }
    static async filterClasses(groupIds, classes, strict = true) {
        if (this.isEverythingGranted(groupIds)) {
            return classes;
        }
        let filteredClasses = [];
        const aclRepository = (0, dataSource_1.getDataSource)().getRepository(acl_entity_1.ACL);
        const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.CLASS } });
        if (rule && rule.length > 0) {
            for (let r of rule) {
                const rClass = r.value0;
                filteredClasses.push(rClass);
                const classIndex = classes.indexOf(rClass);
                classes.splice(classIndex, 1);
            }
        }
        if (!strict) {
            const classesArch = await Architecture_1.Architecture.getClasses(true);
            for (let cClass of classesArch) {
                if (classes.includes(cClass.class)) {
                    const classHwTypes = cClass.hw_types.map((item) => item.hw_type);
                    const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, value0: (0, typeorm_2.In)(classHwTypes) } });
                    if (rule && rule.length > 0) {
                        filteredClasses.push(cClass.class);
                        const classIndex = classes.indexOf(cClass.class);
                        classes.splice(classIndex, 1);
                    }
                }
            }
        }
        return filteredClasses;
    }
    static async filterHwTypes(groupIds, hwTypes, strict = true) {
        if (this.isEverythingGranted(groupIds)) {
            return hwTypes;
        }
        let filteredHwTypes = [];
        const aclRepository = (0, dataSource_1.getDataSource)().getRepository(acl_entity_1.ACL);
        const rule = strict
            ? await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.HWTYPE, value0: (0, typeorm_2.In)(hwTypes) } })
            : await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, value0: (0, typeorm_2.In)(hwTypes) } });
        if (rule && rule.length > 0) {
            for (let r of rule) {
                const hwType = r.value0;
                filteredHwTypes.push(hwType);
                const hwTypeIndex = hwTypes.indexOf(hwType);
                hwTypes.splice(hwTypeIndex, 1);
            }
        }
        if (hwTypes.length > 0) {
            const classesArch = await Architecture_1.Architecture.getClasses(true);
            const rule = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.CLASS } });
            if (rule && rule.length > 0) {
                for (let r of rule) {
                    const rClass = r.value0;
                    const currentClass = classesArch.find(item => item.class === rClass);
                    for (let hwType of currentClass.hw_types) {
                        if (hwTypes.includes(hwType.hw_type)) {
                            filteredHwTypes.push(hwType.hw_type);
                            const hwTypeIndex = hwTypes.indexOf(hwType.hw_type);
                            hwTypes.splice(hwTypeIndex, 1);
                        }
                    }
                }
            }
        }
        return filteredHwTypes;
    }
    static async filterItemsById(groupIds, itemIds) {
        let filteredIds = [];
        const aclRepository = (0, dataSource_1.getDataSource)().getRepository(acl_entity_1.ACL);
        const rules = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.ID, value1: (0, typeorm_2.In)(itemIds) } });
        if (rules && rules.length > 0) {
            filteredIds = rules.map((rule) => rule.value1);
        }
        return filteredIds;
    }
    static async filterItemsBySubsys(groupIds, itemIds) {
        let filteredIds = [];
        let objectItems = await this.getObjectItems(itemIds);
        let hwTypes = objectItems.map(item => item.hw_type);
        let subsystems = objectItems.map(item => item.subsystem);
        const aclRepository = (0, dataSource_1.getDataSource)().getRepository(acl_entity_1.ACL);
        const rules = await aclRepository.find({ relations: { group: true }, where: { group: { group_id: (0, typeorm_2.In)(groupIds) }, type: aclResponse_constants_1.RULE_TYPE.SUBSYS, value0: (0, typeorm_2.In)(hwTypes), value1: (0, typeorm_2.In)(subsystems), value2: (0, typeorm_2.IsNull)() } });
        if (rules && rules.length > 0) {
            for (let rule of rules) {
                objectItems.slice(0).forEach((item, index) => {
                    if ((item.hw_type == rule.value0) && (item.subsystem == rule.value1)) {
                        filteredIds.push(item.item_id);
                        objectItems.splice(index, 1);
                    }
                });
            }
        }
        return filteredIds;
    }
    static async filterItemsByRulesTopDown(groupIds, itemIds, strict = true) {
        if (this.isEverythingGranted(groupIds)) {
            return itemIds;
        }
        let filteredIds = [];
        let rules = await this.getRules(groupIds);
        let objectItems = await this.getObjectItems(itemIds);
        for (let rule of rules) {
            let foundAny = false;
            if (rule.type === aclResponse_constants_1.RULE_TYPE.CLASS) {
                objectItems.slice(0).forEach((item, index) => {
                    if (item.hw_type_entity.class === rule.value0) {
                        filteredIds.push(item.item_id);
                        foundAny = true;
                    }
                });
            }
            else if (rule.type === aclResponse_constants_1.RULE_TYPE.HWTYPE) {
                objectItems.slice(0).forEach((item, index) => {
                    if (item.hw_type === rule.value0) {
                        filteredIds.push(item.item_id);
                        foundAny = true;
                    }
                });
            }
            else if (rule.type === aclResponse_constants_1.RULE_TYPE.ID) {
                objectItems.slice(0).forEach((item, index) => {
                    if ((item.hw_type === rule.value0) && (item.item_id === rule.value1)) {
                        filteredIds.push(item.item_id);
                        foundAny = true;
                    }
                    else {
                        for (let parent of item.parents) {
                            if ((parent._parent.hw_type === rule.value0) && (parent._parent.item_id === rule.value1)) {
                                filteredIds.push(item.item_id);
                                foundAny = true;
                                break;
                            }
                        }
                    }
                });
            }
            else if (rule.type === aclResponse_constants_1.RULE_TYPE.SUBSYS) {
                objectItems.slice(0).forEach((item, index) => {
                    if ((item.hw_type === rule.value0) && (item.subsystem === rule.value1)) {
                        if (rule.value2) {
                            for (let parent of item.parents) {
                                if ((parent._parent.hw_type === rule.value0) && (parent._parent.item_id === rule.value2)) {
                                    filteredIds.push(item.item_id);
                                    foundAny = true;
                                    break;
                                }
                            }
                        }
                        else {
                            filteredIds.push(item.item_id);
                            foundAny = true;
                        }
                    }
                });
            }
            if (foundAny) {
                objectItems = objectItems.filter((item) => { if (!filteredIds.includes(item.item_id)) {
                    return item;
                } });
            }
        }
        return filteredIds;
    }
    static async parseFolderUrl(url) {
        let hwType;
        let _class;
        let subsystem;
        let itemId;
        let path = url.replace(menuResponse_dto_1.MenuItem.API_URL_GEN, '');
        if (path) {
            const basePath = path.split('?');
            const tmpPath = basePath[0].split('/');
            if (tmpPath.length > 0) {
                if (tmpPath[0] === 'hw_type') {
                    hwType = tmpPath[1];
                    const classEntry = await Architecture_1.Architecture.getHwType(hwType);
                    _class = classEntry.class.toString();
                }
                else if (tmpPath[0] === 'class') {
                    _class = tmpPath[1];
                }
                else if (tmpPath[0] === 'sgroup') {
                    _class = 'storage';
                }
            }
            if (tmpPath.length > 2) {
                subsystem = tmpPath[2];
                if (subsystem.includes('-items')) {
                    subsystem = subsystem.replace(/-items/g, '');
                }
            }
            const tmpUrl = new URL('https://do.not.care' + url);
            const tmpIds = tmpUrl.searchParams.get('ids');
            let ids = tmpIds ? tmpIds.split(',') : [];
            if (ids.length > 0) {
                itemId = ids[0];
                const tmpIds = itemId.split(':::');
                itemId = tmpIds[0];
            }
        }
        return {
            hwType: hwType,
            class: _class,
            subsystem: subsystem,
            itemId: itemId
        };
    }
    static async filterMenuFolder(groupIds, menuItems, path, strict = false) {
        let filteredItems = [];
        if (this.isEverythingGranted(groupIds)) {
            return menuItems;
        }
        if (menuItems.length < 1) {
            return filteredItems;
        }
        let folderStatus = await this.getMenuFolderStatus(groupIds, undefined, path, false);
        if (folderStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
            return menuItems;
        }
        else if (folderStatus === aclResponse_constants_1.ITEM_STATUS.denied) {
            return filteredItems;
        }
        return filteredItems;
    }
    static async filterMenuItems(groupIds, menuItems, url, strict = false) {
        let filteredItems = [];
        if (this.isEverythingGranted(groupIds)) {
            return menuItems;
        }
        if (menuItems.length < 1) {
            return filteredItems;
        }
        if (url) {
            let folderStatus = await this.getMenuFolderStatus(groupIds, undefined, url, false);
            if (folderStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                return menuItems;
            }
            else if (folderStatus === aclResponse_constants_1.ITEM_STATUS.denied) {
                return filteredItems;
            }
        }
        const itemIds = menuItems.map((item) => { if (item.itemId) {
            return item.itemId;
        } });
        let filteredIds = await this.filterItemsByRulesTopDown(groupIds, itemIds);
        let filteredIdsPartial = [];
        if (!strict) {
            let itemsWithChildren = await this.getItemsChildren(itemIds);
            if (itemsWithChildren) {
                let allChildrenIds = [];
                for (let [itemId, childrenIds] of Object.entries(itemsWithChildren)) {
                    if (!filteredIds.includes(itemId)) {
                        allChildrenIds.push(...childrenIds);
                    }
                }
                allChildrenIds = await this.filterItemsByEnabledSubsys(allChildrenIds);
                let filteredChildrenIds = await this.filterItemsById(groupIds, allChildrenIds);
                let filteredChildrenIdsBySubsys = await this.filterItemsBySubsys(groupIds, allChildrenIds);
                for (let [itemId, childrenIds] of Object.entries(itemsWithChildren)) {
                    if (!filteredIds.includes(itemId)) {
                        for (let childId of childrenIds) {
                            if (filteredChildrenIds.includes(childId) || filteredChildrenIdsBySubsys.includes(childId)) {
                                filteredIdsPartial.push(itemId);
                                break;
                            }
                        }
                    }
                }
            }
        }
        for (let item of menuItems) {
            const foundAt = filteredItems.indexOf(item);
            if (!item.isLeaf && item.hwType && item.subsystem) {
                if (this.enabledSubsystems[item.hwType]) {
                    if (!this.enabledSubsystems[item.hwType].includes(item.subsystem)) {
                        continue;
                    }
                }
            }
            if (item.itemId) {
                if (foundAt === -1) {
                    if (filteredIds.includes(item.itemId)) {
                        filteredItems.push(item);
                    }
                    else if (filteredIdsPartial.includes(item.itemId)) {
                        if (!item.isLeaf) {
                            filteredItems.push(item);
                        }
                        else {
                            if (item.pageType && item.pageType === 'heatmap') {
                                filteredItems.push(item);
                            }
                        }
                    }
                    continue;
                }
            }
            else {
                if (item.isLeaf && item.pageType) {
                    if (foundAt === -1) {
                        filteredItems.push(item);
                        continue;
                    }
                }
            }
            const itemStatus = await this.getMenuItemStatus(groupIds, item, false);
            if (itemStatus === aclResponse_constants_1.ITEM_STATUS.denied) {
                continue;
            }
            else if (itemStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                if (foundAt === -1) {
                    filteredItems.push(item);
                }
                continue;
            }
            else if (itemStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
                if (foundAt === -1) {
                    filteredItems.push(item);
                }
                if (item.children && item.children.length > 0) {
                    let filteredChildren = [];
                    for (let childItem of item.children) {
                        if (childItem.isLeaf && !childItem.itemId) {
                            filteredChildren.push(childItem);
                            continue;
                        }
                        const childItemStatus = await this.getMenuItemStatus(groupIds, childItem, false);
                        if (childItemStatus !== aclResponse_constants_1.ITEM_STATUS.denied) {
                            if (foundAt === -1) {
                                filteredChildren.push(childItem);
                            }
                            else {
                                if (filteredItems[foundAt].children.indexOf(childItem) !== -1) {
                                    filteredItems[foundAt].children.push(childItem);
                                }
                            }
                        }
                    }
                    if (filteredChildren.length > 0) {
                        filteredItems[filteredItems.indexOf(item)].children = filteredChildren;
                    }
                }
            }
        }
        return filteredItems;
    }
    static async getMenuFolderStatus(groupIds, menuItem, url, strict = false) {
        let hwType;
        let _class;
        let subsystem;
        let itemId;
        if (menuItem && menuItem.url && menuItem.url.length > 0) {
            url = menuItem.url;
        }
        if (url) {
            const currentFolder = await this.parseFolderUrl(url);
            hwType = currentFolder.hwType;
            _class = currentFolder.class;
            subsystem = currentFolder.subsystem;
            itemId = currentFolder.itemId;
        }
        else if (menuItem) {
            hwType = menuItem.hwType;
            _class = menuItem.class;
            subsystem = menuItem.subsystem;
            itemId = menuItem.itemId;
        }
        let status;
        if (hwType) {
            if (subsystem) {
                if (itemId) {
                    if (subsystem === 'vm') {
                        status = await this.getSubsysStatus(groupIds, hwType, subsystem, [itemId], false);
                    }
                    else {
                        status = await this.getItemStatus(groupIds, itemId, false);
                    }
                }
                else {
                    status = await this.getSubsysStatus(groupIds, hwType, subsystem, [], false);
                }
            }
            else {
                status = await this.getHwTypeStatus(groupIds, hwType, false);
            }
        }
        else if (_class) {
            status = await this.getClassStatus(groupIds, _class, false);
        }
        return status;
    }
    static async getMenuItemStatus(groupIds, menuItem, strict = false) {
        if (this.isEverythingGranted(groupIds)) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        let status;
        if (menuItem instanceof menuResponse_dto_1.MenuFolder) {
            status = await this.getMenuFolderStatus(groupIds, menuItem, undefined, strict);
        }
        else {
            for (const groupId of groupIds) {
                const thisStatus = await this.getStatus(menuItem, groupId, strict);
                if (thisStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                    return aclResponse_constants_1.ITEM_STATUS.granted;
                }
                else if (thisStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
                    status = aclResponse_constants_1.ITEM_STATUS.partial;
                }
            }
            if (status !== aclResponse_constants_1.ITEM_STATUS.partial) {
                status = aclResponse_constants_1.ITEM_STATUS.denied;
            }
        }
        return status;
    }
    static async getStatus(menuItem, group_id, strict = true) {
        if (this.isEverythingGranted([group_id])) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        let score = 0;
        const classesRepository = (0, dataSource_1.getDataSource)().getRepository(class_entity_1.Class);
        const objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        if (menuItem.hwType === undefined && (menuItem.isLeaf || menuItem instanceof menuResponse_dto_1.MenuLink)) {
            if (menuItem.itemId) {
                const item = await objectItemsRepository.findOneBy({ item_id: menuItem.itemId });
                menuItem.hwType = item.hw_type;
                menuItem.subsystem = item.subsystem;
            }
            else {
                const splitUrl = menuItem.url.split('/');
                menuItem.hwType = splitUrl[splitUrl.indexOf('page') + 1];
            }
        }
        let parentIds = [];
        if (menuItem.url) {
            let url = new URL('https://do.not.care' + menuItem.url);
            let ids = url.searchParams.get('ids');
            if (ids) {
                parentIds = ids.split(',');
            }
            else if (menuItem.itemId) {
                const itemWithParents = await this.getItemWithParents(menuItem.itemId);
                if (itemWithParents && itemWithParents.parents.length > 0) {
                    parentIds = itemWithParents.parents.map(item => item.parent);
                }
            }
        }
        let itemClass;
        if (menuItem.hwType) {
            if ((menuItem.hwType === 'storage' || menuItem.hwType === 'san' || menuItem.hwType === 'lan') && menuItem.isLeaf && menuItem.itemId === undefined && menuItem.subsystem === undefined) {
                itemClass = menuItem.hwType;
                delete menuItem.hwType;
            }
            else {
                const itemClassEntry = await Architecture_1.Architecture.getHwType(menuItem.hwType);
                itemClass = itemClassEntry.class.toString();
            }
        }
        else {
            const itemClassEntry = await classesRepository.findOneBy({ label: menuItem.title });
            itemClass = itemClassEntry._class.toString();
        }
        const classStatus = await this.getClassStatus([group_id], itemClass, strict);
        if (classStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
            return aclResponse_constants_1.ITEM_STATUS.granted;
        }
        else if (!strict && !menuItem.hwType && classStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
            return aclResponse_constants_1.ITEM_STATUS.partial;
        }
        if (menuItem.hwType) {
            const hwTypeStatus = await this.getHwTypeStatus([group_id], menuItem.hwType, strict);
            if (hwTypeStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                return aclResponse_constants_1.ITEM_STATUS.granted;
            }
            else if (!strict && !menuItem.subsystem && !menuItem.itemId && hwTypeStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
                return aclResponse_constants_1.ITEM_STATUS.partial;
            }
        }
        if (menuItem.subsystem && !menuItem.itemId) {
            let strictSubsys = strict;
            if (this.enabledSubsystems[menuItem.hwType] && !this.enabledSubsystems[menuItem.hwType].includes(menuItem.subsystem)) {
                strictSubsys = true;
            }
            const subsysStatus = await this.getSubsysStatus([group_id], menuItem.hwType, menuItem.subsystem, parentIds, strictSubsys);
            if (subsysStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                return aclResponse_constants_1.ITEM_STATUS.granted;
            }
            else if (!strict && subsysStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
                return aclResponse_constants_1.ITEM_STATUS.partial;
            }
        }
        if (menuItem.itemId) {
            const itemStatus = menuItem.isLeaf
                ? await this.getItemStatus([group_id], menuItem.itemId, true)
                : await this.getItemStatus([group_id], menuItem.itemId, strict);
            if (itemStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                return aclResponse_constants_1.ITEM_STATUS.granted;
            }
            else if (!strict && itemStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
                return aclResponse_constants_1.ITEM_STATUS.partial;
            }
        }
        return aclResponse_constants_1.ITEM_STATUS.denied;
    }
};
exports.ACLCheck = ACLCheck;
ACLCheck.maxLevel = 4;
ACLCheck.enabledSubsystems = {
    'nutanix': ['cluster', 'vm'],
    'oraclevm': ['manager', 'server_pool', 'vm'],
    'ovirt': ['datacenter', 'cluster', 'vm', 'vm_nic', 'disk', 'storage_domain'],
    'power': ['server', 'vm'],
    'proxmox': ['cluster', 'vm'],
    'vmware': ['vcenter', 'cluster', 'vm', 'vm_folder', 'datastore_vm'],
    'linux': ['server', 'group'],
    'docker': ['host'],
    'kubernetes': ['cluster', 'namespace'],
    'openshift': ['cluster', 'project'],
    'isilon': ['device', 'pool'],
    'storeonce': ['device', 'pool'],
    'datadomain': ['device', 'pool'],
    'rubrik': ['device', 'pool'],
    'cohesity': ['device', 'pool'],
    'ibmcoss': ['device', 'pool'],
    'eseries': ['device', 'volume'],
    'vmax': ['device', 'volume'],
    'swiz': ['device', 'volume'],
    'xiv': ['device', 'volume'],
    '3par': ['device', 'volume'],
    'oraclezfs': ['device', 'volume'],
    'compellent': ['device', 'volume'],
    'dothill': ['device', 'volume'],
    'infortrend': ['device', 'volume'],
    'nimble': ['device', 'volume'],
    'pure': ['device', 'volume'],
    'qnap': ['device', 'volume'],
    'powerflex': ['device', 'volume'],
    'vspg': ['device', 'volume'],
    'netapp': ['device', 'volume'],
    'infinibox': ['device', 'volume'],
    'oceanstor': ['device', 'volume'],
    'powerstore': ['device', 'volume'],
    'hcp': ['device', 'namespace'],
    'ddnintelliflash': ['device'],
    'ecs': ['device'],
    'storagegrid': ['device'],
    'oracledb': ['odbfolder', 'instance'],
    'postgres': ['dbfolders', 'db'],
    'sqlserver': ['sqlsfolder', 'db'],
    'db2': ['dbfolder'],
    'lancisco': ['device', 'group'],
    'lanothers': ['device', 'group'],
};
exports.ACLCheck = ACLCheck = ACLCheck_1 = __decorate([
    (0, common_1.Module)({
        imports: [typeorm_1.TypeOrmModule.forFeature([itemRelation_entity_1.ItemRelation])],
    })
], ACLCheck);
//# sourceMappingURL=ACLCheck.js.map