import { OnModuleInit } from '@nestjs/common';
import { Class, IClass, Item, RegexPreviewDTO, Subsystem } from '@xormon/common/architecture.dto';
import { Class as ClassEntity } from '@xormon/entity/class.entity';
import { HostCfg } from '@xormon/entity/hostcfg.entity';
import { HwType } from '@xormon/entity/hwType.entity';
import { ObjectItem } from '@xormon/entity/objectItem.entity';
import { Subsystem as SubsystemEntity } from '@xormon/entity/subsystem.entity';
import { Repository } from 'typeorm';
import { IHwType } from './Architecture';
import { User } from '../users/User';
import { DeviceGroupObjectItems } from '@xormon/common/deviceGroupHwAlerting.dto';
import { AgentMapping } from '@xormon/entity/agentMapping.entity';
export interface IParent {
    [parent_id: string]: Item[];
}
export interface SearchAttributes {
    subsystem?: string;
    hw_type?: string;
    _class?: string;
    parents?: string[];
    withParents?: boolean;
    search?: string;
    childSubsystem?: string;
    withDeleted?: boolean;
}
export declare class ArchitectureService implements OnModuleInit {
    private classesRepository;
    private hwTypesRepository;
    private subsystemsRepository;
    private hostCfgRepository;
    private objectItemsRepository;
    private agentMappingRepository;
    private userRepository;
    constructor(classesRepository: Repository<ClassEntity>, hwTypesRepository: Repository<HwType>, subsystemsRepository: Repository<SubsystemEntity>, hostCfgRepository: Repository<HostCfg>, objectItemsRepository: Repository<ObjectItem>, agentMappingRepository: Repository<AgentMapping>, userRepository: Repository<User>);
    onModuleInit(): Promise<void>;
    getClasses(): Promise<Class[]>;
    getHwTypes(active: boolean, className?: string, list_models?: boolean): Promise<IHwType[]>;
    getHwType(hwType: string): Promise<IHwType>;
    getSubsystem(hwType: string, subsystem: string): Promise<Subsystem>;
    getSubsystems(active: boolean, _class?: string, hw_type?: string, type?: string): Promise<any[] | Subsystem[]>;
    getVirtCrossPlatformParents(): Subsystem[];
    getByHwTypeAndSubsystem(hw_type: string, subsystem: string): Promise<ObjectItem[]>;
    getItems(attributes: SearchAttributes): Promise<ObjectItem[]>;
    getRootItems(hostcfgRelation?: boolean): Promise<ObjectItem[]>;
    getItem(item_id: string): Promise<ObjectItem>;
    getItemWithParents(item_id: string): Promise<ObjectItem>;
    getParentsByIds(itemIds: string[]): Promise<IParent>;
    getItemsByIds(item_ids: string[], withParents?: boolean): Promise<ObjectItem[]>;
    findParents(itemIds: string[], subsystems?: string[]): Promise<{
        [item_id: string]: Item[];
    }>;
    getClassesDef(): IClass[];
    findSubsystem(subsystem: string, hwType?: string, _class?: string): Promise<Subsystem>;
    filterAgentMappings(items: ObjectItem[]): Promise<ObjectItem[]>;
    getDeviceGroupItems(_class: string, hw_type?: string, groupIds?: number[]): Promise<{
        item_id: string;
        label: string;
    }[]>;
    getObjectItemsForDeviceGroup(): Promise<ObjectItem[]>;
    getDeviceGroupItemsWithRegex(regexPreview: RegexPreviewDTO): Promise<DeviceGroupObjectItems[]>;
    getVendors(): string[];
    getVendorDeviceLabel(vendor: string): string[];
}
