/// <reference types="multer" />
import { HttpStatus } from '@nestjs/common';
import { Architecture, ClassesResponseDTO, HwTypesResponseDTO, ItemGroupConfigurationRequest, PropsResponseDTO } from '@xormon/common/architecture.dto';
import { ArrayResponseDTO, BasicResponseDTO } from '@xormon/common/basicResponse.dto';
import { FindProperties, ItemPropertiesDTO } from '@xormon/common/configuration.dto';
import { Fabric, FabricResponseDTO, FabricsResponseDTO, SaveFabric } from '@xormon/common/fabric.dto';
import { HostCfgDTO, HostCfgResponseDTO, HostCfgsResponseDTO } from '@xormon/common/hostCfgResponse.dto';
import { LicenseResponseDTO } from '@xormon/common/license.dto';
import { CreateMailGroupDTO, MailGroup, MailGroupResponseDTO, MailGroupsResponseDTO } from '@xormon/common/mailGroup.dto';
import { MicroserviceDataDTO } from '@xormon/common/microserviceData.dto';
import { MicroserviceJobDTO, MicroserviceJobsResponseDTO } from '@xormon/common/microserviceJob.dto';
import { MicroserviceVariableDTO, MicroserviceVariablesDTO, MicroserviceVariablesResponse } from '@xormon/common/microserviceVariables.dto';
import { MailTestDTO, SettingsItem, SettingsResponseDTO } from '@xormon/common/settings.dto';
import { SaveTargetDestination, TargetDestinationResponseDTO, TargetDestinationsResponseDTO } from '@xormon/common/targetDestination.dto';
import { MailService } from '@xormon/mail/mail.service';
import { Request } from 'express';
import { MetricsTableService } from '../metrics/metricsTable.service';
import { ConfigurationService } from './configuration.service';
import { FabricService } from './fabric.service';
import { HostCfgService } from './hostCfg.service';
import { MailGroupService } from './mailgroup.service';
import { MicroserviceVariablesService } from './microserviceVariable.service';
import { TokenRequestWithURLDTO } from '@xormon/common/tokenRequest.dto';
import { TokenResponseDTO } from '@xormon/common/tokenResponse.dto';
import { PageResponseDTO } from '@xormon/common/pageResponse.dto';
import { AgentStatusDTO } from '@xormon/common/osAgent.dto';
import { MenuGroupService } from './menuGroup.service';
import { MenuGroupDTO, MenuGroupResponseDTO, MenuGroupSubsystems } from '@xormon/common/menuGroup.dto';
export declare class ConfigurationController {
    private readonly configurationService;
    private readonly metricsTableService;
    private readonly mailService;
    private readonly microserviceVariablesService;
    private readonly hostCfgService;
    private readonly fabricService;
    private readonly mailGroupService;
    private readonly menugroupService;
    constructor(configurationService: ConfigurationService, metricsTableService: MetricsTableService, mailService: MailService, microserviceVariablesService: MicroserviceVariablesService, hostCfgService: HostCfgService, fabricService: FabricService, mailGroupService: MailGroupService, menugroupService: MenuGroupService);
    TMP_PATH: string;
    saveConfigrationJSON(data: any): Promise<BasicResponseDTO>;
    saveConfigrationCSV(req: any): Promise<BasicResponseDTO>;
    getConfigurationByUUID(params: any, res: Response): Promise<void>;
    findProperties(find: FindProperties): Promise<ItemPropertiesDTO>;
    getConfigurationProperties(hwType: string, subsystem: string): Promise<PropsResponseDTO>;
    getConfiguration(params: any, res: Response): Promise<void>;
    getConfigurationKeys(uuid: string): Promise<ArrayResponseDTO>;
    saveArchitecture(architecture: Architecture[], agent?: number): Promise<BasicResponseDTO>;
    getTargetDestinations(): Promise<TargetDestinationsResponseDTO>;
    saveTargetDestinations(request: Request, destination: SaveTargetDestination): Promise<TargetDestinationResponseDTO>;
    updateTargetDestinations(request: Request, target_destination_id: string, destination: SaveTargetDestination): Promise<BasicResponseDTO>;
    deleteTargetDestinations(request: Request, target_destination_id: string): Promise<BasicResponseDTO>;
    fetchDestinationToken(tokenAuthDTO: TokenRequestWithURLDTO): Promise<TokenResponseDTO>;
    getHostCfg(_class?: string, hw_type?: string): Promise<HostCfgsResponseDTO>;
    saveHostCfg(request: Request, hostcfg: HostCfgDTO, source?: string): Promise<HostCfgResponseDTO>;
    updateHostCfg(request: Request, headers: any, hostcfg_id: string, hostcfg: HostCfgDTO): Promise<BasicResponseDTO>;
    deleteHostCfg(request: Request, headers: any, hostcfg_id: string): Promise<BasicResponseDTO>;
    disableHostCfg(request: Request, hostcfgs: HostCfgDTO[]): Promise<HostCfgsResponseDTO>;
    updateHostcfgLabels(hostcfgs: HostCfgDTO[]): Promise<BasicResponseDTO>;
    deleteObjectItemWithRelations(request: Request, headers: any, item_id: string): Promise<BasicResponseDTO>;
    getOSAgentStatus(request: Request): Promise<AgentStatusDTO>;
    getFabrics(): Promise<FabricsResponseDTO>;
    saveFabric(fabric: SaveFabric): Promise<FabricResponseDTO>;
    updateFabric(fabric_id: string, fabric: Fabric): Promise<FabricResponseDTO>;
    deleteFabric(fabric_id: string): Promise<BasicResponseDTO>;
    getMenugroups(type: MenuGroupSubsystems): Promise<MenuGroupResponseDTO>;
    saveMenugroup(menuGroup: MenuGroupDTO): Promise<MenuGroupResponseDTO>;
    updateMenuGroup(group_id: string, menuGroup: MenuGroupDTO): Promise<MenuGroupResponseDTO>;
    deleteMenugroup(group_id: string): Promise<BasicResponseDTO>;
    uploadVmwareSdk(file: Express.Multer.File): Promise<BasicResponseDTO>;
    upgrade(file: Express.Multer.File): Promise<BasicResponseDTO>;
    saveLicense(file: Express.Multer.File): Promise<LicenseResponseDTO>;
    saveMicroservicesData(data: MicroserviceDataDTO[]): Promise<BasicResponseDTO>;
    getMicroservicesData(microservice: string, key?: string): Promise<{
        data: {};
    }>;
    saveMicroservicesJob(job: MicroserviceJobDTO): Promise<BasicResponseDTO>;
    getMicroservicesJobStatus(microservice?: string): Promise<MicroserviceJobsResponseDTO>;
    getSettings(filter?: string, prefix?: string): Promise<SettingsResponseDTO>;
    updateSettings(request: Request, headers: any, settings: SettingsItem): Promise<BasicResponseDTO>;
    deleteSettings(request: Request, headers: any, keys: string[]): Promise<BasicResponseDTO>;
    saveCertificate(request: Request, files: {
        crt: Express.Multer.File[];
        key: Express.Multer.File[];
    }): Promise<BasicResponseDTO>;
    mailTest(body: MailTestDTO): Promise<BasicResponseDTO>;
    getClasses(active?: boolean): Promise<ClassesResponseDTO>;
    getHwTypes(active?: boolean): Promise<HwTypesResponseDTO>;
    getSubsystems(hw_type?: string): Promise<{
        statusCode: HttpStatus;
        data: import("../architecture/Architecture").ISubsystem[];
    }>;
    getMailGroups(request: Request): Promise<MailGroupsResponseDTO>;
    createMailGroup(request: Request, mailGroup: CreateMailGroupDTO): Promise<MailGroupResponseDTO>;
    deleteMailGroup(request: Request, mail_group_id: number): Promise<BasicResponseDTO>;
    updateMailGroup(request: Request, mail_group_id: number, mailGroup: MailGroup): Promise<BasicResponseDTO>;
    test(): Promise<any>;
    getMicroserviceVariable(hw_type?: string, hostcfg_id?: string): Promise<MicroserviceVariablesResponse>;
    createMicroserviceVariable(request: Request, createMicroserviceVariablesDTO: MicroserviceVariablesDTO): Promise<{
        statusCode: HttpStatus;
        message: string;
        data: {
            microserviceVariables: MicroserviceVariableDTO[];
        };
    }>;
    deleteMicroserviceVariable(request: Request, deleteIds: {
        ids: string[];
    }): Promise<{
        statusCode: HttpStatus;
        message: string;
    }>;
    updateMicroserviceVariable(request: Request, microserviceVariableDTO: MicroserviceVariableDTO, microservice_variables_id: string): Promise<{
        statusCode: HttpStatus;
        message: string;
        microserviceVariableDTO: MicroserviceVariableDTO;
    }>;
    itemGroupProperties(itemGroupConfigurationRequest: ItemGroupConfigurationRequest): Promise<PropsResponseDTO>;
    selfMonitoring(request: Request): Promise<PageResponseDTO>;
}
