"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ItemGroupWrapper = void 0;
const dataSource_1 = require("../../config/dataSource");
const itemGroup_entity_1 = require("../../entity/itemGroup.entity");
const itemGroupItem_entity_1 = require("../../entity/itemGroupItem.entity");
const itemGroupParent_entity_1 = require("../../entity/itemGroupParent.entity");
const itemGroupParentTag_entity_1 = require("../../entity/itemGroupParentTag.entity");
const itemGroupRegexParent_entity_1 = require("../../entity/itemGroupRegexParent.entity");
const itemGroupTag_entity_1 = require("../../entity/itemGroupTag.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const typeorm_1 = require("typeorm");
const uuid_1 = require("uuid");
const ItemRegexWrapper_1 = require("../architecture/ItemRegexWrapper");
const MetricsWrapper_1 = require("../metrics/MetricsWrapper");
const Stor_1 = require("../menu/menu/Stor");
const TagWrapper_1 = require("../tag/TagWrapper");
class ItemGroupWrapper {
    static async getItemGroup(item_group_id, tool) {
        return await (0, dataSource_1.getDataSource)().getRepository(itemGroup_entity_1.ItemGroup).findOne({
            where: {
                item_group_id: item_group_id
            },
            relations: {
                export: tool && tool == 'export'
                    ? true
                    : undefined,
                alertingGroup: tool && tool == 'alerting'
                    ? true
                    : undefined,
                report_item: tool && tool == 'report'
                    ? true
                    : undefined,
                deviceGroup: tool && tool == 'hw_event'
                    ? true
                    : undefined,
                items: {
                    _item: true
                },
                parents: {
                    _item: true
                },
                regex_parents: true,
                tags: true,
                parent_tags: true
            }
        });
    }
    static transformItemGroupEntity(itemGroupEntity) {
        return {
            item_ids: itemGroupEntity.items && itemGroupEntity.items.length >= 1
                ? itemGroupEntity.items.map(i => i.item_id)
                : undefined,
            tags: itemGroupEntity.tags && itemGroupEntity.tags.length >= 1
                ? itemGroupEntity.tags.map(t => t.tag_value_id)
                : undefined,
            regex: itemGroupEntity.regex ? itemGroupEntity.regex : undefined,
            parents: (itemGroupEntity.parents && itemGroupEntity.parents.length >= 1)
                || (itemGroupEntity.parent_tags && itemGroupEntity.parent_tags.length >= 1)
                || (itemGroupEntity.regex_parents && itemGroupEntity.regex_parents.length >= 1)
                ? {
                    intersection: itemGroupEntity.intersection,
                    item_ids: itemGroupEntity.parents && itemGroupEntity.parents.length >= 1
                        ? itemGroupEntity.parents.map(p => p.item_id)
                        : undefined,
                    tags: itemGroupEntity.parent_tags && itemGroupEntity.parent_tags.length >= 1
                        ? itemGroupEntity.parent_tags.map(t => {
                            return {
                                tag: t.tag_value_id,
                                subsystem: t.subsystem
                            };
                        })
                        : undefined,
                    regex: itemGroupEntity.regex_parents && itemGroupEntity.regex_parents.length >= 1
                        ? itemGroupEntity.regex_parents.map(r => {
                            return {
                                regex: r.regex,
                                subsystem: r.subsystem
                            };
                        })
                        : undefined
                }
                : undefined
        };
    }
    static async saveItemGroup(group, oldItemGroupId) {
        const igId = oldItemGroupId ? oldItemGroupId : (0, uuid_1.v4)();
        await (0, dataSource_1.getDataSource)().transaction(async (transactionalEntityManager) => {
            await transactionalEntityManager.getRepository(itemGroup_entity_1.ItemGroup).save({
                item_group_id: igId,
                regex: group.regex,
                intersection: group.parents
                    ? group.parents.intersection === true
                    : false
            });
            const cgItems = [];
            const parentItems = [];
            const parentRegexItems = [];
            const tags = [];
            const parentTags = [];
            if (group.item_ids) {
                group.item_ids.forEach(itemId => {
                    cgItems.push({
                        item_id: itemId,
                        item_group_id: igId
                    });
                });
            }
            if (group.tags) {
                group.tags.forEach(tag => {
                    tags.push({
                        tag_value_id: tag,
                        item_group_id: igId
                    });
                });
            }
            if (group.parents && group.parents.item_ids) {
                group.parents.item_ids.forEach(itemId => {
                    parentItems.push({
                        item_id: itemId,
                        item_group_id: igId
                    });
                });
            }
            if (group.parents && group.parents.regex) {
                group.parents.regex.forEach(regex => {
                    parentRegexItems.push({
                        regex: regex.regex,
                        subsystem: regex.subsystem,
                        item_group_id: igId
                    });
                });
            }
            if (group.parents && group.parents.tags) {
                group.parents.tags.forEach(tag => {
                    parentTags.push({
                        tag_value_id: tag.tag,
                        subsystem: tag.subsystem,
                        item_group_id: igId
                    });
                });
            }
            if (oldItemGroupId) {
                await transactionalEntityManager.getRepository(itemGroupItem_entity_1.ItemGroupItem).delete({
                    item_group_id: oldItemGroupId
                });
                await transactionalEntityManager.getRepository(itemGroupParent_entity_1.ItemGroupParent).delete({
                    item_group_id: oldItemGroupId
                });
                await transactionalEntityManager.getRepository(itemGroupRegexParent_entity_1.ItemGroupRegexParent).delete({
                    item_group_id: oldItemGroupId
                });
                await transactionalEntityManager.getRepository(itemGroupTag_entity_1.ItemGroupTag).delete({
                    item_group_id: oldItemGroupId
                });
                await transactionalEntityManager.getRepository(itemGroupParentTag_entity_1.ItemGroupParentTag).delete({
                    item_group_id: oldItemGroupId
                });
            }
            if (cgItems.length >= 1) {
                await transactionalEntityManager.getRepository(itemGroupItem_entity_1.ItemGroupItem).save(cgItems);
            }
            if (parentItems.length >= 1) {
                await transactionalEntityManager.getRepository(itemGroupParent_entity_1.ItemGroupParent).save(parentItems);
            }
            if (parentRegexItems.length >= 1) {
                await transactionalEntityManager.getRepository(itemGroupRegexParent_entity_1.ItemGroupRegexParent).save(parentRegexItems);
            }
            if (tags.length >= 1) {
                await transactionalEntityManager.getRepository(itemGroupTag_entity_1.ItemGroupTag).save(tags);
            }
            if (parentTags.length >= 1) {
                await transactionalEntityManager.getRepository(itemGroupParentTag_entity_1.ItemGroupParentTag).save(parentTags);
            }
        });
        return igId;
    }
    static async getItems(item_group_id, tool, withArch = false) {
        const group = await this.getItemGroup(item_group_id, tool);
        if (!group) {
            return [];
        }
        let _class = undefined;
        let hwType = undefined;
        let subsystem = undefined;
        if (tool === 'export' && group.export) {
            _class = group.export.class;
            hwType = group.export.hw_type;
            subsystem = group.export.subsystem;
        }
        else if (tool === 'alerting' && group.alertingGroup) {
            _class = group.alertingGroup.class;
            hwType = group.alertingGroup.hw_type;
            subsystem = group.alertingGroup.subsystem;
        }
        else if (tool === 'report' && group.report_item) {
            _class = group.report_item.class;
            hwType = group.report_item.hw_type;
            subsystem = group.report_item.subsystem;
        }
        else if (tool === 'hw_event' && group.deviceGroup) {
            _class = group.deviceGroup.class;
            hwType = group.deviceGroup.hw_type;
            subsystem = group.deviceGroup.subsystem;
        }
        if (!_class && !hwType) {
            return [];
        }
        return await this.getItemsByData(this.transformItemGroupEntity(group), subsystem, hwType, _class, withArch
            ? {
                hw_type: true,
                subsystem: true
            } : undefined);
    }
    static async getMetrics(item_group_id, tool) {
        let metrics = [];
        const group = await this.getItemGroup(item_group_id, tool);
        let _class = undefined;
        let hwType = undefined;
        let subsystem = undefined;
        if (tool === 'export' && group.export) {
            _class = group.export.class;
            hwType = group.export.hw_type;
            subsystem = group.export.subsystem;
        }
        else if (tool === 'alerting' && group.alertingGroup) {
            _class = group.alertingGroup.class;
            hwType = group.alertingGroup.hw_type;
            subsystem = group.alertingGroup.subsystem;
        }
        else if (tool === 'report' && group.report_item) {
            _class = group.report_item.class;
            hwType = group.report_item.hw_type;
            subsystem = group.report_item.subsystem;
        }
        const excludeType = tool === 'export'
            ? 'exporter'
            : tool === 'report'
                ? 'reporter'
                : tool;
        if (!['san', 'storage', 'lan'].includes(_class) && hwType) {
            return MetricsWrapper_1.MetricsWrapper.handleLongLabel(await MetricsWrapper_1.MetricsWrapper.getMetrics(hwType, subsystem, excludeType));
        }
        const items = await this.getItems(item_group_id, tool, true);
        let hwTypes = [];
        for (const item of items) {
            if (!subsystem) {
                subsystem = item.subsystem;
            }
            if (!hwTypes.includes(item.hw_type)) {
                hwTypes.push(item.hw_type);
            }
        }
        for (const hwType of hwTypes) {
            const hMetrics = MetricsWrapper_1.MetricsWrapper.handleLongLabel(await MetricsWrapper_1.MetricsWrapper.getMetrics(hwType, subsystem, excludeType));
            if (metrics.length === 0) {
                metrics = hMetrics;
                continue;
            }
            else {
                for (const hMetric of hMetrics) {
                    if (metrics.find(metric => metric.metric === hMetric.metric) === undefined) {
                        metrics.push(hMetric);
                    }
                }
            }
        }
        return metrics;
    }
    static async getItemsByData(group, subsystem, hwType, _class, withProps) {
        if (!_class && !hwType) {
            return [];
        }
        let igItems = [];
        if (group.item_ids && group.item_ids.length >= 1) {
            igItems = igItems.concat((await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findBy({
                item_id: (0, typeorm_1.In)(group.item_ids)
            })).map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label,
                    hw_type: withProps && withProps.hw_type
                        ? item.hw_type
                        : undefined
                };
            }));
        }
        else if (group.tags && group.tags.length >= 1) {
            let items = [];
            const filterHwType = hwType ? hwType : _class;
            if (group.parents && group.parents.item_ids && group.parents.item_ids.length >= 1) {
                items = await TagWrapper_1.TagWrapper.getItemsByParents(group.parents.item_ids, filterHwType, subsystem, group.tags, group.parents.intersection);
            }
            else if (group.parents && group.parents.regex && group.parents.regex.length >= 1) {
                items = await TagWrapper_1.TagWrapper.getItemsByRegexParentsRegex(group.parents.regex, filterHwType, subsystem, group.tags, group.parents.intersection);
            }
            else if (group.parents && group.parents.tags && group.parents.tags.length >= 1) {
                items = await TagWrapper_1.TagWrapper.getItemsByParentsTag(group.parents.tags, filterHwType, subsystem, group.tags, group.parents.intersection);
            }
            else {
                items = await TagWrapper_1.TagWrapper.getItems(filterHwType, subsystem, group.tags);
            }
            igItems = igItems.concat(items.map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label,
                    hw_type: withProps && withProps.hw_type
                        ? item.hw_type
                        : undefined
                };
            }));
        }
        else if (group.regex) {
            let items = [];
            const filterHwType = hwType ? hwType : _class;
            if (group.parents && group.parents.item_ids && group.parents.item_ids.length >= 1) {
                items = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegexParents(group.parents.item_ids, filterHwType, subsystem, group.regex, group.parents.intersection);
            }
            else if (group.parents && group.parents.regex && group.parents.regex.length >= 1) {
                items = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegexParentsRegex(group.parents.regex, filterHwType, subsystem, group.regex, group.parents.intersection);
            }
            else if (group.parents && group.parents.tags && group.parents.tags.length >= 1) {
                items = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegexParentsTag(group.parents.tags, filterHwType, subsystem, group.regex, group.parents.intersection);
            }
            else {
                items = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegex(filterHwType, subsystem, group.regex);
            }
            igItems = igItems.concat(items.map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label,
                    hw_type: withProps && withProps.hw_type
                        ? item.hw_type
                        : undefined,
                    subsystem: withProps && withProps.subsystem
                        ? item.hw_type
                        : undefined
                };
            }));
        }
        else {
            return [];
        }
        return igItems;
    }
    static async getMetricsByData(group, subsystem, hwType, _class, type) {
        let metrics = [];
        const items = await this.getItemsByData(group, subsystem, hwType, _class, { hw_type: true });
        let hwTypes = [];
        for (const item of items) {
            if (!hwTypes.includes(item.hw_type)) {
                hwTypes.push(item.hw_type);
            }
        }
        for (const hwType of hwTypes) {
            const hMetrics = MetricsWrapper_1.MetricsWrapper.handleLongLabel(await MetricsWrapper_1.MetricsWrapper.getMetrics(hwType, subsystem, type));
            if (metrics.length === 0) {
                metrics = hMetrics;
                continue;
            }
            else {
                for (const hMetric of hMetrics) {
                    if (metrics.find(metric => metric.metric === hMetric.metric) === undefined) {
                        metrics.push(hMetric);
                    }
                }
            }
        }
        if (_class && _class === 'storage') {
            metrics = (0, Stor_1.sortStorageMetrics)(metrics);
        }
        return metrics;
    }
    static async getPropertiesByData(group, subsystem, hwType, _class) {
        let properties = [];
        const items = await this.getItemsByData(group, subsystem, hwType, _class, { hw_type: true });
        let hwTypes = [];
        for (const item of items) {
            if (!hwTypes.includes(item.hw_type)) {
                hwTypes.push(item.hw_type);
            }
        }
        for (const hwType of hwTypes) {
            const hProperties = await MetricsWrapper_1.MetricsWrapper.getConfigurationFindClass(hwType, subsystem);
            let props = [];
            for (const propertyKey in hProperties) {
                props.push({
                    property_name: propertyKey,
                    label: hProperties[propertyKey] && hProperties[propertyKey].label ? hProperties[propertyKey].label : propertyKey
                });
            }
            if (properties.length === 0) {
                properties = props;
                continue;
            }
            else {
                for (const prop of props) {
                    if (properties.find(property => property.property_name === prop.property_name) === undefined) {
                        properties.push(prop);
                    }
                }
            }
        }
        return properties;
    }
}
exports.ItemGroupWrapper = ItemGroupWrapper;
//# sourceMappingURL=ItemGroupWrapper.js.map