import { CreateCustomGroupDTO, CreateCustomGroupFolder, CustomGroupItemGroup, UpdateCustomGroupDTO } from '@xormon/common/customGroup.dto';
import { CustomGroup } from '@xormon/entity/customGroup.entity';
import { CustomGroupFolder } from '@xormon/entity/customGroupFolder.entity';
import { ItemGroup } from '@xormon/entity/itemGroup.entity';
import { ObjectItem } from '@xormon/entity/objectItem.entity';
import { Repository, TreeRepository } from 'typeorm';
import { CustomGroupFavorite } from '@xormon/entity/customGroupFavorite.entity';
import { MenuItem } from '@xormon/common/menuResponse.dto';
interface CustomGroupFolderItem {
    label: string;
    custom_group_folder_id: string;
    parent: string;
    path: string;
    children?: CustomGroupFolderItem[];
}
interface SearchParams {
    hwType?: string;
    subsystem?: string;
    search?: string;
    page?: number;
}
export declare class CustomGroupService {
    private customGroupsRepository;
    private customGroupFoldersRepository;
    private objectItemsRepository;
    private customGroupFavoriteRepository;
    constructor(customGroupsRepository: Repository<CustomGroup>, customGroupFoldersRepository: TreeRepository<CustomGroupFolder>, objectItemsRepository: Repository<ObjectItem>, customGroupFavoriteRepository: Repository<CustomGroupFavorite>);
    getCustomGroups(folder: CustomGroupFolder, userId?: number, groupIds?: number[]): Promise<CustomGroup[]>;
    getAllFolders(): Promise<CustomGroupFolderItem[]>;
    getFolderChildren(folder: any, parent: string, parentPath: string): CustomGroupFolderItem[];
    getFolderParentPath(folder: any): string;
    updateCustomGroupFolder(custom_group_folder_id: string, custom_group_folder: CreateCustomGroupFolder): Promise<Boolean>;
    getFolders(parent: CustomGroupFolder): Promise<CustomGroupFolder[]>;
    getFoldersTree(user_id?: number, groupIds?: number[]): Promise<Array<CustomGroupFolder>>;
    filterFolderTree(folders: CustomGroupFolder[]): CustomGroupFolder[];
    getRootFolders(userId?: number, groupIds?: number[]): Promise<CustomGroupFolder[]>;
    getFolder(custom_group_folder_id: string): Promise<CustomGroupFolder>;
    saveFolder(folder: CreateCustomGroupFolder, userId?: number): Promise<CustomGroupFolder>;
    deleteFolder2(custom_group_folder_id: string): Promise<Boolean>;
    deleteFolder(custom_group_folder_id: string): Promise<Boolean>;
    getParentsFolders(parent: CustomGroupFolder): Promise<CustomGroupFolder>;
    getParentsIds(parent: CustomGroupFolder): Promise<string[]>;
    findInParent(parent: CustomGroupFolder): string[];
    getPath(custom_group_folder: CustomGroupFolder): Promise<string>;
    findNextPath(folder: CustomGroupFolder): Promise<string>;
    getItems(item_ids: string[]): Promise<ObjectItem[]>;
    getAllCustomGroups(userId: number, groupIds: number[]): Promise<CustomGroup[]>;
    saveCustomGroup(custom_group: CreateCustomGroupDTO, user_id?: number): Promise<CustomGroup>;
    updateCustomGroup(custom_group_id: string, custom_group: Partial<UpdateCustomGroupDTO>): Promise<Boolean>;
    getCustomGroup(custom_group_id: string): Promise<CustomGroup>;
    deleteCustomGroup(custom_group_id: string): Promise<Boolean>;
    getObjectItemsByIds(item_ids: string[]): Promise<ObjectItem[]>;
    search(params: SearchParams): Promise<ObjectItem[]>;
    findParents(item_ids: string[]): Promise<{}>;
    checkCustomGroupOwner(customGroupId: string, userId: number, groupIds: number[]): Promise<boolean>;
    checkCustomGroupFolderOwner(customGroupFolderId: string, userId: number, groupIds: number[]): Promise<boolean>;
    formatCustomGroupItemGroups(groups: ItemGroup[]): CustomGroupItemGroup[];
    getDashboardCustomGroups(userId?: number, groupIds?: number[]): Promise<CustomGroup[]>;
    checkFavoriteCustomGroup(customGroupId: string, userId: number): Promise<boolean>;
    setFavoriteCustomGroup(customGroupId: string, userId: number): Promise<boolean>;
    deleteFavoriteCustomGroup(customGroupId: string, userId: number): Promise<boolean>;
    getCustomGroupDashboardFolder(userId: number): Promise<MenuItem[]>;
}
export {};
