"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DashboardService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const dashboard_dto_1 = require("../../common/dashboard.dto");
const dataSource_1 = require("../../config/dataSource");
const dashboard_entity_1 = require("../../entity/dashboard.entity");
const dashboardFolder_entity_1 = require("../../entity/dashboardFolder.entity");
const dashboardGraph_entity_1 = require("../../entity/dashboardGraph.entity");
const dashboardGroup_entity_1 = require("../../entity/dashboardGroup.entity");
const typeorm_2 = require("typeorm");
class StoreGraph {
}
class StoreGroup {
}
class StoreDashboard {
}
class StoreFolder {
}
let DashboardService = class DashboardService {
    constructor(dashboardsRepository, dashboardFoldersRepository, dashboardGroupsRepository, dashboardGraphsRepository) {
        this.dashboardsRepository = dashboardsRepository;
        this.dashboardFoldersRepository = dashboardFoldersRepository;
        this.dashboardGroupsRepository = dashboardGroupsRepository;
        this.dashboardGraphsRepository = dashboardGraphsRepository;
    }
    async getDashboards(parent, user_id, group_ids) {
        if (parent) {
            return await this.dashboardsRepository.find({
                where: {
                    parent: {
                        dashboard_folder_id: parent
                    },
                    user_id: user_id && !group_ids
                        ? user_id
                        : undefined,
                    group_id: group_ids
                        ? (0, typeorm_2.In)(group_ids)
                        : (0, typeorm_2.IsNull)()
                }, relations: {
                    parent: true
                }
            });
        }
        else {
            return await this.getRootDashboards(user_id);
        }
    }
    async getRootDashboardFolders(user_id, group_ids) {
        if (user_id || group_ids) {
            return await this.dashboardFoldersRepository.find({
                where: {
                    parent: (0, typeorm_2.IsNull)(),
                    user_id: user_id && !group_ids
                        ? user_id
                        : undefined,
                    group_id: group_ids
                        ? (0, typeorm_2.In)(group_ids)
                        : (0, typeorm_2.IsNull)()
                }
            });
        }
        return await this.dashboardFoldersRepository.createQueryBuilder('dashboardFolders').where('dashboardFolders.parent is null').getMany();
    }
    async getRootDashboards(user_id) {
        return await this.dashboardsRepository.find({
            where: {
                parent: (0, typeorm_2.IsNull)(),
                user_id: user_id ? user_id : undefined,
                group_id: (0, typeorm_2.IsNull)()
            }
        });
    }
    async getRootGroupDashboards(group_ids) {
        return await this.dashboardsRepository.find({
            where: {
                parent: (0, typeorm_2.IsNull)(),
                group_id: (0, typeorm_2.In)(group_ids)
            }
        });
    }
    async getAllDashboards(user_id) {
        return await this.dashboardsRepository.find({
            where: {
                user_id: user_id ? user_id : undefined
            }
        });
    }
    async getAllFolders(user_id) {
        if (user_id) {
            const trees = await this.dashboardFoldersRepository.findTrees({ relations: ['created_by'] });
            return trees.filter(tree => tree.created_by.user_id === user_id);
        }
        return await this.dashboardFoldersRepository.findTrees();
    }
    async getFolders(parent) {
        return await this.dashboardFoldersRepository.findDescendantsTree(parent);
    }
    async getParentsFolders(parent) {
        return await (0, dataSource_1.getDataSource)().getTreeRepository(dashboardFolder_entity_1.DashboardFolder).findAncestorsTree(parent);
    }
    async getParentsIds(parent) {
        return this.findInParent(await this.getParentsFolders(parent));
    }
    findInParent(parent) {
        let ids = [parent.dashboard_folder_id];
        if (parent.parent) {
            ids = ids.concat(this.findInParent(parent.parent));
        }
        return ids;
    }
    async getPath(dashboard_folder) {
        const folder = await this.getParentsFolders(dashboard_folder);
        if (folder) {
            let path = await this.findNextPath(folder);
            return '/' + path.split('/').reverse().join('/');
        }
        else {
            return '/';
        }
    }
    async findNextPath(folder) {
        let path = folder.label;
        if (folder.parent) {
            path += '/' + (await this.findNextPath(folder.parent));
        }
        return path;
    }
    async getFolder(dashboard_folder_id) {
        return await this.dashboardFoldersRepository.findOneBy({ dashboard_folder_id: dashboard_folder_id });
    }
    async getAllGroups() {
        return await this.dashboardGroupsRepository.find();
    }
    async getDashboardFolders(parent, user_id, group_ids) {
        return parent
            ? await this.dashboardFoldersRepository.find({
                where: {
                    parent: {
                        dashboard_folder_id: parent.dashboard_folder_id
                    },
                    user_id: user_id && !group_ids
                        ? user_id
                        : undefined,
                    group_id: group_ids
                        ? (0, typeorm_2.In)(group_ids)
                        : (0, typeorm_2.IsNull)()
                }, relations: {
                    parent: true
                }
            })
            : await this.getRootDashboardFolders(user_id ? user_id : undefined, group_ids ? group_ids : undefined);
    }
    async getDashboardDefault(user_id) {
        return await this.dashboardsRepository.findOne({
            where: {
                default: true,
                user_id: user_id ? user_id : undefined
            }
        });
    }
    async setDefault(dashboard_id, user_id) {
        const dashboard = await this.dashboardsRepository.findOne({
            where: {
                dashboard_id: dashboard_id,
                user_id: user_id
            }
        });
        const oldDefault = await this.dashboardsRepository.findOne({
            where: {
                default: true,
                user_id: user_id
            }
        });
        if (dashboard) {
            dashboard.default = true;
            if (oldDefault) {
                oldDefault.default = false;
                return (await this.dashboardsRepository.save(oldDefault)).default === false && (await this.dashboardsRepository.save(dashboard)).default === true;
            }
            return (await this.dashboardsRepository.save(dashboard)).default === true;
        }
        else {
            return false;
        }
    }
    async getDashboard(dashboard_id) {
        return await this.dashboardsRepository.findOne({
            where: { dashboard_id: dashboard_id },
            relations: { dashboard_groups: { dashboard_graphs: true }, parent: true },
        });
    }
    async getDashboardWithGroups(dashboard_id, user_id) {
        return await this.dashboardsRepository.findOne({
            where: {
                dashboard_id: dashboard_id,
                user_id: user_id ? user_id : undefined
            },
            relations: {
                dashboard_groups: true,
                parent: true
            },
        });
    }
    async getDashboardGroup(dashboard_group_id) {
        return await this.dashboardGroupsRepository.findOneBy({ dashboard_group_id: dashboard_group_id });
    }
    async getDashboardGroups(dashboard_id) {
        return dashboard_id
            ? await this.dashboardGroupsRepository.findBy({ dashboard_id: dashboard_id })
            : await this.dashboardGroupsRepository.find();
    }
    async findGraphs(url, tab, user_id, interval, group_ids) {
        let where = {
            url: url,
            tab: tab,
            dashboard_group: {
                dashboard: [{
                        user_id: user_id
                    }, {
                        group_id: (0, typeorm_2.In)(group_ids)
                    }]
            }
        };
        if (interval) {
            where['interval'] = interval;
        }
        return await this.dashboardGraphsRepository.find({
            where: where,
            relations: { dashboard_group: { dashboard: true } },
        });
    }
    async saveGraph(graph) {
        if (graph?.url.includes(":::")) {
            graph.url = graph.url.split(":::")[0];
        }
        return await this.dashboardGraphsRepository.save({
            ...graph,
            x: dashboard_dto_1.DashboardGraph.DEFAULT_COORDINATE,
            y: dashboard_dto_1.DashboardGraph.DEFAULT_COORDINATE
        });
    }
    async saveGroup(group) {
        return await this.dashboardGroupsRepository.save({
            ...group,
            x: dashboard_dto_1.DashboardGraph.DEFAULT_COORDINATE,
            y: dashboard_dto_1.DashboardGraph.DEFAULT_COORDINATE
        });
    }
    async saveDashboardFolder(folder, user_id) {
        if (folder.parent) {
            const parent = await this.dashboardFoldersRepository.findOneBy({ dashboard_folder_id: folder.parent });
            if (parent) {
                return await this.dashboardFoldersRepository.save({
                    parent: parent,
                    label: folder.label,
                    type: folder.type,
                    user_id: user_id ? user_id : null,
                    group_id: parent.group_id
                        ? parent.group_id
                        : folder.group_id
                            ? folder.group_id
                            : null
                });
            }
        }
        return await this.dashboardFoldersRepository.save({
            label: folder.label,
            type: folder.type,
            user_id: user_id ? user_id : null,
            group_id: folder.group_id ? folder.group_id : null
        });
    }
    async findDashboardsInFolder(dashboardFolder, ids) {
        for (const folder of dashboardFolder.children) {
            const subfolder = await this.dashboardFoldersRepository.findOne({
                where: {
                    dashboard_folder_id: folder.dashboard_folder_id
                }, relations: {
                    children: true,
                    dashboards: true
                }
            });
            ids = ids.concat(await this.findDashboardsInFolder(subfolder, ids));
        }
        ids = ids.concat(dashboardFolder.dashboards.map(dashboard => dashboard.dashboard_id));
        return ids;
    }
    async deleteDashboardFolder(dashboard_folder_id) {
        const defaultDashboard = await this.getDashboardDefault();
        if (defaultDashboard) {
            const dashboardFolder = await this.dashboardFoldersRepository.findOne({
                where: {
                    dashboard_folder_id: dashboard_folder_id
                }, relations: {
                    children: true,
                    dashboards: true
                }
            });
            const childrenDashboards = await this.findDashboardsInFolder(dashboardFolder, []);
            if (childrenDashboards.includes(defaultDashboard.dashboard_id)) {
                return false;
            }
        }
        const deleted = await this.dashboardFoldersRepository.delete(dashboard_folder_id);
        return deleted.affected === 1 ? true : false;
    }
    async saveDashboard(dashboard, user_id) {
        const saved = await this.dashboardsRepository.save({ ...dashboard, user_id: user_id ? user_id : null });
        return saved;
    }
    async deleteDashboard(dashboard_id) {
        const deleted = await this.dashboardsRepository.delete(dashboard_id);
        return deleted.affected === 1 ? true : false;
    }
    async getDashboardGraph(dashboard_graph_id) {
        return await this.dashboardGraphsRepository.findOneBy({ dashboard_graph_id: dashboard_graph_id });
    }
    async updateDashboardGraph(dashboard_graph_id, dashboard_graph) {
        if (dashboard_graph?.url.includes(":::")) {
            dashboard_graph.url = dashboard_graph.url.split(":::")[0];
        }
        const updated = await this.dashboardGraphsRepository.update(dashboard_graph_id, { ...dashboard_graph });
        return updated.affected === 1 ? true : false;
    }
    async updateDashboard(dashboard_id, dashboard) {
        const folder = dashboard.parent ? await this.getFolder(dashboard.parent) : dashboard.parent === null ? null : undefined;
        const updated = await this.dashboardsRepository.update(dashboard_id, { ...dashboard, parent: folder });
        return updated.affected === 1 ? true : false;
    }
    async updateDashboardFolder(dashboard_folder_id, dashboard_folder) {
        let updated = null;
        if (dashboard_folder.parent) {
            const parent = await this.dashboardFoldersRepository.findOneBy({ dashboard_folder_id: dashboard_folder.parent });
            if (parent) {
                updated = await this.dashboardFoldersRepository.update(dashboard_folder_id, { label: dashboard_folder.label, parent: parent });
                return updated.affected === 1 ? true : false;
            }
        }
        updated = await this.dashboardFoldersRepository.update(dashboard_folder_id, { label: dashboard_folder.label });
        return updated.affected === 1 ? true : false;
    }
    async deleteDashboardGraph(dashboard_graph_id) {
        const deleted = await this.dashboardGraphsRepository.delete(dashboard_graph_id);
        return deleted.affected === 1 ? true : false;
    }
    async updateDashboardGroup(dashboard_group_id, dashboard_group) {
        const updated = await this.dashboardGroupsRepository.update(dashboard_group_id, { ...dashboard_group });
        return updated.affected === 1 ? true : false;
    }
    async deleteDashboardGroup(dashboard_group_id) {
        const deleted = await this.dashboardGroupsRepository.delete(dashboard_group_id);
        return deleted.affected === 1 ? true : false;
    }
    async resortGroupHorizontal(dashboard_id, y, start_x) {
        let actual = start_x;
        const groups = await this.dashboardGroupsRepository.find({
            where: { x: (0, typeorm_2.MoreThanOrEqual)(actual), y: y, dashboard_id: dashboard_id },
            order: { x: 'ASC' },
        });
        for (const group of groups) {
            actual++;
            if (actual > 12) {
                group.x = 1;
                group.y = y + 1;
                await this.resortGroupHorizontal(dashboard_id, (y = y + 1), (start_x = 1));
            }
            else {
                group.x = actual;
            }
        }
        const saved = await this.dashboardGroupsRepository.save(groups);
        return saved.length >= 1 ? true : false;
    }
    async resortGroup(dashboard_group_id, dashboard_id, x, y) {
        const check = await this.dashboardGroupsRepository.findOneBy({ x: x, y: y, dashboard_id: dashboard_group_id });
        if (check && check.dashboard_group_id !== dashboard_group_id) {
            let sorted = this.resortGroupHorizontal(dashboard_id, y, x);
            return sorted ? true : false;
        }
        else {
            return true;
        }
    }
    async checkDashboardOwner(dashboard_id, user_id, group_ids = []) {
        let where = [{
                dashboard_id: dashboard_id,
                user_id: user_id
            }];
        if (group_ids) {
            where.push({
                dashboard_id: dashboard_id,
                group_id: (0, typeorm_2.In)(group_ids)
            });
        }
        return await this.dashboardsRepository.findOne({
            where: where
        }) !== null;
    }
    async checkDashboardFolderOwner(dashboard_folder_id, user_id, group_ids) {
        let where = [{
                dashboard_folder_id: dashboard_folder_id,
                user_id: user_id
            }];
        if (group_ids) {
            where.push({
                dashboard_folder_id: dashboard_folder_id,
                group_id: (0, typeorm_2.In)(group_ids)
            });
        }
        return await this.dashboardFoldersRepository.findOne({
            where: where
        }) !== null;
    }
};
exports.DashboardService = DashboardService;
exports.DashboardService = DashboardService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(dashboard_entity_1.Dashboard)),
    __param(1, (0, typeorm_1.InjectRepository)(dashboardFolder_entity_1.DashboardFolder)),
    __param(2, (0, typeorm_1.InjectRepository)(dashboardGroup_entity_1.DashboardGroup)),
    __param(3, (0, typeorm_1.InjectRepository)(dashboardGraph_entity_1.DashboardGraph)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.TreeRepository,
        typeorm_2.TreeRepository,
        typeorm_2.TreeRepository])
], DashboardService);
//# sourceMappingURL=dashboard.service.js.map