"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var HealthStatusService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.HealthStatusService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const aclResponse_constants_1 = require("../../common/constants/aclResponse.constants");
const healthStatus_entity_1 = require("../../entity/healthStatus.entity");
const hostcfg_entity_1 = require("../../entity/hostcfg.entity");
const hostcfgRelation_entity_1 = require("../../entity/hostcfgRelation.entity");
const hwType_entity_1 = require("../../entity/hwType.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const ACLCheck_1 = require("../acl/ACLCheck");
const URL_1 = require("../menu/menu/URL");
const typeorm_2 = require("typeorm");
const PM2_1 = require("../configuration/PM2");
const event_emitter_1 = require("@nestjs/event-emitter");
const hwTypeKey_constants_1 = require("../../common/constants/hwTypeKey.constants");
let HealthStatusService = HealthStatusService_1 = class HealthStatusService {
    constructor(objectItemsRepository, hostCfgRepository, hostCfgRelationsRepository, hwTypesRepository, healthStatusRepository, eventEmitter) {
        this.objectItemsRepository = objectItemsRepository;
        this.hostCfgRepository = hostCfgRepository;
        this.hostCfgRelationsRepository = hostCfgRelationsRepository;
        this.hwTypesRepository = hwTypesRepository;
        this.healthStatusRepository = healthStatusRepository;
        this.eventEmitter = eventEmitter;
        this.LAST_DATA_UPDATE_LIMIT = HealthStatusService_1.LAST_DATA_UPDATE_LIMIT;
    }
    onModuleInit() {
        process.on('message', (data) => {
            this.reloadHealthStatus(data);
        });
    }
    reloadHealthStatus(data) {
        const parsed = PM2_1.PM2.parseMessage(data);
        if (parsed.action === 'reload-health-status') {
            this.eventEmitter.emit('reload', {
                event: 'reload',
                message: {
                    reload: 'health-status'
                },
            });
        }
    }
    async getHostCfg(uuid) {
        return this.hostCfgRepository.findOneBy({ hostcfg_id: uuid });
    }
    async getHostsCfg(ignored, blacklist = [hwTypeKey_constants_1.HwTypeKey.PROXMOX, hwTypeKey_constants_1.HwTypeKey.VMWARE]) {
        return this.hostCfgRepository.find({
            where: {
                disabled: false,
                ignore_health_status: ignored !== undefined ? ignored : undefined,
                hw_type: blacklist.length >= 1 ? (0, typeorm_2.Not)((0, typeorm_2.In)(blacklist)) : undefined,
            },
            relations: {
                status: true,
                hw_type_entity: true,
                relations: {
                    _item: {
                        health_status: true
                    }
                }
            }
        });
    }
    async getHostCfgStatus(includeIgnored, specificIds = undefined, blacklist = [hwTypeKey_constants_1.HwTypeKey.PROXMOX, hwTypeKey_constants_1.HwTypeKey.VMWARE]) {
        return this.hostCfgRepository.find({
            where: {
                hostcfg_id: specificIds ? (0, typeorm_2.In)(specificIds) : undefined,
                disabled: false,
                ignore_health_status: includeIgnored !== undefined ? includeIgnored : undefined,
                hw_type: blacklist.length >= 1 ? (0, typeorm_2.Not)((0, typeorm_2.In)(blacklist)) : undefined,
            },
            relations: {
                hw_type_entity: true,
                status: true,
            }
        });
    }
    async getHostCfgByHWType(hw_type = null) {
        return this.hostCfgRepository.findBy({ hw_type: hw_type });
    }
    async getHealthStatuses() {
        return await this.healthStatusRepository.find();
    }
    async getHealthStatus(item_id, status) {
        return await this.healthStatusRepository.findBy({ item_id: (0, typeorm_2.In)(item_id), status: status ? status : undefined });
    }
    async getHostcfgRelations(hostcfg_id) {
        return await this.hostCfgRelationsRepository.findBy({ hostcfg_id: hostcfg_id });
    }
    async getHostcfgRelationsAll() {
        return await this.hostCfgRelationsRepository.find();
    }
    async getItem(item_id) {
        return await this.objectItemsRepository.findOneBy({ item_id: item_id });
    }
    async getItemsByHwType(hw_type) {
        return await this.objectItemsRepository.findBy({ hw_type: hw_type });
    }
    async saveHealthStatus(healthStatus) {
        const insert = await this.healthStatusRepository.save(healthStatus, { chunk: 250 });
        return insert.length;
    }
    async getHwTypes(active) {
        if (active) {
            let hw_types = await this.hwTypesRepository.find();
            let hw_types_active = [];
            for (let hw_type of hw_types) {
                let hostcfg = await this.getHostCfgByHWType(hw_type.hw_type);
                if (hostcfg.length !== 0) {
                    hw_types_active.push(hw_type);
                }
            }
            return hw_types_active;
        }
        else {
            return await this.hwTypesRepository.find();
        }
    }
    async getHwType(hw_type) {
        return await this.hwTypesRepository.findOneBy({ hw_type: hw_type });
    }
    async getDeviceHS(hostcfgs, byStatus, groupIds) {
        let healthStatusHostCfg = [];
        let cleanItemIds = groupIds !== undefined ? await ACLCheck_1.ACLCheck.filterItems(groupIds, hostcfgs.map(device => device.hostcfg_id)) : [];
        for (const hostcfg_entity of hostcfgs) {
            let hostcfgStatus = healthStatus_entity_1.Status.UNKNOWN;
            let timestamp;
            let note = "";
            if (!cleanItemIds.includes(hostcfg_entity.hostcfg_id)) {
                if (groupIds !== undefined && await ACLCheck_1.ACLCheck.getHwTypeStatus(groupIds, hostcfg_entity.hw_type) !== aclResponse_constants_1.ITEM_STATUS.granted) {
                    continue;
                }
            }
            if (hostcfg_entity.relations && hostcfg_entity.relations.length) {
                if (hostcfg_entity.hw_type !== 'nutanix') {
                    const foundHS = hostcfg_entity.relations.find(relation => relation.item_id === hostcfg_entity.hostcfg_id);
                    if (foundHS && foundHS._item && foundHS._item.health_status !== null) {
                        hostcfgStatus = foundHS._item.health_status.status ? foundHS._item.health_status.status : healthStatus_entity_1.Status.UNKNOWN;
                        timestamp = foundHS._item.health_status.updatedAt !== null ? foundHS._item.health_status.updatedAt.valueOf() : null;
                        note = foundHS._item.health_status.data ? foundHS._item.health_status.data : "";
                        if (hostcfgStatus !== healthStatus_entity_1.Status.ERROR && new Date().getTime() - timestamp > this.LAST_DATA_UPDATE_LIMIT) {
                            hostcfgStatus = healthStatus_entity_1.Status.UNKNOWN;
                            note = "No current data";
                        }
                    }
                    else {
                        hostcfgStatus = undefined;
                    }
                }
                else if (hostcfg_entity.hw_type === 'nutanix') {
                    let clusters = hostcfg_entity.relations.filter(relation => {
                        if (relation && relation._item && relation._item.subsystem === 'cluster') {
                            return relation;
                        }
                    });
                    for (const cluster of clusters) {
                        if ('health_status' in cluster._item && cluster._item.health_status) {
                            let currentHS = cluster._item.health_status;
                            if (currentHS.status === healthStatus_entity_1.Status.ERROR || currentHS.status === healthStatus_entity_1.Status.WARNING) {
                                if (hostcfgStatus !== healthStatus_entity_1.Status.ERROR) {
                                    hostcfgStatus = currentHS.status;
                                    timestamp = currentHS.updatedAt.getTime();
                                }
                                note += `${currentHS.data}\n`;
                            }
                        }
                    }
                    if (hostcfgStatus !== healthStatus_entity_1.Status.ERROR && new Date().getTime() - timestamp > this.LAST_DATA_UPDATE_LIMIT) {
                        hostcfgStatus = healthStatus_entity_1.Status.UNKNOWN;
                    }
                }
            }
            else {
                hostcfgStatus = undefined;
            }
            let datacheck = 0;
            let connection = 0;
            if ('status' in hostcfg_entity && hostcfg_entity.status) {
                datacheck = hostcfg_entity.status['data'] ? 0 : 1;
                connection = hostcfg_entity.status['connection'] ? 0 : 1;
            }
            let useId = hostcfg_entity.hw_type === 'nutanix' || hostcfg_entity.hw_type_entity.class === 'database' ? undefined : hostcfg_entity.hostcfg_id;
            healthStatusHostCfg.push({
                class: hostcfg_entity.hw_type_entity.class,
                hw_type: hostcfg_entity.hw_type_entity.hw_type,
                hw_type_label: hostcfg_entity.hw_type_entity.label,
                label: hostcfg_entity.label,
                item_id: hostcfg_entity.hostcfg_id,
                updatedAt: timestamp,
                note: note,
                status: hostcfgStatus,
                data: datacheck,
                connection: connection,
                ignored: hostcfg_entity.ignore_health_status,
                ignore_reason: hostcfg_entity.ignore_health_status_reason,
                url: URL_1.URL.generate(hostcfg_entity.hw_type_entity.hw_type, "health-status", useId),
            });
        }
        return healthStatusHostCfg;
    }
    getStatus(hostcfg_entity, type) {
        let healthStatusHostCfg;
        let status = healthStatus_entity_1.Status.UNKNOWN;
        let timestamp;
        let note = "";
        if ('status' in hostcfg_entity && hostcfg_entity.status !== null) {
            timestamp = new Date(hostcfg_entity.status.updatedAt).getTime();
            if (type === "data") {
                timestamp = new Date(hostcfg_entity.status.lastData).getTime();
            }
            if (new Date().getTime() - timestamp < this.LAST_DATA_UPDATE_LIMIT) {
                if (hostcfg_entity.status[type]) {
                    status = healthStatus_entity_1.Status.OK;
                }
                else {
                    status = healthStatus_entity_1.Status.ERROR;
                    note = hostcfg_entity.status.message;
                }
            }
            else {
                status = healthStatus_entity_1.Status.UNKNOWN;
                note = "No current data";
            }
        }
        else {
            status = healthStatus_entity_1.Status.UNKNOWN;
            note = "No data";
        }
        healthStatusHostCfg = {
            class: hostcfg_entity.hw_type_entity.class,
            hw_type: hostcfg_entity.hw_type_entity.hw_type,
            hw_type_label: hostcfg_entity.hw_type_entity.label,
            label: hostcfg_entity.label,
            item_id: hostcfg_entity.hostcfg_id,
            updatedAt: timestamp,
            note: note,
            status: status,
            ignored: hostcfg_entity.ignore_health_status,
            ignore_reason: hostcfg_entity.ignore_health_status_reason,
            url: URL_1.URL.generate(hostcfg_entity.hw_type_entity.hw_type, "health-status", hostcfg_entity.hostcfg_id),
        };
        return healthStatusHostCfg;
    }
    setStatusBySeverity(currentStatus, status) {
        if (!currentStatus) {
            currentStatus = status;
        }
        else {
            if (this.getStatusPriority(status) > this.getStatusPriority(currentStatus)) {
                currentStatus = status;
            }
        }
        return currentStatus;
    }
    getStatusPriority(status) {
        let statusPriority = new healthStatus_entity_1.StatusPriority();
        let prio = statusPriority.priority[status];
        if (status !== "unknown") {
            prio++;
        }
        return prio;
    }
    getArrayFromResponse(response, elmntType) {
        let arr = [];
        for (const elmnt of response) {
            if (elmnt !== undefined) {
                if (elmnt[elmntType]) {
                    arr.push(elmnt[elmntType]);
                }
            }
        }
        return arr;
    }
};
exports.HealthStatusService = HealthStatusService;
HealthStatusService.LAST_DATA_UPDATE_LIMIT = 60 * 60 * 1000;
exports.HealthStatusService = HealthStatusService = HealthStatusService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(1, (0, typeorm_1.InjectRepository)(hostcfg_entity_1.HostCfg)),
    __param(2, (0, typeorm_1.InjectRepository)(hostcfgRelation_entity_1.HostcfgRelation)),
    __param(3, (0, typeorm_1.InjectRepository)(hwType_entity_1.HwType)),
    __param(4, (0, typeorm_1.InjectRepository)(healthStatus_entity_1.HealthStatus)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        event_emitter_1.EventEmitter2])
], HealthStatusService);
//# sourceMappingURL=healthStatus.service.js.map