import { LdapGroupBase } from '@xormon/common/groupsResponse.dto';
import { LdapSave } from '@xormon/common/ldap.dto';
import { Group } from '@xormon/entity/group.entity';
import { GroupLdapGroup } from '@xormon/entity/groupLdapGroup.entity';
import { Ldap } from '@xormon/entity/ldap.entity';
import { LdapGroup } from '@xormon/entity/ldapGroup.entity';
import { Repository } from 'typeorm';
import { LdapCertService } from './ldap.cert.service';
export interface LdapGroupObject {
    cn: string;
    dn: string;
}
interface LdapWithGroups {
    ldap_id: string;
    groups: string[];
}
export declare class LdapService {
    private ldapCertService;
    private ldapRepository;
    private ldapGroupsRepository;
    private groupLdapGroupsRepository;
    static readonly logger: import("winston").Logger;
    private readonly log;
    private settings;
    constructor(ldapCertService: LdapCertService, ldapRepository: Repository<Ldap>, ldapGroupsRepository: Repository<LdapGroup>, groupLdapGroupsRepository: Repository<GroupLdapGroup>);
    getLdaps(): Promise<Ldap[]>;
    getLdap(ldap_id: string): Promise<Ldap>;
    saveLdap(ldapOrSave: LdapSave | Ldap): Promise<Ldap>;
    deleteLdap(ldap_id: string): Promise<boolean>;
    getLdapGroups(): Promise<LdapGroup[]>;
    getGroupsByLdapGroups(ldap_id: string, ldap_groups: string[]): Promise<Group[]>;
    updateGroupLdapGroups(group_id: number, ldap_groups: LdapGroupBase[]): Promise<({
        group_id: number;
        ldap_id: string;
        ldap_group: string;
    } & GroupLdapGroup)[]>;
    saveLdapGroups(ldapGroups: Partial<LdapGroup>[]): Promise<boolean>;
    deleteLdapGroups(ldap_id: string): Promise<void>;
    testLdapAllGroups(ldapData: LdapSave): Promise<LdapGroupObject[]>;
    private search;
    private createClient;
    private getBindDN;
    private getUserGroups;
    login(username: string, password: string): Promise<LdapWithGroups>;
    private handleLogin;
}
export {};
