"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LdapUtil = void 0;
class LdapUtil {
    constructor() {
        this.dnEscapeChars = {
            '\u005c': '\\\\',
            '\u0022': '\\"',
            '\u0023': '\\#',
            '\u002b': '\\+',
            '\u002c': '\\,',
            '\u003b': '\\;',
            '\u003c': '\\<',
            '\u003d': '\\=',
            '\u003e': '\\>',
        };
    }
    static escapeFilter(dn) {
        let str = dn;
        for (const unescaped in LdapUtil.filterEscapeChars) {
            const escaped = LdapUtil.filterEscapeChars[unescaped];
            str = str.replaceAll(unescaped, escaped);
        }
        return str;
    }
    static trimDN(dn) {
        let result = "";
        const RDNs = dn.split(/(?<!\\5c),/);
        for (const rdn of RDNs) {
            const pair = rdn.split(/(?<!\\5c)=/);
            result += ',' + pair[0].trim() + '=' + pair[1].trim();
        }
        return result.substring(1);
    }
}
exports.LdapUtil = LdapUtil;
LdapUtil.filterEscapeChars = {
    '\u005c': '\\5c',
    '\u0000': '\\00',
    '\u0028': '\\28',
    '\u0029': '\\29',
    '\u002a': '\\2a',
};
LdapUtil.dnExtraChars = {
    '\\5c\\22': /(?<!\\5c)\\5c"/g,
    '\\5c\\2c': /(?<!\\5c)\\5c,/g,
    '\\5c\\3b': /(?<!\\5c)\\5c;/g,
};
//# sourceMappingURL=ldap.util.js.map