"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GCloudOverviewPage = exports.GcloudDatabasePage = exports.GcloudComputePage = exports.GcloudDatabaseFolder = exports.GcloudComputeFolder = exports.GcloudRegiondatabasetotalsPage = exports.GcloudDatabasetotalsPage = exports.GcloudRegioncomputetotalsPage = exports.GcloudComputetotalsPage = exports.GcloudHeatmapPage = exports.GcloudDatabasetopPage = exports.GcloudComputetopPage = exports.GcloudRegionFolder = exports.GcloudProjectinstanceoverviewPage = exports.GcloudOverviewPage = exports.GcloudProjectFolder = exports.GcloudConfigurationPage = exports.GcloudSingleItemPage = exports.GcloudSubsystemFolder = exports.GcloudPage = exports.GcloudRootFolder = exports.GcloudFolder = void 0;
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const Overview_1 = require("./Overview");
const ACLCheck_1 = require("../../acl/ACLCheck");
const Table_1 = require("./Table");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const Top_1 = require("./Top");
const Heatmap_1 = require("./Heatmap");
const URL_1 = require("./URL");
const ConfigurationTable_1 = require("./ConfigurationTable");
const typeorm_1 = require("typeorm");
const AbstractMenu_1 = require("./AbstractMenu");
const Color_1 = require("../../metrics/Color");
const TabFactory_1 = require("./TabFactory");
class GcloudFolder extends AbstractMenu_1.AbstractFolder {
    constructor() {
        super(...arguments);
        this.hwType = 'gcloud';
    }
}
exports.GcloudFolder = GcloudFolder;
class GcloudRootFolder extends GcloudFolder {
    async _get() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration', 'configuration'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Compute TOP', this.hwType + '/compute-top', 'compute-top'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Database TOP', this.hwType + '/database-top', 'database-top'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Overview', this.hwType + '/overview', 'overview'));
        let projects = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'project'
            },
            relations: {
                children: true
            }
        });
        for (let project of projects) {
            if (project.children.length >= 1) {
                let path = this.hwType + '/project?ids=' + project.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(project.label, path, this.hwType, 'project', project.item_id, GcloudProjectFolder));
            }
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(...(await this._get()));
        return menuItems;
    }
    async getChildrenAcl() {
        return this._get();
    }
}
exports.GcloudRootFolder = GcloudRootFolder;
class GcloudPage extends AbstractMenu_1.AbstractPage {
    constructor() {
        super(...arguments);
        this.hwType = 'gcloud';
        this.rootFolder = GcloudRootFolder;
    }
}
exports.GcloudPage = GcloudPage;
class GcloudSubsystemFolder extends GcloudFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId;
        }
    }
}
exports.GcloudSubsystemFolder = GcloudSubsystemFolder;
class GcloudSingleItemPage extends GcloudPage {
}
exports.GcloudSingleItemPage = GcloudSingleItemPage;
class GcloudConfigurationPage extends GcloudPage {
    constructor() {
        super(...arguments);
        this.title = 'Configuration';
    }
    async getPage() {
        let title = 'Configuration';
        let tabs = [];
        let computes = await this.objectItemsRepository.findBy({ hw_type: 'gcloud', subsystem: 'compute' });
        let computeTable = new ConfigurationTable_1.ConfigurationTable(computes.map(compute => compute.item_id), ["status", "cpuPlatform", "ip", "size", "diskSize", "creationTimestamp"], ['project', 'region']);
        computeTable.setItemLabel("Compute name");
        computeTable.setAcl(this.request);
        tabs.push({
            title: 'Compute Engine',
            content: [await computeTable.generate()],
        });
        let databases = await this.objectItemsRepository.findBy({ hw_type: 'gcloud', subsystem: 'database' });
        let databaseTable = new ConfigurationTable_1.ConfigurationTable(databases.map(db => db.item_id), ["status", "ip", "databaseVersion", "size", "dataDiskType", "dataDiskSizeGb"], ['project', 'region']);
        databaseTable.setItemLabel("Database name");
        databaseTable.setAcl(this.request);
        tabs.push({
            title: 'Database Engine',
            content: [await databaseTable.generate()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.GcloudConfigurationPage = GcloudConfigurationPage;
class GcloudProjectFolder extends GcloudSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = "project";
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Compute Totals', this.hwType + '/compute-totals' + '?item_id=' + this.itemId, 'compute-totals', undefined, true));
        menuItems.push(new menuResponse_dto_1.MenuLink('Database Totals', this.hwType + '/database-totals' + '?item_id=' + this.itemId, 'database-totals', undefined, true));
        menuItems.push(new menuResponse_dto_1.MenuLink('Instance overview', this.hwType + '/project-instance-overview' + '?item_id=' + this.itemId, 'instance-overview', undefined, true));
        const children = await this.itemRelationsRepository.find({
            where: {
                parent: this.itemId.toString(),
                _child: {
                    subsystem: 'region'
                }
            },
            relations: {
                _child: true
            }
        });
        const items = [];
        for (const child of children) {
            items.push(child._child);
        }
        for (let item of items) {
            let path = this.hwType + '/region?ids=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, 'gcloud', 'region', item.item_id, GcloudRegionFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.GcloudProjectFolder = GcloudProjectFolder;
class GcloudOverviewPage extends GcloudPage {
    constructor() {
        super(...arguments);
        this.title = 'Overview';
    }
    async getPage() {
        let title = 'Overview';
        let tabs = [];
        let regions = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: "region",
            }
        });
        if (this.request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, regions.map(item => item.item_id));
            regions = regions.filter(item => ids.includes(item.item_id));
        }
        const projectRegionsConf = await this.itemPropertiesRepository.find({
            where: {
                item_id: (0, typeorm_1.In)(regions.map((region) => region.item_id)),
                property_name: (0, typeorm_1.In)(["running", "stopped"])
            },
            relations: {
                _item: {
                    hostcfg_relations: {
                        _hostcfg: true
                    }
                }
            }
        });
        let projectObject = {};
        for (const row of projectRegionsConf) {
            if (!projectObject[row.item_id]) {
                projectObject[row.item_id] = {
                    running: 0,
                    stopped: 0,
                    item: row._item
                };
                projectObject[row.item_id][row.property_name] = +row.property_value;
            }
        }
        let overviewTable = new Table_1.Table()
            .addHeader({ value: 'Project' })
            .addHeader({ value: 'Region' })
            .addHeader({ value: 'Running' })
            .addHeader({ value: 'Stopped' });
        for (const itemId in projectObject) {
            overviewTable.addRow([
                { order: 0, value: projectObject[itemId].item.hostcfg_relations[0]._hostcfg.label },
                { order: 1, value: projectObject[itemId].item.label },
                { order: 2, value: projectObject[itemId].running },
                { order: 3, value: projectObject[itemId].stopped }
            ]);
        }
        overviewTable.sortByIndex(2);
        tabs.push({
            title: "Compute Overview",
            content: [
                overviewTable.generateBox()
            ]
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.GcloudOverviewPage = GcloudOverviewPage;
class GcloudProjectinstanceoverviewPage extends GcloudPage {
    constructor() {
        super(...arguments);
        this.title = 'Overview';
        this.withItemId = true;
    }
    async getPage() {
        let title = 'Overview';
        let tabs = [];
        let regions = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: "region",
                parents: {
                    parent: this.itemId
                }
            }
        });
        if (this.request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, regions.map(item => item.item_id));
            regions = regions.filter(item => ids.includes(item.item_id));
        }
        const projectRegionsConf = await this.itemPropertiesRepository.find({
            where: {
                item_id: (0, typeorm_1.In)(regions.map((region) => region.item_id)),
                property_name: (0, typeorm_1.In)(["running", "stopped"])
            },
            relations: {
                _item: true
            }
        });
        let projectObject = {};
        for (const row of projectRegionsConf) {
            if (!projectObject[row.item_id]) {
                projectObject[row.item_id] = {
                    running: 0,
                    stopped: 0,
                    item: row._item
                };
                projectObject[row.item_id][row.property_name] = +row.property_value;
            }
        }
        let overviewTable = new Table_1.Table()
            .addHeader({ value: 'Region' })
            .addHeader({ value: 'Running' })
            .addHeader({ value: 'Stopped' });
        for (const itemId in projectObject) {
            overviewTable.addRow([
                { order: 0, value: projectObject[itemId].item.label },
                { order: 1, value: projectObject[itemId].running },
                { order: 2, value: projectObject[itemId].stopped }
            ]);
        }
        tabs.push({
            title: "Compute Overview",
            content: [
                overviewTable.generateBox()
            ]
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.GcloudProjectinstanceoverviewPage = GcloudProjectinstanceoverviewPage;
class GcloudRegionFolder extends GcloudSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        const computes = await this.itemRelationsRepository.find({
            where: {
                parent: this.itemId.toString(),
                _child: {
                    subsystem: 'compute'
                }
            }
        });
        const databases = await this.itemRelationsRepository.find({
            where: {
                parent: this.itemId.toString(),
                _child: {
                    subsystem: 'database'
                }
            }
        });
        const subsystems = [{
                "label": "Google Cloud Compute",
                "subsystem": "compute"
            }, {
                "label": "Google Cloud Database",
                "subsystem": "database"
            },
        ];
        const menuLinks = [];
        const menuFolders = [];
        for (const subsystemIndex in subsystems) {
            if (subsystems[subsystemIndex].subsystem === "compute") {
                let path = this.hwType + '/' + subsystems[subsystemIndex].subsystem + '?ids=' + this.itemId.toString();
                menuLinks.push(new menuResponse_dto_1.MenuLink('Compute Totals', this.hwType + '/region-compute-totals?item_id=' + this.itemId, 'region-totals', undefined, true));
                if (computes.length >= 1) {
                    menuFolders.push(new menuResponse_dto_1.MenuFolder(subsystems[subsystemIndex].label, path, this.hwType, subsystems[subsystemIndex].subsystem, this.itemId.toString(), GcloudComputeFolder));
                }
            }
            if (subsystems[subsystemIndex].subsystem === "database" && databases.length >= 1) {
                let path = this.hwType + '/' + subsystems[subsystemIndex].subsystem + '?ids=' + this.itemId.toString();
                menuLinks.push(new menuResponse_dto_1.MenuLink('Database Totals', this.hwType + '/region-database-totals?item_id=' + this.itemId, 'region-totals', undefined, true));
                menuFolders.push(new menuResponse_dto_1.MenuFolder(subsystems[subsystemIndex].label, path, this.hwType, subsystems[subsystemIndex].subsystem, this.itemId.toString(), GcloudDatabaseFolder));
            }
        }
        menuItems.push(...menuLinks);
        menuItems.push(...menuFolders);
        return menuItems;
    }
}
exports.GcloudRegionFolder = GcloudRegionFolder;
class GcloudComputetopPage extends GcloudPage {
    constructor() {
        super(...arguments);
        this.title = 'Compute TOP';
    }
    async getPage() {
        const top = new Top_1.Top();
        top.setSubsystem('gcloud', 'compute');
        top.setParentSubsystem('region');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.GcloudComputetopPage = GcloudComputetopPage;
class GcloudDatabasetopPage extends GcloudPage {
    constructor() {
        super(...arguments);
        this.title = 'Database TOP';
    }
    async getPage() {
        const top = new Top_1.Top();
        top.setSubsystem('gcloud', 'database');
        top.setParentSubsystem('region');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.GcloudDatabasetopPage = GcloudDatabasetopPage;
class GcloudHeatmapPage extends GcloudPage {
    constructor() {
        super(...arguments);
        this.title = 'Heatmap';
    }
    async getPage() {
        let title = 'Heatmap';
        let tabs = [];
        tabs.push({
            title: 'Compute',
            content: Object.values(await new Heatmap_1.Heatmap('gcloud', 'compute', true)
                .computedHeatmapBox(class_constants_1.ClassKey.CLOUD, this.request)),
        });
        let computeTable = new Table_1.Table()
            .addHeader({ value: "Project" })
            .addHeader({ value: 'Region' })
            .addHeader({ value: 'Compute' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let computes = await this.objectItemsRepository.find({
            where: {
                hw_type: 'gcloud',
                subsystem: 'compute',
                parents: {
                    _parent: {
                        subsystem: 'region'
                    }
                }
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                },
                hostcfg_relations: {
                    _hostcfg: true
                }
            }
        });
        if (this.request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, computes.map(item => item.item_id));
            computes = computes.filter(item => ids.includes(item.item_id));
        }
        for (const compute of computes) {
            const region = compute.parents.find(parent => parent._parent.subsystem === 'region');
            const cpu = compute.heatmaps.find(heatmap => heatmap.metric === 'cpu_util');
            const project = compute.hostcfg_relations[0]._hostcfg;
            computeTable.addRow([
                { order: 0, value: project.label },
                { order: 1, value: region._parent.label },
                { order: 2, value: compute.label, url: URL_1.URL.generate('gcloud', 'compute', compute.item_id) },
                { order: 3, value: cpu ? cpu.value : null },
            ]);
        }
        computeTable.sortByIndex(3);
        tabs.push({
            title: 'Compute table',
            content: [computeTable.generateBox()],
        });
        tabs.push({
            title: 'Database',
            content: Object.values(await new Heatmap_1.Heatmap('gcloud', 'database', true)
                .computedHeatmapBox(class_constants_1.ClassKey.CLOUD, this.request)),
        });
        let databaseTable = new Table_1.Table()
            .addHeader({ value: 'Project' })
            .addHeader({ value: 'Region' })
            .addHeader({ value: 'Database' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let databases = await this.objectItemsRepository.find({
            where: {
                hw_type: 'gcloud',
                subsystem: 'database',
                parents: {
                    _parent: {
                        subsystem: 'region'
                    }
                }
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                },
                hostcfg_relations: {
                    _hostcfg: true
                }
            }
        });
        if (this.request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, databases.map(item => item.item_id));
            databases = databases.filter(item => ids.includes(item.item_id));
        }
        for (const database of databases) {
            const cpu = database.heatmaps.find(heatmap => heatmap.metric === 'cpu_util');
            const region = database.parents.find(parent => parent._parent.subsystem === 'region');
            const project = database.hostcfg_relations[0]._hostcfg;
            databaseTable.addRow([
                { order: 0, value: project.label },
                { order: 1, value: region._parent.label },
                { order: 2, value: database.label ? database.label : null, url: database ? URL_1.URL.generate('gcloud', 'database', database.item_id) : undefined },
                { order: 3, value: cpu ? cpu.value : null },
            ]);
        }
        databaseTable.sortByIndex(3);
        tabs.push({
            title: 'Database table',
            content: [databaseTable.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.GcloudHeatmapPage = GcloudHeatmapPage;
const getComputeTotalsTabs = (computes, subsystem) => {
    const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.CLOUD, hwTypeKey_constants_1.HwTypeKey.GCLOUD, subsystem, computes);
    tabFactory.addTab("CPU", { metric: "cpu_util", default: TabFactory_1.MPDefault.LINE_AGGR }, undefined, undefined, true);
    tabFactory.addTab("Disk", [
        { metric: "read_bytes", default: TabFactory_1.MPDefault.READ_LINE_AGGR },
        { metric: "write_bytes", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR }
    ], undefined, undefined, true);
    tabFactory.addTab("LAN", [
        { metric: "received_bytes", default: TabFactory_1.MPDefault.READ_LINE_AGGR },
        { metric: "sent_bytes", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR }
    ], undefined, undefined, true);
    tabFactory.addTab("IO", [
        { metric: "read_ops", default: TabFactory_1.MPDefault.READ_LINE_AGGR },
        { metric: "write_ops", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR }
    ], undefined, undefined, true);
    return tabFactory.getTabs();
};
class GcloudComputetotalsPage extends GcloudSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'project';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let regionIds = (await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: "region",
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        })).map((item) => item.item_id);
        let computes = (await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'compute',
                parents: {
                    parent: (0, typeorm_1.In)(regionIds)
                }
            },
            relations: {
                parents: true
            }
        })).map(item => item.item_id);
        let title = 'Region ' + (item !== null ? item.label : this.itemId);
        this.page = new pageResponse_dto_1.Page(title, getComputeTotalsTabs(computes, "compute"));
        return this.page;
    }
}
exports.GcloudComputetotalsPage = GcloudComputetotalsPage;
class GcloudRegioncomputetotalsPage extends GcloudSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'region';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let computes = (await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'compute',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        })).map(item => item.item_id);
        let title = 'Region ' + (item !== null ? item.label : this.itemId);
        this.page = new pageResponse_dto_1.Page(title, getComputeTotalsTabs(computes, "compute"));
        return this.page;
    }
}
exports.GcloudRegioncomputetotalsPage = GcloudRegioncomputetotalsPage;
const getDatabaseTotalsTabs = (databases, subsystem) => {
    const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.CLOUD, hwTypeKey_constants_1.HwTypeKey.GCLOUD, subsystem, databases);
    tabFactory.addTab("CPU", { metric: "cpu_util", default: TabFactory_1.MPDefault.LINE_AGGR }, undefined, undefined, true);
    tabFactory.addTab("IO", [
        { metric: "read_ops", default: TabFactory_1.MPDefault.READ_LINE_AGGR },
        { metric: "write_ops", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR }
    ], undefined, undefined, true);
    tabFactory.addTab("LAN", [
        { metric: "received_bytes", default: TabFactory_1.MPDefault.READ_LINE_AGGR },
        { metric: "sent_bytes", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR }
    ], undefined, undefined, true);
    tabFactory.addTab("Connections", { metric: "connections", default: TabFactory_1.MPDefault.LINE_AGGR }, undefined, undefined, true);
    tabFactory.addTab("Queries", { metric: "queries", default: TabFactory_1.MPDefault.LINE_AGGR }, undefined, undefined, true);
    tabFactory.addTab("Questions", { metric: "questions", default: TabFactory_1.MPDefault.LINE_AGGR }, undefined, undefined, true);
    tabFactory.addTab("Transactions", { metric: "transaction_count", default: TabFactory_1.MPDefault.LINE_AGGR }, undefined, undefined, true);
    tabFactory.addTab("InnoDB IO", [
        { metric: "innodb_read", default: TabFactory_1.MPDefault.READ_LINE_AGGR },
        { metric: "innodb_write", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR }
    ], undefined, undefined, true);
    tabFactory.addTab("InnoDB buffer", [
        { metric: "innodb_buffer_free", default: TabFactory_1.MPDefault.LINE_AGGR },
        { metric: "innodb_buffer_total", default: TabFactory_1.MPDefault.LINE_AGGR }
    ], undefined, undefined, true);
    tabFactory.addTab("InnoDB Fsyncs", [
        { metric: "innodb_data_fsyncs", default: TabFactory_1.MPDefault.READ_LINE_AGGR },
        { metric: "innodb_os_fsyncs", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR }
    ], undefined, undefined, true);
    return tabFactory.getTabs();
};
class GcloudDatabasetotalsPage extends GcloudSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'project';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let regionIds = (await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: "region",
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        })).map((item) => item.item_id);
        let databases = (await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'database',
                parents: {
                    parent: (0, typeorm_1.In)(regionIds)
                }
            },
            relations: {
                parents: true
            }
        })).map(item => item.item_id);
        let title = 'Region ' + (item !== null ? item.label : this.itemId);
        this.page = new pageResponse_dto_1.Page(title, getDatabaseTotalsTabs(databases, "database"));
        return this.page;
    }
}
exports.GcloudDatabasetotalsPage = GcloudDatabasetotalsPage;
class GcloudRegiondatabasetotalsPage extends GcloudSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'region';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let databases = (await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'database',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        })).map(item => item.item_id);
        let title = 'Region ' + (item !== null ? item.label : this.itemId);
        this.page = new pageResponse_dto_1.Page(title, getDatabaseTotalsTabs(databases, "database"));
        return this.page;
    }
}
exports.GcloudRegiondatabasetotalsPage = GcloudRegiondatabasetotalsPage;
class GcloudComputeFolder extends GcloudSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let children = await this.itemRelationsRepository.find({
            relations: {
                _child: true
            },
            where: {
                parent: this.itemId.toString(),
                _child: {
                    subsystem: 'compute'
                }
            }
        });
        const items = [];
        for (const child of children) {
            items.push(child._child);
        }
        for (let item of items) {
            let path = this.hwType + '/compute?item_id=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, 'compute', item.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.GcloudComputeFolder = GcloudComputeFolder;
class GcloudDatabaseFolder extends GcloudSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let children = await this.itemRelationsRepository.find({
            relations: {
                _child: true
            },
            where: {
                parent: this.itemId.toString(),
                _child: {
                    subsystem: 'database'
                }
            }
        });
        const items = [];
        for (const child of children) {
            items.push(child._child);
        }
        for (let item of items) {
            let path = this.hwType + '/database?item_id=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, 'database', item.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.GcloudDatabaseFolder = GcloudDatabaseFolder;
class GcloudComputePage extends GcloudSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'compute';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Compute ' + (item !== null ? item.label : this.itemId);
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.CLOUD, hwTypeKey_constants_1.HwTypeKey.GCLOUD, "compute", [this.itemId]);
        tabFactory.addTab("CPU", { metric: "cpu_util", default: TabFactory_1.MPDefault.LINE });
        tabFactory.addTab("Disk", [
            { metric: "read_bytes", default: TabFactory_1.MPDefault.LINE },
            { metric: "write_bytes", default: TabFactory_1.MPDefault.LINE }
        ]);
        tabFactory.addTab("LAN", [
            { metric: "received_bytes", default: TabFactory_1.MPDefault.READ_LINE },
            { metric: "sent_bytes", default: TabFactory_1.MPDefault.WRITE_LINE }
        ]);
        tabFactory.addTab("IO", [
            { metric: "read_ops", default: TabFactory_1.MPDefault.READ_LINE },
            { metric: "write_ops", default: TabFactory_1.MPDefault.WRITE_LINE }
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabFactory.getTabs());
        return this.page;
    }
}
exports.GcloudComputePage = GcloudComputePage;
class GcloudDatabasePage extends GcloudSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'database';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Database ' + (item !== null ? item.label : this.itemId);
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.CLOUD, hwTypeKey_constants_1.HwTypeKey.GCLOUD, "database", [this.itemId]);
        tabFactory.addTab("CPU", { metric: "cpu_util", default: TabFactory_1.MPDefault.LINE });
        tabFactory.addTab("Disk Capacity", [
            { metric: "disk_used", color: Color_1.Color.pallete.red, default: TabFactory_1.MPDefault.AREA },
            { metric: "disk_quota", color: Color_1.Color.pallete.blue, default: TabFactory_1.MPDefault.AREA }
        ]);
        tabFactory.addTab("IO", [
            { metric: "read_ops", default: TabFactory_1.MPDefault.READ_LINE },
            { metric: "write_ops", default: TabFactory_1.MPDefault.WRITE_LINE }
        ]);
        tabFactory.addTab("LAN", [
            { metric: "received_bytes", default: TabFactory_1.MPDefault.READ_LINE },
            { metric: "sent_bytes", default: TabFactory_1.MPDefault.WRITE_LINE }
        ]);
        tabFactory.addTab("Memory", [
            { metric: "mem_used", color: Color_1.Color.pallete.red, default: TabFactory_1.MPDefault.AREA },
            { metric: "mem_total", color: Color_1.Color.pallete.blue, default: TabFactory_1.MPDefault.AREA }
        ]);
        tabFactory.addTab("Connections", { metric: "connections", color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE });
        tabFactory.addTab("Queries", { metric: "queries", color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE });
        tabFactory.addTab("Questions", { metric: "questions", color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE });
        tabFactory.addTab("Transactions", { metric: "transaction_count", color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE });
        tabFactory.addTab("InnoDB IO", [
            { metric: "innodb_read", default: TabFactory_1.MPDefault.READ_LINE },
            { metric: "innodb_write", default: TabFactory_1.MPDefault.WRITE_LINE }
        ]);
        tabFactory.addTab("InnoDB buffer", [
            { metric: "innodb_buffer_free", color: Color_1.Color.metric.free, default: TabFactory_1.MPDefault.AREA },
            { metric: "innodb_buffer_total", color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.AREA }
        ]);
        tabFactory.addTab("InnoDB Fsyncs", [
            { metric: "innodb_data_fsyncs", color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: "innodb_os_fsyncs", color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE }
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabFactory.getTabs());
        return this.page;
    }
}
exports.GcloudDatabasePage = GcloudDatabasePage;
class GCloudOverviewPage extends GcloudPage {
    constructor() {
        super(...arguments);
        this.title = 'Overview';
    }
    async getPage() {
        let title = 'Overview';
        let tabs = [];
        const overview = new Overview_1.Overview('gcloud');
        let computes = await this.objectItemsRepository.find({
            where: {
                hw_type: 'gcloud',
                subsystem: 'compute'
            }
        });
        computes = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, computes);
        overview.addCategory('Compute', computes.map(item => {
            return {
                label: item.label,
                item_id: item.item_id
            };
        }));
        tabs.push({
            title: 'Compute',
            content: [
                overview.form()
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.GCloudOverviewPage = GCloudOverviewPage;
//# sourceMappingURL=GCloud.js.map