import { MenuItem, Newable } from '@xormon/common/menuResponse.dto';
import { ChartType, Page, TextBox } from '@xormon/common/pageResponse.dto';
import { Request } from 'express';
import { StorAbstractFolder, StorUniversalGroupAbstractFolder, UniversalGroupAbstractPage, UniversalGroupStorAbstractPage } from './AbstractMenu';
import { ServerGroup } from '@xormon/entity/serverGroup.entity';
import { ServerGroupItems } from '@xormon/entity/serverGroupItems.entity';
export declare const lanGlobalPages: () => MenuItem[];
export declare class LanFolder extends StorUniversalGroupAbstractFolder {
    serverGroupsRepository: import("typeorm").Repository<ServerGroup>;
    serverGroupItemsRepository: import("typeorm").Repository<ServerGroupItems>;
}
export declare class LanSubsystemFolder extends LanFolder {
    subsystem: string;
    itemId: string;
    itemIds: string[];
    constructor(hwType: string, subsystem?: string, itemId?: string[] | string);
}
export declare class LanRootFolder extends LanFolder {
    constructor(hw_type: string, sus?: string, request?: Request);
    getChildren(): Promise<MenuItem[]>;
}
export declare class LanDeviceFolder extends LanSubsystemFolder {
    subsystem: string;
    private groupId?;
    constructor(hwType: string, subsystem: string, itemId: string);
    getChildren(): Promise<MenuItem[]>;
}
export declare class LanSubsystemItemsFolder extends LanSubsystemFolder {
    private groupId;
    constructor(hwType: string, subsystem?: string, itemId?: string);
    getChildren(): Promise<any[]>;
}
export declare class LanSimplePage extends UniversalGroupAbstractPage {
    withItemId: boolean;
    rootFolder: Newable<StorAbstractFolder>;
}
export declare class LanPage extends UniversalGroupStorAbstractPage {
    withItemId: boolean;
    rootFolder: Newable<StorAbstractFolder>;
}
export declare class LanAbstractSingleItemPage extends UniversalGroupStorAbstractPage {
    withItemId: boolean;
    rootFolder: Newable<StorAbstractFolder>;
}
export declare abstract class LanSingleItemPage extends LanAbstractSingleItemPage {
    subsystem: string;
    parentSubsystem: string;
    withItemId: boolean;
}
export declare class LanItemPage extends LanSingleItemPage {
    pageType: string;
    withItemId: boolean;
    constructor(hwType: string, pageType: string, itemId: string, request: Request);
    getPage(): Promise<Page>;
}
export declare class LanTotalsPage extends LanSimplePage {
    title: string;
    type: string;
    withItemId: boolean;
    constructor(pageType: string, itemId: string, request?: Request, group?: boolean);
    getPage(): Promise<Page>;
}
export declare class LanHeatmapPage extends LanSimplePage {
    title: string;
    getTab(tab: string): Promise<{
        title: string;
        content: any[];
        subTabs: ({
            title: string;
            content: (TextBox | {
                type: "HeatmapBox";
                metrics: import("@xormon/common/pageResponse.dto").IHeatmap[];
            })[];
        } | {
            title: string;
            content: import("@xormon/common/pageResponse.dto").TableBox[];
        })[];
    }>;
    getPage(): Promise<Page>;
}
export declare class LanGrouptotalsPage extends LanSimplePage {
    title: string;
    type: string;
    withItemId: boolean;
    constructor(pageType: string, itemId: string, request?: Request, group?: boolean);
    getPage(): Promise<Page>;
}
export declare class LanDataPage extends LanTotalsPage {
    constructor(itemId: string, request?: Request);
}
export declare class LanPacketPage extends LanTotalsPage {
    constructor(itemId: string, request?: Request);
}
export declare class LanCpumemoryPage extends LanTotalsPage {
    constructor(itemId: string, request?: Request);
}
export declare class LanVlantotalPage extends LanTotalsPage {
    constructor(itemId: string, request?: Request);
}
export declare class LanPortchanneltotalPage extends LanTotalsPage {
    constructor(itemId: string, request?: Request);
}
export declare class LanGroupdataPage extends LanGrouptotalsPage {
    constructor(itemId: string, request?: Request);
}
export declare class LanGrouppacketPage extends LanGrouptotalsPage {
    constructor(itemId: string, request?: Request);
}
export declare class LanGroupcpumemoryPage extends LanGrouptotalsPage {
    constructor(itemId: string, request?: Request);
}
export declare class LanGroupvlantotalPage extends LanGrouptotalsPage {
    constructor(itemId: string, request?: Request);
}
export declare class LanGroupportchanneltotalPage extends LanGrouptotalsPage {
    constructor(itemId: string, request?: Request);
}
export declare class LanMetrics {
    static Metric: {
        new (metric: string): {
            getGraph(type: ChartType, stacked?: boolean, order?: number, color?: string): any;
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            aggregate?: import("@xormon/common/pageResponse.dto").MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            color?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
}
export declare class LanHealthstatusPage extends LanSimplePage {
    title: string;
    itemId: string;
    constructor(itemId: string, request: Request);
    getPage(): Promise<Page>;
}
export declare class LanConfigurationPage extends LanSimplePage {
    title: string;
    itemId: string;
    constructor(itemId: string, request: Request);
    getPage(): Promise<Page>;
}
export declare class LanGroupconfigurationPage extends LanSimplePage {
    getPage(): Promise<Page>;
}
export declare class LanGroupFolder extends LanSubsystemFolder {
    subsystem: string;
    getChildren(request?: Request, useFirst?: boolean): Promise<any[]>;
}
