"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.LinuxServergroupPage = exports.LinuxServergrouptotalsPage = exports.LinuxGroupFolder = exports.LinuxgroupsPage = exports.LinuxgroupsSubsystemFolder = exports.LinuxgroupsFolder = exports.LinuxOldMetrics = exports.LinuxMetrics = exports.LinuxServerPage = exports.LinuxTopPage = exports.LinuxHeatmapPage = exports.LinuxGroupconfigurationPage = exports.LinuxTotalsPage = exports.LinuxStatusPage = exports.LinuxSingleItemPage = exports.LinuxPage = exports.LinuxRootFolder = exports.LinuxSubsystemFolder = exports.LinuxFolder = exports.getSelfMonitoringAgent = exports.getLinuxServers = exports.getLinuxMappedChildren = exports.getLinuxCustomGroupSubsystems = exports.getLinuxCustomGroupMetrics = exports.getLinuxExcludedIds = exports.getLinuxMappedIds = exports.getLinuxMapping = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const dataSource_1 = require("../../../config/dataSource");
const itemProperty_entity_1 = require("../../../entity/itemProperty.entity");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const agentMapping_entity_1 = require("../../../entity/agentMapping.entity");
const Color_1 = require("../../metrics/Color");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const Table_1 = require("./Table");
const URL_1 = require("./URL");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const ItemPropertiesWrapper_1 = require("../../configuration/ItemPropertiesWrapper");
const Heatmap_1 = require("./Heatmap");
const ACLCheck_1 = require("../../acl/ACLCheck");
const typeorm_1 = require("typeorm");
const Xormon_1 = require("./Xormon");
const AbstractMenu_1 = require("./AbstractMenu");
const serverGroup_entity_1 = require("../../../entity/serverGroup.entity");
const serverGroupItems_entity_1 = require("../../../entity/serverGroupItems.entity");
const User_1 = require("../../users/User");
const Top_1 = require("./Top");
const getLinuxMapping = async () => {
    return await (0, dataSource_1.getDataSource)().getRepository(agentMapping_entity_1.AgentMapping).find();
};
exports.getLinuxMapping = getLinuxMapping;
const getLinuxMappedIds = async (linuxId) => {
    return (await (0, dataSource_1.getDataSource)().getRepository(agentMapping_entity_1.AgentMapping).find())
        .map(mapping => linuxId ? mapping.agent_id : mapping.item_id);
};
exports.getLinuxMappedIds = getLinuxMappedIds;
const getLinuxExcludedIds = async () => {
    let excluded = await (0, exports.getLinuxMappedIds)(true);
    const selfMonitoringServer = await (0, exports.getSelfMonitoringAgent)();
    if (selfMonitoringServer) {
        excluded.push(selfMonitoringServer.item_id);
    }
    return excluded;
};
exports.getLinuxExcludedIds = getLinuxExcludedIds;
const getLinuxCustomGroupMetrics = () => {
    return {
        server: [
            {
                ...MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, 'linux', 'server', 'mem_used')
            }
        ],
        lan: [
            {
                ...MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, 'linux', 'lan', 'recv_bytes')
            },
            {
                ...MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, 'linux', 'lan', 'trans_bytes')
            }
        ],
        san: [
            {
                ...MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, 'linux', 'san', 'recv_bytes')
            },
            {
                ...MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, 'linux', 'san', 'trans_bytes')
            },
            {
                ...MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, 'linux', 'san', 'iops_in')
            },
            {
                ...MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, 'linux', 'san', 'iops_out')
            }
        ],
    };
};
exports.getLinuxCustomGroupMetrics = getLinuxCustomGroupMetrics;
const getLinuxCustomGroupSubsystems = () => {
    return [
        {
            subsystem: 'server',
            label: 'Server'
        },
        {
            subsystem: 'lan',
            label: 'Interface'
        },
        {
            subsystem: 'san',
            label: 'Interface'
        }
    ];
};
exports.getLinuxCustomGroupSubsystems = getLinuxCustomGroupSubsystems;
const getLinuxMappedChildren = async (itemIds, subsystem) => {
    return (await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
        where: {
            subsystem: subsystem,
            parents: {
                parent: (0, typeorm_1.In)(itemIds)
            }
        },
        relations: {
            parents: true
        }
    })).map(item => {
        return {
            item_id: item.item_id,
            label: item.label
        };
    });
};
exports.getLinuxMappedChildren = getLinuxMappedChildren;
const getLinuxServers = async (itemIds) => {
    return (await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findBy({
        item_id: (0, typeorm_1.In)(itemIds)
    })).map(item => {
        return {
            item_id: item.item_id,
            label: item.label
        };
    });
};
exports.getLinuxServers = getLinuxServers;
const getSelfMonitoringAgent = async () => {
    return await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findOne({
        relations: {
            properties: true
        },
        where: {
            hw_type: 'linux',
            subsystem: 'server',
            properties: {
                property_name: 'ip_address',
                property_value: (0, typeorm_1.In)(['localhost', '127.0.0.1'])
            }
        }
    });
};
exports.getSelfMonitoringAgent = getSelfMonitoringAgent;
class LinuxFolder extends AbstractMenu_1.AbstractFolder {
    constructor() {
        super(...arguments);
        this.hwType = 'linux';
        this.agentMappingRepository = (0, dataSource_1.getDataSource)().getRepository(agentMapping_entity_1.AgentMapping);
    }
}
exports.LinuxFolder = LinuxFolder;
class LinuxSubsystemFolder extends LinuxFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId;
        }
    }
}
exports.LinuxSubsystemFolder = LinuxSubsystemFolder;
class LinuxRootFolder extends LinuxFolder {
    constructor() {
        super(...arguments);
        this.serverGroupsRepository = (0, dataSource_1.getDataSource)().getRepository(serverGroup_entity_1.ServerGroup);
        this.serverGroupItemsRepository = (0, dataSource_1.getDataSource)().getRepository(serverGroupItems_entity_1.ServerGroupItems);
    }
    async getChildren(request) {
        let menuItems = [];
        let subsystem = 'server';
        const user = request ? await User_1.User.getUserByRequest(request) : null;
        const sgItems = user
            ? (await this.serverGroupItemsRepository.find({
                where: {
                    _server_group: {
                        user_id: user.user_id,
                        hw_type: "linux"
                    },
                    isPrimary: true
                },
                relations: {
                    _server_group: true
                }
            })).map(sg => sg.item_id)
            : (await this.serverGroupItemsRepository.find({
                where: {
                    _server_group: {
                        hw_type: "linux"
                    },
                    isPrimary: true
                },
                relations: {
                    _server_group: true
                }
            })).map(sg => sg.item_id);
        let servers = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: subsystem,
            item_id: (0, typeorm_1.Not)((0, typeorm_1.In)((await (0, exports.getLinuxExcludedIds)()).concat(sgItems)))
        });
        const server_ids = servers.map((item) => item.item_id);
        const mappings = await this.agentMappingRepository.findBy({ agent_id: (0, typeorm_1.In)(server_ids) });
        const mapped_server_ids = mappings.map((mapping) => mapping.agent_id);
        servers = servers.filter((item) => !mapped_server_ids.includes(item.item_id));
        if (servers.length >= 1 || sgItems.length >= 1) {
            menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/totals', 'totals'));
            menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
            menuItems.push(new menuResponse_dto_1.MenuLink('TOP', this.hwType + '/top', 'top'));
            menuItems.push(new menuResponse_dto_1.MenuLink('Groups Configuration', this.hwType + '/groupconfiguration', 'configuration'));
        }
        if (user) {
            let serverGroups = await this.serverGroupsRepository.find({
                where: { user_id: user.user_id, hw_type: "linux" },
                relations: {
                    server_group_items: true
                }
            });
            if (serverGroups.length) {
                for (const group of serverGroups.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }))) {
                    if (group && group?.server_group_items.length) {
                        let path = "linux" + '/group?ids=' + group.server_group_id;
                        menuItems.push(new menuResponse_dto_1.MenuFolder(group.label, path, "linux", 'group', group.server_group_id));
                    }
                }
            }
        }
        servers.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let server of servers) {
            let path = this.hwType + '/server?item_id=' + server.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(server.label, path, 'server', server.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        const menuItems = [];
        let servers = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: "server",
            item_id: (0, typeorm_1.Not)((0, typeorm_1.In)((await (0, exports.getLinuxExcludedIds)())))
        });
        for (let server of servers) {
            let path = this.hwType + '/server?item_id=' + server.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(server.label, path, 'server', server.item_id));
        }
        return menuItems;
    }
}
exports.LinuxRootFolder = LinuxRootFolder;
class LinuxPage extends AbstractMenu_1.AbstractPage {
    constructor() {
        super(...arguments);
        this.hwType = 'linux';
        this.rootFolder = LinuxRootFolder;
        this.agentMappingRepository = (0, dataSource_1.getDataSource)().getRepository(agentMapping_entity_1.AgentMapping);
    }
    async getPath() {
        return await new LinuxRootFolder().getChildren(this.request);
    }
}
exports.LinuxPage = LinuxPage;
class LinuxSingleItemPage extends LinuxPage {
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.LinuxSingleItemPage = LinuxSingleItemPage;
class LinuxStatusPage extends LinuxPage {
    async getPage() {
        let tabs = [];
        let table = new Table_1.Table()
            .addHeader({ value: 'Server' })
            .addHeader({ value: 'Agent type' })
            .addHeader({ value: 'Agent version' })
            .addHeader({ value: 'Last perf data', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.timestamp })
            .addHeader({ value: 'IP Address' });
        let servers = await this.objectItemsRepository.find({
            where: {
                hw_type: (0, typeorm_1.In)(['linux', 'as400', 'windows']),
                subsystem: 'server',
            }, relations: {
                properties: true,
                server_group_items: {
                    _server_group: true
                }
            }
        });
        servers.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        let ids = servers.map(item => item.item_id);
        let lastTimestamps = await (new MetricsWrapper_1.MetricsWrapper([], ids)).getLastTimestampFromTable();
        let idsToDoublecheck = ids.filter(id => Object.keys(lastTimestamps).indexOf(id) === -1);
        if (idsToDoublecheck.length > 0) {
            lastTimestamps = { ...lastTimestamps, ...await (new MetricsWrapper_1.MetricsWrapper([], idsToDoublecheck)).getLastTimestampFromTable('1d') };
        }
        const mappedIds = await (0, exports.getLinuxMappedIds)(true);
        const linuxMapping = await (0, exports.getLinuxMapping)();
        const user = await User_1.User.getUserByRequest(this.request);
        for (const server of servers) {
            let url = "";
            if (server.server_group_items && server.server_group_items[0]) {
                url = new URL_1.URL().generateFromObject(server, undefined, undefined, user);
            }
            else {
                url = URL_1.URL.generate(server.hw_type, server.subsystem, server.item_id);
            }
            let prefix = server.item_id.split('_');
            let agentType = "";
            if (prefix.length > 0 && prefix[0].toLowerCase() === 'linux') {
                agentType = "Linux";
            }
            else if (prefix.length > 0 && prefix[0].toLowerCase() === 'power') {
                agentType = "IBM Power";
            }
            else if (server.hw_type === 'windows') {
                agentType = "Windows / Hyper-V";
            }
            else if (server.hw_type === 'as400') {
                agentType = "IBM i";
            }
            if (mappedIds.includes(server.item_id)) {
                const mapping = linuxMapping.find(item => item.agent_id === server.item_id);
                if (mapping) {
                    const objectItem = await this.objectItemsRepository.findOne({ where: { item_id: mapping.item_id } });
                    if (objectItem) {
                        const newURL = URL_1.URL.generate(objectItem.hw_type, objectItem.subsystem, objectItem.item_id);
                        url = newURL;
                    }
                }
            }
            const versionProp = server.properties.find(property => property.property_name === 'version');
            const ipAddressProp = server.properties.find(property => property.property_name === 'ip_address');
            table.addRow([
                { order: 0, value: server.label, url: url },
                { order: 1, value: agentType },
                { order: 2, value: versionProp ? versionProp.property_value : ' ' },
                { order: 3, value: lastTimestamps[server.item_id] ? new Date(lastTimestamps[server.item_id]).getTime() / 1000 : null },
                { order: 4, value: ipAddressProp ? ipAddressProp.property_value : ' ' }
            ]);
        }
        tabs.push({
            title: 'Status',
            content: [
                table.generateBox()
            ]
        });
        this.page = new pageResponse_dto_1.Page("Data collection status", tabs);
        return this.page;
    }
}
exports.LinuxStatusPage = LinuxStatusPage;
class LinuxTotalsPage extends LinuxPage {
    async getPage() {
        let servers = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: "server",
            item_id: (0, typeorm_1.Not)((0, typeorm_1.In)(await (0, exports.getLinuxExcludedIds)()))
        });
        servers = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, servers);
        const server_ids = servers.map((item) => item.item_id);
        const mappings = await this.agentMappingRepository.findBy({ agent_id: (0, typeorm_1.In)(server_ids) });
        const mapped_server_ids = mappings.map((mapping) => mapping.agent_id);
        servers = servers.filter((item) => !mapped_server_ids.includes(item.item_id));
        const ids = servers.map(i => i.item_id);
        let tabs = [];
        let title = "Totals";
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'cpu_util')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: ids,
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Memory Used %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'mem_util')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: ids,
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Memory Used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'mem_used')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: ids,
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Memory Free',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'mem_free')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: ids,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.LinuxTotalsPage = LinuxTotalsPage;
class LinuxGroupconfigurationPage extends LinuxPage {
    async getPage() {
        const menuFormPageItem = {
            hwType: "linux",
            subsystem: "server",
            class: class_constants_1.ClassKey.VIRTUALIZATION
        };
        this.page = new pageResponse_dto_1.Page("Linux Groups Configuration", [], { title: "Linux", hwTypes: [menuFormPageItem] });
        return this.page;
    }
}
exports.LinuxGroupconfigurationPage = LinuxGroupconfigurationPage;
class LinuxHeatmapPage extends LinuxPage {
    constructor() {
        super(...arguments);
        this.lgItemsRepository = (0, dataSource_1.getDataSource)().getRepository(serverGroupItems_entity_1.ServerGroupItems);
    }
    async getPage() {
        let tabs = [];
        tabs.push({
            title: 'Heatmap',
            content: Object.values(await new Heatmap_1.Heatmap('linux', 'server', true)
                .virtHeatmapBox(this.request)),
        });
        let serverTable = new Table_1.Table()
            .addHeader({ value: 'Server' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true })
            .addHeader({ value: 'Memory', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let servers = await this.objectItemsRepository.find({
            where: {
                hw_type: 'linux',
                subsystem: 'server',
                item_id: (0, typeorm_1.Not)((0, typeorm_1.In)(await (0, exports.getLinuxExcludedIds)()))
            },
            relations: {
                heatmaps: true,
            }
        });
        const lgItems = await this.lgItemsRepository.find({
            where: {
                _server_group: {
                    user_id: (await User_1.User.getUserByRequest(this.request)).user_id
                }
            },
            relations: {
                _server_group: true
            }
        });
        servers = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, servers);
        for (const server of servers) {
            const cpu = server.heatmaps.find(heatmap => heatmap.metric === 'cpu_util');
            const memory = server.heatmaps.find(heatmap => heatmap.metric === 'mem_util');
            const inLG = lgItems.find((it => server.item_id === it.item_id));
            serverTable.addRow([
                { order: 0, value: server.label, url: URL_1.URL.generate('linux', 'server', inLG ? server.item_id + ":::" + inLG.server_group_id : server.item_id) },
                { order: 1, value: cpu !== undefined ? cpu.value : null },
                { order: 2, value: memory !== undefined ? memory.value : null },
            ]);
        }
        serverTable.sortByIndex(1);
        tabs.push({
            title: 'Table',
            content: [serverTable.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page("Heatmap", tabs);
        return this.page;
    }
}
exports.LinuxHeatmapPage = LinuxHeatmapPage;
class LinuxTopPage extends LinuxPage {
    constructor() {
        super(...arguments);
        this.title = 'VM TOP';
        this.withItemId = true;
    }
    async getPage() {
        let servers = await this.objectItemsRepository.find({
            where: {
                hw_type: 'linux',
                subsystem: 'server',
                item_id: (0, typeorm_1.Not)((0, typeorm_1.In)(await (0, exports.getLinuxExcludedIds)()))
            }
        });
        const top = new Top_1.Top();
        top.setItemIds(servers.map(i => i.item_id));
        top.setSubsystem('linux', 'server');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.LinuxTopPage = LinuxTopPage;
class LinuxServerPage extends LinuxSingleItemPage {
    constructor(itemId, request) {
        super(itemId, request);
        this.itemId = itemId;
        this.request = request;
        this.subsystem = 'server';
        this.serverGroupItemsRepository = (0, dataSource_1.getDataSource)().getRepository(serverGroupItems_entity_1.ServerGroupItems);
        let split = itemId.split(":::");
        if (split.length > 1) {
            this.itemId = split[0];
            this.parentId = split[1];
        }
    }
    ;
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'VM ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        if (this.itemId.startsWith('Linux_')) {
            tabs.push({
                title: 'CPU OS',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxOldMetrics.CpuIdleMetric().init(),
                            new LinuxOldMetrics.CpuUsrMetric().init(),
                            new LinuxOldMetrics.CpuSysMetric().init(),
                            new LinuxOldMetrics.CpuWaitMetric().init(),
                            new LinuxOldMetrics.CpuStolenMetric().init(),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        else {
            tabs.push({
                title: 'CPU OS',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxOldMetrics.CpuSysMetric().init(),
                            new LinuxOldMetrics.CpuUsrMetric().init(),
                            new LinuxOldMetrics.CpuWaitMetric().init(),
                            new LinuxOldMetrics.CpuIdleMetric().init(),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        if (this.itemId.startsWith('Linux_')) {
            tabs.push({
                title: 'CPU core',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('server', 'logical_processors')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.black),
                            new LinuxMetrics.Metric('server', 'cpu_core')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        if (this.itemId.startsWith('Linux_')) {
            tabs.push({
                title: 'CPU queue',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('server', 'logical_processors')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.black),
                            new LinuxMetrics.Metric('server', 'load_avrg')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark),
                            new LinuxMetrics.Metric('server', 'blocked_processes')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        if (this.itemId.startsWith('power_')) {
            tabs.push({
                title: 'CPU queue',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('server', 'logical_processors')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.black),
                            new LinuxMetrics.Metric('server', 'load_avrg')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark),
                            new LinuxMetrics.Metric('server', 'blocked_processes')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark),
                            new LinuxMetrics.Metric('server', 'blocked_raw')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.greenDark),
                            new LinuxMetrics.Metric('server', 'blocked_direct')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.orangeDark),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        let jobs = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'job',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        const jobCommands = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(jobs.map(job => job.item_id), 'command');
        const jobUsers = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(jobs.map(job => job.item_id), 'user');
        if (jobs.length >= 1) {
            tabs.push({
                title: 'Job',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                        text: "JOB CPU"
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                        graphs: [
                            {
                                metrics: [
                                    new LinuxMetrics.Metric('job', 'time_difference')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                            {
                                metrics: [
                                    new LinuxMetrics.Metric('job', 'time_difference')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.week,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                        ],
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                        text: "Job Memory"
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                        graphs: [
                            {
                                metrics: [
                                    new LinuxMetrics.Metric('job', 'rss')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                            {
                                metrics: [
                                    new LinuxMetrics.Metric('job', 'rss')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.week,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                        ],
                    },
                ],
                help: {
                    text: 'How it works?',
                    url: 'https://xormon.com/job-top.php'
                }
            });
        }
        if (this.itemId.startsWith('Linux_')) {
            tabs.push({
                title: 'Memory',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxOldMetrics.MemoryFreeMetric().init(),
                            new LinuxOldMetrics.MemoryUsedMetric().init(),
                            new LinuxOldMetrics.MemoryFSCacheMetric().init(),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        if (this.itemId.startsWith('power_')) {
            tabs.push({
                title: 'Memory',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxOldMetrics.Memory2Metric().init(),
                            new LinuxOldMetrics.MemoryFreeMetric().init(),
                            new LinuxOldMetrics.MemoryUsedMetric().init(),
                            new LinuxOldMetrics.MemoryFSCacheMetric().init(),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        tabs.push({
            title: 'Paging 1',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'pgs2')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark),
                        new LinuxMetrics.Metric('server', 'pgs1')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Paging 2',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'pgs4')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        const itemProperties_disk = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'disk-total' } });
        if (itemProperties_disk) {
            tabs.push({
                title: 'IO',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('server', 'read_iops')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.metric.read),
                            new LinuxMetrics.Metric('server', 'write_iops')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.metric.write)
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        if (itemProperties_disk) {
            tabs.push({
                title: 'Data',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('server', 'read_data')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.metric.read),
                            new LinuxMetrics.Metric('server', 'write_data')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.metric.write)
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        if (itemProperties_disk) {
            tabs.push({
                title: 'Latency',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('server', 'read_latency')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.metric.read),
                            new LinuxMetrics.Metric('server', 'write_latency')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.metric.write)
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        let lans = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'lan',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        let sans = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'san',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        if (sans.length >= 1) {
            tabs.push({
                title: 'SAN',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('san', 'recv_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('san', 'trans_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: sans.map(san => san.item_id),
                        aggregated: true,
                    },
                ],
            });
            tabs.push({
                title: 'SAN Frames',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('san', 'iops_in')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('san', 'iops_out')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: sans.map(san => san.item_id),
                        aggregated: true,
                    },
                ],
            });
        }
        if (sans.length >= 1) {
            tabs.push({
                title: 'SAN Latency',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('san', 'resp_t_r')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1),
                            new LinuxMetrics.Metric('san', 'resp_t_w')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1)
                        ],
                        uuids: sans.map(san => san.item_id),
                        aggregated: true,
                    },
                ],
            });
        }
        if (sans.length >= 1) {
            const lastValuesSan = await (new MetricsWrapper_1.MetricsWrapper(['san_error'], sans.map(san => san.item_id))).getLastDailyValuesFromTable();
            if (Object.keys(lastValuesSan).length >= 1) {
                for (const key in lastValuesSan) {
                    const sanError = lastValuesSan[key]?.san_error;
                    if (sanError !== null && sanError >= 0) {
                        tabs.push({
                            title: 'SAN Error',
                            content: [
                                {
                                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                                    metrics: [
                                        new LinuxMetrics.Metric('san', 'san_error')
                                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1)
                                    ],
                                    uuids: sans.filter(san => san.label.startsWith("fcs")).map(san => san.item_id),
                                    aggregated: true,
                                },
                            ],
                        });
                        break;
                    }
                }
            }
        }
        if (sans.length >= 1) {
            const lastValuesSan = await (new MetricsWrapper_1.MetricsWrapper(['rx_power'], sans.map(san => san.item_id))).getLastDailyValuesFromTable();
            if (Object.keys(lastValuesSan).length >= 1) {
                for (const key in lastValuesSan) {
                    const sanPower = lastValuesSan[key]?.rx_power;
                    if (sanPower !== null) {
                        tabs.push({
                            title: 'SAN Power',
                            content: [
                                {
                                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                                    metrics: [
                                        new LinuxMetrics.Metric('san', 'rx_power')
                                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1),
                                        new LinuxMetrics.Metric('san', 'tx_power')
                                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1)
                                    ],
                                    uuids: sans.filter(san => san.label.startsWith("fcs")).map(san => san.item_id),
                                    aggregated: true,
                                },
                            ],
                        });
                        break;
                    }
                }
            }
        }
        if (lans.length >= 1) {
            tabs.push({
                title: 'LAN',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('lan', 'recv_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('lan', 'trans_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: lans.map(lan => lan.item_id),
                        aggregated: true,
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                        text: "Packets"
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('lan', 'recv_packets')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('lan', 'trans_packets')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: lans.map(lan => lan.item_id),
                        aggregated: true,
                    },
                ],
            });
        }
        if (lans.length >= 1) {
            const lastValuesLan = await (new MetricsWrapper_1.MetricsWrapper(['lan_error'], lans.map(lan => lan.item_id))).getLastDailyValuesFromTable();
            if (Object.keys(lastValuesLan).length >= 1) {
                for (const key in lastValuesLan) {
                    const lanError = lastValuesLan[key]?.lan_error;
                    if (lanError !== null && lanError >= 0) {
                        tabs.push({
                            title: 'LAN Error',
                            content: [
                                {
                                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                                    metrics: [
                                        new LinuxMetrics.Metric('lan', 'lan_error')
                                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1)
                                    ],
                                    uuids: lans.map(lan => lan.item_id),
                                    aggregated: true,
                                },
                            ],
                        });
                        break;
                    }
                }
            }
        }
        let seas = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'sea',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        if (seas.length >= 1) {
            tabs.push({
                title: 'SEA',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('sea', 'trans_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('sea', 'recv_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: seas.map(sea => sea.item_id),
                        aggregated: true,
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                        text: "Packets"
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('sea', 'trans_packets')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('sea', 'recv_packets')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: seas.map(lan => lan.item_id),
                        aggregated: true,
                    },
                ],
            });
        }
        const itemProperties_ame = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'ame_ratio' } });
        if (this.itemId.startsWith('power_') && itemProperties_ame) {
            tabs.push({
                title: 'AME',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('server', 'ame_compres')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1),
                            new LinuxMetrics.Metric('server', 'ame_deficit')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1)
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        let wlms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'wlm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        if (wlms.length >= 1) {
            tabs.push({
                title: 'WLM-CPU',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('wlm', 'wlm_cpu')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: wlms.filter(wlm => wlm.label.indexOf("TOTAL") === -1).map(wlm => wlm.item_id),
                        aggregated: true,
                    },
                ],
            });
            tabs.push({
                title: 'WLM-MEM',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('wlm', 'wlm_mem')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: wlms.filter(wlm => wlm.label.indexOf("TOTAL") === -1).map(wlm => wlm.item_id),
                        aggregated: true,
                    },
                ],
            });
            tabs.push({
                title: 'WLM-DKIO',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('wlm', 'wlm_dkio')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: wlms.filter(wlm => wlm.label.indexOf("TOTAL") === -1).map(wlm => wlm.item_id),
                        aggregated: true,
                    },
                ],
            });
        }
        const itemProperties = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'FS' } });
        let fsTable = new Table_1.Table()
            .addHeader({ value: 'Filesystem' })
            .addHeader({ value: 'Total[GB]', number: true })
            .addHeader({ value: 'Used[GB]', number: true })
            .addHeader({ value: 'Available[GB]', number: true })
            .addHeader({ value: 'Usage[%]', number: true })
            .addHeader({ value: 'Mounted on' });
        if (itemProperties) {
            let fsLines = JSON.parse(itemProperties.property_value);
            for (const itemId in fsLines) {
                const line = fsLines[itemId];
                fsTable.addRow([
                    { order: 0, value: line.label },
                    { order: 1, value: line.total ? +line.total.replace(/^(\d+\.\d).*$/, '$1') : "" },
                    { order: 2, value: line.used ? +line.used.replace(/^(\d+\.\d).*$/, '$1') : "" },
                    { order: 3, value: line.available ? +line.available.replace(/^(\d+\.\d).*$/, '$1') : "" },
                    { order: 4, value: line.usage ? +line.usage : "" },
                    { order: 5, value: line.mounted_on },
                ]);
            }
        }
        if (itemProperties) {
            fsTable.sortByIndex(4);
            tabs.push({
                title: 'FS',
                content: [
                    fsTable.generateBox(),
                ],
            });
        }
        const itemPropertiesMultiLin = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'MULTILIN' } });
        let LinuxMultiTable = new Table_1.Table()
            .addHeader({ value: 'WWID', trim: false })
            .addHeader({ value: 'Alias', trim: false })
            .addHeader({ value: 'Attributes', trim: false })
            .addHeader({ value: 'Paths', trim: false, linuxMpath: true });
        if (itemPropertiesMultiLin) {
            let MultipathLines = JSON.parse(itemPropertiesMultiLin.property_value);
            for (const itemId in MultipathLines) {
                const line = MultipathLines[itemId];
                let formattedPaths = line.paths.replace(/\//g, '\n').trimEnd();
                LinuxMultiTable.addRow([
                    { order: 0, value: line.wwid },
                    { order: 1, value: line.alias },
                    { order: 2, value: line.attributes },
                    { order: 3, value: formattedPaths },
                ]);
            }
        }
        if (itemPropertiesMultiLin) {
            tabs.push({
                title: 'Multipath',
                content: [
                    LinuxMultiTable.generateBox(),
                ],
            });
        }
        const itemPropertiesLS = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'LSDISK' } });
        let lsdiskTable = new Table_1.Table()
            .addHeader({ value: 'Disk name' })
            .addHeader({ value: 'Disk size[MB]' })
            .addHeader({ value: 'Path properties', trim: false, newLines: true })
            .addHeader({ value: 'Path info', trim: false, newLines: true })
            .addHeader({ value: 'Path status', trim: false, newLines: true })
            .addHeader({ value: 'Status' });
        if (itemPropertiesLS) {
            let lsdiskLines = JSON.parse(itemPropertiesLS.property_value);
            let itemIdMultiLine = {};
            for (const itemId in lsdiskLines) {
                const line = lsdiskLines[itemId];
                itemIdMultiLine[line.label] = itemIdMultiLine[line.label] ? itemIdMultiLine[line.label] : {
                    path_info: "",
                    path_prop: "",
                    path_stat: "",
                    status: "",
                    statusColor: "",
                    disk_size: ""
                };
                if (line && line.status) {
                    if (line.status.includes('OK')) {
                        itemIdMultiLine[line.label].statusColor = Color_1.Color.pallete.green;
                    }
                    else if (line.status.includes('Critical')) {
                        itemIdMultiLine[line.label].statusColor = Color_1.Color.pallete.red;
                    }
                    else if (line.status.includes('Warning')) {
                        itemIdMultiLine[line.label].statusColor = Color_1.Color.pallete.yellow;
                    }
                }
                else {
                    line.status = "";
                }
                itemIdMultiLine[line.label].path_prop = !itemIdMultiLine[line.label].path_prop ? line.path_prop : itemIdMultiLine[line.label].path_prop + "\n" + line.path_prop;
                itemIdMultiLine[line.label].path_info = !itemIdMultiLine[line.label].path_info ? line.path_info : itemIdMultiLine[line.label].path_info + "\n" + line.path_info;
                itemIdMultiLine[line.label].path_stat = !itemIdMultiLine[line.label].path_stat ? line.path_stat : itemIdMultiLine[line.label].path_stat + "\n" + line.path_stat;
                itemIdMultiLine[line.label].status = line.status;
                itemIdMultiLine[line.label].disk_size = line.disk_size;
            }
            for (const key in itemIdMultiLine) {
                lsdiskTable.addRow([
                    { order: 0, value: key },
                    { order: 1, value: +itemIdMultiLine[key].disk_size },
                    { order: 2, value: itemIdMultiLine[key].path_prop },
                    { order: 3, value: itemIdMultiLine[key].path_info },
                    { order: 4, value: itemIdMultiLine[key].path_stat },
                    { order: 5, value: itemIdMultiLine[key].status.trim(), bgColor: itemIdMultiLine[key].statusColor },
                ]);
            }
        }
        lsdiskTable.sortByValues(5, ['Critical', 'Warning', 'OK'], true);
        if (itemPropertiesLS) {
            tabs.push({
                title: 'Multipath',
                content: [
                    lsdiskTable.generateBox(),
                ],
            });
        }
        const selfMonitoringItem = await (0, exports.getSelfMonitoringAgent)();
        if (selfMonitoringItem && selfMonitoringItem.item_id === this.itemId) {
            const tasksPage = await (new Xormon_1.ApplicationTasksPage()).getPage();
            tabs = tabs.concat(tasksPage.tabs);
            if (await this.objectItemsRepository.findOneBy({ hw_type: 'database', subsystem: 'schema' })) {
                const dbSizePage = await (new Xormon_1.DatabaseTotalPage()).getPage();
                tabs = tabs.concat(dbSizePage.tabs);
            }
            const jobs = await this.objectItemsRepository.query(`select * from timescaledb_information.job_stats order by last_run_duration desc limit 50`);
            const toSeconds = (interval) => {
                let seconds = 0;
                if (interval.seconds) {
                    seconds += interval.seconds;
                }
                if (interval.milliseconds) {
                    seconds += (interval.milliseconds / 1000);
                }
                if (interval.minutes) {
                    seconds += (interval.minutes * 60);
                }
                return (Math.round(seconds * 100) / 100);
            };
            let jobsTable = new Table_1.Table()
                .addHeader({ value: 'Table' })
                .addHeader({ value: 'Job' })
                .addHeader({ value: 'Last run started' })
                .addHeader({ value: 'Last successful finish' })
                .addHeader({ value: 'Last run status' })
                .addHeader({ value: 'Last run duration' })
                .addHeader({ value: 'Next start' })
                .addHeader({ value: 'Total runs' })
                .addHeader({ value: 'Total successes' })
                .addHeader({ value: 'Total failures' });
            for (const job of jobs) {
                jobsTable.addRow([
                    { order: 0, value: job.hypertable_name },
                    { order: 1, value: job.job_id },
                    { order: 2, value: job.last_run_started_at },
                    { order: 3, value: job.last_successful_finish },
                    { order: 4, value: job.last_run_status },
                    { order: 5, value: job.last_run_duration ? toSeconds(job.last_run_duration) : null },
                    { order: 6, value: job.next_start },
                    { order: 7, value: job.total_runs },
                    { order: 8, value: job.total_successes },
                    { order: 9, value: job.total_failures },
                ]);
            }
            tabs.push({
                title: 'DB Jobs',
                content: [
                    jobsTable.generateBox(),
                    {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        text: "Top 50 jobs by duration"
                    }
                ],
            });
        }
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        if (this.parentId) {
            for (const item of menuItems) {
                if (item.subsystem === "group") {
                    if (item.itemId === this.parentId) {
                        const sGroup = new LinuxGroupFolder(this.parentId);
                        item.children = await sGroup.getChildren();
                    }
                }
            }
        }
        else {
            for (const item of menuItems) {
                if (item.subsystem === "group") {
                    const sGroup = new LinuxGroupFolder(item.itemId);
                    item.children = await sGroup.getChildren(true);
                    if (item.children.find(i => i.itemId === this.itemId)) {
                        break;
                    }
                }
            }
        }
        return menuItems;
    }
}
exports.LinuxServerPage = LinuxServerPage;
class LinuxMetrics {
}
exports.LinuxMetrics = LinuxMetrics;
LinuxMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor(subsystem, metric) {
        super();
        this.metric = metric;
        const definition = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.LINUX, subsystem, metric);
        if (definition) {
            Object.assign(this, definition);
        }
    }
    getGraph(type, stacked, order, color) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.color = color ? color : undefined;
        return this;
    }
    getAggrGraph(type, stacked, order, aggregate) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.aggregate = aggregate ? aggregate : pageResponse_constants_1.METRIC_AGGREGATE.items;
        return this;
    }
};
class LinuxOldMetrics {
}
exports.LinuxOldMetrics = LinuxOldMetrics;
_a = LinuxOldMetrics;
LinuxOldMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'server';
    }
    init() {
        const metric = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.LINUX, this.subsystem, this.metric);
        if (metric) {
            Object.assign(this, metric);
        }
        return this;
    }
};
LinuxOldMetrics.CpuSysMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'cpu_sys';
        this.order = 2;
        this.color = Color_1.Color.pallete.blue;
    }
};
LinuxOldMetrics.CpuUsrMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'cpu_usr';
        this.order = 3;
        this.color = Color_1.Color.pallete.yellow;
    }
};
LinuxOldMetrics.CpuWaitMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'cpu_wait';
        this.order = 1;
        this.color = Color_1.Color.pallete.red;
    }
};
LinuxOldMetrics.CpuStolenMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'cpu_stolen';
        this.order = 0;
        this.color = Color_1.Color.pallete.blueDark;
    }
};
LinuxOldMetrics.CpuIdleMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'cpu_idle';
        this.order = 4;
        this.color = Color_1.Color.pallete.green;
    }
};
LinuxOldMetrics.SendBytesMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'send_bytes';
        this.order = 3;
        this.color = Color_1.Color.pallete.blueDark;
    }
};
LinuxOldMetrics.ReceivedBytesMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'received_bytes';
        this.order = 3;
        this.color = Color_1.Color.pallete.redDark;
    }
};
LinuxOldMetrics.MemoryTotalMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'mem_total';
        this.order = 0;
        this.color = Color_1.Color.pallete.blueDark;
    }
};
LinuxOldMetrics.Memory2Metric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'mem2';
        this.order = 7;
        this.color = Color_1.Color.pallete.black;
    }
};
LinuxOldMetrics.MemoryFreeMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'mem_free';
        this.order = 5;
        this.color = Color_1.Color.pallete.green;
    }
};
LinuxOldMetrics.Memory4Metric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'mem4';
        this.order = 3;
        this.color = Color_1.Color.pallete.black;
    }
};
LinuxOldMetrics.MemoryUsedMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'mem_used';
        this.order = 2;
        this.color = Color_1.Color.pallete.red;
    }
};
LinuxOldMetrics.MemoryFSCacheMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'mem_fs_cache';
        this.order = 4;
        this.color = Color_1.Color.pallete.blue;
    }
};
LinuxOldMetrics.MemoryWTSizeMetric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'wtsize';
        this.order = 6;
        this.color = Color_1.Color.pallete.black;
    }
};
LinuxOldMetrics.Paging1Metric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'pgs1';
        this.order = 5;
        this.color = Color_1.Color.pallete.blueDark;
    }
};
LinuxOldMetrics.Paging2Metric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'pgs2';
        this.order = 5;
        this.color = Color_1.Color.pallete.redDark;
    }
};
LinuxOldMetrics.Paging3Metric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'pgs3';
        this.order = 5;
        this.color = Color_1.Color.pallete.green;
    }
};
LinuxOldMetrics.Paging4Metric = class extends _a.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = true;
        this.metric = 'pgs4';
        this.order = 5;
        this.color = Color_1.Color.pallete.redDark;
    }
};
class LinuxgroupsFolder {
    constructor() {
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.serverGroupRepository = (0, dataSource_1.getDataSource)().getRepository(serverGroup_entity_1.ServerGroup);
    }
}
exports.LinuxgroupsFolder = LinuxgroupsFolder;
class LinuxgroupsSubsystemFolder extends LinuxgroupsFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId;
        }
    }
}
exports.LinuxgroupsSubsystemFolder = LinuxgroupsSubsystemFolder;
class LinuxgroupsPage {
    constructor(itemId, request) {
        this.itemId = itemId;
        this.request = request;
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.serverGroupRepository = (0, dataSource_1.getDataSource)().getRepository(serverGroup_entity_1.ServerGroup);
        this.itemPropertiesRepository = (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty);
    }
    ;
    async getPath() {
        let root = new LinuxRootFolder();
        let menuItems = await root.getChildren();
        return menuItems;
    }
}
exports.LinuxgroupsPage = LinuxgroupsPage;
class LinuxGroupFolder extends LinuxgroupsSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = "group";
    }
    async getChildren(useFirst) {
        let menuItems = [];
        const sGroup = await this.serverGroupRepository.findOne({
            where: {
                server_group_id: this.itemId,
                server_group_items: {
                    isPrimary: true,
                    item_id: (0, typeorm_1.Not)((0, typeorm_1.In)(await (0, exports.getLinuxExcludedIds)()))
                }
            },
            relations: {
                server_group_items: {
                    _item: true
                }
            }
        });
        if (sGroup) {
            if (sGroup.server_group_items && sGroup.server_group_items.length) {
                menuItems.push(new menuResponse_dto_1.MenuLink('Totals', 'linux/servergroup-totals?item_id=' + this.itemId, 'server-totals', undefined, true));
            }
            for (const { _item: item } of sGroup.server_group_items) {
                menuItems.push(new menuResponse_dto_1.MenuLink(item.label, "linux/server?item_id=" + (useFirst ? item.item_id : item.item_id + ":::" + this.itemId), "server", item.item_id));
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.LinuxGroupFolder = LinuxGroupFolder;
class LinuxServergrouptotalsPage extends LinuxgroupsPage {
    async getPage() {
        let tabs = [];
        let title = "Totals";
        let servers = (await this.serverGroupRepository.findOne({
            where: {
                server_group_id: this.itemId,
                server_group_items: {
                    isPrimary: true,
                    item_id: (0, typeorm_1.Not)((0, typeorm_1.In)(await (0, exports.getLinuxExcludedIds)()))
                }
            },
            relations: {
                server_group_items: {
                    _item: true
                },
            }
        })).server_group_items.map(sIt => sIt._item);
        const ids = (await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, servers)).map(it => it.item_id);
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'cpu_util')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: ids,
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Memory Used %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'mem_util')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: ids,
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Memory Used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'mem_used')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: ids,
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Memory Free',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'mem_free')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: ids,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
    async getPath() {
        const path = await super.getPath();
        for (const item of path) {
            if (item.itemId === this.itemId) {
                const sGroup = new LinuxGroupFolder(this.itemId);
                item.children = await sGroup.getChildren();
            }
        }
        return path;
    }
}
exports.LinuxServergrouptotalsPage = LinuxServergrouptotalsPage;
class LinuxServergroupPage extends LinuxgroupsPage {
    constructor(itemId, request) {
        super(itemId, request);
        this.itemId = itemId;
        this.request = request;
        this.subsystem = 'server';
        let split = itemId.split(":::");
        this.itemId = itemId;
        this.parentId = split[1];
    }
    ;
    async getPath() {
        const path = await super.getPath();
        for (const item of path) {
            if (item.itemId === this.parentId) {
                const sGroup = new LinuxGroupFolder(this.parentId);
                item.children = await sGroup.getChildren();
            }
        }
        return path;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: "linux",
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'VM ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        if (this.itemId.startsWith('Linux_')) {
            tabs.push({
                title: 'CPU OS',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxOldMetrics.CpuIdleMetric().init(),
                            new LinuxOldMetrics.CpuUsrMetric().init(),
                            new LinuxOldMetrics.CpuSysMetric().init(),
                            new LinuxOldMetrics.CpuWaitMetric().init(),
                            new LinuxOldMetrics.CpuStolenMetric().init(),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        else {
            tabs.push({
                title: 'CPU OS',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxOldMetrics.CpuSysMetric().init(),
                            new LinuxOldMetrics.CpuUsrMetric().init(),
                            new LinuxOldMetrics.CpuWaitMetric().init(),
                            new LinuxOldMetrics.CpuIdleMetric().init(),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        if (this.itemId.startsWith('Linux_')) {
            tabs.push({
                title: 'CPU queue',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('server', 'logical_processors')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.black),
                            new LinuxMetrics.Metric('server', 'load_avrg')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark),
                            new LinuxMetrics.Metric('server', 'blocked_processes')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        let jobs = await this.objectItemsRepository.find({
            where: {
                hw_type: "linux",
                subsystem: 'job',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        const jobCommands = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(jobs.map(job => job.item_id), 'command');
        const jobUsers = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(jobs.map(job => job.item_id), 'user');
        if (jobs.length >= 1) {
            tabs.push({
                title: 'Job',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                        text: "JOB CPU"
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                        graphs: [
                            {
                                metrics: [
                                    new LinuxMetrics.Metric('job', 'time_difference')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                            {
                                metrics: [
                                    new LinuxMetrics.Metric('job', 'time_difference')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.week,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                        ],
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                        text: "Job Memory"
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                        graphs: [
                            {
                                metrics: [
                                    new LinuxMetrics.Metric('job', 'rss')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                            {
                                metrics: [
                                    new LinuxMetrics.Metric('job', 'rss')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.week,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                        ],
                    },
                ],
                help: {
                    text: 'How it works?',
                    url: 'https://xormon.com/job-top.php'
                }
            });
        }
        if (this.itemId.startsWith('Linux_')) {
            tabs.push({
                title: 'Memory',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxOldMetrics.MemoryFreeMetric().init(),
                            new LinuxOldMetrics.MemoryUsedMetric().init(),
                            new LinuxOldMetrics.MemoryFSCacheMetric().init(),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
        }
        tabs.push({
            title: 'Paging 1',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'pgs2')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark),
                        new LinuxMetrics.Metric('server', 'pgs1')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Paging 2',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new LinuxMetrics.Metric('server', 'pgs4')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        let lans = await this.objectItemsRepository.find({
            where: {
                hw_type: "linux",
                subsystem: 'lan',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        if (lans.length >= 1) {
            tabs.push({
                title: 'LAN',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('lan', 'recv_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('lan', 'trans_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: lans.map(lan => lan.item_id),
                        aggregated: true,
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                        text: "Packets"
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('lan', 'recv_packets')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('lan', 'trans_packets')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: lans.map(lan => lan.item_id),
                        aggregated: true,
                    },
                ],
            });
        }
        let sans = await this.objectItemsRepository.find({
            where: {
                hw_type: "linux",
                subsystem: 'san',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        if (sans.length >= 1) {
            tabs.push({
                title: 'SAN',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('san', 'recv_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('san', 'trans_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: sans.map(san => san.item_id),
                        aggregated: true,
                    },
                ],
            });
            tabs.push({
                title: 'SAN IO',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('san', 'iops_in')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('san', 'iops_out')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: sans.map(san => san.item_id),
                        aggregated: true,
                    },
                ],
            });
        }
        if (sans.length >= 1) {
            tabs.push({
                title: 'SAN Latency',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('san', 'resp_t_r')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1),
                            new LinuxMetrics.Metric('san', 'resp_t_w')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1)
                        ],
                        uuids: sans.map(san => san.item_id),
                        aggregated: true,
                    },
                ],
            });
        }
        let seas = await this.objectItemsRepository.find({
            where: {
                hw_type: "linux",
                subsystem: 'sea',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        if (seas.length >= 1) {
            tabs.push({
                title: 'SEA',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('sea', 'trans_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('sea', 'recv_bytes')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: seas.map(sea => sea.item_id),
                        aggregated: true,
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                        text: "Packets"
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new LinuxMetrics.Metric('sea', 'trans_packets')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                            new LinuxMetrics.Metric('sea', 'recv_packets')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                        ],
                        uuids: seas.map(lan => lan.item_id),
                        aggregated: true,
                    },
                ],
            });
        }
        const itemProperties = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'FS' } });
        let fsTable = new Table_1.Table()
            .addHeader({ value: 'Filesystem' })
            .addHeader({ value: 'Total[GB]', number: true })
            .addHeader({ value: 'Used[GB]', number: true })
            .addHeader({ value: 'Available[GB]', number: true })
            .addHeader({ value: 'Usage[%]', number: true })
            .addHeader({ value: 'Mounted on' });
        if (itemProperties) {
            let fsLines = JSON.parse(itemProperties.property_value);
            for (const itemId in fsLines) {
                const line = fsLines[itemId];
                fsTable.addRow([
                    { order: 0, value: line.label },
                    { order: 1, value: +line.total.replace(/^(\d+\.\d).*$/, '$1') },
                    { order: 2, value: +line.used.replace(/^(\d+\.\d).*$/, '$1') },
                    { order: 3, value: +line.available.replace(/^(\d+\.\d).*$/, '$1') },
                    { order: 4, value: +line.usage },
                    { order: 5, value: line.mounted_on },
                ]);
            }
        }
        if (itemProperties) {
            fsTable.sortByIndex(4);
            tabs.push({
                title: 'FS',
                content: [
                    fsTable.generateBox(),
                ],
            });
        }
        const itemPropertiesLS = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'LSDISK' } });
        let lsdiskTable = new Table_1.Table()
            .addHeader({ value: 'Disk name' })
            .addHeader({ value: 'Disk size[MB]' })
            .addHeader({ value: 'Path properties', trim: false, newLines: true })
            .addHeader({ value: 'Path info', trim: false, newLines: true })
            .addHeader({ value: 'Path status', trim: false, newLines: true })
            .addHeader({ value: 'Status' });
        if (itemPropertiesLS) {
            let lsdiskLines = JSON.parse(itemPropertiesLS.property_value);
            let itemIdMultiLine = {};
            for (const itemId in lsdiskLines) {
                const line = lsdiskLines[itemId];
                itemIdMultiLine[line.label] = itemIdMultiLine[line.label] ? itemIdMultiLine[line.label] : {
                    path_info: "",
                    path_prop: "",
                    path_stat: "",
                    status: "",
                    statusColor: "",
                    disk_size: ""
                };
                if (line && line.status) {
                    if (line.status.includes('OK')) {
                        itemIdMultiLine[line.label].statusColor = Color_1.Color.pallete.green;
                    }
                    else if (line.status.includes('Critical')) {
                        itemIdMultiLine[line.label].statusColor = Color_1.Color.pallete.red;
                    }
                    else if (line.status.includes('Warning')) {
                        itemIdMultiLine[line.label].statusColor = Color_1.Color.pallete.yellow;
                    }
                }
                else {
                    line.status = "";
                }
                itemIdMultiLine[line.label].path_prop = !itemIdMultiLine[line.label].path_prop ? line.path_prop : itemIdMultiLine[line.label].path_prop + "\n" + line.path_prop;
                itemIdMultiLine[line.label].path_info = !itemIdMultiLine[line.label].path_info ? line.path_info : itemIdMultiLine[line.label].path_info + "\n" + line.path_info;
                itemIdMultiLine[line.label].path_stat = !itemIdMultiLine[line.label].path_stat ? line.path_stat : itemIdMultiLine[line.label].path_stat + "\n" + line.path_stat;
                itemIdMultiLine[line.label].status = line.status;
                itemIdMultiLine[line.label].disk_size = line.disk_size;
            }
            for (const key in itemIdMultiLine) {
                lsdiskTable.addRow([
                    { order: 0, value: key },
                    { order: 1, value: +itemIdMultiLine[key].disk_size },
                    { order: 2, value: itemIdMultiLine[key].path_prop },
                    { order: 3, value: itemIdMultiLine[key].path_info },
                    { order: 4, value: itemIdMultiLine[key].path_stat },
                    { order: 5, value: itemIdMultiLine[key].status.trim(), bgColor: itemIdMultiLine[key].statusColor },
                ]);
            }
        }
        lsdiskTable.sortByValues(5, ['Critical', 'Warning', 'OK'], true);
        if (itemPropertiesLS) {
            tabs.push({
                title: 'Multipath',
                content: [
                    lsdiskTable.generateBox(),
                ],
            });
        }
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.LinuxServergroupPage = LinuxServergroupPage;
//# sourceMappingURL=Linux.js.map