"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Mapping = exports.MappingType = void 0;
const dataSource_1 = require("../../../config/dataSource");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const storageMapping_entity_1 = require("../../../entity/storageMapping.entity");
const Architecture_1 = require("../../architecture/Architecture");
const Table_1 = require("./Table");
const URL_1 = require("./URL");
var MappingType;
(function (MappingType) {
    MappingType["storage"] = "storage";
    MappingType["server"] = "server";
})(MappingType || (exports.MappingType = MappingType = {}));
class Mapping {
    constructor(itemId, type) {
        this.storageMappingRepository = (0, dataSource_1.getDataSource)().getRepository(storageMapping_entity_1.StorageMapping);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
        this.itemId = itemId;
        this.type = type;
    }
    async getMapping() {
        if (this.type == MappingType.storage) {
            this.mappings = await this.storageMappingRepository
                .createQueryBuilder()
                .select(['object_items.item_id', 'object_items.hw_type', 'object_items.label', 'storage_mappings2.wwn'])
                .from(storageMapping_entity_1.StorageMapping, 'storage_mappings')
                .innerJoin('storage_mappings', 'storage_mappings2', 'storage_mappings.wwn = storage_mappings2.wwn')
                .innerJoin('object_items', 'object_items', 'storage_mappings2.item_id = object_items.item_id')
                .where('object_items.subsystem = \'volume\' and storage_mappings.item_id = :item_id', { item_id: this.itemId })
                .getRawMany();
        }
        else if (this.type == MappingType.server) {
            if (this.itemId) {
                this.mappings = await this.storageMappingRepository
                    .createQueryBuilder()
                    .select(['object_items2.item_id as source_id', 'object_items.item_id as item_id', 'object_items2.label as source_label', 'object_items2.hw_type as source_hw_type', 'object_items.hw_type as hw_type', 'object_items.subsystem as subsystem', 'object_items.label as label', 'storage_mappings.wwn'])
                    .from(storageMapping_entity_1.StorageMapping, 'storage_mappings')
                    .innerJoin('storage_mappings', 'storage_mappings2', 'storage_mappings.wwn = storage_mappings2.wwn')
                    .innerJoin('object_items', 'object_items', 'storage_mappings2.item_id = object_items.item_id')
                    .innerJoin('object_items', 'object_items2', 'storage_mappings.item_id = object_items2.item_id')
                    .where('storage_mappings.item_id != storage_mappings2.item_id and storage_mappings.item_id = :item_id', { item_id: this.itemId })
                    .getRawMany();
            }
            else {
                const hwTypes = (await Architecture_1.Architecture.getHwTypes(false, 'storage')).map((hwType) => hwType.hw_type);
                this.mappings = await this.storageMappingRepository
                    .createQueryBuilder()
                    .select(['object_items2.item_id as source_id', 'object_items.item_id as item_id', 'object_items2.label as source_label', 'object_items2.hw_type as source_hw_type', 'object_items.hw_type as hw_type', 'object_items.subsystem as subsystem', 'object_items.label as label', 'storage_mappings.wwn'])
                    .from(storageMapping_entity_1.StorageMapping, 'storage_mappings')
                    .innerJoin('storage_mappings', 'storage_mappings2', 'storage_mappings.wwn = storage_mappings2.wwn')
                    .innerJoin('object_items', 'object_items', 'storage_mappings2.item_id = object_items.item_id')
                    .innerJoin('object_items', 'object_items2', 'storage_mappings.item_id = object_items2.item_id')
                    .where('storage_mappings.item_id != storage_mappings2.item_id and object_items2.subsystem = \'volume\' and object_items2.hw_type IN (:...hw_types)', { hw_types: hwTypes })
                    .getRawMany();
            }
        }
    }
    async isExists() {
        await this.getMapping();
        return this.mappings.length === 0 ? false : true;
    }
    async genTable() {
        await this.getMapping();
        if (this.mappings.length >= 1) {
            let mappedIds;
            if (this.type === MappingType.server) {
                mappedIds = this.mappings.map((item) => item.source_id);
            }
            else {
                mappedIds = this.mappings.map((item) => item.item_id);
            }
            let mappingTable = new Table_1.Table();
            if (this.type === MappingType.storage) {
                let parents = await this.itemRelationsRepository
                    .createQueryBuilder()
                    .select(['oi.item_id', 'oi.hw_type', 'oi.subsystem', 'oi.label', 'ir.child'])
                    .from(itemRelation_entity_1.ItemRelation, 'item_relations')
                    .innerJoin('object_items', 'object_items', 'item_relations.parent = object_items.item_id')
                    .where('item_relations.child IN (:...children)', { children: mappedIds })
                    .getRawMany();
                mappingTable
                    .addHeader({ value: 'Storage' })
                    .addHeader({ value: 'Volume' })
                    .addHeader({ value: 'Pool' })
                    .addHeader({ value: 'WWN' });
                for (const mapping of this.mappings) {
                    let device = parents.find((o) => o.child === mapping.item_id && o.subsystem === 'devide');
                    let pool = parents.find((o) => o.child === mapping.item_id && o.subsystem === 'pool');
                    mappingTable.addRow([
                        { order: 0, value: device ? device.label : 'undefined' },
                        { order: 1, value: mapping.label, url: URL_1.URL.generate(mapping.hw_type, 'volume', mapping.item_id) },
                        { order: 2, value: pool ? pool.label : 'undefined', url: pool ? URL_1.URL.generate(pool.hw_type, 'pool', pool.item_id) : undefined },
                        { order: 3, value: mapping.wwn },
                    ]);
                }
            }
            else if (this.type === MappingType.server) {
                let parents = await this.itemRelationsRepository
                    .createQueryBuilder()
                    .select(['oi.item_id', 'oi.hw_type', 'oi.subsystem', 'oi.label', 'ir.child'])
                    .from(itemRelation_entity_1.ItemRelation, 'item_relations')
                    .innerJoin('object_items', 'object_items', 'item_relations.parent = object_items.item_id')
                    .where('item_relations.child IN (:...children)', { children: mappedIds })
                    .getRawMany();
                mappingTable
                    .addHeader({ value: 'Volume' })
                    .addHeader({ value: 'Pool' })
                    .addHeader({ value: 'Storage' })
                    .addHeader({ value: 'Mapped Hw Type' })
                    .addHeader({ value: 'Mapped subsystem' })
                    .addHeader({ value: 'Label' });
                for (const mapping of this.mappings) {
                    let device = parents.find((o) => o.child === mapping.source_id && o.subsystem === 'device');
                    let pool = parents.find((o) => o.child === mapping.source_id && o.subsystem === 'pool');
                    mappingTable.addRow([
                        { order: 0, value: mapping.source_label, url: URL_1.URL.generate(mapping.source_hw_type, 'volume', mapping.source_id) },
                        { order: 1, value: pool ? pool.label : 'undefined', url: pool ? URL_1.URL.generate(pool.hw_type, 'pool', pool.item_id) : undefined },
                        { order: 2, value: device ? device.label : 'undefined' },
                        { order: 3, value: mapping.hw_type },
                        { order: 4, value: mapping.subsystem },
                        { order: 5, value: mapping.label, url: URL_1.URL.generate(mapping.hw_type, mapping.subsystem, mapping.item_id) },
                    ]);
                }
            }
            return mappingTable.generateBox();
        }
        else {
            return null;
        }
    }
}
exports.Mapping = Mapping;
//# sourceMappingURL=Mapping.js.map