"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Overview = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const dataSource_1 = require("../../../config/dataSource");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const Color_1 = require("../../metrics/Color");
const Definitions_1 = require("../../metrics/Definitions");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const Storage_1 = require("../../metrics/class/Storage");
const ConfigurationTable_1 = require("./ConfigurationTable");
const Nutanix_1 = require("./Nutanix");
const Ovirt_1 = require("./Ovirt");
const PerformanceTable_1 = require("./PerformanceTable");
const Power_1 = require("./Power");
const Proxmox_1 = require("./Proxmox");
const Table_1 = require("./Table");
const Vmware_1 = require("./Vmware");
const class_constants_1 = require("../../../common/constants/class.constants");
const CapacityTableBuilder_1 = require("./CapacityTableBuilder");
class Overview {
    constructor(hw_type) {
        this.categories = [];
        this.hw_type = hw_type;
    }
    addCategory(label, items) {
        this.categories.push({
            label: label,
            items: items
        });
    }
    form() {
        return {
            type: pageResponse_constants_1.BOX_TYPE.OverviewBox,
            categories: this.categories
        };
    }
    static async getPage(request, req) {
        const item = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findOne({
            where: {
                item_id: request.item_id
            },
            relations: {
                hw_type_entity: true
            }
        });
        if (!item) {
            return null;
        }
        let tab = {
            title: "Overview",
            content: []
        };
        if (item.hw_type_entity.class === class_constants_1.ClassKey.STORAGE) {
            tab = await Overview['storage'](item, request, req);
        }
        if (Overview[item.hw_type]) {
            tab = await Overview[item.hw_type](item, request);
        }
        if (tab.content.length !== 0) {
            tab.export = await this.createPdfExport(tab, request, item);
        }
        return tab;
    }
    static async vmware(item, request) {
        let tab = {
            title: "Overview",
            content: []
        };
        if (item.subsystem === 'cluster') {
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                text: "Configuration"
            });
            const esxis = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                where: {
                    subsystem: 'esxi',
                    parents: {
                        parent: item.item_id
                    }
                }, relations: {
                    parents: true
                }
            });
            const respools = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                where: {
                    subsystem: 'respool',
                    parents: {
                        parent: item.item_id
                    }
                }, relations: {
                    parents: true
                }
            });
            const esxiConfigurationTable = new ConfigurationTable_1.ConfigurationTable(esxis.map(item => item.item_id), ['vcente_name', 'cluste_name', 'host_ghz', 'host_memorySize_gb', 'hw_numCpuCores', 'hw_numCpuThreads', 'sw_fullName', 'esxi_uptime_days', 'hw_vendor', 'hw_model', 'hw_cpuModel']);
            esxiConfigurationTable.setItemLabel("ESXi");
            esxiConfigurationTable.setSumProperties(['host_ghz', 'host_memorySize_gb', 'hw_numCpuCores', 'hw_numCpuThreads']);
            tab.content.push(await esxiConfigurationTable.generate());
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                text: "Resource pools configuration"
            });
            const respoolConfigurationTable = new ConfigurationTable_1.ConfigurationTable(respools.map(item => item.item_id), ['cpu_reservation', 'cpu_limit', 'cpu_alloc_type', 'cpu_shares', 'cpu_value', 'mem_reservation', 'mem_limit', 'mem_alloc_type', 'mem_shares', 'mem_value']);
            respoolConfigurationTable.setItemLabel("Resource Pool");
            tab.content.push(await respoolConfigurationTable.generate());
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                text: "Performance"
            });
            const cpuMetrics = [{
                    label: 'CPU Total effective',
                    metric: 'cpu_total',
                    ...Definitions_1.metricTypes.cpu_ghz
                }, {
                    label: 'CPU Utilization',
                    metric: 'cpu_usage',
                    ...Definitions_1.metricTypes.cpu_ghz
                }];
            const memoryMetrics = [{
                    label: 'Memory Total effective',
                    metric: 'memory_total',
                    ...Definitions_1.metricTypes.memory
                }, {
                    label: 'Memory Granted',
                    metric: 'memory_granted',
                    ...Definitions_1.metricTypes.memory
                }, {
                    label: 'Memory Consumed',
                    metric: 'memory_consumed',
                    ...Definitions_1.metricTypes.memory
                }, {
                    label: 'Memory Active',
                    metric: 'memory_active',
                    ...Definitions_1.metricTypes.memory
                }, {
                    label: 'Memory Balloon',
                    metric: 'memory_baloon',
                    ...Definitions_1.metricTypes.memory
                }, {
                    label: 'Memory Swap out',
                    metric: 'memory_swap',
                    ...Definitions_1.metricTypes.memory
                }];
            const maxAvgValues = await new MetricsWrapper_1.MetricsWrapper(cpuMetrics.map(metric => metric.metric).concat(memoryMetrics.map(metric => metric.metric)), [item.item_id]).getAvgAndMaxByTime(request.start, request.end);
            let perfCpuTable = new Table_1.Table()
                .addHeader({ value: item.label })
                .addHeader({ value: 'average', number: true, decimal: Definitions_1.metricTypes.cpu_ghz.defaultPrecision, unit: Definitions_1.metricTypes.cpu_ghz.unit, prefix: Definitions_1.metricTypes.cpu_ghz.defaultPrefix })
                .addHeader({ value: 'maximum', number: true, decimal: Definitions_1.metricTypes.cpu_ghz.defaultPrecision, unit: Definitions_1.metricTypes.cpu_ghz.unit, prefix: Definitions_1.metricTypes.cpu_ghz.defaultPrefix });
            for (const metric of cpuMetrics) {
                perfCpuTable.addRow([
                    { order: 0, value: metric.label },
                    { order: 1, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].avg : null },
                    { order: 2, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].max : null }
                ]);
            }
            tab.content.push(perfCpuTable.generateBox());
            let perfMemoryTable = new Table_1.Table()
                .addHeader({ value: item.label })
                .addHeader({ value: 'average', number: true, decimal: Definitions_1.metricTypes.memory.defaultPrecision, unit: Definitions_1.metricTypes.memory.unit, prefix: Definitions_1.metricTypes.memory.defaultPrefix })
                .addHeader({ value: 'maximum', number: true, decimal: Definitions_1.metricTypes.memory.defaultPrecision, unit: Definitions_1.metricTypes.memory.unit, prefix: Definitions_1.metricTypes.memory.defaultPrefix });
            for (const metric of memoryMetrics) {
                perfMemoryTable.addRow([
                    { order: 0, value: metric.label },
                    { order: 1, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].avg : null },
                    { order: 2, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].max : null }
                ]);
            }
            tab.content.push(perfMemoryTable.generateBox());
            const vms = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                where: {
                    subsystem: 'vm',
                    parents: {
                        parent: item.item_id
                    }
                }, relations: {
                    parents: true
                }
            });
            tab.content.push({
                label: "Cluster",
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                metrics: [
                    new Vmware_1.VmwareMetrics.Metric('cluster', 'cpu_total')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.metric.total),
                    new Vmware_1.VmwareMetrics.Metric('cluster', 'cpu_usage')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.red)
                ],
                uuids: [item.item_id],
                aggregated: false,
                start: request.start,
                end: request.end
            });
            tab.content.push({
                label: "Cluster",
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                metrics: [
                    new Vmware_1.VmwareMetrics.Metric('cluster', 'memory_total')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.metric.total),
                    new Vmware_1.VmwareMetrics.Metric('cluster', 'memory_granted')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark),
                    new Vmware_1.VmwareMetrics.Metric('cluster', 'memory_consumed')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 2, '#00FFFF'),
                    new Vmware_1.VmwareMetrics.Metric('cluster', 'memory_active')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 3, Color_1.Color.pallete.red),
                    new Vmware_1.VmwareMetrics.Metric('cluster', 'memory_baloon')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 4, '#FFFF00'),
                    new Vmware_1.VmwareMetrics.Metric('cluster', 'memory_swap')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 5, '#FFA000'),
                ],
                uuids: [item.item_id],
                aggregated: false,
                start: request.start,
                end: request.end
            });
            tab.content.push({
                label: "Server",
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                metrics: [
                    new Vmware_1.VmwareMetrics.Metric('esxi', 'cpu_usage')
                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                ],
                uuids: esxis.map(esxi => esxi.item_id),
                aggregated: true,
                start: request.start,
                end: request.end
            });
            tab.content.push({
                label: "Server",
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                metrics: [
                    new Vmware_1.VmwareMetrics.Metric('vm', 'network_received')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1, Color_1.Color.pallete.red),
                    new Vmware_1.VmwareMetrics.Metric('vm', 'network_transmitted')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1, Color_1.Color.pallete.blue)
                ],
                uuids: esxis.map(esxi => esxi.item_id),
                aggregated: true,
                start: request.start,
                end: request.end
            });
            tab.content.push({
                label: "VM",
                parent: {
                    label: 'ESXi',
                    subsystem: 'esxi'
                },
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                metrics: [
                    new Vmware_1.VmwareMetrics.Metric('vm', 'cpu_usage')
                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                ],
                uuids: vms.map(vm => vm.item_id),
                aggregated: true,
                start: request.start,
                end: request.end
            });
        }
        return tab;
    }
    static async power(item, request) {
        let tab = {
            title: "Overview",
            content: []
        };
        if (item.subsystem === 'server') {
            let serverTable = new Table_1.Table()
                .addHeader({ value: 'Server', property: 'SystemName', url: '{{this}}' })
                .addHeader({ value: 'State', property: 'State' })
                .addHeader({ value: 'MachineType', property: 'MachineType' })
                .addHeader({ value: 'Model', property: 'Model' })
                .addHeader({ value: 'SerialNumber', property: 'SerialNumber' })
                .addHeader({ value: 'Cores', property: 'InstalledSystemProcessorUnits', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.cpu_core })
                .addHeader({ value: 'Cores Free', property: 'CurrentAvailableSystemProcessorUnits', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.cpu_core })
                .addHeader({ value: 'Memory', property: 'InstalledSystemMemory', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.bytes, prefix: pageResponse_constants_1.UNIT_PREFIX.gibi, decimal: 0 })
                .addHeader({ value: 'Memory Free', property: 'CurrentAvailableSystemMemory', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.bytes, prefix: pageResponse_constants_1.UNIT_PREFIX.gibi, decimal: 0 })
                .addHeader({ value: 'ConfigurableSystemProcessorUnits', property: 'ConfigurableSystemProcessorUnits' })
                .addHeader({ value: 'SystemTime', property: 'SystemTime', unit: subsystemMetricsResponse_constants_1.UNIT.timestamp, number: true })
                .addHeader({ value: 'ConfigurableSystemMemory', property: 'ConfigurableSystemMemory', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.bytes, prefix: pageResponse_constants_1.UNIT_PREFIX.gibi, decimal: 0 })
                .addHeader({ value: 'SystemFirmware', property: 'SystemFirmware' });
            await serverTable.addPropertiesItems([item.item_id]);
            tab.content.push(serverTable.generatePivotBox());
            const vms = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'vm',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            tab.content.push({
                label: "Server",
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                metrics: [
                    new Power_1.PowerMetrics.cpu_physical_cores().init(),
                    new Power_1.PowerMetrics.ServerCpu_utilization_coresMetric().init(),
                    new Power_1.PowerMetrics.ServertotalProcUnitsMetric().init(),
                ],
                uuids: [item.item_id],
                aggregated: false,
                start: request.start,
                end: request.end
            });
            tab.content.push({
                label: "Server",
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                metrics: [
                    new Power_1.PowerMetrics.ServerassignedMemToLparsMetric().init(),
                    new Power_1.PowerMetrics.ServeravailableMemMetric().init(),
                    new Power_1.PowerMetrics.ServersystemFirmwareUtilAssignedMemMetric().init(),
                ],
                uuids: [item.item_id],
                aggregated: false,
                start: request.start,
                end: request.end
            });
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                label: "LPAR Totals CPU phys",
                metrics: [
                    new Power_1.PowerMetrics.VmutilizedProcUnitsAggrMetric().init(),
                ],
                uuids: vms.map(item => item.item_id),
                aggregated: true,
                start: request.start,
                end: request.end
            });
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                label: "LPAR Totals CPU phys %",
                metrics: [
                    new Power_1.PowerMetrics.VmutilizedProcUnitsPercentAggrMetric().init(),
                ],
                uuids: vms.map(item => item.item_id),
                aggregated: true,
                start: request.start,
                end: request.end
            });
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                label: "LPAR Totals CPU usage",
                metrics: [
                    new Power_1.PowerMetrics.VmCpu_utilization_coresMetric().init(),
                ],
                uuids: vms.map(item => item.item_id),
                aggregated: true,
                start: request.start,
                end: request.end
            });
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                label: "LPAR Totals CPU usage %",
                metrics: [
                    new Power_1.PowerMetrics.VmCpu_utilization_percentMetric().init(),
                ],
                uuids: vms.map(item => item.item_id),
                aggregated: true,
                start: request.start,
                end: request.end
            });
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                label: "LPAR Totals Memory",
                metrics: [
                    new Power_1.PowerMetrics.VmLogicalMemAggrMetric().init(),
                ],
                uuids: vms.map(item => item.item_id),
                aggregated: true,
                start: request.start,
                end: request.end
            });
            const sea = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'sea',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            if (sea.length > 0) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "SEA Data",
                    metrics: [
                        new Power_1.PowerMetrics.SeareceivedBytesAggrMetric().init(), new Power_1.PowerMetrics.SeasentBytesAggrMetric().init()
                    ],
                    uuids: sea.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "SEA Packets",
                    metrics: [
                        new Power_1.PowerMetrics.SeareceivedPacketsAggrMetric().init(), new Power_1.PowerMetrics.SeasentPacketsAggrMetric().init()
                    ],
                    uuids: sea.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
            }
            const fibers = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'fiber',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            if (fibers.length > 0) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "SAN Data",
                    metrics: [
                        new Power_1.PowerMetrics.FiberreadBytesAggrMetric().init(), new Power_1.PowerMetrics.FiberwriteBytesAggrMetric().init()
                    ],
                    uuids: fibers.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "SAN IO",
                    metrics: [
                        new Power_1.PowerMetrics.FibernumOfReadsAggrMetric().init(), new Power_1.PowerMetrics.FibernumOfWritesAggrMetric().init()
                    ],
                    uuids: fibers.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
            }
            const generic = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'generic',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            if (generic.length > 0) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "LAN Data",
                    metrics: [
                        new Power_1.PowerMetrics.GenericreceivedBytesAggrMetric().init(), new Power_1.PowerMetrics.GenericsentBytesAggrMetric().init()
                    ],
                    uuids: generic.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "LAN Packets",
                    metrics: [
                        new Power_1.PowerMetrics.GenericreceivedPacketsAggrMetric().init(), new Power_1.PowerMetrics.GenericsentPacketsAggrMetric().init()
                    ],
                    uuids: generic.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
            }
            const genericv = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'genericv',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            if (genericv.length > 0) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "vLAN Data",
                    metrics: [
                        new Power_1.PowerMetrics.GenericvreceivedBytesAggrMetric().init(), new Power_1.PowerMetrics.GenericvsentBytesAggrMetric().init()
                    ],
                    uuids: genericv.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "vLAN Packets",
                    metrics: [
                        new Power_1.PowerMetrics.GenericvreceivedPacketsAggrMetric().init(), new Power_1.PowerMetrics.GenericvsentPacketsAggrMetric().init()
                    ],
                    uuids: genericv.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
            }
            const genericvirtual = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'genericvirtual',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            if (genericvirtual.length > 0) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "vSCSI Data",
                    metrics: [
                        new Power_1.PowerMetrics.GenericvirtualreadBytesAggrMetric().init(), new Power_1.PowerMetrics.GenericvirtualwriteBytesAggrMetric().init()
                    ],
                    uuids: genericvirtual.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "vSCSI IO",
                    metrics: [
                        new Power_1.PowerMetrics.GenericvirtualnumOfReadsAggrMetric().init(), new Power_1.PowerMetrics.GenericvirtualnumOfWritesAggrMetric().init()
                    ],
                    uuids: genericvirtual.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
            }
            const genericphysical = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'genericphysical',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            if (genericphysical.length > 0) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "SAS Data",
                    metrics: [
                        new Power_1.PowerMetrics.GenericphysicalreadBytesAggrMetric().init(), new Power_1.PowerMetrics.GenericphysicalwriteBytesAggrMetric().init()
                    ],
                    uuids: genericphysical.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "SAS IO",
                    metrics: [
                        new Power_1.PowerMetrics.GenericphysicalnumOfReadsAggrMetric().init(), new Power_1.PowerMetrics.GenericphysicalnumOfWritesAggrMetric().init()
                    ],
                    uuids: genericphysical.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
            }
            const hea = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'hea',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            if (hea.length > 0) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "HEA Data",
                    metrics: [
                        new Power_1.PowerMetrics.HeareceivedBytesAggrMetric().init(), new Power_1.PowerMetrics.HeasentBytesAggrMetric().init()
                    ],
                    uuids: hea.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "HEA Packets",
                    metrics: [
                        new Power_1.PowerMetrics.HeareceivedPacketsAggrMetric().init(), new Power_1.PowerMetrics.HeasentPacketsAggrMetric().init()
                    ],
                    uuids: hea.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
            }
            const sharedstoragepools = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'sharedstoragepool',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            if (sharedstoragepools.length > 0) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "Shared Storage Pool Data",
                    metrics: [
                        new Power_1.PowerMetrics.SharedstoragepoolreadBytesAggrMetric().init(), new Power_1.PowerMetrics.SharedstoragepoolwriteBytesAggrMetric().init()
                    ],
                    uuids: hea.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "Shared Storage Pool IO",
                    metrics: [
                        new Power_1.PowerMetrics.SharedstoragepoolnumOfReadsAggrMetric().init(), new Power_1.PowerMetrics.SharedstoragepoolnumOfWritesAggrMetric().init()
                    ],
                    uuids: hea.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
            }
            const sriovlogical = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'sriovlogical',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            if (sriovlogical.length > 0) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "SR-IOV Logical Data",
                    metrics: [
                        new Power_1.PowerMetrics.SriovlogicalreceivedBytesAggrMetric().init(),
                        new Power_1.PowerMetrics.SriovlogicalsentBytesAggrMetric().init()
                    ],
                    uuids: sriovlogical.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "SR-IOV Logical Packets",
                    metrics: [
                        new Power_1.PowerMetrics.SriovlogicalreceivedPacketsAggrMetric().init(),
                        new Power_1.PowerMetrics.SriovlogicalsentPacketsAggrMetric().init()
                    ],
                    uuids: sriovlogical.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
            }
            const sriovphysical = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'sriovphysical',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            if (sriovphysical.length > 0) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "SR-IOV Physical Data",
                    metrics: [
                        new Power_1.PowerMetrics.SriovphysicalreceivedBytesAggrMetric().init(),
                        new Power_1.PowerMetrics.SriovphysicalsentBytesAggrMetric().init()
                    ],
                    uuids: sriovphysical.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "SR-IOV Physical Packets",
                    metrics: [
                        new Power_1.PowerMetrics.SriovphysicalreceivedPacketsAggrMetric().init(),
                        new Power_1.PowerMetrics.SriovphysicalsentPacketsAggrMetric().init()
                    ],
                    uuids: sriovphysical.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
            }
            const virtualethernet = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: 'virtualethernet',
                    parents: {
                        _parent: {
                            item_id: item.item_id
                        }
                    }
                }
            });
            if (virtualethernet.length > 0) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "Virtual Ethernet Data",
                    metrics: [
                        new Power_1.PowerMetrics.VirtualethernetreceivedBytesAggrMetric().init(),
                        new Power_1.PowerMetrics.VirtualethernetsentBytesAggrMetric().init()
                    ],
                    uuids: virtualethernet.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "Virtual Ethernet Packets",
                    metrics: [
                        new Power_1.PowerMetrics.VirtualethernetreceivedPacketsAggrMetric().init(),
                        new Power_1.PowerMetrics.VirtualethernetsentPacketsAggrMetric().init()
                    ],
                    uuids: virtualethernet.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "Virtual Ethernet Data Physical",
                    metrics: [
                        new Power_1.PowerMetrics.VirtualethernetreceivedPhysicalBytesAggrMetric().init(),
                        new Power_1.PowerMetrics.VirtualethernetsentPhysicalBytesAggrMetric().init()
                    ],
                    uuids: virtualethernet.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                    label: "Virtual Ethernet Packets Physical",
                    metrics: [
                        new Power_1.PowerMetrics.VirtualethernetreceivedPhysicalPacketsAggrMetric().init(),
                        new Power_1.PowerMetrics.VirtualethernetsentPhysicalPacketsAggrMetric().init()
                    ],
                    uuids: virtualethernet.map(item => item.item_id),
                    aggregated: true,
                    start: request.start,
                    end: request.end
                });
            }
        }
        else if (item.subsystem === 'vm') {
            let vmTable = new Table_1.Table()
                .addHeader({ value: 'Server', property: '{{parent:server}}' })
                .addHeader({ value: 'Name', property: '{{this}}' })
                .addHeader({ value: 'State', property: 'PartitionState' })
                .addHeader({ value: 'Processing Units', property: 'CurrentProcessingUnits' })
                .addHeader({ value: 'Processors', property: 'CurrentProcessors' })
                .addHeader({ value: 'Desired Processors', property: 'DesiredProcessors' })
                .addHeader({ value: 'Desired Virtual Processors', property: 'DesiredVirtualProcessors' })
                .addHeader({ value: 'Allocated Virtual Processors', property: 'AllocatedVirtualProcessors' })
                .addHeader({ value: 'Run Processors', property: 'RunProcessors' })
                .addHeader({ value: 'Sharing Mode', property: 'CurrentSharingMode' })
                .addHeader({ value: 'Uncapped Weight', property: 'CurrentUncappedWeight' })
                .addHeader({ value: 'SharedPool', property: 'SharedPoolLabel' })
                .addHeader({ value: 'OS Version', property: 'OperatingSystemVersion' });
            await vmTable.addPropertiesItems([item.item_id]);
            tab.content.push(vmTable.generatePivotBox());
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                label: "LPAR CPU",
                metrics: [
                    new Power_1.PowerMetrics.VmEntitledProcUnitsMetric().init(),
                    new Power_1.PowerMetrics.VmMaxVirtualProcessorsMetric().init(),
                    new Power_1.PowerMetrics.VmUtilizedProcUnitsMetric().init(),
                    new Power_1.PowerMetrics.VmUtilizedProcUnitsMaxMetric().init(),
                ],
                uuids: [item.item_id],
                aggregated: false,
                start: request.start,
                end: request.end
            });
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                label: "LPAR Memory",
                metrics: [
                    new Power_1.PowerMetrics.VmLogicalMemAggrMetric().init()
                ],
                uuids: [item.item_id],
                aggregated: false,
                start: request.start,
                end: request.end
            });
        }
        return tab;
    }
    static async ovirt(item, request) {
        let tab = {
            title: "Overview",
            content: []
        };
        const vms = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            relations: {
                parents: true
            },
            where: {
                subsystem: 'vm',
                parents: {
                    parent: item.item_id
                }
            }
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.TextBox,
            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
            text: "Configuration"
        });
        const hostConfigurationTable = new ConfigurationTable_1.ConfigurationTable([item.item_id], ['fqdn_or_ip'], ['datacenter', 'cluster']);
        tab.content.push(await hostConfigurationTable.generate());
        const configurations = await MetricsWrapper_1.MetricsWrapper.getConfigurationFindClass(item.hw_type, item.subsystem);
        const properties = [
            { key: 'memory_size', isNumber: true },
            { key: 'swap_size', isNumber: true },
            { key: 'cpu_model', isNumber: false },
            { key: 'cpu_speed', isNumber: true },
            { key: 'number_of_cores', isNumber: true },
            { key: 'number_of_sockets', isNumber: true },
            { key: 'host_os', isNumber: false },
            { key: 'kernel_version', isNumber: false },
            { key: 'kvm_version', isNumber: false },
            { key: 'threads_per_core', isNumber: true },
            { key: 'hardware_product_name', isNumber: false },
            { key: 'hardware_serial_number', isNumber: false },
        ];
        let hostConfigurationPivotTable = new Table_1.Table();
        for (const property of properties) {
            let isNumber = false;
            if (!configurations[property.key]) {
                hostConfigurationPivotTable.addHeader({
                    value: property.key,
                    number: isNumber ? isNumber : undefined
                });
            }
            else {
                hostConfigurationPivotTable.addHeader({
                    value: configurations[property.key].label ? configurations[property.key].label : property.key,
                    property: property.key,
                    number: property.isNumber ? property.isNumber : undefined,
                    unit: configurations[property.key].unit ? configurations[property.key].unit : undefined,
                    prefix: configurations[property.key].defaultPrefix ? configurations[property.key].defaultPrefix : undefined,
                    decimal: configurations[property.key].decimal !== undefined ? configurations[property.key].decimal : undefined
                });
            }
        }
        await hostConfigurationPivotTable.addPropertiesItems([item.item_id]);
        tab.content.push(hostConfigurationPivotTable.generatePivotBox());
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.TextBox,
            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
            text: "Performance"
        });
        const cpuMetrics = [{
                label: 'CPU Total Cores',
                metric: 'cpu_cores',
                ...Definitions_1.metricTypes.cpu_core
            }, {
                label: 'CPU Cores usage',
                metric: 'cpu_utilization_cores',
                ...Definitions_1.metricTypes.cpu_core
            }];
        const memoryMetrics = [{
                label: 'Memory Used',
                metric: 'used_mem',
                ...Definitions_1.metricTypes.memory
            }, {
                label: 'Memory Free',
                metric: 'free_mem',
                ...Definitions_1.metricTypes.memory
            }];
        const maxAvgValues = await new MetricsWrapper_1.MetricsWrapper(cpuMetrics.map(metric => metric.metric).concat(memoryMetrics.map(metric => metric.metric)), [item.item_id]).getAvgAndMaxByTime(request.start, request.end);
        let perfCpuTable = new Table_1.Table()
            .addHeader({ value: item.label })
            .addHeader({ value: 'average', number: true, decimal: Definitions_1.metricTypes.cpu_core.defaultPrecision, unit: Definitions_1.metricTypes.cpu_core.unit, prefix: Definitions_1.metricTypes.cpu_core.defaultPrefix })
            .addHeader({ value: 'maximum', number: true, decimal: Definitions_1.metricTypes.cpu_core.defaultPrecision, unit: Definitions_1.metricTypes.cpu_core.unit, prefix: Definitions_1.metricTypes.cpu_core.defaultPrefix });
        for (const metric of cpuMetrics) {
            perfCpuTable.addRow([
                { order: 0, value: metric.label },
                { order: 1, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].avg : null },
                { order: 2, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].max : null }
            ]);
        }
        tab.content.push(perfCpuTable.generateBox());
        let perfMemoryTable = new Table_1.Table()
            .addHeader({ value: item.label })
            .addHeader({ value: 'average', number: true, decimal: Definitions_1.metricTypes.memory.defaultPrecision, unit: Definitions_1.metricTypes.memory.unit, prefix: Definitions_1.metricTypes.memory.defaultPrefix })
            .addHeader({ value: 'maximum', number: true, decimal: Definitions_1.metricTypes.memory.defaultPrecision, unit: Definitions_1.metricTypes.memory.unit, prefix: Definitions_1.metricTypes.memory.defaultPrefix });
        for (const metric of memoryMetrics) {
            perfMemoryTable.addRow([
                { order: 0, value: metric.label },
                { order: 1, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].avg : null },
                { order: 2, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].max : null }
            ]);
        }
        tab.content.push(perfMemoryTable.generateBox());
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "Host CPU %",
            metrics: [
                new Ovirt_1.OvirtMetrics.Metric('host', 'cpu_utilization_percent')
                    .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
            ],
            uuids: [item.item_id],
            aggregated: false,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "Host CPU",
            metrics: [
                new Ovirt_1.OvirtMetrics.Metric('host', 'cpu_utilization_cores')
                    .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
            ],
            uuids: [item.item_id],
            aggregated: false,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "Host Memory",
            metrics: [
                new Ovirt_1.OvirtMetrics.Metric('host', 'used_mem')
                    .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1, Color_1.Color.pallete.red),
                new Ovirt_1.OvirtMetrics.Metric('host', 'free_mem')
                    .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 2, Color_1.Color.pallete.green)
            ],
            uuids: [item.item_id],
            aggregated: false,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "VM CPU %",
            metrics: [
                new Ovirt_1.OvirtMetrics.Metric('vm', 'cpu_utilization_percent')
                    .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
            ],
            uuids: vms.map(item => item.item_id),
            aggregated: true,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "VM CPU",
            metrics: [
                new Ovirt_1.OvirtMetrics.Metric('vm', 'cpu_utilization_cores')
                    .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
            ],
            uuids: vms.map(item => item.item_id),
            aggregated: true,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "VM Memory Used",
            metrics: [
                new Ovirt_1.OvirtMetrics.Metric('vm', 'used_mem')
                    .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
            ],
            uuids: vms.map(item => item.item_id),
            aggregated: true,
            start: request.start,
            end: request.end
        });
        return tab;
    }
    static async proxmox(item, request) {
        let tab = {
            title: "Overview",
            content: []
        };
        const vms = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            relations: {
                parents: true
            },
            where: {
                subsystem: 'vm',
                parents: {
                    parent: item.item_id
                }
            }
        });
        const storages = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            relations: {
                parents: true
            },
            where: {
                subsystem: 'storage',
                parents: {
                    parent: item.item_id
                }
            }
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.TextBox,
            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
            text: "Configuration"
        });
        let nodeTable = new Table_1.Table()
            .addHeader({ value: 'Name', property: '{{this}}' })
            .addHeader({ value: 'Cluster', property: 'cluster' })
            .addHeader({ value: 'State', property: 'status' })
            .addHeader({ value: 'Uptime', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.timestamp, property: 'uptime' });
        nodeTable.addPropertiesItems([item.item_id]);
        tab.content.push(nodeTable.generateBox());
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.TextBox,
            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
            text: "Performance"
        });
        const cpuMetrics = [{
                label: 'CPU Total Cores',
                metric: 'maxcpu',
                ...Definitions_1.metricTypes.cpu_core
            }, {
                label: 'CPU Cores usage',
                metric: 'cpu_utilization_cores',
                ...Definitions_1.metricTypes.cpu_core
            }];
        const memoryMetrics = [{
                label: 'Memory Total',
                metric: 'memtotal',
                ...Definitions_1.metricTypes.memory
            }, {
                label: 'Memory Used',
                metric: 'memused',
                ...Definitions_1.metricTypes.memory
            }, {
                label: 'Memory Free',
                metric: 'memfree',
                ...Definitions_1.metricTypes.memory
            }];
        const maxAvgValues = await new MetricsWrapper_1.MetricsWrapper(cpuMetrics.map(metric => metric.metric).concat(memoryMetrics.map(metric => metric.metric)), [item.item_id]).getAvgAndMaxByTime(request.start, request.end);
        let perfCpuTable = new Table_1.Table()
            .addHeader({ value: item.label })
            .addHeader({ value: 'average', number: true, decimal: Definitions_1.metricTypes.cpu_core.defaultPrecision, unit: Definitions_1.metricTypes.cpu_core.unit, prefix: Definitions_1.metricTypes.cpu_core.defaultPrefix })
            .addHeader({ value: 'maximum', number: true, decimal: Definitions_1.metricTypes.cpu_core.defaultPrecision, unit: Definitions_1.metricTypes.cpu_core.unit, prefix: Definitions_1.metricTypes.cpu_core.defaultPrefix });
        for (const metric of cpuMetrics) {
            perfCpuTable.addRow([
                { order: 0, value: metric.label },
                { order: 1, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].avg : null },
                { order: 2, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].max : null }
            ]);
        }
        tab.content.push(perfCpuTable.generateBox());
        let perfMemoryTable = new Table_1.Table()
            .addHeader({ value: item.label })
            .addHeader({ value: 'average', number: true, decimal: Definitions_1.metricTypes.memory.defaultPrecision, unit: Definitions_1.metricTypes.memory.unit, prefix: Definitions_1.metricTypes.memory.defaultPrefix })
            .addHeader({ value: 'maximum', number: true, decimal: Definitions_1.metricTypes.memory.defaultPrecision, unit: Definitions_1.metricTypes.memory.unit, prefix: Definitions_1.metricTypes.memory.defaultPrefix });
        for (const metric of memoryMetrics) {
            perfMemoryTable.addRow([
                { order: 0, value: metric.label },
                { order: 1, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].avg : null },
                { order: 2, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].max : null }
            ]);
        }
        tab.content.push(perfMemoryTable.generateBox());
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "NODE CPU",
            metrics: [new Proxmox_1.ProxmoxMetrics.NodeCpuMetric().init()],
            uuids: [item.item_id],
            aggregated: false,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "NODE Memory",
            metrics: [
                new Proxmox_1.ProxmoxMetrics.NodeMemusedMetric().init(),
                new Proxmox_1.ProxmoxMetrics.NodeMemtotalMetric().init()
            ],
            uuids: [item.item_id],
            aggregated: false,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "VM CPU",
            metrics: [
                new Proxmox_1.ProxmoxMetrics.VmCpuCoresMetric().init()
            ],
            uuids: vms.map(item => item.item_id),
            aggregated: true,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "VM Memory",
            metrics: [
                new Proxmox_1.ProxmoxMetrics.VmMemusedAggrMetric().init()
            ],
            uuids: vms.map(item => item.item_id),
            aggregated: true,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "Storage Capacity",
            metrics: [
                new Proxmox_1.ProxmoxMetrics.StorageUsedAggrMetric().init()
            ],
            uuids: storages.map(item => item.item_id),
            aggregated: true,
            start: request.start,
            end: request.end
        });
        return tab;
    }
    static async nutanix(item, request) {
        let tab = {
            title: "Overview",
            content: []
        };
        const vms = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            relations: {
                parents: true
            },
            where: {
                subsystem: 'vm',
                parents: {
                    parent: item.item_id
                }
            }
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.TextBox,
            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
            text: "Configuration"
        });
        let hostTable = new Table_1.Table()
            .addHeader({ value: 'Host', property: 'name', url: '{{this}}' })
            .addHeader({ value: 'Adress', property: 'address' })
            .addHeader({ value: 'Memory', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.bytes, property: 'memory', prefix: pageResponse_constants_1.UNIT_PREFIX.giga, multiplier: 1024 })
            .addHeader({ value: 'CPU', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.cpu, property: 'cpu_count' })
            .addHeader({ value: 'Socket', property: 'socket_count' })
            .addHeader({ value: 'CPU model', property: 'cpu_model' })
            .addHeader({ value: 'Version', property: 'version' });
        await hostTable.addPropertiesItems([item.item_id]);
        tab.content.push(hostTable.generateBox());
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.TextBox,
            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
            text: "Performance"
        });
        const cpuMetrics = [{
                label: 'CPU Total Cores',
                metric: 'cpu_cores',
                ...Definitions_1.metricTypes.cpu_core
            }, {
                label: 'CPU Cores usage',
                metric: 'cpu_utilization_cores',
                ...Definitions_1.metricTypes.cpu_core
            }];
        const memoryMetrics = [{
                label: 'Memory Total',
                metric: 'memory_total',
                ...Definitions_1.metricTypes.memory
            }, {
                label: 'Memory Used',
                metric: 'memory_used',
                ...Definitions_1.metricTypes.memory
            }, {
                label: 'Memory Free',
                metric: 'memory_free',
                ...Definitions_1.metricTypes.memory
            }];
        const maxAvgValues = await new MetricsWrapper_1.MetricsWrapper(cpuMetrics.map(metric => metric.metric).concat(memoryMetrics.map(metric => metric.metric)), [item.item_id]).getAvgAndMaxByTime(request.start, request.end);
        let perfCpuTable = new Table_1.Table()
            .addHeader({ value: item.label })
            .addHeader({ value: 'average', number: true, decimal: Definitions_1.metricTypes.cpu_core.defaultPrecision, unit: Definitions_1.metricTypes.cpu_core.unit, prefix: Definitions_1.metricTypes.cpu_core.defaultPrefix })
            .addHeader({ value: 'maximum', number: true, decimal: Definitions_1.metricTypes.cpu_core.defaultPrecision, unit: Definitions_1.metricTypes.cpu_core.unit, prefix: Definitions_1.metricTypes.cpu_core.defaultPrefix });
        for (const metric of cpuMetrics) {
            perfCpuTable.addRow([
                { order: 0, value: metric.label },
                { order: 1, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].avg : null },
                { order: 2, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].max : null }
            ]);
        }
        tab.content.push(perfCpuTable.generateBox());
        let perfMemoryTable = new Table_1.Table()
            .addHeader({ value: item.label })
            .addHeader({ value: 'average', number: true, decimal: Definitions_1.metricTypes.memory.defaultPrecision, unit: Definitions_1.metricTypes.memory.unit, prefix: Definitions_1.metricTypes.memory.defaultPrefix })
            .addHeader({ value: 'maximum', number: true, decimal: Definitions_1.metricTypes.memory.defaultPrecision, unit: Definitions_1.metricTypes.memory.unit, prefix: Definitions_1.metricTypes.memory.defaultPrefix });
        for (const metric of memoryMetrics) {
            perfMemoryTable.addRow([
                { order: 0, value: metric.label },
                { order: 1, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].avg : null },
                { order: 2, value: maxAvgValues[item.item_id] && maxAvgValues[item.item_id][metric.metric] ? maxAvgValues[item.item_id][metric.metric].max : null }
            ]);
        }
        tab.content.push(perfMemoryTable.generateBox());
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "NODE CPU",
            metrics: [
                new Nutanix_1.NutanixMetrics.HostCpuCoresUsageMetric().init(),
                new Nutanix_1.NutanixMetrics.HostCpuCoresMetric().init()
            ],
            uuids: [item.item_id],
            aggregated: false,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "NODE Memory",
            metrics: [
                new Nutanix_1.NutanixMetrics.HostMemoryUsageMetric().init(),
                new Nutanix_1.NutanixMetrics.HostMemoryFreeMetric().init()
            ],
            uuids: [item.item_id],
            aggregated: false,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "VM CPU",
            metrics: [
                new Nutanix_1.NutanixMetrics.VmCpuCoresUsageAggrMetric().init()
            ],
            uuids: vms.map(item => item.item_id),
            aggregated: true,
            start: request.start,
            end: request.end
        });
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
            label: "VM Memory",
            metrics: [
                new Nutanix_1.NutanixMetrics.VmMemoryUsageBytesAggrMetric().init()
            ],
            uuids: vms.map(item => item.item_id),
            aggregated: true,
            start: request.start,
            end: request.end
        });
        return tab;
    }
    static async storage(item, request, _request, user = undefined) {
        let tab = {
            title: "Overview",
            content: []
        };
        if (!['ecs'].includes(item.hw_type)) {
            const pools = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: true
                },
                where: {
                    subsystem: 'pool',
                    parents: {
                        parent: item.item_id
                    }
                }
            });
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                text: "Device configuration"
            });
            let configurationTable = new Table_1.Table()
                .addHeader({ value: 'Alias name', property: '{{this}}' })
                .addHeader({ value: 'IP', property: 'ip' })
                .addHeader({ value: 'Model', property: 'model' })
                .addHeader({ value: 'Serial', property: 'serial' })
                .addHeader({ value: 'Version', property: 'version' });
            await configurationTable.addPropertiesItems([item.item_id]);
            tab.content.push(configurationTable.generateBox());
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                text: "Capacity"
            });
            tab.content.push(await new CapacityTableBuilder_1.CapacityTableBuilder("pool", _request)
                .setUser(user)
                .setItemLabel("Pool")
                .addItems(pools.map(i => i.item_id))
                .addDefaultMetrics()
                .addUsed()
                .addTotal()
                .generateBox());
            tab.content.push({
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                text: "Performance statistics"
            });
            const tablesInGroup = [];
            try {
                const poolMetrics = MetricsWrapper_1.MetricsWrapper.getMetricsFromEntity(item.hw_type, 'pool');
                let iopsMetrics = [];
                if (poolMetrics.includes('io_rate')) {
                    iopsMetrics.push({
                        group: "Total",
                        metric: "io_rate",
                        metricType: Definitions_1.metricTypes.iops,
                    });
                }
                if (poolMetrics.includes('read_io')) {
                    iopsMetrics.push({
                        group: "Read",
                        metric: "read_io",
                        metricType: Definitions_1.metricTypes.iops,
                    });
                }
                if (poolMetrics.includes('write_io')) {
                    iopsMetrics.push({
                        group: "Write",
                        metric: "write_io",
                        metricType: Definitions_1.metricTypes.iops,
                    });
                }
                if (iopsMetrics.length !== 0) {
                    let iopsTable = new PerformanceTable_1.PerformanceTable(iopsMetrics, {
                        start: request.start,
                        end: request.end
                    }, "IO");
                    iopsTable.addItemIds(pools.map(pool => pool.item_id));
                    tablesInGroup.push(await iopsTable.genTable());
                }
            }
            catch (e) {
                console.warn(`[Overview] ${item.hw_type} ${e.message}`);
            }
            try {
                const poolMetrics = MetricsWrapper_1.MetricsWrapper.getMetricsFromEntity(item.hw_type, 'pool');
                let dataMetrics = [];
                if (poolMetrics.includes('data_rate')) {
                    dataMetrics.push({
                        group: "Total",
                        metric: "data_rate",
                        metricType: Definitions_1.metricTypes.throughput,
                    });
                }
                if (poolMetrics.includes('read')) {
                    dataMetrics.push({
                        group: "Read",
                        metric: "read",
                        metricType: Definitions_1.metricTypes.throughput,
                    });
                }
                if (poolMetrics.includes('write')) {
                    dataMetrics.push({
                        group: "Write",
                        metric: "write",
                        metricType: Definitions_1.metricTypes.throughput,
                    });
                }
                if (dataMetrics.length !== 0) {
                    let dataTable = new PerformanceTable_1.PerformanceTable(dataMetrics, {
                        start: request.start,
                        end: request.end
                    }, "Data");
                    dataTable.addItemIds(pools.map(pool => pool.item_id));
                    tablesInGroup.push(await dataTable.genTable());
                }
            }
            catch (e) {
                console.warn(`[Overview] ${item.hw_type} ${e.message}`);
            }
            try {
                const poolMetrics = MetricsWrapper_1.MetricsWrapper.getMetricsFromEntity(item.hw_type, 'pool');
                let latencyMetrics = [];
                if (poolMetrics.includes('resp_t')) {
                    latencyMetrics.push({
                        group: "Total",
                        metric: "resp_t",
                        metricType: Definitions_1.metricTypes.latency,
                    });
                }
                if (poolMetrics.includes('resp_t_r')) {
                    latencyMetrics.push({
                        group: "Read",
                        metric: "resp_t_r",
                        metricType: Definitions_1.metricTypes.latency,
                    });
                }
                if (poolMetrics.includes('resp_t_w')) {
                    latencyMetrics.push({
                        group: "Write",
                        metric: "resp_t_w",
                        metricType: Definitions_1.metricTypes.latency,
                    });
                }
                if (latencyMetrics.length !== 0) {
                    let latencyTable = new PerformanceTable_1.PerformanceTable(latencyMetrics, {
                        start: request.start,
                        end: request.end
                    }, "Latency");
                    latencyTable.addItemIds(pools.map(pool => pool.item_id));
                    latencyTable.setHeated();
                    tablesInGroup.push(await latencyTable.genTable());
                }
            }
            catch (e) {
                console.warn(`[Overview] ${item.hw_type} ${e.message}`);
            }
            try {
                const poolMetrics = MetricsWrapper_1.MetricsWrapper.getMetricsFromEntity(item.hw_type, 'pool');
                let blocksizeMetrics = [];
                if (poolMetrics.includes('block_size')) {
                    blocksizeMetrics.push({
                        group: "Total",
                        metric: "block_size",
                        metricType: Definitions_1.metricTypes.block_size,
                    });
                }
                if (poolMetrics.includes('block_size_r')) {
                    blocksizeMetrics.push({
                        group: "Read",
                        metric: "block_size_r",
                        metricType: Definitions_1.metricTypes.block_size,
                    });
                }
                if (poolMetrics.includes('block_size_w')) {
                    blocksizeMetrics.push({
                        group: "Write",
                        metric: "block_size_w",
                        metricType: Definitions_1.metricTypes.block_size,
                    });
                }
                if (blocksizeMetrics.length !== 0) {
                    let blocksizeTable = new PerformanceTable_1.PerformanceTable(blocksizeMetrics, {
                        start: request.start,
                        end: request.end
                    }, "Block size");
                    blocksizeTable.addItemIds(pools.map(pool => pool.item_id));
                    tablesInGroup.push(await blocksizeTable.genTable());
                }
            }
            catch (e) {
                console.warn(`[Overview] ${item.hw_type} ${e.message}`);
            }
            if (tablesInGroup.length >= 1) {
                tab.content.push({
                    type: pageResponse_constants_1.BOX_TYPE.TableGroupBox,
                    tables: tablesInGroup
                });
            }
        }
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.TextBox,
            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
            text: "Graphs"
        });
        const metrics = ['io_rate', 'data_rate', 'resp_t', 'block_size', 'capacity_used'];
        const hwTypeDefinition = MetricsWrapper_1.MetricsWrapper.getHwTypeDefinition(class_constants_1.ClassKey.STORAGE, item.hw_type);
        for (const subsystem in hwTypeDefinition.subsystems) {
            if (subsystem === 'device') {
                continue;
            }
            try {
                const subsystemDefinition = hwTypeDefinition.subsystems[subsystem];
                const items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                    relations: {
                        parents: true
                    },
                    where: {
                        subsystem: subsystem,
                        parents: {
                            parent: item.item_id
                        }
                    }
                });
                if (items.length === 0)
                    continue;
                const columns = MetricsWrapper_1.MetricsWrapper.getMetricsFromEntity(item.hw_type, subsystem);
                const parentType = MetricsWrapper_1.MetricsWrapper.getParent(class_constants_1.ClassKey.STORAGE, item.hw_type, subsystem);
                for (const metric of metrics) {
                    if (columns.includes(metric)) {
                        const definition = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.STORAGE, item.hw_type, subsystem, metric);
                        if (!definition || !definition.metric) {
                            console.warn(`[Overview] [${item.hw_type}] [${subsystem}] metric definition ${metric} not found but exists in database entity`);
                        }
                        const metricObject = Storage_1.Storage.metrics[metric];
                        const metricLabel = metricObject ? metricObject.label : metric;
                        const isRespt = metric === 'resp_t';
                        const isBlocksize = metric === 'block_size';
                        tab.content.push({
                            type: pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox,
                            label: subsystemDefinition.label ? subsystemDefinition.label : subsystem,
                            graphLabel: subsystemDefinition.label ? `${subsystemDefinition.label} - ${metricLabel}` : metricLabel,
                            parent: parentType ? {
                                label: parentType.label,
                                subsystem: parentType.subsystem
                            } : undefined,
                            metrics: [{
                                    ...metricObject,
                                    type: (isRespt || isBlocksize) ? pageResponse_constants_1.CHART_TYPE.line : pageResponse_constants_1.CHART_TYPE.area,
                                    stacked: (isRespt || isBlocksize) ? false : true,
                                    order: 0,
                                }],
                            uuids: items.map(item => item.item_id),
                            aggregated: true,
                            start: request.start,
                            end: request.end
                        });
                    }
                }
            }
            catch (e) {
                console.warn(`[Overview] Error occurred while processing subsystem ${subsystem}: ${e.message}`);
                continue;
            }
        }
        return tab;
    }
    static async createPdfExport(tab, request, item) {
        if (tab.content) {
            const reportItems = await this.tabContentToReportItems(tab, request);
            const report = {
                label: item ? 'Overview: ' + item.label : 'unlabeled-report',
                report_items: reportItems
            };
            return {
                fileName: item ? 'overview-' + item.subsystem + "-" + item.label : 'unlabeled-report',
                url: '/api/reporter/v1/generate',
                method: 'POST',
                body: report
            };
        }
        else {
            return undefined;
        }
    }
    static async tabContentToReportItems(tab, request) {
        const reportItems = [];
        let i = 0;
        if (tab.content) {
            for (const box of tab.content) {
                switch (box.type) {
                    case pageResponse_constants_1.BOX_TYPE.SingleTimeRangeGraphBox:
                        if (box.uuids.length >= 1) {
                            const item = await (0, dataSource_1.getDataSource)()
                                .getRepository(objectItem_entity_1.ObjectItem)
                                .findOneBy({ item_id: box.uuids[0] });
                            reportItems.push({
                                item_index: i,
                                class: 'pdf_export',
                                format: 'graph',
                                type: 'metric',
                                subsystem: item.subsystem,
                                hw_type: item.hw_type,
                                data: {
                                    metrics: box.metrics.map(metric => metric.metric),
                                    item_ids: box.uuids,
                                    stacked: box.aggregated ? true : false,
                                    time_range: {
                                        date_from: new Date(request.start * 1000),
                                        date_to: new Date(request.end * 1000)
                                    }
                                }
                            });
                        }
                        i++;
                        break;
                    case pageResponse_constants_1.BOX_TYPE.TableBox:
                        delete box['export'];
                        reportItems.push({
                            item_index: i,
                            class: 'pdf_export',
                            format: 'table',
                            type: 'box',
                            data: box
                        });
                        i++;
                        break;
                    case pageResponse_constants_1.BOX_TYPE.TextBox:
                        delete box['export'];
                        reportItems.push({
                            item_index: i,
                            class: 'pdf_export',
                            format: 'text',
                            type: 'text',
                            data: { text: box['text'], style: "h3" }
                        });
                        i++;
                        break;
                    case pageResponse_constants_1.BOX_TYPE.CapacityTableBox:
                        delete box['export'];
                        reportItems.push({
                            item_index: i,
                            class: 'pdf_export',
                            format: 'table',
                            type: 'box',
                            data: box
                        });
                        i++;
                        break;
                    case pageResponse_constants_1.BOX_TYPE.TableGroupBox:
                        delete box['export'];
                        reportItems.push({
                            item_index: i,
                            class: 'pdf_export',
                            format: 'table',
                            type: 'box',
                            data: box
                        });
                        i++;
                        break;
                    case pageResponse_constants_1.BOX_TYPE.TablePivotBox:
                        delete box['export'];
                        reportItems.push({
                            item_index: i,
                            class: 'pdf_export',
                            format: 'table',
                            type: 'box',
                            data: box
                        });
                        i++;
                        break;
                    default:
                        break;
                }
            }
        }
        return reportItems;
    }
}
exports.Overview = Overview;
//# sourceMappingURL=Overview.js.map