"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OvirtMetrics = exports.OvirtDiskPage = exports.OvirtStoragedomainPage = exports.OvirtStoragedomaintotalsPage = exports.OvirtVmPage = exports.OvirtHostnicPage = exports.OvirtHostnictotalsPage = exports.OvirtHostPage = exports.OvirtClustertotalsPage = exports.OvirtStoragedomainitemFolder = exports.OvirtStoragedomainFolder = exports.OvirtVmFolder = exports.OvirtHostnicitemsFolder = exports.OvirtHostnicFolder = exports.OvirtHostFolder = exports.OvirtClusterFolder = exports.OvirtDatacenterFolder = exports.OvirtRootFolder = exports.OvirtOverviewPage = exports.OvirtConfigurationPage = exports.OvirtTopPage = exports.OvirtHeatmapPage = exports.OvirtSingleItemPage = exports.OvirtPage = exports.OvirtSubsystemFolder = exports.OvirtFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const ACLCheck_1 = require("../../acl/ACLCheck");
const Color_1 = require("../../metrics/Color");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const Top_1 = require("./Top");
const URL_1 = require("./URL");
const Heatmap_1 = require("./Heatmap");
const Overview_1 = require("./Overview");
const Table_1 = require("./Table");
const ConfigurationTable_1 = require("./ConfigurationTable");
const Prediction_1 = require("../../metrics/Prediction");
const AbstractMenu_1 = require("./AbstractMenu");
const cpu_peak_help = {
    url: "https://xormon.com/max_vrs_average.php",
    text: "How it works?",
};
class OvirtFolder extends AbstractMenu_1.AbstractFolder {
    constructor() {
        super(...arguments);
        this.hwType = 'ovirt';
    }
}
exports.OvirtFolder = OvirtFolder;
class OvirtSubsystemFolder extends OvirtFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId.toString();
        }
    }
}
exports.OvirtSubsystemFolder = OvirtSubsystemFolder;
class OvirtPage extends AbstractMenu_1.AbstractPage {
    constructor() {
        super(...arguments);
        this.hwType = 'ovirt';
        this.rootFolder = OvirtRootFolder;
    }
}
exports.OvirtPage = OvirtPage;
class OvirtSingleItemPage extends OvirtPage {
    constructor() {
        super(...arguments);
        this.withItemId = true;
    }
}
exports.OvirtSingleItemPage = OvirtSingleItemPage;
class OvirtHeatmapPage extends OvirtPage {
    constructor() {
        super(...arguments);
        this.title = 'Heatmap';
    }
    async getPage() {
        let title = 'Heatmap';
        let tabs = [];
        tabs.push({
            title: 'VM',
            content: Object.values(await new Heatmap_1.Heatmap('ovirt', 'vm', true)
                .virtHeatmapBox(this.request)),
        });
        let vmTable = new Table_1.Table()
            .addHeader({ value: 'VM' })
            .addHeader({ value: 'Datacenter' })
            .addHeader({ value: 'Cluster' })
            .addHeader({ value: 'Host' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true })
            .addHeader({ value: 'Memory', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: 'ovirt',
                subsystem: 'vm'
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                }
            }
        });
        vms = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, vms);
        for (const vm of vms) {
            const datacenter = vm.parents.find(parent => parent._parent.subsystem === 'datacenter');
            const cluster = vm.parents.find(parent => parent._parent.subsystem === 'cluster');
            const host = vm.parents.find(parent => parent._parent.subsystem === 'host');
            const cpu = vm.heatmaps.find(heatmap => heatmap.metric === 'cpu_utilization_percent');
            const memory = vm.heatmaps.find(heatmap => heatmap.metric === 'mem_percent');
            vmTable.addRow([
                { order: 0, value: vm.label, url: URL_1.URL.generate('ovirt', 'vm', vm.item_id) },
                { order: 1, value: datacenter ? datacenter._parent.label : 'undefined' },
                { order: 2, value: cluster ? cluster._parent.label : 'undefined' },
                { order: 3, value: host ? host._parent.label : 'undefined' },
                { order: 4, value: cpu ? cpu.value : null },
                { order: 5, value: memory ? memory.value : null },
            ]);
        }
        vmTable.sortByIndex(4);
        tabs.push({
            title: 'VM table',
            content: [vmTable.generateBox()],
        });
        tabs.push({
            title: 'Host',
            content: Object.values(await new Heatmap_1.Heatmap('ovirt', 'host', true)
                .virtHeatmapBox(this.request)),
        });
        let hostTable = new Table_1.Table()
            .addHeader({ value: 'Host' })
            .addHeader({ value: 'Datacenter' })
            .addHeader({ value: 'Cluster' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true })
            .addHeader({ value: 'Memory', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let hosts = await this.objectItemsRepository.find({
            where: {
                hw_type: 'ovirt',
                subsystem: 'host'
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                }
            }
        });
        hosts = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, hosts);
        for (const host of hosts) {
            const datacenter = host.parents.find(parent => parent._parent.subsystem === 'datacenter');
            const cluster = host.parents.find(parent => parent._parent.subsystem === 'cluster');
            const cpu = host.heatmaps.find(heatmap => heatmap.metric === 'cpu_utilization_percent');
            const memory = host.heatmaps.find(heatmap => heatmap.metric === 'mem_percent');
            hostTable.addRow([
                { order: 0, value: host.label, url: URL_1.URL.generate('ovirt', 'host', host.item_id) },
                { order: 1, value: datacenter ? datacenter._parent.label : 'undefined' },
                { order: 2, value: cluster ? cluster._parent.label : 'undefined' },
                { order: 3, value: cpu ? cpu.value : null },
                { order: 4, value: memory ? memory.value : null },
            ]);
        }
        hostTable.sortByIndex(3);
        tabs.push({
            title: 'Host table',
            content: [hostTable.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OvirtHeatmapPage = OvirtHeatmapPage;
class OvirtTopPage extends OvirtPage {
    constructor() {
        super(...arguments);
        this.title = 'VM TOP';
    }
    async getPage() {
        const top = new Top_1.Top();
        top.setSubsystem('ovirt', 'vm');
        top.setParentSubsystem('datacenter');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.OvirtTopPage = OvirtTopPage;
class OvirtConfigurationPage extends OvirtPage {
    constructor() {
        super(...arguments);
        this.title = 'Configuration';
    }
    async getPage() {
        let tabs = [];
        const hosts = await this.objectItemsRepository.findBy({
            hw_type: hwTypeKey_constants_1.HwTypeKey.OVIRT,
            subsystem: 'host'
        });
        const hostTable = new ConfigurationTable_1.ConfigurationTable(hosts.map(item => item.item_id), ['fqdn_or_ip', 'memory_size', 'swap_size', 'cpu_model', 'cpu_speed', 'number_of_cores', 'number_of_sockets', 'host_os', 'kernel_version', 'kvm_version', 'threads_per_core', 'hardware_product_name', 'hardware_serial_number'], ['datacenter', 'cluster']);
        hostTable.setAcl(this.request);
        hostTable.setItemLabel('Host');
        tabs.push({
            title: 'Host',
            content: [await hostTable.generate()],
        });
        const vms = await this.objectItemsRepository.findBy({
            hw_type: hwTypeKey_constants_1.HwTypeKey.OVIRT,
            subsystem: 'vm'
        });
        const vmTable = new ConfigurationTable_1.ConfigurationTable(vms.map(item => item.item_id), ['vm_type', 'memory_size', 'cpu_per_socket', 'number_of_sockets', 'operating_system'], ['datacenter', 'cluster']);
        vmTable.setAcl(this.request);
        vmTable.setItemLabel('VM');
        tabs.push({
            title: 'VM',
            content: [await vmTable.generate()],
        });
        const storageDomains = await this.objectItemsRepository.findBy({
            hw_type: hwTypeKey_constants_1.HwTypeKey.OVIRT,
            subsystem: 'storage_domain'
        });
        const storageDomainTable = new ConfigurationTable_1.ConfigurationTable(storageDomains.map(item => item.item_id), ['total_disk_size', 'free_disk_size', 'used_disk_size'], ['datacenter']);
        storageDomainTable.setAcl(this.request);
        storageDomainTable.setItemLabel('Storage Domain');
        tabs.push({
            title: 'Storage Domain',
            content: [await storageDomainTable.generate()],
        });
        const disks = await this.objectItemsRepository.findBy({
            hw_type: hwTypeKey_constants_1.HwTypeKey.OVIRT,
            subsystem: 'disk'
        });
        const diskTable = new ConfigurationTable_1.ConfigurationTable(disks.map(item => item.item_id), ['vm_disk_size'], ['datacenter', 'storage_domain', 'vm']);
        diskTable.setAcl(this.request);
        diskTable.setItemLabel('Disk');
        tabs.push({
            title: 'Disk',
            content: [await diskTable.generate()],
        });
        let vmDiskTable = new Table_1.Table()
            .addHeader({ value: 'VM', sort: 'asc' })
            .addHeader({ value: 'Datacenter' })
            .addHeader({ value: 'Storage domain' })
            .addHeader({ value: 'Virtual disk' })
            .addHeader({ value: 'Allocated', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.bytes, prefix: pageResponse_constants_1.UNIT_PREFIX.gibi })
            .addHeader({ value: 'Used', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.bytes, prefix: pageResponse_constants_1.UNIT_PREFIX.gibi })
            .addHeader({ value: 'Total allocated per VM', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.bytes, prefix: pageResponse_constants_1.UNIT_PREFIX.gibi })
            .addHeader({ value: 'Total used per VM', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.bytes, prefix: pageResponse_constants_1.UNIT_PREFIX.gibi });
        let vmsWithParents = await this.objectItemsRepository.find({
            where: {
                hw_type: hwTypeKey_constants_1.HwTypeKey.OVIRT,
                subsystem: 'vm'
            },
            relations: {
                parents: {
                    _parent: true
                }
            }
        });
        let disksWithParents = await this.objectItemsRepository.find({
            where: {
                hw_type: hwTypeKey_constants_1.HwTypeKey.OVIRT,
                subsystem: 'disk'
            },
            relations: {
                parents: {
                    _parent: true
                },
                properties: true
            }
        });
        for (const vm of vmsWithParents) {
            let datacenter = vm.parents.find(parent => parent._parent.subsystem === 'datacenter');
            let vmDatacenter = datacenter ? datacenter._parent.label : null;
            let vmLabel = vm.label ? vm.label : null;
            let vmLink = vm ? URL_1.URL.generate(this.hwType, vm.subsystem, vm.item_id) : undefined;
            let allocatedAggr = 0;
            let usedAggr = 0;
            let diskRows = {};
            let childDisks = disksWithParents.filter(disk => disk.parents.find(parent => parent._parent.item_id === vm.item_id));
            for (const disk of childDisks) {
                let storageDomain = disk.parents.find(parent => parent._parent.subsystem === 'storage_domain');
                let diskStorageDomainLabel = storageDomain ? storageDomain._parent.label : null;
                let diskStorageDomainLink = storageDomain ? URL_1.URL.generate(this.hwType, storageDomain._parent.subsystem, storageDomain._parent.item_id) : undefined;
                let diskLabel = disk.label ? disk.label : null;
                let diskLink = disk ? URL_1.URL.generate(this.hwType, disk.subsystem, disk.item_id) : undefined;
                const diskAllocated = disk.properties.find(property => property.property_name === 'vm_disk_size')?.property_value;
                allocatedAggr += Number(diskAllocated);
                const metricsWrapper = await (new MetricsWrapper_1.MetricsWrapper(['size'], [disk.item_id])).getLastDailyValuesFromTable();
                const diskUsed = metricsWrapper[disk.item_id]?.['size'];
                usedAggr += diskUsed;
                diskRows[disk.item_id] = {
                    storage_domain: diskStorageDomainLabel,
                    storage_domain_link: diskStorageDomainLink,
                    disk: diskLabel,
                    disk_link: diskLink,
                    allocated: +diskAllocated,
                    used: +diskUsed
                };
            }
            for (const key in diskRows) {
                vmDiskTable.addRow([
                    { order: 0, value: vmLabel, url: vmLink },
                    { order: 1, value: vmDatacenter },
                    { order: 2, value: diskRows[key].storage_domain, url: diskRows[key].storage_domain_link },
                    { order: 3, value: diskRows[key].disk, url: diskRows[key].disk_link },
                    { order: 4, value: diskRows[key].allocated },
                    { order: 5, value: diskRows[key].used },
                    { order: 6, value: allocatedAggr },
                    { order: 7, value: usedAggr }
                ]);
            }
        }
        tabs.push({
            title: 'VM-Disk',
            content: [vmDiskTable.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.OvirtConfigurationPage = OvirtConfigurationPage;
class OvirtOverviewPage extends OvirtPage {
    constructor() {
        super(...arguments);
        this.title = 'Overview';
    }
    async getPage() {
        let title = 'Overview';
        let tabs = [];
        const overview = new Overview_1.Overview('ovirt');
        let hosts = await this.objectItemsRepository.findBy({ hw_type: hwTypeKey_constants_1.HwTypeKey.OVIRT, subsystem: 'host' });
        hosts = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, hosts);
        overview.addCategory('Host', hosts.map(item => {
            return {
                label: item.label,
                item_id: item.item_id
            };
        }));
        tabs.push({
            title: 'Host',
            content: [
                overview.form()
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OvirtOverviewPage = OvirtOverviewPage;
class OvirtRootFolder extends OvirtFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration', 'configuration'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Storage Domains', this.hwType + '/storage-domain-totals', 'storage-domain-totals'));
        menuItems.push(new menuResponse_dto_1.MenuLink('VM TOP', this.hwType + '/top', 'top'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Overview', this.hwType + '/overview', 'overview'));
        const datacenters = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: 'datacenter'
        });
        datacenters.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let datacenter of datacenters) {
            let path = this.hwType + '/datacenter?ids=' + datacenter.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(datacenter.label, path, this.hwType, 'datacenter', datacenter.item_id, OvirtDatacenterFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        const datacenters = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: 'datacenter'
        });
        datacenters.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const datacenter of datacenters) {
            let path = this.hwType + '/datacenter?ids=' + datacenter.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(datacenter.label, path, this.hwType, 'datacenter', datacenter.item_id, OvirtDatacenterFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.OvirtRootFolder = OvirtRootFolder;
class OvirtDatacenterFolder extends OvirtSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'datacenter';
    }
    async getChildren() {
        let menuItems = [];
        const clusters = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'cluster',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        clusters.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const cluster of clusters) {
            let path = this.hwType + '/cluster?ids=' + cluster.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(cluster.label, path, this.hwType, 'cluster', cluster.item_id, OvirtClusterFolder));
        }
        menuItems.push(new menuResponse_dto_1.MenuFolder('Storage Domain', this.hwType + '/storage-domain?ids=' + this.itemId, this.hwType, null, this.itemId, OvirtStoragedomainFolder));
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        const clusters = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'cluster',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        clusters.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const cluster of clusters) {
            let path = this.hwType + '/cluster?ids=' + cluster.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(cluster.label, path, this.hwType, 'cluster', cluster.item_id, OvirtClusterFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.OvirtDatacenterFolder = OvirtDatacenterFolder;
class OvirtClusterFolder extends OvirtSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'cluster';
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/cluster-totals' + '?item_id=' + this.itemId, this.subsystem + '-totals', this.itemId));
        const hosts = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'host',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        hosts.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const host of hosts) {
            let path = this.hwType + '/host?ids=' + host.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(host.label, path, this.hwType, 'host', host.item_id, OvirtHostFolder));
        }
        menuItems.push(new menuResponse_dto_1.MenuFolder("VM", this.hwType + '/vm?ids=' + this.itemId, this.hwType, 'vm', this.itemId, OvirtVmFolder));
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuFolder("VM", this.hwType + '/vm?ids=' + this.itemId, this.hwType, 'vm', this.itemId, OvirtVmFolder));
        this.children = menuItems;
        return menuItems;
    }
}
exports.OvirtClusterFolder = OvirtClusterFolder;
class OvirtHostFolder extends OvirtSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'host';
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Host', this.hwType + '/host' + '?item_id=' + this.itemId, this.subsystem, this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuFolder("LAN", this.hwType + '/host-nic?ids=' + this.itemId, this.hwType, 'host_nic', this.itemId, OvirtHostnicFolder));
        this.children = menuItems;
        return menuItems;
    }
}
exports.OvirtHostFolder = OvirtHostFolder;
class OvirtHostnicFolder extends OvirtSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'host_nic';
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/host-nic-totals' + '?item_id=' + this.itemId, this.subsystem, this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuFolder('Items', this.hwType + '/host-nic-items?ids=' + this.itemId, this.hwType, this.subsystem, this.itemId, OvirtHostnicitemsFolder));
        this.children = menuItems;
        return menuItems;
    }
}
exports.OvirtHostnicFolder = OvirtHostnicFolder;
class OvirtHostnicitemsFolder extends OvirtSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'host_nic';
    }
    async getChildren() {
        let menuItems = [];
        const nics = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'host_nic',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        nics.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const nic of nics) {
            menuItems.push(new menuResponse_dto_1.MenuLink(nic.label, this.hwType + '/host-nic' + '?item_id=' + nic.item_id, 'host_nic', nic.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.OvirtHostnicitemsFolder = OvirtHostnicitemsFolder;
class OvirtVmFolder extends OvirtSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
    }
    async getChildren() {
        let menuItems = [];
        const vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        vms.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const vm of vms) {
            menuItems.push(new menuResponse_dto_1.MenuLink(vm.label, this.hwType + '/vm' + '?item_id=' + vm.item_id, 'vm', vm.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        const vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        vms.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const vm of vms) {
            menuItems.push(new menuResponse_dto_1.MenuLink(vm.label, this.hwType + '/vm' + '?item_id=' + vm.item_id, 'vm', vm.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.OvirtVmFolder = OvirtVmFolder;
class OvirtStoragedomainFolder extends OvirtSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        const storage_domains = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'storage_domain',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        storage_domains.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const storage_domain of storage_domains) {
            menuItems.push(new menuResponse_dto_1.MenuFolder(storage_domain.label, this.hwType + '/storage-domain-item' + '?ids=' + storage_domain.item_id, this.hwType, 'storage_domain', storage_domain.item_id, OvirtStoragedomainitemFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.OvirtStoragedomainFolder = OvirtStoragedomainFolder;
class OvirtStoragedomainitemFolder extends OvirtSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'storage_domain';
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/storage-domain-totals' + '?item_id=' + this.itemId, "storage-domain-totals", this.itemId, true));
        const disks = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'disk',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        disks.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const disk of disks) {
            menuItems.push(new menuResponse_dto_1.MenuLink(disk.label, this.hwType + '/disk' + '?item_id=' + disk.item_id, 'disk', disk.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.OvirtStoragedomainitemFolder = OvirtStoragedomainitemFolder;
class OvirtClustertotalsPage extends OvirtSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'cluster';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Cluster ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        let hosts = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'host',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('host', 'cpu_utilization_cores')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: hosts.map(host => host.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'CPU %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('host', 'cpu_utilization_percent')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0)
                    ],
                    uuids: hosts.map(host => host.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Memory Used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('host', 'used_mem')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: hosts.map(host => host.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Memory Free',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('host', 'free_mem')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: hosts.map(host => host.item_id),
                    aggregated: true,
                },
            ],
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'VM CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'cpu_utilization_cores')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Host',
                        subsystem: 'host'
                    }
                },
            ],
        });
        tabs.push({
            title: 'VM CPU %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'cpu_utilization_percent')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Host',
                        subsystem: 'host'
                    }
                },
            ],
        });
        tabs.push({
            title: 'VM Memory Used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'used_mem')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'VM Memory Free',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'free_mem')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OvirtClustertotalsPage = OvirtClustertotalsPage;
class OvirtHostPage extends OvirtSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'host';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Host ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('host', 'cpu_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new OvirtMetrics.Metric('host', 'cpu_utilization_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
        });
        tabs.push({
            title: 'CPU max',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('host', 'cpu_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new OvirtMetrics.Metric('host', 'cpu_utilization_cores_max')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
            help: cpu_peak_help,
        });
        tabs.push({
            title: 'CPU %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('host', 'cpu_utilization_percent')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new OvirtMetrics.Metric('host', 'cpu_utilization_percent')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '5m',
                    command: 'predict_cpu',
                    interval: 'month',
                    max: 100
                }),
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new OvirtMetrics.Metric('host', 'cpu_utilization_percent')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '6h',
                    command: 'predict_cpu',
                    interval: 'year',
                    max: 100
                })
            ],
        });
        tabs.push({
            title: 'CPU % max',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('host', 'cpu_utilization_percent_max')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
            help: cpu_peak_help,
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('host', 'used_mem')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1, Color_1.Color.pallete.red),
                        new OvirtMetrics.Metric('host', 'free_mem')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 2, Color_1.Color.pallete.green)
                    ],
                    uuids: [item.item_id],
                },
            ],
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'VM CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'cpu_utilization_cores')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Host',
                        subsystem: 'host'
                    }
                },
            ],
        });
        tabs.push({
            title: 'VM CPU %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'cpu_utilization_percent')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Host',
                        subsystem: 'host'
                    }
                },
            ],
        });
        tabs.push({
            title: 'VM Memory Used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'used_mem')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'VM Memory Free',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'free_mem')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OvirtHostPage = OvirtHostPage;
class OvirtHostnictotalsPage extends OvirtSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'host_nic';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'LAN ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        let nics = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'host_nic',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('host_nic', 'network_received')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new OvirtMetrics.Metric('host_nic', 'network_transmitted')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                    ],
                    uuids: nics.map(nic => nic.item_id),
                    aggregated: true
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OvirtHostnictotalsPage = OvirtHostnictotalsPage;
class OvirtHostnicPage extends OvirtSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'host_nic';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'LAN ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('host_nic', 'network_received')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new OvirtMetrics.Metric('host_nic', 'network_transmitted')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId]
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OvirtHostnicPage = OvirtHostnicPage;
class OvirtVmPage extends OvirtSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'VM ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'cpu_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new OvirtMetrics.Metric('vm', 'cpu_utilization_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
        });
        tabs.push({
            title: 'CPU max',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'cpu_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new OvirtMetrics.Metric('vm', 'cpu_utilization_cores_max')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
            help: cpu_peak_help,
        });
        tabs.push({
            title: 'CPU %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'cpu_utilization_percent')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new OvirtMetrics.Metric('vm', 'cpu_utilization_percent')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '5m',
                    command: 'predict_vm',
                    interval: 'month',
                    max: 100
                }),
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new OvirtMetrics.Metric('vm', 'cpu_utilization_percent')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '6h',
                    command: 'predict_vm',
                    interval: 'year',
                    max: 100
                }),
            ],
        });
        tabs.push({
            title: 'CPU % max',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'cpu_utilization_percent_max')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
            help: cpu_peak_help,
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm', 'free_mem')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, Color_1.Color.pallete.green),
                        new OvirtMetrics.Metric('vm', 'buffered_mem')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, Color_1.Color.pallete.blue),
                        new OvirtMetrics.Metric('vm', 'cached_mem')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, Color_1.Color.pallete.orange),
                        new OvirtMetrics.Metric('vm', 'used_mem')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, Color_1.Color.pallete.red),
                    ],
                    uuids: [item.item_id],
                },
            ],
        });
        const nics = await this.objectItemsRepository.find({
            where: {
                subsystem: 'vm_nic',
                parents: {
                    parent: item.item_id
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('vm_nic', 'network_received')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, true, -1),
                        new OvirtMetrics.Metric('vm_nic', 'network_transmitted')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, true, 1)
                    ],
                    uuids: nics.map(item => item.item_id),
                    aggregated: true
                },
            ],
        });
        const disks = await this.objectItemsRepository.find({
            where: {
                subsystem: 'disk',
                parents: {
                    parent: item.item_id
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read_iops')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new OvirtMetrics.Metric('disk', 'disk_write_iops')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                    ],
                    uuids: disks.map(item => item.item_id),
                    aggregated: true
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new OvirtMetrics.Metric('disk', 'disk_write')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                    ],
                    uuids: disks.map(item => item.item_id),
                    aggregated: true
                },
            ],
        });
        tabs.push({
            title: 'Latency',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read_latency')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1),
                        new OvirtMetrics.Metric('disk', 'disk_write_latency')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1)
                    ],
                    uuids: disks.map(item => item.item_id),
                    aggregated: true
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OvirtVmPage = OvirtVmPage;
class OvirtStoragedomaintotalsPage extends OvirtSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'storage_domain';
    }
    async getPage() {
        let title = 'Storage domains';
        let tabs = [];
        let disks = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'disk'
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read_iops')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new OvirtMetrics.Metric('disk', 'disk_write_iops')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                    ],
                    uuids: disks.map(disk => disk.item_id),
                    aggregated: true
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new OvirtMetrics.Metric('disk', 'disk_write')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                    ],
                    uuids: disks.map(disk => disk.item_id),
                    aggregated: true
                },
            ],
        });
        tabs.push({
            title: 'Latency',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read_latency')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1),
                        new OvirtMetrics.Metric('disk', 'disk_write_latency')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1)
                    ],
                    uuids: disks.map(disk => disk.item_id),
                    aggregated: true
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OvirtStoragedomaintotalsPage = OvirtStoragedomaintotalsPage;
class OvirtStoragedomainPage extends OvirtSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'storage_domain';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Storage domain ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        let disks = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'disk',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read_iops')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new OvirtMetrics.Metric('disk', 'disk_write_iops')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                    ],
                    uuids: disks.map(disk => disk.item_id),
                    aggregated: true
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new OvirtMetrics.Metric('disk', 'disk_write')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                    ],
                    uuids: disks.map(disk => disk.item_id),
                    aggregated: true
                },
            ],
        });
        tabs.push({
            title: 'Latency',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read_latency')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1),
                        new OvirtMetrics.Metric('disk', 'disk_write_latency')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1)
                    ],
                    uuids: disks.map(disk => disk.item_id),
                    aggregated: true
                },
            ],
        });
        tabs.push({
            title: 'Space',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('storage_domain', 'total')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, false, 0, Color_1.Color.pallete.green),
                        new OvirtMetrics.Metric('storage_domain', 'used')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, false, 1, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [this.itemId]
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OvirtStoragedomainPage = OvirtStoragedomainPage;
class OvirtDiskPage extends OvirtSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'disk';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Disk ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read_iops')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new OvirtMetrics.Metric('disk', 'disk_write_iops')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId]
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new OvirtMetrics.Metric('disk', 'disk_write')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
        });
        tabs.push({
            title: 'Latency',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'disk_read_latency')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new OvirtMetrics.Metric('disk', 'disk_write_latency')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId]
                },
            ],
        });
        tabs.push({
            title: 'Used Space',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OvirtMetrics.Metric('disk', 'size')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1)
                    ],
                    uuids: [this.itemId]
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OvirtDiskPage = OvirtDiskPage;
class OvirtMetrics {
}
exports.OvirtMetrics = OvirtMetrics;
OvirtMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor(subsystem, metric) {
        super();
        this.metric = metric;
        const definition = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.OVIRT, subsystem, metric);
        if (definition) {
            Object.assign(this, definition);
        }
    }
    getGraph(type, stacked, order, color) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.color = color ? color : undefined;
        return this;
    }
    getAggrGraph(type, stacked, order, aggregate) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.aggregate = aggregate ? aggregate : pageResponse_constants_1.METRIC_AGGREGATE.items;
        return this;
    }
};
//# sourceMappingURL=Ovirt.js.map