"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RandomMetrics = exports.RandomDataPage = exports.RandomSingleItemPage = exports.RandomTotals5000sumPage = exports.RandomTotals5000stackedPage = exports.RandomTotals5000Page = exports.RandomTotals1000sumPage = exports.RandomTotals1000stackedPage = exports.RandomTotals1000Page = exports.RandomTotals100sumPage = exports.RandomTotals100stackedPage = exports.RandomTotals100Page = exports.RandomTotalssumPage = exports.RandomTotalsstackedPage = exports.RandomTotalsPage = exports.RandomHeatmapPage = exports.RandomPage = exports.RandomRootFolder = exports.RandomFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const dataSource_1 = require("../../../config/dataSource");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const Color_1 = require("../../metrics/Color");
const Heatmap_1 = require("./Heatmap");
class RandomFolder {
    constructor() {
        this.hwType = 'random';
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
}
exports.RandomFolder = RandomFolder;
class RandomRootFolder extends RandomFolder {
    async getChildren() {
        let menuItems = [];
        let subsystem_cluster = 'data';
        menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals 100 line', this.hwType + '/totals100', 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals 100 stacked', this.hwType + '/totals100stacked', 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals 100 sum', this.hwType + '/totals100sum', 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals 1000 line', this.hwType + '/totals1000', 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals 1000 stacked', this.hwType + '/totals1000stacked', 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals 1000 sum', this.hwType + '/totals1000sum', 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals 5000 line', this.hwType + '/totals5000', 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals 5000 stacked', this.hwType + '/totals5000stacked', 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals 5000 sum', this.hwType + '/totals5000sum', 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals all line', this.hwType + '/totals', 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals all stacked', this.hwType + '/totalsstacked', 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals all sum', this.hwType + '/totalssum', 'data'));
        let items = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: subsystem_cluster,
        });
        for (let item of items) {
            let path = this.hwType + '/data?item_id=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, 'data', item.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.RandomRootFolder = RandomRootFolder;
class RandomPage {
    constructor() {
        this.hwType = 'random';
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
    async getPath() {
        let root = new RandomRootFolder();
        let menuItems = await root.getChildren();
        return menuItems;
    }
}
exports.RandomPage = RandomPage;
class RandomHeatmapPage extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Heatmap';
    }
    async getPage() {
        let title = 'Heatmap';
        let tabs = [];
        tabs.push({
            title: 'VM',
            content: [
                await new Heatmap_1.Heatmap('random', 'data')
                    .addMetric({
                    label: 'CPU',
                    metric: 'cpu',
                })
                    .heatmapBox(),
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.RandomHeatmapPage = RandomHeatmapPage;
class RandomTotalsPage extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuLineAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotalsPage = RandomTotalsPage;
class RandomTotalsstackedPage extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuStackedAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotalsstackedPage = RandomTotalsstackedPage;
class RandomTotalssumPage extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuSumAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotalssumPage = RandomTotalssumPage;
class RandomTotals100Page extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
            take: 100,
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuLineAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotals100Page = RandomTotals100Page;
class RandomTotals100stackedPage extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
            take: 100,
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuStackedAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotals100stackedPage = RandomTotals100stackedPage;
class RandomTotals100sumPage extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
            take: 100,
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuSumAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotals100sumPage = RandomTotals100sumPage;
class RandomTotals1000Page extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
            take: 1000,
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuLineAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotals1000Page = RandomTotals1000Page;
class RandomTotals1000stackedPage extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
            take: 1000,
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuStackedAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotals1000stackedPage = RandomTotals1000stackedPage;
class RandomTotals1000sumPage extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
            take: 1000,
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuSumAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotals1000sumPage = RandomTotals1000sumPage;
class RandomTotals5000Page extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
            take: 5000,
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuLineAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotals5000Page = RandomTotals5000Page;
class RandomTotals5000stackedPage extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
            take: 5000,
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuStackedAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotals5000stackedPage = RandomTotals5000stackedPage;
class RandomTotals5000sumPage extends RandomPage {
    constructor() {
        super(...arguments);
        this.title = 'Totals';
    }
    async getPage() {
        let item = await this.objectItemsRepository.find({
            where: {
                hw_type: 'random',
                subsystem: 'data',
            },
            take: 5000,
        });
        const items = item.map((obj) => {
            return obj.item_id;
        });
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuSumAggrMetric()],
                    uuids: items,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomTotals5000sumPage = RandomTotals5000sumPage;
class RandomSingleItemPage extends RandomPage {
    constructor(itemId) {
        super();
        this.itemId = itemId;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomSingleItemPage = RandomSingleItemPage;
class RandomDataPage extends RandomSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'data';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Data ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new RandomMetrics.CpuLineMetric()],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.RandomDataPage = RandomDataPage;
class RandomMetrics {
}
exports.RandomMetrics = RandomMetrics;
RandomMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'data';
        this.unit = subsystemMetricsResponse_constants_1.UNIT.percent;
        this.shortcut = subsystemMetricsResponse_constants_1.SHORTCUT[subsystemMetricsResponse_constants_1.UNIT.percent];
        this.label = 'CPU%';
    }
};
RandomMetrics.FakeMetric = class extends pageResponse_dto_1.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'data';
        this.unit = subsystemMetricsResponse_constants_1.UNIT.cpu;
        this.shortcut = subsystemMetricsResponse_constants_1.SHORTCUT[subsystemMetricsResponse_constants_1.UNIT.cpu];
        this.label = 'CPU';
    }
};
RandomMetrics.CpuLineMetric = class extends RandomMetrics.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu';
        this.order = 0;
        this.color = Color_1.Color.pallete.red;
    }
};
RandomMetrics.CpuLineAggrMetric = class extends RandomMetrics.Metric {
    constructor() {
        super(...arguments);
        this.aggregate = pageResponse_constants_1.METRIC_AGGREGATE.items;
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu';
        this.order = 0;
    }
};
RandomMetrics.CpuStackedAggrMetric = class extends RandomMetrics.FakeMetric {
    constructor() {
        super(...arguments);
        this.aggregate = pageResponse_constants_1.METRIC_AGGREGATE.items;
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'cpu';
        this.order = 0;
    }
};
RandomMetrics.CpuSumAggrMetric = class extends RandomMetrics.FakeMetric {
    constructor() {
        super(...arguments);
        this.aggregate = pageResponse_constants_1.METRIC_AGGREGATE.sum;
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'cpu';
        this.order = 0;
    }
};
//# sourceMappingURL=Random.js.map