import { MenuItem, Newable } from '@xormon/common/menuResponse.dto';
import { ChartType, HtmlTextBox, Page, TextBox, TopologyRequestBox } from '@xormon/common/pageResponse.dto';
import { Request } from 'express';
import { AbstractPage, StorAbstractFolder, StorAbstractPage } from './AbstractMenu';
export declare const sanGlobalPages: () => MenuItem[];
export declare const sanDeviceTopologyBox: (itemId: string) => Promise<TopologyRequestBox | HtmlTextBox>;
export declare const sanGlobalTopologyBox: () => Promise<TopologyRequestBox | HtmlTextBox>;
export declare class SanFolder extends StorAbstractFolder {
}
export declare class SanSubsystemFolder extends SanFolder {
    subsystem: string;
    itemId: string;
    itemIds: string[];
    constructor(hwType: string, subsystem?: string, itemId?: string[] | string);
}
export declare class SanRootFolder extends SanFolder {
    getChildren(): Promise<MenuItem[]>;
}
export declare class SanFabricFolder extends SanSubsystemFolder {
    subsystem: string;
    getChildren(): Promise<MenuItem[]>;
}
export declare class SanDeviceFolder extends SanSubsystemFolder {
    subsystem: string;
    getChildren(): Promise<any[]>;
}
export declare class SanSubsystemItemsFolder extends SanSubsystemFolder {
    constructor(hwType: string, subsystem?: string, itemId?: string);
    getChildren(): Promise<any[]>;
}
export declare class SanPage extends StorAbstractPage {
    withItemId: boolean;
    rootFolder: Newable<StorAbstractFolder>;
}
export declare class SimpleSanPage extends AbstractPage {
    withItemId: boolean;
    rootFolder: Newable<StorAbstractFolder>;
}
export declare abstract class SanSingleItemPage extends SanPage {
    subsystem: string;
    parentSubsystem: string;
}
export declare class SanItemPage extends SanSingleItemPage {
    pageType: string;
    constructor(hwType: string, pageType: string, itemId: string, request: Request);
    getPage(): Promise<Page>;
}
export declare class SanSanportcpumemoryPage extends SimpleSanPage {
    title: string;
    getPage(): Promise<Page>;
}
export declare class SanTotalsPage extends SimpleSanPage {
    title: string;
    type: string;
    constructor(itemId: string, request: Request, type: string);
    getPage(): Promise<Page>;
}
export declare class SanDataPage extends SanTotalsPage {
    constructor(itemId: string, request: Request);
}
export declare class SanFramePage extends SanTotalsPage {
    constructor(itemId: string, request: Request);
}
export declare class SanFabricPage extends SimpleSanPage {
    getPage(): Promise<Page>;
}
export declare class SanCpumemoryPage extends SanTotalsPage {
    constructor(itemId: string, request: Request);
}
export declare class SanConfigurationPage extends SimpleSanPage {
    title: string;
    getPage(): Promise<Page>;
}
export declare class SanSantopologyPage extends SimpleSanPage {
    title: string;
    getPage(): Promise<Page>;
}
export declare class SanIslPage extends SimpleSanPage {
    title: string;
    getPage(): Promise<Page>;
}
export declare class SanHeatmapPage extends SimpleSanPage {
    title: string;
    getTab(tab: string): Promise<{
        title: string;
        content: any[];
        subTabs: ({
            title: string;
            content: (TextBox | {
                type: "HeatmapBox";
                metrics: import("@xormon/common/pageResponse.dto").IHeatmap[];
            })[];
        } | {
            title: string;
            content: import("@xormon/common/pageResponse.dto").TableBox[];
        })[];
    }>;
    getPage(): Promise<Page>;
}
export declare class SanHealthstatusPage extends SimpleSanPage {
    title: string;
    getPage(): Promise<Page>;
}
export declare class SanMetrics {
    static Metric: {
        new (metric: string): {
            getGraph(type: ChartType, stacked?: boolean, order?: number, color?: string): any;
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            aggregate?: import("@xormon/common/pageResponse.dto").MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            color?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
}
