"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SqlserverMetrics = exports.SqlserverRatioPage = exports.SqlserverLocksPage = exports.SqlserverSqlqueryPage = exports.SqlserverLatencyPage = exports.SqlserverHealthstatusPage = exports.SqlserverTopPage = exports.SqlserverDataPage = exports.SqlserverIoPage = exports.SqlserverCapacityPage = exports.SqlserverWaiteventsPage = exports.SqlserverLatchesPage = exports.SqlserverSessionsPage = exports.SqlserverBuffersPage = exports.SqlserverMemoryPage = exports.SqlserverConfigurationPage = exports.SqlserverOverviewPage = exports.SqlserverSingleItemPage = exports.SqlserverPage = exports.SqlserverDbFolder = exports.SqlserverDbsFolder = exports.SqlserverChildSubsystemFolder = exports.SqlserverSqlsfolderFolder = exports.SqlserverRootFolder = exports.SqlserverSubsystemFolder = exports.SqlserverFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const dataSource_1 = require("../../../config/dataSource");
const itemProperty_entity_1 = require("../../../entity/itemProperty.entity");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const HealthStatus_1 = require("./HealthStatus");
const Table_1 = require("./Table");
const Top_1 = require("./Top");
const typeorm_1 = require("typeorm");
const AbstractMenu_1 = require("./AbstractMenu");
const Color_1 = require("../../metrics/Color");
const TabFactory_1 = require("./TabFactory");
class SqlserverFolder extends AbstractMenu_1.AbstractFolder {
    constructor() {
        super(...arguments);
        this.hwType = 'sqlserver';
    }
}
exports.SqlserverFolder = SqlserverFolder;
class SqlserverSubsystemFolder extends SqlserverFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId;
        }
    }
}
exports.SqlserverSubsystemFolder = SqlserverSubsystemFolder;
class SqlserverRootFolder extends SqlserverFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Health Status', this.hwType + '/health-status', 'health-status'));
        menuItems.push(new menuResponse_dto_1.MenuLink('DB TOP', this.hwType + '/top', 'top'));
        let subsystem_server = 'sqlsfolder';
        let servers = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: subsystem_server,
        });
        servers.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let item of servers) {
            let path = this.hwType + '/' + subsystem_server + '?ids=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, this.hwType, subsystem_server, item.item_id, SqlserverSqlsfolderFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.SqlserverRootFolder = SqlserverRootFolder;
class SqlserverSqlsfolderFolder extends SqlserverSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'sqlsfolder';
        this.parentID = this.itemId;
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Overview', this.hwType + '/overview?item_id=' + this.itemId, 'overview', undefined, true));
        menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration?item_id=' + this.itemId, 'configuration'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Memory', this.hwType + '/memory?item_id=' + this.itemId, 'memory'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Buffers', this.hwType + '/buffers?item_id=' + this.itemId, 'buffers'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Sessions', this.hwType + '/sessions?item_id=' + this.itemId, 'sessions'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Latches', this.hwType + '/latches?item_id=' + this.itemId, 'latches'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Locks', this.hwType + '/locks?item_id=' + this.itemId, 'locks'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Wait events', this.hwType + '/wait-events?item_id=' + this.itemId, 'wait-events'));
        let subsystems = [{
                subsystem: "db",
                folder: SqlserverDbsFolder
            }];
        for (let childSubsystem of subsystems) {
            let childSubsystemItems = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: childSubsystem.subsystem,
                    updatedAt: (0, typeorm_1.MoreThanOrEqual)(new Date(new Date().valueOf() - ((1000 * 60 * 60) / 2)))
                }
            });
            if (childSubsystemItems !== null) {
                let path = this.hwType + '/dbs' + (this.itemId !== null ? '?ids=' + this.itemId : '');
                menuItems.push(new menuResponse_dto_1.MenuFolder("Instances", path, this.hwType, childSubsystem.subsystem.toLowerCase(), this.itemId, childSubsystem.folder));
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.SqlserverSqlsfolderFolder = SqlserverSqlsfolderFolder;
class SqlserverChildSubsystemFolder extends SqlserverSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let items = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: this.subsystem,
                updatedAt: (0, typeorm_1.MoreThanOrEqual)(new Date(new Date().valueOf() - ((1000 * 60 * 60) / 2)))
            }
        });
        if (this.itemId) {
            let parentChildren = await this.itemRelationsRepository.find({ where: { parent: this.itemId.toString() } });
            let childrenIds = parentChildren.map((relation) => relation.child);
            for (let item of items) {
                if (childrenIds.includes(item.item_id)) {
                    let path = this.hwType + '/' + this.subsystem + '?item_id=' + item.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, this.subsystem, item.item_id));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.SqlserverChildSubsystemFolder = SqlserverChildSubsystemFolder;
class SqlserverDbsFolder extends SqlserverSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
    }
    async getChildren() {
        let menuItems = [];
        let items = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: this.subsystem,
                updatedAt: (0, typeorm_1.MoreThanOrEqual)(new Date(new Date().valueOf() - ((1000 * 60 * 60) / 2)))
            }
        });
        if (this.itemId) {
            let parentChildren = await this.itemRelationsRepository.find({ where: { parent: this.itemId.toString() } });
            let childrenIds = parentChildren.map((relation) => relation.child);
            items.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (let item of items) {
                if (childrenIds.includes(item.item_id)) {
                    let path = this.hwType + '/' + this.subsystem + '?ids=' + item.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, this.hwType, this.subsystem, item.item_id, SqlserverDbFolder));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.SqlserverDbsFolder = SqlserverDbsFolder;
class SqlserverDbFolder extends SqlserverSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Capacity', this.hwType + '/capacity?item_id=' + this.itemId, 'capacity'));
        menuItems.push(new menuResponse_dto_1.MenuLink('IO', this.hwType + '/io?item_id=' + this.itemId, 'io'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Data', this.hwType + '/Data?item_id=' + this.itemId, 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Latency', this.hwType + '/latency?item_id=' + this.itemId, 'latency'));
        menuItems.push(new menuResponse_dto_1.MenuLink('SQL query', this.hwType + '/sql-query?item_id=' + this.itemId, 'sql-query'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Ratio', this.hwType + '/ratio?item_id=' + this.itemId, 'ratio'));
        this.children = menuItems;
        return menuItems;
    }
}
exports.SqlserverDbFolder = SqlserverDbFolder;
class SqlserverPage extends AbstractMenu_1.AbstractPage {
    constructor() {
        super(...arguments);
        this.hwType = 'sqlserver';
        this.rootFolder = SqlserverRootFolder;
    }
}
exports.SqlserverPage = SqlserverPage;
class SqlserverSingleItemPage extends SqlserverPage {
    constructor() {
        super(...arguments);
        this.parentSubsystem = 'sqlsfolder';
        this.withItemId = true;
    }
}
exports.SqlserverSingleItemPage = SqlserverSingleItemPage;
class SqlserverOverviewPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'Overview';
    }
    async getPage() {
        let title = 'Overview';
        let tabs = [];
        let uuids = await getInstances(this.itemId);
        if (uuids === undefined) {
            uuids[0] = this.itemId;
        }
        uuids.sort();
        tabs.push({
            title: 'Overview',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                    graphs: [
                        {
                            metrics: [
                                await new SqlserverMetrics.BasicDbMetric('io_total', pageResponse_constants_1.CHART_TYPE.line, this.subsystem, false, 0).init()
                            ],
                            uuids: uuids,
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                            aggregated: true,
                        },
                        {
                            type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                            metrics: [
                                await new SqlserverMetrics.BasicDbMetric('data_total', pageResponse_constants_1.CHART_TYPE.line, this.subsystem, false, 0).init()
                            ],
                            uuids: uuids,
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                            aggregated: true,
                        },
                        {
                            type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                            metrics: [
                                await new SqlserverMetrics.BasicDbMetric('used', pageResponse_constants_1.CHART_TYPE.line, this.subsystem, false, 0).init(),
                            ],
                            uuids: uuids,
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                            aggregated: true,
                        },
                        {
                            type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                            metrics: [
                                await new SqlserverMetrics.BasicDbMetric('Transactionssec', pageResponse_constants_1.CHART_TYPE.line, this.subsystem, false, 0).init(),
                            ],
                            uuids: uuids,
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                            aggregated: true,
                        },
                        {
                            type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                            metrics: [await new SqlserverMetrics.BasicDbMetric('NumberofDeadlockssec', pageResponse_constants_1.CHART_TYPE.line, this.subsystem, false, 0).init()],
                            uuids: uuids,
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                            aggregated: true,
                        }
                    ],
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SqlserverOverviewPage = SqlserverOverviewPage;
class SqlserverConfigurationPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.title = 'Configuration';
    }
    async getPage() {
        let title = 'Configuration';
        const itemsPropertiesRepository = (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty);
        let tabs = [];
        let tableType = 'main';
        let id = this.itemId;
        const dict = {
            'host_platform': 'Platform',
            'host_release': 'Release',
            'host_distribution': 'Distribution',
            'physical_memory': 'Physical memory\n[GiB]',
            'virtual_memory': 'Virtual memory\n[GiB]',
            'max_workers_count': 'Max workers',
            'softnuma_configuration': 'Softnuma configuration',
            'socket_count': 'Sockets',
            'cpu_count': 'CPUs',
            'numa_node_count': 'Numa node count',
            'cores_per_socket': 'Cores per socket',
            'hyperthread_ratio': 'Hyperthread ratio',
            'max degree of parallelism': 'Max degree of parallelism',
            'remote access': 'Remote access',
            'remote data archive': 'Remote data archive',
        };
        let serverTable = await gen_table('main', id, dict);
        tabs.push({
            title: 'Configuration',
            content: [serverTable.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SqlserverConfigurationPage = SqlserverConfigurationPage;
class SqlserverMemoryPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'sqlsfolder';
        this.title = '';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, "sqlsfolder", [this.itemId]);
        tabFactory.addTab('Memory', [
            { metric: 'FreeMemory', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'TotalServerMemory', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'TargetServerMemory', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'ReservedServerMemory', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverMemoryPage = SqlserverMemoryPage;
class SqlserverBuffersPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'sqlsfolder';
        this.title = '';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, "sqlsfolder", [this.itemId]);
        tabFactory.addTab('Buffer', [
            { metric: 'Backgroundwriterpagessec', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Checkpointpagessec', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Lazywritessec', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Readahead', [
            { metric: 'Readaheadpagessec', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Readaheadtimesec', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Page RW', [
            { metric: 'Pagereadssec', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Pagewritessec', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Page', [
            { metric: 'Pagelifeexpectancy', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Databasepages', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Extensionallocatedpages', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Extensionfreepages', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Targetpages', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverBuffersPage = SqlserverBuffersPage;
class SqlserverSessionsPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'sqlsfolder';
        this.title = 'Sessions';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, "sqlsfolder", [this.itemId]);
        tabFactory.addTab('Sessions', [
            { metric: 'UserConnections', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'ConnectionResetsec', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Loginssec', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Logoutssec', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverSessionsPage = SqlserverSessionsPage;
class SqlserverLatchesPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'sqlsfolder';
        this.title = '';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, "sqlsfolder", [this.itemId]);
        tabFactory.addTab('IO', [
            { metric: 'PAGEIOLATCH_DT', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'PAGEIOLATCH_EX', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'PAGEIOLATCH_KP', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'PAGEIOLATCH_NL', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'PAGEIOLATCH_SH', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'PAGEIOLATCH_UP', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Buffer', [
            { metric: 'PAGELATCH_DT', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'PAGELATCH_EX', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'PAGELATCH_KP', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'PAGELATCH_NL', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'PAGELATCH_SH', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'PAGELATCH_UP', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Non-Buffer', [
            { metric: 'LATCH_DT', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LATCH_EX', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LATCH_KP', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LATCH_NL', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LATCH_SH', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LATCH_UP', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverLatchesPage = SqlserverLatchesPage;
class SqlserverWaiteventsPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'sqlsfolder';
        this.title = 'Wait events';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, "sqlsfolder", [this.itemId]);
        tabFactory.addTab('Events', [
            { metric: 'RS_SM_QUERY_CMPL', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'SOS_SCHEDULER_YIELD', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'ASYNC_NETWORK_IO', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'MSQL_XP', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'EXECSYNC', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'WRITE_COMPLETION', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'IO_COMPLETION', color: Color_1.Color.pallete.purpleDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'CXPACKET', color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
            { metric: 'WRITELOG', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'THREADPOOL', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Lock events', [
            { metric: 'LCK_M_BU', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_IS', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_IU', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_IX', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_RIn_NL', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_RIn_S', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_RIn_U', color: Color_1.Color.pallete.purpleDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_RIn_X', color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_RS_S', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_RS_U', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_RX_S', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_RX_U', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_RX_X', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_S', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_SCH_M', color: Color_1.Color.pallete.purpleDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_SCH_S', color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_SIU', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_SIX', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_U', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_UIX', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LCK_M_X', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverWaiteventsPage = SqlserverWaiteventsPage;
class SqlserverCapacityPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'Capacity';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, "db", [this.itemId]);
        tabFactory.addTab('Size', [
            { metric: 'unused', color: Color_1.Color.pallete.red, default: TabFactory_1.MPDefault.AREA_STACKED },
            { metric: 'used', color: Color_1.Color.pallete.green, default: TabFactory_1.MPDefault.AREA_STACKED },
            { metric: 'available', color: Color_1.Color.pallete.blue, default: TabFactory_1.MPDefault.AREA_STACKED },
            { metric: 'log_space', color: Color_1.Color.pallete.orange, default: TabFactory_1.MPDefault.AREA_STACKED },
        ]);
        let datafilesTable = {
            'SchemaName': 'Schema',
            'TableName': 'Table',
            'Type': 'Type',
            'TotalSpace': 'Total space\n[MiB]',
            'UsedSpace': 'Used space\n[MiB]',
            'UsedPercent': 'Used\n[%]',
            'rows': 'Rows'
        };
        let serverTable = await gen_table('datafiles2', this.itemId, datafilesTable);
        tabs.push({
            title: 'Relations',
            content: [serverTable.generateBox()],
        });
        let fileTable = {
            'File Group Name': 'Group name',
            'DB File Name': 'Name',
            'File Size': 'Size\n[GiB]',
            'File Path': 'Path'
        };
        serverTable = await gen_table('flgrps', this.itemId, fileTable);
        tabs.push({
            title: 'Filegroups',
            content: [serverTable.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverCapacityPage = SqlserverCapacityPage;
class SqlserverIoPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'IO';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, "db", [this.itemId]);
        tabFactory.addTab('Total', [
            { metric: 'io_total', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Read/Write', [
            { metric: 'io_rd', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'io_wr', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverIoPage = SqlserverIoPage;
class SqlserverDataPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'Data';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, "db", [this.itemId]);
        tabFactory.addTab('Total', [
            { metric: 'data_total', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Read/Write', [
            { metric: 'data_rd', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'data_wr', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverDataPage = SqlserverDataPage;
class SqlserverTopPage extends SqlserverPage {
    constructor() {
        super(...arguments);
        this.title = 'DB TOP';
    }
    async getPage() {
        const top = new Top_1.Top();
        top.setSubsystem('sqlserver', 'db');
        top.setParentSubsystem('sqlsfolder');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.SqlserverTopPage = SqlserverTopPage;
class SqlserverHealthstatusPage extends SqlserverPage {
    constructor() {
        super(...arguments);
        this.title = 'Health Status';
    }
    async getPage() {
        let title = 'Health Status';
        let tabs = [];
        let healthStatus = await new HealthStatus_1.HealthStatus().setAcl(this.request).useDeviceAliasColumn(true).useLinkToItem(true).useClickableAlias(false).getStatusByParameters("db", "database", "Instance", "sqlserver");
        tabs.push({
            title: 'Health Status',
            content: [healthStatus],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SqlserverHealthstatusPage = SqlserverHealthstatusPage;
class SqlserverLatencyPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'Latency';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, "db", [this.itemId]);
        tabFactory.addTab('Total', [
            { metric: 'latency_total', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Read/Write', [
            { metric: 'latency_rd', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'latency_wr', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverLatencyPage = SqlserverLatencyPage;
class SqlserverSqlqueryPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'SQL query';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, "db", [this.itemId]);
        tabFactory.addTab('Transactions', [
            { metric: 'Transactions', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Transactionssec', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Trackedtransactionssec', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'WriteTransactionssec', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'ActiveTransactions', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Cursors', [
            { metric: 'Activecursors', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'CursorRequestssec', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'CachedCursorsCounts', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverSqlqueryPage = SqlserverSqlqueryPage;
class SqlserverLocksPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'sqlsfolder';
        this.title = 'Locks';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, this.subsystem, [this.itemId]);
        tabFactory.addTab('Locks', [
            { metric: 'LocksRequestssec', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LocksWaitssec', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LockTimeoutssec', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Deadlocks', [
            { metric: 'NumberofDeadlockssec', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverLocksPage = SqlserverLocksPage;
class SqlserverRatioPage extends SqlserverSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'Ratio';
    }
    async getPage() {
        let title = this.title;
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, "db", [this.itemId]);
        tabFactory.addTab('Cache hit', [
            { metric: 'CacheHitRatio', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Buffer cache hit', [
            { metric: 'Buffercachehitratio', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Log cache hit', [
            { metric: 'LogCacheHitRatio', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.SqlserverRatioPage = SqlserverRatioPage;
class SqlserverMetrics {
}
exports.SqlserverMetrics = SqlserverMetrics;
SqlserverMetrics.Metric = class extends pageResponse_dto_1.Metric {
    async init() {
        const metric = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, this.subsystem, this.metric);
        if (metric) {
            Object.assign(this, metric);
        }
        return this;
    }
};
SqlserverMetrics.BasicDbMetric = class extends SqlserverMetrics.Metric {
    constructor(metric, type, subsystem, stacked, order, color) {
        super();
        this.metric = metric;
        this.type = type;
        this.subsystem = subsystem;
        this.stacked = stacked;
        this.order = order;
        this.color = color;
    }
};
SqlserverMetrics.CheckpointMetric = class extends SqlserverMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'buffers_checkpoint';
        this.order = 0;
    }
};
async function gen_table(tableType, id, dict) {
    const itemsPropertiesRepository = (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty);
    let serverTable = new Table_1.Table();
    for (const item in dict) {
        let value = dict[item];
        let properties = {};
        if (value.includes("iB]")) {
            if (value.includes("[MiB]")) {
                value = value.replace('\n[MiB]', '');
                properties = {
                    prefix: "Mi",
                    unit: "bytes",
                    shortcut: "MiB",
                    number: true,
                };
            }
            else {
                value = value.replace('\n[GiB]', '');
                properties = {
                    prefix: "Gi",
                    unit: "bytes",
                    shortcut: "GiB",
                    number: true,
                };
            }
        }
        serverTable.addHeader({ property: value, value: value, ...properties });
    }
    let rawProperties = Object.keys(dict);
    let tableRes = await itemsPropertiesRepository.findOneBy({ item_id: id, property_name: tableType });
    if (tableRes) {
        let wholeStuff = JSON.parse(tableRes.property_value);
        for (const dbRow of wholeStuff) {
            let row = [];
            for (const rawHeader of rawProperties) {
                let value = dbRow[rawHeader] ? dbRow[rawHeader] : "";
                row.push({
                    value: value,
                });
            }
            serverTable.addRow(row);
        }
        return serverTable;
    }
    else {
        let row = [];
        row.push({
            order: 1,
            value: "value",
        });
        return serverTable.addRow(row);
    }
}
async function getInstances(item_id) {
    let itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    let items = [];
    let itemRelations = await itemRelationsRepository.find({
        where: {
            parent: item_id,
            updatedAt: (0, typeorm_1.MoreThanOrEqual)(new Date(new Date().valueOf() - ((1000 * 60 * 60) / 2)))
        }
    });
    for (let relation of itemRelations) {
        items.push(relation.child);
    }
    return items;
}
//# sourceMappingURL=Sqlserver.js.map