import { MenuItem, Newable } from '@xormon/common/menuResponse.dto';
import { Page, Tab } from '@xormon/common/pageResponse.dto';
import { HostCfg } from '@xormon/entity/hostcfg.entity';
import { StorageGroup } from '@xormon/entity/storageGroup.entity';
import { IMetric } from '@xormon/service/metrics/Classes';
import { IMetric as IArchMetric } from "@xormon/common/architecture.dto";
import { Request } from 'express';
import { AbstractPage, StorAbstractFolder, StorAbstractPage } from './AbstractMenu';
export declare enum StoragePages {
    totals = "totals",
    capacity = "capacity",
    mapping = "mapping",
    heatmap = "heatmap",
    overview = "overview",
    configuration = "configuration",
    healthStatus = "healthStatus"
}
export declare const storageRegroupTotalMetric: {
    Data: string[];
    IO: string[];
    Latency: string[];
    'Block Size': string[];
};
export declare const storageRegroupMetrics: {
    Read: string[];
    Write: string[];
    Sum: string[];
};
export declare const storageRegroupPages: {
    data: string[];
    io: string[];
    resp_t: string[];
    block_size: string[];
};
export declare const getStorageCustomGroupMetrics: () => IMetric[];
export declare const storageGlobalPages: (parent: string) => MenuItem[];
export declare const sortStorageMetrics: (metrics: IArchMetric[]) => IArchMetric[];
export declare class StorFolder extends StorAbstractFolder {
}
export declare class StorSubsystemFolder extends StorFolder {
    itemIds: string[];
    constructor(hwType: string, subsystem?: string, itemId?: string[] | string);
}
export declare class StorRootFolder extends StorFolder {
    request?: Request;
    constructor(hw_type: string, request?: Request);
    getChildren(): Promise<MenuItem[]>;
}
export declare class StorDeviceFolder extends StorSubsystemFolder {
    subsystem: string;
    getChildren(): Promise<any[]>;
}
export declare class StorSubsystemItemsFolder extends StorSubsystemFolder {
    constructor(hwType: string, subsystem?: string, itemId?: string);
    getChildren(): Promise<any[]>;
}
export declare class StorPage extends StorAbstractPage {
    withItemId: boolean;
    rootFolder: Newable<StorAbstractFolder>;
    storageGroupsRepository: import("typeorm").Repository<StorageGroup>;
}
export declare class SimpleStorPage extends AbstractPage {
    withItemId: boolean;
    rootFolder: Newable<StorAbstractFolder>;
    storageGroupsRepository: import("typeorm").Repository<StorageGroup>;
    hostCfgRepository: import("typeorm").Repository<HostCfg>;
}
export declare class StorHeatmapPage extends SimpleStorPage {
    title: string;
    itemId: string;
    request: Request;
    getTab(tab: string): Promise<Tab>;
    getPage(): Promise<Page>;
}
export declare class StorInactivevolumesPage extends SimpleStorPage {
    title: string;
    itemId: string;
    getPage(): Promise<Page>;
}
export declare class StorConfigurationPage extends SimpleStorPage {
    title: string;
    itemId: string;
    getPage(): Promise<Page>;
}
export declare class StorVolumestopPage extends SimpleStorPage {
    title: string;
    getPage(): Promise<Page>;
}
export declare class StorHoststopPage extends SimpleStorPage {
    title: string;
    getPage(): Promise<Page>;
}
export declare class StorMappingPage extends SimpleStorPage {
    title: string;
    itemId: string;
    getPage(): Promise<Page>;
}
export declare class StorSantopologyPage extends SimpleStorPage {
    title: string;
    getPage(): Promise<Page>;
}
export declare class StorHealthstatusPage extends SimpleStorPage {
    title: string;
    getPage(): Promise<Page>;
}
export declare class StorOverviewPage extends SimpleStorPage {
    title: string;
    itemId: string;
    getPage(): Promise<Page>;
}
export declare class StorTotalPage extends SimpleStorPage {
    title: string;
    itemId: string;
    getPage(): Promise<Page>;
}
export declare class StorDevicetotalPage extends SimpleStorPage {
    title: string;
    itemId: string;
    getPage(): Promise<Page>;
}
export declare class StorDevicePage extends StorDevicetotalPage {
}
export declare class StorCapacityPage extends SimpleStorPage {
    title: string;
    getPage(): Promise<Page>;
}
export declare abstract class StorSingleItemPage extends StorPage {
    subsystem: string;
    itemId: string;
    parentSubsystem: string;
    parentId: string;
    constructor(hwType: string, itemId: string);
}
export declare class StorItemPage extends StorSingleItemPage {
    pageType: string;
    constructor(hwType: string, pageType: string, itemId: string);
    getPage(): Promise<Page>;
}
export declare class StorPortTypePage extends StorPage {
    title: string;
    itemId: string;
    metricType: string;
    constructor(itemId: string, metricType: string, hwType: string);
    getPage(): Promise<any[] | Page>;
}
