"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Table = exports.Properties = exports.HeaderDefinition = exports.PropertiesTableDefinition = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const dataSource_1 = require("../../../config/dataSource");
const itemProperty_entity_1 = require("../../../entity/itemProperty.entity");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const Item_1 = require("../../configuration/Item");
const Metric_1 = require("../../metrics/Metric");
const typeorm_1 = require("typeorm");
const URL_1 = require("./URL");
class PropertiesTableDefinition {
}
exports.PropertiesTableDefinition = PropertiesTableDefinition;
class HeaderDefinition {
}
exports.HeaderDefinition = HeaderDefinition;
class Properties {
}
exports.Properties = Properties;
class Table {
    constructor() {
        this.header = [];
        this.rows = [];
        this.properties = [];
        this.exportProperties = [];
        this.exportIds = [];
        this.exportBox = false;
        this.itemProperties = (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty);
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.lastIndex = 0;
        this.fixedCols = 0;
        this.orientation = 'horizontal';
    }
    setFixedCols(fixedCols) {
        this.fixedCols = fixedCols;
    }
    setOrientation(orientation) {
        this.orientation = orientation;
    }
    setBoxExport(fileName) {
        this.exportBox = true;
        this.exportLabel = fileName;
    }
    addHeader(header) {
        this.header.push({
            order: this.lastIndex,
            value: header.value,
            number: header.number ? header.number : false,
            unit: header.unit ? header.unit : undefined,
            shortcut: header.unit && subsystemMetricsResponse_constants_1.SHORTCUT[header.unit] ? subsystemMetricsResponse_constants_1.SHORTCUT[header.unit] : undefined,
            styleClass: header.styleClass ? header.styleClass : undefined,
            decimal: header.decimal !== undefined ? header.decimal : undefined,
            prefix: header.prefix ? header.prefix : undefined,
            multiplier: header.multiplier ? header.multiplier : undefined,
            sort: header.sort ? header.sort : undefined,
            trim: header.number === true
                ? false
                : header.trim !== undefined
                    ? header.trim
                    : true,
            newLines: header.newLines,
            linuxMpath: header.linuxMpath,
            trimLength: header.trimLength,
            disable: header.disable
        });
        if (header.property) {
            this.properties.push({
                headerIndex: this.lastIndex,
                property: header.property,
                url: header.url ? header.url : undefined,
            });
        }
        this.lastIndex++;
        return this;
    }
    addExport(itemIds, properties, exportLabel) {
        this.exportLabel = exportLabel
            ? exportLabel
            : this.header[0]
                ? 'export-table-' + this.header[0].value
                : 'export-table';
        this.exportProperties = properties;
        this.exportIds = itemIds;
    }
    sortByIndex(i, rev = false, isString = false) {
        if (this.header.length < i + 1) {
            console.warn("The table is sorted by the non-existent column index");
            return;
        }
        this.rows.sort((a, b) => {
            const reverseOrderMultiplier = rev ? -1 : 1;
            if (isString) {
                return a[i].value[0].localeCompare(b[i].value[0], undefined, { numeric: true, sensitivity: 'base' }) * reverseOrderMultiplier;
            }
            else {
                return (a[i].value[0] === null || Number.isNaN(a[i].value[0])) ? 1 :
                    (b[i].value[0] === null || Number.isNaN(b[i].value[0])) ? -1 :
                        (a[i].value[0] < b[i].value[0]) ? 1 * reverseOrderMultiplier :
                            (a[i].value[0] > b[i].value[0]) ? -1 * reverseOrderMultiplier :
                                0;
            }
        });
    }
    sortByValues(i, values, rev = false) {
        let mapping = {};
        let idx = 0;
        for (const value of values) {
            mapping[value] = idx;
            idx++;
        }
        for (const row of this.rows) {
            if (mapping[row[i].value?.[0]] === undefined) {
                mapping[row[i].value?.[0]] = idx;
                idx++;
            }
        }
        if (rev) {
            this.rows.sort((a, b) => (mapping[a[i].value[0]] > mapping[b[i].value[0]]) ? 1 : ((mapping[b[i].value[0]] > mapping[a[i].value[0]]) ? -1 : 0));
        }
        else {
            this.rows.sort((a, b) => (mapping[a[i].value[0]] < mapping[b[i].value[0]]) ? 1 : ((mapping[b[i].value[0]] < mapping[a[i].value[0]]) ? -1 : 0));
        }
    }
    async propertiesTableSimple(item_id, properties) {
        const propertiesFromDB = await this.itemProperties.findBy({ item_id: item_id });
        let reformattedProperties = {};
        for (const propertyFromDB of propertiesFromDB) {
            reformattedProperties[propertyFromDB.property_name] = propertyFromDB.property_value;
        }
        let order = 0;
        let row = [];
        for (const property of properties) {
            row.push({
                order: order,
                value: reformattedProperties[property] ? reformattedProperties[property] : null,
            });
            order++;
        }
        this.rows.push(row);
    }
    async addPropertiesItems(itemIds) {
        let relations = {
            'properties': true
        };
        let where = {
            'item_id': (0, typeorm_1.In)(itemIds)
        };
        let parentSubsystem;
        if (this.properties.find(property => property.property.includes("{{parent:"))) {
            parentSubsystem = this.properties.find(property => property.property.includes("{{parent:")).property.split("{{parent:")[1].replace("}}", "");
            relations['parents'] = {
                '_parent': true
            };
            where['parents'] = {
                '_parent': {
                    'subsystem': parentSubsystem
                }
            };
        }
        const itemsWithProperties = await this.objectItemsRepository.find({
            relations: relations,
            where: where
        });
        const exportProperties = [];
        this.properties.forEach(property => {
            if (!property.property.includes("{{")) {
                exportProperties.push(property.property);
            }
        });
        this.exportProperties = exportProperties;
        this.exportIds = itemIds;
        this.exportLabel = itemsWithProperties[0]
            ? 'export' + itemsWithProperties[0].hw_type + "-" + itemsWithProperties[0].subsystem + "-configuration"
            : 'export-configuration';
        for (const itemWithProperty of itemsWithProperties.sort((a, b) => (a.label.localeCompare(b.label, 'en', { numeric: true }) < b.label.localeCompare(a.label, 'en', { numeric: true }) ? -1 : 1))) {
            let order = 0;
            let row = [];
            let reformattedProperties = {};
            for (const propertyFromDB of itemWithProperty.properties) {
                reformattedProperties[propertyFromDB.property_name] = propertyFromDB.property_value;
            }
            for (const property of this.properties) {
                if (property.property.includes("{{parent:") && parentSubsystem) {
                    const parentItem = itemWithProperty.parents.find(parent => parent._parent.subsystem === parentSubsystem);
                    row.push({
                        order: order,
                        value: [parentItem._parent.label],
                        url: [URL_1.URL.generate(parentItem._parent.hw_type, parentItem._parent.subsystem, parentItem._parent.item_id)],
                    });
                }
                else if (property.property.includes("{{this}}")) {
                    row.push({
                        order: order,
                        value: [itemWithProperty.label],
                        url: [URL_1.URL.generate(itemWithProperty.hw_type, itemWithProperty.subsystem, itemWithProperty.item_id)],
                    });
                }
                else {
                    const _default = this.header[order].number ? null : ' ';
                    row.push({
                        order: order,
                        value: reformattedProperties[property.property] ? [reformattedProperties[property.property]] : [_default],
                        url: undefined,
                    });
                }
                order++;
            }
            this.rows.push(row);
        }
    }
    async addPropertiesItem(item_id) {
        const propertiesFromDB = await this.itemProperties.findBy({ item_id: item_id });
        let reformattedProperties = {};
        for (const propertyFromDB of propertiesFromDB) {
            reformattedProperties[propertyFromDB.property_name] = propertyFromDB.property_value;
        }
        this.exportIds.push(item_id);
        if (this.properties.length >= 1 && this.exportProperties.length === 0) {
            this.exportProperties = this.properties.map(property => property.property);
            const item = await this.objectItemsRepository.findOneBy({ item_id: item_id });
            this.exportLabel = 'export-' + item.hw_type + "-" + item.subsystem + "-configuration";
        }
        let order = 0;
        let row = [];
        for (const property of this.properties) {
            let url;
            if (property.url) {
                if (property.url === '{{this}}') {
                    url = await new URL_1.URL().generateFromUUID(item_id);
                }
                else if (property.url === '{{uuid}}') {
                    url = await new URL_1.URL().generateFromUUID(reformattedProperties[property.property]);
                }
                else if (property.url.includes('{{parent')) {
                    let splitted = property.url.replace(/{{/g, '').replace(/}}/g, '').split(':', 3);
                    let parent = await new Item_1.Item(item_id).findParent(splitted[1], splitted[2]);
                    if (parent) {
                        url = URL_1.URL.generate(splitted[1], splitted[2], parent.parent_id);
                        reformattedProperties[property.property] = parent.parent_label;
                    }
                }
            }
            if (property.property.includes('{{metric:')) {
                let splitted = property.property.replace(/{{/g, '').replace(/}}/g, '').split(':', 2);
                const lastValue = await new Metric_1.Metric(splitted[1], [item_id]).getLastValue();
                row.push({
                    order: order,
                    value: [+lastValue[item_id]],
                    url: url ? [url] : undefined,
                });
                order++;
            }
            else {
                if (this.header[order].number) {
                    row.push({
                        order: order,
                        value: reformattedProperties[property.property] ? [+reformattedProperties[property.property]] : [null],
                        url: url ? [url] : undefined,
                    });
                }
                else {
                    row.push({
                        order: order,
                        value: reformattedProperties[property.property] ? [reformattedProperties[property.property]] : [null],
                        url: url ? [url] : undefined,
                    });
                }
                order++;
            }
        }
        this.rows.push(row);
    }
    addRow(data) {
        if (data.length === this.header.length) {
            this.rows.push(data.map(column => {
                const value = Array.isArray(column.value)
                    ? column.value
                    : [column.value];
                const url = Array.isArray(column.url)
                    ? column.url
                    : [column.url];
                return {
                    ...column,
                    value: value,
                    url: url
                };
            }));
        }
        return this;
    }
    addRows(data) {
        for (const row of data) {
            if (row.length === this.header.length) {
                this.rows.push(row.map(column => {
                    const value = Array.isArray(column.value)
                        ? column.value
                        : [column.value];
                    const url = Array.isArray(column.url)
                        ? column.url
                        : [column.url];
                    return {
                        ...column,
                        value: value,
                        url: url
                    };
                }));
            }
        }
        return this;
    }
    generate() {
        return {
            header: this.header,
            rows: this.rows,
            orientation: this.orientation
        };
    }
    generateBox() {
        let _export;
        if (this.exportBox) {
            let exportBody = {
                headers: this.header.map(h => `${h.value}${h.shortcut ? " [" + h.shortcut + "]" : ""}`),
                values: this.rows.map(r => r.map(c => c.value.toString()))
            };
            _export = {
                fileName: this.exportLabel ? this.exportLabel : 'unlabeled',
                method: 'POST',
                url: '/api/exporter/v1/values',
                body: exportBody
            };
        }
        else if (this.exportProperties.length !== 0 && this.exportIds.length !== 0) {
            _export = {
                fileName: this.exportLabel ? this.exportLabel : 'unlabeled',
                method: 'POST',
                url: '/api/exporter/v1/configuration',
                body: {
                    uuids: this.exportIds,
                    properties: this.exportProperties
                }
            };
        }
        return {
            type: pageResponse_constants_1.BOX_TYPE.TableBox,
            header: this.header,
            rows: this.rows,
            fixedCols: this.fixedCols !== 0 ? this.fixedCols : undefined,
            orientation: this.orientation,
            export: _export
        };
    }
    generateBoxWithManagement() {
        let _export;
        if (this.exportBox) {
            let exportBody = {
                headers: this.header.map(h => `${h.value}${h.shortcut ? " [" + h.shortcut + "]" : ""}`),
                values: this.rows.map(r => r.map(c => {
                    if (c.url && c.value.toString() === 'link') {
                        return c.url.toString();
                    }
                    else {
                        return c.value.toString();
                    }
                }))
            };
            _export = {
                fileName: this.exportLabel ? this.exportLabel : 'unlabeled',
                method: 'POST',
                url: '/api/exporter/v1/values',
                body: exportBody
            };
        }
        else if (this.exportProperties.length !== 0 && this.exportIds.length !== 0) {
            _export = {
                fileName: this.exportLabel ? this.exportLabel : 'unlabeled',
                method: 'POST',
                url: '/api/exporter/v1/configuration',
                body: {
                    uuids: this.exportIds,
                    properties: this.exportProperties
                }
            };
        }
        return {
            type: pageResponse_constants_1.BOX_TYPE.TableBox,
            header: this.header,
            rows: this.rows,
            fixedCols: this.fixedCols !== 0 ? this.fixedCols : undefined,
            orientation: this.orientation,
            export: _export
        };
    }
    generatePivotBox() {
        return {
            type: pageResponse_constants_1.BOX_TYPE.TablePivotBox,
            header: this.header,
            rows: this.rows,
            orientation: this.orientation
        };
    }
}
exports.Table = Table;
//# sourceMappingURL=Table.js.map