"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TimescaledbMetrics = exports.TimescaledbSelfmonitoringPage = exports.TimescaledbHwtypePage = exports.TimescaledbSingleItemPage = exports.TimescaledbPage = exports.TimescaledbRootFolder = exports.TimescaledbFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const dataSource_1 = require("../../../config/dataSource");
const itemProperty_entity_1 = require("../../../entity/itemProperty.entity");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const Color_1 = require("../../metrics/Color");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const typeorm_1 = require("typeorm");
const Table_1 = require("./Table");
class TimescaledbFolder {
    constructor() {
        this.hwType = 'timescaledb';
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
}
exports.TimescaledbFolder = TimescaledbFolder;
class TimescaledbRootFolder extends TimescaledbFolder {
    async getChildren() {
        let menuItems = [];
        let hwTypes = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: 'hw_type',
        });
        for (let hwType of hwTypes) {
            let path = this.hwType + '/hw-type?item_id=' + hwType.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(hwType.label, path, 'hw-type', hwType.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.TimescaledbRootFolder = TimescaledbRootFolder;
class TimescaledbPage {
    constructor() {
        this.hwType = 'timescaledb';
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
    async getPath() {
        let root = new TimescaledbRootFolder();
        let menuItems = await root.getChildren();
        return menuItems;
    }
}
exports.TimescaledbPage = TimescaledbPage;
class TimescaledbSingleItemPage extends TimescaledbPage {
    constructor(itemId) {
        super();
        this.itemId = itemId;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.TimescaledbSingleItemPage = TimescaledbSingleItemPage;
class TimescaledbHwtypePage extends TimescaledbSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'hw_type';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'HW TYPE ' + (item !== null ? item.label : this.itemId);
        const childrenIds = (await this.itemRelationsRepository.find({ where: { parent: this.itemId } })).map((item) => item.child);
        const subsystemsIds = (await this.objectItemsRepository.find({ where: { item_id: (0, typeorm_1.In)(childrenIds), subsystem: 'subsystem' } })).map((item) => item.item_id);
        let tabs = [];
        tabs.push({
            title: 'Size',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                    graphs: [
                        {
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.month,
                            metrics: [new TimescaledbMetrics.Metric('hw_type', 'size').getGraph(pageResponse_constants_1.CHART_TYPE.area, false, 0, Color_1.Color.pallete.red)],
                            uuids: [this.itemId],
                            aggregated: false,
                        },
                        {
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.year,
                            metrics: [new TimescaledbMetrics.Metric('hw_type', 'size').getGraph(pageResponse_constants_1.CHART_TYPE.area, false, 0, Color_1.Color.pallete.red)],
                            uuids: [this.itemId],
                            aggregated: false,
                        },
                    ],
                },
                {
                    type: pageResponse_constants_1.BOX_TYPE.SingleGraphBox,
                    interval: pageResponse_constants_1.GRAPH_INTERVAL.month,
                    metrics: [new TimescaledbMetrics.Metric('hw_type', 'size').getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)],
                    uuids: subsystemsIds,
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.TimescaledbHwtypePage = TimescaledbHwtypePage;
class TimescaledbSelfmonitoringPage extends TimescaledbSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
    }
    async getPage() {
        let title = 'DB';
        let uuid = this.itemId ? this.itemId : 'selfmonitdb';
        let tabs = [];
        tabs.push({
            title: 'DB Size',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new TimescaledbMetrics.Metric('db', 'size').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.red)],
                    uuids: [uuid],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Sessions',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        await new TimescaledbMetrics.Metric('db', 'numbackends').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        await new TimescaledbMetrics.Metric('db', 'active').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.greenDark),
                        await new TimescaledbMetrics.Metric('db', 'idle').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.blueDark),
                        await new TimescaledbMetrics.Metric('db', 'idleintransaction').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.brownDark),
                        await new TimescaledbMetrics.Metric('db', 'idleintransactionaborted').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                        await new TimescaledbMetrics.Metric('db', 'fastpathfunctioncall').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.yellowDark),
                        await new TimescaledbMetrics.Metric('db', 'disabled').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.orangeDark),
                    ],
                    uuids: [uuid],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Txn',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        await new TimescaledbMetrics.Metric('db', 'xactrollback').getGraph(pageResponse_constants_1.CHART_TYPE.line, true, 0, Color_1.Color.pallete.blueDark),
                        await new TimescaledbMetrics.Metric('db', 'xactcommit').getGraph(pageResponse_constants_1.CHART_TYPE.line, true, 1, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [uuid],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Locks',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        await new TimescaledbMetrics.Metric('db', 'deadlocks_max').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        await new TimescaledbMetrics.Metric('db', 'relation').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.greenDark),
                        await new TimescaledbMetrics.Metric('db', 'extend').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.blueDark),
                        await new TimescaledbMetrics.Metric('db', 'page').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.brownDark),
                        await new TimescaledbMetrics.Metric('db', 'tuple').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                        await new TimescaledbMetrics.Metric('db', 'object').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.yellowDark),
                        await new TimescaledbMetrics.Metric('db', 'userlock').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.orangeDark),
                    ],
                    uuids: [uuid],
                    aggregated: false,
                },
            ],
        });
        let serverTable = await gen_table('relations', 'selfmonitdb', {
            'Namespace': 'Namespace',
            'Name': 'Name',
            'Type': 'Type',
            'Size': 'Size[MiB]',
            'Pages': 'Pages',
            'Rows': 'Rows'
        });
        tabs.push({
            title: 'Relations',
            content: [serverTable.generateBox()],
        });
        let mainConf = await gen_table('main', 'selfmonitdb', {
            'max_connections': 'Max connections',
            'max_worker_processes': 'Max workers',
            'deadlock_timeout': 'Deadlock timeout',
            'shared_buffers': 'Shared buffers',
            'work_mem': 'Work memory ',
            'temp_buffers': 'Temp buffers',
            'maintenance_work_mem': 'Maintenance work mem',
            'autovacuum_work_mem': 'Autovacuum work mem',
            'wal_buffers': 'WAL buffers',
            'effective_cache_size': 'Effective cache size',
            'random_page_cost': 'Random page cost',
            'min_wal_size': 'Min WAL size',
            'max_wal_size': 'Max WAL size',
            'wal_sync_method': 'WAL sync method',
        });
        tabs.push({
            title: 'DB configuration',
            content: [mainConf.generatePivotBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        return menuItems;
    }
}
exports.TimescaledbSelfmonitoringPage = TimescaledbSelfmonitoringPage;
async function gen_table(tableType, id, dict) {
    const itemsPropertiesRepository = (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty);
    let serverTable = new Table_1.Table();
    for (const item in dict) {
        let properties = {};
        let value = dict[item];
        if (value.includes("[MiB]")) {
            value = value.replace('[MiB]', '');
            properties = {
                prefix: "Mi",
                unit: "bytes",
                shortcut: "MiB",
                number: true,
            };
        }
        serverTable.addHeader({ property: value, value: value, ...properties });
    }
    let rawProperties = Object.keys(dict);
    let tableRes = await itemsPropertiesRepository.findOneBy({ item_id: id, property_name: tableType });
    if (tableRes) {
        let wholeStuff = JSON.parse(tableRes.property_value);
        for (const dbRow of wholeStuff) {
            let row = [];
            for (const rawHeader of rawProperties) {
                let value = dbRow[rawHeader] ? dbRow[rawHeader] : "";
                row.push({
                    value: value,
                });
            }
            serverTable.addRow(row);
        }
        serverTable.sortByIndex(0, true);
        return serverTable;
    }
    else {
        let row = [];
        row.push({
            order: 1,
            value: "value",
        });
        return serverTable.addRow(row);
    }
}
class TimescaledbMetrics {
}
exports.TimescaledbMetrics = TimescaledbMetrics;
TimescaledbMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor(subsystem, metric) {
        super();
        this.metric = metric;
        const definition = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.TIMESCALEDB, subsystem, metric);
        if (definition) {
            this.unit = definition.unit;
            this.shortcut = definition.shortcut;
            this.label = definition.label;
            this.defaultPrecision = definition.defaultPrecision ? definition.defaultPrecision : undefined;
            this.defaultPrefix = definition.defaultPrefix ? definition.defaultPrefix : undefined;
        }
    }
    getGraph(type, stacked, order, color) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.color = color;
        return this;
    }
};
//# sourceMappingURL=Timescaledb.js.map