"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Top = void 0;
const exporter_constants_1 = require("../../../common/constants/exporter.constants");
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const dataSource_1 = require("../../../config/dataSource");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const top_entity_1 = require("../../../entity/top.entity");
const ItemRegexWrapper_1 = require("../../architecture/ItemRegexWrapper");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const typeorm_1 = require("typeorm");
const Table_1 = require("./Table");
const URL_1 = require("./URL");
const ACLCheck_1 = require("../../acl/ACLCheck");
const subsystem_entity_1 = require("../../../entity/subsystem.entity");
const Linux_1 = require("./Linux");
class Top {
    constructor() {
        this.items = [];
        this.metrics = undefined;
        this.interval = undefined;
        this.mapped = false;
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.topRepository = (0, dataSource_1.getDataSource)().getRepository(top_entity_1.Top);
    }
    async loadSubsystems() {
        if (this.subsystemKey) {
            this.subsystem = await (0, dataSource_1.getDataSource)().getRepository(subsystem_entity_1.Subsystem).findOneBy({
                hw_type: this.hwType,
                subsystem: this.subsystemKey
            });
        }
        if (this.parentSubsystemKey) {
            this.parentSubsystem = await (0, dataSource_1.getDataSource)().getRepository(subsystem_entity_1.Subsystem).findOneBy({
                hw_type: this.hwType,
                subsystem: this.parentSubsystemKey
            });
        }
    }
    setSubsystem(hwType, subsystemKey) {
        this.hwType = hwType;
        this.subsystemKey = subsystemKey;
    }
    setParentSubsystem(parentSubsystem) {
        this.parentSubsystemKey = parentSubsystem;
    }
    setMetrics(metrics) {
        this.metrics = metrics;
    }
    setInterval(interval) {
        this.interval = interval;
    }
    setItemIds(itemIds) {
        this.itemIds = itemIds;
    }
    setAcl(request) {
        this.request = request;
    }
    setMapped() {
        this.mapped = true;
    }
    async getItems() {
        if (this.items.length === 0) {
            await this.loadItems();
        }
        return this.items;
    }
    async loadItems() {
        if (this.mapped) {
            let mappedParents = {};
            const mappedItems = await (0, Linux_1.getLinuxMapping)();
            const mappedIds = mappedItems.map(m => m.item_id);
            const items = await this.objectItemsRepository.find({
                where: {
                    item_id: (0, typeorm_1.In)(mappedIds),
                    parents: {
                        _parent: {
                            subsystem: 'server'
                        }
                    }
                }, relations: {
                    parents: {
                        _parent: true
                    }
                }
            });
            for (const mappedItem of mappedItems) {
                const item = items.find(i => i.item_id === mappedItem.item_id);
                if (item) {
                    mappedParents[mappedItem.agent_id] = {
                        label: item.parents[0]._parent.label,
                        item_id: item.parents[0].parent,
                        source_id: mappedItem.item_id
                    };
                }
            }
            this.mappedParents = mappedParents;
        }
        if (this.itemIds !== undefined && this.items.length === 0) {
            this.items = this.parentSubsystem
                ? (await this.objectItemsRepository.find({
                    relations: {
                        parents: {
                            _parent: true
                        }
                    },
                    where: {
                        item_id: (0, typeorm_1.In)(this.itemIds),
                        parents: {
                            _parent: {
                                subsystem: this.parentSubsystemKey
                            }
                        }
                    }
                })).map(item => {
                    return {
                        label: item.label,
                        item_id: item.item_id,
                        parent_id: item.parents[0] ? item.parents[0]._parent.item_id : undefined,
                        parent_label: item.parents[0] ? item.parents[0]._parent.label : undefined
                    };
                })
                : await this.objectItemsRepository.find({
                    where: {
                        item_id: (0, typeorm_1.In)(this.itemIds)
                    }
                });
        }
        else if (this.hwType && this.subsystemKey) {
            this.items = this.parentSubsystem
                ? (await this.objectItemsRepository.find({
                    relations: {
                        parents: {
                            _parent: true
                        }
                    },
                    where: {
                        hw_type: this.hwType,
                        subsystem: this.subsystemKey,
                        parents: {
                            _parent: {
                                subsystem: this.parentSubsystemKey
                            }
                        }
                    }
                })).map(item => {
                    return {
                        label: item.label,
                        item_id: item.item_id,
                        parent_id: item.parents[0] ? item.parents[0]._parent.item_id : undefined,
                        parent_label: item.parents[0] ? item.parents[0]._parent.label : undefined
                    };
                })
                : await this.objectItemsRepository.find({
                    where: {
                        hw_type: this.hwType,
                        subsystem: this.subsystemKey
                    }
                });
            this.itemIds = this.items.map((item) => item.item_id);
        }
        if (this.request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, this.items.map(item => item.item_id));
            this.items = this.items.filter(item => ids.includes(item.item_id));
            this.itemIds = ids;
        }
    }
    isOld(now, top) {
        let old = false;
        if (top.interval === exporter_constants_1.TOP_INTERVAL.day && now - top.updatedAt.getTime() >= 24 * 60 * 60 * 1000) {
            old = true;
        }
        else if (top.interval === exporter_constants_1.TOP_INTERVAL.week && now - top.updatedAt.getTime() >= 7 * 24 * 60 * 60 * 1000) {
            old = true;
        }
        else if (top.interval === exporter_constants_1.TOP_INTERVAL.month && now - top.updatedAt.getTime() >= 31 * 24 * 60 * 60 * 1000) {
            old = true;
        }
        else if (top.interval === exporter_constants_1.TOP_INTERVAL.year && now - top.updatedAt.getTime() >= 365 * 24 * 60 * 60 * 1000) {
            old = true;
        }
        return old;
    }
    async getValues() {
        await this.loadItems();
        const now = new Date().getTime();
        let values = {};
        let cond = {
            where: {
                item_id: (0, typeorm_1.In)(this.itemIds),
                updatedAt: (0, typeorm_1.MoreThan)(new Date(Date.now() - (48 * 60 * 60 * 1000)))
            }
        };
        if (this.metrics) {
            cond.where['metric'] = (0, typeorm_1.In)(this.metrics);
        }
        if (this.interval) {
            cond.where['interval'] = this.interval;
        }
        const tops = await this.topRepository.find(cond);
        for (const top of tops) {
            const old = this.isOld(now, top);
            if (!old && !this.lastUpdateAt) {
                this.lastUpdateAt = top.updatedAt;
            }
            if (values[top.interval]) {
                if (values[top.interval][top.metric]) {
                    values[top.interval][top.metric][top.item_id] = {
                        avg: !old ? top.avg : null,
                        max: !old ? top.max : null,
                    };
                }
                else {
                    values[top.interval][top.metric] = {
                        [top.item_id]: {
                            avg: !old ? top.avg : null,
                            max: !old ? top.max : null,
                        },
                    };
                }
            }
            else {
                values[top.interval] = {
                    [top.metric]: {
                        [top.item_id]: {
                            avg: !old ? top.avg : null,
                            max: !old ? top.max : null,
                        },
                    },
                };
            }
        }
        return values;
    }
    async genTabs() {
        let tabs = [];
        await this.loadSubsystems();
        let values = await this.getValues();
        let linuxTabs;
        if (this.hwType == hwTypeKey_constants_1.HwTypeKey.POWER) {
            const ids = (await (0, Linux_1.getLinuxMapping)()).filter(i => this.itemIds.includes(i.item_id)).map(i => i.agent_id);
            let servers = await this.objectItemsRepository.find({
                where: {
                    hw_type: 'linux',
                    subsystem: 'server',
                    item_id: (0, typeorm_1.In)(ids)
                }
            });
            const top = new Top();
            top.setMapped();
            top.setItemIds(servers.map(i => i.item_id));
            top.setSubsystem('linux', 'server');
            top.setAcl(this.request);
            linuxTabs = await top.genTabs();
        }
        const _class = await MetricsWrapper_1.MetricsWrapper.findClassKey(this.hwType);
        const subsystem = MetricsWrapper_1.MetricsWrapper.getSubsystemDefinition(_class, this.hwType, this.subsystemKey);
        for (const interval of Top.intervals) {
            let tab = {
                title: interval.label,
                content: [],
                subTabs: []
            };
            for (const group in subsystem.top) {
                const tables = [];
                let subTab = {
                    title: group,
                    content: []
                };
                const topMetrics = subsystem.top[group];
                for (const metric of topMetrics) {
                    if (values[interval.key] && values[interval.key][metric]) {
                        const metricDef = await MetricsWrapper_1.MetricsWrapper.getMetricFindClass(this.hwType, this.subsystemKey, metric);
                        const table = this.genTable(values[interval.key][metric], metricDef);
                        tables.push({
                            label: metricDef ? `${metricDef.label} [${metricDef.defaultPrefix ? metricDef.defaultPrefix : ''}${metricDef.shortcut}]` : metric,
                            export: {
                                url: '/api/exporter/v1/top',
                                method: pageResponse_constants_1.EXPORT_METHOD.POST,
                                fileName: "export-top-" + this.hwType + "-" + this.subsystemKey + "-" + metric,
                                body: {
                                    uuids: this.itemIds,
                                    metric: metric,
                                    interval: interval.key,
                                },
                            },
                            ...table,
                        });
                    }
                }
                const topBox = {
                    type: pageResponse_constants_1.BOX_TYPE.TopBox,
                    tables: tables,
                };
                subTab.content = [topBox, {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        text: "Note that this page is refreshed once a day after midnight." + (this.lastUpdateAt ? " Last refresh on " + this.lastUpdateAt.toDateString() : "Failed to find last update, last refresh happened more than 48 hours ago")
                    },
                ];
                tab.subTabs.push(subTab);
            }
            if (linuxTabs) {
                const linuxTab = linuxTabs.find(tab => tab.title === interval.label);
                if (linuxTab && linuxTab.subTabs) {
                    tab.subTabs.push(...linuxTab.subTabs.filter(subTab => subTab.title !== 'CPU'));
                }
            }
            if (tab.subTabs.length === 1) {
                tab.content = tab.subTabs[0].content;
                tab.subTabs = undefined;
            }
            tabs.push(tab);
        }
        return tabs;
    }
    genTable(metrics, metricDef) {
        let table = new Table_1.Table()
            .addHeader({ value: this.mapped ? 'LPAR' : this.subsystem ? this.subsystem.label : this.subsystemKey });
        if (this.parentSubsystem || this.parentSubsystemKey) {
            table.addHeader({ value: this.parentSubsystem ? this.parentSubsystem.label : this.parentSubsystemKey });
        }
        else if (this.mapped) {
            table.addHeader({ value: "Server" });
        }
        table.addHeader({
            value: 'Avg',
            unit: metricDef ? metricDef.unit : undefined,
            number: true,
            prefix: metricDef ? metricDef.defaultPrefix : undefined,
            decimal: metricDef ? metricDef.defaultPrecision : undefined,
            sort: 'desc'
        })
            .addHeader({
            value: 'Max',
            unit: metricDef ? metricDef.unit : undefined,
            prefix: metricDef ? metricDef.defaultPrefix : undefined,
            decimal: metricDef ? metricDef.defaultPrecision : undefined,
            number: true,
        });
        for (const item of this.items) {
            if (metrics[item.item_id] && metrics[item.item_id]['avg'] !== undefined && metrics[item.item_id]['avg'] !== null) {
                if (this.parentSubsystem || this.mapped) {
                    const url = this.mapped && this.mappedParents?.[item.item_id]
                        ? URL_1.URL.generate(hwTypeKey_constants_1.HwTypeKey.POWER, 'vm', this.mappedParents[item.item_id].source_id)
                        : URL_1.URL.generate(this.hwType, this.subsystemKey, item.item_id);
                    table.addRow([
                        { order: 2, value: item.label, url: url },
                        { order: 3, value: this.mappedParents?.[item.item_id] ? this.mappedParents[item.item_id].label : item.parent_label },
                        { order: 0, value: metrics[item.item_id] ? metrics[item.item_id]['avg'] : null },
                        { order: 1, value: metrics[item.item_id] ? metrics[item.item_id]['max'] : null },
                    ]);
                }
                else {
                    table.addRow([
                        { order: 2, value: item.label, url: item ? URL_1.URL.generate(this.hwType, this.subsystemKey, item.item_id) : undefined },
                        { order: 0, value: metrics[item.item_id] ? metrics[item.item_id]['avg'] : null },
                        { order: 1, value: metrics[item.item_id] ? metrics[item.item_id]['max'] : null },
                    ]);
                }
            }
        }
        table.sortByIndex(this.parentSubsystem || this.parentSubsystemKey || this.mapped ? 2 : 1, true);
        return table.generate();
    }
    static async exportTopData(hwType, subsystem, metric, interval) {
        const top = new Top();
        top.setSubsystem(hwType, subsystem);
        top.setMetrics([metric]);
        top.setInterval(interval);
        const values = await top.getValues();
        const items = await top.getItems();
        if (!values[interval] || !values[interval][metric]) {
            return [];
        }
        let topExport = [];
        for (const item of items) {
            topExport.push({
                label: item.label,
                item_id: item.item_id,
                avg: values[interval][metric][item.item_id] ? values[interval][metric][item.item_id].avg : null,
                max: values[interval][metric][item.item_id] ? values[interval][metric][item.item_id].max : null,
            });
        }
        return topExport;
    }
    static async exportTopDataByItemIds(itemIds, metric, interval, orderBy) {
        const top = new Top();
        top.setItemIds(itemIds);
        top.setMetrics([metric]);
        top.setInterval(interval);
        const values = await top.getValues();
        const items = await top.getItems();
        if (!values[interval] || !values[interval][metric]) {
            return [];
        }
        let topExport = [];
        for (const item of items) {
            topExport.push({
                label: item.label,
                item_id: item.item_id,
                avg: values[interval][metric][item.item_id] ? values[interval][metric][item.item_id].avg : null,
                max: values[interval][metric][item.item_id] ? values[interval][metric][item.item_id].max : null,
            });
        }
        if (orderBy) {
            topExport.sort((a, b) => (a[orderBy] < b[orderBy]) ? 1 : ((b[orderBy] < a[orderBy]) ? -1 : 0));
        }
        return topExport;
    }
    static async exportTopDataByRegex(regex, hwType, subsystem, metric, interval, orderBy) {
        const items = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegex(hwType, subsystem, regex);
        return this.exportTopDataByItemIds(items.map(item => item.item_id), metric, interval, orderBy);
    }
}
exports.Top = Top;
Top.intervals = [
    {
        key: 'day',
        label: 'Last day',
        interval: 1440,
        bucket: '5m'
    },
    {
        key: 'week',
        label: 'Last week',
        interval: 10080,
        bucket: '1h'
    },
    {
        key: 'month',
        label: 'Last month',
        interval: 43200,
        bucket: '6h'
    },
    {
        key: 'year',
        label: 'Last year',
        interval: 525600,
        bucket: '1d'
    },
];
//# sourceMappingURL=Top.js.map