"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.URL = void 0;
const dataSource_1 = require("../../../config/dataSource");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
class URL {
    constructor() {
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
    }
    static checkTransform(hwType, pageType, metric) {
        if (pageType === 'device') {
            if (['sanbrcd', 'sancisco', 'sannav'].includes(hwType)) {
                if (metric && this.metricTransformed['san']['device'][metric]) {
                    return this.metricTransformed['san']['device'][metric];
                }
                return this.transformed['san'][pageType];
            }
            else if (['lanothers', 'lancisco', "lan"].includes(hwType)) {
                return this.transformed['lan'][pageType];
            }
            else {
                return this.transformed['storage'][pageType];
            }
        }
        else if (['lanothers', 'lancisco', "lan"].includes(hwType)) {
            return this.transformed['lan'][pageType] ? this.transformed['lan'][pageType] : pageType;
        }
        else if (this.transformed[hwType] && this.transformed[hwType][pageType]) {
            return this.transformed[hwType][pageType];
        }
        else {
            return pageType;
        }
    }
    static generate(hw_type, pageType, item_id, metric, tab) {
        if (this.withoutUrl[hw_type] && this.withoutUrl[hw_type][pageType]) {
            return null;
        }
        if (item_id) {
            pageType = this.checkTransform(hw_type, pageType, metric);
            return `/api/menu/v1/page/${hw_type}/${pageType}?item_id=${item_id}${tab ? `#tab=${tab}` : ``}`;
        }
        else {
            return `/api/menu/v1/page/${hw_type}/${pageType}${tab ? `#tab=${tab}` : ``}`;
        }
    }
    async generateFromUUID(item_id) {
        const item = await this.objectItemsRepository.findOneBy({ item_id: item_id });
        return item ? '/api/menu/v1/page/' + item.hw_type + '/' + item.subsystem + '?item_id=' + item_id : null;
    }
    generateFromObject(item, metric, path, user, children) {
        if (item) {
            if (URL.withoutUrl[item.hw_type] && URL.withoutUrl[item.hw_type][item.subsystem]) {
                return null;
            }
            if (path) {
                path = path.split(":::").at(-1);
            }
            if (item.subsystem === "esxi_vm") {
                item.item_id = item.item_id.split("_vm_").at(-1);
            }
            let curItem = item;
            let itemId = item.item_id;
            let group = null;
            const pageType = URL.checkTransform(item.hw_type, item.subsystem, metric);
            const partURL = '/api/menu/v1/page/' + item.hw_type + '/' + pageType + '?item_id=';
            if (children && children.server_group_items && children.server_group_items.length) {
                item = children;
                if (path) {
                    group = children.server_group_items.find(i => i._server_group.user_id === user.user_id && i.server_group_id === path);
                }
            }
            else if (["cpu", "memory"].includes(item.subsystem) && item.parents && item.parents[0]) {
                curItem = item.parents[0]._parent;
                if (item.server_group_items && item.server_group_items.length) {
                    if (path) {
                        group = item.server_group_items.find(i => i._server_group.user_id === user.user_id && i.server_group_id === path);
                    }
                }
            }
            if (group) {
                itemId = curItem.item_id + ":::" + group.server_group_id;
            }
            else if (item.server_group_items && item.server_group_items.length) {
                for (const sgItem of item.server_group_items) {
                    if (sgItem._server_group.user_id === user.user_id) {
                        itemId = curItem.item_id + ":::" + (path ? path : sgItem.server_group_id);
                    }
                }
            }
            return partURL + itemId;
        }
        else {
            return null;
        }
    }
}
exports.URL = URL;
URL.transformed = {
    vmware: {
        esxi: 'esxi-cpu',
        vcenter: 'vcenter-totals',
        cluster: 'cluster-totals',
        respool: 'resource-pool',
        esxi_vm: "vm"
    },
    power: {
        cmc: 'cmcconsoleconfiguration'
    },
    storage: {
        device: 'device'
    },
    san: {
        device: 'sanport-data'
    },
    lan: {
        device: 'data',
        cpu: "cpumemory"
    },
    kubernetes: {
        cluster: 'cluster-node-totals'
    },
    nutanix: {
        cluster: 'cluster-host-totals'
    },
    proxmox: {
        cluster: 'cluster-node-totals'
    },
    sqlserver: {
        sqlsfolder: 'overview',
        db: 'capacity'
    },
    postgres: {
        dbfolders: 'overview',
        db: 'size'
    },
    oracledb: {
        instance: 'overview',
        menugroup: 'overview',
        submenugroup: 'overview',
        odbfolder: 'overview'
    },
    db2: {
        bufferpool: '',
        dbfolder: 'overview',
        member: 'io'
    },
    gcloud: {
        project: "compute-totals",
        region: "region-compute-totals"
    },
    azure: {
        mgroup: "vm-mgroup-totals",
        subscription: "vm-subscription-totals",
        rgroup: "vm-rgroup-totals"
    },
    oraclevm: {
        server_pool: "server-pool-totals"
    },
    ovirt: {
        cluster: "cluster-totals",
        host_nic: "host-nic",
        storage_domain: "storage-domain-totals"
    }
};
URL.metricTransformed = {
    san: {
        device: {
            'bb_credits': 'sanport-frames',
            'frames_in': 'sanport-frames',
            'frames_out': 'sanport-frames',
            'crc_errors': 'sanport-frames',
            'pcs_errors': 'sanport-frames',
            'enc_out': 'sanport-frames',
            'disc_c3': 'sanport-frames',
            'link_fail': 'sanport-frames',
            'loss_sync': 'sanport-frames',
            'cpu': 'sanport-cpumemory',
            'memory': 'sanport-cpumemory',
            'data_in': 'sanport-data',
            'data_out': 'sanport-data'
        }
    },
};
URL.withoutUrl = {
    application: {
        task: true
    },
    database: {
        hw_type: true,
        subsystem: true
    },
    linux: {
        job: true,
        san: true,
        lan: true,
        san_resp: true,
        disk_total: true
    },
    vmware: {
        datastore_vm: true,
        ds_folder: true
    },
    power: {
        physicalvolume: true,
        backingsea: true,
        virtualfibrenpiv: true,
        trunk: true,
        freelinkaggregation: true,
        ioadapter: true,
        ioslot: true,
        iobus: true,
        profileioslot: true
    },
    oraclevm: {
        server_interface: true,
        server_disk: true,
        vm_interface: true,
        vm_disk: true
    },
    ovirt: {
        datacenter: true,
        vm_nic: true
    },
    aws: {
        region: true
    },
    compellent: {
        port_type: true
    },
    sancisco: {
        fabric: true
    },
    sanbrcd: {
        fabric: true
    },
    windows: {
        domain: true
    },
    as400: {
        asp: true,
        job: true,
        shrpool: true,
        lan: true
    },
    lancisco: {
        vlancfg: true,
        stackcfg: true,
        cpumemory: true,
        fexcfg: true,
        vsscfg: true,
        cpu: true,
        memory: true
    },
    lanothers: {
        vlancfg: true,
        stackcfg: true,
        cpumemory: true,
        fexcfg: true,
        vsscfg: true,
        cpu: true,
        memory: true
    },
    lan: {
        vlancfg: true,
        stackcfg: true,
        cpumemory: true,
        fexcfg: true,
        vsscfg: true,
        cpu: true,
        memory: true
    }
};
URL.groupHwTypes = {
    "lancisco": true,
    "lanothers": true
};
//# sourceMappingURL=URL.js.map