"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetricsHelper = void 0;
const URL_1 = require("../menu/menu/URL");
const MetricsWrapper_1 = require("./MetricsWrapper");
const User_1 = require("../users/User");
class MetricsHelper {
    static async prepareTable(metricsItem, metadata, uuids, interval, objectItemsById, parentsById, metric, path, request) {
        let items = [];
        for (const item_id of uuids) {
            if (metricsItem[item_id]) {
                let label = objectItemsById[item_id] && objectItemsById[item_id].label ? objectItemsById[item_id].label : item_id;
                let sum, avg, max;
                const filtered = metricsItem[item_id].timeseries.filter((e) => { return e !== null; });
                sum = filtered.reduce((a, b) => +a + +b, 0);
                if (filtered.length > 0) {
                    avg = +Number(sum / filtered.length).toPrecision(5) || 0;
                    max = Number(Math.max(...filtered));
                }
                else {
                    avg = null;
                    max = null;
                }
                items.push({
                    metadata: {
                        uuid: item_id,
                        name: label,
                        parent: parentsById[item_id] ? parentsById[item_id] : undefined,
                        start: metadata[item_id] ? metadata[item_id]['start'] / 1000 : null,
                        end: metadata[item_id] ? metadata[item_id]['end'] / 1000 : null,
                        interval: interval,
                        avg: avg,
                        max: max,
                        url: objectItemsById[item_id] ? new URL_1.URL().generateFromObject(objectItemsById[item_id], metric, path, request ? await (User_1.User.getUserByRequest(request)) : null) : null,
                    },
                    timeseries: metricsItem[item_id].timeseries ? metricsItem[item_id].timeseries : [],
                });
            }
        }
        return items;
    }
    static prepareData(metrics) {
        let prepared = {};
        for (const itemId in metrics) {
            if (!prepared[itemId]) {
                prepared[itemId] = {};
            }
            for (const metric in metrics[itemId]) {
                const timeKeys = Object.keys(metrics[itemId][metric]);
                if (timeKeys.length > 0) {
                    let timeseries = [];
                    for (const timeKey in metrics[itemId][metric]) {
                        timeseries.push(+Number(metrics[itemId][metric][timeKey]).toPrecision(5));
                    }
                    prepared[itemId][metric] = {
                        start: new Date(timeKeys[0]).getTime() / 1000,
                        end: new Date(timeKeys[timeKeys.length - 1]).getTime() / 1000,
                        interval: (new Date(timeKeys[timeKeys.length - 1]).getTime() - new Date(timeKeys[timeKeys.length - 2]).getTime()) / 1000 / 60,
                        timeseries: timeseries
                    };
                }
                else {
                    prepared[itemId][metric] = {
                        start: null,
                        end: null,
                        interval: null,
                        timeseries: []
                    };
                }
            }
        }
        return prepared;
    }
    static calculateWeightedAverage(data) {
        let timeseries = [];
        for (let timeIndex = 0; timeIndex < data.length; timeIndex++) {
            let valueSum = 0;
            let weightSum = 0;
            for (const itemIndex in data[timeIndex]) {
                valueSum += (data[timeIndex][itemIndex].value * data[timeIndex][itemIndex].weight);
                weightSum += data[timeIndex][itemIndex].weight;
            }
            if (weightSum !== 0) {
                timeseries.push(valueSum / weightSum);
            }
            else {
                timeseries.push(null);
            }
        }
        return timeseries;
    }
    static async handleWeightedAverage(metricsItem, metric, start, end) {
        let data = [];
        let ioMetric = '';
        if (metric === 'resp_t_w') {
            ioMetric = 'write_io';
        }
        else if (metric === 'resp_t_r') {
            ioMetric = 'read_io';
        }
        else if (metric === 'resp_t') {
            ioMetric = 'io_rate';
        }
        else {
            return [];
        }
        const ioData = await (new MetricsWrapper_1.MetricsWrapper([ioMetric], Object.keys(metricsItem))).getDataFromTable(start, end);
        const preparedData = this.prepareData(ioData);
        let realStart = null;
        let realEnd = null;
        let interval = null;
        for (const itemId in metricsItem) {
            if (!realStart && metricsItem[itemId]['timeseries'].length > 0 && preparedData[itemId][ioMetric]['timeseries'].length > 0) {
                realStart = preparedData[itemId][ioMetric].start;
                realEnd = preparedData[itemId][ioMetric].end;
                interval = preparedData[itemId][ioMetric].interval;
            }
            for (const index in metricsItem[itemId]['timeseries']) {
                if (preparedData[itemId][ioMetric]['timeseries'][index]) {
                    if (!data[index]) {
                        data[index] = [];
                    }
                    data[index].push({
                        value: metricsItem[itemId]['timeseries'][index],
                        weight: preparedData[itemId][ioMetric]['timeseries'][index]
                    });
                }
            }
        }
        const calculated = this.calculateWeightedAverage(data);
        const filtered = calculated.filter((e) => { return e !== null; });
        const sum = calculated.reduce((a, b) => a + b, 0);
        return [{
                metadata: {
                    uuid: null,
                    name: metric,
                    start: realStart,
                    end: realEnd,
                    interval: interval,
                    avg: +Number(sum / filtered.length).toPrecision(5) || 0,
                    max: Math.max(...filtered),
                    url: null,
                },
                timeseries: calculated,
            }];
    }
}
exports.MetricsHelper = MetricsHelper;
//# sourceMappingURL=MetricsHelper.js.map