"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cloud = void 0;
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const Definitions_1 = require("../Definitions");
class Cloud {
}
exports.Cloud = Cloud;
Cloud.label = "Cloud";
Cloud.order = 6;
Cloud.technology = {
    azure: {
        label: 'Microsoft Azure',
        order: 2,
        subsystems: {
            mgroup: {
                label: "Managment Group",
                excludedTools: [
                    'all'
                ],
            },
            subscription: {
                label: "Subscription",
                excludedTools: [
                    'all'
                ],
            },
            rgroup: {
                label: "Resource Group",
                excludedTools: [
                    'all'
                ],
            },
            vm: {
                label: "Azure Virtual Machine",
                aliases: true,
                top: {
                    'CPU': ['cpu_util']
                },
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_util',
                        percent: 'cpu_util'
                    }],
                configuration: {
                    'ip_adress': {
                        label: 'IP Adress'
                    },
                    'location': {
                        label: 'Location'
                    },
                    'instance_size': {
                        label: 'Instance size'
                    },
                    'disk_size': {
                        label: 'Disk Size [GB]'
                    },
                    'os': {
                        label: 'Operation System',
                    },
                    "state": {
                        label: "Operational Status"
                    }
                },
                metrics: {
                    'cpu_util': {
                        metric: 'cpu_util',
                        longLabel: 'CPU %',
                        label: 'CPU %',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    'read_bytes': {
                        metric: 'read_bytes',
                        label: 'Data Read',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                    'write_bytes': {
                        metric: 'write_bytes',
                        label: 'Data Write',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                    'write_ops': {
                        metric: 'write_ops',
                        label: 'IO Write',
                        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
                    },
                    'read_ops': {
                        metric: 'read_ops',
                        label: 'IO Read',
                        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
                    },
                    'received_bytes': {
                        metric: 'received_bytes',
                        label: 'Network In',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                    'sent_bytes': {
                        metric: 'sent_bytes',
                        label: 'Network Out',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                    freeMemory: {
                        ...Definitions_1.metricTypes.memory,
                        metric: "freeMemory",
                        label: 'Free memory',
                    },
                    usedMemory: {
                        ...Definitions_1.metricTypes.memory,
                        metric: "usedMemory",
                        label: 'Used memory',
                    },
                }
            },
            account: {
                label: "Azure Storage Account",
                aliases: true,
                configuration: {
                    'publicNetworkAccess': {
                        label: 'Public Network Access'
                    },
                    'location': {
                        label: 'Location'
                    },
                    'minimumTlsVersion': {
                        label: 'Minimum Tls Version'
                    },
                    'provisioningState': {
                        label: 'Provisioning State'
                    },
                    'performance': {
                        label: 'Performance Type',
                    },
                    "kind": {
                        label: "Kind"
                    }
                },
                metrics: {
                    'used_capacity': {
                        metric: 'used_capacity',
                        longLabel: 'Used Capacity',
                        label: 'Used Capacity',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'account_transactions': {
                        metric: 'account_transactions',
                        longLabel: 'Transactions',
                        label: 'Transactions',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'account_ingress': {
                        metric: 'account_ingress',
                        label: 'Ingress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'account_egress': {
                        metric: 'account_egress',
                        label: 'Egress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'account_suc_server_lat': {
                        metric: 'account_suc_server_lat',
                        label: 'Success Server Latency',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    'account_suc_e2e_lat': {
                        metric: 'account_suc_e2e_lat',
                        label: 'Success E2E Latency',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    'account_availibility': {
                        metric: 'account_availibility',
                        label: 'Availability %',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    'blob_capacity': {
                        metric: 'blob_capacity',
                        longLabel: 'Blob Capacity',
                        label: 'Blob Capacity',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'blob_count': {
                        metric: 'blob_count',
                        label: 'Blob Count',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'blob_container_count': {
                        metric: 'blob_container_count',
                        label: 'Blob Container Count',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'blob_transactions': {
                        metric: 'blob_transactions',
                        longLabel: 'Transactions',
                        label: 'Transactions',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'blob_ingress': {
                        metric: 'blob_ingress',
                        label: 'Ingress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'blob_egress': {
                        metric: 'blob_egress',
                        label: 'Egress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'blob_suc_server_lat': {
                        metric: 'blob_suc_server_lat',
                        label: 'Success Server Latency',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    'blob_suc_e2e_lat': {
                        metric: 'blob_suc_e2e_lat',
                        label: 'Success E2E Latency',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    'blob_availibility': {
                        metric: 'blob_availibility',
                        label: 'Availability %',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    'file_capacity': {
                        metric: 'file_capacity',
                        longLabel: 'File Capacity',
                        label: 'File Capacity',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'file_count': {
                        metric: 'file_count',
                        label: 'File Count',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'file_share_capacity_quota': {
                        metric: 'file_share_capacity_quota',
                        label: 'File Share Capacity quota',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'file_share_count': {
                        metric: 'file_share_count',
                        label: 'File Share Count',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'file_share_snapshot_count': {
                        metric: 'file_share_snapshot_count',
                        longLabel: 'File Share Snapshot Count',
                        label: 'File Share Snapshot Count',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'file_share_snapshot_size': {
                        metric: 'file_share_snapshot_size',
                        longLabel: 'File Share Snapshot Size',
                        label: 'File Share Snapshot Size',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'file_transactions': {
                        metric: 'file_transactions',
                        longLabel: 'Transactions',
                        label: 'Transactions',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'file_ingress': {
                        metric: 'file_ingress',
                        label: 'Ingress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'file_egress': {
                        metric: 'file_egress',
                        label: 'Egress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'file_suc_server_lat': {
                        metric: 'file_suc_server_lat',
                        label: 'Success Server Latency',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    'file_suc_e2e_lat': {
                        metric: 'file_suc_e2e_lat',
                        label: 'Success E2E Latency',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    'file_availibility': {
                        metric: 'file_availibility',
                        label: 'Availability %',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    'queue_capacity': {
                        metric: 'queue_capacity',
                        longLabel: 'Queue Capacity',
                        label: 'Queue Capacity',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'queue_count': {
                        metric: 'queue_count',
                        label: 'Queue Count',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'queue_message_count': {
                        metric: 'queue_message_count',
                        label: 'Queue Message Count',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'queue_transactions': {
                        metric: 'queue_transactions',
                        longLabel: 'Transactions',
                        label: 'Transactions',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'queue_ingress': {
                        metric: 'queue_ingress',
                        label: 'Ingress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'queue_egress': {
                        metric: 'queue_egress',
                        label: 'Egress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'queue_suc_server_lat': {
                        metric: 'queue_suc_server_lat',
                        label: 'Success Server Latency',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    'queue_suc_e2e_lat': {
                        metric: 'queue_suc_e2e_lat',
                        label: 'Success E2E Latency',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    'queue_availibility': {
                        metric: 'queue_availibility',
                        label: 'Availability %',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    'table_capacity': {
                        metric: 'table_capacity',
                        longLabel: 'Table Capacity',
                        label: 'Table Capacity',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'table_count': {
                        metric: 'table_count',
                        label: 'Table Count',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'table_entity_count': {
                        metric: 'table_entity_count',
                        label: 'Table Entity Count',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'table_transactions': {
                        metric: 'table_transactions',
                        longLabel: 'Transactions',
                        label: 'Transactions',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                    },
                    'table_ingress': {
                        metric: 'table_ingress',
                        label: 'Ingress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'table_egress': {
                        metric: 'table_egress',
                        label: 'Egress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    'table_suc_server_lat': {
                        metric: 'table_suc_server_lat',
                        label: 'Success Server Latency',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    'table_suc_e2e_lat': {
                        metric: 'table_suc_e2e_lat',
                        label: 'Success E2E Latency',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    'table_availibility': {
                        metric: 'table_availibility',
                        label: 'Availability %',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                }
            },
        }
    },
    aws: {
        label: 'Amazon Web Services',
        order: 1,
        subsystems: {
            region: {
                label: 'Region',
                excludedTools: [
                    'all'
                ],
            },
            ec2: {
                label: "Elastic Compute Cloud",
                aliases: true,
                top: {
                    'CPU': ['CPUUtilization']
                },
                heatmap: [{
                        label: 'CPU',
                        key: 'CPUUtilization',
                        percent: 'CPUUtilization'
                    }],
                configuration: {
                    'PrivateIpAddress': {
                        label: 'IP Adress'
                    },
                    'PublicDnsName': {
                        label: 'Public DNS'
                    },
                    'InstanceType': {
                        label: 'Type'
                    },
                    'VirtualizationType': {
                        label: 'Virtualization'
                    },
                    'LaunchTime': {
                        label: 'Launch Time',
                    }
                },
                metrics: {
                    'CPUUtilization': {
                        metric: 'CPUUtilization',
                        longLabel: 'CPU %',
                        label: 'CPU %',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    'DiskReadBytes': {
                        metric: 'DiskReadBytes',
                        label: 'Data Read',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                    'DiskWriteBytes': {
                        metric: 'DiskWriteBytes',
                        label: 'Data Write',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                    'DiskWriteOps': {
                        metric: 'DiskWriteOps',
                        label: 'IO Write',
                        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
                    },
                    'DiskReadOps': {
                        metric: 'DiskReadOps',
                        label: 'IO Read',
                        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
                    },
                    'NetworkIn': {
                        metric: 'NetworkIn',
                        label: 'Network In',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                    'NetworkOut': {
                        metric: 'NetworkOut',
                        label: 'Network Out',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                }
            },
            ebs: {
                label: 'Elastic Block Store',
                metrics: {
                    'VolumeReadBytes': {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'VolumeReadBytes',
                        label: 'Data Read',
                    },
                    'VolumeWriteBytes': {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'VolumeWriteBytes',
                        label: 'Data Write',
                    },
                    'VolumeReadOps': {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'VolumeReadOps',
                        label: 'IO Read',
                    },
                    'VolumeWriteOps': {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'VolumeWriteOps',
                        label: 'IO Write',
                    },
                }
            },
            lambda: {
                label: 'Lambda',
                metrics: {
                    'Invocations': {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        metric: 'Invocations',
                        label: 'Invocations',
                    },
                    'Errors': {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        metric: 'Errors',
                        label: 'Errors',
                    },
                    'Throttles': {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        metric: 'Throttles',
                        label: 'Throttles',
                    },
                    'Duration': {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        metric: 'Duration',
                        label: 'Duration',
                    },
                    'ConcurrentExecutions': {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        metric: 'ConcurrentExecutions',
                        label: 'Concurrent Executions',
                    },
                }
            }
        }
    },
    gcloud: {
        label: 'Google Cloud',
        order: 3,
        subsystems: {
            project: {
                label: "Project",
                excludedTools: [
                    'all'
                ],
            },
            region: {
                label: "Region",
                excludedTools: [
                    'all'
                ],
            },
            compute: {
                label: "Cloud Compute",
                aliases: true,
                top: {
                    'CPU': ["cpu_util"],
                    'IO': ["read_ops", "write_ops"],
                    'Data': ["read_bytes", "write_bytes"],
                    'Net': ["received_bytes", "sent_bytes"]
                },
                heatmap: [{
                        label: "CPU",
                        key: "cpu_util",
                        percent: "cpu_util"
                    }],
                configuration: {
                    name: {
                        label: "Compute label"
                    },
                    status: {
                        label: "State"
                    },
                    cpuPlatform: {
                        label: "CPU Platform"
                    },
                    ip: {
                        label: "IP"
                    },
                    size: {
                        label: "Instance Size"
                    },
                    diskSize: {
                        label: "Disk Size [GB]",
                    },
                    creationTimestamp: {
                        label: "Created",
                        unit: subsystemMetricsResponse_constants_1.UNIT.timestamp
                    }
                },
                metrics: {
                    cpu_util: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: "cpu_util",
                        longLabel: 'CPU %',
                        label: 'CPU %',
                    },
                    read_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: "read_bytes",
                        label: 'Data Read',
                    },
                    write_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: "write_bytes",
                        label: 'Data Write',
                    },
                    read_ops: {
                        ...Definitions_1.metricTypes.iops,
                        metric: "read_ops",
                        label: 'IO Read',
                    },
                    write_ops: {
                        ...Definitions_1.metricTypes.iops,
                        metric: "write_ops",
                        label: 'IO Write',
                    },
                    received_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: "received_bytes",
                        label: 'In',
                    },
                    sent_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: "sent_bytes",
                        label: 'Out',
                    }
                }
            },
            database: {
                label: "Cloud Database",
                top: {
                    'CPU': ["cpu_util"],
                    'Memory': ["mem_used"],
                    'IO': ["read_ops", "write_ops"],
                    'Net': ["received_bytes", "sent_bytes"]
                },
                heatmap: [{
                        label: "CPU",
                        key: "cpu_util",
                        percent: "cpu_util"
                    }],
                configuration: {
                    name: {
                        label: "Compute label"
                    },
                    region: {
                        label: "Region"
                    },
                    status: {
                        label: "State"
                    },
                    ip: {
                        label: "IP"
                    },
                    databaseVersion: {
                        label: "Database Version"
                    },
                    size: {
                        label: "Instance Size"
                    },
                    dataDiskType: {
                        label: "Disk Type"
                    },
                    dataDiskSizeGb: {
                        label: "Disk Size [GB]"
                    }
                },
                metrics: {
                    cpu_util: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: "cpu_util",
                        label: 'CPU %',
                    },
                    disk_used: {
                        ...Definitions_1.metricTypes.memory,
                        metric: "disk_used",
                        label: 'Disk Utilization',
                    },
                    disk_quota: {
                        ...Definitions_1.metricTypes.memory,
                        metric: "disk_quota",
                        label: 'Capacity',
                    },
                    read_ops: {
                        ...Definitions_1.metricTypes.iops,
                        metric: "read_ops",
                        label: 'IO Read',
                    },
                    write_ops: {
                        ...Definitions_1.metricTypes.iops,
                        metric: "write_ops",
                        label: 'IO Write',
                    },
                    mem_total: {
                        ...Definitions_1.metricTypes.memory,
                        metric: "mem_total",
                        label: 'Total memory',
                    },
                    mem_used: {
                        ...Definitions_1.metricTypes.memory,
                        metric: "mem_used",
                        label: 'Used memory',
                    },
                    connections: {
                        metric: "connections",
                        label: 'Connections',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    received_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: "received_bytes",
                        label: 'In',
                    },
                    sent_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: "sent_bytes",
                        label: 'Out',
                    },
                    innodb_read: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: "innodb_read",
                        label: 'InnoDB read',
                    },
                    innodb_write: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: "innodb_write",
                        label: 'InnoDB write',
                    },
                    queries: {
                        metric: "queries",
                        label: 'Queries',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    questions: {
                        metric: "questions",
                        label: 'Questions',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    innodb_buffer_total: {
                        metric: "innodb_buffer_total",
                        label: 'InnoDB buffer total',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    innodb_buffer_free: {
                        metric: "innodb_buffer_free",
                        label: 'InnoDB buffer free',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    innodb_data_fsyncs: {
                        metric: "innodb_data_fsyncs",
                        label: 'InnoDB data fsyncs',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    innodb_os_fsyncs: {
                        metric: "innodb_os_fsyncs",
                        label: 'InnoDB OS fsyncs',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    transaction_count: {
                        metric: "transaction_count",
                        label: 'Transactions',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                }
            }
        }
    },
};
//# sourceMappingURL=Cloud.js.map