"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Database = void 0;
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const Definitions_1 = require("../Definitions");
class Database {
}
exports.Database = Database;
Database.order = 4;
Database.technology = {
    postgres: {
        label: 'PostgreSQL',
        order: 2,
        status: {
            explicit: 'db'
        },
        subsystems: {
            dbfolders: {
                label: "Cluster",
                metrics: {
                    buffersclean: {
                        label: 'Clean',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'buffersclean',
                    },
                    checkpointsreq: {
                        metric: 'checkpointsreq',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        label: 'Requested',
                    },
                    bufferscheckpoint: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        label: 'Checkpoint',
                        metric: 'bufferscheckpoint',
                    },
                    buffersbackend: {
                        metric: 'buffersbackend',
                        label: 'Backend',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    checkpointstimed: {
                        label: 'Scheduled',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        metric: 'checkpointstimed',
                    },
                },
            },
            db: {
                label: "Instance",
                aliases: true,
                excluded: {
                    all: [
                        "page",
                        "LWLockTranche",
                        "ProcArrayLock",
                        "hpage",
                        "transactionid",
                        "tuple",
                        "bufferio",
                        "replicationslotio",
                        "AddinShmemInitLock",
                        "tempfiles",
                        "aextend",
                        "OldSnapshotTimeMapLock",
                        "object",
                        "CommitTsLock",
                        "ControlFileLock",
                        "multixactmember",
                        "hrelation",
                        "ReplicationSlotAllocationLock",
                        "speculativetoken",
                        "WALBufMappingLock",
                        "ShmemIndexLock",
                        "walinsert",
                        "LWLockNamed",
                        "async",
                        "subtrans",
                        "commitratio",
                        "atuple",
                        "htuple",
                        "buffermapping",
                        "virtualxid",
                        "cursors",
                        "deadlocks",
                        "XidGenLock",
                        "TablespaceCreateLock",
                        "currenttransactions",
                        "hobject",
                        "RelCacheInitLock",
                        "ReplicationOriginLock",
                        "AutovacuumScheduleLock",
                        "advisory",
                        "RelationMappingLock",
                        "fastpathfunctioncall",
                        "blkshit",
                        "OldSerXidLock",
                        "CheckpointerCommLock",
                        "SerializableFinishedListLock",
                        "multixactoffset",
                        "userlock",
                        "auserlock",
                        "cachehitratio",
                        "disabled",
                        "frozenid",
                        "WrapLimitsVacuumLock",
                        "WALWriteLock",
                        "buffercontent",
                        "relation",
                        "aobject",
                        "xactrollback",
                        "committimestamp",
                        "AutovacuumLock",
                        "hextend",
                        "SyncScanLock",
                        "xactcommit",
                        "proc",
                        "CheckpointLock",
                        "idleintransactionaborted",
                        "huserlock",
                        "SerializablePredicateLockListLock",
                        "replicationorigin",
                        "tempbytes",
                        "arelation",
                        "clog",
                        "AsyncCtlLock",
                        "apage",
                        "extend",
                        "BackgroundWorkerLock",
                        "predicatelockmanager",
                        "BufferPin",
                        "lockmanager",
                        "MultiXactTruncationLock",
                        "oldserxid",
                        "Lock",
                    ]
                },
                parent: 'dbfolders',
                top: {
                    'DB': ["tupreturned", "blksread", "active", "size"]
                },
                metrics: {
                    size: {
                        label: 'Size',
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'size',
                    },
                    tupreturned: {
                        metric: 'tupreturned',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                        label: 'Returned',
                    },
                    tupinserted: {
                        metric: 'tupinserted',
                        label: 'Inserted',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                    },
                    tupdeleted: {
                        metric: 'tupdeleted',
                        label: 'Deleted',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    blksread: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.blocks,
                        unit: subsystemMetricsResponse_constants_1.UNIT.blocks,
                        label: 'Read',
                        metric: 'blksread',
                    },
                    active: {
                        label: 'Active',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        metric: 'active',
                    },
                    idle: {
                        metric: 'idle',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        label: 'Idle',
                    },
                    tupupdated: {
                        label: 'Updated',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        metric: 'tupupdated',
                    },
                    tupfetched: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                        label: 'Fetched',
                        metric: 'tupfetched',
                    },
                    idleintransaction: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        label: 'Idle in transaction',
                        metric: 'idleintransaction',
                    },
                    page: {
                        label: 'Page',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'page',
                    },
                    LWLockTranche: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'LWLockTranche',
                        metric: 'LWLockTranche',
                    },
                    ProcArrayLock: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'ProcArrayLock',
                        metric: 'ProcArrayLock',
                    },
                    hpage: {
                        label: 'Page',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'hpage',
                    },
                    transactionid: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Transactionid',
                        metric: 'transactionid',
                    },
                    tuple: {
                        label: 'Tuple',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'tuple',
                    },
                    bufferio: {
                        metric: 'bufferio',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Bufferio',
                    },
                    replicationslotio: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Replication slotio',
                        metric: 'replicationslotio',
                    },
                    AddinShmemInitLock: {
                        metric: 'AddinShmemInitLock',
                        label: 'AddinShmemInitLock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    tempfiles: {
                        label: 'Files',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'tempfiles',
                    },
                    aextend: {
                        label: 'Extend',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'aextend',
                    },
                    OldSnapshotTimeMapLock: {
                        metric: 'OldSnapshotTimeMapLock',
                        label: 'OldSnapshotTimeMapLock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    object: {
                        metric: 'object',
                        label: 'Object',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    CommitTsLock: {
                        metric: 'CommitTsLock',
                        label: 'CommitTsLock',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    ControlFileLock: {
                        metric: 'ControlFileLock',
                        label: 'ControlFileLock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    multixactmember: {
                        label: 'Multixact member',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'multixactmember',
                    },
                    hrelation: {
                        metric: 'hrelation',
                        label: 'Relation',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    ReplicationSlotAllocationLock: {
                        label: 'ReplicationSlotAllocationLock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'ReplicationSlotAllocationLock',
                    },
                    speculativetoken: {
                        metric: 'speculativetoken',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Speculative token',
                    },
                    WALBufMappingLock: {
                        metric: 'WALBufMappingLock',
                        label: 'WALBufMappingLock',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    ShmemIndexLock: {
                        metric: 'ShmemIndexLock',
                        label: 'ShmemIndexLock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    walinsert: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Walinsert',
                        metric: 'walinsert',
                    },
                    LWLockNamed: {
                        metric: 'LWLockNamed',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'LWLockNamed',
                    },
                    async: {
                        label: 'Async',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'async',
                    },
                    subtrans: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Subtrans',
                        metric: 'subtrans',
                    },
                    commitratio: {
                        metric: 'commitratio',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        label: 'Commit ratio',
                    },
                    atuple: {
                        label: 'Tuple',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        metric: 'atuple',
                    },
                    htuple: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Tuple',
                        metric: 'htuple',
                    },
                    buffermapping: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Buffermapping',
                        metric: 'buffermapping',
                    },
                    virtualxid: {
                        metric: 'virtualxid',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Virtualxid',
                    },
                    cursors: {
                        label: 'Cursors',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'cursors',
                    },
                    deadlocks: {
                        label: 'Deadlocks',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'deadlocks',
                    },
                    XidGenLock: {
                        label: 'XidGenLock',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'XidGenLock',
                    },
                    TablespaceCreateLock: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'TablespaceCreateLock',
                        metric: 'TablespaceCreateLock',
                    },
                    currenttransactions: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Currenttransactions',
                        metric: 'currenttransactions',
                    },
                    hobject: {
                        metric: 'hobject',
                        label: 'Object',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    RelCacheInitLock: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'RelCacheInitLock',
                        metric: 'RelCacheInitLock',
                    },
                    ReplicationOriginLock: {
                        label: 'ReplicationOriginLock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'ReplicationOriginLock',
                    },
                    AutovacuumScheduleLock: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'AutovacuumScheduleLock',
                        metric: 'AutovacuumScheduleLock',
                    },
                    advisory: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Advisory',
                        metric: 'advisory',
                    },
                    RelationMappingLock: {
                        metric: 'RelationMappingLock',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'RelationMappingLock',
                    },
                    fastpathfunctioncall: {
                        metric: 'fastpathfunctioncall',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        label: 'Fastpath function call',
                    },
                    blkshit: {
                        metric: 'blkshit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Hit',
                    },
                    OldSerXidLock: {
                        metric: 'OldSerXidLock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'OldSerXidLock',
                    },
                    CheckpointerCommLock: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'CheckpointerCommLock',
                        metric: 'CheckpointerCommLock',
                    },
                    SerializableFinishedListLock: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'SerializableFinishedListLock',
                        metric: 'SerializableFinishedListLock',
                    },
                    multixactoffset: {
                        label: 'Multixact offset',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'multixactoffset',
                    },
                    userlock: {
                        metric: 'userlock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Userlock',
                    },
                    auserlock: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Userlock',
                        metric: 'auserlock',
                    },
                    cachehitratio: {
                        metric: 'cachehitratio',
                        label: 'Cache hit ratio',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                    },
                    disabled: {
                        label: 'Disabled',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'disabled',
                    },
                    frozenid: {
                        label: 'Frozenid',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'frozenid',
                    },
                    WrapLimitsVacuumLock: {
                        label: 'WrapLimitsVacuumLock',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'WrapLimitsVacuumLock',
                    },
                    WALWriteLock: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'WALWriteLock',
                        metric: 'WALWriteLock',
                    },
                    buffercontent: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Buffer content',
                        metric: 'buffercontent',
                    },
                    relation: {
                        metric: 'relation',
                        label: 'Relation',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    aobject: {
                        metric: 'aobject',
                        label: 'Object',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    xactrollback: {
                        metric: 'xactrollback',
                        label: 'Rollbacks',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    committimestamp: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Commit timestamp',
                        metric: 'committimestamp',
                    },
                    AutovacuumLock: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'AutovacuumLock',
                        metric: 'AutovacuumLock',
                    },
                    hextend: {
                        metric: 'hextend',
                        label: 'Extend',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    SyncScanLock: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'SyncScanLock',
                        metric: 'SyncScanLock',
                    },
                    xactcommit: {
                        metric: 'xactcommit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Commits',
                    },
                    proc: {
                        metric: 'proc',
                        label: 'Proc',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    CheckpointLock: {
                        metric: 'CheckpointLock',
                        label: 'CheckpointLock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    idleintransactionaborted: {
                        metric: 'idleintransactionaborted',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        label: 'Idle in transaction aborted',
                    },
                    huserlock: {
                        label: 'Userlock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'huserlock',
                    },
                    SerializablePredicateLockListLock: {
                        metric: 'SerializablePredicateLockListLock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'SerializablePredicateLockListLock',
                    },
                    replicationorigin: {
                        metric: 'replicationorigin',
                        label: 'Replication origin',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    tempbytes: {
                        label: 'Bytes',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        metric: 'tempbytes',
                    },
                    arelation: {
                        label: 'Relation',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'arelation',
                    },
                    clog: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Clog',
                        metric: 'clog',
                    },
                    AsyncCtlLock: {
                        label: 'AsyncCtlLock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'AsyncCtlLock',
                    },
                    apage: {
                        metric: 'apage',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Page',
                    },
                    extend: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Extend',
                        metric: 'extend',
                    },
                    BackgroundWorkerLock: {
                        label: 'BackgroundWorkerLock',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'BackgroundWorkerLock',
                    },
                    predicatelockmanager: {
                        label: 'Predicate lock manager',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'predicatelockmanager',
                    },
                    BufferPin: {
                        label: 'BufferPin',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'BufferPin',
                    },
                    lockmanager: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Lock manager',
                        metric: 'lockmanager',
                    },
                    MultiXactTruncationLock: {
                        metric: 'MultiXactTruncationLock',
                        label: 'MultiXactTruncationLock',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    oldserxid: {
                        label: 'Oldserxid',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'oldserxid',
                    },
                    Lock: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Lock',
                        metric: 'Lock',
                    },
                },
            },
        },
    },
    db2: {
        label: 'IBM Db2',
        order: 4,
        status: {
            explicit: 'dbfolder'
        },
        subsystems: {
            dbfolder: {
                label: "DB Folder",
            },
            bufferpool: {
                label: "Buffer pool",
                excluded: {
                    all: [
                        "POOLDATAWRITES",
                        "POOLTEMPDATAPREADS",
                        "POOLDATALREADS",
                        "POOLTEMPINDEXPREADS",
                        "POOLXDAPREADS",
                        "POOLINDEXPREADS",
                        "POOLTEMPXDALREADS",
                        "POOLTEMPXDAPREADS",
                        "POOLXDAWRITES",
                        "POOLINDEXWRITES",
                        "POOLINDEXLREADS",
                        "POOLDATAPREADS",
                        "POOLTEMPINDEXLREADS",
                        "POOLXDALREADS",
                        "POOLTEMPDATALREADS",
                        "DIRECTREADTIME",
                        "DIRECTWRITETIME",
                        "RATIOCACHE",
                        "RATIOBUFFER",
                        "RATIOCPUT"
                    ]
                },
                metrics: {
                    LOGICALREADS: {
                        metric: 'LOGICALREADS',
                        label: 'Logical',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    PHYSICALREADS: {
                        metric: 'PHYSICALREADS',
                        label: 'Physical',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    DIRECTREADS: {
                        metric: 'DIRECTREADS',
                        label: 'Direct',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    WRITES: {
                        metric: 'WRITES',
                        label: 'Writes',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    DIRECTWRITES: {
                        metric: 'DIRECTWRITES',
                        label: 'Direct writes',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    RATINLOG: {
                        metric: 'RATINLOG',
                        label: 'Hit ratio',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    POOLWRITES: {
                        metric: 'POOLWRITES',
                        label: 'Pool writes',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLREADS: {
                        metric: 'POOLREADS',
                        label: 'Pool reads',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLREADTIME: {
                        metric: 'POOLREADTIME',
                        label: 'Pool read time',
                        ...Definitions_1.metricTypes.latency,
                    },
                    POOLWRITETIME: {
                        metric: 'POOLWRITETIME',
                        label: 'Pool write time',
                        ...Definitions_1.metricTypes.latency,
                    },
                    POOLDATAWRITES: {
                        metric: 'POOLDATAWRITES',
                        label: 'Pool Data writes',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    POOLTEMPDATAPREADS: {
                        metric: 'POOLTEMPDATAPREADS',
                        label: 'Pool temp data reads',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    POOLDATALREADS: {
                        metric: 'POOLDATALREADS',
                        label: 'Pool data l reads',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    POOLTEMPINDEXPREADS: {
                        metric: 'POOLTEMPINDEXPREADS',
                        label: 'Pool temp index p reads',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLXDAPREADS: {
                        metric: 'POOLXDAPREADS',
                        label: 'Pool xda p reads',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLINDEXPREADS: {
                        metric: 'POOLINDEXPREADS',
                        label: 'Pool index p reads',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLTEMPXDALREADS: {
                        metric: 'POOLTEMPXDALREADS',
                        label: 'Pool temp xda l reads',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLTEMPXDAPREADS: {
                        metric: 'POOLTEMPXDAPREADS',
                        label: 'Pool temp xda p reads',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLXDAWRITES: {
                        metric: 'POOLXDAWRITES',
                        label: 'Pool xda writes',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLINDEXWRITES: {
                        metric: 'POOLINDEXWRITES',
                        label: 'Pool index writes',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLINDEXLREADS: {
                        metric: 'POOLINDEXLREADS',
                        label: 'Pool index l reads',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLDATAPREADS: {
                        metric: 'POOLDATAPREADS',
                        label: 'Pool data p reads',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    POOLTEMPINDEXLREADS: {
                        metric: 'POOLTEMPINDEXLREADS',
                        label: 'Pool temp index l reads',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLXDALREADS: {
                        metric: 'POOLXDALREADS',
                        label: 'Pool XDALREADS',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    POOLTEMPDATALREADS: {
                        metric: 'POOLTEMPDATALREADS',
                        label: 'Pool temp data l reads',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    DIRECTREADTIME: {
                        metric: 'DIRECTREADTIME',
                        label: 'Direct read time',
                        ...Definitions_1.metricTypes.latency,
                    },
                    DIRECTWRITETIME: {
                        metric: 'DIRECTWRITETIME',
                        label: 'Direct write time',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    RATIOCACHE: {
                        metric: 'RATIOCACHE',
                        label: 'Cache hit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    RATIOBUFFER: {
                        metric: 'RATIOBUFFER',
                        label: 'Buffer hit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    RATIOCPUT: {
                        metric: 'RATIOCPUT',
                        label: 'CPU time',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                },
            },
            member: {
                label: "Member",
                excluded: {
                    all: [
                        "AGENTSTOP",
                        "NUMPOOLEDAGENTS",
                        "NUMASSOCAGENTS",
                        "NUMCOORDAGENTS",
                        "NUMLOCKSHELD",
                        "NUMLOCKSWAITING",
                        "LOCKESCALS",
                        "LOCKTIMEOUTS",
                        "DEADLOCKS",
                        "PHYSICALREADS",
                        "DIRECTREADTIME",
                        "DIRECTWRITETIME",
                        "POOLREADTIME",
                        "POOLWRITETIME",
                        "ROWSMODIFIED",
                        "INTROWSDELETED",
                        "INTROWSINSERTED",
                        "INTROWSUPDATED",
                        "FEDROWSRETURNED",
                        "FEDROWSREAD",
                        "FEDROWSUPDATED",
                        "FEDROWSDELETED",
                        "FEDROWSINSERTED",
                        "IPCSENDVOLUME",
                        "IPCRECVVOLUME",
                        "FCMSENDVOLUME",
                        "FCMRECVVOLUME",
                        "PKGCACHEINSERTS",
                        "PKGCACHELOOKUPS",
                        "TOTALAPPCOMMITS",
                        "TOTALAPPROLLBACKS",
                        "LOGDISKWAITTIME",
                        "TCPIPSENDWAITTIME",
                        "TCPIPRECVWAITTIME",
                        "IPCSENDWAITTIME",
                        "IPCRECVWAITTIME",
                        "FCMSENDWAITTIME",
                        "FCMRECVWAITTIME",
                        "CFWAITTIME",
                        "LOCKWAITTIME",
                        "AGENTWAITTIME",
                        "WLMQUEUETIMETOTAL",
                        "RATIOCACHE",
                        "RATIOBUFFER",
                        "RATIOCPUT",
                        "AGENTSREGISTERED",
                        "IDLEAGENTS",
                        "AGENTSSTOLEN",
                    ]
                },
                top: {
                    'DB': [
                        "LOGICALREADS",
                        "PHYSICALREADS",
                        "DIRECTREADS",
                        "WRITES",
                        "DIRECTWRITES",
                        "TOTALCONS",
                    ]
                },
                metrics: {
                    LOGICALREADS: {
                        metric: 'LOGICALREADS',
                        label: 'Logical',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    PHYSICALREADS: {
                        metric: 'PHYSICALREADS',
                        label: 'Physical',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    DIRECTREADS: {
                        metric: 'DIRECTREADS',
                        label: 'Direct',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    WRITES: {
                        metric: 'WRITES',
                        label: 'Writes',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    DIRECTWRITES: {
                        metric: 'DIRECTWRITES',
                        label: 'Direct writes',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    ROWSREAD: {
                        metric: 'ROWSREAD',
                        label: 'Read',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    ROWSRETURNED: {
                        metric: 'ROWSRETURNED',
                        label: 'Returned',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    ROWSDELETED: {
                        metric: 'ROWSDELETED',
                        label: 'Deleted',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    TCPIPSENDVOLUME: {
                        metric: 'TCPIPSENDVOLUME',
                        label: 'TCPIP out',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    TCPIPRECVVOLUME: {
                        metric: 'TCPIPRECVVOLUME',
                        label: 'TCPIP in',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    ROWSUPDATED: {
                        metric: 'ROWSUPDATED',
                        label: 'Updated',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    CONNECTIONSTOP: {
                        metric: 'CONNECTIONSTOP',
                        label: 'Max conns',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                    },
                    TOTALCONS: {
                        metric: 'TOTALCONS',
                        label: 'Conns/s',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                    },
                    TOTALSECCONS: {
                        metric: 'TOTALSECCONS',
                        label: 'Second conns/s',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                    },
                    APPLSCURCONS: {
                        metric: 'APPLSCURCONS',
                        label: 'Current conns',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                    },
                    ROWSINSERTED: {
                        metric: 'ROWSINSERTED',
                        label: 'Inserted',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    RATINLOG: {
                        metric: 'RATINLOG',
                        label: 'Hit ratio',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    AGENTSTOP: {
                        metric: 'AGENTSTOP',
                        label: 'Agents top',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    NUMPOOLEDAGENTS: {
                        metric: 'NUMPOOLEDAGENTS',
                        label: 'Pooled',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    NUMASSOCAGENTS: {
                        metric: 'NUMASSOCAGENTS',
                        label: 'Associated',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    NUMCOORDAGENTS: {
                        metric: 'NUMCOORDAGENTS',
                        label: 'Coordinated',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    NUMLOCKSHELD: {
                        metric: 'NUMLOCKSHELD',
                        label: 'Held',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    NUMLOCKSWAITING: {
                        metric: 'NUMLOCKSWAITING',
                        label: 'Waiting',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    LOCKESCALS: {
                        metric: 'LOCKESCALS',
                        label: 'Escals',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    LOCKTIMEOUTS: {
                        metric: 'LOCKTIMEOUTS',
                        label: 'Timeouts',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    DEADLOCKS: {
                        metric: 'DEADLOCKS',
                        label: 'Deadlocks',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    DIRECTREADTIME: {
                        metric: 'DIRECTREADTIME',
                        label: 'Direct read time',
                        ...Definitions_1.metricTypes.latency,
                    },
                    DIRECTWRITETIME: {
                        metric: 'DIRECTWRITETIME',
                        label: 'Direct write time',
                        ...Definitions_1.metricTypes.latency,
                    },
                    POOLREADTIME: {
                        metric: 'POOLREADTIME',
                        label: 'Pool read time',
                        ...Definitions_1.metricTypes.latency,
                    },
                    POOLWRITETIME: {
                        metric: 'POOLWRITETIME',
                        label: 'Pool write time',
                        ...Definitions_1.metricTypes.latency,
                    },
                    ROWSMODIFIED: {
                        metric: 'ROWSMODIFIED',
                        label: 'Modified',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    INTROWSDELETED: {
                        metric: 'INTROWSDELETED',
                        label: 'Deleted',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    INTROWSINSERTED: {
                        metric: 'INTROWSINSERTED',
                        label: 'Inserted',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    INTROWSUPDATED: {
                        metric: 'INTROWSUPDATED',
                        label: 'Updated',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    FEDROWSRETURNED: {
                        metric: 'FEDROWSRETURNED',
                        label: 'Returned',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    FEDROWSREAD: {
                        metric: 'FEDROWSREAD',
                        label: 'Read',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    FEDROWSUPDATED: {
                        metric: 'FEDROWSUPDATED',
                        label: 'Updated',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    FEDROWSDELETED: {
                        metric: 'FEDROWSDELETED',
                        label: 'Deleted',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    FEDROWSINSERTED: {
                        metric: 'FEDROWSINSERTED',
                        label: 'Inserted',
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                    },
                    IPCSENDVOLUME: {
                        metric: 'IPCSENDVOLUME',
                        label: 'IPC out',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    IPCRECVVOLUME: {
                        metric: 'IPCRECVVOLUME',
                        label: 'FCM in',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    FCMSENDVOLUME: {
                        metric: 'FCMSENDVOLUME',
                        label: 'FCM out',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    FCMRECVVOLUME: {
                        metric: 'FCMRECVVOLUME',
                        label: 'FCM in',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    PKGCACHEINSERTS: {
                        metric: 'PKGCACHEINSERTS',
                        label: 'Cache inserts',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    PKGCACHELOOKUPS: {
                        metric: 'PKGCACHELOOKUPS',
                        label: 'Cache lookups',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    TOTALAPPCOMMITS: {
                        metric: 'TOTALAPPCOMMITS',
                        label: 'Commits',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    TOTALAPPROLLBACKS: {
                        metric: 'TOTALAPPROLLBACKS',
                        label: 'Rollbacks ',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    LOGDISKWAITTIME: {
                        metric: 'LOGDISKWAITTIME',
                        label: 'Log disk',
                        ...Definitions_1.metricTypes.latency,
                    },
                    TCPIPSENDWAITTIME: {
                        metric: 'TCPIPSENDWAITTIME',
                        label: 'TCPIP out',
                        ...Definitions_1.metricTypes.latency,
                    },
                    TCPIPRECVWAITTIME: {
                        metric: 'TCPIPRECVWAITTIME',
                        label: 'TCPIP in',
                        ...Definitions_1.metricTypes.latency,
                    },
                    IPCSENDWAITTIME: {
                        metric: 'IPCSENDWAITTIME',
                        label: 'IPC out',
                        ...Definitions_1.metricTypes.latency,
                    },
                    IPCRECVWAITTIME: {
                        metric: 'IPCRECVWAITTIME',
                        label: 'IPC in',
                        ...Definitions_1.metricTypes.latency,
                    },
                    FCMSENDWAITTIME: {
                        metric: 'FCMSENDWAITTIME',
                        label: 'FCM out',
                        ...Definitions_1.metricTypes.latency,
                    },
                    FCMRECVWAITTIME: {
                        metric: 'FCMRECVWAITTIME',
                        label: 'FCM in',
                        ...Definitions_1.metricTypes.latency,
                    },
                    CFWAITTIME: {
                        metric: 'CFWAITTIME',
                        label: 'CF',
                        ...Definitions_1.metricTypes.latency,
                    },
                    LOCKWAITTIME: {
                        metric: 'LOCKWAITTIME',
                        label: 'Lock',
                        ...Definitions_1.metricTypes.latency,
                    },
                    AGENTWAITTIME: {
                        metric: 'AGENTWAITTIME',
                        label: 'Agent',
                        ...Definitions_1.metricTypes.latency,
                    },
                    WLMQUEUETIMETOTAL: {
                        metric: 'WLMQUEUETIMETOTAL',
                        label: 'Wlm queue total',
                        ...Definitions_1.metricTypes.latency,
                    },
                    RATIOCACHE: {
                        metric: 'RATIOCACHE',
                        label: 'Cache hit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    RATIOBUFFER: {
                        metric: 'RATIOBUFFER',
                        label: 'Buffer hit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    RATIOCPUT: {
                        metric: 'RATIOCPUT',
                        label: 'CPU time',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    AGENTSREGISTERED: {
                        metric: 'AGENTSREGISTERED',
                        label: 'Registered',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    IDLEAGENTS: {
                        metric: 'IDLEAGENTS',
                        label: 'Idle',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    AGENTSSTOLEN: {
                        metric: 'AGENTSSTOLEN',
                        label: 'Stolen',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                },
            },
        },
    },
    oracledb: {
        label: 'Oracle Database',
        order: 1,
        status: {
            explicit: 'instance'
        },
        subsystems: {
            menugroup: {
                excludedTools: [
                    'all'
                ],
                label: "Menu group",
            },
            submenugroup: {
                excludedTools: [
                    'all'
                ],
                label: "Sub group",
            },
            host: {
                excludedTools: [
                    'all'
                ],
                label: "Host",
            },
            odbfolder: {
                excludedTools: [
                    'all'
                ],
                label: "DB folder",
            },
            instance: {
                label: "Instance",
                aliases: true,
                excluded: {
                    all: [
                        "gccurrentgrant2way",
                        "flashbacklogfilesync",
                        "OpenCursorsPerSec",
                        "CellPhysicalIOInterconnectBytes",
                        "HardParseCountPerSec",
                        "gcbufferbusyacquire",
                        "ExecutionsPerSec",
                        "GlobalCacheBlocksCorrupted",
                        "PhysicalReadBytesPerSec",
                        "MemorySortsRatio",
                        "NetworkTrafficVolumePerSec",
                        "RedoGeneratedPerSec",
                        "DBBlockChangesPerSec",
                        "GCAvgCURBlockreceivems",
                        "GCAvgCRBlockreceivems",
                        "gccrblockcongested",
                        "LogicalReadsPerSec",
                        "BufferCacheHitRatio",
                        "gccrblockthreeway",
                        "UserCommitsPerSec",
                        "gcremaster",
                        "gccrgrant2way",
                        "CurrentOpenCursorsCount",
                        "PhysicalReadsPerSec",
                        "free",
                        "gccurrentretry",
                        "gccurrentblock2way",
                        "used",
                        "PhysicalWritesPerSec",
                        "gccurrentblocklost",
                        "gccrblock2way",
                        "DatabaseWaitTimeRatio",
                        "DatabaseCPUTimeRatio",
                        "gccurrentmultiblockrequest",
                        "gccurrentsplit",
                        "gccrblockbusy",
                        "gccurrentblockcongested",
                        "GCCRBlockReceivedPerSecond",
                        "gccrdiskread",
                        "gccrgrantcongested",
                        "gccurrentblockthreeway",
                        "PhysicalWriteBytesPerSec",
                        "GlobalCacheAverageCRGetTime",
                        "gcbufferbusyrelease",
                        "gccurrentblockbusy",
                        "UserTransactionPerSec",
                        "gccrmultiblockrequest",
                        "gccrfailure",
                        "GlobalCacheAverageCurrentGetTime",
                        "gccurrentgrantcongested",
                        "gccurrentgrantbusy",
                        "GCCurrentBlockReceivedPerSecond",
                        "gccrblocklost",
                        "logcapacity",
                        "GlobalCacheBlocksLost",
                        "SoftParseRatio",
                        "PGACacheHit"
                    ],
                },
                top: {
                    'DB': ["CPUUsagePerSec", "CurrentLogonsCount", "IORequestsperSecond", "IOMegabytesperSecond"]
                },
                metrics: {
                    HostCPUUtilization: {
                        label: 'CPU %',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        metric: 'HostCPUUtilization',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        longLabel: 'CPU %'
                    },
                    CPUUsagePerSec: {
                        label: 'CPU usage',
                        longLabel: 'CPU Usage/sec',
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'CPUUsagePerSec',
                    },
                    CPUUsagePerTxn: {
                        label: 'CPU usage',
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'CPUUsagePerTxn',
                        longLabel: 'CPU Usage/txn'
                    },
                    IORequestsperSecond: {
                        label: 'IO',
                        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
                        metric: 'IORequestsperSecond',
                    },
                    IOMegabytesperSecond: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'IOMegabytesperSecond',
                        label: 'Data',
                    },
                    LogonsPerSec: {
                        label: 'Logons Per Sec',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        metric: 'LogonsPerSec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                    },
                    ActiveParallelSessions: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        metric: 'ActiveParallelSessions',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        label: 'Active Parallel Sessions',
                    },
                    CurrentLogonsCount: {
                        label: 'Current Logons Count',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        metric: 'CurrentLogonsCount',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                    },
                    ActiveSerialSessions: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        metric: 'ActiveSerialSessions',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        label: 'Active Serial Sessions',
                    },
                    AverageActiveSessions: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        metric: 'AverageActiveSessions',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        label: 'Average Active Sessions',
                    },
                    dbfilesequentialread: {
                        label: 'DB file sequential read',
                        metric: 'dbfilesequentialread',
                        ...Definitions_1.metricTypes.latency,
                    },
                    dbfileparallelwrite: {
                        label: 'DB file parallel write',
                        metric: 'dbfileparallelwrite',
                        ...Definitions_1.metricTypes.latency,
                    },
                    logfilesync: {
                        metric: 'logfilesync',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Log file sync',
                    },
                    dbfilescatteredread: {
                        metric: 'dbfilescatteredread',
                        ...Definitions_1.metricTypes.latency,
                        label: 'DB file scattered read',
                    },
                    dbfilesinglewrite: {
                        metric: 'dbfilesinglewrite',
                        ...Definitions_1.metricTypes.latency,
                        label: 'DB file single write',
                    },
                    logfilesinglewrite: {
                        metric: 'logfilesinglewrite',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Log file single write',
                    },
                    logfileparallelwrite: {
                        label: 'Log file parallel write',
                        ...Definitions_1.metricTypes.latency,
                        metric: 'logfileparallelwrite',
                    },
                    gccurrentgrant2way: {
                        metric: 'gccurrentgrant2way',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Current grant 2way',
                    },
                    flashbacklogfilesync: {
                        label: 'Flashback log file sync',
                        metric: 'flashbacklogfilesync',
                        ...Definitions_1.metricTypes.latency,
                    },
                    OpenCursorsPerSec: {
                        label: 'Open Cursors/s',
                        metric: 'OpenCursorsPerSec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    CellPhysicalIOInterconnectBytes: {
                        label: 'Cell Physical IO Interconnect',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                        defaultPrecision: 2,
                        metric: 'CellPhysicalIOInterconnectBytes',
                    },
                    HardParseCountPerSec: {
                        label: 'Hard Parse',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'HardParseCountPerSec',
                    },
                    gcbufferbusyacquire: {
                        label: 'Buffer busy acquire',
                        ...Definitions_1.metricTypes.latency,
                        metric: 'gcbufferbusyacquire',
                    },
                    ExecutionsPerSec: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'ExecutionsPerSec',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Executions',
                    },
                    GlobalCacheBlocksCorrupted: {
                        metric: 'GlobalCacheBlocksCorrupted',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Global Cache Blocks Corrupted',
                    },
                    PhysicalReadBytesPerSec: {
                        label: 'Read Bytes',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                        defaultPrecision: 2,
                        metric: 'PhysicalReadBytesPerSec',
                    },
                    MemorySortsRatio: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        metric: 'MemorySortsRatio',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        label: 'Memory Sorts',
                    },
                    NetworkTrafficVolumePerSec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                        defaultPrecision: 2,
                        metric: 'NetworkTrafficVolumePerSec',
                        label: 'Traffic Volume',
                    },
                    RedoGeneratedPerSec: {
                        metric: 'RedoGeneratedPerSec',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                        defaultPrecision: 2,
                        label: 'Redo',
                    },
                    DBBlockChangesPerSec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'DBBlockChangesPerSec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Block Changes',
                    },
                    GCAvgCURBlockreceivems: {
                        label: 'Avg CUR Block receive ms',
                        metric: 'GCAvgCURBlockreceivems',
                        ...Definitions_1.metricTypes.latency,
                    },
                    GCAvgCRBlockreceivems: {
                        label: 'Avg CR Block receive ms',
                        metric: 'GCAvgCRBlockreceivems',
                        ...Definitions_1.metricTypes.latency,
                    },
                    gccrblockcongested: {
                        metric: 'gccrblockcongested',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Cr block congested',
                    },
                    LogicalReadsPerSec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'LogicalReadsPerSec',
                        label: 'Logical Reads',
                    },
                    BufferCacheHitRatio: {
                        metric: 'BufferCacheHitRatio',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        label: 'Buffer Cache Hit',
                    },
                    gccrblockthreeway: {
                        label: 'Cr block threeway',
                        metric: 'gccrblockthreeway',
                        ...Definitions_1.metricTypes.latency,
                    },
                    UserCommitsPerSec: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'UserCommitsPerSec',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'User Commits',
                    },
                    gcremaster: {
                        label: 'Remaster',
                        metric: 'gcremaster',
                        ...Definitions_1.metricTypes.latency,
                    },
                    gccrgrant2way: {
                        metric: 'gccrgrant2way',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Cr grant 2way',
                    },
                    CurrentOpenCursorsCount: {
                        metric: 'CurrentOpenCursorsCount',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Current Open Cursors',
                    },
                    PhysicalReadsPerSec: {
                        label: 'Reads',
                        metric: 'PhysicalReadsPerSec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.blocks_per_second,
                        unit: subsystemMetricsResponse_constants_1.UNIT.blocks_per_second,
                    },
                    free: {
                        label: 'Free',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        metric: 'free',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                    },
                    gccurrentretry: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'gccurrentretry',
                        label: 'Current retry',
                    },
                    gccurrentblock2way: {
                        label: 'Current block 2way',
                        ...Definitions_1.metricTypes.latency,
                        metric: 'gccurrentblock2way',
                    },
                    used: {
                        label: 'Used',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        metric: 'used',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                    },
                    PhysicalWritesPerSec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.blocks_per_second,
                        metric: 'PhysicalWritesPerSec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.blocks_per_second,
                        label: 'Writes',
                    },
                    gccurrentblocklost: {
                        label: 'Current block lost',
                        metric: 'gccurrentblocklost',
                        ...Definitions_1.metricTypes.latency,
                    },
                    gccrblock2way: {
                        metric: 'gccrblock2way',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Cr block 2way',
                    },
                    DatabaseWaitTimeRatio: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        metric: 'DatabaseWaitTimeRatio',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        label: 'Wait Time',
                    },
                    DatabaseCPUTimeRatio: {
                        metric: 'DatabaseCPUTimeRatio',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        label: 'CPU Time',
                    },
                    gccurrentmultiblockrequest: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'gccurrentmultiblockrequest',
                        label: 'Current multi block request',
                    },
                    gccurrentsplit: {
                        label: 'Current split',
                        metric: 'gccurrentsplit',
                        ...Definitions_1.metricTypes.latency,
                    },
                    gccrblockbusy: {
                        label: 'Cr block busy',
                        ...Definitions_1.metricTypes.latency,
                        metric: 'gccrblockbusy',
                    },
                    gccurrentblockcongested: {
                        label: 'Current block congested',
                        ...Definitions_1.metricTypes.latency,
                        metric: 'gccurrentblockcongested',
                    },
                    GCCRBlockReceivedPerSecond: {
                        metric: 'GCCRBlockReceivedPerSecond',
                        ...Definitions_1.metricTypes.latency,
                        label: 'CR Block Received Per Second',
                    },
                    gccrdiskread: {
                        metric: 'gccrdiskread',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Cr disk read',
                    },
                    gccrgrantcongested: {
                        label: 'Cr grant congested',
                        ...Definitions_1.metricTypes.latency,
                        metric: 'gccrgrantcongested',
                    },
                    gccurrentblockthreeway: {
                        label: 'Current block threeway',
                        metric: 'gccurrentblockthreeway',
                        ...Definitions_1.metricTypes.latency,
                    },
                    PhysicalWriteBytesPerSec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'PhysicalWriteBytesPerSec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Write Bytes',
                    },
                    GlobalCacheAverageCRGetTime: {
                        metric: 'GlobalCacheAverageCRGetTime',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Global Cache Average CR Get Time',
                    },
                    gcbufferbusyrelease: {
                        label: 'Buffer busy release',
                        metric: 'gcbufferbusyrelease',
                        ...Definitions_1.metricTypes.latency,
                    },
                    gccurrentblockbusy: {
                        label: 'Current block busy',
                        metric: 'gccurrentblockbusy',
                        ...Definitions_1.metricTypes.latency,
                    },
                    UserTransactionPerSec: {
                        label: 'User Transaction',
                        metric: 'UserTransactionPerSec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    gccrmultiblockrequest: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'gccrmultiblockrequest',
                        label: 'Cr multi block request',
                    },
                    gccrfailure: {
                        label: 'Cr failure',
                        metric: 'gccrfailure',
                        ...Definitions_1.metricTypes.latency,
                    },
                    GlobalCacheAverageCurrentGetTime: {
                        label: 'Global Cache Average Current Get Time',
                        metric: 'GlobalCacheAverageCurrentGetTime',
                        ...Definitions_1.metricTypes.latency,
                    },
                    gccurrentgrantcongested: {
                        label: 'Current grant congested',
                        ...Definitions_1.metricTypes.latency,
                        metric: 'gccurrentgrantcongested',
                    },
                    gccurrentgrantbusy: {
                        label: 'Current grant busy',
                        metric: 'gccurrentgrantbusy',
                        ...Definitions_1.metricTypes.latency,
                    },
                    GCCurrentBlockReceivedPerSecond: {
                        label: 'Current Block Received Per Second',
                        ...Definitions_1.metricTypes.latency,
                        metric: 'GCCurrentBlockReceivedPerSecond',
                    },
                    gccrblocklost: {
                        metric: 'gccrblocklost',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Cr block lost',
                    },
                    log_capacity: {
                        label: 'Log',
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'log_capacity',
                    },
                    GlobalCacheBlocksLost: {
                        metric: 'GlobalCacheBlocksLost',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Global Cache Blocks Lost',
                    },
                    SoftParseRatio: {
                        label: 'Soft Parse',
                        metric: 'SoftParseRatio',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                    },
                    PGACacheHit: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        metric: 'PGACacheHit',
                        label: 'PGA Cache Hit',
                    },
                    control_files: {
                        label: 'Control files',
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'control_files',
                    },
                    temp_files: {
                        label: 'Temp files',
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'temp_files',
                    },
                    recoverysize: {
                        label: 'Total',
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'recoverysize',
                    },
                    recoveryused: {
                        label: 'Used',
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'recoveryused',
                    },
                },
            },
        },
    },
    sqlserver: {
        label: 'Microsoft SQL Server',
        order: 3,
        status: {
            explicit: 'db'
        },
        subsystems: {
            db: {
                label: "Instance",
                aliases: true,
                excluded: {
                    all: [
                        "CacheHitRatio",
                        "latency_wr",
                        "Transactionssec",
                        "data_wr",
                        "LogCacheHitRatio",
                        "io_rd",
                        "LocksRequestssec",
                        "latency_total",
                        "Transactions",
                        "LocksWaitssec",
                        "log_space",
                        "Trackedtransactionssec",
                        "unused",
                        "LockTimeoutssec",
                        "NumberofDeadlockssec",
                        "io_wr",
                        "Buffercachehitratio",
                        "latency_rd",
                        "WriteTransactionssec",
                        "ActiveTransactions",
                        "data_rd",
                        "CachedCursorsCounts",
                        "Activecursors",
                        "CursorRequestssec"
                    ]
                },
                parent: 'sqlsfolder',
                top: {
                    'IO': ["io_total"],
                    'Data': ["data_total"]
                },
                metrics: {
                    io_total: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
                        label: 'IO',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
                        metric: 'io_total',
                    },
                    data_total: {
                        metric: 'data_total',
                        ...Definitions_1.metricTypes.throughput,
                        label: 'Data',
                    },
                    latency_total: {
                        metric: 'latency_total',
                        label: 'Latency',
                        ...Definitions_1.metricTypes.latency,
                    },
                    used: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        metric: 'used',
                        label: 'Used',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                    },
                    available: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        label: 'Available',
                        metric: 'available',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    CacheHitRatio: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        label: 'Cache Hit Ratio',
                        metric: 'CacheHitRatio',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    latency_wr: {
                        label: 'Latency Write',
                        ...Definitions_1.metricTypes.latency,
                        metric: 'latency_wr',
                    },
                    Transactionssec: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'Transactionssec',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Transactions/sec',
                    },
                    data_wr: {
                        metric: 'data_wr',
                        label: 'Data Write',
                        ...Definitions_1.metricTypes.throughput,
                    },
                    LogCacheHitRatio: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        metric: 'LogCacheHitRatio',
                        label: 'Log Cache Hit Ratio',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                    },
                    io_rd: {
                        label: 'IO Read',
                        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
                        metric: 'io_rd',
                    },
                    Transactions: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'Transactions',
                        label: 'Transactions',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    log_space: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        metric: 'log_space',
                        label: 'LOG space',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                    },
                    Trackedtransactionssec: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'Trackedtransactionssec',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Tracked transactions/sec',
                    },
                    unused: {
                        label: 'Unused',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        metric: 'unused',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    io_wr: {
                        metric: 'io_wr',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
                        label: 'IO Write',
                        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
                    },
                    Buffercachehitratio: {
                        metric: 'Buffercachehitratio',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        label: 'Buffer cache hit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                    },
                    latency_rd: {
                        metric: 'latency_rd',
                        ...Definitions_1.metricTypes.latency,
                        label: 'Latency Read',
                    },
                    WriteTransactionssec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Write Transactions/sec',
                        metric: 'WriteTransactionssec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    ActiveTransactions: {
                        metric: 'ActiveTransactions',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Active Transactions',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    data_rd: {
                        metric: 'data_rd',
                        ...Definitions_1.metricTypes.throughput,
                        label: 'Data Read',
                    },
                    CachedCursorsCounts: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'CachedCursorsCounts',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'CachedCursorsCounts',
                    },
                    Activecursors: {
                        metric: 'Activecursors',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Active cursors',
                    },
                    CursorRequestssec: {
                        metric: 'CursorRequestssec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Cursor Requests/sec',
                    },
                },
            },
            sqlsfolder: {
                label: "Cluster",
                excluded: {
                    all: [
                        "ASYNC_NETWORK_IO",
                        "LCK_M_BU",
                        "PAGEIOLATCH_SH",
                        "Readaheadtimesec",
                        "LCK_M_RIn_U",
                        "LCK_M_RIn_X",
                        "MSQL_XP",
                        "LCK_M_RS_U",
                        "PAGELATCH_UP",
                        "LCK_M_U",
                        "Extensionallocatedpages",
                        "LCK_M_RX_S",
                        "Readaheadpagessec",
                        "LCK_M_SCH_M",
                        "IOLATCH_UP",
                        "Extensionfreepages",
                        "ConnectionResetsec",
                        "PAGELATCH_KP",
                        "IO_COMPLETION",
                        "TotalServerMemory",
                        "ReservedServerMemory",
                        "PAGELATCH_SH",
                        "WRITE_COMPLETION",
                        "Pagelifeexpectancy",
                        "PAGELATCH_NL",
                        "LCK_M_UIX",
                        "PAGEIOLATCH_EX",
                        "LATCH_SH",
                        "LCK_M_RS_S",
                        "Databasepages",
                        "Targetpages",
                        "LCK_M_IS",
                        "Checkpointpagessec",
                        "LCK_M_IX",
                        "Logoutssec",
                        "PAGEIOLATCH_DT",
                        "PAGELATCH_EX",
                        "CachedCursorsCounts",
                        "LCK_M_RIn_S",
                        "LCK_M_RX_X",
                        "LATCH_KP",
                        "LATCH_EX",
                        "FreeMemory",
                        "EXECSYNC",
                        "TargetServerMemory",
                        "LATCH_NL",
                        "SOS_SCHEDULER_YIELD",
                        "THREADPOOL",
                        "PAGEIOLATCH_NL",
                        "LATCH_UP",
                        "Activecursors",
                        "LATCH_DT",
                        "LCK_M_SCH_S",
                        "WRITELOG",
                        "LCK_M_SIX",
                        "LCK_M_X",
                        "LCK_M_RX_U",
                        "LCK_M_SIU",
                        "Lazywritessec",
                        "LCK_M_S",
                        "CXPACKET",
                        "PAGEIOLATCH_KP",
                        "LCK_M_RIn_NL",
                        "LCK_M_IU",
                        "PAGELATCH_DT",
                        "RS_SM_QUERY_CMPL",
                    ]
                },
                top: {
                    'Connections': ["UserConnections"]
                },
                metrics: {
                    LocksWaitssec: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'LocksWaitssec',
                        label: 'LocksWaitssec',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    LocksRequestssec: {
                        label: 'Locks rqst/sec',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'LocksRequestssec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    LockTimeoutssec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Lock Timeouts/sec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'LockTimeoutssec',
                    },
                    NumberofDeadlockssec: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'NumberofDeadlockssec',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Number of Deadlocks/sec',
                    },
                    UserConnections: {
                        label: 'User Connections',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        metric: 'UserConnections',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                    },
                    Loginssec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Logins/s',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'Loginssec',
                    },
                    Pagereadssec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Page reads/s',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'Pagereadssec',
                    },
                    Pagewritessec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Page writes/s',
                        metric: 'Pagewritessec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    Backgroundwriterpagessec: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'Backgroundwriterpagessec',
                        label: 'Background writer pages/s',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    ASYNC_NETWORK_IO: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'ASYNC_NETWORK_IO',
                        metric: 'ASYNC_NETWORK_IO',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    LCK_M_BU: {
                        label: 'LCK_M_BU',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LCK_M_BU',
                    },
                    PAGEIOLATCH_SH: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'PAGEIOLATCH_SH',
                        metric: 'PAGEIOLATCH_SH',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    Readaheadtimesec: {
                        metric: 'Readaheadtimesec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Readahead time/s',
                    },
                    LCK_M_RIn_U: {
                        metric: 'LCK_M_RIn_U',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        label: 'LCK_M_RIn_U',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    LCK_M_RIn_X: {
                        label: 'LCK_M_RIn_X',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        metric: 'LCK_M_RIn_X',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    MSQL_XP: {
                        metric: 'MSQL_XP',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'MSQL_XP',
                    },
                    LCK_M_RS_U: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'LCK_M_RS_U',
                        metric: 'LCK_M_RS_U',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    PAGELATCH_UP: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'PAGELATCH_UP',
                        label: 'PAGELATCH_UP',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    LCK_M_U: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'LCK_M_U',
                        metric: 'LCK_M_U',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    Extensionallocatedpages: {
                        metric: 'Extensionallocatedpages',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Extensional located pages',
                    },
                    LCK_M_RX_S: {
                        label: 'LCK_M_RX_S',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LCK_M_RX_S',
                    },
                    Readaheadpagessec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Readahead pages/sec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'Readaheadpagessec',
                    },
                    LCK_M_SCH_M: {
                        label: 'LCK_M_SCH_M',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        metric: 'LCK_M_SCH_M',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    PAGEIOLATCH_UP: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'PAGEIOLATCH_UP',
                        metric: 'PAGEIOLATCH_UP',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    Extensionfreepages: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'Extensionfreepages',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Extension free pages',
                    },
                    ConnectionResetsec: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        metric: 'ConnectionResetsec',
                        label: 'Connection Reset/sec',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                    },
                    PAGELATCH_KP: {
                        label: 'PAGELATCH_KP',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'PAGELATCH_KP',
                    },
                    IO_COMPLETION: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'IO_COMPLETION',
                        label: 'IO_COMPLETION',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    TotalServerMemory: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        metric: 'TotalServerMemory',
                        label: 'Total Server',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                    },
                    ReservedServerMemory: {
                        metric: 'ReservedServerMemory',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        label: 'Reserved Server',
                    },
                    PAGELATCH_SH: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'PAGELATCH_SH',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'PAGELATCH_SH',
                    },
                    WRITE_COMPLETION: {
                        label: 'WRITE_COMPLETION',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'WRITE_COMPLETION',
                    },
                    Pagelifeexpectancy: {
                        label: 'Life expectancy',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'Pagelifeexpectancy',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    PAGELATCH_NL: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'PAGELATCH_NL',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'PAGELATCH_NL',
                    },
                    LCK_M_UIX: {
                        label: 'LCK_M_UIX',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LCK_M_UIX',
                    },
                    PAGEIOLATCH_EX: {
                        label: 'PAGEIOLATCH_EX',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        metric: 'PAGEIOLATCH_EX',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    LATCH_SH: {
                        label: 'LATCH_SH',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LATCH_SH',
                    },
                    LCK_M_RS_S: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LCK_M_RS_S',
                        label: 'LCK_M_RS_S',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    Databasepages: {
                        metric: 'Databasepages',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Database pages',
                    },
                    Targetpages: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Target pages',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'Targetpages',
                    },
                    LCK_M_IS: {
                        metric: 'LCK_M_IS',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        label: 'LCK_M_IS',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    Checkpointpagessec: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'Checkpointpagessec',
                        label: 'Checkpoint pages/sec',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    LCK_M_IX: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'LCK_M_IX',
                        metric: 'LCK_M_IX',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    Logoutssec: {
                        label: 'Logouts/s',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        metric: 'Logoutssec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                    },
                    PAGEIOLATCH_DT: {
                        metric: 'PAGEIOLATCH_DT',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'PAGEIOLATCH_DT',
                    },
                    PAGELATCH_EX: {
                        label: 'PAGELATCH_EX',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        metric: 'PAGELATCH_EX',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    CachedCursorsCounts: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Cached Cursors',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'CachedCursorsCounts',
                    },
                    LCK_M_RIn_S: {
                        metric: 'LCK_M_RIn_S',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        label: 'LCK_M_RIn_S',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    LCK_M_RX_X: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'LCK_M_RX_X',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LCK_M_RX_X',
                    },
                    LATCH_KP: {
                        label: 'LATCH_KP',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        metric: 'LATCH_KP',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    LATCH_EX: {
                        metric: 'LATCH_EX',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'LATCH_EX',
                    },
                    FreeMemory: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        label: 'Free Memory',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        metric: 'FreeMemory',
                    },
                    EXECSYNC: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'EXECSYNC',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'EXECSYNC',
                    },
                    TargetServerMemory: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        label: 'Target Server',
                        metric: 'TargetServerMemory',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    LATCH_NL: {
                        label: 'LATCH_NL',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LATCH_NL',
                    },
                    SOS_SCHEDULER_YIELD: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'SOS_SCHEDULER_YIELD',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'SOS_SCHEDULER_YIELD',
                    },
                    THREADPOOL: {
                        metric: 'THREADPOOL',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        label: 'THREADPOOL',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    PAGEIOLATCH_NL: {
                        metric: 'PAGEIOLATCH_NL',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        label: 'PAGEIOLATCH_NL',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    LATCH_UP: {
                        metric: 'LATCH_UP',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        label: 'LATCH_UP',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    Activecursors: {
                        metric: 'Activecursors',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Active cursors',
                    },
                    LATCH_DT: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LATCH_DT',
                        label: 'LATCH_DT',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    LCK_M_SCH_S: {
                        metric: 'LCK_M_SCH_S',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'LCK_M_SCH_S',
                    },
                    WRITELOG: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'WRITELOG',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'WRITELOG',
                    },
                    LCK_M_SIX: {
                        label: 'LCK_M_SIX',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        metric: 'LCK_M_SIX',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    LCK_M_X: {
                        metric: 'LCK_M_X',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'LCK_M_X',
                    },
                    LCK_M_RX_U: {
                        metric: 'LCK_M_RX_U',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        label: 'LCK_M_RX_U',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    LCK_M_SIU: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LCK_M_SIU',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        label: 'LCK_M_SIU',
                    },
                    Lazywritessec: {
                        metric: 'Lazywritessec',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Lazy writes/s',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    LCK_M_S: {
                        label: 'LCK_M_S',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LCK_M_S',
                    },
                    CXPACKET: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'CXPACKET',
                        label: 'CXPACKET',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    PAGEIOLATCH_KP: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'PAGEIOLATCH_KP',
                        label: 'PAGEIOLATCH_KP',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    LCK_M_RIn_NL: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LCK_M_RIn_NL',
                        label: 'LCK_M_RIn_NL',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    LCK_M_IU: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        metric: 'LCK_M_IU',
                        label: 'LCK_M_IU',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                    PAGELATCH_DT: {
                        label: 'PAGELATCH_DT',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                        metric: 'PAGELATCH_DT',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                    },
                    RS_SM_QUERY_CMPL: {
                        metric: 'RS_SM_QUERY_CMPL',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
                        label: 'RS_SM_QUERY_CMPL',
                        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
                    },
                },
            },
        },
    },
    timescaledb: {
        label: 'TimescaleDB',
        order: 5,
        subsystems: {
            db: {
                metrics: {
                    buffersclean: {
                        label: 'Clean',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'buffersclean',
                    },
                    checkpointsreq: {
                        metric: 'checkpointsreq',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        label: 'Requested',
                    },
                    bufferscheckpoint: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        label: 'Checkpoint',
                        metric: 'bufferscheckpoint',
                    },
                    buffersbackend: {
                        metric: 'buffersbackend',
                        label: 'Backend',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    checkpointstimed: {
                        label: 'Scheduled',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        metric: 'checkpointstimed',
                    },
                    size: {
                        label: 'Size',
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'size',
                    },
                    numbackends: {
                        label: 'All connections',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        metric: 'numbackends',
                    },
                    active: {
                        label: 'Active',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        metric: 'active',
                    },
                    idle: {
                        metric: 'idle',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        label: 'Idle',
                    },
                    idleintransaction: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        label: 'Idle in transaction',
                        metric: 'idleintransaction',
                    },
                    hpage: {
                        label: 'Page',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'hpage',
                    },
                    transactionid: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Transactionid',
                        metric: 'transactionid',
                    },
                    tuple: {
                        label: 'Tuple',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'tuple',
                    },
                    bufferio: {
                        metric: 'bufferio',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Bufferio',
                    },
                    replicationslotio: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Replication slotio',
                        metric: 'replicationslotio',
                    },
                    tempfiles: {
                        label: 'Files',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'tempfiles',
                    },
                    aextend: {
                        label: 'Extend',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'aextend',
                    },
                    object: {
                        metric: 'object',
                        label: 'Object',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    multixactmember: {
                        label: 'Multixact member',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'multixactmember',
                    },
                    hrelation: {
                        metric: 'hrelation',
                        label: 'Relation',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    speculativetoken: {
                        metric: 'speculativetoken',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Speculative token',
                    },
                    walinsert: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Walinsert',
                        metric: 'walinsert',
                    },
                    async: {
                        label: 'Async',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'async',
                    },
                    subtrans: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Subtrans',
                        metric: 'subtrans',
                    },
                    commitratio: {
                        metric: 'commitratio',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        label: 'Commit ratio',
                    },
                    atuple: {
                        label: 'Tuple',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.rows,
                        unit: subsystemMetricsResponse_constants_1.UNIT.rows,
                        metric: 'atuple',
                    },
                    htuple: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Tuple',
                        metric: 'htuple',
                    },
                    buffermapping: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Buffermapping',
                        metric: 'buffermapping',
                    },
                    virtualxid: {
                        metric: 'virtualxid',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Virtualxid',
                    },
                    cursors: {
                        label: 'Cursors',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'cursors',
                    },
                    deadlocks: {
                        label: 'Deadlocks',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'deadlocks',
                    },
                    deadlocks_max: {
                        label: 'Deadlocks',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'deadlocks_max',
                    },
                    currenttransactions: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Currenttransactions',
                        metric: 'currenttransactions',
                    },
                    hobject: {
                        metric: 'hobject',
                        label: 'Object',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    advisory: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Advisory',
                        metric: 'advisory',
                    },
                    RelationMappingLock: {
                        metric: 'RelationMappingLock',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'RelationMappingLock',
                    },
                    fastpathfunctioncall: {
                        metric: 'fastpathfunctioncall',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        label: 'Fastpath function call',
                    },
                    blkshit: {
                        metric: 'blkshit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Hit',
                    },
                    multixactoffset: {
                        label: 'Multixact offset',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'multixactoffset',
                    },
                    userlock: {
                        metric: 'userlock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Userlock',
                    },
                    auserlock: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Userlock',
                        metric: 'auserlock',
                    },
                    cachehitratio: {
                        metric: 'cachehitratio',
                        label: 'Cache hit ratio',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                    },
                    disabled: {
                        label: 'Disabled',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'disabled',
                    },
                    frozenid: {
                        label: 'Frozenid',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'frozenid',
                    },
                    buffercontent: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Buffer content',
                        metric: 'buffercontent',
                    },
                    relation: {
                        metric: 'relation',
                        label: 'Relation',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    aobject: {
                        metric: 'aobject',
                        label: 'Object',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    xactrollback: {
                        metric: 'xactrollback',
                        label: 'Rollbacks',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    committimestamp: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Commit timestamp',
                        metric: 'committimestamp',
                    },
                    hextend: {
                        metric: 'hextend',
                        label: 'Extend',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                    },
                    xactcommit: {
                        metric: 'xactcommit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        label: 'Commits',
                    },
                    proc: {
                        metric: 'proc',
                        label: 'Proc',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    idleintransactionaborted: {
                        metric: 'idleintransactionaborted',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
                        label: 'Idle in transaction aborted',
                    },
                    huserlock: {
                        label: 'Userlock',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'huserlock',
                    },
                    replicationorigin: {
                        metric: 'replicationorigin',
                        label: 'Replication origin',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    tempbytes: {
                        label: 'Bytes',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        metric: 'tempbytes',
                    },
                    arelation: {
                        label: 'Relation',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'arelation',
                    },
                    clog: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Clog',
                        metric: 'clog',
                    },
                    apage: {
                        metric: 'apage',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Page',
                    },
                    extend: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Extend',
                        metric: 'extend',
                    },
                    predicatelockmanager: {
                        label: 'Predicate lock manager',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        metric: 'predicatelockmanager',
                    },
                    lockmanager: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Lock manager',
                        metric: 'lockmanager',
                    },
                    oldserxid: {
                        label: 'Oldserxid',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        metric: 'oldserxid',
                    },
                    Activity: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Activity',
                        metric: 'Activity',
                    },
                    BufferPin: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'BufferPin',
                        metric: 'BufferPin',
                    },
                    Client: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Client',
                        metric: 'Client',
                    },
                    Extension: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Extension',
                        metric: 'Extension',
                    },
                    IO: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'IO',
                        metric: 'IO',
                    },
                    IPC: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'IPC',
                        metric: 'IPC',
                    },
                    Lock: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Lock',
                        metric: 'Lock',
                    },
                    LWLock: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'LWLock',
                        metric: 'LWLock',
                    },
                    Timeout: {
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        label: 'Timeout',
                        metric: 'Timeout',
                    },
                }
            },
            schema: {
                metrics: {
                    size: {
                        label: 'Size',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        metric: 'size',
                    },
                },
            },
            hw_type: {
                metrics: {
                    size: {
                        label: 'Size',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        metric: 'size',
                    },
                },
            },
            subsystem: {
                metrics: {
                    size: {
                        label: 'Size',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        metric: 'size',
                    },
                },
            },
            table: {
                metrics: {
                    size: {
                        label: 'Size',
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        metric: 'size',
                    },
                },
            },
        },
    },
};
//# sourceMappingURL=Database.js.map