"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var Microservice_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Microservice = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const dataSource_1 = require("../../config/dataSource");
const microservice_entity_1 = require("../../entity/microservice.entity");
const Format_1 = require("../log/Format");
const winston = require("winston");
const App_1 = require("../configuration/App");
const PM2_1 = require("../configuration/PM2");
let Microservice = Microservice_1 = class Microservice {
    constructor() {
    }
    async init(microservice) {
        this.microservicesRepository = (0, dataSource_1.getDataSource)().getRepository(microservice_entity_1.Microservice);
        this.microservice = microservice;
        let service = await this.getMicroserviceDB(microservice);
        let linux_port = null;
        if (service) {
            if (this.microservice === "linux")
                linux_port = await App_1.App.checkGlobalMsVariables("PORT", "linux");
            this.registered = true;
            this.script = service.script;
            this.host = service.host;
            this.hw_type = service.hw_type;
            this.label = service.label;
            this.port = this.microservice !== 'linux' && process.env['MS_PORT_PREFIX']
                ? (+process.env['MS_PORT_PREFIX'] * 1000) + (+service.port.substring(1))
                : this.microservice === "linux"
                    ? linux_port
                        ? +linux_port
                        : +service.port
                    : +service.port;
            this.needConf = service.needConf;
            let servicePM2 = await this.getMicroservicePM2(microservice);
            if (servicePM2[0]) {
                this.available = true;
                this.active = servicePM2[0].pm2_env.status === 'online' ? true : false;
                this.pid = servicePM2[0].pid;
                this.version = servicePM2[0].pm2_env.version;
                this.monitor = {
                    memory_byte: servicePM2[0].monit.memory,
                    cpu_percent: servicePM2[0].monit.cpu,
                };
            }
            else {
                this.available, this.active = false;
            }
        }
        else {
            this.registered, this.available, this.active = false;
        }
    }
    isRegistered() {
        return this.registered;
    }
    isActive() {
        return this.active;
    }
    isAvailable() {
        return this.available;
    }
    needsConf() {
        return this.needConf;
    }
    getData() {
        return {
            label: this.label,
            microservice: this.microservice,
            host: this.host,
            port: this.port,
            script: this.script,
            pid: this.pid ? this.pid : null,
            version: this.version,
            active: this.active,
            monitor: this.monitor,
        };
    }
    async reloadData() {
        let servicePM2 = await this.getMicroservicePM2(this.microservice);
        this.available = true;
        this.active = servicePM2[0].pm2_env.status === 'online' ? true : false;
        this.pid = servicePM2[0].pid;
        this.version = servicePM2[0].pm2_env.version;
        this.monitor = {
            memory_byte: servicePM2[0].monit.memory,
            cpu_percent: servicePM2[0].monit.cpu,
        };
    }
    async getMicroserviceDB(microservice) {
        return this.microservicesRepository.findOneBy({ microservice: microservice });
    }
    async getMicroservicePM2(name) {
        let pm2 = require('pm2');
        return new Promise((resolve, reject) => {
            pm2.describe(name, (err, proc) => {
                if (err) {
                    reject(err);
                }
                resolve(proc);
            });
        });
    }
    async start() {
        try {
            if (!this.isAvailable()) {
                let data = JSON.parse('{ "name": "' +
                    this.microservice +
                    '", "script": "./microservices/' +
                    this.microservice +
                    '/' +
                    this.script +
                    '", "cwd": "./microservices/' +
                    this.microservice +
                    '/", "error_file": "' + App_1.App.LOGS_PATH + '/' +
                    this.microservice +
                    '-error.log", "out_file": "' + App_1.App.LOGS_PATH + '/' +
                    this.microservice +
                    '.log", "log_date_format": "YYYY-MM-DD HH:mm:ss" }');
                await this.register(data);
            }
            else {
                await PM2_1.PM2.startService(this.microservice);
                await this.reloadData();
            }
            if (this.needsConf()) {
                require('dotenv').config();
                const jwt = require('jsonwebtoken');
                setTimeout(async () => {
                    let token = await jwt.sign({ microservice: this.microservice }, App_1.App.getSecret());
                    let message = {
                        jwt: token,
                    };
                    let received;
                    try {
                        received = await this.sendData(JSON.stringify(message));
                    }
                    catch (error) {
                        console.error('[' + this.microservice + '] unable to send configuration');
                    }
                }, 3000);
            }
        }
        catch (e) {
            console.error("[" + this.microservice + "] MicroserviceStartError: " + e);
        }
    }
    async changeOnStartup(onStartup) {
        const microservice = await this.microservicesRepository.findOneBy({ microservice: this.microservice });
        microservice.onStartup = onStartup;
        await this.microservicesRepository.save(microservice);
    }
    async stop() {
        await PM2_1.PM2.stopService(this.microservice);
        await this.reloadData();
    }
    async register(data) {
        try {
            await this.registerService(data);
            await this.reloadData();
        }
        catch (e) {
            console.error("[" + this.microservice + "] MicroserviceRegisterError: " + e);
        }
    }
    async registerService(data) {
        let pm2 = require('pm2');
        return new Promise((resolve, reject) => {
            pm2.start(data, (err, proc) => {
                if (err) {
                    reject(err);
                }
                resolve(proc);
            });
        });
    }
    async sendData(data) {
        const winstonLogger = winston.createLogger({
            format: winston.format.combine(winston.format.timestamp(), Format_1.logFormat),
            transports: [
                new winston.transports.File({ filename: 'logs/microservices-error.log', level: 'error' }),
                new winston.transports.File({ filename: 'logs/microservices.log' }),
            ],
        });
        return new Promise((resolve, reject) => {
            const net = require('net');
            let client = new net.Socket();
            client.connect(this.port, this.host, () => {
                client.write(data);
                winstonLogger.log('info', '[' + this.microservice + '] Connected');
            });
            client.on('data', (data) => {
                winstonLogger.log('info', '[' + this.microservice + '] Received: ' + data);
                resolve(data);
                client.destroy();
            });
            client.on('close', () => {
                winstonLogger.end();
                client.destroy();
            });
            client.on('error', (error) => {
                winstonLogger.log('error', '[' + this.microservice + '] Error: ' + error);
                winstonLogger.end();
                reject();
            });
        });
    }
    static async checkAndStart(ms) {
        console.log("[CheckMS]: checking " + ms);
        const microservice = new Microservice_1();
        await microservice.init(ms);
        if (microservice.isRegistered()) {
            if (!microservice.isActive()) {
                console.log("[CheckMS]: starting " + ms);
                await microservice.start();
                await microservice.changeOnStartup(true);
            }
            else {
                console.log("[CheckMS]: " + ms + " already started");
            }
        }
        else {
            console.error("[CheckMS]: " + ms + " is not registered!");
        }
    }
    static getCoreServices() {
        let coreServices = ['analyzer'];
        let pm2_json = require('../../../production.config.js');
        for (let i = 0; i < pm2_json.apps.length; i++) {
            coreServices.push(pm2_json.apps[i].name);
        }
        return coreServices;
    }
    static list() {
        return [
            {
                label: 'OS Agent',
                microservice: 'linux',
                script: 'linux-service.pl',
                host: '127.0.0.1',
                port: '8162',
                hw_type: 'linux',
                needConf: true,
            },
            {
                label: 'Kubernetes',
                microservice: 'kubernetes',
                script: 'kubernetes-service.pl',
                host: '127.0.0.1',
                port: '3601',
                hw_type: 'kubernetes',
                needConf: false,
            },
            {
                label: 'Openshift',
                microservice: 'openshift',
                script: 'openshift-service.pl',
                host: '127.0.0.1',
                port: '3602',
                hw_type: 'openshift',
                needConf: false,
            },
            {
                label: 'Azure',
                microservice: 'azure',
                script: 'azure-service.pl',
                host: '127.0.0.1',
                port: '3102',
                hw_type: 'azure',
                needConf: false,
            },
            {
                label: 'Proxmox',
                microservice: 'proxmox',
                script: 'proxmox-service.pl',
                host: '127.0.0.1',
                port: '3103',
                hw_type: 'proxmox',
                needConf: false,
            },
            {
                label: 'CloudStack',
                microservice: 'cloudstack',
                script: 'cloudstack-service.pl',
                host: '127.0.0.1',
                port: '3104',
                hw_type: 'cloudstack',
                needConf: false,
            },
            {
                label: 'Nutanix',
                microservice: 'nutanix',
                script: 'nutanix-service.pl',
                host: '127.0.0.1',
                port: '3105',
                hw_type: 'nutanix',
                needConf: false,
            },
            {
                label: 'IBM Power',
                microservice: 'power',
                script: 'power-service.pl',
                host: '127.0.0.1',
                port: '3106',
                hw_type: 'power',
                needConf: false,
            },
            {
                label: 'VMware',
                microservice: 'vmware',
                script: 'vmware-service.pl',
                host: '127.0.0.1',
                port: '3107',
                hw_type: 'vmware',
                needConf: false,
            },
            {
                label: 'E-series',
                microservice: 'eseries',
                script: 'eseries-service.pl',
                host: '127.0.0.1',
                port: '3201',
                hw_type: 'eseries',
                needConf: false,
            },
            {
                label: 'Storwize',
                microservice: 'swiz',
                script: 'swiz-service.pl',
                host: '127.0.0.1',
                port: '3202',
                hw_type: 'swiz',
                needConf: false,
            },
            {
                label: 'VMAX',
                microservice: 'vmax',
                script: 'vmax-service.pl',
                host: '127.0.0.1',
                port: '3216',
                hw_type: 'vmax',
                needConf: false,
            },
            {
                label: 'XIV',
                microservice: 'xiv',
                script: 'xiv-service.pl',
                host: '127.0.0.1',
                port: '3222',
                hw_type: 'xiv',
                needConf: false,
            },
            {
                label: 'XtremIO',
                microservice: 'xtremio',
                script: 'xtremio-service.pl',
                host: '127.0.0.1',
                port: '3223',
                hw_type: 'xtremio',
                needConf: false,
            },
            {
                label: '3PAR',
                microservice: '3par',
                script: '3par-service.pl',
                host: '127.0.0.1',
                port: '3203',
                hw_type: '3par',
                needConf: false,
            },
            {
                label: 'Isilon',
                microservice: 'isilon',
                script: 'isilon-service.pl',
                host: '127.0.0.1',
                port: '3204',
                hw_type: 'isilon',
                needConf: false,
            },
            {
                label: 'OracleZFS',
                microservice: 'oraclezfs',
                script: 'oraclezfs-service.pl',
                host: '127.0.0.1',
                port: '3208',
                hw_type: 'oraclezfs',
                needConf: false,
            },
            {
                label: 'Dothill',
                microservice: 'dothill',
                script: 'dothill-service.pl',
                host: '127.0.0.1',
                port: '3209',
                hw_type: 'dothill',
                needConf: false,
            },
            {
                label: 'Compellent',
                microservice: 'compellent',
                script: 'compellent-service.pl',
                host: '127.0.0.1',
                port: '3210',
                hw_type: 'compellent',
                needConf: false,
            },
            {
                label: 'PowerFlex',
                microservice: 'powerflex',
                script: 'powerflex-service.pl',
                host: '127.0.0.1',
                port: '3211',
                hw_type: 'powerflex',
                needConf: false,
            },
            {
                label: 'Rubrik',
                microservice: 'rubrik',
                script: 'rubrik-service.pl',
                host: '127.0.0.1',
                port: '3213',
                hw_type: 'rubrik',
                needConf: false,
            },
            {
                label: 'Cohesity',
                microservice: 'cohesity',
                script: 'cohesity-service.pl',
                host: '127.0.0.1',
                port: '3219',
                hw_type: 'cohesity',
                needConf: false,
            },
            {
                label: 'StoreOnce',
                microservice: 'storeonce',
                script: 'storeonce-service.pl',
                host: '127.0.0.1',
                port: '3206',
                hw_type: 'storeonce',
                needConf: false,
            },
            {
                label: 'PostgreSQL',
                microservice: 'postgres',
                script: 'postgres-service.pl',
                host: '127.0.0.1',
                port: '3301',
                hw_type: 'postgres',
                needConf: false,
            },
            {
                label: 'SQL Server',
                microservice: 'sqlserver',
                script: 'sqlserver-service.pl',
                host: '127.0.0.1',
                port: '3302',
                hw_type: 'sqlserver',
                needConf: false,
            },
            {
                label: 'Oracle DB',
                microservice: 'oracledb',
                script: 'oracledb-service.pl',
                host: '127.0.0.1',
                port: '3303',
                hw_type: 'oracledb',
                needConf: false,
            },
            {
                label: 'SAN Brocade',
                microservice: 'sanbrcd',
                script: 'sanbrcd-service.pl',
                host: '127.0.0.1',
                port: '3401',
                hw_type: 'sanbrcd',
                needConf: false,
            },
            {
                label: 'SAN Cisco',
                microservice: 'sancisco',
                script: 'sancisco-service.pl',
                host: '127.0.0.1',
                port: '3402',
                hw_type: 'sancisco',
                needConf: false,
            },
            {
                label: 'SANnav',
                microservice: 'sannav',
                script: 'sannav-service.pl',
                host: '127.0.0.1',
                port: '3406',
                hw_type: 'sannav',
                needConf: false,
            },
            {
                label: 'LAN Cisco',
                microservice: 'lancisco',
                script: 'lancisco-service.pl',
                host: '127.0.0.1',
                port: '3404',
                hw_type: 'lancisco',
                needConf: false,
            },
            {
                label: 'LAN Others',
                microservice: 'lanothers',
                script: 'lanothers-service.pl',
                host: '127.0.0.1',
                port: '3403',
                hw_type: 'lanothers',
                needConf: false,
            },
            {
                label: 'IBM COS',
                microservice: 'ibmcoss',
                script: 'ibmcoss-service.pl',
                host: '127.0.0.1',
                port: '3214',
                hw_type: 'ibmcoss',
                needConf: false,
            },
            {
                label: 'QNAP',
                microservice: 'qnap',
                script: 'qnap-service.pl',
                host: '127.0.0.1',
                port: '3205',
                hw_type: 'qnap',
                needConf: false,
            },
            {
                label: 'DataDomain',
                microservice: 'datadomain',
                script: 'datadomain-service.pl',
                host: '127.0.0.1',
                port: '3212',
                hw_type: 'datadomain',
                needConf: false,
            },
            {
                label: 'AWS',
                microservice: 'aws',
                script: 'aws-service.pl',
                host: '127.0.0.1',
                port: '3501',
                hw_type: 'aws',
                needConf: false,
            },
            {
                label: 'Nimble',
                microservice: 'nimble',
                script: 'nimble-service.pl',
                host: '127.0.0.1',
                port: '3215',
                hw_type: 'nimble',
                needConf: false,
            },
            {
                label: 'StorageGRID',
                microservice: 'storagegrid',
                script: 'storagegrid-service.pl',
                host: '127.0.0.1',
                port: '3217',
                hw_type: 'storagegrid',
                needConf: false,
            },
            {
                label: 'Infortrend',
                microservice: 'infortrend',
                script: 'infortrend-service.pl',
                host: '127.0.0.1',
                port: '3218',
                hw_type: 'infortrend',
                needConf: false,
            },
            {
                label: 'VSPG',
                microservice: 'vspg',
                script: 'vspg-service.pl',
                host: '127.0.0.1',
                port: '3220',
                hw_type: 'vspg',
                needConf: false,
            },
            {
                label: 'Pure',
                microservice: 'pure',
                script: 'pure-service.pl',
                host: '127.0.0.1',
                port: '3221',
                hw_type: 'pure',
                needConf: false,
            },
            {
                label: 'Db2',
                microservice: 'db2',
                script: 'db2-service.pl',
                host: '127.0.0.1',
                port: '3304',
                hw_type: 'db2',
                needConf: false,
            },
            {
                label: 'NetApp',
                microservice: 'netapp',
                script: 'netapp-service.pl',
                host: '127.0.0.1',
                port: '3207',
                hw_type: 'netapp',
                needConf: false,
            },
            {
                label: 'InfiniBox',
                microservice: 'infinibox',
                script: 'infinibox-service.pl',
                host: '127.0.0.1',
                port: '3224',
                hw_type: 'infinibox',
                needConf: false,
            },
            {
                label: 'OceanStor',
                microservice: 'oceanstor',
                script: 'oceanstor-service.pl',
                host: '127.0.0.1',
                port: '3227',
                hw_type: 'oceanstor',
                needConf: false,
            },
            {
                label: 'PowerStore',
                microservice: 'powerstore',
                script: 'powerstore-service.pl',
                host: '127.0.0.1',
                port: '3226',
                hw_type: 'powerstore',
                needConf: false,
            },
            {
                label: 'Google Cloud',
                microservice: 'gcloud',
                script: 'gcloud-service.pl',
                host: '127.0.0.1',
                port: '3502',
                hw_type: 'gcloud',
                needConf: false,
            },
            {
                label: 'ECS',
                microservice: 'ecs',
                script: 'ecs-service.pl',
                host: '127.0.0.1',
                port: '3225',
                hw_type: 'ecs',
                needConf: false,
            },
            {
                label: 'OracleVM',
                microservice: 'oraclevm',
                script: 'oraclevm-service.pl',
                host: '127.0.0.1',
                port: '3108',
                hw_type: 'oraclevm',
                needConf: false,
            },
            {
                label: 'oVirt',
                microservice: 'ovirt',
                script: 'ovirt-service.pl',
                host: '127.0.0.1',
                port: '3110',
                hw_type: 'ovirt',
                needConf: false,
            },
            {
                label: 'HCP',
                microservice: 'hcp',
                script: 'hcp-service.pl',
                host: '127.0.0.1',
                port: '3230',
                hw_type: 'hcp',
                needConf: false,
            },
            {
                label: 'DDNIntelliFlash',
                microservice: 'ddnintelliflash',
                script: 'ddnintelliflash-service.pl',
                host: '127.0.0.1',
                port: '3229',
                hw_type: 'ddnintelliflash',
                needConf: false,
            },
            {
                label: 'Datacore',
                microservice: 'datacore',
                script: 'datacore-service.pl',
                host: '127.0.0.1',
                port: '3231',
                hw_type: 'datacore',
                needConf: false,
            },
            {
                label: 'OceanStorPacific',
                microservice: 'oceanstorpacific',
                script: 'oceanstorpacific-service.pl',
                host: '127.0.0.1',
                port: '3228',
                hw_type: 'oceanstorpacific',
                needConf: false,
            },
            {
                label: 'OceanStorPacificNAS',
                microservice: 'oceanstorpacificnas',
                script: 'oceanstorpacificnas-service.pl',
                host: '127.0.0.1',
                port: '3233',
                hw_type: 'oceanstorpacificnas',
                needConf: false,
            },
            {
                label: 'OceanStorNAS',
                microservice: 'oceanstornas',
                script: 'oceanstornas-service.pl',
                host: '127.0.0.1',
                port: '3234',
                hw_type: 'oceanstornas',
                needConf: false,
            },
            {
                label: 'Synology',
                microservice: 'synology',
                script: 'synology-service.pl',
                host: '127.0.0.1',
                port: '3235',
                hw_type: 'synology',
                needConf: false,
            },
            {
                label: 'Unity',
                microservice: 'unity',
                script: 'unity-service.pl',
                host: '127.0.0.1',
                port: '3232',
                hw_type: 'unity',
                needConf: false,
            }
        ];
    }
    static jobs() {
        return [
            {
                microservice: 'nutanix',
                job: 'fetch',
                interval: 20,
            },
            {
                microservice: 'proxmox',
                job: 'fetch',
                interval: 20,
            },
            {
                microservice: 'power',
                job: 'perf',
                interval: 20,
            },
            {
                microservice: 'power',
                job: 'conf',
                interval: 60,
            },
            {
                microservice: 'power',
                job: 'cmc',
                interval: 20,
            },
            {
                microservice: 'vmware',
                job: 'fetch',
                interval: 20,
            },
            {
                microservice: '3par',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'eseries',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'isilon',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'oraclezfs',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'dothill',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'compellent',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'postgres',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'sqlserver',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'powerflex',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'swiz',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'vmax',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'xiv',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'xtremio',
                job: 'fetch',
                interval: 60,
            },
            {
                microservice: 'xtremio',
                job: 'conf',
                interval: 360,
            },
            {
                microservice: 'storeonce',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'oracledb',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'rubrik',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'cohesity',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'sanbrcd',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'synology',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'sancisco',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'sannav',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'lancisco',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'lanothers',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'ibmcoss',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'datadomain',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'qnap',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'aws',
                job: 'fetch',
                interval: 20,
            },
            {
                microservice: 'nimble',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'kubernetes',
                job: 'fetch',
                interval: 20,
            },
            {
                microservice: 'openshift',
                job: 'fetch',
                interval: 20,
            },
            {
                microservice: 'storagegrid',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'infortrend',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'vspg',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'pure',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'db2',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'netapp',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'infinibox',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'oceanstor',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'powerstore',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'gcloud',
                job: 'fetch',
                interval: 20,
            },
            {
                microservice: 'ecs',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'oraclevm',
                job: 'fetch',
                interval: 20,
            },
            {
                microservice: 'ovirt',
                job: 'fetch',
                interval: 20,
            },
            {
                microservice: 'datacore',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'azure',
                job: 'fetch',
                interval: 20,
            }, {
                microservice: 'ddnintelliflash',
                job: 'fetch',
                interval: 5,
            }, {
                microservice: 'hcp',
                job: 'fetch',
                interval: 5,
            }, {
                microservice: 'hcp',
                job: 'perf',
                interval: 1,
            },
            {
                microservice: 'unity',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'oceanstorpacific',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'oceanstorpacificnas',
                job: 'fetch',
                interval: 5,
            },
            {
                microservice: 'oceanstornas',
                job: 'fetch',
                interval: 5,
            },
        ];
    }
};
exports.Microservice = Microservice;
exports.Microservice = Microservice = Microservice_1 = __decorate([
    (0, common_1.Module)({
        imports: [typeorm_1.TypeOrmModule.forFeature([microservice_entity_1.Microservice])],
    }),
    __metadata("design:paramtypes", [])
], Microservice);
//# sourceMappingURL=Microservice.js.map