"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExporterPublicController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const exporter_constants_1 = require("../../../common/constants/exporter.constants");
const exporter_dto_1 = require("../../../common/exporter.dto");
const joiValidation_pipe_1 = require("../../../pipe/joiValidation.pipe");
const Exporter_1 = require("../../exporter/Exporter");
const ExporterFormatter_1 = require("../../exporter/ExporterFormatter");
const exporter_service_1 = require("../../exporter/exporter.service");
const Joi = require("joi");
const ApiKey_1 = require("../ApiKey");
const ItemGroupWrapper_1 = require("../../custom_group/ItemGroupWrapper");
const CustomGroupWrapper_1 = require("../../custom_group/CustomGroupWrapper");
const License_1 = require("../../../config/License");
const xormon_exception_1 = require("../../../exception/xormon.exception");
const dataSource_1 = require("../../../config/dataSource");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const typeorm_1 = require("typeorm");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
let ExporterPublicController = class ExporterPublicController {
    constructor(exporterService) {
        this.exporterService = exporterService;
    }
    async exports(request) {
        const user = await ApiKey_1.ApiKey.getUserByRequest(request);
        if (!user) {
            throw new common_1.UnauthorizedException();
        }
        const exports = await this.exporterService.getExports(user.user_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: exports.map((hist) => {
                return {
                    export_id: hist.export_id,
                    label: hist.label,
                    format: hist.format,
                    type: hist.type,
                    last_run: hist.last_run,
                    recurrence: hist.recurrence,
                    created: hist.created.toISOString(),
                };
            }),
        };
    }
    async export(request, res, export_id) {
        const user = await ApiKey_1.ApiKey.getUserByRequest(request);
        if (!user) {
            throw new common_1.UnauthorizedException();
        }
        if (!this.exporterService.checkExportOwner(export_id, user.user_id)) {
            throw new common_1.ForbiddenException();
        }
        const detail = await this.exporterService.getExport(export_id);
        if (!detail) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Export not found',
                error: 'Export not found',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        const technology = detail.hw_type
            ? detail.hw_type
            : detail.class
                ? detail.class
                : undefined;
        if (technology && !(await License_1.License.checkTechnology(technology, true))) {
            throw new xormon_exception_1.XormonException("You do not have an active API license for the given technology", common_1.HttpStatus.PAYMENT_REQUIRED);
        }
        let items = undefined;
        if (detail.item_group_id) {
            items = await ItemGroupWrapper_1.ItemGroupWrapper.getItems(detail.item_group_id, 'export');
        }
        else if (detail.custom_group_id) {
            items = await CustomGroupWrapper_1.CustomGroupWrapper.getItems(detail.custom_group_id);
        }
        switch (detail.type) {
            case exporter_constants_1.EXPORT_TYPE.timeseries: {
                const data = detail.data;
                if (detail.recurrence) {
                    const interval = Exporter_1.Exporter.getTimeseriesInterval(data, detail.recurrence.freq);
                    if (interval.start && interval.end) {
                        data.start = interval.start;
                        data.end = interval.end;
                    }
                }
                await this.exporterService.saveUsedTime(export_id);
                return await this.timeseries(res, {
                    ...detail.data,
                    format: detail.format,
                    uuids: items ? items.map(item => item.item_id) : []
                });
            }
            case exporter_constants_1.EXPORT_TYPE.mapping: {
                await this.exporterService.saveUsedTime(export_id);
                return await this.mapping(request, res, {
                    ...detail,
                    uuids: items ? items.map(item => item.item_id) : undefined
                });
            }
            case exporter_constants_1.EXPORT_TYPE.configuration: {
                await this.exporterService.saveUsedTime(export_id);
                return await this.configuration(request, res, {
                    properties: detail.data.properties,
                    uuids: items ? items.map(item => item.item_id) : undefined
                });
            }
        }
        throw new common_1.HttpException({
            statusCode: common_1.HttpStatus.BAD_REQUEST,
            message: 'bad request',
            error: 'bad request',
        }, common_1.HttpStatus.BAD_REQUEST);
    }
    async timeseries(res, request) {
        if (!Exporter_1.Exporter.isValid(request)) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'filter is missing',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        const items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                item_id: (0, typeorm_1.In)(request.uuids)
            }, relations: {
                hw_type_entity: true
            }
        });
        let technologies = [];
        for (const item of items) {
            let technology = ['san', 'storage'].includes(item.hw_type_entity.class)
                ? item.hw_type_entity.class
                : item.hw_type;
            if (!technologies.includes(technology)) {
                technologies.push(technology);
            }
        }
        for (const technology of technologies) {
            if (!(await License_1.License.checkTechnology(technology, true))) {
                throw new xormon_exception_1.XormonException("You do not have an active API license for the given technology", common_1.HttpStatus.PAYMENT_REQUIRED);
            }
        }
        const exportData = await Exporter_1.Exporter.timeseries(request);
        if (!exportData || exportData.items.length === 0) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'no items found',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        const formatter = new ExporterFormatter_1.ExporterFormatter(exportData);
        formatter.setStartEnd({ start: request.start, end: request.end });
        return await formatter.exportTimeseriesToResponse(request.format ? request.format : 'json', res);
    }
    async configuration(request, res, exportRequest) {
        const items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                item_id: (0, typeorm_1.In)(exportRequest.uuids)
            }, relations: {
                hw_type_entity: true
            }
        });
        let technologies = [];
        for (const item of items) {
            let technology = ['san', 'storage'].includes(item.hw_type_entity.class)
                ? item.hw_type_entity.class
                : item.hw_type;
            if (!technologies.includes(technology)) {
                technologies.push(technology);
            }
        }
        for (const technology of technologies) {
            if (!(await License_1.License.checkTechnology(technology, true))) {
                throw new xormon_exception_1.XormonException("You do not have an active API license for the given technology", common_1.HttpStatus.PAYMENT_REQUIRED);
            }
        }
        const data = await Exporter_1.Exporter.configuration(exportRequest, request, exportRequest.format, res);
        if (exportRequest.format !== 'xlsx') {
            res.send(data);
        }
    }
    async mapping(request, res, exportRequest) {
        const items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                item_id: (0, typeorm_1.In)(exportRequest.uuids)
            }, relations: {
                hw_type_entity: true
            }
        });
        let technologies = [];
        for (const item of items) {
            let technology = ['san', 'storage'].includes(item.hw_type_entity.class)
                ? item.hw_type_entity.class
                : item.hw_type;
            if (!technologies.includes(technology)) {
                technologies.push(technology);
            }
        }
        for (const technology of technologies) {
            if (!(await License_1.License.checkTechnology(technology, true))) {
                throw new xormon_exception_1.XormonException("You do not have an active API license for the given technology", common_1.HttpStatus.PAYMENT_REQUIRED);
            }
        }
        const data = await Exporter_1.Exporter.mapping(exportRequest, request, exportRequest.format, res);
        if (exportRequest.format !== 'xlsx') {
            res.send(data);
        }
    }
    async capacity(request, res, exportRequest) {
        const items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                item_id: (0, typeorm_1.In)(exportRequest.uuids)
            }, relations: {
                hw_type_entity: true
            }
        });
        let technologies = [];
        for (const item of items) {
            let technology = ['san', 'storage'].includes(item.hw_type_entity.class)
                ? item.hw_type_entity.class
                : item.hw_type;
            if (!technologies.includes(technology)) {
                technologies.push(technology);
            }
        }
        for (const technology of technologies) {
            if (!(await License_1.License.checkTechnology(technology, true))) {
                throw new xormon_exception_1.XormonException("You do not have an active API license for the given technology", common_1.HttpStatus.PAYMENT_REQUIRED);
            }
        }
        const data = await Exporter_1.Exporter.capacity(exportRequest, request, exportRequest.format, res);
        if (exportRequest.format !== 'xlsx') {
            res.send(data);
        }
    }
    async getConfigurationProperties(hwType, subsystem) {
        let props = [];
        const properties = await MetricsWrapper_1.MetricsWrapper.getConfigurationFindClass(hwType, subsystem);
        for (const propertyKey in properties) {
            props.push({
                property_name: propertyKey,
                label: properties[propertyKey] && properties[propertyKey].label ? properties[propertyKey].label : propertyKey
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: props
        };
    }
};
exports.ExporterPublicController = ExporterPublicController;
__decorate([
    (0, common_1.Get)('exports'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/exporter.dto").ExporterHistoryResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ExporterPublicController.prototype, "exports", null);
__decorate([
    openapi.ApiOperation({ description: "Run the export" }),
    (0, common_1.Get)('export/:export_id'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)()),
    __param(2, (0, common_1.Param)('export_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, String]),
    __metadata("design:returntype", Promise)
], ExporterPublicController.prototype, "export", null);
__decorate([
    openapi.ApiOperation({ description: "Export timeseries data" }),
    (0, common_1.Post)('timeseries'),
    (0, common_1.UsePipes)(new joiValidation_pipe_1.JoiValidationPipe(Joi.object({
        start: Joi.number().required(),
        end: Joi.number().required(),
        uuids: Joi.array().items(Joi.string(), Joi.number()).required(),
        metric: Joi.array().items(Joi.string()).required(),
        format: Joi.string().optional()
    }))),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Res)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, exporter_dto_1.PublicExportTimeseriesRequestDTO]),
    __metadata("design:returntype", Promise)
], ExporterPublicController.prototype, "timeseries", null);
__decorate([
    openapi.ApiOperation({ description: "Export configuration data" }),
    (0, common_1.Post)('configuration'),
    (0, common_1.UsePipes)(new joiValidation_pipe_1.JoiValidationPipe(Joi.object({
        uuids: Joi.array().items(Joi.string(), Joi.number()).required(),
        properties: Joi.array().items(Joi.string()),
        format: Joi.string().optional()
    }))),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)()),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, exporter_dto_1.PublicExportConfigurationRequestDTO]),
    __metadata("design:returntype", Promise)
], ExporterPublicController.prototype, "configuration", null);
__decorate([
    openapi.ApiOperation({ description: "Export mapping data" }),
    (0, common_1.Post)('mapping'),
    (0, common_1.UsePipes)(new joiValidation_pipe_1.JoiValidationPipe(Joi.object({
        uuids: Joi.array().items(Joi.string(), Joi.number()).required(),
        format: Joi.string().optional()
    }))),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)()),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, exporter_dto_1.PublicExportMappingRequestDTO]),
    __metadata("design:returntype", Promise)
], ExporterPublicController.prototype, "mapping", null);
__decorate([
    openapi.ApiOperation({ description: "Export capacity data" }),
    (0, common_1.Post)('capacity'),
    (0, common_1.UsePipes)(new joiValidation_pipe_1.JoiValidationPipe(Joi.object({
        uuids: Joi.array().items(Joi.string(), Joi.number()).required(),
        format: Joi.string().optional()
    }))),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)()),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, exporter_dto_1.PublicExportCapacityRequestDTO]),
    __metadata("design:returntype", Promise)
], ExporterPublicController.prototype, "capacity", null);
__decorate([
    (0, common_1.Get)('v1/props/:hw_type/:subsystem'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/architecture.dto").PropsResponseDTO }),
    __param(0, (0, common_1.Param)('hw_type')),
    __param(1, (0, common_1.Param)('subsystem')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], ExporterPublicController.prototype, "getConfigurationProperties", null);
exports.ExporterPublicController = ExporterPublicController = __decorate([
    (0, swagger_1.ApiTags)('PublicExporter'),
    (0, common_1.Controller)('api/public/v1/exporter'),
    __metadata("design:paramtypes", [exporter_service_1.ExporterService])
], ExporterPublicController);
//# sourceMappingURL=exporter.public.controller.js.map