"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var ReportTemplateService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReportTemplateService = void 0;
const common_1 = require("@nestjs/common");
const reportItem_constants_1 = require("../../common/constants/reportItem.constants");
let ReportTemplateService = ReportTemplateService_1 = class ReportTemplateService {
    async getReportTemplates() {
        return ReportTemplateService_1.reportTemplates;
    }
    async getReportTemplateById(reportTemplateId) {
        return ReportTemplateService_1.reportTemplates.find(reportTemplate => reportTemplate.report_template_id === reportTemplateId);
    }
    checkData(data, reportTemplate) {
        if (reportTemplate.report_template_id !== ReportTemplateService_1.OverviewReportTemplate.report_template_id) {
            throw new common_1.NotImplementedException(`checkData() for report template "${reportTemplate.name}" with id "${reportTemplate.report_template_id} has not been implemented yet.`);
        }
        if (!data) {
            return `Missing parameter "data" required for template "${reportTemplate.name}"`;
        }
        if (typeof data !== 'object') {
            return `Wrong format of parameter "data" (should be object)`;
        }
        for (const fieldToFill of reportTemplate.fields_to_fill) {
            if (fieldToFill === 'item_ids_or_regex') {
                if ('item_ids' in data && Array.isArray(data.item_ids) && data.item_ids.length) {
                    continue;
                }
                if ('regex' in data && Array.isArray(data.regex) && data.regex.length) {
                    continue;
                }
                return `Wrong content of parameter "data": at least one of "item_ids" and "regex" should be a non-empty array`;
            }
            if (fieldToFill in data && data[fieldToFill]) {
                continue;
            }
            return `Wrong content of parameter "data": field "${fieldToFill}" is required`;
        }
        return null;
    }
    async makeDefaultReportLabel(reportTemplate, data) {
        if (reportTemplate.report_template_id !== ReportTemplateService_1.OverviewReportTemplate.report_template_id) {
            throw new common_1.NotImplementedException(`checkData() for report template "${reportTemplate.name}" with id "${reportTemplate.report_template_id} has not been implemented yet.`);
        }
        return reportTemplate.name;
    }
    async makeReportItems(reportTemplate, data) {
        if (reportTemplate.report_template_id !== ReportTemplateService_1.OverviewReportTemplate.report_template_id) {
            throw new common_1.NotImplementedException(`checkData() for report template "${reportTemplate.name}" with id "${reportTemplate.report_template_id} has not been implemented yet.`);
        }
        const start = {
            format: reportItem_constants_1.REPORT_ITEM_FORMAT.text,
            type: reportItem_constants_1.REPORT_ITEM_TYPE.text,
            data: {
                text: `Start of template items from template: ${reportTemplate.name}`,
                style: "h2",
            },
        };
        const end = {
            format: reportItem_constants_1.REPORT_ITEM_FORMAT.text,
            type: reportItem_constants_1.REPORT_ITEM_TYPE.text,
            data: {
                text: `End of template items from template: ${reportTemplate.name}`,
                style: 'h3',
            },
        };
        const healthStatus = {
            format: reportItem_constants_1.REPORT_ITEM_FORMAT.table,
            type: reportItem_constants_1.REPORT_ITEM_TYPE.health_status,
            data: data,
        };
        delete healthStatus.data.range;
        return [
            start,
            healthStatus,
            end,
        ];
    }
};
exports.ReportTemplateService = ReportTemplateService;
ReportTemplateService.OverviewReportTemplate = {
    report_template_id: 'aaaaaaaa-0001-0001-0001-000000000001',
    name: 'Overview',
    fields_to_fill: [
        'class',
        'hw_type',
        'subsystem',
        'item_ids_or_regex',
        'range',
    ],
};
ReportTemplateService.SomeFutureReportTemplate = {
    report_template_id: 'aaaaaaaa-0001-0001-0001-000000000999',
    name: 'Some future report template (not implemented now)',
    fields_to_fill: [
        'class',
        'hw_type',
        'subsystem',
        'item_ids_or_regex',
        'range',
    ],
};
ReportTemplateService.reportTemplates = [
    ReportTemplateService_1.OverviewReportTemplate,
    ReportTemplateService_1.SomeFutureReportTemplate,
];
exports.ReportTemplateService = ReportTemplateService = ReportTemplateService_1 = __decorate([
    (0, common_1.Injectable)()
], ReportTemplateService);
//# sourceMappingURL=ReportTemplate.service.js.map