"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _a, _PdfMakeWrapper_elementHasClass, _PdfMakeWrapper_filterEmptyNodes, _PdfMakeWrapper_nodesAreOnSamePage, _PdfMakeWrapper_pageBreakBeforeCaption, _PdfMakeWrapper_pageBreakBeforeCaptionWhenViaHtml, _PdfMakeWrapper_footerWithPageNumbers, _PdfMakeWrapper_prepareDocDefinition, _PdfMakeWrapper_preparePdfStreamWithRobotoFonts, _PdfMakeWrapper_preparePdfStream, _LongStringHandler_instances, _b, _LongStringHandler_treatRecursively, _LongStringHandler_splitAndJoin, _LongStringHandler_treatNumber, _LongStringHandler_treatWord;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Reporter = void 0;
const reportItem_constants_1 = require("../../common/constants/reportItem.constants");
const dataSource_1 = require("../../config/dataSource");
const hwType_entity_1 = require("../../entity/hwType.entity");
const itemProperty_entity_1 = require("../../entity/itemProperty.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const reportFile_entity_1 = require("../../entity/reportFile.entity");
const Format_1 = require("../log/Format");
const promises_1 = require("fs/promises");
const path_1 = require("path");
const promises_2 = require("stream/promises");
const typeorm_1 = require("typeorm");
const uuid_1 = require("uuid");
const winston = require("winston");
const CustomTop_1 = require("../menu/menu/CustomTop");
const Color_1 = require("../metrics/Color");
const MetricsWrapper_1 = require("../metrics/MetricsWrapper");
const GraphData_1 = require("./GraphData");
const GraphData_factory_1 = require("./GraphData.factory");
const MetricTools_1 = require("./MetricTools");
const reporter_configuration_1 = require("./reporter.configuration");
const fs = require("fs");
const pdfMakeForServer = require("pdfmake");
const pdfMakeForBrowser = require("pdfmake/build/pdfmake");
const pdfFonts = require("pdfmake/build/vfs_fonts");
const CustomGroupWrapper_1 = require("../custom_group/CustomGroupWrapper");
const ItemGroupWrapper_1 = require("../custom_group/ItemGroupWrapper");
const ACLCheck_1 = require("../acl/ACLCheck");
const Storage_1 = require("../metrics/class/Storage");
const class_constants_1 = require("../../common/constants/class.constants");
;
const Overview_1 = require("../menu/menu/Overview");
const User_1 = require("../users/User");
class HwTypeLabel {
    static async initialize() {
        HwTypeLabel.data = await (0, dataSource_1.getDataSource)().getRepository(hwType_entity_1.HwType).find({
            relations: {
                subsystems: true,
            }
        });
        for (const hwt of HwTypeLabel.data) {
            HwTypeLabel.hwTypeLabels.set(hwt.hw_type, hwt.label);
            const hwtSubsystemLabels = new Map();
            if (hwt.subsystems) {
                for (const subs of hwt.subsystems) {
                    hwtSubsystemLabels.set(subs.subsystem, subs.label);
                }
            }
            HwTypeLabel.subsystemLabels.set(hwt.hw_type, hwtSubsystemLabels);
        }
        return;
    }
    static async getLabel(hwType, subsystem = undefined, separator = ' ') {
        return (subsystem.charAt(0).toUpperCase() + subsystem.slice(1)).replaceAll('_', ' ');
    }
}
HwTypeLabel.data = undefined;
HwTypeLabel.hwTypeLabels = new Map();
HwTypeLabel.subsystemLabels = new Map();
class PdfMakeWrapper {
    static makeAndSave(content, saveToFilePath, viaHtml = false) {
        const pdfDoc = _a.makePdfStream(content, viaHtml);
        const file = fs.createWriteStream(saveToFilePath);
        return (0, promises_2.pipeline)(pdfDoc, file);
    }
    static makeTable(content, includeTextFields = true) {
        let headers;
        if (content.additionalHead) {
            headers = [content.additionalHead, content.table.head.map(t => ({ text: t, style: "tableHeader", }))];
        }
        else {
            headers = [content.table.head.map(t => ({ text: t, style: "tableHeader", }))];
            delete content.additionalHead;
        }
        let result = [
            {
                text: [
                    {
                        text: `${content.caption}\n`,
                        style: 'caption',
                        lineHeight: 1,
                    },
                    {
                        text: content.addtitionalText,
                        style: 'text',
                        lineHeight: 1,
                    },
                ],
                margin: [0, 20, 0, 5]
            },
            {
                layout: 'standardTableLayout',
                table: {
                    headerRows: 1,
                    keepWithHeaderRows: 1,
                    body: [
                        ...headers,
                        ...content.table.body,
                    ],
                },
                margin: [0, 10, 0, 10],
            }
        ];
        if (!includeTextFields) {
            result.shift();
        }
        const unbreakable = content.table.body.length < 15;
        if (unbreakable) {
            result = {
                unbreakable: true,
                stack: result,
            };
        }
        return result;
    }
    static makePdfStream(content, viaHtml = false) {
        const config = reporter_configuration_1.default.pdfMake;
        const docDefinition = __classPrivateFieldGet(_a, _a, "m", _PdfMakeWrapper_prepareDocDefinition).call(_a, content, config, viaHtml);
        if (config.useFontDefinition && (config.useFontDefinition in config.fontDefinitions)) {
            return __classPrivateFieldGet(_a, _a, "m", _PdfMakeWrapper_preparePdfStream).call(_a, docDefinition, config.tableLayouts, config.fontDefinitions[config.useFontDefinition]);
        }
        else {
            return __classPrivateFieldGet(_a, _a, "m", _PdfMakeWrapper_preparePdfStreamWithRobotoFonts).call(_a, docDefinition, config.tableLayouts);
        }
    }
}
_a = PdfMakeWrapper, _PdfMakeWrapper_prepareDocDefinition = function _PdfMakeWrapper_prepareDocDefinition(content, config, viaHtml = false) {
    const docDefinition = config.docDefinition ?? {};
    docDefinition['content'] = content;
    if (viaHtml) {
        docDefinition['pageBreakBefore'] = __classPrivateFieldGet(_a, _a, "f", _PdfMakeWrapper_pageBreakBeforeCaptionWhenViaHtml);
    }
    else {
        docDefinition['pageBreakBefore'] = __classPrivateFieldGet(_a, _a, "f", _PdfMakeWrapper_pageBreakBeforeCaption);
    }
    if (config.addPageNumberFooter) {
        docDefinition['footer'] = __classPrivateFieldGet(_a, _a, "f", _PdfMakeWrapper_footerWithPageNumbers);
    }
    return docDefinition;
}, _PdfMakeWrapper_preparePdfStreamWithRobotoFonts = function _PdfMakeWrapper_preparePdfStreamWithRobotoFonts(docDefinition, tableLayouts) {
    pdfMakeForBrowser.vfs = pdfFonts.pdfMake.vfs;
    const pdfDocGenerator = pdfMakeForBrowser.createPdf(docDefinition, tableLayouts);
    const pdfDoc = pdfDocGenerator.getStream();
    pdfDoc.end();
    return pdfDoc;
}, _PdfMakeWrapper_preparePdfStream = function _PdfMakeWrapper_preparePdfStream(docDefinition, tableLayouts, fonts) {
    const printer = new pdfMakeForServer(fonts);
    const pdfDoc = printer.createPdfKitDocument(docDefinition, { tableLayouts: tableLayouts });
    pdfDoc.end();
    return pdfDoc;
};
_PdfMakeWrapper_elementHasClass = { value: function (node, className) {
        if (!('style' in node)) {
            return false;
        }
        if (typeof node.style === 'string') {
            return node.style === className;
        }
        if (Array.isArray(node.style)) {
            return node.style.includes(className);
        }
        return false;
    } };
_PdfMakeWrapper_filterEmptyNodes = { value: function (nodes) {
        return nodes.filter((n) => !((('text' in n) && (typeof (n.text) === 'string') && n.text.trim() === '') || __classPrivateFieldGet(_a, _a, "f", _PdfMakeWrapper_elementHasClass).call(_a, n, 'footer')));
    } };
_PdfMakeWrapper_nodesAreOnSamePage = { value: function (node1, node2) {
        if (node1.pageNumbers.length !== node2.pageNumbers.length) {
            return false;
        }
        for (const pageNumber of node1.pageNumbers) {
            if (!node2.pageNumbers.includes(pageNumber)) {
                return false;
            }
        }
        return true;
    } };
_PdfMakeWrapper_pageBreakBeforeCaption = { value: function (currentNode, followingNodesOnPage, nodesOnNextPage, previousNodesOnPage) {
        if (__classPrivateFieldGet(_a, _a, "f", _PdfMakeWrapper_elementHasClass).call(_a, currentNode, 'caption')) {
            if (followingNodesOnPage.length === 0) {
                return true;
            }
            if (__classPrivateFieldGet(_a, _a, "f", _PdfMakeWrapper_nodesAreOnSamePage).call(_a, currentNode, followingNodesOnPage[0])) {
                return false;
            }
            return (currentNode.startPosition.top > (currentNode.startPosition.pageInnerHeight * 0.7));
        }
        return false;
    } };
_PdfMakeWrapper_pageBreakBeforeCaptionWhenViaHtml = { value: function (currentNode, followingNodesOnPage, nodesOnNextPage, previousNodesOnPage) {
        if (__classPrivateFieldGet(_a, _a, "f", _PdfMakeWrapper_elementHasClass).call(_a, currentNode, 'caption')) {
            const followingNonEmptyNodesOnPage = __classPrivateFieldGet(_a, _a, "f", _PdfMakeWrapper_filterEmptyNodes).call(_a, followingNodesOnPage);
            if (followingNonEmptyNodesOnPage.length === 0) {
                return true;
            }
            return !__classPrivateFieldGet(_a, _a, "f", _PdfMakeWrapper_nodesAreOnSamePage).call(_a, currentNode, followingNonEmptyNodesOnPage[0]);
        }
        return false;
    } };
_PdfMakeWrapper_footerWithPageNumbers = { value: function (currentPage, pageCount, pageSize) {
        return {
            text: `${currentPage.toString()} / ${pageCount}`,
            alignment: 'right',
            style: ['footer'],
            margin: [0, 5, 30, 0],
        };
    } };
class LongStringHandler {
    constructor(maxLength) {
        _LongStringHandler_instances.add(this);
        this.maxLength = maxLength ? maxLength : 1;
    }
    static treat(input) {
        return __classPrivateFieldGet(_b, _b, "m", _LongStringHandler_treatRecursively).call(_b, input, _b.pageWidth);
    }
    treatString(input) {
        if (input.length <= this.maxLength) {
            return input;
        }
        if (/\s/.test(input)) {
            return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, input, /\s+/, ' ');
        }
        input = input.replace(/([,?!])([^,?!])/g, '$1\u200b$2');
        if (input.includes('\u200b')) {
            return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, input, '\u200b', '\u200b');
        }
        input = input.replace(/(\p{Ll})(\p{Lu})/gu, '$1\u200b$2');
        if (input.includes('\u200b')) {
            return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, input, '\u200b', '\u200b');
        }
        input = input.replace(/(\P{L})(\p{L})/gu, '$1\u200b$2');
        if (input.includes('\u200b')) {
            return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, input, '\u200b', '\u200b');
        }
        input = input.replace(/(\p{L})(\P{L})/gu, '$1\u200b$2');
        if (input.includes('\u200b')) {
            return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, input, '\u200b', '\u200b');
        }
        input = input.replace(/([^0-9])([0-9])/g, '$1\u200b$2');
        if (input.includes('\u200b')) {
            return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, input, '\u200b', '\u200b');
        }
        input = input.replace(/([0-9])([^0-9])/g, '$1\u200b$2');
        if (input.includes('\u200b')) {
            return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, input, '\u200b', '\u200b');
        }
        if (/^[0-9]+$/.test(input)) {
            return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_treatNumber).call(this, input);
        }
        if (/^\p{L}+$/u.test(input)) {
            return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_treatWord).call(this, input);
        }
        input = input.replace(/([\p{P}\p{S}])/gu, '\u200b$1').replace(/^\u200b*/, '');
        if (input.includes('\u200b')) {
            return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, input, '\u200b', '\u200b');
        }
        return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, input, '', '\u200b');
    }
}
_b = LongStringHandler, _LongStringHandler_instances = new WeakSet(), _LongStringHandler_treatRecursively = function _LongStringHandler_treatRecursively(input, maxLength) {
    if (['number', 'bigint'].includes(typeof (input))) {
        input = input.toString();
    }
    if (typeof (input) === 'string') {
        const handler = new _b(maxLength);
        return handler.treatString(input);
    }
    if (input && (typeof (input) === 'object')) {
        const childMaxLength = Math.floor(_b.pageWidth / Object.keys(input).length);
        for (const i in input) {
            input[i] = __classPrivateFieldGet(_b, _b, "m", _LongStringHandler_treatRecursively).call(_b, input[i], childMaxLength);
        }
    }
    else if (input) {
        console.warn('[GenerateReportWarning]: LongTextHandler: weird field data ', input);
    }
    return input;
}, _LongStringHandler_splitAndJoin = function _LongStringHandler_splitAndJoin(input, separator, glue) {
    let parts = input.split(separator);
    for (const i in parts) {
        parts[i] = this.treatString(parts[i]);
    }
    return parts.join(glue);
}, _LongStringHandler_treatNumber = function _LongStringHandler_treatNumber(input) {
    if (this.maxLength < 3) {
        return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, input, '', '\u200b');
    }
    const length = input.length;
    const openingTripletLength = (length % 3) || 3;
    const additionalTripletsForFirstSequence = Math.floor((this.maxLength - openingTripletLength) / 3);
    const firstSequenceLegth = openingTripletLength + additionalTripletsForFirstSequence * 3;
    const nextSequenceLength = Math.floor(this.maxLength / 3) * 3;
    const parts = [];
    let i = 0;
    let start = 0;
    let end = firstSequenceLegth;
    while (start < length) {
        parts[i] = input.slice(start, end);
        i++;
        start = end;
        end += nextSequenceLength;
    }
    return parts.join('\u200b');
}, _LongStringHandler_treatWord = function _LongStringHandler_treatWord(input) {
    if (this.maxLength < 3) {
        return __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, input, '', '\u200b');
    }
    const VCVRegex = /([aeiouy])((?:wr)|(?:wh)|(?:zh)|(?:th)|(?:rh)|(?:gh)|(?:ph)|(?:ch)|(?:kn)|(?:sh)|(?:sc)|(?:sk)|(?:sm)|(?:sn)|(?:st)|(?:sp)|(?:sw)|(?:sl)|(?:bl)|(?:fl)|(?:gl)|(?:cl)|(?:pl)|(?:br)|(?:cr)|(?:dr)|(?:fr)|(?:gr)|(?:pr)|(?:tr)|(?:dw)|(?:sw)|(?:tw)|(?:scr)|(?:shr)|(?:spl)|(?:spr)|(?:str)|(?:thr)|[^aeiouy\u200b])([aeiouy])/gu;
    const VCCVLazyRegex = /([aeiouy])((?:th)|(?:gh)|(?:ph)|(?:ch)|(?:ck)|(?:sh)|[^aeiouy\u200b])((?:wr)|(?:wh)|(?:zh)|(?:th)|(?:rh)|(?:gh)|(?:ph)|(?:ch)|(?:kn)|(?:sh)|(?:sc)|(?:sk)|(?:sm)|(?:sn)|(?:st)|(?:sp)|(?:sw)|(?:sl)|(?:bl)|(?:fl)|(?:gl)|(?:cl)|(?:pl)|(?:br)|(?:cr)|(?:dr)|(?:fr)|(?:gr)|(?:pr)|(?:tr)|(?:dw)|(?:sw)|(?:tw)|(?:scr)|(?:shr)|(?:spl)|(?:spr)|(?:str)|(?:thr)|[^aeiouy\u200b])([aeiouy])/gu;
    const VCCVEagerRegex = /([aeiouy])((?:zh)|(?:th)|(?:rh)|(?:gh)|(?:ph)|(?:ch)|(?:ck)|(?:sh)|(?:sc)|(?:ct)|(?:ft)|(?:lt)|(?:nt)|(?:pt)|(?:st)|(?:xt)|(?:lb)|(?:ld)|(?:lf)|(?:lk)|(?:lp)|(?:lt)|(?:nd)|(?:nk)|(?:nt)|(?:ng)|(?:sk)|(?:sp)|(?:mp)|(?:nch)|[^aeiouy\u200b])((?:wr)|(?:wh)|(?:zh)|(?:th)|(?:rh)|(?:gh)|(?:ph)|(?:ch)|(?:kn)|(?:sh)|(?:sc)|(?:sk)|(?:sm)|(?:sn)|(?:st)|(?:sp)|(?:sw)|(?:sl)|(?:bl)|(?:fl)|(?:gl)|(?:cl)|(?:pl)|(?:br)|(?:cr)|(?:dr)|(?:fr)|(?:gr)|(?:pr)|(?:tr)|(?:dw)|(?:sw)|(?:tw)|(?:scr)|(?:shr)|(?:spl)|(?:spr)|(?:str)|(?:thr)|[^aeiouy\u200b])([aeiouy])/gu;
    input = input.replace(VCVRegex, '$1\u200b$2$3');
    input = input.replace(VCVRegex, '$1\u200b$2$3');
    input = input.replace(VCCVLazyRegex, '$1$2\u200b$3$4');
    input = input.replace(VCCVLazyRegex, '$1$2\u200b$3$4');
    input = input.replace(VCCVEagerRegex, '$1$2\u200b$3$4');
    input = input.replace(VCCVEagerRegex, '$1$2\u200b$3$4');
    const parts = [];
    let j = 0;
    const syllables = input.split('\u200b');
    for (let i = 0; i < syllables.length; i++) {
        if (syllables[i].length > this.maxLength) {
            j += (j in parts) ? 1 : 0;
            parts[j] = __classPrivateFieldGet(this, _LongStringHandler_instances, "m", _LongStringHandler_splitAndJoin).call(this, syllables[i], '', '\u200b');
            j++;
            continue;
        }
        parts[j] ??= '';
        if (parts[j].length + syllables[i].length > this.maxLength) {
            j++;
            parts[j] = '';
        }
        parts[j] += syllables[i];
    }
    return parts.join('\u200b');
};
LongStringHandler.pageWidth = reporter_configuration_1.default.longStringHandler.pageWidth;
class ReportTitleData {
    constructor(title) {
        this.title = title;
    }
    getContentForPdfMake() {
        return [
            {
                text: " ",
                margin: [0, 10, 0, 0],
            },
            {
                table: {
                    widths: ['*'],
                    body: [[{
                                text: this.title,
                                style: 'title',
                                border: [false, false, false, false]
                            }]],
                }
            }
        ];
    }
}
class ReportGeneratedAtData {
    constructor(generatedAt) {
        this.generatedAt = generatedAt;
    }
    getContentForPdfMake() {
        const formatter = new Intl.DateTimeFormat('en-GB', { dateStyle: "full", timeStyle: "medium" });
        return {
            text: `Report generated on ${formatter.format(this.generatedAt)}`,
            style: 'generatedAt'
        };
    }
}
class ReportItemData {
    constructor(item, timeRange = null) {
        this.reportItem = item;
        this.timeRange = timeRange;
    }
    getContentForPdfMake() {
        return `Not implemented yet.  format: ${this.reportItem.format}, type: ${this.reportItem.type}.`;
    }
    promiseInitialization() {
        return Promise.resolve(this);
    }
    static getReportItemDataClass(item) {
        if (item.format === reportItem_constants_1.REPORT_ITEM_FORMAT.text) {
            return ReportTextItemData;
        }
        if (item.format === reportItem_constants_1.REPORT_ITEM_FORMAT.table) {
            if (item.type == reportItem_constants_1.REPORT_ITEM_TYPE.configuration) {
                return ReportConfigurationTableItemData;
            }
            if (item.type == reportItem_constants_1.REPORT_ITEM_TYPE.metric) {
                return ReportMetricsTableItemData;
            }
            if (item.type == reportItem_constants_1.REPORT_ITEM_TYPE.mapping) {
                return ReportMappingTableItemData;
            }
            if (item.type == reportItem_constants_1.REPORT_ITEM_TYPE.health_status) {
                return ReportHealthStatusTableItemData;
            }
            if (item.type == reportItem_constants_1.REPORT_ITEM_TYPE.top) {
                return ReportTopTableItemData;
            }
            if (item.type == reportItem_constants_1.REPORT_ITEM_TYPE.box) {
                return ReportBoxTableItemData;
            }
        }
        if (item.format === reportItem_constants_1.REPORT_ITEM_FORMAT.graph) {
            if (item.type == reportItem_constants_1.REPORT_ITEM_TYPE.metric) {
                return ReportMetricGraphItemData;
            }
        }
        return ReportUnsupportedItemData;
    }
    static makeDataPromise(item, timeRange) {
        const itemDataClass = ReportItemData.getReportItemDataClass(item);
        const itemData = new itemDataClass(item, timeRange);
        return itemData.promiseInitialization();
    }
}
class ReportSupportedItemData extends ReportItemData {
}
class ReportItemDelayedData extends ReportSupportedItemData {
    constructor() {
        super(...arguments);
        this.hwTypeLabel = undefined;
        this.metricForLabel = undefined;
        this.observedItemIds = [];
        this.observedItemsArray = [];
        this.observedItems = {};
        this.contentForPdfMake = 'Not defined.';
        this.reloadObservedItemsWithRelations = false;
    }
    getContentForPdfMake() {
        return this.contentForPdfMake;
    }
    setMetricLabel(metricString, metricSpecification = undefined) {
        if (metricSpecification) {
            if (metricSpecification.label) {
                this.metricForLabel = metricSpecification.longLabel ? metricSpecification.longLabel : metricSpecification.label;
                return this.metricForLabel;
            }
            if (metricSpecification.metric) {
                this.metricForLabel = metricSpecification.metric;
                return this.metricForLabel;
            }
        }
        this.metricForLabel = metricString;
        return this.metricForLabel;
    }
    getIntervalLabel(interval) {
        if (!interval) {
            return undefined;
        }
        switch (interval) {
            case 'week':
                return 'weekly';
            case 'month':
                return 'monthly';
            case 'year':
                return 'yearly';
            case 'day':
                return 'daily';
            default:
                Reporter.winstonLogger.warn(`Unknown interval ${interval}`);
                return undefined;
        }
    }
    retrieveObservedItems(itemIds) {
        return (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                item_id: (0, typeorm_1.In)(itemIds)
            }
        });
    }
    getCaption() {
        const parts = [this.captionPrefix, this.hwTypeLabel, this.metricForLabel];
        const rangeLabel = this.getIntervalLabel(this.reportItem.data['interval']);
        let valueType = this.reportItem.data['order_by'] ? this.reportItem.data['order_by'] : undefined;
        if (rangeLabel) {
            let part = " (";
            part += rangeLabel;
            if (valueType) {
                part += ` ${valueType}`;
                valueType = undefined;
            }
            part += ")";
            parts.push(part);
        }
        if (valueType) {
            parts.push(`(${valueType})`);
        }
        return parts.filter(x => x).join(' ');
    }
    getAdditionalText() {
        const parts = [];
        const time_range = this.reportItem.data['time_range'];
        if (time_range) {
            parts.push(`${this.formatDate(time_range.date_from)} - ${this.formatDate(time_range.date_to)}`);
        }
        return parts.filter(x => x).join(' ');
    }
    formatDate(date) {
        let formattedDate = new Intl.DateTimeFormat('sv-SE', {
            year: 'numeric',
            month: 'numeric',
            day: 'numeric',
            hour: 'numeric',
            minute: 'numeric'
        }).format(new Date(date).getTime());
        return formattedDate;
    }
    async promiseInitialization() {
        const data = this.reportItem.data;
        let hw_type = this.reportItem.hw_type ? this.reportItem.hw_type : this.reportItem.class;
        let subsystem = this.reportItem.subsystem;
        if (this.reportItem.class === 'pdf_export') {
            if (this.reportItem.type === "box") {
                return this.specificPromiseInitialization();
            }
            this.hwTypeLabel = await HwTypeLabel.getLabel(hw_type, subsystem);
            this.observedItemIds = data['item_ids'] ?? [];
        }
        else {
            const { ids, items } = await Reporter.getIds(this.reportItem.item_group_id, this.reportItem.custom_group_id);
            this.observedItemIds = ids;
            this.observedItemsArray = items;
            if (!this.observedItemIds) {
                return this.handleMissingData();
            }
        }
        await this.enhanceObservedItems([], hw_type, subsystem);
        if (!this.observedItemIds.length) {
            return this.handleMissingData();
        }
        return this.specificPromiseInitialization();
    }
    async enhanceObservedItems(regexArray, hw_type, subsystem) {
        const reload = this.reloadObservedItemsWithRelations || this.observedItemIds.length;
        if (this.observedItemsArray.length) {
            this.observedItemIds.push(...this.observedItemsArray.map(item => item.item_id));
            this.observedItemIds = this.observedItemIds.filter((value, index) => this.observedItemIds.indexOf(value) === index);
        }
        if (!this.observedItemIds.length) {
            Reporter.winstonLogger.warn('Item IDs (regex or item_ids) not provided.');
            return;
        }
        if (reload) {
            this.observedItemsArray = await this.retrieveObservedItems(this.observedItemIds);
        }
        this.observedItemsArray.forEach(item => { this.observedItems[item.item_id] = item; });
    }
    async specificPromiseInitialization() {
        return Promise.resolve(this);
    }
    handleMissingData() {
        const parts = [this.captionPrefix, this.hwTypeLabel];
        const specification = parts.filter(x => x).join(' ');
        return Promise.resolve(new ReportItemMissingData(this.reportItem, specification));
    }
    handleError(errorPrefix = 'Some Error', error) {
        const errorText = `${errorPrefix} when trying to render ${this.getCaption()}`;
        Reporter.winstonLogger.error(error ? `${errorText} : ${error}` : errorText);
        return Promise.resolve(new ReportItemError(this.reportItem, errorText));
    }
}
class ReportItemError extends ReportItemDelayedData {
    constructor(item, error) {
        super(item);
        this.error = error;
    }
    getContentForPdfMake() {
        return {
            text: this.error,
            style: 'error',
        };
    }
}
class ReportItemMissingData extends ReportItemError {
    constructor(item, specification = undefined) {
        const error = specification ? `Data required for ${specification} are missing` : 'Required data are missing';
        super(item, error);
    }
}
class ReportTextItemData extends ReportSupportedItemData {
    constructor(item) {
        const data = item.data;
        super(item);
        this.text = String(data.text).trim();
        this.style = data.style || undefined;
    }
    getContentForPdfMake() {
        return {
            text: this.text || 'wrong text data',
            style: this.style || 'textItem',
        };
    }
}
class ReportTableItemData extends ReportItemDelayedData {
    constructor() {
        super(...arguments);
        this.specifications = {};
    }
    getContentForPdfMake() {
        return PdfMakeWrapper.makeTable({
            caption: this.getCaption(),
            addtitionalText: this.getAdditionalText(),
            table: this.getTableContent(),
        });
    }
    getColumnKeys() {
        return ['Item'].concat(this.valueColumnKeys);
    }
    getTableHead() {
        const specs = this.specifications;
        const niceLabels = this.valueColumnKeys.map(key => specs[key] ? MetricTools_1.MetricTools.makeLabel(specs[key], true) : key);
        return ['Item'].concat(niceLabels);
    }
    getTableContent() {
        const body = this.prepareTableBody(this.observedItemIds, this.observedItems, this.tableValues, this.getColumnKeys(), ' ');
        return {
            head: LongStringHandler.treat(this.getTableHead()),
            body: LongStringHandler.treat(body),
        };
    }
    formatValues(data, keys, specifications, observedItemIds, valueType = "avg") {
        const result = {};
        for (const id of observedItemIds) {
            result[id] = {};
            for (const key of keys) {
                if (data[id] && data[id][key]) {
                    const value = (valueType !== null) ? data[id][key][valueType] : data[id][key];
                    const spec = specifications[key];
                    result[id][key] = MetricTools_1.MetricTools.formatNumber(value, spec);
                }
            }
        }
        return result;
    }
    prepareTableBody(observedItemIds, observedItems, data, keys, defaultValue = null) {
        const tableRows = [];
        let cell;
        for (const id of observedItemIds) {
            const tableRow = [];
            for (const key of keys) {
                if (key === 'Item') {
                    const label = observedItems[id]['label'];
                    cell = { text: label, style: 'tableItemLabel', alignment: 'left' };
                }
                else if (data[id] && data[id][key]) {
                    let align = 'right';
                    if (isNaN(+data[id][key])) {
                        align = 'left';
                    }
                    cell = { text: String(data[id][key]), style: 'tableItemValue', alignment: align };
                }
                else {
                    cell = { text: defaultValue, style: 'tableItemNoValue', alignment: 'center' };
                }
                tableRow.push(cell);
            }
            tableRows.push(tableRow);
        }
        return tableRows;
    }
}
class ReportConfigurationTableItemData extends ReportTableItemData {
    constructor() {
        super(...arguments);
        this.captionPrefix = 'Configuration';
    }
    async specificPromiseInitialization() {
        const data = this.reportItem.data;
        if (data.tablePerItem) {
            return Promise.resolve(new ReportConfigurationTableItemSeparateData(this.reportItem, this.timeRange).promiseInitialization());
        }
        this.valueColumnKeys = data.property_keys ?? data.properties_key;
        let hwType = this.reportItem.hw_type;
        let subsystem = this.reportItem.subsystem;
        if (!this.valueColumnKeys) {
            return this.handleMissingData();
        }
        if (this.reportItem.class) {
            hwType = this.observedItems[Object.keys(this.observedItems)[0]].hw_type;
            subsystem = this.observedItems[Object.keys(this.observedItems)[0]].subsystem;
        }
        else {
            hwType = this.reportItem.hw_type;
            subsystem = this.reportItem.subsystem;
        }
        const classKey = await MetricsWrapper_1.MetricsWrapper.findClassKey(hwType);
        this.specifications = MetricsWrapper_1.MetricsWrapper.getConfiguration(classKey, hwType, subsystem);
        if (!this.observedItemIds) {
            const { ids, items } = await Reporter.getIds(this.reportItem.item_group_id, this.reportItem.custom_group_id);
            this.observedItemIds = ids;
            this.observedItemsArray = items;
        }
        const itemProperties = await (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty).findBy({
            item_id: (0, typeorm_1.In)(this.observedItemIds),
            property_name: (0, typeorm_1.In)(this.valueColumnKeys)
        });
        const rawValues = ReportConfigurationTableItemData.processProperties(itemProperties);
        this.tableValues = this.formatValues(rawValues, this.valueColumnKeys, this.specifications, this.observedItemIds, null);
        return Promise.resolve(this);
    }
    static processProperties(properties) {
        let props = {};
        for (const property of properties) {
            if (!props[property.item_id]) {
                props[property.item_id] = {
                    [property.property_name]: property.property_value
                };
            }
            else {
                props[property.item_id][property.property_name] = property.property_value;
            }
        }
        return props;
    }
}
class ReportTablePropValData extends ReportItemDelayedData {
    constructor() {
        super(...arguments);
        this.specifications = {};
    }
    getContentForPdfMake() {
        let tables = [];
        for (const id of this.observedItemIds) {
            tables.push(PdfMakeWrapper.makeTable({
                caption: this.getCaption(id),
                addtitionalText: this.getAdditionalText(),
                table: this.getTableContent(id)
            }));
        }
        return tables;
    }
    getCaption(id) {
        return `${this.captionPrefix} - ${this.observedItems[id].label}`;
    }
    getTableHead() {
        return ['Property', 'Value'];
    }
    getTableContent(id) {
        const body = this.prepareTableBody(id, this.tableValues);
        return {
            head: LongStringHandler.treat(this.getTableHead()),
            body: LongStringHandler.treat(body),
        };
    }
    formatValues(data, keys, specifications, observedItemIds, valueType = "avg") {
        const result = {};
        for (const id of observedItemIds) {
            result[id] = {};
            for (const key of keys) {
                if (data[id] && data[id][key]) {
                    const value = (valueType !== null) ? data[id][key][valueType] : data[id][key];
                    const spec = specifications[key];
                    result[id][key] = MetricTools_1.MetricTools.formatNumber(value, spec);
                }
            }
        }
        return result;
    }
    prepareTableBody(id, data) {
        const tableRows = [];
        let cell;
        const specs = this.specifications;
        for (const key in data[id]) {
            const tableRow = [];
            cell = { text: specs[key] ? MetricTools_1.MetricTools.makeLabel(specs[key], true) : key, style: 'tableItemLabel', alignment: 'left' };
            tableRow.push(cell);
            let align = 'right';
            cell = { text: data[id][key], style: 'tableItemValue', alignment: align };
            tableRow.push(cell);
            tableRows.push(tableRow);
        }
        return tableRows;
    }
}
class ReportConfigurationTableItemSeparateData extends ReportTablePropValData {
    constructor() {
        super(...arguments);
        this.captionPrefix = 'Configuration';
    }
    async specificPromiseInitialization() {
        const data = this.reportItem.data;
        this.valueColumnKeys = data.property_keys ?? data.properties_key;
        let hwType = this.reportItem.hw_type;
        let subsystem = this.reportItem.subsystem;
        if (!this.valueColumnKeys) {
            return this.handleMissingData();
        }
        if (this.reportItem.class) {
            hwType = this.observedItems[Object.keys(this.observedItems)[0]].hw_type;
            subsystem = this.observedItems[Object.keys(this.observedItems)[0]].subsystem;
        }
        else {
            hwType = this.reportItem.hw_type;
            subsystem = this.reportItem.subsystem;
        }
        const classKey = await MetricsWrapper_1.MetricsWrapper.findClassKey(hwType);
        this.specifications = MetricsWrapper_1.MetricsWrapper.getConfiguration(classKey, hwType, subsystem);
        if (!this.observedItemIds) {
            const { ids, items } = await Reporter.getIds(this.reportItem.item_group_id, this.reportItem.custom_group_id);
            this.observedItemIds = ids;
            this.observedItemsArray = items;
        }
        const itemProperties = await (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty).findBy({
            item_id: (0, typeorm_1.In)(this.observedItemIds),
            property_name: (0, typeorm_1.In)(this.valueColumnKeys)
        });
        const rawValues = ReportConfigurationTableItemData.processProperties(itemProperties);
        this.tableValues = this.formatValues(rawValues, this.valueColumnKeys, this.specifications, this.observedItemIds, null);
        return Promise.resolve(this);
    }
}
class ReportMetricsTableItemData extends ReportTableItemData {
    constructor() {
        super(...arguments);
        this.captionPrefix = 'Metrics';
    }
    async specificPromiseInitialization() {
        const data = this.reportItem.data;
        this.valueColumnKeys = data.metrics;
        const hwType = this.observedItems[Object.keys(this.observedItems)[0]].hw_type;
        const subsystem = this.reportItem.subsystem;
        const classKey = this.reportItem.class ? this.reportItem.class : await MetricsWrapper_1.MetricsWrapper.findClassKey(hwType);
        if (!(this.valueColumnKeys)) {
            return this.handleMissingData();
        }
        let specificationsArray;
        if (this.reportItem.custom_group_id) {
            specificationsArray = await CustomGroupWrapper_1.CustomGroupWrapper.getMetrics(this.reportItem['custom_group_id']);
        }
        else {
            if (classKey && classKey === "storage") {
                specificationsArray = this.valueColumnKeys.map(metric => Storage_1.Storage.metrics[metric] ? Storage_1.Storage.metrics[metric] : {});
            }
            else {
                const metricSpecificationPromises = this.valueColumnKeys.map(metric => classKey ? MetricsWrapper_1.MetricsWrapper.getMetric(classKey, hwType, subsystem, metric) : {});
                specificationsArray = await Promise.all(metricSpecificationPromises);
            }
        }
        specificationsArray.forEach(s => { if ((typeof s === "object") && s.metric) {
            this.specifications[s.metric] = s;
        } });
        if (data.order_by !== 'max' && data.order_by !== 'avg') {
            let rawValues = await new MetricsWrapper_1.MetricsWrapper(this.valueColumnKeys, this.observedItemIds).getLastHourValueFromTable();
            for (const uuid in rawValues) {
                for (const metric in rawValues[uuid]) {
                    rawValues[uuid][metric] = { [data.order_by]: rawValues[uuid][metric] };
                }
            }
            this.tableValues = this.formatValues(rawValues, this.valueColumnKeys, this.specifications, this.observedItemIds, data.order_by);
        }
        else {
            const { start, end } = Reporter.getStartAndEnd(data.time_range, data.interval, undefined);
            const rawValues = await new MetricsWrapper_1.MetricsWrapper(this.valueColumnKeys, this.observedItemIds).getAvgAndMaxByTime(start, end);
            this.tableValues = this.formatValues(rawValues, this.valueColumnKeys, this.specifications, this.observedItemIds, data.order_by);
        }
        return Promise.resolve(this);
    }
}
class ReportTopTableItemData extends ReportTableItemData {
    constructor() {
        super(...arguments);
        this.captionPrefix = 'Top';
        this.valueColumnKeys = ['avg', 'max'];
    }
    ucfirst(text) {
        return `${text.at(0).toUpperCase()}${text.slice(1)}`;
    }
    getTableHead() {
        const spec = this.metricSpecification;
        const niceLabels = this.valueColumnKeys.map(key => {
            let label = this.ucfirst(key);
            if (spec) {
                label = `${label} ${MetricTools_1.MetricTools.makeLabel(spec, true)}`;
            }
            return label;
        });
        return ['Item'].concat(niceLabels);
    }
    formatValues(data, keys, metricSpecification) {
        const result = {};
        for (const row of data) {
            const id = row.item_id;
            result[id] = {};
            for (const key of keys) {
                if (row[key]) {
                    result[id][key] = MetricTools_1.MetricTools.formatNumber(row[key], metricSpecification);
                }
            }
        }
        return result;
    }
    async specificPromiseInitialization() {
        const data = this.reportItem.data;
        const metric = data.metric;
        const orderBy = data.order_by;
        const hwType = this.observedItems[Object.keys(this.observedItems)[0]].hw_type;
        const subsystem = this.observedItems[Object.keys(this.observedItems)[0]].subsystem;
        if (!metric) {
            return this.handleMissingData();
        }
        let ms = await MetricsWrapper_1.MetricsWrapper.getMetricFindClass(hwType, subsystem, metric);
        this.metricSpecification = ms.metric ? ms : undefined;
        this.setMetricLabel(metric, this.metricSpecification);
        let rawValues = [];
        const { start, end } = Reporter.getStartAndEnd('time_range' in data ? data.time_range : undefined, 'interval' in data ? data.interval : undefined, undefined);
        const customTop = new CustomTop_1.CustomTop(hwType, subsystem, [metric], start, end, orderBy);
        await customTop.fetchMetricsData();
        let customGroupData = customTop.getData();
        for (const key of Object.keys(customGroupData)) {
            if (this.observedItemIds.includes(key)) {
                let currentValues = {
                    'item_id': key,
                    'item_ids': key,
                    'avg': customGroupData[key][metric].avg,
                    'max': customGroupData[key][metric].max,
                };
                rawValues.push(currentValues);
            }
        }
        rawValues.sort(function (a, b) {
            return parseFloat(b[orderBy]) - parseFloat(a[orderBy]);
        });
        this.observedItemIds = rawValues.map(row => row.item_id);
        this.tableValues = this.formatValues(rawValues, this.valueColumnKeys, this.metricSpecification);
        return Promise.resolve(this);
    }
}
class ReportCustomTableData extends ReportTableItemData {
    constructor() {
        super(...arguments);
        this.specifications = {};
        this.includeText = false;
    }
    getContentForPdfMake() {
        return PdfMakeWrapper.makeTable({
            caption: this.getCaption(),
            addtitionalText: this.getAdditionalText(),
            table: this.getTableContent(),
            additionalHead: this.getAddtionalHead(),
        }, this.includeText);
    }
    getColumnKeys() {
        return this.valueColumnKeys;
    }
    getTableHead() {
        const header = this.tableValues.header;
        const boxType = this.tableValues.type;
        let fullHeader = [];
        if (header.every(entry => !Array.isArray(entry))) {
            header.forEach(function (key) {
                let headerSpecs = "";
                let prefix = "";
                let shortcut = "";
                if (boxType && boxType === "CapacityTableBox") {
                    if (key.prefix) {
                        prefix = key.prefix ? key.prefix : "";
                        shortcut = key.shortcut ? key.shortcut : "B";
                    }
                    if (key.thermometer) {
                        prefix = "%";
                    }
                }
                else if (boxType && boxType !== "TableGroupBox") {
                    prefix = key.prefix ? key.prefix : "";
                    shortcut = key.shortcut ? key.shortcut : "";
                    if (key.shortcut === "timestamp") {
                        shortcut = "";
                    }
                }
                headerSpecs = `[${prefix}${shortcut}]`;
                headerSpecs = headerSpecs !== "[]" ? headerSpecs : "";
                fullHeader.push(`${key.value} ${headerSpecs}`);
            });
        }
        return fullHeader;
    }
    setIncludeText(value) {
        this.includeText = value;
    }
    getAddtionalHead() {
        let tableRow = [];
        if (!this.tableValues.groups) {
            return undefined;
        }
        const groups = this.tableValues.groups;
        groups.forEach(function (group, i) {
            group.prefix = group.prefix ? group.prefix : "";
            group.shortcut = group.shortcut ? group.shortcut : "";
            let headerSpecs = `[${group.prefix}${group.shortcut}]`;
            if (i > 0) {
                for (let j = 0; j < group.colCount; j++) {
                    tableRow.push({ text: `${group.group} ${headerSpecs}`, style: 'tableHeader', colSpan: group.colCount, alignment: 'center' });
                }
            }
            else {
                tableRow.push({ text: group.group, rowSpan: 2, style: 'tableHeader', alignment: 'center', margin: [0, 8, 0, 0] });
            }
        });
        if (this.tableValues.type === "TableGroupBox") {
            return tableRow;
        }
        else {
            return undefined;
        }
    }
    getTableContent() {
        const body = this.myPrepareTableBody(undefined, undefined, this.tableValues, this.getColumnKeys(), '');
        return {
            head: LongStringHandler.treat(this.getTableHead()),
            body: LongStringHandler.treat(body),
        };
    }
    formatValues(data, keys, specifications, observedItemIds, valueType = "avg") {
        const result = {};
        for (const id of observedItemIds) {
            result[id] = {};
            for (const key of keys) {
                if (data[id] && data[id][key]) {
                    const value = (valueType !== null) ? data[id][key][valueType] : data[id][key];
                    const spec = specifications[key];
                    result[id][key] = MetricTools_1.MetricTools.formatNumber(value, spec);
                }
            }
        }
        return result;
    }
    myPrepareTableBody(observedItemIds, observedItems, data, keys, defaultValue = null) {
        const tableRows = [];
        const header = data.header;
        const rows = data.rows;
        rows.forEach(function (row, i) {
            const tableRow = [];
            header.forEach(function (key, j) {
                let cell;
                let alignment = key.number || key.thermometer ? "right" : "left";
                let label = row[j].value !== undefined ? row[j].value : defaultValue;
                if (key.number && !key.thermometer) {
                    let value = row[j].value;
                    value = Number.parseFloat(value);
                    if (key.prefix) {
                        value /= MetricTools_1.MetricTools.prefixToValue(key.prefix);
                    }
                    const precision = key.decimal;
                    if (precision !== undefined) {
                        value *= Math.pow(10, precision);
                        value = (value > 0) ? Math.ceil(value) : Math.floor(value);
                        value /= Math.pow(10, precision);
                        label = value.toFixed(precision);
                    }
                }
                if (key.shortcut === "timestamp") {
                    label = new Date((row[j].value * 1000)).toISOString();
                }
                cell = { text: label, style: 'tableItemValue', alignment: alignment };
                if (key.thermometer) {
                    if (+label >= 95) {
                        cell.fillColor = "red";
                        cell.color = '#fff';
                    }
                    else if (+label >= 80) {
                        cell.fillColor = "#ffbd4f";
                        cell.color = '#fff';
                    }
                    else {
                        cell.fillColor = "#008000";
                        cell.color = '#fff';
                    }
                }
                if (row[j].bgColor) {
                    cell.fillColor = row[j].bgColor;
                    cell.color = '#fff';
                }
                tableRow.push(cell);
            });
            tableRows.push(tableRow);
        });
        return tableRows;
    }
}
class ReportBoxTableItemData extends ReportCustomTableData {
    constructor() {
        super(...arguments);
        this.captionPrefix = 'TableBox';
    }
    ucfirst(text) {
        return `${text.at(0).toUpperCase()}${text.slice(1)}`;
    }
    formatData(data) {
        let myData = data;
        for (let i = 0; i < myData.header.length; i++) {
            if (i === 0) {
                let group = {
                    group: myData.header[i].value,
                    colCount: 1,
                };
                myData.groups.unshift(group);
            }
            for (let j = 0; j < myData.groups.length; j++) {
                if (myData.header[i].group === myData.groups[j].group) {
                    myData.groups[j].colCount = myData.groups[j].colCount ? myData.groups[j].colCount += 1 : 1;
                    for (const key in myData.groups[j]) {
                        myData.header[i][key] = myData.groups[j][key];
                    }
                    continue;
                }
            }
        }
        return myData;
    }
    adjustPivot(data) {
        let adjustedData = data;
        let rows = [];
        for (let i = 0; i < data.header.length; i++) {
            rows.push([{ value: data.header[i].value }, { value: data.rows[0][i].value }]);
        }
        adjustedData.header = [{ value: "Property" }, { value: "Value" }];
        adjustedData.rows = rows;
        return adjustedData;
    }
    async specificPromiseInitialization() {
        const data = this.reportItem.data;
        this.metricSpecification = undefined;
        let header = [];
        let rawValues = [];
        if (data.type === "CapacityTableBox" || data.type === "TableBox") {
            let newData = data;
            for (const obj of newData.header) {
                if ('value' in obj && obj.value) {
                    header.push(obj.value.toString());
                }
            }
            this.valueColumnKeys = header;
            this.tableValues = newData;
        }
        else if (data.type === "TableGroupBox") {
            let newData = data;
            for (const obj of newData['header']) {
                if ('value' in obj && obj.value) {
                    header.push(obj.value.toString());
                }
            }
            this.valueColumnKeys = header;
            this.tableValues = this.formatData(newData);
        }
        else if (data.type === "TablePivotBox") {
            this.tableValues = this.adjustPivot(data);
            this.valueColumnKeys = ["Property", "Value"];
        }
        else {
            return this.handleMissingData();
        }
        return Promise.resolve(this);
    }
}
class ReportMappingTableItemData extends ReportCustomTableData {
    constructor() {
        super(...arguments);
        this.captionPrefix = 'Mapping';
    }
    async specificPromiseInitialization() {
        const subsystem = this.reportItem.subsystem;
        this.valueColumnKeys = ["Host", "Device", "Volume"];
        this.setIncludeText(true);
        this.setCaptionPrefix(`Mapping ${subsystem}`);
        const allowSubsystems = ['host', 'device', 'volume'];
        let tableBody = [];
        let hosts = [];
        if (allowSubsystems.includes(subsystem)) {
            hosts = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                where: {
                    hw_type_entity: {
                        class: class_constants_1.ClassKey.STORAGE
                    },
                    subsystem: 'host',
                    item_id: subsystem === "host" ? (0, typeorm_1.In)(this.observedItemIds) : undefined,
                    parents: {
                        _parent: {
                            subsystem: 'device',
                            item_id: subsystem === "device" ? (0, typeorm_1.In)(this.observedItemIds) : undefined
                        }
                    },
                    children: {
                        _child: {
                            subsystem: 'volume',
                            item_id: subsystem === "volume" ? (0, typeorm_1.In)(this.observedItemIds) : undefined,
                        }
                    }
                },
                relations: {
                    children: {
                        _child: true
                    },
                    parents: {
                        _parent: true
                    },
                    hw_type_entity: true
                }
            });
        }
        for (const hostInfo of hosts) {
            let children = [];
            for (const child of hostInfo.children) {
                children.push(child._child.label);
            }
            tableBody.push([{ value: hostInfo.label }, { value: hostInfo.parents[0]._parent.label }, { value: children.join(", ") }]);
        }
        this.tableValues = {
            type: "Mapping",
            header: [{ value: "Host" }, { value: "Device" }, { value: "Volume" }],
            rows: tableBody,
        };
        return Promise.resolve(this);
    }
    setCaptionPrefix(caption) {
        this.captionPrefix = caption;
    }
}
class ReportHealthStatusTableItemData extends ReportTableItemData {
    constructor() {
        super(...arguments);
        this.captionPrefix = 'Health status';
        this.reloadObservedItemsWithRelations = true;
    }
    retrieveObservedItems(itemIds) {
        return (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                item_id: (0, typeorm_1.In)(itemIds)
            }, relations: {
                health_status: true
            }
        });
    }
    getTableContent() {
        const body = this.observedItemIds.map(id => {
            const item = this.observedItems[id];
            return item.health_status ? [item.health_status.status, item.label, item.health_status.data] : [' ', item.label, ''];
        });
        return {
            head: ['Status', 'Item', 'Reason'],
            body: LongStringHandler.treat(body),
        };
    }
}
class ReportGraphItemData extends ReportItemDelayedData {
    getContentForPdfMake() {
        let result = [
            {
                text: this.getCaption(),
                style: 'caption',
            },
            this.formatImage(),
            this.formatTables(),
        ];
        const graphSummaryMaxAllowedItems = this.maxItems !== undefined ? this.maxItems : ReportGraphItemData.getGraphSummaryMaxItems();
        const graphSummaryTotalItems = this.graphData.getTimeserieItems().length;
        if (graphSummaryMaxAllowedItems > 0 && graphSummaryTotalItems > graphSummaryMaxAllowedItems) {
            result.push({
                text: `Note: Out of a total of ${graphSummaryTotalItems} items only the first ${graphSummaryMaxAllowedItems} are listed here.`,
                style: 'graphMoreItemsNote',
            });
        }
        result = {
            unbreakable: true,
            stack: result,
        };
        return result;
    }
    formatImage() {
        let imageElement = (this.format === 'svg') ? { svg: this.image } : { image: `data:image/png;base64,${this.image}` };
        if (this.width) {
            imageElement['width'] = this.width;
        }
        if (this.height) {
            imageElement['height'] = this.height;
        }
        return imageElement;
    }
    static formatTable(table, unitString) {
        return {
            layout: 'standardTableLayout',
            table: {
                headerRows: 1,
                keepWithHeaderRows: 1,
                widths: [3, '*', 'auto', 'auto'],
                body: [
                    [
                        { text: 'Item', colSpan: 2, alignment: 'center' },
                        '',
                        { text: `Avg${unitString}`, alignment: 'center' },
                        { text: `Max${unitString}`, alignment: 'center' },
                    ],
                    ...table.body.map(({ label, avg, max, color }) => [
                        { fillColor: color, text: '', margin: [0, -2] },
                        { text: label, noWrap: true, style: 'graphSummaryItemLabel', alignment: 'left', margin: [0, -2] },
                        { text: avg, alignment: 'right', margin: [0, -2] },
                        { text: max, alignment: 'right', margin: [0, -2] },
                    ]),
                ],
            },
        };
    }
    formatTables() {
        const ms = this.graphData.getMetricSpecification();
        let unitString = (ms && 'unit' in ms && ms.unit) ? MetricTools_1.MetricTools.makeUnitString(ms.unit, ms.defaultPrefix) : '';
        unitString = unitString ? ` [${unitString}]` : '';
        return {
            columnGap: 10,
            columns: (this.tables.length > 1) ?
                this.tables.map(t => ReportGraphItemData.formatTable(t, unitString))
                :
                    [
                        ReportGraphItemData.formatTable(this.tables[0], unitString),
                        { width: '50%', text: '' },
                    ]
        };
    }
    static prepareGraphData(data, itemIds, observedItems, timeRange, now) {
        const { start, end } = Reporter.getStartAndEnd(timeRange, data.interval, now);
        const graphData = new GraphData_1.GraphData(data.metric, itemIds, start, end, Reporter.getUserTimezone());
        graphData.stacked = data.stacked;
        graphData.setObjectItems(observedItems);
        return graphData;
    }
    static addImageProperties(graphData, config) {
        graphData.format = config.format;
        graphData.width = (graphData.format === "png") ? config.pngWidth : config.width;
        graphData.height = (graphData.format === "png") ? config.pngHeight : config.height;
        return graphData;
    }
    static getGraphSummaryMaxItems() {
        return reporter_configuration_1.default.graphSummary.maxItems;
    }
    static makeGraphTables(items, metricSpecification, maxItems) {
        const maxLength = reporter_configuration_1.default.graphSummary.labelMaxLength;
        const columns = reporter_configuration_1.default.graphSummary.columns;
        const maxItemsInLegend = maxItems !== undefined ? maxItems : ReportGraphItemData.getGraphSummaryMaxItems();
        const tableRows = [];
        const limit = items.length < maxItemsInLegend || maxItemsInLegend === 0 ? items.length : maxItemsInLegend;
        for (let i = 0; i < limit; i++) {
            const timeserieItem = items[i];
            let label = timeserieItem.metadata.name;
            if (label.length > maxLength) {
                label = label.substring(0, maxLength - 3) + '...';
            }
            let tableRow = {
                label: label,
                avg: MetricTools_1.MetricTools.formatNumber(timeserieItem.metadata.avg, metricSpecification),
                max: MetricTools_1.MetricTools.formatNumber(timeserieItem.metadata.max, metricSpecification),
                color: Color_1.Color.getIndexColor(i),
            };
            tableRows.push(tableRow);
        }
        const sepNumber = Math.ceil(limit / columns) < 6 ? 5 : Math.ceil((limit / columns));
        const tables = [];
        for (let i = 0; i < columns; i++) {
            const part = tableRows.slice(i * sepNumber, (i + 1) * sepNumber);
            if (part.length) {
                tables.push({ body: part });
            }
        }
        return tables;
    }
    async specificPromiseInitialization() {
        this.width = reporter_configuration_1.default.graph.width;
        this.height = reporter_configuration_1.default.graph.height;
        const format = reporter_configuration_1.default.graph.format;
        if (format === "svg" || format === "png") {
            this.format = format;
        }
        else {
            return this.handleError(`Format '${format}' is not supported`);
        }
        const data = this.reportItem.data;
        const interval = data.interval;
        this.timeRange = data.time_range;
        if (!(data.metric && (interval || this.timeRange))) {
            return this.handleMissingData();
        }
        this.setMaxItems(data.maxItems);
        this.graphData = ReportGraphItemData.prepareGraphData(data, this.observedItemIds, this.observedItemsArray, this.timeRange);
        ReportGraphItemData.addImageProperties(this.graphData, reporter_configuration_1.default.graph);
        await GraphData_factory_1.GraphDataFactory.fillGraphData(this.graphData);
        let items = this.graphData.getTimeserieItems();
        let metricSpecification = this.graphData.getMetricSpecification();
        this.setMetricLabel(data.metric, metricSpecification);
        if (!this.graphData.haveData() || items.length <= 0) {
            const { start, end } = Reporter.getStartAndEnd(data.time_range, data.interval, undefined);
            let timeserieItems = [];
            for (const label of this.graphData.getLabels()) {
                timeserieItems.push({
                    metadata: {
                        uuid: "itemId",
                        name: label,
                        start: start,
                        end: end,
                        interval: 1,
                        avg: "-",
                        max: "-",
                    },
                    timeseries: [{ start: null, end: null }]
                });
            }
            items = timeserieItems;
        }
        if (this.observedItemIds.length < 1) {
            return this.handleError('No item ids found');
        }
        else if (items.length < 1) {
            return this.handleError('No data found');
        }
        else if (!metricSpecification) {
            return this.handleError('No metric definitions found');
        }
        this.tables = ReportGraphItemData.makeGraphTables(items, metricSpecification, this.maxItems);
        try {
            this.image = await GraphData_factory_1.GraphDataFactory.getGraphImageData(this.graphData);
        }
        catch (e) {
            return this.handleError('An error occured', e);
        }
        return Promise.resolve(this);
    }
    setMaxItems(maxItems) {
        this.maxItems = maxItems !== undefined ? +maxItems : undefined;
    }
}
class ReportMetricGraphItemData extends ReportGraphItemData {
    constructor() {
        super(...arguments);
        this.captionPrefix = undefined;
    }
}
class ReportUnsupportedItemData extends ReportItemData {
    constructor(item) {
        super(item);
        this.specification = `${this.reportItem.format} ${this.reportItem.type}`;
    }
    getContentForPdfMake() {
        Reporter.winstonLogger.warn(`ReporterUnsupportedItemData: Unsupported item type ${this.specification}`);
        return {
            text: `Requested item ${this.specification} is not supported yet`,
            style: 'error',
        };
    }
}
class Reporter {
    static async ensureExportPath() {
        return (0, promises_1.mkdir)(Reporter.getReportTmpPath(), { recursive: true });
    }
    static getReportFilePath(reportFile) {
        return `${Reporter.EXPORT_PATH}/${reportFile.relative_path}`;
    }
    static getReportTmpPath() {
        return `${Reporter.EXPORT_PATH}/tmp`;
    }
    static sanitizeFileName(name) {
        const normalized = name.normalize("NFKD");
        let sanitized = normalized.replace(/\p{M}/gu, '');
        sanitized = sanitized.replace(/[^a-zA-Z0-9-]/g, '_');
        return sanitized;
    }
    static async generateReportPdfMakeContent(report, generatedAt, _request) {
        const timeRange = (report.time_range && report.time_range.date_from) ? report.time_range : null;
        report.report_items.sort((a, b) => (a.item_index > b.item_index ? 1 : -1));
        const dataPromises = [];
        let expandedReportItems = [];
        for (const reportItem of report.report_items) {
            if (reportItem.type === 'overview') {
                let data = reportItem.data;
                const { start, end } = this.getStartAndEnd(data.time_range, data.interval, undefined);
                const { ids, items } = await this.getIds(reportItem.item_group_id, reportItem.custom_group_id);
                for (const currentObjectItem of items) {
                    currentObjectItem;
                    let request = {
                        item_id: currentObjectItem.item_id,
                        start: start,
                        end: end
                    };
                    let tabs;
                    if (reportItem.class == 'storage') {
                        tabs = await Overview_1.Overview.storage(currentObjectItem, request, _request, this.user);
                    }
                    else if (reportItem['hw_type'] && reportItem.hw_type == 'power') {
                        tabs = await Overview_1.Overview.power(currentObjectItem, request);
                    }
                    else if (reportItem['hw_type'] && reportItem.hw_type == 'proxmox') {
                        tabs = await Overview_1.Overview.proxmox(currentObjectItem, request);
                    }
                    else if (reportItem['hw_type'] && reportItem.hw_type == 'nutanix') {
                        tabs = await Overview_1.Overview.nutanix(currentObjectItem, request);
                    }
                    else if (reportItem['hw_type'] && reportItem.hw_type == 'vmware') {
                        tabs = await Overview_1.Overview.vmware(currentObjectItem, request);
                    }
                    if (tabs) {
                        expandedReportItems.push(...await Overview_1.Overview.tabContentToReportItems(tabs, request));
                    }
                }
            }
            else {
                expandedReportItems.push(reportItem);
            }
        }
        for (const reportItem of expandedReportItems) {
            if (reportItem.type === "box") {
                const data = reportItem.data;
                if (data.type === "TableGroupBox") {
                    const newData = reportItem.data;
                    newData.tables.forEach(table => {
                        const individualizedData = { ...table, type: newData.type };
                        const individualized = { ...reportItem, data: individualizedData };
                        dataPromises.push(ReportItemData.makeDataPromise(individualized, timeRange));
                    });
                }
                else if (data.type === "CapacityTableBox" || data.type === "TableBox" || data.type === "TablePivotBox") {
                    dataPromises.push(ReportItemData.makeDataPromise(reportItem, timeRange));
                }
            }
            else if ((reportItem.format === reportItem_constants_1.REPORT_ITEM_FORMAT.graph && reportItem.type === reportItem_constants_1.REPORT_ITEM_TYPE.metric)
                || (reportItem.format === reportItem_constants_1.REPORT_ITEM_FORMAT.graph && reportItem.type === reportItem_constants_1.REPORT_ITEM_TYPE.custom_group)
                || (reportItem.format === reportItem_constants_1.REPORT_ITEM_FORMAT.table && reportItem.type === reportItem_constants_1.REPORT_ITEM_TYPE.top)) {
                let metrics = [undefined];
                if (Array.isArray(reportItem.data['metrics']) && (reportItem.data['metrics'].length > 0)) {
                    metrics = reportItem.data['metrics'];
                }
                else if (Array.isArray(reportItem.data['metric']) && (reportItem.data['metric'].length > 0)) {
                    Reporter.winstonLogger.warn('Deprecated attribute metric with array used (use metrics instead)');
                    metrics = reportItem.data['metric'];
                }
                else if (typeof reportItem.data['metric'] === "string") {
                    Reporter.winstonLogger.warn('Deprecated attribute metric with string used (use metrics with array instead');
                    metrics = [reportItem.data['metric']];
                }
                for (const metric of metrics) {
                    const individualizedData = { ...reportItem.data, metric: metric };
                    const individualized = { ...reportItem, data: individualizedData };
                    dataPromises.push(ReportItemData.makeDataPromise(individualized, timeRange));
                }
            }
            else {
                dataPromises.push(ReportItemData.makeDataPromise(reportItem, timeRange));
            }
        }
        const reportItemDataSet = await Promise.all(dataPromises);
        const collectedData = [
            new ReportTitleData(report.label),
            new ReportGeneratedAtData(generatedAt),
            ...reportItemDataSet,
        ];
        const contentForPdfMake = collectedData.map(i => i.getContentForPdfMake());
        return contentForPdfMake;
    }
    static async generateReport(report, request) {
        try {
            const generatedAt = new Date();
            const sanitizedLabel = Reporter.sanitizeFileName(report.label);
            const fileName = `report-${sanitizedLabel}-${(new Date()).getTime()}.pdf`;
            const reportFile = new reportFile_entity_1.ReportFile();
            reportFile.created_at = generatedAt;
            reportFile.file_name = fileName;
            if ('report_id' in report) {
                reportFile.report_id = report.report_id;
                reportFile.relative_path = `${(!report.on_generate_create_report_file) ? 'tmp/' : ''}${report.report_id}-${fileName}`;
            }
            else {
                reportFile.relative_path = `tmp/on_the_fly_${(0, uuid_1.v4)()}-${fileName}`;
            }
            if (!Array.isArray(report.report_items)) {
                Reporter.winstonLogger.warn("[GenerateReportWarning]: Report without array of report items.");
                report.report_items = [];
            }
            if (request || report['user_id']) {
                await this.setUser(request, report['user_id']);
                this.setAclGroups();
            }
            const path = Reporter.getReportFilePath(reportFile);
            const content = await Reporter.generateReportPdfMakeContent(report, generatedAt, request);
            const contentMadeAt = new Date();
            Reporter.winstonLogger.info(`Reporter: PdfMake Content for ${fileName} done in ${contentMadeAt.valueOf() - generatedAt.valueOf()} miliseconds.`);
            await PdfMakeWrapper.makeAndSave(content, path);
            const renderedAt = new Date();
            Reporter.winstonLogger.info(`Reporter: Rendering of ${fileName} done in ${renderedAt.valueOf() - generatedAt.valueOf()} miliseconds.`);
            return reportFile;
        }
        catch (e) {
            Reporter.winstonLogger.error("[GenerateReportError]: " + e);
            throw e;
        }
    }
    static async getIds(_item_group_id, _custom_group_id) {
        let items;
        if (_custom_group_id) {
            items = await CustomGroupWrapper_1.CustomGroupWrapper.getItems(_custom_group_id, true);
        }
        else {
            items = await ItemGroupWrapper_1.ItemGroupWrapper.getItems(_item_group_id, 'report', true);
        }
        let ids = items.map(item => item.item_id);
        if (this.aclGroups) {
            ids = await ACLCheck_1.ACLCheck.filterItems(this.aclGroups, ids);
            items = items.filter(item => ids.includes(item.item_id));
        }
        return { ids: ids, items: items };
    }
    static getStartAndEnd(timeRange, interval, endDate) {
        let start;
        let end = this.makeTimeNumberFromDate(endDate);
        if (timeRange) {
            start = this.makeTimeNumberFromDate(timeRange.date_from);
            if (timeRange.date_to) {
                end = this.makeTimeNumberFromDate(timeRange.date_to);
            }
        }
        else {
            start = end - this.secondsInTimeInterval(interval);
        }
        return { start: start, end: end };
    }
    static makeTimeNumberFromDate(date = undefined) {
        return Math.floor((date ? new Date(date) : new Date()).getTime() / 1000);
    }
    static secondsInTimeInterval(interval) {
        switch (interval) {
            case 'week':
                return 7 * 24 * 60 * 60;
            case 'month':
                return 31 * 24 * 60 * 60;
            case 'year':
                return 365 * 24 * 60 * 60;
            case 'day':
            default:
                return 1 * 24 * 60 * 60;
        }
    }
    static setAclGroups() {
        this.aclGroups = this.user.groups.map((group) => group.group_id);
    }
    static async setUser(request, userId) {
        if (userId) {
            this.user = await User_1.User.getUserById(userId, true);
        }
        else {
            this.user = await User_1.User.getUserByRequest(request, true);
        }
    }
    static getUserTimezone() {
        return this.user && 'timezone' in this.user ? this.user.timezone : "Etc/Utc";
    }
}
exports.Reporter = Reporter;
Reporter.EXPORT_PATH = (0, path_1.resolve)('./files/reporter');
Reporter.LOGS_PATH = (0, path_1.resolve)('./logs');
Reporter.winstonLogger = winston.createLogger({
    format: winston.format.combine(winston.format.timestamp(), Format_1.logFormat),
    transports: [
        new winston.transports.File({ filename: Reporter.LOGS_PATH + '/reporter-error.log', level: 'error' }),
        new winston.transports.File({ filename: Reporter.LOGS_PATH + '/reporter.log' }),
        new winston.transports.Console(),
    ],
});
//# sourceMappingURL=Reporter.js.map