"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReporterController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const report_dto_1 = require("../../common/report.dto");
const reportTemplate_dto_1 = require("../../common/reportTemplate.dto");
const fs_1 = require("fs");
const ReportTemplate_service_1 = require("./ReportTemplate.service");
const Reporter_1 = require("./Reporter");
const reporter_service_1 = require("./reporter.service");
const PM2_1 = require("../configuration/PM2");
const License_1 = require("../../config/License");
const xormon_exception_1 = require("../../exception/xormon.exception");
let ReporterController = class ReporterController {
    constructor(reporterService, reportTemplateService) {
        this.reporterService = reporterService;
        this.reportTemplateService = reportTemplateService;
    }
    async onModuleInit() {
        if (PM2_1.PM2.isMasterBackend()) {
            return Reporter_1.Reporter.ensureExportPath();
        }
    }
    async generateReportFromSchema(request, res, plainReport) {
        const reportFile = await Reporter_1.Reporter.generateReport(plainReport, request);
        const path = Reporter_1.Reporter.getReportFilePath(reportFile);
        const file = (0, fs_1.createReadStream)(path);
        res.set({
            'Content-Type': 'application/pdf',
            'Content-Disposition': `attachment; filename="${reportFile.file_name}"`,
        });
        return new common_1.StreamableFile(file);
    }
    async reports(request) {
        let userId = await this.reporterService.getUserId(request);
        const reports = await this.reporterService.getReportsOnly(userId);
        return {
            data: reports
        };
    }
    async report(request, report_id) {
        const report = await this.reporterService.getReportWithRelations(report_id);
        if (!report) {
            throw new common_1.NotFoundException(`Requested report with ID ${report_id} not found.`);
        }
        let userId = await this.reporterService.getUserId(request);
        if (report.user_id !== userId) {
            throw new common_1.ForbiddenException('You are not the owner of this report.');
        }
        delete report.user_id;
        report.report_items = this.reporterService.transformReportItem(report.report_items);
        return {
            data: report
        };
    }
    async generateReport(request, res, report_id) {
        const report = await this.reporterService.getReportWithItems(report_id);
        if (!report) {
            throw new common_1.NotFoundException(`Requested report with ID ${report_id} not found.`);
        }
        let userId = await this.reporterService.getUserId(request);
        if (report.user_id !== userId) {
            throw new common_1.ForbiddenException('You are not the owner of this report.');
        }
        const reportFile = await Reporter_1.Reporter.generateReport(report, request);
        if (report.on_generate_create_report_file) {
            await this.reporterService.saveReportFile(reportFile);
        }
        await this.reporterService.updateReportLastRunAt(report, reportFile.created_at);
        const path = Reporter_1.Reporter.getReportFilePath(reportFile);
        const file = (0, fs_1.createReadStream)(path);
        res.set({
            'Content-Type': 'application/pdf',
            'Content-Disposition': `attachment; filename="${reportFile.file_name}"`,
        });
        return new common_1.StreamableFile(file);
    }
    async updateReport(request, updateReport, report_id) {
        if (updateReport['report_id'] && updateReport['report_id'] !== report_id) {
            throw new common_1.BadRequestException('report_id mismatch');
        }
        if ('data' in updateReport) {
            throw new common_1.BadRequestException('Report PUT api called with deprecated attribute data');
        }
        if (Array.isArray(updateReport.report_items)) {
            updateReport.report_items.forEach(i => {
                if (i.report_id && i.report_id !== report_id) {
                    throw new common_1.BadRequestException('report_id in ReportItem mismatch');
                }
            });
        }
        const oldReport = await this.reporterService.getReportWithItems(report_id);
        if (!oldReport) {
            throw new common_1.NotFoundException(`Requested report with ID ${report_id} not found.`);
        }
        let userId = await this.reporterService.getUserId(request);
        if (oldReport.user_id !== userId) {
            throw new common_1.ForbiddenException('You are not the owner of this report.');
        }
        await this.reporterService.saveReport(updateReport, oldReport);
        const reportForResponse = await this.reporterService.getReportWithRelations(report_id);
        delete reportForResponse.user_id;
        reportForResponse.report_items = this.reporterService.transformReportItem(reportForResponse.report_items);
        return {
            message: `Updated report with id: ${report_id}`,
            data: reportForResponse
        };
    }
    async saveReport(request, newReport) {
        let userId = await this.reporterService.getUserId(request);
        newReport.user_id = userId;
        const isFree = !(await License_1.License.hasAnyValid());
        if (isFree) {
            const reports = await this.reporterService.getReportsOnly(userId);
            if (reports.length >= 1) {
                throw new xormon_exception_1.XormonException("You can  only have 1 report in free version", common_1.HttpStatus.PAYMENT_REQUIRED);
            }
        }
        if ('report_id' in newReport) {
            throw new common_1.MethodNotAllowedException('Report POST api called with Entity containing report_id. For update use PUT instead.');
        }
        if (Array.isArray(newReport.report_items)) {
            newReport.report_items.forEach(reportItem => {
                if ('report_id' in reportItem) {
                    throw new common_1.BadRequestException('existing report_id in ReportItem in new Report');
                }
                ;
                if ('report_item_id' in reportItem) {
                    throw new common_1.BadRequestException('existing report_item_id in ReportItem in new Report');
                }
                ;
            });
        }
        const savedReport = await this.reporterService.saveReport(newReport);
        const reportForResponse = await this.reporterService.getReportWithRelations(savedReport.report_id);
        reportForResponse.report_items = this.reporterService.transformReportItem(reportForResponse.report_items);
        delete reportForResponse.user_id;
        return {
            message: `A new report saved with id: ${savedReport.report_id}`,
            data: reportForResponse
        };
    }
    async cloneReport(request, report_id) {
        let userId = await this.reporterService.getUserId(request);
        const isFree = !(await License_1.License.hasAnyValid());
        if (isFree) {
            const reports = await this.reporterService.getReportsOnly(userId);
            if (reports.length >= 1) {
                throw new xormon_exception_1.XormonException("You can only have 1 report in free version", common_1.HttpStatus.PAYMENT_REQUIRED);
            }
        }
        let newReport = await this.reporterService.getReportWithRelations(report_id, false);
        delete newReport.report_id;
        let formattedDate = new Intl.DateTimeFormat('sv-SE', {
            year: 'numeric',
            month: 'numeric',
            day: 'numeric',
            hour: 'numeric',
            minute: 'numeric'
        }).format(new Date());
        newReport.label = `${newReport.label} clone ${formattedDate}`;
        if (Array.isArray(newReport.report_items)) {
            newReport.report_items.forEach((reportItem, index) => {
                newReport.report_items[index]['item_ids'] = reportItem._item_group.items.map(item => item.item_id);
                delete newReport.report_items[index].report_id;
                delete newReport.report_items[index].report_item_id;
                if (reportItem._item_group.parents) {
                    newReport.report_items[index]['parents'] = {
                        item_ids: reportItem._item_group.parents.map(item => item.item_id)
                    };
                }
                newReport.report_items[index]['regex'] = reportItem._item_group.regex;
                newReport.report_items[index]['regex_parents'] = reportItem._item_group.regex_parents;
                newReport.report_items[index]['tags'] = reportItem._item_group.tags.map(tag => tag.tag_value_id);
                newReport.report_items[index]['parent_tags'] = reportItem._item_group.parent_tags;
                delete reportItem._item_group;
            });
        }
        const savedReport = await this.reporterService.saveReport(newReport);
        const reportForResponse = await this.reporterService.getReportWithRelations(savedReport.report_id);
        reportForResponse.report_items = this.reporterService.transformReportItem(reportForResponse.report_items);
        delete reportForResponse.user_id;
        return {
            message: `A new report saved with id: ${savedReport.report_id}`,
            data: reportForResponse
        };
    }
    async deleteReport(request, report_id) {
        const oldReport = await this.reporterService.getReportWithRelations(report_id);
        if (!oldReport) {
            throw new common_1.NotFoundException(`Requested report with ID ${report_id} not found.`);
        }
        let userId = await this.reporterService.getUserId(request);
        if (oldReport.user_id !== userId) {
            throw new common_1.ForbiddenException('You are not the owner of this report.');
        }
        await this.reporterService.removeReport(oldReport);
        return {
            message: `Deleted report with id: ${report_id}`
        };
    }
    async getReportFile(request, report_file_id) {
        const reportFile = await this.reporterService.getReportFile(report_file_id);
        if (!reportFile) {
            throw new common_1.NotFoundException(`Requested report file with ID ${report_file_id} not found.`);
        }
        let userId = await this.reporterService.getUserId(request);
        if (reportFile.report.user_id !== userId) {
            throw new common_1.ForbiddenException('You are not the owner of related report.');
        }
        delete reportFile.report;
        return {
            data: reportFile
        };
    }
    async downloadReportFilePDF(request, res, report_file_id) {
        const reportFile = await this.reporterService.getReportFile(report_file_id);
        if (!reportFile) {
            throw new common_1.NotFoundException(`Requested report file with ID ${report_file_id} not found.`);
        }
        let userId = await this.reporterService.getUserId(request);
        if (reportFile.report.user_id !== userId) {
            throw new common_1.ForbiddenException('You are not the owner of related report.');
        }
        const path = Reporter_1.Reporter.getReportFilePath(reportFile);
        const file = (0, fs_1.createReadStream)(path);
        res.set({
            'Content-Type': 'application/pdf',
            'Content-Disposition': `attachment; filename="${reportFile.file_name}"`,
        });
        return new common_1.StreamableFile(file);
    }
    async deleteReportFile(request, report_file_id) {
        const reportFile = await this.reporterService.getReportFile(report_file_id);
        if (!reportFile) {
            throw new common_1.NotFoundException(`Requested report file with ID ${report_file_id} not found.`);
        }
        let userId = await this.reporterService.getUserId(request);
        if (reportFile.report.user_id !== userId) {
            throw new common_1.ForbiddenException('You are not the owner of related report.');
        }
        await this.reporterService.removeReportFile(reportFile);
        return {
            message: `Deleted report file with id: ${report_file_id}`
        };
    }
    async getReportTemplates() {
        return {
            data: await this.reportTemplateService.getReportTemplates(),
        };
    }
    async generateReportFromTemplate(request, templateData) {
        if (typeof templateData !== 'object' || !templateData) {
            throw new common_1.BadRequestException("Request body is required and should be an object");
        }
        if (!templateData.report_template_id) {
            throw new common_1.BadRequestException("Missing required parameter 'report_template_id'");
        }
        const reportTemplate = await this.reportTemplateService.getReportTemplateById(templateData.report_template_id);
        if (!reportTemplate) {
            throw new common_1.BadRequestException(`Report template with id "${templateData.report_template_id}" not found.`);
        }
        const errorInData = this.reportTemplateService.checkData(templateData.data, reportTemplate);
        if (errorInData) {
            throw new common_1.BadRequestException(errorInData);
        }
        return {
            data: {
                default_label: await this.reportTemplateService.makeDefaultReportLabel(reportTemplate, templateData.data),
                report_items: await this.reportTemplateService.makeReportItems(reportTemplate, templateData.data),
            }
        };
    }
};
exports.ReporterController = ReporterController;
__decorate([
    (0, common_1.Post)('v1/generate'),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)({ passthrough: true })),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, report_dto_1.PlainReport]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "generateReportFromSchema", null);
__decorate([
    (0, common_1.Get)('v1/reports'),
    openapi.ApiResponse({ status: 200, type: require("../../common/report.dto").GetReportsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "reports", null);
__decorate([
    (0, common_1.Get)('v1/reports/:report_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/report.dto").GetReportResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('report_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "report", null);
__decorate([
    (0, common_1.Get)('v1/reports/:report_id/generate'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)({ passthrough: true })),
    __param(2, (0, common_1.Param)('report_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, String]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "generateReport", null);
__decorate([
    (0, common_1.Put)('v1/reports/:report_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/report.dto").SaveReportResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Param)('report_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, report_dto_1.UpdateReport, String]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "updateReport", null);
__decorate([
    (0, common_1.Post)('v1/reports'),
    openapi.ApiResponse({ status: 201, type: require("../../common/report.dto").SaveReportResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, report_dto_1.NewReport]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "saveReport", null);
__decorate([
    (0, common_1.Post)('v1/clone/:report_id'),
    openapi.ApiResponse({ status: 201, type: require("../../common/report.dto").SaveReportResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('report_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "cloneReport", null);
__decorate([
    (0, common_1.Delete)('v1/reports/:report_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('report_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "deleteReport", null);
__decorate([
    (0, common_1.Get)('v1/files/:report_file_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/reportFile.dto").GetReportFileResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('report_file_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "getReportFile", null);
__decorate([
    (0, common_1.Get)('v1/files/:report_file_id/download'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)({ passthrough: true })),
    __param(2, (0, common_1.Param)('report_file_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, String]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "downloadReportFilePDF", null);
__decorate([
    (0, common_1.Delete)('v1/files/:report_file_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('report_file_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "deleteReportFile", null);
__decorate([
    (0, common_1.Get)('v1/templates'),
    openapi.ApiResponse({ status: 200, type: require("../../common/reportTemplate.dto").GetReportTemplatesResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "getReportTemplates", null);
__decorate([
    (0, common_1.Post)('v1/templates/generate'),
    openapi.ApiResponse({ status: 201, type: require("../../common/reportTemplate.dto").generateReportFromTemplateResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, reportTemplate_dto_1.generateReportFromTemplateRequestDTO]),
    __metadata("design:returntype", Promise)
], ReporterController.prototype, "generateReportFromTemplate", null);
exports.ReporterController = ReporterController = __decorate([
    (0, swagger_1.ApiTags)('Reporter'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/reporter'),
    __metadata("design:paramtypes", [reporter_service_1.ReporterService,
        ReportTemplate_service_1.ReportTemplateService])
], ReporterController);
//# sourceMappingURL=reporter.controller.js.map