"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SearchController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const Item_1 = require("../configuration/Item");
const URL_1 = require("../menu/menu/URL");
const RecentlySearchedWrapper_1 = require("./RecentlySearchedWrapper");
const search_interceptor_1 = require("../../interceptor/search.interceptor");
const Definitions_1 = require("../metrics/Definitions");
let SearchController = class SearchController {
    constructor() { }
    async searchAllItem(page, limit) {
        return await this.searchItem('', page, limit);
    }
    async searchItem(search, page, limit) {
        let searchResult;
        const regexKeywords = ['regex:', 'reg:', 'rg:', 're:'];
        const regexKeyword = regexKeywords.find(keyword => search.startsWith(keyword));
        const tagKeywords = ['tag:', '#'];
        const tagKeyword = tagKeywords.find(keyword => search.startsWith(keyword));
        if (search.startsWith(regexKeyword)) {
            const regex = search.slice(regexKeyword.length);
            searchResult = (0, Definitions_1.addRootParent)((0, Definitions_1.ObjectItemToArchItem)(await Item_1.Item.search(regex, true, false)));
        }
        else if (search.startsWith(tagKeyword)) {
            const tag = search.slice(tagKeyword.length);
            searchResult = (0, Definitions_1.addRootParent)((0, Definitions_1.ObjectItemToArchItem)(await Item_1.Item.search(tag, false, true)));
        }
        else {
            searchResult = (0, Definitions_1.addRootParent)((0, Definitions_1.ObjectItemToArchItem)(await Item_1.Item.search(search)));
        }
        let items = searchResult.map((result) => {
            let res = {
                item_id: result.item_id,
                label: result.label,
                hw_type: result.hw_type,
                subsystem: result.subsystem,
                class: result.class,
                root_parent: result.root_parent,
                url: URL_1.URL.generate(result.hw_type, result.subsystem, result.item_id),
            };
            return res;
        });
        items = items.filter(item => item.url !== null);
        limit = limit ? limit : items.length;
        const metadata = {
            start: page ? page * limit : 0,
            end: page
                ? page * limit + limit - 1 >= items.length
                    ? items.length
                    : page * limit + limit - 1
                : limit >= items.length
                    ? items.length - 1
                    : limit - 1,
            max: items.length - 1,
            page: page ? +page : 0,
        };
        if (metadata.start > metadata.max) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    metadata: {
                        start: metadata.start,
                        end: metadata.end,
                        max: metadata.max,
                        page: metadata.page,
                        info: 'the page number has a higher start index than the max index',
                    },
                    items: [],
                },
            };
        }
        if (items.length >= 1) {
            items = items.slice(metadata.start, metadata.end + 1);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: {
                metadata: metadata,
                items: items,
            },
        };
    }
    async getAllRecentlySearched(user_id, limit) {
        return await new RecentlySearchedWrapper_1.RecentlySearchedWrapper().getRecentlySearched(user_id, limit);
    }
    async addRecentlySearched(user_id, search_query) {
        if (!search_query || !user_id) {
            return {
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: "Can't add an empty query. Please provide both user_id and search_query.",
                data: [],
            };
        }
        await new RecentlySearchedWrapper_1.RecentlySearchedWrapper().addRecentlySearched(user_id, search_query);
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: 'Successfully added query: ' + "'" + search_query + "'" + ' to recently searched',
            data: await new RecentlySearchedWrapper_1.RecentlySearchedWrapper().findRecentlySearched(user_id, search_query).then((res) => res.data),
        };
    }
    async deleteRecentlySearched(user_id, search_query) {
        if (!search_query || !user_id) {
            return {
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Please provide both user_id and search_query, or consider deleting by ID instead.',
                data: [],
            };
        }
        await new RecentlySearchedWrapper_1.RecentlySearchedWrapper().deleteRecentlySearched(user_id, search_query);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Successfully deleted query: ' + "'" + search_query + "'" + ' from recently searched',
            data: [],
        };
    }
    async deleteRecentlySearchedId(recent_search_id) {
        if (!recent_search_id) {
            return {
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Please provide a recent_search_id to delete.',
                data: [],
            };
        }
        const deletedQuery = await new RecentlySearchedWrapper_1.RecentlySearchedWrapper().deleteRecentlySearchedById(recent_search_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Successfully deleted query: ' + "'" + deletedQuery + "'" + " with ID " + recent_search_id + ' from recently searched',
            data: [],
        };
    }
    async deleteAllRecentlySearched() {
        await new RecentlySearchedWrapper_1.RecentlySearchedWrapper().deleteAllRecentlySearched();
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Successfully deleted all queries from recently searched',
            data: [],
        };
    }
};
exports.SearchController = SearchController;
__decorate([
    (0, common_1.Get)('v1/item/'),
    (0, common_1.UseInterceptors)(search_interceptor_1.SearchInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/searchResponse.dto").SearchItemResponseDTO }),
    __param(0, (0, common_1.Query)('page')),
    __param(1, (0, common_1.Query)('limit')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, Number]),
    __metadata("design:returntype", Promise)
], SearchController.prototype, "searchAllItem", null);
__decorate([
    (0, common_1.Get)('v1/item/:search'),
    (0, common_1.UseInterceptors)(search_interceptor_1.SearchInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/searchResponse.dto").SearchItemResponseDTO }),
    __param(0, (0, common_1.Param)('search')),
    __param(1, (0, common_1.Query)('page')),
    __param(2, (0, common_1.Query)('limit')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Number, Number]),
    __metadata("design:returntype", Promise)
], SearchController.prototype, "searchItem", null);
__decorate([
    (0, common_1.Get)('v1/recently_searched'),
    openapi.ApiResponse({ status: 200, type: require("../../common/recentlySearched.dto").RecentlySearchedResponseDTO }),
    __param(0, (0, common_1.Query)('user_id')),
    __param(1, (0, common_1.Query)('limit')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Number]),
    __metadata("design:returntype", Promise)
], SearchController.prototype, "getAllRecentlySearched", null);
__decorate([
    (0, common_1.Post)('v1/recently_searched/add'),
    openapi.ApiResponse({ status: 201, type: require("../../common/recentlySearched.dto").RecentlySearchedResponseDTO }),
    __param(0, (0, common_1.Query)('user_id')),
    __param(1, (0, common_1.Query)('search_query')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], SearchController.prototype, "addRecentlySearched", null);
__decorate([
    (0, common_1.Post)('v1/recently_searched/delete'),
    openapi.ApiResponse({ status: 201, type: require("../../common/recentlySearched.dto").RecentlySearchedResponseDTO }),
    __param(0, (0, common_1.Query)('user_id')),
    __param(1, (0, common_1.Query)('search_query')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], SearchController.prototype, "deleteRecentlySearched", null);
__decorate([
    (0, common_1.Post)('v1/recently_searched/delete/id'),
    openapi.ApiResponse({ status: 201, type: require("../../common/recentlySearched.dto").RecentlySearchedResponseDTO }),
    __param(0, (0, common_1.Query)('recent_search_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], SearchController.prototype, "deleteRecentlySearchedId", null);
__decorate([
    (0, common_1.Post)('v1/recently_searched/delete/all'),
    openapi.ApiResponse({ status: 201, type: require("../../common/recentlySearched.dto").RecentlySearchedResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], SearchController.prototype, "deleteAllRecentlySearched", null);
exports.SearchController = SearchController = __decorate([
    (0, swagger_1.ApiTags)('Search'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/search'),
    __metadata("design:paramtypes", [])
], SearchController);
//# sourceMappingURL=search.controller.js.map