"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServerGroupService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const dataSource_1 = require("../../config/dataSource");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const serverGroup_entity_1 = require("../../entity/serverGroup.entity");
const serverGroupItems_entity_1 = require("../../entity/serverGroupItems.entity");
const typeorm_2 = require("typeorm");
const Linux_1 = require("../menu/menu/Linux");
const itemRelation_entity_1 = require("../../entity/itemRelation.entity");
let ServerGroupService = class ServerGroupService {
    constructor(objectItemRepository, serverGroupRepository, serverGroupItemsRepository) {
        this.objectItemRepository = objectItemRepository;
        this.serverGroupRepository = serverGroupRepository;
        this.serverGroupItemsRepository = serverGroupItemsRepository;
    }
    async getServerGroups(user_id, hw_types, _class) {
        return (await this.serverGroupRepository.find({
            where: [
                {
                    user_id: user_id,
                    hw_type: _class ? undefined : (0, typeorm_2.In)(hw_types),
                    class: _class ? _class : undefined,
                    server_group_items: {
                        isPrimary: true
                    }
                },
                {
                    user_id: user_id,
                    hw_type: _class ? undefined : (0, typeorm_2.In)(hw_types),
                    class: _class ? _class : undefined,
                }
            ],
            relations: {
                server_group_items: {
                    _item: true
                }
            }
        }));
    }
    async getServerGroupWithRelation(server_group_id) {
        return await this.serverGroupRepository.findOne({
            where: {
                server_group_id: server_group_id,
                server_group_items: {
                    isPrimary: true
                }
            },
            relations: {
                server_group_items: {
                    _item: true,
                }
            }
        });
    }
    async saveServerGroup(server_group) {
        let sGroup = null;
        let items;
        const queryRunner = (0, dataSource_1.getDataSource)().createQueryRunner();
        await queryRunner.connect();
        await queryRunner.startTransaction();
        try {
            sGroup = await queryRunner.manager.getRepository(serverGroup_entity_1.ServerGroup).save({
                label: server_group.label,
                description: server_group.description,
                user_id: server_group.user_id,
                hw_type: server_group.class ? undefined : server_group.hw_type,
                class: server_group.class
            });
            const children = await queryRunner.manager.getRepository(itemRelation_entity_1.ItemRelation).findBy({
                parent: (0, typeorm_2.In)(server_group.items.map(i => i))
            });
            const ids = server_group.items.concat(children.map(i => i.child));
            items = server_group.items.map((item) => {
                return {
                    server_group_id: sGroup.server_group_id,
                    item_id: item,
                    isPrimary: true
                };
            });
            await queryRunner.manager.getRepository(serverGroupItems_entity_1.ServerGroupItems).save(items.concat(children.map((item) => {
                return {
                    server_group_id: sGroup.server_group_id,
                    item_id: item.child,
                    isPrimary: false
                };
            })));
            const parent = await queryRunner.manager.getRepository(objectItem_entity_1.ObjectItem).save({
                item_id: sGroup.server_group_id,
                hw_type: server_group.hw_type,
                object_id: server_group.hw_type,
                subsystem: "group",
                label: sGroup.label
            });
            await queryRunner.manager.getRepository(itemRelation_entity_1.ItemRelation).save(ids.map((id) => {
                return {
                    parent: parent.item_id,
                    child: id,
                    origin: itemRelation_entity_1.RelationOrigin.parent
                };
            }));
            sGroup.server_group_items = items;
            await queryRunner.commitTransaction();
        }
        catch (err) {
            console.log(err);
            await queryRunner.rollbackTransaction();
            await queryRunner.release();
            throw err;
        }
        finally {
            await queryRunner.release();
        }
        return sGroup;
    }
    async getValidItems(hw_type, _class) {
        if (_class) {
            return await this.objectItemRepository.find({
                where: {
                    hw_type_entity: {
                        class: _class
                    },
                    subsystem: (0, typeorm_2.In)(['device', 'server', 'cluster', 'db', 'vcenter', 'host']),
                },
                relations: {
                    hw_type_entity: true
                }
            });
        }
        else if (hw_type && hw_type !== "linux") {
            return await this.objectItemRepository.findBy({
                hw_type: hw_type,
                subsystem: (0, typeorm_2.In)(['device', 'server', 'cluster', 'db', 'vcenter', 'host']),
            });
        }
        return await this.objectItemRepository.findBy({
            hw_type: (0, typeorm_2.In)(["linux"]),
            subsystem: "server",
            item_id: (0, typeorm_2.Not)((0, typeorm_2.In)(await (0, Linux_1.getLinuxExcludedIds)()))
        });
    }
    async updateServerGroup(server_group_id, server_group) {
        const queryRunner = (0, dataSource_1.getDataSource)().createQueryRunner();
        await queryRunner.connect();
        await queryRunner.startTransaction();
        let items = [];
        try {
            await this.serverGroupRepository.update({
                server_group_id: server_group_id
            }, {
                label: server_group.label ? server_group.label : undefined,
                description: server_group.description ? server_group.description : undefined
            });
            const existingItems = await this.serverGroupItemsRepository.findBy({ server_group_id: server_group_id });
            const idsToDelete = existingItems.filter((sgItem => !server_group.items.includes(sgItem.item_id)));
            await this.serverGroupItemsRepository.remove(idsToDelete.map(sgIt => {
                return {
                    server_group_id: server_group_id,
                    item_id: sgIt.item_id,
                    isPrimary: sgIt.isPrimary
                };
            }));
            const children = await queryRunner.manager.getRepository(itemRelation_entity_1.ItemRelation).findBy({
                parent: (0, typeorm_2.In)(server_group.items.map(i => i))
            });
            const ids = server_group.items.concat(children.map(i => i.child));
            items = server_group.items.map((item) => {
                return {
                    server_group_id: server_group_id,
                    item_id: item,
                    isPrimary: true
                };
            });
            await queryRunner.manager.getRepository(serverGroupItems_entity_1.ServerGroupItems).save(items.concat(children.map((item) => {
                return {
                    server_group_id: server_group_id,
                    item_id: item.child,
                    isPrimary: false
                };
            })));
            await queryRunner.manager.getRepository(itemRelation_entity_1.ItemRelation).delete({
                parent: server_group_id
            });
            await queryRunner.manager.getRepository(itemRelation_entity_1.ItemRelation).save(ids.map((id) => {
                return {
                    parent: server_group_id,
                    child: id,
                    origin: itemRelation_entity_1.RelationOrigin.parent
                };
            }));
            await queryRunner.commitTransaction();
        }
        catch (err) {
            console.log(err);
            await queryRunner.rollbackTransaction();
            await queryRunner.release();
            throw err;
        }
        finally {
            await queryRunner.release();
        }
        return await this.serverGroupRepository.findOne({
            where: {
                server_group_id: server_group_id
            },
            relations: {
                server_group_items: {
                    _item: true
                }
            }
        });
    }
    async deleteServerGroup(server_group_id) {
        const queryRunner = (0, dataSource_1.getDataSource)().createQueryRunner();
        await queryRunner.connect();
        await queryRunner.startTransaction();
        let deleted = null;
        try {
            await queryRunner.manager.getRepository(objectItem_entity_1.ObjectItem).delete({
                item_id: server_group_id
            });
            deleted = await this.serverGroupRepository.delete(server_group_id);
            await queryRunner.commitTransaction();
        }
        catch (err) {
            console.log(err);
            await queryRunner.rollbackTransaction();
            throw err;
        }
        finally {
            await queryRunner.release();
        }
        return deleted;
    }
    async checkOwnership(server_group_id, user_id) {
        return await this.serverGroupRepository.findOneBy({
            server_group_id: server_group_id,
            user_id: user_id
        }) ? true : false;
    }
};
exports.ServerGroupService = ServerGroupService;
exports.ServerGroupService = ServerGroupService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(1, (0, typeorm_1.InjectRepository)(serverGroup_entity_1.ServerGroup)),
    __param(2, (0, typeorm_1.InjectRepository)(serverGroupItems_entity_1.ServerGroupItems)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], ServerGroupService);
//# sourceMappingURL=serverGroup.service.js.map