"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StorageGroupService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const storageGroup_entity_1 = require("../../entity/storageGroup.entity");
const storageGroupDevice_entity_1 = require("../../entity/storageGroupDevice.entity");
const storageGroupRegex_entity_1 = require("../../entity/storageGroupRegex.entity");
const typeorm_2 = require("typeorm");
let StorageGroupService = class StorageGroupService {
    constructor(storageGroupsRepository, storageGroupDevicesRepository, storageGroupRegexRepository, objectItemsRepository) {
        this.storageGroupsRepository = storageGroupsRepository;
        this.storageGroupDevicesRepository = storageGroupDevicesRepository;
        this.storageGroupRegexRepository = storageGroupRegexRepository;
        this.objectItemsRepository = objectItemsRepository;
    }
    async getStorageGroups(userId) {
        return await this.storageGroupsRepository.find({
            relations: {
                storage_group_devices: true
            },
            where: {
                user_id: userId,
                group_id: (0, typeorm_2.IsNull)()
            }
        });
    }
    async getGroupStorageGroups(groupIds) {
        return await this.storageGroupsRepository.find({
            relations: {
                storage_group_devices: true
            },
            where: {
                group_id: (0, typeorm_2.In)(groupIds)
            }
        });
    }
    async getStorageGroupsWithRelations(userId, groupIds) {
        return await this.storageGroupsRepository.find({
            relations: {
                storage_group_regex: true,
                storage_group_devices: {
                    _item: true
                },
                _group: true
            },
            where: [{
                    user_id: userId,
                    group_id: (0, typeorm_2.IsNull)()
                }, {
                    group_id: (0, typeorm_2.In)(groupIds)
                }]
        });
    }
    async findStorageGroupByLabel(label) {
        return await this.storageGroupsRepository.findOne({
            where: { label: label },
            relations: {
                _group: true
            }
        });
    }
    async getStorageGroup(storage_group_id) {
        return await this.storageGroupsRepository.findOne({
            where: { storage_group_id: storage_group_id },
            relations: {
                storage_group_regex: true,
                storage_group_devices: {
                    _item: true
                },
                _group: true
            }
        });
    }
    async saveStorageGroup(storage_group, user_id) {
        const sgroup = await this.storageGroupsRepository.save({
            label: storage_group.label,
            description: storage_group.description,
            user_id: user_id ? user_id : undefined,
            group_id: storage_group.group_id ? storage_group.group_id : undefined
        });
        if (storage_group.devices && storage_group.devices.length >= 1) {
            const devices = await this.objectItemsRepository.find({ where: { subsystem: 'device', item_id: (0, typeorm_2.In)(storage_group.devices) } });
            const sgroupDevices = devices.map((device) => {
                return {
                    storage_group_id: sgroup.storage_group_id,
                    item_id: device.item_id,
                };
            });
            await this.storageGroupDevicesRepository.save(sgroupDevices);
        }
        if (storage_group.regex && storage_group.regex.length >= 1) {
            const sgroupRegex = storage_group.regex.map((regex) => {
                return {
                    storage_group_id: sgroup.storage_group_id,
                    regex: regex,
                };
            });
            await this.storageGroupRegexRepository.save(sgroupRegex);
        }
        return await this.getStorageGroup(sgroup.storage_group_id);
    }
    async updateStorageGroup(storage_group_id, storage_group) {
        const sgroup = await this.storageGroupsRepository.save({
            storage_group_id: storage_group_id,
            label: storage_group.label,
            description: storage_group.description,
        });
        if (storage_group.devices && storage_group.devices.length >= 1) {
            const devices = await this.objectItemsRepository.find({ where: { subsystem: 'device', item_id: (0, typeorm_2.In)(storage_group.devices) } });
            const sgroupDevices = devices.map((device) => {
                return {
                    storage_group_id: sgroup.storage_group_id,
                    item_id: device.item_id,
                };
            });
            await this.storageGroupDevicesRepository.delete({ storage_group_id: sgroup.storage_group_id });
            await this.storageGroupDevicesRepository.save(sgroupDevices);
        }
        if (storage_group.regex && storage_group.regex.length >= 1) {
            const sgroupRegex = storage_group.regex.map((regex) => {
                return {
                    storage_group_id: sgroup.storage_group_id,
                    regex: regex,
                };
            });
            await this.storageGroupRegexRepository.delete({ storage_group_id: sgroup.storage_group_id });
            await this.storageGroupRegexRepository.save(sgroupRegex);
        }
        return await this.getStorageGroup(sgroup.storage_group_id);
    }
    async checkStorageGroupOwner(storage_group_id, user_id) {
        const sg = await this.storageGroupsRepository.findOneBy({
            storage_group_id: storage_group_id,
            user_id: user_id
        });
        return sg ? true : false;
    }
    async deleteStorageGroup(storage_group_id) {
        return await this.storageGroupsRepository.delete(storage_group_id);
    }
    async regex(regex) {
        if (regex.length === 0) {
            return [];
        }
        let regexRule;
        for (const regexItem of regex) {
            if (!regexRule) {
                regexRule = "(object_items.label ~ '" + regexItem + "'";
            }
            else {
                regexRule += " OR object_items.label ~ '" + regexItem + "'";
            }
        }
        regexRule += ')';
        return await this.objectItemsRepository
            .createQueryBuilder('object_items')
            .where("object_items.subsystem = 'DEVICE' AND " + regexRule)
            .getMany();
    }
};
exports.StorageGroupService = StorageGroupService;
exports.StorageGroupService = StorageGroupService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(storageGroup_entity_1.StorageGroup)),
    __param(1, (0, typeorm_1.InjectRepository)(storageGroupDevice_entity_1.StorageGroupDevice)),
    __param(2, (0, typeorm_1.InjectRepository)(storageGroupRegex_entity_1.StorageGroupRegex)),
    __param(3, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], StorageGroupService);
//# sourceMappingURL=storage_group.service.js.map