"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TagWrapper = void 0;
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const typeorm_1 = require("typeorm");
const dataSource_1 = require("../../config/dataSource");
const Classes_1 = require("../metrics/Classes");
const hwType_entity_1 = require("../../entity/hwType.entity");
const URL_1 = require("../menu/menu/URL");
const tagValueItem_entity_1 = require("../../entity/tagValueItem.entity");
const ItemRegexWrapper_1 = require("../architecture/ItemRegexWrapper");
const tag_entity_1 = require("../../entity/tag.entity");
const tagValue_entity_1 = require("../../entity/tagValue.entity");
const Definitions_1 = require("../metrics/Definitions");
class TagWrapper {
    static async getItemsByTagParents(parents, tags, hwTypeOrClass, intersection = false, subsystem) {
        if (tags.length === 0) {
            return [];
        }
        if (parents.length === 0) {
            return await TagWrapper.getItemsByTag(tags, hwTypeOrClass, intersection, subsystem);
        }
        let parentsString = '';
        const operatedParents = parents.map(parent => 'parents.parent = \'' + parent + '\'');
        parentsString = intersection ? operatedParents.join(' AND ') : operatedParents.join(' OR ');
        const itemTemp = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem)
            .createQueryBuilder('object_items')
            .leftJoin('object_items.parents', 'parents')
            .where(parentsString);
        if (hwTypeOrClass === 'storage' || hwTypeOrClass === 'san') {
            const hwTypes = (await (0, dataSource_1.getDataSource)().getRepository(hwType_entity_1.HwType).findBy({ class: hwTypeOrClass }))
                .map(hwType => hwType.hw_type);
            itemTemp.andWhere('object_items.hw_type IN (:...hwTypes)', {
                hwTypes: hwTypes
            });
        }
        else {
            itemTemp.andWhere('object_items.hw_type = :hwType', {
                hwType: hwTypeOrClass
            });
        }
        if (subsystem) {
            itemTemp.andWhere('object_items.subsystem = :subsystem', {
                subsystem: subsystem
            });
        }
        if (tags) {
            let tagsString = '';
            const operatedTags = tags.map(tag => 'tags.tag = \'' + tag + '\'');
            tagsString = operatedTags.join(' OR ');
            itemTemp.andWhere('object_items.item_id IN ' +
                '(SELECT tag_items.item_id FROM tag_items ' +
                'INNER JOIN tags ON tag_items.tag_id = tags.tag_id ' +
                'WHERE ' + tagsString + ')');
        }
        const objectItems = await itemTemp.orderBy('object_items.label', 'ASC').getMany();
        return objectItems.map(objectItem => ({
            item_id: objectItem.item_id,
            label: objectItem.label,
            hw_type: objectItem.hw_type,
            subsystem: objectItem.subsystem,
        }));
    }
    static async getItemsByTag(tags, hwTypeOrClass, intersection = false, subsystem) {
        if (tags.length === 0) {
            return [];
        }
        let hwTypes = [];
        if (hwTypeOrClass === 'storage') {
            hwTypes = Object.keys(Classes_1.Storage.technology);
        }
        else if (hwTypeOrClass === 'san') {
            hwTypes = Object.keys(Classes_1.San.technology);
        }
        else {
            hwTypes.push(hwTypeOrClass);
        }
        let tagsString = '';
        const operatedTags = tags.map(tag => 'tags.tag = \'' + tag + '\'');
        tagsString = intersection ? operatedTags.join(' AND ') : operatedTags.join(' OR ');
        let objectItems = await (0, dataSource_1.getDataSource)()
            .getRepository(objectItem_entity_1.ObjectItem)
            .createQueryBuilder('object_items')
            .where('object_items.hw_type IN (:...hwTypes)', { hwTypes: hwTypes })
            .andWhere(subsystem ? 'object_items.subsystem = :subsystem' : '1=1', { subsystem: subsystem })
            .andWhere('object_items.item_id IN ' +
            '(SELECT tag_items.item_id FROM tag_items ' +
            'INNER JOIN tags ON tag_items.tag_id = tags.tag_id ' +
            'WHERE ' + tagsString + ')')
            .orderBy('object_items.label', 'ASC')
            .getMany();
        return objectItems.map(objectItem => ({
            item_id: objectItem.item_id,
            label: objectItem.label,
            hw_type: objectItem.hw_type,
            subsystem: objectItem.subsystem,
        }));
    }
    static getFullItemsForTag(tagToUse) {
        return (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem)
            .createQueryBuilder('object_items')
            .leftJoinAndSelect("object_items.hw_type_entity", "hw_types")
            .leftJoinAndSelect("hw_types._class", "class")
            .leftJoinAndSelect("object_items.tags", "tag_items")
            .leftJoinAndSelect("tag_items._tag", "tags")
            .where('tags.tag = :tagToUse', { tagToUse: tagToUse })
            .getMany()
            .then(items => {
            return items.map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label,
                    hw_type: item.hw_type,
                    subsystem: item.subsystem,
                    class: item.hw_type_entity._class?.label,
                    parents: item.parents ? item.parents.map(parent => ({
                        item_id: parent._parent.item_id,
                        label: parent._parent.label,
                        hw_type: parent._parent.hw_type,
                        subsystem: parent._parent.subsystem,
                        class: parent._parent.hw_type_entity?._class?.label,
                        url: URL_1.URL.generate(parent._parent.hw_type, parent._parent.subsystem, parent._parent.item_id),
                    })) : [],
                    url: URL_1.URL.generate(item.hw_type, item.subsystem, item.item_id),
                };
            });
        });
    }
    static getFullItemsForTagById(tagIdToUse) {
        return (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem)
            .createQueryBuilder('object_items')
            .leftJoinAndSelect("object_items.hw_type_entity", "hw_types")
            .leftJoinAndSelect("hw_types._class", "class")
            .leftJoinAndSelect("object_items.tags", "tags")
            .where('tags.tag_id = :tagIdToUse', { tagIdToUse: tagIdToUse })
            .getMany()
            .then(items => {
            return items.map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label,
                    hw_type: item.hw_type,
                    subsystem: item.subsystem,
                    class: item.hw_type_entity._class?.label,
                    parents: null,
                    url: URL_1.URL.generate(item.hw_type, item.subsystem, item.item_id),
                };
            });
        });
    }
    static async getFullItemsForItemIds(itemIds) {
        return (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem)
            .createQueryBuilder('object_items')
            .leftJoinAndSelect("object_items.hw_type_entity", "hw_types")
            .leftJoinAndSelect("hw_types._class", "class")
            .leftJoinAndSelect("object_items.parents", "parents")
            .leftJoinAndSelect("parents._parent", "parent_items")
            .where('object_items.item_id IN (:...itemIds)', { itemIds: itemIds })
            .getMany()
            .then(itemsWithRelations => {
            let transformedItems = itemsWithRelations.map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label,
                    hw_type: item.hw_type,
                    subsystem: item.subsystem,
                    class: item.hw_type_entity?._class?.label,
                    parents: item.parents ? (0, Definitions_1.ObjectItemToArchItem)(item.parents.map(parent => parent._parent)) : [],
                    url: URL_1.URL.generate(item.hw_type, item.subsystem, item.item_id),
                };
            });
            let itemsWithRootParents = (0, Definitions_1.addRootParent)(transformedItems);
            return itemsWithRootParents;
        });
    }
    static async getFilteredItemsForItemIds(item_ids, hw_type, subsystem, class_name) {
        return (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            relations: {
                hw_type_entity: {
                    _class: true
                },
                parents: {
                    _parent: {
                        hw_type_entity: true
                    }
                }
            },
            where: {
                item_id: (0, typeorm_1.In)(item_ids),
                hw_type: hw_type ? hw_type : undefined,
                subsystem: subsystem ? subsystem : undefined,
                hw_type_entity: {
                    class: class_name ?? undefined
                }
            }
        }).then(itemsWithRelations => {
            let transformedItems = itemsWithRelations.map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label,
                    hw_type: item.hw_type,
                    subsystem: item.subsystem,
                    class: item.hw_type_entity?._class?.label,
                    parents: item.parents ? (0, Definitions_1.ObjectItemToArchItem)(item.parents.map(parent => parent._parent)) : [],
                    url: URL_1.URL.generate(item.hw_type, item.subsystem, item.item_id),
                };
            });
            let itemsWithRootParents = (0, Definitions_1.addRootParent)(transformedItems);
            return itemsWithRootParents;
        });
    }
    static async getFullObjectItemsForItemIds(itemIds) {
        return (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem)
            .createQueryBuilder('object_items')
            .leftJoinAndSelect("object_items.hw_type_entity", "hw_types")
            .leftJoinAndSelect("hw_types._class", "class")
            .leftJoinAndSelect("object_items.parents", "parents")
            .leftJoinAndSelect("parents._parent", "parent_items")
            .where('object_items.item_id IN (:...itemIds)', { itemIds: itemIds })
            .getMany();
    }
    static async getItems(hwTypeOrClass, subsystem, tags) {
        const hwTypes = ['storage', 'san', 'virtualization', 'lan'].includes(hwTypeOrClass)
            ? ItemRegexWrapper_1.ItemRegexWrapper.getClassHwTypes(hwTypeOrClass)
            : [hwTypeOrClass];
        let items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            relations: {
                tag_value_items: true
            }, where: {
                subsystem: subsystem,
                hw_type: (0, typeorm_1.In)(hwTypes),
                tag_value_items: {
                    tag_value_id: (0, typeorm_1.In)(tags)
                }
            }
        });
        return items;
    }
    static async getItemsByParents(parents, hwTypeOrClass, subsystem, tags, intersection = false) {
        if (tags.length === 0) {
            return [];
        }
        if (!parents || parents.length === 0) {
            return await this.getItems(hwTypeOrClass, subsystem, tags);
        }
        const hwTypes = ['storage', 'san', 'virtualization', 'lan'].includes(hwTypeOrClass)
            ? ItemRegexWrapper_1.ItemRegexWrapper.getClassHwTypes(hwTypeOrClass)
            : [hwTypeOrClass];
        let items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            relations: {
                parents: true,
                tag_value_items: true
            }, where: {
                subsystem: subsystem,
                hw_type: (0, typeorm_1.In)(hwTypes),
                parents: {
                    parent: (0, typeorm_1.In)(parents)
                },
                tag_value_items: {
                    tag_value_id: (0, typeorm_1.In)(tags)
                }
            }
        });
        if (intersection) {
            items = items.filter(item => item.parents.length === parents.length);
        }
        return items;
    }
    static async getItemsByRegexParentsRegex(parentsRegex, hwTypeOrClass, subsystem, tags, intersection = false) {
        if (tags.length === 0) {
            return [];
        }
        if (!parentsRegex || parentsRegex.length === 0) {
            return await this.getItems(hwTypeOrClass, subsystem, tags);
        }
        if (intersection) {
            let queryBuilder = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem)
                .createQueryBuilder('object_items')
                .leftJoin("object_items.parents", "parents")
                .leftJoin("object_items.tag_value_items", "tag_value_items");
            let c = 1;
            for (const parentRegex of parentsRegex) {
                queryBuilder = queryBuilder.leftJoin("object_items.parents", `parents${c}`)
                    .leftJoin(`parents${c}._parent`, `parent${c}`);
                c++;
            }
            if (['storage', 'san', 'virtualization', 'lan'].includes(hwTypeOrClass)) {
                const hwTypes = ItemRegexWrapper_1.ItemRegexWrapper.getClassHwTypes(hwTypeOrClass);
                queryBuilder = queryBuilder.where('object_items.hw_type IN (:...hwTypes) AND object_items.subsystem = :subsystem AND tag_value_items.tag_value_id IN (:...tags)', {
                    hwTypes: hwTypes,
                    subsystem: subsystem,
                    tags: tags
                });
            }
            else {
                queryBuilder = queryBuilder.where('object_items.hw_type = :hwType AND object_items.subsystem = :subsystem AND tag_value_items.tag_value_id IN (:...tags)', {
                    hwType: hwTypeOrClass,
                    subsystem: subsystem,
                    tags: tags
                });
            }
            const useAlias = await ItemRegexWrapper_1.ItemRegexWrapper.isAnyAlias();
            queryBuilder = queryBuilder.andWhere(new typeorm_1.Brackets((qb) => {
                let i = 1;
                for (const parentRegex of parentsRegex) {
                    const parentRegexFilter = useAlias
                        ? `((parent${i}.label ~ :regex${i} and parent${i}.alias is null) OR (parent${i}.alias ~ :regex${i} and parent${i}.alias is not null))`
                        : `parent${i}.label ~ :regex${i}`;
                    if (i === 1) {
                        qb.where(`parent${i}.subsystem = :subsystem${i} AND ${parentRegexFilter}`, {
                            [`subsystem${i}`]: parentRegex.subsystem,
                            [`regex${i}`]: parentRegex.regex,
                        });
                    }
                    else {
                        qb.andWhere(`parent${i}.subsystem = :subsystem${i} AND ${parentRegexFilter}`, {
                            [`subsystem${i}`]: parentRegex.subsystem,
                            [`regex${i}`]: parentRegex.regex,
                        });
                    }
                    i++;
                }
            }));
            return await queryBuilder.orderBy('object_items.label', 'ASC')
                .getMany();
        }
        else {
            let parents = [];
            for (const parentRegex of parentsRegex) {
                const items = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegex(hwTypeOrClass, parentRegex.subsystem, [parentRegex.regex]);
                parents = parents.concat(items.map(item => item.item_id));
            }
            if (parents.length === 0) {
                return [];
            }
            return this.getItemsByParents(parents, hwTypeOrClass, subsystem, tags);
        }
    }
    static async getItemsByParentsTag(parentTags, hwTypeOrClass, subsystem, tags, intersection = false) {
        if (tags.length === 0) {
            return [];
        }
        if (!parentTags || parentTags.length === 0) {
            return await this.getItems(hwTypeOrClass, subsystem, tags);
        }
        if (intersection) {
            let queryBuilder = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem)
                .createQueryBuilder('object_items')
                .leftJoin("object_items.tag_value_items", "tag_value_items");
            let c = 1;
            for (const parentTag of parentTags) {
                queryBuilder = queryBuilder.leftJoin("object_items.parents", `parents${c}`)
                    .leftJoin(`parents${c}._parent`, `parent${c}`)
                    .leftJoin(`parent${c}.tag_value_items`, `tag_value_items${c}`);
                c++;
            }
            if (['storage', 'san', 'virtualization', 'lan'].includes(hwTypeOrClass)) {
                const hwTypes = ItemRegexWrapper_1.ItemRegexWrapper.getClassHwTypes(hwTypeOrClass);
                queryBuilder = queryBuilder.where('object_items.hw_type IN (:...hwTypes) AND object_items.subsystem = :subsystem AND tag_value_items.tag_value_id IN (:...tags)', {
                    hwTypes: hwTypes,
                    subsystem: subsystem,
                    tags: tags
                });
            }
            else {
                queryBuilder = queryBuilder.where('object_items.hw_type = :hwType AND object_items.subsystem = :subsystem AND tag_value_items.tag_value_id IN (:...tags)', {
                    hwType: hwTypeOrClass,
                    subsystem: subsystem,
                    tags: tags
                });
            }
            queryBuilder = queryBuilder.andWhere(new typeorm_1.Brackets((qb) => {
                let i = 1;
                for (const parentTag of parentTags) {
                    if (i === 1) {
                        qb.where(`tag_value_items${i}.tag_value_id = :tag${i} AND parent${i}.subsystem = :subsystem${i}`, {
                            [`tag${i}`]: parentTag.tag,
                            [`subsystem${i}`]: parentTag.subsystem,
                        });
                    }
                    else {
                        qb.andWhere(`tag_value_items${i}.tag_value_id = :tag${i} AND parent${i}.subsystem = :subsystem${i}`, {
                            [`tag${i}`]: parentTag.tag,
                            [`subsystem${i}`]: parentTag.subsystem,
                        });
                    }
                    i++;
                }
            }));
            return await queryBuilder.orderBy('object_items.label', 'ASC')
                .getMany();
        }
        else {
            let parents = [];
            const tagValueItems = await (0, dataSource_1.getDataSource)().getRepository(tagValueItem_entity_1.TagValueItem).find({
                relations: {
                    _item: true
                },
                where: parentTags.map(pTag => {
                    return {
                        tag_value_id: pTag.tag,
                        _item: {
                            subsystem: pTag.subsystem
                        }
                    };
                })
            });
            parents = tagValueItems.map(vItem => vItem.item_id);
            if (parents.length === 0) {
                return [];
            }
            return this.getItemsByParents(parents, hwTypeOrClass, subsystem, tags);
        }
    }
    static async tagSearch(tag_label, value) {
        const tag = await (0, dataSource_1.getDataSource)().getRepository(tag_entity_1.Tag).findOne({ where: { label: tag_label } });
        if (!tag) {
            return [];
        }
        const tagValue = await (0, dataSource_1.getDataSource)().getRepository(tagValue_entity_1.TagValue).findOne({ where: { tag_id: tag.tag_id, value: value } });
        if (!tagValue) {
            return [];
        }
        const item_ids = [];
        const tagValueItems = await (0, dataSource_1.getDataSource)().getRepository(tagValueItem_entity_1.TagValueItem).find({ where: { tag_value_id: tagValue.tag_value_id } });
        for (const tagValueItem of tagValueItems) {
            item_ids.push(tagValueItem.item_id);
        }
        if (item_ids.length === 0) {
            return [];
        }
        return await this.getFullObjectItemsForItemIds(item_ids);
    }
}
exports.TagWrapper = TagWrapper;
//# sourceMappingURL=TagWrapper.js.map