"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var TokenController_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.TokenController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const config_1 = require("@nestjs/config");
const event_emitter_1 = require("@nestjs/event-emitter");
const swagger_1 = require("@nestjs/swagger");
const tokenRequest_dto_1 = require("../../common/tokenRequest.dto");
const auditEvent_entity_1 = require("../../entity/auditEvent.entity");
const joiValidation_pipe_1 = require("../../pipe/joiValidation.pipe");
const Audit_1 = require("../configuration/Audit");
const Settings_1 = require("../configuration/Settings");
const User_1 = require("../users/User");
const users_service_1 = require("../users/users.service");
const Joi = require("joi");
const App_1 = require("../configuration/App");
const ldap_service_1 = require("../ldap/ldap.service");
const XORMON_TOKEN = 'xormon-token';
const COOKIE_OPTIONS = {
    sameSite: 'none',
};
let TokenController = TokenController_1 = class TokenController {
    constructor(configService, usersService, ldapService, eventEmitter) {
        this.configService = configService;
        this.usersService = usersService;
        this.ldapService = ldapService;
        this.eventEmitter = eventEmitter;
        this.logger = new common_1.Logger(TokenController_1.name);
        this.debug = configService.get('APP_DEBUG');
    }
    async logout(req, response) {
        response.clearCookie(XORMON_TOKEN, COOKIE_OPTIONS);
    }
    async generateToken(request, res, tokenRequest) {
        if (tokenRequest.username && tokenRequest.password) {
            let user = await User_1.User.authByUsername(tokenRequest.username, tokenRequest.password);
            if (!user) {
                try {
                    const ldap = await this.ldapService.login(tokenRequest.username, tokenRequest.password);
                    if (ldap) {
                        user = await this.usersService.saveLdapUser(ldap.ldap_id, tokenRequest.username);
                        user.groups = await this.ldapService.getGroupsByLdapGroups(ldap.ldap_id, ldap.groups);
                    }
                    else {
                        throw new common_1.UnauthorizedException({
                            statusCode: common_1.HttpStatus.UNAUTHORIZED,
                            message: 'login failed',
                            error: 'bad credentials',
                            data: {
                                state: 'failed',
                                token: null,
                            }
                        });
                    }
                }
                catch (e) {
                    console.error("[GenerateTokenLdapCheck] error: " + e);
                    throw new common_1.UnauthorizedException({
                        statusCode: common_1.HttpStatus.UNAUTHORIZED,
                        message: 'login failed',
                        error: e?.response?.error || 'bad credentials',
                        data: {
                            state: 'failed',
                            token: null,
                        }
                    });
                }
            }
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.LOGIN, null, user.user_id);
            User_1.User.setLastLogged(user.user_id);
            if (user && user.active) {
                const secret = App_1.App.getSecret();
                const settings = await Settings_1.Settings.get({ keys: ['GLOB_TOKEN_EXPIRATION'] });
                const expiration = settings['GLOB_TOKEN_EXPIRATION'] && parseInt(settings['GLOB_TOKEN_EXPIRATION']) != 0 ? { expiresIn: settings['GLOB_TOKEN_EXPIRATION'] + "m" } : undefined;
                const jwt = require('jsonwebtoken');
                const groups = user.groups ? user.groups : (await this.usersService.getUser(user.user_id)).groups;
                const isAdmin = groups.find((group) => group.group_id === 1) ? true : false;
                const token = await jwt.sign({
                    username: tokenRequest.username,
                    email: user.email,
                    isAdmin: isAdmin,
                    timezone: user.timezone,
                    configuration: user.configuration,
                    updated: user.updated,
                    groupLabels: groups.map(group => group.name)
                }, secret, expiration);
                if (this.debug === 'true') {
                    this.logger.debug(token);
                }
                user.password = undefined;
                res.status(common_1.HttpStatus.OK);
                res.cookie(XORMON_TOKEN, token, COOKIE_OPTIONS);
                return {
                    statusCode: common_1.HttpStatus.OK,
                    message: 'successfully logged in',
                    data: {
                        state: 'success',
                        token: token,
                        user: {
                            ...user,
                            isAdmin: isAdmin
                        },
                    },
                };
            }
            else {
                throw new common_1.HttpException({
                    statusCode: common_1.HttpStatus.UNAUTHORIZED,
                    message: 'login failed',
                    error: 'user is not active',
                    data: {
                        state: 'failed',
                        token: null,
                    },
                }, common_1.HttpStatus.UNAUTHORIZED);
            }
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.UNAUTHORIZED,
                message: 'login failed',
                error: 'missing credentials',
                data: {
                    state: 'failed',
                    token: null,
                },
            }, common_1.HttpStatus.UNAUTHORIZED);
        }
    }
    async proxyGenerate(request, tokenRequestDTO) {
        const tokenRequest = tokenRequestDTO["body"] ? tokenRequestDTO["body"] : tokenRequestDTO;
        const host = request.ip;
        if (tokenRequest.username && tokenRequest.password) {
            let user = await User_1.User.authByUsername(tokenRequest.username, tokenRequest.password);
            if (user) {
                const groups = user.groups ? user.groups : (await this.usersService.getUser(user.user_id)).groups;
                const isAdmin = groups.find((group) => group.group_id === 1) ? true : false;
                if (isAdmin) {
                    const secret = App_1.App.getSecret();
                    const jwt = require('jsonwebtoken');
                    const token = await jwt.sign({
                        proxy: true,
                        host: host,
                    }, secret);
                    const metricsPort = process.env['METRICS_SERVER_PORT'] !== undefined
                        ? +process.env['METRICS_SERVER_PORT']
                        : 3030;
                    return {
                        statusCode: common_1.HttpStatus.OK,
                        message: 'successfully logged in',
                        data: {
                            state: 'success',
                            token: token,
                            port: metricsPort
                        },
                    };
                }
                else {
                    throw new common_1.HttpException({
                        statusCode: common_1.HttpStatus.UNAUTHORIZED,
                        message: 'User unathorized for generating proxy token',
                        error: 'Unauthorized',
                        data: {
                            state: 'failed',
                            token: null,
                        },
                    }, common_1.HttpStatus.UNAUTHORIZED);
                }
            }
            else {
                throw new common_1.HttpException({
                    statusCode: common_1.HttpStatus.UNAUTHORIZED,
                    message: 'login failed',
                    error: 'Bad credentials',
                    data: {
                        state: 'failed',
                        token: null,
                    },
                }, common_1.HttpStatus.UNAUTHORIZED);
            }
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.UNAUTHORIZED,
                message: 'login failed',
                error: 'missing credentials',
                data: {
                    state: 'failed',
                    token: null,
                },
            }, common_1.HttpStatus.UNAUTHORIZED);
        }
    }
    async refresh(tokenRequest) {
        return null;
    }
};
exports.TokenController = TokenController;
__decorate([
    (0, common_1.Get)('v1/remove'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)({ passthrough: true })),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TokenController.prototype, "logout", null);
__decorate([
    openapi.ApiOperation({ description: "generating a new JWT token" }),
    (0, common_1.Post)('v1/generate'),
    (0, common_1.UsePipes)(new joiValidation_pipe_1.JoiValidationPipe(Joi.object({
        username: Joi.string().required(),
        password: Joi.string().required(),
    }))),
    openapi.ApiResponse({ status: 201, type: require("../../common/tokenResponse.dto").TokenResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)({ passthrough: true })),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, tokenRequest_dto_1.TokenRequestDTO]),
    __metadata("design:returntype", Promise)
], TokenController.prototype, "generateToken", null);
__decorate([
    (0, common_1.Post)("v1/generate/proxy"),
    openapi.ApiResponse({ status: 201, type: require("../../common/tokenResponse.dto").TokenResponseMetricsPort }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, tokenRequest_dto_1.TokenRequestDTO]),
    __metadata("design:returntype", Promise)
], TokenController.prototype, "proxyGenerate", null);
__decorate([
    openapi.ApiOperation({ description: "refresh JWT token" }),
    (0, common_1.Post)('v1/refresh'),
    openapi.ApiResponse({ status: 201, type: require("../../common/tokenResponse.dto").TokenResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [tokenRequest_dto_1.TokenRefreshRequestDTO]),
    __metadata("design:returntype", Promise)
], TokenController.prototype, "refresh", null);
exports.TokenController = TokenController = TokenController_1 = __decorate([
    (0, swagger_1.ApiTags)('Token'),
    (0, common_1.Controller)('api/token'),
    __metadata("design:paramtypes", [config_1.ConfigService,
        users_service_1.UsersService,
        ldap_service_1.LdapService,
        event_emitter_1.EventEmitter2])
], TokenController);
//# sourceMappingURL=token.controller.js.map