"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DestinationSender = void 0;
const typeorm_1 = require("typeorm");
const https = require("https");
const License_1 = require("../config/License");
class DestinationSender {
    constructor(destinationRepository, destinationHostcfg, type) {
        this.destinationRepository = destinationRepository;
        this.destinationHostcfg = destinationHostcfg;
        this.type = type;
        this.destinationMap = {};
        this.globalItems = [];
        this.options = {
            agent: new https.Agent({
                rejectUnauthorized: false,
            }),
            method: "POST",
            timeout: 20000,
            headers: {
                'Content-Type': 'application/json'
            }
        };
    }
    ;
    async init() {
        this.hasLicense = await License_1.License.hasAnyValid();
        this.globalDestinations = !this.hasLicense ? [] : await this.destinationRepository.findBy({
            target_destination_id: (0, typeorm_1.Not)((0, typeorm_1.In)((await this.destinationHostcfg.find())
                .map(destinationHostcfg => destinationHostcfg.target_destination_id)))
        });
    }
    addItems(item, hItem) {
        if (!this.hasLicense)
            return;
        if (item.hostcfg_relations && item.hostcfg_relations[0] && item.hostcfg_relations[0]._hostcfg.destinations) {
            for (const destination of item.hostcfg_relations[0]._hostcfg.destinations) {
                if (this.destinationMap[destination._target_destination.destination]) {
                    this.destinationMap[destination._target_destination.destination].items.push(hItem);
                }
                else {
                    this.destinationMap[destination._target_destination.destination] = {
                        destination: destination._target_destination,
                        items: [hItem]
                    };
                }
            }
        }
        if (this.globalDestinations) {
            this.globalItems.push(hItem);
        }
    }
    doRequest(url, body, token) {
        this.options.headers.authorization = `Bearer ${token}`;
        return new Promise((resolve, reject) => {
            const req = https.request(url, this.options, (res) => {
                res.setEncoding('utf8');
                let responseBody = '';
                if (res.statusCode !== 201) {
                    if (res.statusCode === 401) {
                        reject("Wrong credentials");
                    }
                    reject(res.statusMessage + ", code: " + res.statusCode);
                }
                res.on('data', (chunk) => {
                    responseBody += chunk;
                });
                res.on('end', () => {
                    resolve(JSON.parse(responseBody));
                });
            });
            req.on('timeout', () => {
                reject("Target destination unreachable");
            });
            req.on('error', (err) => {
                reject(err);
            });
            req.write(JSON.stringify(body));
            req.end();
        });
    }
    async sendBatch() {
        if (!this.hasLicense)
            return;
        const localDestiMap = { ...this.destinationMap };
        const localGlobalItems = [...this.globalItems];
        for (const { destination, items } of Object.values(localDestiMap)) {
            this.doRequest(`https://${destination.destination}:${destination.port}/api/proxy/v1/${this.type}`, items, destination.token).catch((err) => {
                console.error(`[Proxy Sender ${this.type}] ${err}, destination: ${destination.destination}:${destination.port}`);
            });
        }
        for (const destination of this.globalDestinations) {
            this.doRequest(`https://${destination.destination}:${destination.port}/api/proxy/v1/${this.type}`, localGlobalItems, destination.token).catch((err) => {
                console.error(`[Proxy Sender ${this.type}] ${err}, global destination: ${destination.destination}:${destination.port}`);
                if (err === "Wrong credentials") {
                    this.globalDestinations = this.globalDestinations.filter((dest) => dest.destination !== destination.destination);
                }
            });
        }
        this.globalItems = [];
        this.destinationMap = {};
    }
}
exports.DestinationSender = DestinationSender;
//# sourceMappingURL=destinationSender.js.map