"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HealthStatusTask = exports.createItemLink = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const healthStatus_entity_1 = require("../entity/healthStatus.entity");
const PM2_1 = require("../service/configuration/PM2");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const taskServerCache_1 = require("./task-queue/taskServerCache");
const typeorm_2 = require("typeorm");
const healthStatusHistory_entity_1 = require("../entity/healthStatusHistory.entity");
const objectItem_entity_1 = require("../entity/objectItem.entity");
const Settings_1 = require("../service/configuration/Settings");
const mail_service_1 = require("../mail/mail.service");
const deviceGroupHwAlerting_entity_1 = require("../entity/deviceGroupHwAlerting.entity");
const snmp_handler_1 = require("./snmp.handler");
const serviceNow_handler_1 = require("./serviceNow.handler");
const Definitions_1 = require("../service/metrics/Definitions");
const schedule_1 = require("@nestjs/schedule");
function createItemLink(_class, hw_type, alias, item_id, settings) {
    if (!settings['ALERTING_XORMON_URL']) {
        return null;
    }
    if (_class === "storage") {
        return settings['ALERTING_XORMON_URL'] + _class + "/device/" + alias + "/health-status?item_id=" + item_id;
    }
    if (hw_type === 'power') {
        return settings['ALERTING_XORMON_URL'] + _class + "/type/" + hw_type + "/health-status";
    }
    return settings['ALERTING_XORMON_URL'] + _class + "/type/" + hw_type + "/health-status?item_id=" + item_id;
}
exports.createItemLink = createItemLink;
class MailInfo {
    constructor() {
        this.mails = [];
    }
}
let HealthStatusTask = class HealthStatusTask {
    constructor(healthStatusRepository, healthStatusHistoryRepository, objectItemRepository, deviceGroupRepository, snmpHandler, serviceNowHandler, mailService) {
        this.healthStatusRepository = healthStatusRepository;
        this.healthStatusHistoryRepository = healthStatusHistoryRepository;
        this.objectItemRepository = objectItemRepository;
        this.deviceGroupRepository = deviceGroupRepository;
        this.snmpHandler = snmpHandler;
        this.serviceNowHandler = serviceNowHandler;
        this.mailService = mailService;
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('hw_events');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    sendReload() {
        try {
            PM2_1.PM2.sendMessage('backend', {
                action: "reload-health-status"
            });
        }
        catch (e) {
            this.winstonLogger.warn("Emit ws event failed");
        }
    }
    async healthStatusInsert(payload) {
        const cache = (0, taskServerCache_1.getGlobalTaskCache)();
        let cacheItem = cache.get(taskServerCache_1.CacheKeys.HEALTH_STATUS);
        if (!cacheItem) {
            console.log("Cache didnt load on init??");
            const value = await this.loadCacheItems();
            cache.set(taskServerCache_1.CacheKeys.HEALTH_STATUS, value);
            cacheItem = cache.get(taskServerCache_1.CacheKeys.HEALTH_STATUS);
        }
        const idMap = cacheItem[0];
        const mailIdMap = cacheItem[1];
        const lastHsHistory = cacheItem[2];
        const settings = cacheItem[3];
        let snmpIdMap = cacheItem[4];
        let servicenowIdMap = cacheItem[5];
        const newHsHistory = [];
        const hsHistoryToEnd = [];
        const start = performance.now();
        let mailQueue = [];
        for (const entity of payload) {
            let mail = {
                healthStatus: entity,
                mails: [],
            };
            if (mailIdMap[entity.item_id]) {
                mailIdMap[entity.item_id].forEach(m => mail.mails.push(m));
            }
            const lastHistory = lastHsHistory.find((history) => history.item_id === entity.item_id && history.type === entity.type);
            const item = idMap[entity.item_id] ? idMap[entity.item_id] : null;
            mail.item = item;
            if (entity.data === "No current data") {
                entity.status = healthStatus_entity_1.Status.UNKNOWN;
            }
            if (item) {
                if (lastHistory) {
                    if (lastHistory.status === entity.status) {
                        return;
                    }
                    else {
                        lastHistory.endedAt = new Date();
                        hsHistoryToEnd.push(lastHistory);
                        this.winstonLogger.log("info", "[HW Events]: Ending old Health Status History: [" + lastHistory.item_id + "] " + (item ? item.label : undefined));
                        [mail.header, mail.message] = this.getMail(item, entity, settings);
                    }
                }
                else {
                    if (entity.status !== healthStatus_entity_1.Status.OK && entity.status !== healthStatus_entity_1.Status.UNKNOWN) {
                        [mail.header, mail.message] = this.getMail(item, entity, settings);
                    }
                }
                if ([healthStatus_entity_1.Status.WARNING, healthStatus_entity_1.Status.ERROR].includes(entity.status)) {
                    newHsHistory.push({
                        item_id: entity.item_id,
                        status: entity.status,
                        reason: entity.data,
                        type: entity.type
                    });
                    this.winstonLogger.log("info", "[HW Events]: Saving new Health Status History: [" + entity.item_id + "] [" + entity.type + "] " + (item ? item.label : undefined));
                }
            }
            if (mail.header && mail.message) {
                mailQueue.push(mail);
                this.winstonLogger.log("info", "[HW Events]: Health Status changed for: [" + entity.item_id + "] [" + entity.type + "] " + (item ? item.label : undefined) + ", changed to " + entity.status);
            }
        }
        if (newHsHistory.length || hsHistoryToEnd.length) {
            this.sendReload();
            const idMap = {};
            for (const item of hsHistoryToEnd) {
                idMap[item.health_status_history_id] = true;
            }
            const itemsToSave = cacheItem[2].filter((val) => !idMap[val.health_status_history_id]).concat(...newHsHistory);
            cache.update(taskServerCache_1.CacheKeys.HEALTH_STATUS, [cacheItem[0], cacheItem[1], itemsToSave, cacheItem[3], cacheItem[4], cacheItem[5]]);
        }
        await this.healthStatusHistoryRepository.save(hsHistoryToEnd.concat(...newHsHistory));
        if (mailQueue.length) {
            if (!this.mailService.isTransportSetUp()) {
                await this.mailService.refreshTransport();
            }
            for (const mail of mailQueue) {
                const mailsString = mail.mails.join(", ");
                this.winstonLogger.log("info", "[HW Events]: Sending mail with header: " + mail.header + ", for mail adresses: " + mailsString);
                if (mail.mails && mail.mails.length) {
                    this.mailService.send(mail.mails, mail.header, mail.message);
                }
                if (mail.healthStatus) {
                    if (snmpIdMap[mail.healthStatus.item_id]) {
                        this.sendSnmp(settings, mail.header, mail.message, mail.item, mail.healthStatus);
                    }
                    if (servicenowIdMap[mail.healthStatus.item_id]) {
                        this.sendServicenow(settings, mail.header, mail.message, mail.item, mail.healthStatus);
                    }
                }
            }
        }
        this.winstonLogger.log("info", "[HW Events]: healthStatus check took: " + (performance.now() - start) + " ms");
    }
    getMail(item, entity, settings) {
        let link = createItemLink(item.hw_type_entity.class, item.hw_type, item.label, item.item_id, settings);
        const header = `Xormon HW Alert [${entity.status}] ${item.label}`;
        const message = `
                        <b>Xormon Alert</b><br>
                        ${entity.type === "health" ? "Health status event" : entity.type === "data" ? "Performance data event" : "Connection test event"} for device ${item.label}<br>
                        Status changed back to ${entity.status}<br>
                        Reason: ${entity.data ? entity.data.replaceAll("\n", "<br>") : ''}<br>
                        ${link && entity.type === "health" ? "Check it out in the Xormon NG UI: <a href='" + link + "'>" + link + "</a>" : ""}
                        `;
        return [header, message];
    }
    async reloadHsTaskCache() {
        const cache = (0, taskServerCache_1.getGlobalTaskCache)() ? (0, taskServerCache_1.getGlobalTaskCache)() : (0, taskServerCache_1.createGlobalTaskCache)();
        const value = await this.loadCacheItems();
        cache.set(taskServerCache_1.CacheKeys.HEALTH_STATUS, value);
    }
    async loadCacheItems() {
        let hwAlertSubsystems = new Set();
        const possibleSubsys = (0, Definitions_1.findStatusSubsystemsPerHwtype)();
        for (const hw_type in possibleSubsys) {
            for (const subsys of possibleSubsys[hw_type]) {
                hwAlertSubsystems.add(subsys);
            }
        }
        const rootObjectItems = await this.objectItemRepository.find({
            where: [
                {
                    subsystem: (0, typeorm_2.In)([...hwAlertSubsystems.values()]),
                    hw_type: (0, typeorm_2.Not)((0, typeorm_2.In)(['timescaledb', 'linux'])),
                    parents: {
                        parent: (0, typeorm_2.IsNull)()
                    },
                }, {
                    subsystem: "device",
                    parents: {
                        _parent: {
                            subsystem: "fabric"
                        }
                    }
                }
            ],
            relations: {
                parents: {
                    _parent: true
                },
                hw_type_entity: {
                    _class: true
                }
            }
        });
        let idMap = {};
        let hwTypeMap = {};
        let classMap = {};
        let snmpIdMap = {};
        let servicenowIdMap = {};
        const itemIdMailMap = {};
        for (const item of rootObjectItems) {
            idMap[item.item_id] = item;
            if (item.hw_type in hwTypeMap) {
                hwTypeMap[item.hw_type].push(item.item_id);
            }
            else {
                hwTypeMap[item.hw_type] = [item.item_id];
            }
            if (item.hw_type_entity._class._class in classMap) {
                classMap[item.hw_type_entity._class._class].push(item.item_id);
            }
            else {
                classMap[item.hw_type_entity._class._class] = [item.item_id];
            }
            itemIdMailMap[item.item_id] = new Set();
        }
        let lastHealthStatusHistory = await this.healthStatusHistoryRepository.findBy({
            endedAt: (0, typeorm_2.IsNull)()
        });
        ;
        let settings = await Settings_1.Settings.get({
            keys: ['ALERTING_XORMON_URL', 'SERVICENOW_INSTANCE', 'SERVICENOW_USER', 'SERVICENOW_PASS', 'SERVICENOW_TYPE', 'SERVICENOW_CLASS', 'GLOB_SNMP_TRAP_HOST', 'GLOB_SNMP_COMMUNITY']
        });
        const canUseServicenow = !(!settings.SERVICENOW_INSTANCE || !settings.SERVICENOW_USER || !settings.SERVICENOW_PASS);
        const canUseSnmpTrap = !(settings.GLOB_SNMP_TRAP_HOST || settings.GLOB_SNMP_COMMUNITY);
        const hwAlertDeviceGroups = await this.deviceGroupRepository.find({
            where: {
                enabled: true
            },
            relations: {
                mail_groups: true,
                _item_group: {
                    items: true
                }
            }
        });
        for (const group of hwAlertDeviceGroups) {
            if (group.class === "all") {
                for (const key in itemIdMailMap) {
                    if (group.integrations) {
                        servicenowIdMap[key] = group.integrations.servicenow && canUseServicenow ? true : false;
                        snmpIdMap[key] = group.integrations.snmp && canUseSnmpTrap ? true : false;
                    }
                    for (const mailGroup of group.mail_groups) {
                        for (const mail of mailGroup.mails) {
                            itemIdMailMap[key].add(mail);
                        }
                    }
                }
            }
            else if (group.hw_type === "all") {
                if (classMap[group.class])
                    this.addMailsToItems(group, group.class, classMap, itemIdMailMap, snmpIdMap, servicenowIdMap, canUseSnmpTrap, canUseServicenow);
            }
            else if (!group._item_group.regex && !group._item_group.items) {
                if (group.hw_type) {
                    if (hwTypeMap[group.hw_type])
                        this.addMailsToItems(group, group.hw_type, hwTypeMap, itemIdMailMap, snmpIdMap, servicenowIdMap, canUseSnmpTrap, canUseServicenow);
                }
                else {
                    if (classMap[group.class])
                        this.addMailsToItems(group, group.class, classMap, itemIdMailMap, snmpIdMap, servicenowIdMap, canUseSnmpTrap, canUseServicenow);
                }
            }
            else if (group._item_group.regex) {
                for (const key in itemIdMailMap) {
                    if (idMap[key].hw_type === group.hw_type) {
                        if (group.integrations) {
                            servicenowIdMap[key] = group.integrations.servicenow && canUseServicenow ? true : false;
                            snmpIdMap[key] = group.integrations.snmp && canUseSnmpTrap ? true : false;
                        }
                        if (key.match(group._item_group.regex[0])) {
                            for (const mailGroup of group.mail_groups) {
                                for (const mail of mailGroup.mails) {
                                    itemIdMailMap[key].add(mail);
                                }
                            }
                        }
                    }
                }
            }
            else if (group._item_group.items && group._item_group.items.length) {
                for (const { item_id } of group._item_group.items) {
                    if (group.integrations) {
                        servicenowIdMap[item_id] = group.integrations.servicenow && canUseServicenow ? true : false;
                        snmpIdMap[item_id] = group.integrations.snmp && canUseSnmpTrap ? true : false;
                    }
                    for (const mailGroup of group.mail_groups) {
                        for (const mail of mailGroup.mails) {
                            itemIdMailMap[item_id].add(mail);
                        }
                    }
                }
            }
        }
        if (settings['ALERTING_XORMON_URL'] && settings['ALERTING_XORMON_URL'][settings['ALERTING_XORMON_URL'].length - 1] !== '/') {
            settings['ALERTING_XORMON_URL'] = settings['ALERTING_XORMON_URL'] + '/';
        }
        return [idMap, itemIdMailMap, lastHealthStatusHistory, settings, snmpIdMap, servicenowIdMap];
    }
    async sendSnmp(settings, header, message, item, healthStatus) {
        const enterpriseOid = '1.3.6.1.4.1.40540';
        const oidValues = {
            [`${enterpriseOid}.1.1`]: header,
            [`${enterpriseOid}.1.2`]: item.subsystem,
            [`${enterpriseOid}.1.3`]: item.label,
            [`${enterpriseOid}.1.4`]: message,
            [`${enterpriseOid}.1.5`]: "Severity: " + healthStatus.status
        };
        await this.snmpHandler.sendSNMPTrap(settings.GLOB_SNMP_TRAP_HOST, settings.GLOB_SNMP_COMMUNITY, oidValues).catch(error => {
            this.winstonLogger.error(`[AG ${settings.GLOB_SNMP_TRAP_HOST}] Error sending SNMP trap: ${error.message}`);
        });
    }
    async sendServicenow(settings, header, message, item, healthStatus) {
        const body = {
            'records': [
                {
                    'source': 'xormon-ng',
                    'event_class': "health_status",
                    'resource': item.label,
                    'node': item.hw_type,
                    'metric_name': "health",
                    'type': settings.SERVICENOW_TYPE,
                    'severity': healthStatus.status,
                    'description': message,
                }
            ]
        };
        await this.serviceNowHandler.sendserviceNow(settings.SERVICENOW_INSTANCE, body, {
            user: settings.SERVICENOW_USER,
            password: settings.SERVICENOW_PASS
        }).catch(error => {
            this.winstonLogger.error(`[HS ${header}] Error sending ServiceNow: ${error.message}`);
        });
    }
    addMailsToItems(group, mapKey, map, mailMap, snmpMap, snMap, snmp, servicenow) {
        for (const key of map[mapKey]) {
            if (group.integrations) {
                snMap[key] = group.integrations.servicenow && servicenow ? true : false;
                snmpMap[key] = group.integrations.snmp && snmp ? true : false;
            }
            for (const mailGroup of group.mail_groups) {
                for (const mail of mailGroup.mails) {
                    mailMap[key].add(mail);
                }
            }
        }
    }
};
exports.HealthStatusTask = HealthStatusTask;
__decorate([
    (0, schedule_1.Cron)(schedule_1.CronExpression.EVERY_5_MINUTES),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], HealthStatusTask.prototype, "reloadHsTaskCache", null);
exports.HealthStatusTask = HealthStatusTask = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(healthStatus_entity_1.HealthStatus)),
    __param(1, (0, typeorm_1.InjectRepository)(healthStatusHistory_entity_1.HealthStatusHistory)),
    __param(2, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(3, (0, typeorm_1.InjectRepository)(deviceGroupHwAlerting_entity_1.HwEventDeviceGroup)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        snmp_handler_1.SnmpHandler,
        serviceNow_handler_1.ServiceNowHandler,
        mail_service_1.MailService])
], HealthStatusTask);
//# sourceMappingURL=healthStatus.task.js.map