"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HeatmapTask = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const heatmap_entity_1 = require("../entity/heatmap.entity");
const heatmapTier_entity_1 = require("../entity/heatmapTier.entity");
const heatmapTierItem_entity_1 = require("../entity/heatmapTierItem.entity");
const itemProperty_entity_1 = require("../entity/itemProperty.entity");
const objectItem_entity_1 = require("../entity/objectItem.entity");
const Classes_1 = require("../service/metrics/Classes");
const MetricsWrapper_1 = require("../service/metrics/MetricsWrapper");
const typeorm_2 = require("typeorm");
const itemRelation_entity_1 = require("../entity/itemRelation.entity");
const Heatmap_1 = require("../service/menu/menu/Heatmap");
const hwTypeKey_constants_1 = require("../common/constants/hwTypeKey.constants");
const TaskMonitor_1 = require("./TaskMonitor");
const targetDestination_entity_1 = require("../entity/targetDestination.entity");
const targetDestinationHostcfg_entity_1 = require("../entity/targetDestinationHostcfg.entity");
const destinationSender_1 = require("./destinationSender");
const defaultIntervals = {
    read: [
        { limit: 2, color: '#008000' },
        { limit: 2.4, color: '#29f929' },
        { limit: 2.8, color: '#81fa51' },
        { limit: 3.2, color: '#c9f433' },
        { limit: 3.6, color: '#FFFF66' },
        { limit: 4, color: '#ffff00' },
        { limit: 4.6, color: '#FFCC00' },
        { limit: 5, color: '#ffa500' },
        { limit: 5.4, color: '#fa610e' },
        { limit: 5.8, color: '#ff0000' }
    ],
    write: [
        { limit: 2, color: '#008000' },
        { limit: 2.4, color: '#29f929' },
        { limit: 2.8, color: '#81fa51' },
        { limit: 3.2, color: '#c9f433' },
        { limit: 3.6, color: '#FFFF66' },
        { limit: 4, color: '#ffff00' },
        { limit: 4.6, color: '#FFCC00' },
        { limit: 5, color: '#ffa500' },
        { limit: 5.4, color: '#fa610e' },
        { limit: 5.8, color: '#ff0000' }
    ]
};
let HeatmapTask = class HeatmapTask {
    constructor(heatmapRepository, objectItemsRepository, itemPropertiesRepository, heatmapTierRepository, heatmapTierItemRepository, itemRelationRepository, targetDestinationRepository, targetDestinationHcfgRepository) {
        this.heatmapRepository = heatmapRepository;
        this.objectItemsRepository = objectItemsRepository;
        this.itemPropertiesRepository = itemPropertiesRepository;
        this.heatmapTierRepository = heatmapTierRepository;
        this.heatmapTierItemRepository = heatmapTierItemRepository;
        this.itemRelationRepository = itemRelationRepository;
        this.targetDestinationRepository = targetDestinationRepository;
        this.targetDestinationHcfgRepository = targetDestinationHcfgRepository;
        this.PAGE_LIMIT = 500;
        this.roundTo = function (num, places) {
            const factor = 10 ** places;
            return Math.round(num * factor) / factor;
        };
    }
    async createHeatmapTier() {
        await this.heatmapTierRepository.save([{ label: 'tier1', limit_pool: 100, limit_volume: 25, read_intervals: defaultIntervals.read, write_intervals: defaultIntervals.write }]);
        const pools = await this.objectItemsRepository.findBy({ subsystem: 'pool' });
        const tier1 = (await this.heatmapTierRepository.findOneBy({ label: 'tier1' })).heatmap_tier_id;
        let heatmapTierItems = tier1
            ? pools.map((pool) => {
                return { heatmap_tier_id: tier1, item_id: pool.item_id };
            })
            : [];
        if (heatmapTierItems && heatmapTierItems.length > 0)
            await this.heatmapTierItemRepository.save(heatmapTierItems);
    }
    async reloadHeatmap() {
        const taskMonitor = new TaskMonitor_1.TaskMonitor(this.reloadHeatmap.name, 'Storage heatmap reload');
        const now = new Date();
        const tiers = await this.heatmapTierRepository.find();
        if (!tiers || tiers.length === 0) {
            await this.createHeatmapTier();
        }
        else {
            const tieredItemIds = (await this.heatmapTierItemRepository.find())
                .map(heatmapTierItem => heatmapTierItem.item_id);
            const pools = await this.objectItemsRepository.find({
                where: {
                    subsystem: 'pool',
                    item_id: (0, typeorm_2.Not)((0, typeorm_2.In)(tieredItemIds))
                }
            });
            const tier1 = await this.heatmapTierRepository.findOneBy({ label: 'tier1' });
            let heatmapTierItems = [];
            if (tier1) {
                heatmapTierItems = pools.map((pool) => {
                    return { heatmap_tier_id: tier1.heatmap_tier_id, item_id: pool.item_id };
                });
            }
            if (heatmapTierItems) {
                await this.heatmapTierItemRepository.save(heatmapTierItems);
            }
        }
        const subsystems = ['pool', 'volume'];
        const metrics = ['read_io', 'write_io', 'io_rate', 'resp_t_r', 'resp_t_w', 'resp_t'];
        const destination = await this.targetDestinationRepository.findOneBy({});
        const proxySender = destination ? new destinationSender_1.DestinationSender(this.targetDestinationRepository, this.targetDestinationHcfgRepository, "heatmap") : null;
        if (proxySender) {
            await proxySender.init();
        }
        for (const subsystem of subsystems) {
            let heatmapItems = await this.objectItemsRepository.find({
                where: {
                    subsystem: subsystem,
                    parents: {
                        _parent: {
                            subsystem: 'device',
                        }
                    }
                },
                relations: {
                    parents: {
                        _parent: true
                    },
                    hostcfg_relations: proxySender ? {
                        _hostcfg: {
                            destinations: {
                                _target_destination: true
                            }
                        }
                    } : undefined
                }
            });
            const count = heatmapItems.length;
            taskMonitor.log(`Found ${count} ${subsystem}s`);
            let repetitions = Math.ceil((count / this.PAGE_LIMIT));
            taskMonitor.log(`Processing [${repetitions}]`);
            for (let index = 1; index <= repetitions; index++) {
                let bundle = this.getBundle(heatmapItems, index - 1, this.PAGE_LIMIT);
                taskMonitor.log(`Processing [${bundle.length + ((index - 1) * this.PAGE_LIMIT)} / ${count}]`);
                for (const metric of metrics) {
                    const values = await (new MetricsWrapper_1.MetricsWrapper([metric], bundle.map(item => item.item_id))).getLastHourValueFromTable();
                    let data = [];
                    taskMonitor.log(`Processing ${metric}`);
                    for (const item of bundle) {
                        if (values[item.item_id] !== undefined && values[item.item_id][metric] !== undefined) {
                            let count_decimals = metric.match(/io/g) ? 0 : 2;
                            data.push({
                                item_id: item.item_id,
                                metric: metric,
                                value: this.roundTo(values[item.item_id][metric], count_decimals),
                                device: item.parents[0]._parent.label,
                                label: item.label,
                                subsystem: subsystem,
                                updatedAt: now
                            });
                            if (proxySender) {
                                proxySender.addItems(item, data.at(-1));
                            }
                        }
                    }
                    try {
                        await this.heatmapRepository.upsert(data, ['item_id', 'metric']);
                        if (proxySender) {
                            proxySender.sendBatch();
                        }
                    }
                    catch (error) {
                        console.error(`[ReloadHeatmap] [${metric}] error: ${error}`);
                    }
                }
            }
        }
    }
    async reloadStorCapacityHeatmap() {
        const taskMonitor = new TaskMonitor_1.TaskMonitor(this.reloadStorCapacityHeatmap.name, 'Storage capacity reload');
        const now = new Date();
        const storage = Classes_1.Storage.technology;
        const destination = await this.targetDestinationRepository.findOneBy({});
        const proxySender = destination ? new destinationSender_1.DestinationSender(this.targetDestinationRepository, this.targetDestinationHcfgRepository, "heatmap") : null;
        if (proxySender) {
            await proxySender.init();
        }
        for (const hwType in storage) {
            try {
                let subsystem;
                if (storage[hwType].subsystems && storage[hwType].subsystems['pool']) {
                    subsystem = 'pool';
                }
                else if (hwType === hwTypeKey_constants_1.HwTypeKey.STORAGEGRID) {
                    subsystem = 'site';
                }
                if (subsystem !== undefined) {
                    taskMonitor.log(`Processing ${storage[hwType].label ? storage[hwType].label : hwType}`);
                    const heatmap = new Heatmap_1.Heatmap(hwType, subsystem, true);
                    heatmap.addMetric({
                        label: 'capacity_used_proc',
                        metricUsed: 'capacity_used',
                        metricTotal: 'capacity_total',
                    });
                    let heatmapToSave = [];
                    const heatmapData = await heatmap.getData(proxySender ? true : false);
                    for (const heatmapMetric of heatmapData) {
                        for (const metricItem of heatmapMetric['items']) {
                            heatmapToSave.push({
                                item_id: metricItem.item_id,
                                metric: heatmapMetric.metric,
                                value: metricItem.percent,
                                updatedAt: now
                            });
                            if (proxySender) {
                                await proxySender.addItems(metricItem, heatmapToSave.at(-1));
                            }
                        }
                    }
                    await this.heatmapRepository.upsert(heatmapToSave, ['item_id', 'metric']);
                    if (proxySender) {
                        proxySender.sendBatch();
                    }
                }
            }
            catch (e) {
                taskMonitor.log(`Error: ${e}`);
            }
        }
    }
    async reloadComputedHeatmap() {
        const taskMonitor = new TaskMonitor_1.TaskMonitor(this.reloadComputedHeatmap.name, 'Virtualization & Container & Cloud heatmap reload');
        const now = new Date();
        const destination = await this.targetDestinationRepository.findOneBy({});
        const proxySender = destination ? new destinationSender_1.DestinationSender(this.targetDestinationRepository, this.targetDestinationHcfgRepository, "heatmap") : null;
        if (proxySender) {
            await proxySender.init();
        }
        for (const _class of [Classes_1.Virtualization, Classes_1.Container, Classes_1.Cloud]) {
            const technologies = _class.technology;
            for (const hwType in technologies) {
                for (const subsystem in technologies[hwType].subsystems) {
                    if (technologies[hwType].subsystems[subsystem].heatmap) {
                        try {
                            taskMonitor.log(`Processing ${technologies[hwType].label ? technologies[hwType].label : hwType} ${technologies[hwType].subsystems[subsystem].label ? technologies[hwType].subsystems[subsystem].label : subsystem}`);
                            const heatmap = new Heatmap_1.Heatmap(hwType, subsystem, true);
                            const heatmapMetrics = technologies[hwType].subsystems[subsystem].heatmap;
                            for (const heatmapMetric of heatmapMetrics) {
                                if (heatmapMetric.percent !== undefined) {
                                    heatmap.addMetric({
                                        label: heatmapMetric.key,
                                        metric: heatmapMetric.percent
                                    });
                                }
                                else if (heatmapMetric.used !== undefined && heatmapMetric.total !== undefined) {
                                    heatmap.addMetric({
                                        label: heatmapMetric.key,
                                        metricUsed: heatmapMetric.used,
                                        metricTotal: heatmapMetric.total,
                                    });
                                }
                                else if (heatmapMetric.free !== undefined && heatmapMetric.total !== undefined) {
                                    heatmap.addMetric({
                                        label: heatmapMetric.key,
                                        metricFree: heatmapMetric.free,
                                        metricTotal: heatmapMetric.total,
                                    });
                                }
                            }
                            let heatmapToSave = [];
                            const heatmapData = await heatmap.getData(proxySender ? true : false);
                            for (const heatmapMetric of heatmapData) {
                                for (const metricItem of heatmapMetric['items']) {
                                    heatmapToSave.push({
                                        item_id: metricItem.item_id,
                                        metric: heatmapMetric.metric,
                                        value: metricItem.percent,
                                        updatedAt: now
                                    });
                                    if (proxySender) {
                                        proxySender.addItems(metricItem, heatmapToSave.at(-1));
                                    }
                                }
                            }
                            await this.heatmapRepository.upsert(heatmapToSave, ['item_id', 'metric']);
                            if (proxySender) {
                                proxySender.sendBatch();
                            }
                        }
                        catch (e) {
                            taskMonitor.log(`Error: ${e}`);
                        }
                    }
                }
            }
        }
    }
    async reloadSanHetmap() {
        const taskMonitor = new TaskMonitor_1.TaskMonitor(this.reloadSanHetmap.name, 'SAN heatmap reload');
        const now = new Date();
        const destination = await this.targetDestinationRepository.findOneBy({});
        const proxySender = destination ? new destinationSender_1.DestinationSender(this.targetDestinationRepository, this.targetDestinationHcfgRepository, "heatmap") : null;
        if (proxySender) {
            await proxySender.init();
        }
        const options = {
            relations: {
                hw_type_entity: true,
                hostcfg_relations: proxySender ? {
                    _hostcfg: {
                        destinations: {
                            _target_destination: true
                        }
                    }
                } : undefined
            },
            where: {
                hw_type_entity: {
                    class: 'san'
                },
                subsystem: 'sanport'
            }
        };
        const count = await this.objectItemsRepository.count(options);
        taskMonitor.log(`Found ${count} ports`);
        for (let skip = 0; skip < count; skip += this.PAGE_LIMIT) {
            taskMonitor.log(`Processing [${skip + this.PAGE_LIMIT < count ? skip + this.PAGE_LIMIT : count} / ${count}]`);
            const ports = await this.objectItemsRepository.find({
                ...options,
                take: this.PAGE_LIMIT,
                skip: skip
            });
            const itemIds = ports.map(item => item.item_id);
            const metrics = await (new MetricsWrapper_1.MetricsWrapper(['utilization_in', 'utilization_out'], itemIds)).getLastHourValueFromTable();
            let heatmapToSave = [];
            for (const item of ports) {
                if (metrics[item.item_id]) {
                    if (metrics[item.item_id]['utilization_in'] !== undefined) {
                        heatmapToSave.push({
                            item_id: item.item_id,
                            metric: 'data_in_util',
                            value: metrics[item.item_id]['utilization_in'],
                            updatedAt: now
                        });
                        if (proxySender) {
                            proxySender.addItems(item, heatmapToSave.at(-1));
                        }
                    }
                    if (metrics[item.item_id]['utilization_out'] !== undefined) {
                        heatmapToSave.push({
                            item_id: item.item_id,
                            metric: 'data_out_util',
                            value: metrics[item.item_id]['utilization_out'],
                            updatedAt: now
                        });
                        if (proxySender) {
                            proxySender.addItems(item, heatmapToSave.at(-1));
                        }
                    }
                }
            }
            try {
                await this.heatmapRepository.upsert(heatmapToSave, ['item_id', 'metric']);
                if (proxySender) {
                    proxySender.sendBatch();
                }
            }
            catch (e) {
                taskMonitor.log(`Error: ${e}`);
            }
        }
    }
    async reloadLanHetmap() {
        const taskMonitor = new TaskMonitor_1.TaskMonitor(this.reloadLanHetmap.name, 'LAN heatmap reload');
        const now = new Date();
        const destination = await this.targetDestinationRepository.findOneBy({});
        const proxySender = destination ? new destinationSender_1.DestinationSender(this.targetDestinationRepository, this.targetDestinationHcfgRepository, "heatmap") : null;
        if (proxySender) {
            await proxySender.init();
        }
        const options = {
            relations: {
                hw_type_entity: true,
                hostcfg_relations: proxySender ? {
                    _hostcfg: {
                        destinations: {
                            _target_destination: true
                        }
                    }
                } : undefined
            },
            where: {
                hw_type_entity: {
                    class: 'lan'
                },
                subsystem: 'physical'
            }
        };
        const count = await this.objectItemsRepository.count(options);
        taskMonitor.log(`Found ${count} ports`);
        for (let skip = 0; skip < count; skip += this.PAGE_LIMIT) {
            taskMonitor.log(`Processing [${skip + this.PAGE_LIMIT < count ? skip + this.PAGE_LIMIT : count} / ${count}]`);
            const ports = await this.objectItemsRepository.find({
                ...options,
                take: this.PAGE_LIMIT,
                skip: skip
            });
            const itemIds = ports.map(item => item.item_id);
            const metrics = await (new MetricsWrapper_1.MetricsWrapper(['utilization_in', 'utilization_out'], itemIds)).getLastHourValueFromTable();
            let heatmapToSave = [];
            for (const item of ports) {
                if (metrics[item.item_id]) {
                    if (metrics[item.item_id]['utilization_in'] !== undefined) {
                        heatmapToSave.push({
                            item_id: item.item_id,
                            metric: 'data_in_util',
                            value: metrics[item.item_id]['utilization_in'],
                            updatedAt: now
                        });
                        if (proxySender) {
                            proxySender.addItems(item, heatmapToSave.at(-1));
                        }
                    }
                    if (metrics[item.item_id]['utilization_out'] !== undefined) {
                        heatmapToSave.push({
                            item_id: item.item_id,
                            metric: 'data_out_util',
                            value: metrics[item.item_id]['utilization_out'],
                            updatedAt: now
                        });
                        if (proxySender) {
                            proxySender.addItems(item, heatmapToSave.at(-1));
                        }
                    }
                }
            }
            try {
                await this.heatmapRepository.upsert(heatmapToSave, ['item_id', 'metric']);
                if (proxySender) {
                    proxySender.sendBatch();
                }
            }
            catch (e) {
                taskMonitor.log(`Error: ${e}`);
            }
        }
    }
    getBundle(items, index, multiplier) {
        let start = index * multiplier;
        let end = (index + 1) * multiplier;
        let bundle = items.slice(start, end < items.length ? end : items.length);
        return bundle;
    }
};
exports.HeatmapTask = HeatmapTask;
exports.HeatmapTask = HeatmapTask = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(heatmap_entity_1.Heatmap)),
    __param(1, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(2, (0, typeorm_1.InjectRepository)(itemProperty_entity_1.ItemProperty)),
    __param(3, (0, typeorm_1.InjectRepository)(heatmapTier_entity_1.HeatmapTier)),
    __param(4, (0, typeorm_1.InjectRepository)(heatmapTierItem_entity_1.HeatmapTierItem)),
    __param(5, (0, typeorm_1.InjectRepository)(itemRelation_entity_1.ItemRelation)),
    __param(6, (0, typeorm_1.InjectRepository)(targetDestination_entity_1.TargetDestination)),
    __param(7, (0, typeorm_1.InjectRepository)(targetDestinationHostcfg_entity_1.TargetDestinationHostcfg)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], HeatmapTask);
//# sourceMappingURL=heatmap.task.js.map