"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MonitoringTask = void 0;
const common_1 = require("@nestjs/common");
const config_1 = require("@nestjs/config");
const typeorm_1 = require("@nestjs/typeorm");
const dataSource_1 = require("../config/dataSource");
const hwType_entity_1 = require("../entity/hwType.entity");
const itemRelation_entity_1 = require("../entity/itemRelation.entity");
const objectItem_entity_1 = require("../entity/objectItem.entity");
const subsystem_entity_1 = require("../entity/subsystem.entity");
const App_1 = require("../service/configuration/App");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const typeorm_2 = require("typeorm");
const uuid_1 = require("uuid");
const TaskMonitor_1 = require("./TaskMonitor");
let MonitoringTask = class MonitoringTask {
    constructor(configService, objectItemsRepository, itemRelationsRepository) {
        this.configService = configService;
        this.objectItemsRepository = objectItemsRepository;
        this.itemRelationsRepository = itemRelationsRepository;
        this.namespace = App_1.App.DEFAULT_NAMESPACE;
        this.enabled = this.configService.get('APP_MONITORING');
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('monitoring');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async onModuleInit() {
        await TaskMonitor_1.TaskMonitor.checkInternalMonitoringTable();
    }
    async monitoring() {
        if (!App_1.App.isProduction() && !(await App_1.App.isMaster())) {
            return;
        }
        if (!this.enabled) {
            return;
        }
        const entityManager = (0, dataSource_1.getDataSource)();
        let tables = {
            total: 0,
            hw_types: {},
        };
        try {
            this.winstonLogger.log("info", "Job started");
            const exclude_list = ['metrics', 'metrics_5m', 'metrics_1h', 'metrics_6h', 'metrics_1d'];
            const hypertables = await entityManager.query("SELECT hypertable_name, hypertable_size(format('%I.%I', hypertable_schema, hypertable_name)::regclass) FROM timescaledb_information.hypertables;");
            for (const hypertable of hypertables) {
                if (exclude_list.includes(hypertable['hypertable_name'])) {
                    continue;
                }
                const splitted = hypertable['hypertable_name'].split('_', 2);
                tables['total'] += +hypertable['hypertable_size'];
                if (tables['hw_types'][splitted[1]]) {
                    if (!tables['hw_types'][splitted[1]]['subsystems'][hypertable['hypertable_name'].split(splitted[1] + '_')[1]]) {
                        tables['hw_types'][splitted[1]]['subsystems'][hypertable['hypertable_name'].split(splitted[1] + '_')[1]] = {
                            total: 0,
                            tables: {},
                        };
                    }
                    tables['hw_types'][splitted[1]]['subsystems'][hypertable['hypertable_name'].split(splitted[1] + '_')[1]]['total'] +=
                        +hypertable['hypertable_size'];
                    tables['hw_types'][splitted[1]]['total'] += +hypertable['hypertable_size'];
                }
                else {
                    tables['hw_types'][splitted[1]] = {
                        total: +hypertable['hypertable_size'],
                        subsystems: {},
                    };
                    tables['hw_types'][splitted[1]]['subsystems'][hypertable['hypertable_name'].split(splitted[1] + '_')[1]] = {
                        total: +hypertable['hypertable_size'],
                        tables: {},
                    };
                    tables['hw_types'][splitted[1]]['total'] += +hypertable['hypertable_size'];
                }
            }
            const views = await entityManager.query("SELECT view_name, hypertable_size(format('%I.%I', materialization_hypertable_schema , materialization_hypertable_name )::regclass) FROM timescaledb_information.continuous_aggregates;");
            for (const view of views) {
                const splitted = view['view_name'].split('_');
                if (splitted[2]) {
                    tables['total'] += +view['hypertable_size'];
                    const subsystem = view['view_name'].split(splitted[1] + '_')[1].replace(/_1m|_5m|_1h|_6h|_1d/g, '');
                    if (tables['hw_types'][splitted[1]]) {
                        tables['hw_types'][splitted[1]]['subsystems'][subsystem]['total'] += +view['hypertable_size'];
                        tables['hw_types'][splitted[1]]['total'] += +view['hypertable_size'];
                    }
                }
            }
            const now = new Date().toISOString();
            const hwTypeEntities = await (0, dataSource_1.getDataSource)().getRepository(hwType_entity_1.HwType).find();
            const subsystemEntities = await (0, dataSource_1.getDataSource)().getRepository(subsystem_entity_1.Subsystem).find();
            let objectItems = [];
            let itemRelations = [];
            let metrics = {};
            objectItems.push({
                item_id: (0, uuid_1.v5)('schema-metrics', this.namespace),
                label: 'Metrics',
                object_id: 'database',
                hw_type: 'database',
                subsystem: 'schema',
            });
            metrics['schema'] = [
                {
                    item_id: (0, uuid_1.v5)('schema-metrics', this.namespace),
                    time: now,
                    size: tables['total'],
                },
            ];
            for (const hwType in tables['hw_types']) {
                const entity = hwTypeEntities.find(entity => entity.hw_type === hwType);
                objectItems.push({
                    item_id: (0, uuid_1.v5)('hw_type-' + hwType, this.namespace),
                    label: entity ? entity.label : hwType,
                    object_id: 'database',
                    hw_type: 'database',
                    subsystem: 'hw_type',
                });
                if (!metrics['hw_type']) {
                    metrics['hw_type'] = [];
                }
                metrics['hw_type'].push({
                    item_id: (0, uuid_1.v5)('hw_type-' + hwType, this.namespace),
                    time: now,
                    size: tables['hw_types'][hwType]['total'],
                });
                for (const subsystem in tables['hw_types'][hwType]['subsystems']) {
                    const subsysEntity = subsystemEntities.find(entity => entity.hw_type === hwType && entity.subsystem === subsystem);
                    objectItems.push({
                        item_id: (0, uuid_1.v5)(hwType + '-subsystem-' + subsystem, this.namespace),
                        label: subsysEntity ? subsysEntity.label : subsystem,
                        object_id: 'database',
                        hw_type: 'database',
                        subsystem: 'subsystem',
                    });
                    itemRelations.push({
                        parent: (0, uuid_1.v5)('hw_type-' + hwType, this.namespace),
                        child: (0, uuid_1.v5)(hwType + '-subsystem-' + subsystem, this.namespace),
                    });
                    if (!metrics['subsystem']) {
                        metrics['subsystem'] = [];
                    }
                    metrics['subsystem'].push({
                        item_id: (0, uuid_1.v5)(hwType + '-subsystem-' + subsystem, this.namespace),
                        time: now,
                        size: tables['hw_types'][hwType]['subsystems'][subsystem]['total'],
                    });
                }
            }
            await this.objectItemsRepository.save(objectItems);
            await this.itemRelationsRepository.save(itemRelations);
            for (const metric in metrics) {
                this.winstonLogger.log("info", "Saving " + metric);
                const repository = (0, dataSource_1.getDataSource)().getRepository('metrics.metrics_database_' + metric);
                const saved = await repository.createQueryBuilder().insert().values(metrics[metric]).orIgnore().execute();
            }
            this.winstonLogger.log("info", "Job ended: success");
        }
        catch (e) {
            this.winstonLogger.log("error", e);
            this.winstonLogger.log("info", "Job ended: error");
        }
    }
    async handlePerf() {
        const manager = (0, dataSource_1.getDataSource)();
        let queries = this.getQueries('perf', manager.options.database);
        let responses = {};
        for (const key in queries) {
            if (Object.prototype.hasOwnProperty.call(queries, key)) {
                try {
                    responses[key] = await manager.query(queries[key]);
                }
                catch (error) {
                    console.warn(`query ${key}  error: ${error.message}`);
                }
            }
        }
        let cleanResponses = {};
        for (const key in responses) {
            let newObject = {};
            if (Object.prototype.hasOwnProperty.call(responses, key)) {
                const element = responses[key];
                for (const metric in element[0]) {
                    let newMetric = metric.replaceAll('_', '');
                    newObject[newMetric] = responses[key][0][metric];
                }
                responses[key][0] = newObject;
            }
        }
        responses['clean'] = [{}];
        for (const dataType of ['sessions', 'locks', 'wait_event']) {
            if (dataType in responses) {
                for (const type of responses[dataType]) {
                    cleanResponses[(type.type).toLowerCase()] = type.count;
                }
            }
        }
        let currentData = {
            perSecond: { ...responses['dbs'][0], ...responses['cluster'][0] },
            counters: {
                deadlocks: responses['dbs'][0]['deadlocks'] ? responses['dbs'][0]['deadlocks'] : 0,
            },
            clean: {
                size: responses['dbs'][0]['size'] ? responses['dbs'][0]['size'] : 0,
                numbackends: responses['dbs'][0]['numbackends'] ? responses['dbs'][0]['numbackends'] : 0,
                ...cleanResponses,
            }
        };
        let objectItems = [];
        objectItems.push({
            item_id: 'selfmonitdb',
            label: 'selfmonitdb',
            object_id: 'database',
            hw_type: 'timescaledb',
            subsystem: 'db',
            updatedAt: new Date()
        });
        await this.objectItemsRepository.save(objectItems);
        const repository = (0, dataSource_1.getDataSource)().getRepository('metrics.metrics_timescaledb_db');
        const counters = (0, dataSource_1.getDataSource)().getRepository('counters');
        let lastData = (await counters.createQueryBuilder().select('*').where({ item_id: "selfmonitdb" }).execute())[0];
        const leCounters = await counters.createQueryBuilder().insert()
            .values({ item_id: "selfmonitdb", counters: currentData, updatedAt: new Date() })
            .orUpdate(['counters', 'updated_at'], ['item_id']).execute();
        if (!lastData) {
            this.winstonLogger.log("info", "[end] Postgres performance selfmoniroring: no counters found");
            return;
        }
        if (lastData) {
            let timeDifference = (Math.ceil(new Date().valueOf() - lastData.updated_at.valueOf()) / 1000);
            for (const key in currentData.counters) {
                currentData.counters[key] = currentData.counters[key] - lastData.counters['counters'][key];
            }
            for (const key in currentData.perSecond) {
                currentData.perSecond[key] = (currentData.perSecond[key] - lastData.counters['perSecond'][key]) / timeDifference;
            }
        }
        let cleanData = {
            "item_id": "selfmonitdb",
            "time": new Date(),
        };
        cleanData = { ...cleanData, ...currentData.perSecond, ...currentData.clean };
        const saved = await repository.createQueryBuilder().insert().values(cleanData).orIgnore().execute();
        this.winstonLogger.log("info", "[end] Postgres performance selfmoniroring");
    }
    async handleConf() {
        let rel_types = {
            "r": "ordinary table",
            "i": "index",
            "S": "sequence",
            "v": "view",
            "m": "materialized view",
            "c": "composite type",
            "t": "TOAST table",
            "f": "foreign table",
        };
        let conf_sa = [
            "shared_buffers",
            "work_mem",
            "max_connections",
            "temp_buffers",
            "autovacuum_work_mem",
            "maintenance_work_mem",
            "wal_buffers",
            "effective_cache_size",
            "random_page_cost",
            "min_wal_size",
            "max_wal_size",
            "wal_sync_method",
            "deadlock_timeout",
            "max_worker_processes",
        ];
        const manager = (0, dataSource_1.getDataSource)();
        let responses = {};
        let queriesConf = this.getQueries('conf', manager.options.database);
        for (const key in queriesConf) {
            if (Object.prototype.hasOwnProperty.call(queriesConf, key)) {
                try {
                    responses[key] = await manager.query(queriesConf[key]);
                }
                catch (error) {
                    console.warn(error.message);
                }
            }
        }
        let tempObject = {};
        for (const element of responses['main']) {
            if ('name' in element && 'setting' in element && conf_sa.includes(element['name'])) {
                tempObject[element['name']] = element['setting'];
            }
        }
        responses['main'] = [tempObject];
        const properties = (0, dataSource_1.getDataSource)().getRepository('item_properties');
        const savedMain = await properties.createQueryBuilder().insert()
            .values({ item_id: 'selfmonitdb', property_name: 'main', property_value: JSON.stringify([...responses['main']]) })
            .orUpdate(['property_value'], ['item_id', 'property_name']).execute();
        const savedRelations = await properties.createQueryBuilder().insert()
            .values({ item_id: 'selfmonitdb', property_name: 'relations', property_value: JSON.stringify([...responses['relations']]) })
            .orUpdate(['property_value'], ['item_id', 'property_name']).execute();
        this.winstonLogger.log("info", "[end] Postgres configuration selfmoniroring");
    }
    getQueries(type, database = '') {
        if (type === 'perf') {
            let queries = {
                "dbs": `SELECT blks_read, blks_hit, tup_returned, tup_fetched, tup_inserted, tup_updated, tup_deleted, xact_commit, xact_rollback, deadlocks, temp_files,temp_bytes, pg_database_size(datname) as \"size\", numbackends FROM pg_stat_database
                         WHERE datname = '${database}';`,
                "cluster": `SELECT buffers_backend, buffers_clean, buffers_checkpoint, checkpoints_req, checkpoints_timed FROM pg_stat_bgwriter;`,
                "locks": `SELECT locktype as type, count(locktype) FROM pg_locks where granted = 'true' AND 
                         database is not null group by locktype;`,
                "sessions": `SELECT state as type, count(state) FROM pg_stat_activity
                         WHERE state != 'undefined' AND datname = '${database}' group by state;`,
                "wait_event": `SELECT wait_event_type as type, count(wait_event_type) FROM pg_stat_activity
                         WHERE wait_event_type != 'undefined' AND wait_event != 'undefined' AND datname = '${database}' group by wait_event_type;`,
            };
            return queries;
        }
        else if (type === 'conf') {
            let queries = {
                "main": `show all;`,
                "relations": `SELECT nspname AS \"Namespace\", relname AS \"Name\", relkind AS \"Type\",
                                 pg_relation_size(C.oid) AS \"Size\", relpages AS \"Pages\",reltuples AS \"Rows\"
                          FROM pg_class C
                          LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace)
                          WHERE nspname IN ('metrics', 'public')
                          AND relkind != 'i'
                          ORDER BY pg_relation_size(C.oid) DESC
                          LIMIT 50;`
            };
            return queries;
        }
    }
};
exports.MonitoringTask = MonitoringTask;
exports.MonitoringTask = MonitoringTask = __decorate([
    (0, common_1.Injectable)(),
    __param(1, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(2, (0, typeorm_1.InjectRepository)(itemRelation_entity_1.ItemRelation)),
    __metadata("design:paramtypes", [config_1.ConfigService,
        typeorm_2.Repository,
        typeorm_2.Repository])
], MonitoringTask);
//# sourceMappingURL=monitoring.task.js.map