"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskWorker = void 0;
const dataSource_1 = require("../../config/dataSource");
const TaskMonitor_1 = require("../TaskMonitor");
const masterProcess_1 = require("./masterProcess");
class TaskWorker {
    constructor(pollIntervalInMs, jobMap, workerId) {
        this.pollIntervalInMs = pollIntervalInMs;
        this.jobMap = jobMap;
        this.workerId = workerId;
        this.status = "stopped";
        this.taskMonitor = new TaskMonitor_1.TaskMonitor("Worker", "Label", "WorkerMonitor[" + this.workerId + "]" + "[" + (0, masterProcess_1.getCurrentProcessId)() + "]");
    }
    ;
    async pollForTasks() {
        this.clear();
        if (this.status === "waiting") {
            this.status = "running";
            const queryRunner = (0, dataSource_1.getDataSource)().createQueryRunner();
            await queryRunner.connect();
            await queryRunner.startTransaction();
            try {
                const queryResult = await queryRunner.query("DELETE FROM queue.task_queue WHERE id = (SELECT id FROM queue.task_queue ORDER BY priority DESC FOR UPDATE SKIP LOCKED LIMIT 1) RETURNING id, type, payload, status, number_of_items, priority");
                let pollAgain = false;
                if (queryResult && queryResult[0].length && "type" in queryResult[0][0]) {
                    pollAgain = true;
                    const task = queryResult[0][0];
                    const job = this.jobMap[task["type"]];
                    this.taskMonitor.setTask(job.taskType);
                    if (job) {
                        if (job.taskLabel) {
                            this.taskMonitor.setLabel(job.taskLabel);
                            this.taskMonitor.setTask(job.taskType);
                            this.taskMonitor.start();
                        }
                        try {
                            await job.service[job.taskName](task.payload ? task.payload : undefined);
                        }
                        catch (error) {
                            console.error(error);
                        }
                    }
                    else {
                        throw new Error("No job with type " + task["type"] + " exists in TaskQueueService job map");
                    }
                }
                await queryRunner.commitTransaction();
            }
            catch (error) {
                console.error(error);
                if (error.message && error.message.includes("FK_99e3a748292ea5203b62e0b69b6")) {
                    console.error(error.detail);
                }
                await queryRunner.rollbackTransaction();
            }
            finally {
                if (this.taskMonitor.getWorkerStarted()) {
                    this.taskMonitor.end();
                }
                await queryRunner.release();
                this.start();
            }
        }
        else {
            if (this.interval) {
                this.clear();
            }
            return;
        }
    }
    start() {
        if (this.status !== "waiting") {
            this.status = "waiting";
            this.interval = setInterval(() => { this.pollForTasks(); }, this.pollIntervalInMs);
        }
    }
    stop() {
        this.clear();
        this.status = "stopped";
    }
    setPollInterval(interval) {
        this.pollIntervalInMs = interval;
    }
    getStatus() {
        return this.status;
    }
    notify() {
        this.pollForTasks();
    }
    clear() {
        if (this.interval) {
            clearInterval(this.interval);
            this.interval = null;
        }
    }
}
exports.TaskWorker = TaskWorker;
//# sourceMappingURL=TaskWorker.js.map