"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMasterProcessId = exports.getProcessesIds = exports.getProcesses = exports.getCurrentProcessId = void 0;
const pm2 = require("pm2");
const util_1 = require("util");
const baseConfig = {
    instanceStatus: ["online"]
};
const initConnection = (function () {
    let isConnected = null;
    return () => {
        if (isConnected)
            return isConnected;
        pm2.connect(true, (err) => {
            isConnected = err ? false : true;
        });
        global.pm2Connected = isConnected;
        return isConnected;
    };
})();
const isConnected = () => {
    return global.pm2Connected;
};
function getProcessId(process) {
    const processId = Number(process.pm_id ?? process.pm2_env?.pm_id);
    return Number.isNaN(processId) ? null : processId;
}
function getCurrentProcessId() {
    const processId = Number(process.env.pm_id);
    return Number.isNaN(processId) ? null : processId;
}
exports.getCurrentProcessId = getCurrentProcessId;
async function getProcesses({ instanceStatus = [] }) {
    if (!isConnected()) {
        await initConnection();
    }
    const processes = await (0, util_1.promisify)(pm2.list).bind(pm2)();
    if (processes.length === 0) {
        return [];
    }
    const curProcessId = getCurrentProcessId();
    const curProcess = processes.find((process) => getProcessId(process) === curProcessId);
    if (!curProcess?.name) {
        throw new Error(`Invalid instance name in pm2!`);
    }
    return processes
        .filter((process) => process.name === curProcess.name)
        .filter((process) => instanceStatus.length === 0 || instanceStatus.includes(process.pm2_env?.status));
}
exports.getProcesses = getProcesses;
async function getProcessesIds(config = baseConfig) {
    const curProcessId = getCurrentProcessId();
    if (curProcessId === null) {
        config.logger.warn('Not running in PM2.');
        return [];
    }
    const processes = await getProcesses(config);
    if (processes.length === 0) {
        return [curProcessId];
    }
    return processes
        .map(getProcessId)
        .filter((id) => id !== null);
}
exports.getProcessesIds = getProcessesIds;
async function getMasterProcessId(config = baseConfig) {
    const processesIds = await getProcessesIds(config);
    if (processesIds.length === 0) {
        return null;
    }
    return Math.min(...processesIds);
}
exports.getMasterProcessId = getMasterProcessId;
//# sourceMappingURL=masterProcess.js.map