"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskQueueService = exports.TaskDefinition = void 0;
const common_1 = require("@nestjs/common");
const schedule_1 = require("@nestjs/schedule");
const typeorm_1 = require("@nestjs/typeorm");
const taskQueue_constants_1 = require("../common/constants/taskQueue.constants");
const taskQueue_entity_1 = require("../entity/taskQueue.entity");
const typeorm_2 = require("typeorm");
const heatmap_task_1 = require("./heatmap.task");
const top_task_1 = require("./top.task");
const cron_1 = require("cron");
const cleanup_task_1 = require("./cleanup.task");
const alert_task_1 = require("./alert.task");
const exporter_task_1 = require("./exporter.task");
const reporter_task_1 = require("./reporter.task");
const monitoring_task_1 = require("./monitoring.task");
const aggregation_task_1 = require("./aggregation.task");
const log_task_1 = require("./log.task");
const custom_task_1 = require("./custom.task");
const prediction_task_1 = require("./prediction.task");
const stuckJob_task_1 = require("./stuckJob.task");
const TaskWorker_1 = require("./task-queue/TaskWorker");
const PM2_1 = require("../service/configuration/PM2");
const taskServerCache_1 = require("./task-queue/taskServerCache");
const healthStatus_task_1 = require("./healthStatus.task");
const masterProcess_1 = require("./task-queue/masterProcess");
class TaskDefinition {
    constructor() {
        this.typeExists = true;
    }
}
exports.TaskDefinition = TaskDefinition;
let TaskQueueService = class TaskQueueService {
    constructor(taskQueueRepository, schedulerRegistry, hetmapTasks, topTasks, cleanupTasks, alertTasks, exporterTasks, reporterTasks, monitoringTasks, aggregationTasks, logTasks, customTasks, predictionTasks, stuckJobTasks, healthStatusTask) {
        this.taskQueueRepository = taskQueueRepository;
        this.schedulerRegistry = schedulerRegistry;
        this.hetmapTasks = hetmapTasks;
        this.topTasks = topTasks;
        this.cleanupTasks = cleanupTasks;
        this.alertTasks = alertTasks;
        this.exporterTasks = exporterTasks;
        this.reporterTasks = reporterTasks;
        this.monitoringTasks = monitoringTasks;
        this.aggregationTasks = aggregationTasks;
        this.logTasks = logTasks;
        this.customTasks = customTasks;
        this.predictionTasks = predictionTasks;
        this.stuckJobTasks = stuckJobTasks;
        this.healthStatusTask = healthStatusTask;
        this.workers = [];
        this.workerLimit = taskQueue_constants_1.WORKER_LIMIT;
        this.jobMap = {
            [taskQueue_constants_1.TaskTypes.HEALT_STATUS]: {
                taskType: taskQueue_constants_1.TaskTypes.HEALT_STATUS,
                priority: 16,
                typeExists: false,
                taskName: "healthStatusInsert",
                service: this.healthStatusTask
            },
            [taskQueue_constants_1.TaskTypes.HEATMAP_STOR]: {
                taskType: taskQueue_constants_1.TaskTypes.HEATMAP_STOR,
                priority: 4,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_HOUR,
                taskLabel: 'Storage heatmap reload',
                taskName: "reloadHeatmap",
                service: this.hetmapTasks
            },
            [taskQueue_constants_1.TaskTypes.HEATMAP_SAN]: {
                taskType: taskQueue_constants_1.TaskTypes.HEATMAP_SAN,
                priority: 4,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_HOUR,
                taskLabel: 'SAN heatmap reload',
                taskName: "reloadSanHetmap",
                service: this.hetmapTasks
            },
            [taskQueue_constants_1.TaskTypes.HEATMAP_LAN]: {
                taskType: taskQueue_constants_1.TaskTypes.HEATMAP_LAN,
                priority: 4,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_HOUR,
                taskLabel: 'LAN heatmap reload',
                taskName: "reloadLanHetmap",
                service: this.hetmapTasks
            },
            [taskQueue_constants_1.TaskTypes.HEATMAP_VIRTUALIZATION]: {
                taskType: taskQueue_constants_1.TaskTypes.HEATMAP_VIRTUALIZATION,
                priority: 4,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_HOUR,
                taskLabel: 'Virtualization & Container & Cloud heatmap reload',
                taskName: "reloadComputedHeatmap",
                service: this.hetmapTasks
            },
            [taskQueue_constants_1.TaskTypes.HEATMAP_CAPACITY]: {
                taskType: taskQueue_constants_1.TaskTypes.HEATMAP_CAPACITY,
                priority: 4,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_HOUR,
                taskLabel: 'Storage capacity heatmap reload',
                taskName: "reloadStorCapacityHeatmap",
                service: this.hetmapTasks
            },
            [taskQueue_constants_1.TaskTypes.TOP]: {
                taskType: taskQueue_constants_1.TaskTypes.TOP,
                priority: 2,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_DAY_AT_MIDNIGHT,
                taskLabel: 'TOP reload',
                taskName: "top",
                service: this.topTasks
            },
            [taskQueue_constants_1.TaskTypes.CLEANUP]: {
                taskType: taskQueue_constants_1.TaskTypes.CLEANUP,
                priority: 2,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_HOUR,
                monitorBlacklist: true,
                taskName: "handleInterval",
                service: this.cleanupTasks
            },
            [taskQueue_constants_1.TaskTypes.ALERTS]: {
                taskType: taskQueue_constants_1.TaskTypes.ALERTS,
                priority: 16,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_5_MINUTES,
                monitorBlacklist: true,
                taskName: "checkAlerts",
                service: this.alertTasks
            },
            [taskQueue_constants_1.TaskTypes.HW_ALERTS]: {
                taskType: taskQueue_constants_1.TaskTypes.HW_ALERTS,
                priority: 16,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_30_MINUTES,
                monitorBlacklist: true,
                taskName: "checkHwAlerts",
                service: this.alertTasks
            },
            [taskQueue_constants_1.TaskTypes.HS_HISTORY_CLOSE]: {
                taskType: taskQueue_constants_1.TaskTypes.HS_HISTORY_CLOSE,
                priority: 2,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_30_MINUTES,
                monitorBlacklist: true,
                taskName: "closeDifferingHealthStatusHistory",
                service: this.alertTasks
            },
            [taskQueue_constants_1.TaskTypes.EXPORTER]: {
                taskType: taskQueue_constants_1.TaskTypes.EXPORTER,
                priority: 8,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_DAY_AT_2AM,
                monitorBlacklist: true,
                taskName: "exporter",
                service: this.exporterTasks
            },
            [taskQueue_constants_1.TaskTypes.REPORTER]: {
                taskType: taskQueue_constants_1.TaskTypes.REPORTER,
                priority: 8,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_DAY_AT_2AM,
                monitorBlacklist: true,
                taskName: "reporter",
                service: this.reporterTasks
            },
            [taskQueue_constants_1.TaskTypes.MONITORING]: {
                taskType: taskQueue_constants_1.TaskTypes.MONITORING,
                priority: 2,
                typeExists: true,
                cron: "33 */2 * * *",
                taskName: "monitoring",
                service: this.monitoringTasks
            },
            [taskQueue_constants_1.TaskTypes.MONITORING_HANDLE_CONF]: {
                taskType: taskQueue_constants_1.TaskTypes.MONITORING_HANDLE_CONF,
                priority: 2,
                typeExists: true,
                cron: "51 */2 * * *",
                taskLabel: "DB monitoring Conf",
                taskName: "handleConf",
                service: this.monitoringTasks
            },
            [taskQueue_constants_1.TaskTypes.MONITORING_HANDLE_PERF]: {
                taskType: taskQueue_constants_1.TaskTypes.MONITORING_HANDLE_PERF,
                priority: 2,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_5_MINUTES,
                taskLabel: "DB monitoring Perf",
                taskName: "handlePerf",
                service: this.monitoringTasks
            },
            [taskQueue_constants_1.TaskTypes.AGGREGATION]: {
                taskType: taskQueue_constants_1.TaskTypes.AGGREGATION,
                priority: 4,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_HOUR,
                taskName: "handleInterval",
                service: this.aggregationTasks
            },
            [taskQueue_constants_1.TaskTypes.LOGS]: {
                taskType: taskQueue_constants_1.TaskTypes.LOGS,
                priority: 2,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_DAY_AT_10PM,
                taskName: "_trimLogs",
                service: this.logTasks
            },
            [taskQueue_constants_1.TaskTypes.CUSTOM_TASK]: {
                taskType: taskQueue_constants_1.TaskTypes.CUSTOM_TASK,
                priority: 16,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_HOUR,
                taskLabel: "Custom tasks",
                taskName: "handleTask",
                service: this.customTasks
            },
            [taskQueue_constants_1.TaskTypes.PREDICTION]: {
                taskType: taskQueue_constants_1.TaskTypes.PREDICTION,
                priority: 2,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_DAY_AT_MIDNIGHT,
                taskName: "handle",
                service: this.predictionTasks
            },
            [taskQueue_constants_1.TaskTypes.STUCK_JOB_DEAD]: {
                taskType: taskQueue_constants_1.TaskTypes.STUCK_JOB_DEAD,
                priority: 2,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_30_MINUTES,
                taskName: "checkDeadTasks",
                service: this.stuckJobTasks
            },
            [taskQueue_constants_1.TaskTypes.STUCK_JOB_NUL]: {
                taskType: taskQueue_constants_1.TaskTypes.STUCK_JOB_DEAD,
                priority: 2,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_DAY_AT_MIDNIGHT,
                taskName: "checkNullStartTasks",
                service: this.stuckJobTasks
            },
            [taskQueue_constants_1.TaskTypes.DAEMON_KILLER]: {
                taskType: taskQueue_constants_1.TaskTypes.DAEMON_KILLER,
                priority: 2,
                typeExists: true,
                cron: schedule_1.CronExpression.EVERY_30_MINUTES,
                taskName: "killStuckDaemonProcesses",
                service: this.stuckJobTasks
            }
        };
        for (let i = 1; i <= this.workerLimit; i++) {
            setTimeout(() => {
                console.log("Spawning worker " + (i));
                this.workers.push(new TaskWorker_1.TaskWorker(taskQueue_constants_1.WORKER_POLL_INTERVAL, this.jobMap, i));
                this.workers.at(-1).start();
            }, i * 5000);
        }
    }
    ;
    async isMasterProcess() {
        return this.isMaster = await (0, masterProcess_1.getMasterProcessId)() === (0, masterProcess_1.getCurrentProcessId)() ? true : false;
    }
    async onApplicationBootstrap() {
        await this.isMasterProcess();
        const cache = (0, taskServerCache_1.createGlobalTaskCache)();
        this.healthStatusTask.loadCacheItems().then(value => {
            cache.set(taskServerCache_1.CacheKeys.HEALTH_STATUS, value);
        });
        if (this.isMaster) {
            for (const key in this.jobMap) {
                if (this.jobMap[key].cron) {
                    this.schedulerRegistry.addCronJob(key, new cron_1.CronJob(this.jobMap[key].cron, async () => {
                        await this.insertTaskObject(this.jobMap[key]);
                    }, null, true));
                }
            }
        }
    }
    async onModuleInit() {
        await this.isMasterProcess();
        if (this.isMaster) {
            process.on('message', (data) => {
                const parsed = PM2_1.PM2.parseMessageTask(data);
                if (parsed.action === 'insert-task') {
                    if (this.jobMap[parsed.taskType]) {
                        this.insertTaskObject({ ...this.jobMap[parsed.taskType], payload: parsed.taskPayload, number_of_items: parsed.number_of_items });
                    }
                    else {
                        console.error(`Task ${parsed.taskType} not in jobMap.`);
                    }
                }
            });
        }
    }
    async insertTask(taskType, priority, payload, number_of_items = undefined, typeExists = true) {
        if (typeExists) {
            if (await this.checkTask(taskType))
                return;
        }
        await this.taskQueueRepository.save({
            type: taskType,
            status: taskQueue_constants_1.TaskStatus.PENDING,
            priority: priority,
            number_of_items: number_of_items,
            payload: payload
        });
        this.notifyIdleWorker();
    }
    async insertTaskObject(task) {
        await this.insertTask(task.taskType, task.priority, task.payload, task.number_of_items, task.typeExists);
    }
    async checkTask(type) {
        const currentTask = await this.taskQueueRepository.findOneBy({ type: type });
        if (currentTask) {
            await this.taskQueueRepository.save({ ...currentTask, priority: currentTask.priority > 0 ? currentTask.priority * 2 : 2 });
            return true;
        }
        return false;
    }
    getIdleWorker() {
        return this.workers.find((w) => w.getStatus() === "waiting");
    }
    notifyIdleWorker() {
        const worker = this.getIdleWorker();
        if (worker) {
            worker.notify();
        }
    }
    getJob(type) {
        return this.jobMap[type];
    }
};
exports.TaskQueueService = TaskQueueService;
exports.TaskQueueService = TaskQueueService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(taskQueue_entity_1.TaskQueue)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        schedule_1.SchedulerRegistry,
        heatmap_task_1.HeatmapTask,
        top_task_1.TopTask,
        cleanup_task_1.CleanupTask,
        alert_task_1.AlertTask,
        exporter_task_1.ExporterTask,
        reporter_task_1.ReporterTask,
        monitoring_task_1.MonitoringTask,
        aggregation_task_1.AggregationTask,
        log_task_1.LogTask,
        custom_task_1.CustomTask,
        prediction_task_1.PredictionTask,
        stuckJob_task_1.StuckJobTask,
        healthStatus_task_1.HealthStatusTask])
], TaskQueueService);
//# sourceMappingURL=taskQueue.service.js.map