#
# 3PAR.pm
#
# v1.7.1

# Changes:
# New release
# Add showsys space

package HP3PAR;    #

use Carp;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use Xormon;
use JSON qw(encode_json decode_json);
require Exporter;
our @ISA       = qw(Exporter);
our @EXPORT    = qw();
our @EXPORT_OK = qw();
our $VERSION   = 1.7.1;

# Modules
use strict;
use Storable qw(retrieve store);
#use Data::Dumper;

#
my %storids;
my %stordata;
my $debug;
my %storstate;
my @error_message = ();

#NG
my %conf = ();

$storstate{'node'} = { "OK"     => "Normal", "Degraded" => "Critical", "Failed" => "Critical" };
$storstate{'ld'}   = { "normal" => "Normal", "degraded" => "Warning", "failed" => "Critical" };
$storstate{'vvol'} = { "normal" => "Normal", "degraded" => "Warning", "failed" => "Critical" };
$storstate{'vlun'} = { "active" => "Normal", "failed"   => "Warning" };
$storstate{'port'} = {
  "ready"     => "Normal",   "loss_sync"           => "Warning",  "config_wait"      => "Warning", "login_wait"    => "Warning",
  "alpa_wait" => "Warning",  "non_participate"     => "Warning",  "taking_coredump"  => "Warning",
  "offline"   => "Warning",  "link_idle_for_reset" => "Warning",  "dhcp_in_progress" => "Warning", "pending_reset" => "Warning",
  "unknown"   => "Critical", "fw_dead"             => "Critical", "error"            => "Critical"
};
$storstate{'power'} = { "OK"     => "Normal", "Degraded" => "Critical", "NotPresent" => "Warning", "Failed" => "Critical" };
$storstate{'pd'}    = { "normal" => "Normal", "degraded" => "Critical", "new"        => "Normal",  "failed" => "Critical" };
$storstate{'rcopy_target'}  = { "New" => "Normal", "ready" => "Normal", "unsupported" => "Critical", "failing" => "Critical", "failed" => "Critical" };
$storstate{'rcopy_link'}  = { "Not Started" => "Warning", "Down" => "Critical", "Up" => "Normal", "Failed" => "Critical" };
$storstate{'rcopy_group'}  = { 
  "New" => "Normal", "Starting" => "Normal", "Started" => "Normal", "Stopped" => "Critical", "Stopped(Grow_Failed)" => "Critical", 
  "Backup" => "Normal", "Restart" => "Normal", "Failsafe" => "Critical", "Logging" => "Critical" 
};
$storstate{'rcopy_map'} = { 
  "New" => "Normal", "New-SyncFromSnap" => "Normal", "New-PreSynced" => "Normal", "Syncing" => "Normal", "Synced" => "Normal", 
  "NotSynced" => "Critical", "Failsafe" => "Critical", "Stale" => "Warning", "Logging" => "Warning"
};

sub process_files {
  my ( $servers, $dir, $storname, $time_str, $interval, $vvolinterval, $dbg ) = @_;
  $stordata{'server_time'}    = $time_str;
  $stordata{'interval'}       = $interval;
  $stordata{'vvolinterval'}   = $vvolinterval;
  $stordata{'out_conf_file'}  = $dir . "/" . $storname . "_hp3parconf_" . $time_str . ".out";
  $stordata{'out_perf_file'}  = $dir . "/" . $storname . "_hp3parperf_" . $time_str . ".out";
  $stordata{'out_state_file'} = $dir . "/" . $storname . "_hp3parstate_" . $time_str . ".out";
  $debug                      = $dbg;
  #
  my $tmpdir = "$dir/tmp/";    # Directory for temporary files
                               #
  foreach my $name ( @{$servers} ) {
    my $filename = $tmpdir . $storname . "." . $name . "." . $time_str . ".txt";

    #&message("Process file: ".$filename);
    if ( -r $filename ) {
      &message( "Process file: " . $filename );
      &parse_showcfg($filename)  if ( $name eq "showcfg" );
      &parse_statpd($filename)   if ( $name eq "statpd" );
      &parse_statvv($filename)   if ( $name eq "statvv" );
      &parse_statvlun($filename) if ( $name eq "statvlun" );
      &parse_statcpu($filename)  if ( $name eq "statcpu" );
      &parse_statport($filename) if ( $name eq "statport" );
      &parse_statcmp($filename)  if ( $name eq "statcmp" );
    } ## end if ( -r $filename )
  } ## end foreach my $name ( @{$servers...})

  # Print output perf file
  foreach my $key ( sort( keys(%stordata) ) ) {
    &print_perf_cpu   if ( $key eq "cpu" );
    &print_conf_vvol  if ( $key eq "vvol" );
    &print_perf_vvol  if ( $key eq "vvol" );
    &print_state_vvol if ( $key eq "vvol" );
    &print_conf_cpg   if ( $key eq "cpg" );
    &print_conf_port  if ( $key eq "port" );
    &print_perf_port  if ( $key eq "port" );
    &print_state_port if ( $key eq "port" );
    &print_conf_host  if ( $key eq "host" );
    &print_perf_host  if ( $key eq "host" );
    &print_conf_sys   if ( $key eq "sys" );
    &print_state_node if ( $key eq "node" );
    &print_conf_pd    if ( $key eq "pd" );
    &print_perf_pd    if ( $key eq "pd" );
    &print_state_pd   if ( $key eq "pd" );
    &print_perf_cmp   if ( $key eq "cmp" );
    &print_conf_rcopy  if ( $key eq "rcopy_map" );
    &print_state_rcopy if ( $key eq "rcopy_map" );
    &print_state_psu   if ( $key eq "psu" );
    &print_state_cage  if ( $key eq "cage" )

  } ## end foreach my $key ( sort( keys...))

  # Check storage release version
  my $tmp_version_file = $tmpdir . $storname . ".showversion.txt";
  my $fwversion        = &get_version($tmp_version_file);
  if ( defined $fwversion && defined $stordata{'sys'}->{'Release version'} ) {
    if ( $stordata{'sys'}->{'Release version'} =~ /^(\d+\.\d+\.\d+)\s+/ ) {
      if ( $fwversion ne $1 ) {
        &warning( "Storage release version changed [ " . $fwversion . " -> " . $1 . " ]." );
        system( "/bin/rm -f " . $tmp_version_file );
      }
    } ## end if ( $stordata{'sys'}->...)
  } ## end if ( defined $fwversion...)

  #print("__DBG__: IDs\n",Dumper(\%storids),"\n");
  #print("__DBG__: IDs CPG\n",Dumper($storids{'cpg'}),"\n");
  #print("__DBG__: DATA CPG\n",Dumper($stordata{'cpg'}->{0}),"\n");
  #print("__DBG__: IDs HOST\n",Dumper($storids{'host'}),"\n");
  #print("__DBG__: DATA HOST\n",Dumper($stordata{'host'}),"\n");
  #print("__DBG__: IDs LD\n",Dumper($storids{'ld'}),"\n");
  #print("__DBG__: DATA LD\n",Dumper($stordata{'ld'}->{10}),"\n");
  #print("__DBG__: IDs PORT\n",Dumper(\$storids{'port'}),"\n");
  #print("__DBG__: DATA PORT\n",Dumper($stordata{'port'}->{'0:3:1'}),"\n");
  #print("__DBG__: DATA VVOL\n",Dumper($stordata{'vvol'}),"\n");
  #print("__DBG__: DATA VLUN\n",Dumper($stordata{'vlun'}),"\n");
  #print("__DBG__: DATA CPU\n",Dumper($stordata{'cpu'}),"\n");
  #print("__DBG__: DATA CPU\n",Dumper($stordata{'cpu'}->{'0,total'}),"\n");
  #print("__DBG__: DATA NODE STATE\n",Dumper($stordata{'node'}),"\n");
  #print("__DBG__: DATA NODE POWER\n",Dumper($stordata{'power'}),"\n");
  #print("__DBG__: DATA VVOL 2:\n",Dumper($stordata{'vvol'}->{'2'}),"\n"); #'2016-04-04 14:41:53'
  #print("__DBG__: DATA SYS\n",Dumper($stordata{'sys'}),"\n");
  #print("__DBG__: DATA PD 0:\n",Dumper($stordata{'pd'}->{'0'}),"\n");
  #print("__DBG__: DATA CMP 0:\n",Dumper($stordata{'cmp'}->{'0'}),"\n");
  #print("__DBG__: IDs\n",Dumper($storids{'rcopy_link'}),"\n");
  #print("__DBG__: DATA CMP 0:\n",Dumper($stordata{'rcopy_link'}->{7}),"\n");
  #print("__DBG__: IDs\n",Dumper($storids{'rcopy_group'}),"\n");
  #print("__DBG__: DATA CMP 0:\n",Dumper($stordata{'rcopy_group'}),"\n");
  #print("__DBG__: IDs\n",Dumper($storids{'rcopy_map'}),"\n");
  #print("__DBG__: DATA CMP 0:\n",Dumper($stordata{'rcopy_map'}),"\n");
} ## end sub process_files

sub parse_showcfg {
  my $filename = shift;
  if ( not defined $stordata{'sys'} ) {
    my %d;
    $stordata{'sys'} = \%d;
  }
  $stordata{'sys'}->{'ID'}      = "";
  $stordata{'sys'}->{'Name'}    = "";
  $stordata{'sys'}->{'Model'}   = "";
  $stordata{'sys'}->{'Serial'}  = "";
  $stordata{'sys'}->{"Volumes"} = "";
  #
  &message( "parse_showcfg: Process file: " . $filename );
  open( my $data, '<', $filename ) || die( "Cannot open file: " . $filename . "\n" );
  my $flag = 0;
  my $key  = "_none_";
  my $key2 = 0;
  my $rcgroupid;
  my $cage_name;

  while ( my $ln = <$data> ) {
    chomp $ln;

    #print("__DBG__: [$flag]-$key-${key2}-|".$ln."\n");
    if ( $ln eq "" && $key ne "cage" ) { $flag = 0; $key = "_none_"; next; }

    # there is several varianst of the header based on storage firmware, this should work for all
    if ( $ln =~ /ID.*Name.*Model.*Serial.*Nodes/ ) { $flag = 1; $key = "sys"; $key2 = 1; next; }
    if ( $ln =~ /.*[-]+\s*System\s+Capacity.*/ )   { $flag = 1; $key = "sys"; $key2 = 2; next; }
    if ( $ln =~ /.*\s*(Release\s+version)\s+(.*)/ ) {
      $flag                 = 1;
      $key                  = "sys";
      $key2                 = 2;
      $stordata{$key}->{$1} = $2;
      next;
    }
    if ( $ln =~ /.*Node\s+-State-\s+/ )      { $flag = 1; $key = "node";  next; }
    if ( $ln =~ /.*Node\s+PS\s+/ )           { $flag = 1; $key = "power"; next; }
    if ( $ln =~ /.*Id\s+Name\s+Persona\s+/ ) { $flag = 1; $key = "host";  next; }

    # Keep exact order here! firwares provides different outputs
    if ( $ln =~ /^.*Id\s+Name\s+Warn%\s+VVs\s+TPVVs\s+TDVVs\s+Usr\s+Snp\s+Base/ ) { $flag = 1; $key = "cpg"; $key2 = 3; next; }
    if ( $ln =~ /^.*Id\s+Name\s+Warn%\s+VVs\s+TPVVs\s+TDVVs\s+/ )                 { $flag = 1; $key = "cpg"; $key2 = 1; next; }
    if ( $ln =~ /^.*Id\s+Name\s+Warn%\s+VVs\s+TPVVs\s+Usr\s+/ )                   { $flag = 1; $key = "cpg"; $key2 = 2; next; }

    if ( $ln =~ /^.*Id\s+Name\s+RAID\s+/ )                                                          { $flag = 1; $key = "ld"; $key2 = "normal"; next; }
    if ( $ln =~ /^.*Id\s+Name\s+CPG\s+/ )                                                           { $flag = 1; $key = "ld"; $key2 = "detail"; next; }
    if ( $ln =~ /^.*Id\s+Name\s+-State-\s+/ )                                                       { $flag = 1; $key = "ld"; $key2 = "state"; next; }
    if ( $ln =~ /^.*Id\s+Name\s+Prov\s+Type\s+CopyOf\s+/ )                                          { $flag = 1; $key = "vvol"; $key2 = "normal"; next; }
    if ( $ln =~ /^.*Name\s+CPG\s+Adm\s+Snp\s+Usr\s+/ )                                              { $flag = 1; $key = "vvol"; $key2 = "cpg"; next; }
    if ( $ln =~ /^.*Lun\s+VVName\s+HostName\s+-Host_WWN\/iSCSI_Name-\s+Port\s+Type\s+Status\s+ID/ ) { $flag = 1; $key = "vlun"; next; }
    if ( $ln =~ /^.*N:S:P\s+Mode\s+State\s+/ )                                                      { $flag = 1; $key = "port"; next; }
    if ( $ln =~ /^\s*Id\s+CagePos\s+Type\s+/ )                                                      { $flag = 1; $key = "pd"; next; }
    if ( $ln =~ /^.*Name\s+ID\s+Type\s+Status\s+Options\s+Version\s+Policy\s+Socks\s+/ )            { $flag = 1; $key = "rcopy"; $key2 = "target"; next; }
    if ( $ln =~ /^.*Target\s+Name\s+IPC\s+ID\s+Node\s+Address\s+/ )                                 { $flag = 1; $key = "rcopy"; $key2 = "link"; next; }
    if ( $ln =~ /^.*Name\s+ID\s+Target\s+Status\s+Role\s+Mode\s+LocalUserCpg\s+/ )                  { $flag = 1; $key = "rcopy"; $key2 = "group"; next; }
    if ( $ln =~ /^.*LocalVV\s+ID\s+RemoteVV\s+ID\s+SyncStatus\s+Resync_ss\s+/ )                     { $flag = 1; $key = "rcopy"; $key2 = "localvv"; next; }

    if ( $ln =~ /^.*PS\s+PSState\s+ACState\s+DCState\s+Fan\s+State\s+/ ) { $flag = 1; $key = "cage"; $key2 = "psu"; next; }
    if ( $ln =~ /^.*Cage\s+detail\s+info\s+for\s+/ ) { $flag = 1; $key = "cage"; $key2 = "cage"; }
    #
    if ( $ln =~ /^[-]+\s*$/ && $key ne "cage" ) { $flag = 0; $key = "_none_"; next; }
    if ( $ln =~ /.*cli%/ )                      { $flag = 0; $key = "_none_"; next; }
    if ( $flag && $key eq "sys" && $key2 == 1 ) {

      # 44035 3par01 HP_3PAR 7400 1644035      2      0  3342336  2878464  463872         0
      #if ( $ln =~ /^\s*([-\w]+)\s+([-\w]+)\s+([-\w]+\s+\d+[a-z]*)\s+(\d+).*/ ) {
      # 0x24B70 HYD-HPE-3PAR-8450-2749-03 HPE_3PAR 8450 4C19422749     4      0 Off        234356736 28080128 206276608         0
      #if ( $ln =~ /^\s*([-\w]+)\s+([-\w]+)\s+([-\w]+\s+\d+[a-z]*)\s+([-\w]+)\s+.*/ ) {
      if ( $ln =~ /^\s*([-\w]+)\s+([-\w]+)\s+([-\w]+\s+[-\w]+)\s+([-\w]+)\s+.*/ ) {
        $stordata{$key}->{'ID'}     = $1;
        $stordata{$key}->{'Name'}   = $2;
        $stordata{$key}->{'Model'}  = $3;
        $stordata{$key}->{'Serial'} = $4;
      }
    }
    if ( $flag && $key eq "sys" && $key2 == 2 ) {
      if ( $ln =~ /^\s*(Patches):\s+(.*)\s*/ ) {
        $stordata{$key}->{$1} = $2;
      }
      elsif ( $ln =~ /^\s*([\w]+)\s+:\s+([-\d\.]*)\s*/ ) {
        $stordata{$key}->{$1} = $2;
      }
      elsif ( $ln =~ /^\s*(Total Capacity)\s+:\s+([-\d\.]*)\s*/ ) {
        $stordata{$key}->{$1} = $2;
      }
    }
    if ( $flag && $key eq "node" ) {
      if ( $ln =~ /^\s*(\d+)\s+([-\w\.]+)\s+([-\w\.]*)/ ) {
        $stordata{$key}->{$1}->{'state'}          = $2;
        $stordata{$key}->{$1}->{'detailed_state'} = $3;
      }
    }
    if ( $flag && $key eq "power" ) {
      if ( $ln =~ /^\s*(\d+,\d+)\s+(.*)/ ) {

        #Node PS -Assy_Part- -Assy_Serial-- ACState DCState PSState
        my @a = split( /,/,   $1 );
        my @b = split( /\s+/, $2 );

        #print("__DBG_3: Node ID: ".$1." = ".$a[0]." - State: ".Dumper(\@b)."\n");
        $stordata{$key}->{ $b[0] }->{'ACState'} = $b[3];
        $stordata{$key}->{ $b[0] }->{'DCState'} = $b[4];
        $stordata{$key}->{ $b[0] }->{'PSState'} = $b[5];
      }
    }
    if ( $flag && $key eq "host" ) {

      #my %ids; $storids{$key} = \%ids;
      if ( $flag && $ln =~ /^\s*(\d+)\s+([-\w\.]+)\s+(.*)\s*/ ) {

        #$ids{$2} = $1;
        $storids{$key}->{$2}            = $1;
        $stordata{$key}->{$1}->{'id'}   = $1;
        $stordata{$key}->{$1}->{'name'} = $2;
        my @a = split( /\s+/, $3 );
        &host_conf( $1, $2, \@a );
      }
    }
    if ( $flag && $key eq "cpg" ) {
      if ( $ln =~ /^\s*(\d+)\s+([-\w\.]+)\s+([-\d\.\s]*)/ ) {

        #print("__DBG_2: CPG ID: ".$1." - Name: ".$2." === $key2 : $ln \n");
        $storids{$key}->{$2} = $1;
        my @a = split( /\s+/, $3 );
        &cpg_conf( $1, $2, \@a )   if ( $key2 == 1 );
        &cpg_conf_2( $1, $2, \@a ) if ( $key2 == 2 );
        &cpg_conf_3( $1, $2, \@a ) if ( $key2 == 3 );
      }
    }
    if ( $flag && $key eq "ld" ) {
      if ( $key2 eq "normal" && $ln =~ /^\s*(\d+)\s+([-\w\.]+)\s+(.*)\s*/ ) {
        $storids{$key}->{$2}            = $1;
        $stordata{$key}->{$1}->{'id'}   = $1;
        $stordata{$key}->{$1}->{'name'} = $2;
        my @a = split( /\s+/, $3 );
        &ld_conf( $1, $2, \@a );
      }
      if ( $key2 eq "detail" && $ln =~ /^\s*(\d+)\s+([-\w\.]+)\s+(.*)\s*/ ) {
        $storids{$key}->{$2}            = $1;
        $stordata{$key}->{$1}->{'id'}   = $1;
        $stordata{$key}->{$1}->{'name'} = $2;
        my @a = split( /\s+/, $3 );
        &ld_conf_detail( $1, $2, \@a );
      }
      if ( $key2 eq "state" && $ln =~ /^\s*(\d+)\s+([-\w\.]+)\s+([-\w\.]+)\s+([-\w\.]+)\s*/ ) {

        #print("__DBG_2: LD ID: ".$1." - Name: ".$2."\n");
        $storids{$key}->{$2}                      = $1;
        $stordata{$key}->{$1}->{'id'}             = $1;
        $stordata{$key}->{$1}->{'name'}           = $2;
        $stordata{$key}->{$1}->{'state'}          = $3;
        $stordata{$key}->{$1}->{'detailed_state'} = $4;
      }
    }
    if ( $flag && $key eq "vvol" ) {
      if ( $key2 eq "normal" && $ln =~ /^\s*(\d+)\s+([-\w\.]+)\s+(.*)\s*/ ) {

        #print("__DBG_2: VVol ID: ".$1." - Name: ".$2."\n");
        $storids{$key}->{$2}            = $1;
        $stordata{$key}->{$1}->{'id'}   = $1;
        $stordata{$key}->{$1}->{'name'} = $2;
        my @a = split( /\s+/, $3 );
        &vvol_conf( $1, $2, \@a );
      }
      if ( $key2 eq "cpg" && $ln =~ /^\s*([-\w\.]+)\s+([-\w\.]+)\s+(\d+)\s+(\d+)\s+(\d+)\s+([-\d\.\s]*)/ ) {
        my $vvid    = $storids{'vvol'}->{$1};
        my $cpgname = $2;
        my $adm     = $3;
        my $snp     = $4;
        my $usr     = $5;

        # print STDERR ("__DBG_2: VV ID: $vvid == CPG Name: $cpgname - Snap $snp - User $usr\n");
        if ( $cpgname ne "---" && $usr != 0 ) {
          my $cpgid = $storids{'cpg'}->{$cpgname};

          # print STDERR ("__DBG_3: VV ID: $vvid == CPG ID: $cpgid - Name: $cpgname\n");
          # print STDERR ("__DBG_4: DATA VVOL $vvid Name: ",$stordata{'vvol'}->{$vvid}->{'name'});
          if ( not defined $stordata{$key}->{$vvid}->{'cpgname'} ) {
            $stordata{$key}->{$vvid}->{'cpgid'}   = $cpgid;
            $stordata{$key}->{$vvid}->{'cpgname'} = $cpgname;
          }
        }
        elsif ( $cpgname eq "---" ) {
          $stordata{$key}->{$vvid}->{'cpgid'}   = "-";
          $stordata{$key}->{$vvid}->{'cpgname'} = $cpgname;
        }
      }
    }
    if ( $flag && $key eq "vlun" ) {

      # Lun VVName      HostName  -Host_WWN/iSCSI_Name-  Port Type Status ID
      #   0 vol02-RAID1 ESXi2-new 2100001B3291D55C      0:1:1 host active  0

      if ( $ln =~ /^\s*(\d+)\s+([-\w\.]+)\s+([-\w\.]+)\s+(.*)\s*/ ) {

        #print("__DBG_7: VVname: ".$2." - HostName: ".$3."\n");
        my $hostid;
        if ( defined $storids{'host'}->{$3} ) {
          $hostid = $storids{'host'}->{$3};
        }
        else {
          warning( "vlun: Host ID is not defined for Host name " . $3 );
          next;
        }
        if ( not $2 ) { next; }
        &vlun_conf( $hostid, $3, $2 );
      }
    }
    if ( $flag && $key eq "port" ) {
      if ( $ln =~ /^\s*([:\d]+)\s+(.*)\s*/ ) {
        $storids{$key}->{ 'P' . $1 } = $1;
        $stordata{$key}->{$1}->{'id'} = $1;
        my @a = split( /\s+/, $2 );
        &port_conf( $1, \@a );
      }
    }
    if ( $flag && $key eq "pd" ) {
      if ( $ln =~ /^\s*(\d+)\s+(.*)\s*/ ) {

        #print("__DBG_2: VVol ID: ".$1." - ".$2."\n");
        $stordata{$key}->{$1}->{'id'} = $1;
        my @a = split( /\s+/, $2 );
        &pd_conf( $1, \@a );
      }
    }
    if ( $flag && $key eq "rcopy" ) {
      if ( $key2 eq "target" && $ln =~ /^\s*([-\w\.]+)\s+(\d+)\s+(.*)\s*/ ) {

        #print("__DBG_RC_1: RC_Taget ID: ".$1." - Name: ".$2."\n");
        $storids{ $key . '_' . $key2 }->{$1}            = $2;
        $stordata{ $key . '_' . $key2 }->{$2}->{'id'}   = $2;
        $stordata{ $key . '_' . $key2 }->{$2}->{'name'} = $1;
        my @a = split( /\s+/, $ln );
        &rcopy_target_conf( $2, \@a );
      }
      if ( $key2 eq "link" && $ln =~ /^\s*([-\w\.]+)\s+([-\w\.]+)\s+([-\w\.]+)\s*(\d+)\s+(.*)\s*/ ) {

        #print("__DBG_RC_2: RC_Link ID: ".$4." - Name: ".$2."\n");
        $storids{ $key . '_' . $key2 }->{$2} = $4;
        $stordata{ $key . '_' . $key2 }->{$4}->{'id'} = $4;
        &rcopy_link_conf( $4, $ln );
      }
      if ( $key2 eq "group" && $ln =~ /^([-\w\.]+)\s+(\d+)\s+(.*)\s*/ ) {

        #print("__DBG_RC_3: RC_Group ID: ".$2." - Name: ".$1."\n");
        $storids{ $key . '_' . $key2 }->{$1}          = $2;
        $stordata{ $key . '_' . $key2 }->{$2}->{'id'} = $2;
        $rcgroupid                                    = $2;
        my @a = split( /\s+/, $ln );
        &rcopy_group_conf( $2, \@a );
      }
      if ( $key2 eq "localvv" && $ln =~ /^\s+([-\w\.]+)\s+(\d+)\s+(.*)\s*/ ) {

        #print("__DBG_RC_4: RC_Map ID: ".$2." - Name: ".$1."\n");
        $storids{ $key . '_' . "map" }->{$1}                = $2;
        $stordata{ $key . '_' . "map" }->{$2}->{'id'}       = $2;
        $stordata{ $key . '_' . "map" }->{$2}->{'group_id'} = $rcgroupid;
        my @a = split( /\s+/, $ln );
        shift(@a);    # Suppress first element
        &rcopy_map_conf( $2, \@a );
      }
    }
    if ( $flag && $key eq "cage" ) {
      if ( $key2 eq "cage" ) {
        if ( $ln =~ m/^$/ ) { next; }

        #Get Cage Name
        if ( $ln =~ /^.*Cage\s+detail\s+info\s+for\s+/ ) {
          $cage_name = $ln;
          $cage_name =~ s/^.*Cage\s+detail\s+info\s+for//;
          $cage_name =~ s/-//g;
          $cage_name =~ s/ //g;
          chomp($cage_name);
        }

        #Get Cage State
        if ( $ln =~ /^.*OverallState/ ) {
          my $state = $ln;
          $state =~ s/^.*OverallState:\s+//;
          $state =~ s/ //g;
          chomp($state);
          $stordata{$key2}->{$cage_name}->{'id'}    = $cage_name;
          $stordata{$key2}->{$cage_name}->{'state'} = $state;
        }
      }
      if ( $key2 eq "psu" ) {
        if ( $ln =~ m/^$/ ) { $flag = 0; $key = "_none_"; next; }
        $ln =~ s/^\s*(.*?)\s*$/$1/;
        my ( $id, $state ) = split( /\s+/, $ln );
        chomp($id);
        chomp($state);
        $stordata{$key2}->{$cage_name}->{$id}->{'id'}    = $id;
        $stordata{$key2}->{$cage_name}->{$id}->{'state'} = $state;
        $stordata{$key2}->{$cage_name}->{$id}->{'cage'}  = $cage_name;
      }
    }    #end if cage
  }
  close $data;

}

sub get_version {
  my $filename = shift;
  if ( not defined $stordata{'sys'} ) {
    my %d;
    $stordata{'sys'} = \%d;
  }
  #
  &message( "get_version: Process file: " . $filename );
  open( my $data, '<', $filename ) || die( "Cannot open file: " . $filename . "\n" );
  my $ver;
  while ( my $ln = <$data> ) {
    chomp $ln;
    if ( $ln =~ /.*\s*Release\s+version\s+(\d+\.\d+\.\d+).*/ ) {
      $ver = $1;
      next;
    }
  } ## end while ( my $ln = <$data> )
  close $data;
  if ($ver) {
    return ($ver);
  }
  else {
    return (undef);
  }
} ## end sub get_version

sub print_conf_sys {
  open( my $fh, '>>', $stordata{'out_conf_file'} );
  #
  print $fh "\nSystem Level Configuration\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  # host_id,id,name,
  # port_count,iogrp_count,status,
  # IQN WWPN,Volume IDs,Volume Names
  #
  print $fh "Total Capacity," . $stordata{'sys'}->{"Total Capacity"} . "\n";
  print $fh "Allocated," . $stordata{'sys'}->{"Allocated"} . "\n";
  print $fh "Allocated_Volumes," . $stordata{'sys'}->{"Volumes"} . "\n";
  print $fh "Allocated_System," . $stordata{'sys'}->{"System"} . "\n";
  print $fh "Free," . $stordata{'sys'}->{"Free"} . "\n";
  print $fh "Failed," . $stordata{'sys'}->{"Failed"} . "\n";
  print $fh "Release version," . $stordata{'sys'}->{"Release version"} . "\n";
  if ( defined ($stordata{'sys'}->{"Patches"}) && ! $stordata{'sys'}->{"Patches"} eq '') {
    # newer 3PARs do not provide patch level
    print $fh "Patches," . $stordata{'sys'}->{"Patches"} . "\n";
  }
  print $fh "Storage System ID," . $stordata{'sys'}->{"ID"} . "\n";
  print $fh "Storage System Name," . $stordata{'sys'}->{"Name"} . "\n";
  print $fh "Storage System Model," . $stordata{'sys'}->{"Model"} . "\n";
  print $fh "Storage System Serial," . $stordata{'sys'}->{"Serial"} . "\n";

  close($fh);
} ## end sub print_conf_sys

sub print_state_node {
  open( my $fh, '>>', $stordata{'out_state_file'} );
  #
  print $fh "\nNode Level State\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #
  print $fh "Node,State,Level,Detailed_State\n";
  foreach my $id ( sort { $a <=> $b } ( keys( %{ $stordata{node} } ) ) ) {
    my $state = $stordata{'node'}->{$id}->{"state"};
    printf $fh ( "%d,%s,%s,%s,\n", $id, $state, $storstate{'node'}->{$state}, $stordata{'node'}->{$id}->{"detailed_state"} );
  }
  print $fh "\nPower Level State\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #
  print $fh "PowerID,State,Level\n";
  foreach my $id ( sort { $a <=> $b } ( keys( %{ $stordata{power} } ) ) ) {
    my $state = $stordata{'power'}->{$id}->{'PSState'};
    printf $fh ( "%d,%s,%s,\n", $id, $state, $storstate{'power'}->{$state} );
  }

  close($fh);
} ## end sub print_state_node

sub cpg_conf {
  my ( $id, $name, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$a->[6]."\n");
  #                                   ------------------(MB)------------------
  #           ----Volumes---- -Usage- ----- Usr ----- ---- Snp ---- -- Adm ---
  #     Warn% VVs TPVVs TDVVs Usr Snp   Total    Used   Total  Used Total Used
  #         -   1     1     0   1   1     512     512   34304   512  8192  128
  #         0   1     2     3   4   5       6       7       8     9    10   11

  my $stat;
  if ( not defined $stordata{'cpg'}->{$id} ) {
    my %d;
    $stat                             = \%d;
    $stordata{'cpg'}->{$id}           = $stat;
    $stordata{'cpg'}->{$id}->{'name'} = $name;
    $stordata{'cpg'}->{$id}->{'id'}   = $id;
  } ## end if ( not defined $stordata...)
  $stordata{'cpg'}->{$id}->{'usr_tot_cap'} = $a->[6];
  $stordata{'cpg'}->{$id}->{'usr_use_cap'} = $a->[7];
  $stordata{'cpg'}->{$id}->{'snp_tot_cap'} = $a->[8];
  $stordata{'cpg'}->{$id}->{'snp_use_cap'} = $a->[9];
  if ( defined $a->[10] ) {
    $stordata{'cpg'}->{$id}->{'adm_tot_cap'} = $a->[10];
  }
  else {
    $stordata{'cpg'}->{$id}->{'adm_tot_cap'} = 0;
  }
  if ( defined $a->[11] ) {
    $stordata{'cpg'}->{$id}->{'adm_use_cap'} = $a->[11];
  }
  else {
    $stordata{'cpg'}->{$id}->{'adm_use_cap'} = 0;
  }
} ## end sub cpg_conf

sub cpg_conf_2 {
  my ( $id, $name, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$a->[6]."\n");
  #                                   ------------------(MB)------------------
  #           -Volumes- -Usage- ---- Usr ---- -- Snp --- -- Adm ---
  #     Warn% VVs TPVVs Usr Snp  Total   Used Total Used Total Used
  #         -   1     1   1   0    512    512 32256    0  8192  128
  #         0   1     2   3   4      5      6     7    8     9   10
  my $stat;
  if ( not defined $stordata{'cpg'}->{$id} ) {
    my %d;
    $stat                             = \%d;
    $stordata{'cpg'}->{$id}           = $stat;
    $stordata{'cpg'}->{$id}->{'name'} = $name;
    $stordata{'cpg'}->{$id}->{'id'}   = $id;
  } ## end if ( not defined $stordata...)
  $stordata{'cpg'}->{$id}->{'usr_tot_cap'} = $a->[5];
  $stordata{'cpg'}->{$id}->{'usr_use_cap'} = $a->[6];
  $stordata{'cpg'}->{$id}->{'snp_tot_cap'} = $a->[7];
  $stordata{'cpg'}->{$id}->{'snp_use_cap'} = $a->[8];
  $stordata{'cpg'}->{$id}->{'adm_tot_cap'} = $a->[9];
  $stordata{'cpg'}->{$id}->{'adm_use_cap'} = $a->[10];
} ## end sub cpg_conf_2


sub cpg_conf_3 {
  my ( $id, $name, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$a->[6]."\n");
  #           ----Volumes---- -Usage- -----------(MiB)-----------
  #     Warn% VVs TPVVs TDVVs Usr Snp    Base   Snp  Free   Total
  #         -  30    30     0  30  30 9554816 30720 48256 9633792
  #         0   1     2     3   4   5      6     7    8       9 
  #-------------------------------------------------------------------
  my $stat;
  if ( not defined $stordata{'cpg'}->{$id} ) {
    my %d;
    $stat                             = \%d;
    $stordata{'cpg'}->{$id}           = $stat;
    $stordata{'cpg'}->{$id}->{'name'} = $name;
    $stordata{'cpg'}->{$id}->{'id'}   = $id;
  } ## end if ( not defined $stordata...)
  $stordata{'cpg'}->{$id}->{'usr_tot_cap'} = $a->[9];
  $stordata{'cpg'}->{$id}->{'usr_use_cap'} = $a->[6];
  $stordata{'cpg'}->{$id}->{'snp_tot_cap'} = $a->[7];
  $stordata{'cpg'}->{$id}->{'snp_use_cap'} = $a->[7];
  $stordata{'cpg'}->{$id}->{'adm_tot_cap'} = 0;
  $stordata{'cpg'}->{$id}->{'adm_use_cap'} = 0;
#print "==== $a->[9] , $a->[6] , $a->[7] \n";
} ## end sub cpg_conf_3

sub vvol_conf {
  my ( $id, $name, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$a->[6]."\n");
  # Prov Type  CopyOf         BsId Rd State  Adm_Rsvd_MB Snp_Rsvd_MB Usr_Rsvd_MB VSize_MB Detailed_State
  # 0    1     2              3    4  5      6           7           8           9        10
  #
  my $stat;
  if ( defined $stordata{'vvol'}->{$id} ) {
    $stat = $stordata{'vvol'}->{$id};
  }
  else {
    my %d;
    $stat                              = \%d;
    $stordata{'vvol'}->{$id}           = $stat;
    $stordata{'vvol'}->{$id}->{'name'} = $name;
    $stordata{'vvol'}->{$id}->{'id'}   = $id;
  } ## end else [ if ( defined $stordata...)]
  $stordata{'vvol'}->{$id}->{'prov'}           = $a->[0];
  $stordata{'vvol'}->{$id}->{'type'}           = $a->[1];
  $stordata{'vvol'}->{$id}->{'copyof'}         = $a->[2];
  $stordata{'vvol'}->{$id}->{'bsid'}           = $a->[3];
  $stordata{'vvol'}->{$id}->{'rd'}             = $a->[4];
  $stordata{'vvol'}->{$id}->{'state'}          = $a->[5];
  $stordata{'vvol'}->{$id}->{'adm_size'}       = $a->[6];
  $stordata{'vvol'}->{$id}->{'snp_size'}       = $a->[7];
  $stordata{'vvol'}->{$id}->{'usr_size'}       = $a->[8];
  $stordata{'vvol'}->{$id}->{'virt_size'}      = $a->[9];
  $stordata{'vvol'}->{$id}->{'detailed_state'} = $a->[10];
  $stordata{'vvol'}->{$id}->{'vv_wwn'}         = $a->[11];

  if ( $a->[8] ne "--" ) {
    $stordata{'vvol'}->{$id}->{'real_size'} = $a->[6] + $a->[7] + $a->[8];
  }
  else {
    $stordata{'vvol'}->{$id}->{'real_size'} = 0;
  }
} ## end sub vvol_conf

sub parse_statvv {
  my $filename = shift;
  if ( not defined $stordata{'vvol'} ) {
    my %d;
    $stordata{'vvol'} = \%d;
  }
  my $ids = $storids{'vvol'};
  &message( "parse_statvv: Process file: " . $filename );
  open( my $data, '<', $filename ) || die( "Cannot open file: " . $filename . "\n" );
  my $flag = 0;
  my $time;
  my $date;
  while ( my $ln = <$data> ) {
    chomp $ln;

    #print("__DBG__: [$flag]".$ln."\n");
    # 00:26:09 01/09/2016 -> 2015-07-13 16:09:33
    if ( $ln =~ /^\s*(\d{2}:\d{2}:\d{2})\s+(\d{2})\/(\d{2})\/(\d{4}).*/ ) {
      $flag = 1;
      $time = $4 . "-" . $2 . "-" . $3 . " " . $1;
      next;
    }
    if ( $ln =~ /^[-]+\s*/ ) { $flag = 0; next; }
    if ( $flag && $ln =~ /^\s*([-\w\.]+)\s+([rwt]{1})\s+([-\d\.\s]*)/ ) {
      my $name   = $1;
      my $rwflag = $2;
      my $id     = $ids->{$name};
      if ( not defined $id ) {
        &message( "parse_statvv: Volume: " . $name . " has not ID" );
        next;
      }

      #			my %d;
      #			$d{name} = $name;
      #			$d{id} = $id;
      my @a = split( /\s+/, $3 );
      &vvol_data( $id, $name, $rwflag, $time, \@a );
    } ## end if ( $flag && $ln =~ /^\s*([-\w\.]+)\s+([rwt]{1})\s+([-\d\.\s]*)/)
  } ## end while ( my $ln = <$data> )
  close $data;
} ## end sub parse_statvv

sub vvol_data {
  my ( $id, $name, $rwflag, $time, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$rwflag." - ".$a->[6]."\n");
  #     I/O per second    KBytes per sec    Svt ms      IOSz KB
  #     Cur  Avg  Max     Cur  Avg  Max     Cur  Avg    Cur  Avg    Qlen
  #     0    1    2       3    4    5       6    7      8    9      10
  #
  my $stat;
  if ( not defined $stordata{'vvol'}->{$id} ) {
    $stordata{'vvol'}->{$id}->{'name'} = $name;
    $stordata{'vvol'}->{$id}->{'id'}   = $id;
  }
  if ( not defined $stordata{'vvol'}->{$id}->{'stat'}->{$time} ) {
    my %d;
    $stat = \%d;
    $stordata{'vvol'}->{$id}->{'stat'}->{$time} = $stat;

    #		$stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'name'} = $name;
    #		$stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'id'} = $id;
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'time'} = $time;
  } ## end if ( not defined $stordata...)

  if ( $rwflag eq "r" ) {
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'rops'}     = $a->[0];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'rops-avg'} = $a->[1];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'rops-max'} = $a->[2];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'rdps'}     = $a->[3];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'rdps-avg'} = $a->[4];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'rdps-max'} = $a->[5];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'rlpo'}     = $a->[6];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'rlpo-avg'} = $a->[7];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'rtpo'}     = $a->[8];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'rtpo-avg'} = $a->[9];
  } ## end if ( $rwflag eq "r" )
  elsif ( $rwflag eq "w" ) {
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'wops'}     = $a->[0];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'wops-avg'} = $a->[1];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'wops-max'} = $a->[2];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'wdps'}     = $a->[3];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'wdps-avg'} = $a->[4];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'wdps-max'} = $a->[5];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'wlpo'}     = $a->[6];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'wlpo-avg'} = $a->[7];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'wtpo'}     = $a->[8];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'wtpo-avg'} = $a->[9];
  } ## end elsif ( $rwflag eq "w" )
  elsif ( $rwflag eq "t" ) {
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'tops'}     = $a->[0];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'tops-avg'} = $a->[1];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'tops-max'} = $a->[2];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'tdps'}     = $a->[3];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'tdps-avg'} = $a->[4];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'tdps-max'} = $a->[5];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'tlpo'}     = $a->[6];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'tlpo-avg'} = $a->[7];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'ttpo'}     = $a->[8];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'ttpo-avg'} = $a->[9];
    $stordata{'vvol'}->{$id}->{'stat'}->{$time}->{'tq'}       = $a->[10];
  } ## end elsif ( $rwflag eq "t" )

} ## end sub vvol_data

sub print_conf_vvol {
  open( my $fh, '>>', $stordata{'out_conf_file'} );
  #
  print $fh "\nVolume Level Configuration\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #volume id:volume name:pool id/name:size:used/free
  print $fh "Volume ID,Volume name,CPG ID,CPG name,State,";
  print $fh "Provisioning,Type,Copy Of,Admin size [MB],Snap size [MB],User size [MB],Real size [MB],Virtual size [MB],Volume UID,\n";

  foreach my $id ( sort { $a <=> $b } ( values( %{ $storids{vvol} } ) ) ) {
    #
    my $cpid;
    my $v = $stordata{'vvol'}->{$id};
    if ( $v->{'copyof'} ne "---" ) {
      $v->{'cpgid'}   = "-";
      $v->{'cpgname'} = "---";
    }
    if ( not defined $v->{'cpgid'} ) {
      $v->{'cpgid'}   = "-";
      $v->{'cpgname'} = "---";
    }

    # Volume ID,Volume name,CPG ID,CPG name,Status,
    printf $fh ( "%d,%s,%s,%s,%s,", $v->{'id'}, $v->{'name'}, $v->{'cpgid'}, $v->{'cpgname'}, $v->{'state'} );

    # Provisioning,Type,Copy Of,Admin size,Snap size,User size,Real size,Virtual size
    printf $fh ( "%s,%s,%s,%s,%s,%s,%d,%d,%s,\n", $v->{'prov'}, $v->{'type'}, $v->{'copyof'}, $v->{'adm_size'}, $v->{'snp_size'}, $v->{'usr_size'}, $v->{'real_size'}, $v->{'virt_size'}, $v->{'vv_wwn'} );
  } ## end foreach my $id ( sort { $a ...})
  close($fh);
} ## end sub print_conf_vvol

sub print_state_vvol {
  my @volumes = ();
  open( my $fh, '>>', $stordata{'out_state_file'} );
  #
  print $fh "\nVolume Level State\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #
  print $fh "Volume ID,Volume name,State,Level,Detailed_State\n";
  foreach my $id ( sort { $a <=> $b } ( values( %{ $storids{'vvol'} } ) ) ) {
    #
    my $v = $stordata{'vvol'}->{$id};
    if ( not defined $v->{'id'} ) {
      &message( "print_conf_vvol: Volume ID: " . $id . " has not state data" );
      next;
    }
    printf $fh ( "%d,%s,%s,%s,%s,\n", $v->{'id'}, $v->{'name'}, $v->{'state'}, $storstate{'vvol'}->{ $v->{'state'} }, $v->{'detailed_state'} );
    my %data;
    $data{id}         = $v->{'id'};
    $data{name}       = $v->{'name'};
    $data{state}      = $storstate{'vvol'}->{ $v->{'state'} };
    $data{dstate}     = $v->{'detailed_state'};
    push(@volumes,\%data);
  }

  $conf{$ENV{'HP3PAR_HOSTCFG_ID'}}{'epar_volumes'} = encode_json(\@volumes);
  close($fh);
}


sub print_perf_vvol {
  open( my $fh, '>>', $stordata{'out_perf_file'} );

  #Volume Level Statistics
  #Interval Start:   2015-07-13 16:08:33
  #Interval End:     2015-07-13 16:09:33
  #Interval Length:  60 seconds
  #---------------------
  print $fh "\nVolume Level Statistics\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #Volume ID,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),Peak Read Response Time (ms),Peak Write Response Time (ms),Host Delay (assuming that all host delay is writes) (ms),Host Delay (assuming that host delay is evenly spread between read and writes) (ms),Read Hits,Write Hits,Data Read (KB),Data Written (KB),Volume (Vdisk) Name,Managed Disk Group ID,Managed Disk Group Name,IO Group ID,IO Group Name,Remote Copy relationship ID,Remote Copy relationship name,Remote Copy Change Volume relationship,FlashCopy map ID,FlashCopy map name,FlashCopy map count,Copy Count,Space Efficient Copy Count,Cache state,Easy Tier On/Off,Easy Tier Status,Preferred Node ID,Capacity (TB),Real Capacity (TB),Used Capacity (TB),Auto Expand,Grain Size,Throttle Unit,Throttle Rate,UDID (for HPUX),
  #0000,2015-07-13 16:09:33,60,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,tsmplinux_rvg,0,SAS450,0,io_grp0,,,no,,,0,1,0,empty,,,,53687091200.000,0,0,0,0,,0,,
  # Volume ID,Time,Interval (s),
  # Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),
  # Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),
  # Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),
  # Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),
  # Peak Read Response Time (ms),Peak Write Response Time (ms),
  # Host Delay (assuming that all host delay is writes) (ms),Host Delay (assuming that host delay is evenly spread between read and writes) (ms),
  # Read Hits,Write Hits,Data Read (KB),Data Written (KB),
  # Volume (Vdisk) Name,Managed Disk Group ID,Managed Disk Group Name,IO Group ID,IO Group Name,
  # Remote Copy relationship ID,Remote Copy relationship name,Remote Copy Change Volume relationship,
  # FlashCopy map ID,FlashCopy map name,FlashCopy map count,
  # Copy Count,Space Efficient Copy Count,Cache state,Easy Tier On/Off,Easy Tier Status,
  # Preferred Node ID,Capacity (TB),Real Capacity (TB),Used Capacity (TB),
  # Auto Expand,Grain Size,Throttle Unit,Throttle Rate,UDID (for HPUX),
  #
  printf $fh "Volume ID,Time,Interval (s),";
  printf $fh "Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),";
  printf $fh "Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),";
  printf $fh ",,,,";
  printf $fh ",,,,";
  printf $fh "Volume (Vdisk) Name,CPG ID,CPG Name,,,";
  printf $fh ",,,\n";

  #printf $fh ",,,,";
  #printf $fh ",,,,";
  #printf $fh "Capacity (TB),Real Capacity (TB),Used Capacity (TB),,,,,,\n";

  foreach my $id ( sort { $a <=> $b } ( values( %{ $storids{vvol} } ) ) ) {
    #
    my $vvol = $stordata{'vvol'}->{$id};
    if ( not defined $vvol->{'stat'} ) {
      &message( "print_perf_vvol: Volume ID: " . $id . " has not perf data" );
      next;
    }
    foreach my $tm ( sort( keys( %{ $vvol->{'stat'} } ) ) ) {
      my $v = $vvol->{'stat'}->{$tm};
      if ( not defined $v->{time} ) { next; }
      &message( "VVOL: " . $vvol->{'name'} . " Time - server: " . $stordata{'server_time'} . " - storage:" . $vvol->{'stat'}->{$tm}->{'time'} );

      #"Volume ID,Time,Interval (s),
      if ( $stordata{'vvolinterval'} > 0 ) {
        printf $fh ( "%d,%s,%s,", $vvol->{'id'}, $v->{time}, $stordata{'vvolinterval'}, );
      }
      else {
        printf $fh ( "%d,%s,%s,", $vvol->{'id'}, $v->{time}, $stordata{'interval'}, );
      }

      #"Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s)"
      printf $fh ( "%d,%d,%d,%d,%d,%d,", $v->{'rops'}, $v->{'wops'}, $v->{'tops'}, $v->{'rdps'}, $v->{'wdps'}, $v->{'tdps'} );

      #"Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms)"
      printf $fh ( "%.2f,%.2f,%.2f,%.2f,%.2f,%.2f,", $v->{'rtpo'}, $v->{'wtpo'}, $v->{'ttpo'}, $v->{'rlpo'}, $v->{'wlpo'}, $v->{'tlpo'} );

      #"Peak Read Response Time (ms),Peak Write Response Time (ms),Host Delay (assuming that all host delay is writes) (ms),Host Delay (assuming that host delay is evenly spread between read and writes) (ms)"
      printf $fh (",,,,");

      #"Read Hits,Write Hits,Data Read (KB),Data Written (KB)";
      printf $fh (",,,,");

      #"Volume (Vdisk) Name,Managed Disk Group ID,Managed Disk Group Name,IO Group ID,IO Group Name";
      if ( defined $vvol->{'cpgid'} ) {
        printf $fh ( "%s,%s,%s,,,", $vvol->{'name'}, $vvol->{'cpgid'}, $vvol->{'cpgname'} );
      }
      else {
        printf $fh ( "%s,,,,,", $vvol->{'name'} );
      }

      #"Remote Copy relationship ID,Remote Copy relationship name,Remote Copy Change Volume relationship";
      printf $fh (",,,\n");

      #"FlashCopy map ID,FlashCopy map name,FlashCopy map count,Copy Count";
      #printf $fh  (",,,,");
      #"Space Efficient Copy Count,Cache state,Easy Tier On/Off,Easy Tier Status,Preferred Node ID";
      #printf $fh  (",,,,,");
      #"Capacity (TB),Real Capacity (TB),Used Capacity (TB),Auto Expand,Grain Size,Throttle Unit,Throttle Rate,UDID (for HPUX),\n";
      #printf $fh  (",,,,,,,0,\n");
    } ## end foreach my $tm ( sort( keys...))
  } ## end foreach my $id ( sort { $a ...})
  close($fh);
} ## end sub print_perf_vvol

sub parse_statvlun {
  my $filename = shift;

  #if ( not defined $stordata{'vlun'} ) {
  #	my %d; $stordata{'vlun'} = \%d;
  #}
  my $ids = $storids{'vlun'};
  &message( "parse_statvlun: Process file: " . $filename );
  open( my $data, '<', $filename ) || die( "Cannot open file: " . $filename . "\n" );
  my $flag = 0;
  my $time;
  my $date;
  while ( my $ln = <$data> ) {
    chomp $ln;

    #print("__DBG__: [$flag]".$ln."\n");
    # 00:26:09 01/09/2016 -> 2015-07-13 16:09:33
    if ( $ln =~ /^\s*(\d{2}:\d{2}:\d{2})\s+(\d{2})\/(\d{2})\/(\d{4}).*/ ) {
      $flag = 1;
      $time = $4 . "-" . $2 . "-" . $3 . " " . $1;
      next;
    }
    if ( $ln =~ /^[-]+\s*/ ) { $flag = 0; next; }

    #         02:20:14 04/13/2016 r/w I/O per second KBytes per sec    Svt ms IOSz KB
    #Lun      VVname  Host  Port      Cur  Avg  Max  Cur  Avg  Max  Cur  Avg Cur Avg Qlen
    #  0 vol02-RAID1 ESXi3 1:1:1   r    0    0    0    0    0    0 0.00 0.00 0.0 0.0    -

    if ( $flag && $ln =~ /^\s*\d+\s+([-\w\.]+)\s+([-\w\.]+)\s+\d+:\d+:\d+\s+([rwt]{1})\s+([-\d\.\s]*)/ ) {
      my $vvname = $1;
      my $host   = $2;
      my $rwflag = $3;
      my @a      = split( /\s+/, $4 );
      &vlun_data( $vvname, $host, $rwflag, $time, \@a );
    } ## end if ( $flag && $ln =~ /^\s*\d+\s+([-\w\.]+)\s+([-\w\.]+)\s+\d+:\d+:\d+\s+([rwt]{1})\s+([-\d\.\s]*)/)
  } ## end while ( my $ln = <$data> )
  close $data;
} ## end sub parse_statvlun

sub vlun_data {
  my ( $vvn, $hst, $rwflag, $time, $a ) = @_;

  #print("__DBG_2: VLUN ID: ".$hst." - Name: ".$vvn." - ".$rwflag." - ".$a->[0]."\n");
  # I/O per second KBytes per sec    Svt ms IOSz KB
  #  Cur  Avg  Max  Cur  Avg  Max  Cur  Avg Cur Avg Qlen
  #    0    0    0    0    0    0 0.00 0.00 0.0 0.0    -
  #    0    1    2    3    4    5    6    7   8   9   10
  #
  my $id = $hst . ":" . $vvn;
  if ( not defined $stordata{'vlun'}->{$id} ) {
    my %d;
    $stordata{'vlun'}->{$id} = \%d;
    $stordata{'vlun'}->{$id}->{'id'} = $id;
  }

  $stordata{'vlun'}->{$id}->{'name'} = $vvn;
  $stordata{'vlun'}->{$id}->{'host'} = $hst;
  $stordata{'vlun'}->{$id}->{'time'} = $time;

  if ( $rwflag eq "r" ) {
    if ( defined $stordata{'vlun'}->{$id}->{'rops'} ) {
      $stordata{'vlun'}->{$id}->{'rops'}     = $stordata{'vlun'}->{$id}->{'rops'} + $a->[0];
      $stordata{'vlun'}->{$id}->{'rops-avg'} = $stordata{'vlun'}->{$id}->{'rops-avg'} + $a->[1];
      $stordata{'vlun'}->{$id}->{'rops-max'} = $stordata{'vlun'}->{$id}->{'rops-max'} + $a->[2];
      $stordata{'vlun'}->{$id}->{'rdps'}     = $stordata{'vlun'}->{$id}->{'rdps'} + $a->[3];
      $stordata{'vlun'}->{$id}->{'rdps-avg'} = $stordata{'vlun'}->{$id}->{'rdps-avg'} + $a->[4];
      $stordata{'vlun'}->{$id}->{'rdps-max'} = $stordata{'vlun'}->{$id}->{'rdps-max'} + $a->[5];
      $stordata{'vlun'}->{$id}->{'rlpo'}     = ( $stordata{'vlun'}->{$id}->{'rlpo'} + $a->[6] ) / 2;
      $stordata{'vlun'}->{$id}->{'rlpo-avg'} = ( $stordata{'vlun'}->{$id}->{'rlpo-avg'} + $a->[7] ) / 2;
      $stordata{'vlun'}->{$id}->{'rtpo'}     = ( $stordata{'vlun'}->{$id}->{'rtpo'} + $a->[8] ) / 2;
      $stordata{'vlun'}->{$id}->{'rtpo-avg'} = ( $stordata{'vlun'}->{$id}->{'rtpo-avg'} + $a->[9] ) / 2;
    } ## end if ( defined $stordata...)
    else {
      $stordata{'vlun'}->{$id}->{'rops'}     = $a->[0];
      $stordata{'vlun'}->{$id}->{'rops-avg'} = $a->[1];
      $stordata{'vlun'}->{$id}->{'rops-max'} = $a->[2];
      $stordata{'vlun'}->{$id}->{'rdps'}     = $a->[3];
      $stordata{'vlun'}->{$id}->{'rdps-avg'} = $a->[4];
      $stordata{'vlun'}->{$id}->{'rdps-max'} = $a->[5];
      $stordata{'vlun'}->{$id}->{'rlpo'}     = $a->[6];
      $stordata{'vlun'}->{$id}->{'rlpo-avg'} = $a->[7];
      $stordata{'vlun'}->{$id}->{'rtpo'}     = $a->[8];
      $stordata{'vlun'}->{$id}->{'rtpo-avg'} = $a->[9];
    } ## end else [ if ( defined $stordata...)]
  } ## end if ( $rwflag eq "r" )
  elsif ( $rwflag eq "w" ) {
    if ( defined $stordata{'vlun'}->{$id}->{'wops'} ) {
      $stordata{'vlun'}->{$id}->{'wops'}     = $stordata{'vlun'}->{$id}->{'wops'} + $a->[0];
      $stordata{'vlun'}->{$id}->{'wops-avg'} = $stordata{'vlun'}->{$id}->{'wops-avg'} + $a->[1];
      $stordata{'vlun'}->{$id}->{'wops-max'} = $stordata{'vlun'}->{$id}->{'wops-max'} + $a->[2];
      $stordata{'vlun'}->{$id}->{'wdps'}     = $stordata{'vlun'}->{$id}->{'wdps'} + $a->[3];
      $stordata{'vlun'}->{$id}->{'wdps-avg'} = $stordata{'vlun'}->{$id}->{'wdps-avg'} + $a->[4];
      $stordata{'vlun'}->{$id}->{'wdps-max'} = $stordata{'vlun'}->{$id}->{'wdps-max'} + $a->[5];
      $stordata{'vlun'}->{$id}->{'wlpo'}     = ( $stordata{'vlun'}->{$id}->{'wlpo'} + $a->[6] ) / 2;
      $stordata{'vlun'}->{$id}->{'wlpo-avg'} = ( $stordata{'vlun'}->{$id}->{'wlpo-avg'} + $a->[7] ) / 2;
      $stordata{'vlun'}->{$id}->{'wtpo'}     = ( $stordata{'vlun'}->{$id}->{'wtpo'} + $a->[8] ) / 2;
      $stordata{'vlun'}->{$id}->{'wtpo-avg'} = ( $stordata{'vlun'}->{$id}->{'wtpo-avg'} + $a->[9] ) / 2;
    } ## end if ( defined $stordata...)
    else {
      $stordata{'vlun'}->{$id}->{'wops'}     = $a->[0];
      $stordata{'vlun'}->{$id}->{'wops-avg'} = $a->[1];
      $stordata{'vlun'}->{$id}->{'wops-max'} = $a->[2];
      $stordata{'vlun'}->{$id}->{'wdps'}     = $a->[3];
      $stordata{'vlun'}->{$id}->{'wdps-avg'} = $a->[4];
      $stordata{'vlun'}->{$id}->{'wdps-max'} = $a->[5];
      $stordata{'vlun'}->{$id}->{'wlpo'}     = $a->[6];
      $stordata{'vlun'}->{$id}->{'wlpo-avg'} = $a->[7];
      $stordata{'vlun'}->{$id}->{'wtpo'}     = $a->[8];
      $stordata{'vlun'}->{$id}->{'wtpo-avg'} = $a->[9];
    } ## end else [ if ( defined $stordata...)]
  } ## end elsif ( $rwflag eq "w" )
  elsif ( $rwflag eq "t" ) {
    if ( defined $stordata{'vlun'}->{$id}->{'tops'} ) {
      $stordata{'vlun'}->{$id}->{'tops'}     = $stordata{'vlun'}->{$id}->{'tops'} + $a->[0];
      $stordata{'vlun'}->{$id}->{'tops-avg'} = $stordata{'vlun'}->{$id}->{'tops-avg'} + $a->[1];
      $stordata{'vlun'}->{$id}->{'tops-max'} = $stordata{'vlun'}->{$id}->{'tops-max'} + $a->[2];
      $stordata{'vlun'}->{$id}->{'tdps'}     = $stordata{'vlun'}->{$id}->{'tdps'} + $a->[3];
      $stordata{'vlun'}->{$id}->{'tdps-avg'} = $stordata{'vlun'}->{$id}->{'tdps-avg'} + $a->[4];
      $stordata{'vlun'}->{$id}->{'tdps-max'} = $stordata{'vlun'}->{$id}->{'tdps-max'} + $a->[5];
      $stordata{'vlun'}->{$id}->{'tlpo'}     = ( $stordata{'vlun'}->{$id}->{'tlpo'} + $a->[6] ) / 2;
      $stordata{'vlun'}->{$id}->{'tlpo-avg'} = ( $stordata{'vlun'}->{$id}->{'tlpo-avg'} + $a->[7] ) / 2;
      $stordata{'vlun'}->{$id}->{'ttpo'}     = ( $stordata{'vlun'}->{$id}->{'ttpo'} + $a->[8] ) / 2;
      $stordata{'vlun'}->{$id}->{'ttpo-avg'} = ( $stordata{'vlun'}->{$id}->{'ttpo-avg'} + $a->[9] ) / 2;
      $stordata{'vlun'}->{$id}->{'tq'}       = $stordata{'vlun'}->{$id}->{'tq'} + $a->[10];
    } ## end if ( defined $stordata...)
    else {
      $stordata{'vlun'}->{$id}->{'tops'}     = $a->[0];
      $stordata{'vlun'}->{$id}->{'tops-avg'} = $a->[1];
      $stordata{'vlun'}->{$id}->{'tops-max'} = $a->[2];
      $stordata{'vlun'}->{$id}->{'tdps'}     = $a->[3];
      $stordata{'vlun'}->{$id}->{'tdps-avg'} = $a->[4];
      $stordata{'vlun'}->{$id}->{'tdps-max'} = $a->[5];
      $stordata{'vlun'}->{$id}->{'tlpo'}     = $a->[6];
      $stordata{'vlun'}->{$id}->{'tlpo-avg'} = $a->[7];
      $stordata{'vlun'}->{$id}->{'ttpo'}     = $a->[8];
      $stordata{'vlun'}->{$id}->{'ttpo-avg'} = $a->[9];
      $stordata{'vlun'}->{$id}->{'tq'}       = $a->[10];
    } ## end else [ if ( defined $stordata...)]
  } ## end elsif ( $rwflag eq "t" )
} ## end sub vlun_data

sub print_perf_vlun {
  open( my $fh, '>>', $stordata{'out_perf_file'} );

  #Volume Level Statistics
  #Interval Start:   2015-07-13 16:08:33
  #Interval End:     2015-07-13 16:09:33
  #Interval Length:  60 seconds
  #---------------------
  print $fh "\nvLUN Level Statistics\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #Volume ID,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),Peak Read Response Time (ms),Peak Write Response Time (ms),Host Delay (assuming that all host delay is writes) (ms),Host Delay (assuming that host delay is evenly spread between read and writes) (ms),Read Hits,Write Hits,Data Read (KB),Data Written (KB),Volume (Vdisk) Name,Managed Disk Group ID,Managed Disk Group Name,IO Group ID,IO Group Name,Remote Copy relationship ID,Remote Copy relationship name,Remote Copy Change Volume relationship,FlashCopy map ID,FlashCopy map name,FlashCopy map count,Copy Count,Space Efficient Copy Count,Cache state,Easy Tier On/Off,Easy Tier Status,Preferred Node ID,Capacity (TB),Real Capacity (TB),Used Capacity (TB),Auto Expand,Grain Size,Throttle Unit,Throttle Rate,UDID (for HPUX),
  #0000,2015-07-13 16:09:33,60,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,tsmplinux_rvg,0,SAS450,0,io_grp0,,,no,,,0,1,0,empty,,,,53687091200.000,0,0,0,0,,0,,
  # Volume ID,Time,Interval (s),
  # Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),
  # Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),
  # Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),
  # Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),
  # Peak Read Response Time (ms),Peak Write Response Time (ms),
  # Host Delay (assuming that all host delay is writes) (ms),Host Delay (assuming that host delay is evenly spread between read and writes) (ms),
  # Read Hits,Write Hits,Data Read (KB),Data Written (KB),
  # Volume (Vdisk) Name,Managed Disk Group ID,Managed Disk Group Name,IO Group ID,IO Group Name,
  # Remote Copy relationship ID,Remote Copy relationship name,Remote Copy Change Volume relationship,
  # FlashCopy map ID,FlashCopy map name,FlashCopy map count,
  # Copy Count,Space Efficient Copy Count,Cache state,Easy Tier On/Off,Easy Tier Status,
  # Preferred Node ID,Capacity (TB),Real Capacity (TB),Used Capacity (TB),
  # Auto Expand,Grain Size,Throttle Unit,Throttle Rate,UDID (for HPUX),
  #
  printf $fh "Volume ID,Time,Interval (s),";
  printf $fh "Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),";
  printf $fh "Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),";
  printf $fh ",,,,";
  printf $fh ",,,,";
  printf $fh "Volume (Vdisk) Name,CPG ID,CPG Name,,,";
  printf $fh ",,,\n";

  #printf $fh ",,,,";
  #printf $fh ",,,,";
  #printf $fh "Capacity (TB),Real Capacity (TB),Used Capacity (TB),,,,,,\n";

  foreach my $id ( sort { $a <=> $b } ( values( %{ $storids{vvol} } ) ) ) {
    #
    my $vlun = $stordata{'vlun'}->{$id};
    if ( not defined $vlun->{'stat'} ) {
      &message( "print_perf_vlun: Volume ID: " . $id . " has not perf data" );
      next;
    }
    foreach my $tm ( sort( keys( %{ $vlun->{'stat'} } ) ) ) {
      my $v = $vlun->{'stat'}->{$tm};
      if ( not defined $v->{time} ) { next; }
      &message( "VLUN: " . $vlun->{'name'} . " Time - server: " . $stordata{'server_time'} . " - storage:" . $vlun->{'stat'}->{$tm}->{'time'} );

      #"Volume ID,Time,Interval (s),
      printf $fh ( "%d,%s,%s,", $vlun->{'id'}, $v->{time}, $stordata{'interval'}, );

      #"Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s)"
      printf $fh ( "%d,%d,%d,%d,%d,%d,", $v->{'rops'}, $v->{'wops'}, $v->{'tops'}, $v->{'rdps'}, $v->{'wdps'}, $v->{'tdps'} );

      #"Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms)"
      printf $fh ( "%.2f,%.2f,%.2f,%.2f,%.2f,%.2f,", $v->{'rtpo'}, $v->{'wtpo'}, $v->{'ttpo'}, $v->{'rlpo'}, $v->{'wlpo'}, $v->{'tlpo'} );

      #"Peak Read Response Time (ms),Peak Write Response Time (ms),Host Delay (assuming that all host delay is writes) (ms),Host Delay (assuming that host delay is evenly spread between read and writes) (ms)"
      printf $fh (",,,,");

      #"Read Hits,Write Hits,Data Read (KB),Data Written (KB)";
      printf $fh (",,,,");

      #"Volume (Vdisk) Name,Managed Disk Group ID,Managed Disk Group Name,IO Group ID,IO Group Name";
      if ( defined $vlun->{'cpgid'} ) {
        printf $fh ( "%s,%s,%s,,,", $vlun->{'name'}, $vlun->{'cpgid'}, $vlun->{'cpgname'} );
      }
      else {
        printf $fh ( "%s,,,,,", $vlun->{'name'} );
      }

      #"Remote Copy relationship ID,Remote Copy relationship name,Remote Copy Change Volume relationship";
      printf $fh (",,,\n");

      #"FlashCopy map ID,FlashCopy map name,FlashCopy map count,Copy Count";
      #printf $fh  (",,,,");
      #"Space Efficient Copy Count,Cache state,Easy Tier On/Off,Easy Tier Status,Preferred Node ID";
      #printf $fh  (",,,,,");
      #"Capacity (TB),Real Capacity (TB),Used Capacity (TB),Auto Expand,Grain Size,Throttle Unit,Throttle Rate,UDID (for HPUX),\n";
      #printf $fh  (",,,,,,,0,\n");
    } ## end foreach my $tm ( sort( keys...))
  } ## end foreach my $id ( sort { $a ...})
  close($fh);
} ## end sub print_perf_vlun

sub parse_statcpu {
  my $filename = shift;
  if ( not defined $stordata{'cpu'} ) {
    my %d;
    $stordata{'cpu'} = \%d;
  }
  my $ids = $storids{'cpu'};
  &message( "parse_statcpu: Process file: " . $filename );
  open( my $data, '<', $filename ) || die( "Cannot open file: " . $filename . "\n" );
  my $time = " ";
  while ( my $ln = <$data> ) {
    chomp $ln;

    #print("__DBG_1: ".$ln."\n");
    # 00:26:09 01/09/2016 -> 2015-07-13 16:09:33
    if ( $ln =~ /^\s*(\d{2}:\d{2}:\d{2})\s+(\d{2})\/(\d{2})\/(\d{4}).*/ ) {
      $time = $4 . "-" . $2 . "-" . $3 . " " . $1;
      next;
    }

    #   node,cpu user sys idle intr/s ctxt/s
    if ( $ln =~ /^\s*node,cpu\s+user\s+sys\s+.*/ ) { next; }

    #        0,7    0   0  100
    #    0,total    0   1   99   1957   3685
    if ( $ln =~ /^\s*(\d+,\d+)\s+(\d+)\s+(\d+)\s+(\d+)/ ) {
      my $id = $1;
      my %d;
      $d{'id'}                = $id;
      $d{'time'}              = $time;
      $d{'user'}              = $2;
      $d{'sys'}               = $3;
      $d{'idle'}              = $4;
      $stordata{'cpu'}->{$id} = \%d;
    } ## end if ( $ln =~ /^\s*(\d+,\d+)\s+(\d+)\s+(\d+)\s+(\d+)/)
    if ( $ln =~ /^\s*(\d+,total)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/ ) {
      my $id = $1;
      my %d;
      $d{'id'}                = $id;
      $d{'time'}              = $time;
      $d{'user'}              = $2;
      $d{'sys'}               = $3;
      $d{'idle'}              = $4;
      $d{'intrps'}            = $5;
      $d{'ctxtps'}            = $6;
      $stordata{'cpu'}->{$id} = \%d;
    } ## end if ( $ln =~ /^\s*(\d+,total)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/)
  } ## end while ( my $ln = <$data> )
  close $data;
} ## end sub parse_statcpu

sub print_perf_cpu {
  open( my $fh, '>>', $stordata{'out_perf_file'} );

  print $fh "\nCPU-Node Level Statistics\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  # Node,Time,Interval,CPU Utilization - User,CPU Utilization - System,CPU Utilization - Idle,intr/s,ctxt/s,
  # node1_core00,2015-07-13 16:03:33,60,node1,0,1.597,0.000,
  print $fh "Node,Time,Interval (s),";
  print $fh "CPU Utilization - User,CPU Utilization - System,CPU Utilization - Idle,";
  print $fh "intr/s,ctxt/s,\n";
  #
  foreach my $id ( sort( keys( %{ $stordata{'cpu'} } ) ) ) {
    #
    my $v = $stordata{'cpu'}->{$id};
    my ( $node, $cpu ) = split( /,/, $v->{'id'} );
    if ( $cpu ne "total" ) { next; }

    #"Node-CPU,Time,Interval (s),Node,CPU Core ID,
    printf $fh ( "%d,%s,%s,", $node, $v->{time}, $stordata{'interval'} );

    # CPU Utilization - User,CPU Utilization - System,CPU Utilization - Idle,
    printf $fh ( "%d,%d,%d,", $v->{'user'}, $v->{'sys'}, $v->{'idle'} );

    # intr/s,ctxt/s,
    printf $fh ( "%d,%d,\n", $v->{'intrps'}, $v->{'ctxtps'} );
  } ## end foreach my $id ( sort( keys...))

  print $fh "\nCPU-Core Level Statistics\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  # Node,CPU Core,Time,Interval,Node,CPU Core,CPU Utilization - User,CPU Utilization - System,CPU Utilization - Idle,
  # node1_core00,2015-07-13 16:03:33,60,node1,0,1.597,0.000,
  #
  print $fh "Node-CPU Core,Time,Interval (s),Node,CPU Core,";
  print $fh "CPU Utilization - User,CPU Utilization - System,CPU Utilization - Idle,\n";

  foreach my $id ( sort( keys( %{ $stordata{'cpu'} } ) ) ) {
    #
    my $v = $stordata{'cpu'}->{$id};
    my ( $node, $cpu ) = split( /,/, $v->{'id'} );
    if ( $cpu eq "total" ) { next; }

    #"Node-CPU,Time,Interval (s),Node,CPU Core ID,
    printf $fh ( "%d-%d,%s,%s,%d,%d,", $node, $cpu, $v->{time}, $stordata{'interval'}, $node, $cpu );

    # CPU Utilization - User,CPU Utilization - System,CPU Utilization - Idle,
    printf $fh ( "%d,%d,%d,\n", $v->{'user'}, $v->{'sys'}, $v->{'idle'} );
  } ## end foreach my $id ( sort( keys...))

  close($fh);
} ## end sub print_perf_cpu

sub print_conf_cpg {
  open( my $fh, '>>', $stordata{'out_conf_file'} );
  #
  print $fh "\nPool Level Configuration\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #volume id:volume name:pool id/name:size:used/free
  print $fh "CPG ID,CPG name,";
  print $fh "User Total Capacity [MB],User Used Capacity [MB],";
  print $fh "Snapshot Total Capacity [MB],Snapshot Used Capacity [MB],";
  print $fh "Admin Total Capacity [MB],Admin Used Capacity [MB],\n";

  foreach my $id ( sort { $a <=> $b } ( values( %{ $storids{'cpg'} } ) ) ) {
    #
    my $cpid;
    my $v = $stordata{'cpg'}->{$id};

    # CPG ID,CPG name,
    printf $fh ( "%d,%s,", $v->{'id'}, $v->{'name'} );

    # User Total Capacity [MB],User Used Capacity [MB],
    printf $fh ( "%d,%d,", $v->{'usr_tot_cap'}, $v->{'usr_use_cap'} );

    # Snapshot Total Capacity [MB],Snapshot Used Capacity [MB],
    printf $fh ( "%d,%d,", $v->{'snp_tot_cap'}, $v->{'snp_use_cap'} );

    # Admin Total Capacity [MB],Admin Used Capacity [MB]
    printf $fh ( "%d,%d,\n", $v->{'adm_tot_cap'}, $v->{'adm_use_cap'} );
  } ## end foreach my $id ( sort { $a ...})
  close($fh);
} ## end sub print_conf_cpg

sub ld_conf {
  my ( $id, $name, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$a->[6]."\n");
  # RAID -Detailed_State- Own  SizeMB  UsedMB Use  Lgct LgId WThru MapV
  #    1 normal           0/1    3072    3072 V       0  ---     N    Y
  #    0                1   2       3       4   5     6    7     8    9
  #
  my $stat;
  if ( defined $stordata{'ld'}->{$id} ) {
    $stat = $stordata{'ld'}->{$id};
  }
  else {
    my %d;
    $stat                            = \%d;
    $stordata{'ld'}->{$id}           = $stat;
    $stordata{'ld'}->{$id}->{'name'} = $name;
    $stordata{'ld'}->{$id}->{'id'}   = $id;
  } ## end else [ if ( defined $stordata...)]
  $stordata{'ld'}->{$id}->{'raid'}       = $a->[0];
  $stordata{'ld'}->{$id}->{'status'}     = $a->[1];
  $stordata{'ld'}->{$id}->{'own'}        = $a->[2];
  $stordata{'ld'}->{$id}->{'total_size'} = $a->[3];
  $stordata{'ld'}->{$id}->{'used_size'}  = $a->[4];
  $stordata{'ld'}->{$id}->{'use'}        = $a->[5];
  $stordata{'ld'}->{$id}->{'lgct'}       = $a->[6];
  $stordata{'ld'}->{$id}->{'lgid'}       = $a->[7];
  $stordata{'ld'}->{$id}->{'wthru'}      = $a->[8];
  $stordata{'ld'}->{$id}->{'mapv'}       = $a->[9];
} ## end sub ld_conf

sub ld_conf_detail {
  my ( $id, $name, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$a->[6]."\n");
  # CPG   RAID Own  SizeMB RSizeMB RowSz StepKB SetSz Refcnt Avail CAvail ------CreationTime------ ---CreationPattern----
  # ---      1 0/1    3072    6144     3    256     2      0 mag   mag    2014-10-06 09:35:23 CEST --
  #   0      1   2       3       4     5      6     7      8   9       10                       11                 12
  #
  my $stat;
  if ( defined $stordata{'ld'}->{$id} ) {
    $stat = $stordata{'ld'}->{$id};
  }
  else {
    my %d;
    $stat                            = \%d;
    $stordata{'ld'}->{$id}           = $stat;
    $stordata{'ld'}->{$id}->{'name'} = $name;
    $stordata{'ld'}->{$id}->{'id'}   = $id;
  } ## end else [ if ( defined $stordata...)]
  $stordata{'ld'}->{$id}->{'cpgname'}  = $a->[0];
  $stordata{'ld'}->{$id}->{'raid'}     = $a->[1];
  $stordata{'ld'}->{$id}->{'own'}      = $a->[2];
  $stordata{'ld'}->{$id}->{'size'}     = $a->[3];
  $stordata{'ld'}->{$id}->{'rsize'}    = $a->[4];
  $stordata{'ld'}->{$id}->{'row_size'} = $a->[5];
  $stordata{'ld'}->{$id}->{'step'}     = $a->[6];
  $stordata{'ld'}->{$id}->{'set_size'} = $a->[7];
  $stordata{'ld'}->{$id}->{'ref_cnt'}  = $a->[8];
  $stordata{'ld'}->{$id}->{'avail'}    = $a->[9];
  $stordata{'ld'}->{$id}->{'c_avail'}  = $a->[10];
  $stordata{'ld'}->{$id}->{'cre_time'} = $a->[11];
  $stordata{'ld'}->{$id}->{'cre_patt'} = $a->[12];
} ## end sub ld_conf_detail

sub print_conf_ld {
  open( my $fh, '>>', $stordata{'out_conf_file'} );
  #
  print $fh "\nLogical Drive Level Configuration\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #volume id:volume name:pool id/name:size:used/free
  print $fh "LD ID,LD name,CPG ID,CPG name,RAID,Status,";
  print $fh "Owner,Use,Total size [MB],Used size [MB],\n";

  foreach my $id ( sort { $a <=> $b } ( values( %{ $storids{'ld'} } ) ) ) {
    #
    my $v = $stordata{'ld'}->{$id};
    my $cpgid;
    if ( $v->{'cpgname'} eq "---" ) {
      $cpgid = "-";
    }
    else {
      $cpgid = $storids{'cpg'}->{ $v->{'cpgname'} };
    }
    my $use = $v->{'use'};
    $use =~ s/,/-/;

    # LD ID,LD name,CPG ID,CPG name,RAID,Status
    printf $fh ( "%d,%s,%s,%s,", $v->{'id'}, $v->{'name'}, $cpgid, $v->{'cpgname'} );

    # RAID,Status
    printf $fh ( "%d,%s,", $v->{'raid'}, $v->{'status'} );

    # Owner,Use,Total size [MB],Used size [MB]
    printf $fh ( "%s,%s,%d,%d,\n", $v->{'own'}, $use, $v->{'total_size'}, $v->{'used_size'} );
  } ## end foreach my $id ( sort { $a ...})
  close($fh);
} ## end sub print_conf_ld

sub print_state_ld {
  open( my $fh, '>>', $stordata{'out_state_file'} );
  #
  print $fh "\nLogical Drive Level State\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  # host_id,id,name,
  # port_count,iogrp_count,status,
  # IQN WWPN,Volume IDs,Volume Names
  #
  print $fh "LD ID,LD name,State,Level,Detailed_State\n";
  foreach my $id ( sort { $a <=> $b } ( values( %{ $storids{'ld'} } ) ) ) {
    #
    my $v = $stordata{'ld'}->{$id};
    printf $fh ( "%d,%s,%s,%s,%s,\n", $v->{'id'}, $v->{'name'}, $v->{'state'}, $storstate{'ld'}->{ $v->{'state'} }, $v->{'detailed_state'} );
  }

  close($fh);
} ## end sub print_state_ld

sub parse_statld {
  my $filename = shift;
  if ( not defined $stordata{'ld'} ) {
    my %d;
    $stordata{'ld'} = \%d;
  }
  my $ids = $storids{'ld'};
  &message( "parse_statld: Process file: " . $filename );
  open( my $data, '<', $filename ) || die( "Cannot open file: " . $filename . "\n" );

  #        00:12:13 01/19/2016 r/w I/O per second KBytes per sec      Svt ms   IOSz KB
  #                    Ldname      Cur  Avg  Max  Cur  Avg  Max   Cur   Avg  Cur  Avg Qlen
  #               admin.usr.0   r    0    0    0    0    0    0  0.00  0.00  0.0  0.0    -
  #               admin.usr.0   w    1    1    1    3    3    3 14.33 14.33  5.2  5.2    -
  my $flag = 0;
  my $time;
  my $date;
  while ( my $ln = <$data> ) {
    chomp $ln;

    # 00:26:09 01/09/2016 -> 2015-07-13 16:09:33
    if ( $ln =~ /^\s*(\d{2}:\d{2}:\d{2})\s+(\d{2})\/(\d{2})\/(\d{4}).*/ ) {
      $time = $4 . "-" . $2 . "-" . $3 . " " . $1;
      next;
    }
    if ( $ln =~ /^\s*Ldname\s+Cur\s+Avg\s+Max/ ) { $flag = 1; next; }
    if ( $ln =~ /^[-]+\s*/ ) { $flag = 0; last; }
    if ( $flag && $ln =~ /^\s*([-\w\.]+)\s+([rwt]{1})\s+([-\d\.\s]*)/ ) {
      my $name   = $1;
      my $rwflag = $2;
      my $id     = $ids->{$name};
      if ( not defined $id ) {
        &message( "parse_statld: Volume: " . $name . " has not ID" );
        next;
      }
      my %d;
      $d{name} = $name;
      $d{id}   = $id;
      my @a = split( /\s+/, $3 );
      &ld_data( $id, $name, $rwflag, $time, \@a );
    } ## end if ( $flag && $ln =~ /^\s*([-\w\.]+)\s+([rwt]{1})\s+([-\d\.\s]*)/)
  } ## end while ( my $ln = <$data> )
  close $data;
} ## end sub parse_statld

sub ld_data {
  my ( $id, $name, $rwflag, $time, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$rwflag." - ".$a->[6]."\n");
  #     I/O per second    KBytes per sec    Svt ms      IOSz KB
  #     Cur  Avg  Max     Cur  Avg  Max     Cur  Avg    Cur  Avg    Qlen
  #     0    1    2       3    4    5       6    7      8    9      10
  #
  my $stat;
  if ( defined $stordata{'ld'}->{$id} ) {
    $stat = $stordata{'ld'}->{$id};
    $stordata{'ld'}->{$id}->{'time'} = $time;
  }
  else {
    my %d;
    $stat                            = \%d;
    $stordata{'vvol'}->{$id}         = $stat;
    $stordata{'ld'}->{$id}->{'name'} = $name;
    $stordata{'ld'}->{$id}->{'id'}   = $id;
    $stordata{'ld'}->{$id}->{'time'} = $time;
  } ## end else [ if ( defined $stordata...)]

  if ( $rwflag eq "r" ) {
    $stordata{'ld'}->{$id}->{'rops'}     = $a->[0];
    $stordata{'ld'}->{$id}->{'rops-avg'} = $a->[1];
    $stordata{'ld'}->{$id}->{'rops-max'} = $a->[2];
    $stordata{'ld'}->{$id}->{'rdps'}     = $a->[3];
    $stordata{'ld'}->{$id}->{'rdps-avg'} = $a->[4];
    $stordata{'ld'}->{$id}->{'rdps-max'} = $a->[5];
    $stordata{'ld'}->{$id}->{'rlpo'}     = $a->[6];
    $stordata{'ld'}->{$id}->{'rlpo-avg'} = $a->[7];
    $stordata{'ld'}->{$id}->{'rtpo'}     = $a->[8];
    $stordata{'ld'}->{$id}->{'rtpo-avg'} = $a->[9];
  } ## end if ( $rwflag eq "r" )
  elsif ( $rwflag eq "w" ) {
    $stordata{'ld'}->{$id}->{'wops'}     = $a->[0];
    $stordata{'ld'}->{$id}->{'wops-avg'} = $a->[1];
    $stordata{'ld'}->{$id}->{'wops-max'} = $a->[2];
    $stordata{'ld'}->{$id}->{'wdps'}     = $a->[3];
    $stordata{'ld'}->{$id}->{'wdps-avg'} = $a->[4];
    $stordata{'ld'}->{$id}->{'wdps-max'} = $a->[5];
    $stordata{'ld'}->{$id}->{'wlpo'}     = $a->[6];
    $stordata{'ld'}->{$id}->{'wlpo-avg'} = $a->[7];
    $stordata{'ld'}->{$id}->{'wtpo'}     = $a->[8];
    $stordata{'ld'}->{$id}->{'wtpo-avg'} = $a->[9];
  } ## end elsif ( $rwflag eq "w" )
  elsif ( $rwflag eq "t" ) {
    $stordata{'ld'}->{$id}->{'tops'}     = $a->[0];
    $stordata{'ld'}->{$id}->{'tops-avg'} = $a->[1];
    $stordata{'ld'}->{$id}->{'tops-max'} = $a->[2];
    $stordata{'ld'}->{$id}->{'tdps'}     = $a->[3];
    $stordata{'ld'}->{$id}->{'tdps-avg'} = $a->[4];
    $stordata{'ld'}->{$id}->{'tdps-max'} = $a->[5];
    $stordata{'ld'}->{$id}->{'tlpo'}     = $a->[6];
    $stordata{'ld'}->{$id}->{'tlpo-avg'} = $a->[7];
    $stordata{'ld'}->{$id}->{'ttpo'}     = $a->[8];
    $stordata{'ld'}->{$id}->{'ttpo-avg'} = $a->[9];
    $stordata{'ld'}->{$id}->{'tq'}       = $a->[10];
  } ## end elsif ( $rwflag eq "t" )
} ## end sub ld_data

sub print_perf_ld {
  open( my $fh, '>>', $stordata{'out_perf_file'} );

  #Volume Level Statistics
  #Interval Start:   2015-07-13 16:08:33
  #Interval End:     2015-07-13 16:09:33
  #Interval Length:  60 seconds
  #---------------------
  print $fh "\nLogical Drive Level Statistics\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";
  #
  printf $fh "Logical Drive ID,Time,Interval (s),";
  printf $fh "Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),";
  printf $fh "Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),";
  printf $fh ",,,,";
  printf $fh ",,,,";
  printf $fh "Logical Drive Name,CPG ID,CPG Name,RAID,,";
  printf $fh ",,,\n";

  foreach my $id ( sort { $a <=> $b } ( values( %{ $storids{ld} } ) ) ) {
    #
    my $v = $stordata{'ld'}->{$id};
    if ( not defined $v->{'rops'} ) {
      &message( "print_perf_ld: Logical Drive ID: " . $id . " has not perf data" );
      next;
    }
    my $cpgid;
    if ( $v->{'cpgname'} eq "---" ) {
      $cpgid = "-";
    }
    else {
      $cpgid = $storids{'cpg'}->{ $v->{'cpgname'} };
    }

    #"Volume ID,Time,Interval (s),
    printf $fh ( "%d,%s,%s,", $v->{'id'}, $v->{time}, $stordata{'interval'}, );

    #"Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s)"
    printf $fh ( "%d,%d,%d,%d,%d,%d,", $v->{'rops'}, $v->{'wops'}, $v->{'tops'}, $v->{'rdps'}, $v->{'wdps'}, $v->{'tdps'} );

    #"Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms)"
    printf $fh ( "%.2f,%.2f,%.2f,%.2f,%.2f,%.2f,", $v->{'rtpo'}, $v->{'wtpo'}, $v->{'ttpo'}, $v->{'rlpo'}, $v->{'wlpo'}, $v->{'tlpo'} );
    #
    printf $fh (",,,,");

    #;
    printf $fh (",,,,");

    # Logical Drive Name,CPG ID,CPG Name,RAID,,;
    printf $fh ( "%s,%s,%s,%d,,", $v->{'name'}, $cpgid, $v->{'cpgname'}, $v->{'raid'} );
    #
    printf $fh (",,,\n");
  } ## end foreach my $id ( sort { $a ...})

  close($fh);
} ## end sub print_perf_ld

sub port_conf {
  my ( $id, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$a->[6]."\n");
  #      Mode     State ----Node_WWN---- -Port_WWN/HW_Addr-  Type Protocol Label Partner FailoverState
  # initiator     ready 50002ACFF70046A8   50002AC0010046A8  disk      SAS  DP-1       -             -
  #         0         1                2                  3     4        5     6       7             8
  #
  my $stat;
  if ( defined $stordata{'port'}->{$id} ) {
    $stat = $stordata{'port'}->{$id};
  }
  else {
    my %d;
    $stat                            = \%d;
    $stordata{'port'}->{$id}         = $stat;
    $stordata{'port'}->{$id}->{'id'} = $id;
  } ## end else [ if ( defined $stordata...)]
  $stordata{'port'}->{$id}->{'mode'}          = $a->[0];
  $stordata{'port'}->{$id}->{'state'}         = $a->[1];
  $stordata{'port'}->{$id}->{'wwnn'}          = $a->[2];
  $stordata{'port'}->{$id}->{'wwpn'}          = $a->[3];
  $stordata{'port'}->{$id}->{'type'}          = $a->[4];
  $stordata{'port'}->{$id}->{'protocol'}      = $a->[5];
  $stordata{'port'}->{$id}->{'label'}         = $a->[6];
  $stordata{'port'}->{$id}->{'partner'}       = $a->[7];
  $stordata{'port'}->{$id}->{'failoverstate'} = $a->[8];
} ## end sub port_conf

sub print_conf_port {
  open( my $fh, '>>', $stordata{'out_conf_file'} );
  #
  print $fh "\nPort Level Configuration\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  # N:S:P      Mode     State ----Node_WWN---- -Port_WWN/HW_Addr-  Type Protocol Label Partner FailoverState
  # 0:1:1    target     ready 2FFFFFFFFFFFFFFF   2FFFFFFFFFFFFFF8  host       FC     -   1:1:1          none
  print $fh "N:S:P,Mode,State,Node WWN,Port WWN,";
  print $fh "Type,Protocol,Label,Partner,Failover State,\n";

  foreach my $id ( sort( values( %{ $storids{'port'} } ) ) ) {
    #
    my $v = $stordata{'port'}->{$id};

    # N:S:P,Mode,State,Node WWN,PortWWN,
    printf $fh ( "%s,%s,%s,%s,%s,", $v->{'id'}, $v->{'mode'}, $v->{'state'}, $v->{'wwnn'}, $v->{'wwpn'} );

    # Type,Protocol,Label,Partner,Failover State
    printf $fh ( "%s,%s,%s,%s,%s,\n", $v->{'type'}, $v->{'protocol'}, $v->{'label'}, $v->{'partner'}, $v->{'failoverstate'} );
  } ## end foreach my $id ( sort( values...))
  close($fh);
} ## end sub print_conf_port

sub print_state_port {
  open( my $fh, '>>', $stordata{'out_state_file'} );
  my @ports = ();
  #
  print $fh "\nPort Level State\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #
  print $fh "N:S:P,Type,State,Level,\n";

  foreach my $id ( sort( values( %{ $storids{'port'} } ) ) ) {
    #
    my $v = $stordata{'port'}->{$id};
    printf $fh ( "%s,%s,%s,%s,\n", $v->{'id'}, $v->{'type'}, $v->{'state'}, $storstate{'port'}->{ $v->{'state'} } );
    #$table_tr .= "<tr><td>$v->{'id'}</td><td>$v->{'type'}</td><td>$v->{'state'}</td><td>$storstate{'port'}->{ $v->{'state'} }</td></tr>\n";

    my %data;
    $data{nsp}        = $v->{'id'};
    $data{type}       = $v->{'type'};
    $data{state}      = $v->{'state'};
    $data{failover}   = $storstate{'port'}->{ $v->{'state'} };
    push(@ports,\%data);
  }
  $conf{$ENV{'HP3PAR_HOSTCFG_ID'} }{'epar_ports'} = encode_json(\@ports);
  close($fh);
}


sub port_data {
  my ( $id, $rwflag, $time, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$rwflag." - ".$a->[6]."\n");
  #  I/O per second KBytes per sec      Svt ms   IOSz KB
  #  Cur  Avg  Max  Cur  Avg  Max   Cur   Avg  Cur  Avg Qlen
  #    7    7    7  153  153  153  4.21  4.21 20.7 20.7    -
  #    0    1    2    3    4    5     6     7    8    9    10
  #
  my $stat;
  if ( defined $stordata{'port'}->{$id} ) {
    $stat = $stordata{'port'}->{$id};
    $stordata{'port'}->{$id}->{'time'} = $time;
  }
  else {
    my %d;
    $stat                              = \%d;
    $stordata{'vvol'}->{$id}           = $stat;
    $stordata{'port'}->{$id}->{'id'}   = $id;
    $stordata{'port'}->{$id}->{'time'} = $time;
  } ## end else [ if ( defined $stordata...)]
  if ( $rwflag eq "r" ) {
    $stordata{'port'}->{$id}->{'rops'}     = $a->[0];
    $stordata{'port'}->{$id}->{'rops-avg'} = $a->[1];
    $stordata{'port'}->{$id}->{'rops-max'} = $a->[2];
    $stordata{'port'}->{$id}->{'rdps'}     = $a->[3];
    $stordata{'port'}->{$id}->{'rdps-avg'} = $a->[4];
    $stordata{'port'}->{$id}->{'rdps-max'} = $a->[5];
    $stordata{'port'}->{$id}->{'rlpo'}     = $a->[6];
    $stordata{'port'}->{$id}->{'rlpo-avg'} = $a->[7];
    $stordata{'port'}->{$id}->{'rtpo'}     = $a->[8];
    $stordata{'port'}->{$id}->{'rtpo-avg'} = $a->[9];
  } ## end if ( $rwflag eq "r" )
  elsif ( $rwflag eq "w" ) {
    $stordata{'port'}->{$id}->{'wops'}     = $a->[0];
    $stordata{'port'}->{$id}->{'wops-avg'} = $a->[1];
    $stordata{'port'}->{$id}->{'wops-max'} = $a->[2];
    $stordata{'port'}->{$id}->{'wdps'}     = $a->[3];
    $stordata{'port'}->{$id}->{'wdps-avg'} = $a->[4];
    $stordata{'port'}->{$id}->{'wdps-max'} = $a->[5];
    $stordata{'port'}->{$id}->{'wlpo'}     = $a->[6];
    $stordata{'port'}->{$id}->{'wlpo-avg'} = $a->[7];
    $stordata{'port'}->{$id}->{'wtpo'}     = $a->[8];
    $stordata{'port'}->{$id}->{'wtpo-avg'} = $a->[9];
  } ## end elsif ( $rwflag eq "w" )
  elsif ( $rwflag eq "t" ) {
    $stordata{'port'}->{$id}->{'tops'}     = $a->[0];
    $stordata{'port'}->{$id}->{'tops-avg'} = $a->[1];
    $stordata{'port'}->{$id}->{'tops-max'} = $a->[2];
    $stordata{'port'}->{$id}->{'tdps'}     = $a->[3];
    $stordata{'port'}->{$id}->{'tdps-avg'} = $a->[4];
    $stordata{'port'}->{$id}->{'tdps-max'} = $a->[5];
    $stordata{'port'}->{$id}->{'tlpo'}     = $a->[6];
    $stordata{'port'}->{$id}->{'tlpo-avg'} = $a->[7];
    $stordata{'port'}->{$id}->{'ttpo'}     = $a->[8];
    $stordata{'port'}->{$id}->{'ttpo-avg'} = $a->[9];
    $stordata{'port'}->{$id}->{'tq'}       = $a->[10];
  } ## end elsif ( $rwflag eq "t" )

} ## end sub port_data

sub print_perf_port {
  open( my $fh, '>>', $stordata{'out_perf_file'} );

  #Volume Level Statistics
  #Interval Start:   2015-07-13 16:08:33
  #Interval End:     2015-07-13 16:09:33
  #Interval Length:  60 seconds
  #---------------------
  print $fh "\nPort Level Statistics\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";
  #
  printf $fh "N:S:P,Time,Interval (s),";
  printf $fh "Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),";
  printf $fh "Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),";
  printf $fh ",,,,";
  printf $fh ",,,,";
  printf $fh "Type,Protocol,Label,,";
  printf $fh ",,,\n";

  foreach my $id ( sort( values( %{ $storids{'port'} } ) ) ) {
    #
    my $v = $stordata{'port'}->{$id};
    if ( not defined $v->{'rops'} ) {
      &message( "print_perf_port: Port ID: " . $id . " has not perf data" );
      next;
    }
    if ( not defined $v->{time} ) { next; }

    #"Volume ID,Time,Interval (s),
    printf $fh ( "%s,%s,%s,", $v->{'id'}, $v->{time}, $stordata{'interval'}, );

    #"Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s)"
    printf $fh ( "%d,%d,%d,%d,%d,%d,", $v->{'rops'}, $v->{'wops'}, $v->{'tops'}, $v->{'rdps'}, $v->{'wdps'}, $v->{'tdps'} );

    #"Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms)"
    printf $fh ( "%.2f,%.2f,%.2f,%.2f,%.2f,%.2f,", $v->{'rtpo'}, $v->{'wtpo'}, $v->{'ttpo'}, $v->{'rlpo'}, $v->{'wlpo'}, $v->{'tlpo'} );
    #
    printf $fh (",,,,");

    #;
    printf $fh (",,,,");

    # Logical Drive Name,CPG ID,CPG Name,RAID,,;
    printf $fh ( "%s,%s,%s,,,", $v->{'type'}, $v->{'protocol'}, $v->{'label'} );
    #
    printf $fh (",,,\n");
  } ## end foreach my $id ( sort( values...))

  close($fh);
} ## end sub print_perf_port

sub parse_statport {
  my $filename = shift;
  if ( not defined $stordata{'port'} ) {
    my %d;
    $stordata{'port'} = \%d;
  }
  my $ids = $storids{'port'};
  &message( "parse_statport: Process file: " . $filename );
  open( my $data, '<', $filename ) || die( "Cannot open file: " . $filename . "\n" );

  # 23:36:14 01/18/2016 r/w I/O per second KBytes per sec      Svt ms   IOSz KB
  #      Port       D/C      Cur  Avg  Max  Cur  Avg  Max   Cur   Avg  Cur  Avg Qlen
  #     0:0:1      Data   r    7    7    7  153  153  153  4.21  4.21 20.7 20.7    -
  my $flag = 0;
  my $time;
  while ( my $ln = <$data> ) {
    chomp $ln;

    #print("__DBG__: [$flag]".$ln."\n");
    # 00:26:09 01/09/2016 -> 2015-07-13 16:09:33
    if ( $ln =~ /^\s*(\d{2}:\d{2}:\d{2})\s+(\d{2})\/(\d{2})\/(\d{4}).*/ ) {
      $time = $4 . "-" . $2 . "-" . $3 . " " . $1;
      next;
    }
    if ( $ln =~ /^\s*Port\s+D\/C\s+Cur/ ) { $flag = 1; next; }
    if ( $ln =~ /^[-]+\s*/ ) { $flag = 0; last; }
    if ( $flag && $ln =~ /^\s*([:\d]+)\s+\w+\s+([rwt]{1})\s+(.*)/ ) {
      my $id     = $1;
      my $rwflag = $2;
      my %d;
      $d{id} = $id;
      my @a = split( /\s+/, $3 );
      &port_data( $id, $rwflag, $time, \@a );
    } ## end if ( $flag && $ln =~ /^\s*([:\d]+)\s+\w+\s+([rwt]{1})\s+(.*)/)
  } ## end while ( my $ln = <$data> )
  close $data;
} ## end sub parse_statport

sub vlun_conf {
  my ( $id, $host, $vvol ) = @_;

  #print("__DBG_8:  Host ID: ".$id." - Host Name: ".$host." - VVol Name: ".$vvol."\n");
  #
  my $stat;
  if ( defined $stordata{'host'}->{$id} ) {
    $stat = $stordata{'host'}->{$id};
  }
  else {
    my %d;
    $stat                              = \%d;
    $stordata{'host'}->{$id}           = $stat;
    $stordata{'host'}->{$id}->{'name'} = $host;
    $stordata{'host'}->{$id}->{'id'}   = $id;
  } ## end else [ if ( defined $stordata...)]

  # VVol IDs
  my $vvid = $storids{'vvol'}->{$vvol};
  if ( not defined $vvid ) {
    &warning( "Cannot found VVol ID for VVol Name: " . $vvol . " - LUN ID: " . $id . " Host Name: " . $host ) if ($debug);
    return;
  }
  if ( defined $stordata{'host'}->{$id}->{'vvol_ids'} ) {
    my @d = split( / /, $stordata{'host'}->{$id}->{'vvol_ids'} );
    my $flag = 1;
    for my $ent (@d) {
      if ( $ent eq $vvid ) { $flag = 0; last; }
    }
    if ($flag) {
      push( @d, $vvid );
      $stordata{'host'}->{$id}->{'vvol_ids'} = join( " ", @d );
    }
  } ## end if ( defined $stordata...)
  else {
    $stordata{'host'}->{$id}->{'vvol_ids'} = $vvid;
  }

  # VVol Names
  if ( defined $stordata{'host'}->{$id}->{'vvol_names'} ) {
    my @d = split( / /, $stordata{'host'}->{$id}->{'vvol_names'} );
    my $flag = 1;
    for my $ent (@d) {
      if ( $ent eq $vvol ) { $flag = 0; last; }
    }
    if ($flag) {
      push( @d, $vvol );
      $stordata{'host'}->{$id}->{'vvol_names'} = join( " ", @d );
    }
  } ## end if ( defined $stordata...)
  else {
    $stordata{'host'}->{$id}->{'vvol_names'} = $vvol;
  }
} ## end sub vlun_conf

sub host_conf {
  my ( $id, $name, $a ) = @_;

  # Persona       ---------WWN/iSCSI_Name--------- Port  IP_addr
  # 0             1                                2     3
  #
  my $stat;
  if ( defined $stordata{'host'}->{$id} ) {
    $stat = $stordata{'host'}->{$id};
  }
  else {
    my %d;
    $stat                              = \%d;
    $stordata{'host'}->{$id}           = $stat;
    $stordata{'host'}->{$id}->{'name'} = $name;
    $stordata{'host'}->{$id}->{'id'}   = $id;
  } ## end else [ if ( defined $stordata...)]
  $stordata{'host'}->{$id}->{'persona'} = $a->[0];

  # WWN/iSCSI
  if ( defined $stordata{'host'}->{$id}->{'wwn_iscsi'} ) {
    my @d = split( / /, $stordata{'host'}->{$id}->{'wwn_iscsi'} );
    my $flag = 1;
    for my $ent (@d) {
      if ( $ent eq $a->[1] ) { $flag = 0; last; }
    }
    if ($flag) {
      push( @d, $a->[1] );
      $stordata{'host'}->{$id}->{'wwn_iscsi'} = join( " ", @d );
    }
  } ## end if ( defined $stordata...)
  else {
    $stordata{'host'}->{$id}->{'wwn_iscsi'} = $a->[1];
  }

  # Port
  if ( defined $stordata{'host'}->{$id}->{'port'} ) {
    my @d = split( / /, $stordata{'host'}->{$id}->{'port'} );
    my $flag = 1;
    for my $ent (@d) {
      if ( $ent eq $a->[2] ) { $flag = 0; last; }
    }
    if ($flag) {
      push( @d, $a->[2] );
      $stordata{'host'}->{$id}->{'port'} = join( " ", @d );
    }
  } ## end if ( defined $stordata...)
  else {
    $stordata{'host'}->{$id}->{'port'} = $a->[2];
  }
  $stordata{'host'}->{$id}->{'ip_addr'} = $a->[3];
} ## end sub host_conf

sub print_conf_host {
  open( my $fh, '>>', $stordata{'out_conf_file'} );
  #
  print $fh "\nHost Level Configuration\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  # host_id,id,name,
  # port_count,iogrp_count,status,
  # IQN WWPN,Volume IDs,Volume Names
  #
  print $fh "host_id,id,name,";
  print $fh "port_count,,status,";
  print $fh "IQN WWPN,Volume IDs,Volume Names,\n";

  foreach my $id ( sort { $a <=> $b } ( values( %{ $storids{'host'} } ) ) ) {
    #
    my $cpid;
    my $v = $stordata{'host'}->{$id};
    my @port = split( / /, $v->{'port'} );

    # host_id,id,name,
    printf $fh ( "%06d,%d,%s,", $v->{'id'}, $v->{'id'}, $v->{'name'} );

    # port_count,,status,
    printf $fh ( "%s,,,", scalar(@port) );

    # IQN WWPN,Volume IDs,Volume Names,
    if ( defined $v->{'vvol_names'} ) {
      printf $fh ( "%s,%s,%s,\n", $v->{'wwn_iscsi'}, $v->{'vvol_ids'}, $v->{'vvol_names'} );
    }
    else {
      printf $fh ( "%s,,,\n", $v->{'wwn_iscsi'} );
    }

  } ## end foreach my $id ( sort { $a ...})
  close($fh);
} ## end sub print_conf_host

sub print_perf_host {
  open( my $fh, '>>', $stordata{'out_perf_file'} );

  #Volume Level Statistics
  #Interval Start:   2015-07-13 16:08:33
  #Interval End:     2015-07-13 16:09:33
  #Interval Length:  60 seconds
  #---------------------
  print $fh "\nHost Level Statistics\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";
  #
  printf $fh "host_id,Time,Interval (s),";
  printf $fh "Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),";
  printf $fh "Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),";
  printf $fh ",,,,";
  printf $fh ",,,,";
  printf $fh "Host Name,Volume (Vdisk) Name,,,,";
  printf $fh ",,,\n";

  foreach my $id ( sort( keys( %{ $stordata{'vlun'} } ) ) ) {
    #
    my $v = $stordata{'vlun'}->{$id};

    #if( not defined $v->{'tops'} || not defined $v->{'rops'} || not defined $v->{'wops'}) {
    if ( not defined $v->{'rops'} ) {
      &message( "print_perf_host: Physical Disk ID: " . $id . " has not perf data" );
      next;
    }

    #"Volume ID,Time,Interval (s),
    printf $fh ( "%s,%s,%s,", $v->{'id'}, $v->{time}, $stordata{'interval'}, );

    #"Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s)"
    printf $fh ( "%d,%d,%d,%d,%d,%d,", $v->{'rops'}, $v->{'wops'}, $v->{'tops'}, $v->{'rdps'}, $v->{'wdps'}, $v->{'tdps'} );

    #"Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms)"
    printf $fh ( "%.2f,%.2f,%.2f,%.2f,%.2f,%.2f,", $v->{'rtpo'}, $v->{'wtpo'}, $v->{'ttpo'}, $v->{'rlpo'}, $v->{'wlpo'}, $v->{'tlpo'} );
    #
    printf $fh (",,,,");

    #;
    printf $fh (",,,,");

    # Cage Position,Type,RPM,;
    printf $fh ( "%s,%s,%s,%s,,", $v->{'host'}, $v->{'name'}, "", "" );
    #
    printf $fh (",,,\n");
    &message( "VLUN: " . $v->{'id'} . " Time - server: " . $stordata{'server_time'} . " - storage:" . $v->{'host'} );
  } ## end foreach my $id ( sort( keys...))

  close($fh);
} ## end sub print_perf_host

sub pd_conf {
  my ( $id, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$a->[6]."\n");
  #                         ----Size(MB)----- ----Ports----
  # CagePos Type RPM State     Total     Free A      B      Capacity(GB)
  # 0:0:0   FC    15 normal   278528   106496 1:0:1* 0:0:1           300
  #
  my $stat;
  if ( defined $stordata{'pd'}->{$id} ) {
    $stat = $stordata{'pd'}->{$id};
  }
  else {
    my %d;
    $stat                          = \%d;
    $stordata{'pd'}->{$id}         = $stat;
    $stordata{'pd'}->{$id}->{'id'} = $id;
  } ## end else [ if ( defined $stordata...)]
  $stordata{'pd'}->{$id}->{'cage_pos'}      = $a->[0];
  $stordata{'pd'}->{$id}->{'type'}          = $a->[1];
  $stordata{'pd'}->{$id}->{'rpm'}           = $a->[2];
  $stordata{'pd'}->{$id}->{'state'}         = $a->[3];
  $stordata{'pd'}->{$id}->{'total_size_mb'} = $a->[4];
  $stordata{'pd'}->{$id}->{'free_size_mb'}  = $a->[5];
  $stordata{'pd'}->{$id}->{'port_A'}        = $a->[6];
  $stordata{'pd'}->{$id}->{'port_B'}        = $a->[7];
  $stordata{'pd'}->{$id}->{'capacity_GB'}   = $a->[8];
} ## end sub pd_conf

sub print_conf_pd {
  open( my $fh, '>>', $stordata{'out_conf_file'} );
  #
  print $fh "\nPhysical Disk Configuration\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #volume id:volume name:pool id/name:size:used/free
  print $fh "Physical Disk ID,Cage Position,Type,RPM,State,";
  print $fh "Total Size [MB],Free Size [MB],Port A,Port B,Capacity [GB],\n";

  foreach my $id ( sort { $a <=> $b } ( keys( %{ $stordata{'pd'} } ) ) ) {
    #
    my $v = $stordata{'pd'}->{$id};

    # Physical Disk ID,Cage Position,Type,RPM,State,
    printf $fh ( "%d,%s,%s,%s,%s,", $v->{'id'}, $v->{'cage_pos'}, $v->{'type'}, $v->{'rpm'}, $v->{'state'} );

    # Total Size [MB],Free Size [MB],Port A,Port B,Capacity [GB],
    printf $fh ( "%d,%d,%s,%s,%d,\n", $v->{'total_size_mb'}, $v->{'free_size_mb'}, $v->{'port_A'}, $v->{'port_B'}, $v->{'capacity_GB'} );
  } ## end foreach my $id ( sort { $a ...})
  close($fh);
} ## end sub print_conf_pd

sub print_state_pd {
  open( my $fh, '>>', $stordata{'out_state_file'} );
  my @disks = ();
  print $fh "\nPhysical Disk State\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  print $fh "Physical Disk ID,Cage Position,State,Level\n";
  foreach my $id ( sort { $a <=> $b } ( keys( %{ $stordata{'pd'} } ) ) ) {
    #
    my $v = $stordata{'pd'}->{$id};
    #$table_tr .= "<tr><td>$v->{'id'}</td><td>$v->{'cage_pos'}</td><td>$v->{'state'}</td><td>$storstate{'pd'}->{ $v->{'state'} }</td></tr>\n";
    printf $fh ( "%d,%s,%s,%s,\n", $v->{'id'}, $v->{'cage_pos'}, $v->{'state'}, $storstate{'pd'}->{ $v->{'state'} } );
    my %data;
    $data{id}        = $v->{'id'};
    $data{position}  = $v->{'cage_pos'};
    $data{state}     = $v->{'state'};
    push(@disks,\%data);
  }

  $conf{$ENV{'HP3PAR_HOSTCFG_ID'} }{'epar_disks'} = encode_json(\@disks);

  close($fh);
}


sub parse_statpd {
  my $filename = shift;
  if ( not defined $stordata{'pd'} ) {
    my %d;
    $stordata{'pd'} = \%d;
  }
  &message( "parse_statpd: Process file: " . $filename );
  open( my $data, '<', $filename ) || die( "Cannot open file: " . $filename . "\n" );

  #   14:47:47 04/04/2016 r/w I/O per second KBytes per sec      Svt ms   IOSz KB     Util%
  #       Pdid      Port      Cur  Avg  Max  Cur  Avg  Max   Cur   Avg   Cur   Avg Qlen  Cur   Avg
  #          0     0:0:0   t  10   10   50   50   10   90  0.00  0.00   0.0   0.0    0   30    10
  #          1     0:1:0   t  20   10   60   50   20   90  0.00  0.00   0.0   0.0    0   40    20
  #          2     0:2:0   t  30   20   50   60   30   70  0.00  0.00   0.0   0.0    0   10    20
  # 14:47:47 04/04/2016 r/w I/O per second KBytes per sec    Svt ms   IOSz KB       Idle %
  #       ID        Port      Cur  Avg  Max  Cur  Avg  Max  Cur  Avg  Cur  Avg Qlen Cur Avg
  #        0       0:1:1   r    0    0    0    0    0    0 0.00 0.00  0.0  0.0    -   -   -
  #        0       0:1:1   w    0    0    0    0    0    0 0.00 0.00  0.0  0.0    -   -   -
  #        0       0:1:1   t    0    0    0    0    0    0 0.00 0.00  0.0  0.0    0 100 100
  #        1       1:1:1   r    0    0    0    0    0    0 0.00 0.00  0.0  0.0    -   -   -
  #        1       1:1:1   w    0    0    0    0    0    0 0.00 0.00  0.0  0.0    -   -   -
  #        1       1:1:1   t    0    0    0    0    0    0 0.00 0.00  0.0  0.0    0 100 100
  
  # 08:43:52 05/28/2019 r/w  I/O per second   KBytes per sec   Svt ms     IOSz KB        Idle %
  #       ID        Port      Cur  Avg  Max   Cur   Avg   Max   Cur  Avg   Cur  Avg Qlen Cur Avg
  #       55       0:0:2   t 1324  922 1324 29258 15522 29258  1.87 0.92  22.1 16.8    1  87  91
  #       55       1:0:2   t   45   56  109   546   756  1742  0.26 0.27  12.2 13.6    0  99  99
  #       56       0:0:1   t 1244  908 1244 26239 15064 26239  0.78 0.48  21.1 16.6    0  91  94
  #       56       1:0:1   t   33   48  112   421   673  1835  0.13 0.14  12.8 14.2    0 100  99
  
  my $flag = 0;
  my $time;
  my $date;
  while ( my $ln = <$data> ) {
    chomp $ln;

    #
    if ( $ln =~ /^\s*(\d{2}:\d{2}:\d{2})\s+(\d{2})\/(\d{2})\/(\d{4}).*/ ) {
      $time = $4 . "-" . $2 . "-" . $3 . " " . $1;
      next;
    }
    if ( $ln =~ /^\s*ID\s+Port\s+Cur\s+Avg\s+Max/ ) { $flag = 1; next; }
    if ( $ln =~ /^[-]+\s*/ ) { $flag = 0; last; }
    if ( $flag && $ln =~ /^\s*(\d+)\s+(\d+:\d+:\d+)\s+([rwt]{1})\s+([-\d\.\s]*)/ ) {
      my $id     = $1;
      my $port   = $2;
      my $rwflag = $3;
      my %d;
      $d{id} = $id;
      my @a = split( /\s+/, $4 );
      &pd_data( $id, $port, $rwflag, $time, \@a );
    } ## end if ( $flag && $ln =~ /^\s*(\d+)\s+(\d+:\d+:\d+)\s+([rwt]{1})\s+([-\d\.\s]*)/)
  } ## end while ( my $ln = <$data> )
  close $data;
} ## end sub parse_statpd

sub pd_data {
  my ( $id, $port, $rwflag, $time, $a ) = @_;

  #print("__DBG_2: VVol ID: ".$id." - Name: ".$name." - ".$rwflag." - ".$a->[6]."\n");
  #     I/O per second    KBytes per sec    Svt ms      IOSz KB           Idle%
  #     Cur  Avg  Max     Cur  Avg  Max     Cur  Avg    Cur  Avg    Qlen  Cur   Avg
  #     0    1    2       3    4    5       6    7      8    9      10     11    12
  #
  my $stat;
  if ( defined $stordata{'pd'}->{$id} ) {
    $stat = $stordata{'pd'}->{$id};
    $stordata{'pd'}->{$id}->{'time'} = $time;
  }
  else {
    my %d;
    $stat                                = \%d;
    $stordata{'pd'}->{$id}               = $stat;
    $stordata{'pd'}->{$id}->{'cage_pos'} = $port;
    $stordata{'pd'}->{$id}->{'id'}       = $id;
    $stordata{'pd'}->{$id}->{'time'}     = $time;
  } ## end else [ if ( defined $stordata...)]

  if ( $rwflag eq "r" ) {
    if ( defined $stordata{'pd'}->{$id}->{'rops'} ) {
      $stordata{'pd'}->{$id}->{'rops'}     = $stordata{'pd'}->{$id}->{'rops'} + $a->[0];
      $stordata{'pd'}->{$id}->{'rops-avg'} = $stordata{'pd'}->{$id}->{'rops-avg'} + $a->[1];
      $stordata{'pd'}->{$id}->{'rops-max'} = $stordata{'pd'}->{$id}->{'rops-max'} + $a->[2];
      $stordata{'pd'}->{$id}->{'rdps'}     = $stordata{'pd'}->{$id}->{'rdps'} + $a->[3];
      $stordata{'pd'}->{$id}->{'rdps-avg'} = $stordata{'pd'}->{$id}->{'rdps-avg'} + $a->[4];
      $stordata{'pd'}->{$id}->{'rdps-max'} = $stordata{'pd'}->{$id}->{'rdps-max'} + $a->[5];
      $stordata{'pd'}->{$id}->{'rlpo'}     = ( $stordata{'pd'}->{$id}->{'rlpo'} + $a->[6] ) / 2;
      $stordata{'pd'}->{$id}->{'rlpo-avg'} = ( $stordata{'pd'}->{$id}->{'rlpo-avg'} + $a->[7] ) / 2;
      $stordata{'pd'}->{$id}->{'rtpo'}     = ( $stordata{'pd'}->{$id}->{'rtpo'} + $a->[8] ) / 2;
      $stordata{'pd'}->{$id}->{'rtpo-avg'} = ( $stordata{'pd'}->{$id}->{'rtpo-avg'} + $a->[9] ) / 2;
    } else {
      $stordata{'pd'}->{$id}->{'rops'}     = $a->[0];
      $stordata{'pd'}->{$id}->{'rops-avg'} = $a->[1];
      $stordata{'pd'}->{$id}->{'rops-max'} = $a->[2];
      $stordata{'pd'}->{$id}->{'rdps'}     = $a->[3];
      $stordata{'pd'}->{$id}->{'rdps-avg'} = $a->[4];
      $stordata{'pd'}->{$id}->{'rdps-max'} = $a->[5];
      $stordata{'pd'}->{$id}->{'rlpo'}     = $a->[6];
      $stordata{'pd'}->{$id}->{'rlpo-avg'} = $a->[7];
      $stordata{'pd'}->{$id}->{'rtpo'}     = $a->[8];
      $stordata{'pd'}->{$id}->{'rtpo-avg'} = $a->[9];
    }
  } ## end if ( $rwflag eq "r" )
  elsif ( $rwflag eq "w" ) {
    if ( defined $stordata{'pd'}->{$id}->{'wops'} ) {
      $stordata{'pd'}->{$id}->{'wops'}     = $stordata{'pd'}->{$id}->{'wops'} + $a->[0];
      $stordata{'pd'}->{$id}->{'wops-avg'} = $stordata{'pd'}->{$id}->{'wops-avg'} + $a->[1];
      $stordata{'pd'}->{$id}->{'wops-max'} = $stordata{'pd'}->{$id}->{'wops-max'} + $a->[2];
      $stordata{'pd'}->{$id}->{'wdps'}     = $stordata{'pd'}->{$id}->{'wdps'} + $a->[3];
      $stordata{'pd'}->{$id}->{'wdps-avg'} = $stordata{'pd'}->{$id}->{'wdps-avg'} + $a->[4];
      $stordata{'pd'}->{$id}->{'wdps-max'} = $stordata{'pd'}->{$id}->{'wdps-max'} + $a->[5];
      $stordata{'pd'}->{$id}->{'wlpo'}     = ( $stordata{'pd'}->{$id}->{'wlpo'} + $a->[6] ) / 2;
      $stordata{'pd'}->{$id}->{'wlpo-avg'} = ( $stordata{'pd'}->{$id}->{'wlpo-avg'} + $a->[7] ) / 2;
      $stordata{'pd'}->{$id}->{'wtpo'}     = ( $stordata{'pd'}->{$id}->{'wtpo'} + $a->[8] ) / 2;
      $stordata{'pd'}->{$id}->{'wtpo-avg'} = ( $stordata{'pd'}->{$id}->{'wtpo-avg'} + $a->[9] ) / 2;
    } else {
      $stordata{'pd'}->{$id}->{'wops'}     = $a->[0];
      $stordata{'pd'}->{$id}->{'wops-avg'} = $a->[1];
      $stordata{'pd'}->{$id}->{'wops-max'} = $a->[2];
      $stordata{'pd'}->{$id}->{'wdps'}     = $a->[3];
      $stordata{'pd'}->{$id}->{'wdps-avg'} = $a->[4];
      $stordata{'pd'}->{$id}->{'wdps-max'} = $a->[5];
      $stordata{'pd'}->{$id}->{'wlpo'}     = $a->[6];
      $stordata{'pd'}->{$id}->{'wlpo-avg'} = $a->[7];
      $stordata{'pd'}->{$id}->{'wtpo'}     = $a->[8];
      $stordata{'pd'}->{$id}->{'wtpo-avg'} = $a->[9];
    }
  } ## end elsif ( $rwflag eq "w" )
  elsif ( $rwflag eq "t" ) {
    if ( defined $stordata{'pd'}->{$id}->{'tops'} ) {
      $stordata{'pd'}->{$id}->{'tops'}     = $stordata{'pd'}->{$id}->{'tops'} + $a->[0];
      $stordata{'pd'}->{$id}->{'tops-avg'} = $stordata{'pd'}->{$id}->{'tops-avg'} + $a->[1];
      $stordata{'pd'}->{$id}->{'tops-max'} = $stordata{'pd'}->{$id}->{'tops-max'} + $a->[2];
      $stordata{'pd'}->{$id}->{'tdps'}     = $stordata{'pd'}->{$id}->{'tdps'} + $a->[3];
      $stordata{'pd'}->{$id}->{'tdps-avg'} = $stordata{'pd'}->{$id}->{'tdps-avg'} + $a->[4];
      $stordata{'pd'}->{$id}->{'tdps-max'} = $stordata{'pd'}->{$id}->{'tdps-max'} + $a->[5];
      $stordata{'pd'}->{$id}->{'tlpo'}     = ( $stordata{'pd'}->{$id}->{'tlpo'} + $a->[6] ) / 2;
      $stordata{'pd'}->{$id}->{'tlpo-avg'} = ( $stordata{'pd'}->{$id}->{'tlpo-avg'} + $a->[7] ) / 2;
      $stordata{'pd'}->{$id}->{'ttpo'}     = ( $stordata{'pd'}->{$id}->{'ttpo'} + $a->[8] ) / 2;
      $stordata{'pd'}->{$id}->{'ttpo-avg'} = ( $stordata{'pd'}->{$id}->{'ttpo-avg'} + $a->[9] ) / 2;
      $stordata{'pd'}->{$id}->{'tq'}       = $stordata{'pd'}->{$id}->{'tq'} + $a->[10];
      $stordata{'pd'}->{$id}->{'tidl'}     = ( $stordata{'pd'}->{$id}->{'tidl'} + $a->[11] ) / 2;
      $stordata{'pd'}->{$id}->{'tidl-avg'} = ( $stordata{'pd'}->{$id}->{'tidl-avg'} + $a->[12] ) / 2;
    } else {
      $stordata{'pd'}->{$id}->{'tops'}     = $a->[0];
      $stordata{'pd'}->{$id}->{'tops-avg'} = $a->[1];
      $stordata{'pd'}->{$id}->{'tops-max'} = $a->[2];
      $stordata{'pd'}->{$id}->{'tdps'}     = $a->[3];
      $stordata{'pd'}->{$id}->{'tdps-avg'} = $a->[4];
      $stordata{'pd'}->{$id}->{'tdps-max'} = $a->[5];
      $stordata{'pd'}->{$id}->{'tlpo'}     = $a->[6];
      $stordata{'pd'}->{$id}->{'tlpo-avg'} = $a->[7];
      $stordata{'pd'}->{$id}->{'ttpo'}     = $a->[8];
      $stordata{'pd'}->{$id}->{'ttpo-avg'} = $a->[9];
      $stordata{'pd'}->{$id}->{'tq'}       = $a->[10];
      $stordata{'pd'}->{$id}->{'tidl'}     = $a->[11];
      $stordata{'pd'}->{$id}->{'tidl-avg'} = $a->[12];
    }
  } ## end elsif ( $rwflag eq "t" )
} ## end sub pd_data

sub print_perf_pd {
  open( my $fh, '>>', $stordata{'out_perf_file'} );

  #Volume Level Statistics
  #Interval Start:   2015-07-13 16:08:33
  #Interval End:     2015-07-13 16:09:33
  #Interval Length:  60 seconds
  #---------------------
  print $fh "\nPhysical Disk Level Statistics\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";
  #
  printf $fh "Physical Disk ID,Time,Interval (s),";
  printf $fh "Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),";
  printf $fh "Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),";
  printf $fh ",,,,";
  printf $fh ",,,,";
  printf $fh "Cage Position,Type,RPM,,,";
  printf $fh ",,,\n";

  foreach my $id ( sort { $a <=> $b } ( keys( %{ $stordata{'pd'} } ) ) ) {
    #
    my $v = $stordata{'pd'}->{$id};

    #if( not defined $v->{'tops'} || not defined $v->{'rops'} || not defined $v->{'wops'}) {
    if ( not defined $v->{'rops'} ) {
      &message( "print_perf_pd: Physical Disk ID: " . $id . " has not perf data" );
      next;
    }

    #"Volume ID,Time,Interval (s),
    printf $fh ( "%d,%s,%s,", $v->{'id'}, $v->{time}, $stordata{'interval'}, );

    #"Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s)"
    printf $fh ( "%d,%d,%d,%d,%d,%d,", $v->{'rops'}, $v->{'wops'}, $v->{'tops'}, $v->{'rdps'}, $v->{'wdps'}, $v->{'tdps'} );

    #"Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms)"
    printf $fh ( "%.2f,%.2f,%.2f,%.2f,%.2f,%.2f,", $v->{'rtpo'}, $v->{'wtpo'}, $v->{'ttpo'}, $v->{'rlpo'}, $v->{'wlpo'}, $v->{'tlpo'} );
    #
    printf $fh (",,,,");

    #;
    printf $fh (",,,,");

    # Cage Position,Type,RPM,;
    printf $fh ( "%s,%s,%s,%s,,", $v->{'cage_pos'}, $v->{'type'}, $v->{'rpm'}, "" );
    #
    printf $fh (",,,\n");
    &message( "PD: " . $v->{'id'} . " Time - server: " . $stordata{'server_time'} . " - storage:" . $v->{'cage_pos'} ) if ($debug);
  } ## end foreach my $id ( sort { $a ...})

  close($fh);
} ## end sub print_perf_pd

sub parse_statcmp {
  my $filename = shift;
  if ( not defined $stordata{'pd'} ) {
    my %d;
    $stordata{'pd'} = \%d;
  }
  &message( "parse_statcmp: Process file: " . $filename );
  open( my $data, '<', $filename ) || die( "Cannot open file: " . $filename . "\n" );

  #02:20:14 04/13/2016  ---- Current ----- --------- Total ----------
  #    Node Type       Accesses Hits Hit% Accesses Hits Hit% LockBlk
  #       0 Read            216  212   98      216  212   98       0
  #       0 Write           641  304   47      641  304   47       3
  #       1 Read            390  385   99      390  385   99       0
  #       1 Write           531  123   23      531  123   23       0
  #
  #        Queue Statistics
  #Node  Free  Clean Write1 WriteN WrtSched Writing DcowPend DcowProc
  #   0 32288 404022      0      0        7       0        0        0
  #   1 31534 404224      6      1        2       0        0        0
  #
  #        Temporary and Page Credits
  #Node Node0 Node1 Node2 Node3 Node4 Node5 Node6 Node7
  #   0     0 30165   ---   ---   ---   ---   ---   ---
  #   1 30738     0   ---   ---   ---   ---   ---   ---
  #
  #        Page Statistics
  #     ---------CfcDirty--------- ----------CfcMax---------- ----------DelAck----------
  #Node FC_10KRPM FC_15KRPM NL SSD FC_10KRPM FC_15KRPM NL SSD FC_10KRPM FC_15KRPM NL SSD
  #   0         7         0  0   0      9600         0  0   0     13319         0  0   0
  #   1         9         0  0   0      9600         0  0   0      9938         0  0   0

  my $flag = 0;
  my $key  = "_none_";
  my $key2 = 0;
  my $time = " ";
  my $date;
  my @tiers;
  while ( my $ln = <$data> ) {
    chomp $ln;

    #print("__DBG__: [$key2]-[$flag]-$key-|".$ln."\n");
    #
    if ( $ln =~ /^\s*(\d{2}:\d{2}:\d{2})\s+(\d{2})\/(\d{2})\/(\d{4}).*/ ) {
      $time = $4 . "-" . $2 . "-" . $3 . " " . $1;
      next;
    }
    if ( $ln =~ /^\s*Node\s+Type\s+Accesses\s+Hits\s+/ ) { $flag = 1; $key = "base";   next; }
    if ( $ln =~ /^\s*Page\s+Statistics\s*/ )             { $flag = 1; $key = "page";   $key2++; next; }
    if ( $ln =~ /^\s*$/ )                                { $flag = 0; $key = "_none_"; next; }
    if ( $key2 == 1 && $flag && $key eq "base" && $ln =~ /^\s*(\d+)\s+(\w+)\s+([-\d\.\s]*)/ ) {
      my $id     = $1;
      my $rwflag = $2;
      my @a      = split( /\s+/, $3 );
      &cmp_base_data( $id, $rwflag, $time, \@a );
    } ## end if ( $key2 == 1 && $flag...)
    if ( $key2 == 1 && $flag && $key eq "page" ) {
      if ( $ln =~ /^\s*Node\s+([-\w\.\s]*)\s*/ ) {
        my @a = split( /\s+/, $1 );
        foreach my $type (@a) {
          if ( grep /$type/, @tiers ) {
            next;
          }
          else {
            push( @tiers, $type );
          }
        } ## end foreach my $type (@a)
        $storids{'tiers'} = \@tiers;
      } ## end if ( $ln =~ /^\s*Node\s+([-\w\.\s]*)\s*/)
      if ( $ln =~ /^\s*(\d+)\s+([-\d\.\s]*)/ ) {
        my $id = $1;
        my @a = split( /\s+/, $2 );
        &cmp_page_data_start( $id, $time, \@tiers, \@a );
      }
    } ## end if ( $key2 == 1 && $flag...)
    if ( $key2 == 2 && $flag && $key eq "page" ) {
      if ( $ln =~ /^\s*Node\s+([-\w\.\s]*)\s*/ ) {
        my @a = split( /\s+/, $1 );
        foreach my $type (@a) {
          if ( grep /$type/, @tiers ) {
            next;
          }
          else {
            push( @tiers, $type );
          }
        } ## end foreach my $type (@a)
        $storids{'tiers'} = \@tiers;
      } ## end if ( $ln =~ /^\s*Node\s+([-\w\.\s]*)\s*/)
      if ( $ln =~ /^\s*(\d+)\s+([-\d\.\s]*)/ ) {
        my $id = $1;
        my @a = split( /\s+/, $2 );
        &cmp_page_data( $id, $time, \@tiers, \@a );
      }
    } ## end if ( $key2 == 2 && $flag...)
  } ## end while ( my $ln = <$data> )
  close $data;
} ## end sub parse_statcmp

sub cmp_base_data {
  my ( $id, $rwflag, $time, $a ) = @_;

  #   ---- Current ----- --------- Total ----------
  #   Accesses Hits Hit% Accesses Hits Hit% LockBlk
  #        216  212   98      216  212   98       0
  #        641  304   47      641  304   47       3
  #
  if ( defined $stordata{'cmp'}->{$id} ) {
    $stordata{'cmp'}->{$id}->{'time'} = $time;
  }
  else {
    my %d;
    $stordata{'cmp'}->{$id}           = \%d;
    $stordata{'cmp'}->{$id}->{'id'}   = $id;
    $stordata{'cmp'}->{$id}->{'time'} = $time;
  } ## end else [ if ( defined $stordata...)]

  if ( $rwflag eq "Read" ) {
    $stordata{'cmp'}->{$id}->{'ro-cur-acc'}     = $a->[0];
    $stordata{'cmp'}->{$id}->{'ro-cur-hits'}    = $a->[1];
    $stordata{'cmp'}->{$id}->{'ro-cur-hitperc'} = $a->[2];
    $stordata{'cmp'}->{$id}->{'ro-tot-acc'}     = $a->[3];
    $stordata{'cmp'}->{$id}->{'ro-tot-hits'}    = $a->[4];
    $stordata{'cmp'}->{$id}->{'ro-tot-hitperc'} = $a->[5];
    $stordata{'cmp'}->{$id}->{'ro-tot-lock'}    = $a->[6];
  } ## end if ( $rwflag eq "Read")
  elsif ( $rwflag eq "Write" ) {
    $stordata{'cmp'}->{$id}->{'wo-cur-acc'}     = $a->[0];
    $stordata{'cmp'}->{$id}->{'wo-cur-hits'}    = $a->[1];
    $stordata{'cmp'}->{$id}->{'wo-cur-hitperc'} = $a->[2];
    $stordata{'cmp'}->{$id}->{'wo-tot-acc'}     = $a->[3];
    $stordata{'cmp'}->{$id}->{'wo-tot-hits'}    = $a->[4];
    $stordata{'cmp'}->{$id}->{'wo-tot-hitperc'} = $a->[5];
    $stordata{'cmp'}->{$id}->{'wo-tot-lock'}    = $a->[6];
  } ## end elsif ( $rwflag eq "Write")
  $stordata{'cmp'}->{$id}->{'ro-tot-lock'} = 0  if ( not defined $stordata{'cmp'}->{$id}->{'ro-tot-lock'} );
  $stordata{'cmp'}->{$id}->{'wo-tot-lock'} = 0  if ( not defined $stordata{'cmp'}->{$id}->{'wo-tot-lock'} );
  $stordata{'cmp'}->{$id}->{'time'}        = "" if ( not defined $stordata{'cmp'}->{$id}->{'time'} );
} ## end sub cmp_base_data

sub cmp_page_data_start {
  my ( $id, $time, $trs, $a ) = @_;

  # ---------CfcDirty--------- ----------CfcMax---------- ----------DelAck----------
  # FC_10KRPM FC_15KRPM NL SSD FC_10KRPM FC_15KRPM NL SSD FC_10KRPM FC_15KRPM NL SSD
  #         7         0  0   0      9600         0  0   0     13319         0  0   0
  #
  if ( defined $stordata{'cmp'}->{$id} ) {
    $stordata{'cmp'}->{$id}->{'time'} = $time;
  }
  else {
    my %d;
    $stordata{'cmp'}->{$id}           = \%d;
    $stordata{'cmp'}->{$id}->{'id'}   = $id;
    $stordata{'cmp'}->{$id}->{'time'} = $time;
  } ## end else [ if ( defined $stordata...)]
  my $tier;
  foreach $tier ( @{$trs} ) {
    shift( @{$a} );
  }
  foreach $tier ( @{$trs} ) {
    shift( @{$a} );
  }
  foreach $tier ( @{$trs} ) {
    $stordata{'cmp'}->{$id}->{'DelAckStart'}->{$tier} = shift( @{$a} );
  }
} ## end sub cmp_page_data_start

sub cmp_page_data {
  my ( $id, $time, $trs, $a ) = @_;

  # ---------CfcDirty--------- ----------CfcMax---------- ----------DelAck----------
  # FC_10KRPM FC_15KRPM NL SSD FC_10KRPM FC_15KRPM NL SSD FC_10KRPM FC_15KRPM NL SSD
  #         7         0  0   0      9600         0  0   0     13319         0  0   0
  #
  if ( defined $stordata{'cmp'}->{$id} ) {
    $stordata{'cmp'}->{$id}->{'time'} = $time;
  }
  else {
    my %d;
    $stordata{'cmp'}->{$id}           = \%d;
    $stordata{'cmp'}->{$id}->{'id'}   = $id;
    $stordata{'cmp'}->{$id}->{'time'} = $time;
  } ## end else [ if ( defined $stordata...)]
  my $tier;
  foreach $tier ( @{$trs} ) {
    $stordata{'cmp'}->{$id}->{'CfcDirty'}->{$tier} = shift( @{$a} );
  }
  foreach $tier ( @{$trs} ) {
    $stordata{'cmp'}->{$id}->{'CfcMax'}->{$tier} = shift( @{$a} );
  }
  foreach $tier ( @{$trs} ) {
    $stordata{'cmp'}->{$id}->{'DelAckEnd'}->{$tier} = shift( @{$a} );
    $stordata{'cmp'}->{$id}->{'DelAck'}->{$tier}    = $stordata{'cmp'}->{$id}->{'DelAckEnd'}->{$tier} - $stordata{'cmp'}->{$id}->{'DelAckStart'}->{$tier};
  }
} ## end sub cmp_page_data

sub print_perf_cmp {
  open( my $fh, '>>', $stordata{'out_perf_file'} );

  #Volume Level Statistics
  #Interval Start:   2015-07-13 16:08:33
  #Interval End:     2015-07-13 16:09:33
  #Interval Length:  60 seconds
  #---------------------
  print $fh "\nNode Cache Level Statistics\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";
  #
  my $tier;
  printf $fh "Node,Time,Interval (s),";
  printf $fh "Current Read IO Accesses,Current Write IO Accesses,Current Read IO Hits,Current Write IO Hits,Current Read IO Hits Percent,Current Write IO Hits Percent,";
  printf $fh "Total Read IO Accesses,Total Write IO Accesses,Total Read IO Hits,Total Write IO Hits,Total Read IO Hits Percent,Total Write IO Hits Percent,";
  printf $fh "Total Read Lock Blocked,Total Write Lock Blocked,,,";
  printf $fh ",,,,";
  for $tier ( @{ $storids{'tiers'} } ) {
    printf $fh "Tier Name,CfcDirty,CfcMax,DelAck,";
  }
  printf $fh "\n";

  foreach my $id ( sort { $a <=> $b } ( keys( %{ $stordata{'cmp'} } ) ) ) {
    #
    my $v = $stordata{'cmp'}->{$id};

    #if( not defined $v->{'tops'} || not defined $v->{'rops'} || not defined $v->{'wops'}) {
    if ( not defined $v->{'ro-cur-acc'} ) {
      &message( "print_perf_cmp: Node: " . $id . " has not perf data" );
      next;
    }

    #"Node,Time,Interval (s),
    printf $fh ( "%d,%s,%s,", $v->{'id'}, $v->{'time'}, $stordata{'interval'}, );

    #"Current Read IO Accesses,Current Write IO Accesses,Current Read IO Hits,Current Write IO Hits,Current Read IO Hits Percent,Current Write IO Hits Percent,"
    printf $fh ( "%d,%d,%d,%d,%d,%d,", $v->{'ro-cur-acc'}, $v->{'wo-cur-acc'}, $v->{'ro-cur-hits'}, $v->{'wo-cur-hits'}, $v->{'ro-cur-hitperc'}, $v->{'wo-cur-hitperc'} );

    #"Total Read IO Accesses,Total Write IO Accesses,Total Read IO Hits,Total Write IO Hits,Total Read IO Hits Percent,Total Write IO Hits Percent,"
    printf $fh ( "%d,%d,%d,%d,%d,%d,", $v->{'ro-tot-acc'}, $v->{'wo-tot-acc'}, $v->{'ro-tot-hits'}, $v->{'wo-tot-hits'}, $v->{'ro-tot-hitperc'}, $v->{'wo-tot-hitperc'} );

    #"Total Read Lock Blocked,Total Write Lock Blocked,,,"
    printf $fh ( "%d,%d,,,", $v->{'ro-tot-lock'}, $v->{'ro-tot-lock'} );
    #
    printf $fh (",,,,");
    #
    for $tier ( @{ $storids{'tiers'} } ) {

      #"Tier Name,CfcDirty,CfcMax,DelAck,"
      printf $fh ( "%s,%d,%d,%d,", $tier, $v->{'CfcDirty'}->{$tier}, $v->{'CfcMax'}->{$tier}, $v->{'DelAck'}->{$tier} );
    }
    printf $fh "\n";
    &message( "CMP: " . $v->{'id'} . " Time - server: " . $stordata{'server_time'} . " - HitPerc:" . $v->{'ro-tot-hitperc'} ) if ($debug);
  } ## end foreach my $id ( sort { $a ...})

  close($fh);
} ## end sub print_perf_cmp

### health status ###

sub set_global_health_status_summary {
  my ( $storname, $time_str ) = @_;
  my $file_state = $stordata{'out_state_file'};
  # Global health check
  my $act_timestamp = time();

  #my $act_time = localtime();
  my $status   = "ok";
  my $component_name = $storname;

  #$component_name =~ s/\s+//g;
  my $storage_status = set_health_status_html( $storname, $file_state );
  my $alert_status = 0;

  if ( $storage_status !~ "optimal" ) { $status = "error"; }
  
    my %device_status = (
    "item_id" => $ENV{'HP3PAR_HOSTCFG_ID'} ,
    "status" => "$status",
    "data" => "",
  );

  my $encode_tables = encode_json \%conf;
  my $encode_hs     = encode_json \%device_status;
  my $table = "$ENV{'HP3PAR_DIR'}/health_status/tables.json";
  my $hs    = "$ENV{'HP3PAR_DIR'}/health_status/hs.json";

  if ( defined $encode_tables && $encode_tables ne "" ) {
    open( JSON, "> $table" ) || error(" Cannot open file $table") && exit;
    print JSON "$encode_tables";
    close(JSON);
  }

  if ( defined $encode_hs && $encode_hs ne "" ) {
    open( JSON, "> $hs" ) || error(" Cannot open file $hs") && exit;
    print JSON "$encode_hs";
    close(JSON);
  }


  unlink($file_state);
} ## end sub set_global_health_status_summary

sub set_health_status_html {
  my $STORAGE_NAME       = shift;
  my $file_state         = shift;
  my $time               = localtime();
  my $status;
  open( FH, "< $file_state" ) || error( "Cannot read $file_state: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @hw_all = <FH>;
  close(FH);

  if ( check_health_status( \@hw_all ) == 0 ) {
    $status = "optimal";
    foreach my $line (@hw_all) {
      chomp $line;
      #print "$line\n";
    }
  } ## end if ( check_health_status...)
  else {
    $status = "error";
    foreach my $line (@hw_all) {
      chomp $line;
      #print "$line\n";
    }
  } ## end else [ if ( check_health_status...)]
  return $status;
} ## end sub set_health_status_html

sub check_health_status {
  my ($hw_all) = @_;
  my @data     = @{$hw_all};
  my $active   = 0;
  my $index;

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) {
      $index  = "";
      $active = 0;
      next;
    }
    $line =~ s/^\s+|\s+$//g;
    if ( $line =~ /----/ ) {
      $index  = "";
      $active = 1;
      next;
    }
    if ( $active == 1 ) {
      my @header = split( ",", $line );
      my $i = -1;
      foreach my $element (@header) {
        $i++;
        chomp $element;
        if ( $element eq "" ) { next; }
        $element =~ s/^\s+|\s+$//g;
        if ( $element eq "Level" ) {
          $index = $i;
          last;
        }
      } ## end foreach my $element (@header)
      if ( $index ne "" && defined $index ) {
        $active = 2;
        next;
      }
    } ## end if ( $active == 1 )
    if ( $active == 2 ) {
      my @status = split( ",", $line );
      chomp $status[$index];
      $status[$index] =~ s/^\s+|\s+$//g;
      my $upper_status = uc( $status[$index] );
      if ( $upper_status eq "CRITICAL" ) {
        push (@error_message, "$line\n");
        return 1;
      }
    } ## end if ( $active == 2 )
  } ## end foreach my $line (@data)

  return 0;

} ## end sub check_health_status

sub error_die {
  my $message = shift;
  print STDERR "$message\n";
  exit(1);

} ## end sub error_die

sub message {
  my $msg = shift;
  my $tm  = localtime();
  print( $tm. " INFO    :" . " HP3PAR: " . $msg . "\n" ) if ($debug);
}

sub warning {
  my ( $msg, $rc ) = @_;
  my $tm = localtime();
  print STDERR ( $tm . " WARNING :" . " HP3PAR: " . $msg . "\n" );
}

sub error {
  my ( $msg, $rc ) = @_;
  my $tm = localtime();
  if ( defined $rc ) {
    print STDERR ( $tm . " ERROR   :" . " HP3PAR: " . $msg . " - return code: " . $rc . "\n" );
  }
  else {
    print STDERR ( $tm . " ERROR   :" . " HP3PAR: " . $msg . "\n" );
  }

} ## end sub error

sub rcopy_target_conf {
  my ( $id, $a ) = @_;
  # Name         ID Type Status Options Version Policy        Socks  Buffsz SysName        SysID RemoteName     RemoteID
  #
  $stordata{'rcopy_target'}->{$id}->{'name'}           = $a->[0];
  $stordata{'rcopy_target'}->{$id}->{'type'}           = $a->[2];
  $stordata{'rcopy_target'}->{$id}->{'status'}           = $a->[3];
  $stordata{'rcopy_target'}->{$id}->{'options'}         = $a->[4];
  $stordata{'rcopy_target'}->{$id}->{'version'}           = $a->[5];
  $stordata{'rcopy_target'}->{$id}->{'policy'}             = $a->[6];
  $stordata{'rcopy_target'}->{$id}->{'socks'}          = $a->[7];
  $stordata{'rcopy_target'}->{$id}->{'buffsz'}       = $a->[8];
  $stordata{'rcopy_target'}->{$id}->{'sys_name'}       = $a->[9];
  $stordata{'rcopy_target'}->{$id}->{'sys_id'}       = $a->[10];
  $stordata{'rcopy_target'}->{$id}->{'remote_name'}      = $a->[11];
  $stordata{'rcopy_target'}->{$id}->{'remote_id'} = $a->[12];
} ## end sub rcopy_target_conf

sub rcopy_link_conf {
  my ( $id, $ln ) = @_;
  if ( $ln =~ /^\s*([-\w\.]+)\s+([-\w\.]+)\s+([-\w\.]+)\s*(\d+)\s*([-\w\.]+)\s*([-\w\.]+)\s*('Not Started'|[-\w\.]+)\s+(.*)$/ ) {
    # Target       Name               IPC   ID Node  Address       Status Options
    #
    $stordata{'rcopy_link'}->{$id}->{'target'}      = $0;
    $stordata{'rcopy_link'}->{$id}->{'name'}        = $1;
    $stordata{'rcopy_link'}->{$id}->{'ipc'}         = $2;
    $stordata{'rcopy_link'}->{$id}->{'node'}        = $4;
    $stordata{'rcopy_link'}->{$id}->{'address'}     = $5;
    $stordata{'rcopy_link'}->{$id}->{'status'}      = $6;
    $stordata{'rcopy_link'}->{$id}->{'options'}     = $7;
  }
} ## end sub rcopy_link_conf

sub rcopy_group_conf {
  my ( $id, $a ) = @_;
  # Name               ID   Target       Status   Role       Mode     LocalUserCpg LocalSnapCpg RmUserCpg    RmSnapCpg    Options                  
  #
  $stordata{'rcopy_group'}->{$id}->{'name'}             = $a->[0];
  $stordata{'rcopy_group'}->{$id}->{'target'}           = $a->[2];
  $stordata{'rcopy_group'}->{$id}->{'status'}           = $a->[3];
  $stordata{'rcopy_group'}->{$id}->{'role'}             = $a->[4];
  $stordata{'rcopy_group'}->{$id}->{'mode'}             = $a->[5];
  $stordata{'rcopy_group'}->{$id}->{'local_user_cpg'}   = $a->[6];
  $stordata{'rcopy_group'}->{$id}->{'local_snap_cpg'}   = $a->[7];
  $stordata{'rcopy_group'}->{$id}->{'remote_user_cpg'}  = $a->[8];
  $stordata{'rcopy_group'}->{$id}->{'remote_snap_cpg'}  = $a->[9];
  $stordata{'rcopy_group'}->{$id}->{'options'}          = $a->[10];
} ## end sub rcopy_link_conf

sub rcopy_map_conf {
  my ( $id, $a ) = @_;
  #   LocalVV            ID   RemoteVV          ID   SyncStatus    Resync_ss Sync_ss  VV_iter     R_iter S_iter LastSyncTime           
  #
  $stordata{'rcopy_map'}->{$id}->{'local_vv_name'}     = $a->[0];
  $stordata{'rcopy_map'}->{$id}->{'local_vv_id'}       = $a->[1];
  $stordata{'rcopy_map'}->{$id}->{'remote_vv_name'}    = $a->[2];
  $stordata{'rcopy_map'}->{$id}->{'remote_vv_id'}      = $a->[3];
  $stordata{'rcopy_map'}->{$id}->{'sync_status'}       = $a->[4];
  $stordata{'rcopy_map'}->{$id}->{'resync_ss'}         = $a->[5];
  $stordata{'rcopy_map'}->{$id}->{'sync_ss'}           = $a->[6];
  $stordata{'rcopy_map'}->{$id}->{'vv_iter'}           = $a->[7];
  $stordata{'rcopy_map'}->{$id}->{'r_iter'}            = $a->[8];
  $stordata{'rcopy_map'}->{$id}->{'s_iter'}            = $a->[9];
  $stordata{'rcopy_map'}->{$id}->{'last_sync_time'}    = $a->[10];
  $stordata{'rcopy_map'}->{$id}->{'group_name'}        = $stordata{'rcopy_group'}->{$stordata{'rcopy_map'}->{$id}->{'group_id'}}->{'name'};

} ## end sub rcopy_link_conf

sub print_conf_rcopy {
  #return;
  open( my $fh, '>>', $stordata{'out_conf_file'} );
  #
  print $fh "\nRemote Copy Configuration\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #volume id:volume name:pool id/name:size:used/free
  print $fh "Local VV ID,Local VV Name,Remote system name,Remote VV ID,Remote VV Name,Status,Syncing Level,";
  print $fh "Remote Copy Group ID,Remote Copy Group Name,Remote Copy Group Mode,Remote Copy Group Status,Remote Copy Group Syncing Level,Remote Copy Group Role,\n";

  foreach my $id ( sort { $a <=> $b } ( keys( %{ $stordata{'rcopy_map'} } ) ) ) {
    #
    my $v = $stordata{'rcopy_map'}->{$id};
    my $g = $stordata{'rcopy_group'}->{$v->{'group_id'}};

    # Local VV ID,Local VV Name,
    printf $fh ( "%d,%s,", $v->{'local_vv_id'}, $v->{'local_vv_name'} );
    # Remote system name,Remote VV ID,Remote VV Name,Status,Syncing Level,
    printf $fh ( "%s,%d,%s,%s,,", $g->{'target'}, $v->{'remote_vv_id'}, $v->{'remote_vv_name'}, $v->{'sync_status'} );
    # Remote Copy Group ID,Remote Copy Group Name,Remote Copy Group Mode,Remote Copy Group Status,Remote Copy Group Syncing Level,Remote Copy Group Role,
    printf $fh ( "%d,%s,%s,%s,,%s,\n", $g->{'id'}, $g->{'name'}, $g->{'mode'}, $g->{'status'}, $g->{'role'} );
  } ## end foreach my $id ( sort { $a ...})
  close($fh);
} ## end sub print_conf_rcopy

sub print_state_rcopy {
  open( my $fh, '>>', $stordata{'out_state_file'} );
  my @remote_copy = ();
  print $fh "\nRemote Copy State\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  #
  print $fh "Local VV ID,Local VV Name,Remote system name,Remote VV ID,Remote VV Name,Sync Status,Sync State,";
  print $fh "Remote Copy Group ID,Remote Copy Group Name,Remote Copy Group Status,Remote Copy Group State,\n";

  foreach my $id ( sort { $a <=> $b } ( keys( %{ $stordata{'rcopy_map'} } ) ) ) {
    #
    my $v = $stordata{'rcopy_map'}->{$id};
    my $g = $stordata{'rcopy_group'}->{ $v->{'group_id'} };

    # Local VV ID,Local VV Name,
    printf $fh ( "%d,%s,", $v->{'local_vv_id'}, $v->{'local_vv_name'} );

    # Remote system name,Remote VV ID,Remote VV Name,
    printf $fh ( "%s,%d,%s,", $g->{'target'}, $v->{'remote_vv_id'}, $v->{'remote_vv_name'} );

    # Sync Status, Sync State,
    printf $fh ( "%s,%s,", $v->{'sync_status'}, $storstate{'rcopy_map'}->{ $v->{'sync_status'} } );

    # Remote Copy Group ID,Remote Copy Group Name,Remote Copy Group Status,Remote Copy Group State,
    printf $fh ( "%d,%s,%s,%s,\n", $g->{'id'}, $g->{'name'}, $g->{'status'}, $storstate{'rcopy_group'}->{ $g->{'status'} } );

    #table
    #$table_tr .= "<tr><td>$v->{'local_vv_id'}</td><td>$v->{'local_vv_name'}</td><td>$g->{'target'}</td><td>$v->{'remote_vv_id'}</td><td>$v->{'remote_vv_name'}</td>";
    #$table_tr .= "<td>$v->{'sync_status'}</td><td>$storstate{'rcopy_map'}->{ $v->{'sync_status'} }</td><td>$g->{'id'}</td><td>$g->{'name'}</td><td>$g->{'status'}</td><td>$storstate{'rcopy_group'}->{ $g->{'status'} }</td></tr>\n";

    my %data;
    $data{mode}         = $g->{'mode'};
    $data{status}       = $g->{'status'};
    push(@remote_copy,\%data);
  }

  $conf{$ENV{'HP3PAR_HOSTCFG_ID'} }{'epar_remotecopy'} = encode_json(\@remote_copy);

  close($fh);
}
sub print_state_psu {
  open( my $fh, '>>', $stordata{'out_state_file'} );
  my @psu = ();

  print $fh "\nPS Level State\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  foreach my $cage ( sort ( keys( %{ $stordata{'psu'} } ) ) ) {
    foreach my $id ( sort ( values( %{ $stordata{'psu'}{$cage} } ) ) ) {
      my $level = "";
      if ( $id->{'state'} eq "OK" ) {
        print $fh "$id->{'cage'},$id->{'id'},$id->{'state'},Normal\n";
        $level = "Normal";
      }
      else {
        print $fh "$id->{'cage'},$id->{'id'},$id->{'state'},Critical\n";
        $level = "Critical";
      }
      my %data;
      $data{cage}        = $id->{'cage'};
      $data{ps}          = $id->{'id'};
      $data{state}       = $id->{'state'};
      $data{level}       = $level;
      push(@psu,\%data);
    }
  }
  $conf{$ENV{'HP3PAR_HOSTCFG_ID'} }{'epar_psu'} = encode_json(\@psu);
  
  close($fh);
}

sub print_state_cage {
  open( my $fh, '>>', $stordata{'out_state_file'} );
  my @cage = ();
  my $level = "";

  print $fh "\nCage Level State\n";
  print $fh "\tInterval Start:   ", $stordata{'server_time'}, "\t(STOR2RRD Server time)\n";
  print $fh "\tInterval Length:  ", $stordata{'interval'},    " seconds\n";
  print $fh "---------------------\n";

  print $fh "Cage,CageState,Level\n";
  foreach my $id ( sort( values( %{ $stordata{'cage'} } ) ) ) {
    if ( $id->{'state'} eq "Normal" ) {
      $level = "Normal";
      print $fh "$id->{'id'},$id->{'state'},Normal\n";
    }
    else {
      $level = "Critical";
      print $fh "$id->{'id'},$id->{'state'},Critical\n";
    }
    my %data;
    $data{cage}        = $id->{'id'};
    $data{state}       = $id->{'state'};
    $data{level}       = $level;
    push(@cage,\%data);
  }

  $conf{$ENV{'HP3PAR_HOSTCFG_ID'} }{'epar_cage'} = encode_json(\@cage);
  close($fh);
}
1;