"""

TODO: log data stats
TODO: typing of results + check structure
TODO: solve empty in this module

Lower priority:
TODO: change method to custom nan handling
TODO: integrity check of translate_freq

"""
import pandas as pd
import numpy as np
import os

import importer.dbwork as dbwork

from utility.logger import log

DEVELOP_SAVE = False

def collect_db_data(schema, table, metric, item_id, time_name="time"):
    """
    Wraps dbwork.metrics_data
    """
    if schema is None or not schema:
        log("schema is None!")
        return []
    elif table is None or not table:
        log("table is None!")
        return []
    elif metric is None or not metric:
        log("metric is None!")
        return []

    log(f"Received: schema:{schema} table:{table} metric:{metric}")

    time_series = dbwork.metrics_data(table, metric, item_id, time_name)

    log("DATA LOADED")

    points_num = len(time_series)

    # TODO: !!! testing limit, move to request/remove, 
    # move into dbwork request
    if points_num > 100000:
        time_series = time_series[-100000:]

    return time_series

def processing_of_import(time_series_data, frequency, label="data"):
    """
    Function takes array in shape of [[time, value]] to prepare pandas dataframe
    with columns ['Date', 'Value', 'valid_map'].

    Data operations:
    1) prepare valid_map: [boolean], saved to dataframe['valid_map']
    2) resample to proper frequency: input data could have empty
       Date points, data are later processed as np.arrays
    3) fill possible remaining gaps

    (Optional) DEVELOP_SAVE=True => save used dataframe to CSV for development purposes

    Optional improvements:
    TODO: check shape of input / typing of input
    TODO: check data

    """
    timeseries = np.array(time_series_data, dtype=object)
    df = pd.DataFrame(timeseries, columns = ['Date', 'Value'])
    df = df.sort_values("Date")

    df['Date'] = pd.to_datetime(df['Date'])
    df.set_index('Date', inplace=True)

    translate_freq = {
        '5m': '5min',
        '6h': '6h',
        '1d': '1D',
    }

    # older version of pandas supported following arguments
    translate_freq_backup = {
        '5m': '5T',
        '6h': '6H',
        '1d': '1D',
    }

    try:
        df = df.asfreq(translate_freq[frequency])
        empty_map = pd.Series(df.asfreq(translate_freq[frequency])["Value"]).isna()
    except:
        log("Exception: pandas translate frequency")
        df = df.asfreq(translate_freq_backup[frequency])
        empty_map = pd.Series(df.asfreq(translate_freq_backup[frequency])["Value"]).isna()

    df["valid_map"] = np.logical_not(empty_map)

    # for currently used pandas version
    try:
        df["Value"] = df['Value'].infer_objects(copy=False).interpolate()
    except:
        log("Exception: pandas cannot use linear gapfilling")
        try:
            df = df.bfill()
        except:
            df = df.fillna(method="bfill")

    """
    If there is new metric added via migration to view table,
    then its retrieved values are null since the first
    available date in table.
    """
    log("Data head precut:")
    print(df.head())
    log("Data tail precut:")
    print(df.tail())

    if not df["valid_map"].iloc[0]:
        save = 0
        for i in df.index:
            if df["valid_map"][i]:
                save = i
                break
        df = df.loc[save:]

    log("Data head:")
    print(df.head())
    log("Data tail:")
    print(df.tail())


    if DEVELOP_SAVE:
        # manual setup of DEVELOP_SAVE
        # provides simple external access to used dataframe for dev purposes
        # make sure you have dev_dir
        df_save = pd.DataFrame(df["Value"].tolist())
        dev_dir = "/home/lpar2rrd/nextgen/nextgen/server-nest/microservices/analyzer/data"
        try:
            df_save.to_csv(f"{dev_dir}/last_{label}_{frequency}.csv")
            df_save.to_csv(f"{dev_dir}/last_{frequency}.csv")
        except:
            print("Data are not saved. Something is wrong.")

    return df


