"""
DONE:
- parse request structure
- check request keys
- check prediction response keys
- check implementation of request -> command
- check implementation of received pair command-frequency 

TO DECIDE:
check command

TODO: add relevancy check of commands (this feature might be problematic: excessive reading of db)
TODO: logging module

Low priority:
TODO: typing or request object
TODO: common check to make sure that command_is_implemented
TODO: restructure all requests from {request: {*key: value}} to {*key: value}


check incoming request formed as {"request" : {key:value}}
VS
check naked request {key:value}

"""
from utility.logger import log

class Request:
    def __init__(self, request):
        """
        Check request on initialization. Then there will be no problems
        with data loading.
        """
        if check_request_keys(request):
            self.request_ok = True
            self.request = request
        else:
            log("Wrong request structure.")
            log(request)
            self.request_ok = False
            self.request = {}

    def is_valid(self):
        return self.request_ok
#
# Checking and reading
#
def read_request(request, show_read=False, is_naked=False):
    """
    is_naked:
        is_naked=False => structure {"request" : {key:value}}
        is_naked=True  => structure {key:value}
    """
    log("Reading request")

    if 'request' in request.keys():
        request = request['request']

    if check_request_keys(request):

        analyzer_command  = request['command']
        schema            = request['schema']
        table             = request['table']
        metric            = request['column']
        item_id           = request['id']
        frequency         = request['frequency']
        maximal_value     = request['max']
        time_name         = "bucket"
    else:
        log("WRONG KEYS")
        return []
 
    if show_read:
        print()
        print("ACCEPTED PARAMS")
        print("--------------------------------------")
        print(f"Analyzer command: {analyzer_command}")
        print(f"Schema:           {schema}")
        print(f"Table:            {table}")
        print(f"Metric:           {metric}")
        print(f"Item ID:          {item_id}")
        print(f"Frequency:        {frequency}")
        print(f"Maximal value:    {maximal_value}")
        print(f"Time name:        {time_name}")
        print("--------------------------------------")
        print()

    return (analyzer_command, schema, table, metric, time_name, item_id, frequency, maximal_value)

def overall_request_check(request):
    if not check_request_keys(request):
        return False

    if "request" in request.keys():
        request = request["request"]

    # request checked, check value implementation
    if not command_is_implemented(request["command"]):
        return False

    if not frequency_is_implemented(request["command"],
                                    request["frequency"]):
        return False

    # MORE TO CHECK - db: 'schema', 'table', 'column', 'id'

    return True

def check_request_keys(request):
    # add check on string structure
    log("")

    if "request" in request.keys():
        request = request["request"]

    if type(request) is not dict:
        log("WRONG request datatype!")
        return False

    must_have = ('command', 'schema', 'table', 'column', 'id', 'frequency', 'max')

    for key_key in must_have:
        if not key_key in request:
            log(f"[!] Missing key - {key_key}")
            return False
    return True

def command_is_implemented(command):
    log("Checking command implementation")
    implemented_commands = (
        "predict_capacity",
        "predict_cpu",
        "predict_vm"
    )

    if command in implemented_commands:
        return True
    else:
        log(f"Command {command} is not implemented!")
        return False

def frequency_is_implemented(command, frequency):
    log("Checking frequency")

    command_frequencies = {
        "predict_capacity": ["1d"],
        "predict_cpu": ["5m", "6h"],
        "predict_vm": ["5m", "6h"]
    }

    if str(frequency) in command_frequencies[command]:
        return True
    else:
        log(command, frequency, command_frequencies[command])
        log(f"Frequency {frequency} for command {command} is not implemented!")
        return False

def check_prediction_response_structure(prediction):
    log("Checking response structure")

    names_to_check = ('prediction', 'name', 'start_time', 'days_to_100_percent_fill')

    for prediction_name in prediction.keys():
        for name_to_check in names_to_check:
            if not name_to_check in prediction[prediction_name].keys():
                log(f"[!] Name {name_to_check} is not in prediction {prediction_name}")

    pass

"""
import json

def save_response(response, save_command):
    if not response:
        log("No response!")
    else:
        log("Saving response")
        filename = f"{total_path}/xormonyzer/temporary_data/{save_command}.json"
        filename = f"/home/lpar2rrd/nextgen/nextgen/server-nest/microservices/analyzer/tmp_data/{save_command}.json"

        pd.DataFrame(response).to_csv(filename)
        with open(filename, 'w') as outfile:
            json.dump(response, outfile)

    return save_command
"""
