package COHESITY;

use strict;
use warnings;
use lib '../../../perl-modules';

use Digest::MD5 qw(md5 md5_hex md5_base64);
use Data::Dumper;
use Date::Parse;
use Socket;
use JSON qw(encode_json decode_json);
use Time::Local;

use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder

use Xormon;
use XormonLogger;
use Test;


my $DEBUG = 0;
my $CYCLE_DEBUG = 0;
my $TEST_DATA_IN_GUI = 0;
my $REVEAL_PERF_CSV = 0;

## commented out for clarity
##
## SAVE_RESPONSE toggle: 
##   save every JSON that is accesed throught general_data_request
##   save timestamp
## LOAD_RESPONSES toggle:
##   provides access to saved responses
##   enables proper saving of past values as latest values
#
#use Response_saver; # module providing necessary functionality
#
#my $SAVE_RESPONSE   = 0;
#my $LOAD_RESPONSES  = 1;


# === CONTROL
# sub collect

# === DEVICE CONNECTION CONFIGURATION
# sub entirety_check
# sub global_cfg_load
my $cfg;
my $xormon;
my ($api_ip, $username, $password, $storage_name);
my $token;
my $storage_uuid; # hostcfg_id

# === MISCELLANEOUS
# sub global_queries
my $query_ip;
my $baseUr;
# sub generate_ids_from_schemaName
# sub generate_ids_from_schemaName_interface
# sub check_limit_to_sleep

# === TESTS
# sub connection_test
# sub test_perf_data
# sub test_performance_value

# === CONNECTION: REQUESTS
# sub generate_new_token
# sub general_data_request
# sub general_hash_request
# sub general_array_request
my %current_hash;

# === TIME
# sub time_handling
my ($query_time, $delayed_time);

# === PERFORMANCE
# sub performance_data
my %data;

# === PERFORMANCE: LOAD DATA TO HASH
# sub local_data
# sub actualize_performance_hash
# sub actualize_performance_hash_with_values
# sub load_data_from_timeSeriesStats
# sub load_interface_data_from_timeSeriesStats
my $rollupFunction = 'average';
my $rollupIntervalSecs = 300;

# === CONFIGURATION
# sub configuration_data
# sub collect_pool
# sub collect_port
# sub collect_cluster
# sub collect_nodes
# sub collect_drives
my %conf;

# === HS
# sub health_status

# === MODIFY
# sub modify_perf_data
# sub make_typical_totals
# sub data_hash_total
# sub data_hash_average
# sub total_data
# sub average_data
my @names_to_total;
my %average_to_rate;
my %local_hash; # ! weird global name
my $Ti = 1024*1024*1024*1024;

my %url_file_map = ();

# === SAVE
# sub save_data
my $csv_string = "";

# === MESSAGE
# sub error
# sub debug_message
my @rows;

# === QUERIES
# sub generate_id_query
# sub generate_query
# sub generate_performance_query

# ----------------------------------------------------------------------------------------
# INITIAL CODE NAMES
# ----------------------------------------------------------------------------------------
my %conf_hash;
my %capacity_hash;

my %name_and_codename;
my %section_name_path;
my %nodeID_nodeName;

my $value_type;
my $type;
my $specific;
my $schemaName;
my $metricName;
my $value_name;
my $section;
my $name;

my @Ids;
my $Id;

my $useId;

my $query;

# Global translations of standardized names to API names
my %name_and_metric;
my %metric_and_name;
my %type_and_schema;
my @metric_names;

my $valueType;
my $logger;

sub global_cfg_load {
  my $storage_configuration = shift; # cfg

  # ADD || logging...
  my $apiHost     = $storage_configuration->{'apiHost'};
  my $apiPort     = $storage_configuration->{'apiPort'};
  my $apiUsername = $storage_configuration->{'apiUsername'};
  my $apiPassword = $storage_configuration->{'apiPassword'};
  my $hostalias   = $storage_configuration->{'hostalias'};

  $storage_uuid   = $storage_configuration->{'hostcfg_id'};

  $api_ip       = "${apiHost}:${apiPort}";
  $username     = $apiUsername;
  $password     = $apiPassword;
  $storage_name = $hostalias;

  $logger = XormonLogger->new( 'cohesity', $storage_uuid, $hostalias );
}

sub collect {
  $xormon  = shift;
  $cfg     = shift;

  ($query_time, $delayed_time)  = time_handling();

  #
  # Config check and load
  #
  entirety_check($cfg);
  global_cfg_load($cfg);

  global_queries();

  #
  # Token
  #
  infolog("GENERATE TOKEN");
  $token = generate_new_token();

  if ( ! $token || ! defined $token ) {
    infolog("NO TOKEN");
    $xormon->isConnected(0, "Token not granted");
    exit;
  }
  else {
    infolog("TOKEN ACCESSED");
    $xormon->isConnected(1);
  }
  $cfg->{'timestamp'} = $delayed_time;

  #
  # PERFORMANCE
  #
  # global %data
  performance_data();
  modify_perf_data();

  #
  # CONFIGURATION
  #
  my $configurations = configuration_data();

  #
  # SAVE
  #
  save_data($xormon, $configurations);

}

sub entirety_check {

  my @cfg_check_names = (
    'hostcfg_id',
    'hostalias',
    'apiHost',
    'apiPort',
    'apiUsername',
    'apiPassword',
  );

  # $cfg entirety check
  for my $cfg_check_name (@cfg_check_names){
    unless ( exists $cfg->{$cfg_check_name} ){
      Xormon::error("Cannot continue, '$cfg_check_name' is not configured!") && return "error";
    }
  }

  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
}

sub generate_new_token {

  # development purposes
  # look for $LOAD_RESPONSES and $SAVE_RESPONSE for context
  #if ($LOAD_RESPONSES) {
  #  return "TEST_TOKEN";
  #}

  my $token_query = q(/public/accessTokens);

  debug_message("NEW TOKEN");

  my $url = 'https://'.$api_ip.'/irisservices/api/v1'.$token_query;

  my $user_agent = LWP::UserAgent->new(
    ssl_opts => {
      SSL_cipher_list => 'DEFAULT:!DH',
      verify_hostname => 0,
      SSL_verify_mode => 0
    }
  );

  my $request = HTTP::Request->new( POST => $url );
  $request->header( 'Content-Type' => 'application/json' );
  $request->header( 'Accept'       => 'application/json' );
  $request->header( 'Connection'   => 'keep-alive' );

  my $post_data = qq({ "username": "$username","password": "$password"});

  $request->content($post_data);

  my $result = $user_agent->request($request);

  my %content;

  eval{
    %content = %{decode_json($result->{'_content'})};
  };
  if($@){
    # Anonymize for logging
    undef $result->{'_request'};

    my $error_message = "";
    $error_message .= "PROBLEM DURING TOKEN ACCESS \n";

    error($error_message);
    error(Dumper $result);

    infolog( "\n REQUEST-CLEAN RESPONSE: \n");
    infolog(Dumper $result);

    exit;
  }

  # Work with request limits (Cohesity specific):
  ## number of available requests on reset
  my $x_ratelimit_limit     = $result->{'_headers'}{'x-ratelimit-limit'};
  my $x_ratelimit_remaining = $result->{'_headers'}{'x-ratelimit-remaining'};
  ## unixtime of reset
  my $x_ratelimit_reset     = $result->{'_headers'}{'x-ratelimit-reset'};

  ## check limit; if < 2: sleep until limit reset
  if ( defined $x_ratelimit_limit && defined $x_ratelimit_remaining && defined $x_ratelimit_reset ) {
    check_limit_to_sleep($x_ratelimit_remaining, $x_ratelimit_limit, $x_ratelimit_reset);
  }

  my $new_token = $content{accessToken};

  return $new_token;
}

# Some environments have limit on number of requests in time interval.
sub check_limit_to_sleep {
  my $limit_remaining   = shift;
  my $total_limit       = shift;
  my $moment_of_reset   = shift;

  my $actual_time = time();
  debug_message("REMAINING LIMIT: $limit_remaining | TOTAL LIMIT: $total_limit | RESET TIME: $moment_of_reset | TIMESTAMP NOW: $actual_time");

  my $possible_waiting_interval = $moment_of_reset - $actual_time + 1;

  if ( $limit_remaining < 2 ) {
    debug_message( "WAITING: $possible_waiting_interval ");
    sleep($possible_waiting_interval);
  }

}

sub time_handling {
  # ----------------------------------------------------------------------------------------
  # CURRENT TIME, TIME HANDLING
  # NOTE: Script is collecting lagged values due to complications in actual values returned by API
  # ----------------------------------------------------------------------------------------
  # Delayed time: APIs values from latest interval differ from delayed intervals
  my $delay_interval = 600;
  my $use_time = time() - $delay_interval;

  my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($use_time);

  # Time in 5 minute ticks
  my $mod_min = $min;
  $mod_min %= 5;
  $min = $min - $mod_min;
  $min %= 60;

  my $upd_time = timelocal(0,$min,$hour,$day,$month,$year + 1900);

  my $time_now = $upd_time;

  my $startMsecs = ($time_now-3600)*1000;
  my $endMsecs = ($time_now)*1000;

  my $q_time = "startTimeMsecs=${startMsecs}&endTimeMsecs=${endMsecs}";

  return( $q_time, $time_now )
}

sub global_queries {
  # use after global_cfg_load($cfg);
  $baseUr = 'https://'.$api_ip.'/irisservices/api/v1';
  $query_ip = 'https://'.$api_ip;
}

sub connection_test {
  my $client  = shift;
  $cfg        = shift;

  my $test = Test->new($client);

  #
  # [1] CFG TEST
  #
  ($query_time, $delayed_time)  = time_handling();

  my @cfg_check_names = (
    'hostcfg_id',
    'hostalias',
    'apiHost',
    'apiPort',
    'apiUsername',
    'apiPassword',
  );

  for my $cfg_check_name (@cfg_check_names){
    unless ( exists $cfg->{$cfg_check_name} ){
      $test->addSection("Device cfg", 0 , "Cannot continue, $cfg_check_name is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Device cfg", 1 , "");

  global_cfg_load($cfg);

  global_queries();

  #
  # [2] CONNECTION TEST
  #
  my $connected = Test::connTest($cfg->{apiHost}, $cfg->{apiPort});

  if ($connected == 1) {
    $test->addSection("API host network connection", 1, "TCP connection to " . $cfg->{apiHost} . ":" . $cfg->{apiPort} . " is OK.");
  }
  else {
    $test->addSection("API host network connection", 0, "TCP connection to " . $cfg->{apiHost} . ":" . $cfg->{apiPort} . " has failed!");
    $test->stop();
  }

  #
  # [3-1] API AUTH: TOKEN
  #
  my $configuration_data_result = 1;

  $token   = generate_new_token();

  my %system_information = general_hash_request('GET', generate_query('cluster'));

  my $token_works = "";
  $token_works = "yes" if $system_information{name};

  if ($token_works) {
    push(@rows, "Token is available. \n\n");
  }
  else {
    $test->addSection("API Authorization", 0, "Token is not available!");
    $test->stop();
  }

  #
  # [3-2] API AUTH: CLUSTER CONFIGURATION
  #
  %system_information = general_hash_request('GET', generate_query('cluster'));

  my @names = (
    'Name',
    'IPs',
    'ID',
    'Incarnation ID',
    'SW version',
    'Target SW version',
    'Type',
    'Node Count',
  );

  my %name_path = (
    'Name'              => 'name',
    'IPs'               => 'nodeIps',
    'ID'                => 'id',
    'Incarnation ID'    => 'incarnationId',
    'SW version'        => 'clusterSoftwareVersion',
    'Target SW version' => 'targetSoftwareVersion',
    'Type'              => 'clusterType',
    'Node Count'        => 'nodeCount',
  );


  for my $name (@names){
    my $path = $name_path{$name};
    if ( defined $system_information{$path} ) {
      push(@rows, "$name:  $system_information{$path}\n");
    }
    else {
      $configuration_data_result = 0;
    }
    undef $path;
  }

  if ( $configuration_data_result ) {
    $test->addSection("API Authorization", 1, \@rows);
  }
  else {
    # do not stop, try performance
    $test->addSection("API Authorization", 0, \@rows);
  }

  #
  # [4] PERFORMACE DATA TEST
  #
  my $performance_is_collectable = test_perf_data();

  if ($performance_is_collectable) {
    $test->addSection("API Data Test", 1, "Performance data OK!");
  }
  else {
    $test->addSection("API Data Test", 0, "Performance data not found!");
  }

  return $test->stop();
}

sub test_perf_data {

  my $test_result = 1;

  my %schema_metric = (
    'kSentryClusterStats'         => 'kCpuUsagePct',
    'kBridgeClusterStats'         => 'kReadIos',
    'kBridgeViewBoxLogicalStats'  => 'kReadIos',
    'kBridgeViewBoxLogicalStats'  => 'kReadLatencyUsecs',
    'kSentryNodeStats'            => 'kCpuUsagePct',
  );

  for $schemaName (keys %schema_metric){
    for $metricName ($schema_metric{$schemaName}){
      if ( ! test_performance_value($schemaName, $metricName)){
        $test_result = 0;
        return $test_result;
      }
    }
  }

  return $test_result;

}

sub test_performance_value {
  $schemaName = shift;
  $metricName = shift;

  @Ids = generate_ids_from_schemaName($schemaName);
  $Id = $Ids[0];

  my $query = generate_performance_query();
  %current_hash = general_hash_request('GET', $query);

  if (! defined $current_hash{metricName}) {
    return 0;
  }
  else {
    return 1;
  }
}

sub health_status {
  # ----------------------------------------------------------------------------------------
  # HEALTH STATUS
  # OUTPUT: events, global hs, custom hs
  # SPECIFICS: time used in query is in microseconds
  #
  # HASH STRUCTURE:
  # OUT: %health_hash_named = Health category => Health specifics
  #      @events = NUMBER => alerting event specifics
  #      $global_hs = [ok|error]
  # ----------------------------------------------------------------------------------------
  my %health_hash;
  my %health_hash_named;
  my $category;
  my $global_hs = 'ok';
  #my $time_start = time();
  my %types;
  my %alerting_hash;
  my @alerting_queries;

  my $health_time_start = time()*1000-604800000;
  my $formated_time;

  # string was used due to some problems on AIX in STOR
  my $hs_time_start = time()-604800;
  $hs_time_start = "$hs_time_start"."000000";

  my $hs_time_end = time();
  $hs_time_end = "$hs_time_end"."000000";

  my $critical_message;
  my @wild_hash;
  my %wild_hash;
  my @events =();

  # GLOBAL HEALTH STATUS
  global_queries();

  $query = $query_ip .  '/v2/alerts-summary?startTimeUsecs='."$hs_time_start"."&endTimeUsecs=$hs_time_end";
  %wild_hash = general_hash_request('GET', $query);

  for (my $i=0; $i < scalar(@{$wild_hash{alertsSummary}}); $i++){
    if ($wild_hash{alertsSummary}[$i]{criticalCount}){
      $global_hs = 'error';
    }
  }

  # HEALTH STATUS SUMMARY
  for (my $i=0; $i < scalar(@{$wild_hash{alertsSummary}}); $i++){
    $type = $wild_hash{alertsSummary}[$i]{type};
    $category = $wild_hash{alertsSummary}[$i]{category};
    $health_hash{$type}{$category}{'Warning Count'}=$wild_hash{alertsSummary}[$i]{warningCount};
    $health_hash{$type}{$category}{'Critical Count'}=$wild_hash{alertsSummary}[$i]{criticalCount};
  }

  # HEALTH EVENTS
  %types = (
    "Hardware" =>
    ["Cluster",  "Power Supply", "Firmware", "CPU", "Memory",
     "Temperature", "Node", "Fan", "Chassis", "Disk", "NIC"],
    "Software" =>
    ["System Service", "Data Path", "Node Health", "Indexing",
     "App MarketPlace", "Operating System", "Metadata"],
    "Maintenance" =>
    ["Storage Usage", "Fault Tolerance", "Cluster Management",
    "Upgrade", "Security", "License", "Configuration", "Networking"],
    "Data Service" =>
    ["View Failover", "Backup Restore", "Archival Restore",
    "Disaster Recovery", "CDP",  "Remote Replication", "Quota"],
  );

  my $order_counter = 0;
  my %help_hash;

  my $event_name;
  my $modified_event_name;
  for $event_name ( keys %types){
    $modified_event_name = 'k'.$event_name;
    $modified_event_name =~ s/\s//g;
    for my $name1 ( @{$types{$event_name}}){
      my $name2 = 'k'.$name1;
      $name2 =~ s/\s//g;
      $health_hash_named{$event_name}{$name1} = $health_hash{$modified_event_name}{$name2};
      $health_hash_named{$event_name}{$name1}{Name}=$name1;
    }
  }

  $query = $query_ip . '/irisservices/api/v1/public/alerts?maxAlerts=100&alertStateList=kOpen&alertSeverityList=kWarning&startDateUsecs='."$health_time_start"."000";
  push (@alerting_queries, $query);
  $query = $query_ip . '/irisservices/api/v1/public/alerts?maxAlerts=100&alertStateList=kOpen&alertSeverityList=kCritical&startDateUsecs='."$health_time_start"."000";
  push (@alerting_queries, $query);

  for my $alerting_query (@alerting_queries){
    @wild_hash = general_array_request('GET', $alerting_query);
    for (my $i=0; $i < scalar(@wild_hash); $i++){

      $formated_time = localtime($wild_hash[$i]{latestTimestampUsecs}/1000000);
      $alerting_hash{'Severity'}            = $wild_hash[$i]{severity},
      $alerting_hash{'Id'}                  = $wild_hash[$i]{id},
      $alerting_hash{'Time'}                = $formated_time,
      $alerting_hash{'Alert Name'}          = $wild_hash[$i]{alertDocument}{alertName},
      $alerting_hash{'Alert Category'}      = $wild_hash[$i]{alertCategory},
      $alerting_hash{'Alert Code'}          = $wild_hash[$i]{alertCode},
      $alerting_hash{'Alert Description'}   = $wild_hash[$i]{alertDocument}{alertDescription};
      if ( $alerting_hash{'Severity'} =~ /Critical/){
        $critical_message = $alerting_hash{'Alert Description'};
      }
      my %saving_hash = %alerting_hash;
      push(@events,\%saving_hash);
    }
  }

  my @hs = (
    {
      "item_id" => $storage_uuid,
      "status" => $global_hs,
      "data" => $critical_message,
    }
  );

  $xormon->saveStatus(\@hs);

  # TODO: change to keys, move somewhere else
  my %health_table_order = (
    "Hardware"      => ["Name", "Warning Count", "Critical Count"],
    "Software"      => ["Name", "Warning Count", "Critical Count"],
    "Maintenance"   => ["Name", "Warning Count", "Critical Count"],
    "Data Service"  => ["Name", "Warning Count", "Critical Count"]
  );

  $conf{$storage_uuid}{'events'} = encode_json(\@events);
  $conf{$storage_uuid}{'health_summary'} = encode_json(\%health_hash_named);
  $conf{$storage_uuid}{'health_summary_order'} = encode_json(\%health_table_order);
}

my $metricNames;

sub performance_data {
  #=========================================================================================
  # PERFORMANCE
  #=========================================================================================

  my $url_v2_timeseries;
  my $url_v2    = "https://$api_ip";

  my $startTimeMsecs  = ($delayed_time-3600)*1000;
  my $endTimeMsecs    = ($delayed_time)*1000;

  #-----------------------------------------------------------------------------------------
  # PERFORMANCE: CLUSTER
  #-----------------------------------------------------------------------------------------
  debug_message("STARTING CLUSTER SECTION");


  # PERFORMANCE: CLUSTER: CPU/MEMORY
  $type = 'CLUSTER';

  $valueType      = 'doubleValue';
  $schemaName     = 'kSentryClusterStats';
  $rollupFunction = 'average';
  my $cluster_id;
  @Ids = generate_ids_from_schemaName($schemaName);

  foreach my $saveId (@Ids){
    $Id = $saveId;
    $cluster_id = $saveId;

    actualize_performance_hash_with_values('sys', 'kCpuUsagePct');
    actualize_performance_hash_with_values('memory', 'kMemoryUsagePct');
  }

  # PERFORMANCE: CLUSTER: PHYSICAL/LOGICAL
  %type_and_schema = (
    'CLUSTER' => 'kBridgeClusterLogicalStats'
  );
  # PERFORMANCE: CLUSTER: PHYSICAL/LOGICAL: IOPS/DATA/LATENCY/ERRORS
  # Additional metric scheme: 'Cluster_logical' => 'kBridgeClusterLogicalStats'
  # LOGICAL VS PHYSICAL: LOGICAL == POOL, PHYSICAL == NODE
  $valueType = 'int64Value';
  $rollupFunction = 'rate';
  %name_and_metric = (
    'read'      => 'kNumBytesRead',
    'write'     => 'kNumBytesWritten',
    'resp_t_r'  => 'kReadLatencyUsecs',
    'resp_t_w'  => 'kWriteLatencyUsecs',
  );

  foreach my $saveId (@Ids){
    $Id = $saveId;
    foreach my $name1 (keys %type_and_schema){
      $type = $name1;
      $schemaName = $type_and_schema{$type};
      foreach my $name2 (keys %name_and_metric){
        $specific = $name2;
        $metricName = $name_and_metric{$specific};
        actualize_performance_hash();
      }
      make_typical_totals();
    }
  }

  $rollupFunction = 'average';
  %name_and_metric = (
    'read_io'   => 'kReadIos',
    'write_io'  => 'kWriteIos',
  );

  foreach my $saveId (@Ids){
    $Id = $saveId;
    foreach my $name0 (keys %name_and_metric){
      $specific = $name0;
      $metricName = $name_and_metric{$specific};
      actualize_performance_hash();
    }
    make_typical_totals();
  }

  #-----------------------------------------------------------------------------------------
  # PERFORMANCE: POOL (=STORAGE DOMAINS)
  #-----------------------------------------------------------------------------------------
  debug_message("STARTING POOL SECTION");

  $type = 'POOL';
  $query = $baseUr . '/public/viewBoxes';
  my @Id_hashes = general_array_request('GET', $query);
  @Ids = ();

  for (my $i = 0; $i < scalar(@Id_hashes); $i++){
    push (@Ids, $Id_hashes[$i]{id});
  }

  # PERFORMANCE: POOL (=STORAGE DOMAINS): IOPS/DATA/LATENCY/ERRORS
  $schemaName = 'kBridgeViewBoxLogicalStats';
  $valueType = 'int64Value';

  $rollupFunction = 'rate';
  %name_and_metric = (
    'read'          => 'kNumBytesRead',
    'write'         => 'kNumBytesWritten',
    #'Errors_read'   => 'kNumReadErrors',
    #'Errors_write'  => 'kNumWriteErrors',
    'resp_t_r'      => 'kReadLatencyUsecs',
    'resp_t_w'      => 'kWriteLatencyUsecs',
  );

  foreach my $saveId (@Ids){
    $Id = $saveId;
    foreach my $name0 (keys %name_and_metric){
      $specific = $name0;
      $metricName = $name_and_metric{$specific};
      actualize_performance_hash();
    }
  }

  $rollupFunction = 'average';
  %name_and_metric = (
    'read_io'   => 'kReadIos',
    'write_io'  => 'kWriteIos',
  );

  foreach my $saveId (@Ids){
    $Id = $saveId;
    foreach my $name0 (keys %name_and_metric){
      $specific = $name0;
      $metricName = $name_and_metric{$specific};
      actualize_performance_hash();
    }
    make_typical_totals();
  }

  #-----------------------------------------------------------------------------------------
  # PERFORMANCE: NODES
  #-----------------------------------------------------------------------------------------
  debug_message("STARTING NODE SECTION");

  $type = 'NODE';
  $schemaName = 'kSentryNodeStats';
  $rollupFunction = 'average';
  @Ids = generate_ids_from_schemaName($schemaName);

  foreach my $saveId (@Ids){
    $Id = $saveId;

    # PERFORMANCE: NODES: CPU/MEMORY
    $schemaName = 'kSentryNodeStats';
    $valueType = 'doubleValue';
    $rollupFunction = 'average';

    $specific = 'cpu_system';
    $metricName = 'kCpuUsagePct';
    actualize_performance_hash();

    $specific = 'memory_used_percent';
    $metricName = 'kMemoryUsagePct';
    actualize_performance_hash();

    actualize_performance_hash_with_values('cpu_system', 'kCpuUsagePct');
    actualize_performance_hash_with_values('memory_used_percent', 'kMemoryUsagePct');

    #if (defined $data{$type}{$Id}{$specific}{value}){
    #  $data{$type}{$Id}{$specific}{value} = $data{$type}{$Id}{$specific}{value} / scalar(@Ids);
    #}
  }

  # PERFORMANCE: NODES: IOPS/DATA/LATENCY/ERRORS
  $schemaName = 'kBridgeNodeStats';
  $valueType = 'int64Value';
  $rollupFunction = 'rate';

  %name_and_metric = (
    'read'          => 'kNumBytesRead',
    'write'         => 'kNumBytesWritten',
    #'Errors_read'  => 'kNumReadErrors',
    #'Errors_write' => 'kNumWriteErrors',
    'resp_t_r'      => 'kReadLatencyUsecs',
    'resp_t_w'      => 'kWriteLatencyUsecs',
  );

  foreach my $saveId (@Ids){
    $Id = $saveId;
    foreach my $name0 (keys %name_and_metric){
      $specific = $name0;
      $metricName = $name_and_metric{$specific};
      actualize_performance_hash();
    }
    make_typical_totals();
  }

  $rollupFunction = 'average';
  %name_and_metric = (
    'read_io'   => 'kReadIos',
    'write_io'  => 'kWriteIos',
  );

  foreach my $saveId (@Ids){
    $Id = $saveId;
    foreach my $name0 (keys %name_and_metric){
      $specific = $name0;
      $metricName = $name_and_metric{$specific};
      actualize_performance_hash();
    }
    make_typical_totals();
  }

  #-----------------------------------------------------------------------------------------
  # PERFORMANCE: DISK==>DRIVE
  #-----------------------------------------------------------------------------------------
  debug_message("STARTING DRIVE SECTION");

  $type = 'DRIVE';
  # PERFORMANCE: DISK: UTILIZATION/RESPONSE TIME/DATA/IOPS

  $schemaName     = 'kBridgeDiskStats';
  $rollupFunction = 'kAverage';
  $valueType      = 'int64Value';

  @Ids = generate_ids_from_schemaName($schemaName);

  %name_and_metric = (
    'read'      => 'kNumBytesRead',
    'write'     => 'kNumBytesWritten',
    'resp_t_r'  => 'kReadLatencyUsecs',
    'resp_t_w'  => 'kWriteLatencyUsecs',
    'read_io'   => 'kReadIos',
    'write_io'  => 'kWriteIos',
  );

  # Decision object
  # Everything is called with average rollup to minimize
  # number of requests. Equation within the same rollup
  # interval is: rate = average / time_interval
  %average_to_rate = (
    'read'      => '1',
    'write'     => '1',
    'resp_t_r'  => '1',
    'resp_t_w'  => '1',
  );

  %metric_and_name = reverse %name_and_metric;
  @metric_names    = keys %metric_and_name;

  $metricNames = join(",", sort @metric_names);

  $url_v2_timeseries = "${url_v2}/v2/stats/time-series-stats";

  foreach my $ID (@Ids) {
    $Id = $ID;

    $query  = "?";
    $query .= "entityId=$Id";
    $query .= "&schemaName=$schemaName";
    $query .= "&rollupIntervalSecs=30";
    $query .= "&rollupFunction=$rollupFunction";
    $query .= "&startTimeMsecs=$startTimeMsecs";
    $query .= "&endTimeMsecs=$endTimeMsecs";

    $query .= "&metricNames=$metricNames";

    my $url = $url_v2_timeseries . $query;

    %local_hash = general_data_request("GET", $url, "HASH");

    load_data_from_timeSeriesStats();

    make_typical_totals();
  }

  sub load_data_from_timeSeriesStats {
    # Problem: API returns "random" uniform intervals with no connection to rollupInterval
    if ( %local_hash && ref($local_hash{"timeSeriesStats"}) eq "ARRAY") {
      # Cycle over blocks with metric
      # Note that time is in milliseconds
      for ( my $i = 0; $i < scalar (@{$local_hash{"timeSeriesStats"}}); $i++ ) {

        my $current_metric;
        if ( defined $local_hash{"timeSeriesStats"}[$i]{"metricName"} ) {
          $current_metric = $local_hash{"timeSeriesStats"}[$i]{"metricName"};
          $specific = $metric_and_name{$current_metric};
        }

        # Latest timestamp: used to
        # 1. try to get returned interval
        # 2. break cycle below
        my $latest_timestamp;
        if ( defined $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-1]{"timestampMsecs"} ) {
          $latest_timestamp  = $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-1]{"timestampMsecs"};
        }

        # Interval is not always equal to rollupInterval
        # Probably bug in API
        # When called with 300 || 30 ... >> returns 10 second rollupIntervals
        # Treat Undefs as 0: Cohesity API does that with pools -> expect it here
        my $final_to_average;

        # used weight and interval: weight for Average work
        #                           interval to change Average to Rate
        # >> Weight: used to average of sum of values in "smaller intervals"
        #         (weight = number of "smaller intervals")
        my $weight = 0;
        # >> Get returned interval (or expect rollup) == "smaller interval":
        my $interval = $rollupIntervalSecs; # should be rollup interval
        if ( defined $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-2]{"timestampMsecs"} ) {
          $interval  = $latest_timestamp - $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-2]{"timestampMsecs"};
          $interval /= 1000;
        }

        # Data loading cycle:
        # Sum values to $final_to_average
        if ( defined $local_hash{"timeSeriesStats"}[$i]{"dataPoints"} && ref($local_hash{"timeSeriesStats"}[$i]{"dataPoints"}) eq "ARRAY" ) {
          $final_to_average = 0;
          for ( my $j = 1; $j < (scalar (@{$local_hash{"timeSeriesStats"}[$i]{"dataPoints"}}) -1 ); $j++ ){
            if ( defined $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-$j]{"timestampMsecs"} ) {

              my $local_interval = $latest_timestamp - $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-$j]{"timestampMsecs"};
              if ( $local_interval >= 300000 ) {
                last;
              }

              $weight ++;

              if ( defined $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-$j]{"int64Value"} and $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-$j]{"int64Value"} ) {
                $final_to_average += $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-$j]{"int64Value"};
              }

            }

          }
        }

        $data{$type}{$Id}{$specific}{value}  = $final_to_average;

        # Average over total number of small intervals
        if ( defined $data{$type}{$Id}{$specific}{value} && $weight ) {
          $data{$type}{$Id}{$specific}{value} = $data{$type}{$Id}{$specific}{value} / $weight;
        }

        # Averages to rate
        if ( defined $data{$type}{$Id}{$specific}{value} && defined $average_to_rate{$specific} && $interval ) {
          $data{$type}{$Id}{$specific}{value} = $data{$type}{$Id}{$specific}{value} / $interval;
        }

      }
    }
  }

  $schemaName     = 'kDiskIoStats';
  $valueType      = 'doubleValue';
  $rollupFunction = 'average';

  %name_and_metric = (
    'utilization' => 'kPctUtilization',
  );

  foreach my $saveId (@Ids){
    $Id = $saveId;
    foreach my $name0 (keys %name_and_metric){
      $specific = $name0;
      $metricName = $name_and_metric{$specific};
      actualize_performance_hash();
    }
    make_typical_totals();
  }

  #-------------------------------------------------------------------
  # PERFORMANCE: PORT
  #-------------------------------------------------------------------
  debug_message("STARTING PORT SECTION");

  $type = 'PORT';

  $valueType = 'doubleValue';
  $schemaName = 'kInterfaceStats';
  $rollupFunction = 'kAverage';

  @Ids = generate_ids_from_schemaName_interface($schemaName);
  %name_and_metric = (
    'write_packet'  =>  'kRxPackets',
    'write'         =>  'kRxBytes',
    'write_error'   =>  'kRxErrors',
    'read_packet'   =>  'kTxPackets',
    'read'          =>  'kTxBytes',
    'read_error'    =>  'kTxErrors',
  );

  %metric_and_name = reverse %name_and_metric;
  @metric_names    = keys %metric_and_name;

  $metricNames = join(",", sort @metric_names);

  $url_v2_timeseries = "${url_v2}/v2/stats/time-series-stats";

  foreach my $ID (@Ids) {
    $Id = $ID;

    $query  = "?";
    $query .= "entityId=$Id";
    $query .= "&schemaName=$schemaName";
    $query .= "&metricNames=$metricNames";
    $query .= "&rollupIntervalSecs=30";
    $query .= "&rollupFunction=$rollupFunction";
    $query .= "&startTimeMsecs=$startTimeMsecs";
    $query .= "&endTimeMsecs=$endTimeMsecs";

    my $url = $url_v2_timeseries . $query;

    %local_hash = general_data_request("GET", $url, "HASH");

    load_interface_data_from_timeSeriesStats();

    make_typical_totals();
  }


  sub load_interface_data_from_timeSeriesStats {
    if ( %local_hash && ref($local_hash{"timeSeriesStats"}) eq "ARRAY" ){
      for ( my $i = 0; $i < scalar (@{$local_hash{"timeSeriesStats"}}); $i++ ) {
        # INTERFACE stats: counters
        # Problem: API returns "random" numbers with no connection to rollupInterval
        # NOTE: return are counters
        # Solution: get $oldest_timestamp, run over returned values, find longest interval ~ 300 sec, save its index, use index to
        my $latest_timestamp;
        my $save_index = -2;
        my $save_interval = 300;

        if ( defined $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-1]{"timestampMsecs"} ) {
          $latest_timestamp  = $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-1]{"timestampMsecs"};
        }

        my $current_metric = $local_hash{"timeSeriesStats"}[$i]{"metricName"};
        $specific = $metric_and_name{$current_metric};

        # Cycle for time range
        if ( defined $local_hash{"timeSeriesStats"}[$i]{"dataPoints"} && ref($local_hash{"timeSeriesStats"}[$i]{"dataPoints"}) eq "ARRAY" ) {

          for ( my $j = 2; $j < (scalar (@{$local_hash{"timeSeriesStats"}[$i]{"dataPoints"}}) -1 ); $j++ ){
            if ( defined $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-$j]{"timestampMsecs"} ) {
              my $local_interval = $latest_timestamp - $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-$j]{"timestampMsecs"};


              if ( $local_interval >= 300000 ) {
                $save_interval = $local_interval / 1000;
                last;
              }
              $save_index = -$j;
            }
          }

          if ( defined $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-1]{"doubleValue"} ) {
            if (defined $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[$save_index]{"doubleValue"} and defined $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-1]{"doubleValue"}){
              $data{$type}{$Id}{$specific}{value}  = $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[-1]{"doubleValue"};
              $data{$type}{$Id}{$specific}{value} -= $local_hash{"timeSeriesStats"}[$i]{"dataPoints"}[$save_index]{"doubleValue"};
            }
          }

        }

        ## Average over total number of small intervals
        #if ( defined $data{$type}{$Id}{$specific}{value} && $weight ) {
        #  $data{$type}{$Id}{$specific}{value} = $data{$type}{$Id}{$specific}{value} / $weight;
        #}

        # all to rate
        if ( defined $data{$type}{$Id}{$specific}{value} && $save_interval ) {
          $data{$type}{$Id}{$specific}{value} = $data{$type}{$Id}{$specific}{value}/$save_interval;
        }

      }
    }

  }

  #=========================================================================================
  # CAPACITY
  #=========================================================================================
  debug_message("STARTING CAPACITY SECTION");

  # CAPACITY: POOL:
  debug_message("STARTING CAPACITY: POOL SECTION");

  $query = $baseUr . '/public/stats/viewBoxes?';#msecsBeforeCurrentTimeToCompare=604800000';
  my %wild_hash = general_hash_request('GET', $query);

  $type = 'POOL';

  my $Ti = 1024*1024*1024*1024;

  my %name_and_codename = (

    'capacity_data_in'               => 'dataInBytes',
    'capacity_data_written'          => 'dataWrittenBytes',
    'capacity_storage_consumed'      => 'storageConsumedBytes',
    'capacity_used'                  => 'storageConsumedBytes',
    'capacity_physical_usage'        => 'totalPhysicalUsageBytes',
    'capacity_local_physical_usage'  => 'localTotalPhysicalUsageBytes',
    'capacity_logical_usage'         => 'totalLogicalUsageBytes',

    'capacity_total'        => 'localTotalPhysicalUsageBytes',
    'capacity_real'         => 'totalPhysicalUsageBytes',
  );

  # NOTE: Default pool is not in here
  if (defined $wild_hash{statsList} && ref($wild_hash{statsList}) eq "ARRAY" ){
    for (my $i = 0; $i < (scalar @{$wild_hash{statsList}}); $i++){
      $Id = $wild_hash{statsList}[$i]{id};
      $data{$type}{$Id}{Id}{value}=$Id;
      $data{$type}{$Id}{Name}{value}=$wild_hash{statsList}[$i]{name};

      foreach my $name (keys %name_and_codename){

        my $value_name = $name_and_codename{$name};

        if (defined $wild_hash{statsList}[$i]{stats}{$value_name} && $wild_hash{statsList}[$i]{stats}{$value_name}){
          $data{$type}{$Id}{$name}{value} = $wild_hash{statsList}[$i]{stats}{$value_name};
          #$data{$type}{$Id}{$name}{value} /= $Ti;
        }
        else {
          # Tested: pools with 0 value returned undef
          $data{$type}{$Id}{$name}{value} = 0;
        }
        if (! defined $data{$type}{$Id}{$name}{value}){
          $data{$type}{$Id}{$name}{value} = 0;
        }

        if ( $TEST_DATA_IN_GUI ) {
          $data{$type}{$Id}{$name}{value} = 1000000000;
        }

      }

      if (! $data{$type}{$Id}{capacity_data_written}{value} == 0){
        $data{$type}{$Id}{capacity_data_reduction}{value} = divide($data{$type}{$Id}{capacity_logical_usage}{value}, $data{$type}{$Id}{capacity_data_written}{value});
      }

    }

    if (defined $data{$type}{$Id}{capacity_total}{value} && defined $data{$type}{$Id}{capacity_used}{value} ){
      $data{$type}{$Id}{capacity_free}{value} = $data{$type}{$Id}{capacity_total}{value} - $data{$type}{$Id}{capacity_used}{value};
    }

  }


  #-----------------------------------------------------------------------------------------
  # CAPACITY: CLUSTER
  #-----------------------------------------------------------------------------------------
  $query = $baseUr . '/public/cluster?fetchStats=true&fetchTimeSeriesSchema=true';
  %wild_hash = general_hash_request('GET', $query);
  $type = 'CLUSTER';

  $specific = 'localUsagePerfStats';
  %name_and_codename = (
    'capacity_total'  => 'physicalCapacityBytes',
    'capacity_used'   => 'totalPhysicalUsageBytes',
    'capacity_real'   => 'totalPhysicalUsageBytes',
  );

  foreach my $name (keys %name_and_codename){
    my $value_name = $name_and_codename{$name};
    $Id = $cluster_id;
    if (defined $wild_hash{stats}{usagePerfStats}{$value_name}){
      $data{$type}{$Id}{$name}{value} = $wild_hash{stats}{usagePerfStats}{$value_name};
    }
  }
  if (defined $data{$type}{$Id}{capacity_total}{value} and defined $data{$type}{$Id}{capacity_used}{value}){
    $data{$type}{$Id}{capacity_free}{value} = $data{$type}{$Id}{capacity_total}{value} - $data{$type}{$Id}{capacity_used}{value};

    $data{$type}{$Id}{capacity_used_percent}{value} = divide( $data{$type}{$Id}{capacity_used}{value}, $data{$type}{$Id}{capacity_total}{value} );
    $data{$type}{$Id}{capacity_used_percent}{value} *= 100;

  }

  #-----------------------------------------------------------------------------------------
  # CAPACITY: CLUSTER
  #-----------------------------------------------------------------------------------------
  debug_message("STARTING CAPACITY: CLUSTER SECTION");

  $query = $baseUr . '/public/cluster?fetchStats=true&fetchTimeSeriesSchema=true';

  %wild_hash = general_hash_request('GET', $query);
  $type = 'CLUSTER';
  $specific = 'dataUsageStats';
  %name_and_codename = (
    'capacity_data_in'           => 'dataInBytes',
    'capacity_data_written'      => 'dataWrittenBytes',
    'capacity_storage_consumed'  => 'localTotalPhysicalUsageBytes',
    'capacity_logical_usage'     => 'totalLogicalUsageBytes',
  );

  foreach my $name (keys %name_and_codename){
    my $value_name = $name_and_codename{$name};
    $data{$type}{$cluster_id}{$name}{value} = $wild_hash{stats}{$specific}{$value_name};
  }
  if (defined $wild_hash{stats}{dataReductionRatio}){
    $data{$type}{$cluster_id}{capacity_data_reduction}{value} = $wild_hash{stats}{dataReductionRatio};
  }

}

sub configuration_data {

  # ----------------------------------------------------------------------------------------
  # INITIAL CODE NAMES
  # ----------------------------------------------------------------------------------------
  my %wild_hash;
  my $query;
  my %capacity_hash;
  my $section;
  my %name_and_codename;
  my %section_name_path;

  my $process_num = 5;
  $token = generate_new_token();

  infolog("COLLECTING [1/$process_num] - POOL SECTION");
  collect_pool();

  infolog("COLLECTING [2/$process_num] - PORT SECTION");
  collect_port();

  infolog("COLLECTING [3/$process_num] - CLUSTER SECTION");
  collect_cluster();

  my %nodeID_nodeName;
  infolog("COLLECTING [4/$process_num] - NODE SECTION");
  collect_nodes();

  # node names for device configuration table
  my @node_names = ();
  for my $i (keys @{$conf_hash{'NODE'}}) {
    push (@node_names, $conf_hash{'NODE'}[$i]{"Name"});
  }
  $conf_hash{'CLUSTER'}[0]{'node_name'} = join(',', @node_names);


  infolog("COLLECTING [5/$process_num] - DRIVE SECTION");
  collect_drives();

  # CHANGE CONF_HASH = type => $number => data
  # TO     CONF      = hash_key        => data
  # where hash_key is made from $storage_uuid."$type".$ID

  # !!! LATER: CHANGE THE FOLLOWING PART:
  # better compatibility with changes in STOR

  # TODO: CLUSTER == DEVICE

  # %rename_section_metric
  # contains all collected configuration data
  # provides possibility to monitor and filter usage

  # PROVIDES:
  # 1. available metrics
  # 2. untracked metrics
  # 3. filter on saved materics + renaming space
  my %rename_section_metric = (
    'DRIVE' => {
      'ID'    => 'ID',
      'Name'  => 'Name',

      'maxUsableCapacity_TB' => '',
      'storageTier' => '',
      'maxPhysicalCapacity_TB' => '',
      'nodeName' => '',
      'maxUsableCapacityBytes' => '',
      'nodeID' => '',
    },
    'NODE' => {
      'Name'          => 'Name',
      'SWversion'     => 'sw_version',
      'Type'          => 'node_type',
      'DiskCount'     => 'disk_count',
      'RemovalState'  => 'removal_state',
      'MaxCapacity'   => 'capacity_node_total',
      'ID'            => 'ID',
      'IP'            => 'IP',

      'NodeIncarnationID' => '',
      'ProductModel' => '',
      'ClusterPartitionName' => '',
      'ChassisName' => '',
      'ChassisLocation' => '',
      'ChassisId' => '',
      'MarkedForRemoval' => '',
      'ClusterPartitionID' => '',
      'Hostname' => '',
      'ChassisSerial' => '',
    },
    'POOL' => {
      'Name'                    => 'Name',
      'ID'                      => 'ID',

      'capacity_total'          => 'capacity_total',
      'data_in_B'               => 'capacity_data_in',
      'physical_usage_B'        => 'capacity_physical_usage',
      'data_written_B'          => 'capacity_data_written',
      'storage_consumed_B'      => 'capacity_storage_consumed',
      'used_B'                  => 'capacity_used',
      #'storage_consumed'        => 'capacity_storage_consumed',
      'logical_usage_B'         => 'capacity_logical_usage',
      'local_physical_usage_B'  => 'capacity_local_physical_usage',
    },
    'CLUSTER' => {
      'ID' => 'ID',
      'Name'                  => 'Name',
      'node_name'             => 'node_name',
      'MetadataFree'          => 'metadata_available',
      'MetadataUsed'          => 'metadata_used',
      'ChassisCount'          => 'chassis_count',
      'NodeCount'             => 'node_count',
      'Model'                 => 'model',

      'capacity_used'         => 'capacity_used',
      'capacity_free'         => 'capacity_free',
      'capacity_used_percent' => 'capacity_used_percent',

      'version'               => 'version',
      'model'                 => 'model',

      'CurrentOperation' => '',
      'Type' => '',
      'capacity_total' => '',
      'incarnationID' => '',
      'IPs' => '',
      'SWversion' => '',
      'FaultTolerance' => '',
      'TargetSWversion' => '',
    },
    'PORT' => {
      'Name'  => 'Name',
      'ID'    => 'ID',
    },
  );

  my $reveal_conf_names = 0;

  for $type (keys %conf_hash){

    # REVEAL
    if ( $reveal_conf_names ) { print "$type \n"; }

    for (my $i=0; $i<scalar @{$conf_hash{$type}}; $i++){

      # !!! TODO: CHANGE [$i] to {$Id}, ARRAY IS CRAP!
      $Id = $conf_hash{$type}[$i]{"ID"};

      # KEY
      my $hash_key = Xormon::create_hash_key($storage_uuid."$type".$Id);
      $hash_key = lc($hash_key);

      for my $name (keys %{$conf_hash{$type}[$i]}){
        my $rename = $name;
        # REVEAL
        if ( $reveal_conf_names ) { print "\'$name\' => \'\',\n"; }

        # RENAME, ONLY RENAMED WILL BE USED
        # = usage filter
        if ( defined $rename_section_metric{$type}{$name} && $rename_section_metric{$type}{$name}){
          $rename = $rename_section_metric{$type}{$name};
        }
        elsif ( ! defined $rename_section_metric{$type}{$name} ) {
          #print "UNTRACKED CONFIGURATION NAME: $type $name";
          #$name = $name
          next;
        }
        else {
          next;
          #$name = $name
        }

        # RELOAD DATA TO NG FORMAT
        $conf{$hash_key}{$rename} =  $conf_hash{$type}[$i]{$name};

        if ( $TEST_DATA_IN_GUI && !($name eq "Name" || $name eq "ID" || $name eq "name") ) {
          $conf{$hash_key}{$rename} =  1;
        }

        if ( $type eq "CLUSTER" ) {
          $conf{$storage_uuid}{$rename} =  $conf_hash{$type}[$i]{$name};

          if ( $name eq "Name") {
            $conf{$storage_uuid}{"name"} = $conf_hash{$type}[$i]{"Name"};
          }

          if ( $TEST_DATA_IN_GUI && !($name eq "Name" || $name eq "ID" || $name eq "name") ) {
            $conf{$storage_uuid}{$rename} =  1;
          }

        }

      }
    }

    # REVEAL
    if ( $reveal_conf_names ) { print "\n\n"; }

  }

  return \%conf;
}

sub collect_pool {
  my @raw_pool_conf = general_array_request('GET', generate_query('viewBoxes'));
  $type = 'POOL';
  %section_name_path = (
    'Configuration' => {
      'Name'  => 'name',
      'ID'    => 'id',
    }
  );

  # translation from pool id to array number
  my %poolID_number;
  my $local_id;

  for (my $i=0; $i < scalar(@raw_pool_conf); $i++){

    foreach $section (keys %section_name_path){

      foreach $name (keys %{$section_name_path{$section}}){

        my $path = $section_name_path{$section}{$name};
        $conf_hash{$type}[$i]{$name} = $raw_pool_conf[$i]{$path};

        if ($name eq "ID"){
          $local_id = $raw_pool_conf[$i]{$path};
          $poolID_number{$local_id} = $i;
        }

      }
    }
  }


  # CAPACITY: POOL:
  $query = $baseUr . '/public/stats/viewBoxes';#msecsBeforeCurrentTimeToCompare=604800000';

  my %raw_pool = general_hash_request('GET', $query);

  ##!!!TEST
  #for ( my $x = 0 ; $x<1000; $x++) {
  #  %raw_pool = general_hash_request('GET', $query);
  #}

  $type = 'POOL';

  my $Ti = 1;#1024*1024*1024*1024;
  %name_and_codename = (
    'data_in_B'               => 'dataInBytes',
    'data_written_B'          => 'dataWrittenBytes',
    'storage_consumed_B'      => 'storageConsumedBytes',
    'used_B'                  => 'localTotalPhysicalUsageBytes',
    'capacity_used'           => 'localTotalPhysicalUsageBytes',
    'physical_usage_B'        => 'totalPhysicalUsageBytes',
    'local_physical_usage_B'  => 'localTotalPhysicalUsageBytes',
    'logical_usage_B'         => 'totalLogicalUsageBytes',
  );

  my $array_number;

  if ( defined \@{$raw_pool{statsList}}){
    for (my $i = 0; $i < (scalar @{$raw_pool{statsList}}); $i++){

      $Id = $raw_pool{statsList}[$i]{id};

      if ($poolID_number{$Id}){

        $array_number = $poolID_number{$Id};

        undef $poolID_number{$Id};

        foreach my $name (keys %name_and_codename){

          my $value_name = $name_and_codename{$name};
          if (defined $raw_pool{statsList}[$i]{stats}{$value_name}){
            $conf_hash{$type}[$array_number]{$name} = $raw_pool{statsList}[$i]{stats}{$value_name};
            #$conf_hash{$type}[$array_number]{$name} /= $Ti;
          }
          else {
            $conf_hash{$type}[$array_number]{$name} = 0;
          }

          if (! defined $conf_hash{$type}[$array_number]{$name}){
            $conf_hash{$type}[$array_number]{$name} = 0;
          }

          if ( $TEST_DATA_IN_GUI ) {
             $conf_hash{$type}[$array_number]{$name} = 1;
          }

        }
        if (! $conf_hash{$type}[$array_number]{data_written_B} == 0){
          $conf_hash{$type}[$array_number]{data_reduction_B} = $conf_hash{$type}[$array_number]{logical_usage_B} / $conf_hash{$type}[$array_number]{data_written_B};
        }
      }
    }
  }
  else {
    infolog("POOL CONFIGURATION STATS are incomplete! (NOTE: could be licence)");
  }

  foreach my $rest_id (keys %poolID_number){

    if (defined $poolID_number{$rest_id}){
      $array_number = $poolID_number{$rest_id};
      foreach my $name (keys %name_and_codename){

        $conf_hash{$type}[$array_number]{$name} = 0;

      }
    }
  }
}

sub collect_port {
  $type = 'PORT';
  $value_type = 'doubleValue';
  $schemaName = 'kInterfaceStats';
  $rollupFunction = 'average';
  my @Ids = generate_ids_from_schemaName_interface($schemaName);
  for (my $i=0; $i < scalar(@Ids); $i++){

    my $id = $Ids[$i];
    my $name = $id;
    $id =~ s/[\(\)\.]//g;
    $conf_hash{$type}[$i]{ID}=$id;
    $conf_hash{$type}[$i]{Name}=$name;
  }
}

sub collect_cluster {
  # CONFIGURATION: CLUSTER: INFORMATION

  my %raw_cluster = general_hash_request('GET', generate_query('cluster'));
  $type = 'CLUSTER';
  %section_name_path = (
    'Configuration' => {
      'Name'              => 'name',
      'IPs'               => 'nodeIps',
      'ID'                => 'id',
      'incarnationID'     => 'incarnationId',
      'SWversion'         => 'clusterSoftwareVersion',
      'TargetSWversion'   => 'targetSoftwareVersion',
      'Type'              => 'clusterType',
      'FaultTolerance'    => 'faultToleranceLevel',
      'CurrentOperation'  => 'currentOperation',
      'ChassisCount'      => 'chassisCount',
      'NodeCount'         => 'nodeCount',
      'MetadataFree'      => 'availableMetadataSpace',
      'MetadataUsed'      => 'usedMetadataSpacePct',
      'version'           => 'clusterSoftwareVersion',

    }
  );
  #"Storage System Name"
  #"Storage System Model"
  #"Storage System Version"
  # CONFIGURATION: CLUSTER: CONFIGURATION/DISKS/CAPACITIES FROM SCALARS
  foreach $section (keys %section_name_path){

    foreach $name (keys %{$section_name_path{$section}}){

      my $path = $section_name_path{$section}{$name};
      $conf_hash{$type}[0]{$name} = $raw_cluster{$path};
      undef $path;
    }
  }

  $conf_hash{$type}[0]{"Model"} = $raw_cluster{'hardwareInfo'}{'hardwareModels'}[0];
  $conf_hash{$type}[0]{"model"} = $raw_cluster{'hardwareInfo'}{'hardwareModels'}[0];

  my $Ti = 1024*1024*1024*1024;
  if ( defined $conf_hash{$type}[0]{MetadataFree} ) {
    #$conf_hash{$type}[0]{MetadataFree}/=$Ti;
    $conf_hash{$type}[0]{capacity_free} = (sprintf "%.2f",$conf_hash{$type}[0]{MetadataFree});
    $conf_hash{$type}[0]{MetadataFree} = (sprintf "%.2f",$conf_hash{$type}[0]{MetadataFree});
  }
  if ( defined $conf_hash{$type}[0]{MetadataUsed} ) {
    $conf_hash{$type}[0]{capacity_used} = (sprintf "%.0f",$conf_hash{$type}[0]{MetadataUsed});
    $conf_hash{$type}[0]{MetadataUsed} = (sprintf "%.0f",$conf_hash{$type}[0]{MetadataUsed});
  }

  if ( $TEST_DATA_IN_GUI ) {
    $conf_hash{$type}[0]{capacity_used} = 1;
    $conf_hash{$type}[0]{capacity_free} = 0;
  }

  $conf_hash{$type}[0]{capacity_total} = 0;
  $conf_hash{$type}[0]{capacity_total} += $conf_hash{$type}[0]{capacity_used} || 0;
  $conf_hash{$type}[0]{capacity_total} += $conf_hash{$type}[0]{capacity_free} || 0;


  $conf_hash{$type}[0]{capacity_used_percent} = divide( $conf_hash{$type}[0]{capacity_used}, $conf_hash{$type}[0]{capacity_total} );

  if ( defined $conf_hash{$type}[0]{capacity_used_percent} ) {
    $conf_hash{$type}[0]{capacity_used_percent} *= 100;
  }
  ## CONFIGURATION: BACKUP_JOB:
  #$type = 'BACKUP_JOB';
  #$schemaName = 'kMagnetoBackupJobStats';
  #my @raw_backup_job = general_array_request('GET', generate_query('entities?schemaName=kMagnetoBackupJobStats'));
  #
  #for (my $i = 0; $i < scalar(@raw_backup_job); $i++){
  #  $conf_hash{$type}[$i]{"ID"}=$raw_backup_job[$i]{entityId}{entityId}{data}{int64Value};
  #  for (my $j = 0; $j < scalar(@{$raw_backup_job[$i]{attributeVec}}); $j++){
  #    if ($raw_backup_job[$i]{attributeVec}{key} eq "Name"){
  #      $conf_hash{$type}[$i]{"Name"}=$raw_backup_job[$i]{attributeVec}{value}{data}{int64Value};
  #    }
  #  }
  #}
}

sub collect_nodes {
  $type = "NODE";
  my @raw_nodes = general_array_request('GET', generate_query('nodes'));
  %section_name_path = (
    'Configuration' => {
      'ID'                    => 'id',
      'NodeIncarnationID'     => 'nodeIncarnationId',
      'Hostname'              => 'hostName',
      'Name'                  => 'hostName',
      'IP'                    => 'ip',
      'SWversion'             => 'nodeSoftwareVersion',
      'Type'                  => 'nodeType',
      'ClusterPartitionID'    => 'clusterPartitionId',
      'ClusterPartitionName'  => 'clusterPartitionName',
      'RemovalState'          => 'removalState',
      'MarkedForRemoval'      => 'isMarkedForRemoval',
      'ProductModel'          => 'productModel',
      'DiskCount'             => 'diskCount',
      'MaxCapacity'           => 'maxPhysicalCapacityBytes'
    },
  );

  for (my $i=0; $i < scalar(@raw_nodes); $i++){

    my %raw_local = %{$raw_nodes[$i]};
    $type = "NODE";

    # CONFIGURATION: NODE: CONFIGURATION/DISKS/CAPACITIES FROM SIMPLE PATHS
    foreach $section (keys %section_name_path){

      foreach $name (keys %{$section_name_path{$section}}){

        $conf_hash{$type}[$i]{$name} = $raw_local{$section_name_path{$section}{$name}};
      }
    }

    my $nodeID    = $conf_hash{$type}[$i]{ID};
    my $nodeName  = $conf_hash{$type}[$i]{Name};

    $nodeID_nodeName{$nodeID} = $nodeName;

    #$conf_hash{$type}[$i]{MaxCapacity} /= $Ti;
    $conf_hash{$type}[$i]{MaxCapacity}  = (sprintf "%.2f",$conf_hash{$type}[$i]{MaxCapacity});

    $section = "Configuration";
    $conf_hash{$type}[$i]{ChassisName}      = $raw_local{chassisInfo}{chassisName};
    $conf_hash{$type}[$i]{ChassisId}        = $raw_local{chassisInfo}{chassisId};
    $conf_hash{$type}[$i]{ChassisSerial}    = $raw_local{chassisInfo}{chassisSerial};
    $conf_hash{$type}[$i]{ChassisLocation}  = $raw_local{chassisInfo}{location};


    #$specific = 'diskCountByTier';

    #for (my $j = 0; $j < (scalar(@{$raw_local{$specific}})); $j++){

    #  $conf_hash{$type}[$j]{"diskCount"} = $raw_local{$specific}[$j]{diskCount};
    #  $conf_hash{$type}[$j]{"storageTier"} = $raw_local{$specific}[$j]{storageTier};

    #}
    #
    ## CONFIGURATION: NODE: CAPACITIES FROM ARRAY
    #for (my $j = 0; $j < (scalar(@{$raw_local{capacityByTier}})); $j++){

    #  $conf_hash{$type}[$j]{"capacity_B"} = $raw_local{capacityByTier}[$j]{tierMaxPhysicalCapacityBytes}/$Ti;
    #  $conf_hash{$type}[$j]{"capacity_B"} = (sprintf "%.2f",$conf_hash{$type}[$j]{"capacity_B"});
    #  $conf_hash{$type}[$j]{node} = $conf_hash{NODE}[$i]{Name};
    #}
  }
}

sub collect_drives {
  $type = 'DRIVE';
  $specific = 'hardwareInfo';
  $schemaName = 'kDiskIoStats';

  my @Ids = generate_ids_from_schemaName($schemaName);

  $specific = 'diskCountByTier';

  #if (defined \@{$raw_cluster{$specific}}){
  #  for (my $i = 0; $i < scalar(@{$raw_cluster{$specific}}); $i++){
  #    warn Dumper @{$raw_cluster{$specific}};
  #    $conf_hash{$type}[$i]{"ID"}         = $Ids[$i];
  #    $conf_hash{$type}[$i]{"Name"}       = $Ids[$i];
  #    $conf_hash{$type}[$i]{"diskCount"}  = $raw_cluster{$specific}[$i]{diskCount};
  #    $conf_hash{$type}[$i]{"DiskAmount"} = scalar(@{$raw_cluster{$specific}});
  #  }
  #}

  $query = $baseUr . "/disks";

  my @raw_disk = general_array_request('GET', $query);

  debug_message("DRIVE SECTION - MORE");

  for (my $i = 0; $i < scalar(@raw_disk); $i++){

    $conf_hash{$type}[$i]{"ID"}           = $raw_disk[$i]{id};
    $conf_hash{$type}[$i]{"Name"}         = "$raw_disk[$i]{id}-$raw_disk[$i]{storageTier}";

    $conf_hash{$type}[$i]{"storageTier"}  = $raw_disk[$i]{storageTier};
    $conf_hash{$type}[$i]{"nodeID"}       = $raw_disk[$i]{currentNodeId};

    $conf_hash{$type}[$i]{"maxUsableCapacityBytes"}     = $raw_disk[$i]{maxUsableCapacityBytes};
    $conf_hash{$type}[$i]{"maxUsableCapacity_TB"}       = (sprintf "%.2f",$raw_disk[$i]{maxUsableCapacityBytes}/1024/1024/1024/1024);
    $conf_hash{$type}[$i]{"maxPhysicalCapacity_TB"}     = (sprintf "%.2f",$raw_disk[$i]{maxPhysicalCapacityBytes}/1024/1024/1024/1024);

    my $current_node_ID                   = $conf_hash{$type}[$i]{nodeID};
    $conf_hash{$type}[$i]{"nodeName"}     = $nodeID_nodeName{$current_node_ID};
  }
}

sub modify_perf_data {
  # ----------------------------------------------------------------------------------------
  # FINAL DATA MODIFICATIONS
  # ----------------------------------------------------------------------------------------
  my $Mi = 1024*1024;
  my $Ti = 1024*1024*1024*1024;
  # resp_t_r is in 'kReadLatencyUsecs'
  # Possibly rewrite to generally resp_t** => 0.001
  my %value_multiplier = (
  'CLUSTER' => {
    'resp_t' => 0.001, 'resp_t_r' => 0.001, 'resp_t_w'=>0.001
    },
  'DRIVE' => {
    'resp_t' => 0.001, 'resp_t_r' => 0.001, 'resp_t_w'=>0.001
    },
  'NODE' => {
    'resp_t' => 0.001, 'resp_t_r' => 0.001, 'resp_t_w'=>0.001
    },
  'POOL' => {
    'resp_t' => 0.001, 'resp_t_r' => 0.001, 'resp_t_w'=>0.001
    },

  );

  #my %value_divisitor = (
  #
  #);

  for my $name0 (keys %data){
    for my $name1 (keys %{$data{$name0}}){
      for my $name2 (keys %{$data{$name0}{$name1}}){
        if (defined $value_multiplier{$name0}{$name2}){
          if ($data{$name0}{$name1}{$name2}{value}){
            $data{$name0}{$name1}{$name2}{value}*=$value_multiplier{$name0}{$name2};
          }
        }
        #if (defined $value_divisitor{$name0}{$name2}){
        #  $data{$name0}{$name1}{$name2}{value}/=$value_divisitor{$name0}{$name2};
        #}
      }
    }
  }
}

sub save_data {
  $xormon           = shift;
  my $configuration = shift;

  %conf = %{$configuration};

  # Used for data check
  my $has_data = 0;

  # EVENTS AND HS: saved in sub health_status

  # ----------------------------------------------------------------------------------------
  # PERFORMANCE DATA
  # ----------------------------------------------------------------------------------------

  # SELECTION OF DATA TO CREATE CSV FILE
  my %data_to_collect = (
    'CLUSTER' => [
      'io_rate', 'read_io', 'write_io',
      'data_rate', 'write', 'read',
      'resp_t', 'resp_t_r', 'resp_t_w',

      'capacity_data_in', 'capacity_data_written',

      'capacity_total', 'capacity_free', 'capacity_used',
      'capacity_used_percent',

      'capacity_storage_consumed',
      'capacity_logical_usage', 'capacity_data_reduction',
      ],

    'DRIVE' => [
      'io_rate', 'read_io', 'write_io',
      'data_rate', 'write', 'read',
      'resp_t', 'resp_t_r', 'resp_t_w',

      'utilization'
      ],

    'POOL' => [
      'io_rate', 'read_io', 'write_io',
      'data_rate', 'write', 'read',
      'resp_t', 'resp_t_r', 'resp_t_w',

      'capacity_total', 'capacity_free', 'capacity_used',
      'capacity_used_percent',

      'capacity_data_in', 'capacity_data_written', 'capacity_storage_consumed',
      'capacity_logical_usage', 'capacity_data_reduction'
      ],

    'NODE' => [
      'cpu_system', 'memory_used_percent',
      'io_rate', 'read_io', 'write_io',
      'data_rate', 'write', 'read',
      'resp_t', 'resp_t_r', 'resp_t_w'
      ],

    'PORT' => [
      'data_rate', 'read', 'write',
      'packet_rate', 'read_packet', 'write_packet',
      'read_error', 'write_error'
      ],
  );

  # RULES FOR ROUNDING OF VALUES
  my %rounding_rules = (
    'io_rate'=>0, 'read_io'=>0, 'write_io'=>0,
    'data_rate'=>3, 'write'=>3, 'read'=>3,
    'resp_t'=>2, 'resp_t_r'=>2, 'resp_t_w'=>2,
    'sys'=>0, 'memory'=>0, 'utilization'=>0,
    'cpu_system'=>0, 'memory_used_percent'=>0,

    'packet_rate'=>0, 'read_packet'=>0, 'write_packet'=>0,
    'read_error'=>0, 'write_error'=>0,

    'capacity_total' => 0, 'capacity_free' => 0, 'capacity_used' => 0, 'capacity_real' => 0,
    'capacity_used_percent' => 1,

    'capacity_data_in'=>3, 'capacity_data_written'=>3, 'capacity_storage_consumed'=>3,
    'capacity_physical_usage'=>3, 'capacity_logical_usage'=>3, 'capacity_data_reduction' => 1
  );

  #my %rounding_rules = (
  #  'io_rate' => 0, 'read_io' => 0, 'write_io' => 0,
  #  'data_rate' => 0, 'write' => 0, 'read' => 0,
  #  'resp_t' => 3, 'resp_t_r' => 3, 'resp_t_w' => 3,
  #  'sys' => 0, 'memory' => 0, 'utilization' => 0,
  #  'capacity_total' => 0, 'capacity_free' => 0, 'capacity_used' => 0, 'capacity_real' => 0,
  #  'packet_rate' => 0, 'read_packet' => 0, 'write_packet' => 0,
  #  'read_error' => 0, 'write_error' => 0,
  #  'data_in' => 0, 'data_written' => 0, 'storage_consumed' => 0,
  #  'logical_usage' => 0, 'data_reduction' => 1
  #);

  my $time_to_save = $delayed_time;
  infolog("TIME: $time_to_save");

  ##
  ## Following part does not run in production environment.
  ## development purposes, look for $LOAD_RESPONSES and $SAVE_RESPONSE for context
  ##
  #if ( $SAVE_RESPONSE ) {
  #  infolog("DEV: saving timestamp");
  #  Response_saver::write_to_file("/home/lpar2rrd/nextgen/nextgen/server-nest/microservices/cohesity/data/timestamp", $time_to_save);
  #}
  #if ( $LOAD_RESPONSES ) {
  #  infolog("DEV: change timestamp");
  #  my ($query_time, $time_now)  = time_handling();
  #  $time_to_save = $time_now;
  #}

  my $rounded_value;
  my $type_to_save;

  my $rc;
  my $status;

  my $arch;
  my @architecture;

  my @line_data;

  # CREATE CSV FILE STRING
  my @use_parent;
  my $parent = $storage_uuid;

  for $type (keys %data_to_collect) {
    $type_to_save = lc($type);

    local $" = ';';

    # FIRST LINE
    $csv_string = "";

    $csv_string .= "Id;";
    $csv_string .= "time;";
    $csv_string .= "@{$data_to_collect{$type}}".";"."\n";

    # DEVICE == CLUSTER: DUPLICITY, REMOVE LATER !!!
    if ($type_to_save eq "cluster"){
      @line_data = ();

      $arch = {
        "item_id"     => $parent,
        "label"       => $storage_name,
        "hw_type"     => "cohesity",
        "subsystem"   => "device",
        "parents"     => [],
        "hostcfg_id"  => $parent,
      };

      push(@architecture, $arch);

       # DEVICE DATA
      $csv_string = "";
      $csv_string .= "Id;";
      $csv_string .= "time;";
      $csv_string .= "@{$data_to_collect{$type}};\n";

      my $hash_key = $parent;
      push(@line_data, $hash_key);
      push(@line_data, $time_to_save);

      for $Id (sort keys %{$data{$type}}){
        for my $name (@{$data_to_collect{$type}}){

          if (defined $data{$type}{$Id}{$name}{value} ){

            if ( defined $rounding_rules{$name} ) {
              $rounded_value = (sprintf "%.$rounding_rules{$name}f",$data{$type}{$Id}{$name}{value});
            }
            else {
              print "UNKNOWN rounding rule for $name!";
              $rounded_value = (sprintf "%.3f",$data{$type}{$Id}{$name}{value});
            }

            push(@line_data, $rounded_value);

            # Used for data check
            if ( ! $has_data ) {
              $has_data = 1;
            }

          }
          else {
            push(@line_data, "");
          }

        }
      }

       # APPEND LINE TO RESULT
      $csv_string .= "@line_data".";"."\n";

      $rc = $xormon->saveCSV($csv_string,"cohesity","device");

    }

    $csv_string = "";
    $csv_string .= "Id;";
    $csv_string .= "time;";
    $csv_string .= "@{$data_to_collect{$type}}".";"."\n";

    for $Id (sort keys %{$data{$type}}){
      @line_data = ();

      # Modify ID
      # Ports have IP in port id
      $useId = $Id;
      $useId =~ s/[\(\)\.]//g;

      my $hash_key = Xormon::create_hash_key($storage_uuid."$type".$useId);
      $hash_key = lc($hash_key);

      @use_parent = [$parent];

      #if ($type_to_save eq "cluster"){
      #  $hash_key = $cfg->{'$storage_uuid'};
      #}

      $arch = {
        "item_id"     => $hash_key,
        "label"       => $conf{$hash_key}{'Name'},
        "hw_type"     => "cohesity",
        "subsystem"   => "$type_to_save",
        "parents"     => @use_parent,
        "hostcfg_id"  => $parent,
      };

      push(@architecture, $arch);

      # FILL LINE
      push(@line_data, $hash_key);
      push(@line_data, $time_to_save);

      for my $name (@{$data_to_collect{$type}}){
        if (defined $data{$type}{$Id}{$name}{value}){

          if ( defined $rounding_rules{$name} ) {
            $rounded_value = (sprintf "%.$rounding_rules{$name}f",$data{$type}{$Id}{$name}{value});
          }
          else {
            print "UNKNOWN rounding rule for $name!";
            $rounded_value = (sprintf "%.3f",$data{$type}{$Id}{$name}{value});
          }

          push(@line_data, $rounded_value);

          # Used for data check
          if ( ! $has_data ) {
            $has_data = 1;
          }

        }
        else {
          push(@line_data, "");
        }
      }

      # APPEND LINE TO RESULT
      $csv_string .= "@line_data".";"."\n";
    }

    # SAVE DATA PER TYPE
    $type_to_save = lc($type);

    if ( $REVEAL_PERF_CSV ) {
      infolog("$csv_string");
    }

    $rc = $xormon->saveCSV($csv_string, "cohesity", "$type_to_save");
    $csv_string = "";

    # STATUS CODE
    if ($rc eq "201") {
      $status = "successful";
    }
    elsif ($rc eq "401") {
      $status = "unauthorized";
    }
    else {
      $status = "error";
    }

    Xormon::log("Save status for $type_to_save : $status");
  }

  # DATA CHECK
  if ( $has_data ) {
    infolog("HAS DATA: $has_data");
    $xormon->hasData(1);
  }
  else {
    infolog("NO DATA: $has_data");
    $xormon->hasData(0, "No data");
  }

  # SAVE CONFIGURATION
  health_status();
  $xormon->saveConfJSON(\%conf);

  # SAVE ARCHITECTURE
  $xormon->saveArchitecture(\@architecture);

  $xormon->endJob('fetch', $storage_name, $status, $$);

  Xormon::log("Task for ${storage_name} is done. Status: $status");

}

# ----------------------------------------------------------------------------------------
# PROCEDURES AND FUNCTIONS FOR DATA COLLECTING
# ----------------------------------------------------------------------------------------
# HASH LOADING: PROCEDURES AND FUNCTIONS
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  $logger->error("$text");

  return 1;
}

sub general_data_request {
  # token authorization
  my $method          = shift; # GET POST
  my $url             = shift;
  my $requested_type  = shift; # ARRAY HASH ...

  my $result_type = "";
  my $decoded_json;

  ## Response loading: development purposes:
  ## look for $LOAD_RESPONSES and $SAVE_RESPONSE for context
  ## This code loads pre-saved responses from all used url endpoints
  #if ($LOAD_RESPONSES) {
  #  infolog("Loading saved responses...");
  #  my $url_name = Response_saver::url_save_name($url);
  #  infolog("\n /home/lpar2rrd/nextgen/nextgen/server-nest/microservices/cohesity/data/$url_name");
  #  $decoded_json = decode_json(Response_saver::file_to_string("/home/lpar2rrd/nextgen/nextgen/server-nest/microservices/cohesity/data/$url_name"));
  #  
  #  $result_type = ref($decoded_json);
  #  infolog("\n$result_type  $requested_type\n");
  #  if ($requested_type eq $result_type){
  #    if ( $requested_type eq "HASH" ) {
  #      return %{$decoded_json};
  #    }
  #    elsif ( $requested_type eq "ARRAY" ) {
  #      return @{$decoded_json};
  #    }
  #    else {
  #      return ${$decoded_json};
  #    }
  #  }
  #} # End of response loading

  debug_message("$method $url - expected type: $requested_type");

  my $request = HTTP::Request->new( $method => $url );

  $request->header( 'Authorization' => "Bearer $token" );
  $request->header( 'Accept'        => 'application/json' );

  my $user_agent    = LWP::UserAgent->new(
    ssl_opts => {
      SSL_cipher_list => 'DEFAULT:!DH',
      verify_hostname => 0,
      SSL_verify_mode => 0
    }
  );

  my $result = $user_agent->request($request);

  # TEST: return unexpected type:
  #$result->{'_content'}='null';
  #if ($storage_name eq '009_Cohesity' and $type eq 'PORT'){
  #  $result->{'_content'}='null';
  #}

  eval{
    $decoded_json = decode_json($result->{'_content'});
  };

  if($@){
    error($@);

    error("PROBLEM OCCURED during decode_json with url $url!");

    error(" --- RESULT->_content --- ");
    error( Dumper $result->{'_content'});

    error("--- RESULT->_headers --- \n");
    error(Dumper $result->{'_headers'});

    # Anonymization for logger
    undef $result->{'_request'};

    # Always log into output log in case of error
    infolog("\n REQUEST-CLEAN RESPONSE: \n");
    infolog(Dumper $result);

    if ($requested_type eq "HASH") {
      return ();
    }
    elsif($requested_type eq "ARRAY") {
      return [];
    }
    else {
      return '';
    }
  }

  # Work with request limits (Cohesity specific):
  ## number of available requests on reset
  my $x_ratelimit_limit     = $result->{'_headers'}{'x-ratelimit-limit'};
  my $x_ratelimit_remaining = $result->{'_headers'}{'x-ratelimit-remaining'};
  ## unixtime of reset
  my $x_ratelimit_reset     = $result->{'_headers'}{'x-ratelimit-reset'};

  ## check limit, if < 2: sleep until limit reset
  if ( defined $x_ratelimit_limit && defined $x_ratelimit_remaining && defined $x_ratelimit_reset ) {
    check_limit_to_sleep($x_ratelimit_remaining, $x_ratelimit_limit, $x_ratelimit_reset);
  }

  $result_type = ref($decoded_json);

  ## Response saving: development purposes:
  ## This method translates urls to simple names that will be used as file names.
  ## to load these responses use variable $LOAD_REQUESTS
  #if ($SAVE_RESPONSE){
  #  infolog("Saving responses...");
  #  my $url_name = Response_saver::url_save_name($url);
  #
  #  Response_saver::dir_treat("/home/lpar2rrd/nextgen/nextgen/server-nest/microservices/cohesity/data");
  #
  #  $url_file_map{$url_name} = $url_name;
  #  Response_saver::update_url_map(\%url_file_map);
  #  Response_saver::save_response($decoded_json, $url_name, $requested_type, $result_type);
  #
  #} # end of response saving

  if ($requested_type eq $result_type){
    if ( $requested_type eq "HASH" ) {
      return %{$decoded_json};
    }
    elsif ( $requested_type eq "ARRAY" ) {
      return @{$decoded_json};
    }
    else {
      return ${$decoded_json};
    }
  }
  else {
    my $error_message = "";
    $error_message .= "PROBLEM with $url!" ;
    $error_message .= "\n --- RESULT->_content --- \n $result->{'_content'} \n";
    error($error_message);

    # anonymize for logging
    undef $result->{'_request'};

    infolog("\n REQUEST-CLEAN RESPONSE: \n");
    infolog(Dumper $result);

    # Return prevents crash
    if ($requested_type eq "HASH"){
      return ();
    }
    elsif($requested_type eq "ARRAY"){
      return [];
    }
    else{
      return '';
    }
  }
}

sub general_hash_request {
  my $method  = shift;
  my $url     = shift;

  my %decoded_json = general_data_request($method, $url, 'HASH');

  return %decoded_json;
}

sub general_array_request {
  my $method  = shift;
  my $url     = shift;

  my @decoded_json = general_data_request($method, $url, 'ARRAY');

  return @decoded_json;
}

# Query to get Ids from
sub generate_id_query {
  $schemaName = shift;

  my $query_out = $baseUr . '/public/statistics/entities';

  $query_out .= "?schemaName=${schemaName}";

  return $query_out;
}

sub generate_query {
  $type  = shift;

  debug_message("from $type");

  my $query_out = $baseUr . "/public/$type";
  return $query_out;
}

## Makes sum from datahash values
sub data_hash_total {
  my @names = @_;

  debug_message("$type $Id @names");

  my $value;
  foreach my $name (@names){
    if (defined $data{$type}{$Id}{$name}{value}){
      $value += $data{$type}{$Id}{$name}{value};
    }
  }
  return $value;
}

# Makes average from datahash values
sub data_hash_average {
  my @names = @_;

  debug_message("");

  my $value;
  foreach $specific (@names){
    if (defined $data{$type}{$Id}{$specific}{value}){
      $value += $data{$type}{$Id}{$specific}{value}/scalar(@names);
    }
  }
  return $value;
}

# Loads sums of data from datahash to datahash
sub total_data {
  $data{$type}{$Id}{$specific}{timestampMsecs} = $data{$type}{$Id}{$names_to_total[0]}{timestampMsecs};
  $data{$type}{$Id}{$specific}{value} = data_hash_total(@names_to_total);
}

# Loads averages of data from datahash to datahash
sub average_data {
  $data{$type}{$Id}{$specific}{timestampMsecs} = $data{$type}{$Id}{$names_to_total[0]}{timestampMsecs};
  $data{$type}{$Id}{$specific}{value} = data_hash_average(@names_to_total);
}

# Internal names are identical for all types, therefore totals are of typical structure
# task: change to DRY standard
#my @names_to_total;
sub make_typical_totals {
  # describes how to sum metrics
  my %cname_names = (
    'read_io'     => 'io_rate',
    'write_io'    => 'io_rate',
    'read'        => 'data_rate',
    'write'       => 'data_rate',
    'resp_t_r'    => 'resp_t',
    'resp_t_w'    => 'resp_t',
    'read_error'  => 'error_total',
    'write_error' => 'error_total',
  );

  if (defined $data{$type}{$Id}{read_io}){
    $specific = 'io_rate';
    @names_to_total = ('read_io', 'write_io');
    total_data();
  }
  if (defined $data{$type}{$Id}{read}){
    $specific = 'data_rate';
    @names_to_total = ('read', 'write');
    total_data();
  }
  if (defined $data{$type}{$Id}{resp_t_w}){
    $specific = 'resp_t';
    @names_to_total = ('resp_t_r', 'resp_t_w');
    average_data();
  }
  if (defined $data{$type}{$Id}{read_error}){
    $specific = 'error_total';
    @names_to_total = ('read_error', 'write_error');
    total_data();
  }
  if (defined $data{$type}{$Id}{write_packet}){
    $specific = 'packet_rate';
    @names_to_total = ('write_packet', 'read_packet');
    total_data();
  }
}

# Creates specific query
sub generate_performance_query {
  my $url = $baseUr . '/public/statistics/timeSeriesStats';


  $url .= "?entityId=$Id&schemaName=$schemaName&metricName=$metricName&rollupIntervalSecs=$rollupIntervalSecs&rollupFunction=$rollupFunction";
  $url .= '&' .  $query_time;

  return $url;
}

# Procedure to fill datahash with last values from requested hash
sub local_data {

  if ( defined $current_hash{dataPointVec}[2]{data} ) {
    $data{$type}{$Id}{$specific}{metricName} = $current_hash{metricName};
    $data{$type}{$Id}{$specific}{timestampMsecs} = $current_hash{dataPointVec}[-1]{timestampMsecs};
    $data{$type}{$Id}{$specific}{value} = $current_hash{dataPointVec}[-1]{data}{$valueType};
  }

}

## Internal hash loading procedure
sub actualize_performance_hash {

  $query = generate_performance_query();
  %current_hash = general_hash_request('GET', $query);

  local_data();
}

# Internal hash loading function
sub actualize_performance_hash_with_values {
  $specific   = shift;
  $metricName = shift;

  $query = generate_performance_query();
  %current_hash = general_hash_request('GET', $query);

  local_data();
}

# Makes array of Ids from schemaName
sub generate_ids_from_schemaName {
  $schemaName  = shift;

  $query = generate_id_query($schemaName);

  my @Id_hashes = general_array_request('GET', $query);

  @Ids = ();

  for (my $i = 0; $i < scalar(@Id_hashes); $i++){
    push (@Ids, $Id_hashes[$i]{entityId}{entityId}{data}{int64Value});
  }

  return @Ids;
}

# Makes array of Ids from schemaName for PORT
sub generate_ids_from_schemaName_interface {
  $schemaName  = shift;

  $query = generate_id_query($schemaName);
  my @Id_hashes = general_array_request('GET', $query);

  @Ids = ();

  for (my $i = 0; $i < scalar(@Id_hashes); $i++){
    push (@Ids, $Id_hashes[$i]{entityId}{entityId}{data}{stringValue});
  }
  return @Ids;
}

sub infolog {
  my $message = shift || "Not received!";
  $logger->info("$message");
}

sub divide {
  my $up    = shift || "U";
  my $under = shift || "U";

  if (
    $under &&
    ($under ne "U") &&
    ($up ne "U" )
  ) {
    return ( $up / $under )
  }
  else {
    return undef
  }
}

sub debug_message {
  if ( $DEBUG ) {
    my $message = shift;

    my $debug_time = localtime();
    my ($sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst) = localtime();

    $debug_time = sprintf( "%4d-%02d-%02d %02d:%02d:%02d", $year + 1900, $month + 1, $day, $hour, $min, $sec );

    # caller(x)[3]: returns x-th previously called subroutine
    my $function_message = (caller(1))[3] || "MAIN";
    if ( $function_message eq "MAIN" ) {
      $debug_time = "\n$debug_time";
    }

    $message =~ s/main:://g;
    $function_message =~ s/main:://g;

    $logger->info("$function_message - $message");

  }
}

1;
