package ECS;

use strict;
use warnings;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use Xormon;
use JSON qw(encode_json decode_json);
use Test;
use XormonLogger;
use HTTP::Date;

my $debug = 0;
my $ecs_debug = 0; ## set also debug to 1
$ecs_debug = 0 if !$debug;

my $xormon;
my $cfg;
my $logger;
my $act_time;
my $cookie;
my $ip = $ecs_debug ? 'debug' : undef;
my $version;
my $run_conf;
my $vdc_alias;
my $port = 4443;
my ($hostcfg_status, $hostcfg_data) = (1,1);

### hs 1 == OK, 0 == NOK
my $hs_vdc;
my $hs_global = 1;
my $int_max = 2147483647;

my (%perf_data, %conf_data, %perf_data_last_hour);
my ($ecs_once, $vdc_id, $vdc_name, @replic_group, $namespace, @rgids, $replic_group);
my %api_data;
my %influx_data;      ### if version >= 3.6 data for VDC and NODE are colected from influxDB
my $list_conf = list_conf();
my $list_perf = list_perf();
my %device_perf; # data, latency, capacity

sub collect {
  $xormon = shift;
  $cfg    = shift;

  if ( !exists $cfg->{hostcfg_id} ) {
    Xormon::error("Cannot continue, 'hostcfg_id' is not configured!");
    return;
  }

  $logger = XormonLogger->new('ecs', $cfg->{hostcfg_id}, $cfg->{hostalias});
  $act_time = time;
  $cfg->{timestamp} = $act_time;
  my $data = $xormon->getData();
  my $last_conf_run = $data ? $data->{last_conf_run} : $act_time - 3700;
  %perf_data_last_hour = %{$data->{last_hour_data}} if $data && exists $data->{last_hour_data};
  $run_conf = $act_time - $last_conf_run >= 3600 || $debug ? 1 : 0;
  foreach (@{$cfg->{vdc}}) {
    vdc_collect($_);
  }

  $device_perf{capacity_used_percent} = Xormon::isdigit($device_perf{capacity_used}) && exists $device_perf{capacity_used} && Xormon::isdigit($device_perf{capacity_total}) && exists $device_perf{capacity_total} ? $device_perf{capacity_used} / $device_perf{capacity_total} * 100 : undef;
  if ($run_conf) {
    process_conf_data();
    save_conf_data();
    $xormon->saveData({
      last_conf_run => $act_time,
      last_hour_data => \%perf_data
    });
  }

  process_perf_data();

  if ( $hostcfg_status ) {
    $xormon->isConnected(1);
  } else {
    $xormon->isConnected(0, "auth failed");
    return;
  }

  if ( $hostcfg_data ) {
    $xormon->hasData(1);
  } else {
    $xormon->hasData(0, "Unable to get API data");
    return;
  }

  return 'successful';
}

my $set_origin_cookie;

sub vdc_collect {
  my $vdc = shift;
  $hs_vdc = 1;
  $set_origin_cookie = sub {
    if ($ecs_debug) {
      $cookie = 'debug';
      return;
    }

    $cookie = set_cookie($_);
  };

  $set_origin_cookie->();
  if (!$cookie) {
    $hostcfg_status = 0;
    return;
  }

  $vdc_alias = $vdc->{'alias'};
  $version = version();

  if (!$version) {
    $hostcfg_data = 0;
    return;
  }

  influxdata() if $version >= 360 && !%influx_data;
  push(@{$perf_data{VDC}}, get_perf_data());
  push(@{$conf_data{VDC}}, get_conf_data()) if $run_conf;

  logout() if !$ecs_debug;
  $cookie = undef;
  $version = undef;
  %influx_data = ();
}

sub get_perf_data {
  my $vdc_data = {
    vdc => VDC_perf('VDC_m',300) || undef,
    node => NODE_perf('NODE_m',300) || undef
  };

  my ($vdc_h, $node_h);

  if ($run_conf) {
    $vdc_h = VDC_perf('VDC_h',3600) || undef;
    $node_h = NODE_perf('NODE_h',3600) || undef;
    $vdc_data->{'replication_group'} = RG_perf() || undef;
    $vdc_data->{'replication_group_link'} = RGL_perf() || undef;
  } else {
    foreach my $vdc (@{$perf_data_last_hour{VDC}}) {
      next if $vdc->{vdc}[0]{id} ne $vdc_data->{vdc}[0]{id};
      foreach (@{[qw(capacity_total capacity_used capacity_free)]}) {
        my $metric = sub { return $device_perf{$_} + $vdc->{vdc}[0]{$_}; };
        device_metric($_, $vdc->{vdc}[0]{$_}, $metric);
      }

      $vdc_h = $vdc->{vdc};
      $node_h = $vdc->{node};
      $vdc_data->{'replication_group'} = $vdc->{replication_group};
      $vdc_data->{'replication_group_link'} = $vdc->{replication_group_link};
      last;
    }
  }

  my @vdc_hour_metrics = (keys %{$list_perf->{VDC_h}{metric}}, keys %{$list_perf->{VDCCAP}{metric}});
  foreach my $k ( @vdc_hour_metrics) {
    $vdc_data->{vdc}[0]{$k} = $vdc_h->[0]{$k};
  }

  my @node_hour_metrics = (keys %{$list_perf->{NODE_h}{metric}}, keys %{$list_perf->{NODECAP}{metric}});
  if (defined $vdc_data->{node}) {
    foreach my $node_data_h (@$node_h) {
      foreach my $node_data (@{$vdc_data->{node}}) {
        next if $node_data->{id} ne $node_data_h->{id};
        foreach my $k ( @node_hour_metrics) {
          $node_data->{$k} = $node_data_h->{$k};
        }

        last;
      }
    }
  }

  return $vdc_data if exists $perf_data{namespace};

  my ($ns, $bkt);
  if ($run_conf) {
    $ns = NS_perf();
    $bkt = BKT_perf();
  } else {
    $ns = $perf_data_last_hour{namespace};
    $bkt = $perf_data_last_hour{bucket};
  }

  return $vdc_data if !$ns;
  $perf_data{namespace} = $ns;
  $perf_data{bucket} = $bkt;

  return $vdc_data;
}

sub get_conf_data {
  my $vdc_data = {
    vdc => VDC() || undef,
    ALERTS => ALERT() || undef,
    HS => HS() || undef,
    node => NODE() || undef,
    'replication_group' => RG() || undef,
    'replication_group_link' => RGL() || undef
  };

  return $vdc_data if exists $conf_data{namespace};

  my $ns = NS();
  my $bkt = BKT();

  return $vdc_data if !$ns;

  $conf_data{namespace} = $ns;
  $conf_data{bucket} = $bkt;

  return $vdc_data;
}

sub save_conf_data {
  my $st_serial = $cfg->{'hostcfg_id'};
  my %conf_save;
  # my @vdc_status;

  my @health_status = ({
    "item_id" => $st_serial,
    "status" => $hs_global ? "ok" : "error"
  });

  foreach my $vdc (@{$conf_data{VDC}}) {
    next if !defined $vdc->{vdc};
    my $vdc_hash_key = lc(Xormon::create_hash_key($st_serial.'VDC'.$vdc->{vdc}[0]{id}));

    # Glob configuration VDC table
    push(@{$conf_save{$st_serial}{conf}{vdc}}, {
      alias => $vdc->{vdc}[0]{name},
      name => $vdc->{vdc}[0]{name_real},
      serial => $vdc->{vdc}[0]{vdcSerial},
    });

    # Health Status VDC
    # push(@vdc_status, {
    #   name => $vdc->{vdc}[0]{name},
    #   status => $vdc->{vdc}[0]{vdcStatus} ? "ok" : "error"
    # });

    # VDC configuration & health status
    $conf_save{$vdc_hash_key} = {
      hs => encode_json({
        hs => $vdc->{HS},
        alerts => $vdc->{ALERTS},
      }),
      conf => {
        vdc => {
          Alias => $vdc->{vdc}[0]{name},
          Name => $vdc->{vdc}[0]{name_real},
          Serial => $vdc->{vdc}[0]{vdcSerial},
        },
        node => []
      }
    };

    push(@health_status, {
      item_id => $vdc_hash_key,
      status => $vdc->{vdc}[0]{vdcStatus} ? "ok" : "error"
    });

    # Node
    foreach my $node (@{$vdc->{node}}) {
      my $hash_key = lc(Xormon::create_hash_key($st_serial.'NODE'.$node->{id}));
      $conf_save{$hash_key} = {
        TYPE => 'node',
        name => $node->{name},
        label => "$vdc->{vdc}[0]{name}-$node->{name}",
        serial_number => $vdc->{vdc}[0]{vdcSerial},
        host => $node->{ip},
        version => $node->{nodeEcsVer}
      };

      # Glob configuration NODE table
      push(@{$conf_save{$st_serial}{conf}{node}}, {
        name => $node->{name},
        vdc => $vdc->{vdc}[0]{name},
        host => $node->{ip},
        version => $node->{nodeEcsVer}
      });

      # VDC configuration NODE table
      push(@{$conf_save{$vdc_hash_key}{conf}{node}}, {
        name => $node->{name},
        host => $node->{ip},
        version => $node->{nodeEcsVer}
      });
    }
  }

  $xormon->saveConfJSON(\%conf_save);
  $xormon->saveStatus(\@health_status);
}

sub process_conf_data {
  my @architecture;
  my $st_serial = $cfg->{'hostcfg_id'};

  # device
  push(@architecture, {
    "item_id"     => $st_serial,
    "label"       => $cfg->{'hostalias'},
    "hw_type"     => "ecs",
    "subsystem"   => "device",
    "hostcfg_id"  => $st_serial,
    "parents"     => [],
  });

  # Level 1
  foreach my $type (@{[qw( namespace bucket )]}) {
    foreach (@{$conf_data{$type}}) {
      my $hash_key = lc(Xormon::create_hash_key($st_serial.uc($type).$_->{id}));
      push(@architecture, {
        "item_id" => $hash_key,
        "label" => $_->{name},
        "hw_type" => "ecs",
        "subsystem" => $type,
        "hostcfg_id" => $st_serial,
        "parents" => [$st_serial],
      });
    }
  }

  # VDC + Level 2
  foreach my $vdc_obj (@{$conf_data{VDC}}) {
    my $vdc = $vdc_obj->{vdc}[0];
    my $vdc_key = lc(Xormon::create_hash_key($st_serial . "VDC" . $vdc->{id}));
    push(@architecture, {
      "item_id" => $vdc_key,
      "label" => $vdc->{name},
      "hw_type" => "ecs",
      "subsystem" => "vdc",
      "hostcfg_id" => $st_serial,
      "parents" => [$st_serial],
    });

    foreach my $type (@{['node', 'replication_group', 'replication_group_link']}) {
      foreach my $obj (@{$vdc_obj->{$type}}) {
        my $key = lc(Xormon::create_hash_key($st_serial . uc($type) . $obj->{id}));
        push(@architecture, {
          "item_id" => $key,
          "label" => $obj->{name},
          "hw_type" => "ecs",
          "subsystem" => $type,
          "hostcfg_id" => $st_serial,
          "parents" => [$st_serial, $vdc_key],
        });
      }
    }
  }

  $xormon->saveArchitecture(\@architecture);
}

sub process_perf_data {
  my $st_serial = $cfg->{'hostcfg_id'};

  # !refactor shortcuts to full name
  my $types = {
    bucket => 'BKT',
    namespace => 'NS',
    node => 'NODE',
    replication_group => 'RG',
    replication_group_link => 'RGL',
    vdc => 'VDC',
    device => 'device'
  };

  my $data2csv = sub {
    my($type, $data) = @_;

    return if !$data;

    my $data_out = "id;time";
    foreach ( @{$list_perf->{$types->{$type}}{names}} ) {
      $data_out .= ";$_";
    }

    $data_out .= "\n";
    foreach (@{$data}) {
      my $hash_key = $type ne 'device' ? lc(Xormon::create_hash_key($st_serial.uc($type).$_->{id})) : $st_serial;
      $data_out .= "$hash_key;$act_time";

      foreach my $metric ( @{$list_perf->{$types->{$type}}{names}} ) {
        $data_out .= defined $_->{$metric} ? ";$_->{$metric}" : ";";
      }

      $data_out .= "\n";
      my $rc = $xormon->saveCSV($data_out,"ecs",$type);
      error("Attempt to save data of type '$type' ended with code: $rc") if $rc ne 201;
    }
  };

  $data2csv->('device', [rounding( \%device_perf, 'device', $list_perf )]);

  foreach my $type (@{[qw( namespace bucket )]}) {
    $data2csv->($type, $perf_data{$type});
  }
  foreach my $vdc (@{$perf_data{VDC}}) {
    foreach my $type (@{[qw( vdc node replication_group replication_group_link )]}) {
      $data2csv->($type, $vdc->{$type});
    }
  }
}

sub influxdata {
  my $type = 'influxdata';
  my $get = sorting('VDC');
  if (!$get) {
    error_debug("$type-VDC");
    return;
  }

  my $call = make_request($type);
  my %node2vdc;         ### mapping for influx db version >= 3.6
  foreach my $vdc (@{$get}) {
    my $get_node = $call->('NODE-CONF', $vdc->{name} );
    next if !$get_node;

    foreach ( @{ $get_node->{node} } ) {
      push(@{$node2vdc{$vdc->{name}}}, $_->{nodeid} );
    }
  }

  if (!%node2vdc) {
    error_debug("$type-mapping");
    return;
  }

  my %data;    ### data for every node
  my @timeList = $run_conf ? (300, 3600) : (300);
  foreach my $sampleT (@timeList) {
    foreach my $db ( @{ $list_perf->{influx}{db}{$sampleT} } ) {
      foreach my $table ( keys %{ $db->{names} } ) {
        ###
        # Because the range function have sample limit an hour, we have to request another 5min.
        # With this sample we can get value for first 5 min samle in an hour range.
        #
        # When requesting range wider then an hour:
        # "error" : "failed to compile query: validation failed: range 1h5m0s (start: 2021-12-09T10:04:27Z; stop: 2021-12-09T11:09:27Z) is longer than allowed 1 hour"
        #
        # Maybe there is way, how to set influx return more wide range?
        ###
        my ( %request, $post, %dataSec, $iV, $iM, $iN );
        if ( $sampleT == 3600 ) {
          %request = ( query => "from(bucket:\"$db->{db}\") |> range(start: -65m, stop: -60m) |> filter(fn: (r) => r._measurement == \"$table\")" );
          $post = make_request($type, 'POST')->(\%request, 1, $table, 1);
          if ($post) {
            ( $iV, $iM, $iN ) = indexOfcolumn( \@{ $post->{Series}[0]{Columns} } );
            foreach ( @{ $post->{Series}[0]{Values} } ) {
              next if !$_->[$iN] || !$_->[$iM];
              $dataSec{ $_->[$iN] }{ $_->[$iM] } = $_->[$iV];
            }
          }
        }

        my $qStart = $sampleT / 60;
        $qStart += 5 if $sampleT == 300;
        $qStart .= "m";

        %request = ( query => "from(bucket:\"$db->{db}\") |> range(start: -$qStart) |> filter(fn: (r) => r._measurement == \"$table\")" );
        $post = make_request($type, 'POST')->(\%request, 1, $table);

        next if !$post;

        ### Latency && Errors have specific data colection. Detailed explenation in function latenceData.
        if ( $table =~ m/^statDataHead_performance_internal_(error|latency)$/ ) {
          specialData( $post, \%data, $table );    ### saving to data hash
          next;
        }

        next if $sampleT == 3600 && !%dataSec;
        ( $iV, $iM, $iN ) = indexOfcolumn( \@{ $post->{Series}[0]{Columns} } );

        my ( $value, $metric, $nodeid, $counter );

        foreach my $sample ( @{ $post->{Series}[0]{Values} } ) {
          ### diskio has different array size!!!
          if ( @{$sample} != @{ $post->{Series}[0]{Columns} } ) {
            print Dumper \@{$sample} if $debug && !$ecs_debug;
            next;
          }

          ### array contain only non-counter metric
          my $is_counter = ( @{ $db->{names}{$table} } && grep( /^$sample->[$iM]$/, @{ $db->{names}{$table} } ) ) ? 0 : 1;
          if ($is_counter) {
            if ( !$metric || $metric ne $sample->[$iM] ) {
              $metric  = $sample->[$iM];
              $nodeid  = $sample->[$iN];
              $counter = ( $sampleT == 300 ) ? $sample->[$iV] : $dataSec{$nodeid}{$metric};
              next if $sampleT == 300;
            }

            $value = $sample->[$iV];
            ### overfllowing counter
            if ( $value >= $counter ) { $value = $value - $counter; }
            else { $value = ( $int_max - $counter ) + $value; }

            $value /= $sampleT if $metric ne "utilization";    ### average per s

            my $metName = "$metric" . "_$table";

            if ( exists $data{$nodeid}{$metName} ) { $data{$nodeid}{$metName} = ( $data{$nodeid}{$metName} + $value ) / 2; }
            else { $data{$nodeid}{$metName} = $value; }

            $counter = $sample->[$iV];
          } else {
            $value  = $sample->[$iV];
            $metric = $sample->[$iM];
            $nodeid = $sample->[$iN];
             my $metName = "$metric" . "_$table";
            $data{$nodeid}{$metName} = $value;
          }
        }
      }
    }
  }

  return if !%data;   ### no data from influx db

  ### our metric
  foreach my $node ( keys %data ) {
    my $nd = $data{$node};

    ### error_statDataHead_performance_internal_error
    my ( $table, $val ) = ( '_statDataHead_performance_internal_error', undef );
    my @agg = (qw( system_errors user_errors ));
    foreach (@agg) {
      my $met = "$_" . "$table";
      $val += $nd->{$met} if exists $nd->{$met};
    }

    $nd->{error_rate_statDataHead_performance_internal_error} = $val if defined $val;

    next if !$run_conf; ### following part only for 1h perf

    ### util_cpu
    ( $table, $val ) = ( '_cpu', undef );
    @agg = (qw( usage_irq usage_nice usage_system usage_user ));
    foreach (@agg) {
      my $met = "$_" . "$table";
      $val += $nd->{$met} if exists $nd->{$met};
    }

    $nd->{util_cpu} = $val if defined $val;

    ### util_net
    if ( exists $nd->{speed_net} && exists $nd->{utilization_net} ) {
      $nd->{util_net} = ( $nd->{utilization_net} / $nd->{speed_net} ) * 100;
      if ( exists $nd->{packets_sent_net} && exists $nd->{packets_recv_net} ) {
        ### read_util_net
        $nd->{read_util_net} = $nd->{packets_sent_net} / ( ( $nd->{packets_sent_net} + $nd->{packets_recv_net} ) / 100 ) * ( $nd->{util_net} / 100 );
        ### write_util_net
        $nd->{write_util_net} = $nd->{packets_recv_net} / ( ( $nd->{packets_sent_net} + $nd->{packets_recv_net} ) / 100 ) * ( $nd->{util_net} / 100 );
      }
    }

    if ( exists $nd->{used_disk} )         { $nd->{used_disk_sp}         = $nd->{used_disk}; }            ### used_disk
    if ( exists $nd->{free_disk} )         { $nd->{free_disk_sp}         = $nd->{free_disk}; }            ### free_disk
    if ( exists $nd->{total_disk} )        { $nd->{total_disk_sp}        = $nd->{total_disk}; }           ### total_disk
    if ( exists $nd->{used_percent_disk} ) { $nd->{used_percent_disk_sp} = $nd->{used_percent_disk}; }    ### used_percent_disk
  }

  ### our metrics naming
  foreach my $vdc ( keys %node2vdc ) {
    foreach my $node ( @{ $node2vdc{$vdc} } ) {
      foreach my $metric ( keys %{ $list_perf->{influx}{naming}{node} } ) {

        my $node_name = $list_perf->{influx}{naming}{node}{$metric};
        $influx_data{$node}{$node_name} = ( exists $data{$node}{$metric} ) ? $data{$node}{$metric} : undef;

        next if $metric =~ m/disk_sp$/;

        my $vdc_n = $list_perf->{influx}{naming}{vdc}{$metric};

        if (exists $data{$node}{$metric}) {
          delete( $influx_data{$vdc}{$vdc_n} ) if exists $influx_data{$vdc}{$vdc_n} && !$influx_data{$vdc}{$vdc_n};

          if ( exists $influx_data{$vdc}{$vdc_n} ) {
            if ($vdc_n =~ m/cpu_|util|percent$|resp/) { $influx_data{$vdc}{$vdc_n} = ( $influx_data{$vdc}{$vdc_n} + $data{$node}{$metric} ) / 2; }
            else { $influx_data{$vdc}{$vdc_n} += $data{$node}{$metric}; }
          } else {
            $influx_data{$vdc}{$vdc_n} = $data{$node}{$metric};
          }
        } elsif ( !$influx_data{$vdc}{$vdc_n} ) {
          $influx_data{$vdc}{$vdc_n} = undef;
        }
      }
    }
  }
}

sub specialData {
  ### FOR resp_t
  # API doesn`t return latency counter. It returns bucket list of time ranges representing latency range in which ecs job is. It is counting jobs not specific resp_t value.
  # It needs to be devided by process column, becouse _field contain min time value of one time bucket range.
  #
  # API example:
  #
  # "0",                                          <== table
  # "2021-12-13T08:52:49.019453262Z",             <== _start
  # "2021-12-13T08:57:49.019453262Z",             <== _stop
  # "2021-12-13T08:53:38Z",                       <== _time
  # "560194034",                                  <== _value
  # "+Inf",                                       <== _field time range min value
  # "statDataHead_performance_internal_latency",  <== _measurement
  # "czes.cz",                                    <== host
  # "ttfb_read",                                  <== id ( it decide if read or wite latency)
  # "e88684a4-b52fed7e2222",                      <== node_id
  # "statDataHead",                               <== process
  # "dashboard"                                   <== tag
  #
  ###

  my ( $post, $data, $table ) = @_;
  my ( $metric, $nodeid, $counter, $bucket );
  my ( $iC, $iB, $iN, $iM, $iT ) = indexOfcolumn( \@{ $post->{Series}[0]{Columns} } );

  if ( $table eq 'statDataHead_performance_internal_latency' ) {
    my @bucketOrder = (qw(
      0.0
      1.0
      4.814963904455889
      23.183877401213103
      111.6295328521717
      537.4921713544796
      2588.0054039994393
      12461.15260479408
      59999.999999999985
      +Inf
    ));

    my %respTable = (
      $bucketOrder[0] => 0.5,
      $bucketOrder[1] => 1.907481952,
      $bucketOrder[2] => 9.184456748,
      $bucketOrder[3] => 44.22282773,
      $bucketOrder[4] => 212.9313193,
      $bucketOrder[5] => 1025.256616,
      $bucketOrder[6] => 4936.5736,
      $bucketOrder[7] => 23769.4237,
      $bucketOrder[8] => 180000,
      $bucketOrder[9] => 600000
    );

    my %proccesed_data;
    foreach ( @{ $post->{Series}[0]{Values} } ) {
      if ( @{$_} != @{ $post->{Series}[0]{Columns} } ) {
        print Dumper \@{$_} if $debug;
        next;
      }

      push(@{$proccesed_data{$_->[$iN]}{$_->[$iM]}{$_->[$iB]}}, {
        'time' => $_->[$iT],
        'counter' => $_->[$iC]
      });
    }

    foreach my $node ( keys %proccesed_data ) {
      foreach my $metric ( keys %{$proccesed_data{$node}} ) {
        my $metricName = ( $metric =~ m/write$/ ) ? 'write' : 'read';
        $metricName .= "_statDataHead_performance_internal_latency";
        my ( $respT, $ioLast ) = ( 0, 0 );
        foreach my $bucketName ( @bucketOrder ) {
          my $bucket = \@{$proccesed_data{$node}{$metric}{$bucketName}};
          next if @{$bucket} != 2;
          my $t1 = str2time($bucket->[0]{time});
          my $t2 = str2time($bucket->[1]{time});
          my ($counter, $counterPrev);
          if ($t1 > $t2) {
            $counter = $bucket->[0]{counter};
            $counterPrev = $$bucket->[1]{counter};
          } else {
            $counter = $bucket->[1]{counter};
            $counterPrev = $bucket->[0]{counter};
          }

          my $ioDiff = ($counterPrev > $counter) ?  $int_max - $counterPrev + $counter : $counter - $counterPrev;
          $respT += ($ioDiff - $ioLast) * $respTable{$bucketName};
          $ioLast = $ioDiff;
        }

        $data->{$node}{$metricName} = ($ioLast) ? $respT / $ioLast : 0; ## ms
      }
    }

    return;
  }

  ### for every metrics which are not devided by 300
  foreach ( @{ $post->{Series}[0]{Values} } ) {
    if ( !$metric || $metric ne $_->[$iB] ) {
      $metric  = $_->[$iB];
      $nodeid  = $_->[$iN];
      $counter = $_->[$iC];
      next;
    }

    my $value = $_->[$iC];

    ### overfllowing counter
    $value = ( $value >= $counter ) ? $value - $counter : $int_max - $counter + $value;

    my $metName = $metric . "_$table";
    $data->{$nodeid}{$metName} += $value;
    $counter = $_->[$iC];
  }
}

sub indexOfcolumn {
  ### When requesting influx query without some Columns. Never know exactly where in array is what.
  my $colm = shift;
  my ( $iV, $iM, $iN, $iId, $iT );

  for ( 0 .. $#{$colm} ) {
    if    ( $colm->[$_] eq '_value' )  { $iV  = $_; }    ### value
    elsif ( $colm->[$_] eq '_field' )  { $iM  = $_; }    ### metric
    elsif ( $colm->[$_] eq 'node_id' ) { $iN  = $_; }    ### nodeid
    elsif ( $colm->[$_] eq 'id' )      { $iId = $_; }    ### id
    elsif ( $colm->[$_] eq '_time' )   { $iT = $_; }    ### time
  }

  return $iV, $iM, $iN, $iId, $iT;
}

sub BKT_perf {
  return if !$namespace;
  my $type = 'BKT';
  my $which = 3600;

  my @data_wrapp;
  my $call = make_request($type);
  foreach my $ns ( @{$namespace} ) {
    my $get = $call->("$type-CONF", $ns->{id} );
    next if !$get;
    my $conf = sorting($type, $get);
    next if !$conf;
    foreach my $vdc (@{$cfg->{vdc}}) {
      $cookie = set_cookie($vdc) if !$ecs_debug;
      next if !$cookie;

      foreach my $bkt_conf ( @{$conf} ) {
        next if !$bkt_conf;
        my $bkt_get = $call->("$type-DATA", $ns->{id}, $bkt_conf->{name},$which);
        if (!$bkt_get) {
          error_debug("$type-$bkt_conf->{name}-$ns->{id}-DATA");
          next;
        }

        my $index = 0;
        $index++ until $conf->[$index] && $conf->[$index]{name} eq $bkt_conf->{name};
        my $data = data_perf( $bkt_get, $type, $which );
        my $bkt = { %$bkt_conf, %$data };
        if (exists $bkt->{hard_quota} && Xormon::isdigit($bkt->{hard_quota}) && $bkt->{hard_quota} ne -1 && exists $bkt->{size} && Xormon::isdigit($bkt->{size}) && $bkt->{size} ne -1) {
          $bkt->{capacity_free} = $bkt->{hard_quota} - $bkt->{size};
        }

        my $ref = rounding($bkt, $type, $list_perf);
        push( @data_wrapp, $ref );
        delete( $conf->[$index] );
      }
    }
  }

  ## return original credentials
  $set_origin_cookie->();
  return \@data_wrapp;
}

sub BKT {
  return if !$namespace;

  my $type = 'BKT';
  my @data_wrapp;
  my $call = make_request($type);
  foreach my $ns ( @{$namespace} ) {
    my $get = $call->( "$type-CONF", $ns->{id} );
    if (!$get) {
      error_debug($type);
      next;
    }

    my $conf = sorting($type, $get);
    if (!$conf) {
      error_debug("$type-conf");
      next;
    }

    foreach my $vdc (@{$cfg->{vdc}}) {
      $cookie = set_cookie( $vdc);
      next if !$cookie;

      foreach my $bkt_conf ( @{$conf} ) {
        next if !$bkt_conf;

        my $bkt_get = $call->("$type-DATA", $ns->{id}, $bkt_conf->{name}, 3600);
        next if !$bkt_get || !exists $bkt_get->{vpool_id};

        my $get_vdc = $call->('NOW');
        $bkt_conf->{vdc} = $get_vdc->{name} if $get_vdc;
        $bkt_conf->{vdcAlias} = $vdc->{alias};

        foreach (@rgids) {
          next if $_->{id} ne $bkt_get->{vpool_id};
          $bkt_conf->{bktRGName} = $_->{name};
          last;
        }

        my $index = 0;
        $index++ until $conf->[$index] && $conf->[$index]{name} eq $bkt_conf->{name};
        my $mergeData = { %$bkt_get, %$bkt_conf };
        my $data = data( $mergeData, $type );
        my $bkt  = {%$bkt_conf, %$data};
        push( @data_wrapp, $bkt );
        delete( $conf->[$index] );
      }
    }
  }

  ## return original credentials
  $set_origin_cookie->();
  return \@data_wrapp;
}

sub NS_perf {
  my $type = 'NS';
  my $which = 3600;

  my $conf = sorting($type);
  if (!$conf) {
    error_debug("$type-conf");
    return;
  }

  ### NS configurations for BKT
  $namespace = $conf;

  my @data_wrapp;
  foreach my $ns_conf ( @{$conf} ) {
    my $get = make_request($type)->("$type-DATA", $ns_conf->{id}, undef, $which);
    next if !$get;

    my $data = data_perf( $get, $type, $which );
    my $ns   = {%$ns_conf, %$data };
    if (exists $ns->{hard_quota} && Xormon::isdigit($ns->{hard_quota}) && $ns->{hard_quota} ne -1 && exists $ns->{size} && Xormon::isdigit($ns->{size}) && $ns->{size} ne -1) {
      $ns->{capacity_free} = $ns->{hard_quota} - $ns->{size};
    }
    
    my $ref  = rounding($ns, $type, $list_perf);
    push( @data_wrapp, $ref );
  }

  return \@data_wrapp;
}

sub NS {
  my $type = 'NS';
  my @data_wrapp;

  my $conf = sorting($type);
  if (!$conf) {
    error_debug($type);
    return;
  }

  ### NS configurations for BKT
  $namespace = $conf;
  foreach my $ns_conf (@{$conf}) {
    my $get = make_request($type)->("$type-DATA", $ns_conf->{id}, undef, 3600);
    if (!$get) {
      error_debug("$type-get");
      next;
    }

    my $mergeData = {%$get, %$ns_conf};
    my $data = data($mergeData, $type);
    my $ns = {%$ns_conf, %$data};
    push(@data_wrapp, $ns);
  }

  return \@data_wrapp;
}

sub RG_perf {
  my $type = 'RG';
  my $which = 3600;

  my $conf = sorting($type);
  if ($conf) {
    error_debug("$type-conf");
    return;
  }

  ### RG configurations for RGL
  $replic_group = $conf;
  my $get = make_request($type)->("$type-DATA", $which);
  if ($get) {
    error_debug("$type-conf");
    return;
  }

  my @data_wrapp;
  foreach my $rg_conf ( @{$conf} ) {
    foreach ( @{ $get->{_embedded}{_instances} } ) {
      $_->{id} =~ s/^Replication_Group://;
      next if $rg_conf->{rgId} ne $_->{id};
      my $data = data_perf($_, $type, $which);
      $data->{vdcId} = $vdc_id;
      my %rg = ( %{$rg_conf}, %{$data} );
      push( @data_wrapp, \%rg );

      last;
    }
  }

  return \@data_wrapp;
}

sub RG {
  my $type = "RG";

  my $conf = sorting($type);
  if (!$conf) {
    error_debug($type);
    return;
  }

  ### RG configurations for RGL
  @replic_group = map {
    {%$_}
  } @{$conf};

  return $conf;
}

sub RGL_perf {
  return if !$replic_group;

  my $type = 'RGL';
  my $which = 3600;

  my @data_wrapp;
  foreach my $rg ( @{$replic_group} ) {
    my $get = make_request($type)->($type, $which, $rg->{rgId});
    next if !$get;
    my $conf = sorting($type, \%{{rgl => \@{ $get->{_embedded}{_instances}}}});
    next if !$conf;
    foreach my $rgl_conf ( @{$conf} ) {
      foreach (@{ $get->{_embedded}{_instances} }) {
        next if $rgl_conf->{rgLnkId} ne $_->{id};
        $rgl_conf->{vdcId}     = $vdc_id;
        $rgl_conf->{rgId}      = $rg->{rgId};
        $rgl_conf->{rgLnkName} = "$rg->{rgName}::$vdc_name-->$rgl_conf->{rgLnkName}";
        my $data = data( $_, $type, $which );
        my %rgl  = ( %{$rgl_conf}, %{$data} );
        push( @data_wrapp, \%rgl );

        last;
      }
    }
  }

  $replic_group = undef;
  return \@data_wrapp;
}


sub RGL {
  return if !@replic_group;

  my $type = 'RGL';
  my @data_wrapp;
  foreach my $rg (@replic_group) {
    my $get = make_request($type)->($type, $rg->{id});
    if (!$get) {
      error_debug($type);
      next;
    }

    my $conf = sorting( $type, \%{ { rgl => \@{ $get->{_embedded}{_instances} } } } );
    if (!$conf) {
      error_debug("$type-conf");
      next;
    }

    foreach my $rgl_conf ( @{$conf} ) {
      foreach ( @{ $get->{_embedded}{_instances} } ) {
        next if $rgl_conf->{rgLnkId} ne $_->{id};
        $rgl_conf->{vdcId}     = $vdc_id;
        $rgl_conf->{rgId}      = $rg->{id};
        $rgl_conf->{rgLnkName} = "$rg->{rgName}::$vdc_name-->$rgl_conf->{rgLnkName}";
        my $data = data( $_, $type );
        my %rgl  = ( %{$rgl_conf}, %{$data} );
        push( @data_wrapp, \%rgl );
        last;
      }
    }
  }

  @replic_group = undef;
  return \@data_wrapp;
}

sub NODE_perf {
  my ($type, $which) = @_;

  my $conf = sorting('NODE', __LINE__);
  if (!$conf) {
    error_debug("$type-conf");
    return;
  }

  my @data_wrapp;
  my $call = make_request($type);
  if ( $version < 360 ) {
    my $get = $call->('NODE-DATA',undef,undef,$which);
    my $get_capacity = $call->('NOW-NODE') if $type eq 'NODE_h';
    if (!$get) {
      error_debug("$type-data");
      return;
    }

    if (!$get_capacity && $type eq 'NODE_h' ) {
      error_debug("$type-capacity-data");
      return;
    }

    foreach my $n_conf ( @{$conf} ) {
      my $node_cap;
      if ($type eq 'NODE_h') {
        foreach ( @{ $get_capacity->{_embedded}{_instances} } ) {
          if ( $n_conf->{id} eq $_->{id} ) {
            $node_cap = $_;
            last;
          }
        }
      }

      foreach ( @{ $get->{_embedded}{_instances} } ) {
        next if $n_conf->{id} ne $_->{id};
        my %node = $type eq 'NODE_m' ? (%{$n_conf}, %{data_perf( $_, $type, $which )}) : (%{$n_conf}, %{data_perf( $_, $type, $which )}, %{data_perf($node_cap, 'NODECAP', $which)});
        push( @data_wrapp, \%node );
        last;
      }
    }

    return \@data_wrapp;
  }

  my $get = $call->('NOW-NODE');
  if (!$get) {
    error_debug("$type-data");
    return;
  }

  foreach my $n_conf ( @{$conf} ) {
    foreach ( @{ $get->{_embedded}{_instances} } ) {
      next if $n_conf->{id} ne $_->{id};
      my %from_influx;
      foreach ( keys %{ $list_perf->{$type}{metric} } ) {
        $from_influx{$_} = $influx_data{ $n_conf->{id} }{$_};
      }

      my %node = ( $type eq 'NODE_m' ) ? ( %{$n_conf}, %{ aggregated_metrics( \%from_influx, $type, $list_perf ) } ) : ( %{$n_conf}, %{ aggregated_metrics( \%from_influx, $type, $list_perf ) }, %{ data_perf( $_, 'NODECAP', $which ) } );
      push( @data_wrapp, \%node );

      last;
    }
  }

  return \@data_wrapp;
}

sub NODE {
  my $type = 'NODE';
  my $call = make_request($type);

  my $get = $call->("$type-DATA");
  if (!$get) {
    error_debug("$type-DATA");
    return;
  }

  my @data_wrapp;
  foreach ( @{ $get->{_embedded}{_instances} } ) {
    my $data = data( $_, $type );
    push(@data_wrapp, $data);
  }

  my $get_version = $call->("$type-VERSION");
  if (!$get_version) {
    error_debug("$type-VERSION");
    return \@data_wrapp;
  }

  foreach my $node (@{$get_version->{node}}) {
    foreach (@data_wrapp) {
      next if $_->{id} ne $node->{nodeid};
      $_->{nodeEcsVer} = $node->{version};
      $_->{ip}         = $node->{ip};
      last;
    }
  }

  return \@data_wrapp;
}

sub HS {
  my $type = 'HS';
  my $call = make_request($type);

  my $get = $call->('NOW');
  if (!$get) {
    error_debug("$type-NOW");
    ($hs_vdc, $hs_global) = (0, 0);
    return;
  }

  my $sorted = data( $get, 'HS' );

  ### with ECS 3.6+ is diffcult to get aletr counters from dashboard. Easier way is selecting alerts which ocurred in the last hour.
  my $get_alerts = $call->('AL');
  if (!$get_alerts) {
    error_debug("$type-AL");
    return $sorted;
  }

  ### If api returns data, init each alert counter to 0.
  foreach (@{['warning alerts', 'error alerts', 'critical alerts']}) {
    $sorted->{$_} = 0;
  }

  foreach ( @{ $get_alerts->{alert} } ) {
    ### alerts which ocurred in the last hour
    if ( $_->{severity} eq "INFO" ) { next; }
    elsif ( $_->{severity} eq "WARNING" )  { $sorted->{'warning alerts'}++; }
    elsif ( $_->{severity} eq "ERROR" )    { $sorted->{'error alerts'}++; $hs_vdc = 0; }
    elsif ( $_->{severity} eq "CRITICAL" ) { $sorted->{'critical alerts'}++; $hs_vdc = 0; }
  }

  $hs_global = 0 if !$hs_vdc;

  return $sorted;
}

sub ALERT {
  my $type = 'AL';
  my @alerts;

  my $get = make_request($type)->($type);
  if (!$get) {
    error_debug($type);
    return;
  }

  foreach (@{ $get->{alert} }) {
    next if $_->{severity} eq "INFO";
    push( @alerts, data( $_, $type ) );
  }

  return \@alerts;
}

sub VDC_perf {
  my ($type, $which) = @_;

  my $conf = sorting('VDC');
  if (!$conf) {
    endJob("No data from VDC");
    return;
  }

  my $config = \%{ $conf->[0] };
  $vdc_name = $config->{name};
  $vdc_id   = $config->{id};

  $config->{name} = $vdc_alias;    ### our vdcname => vdc_alias
  $config->{name} =~ s/^-//;

  my $call = make_request($type);
  if ( $version < 360 ) {
    my $get = $call->('VDC',undef,undef,$which);

    my $get_capacity = $call->('NOW') if $type eq 'VDC_h';

    if (!$get) {
      error_debug("$type-get");
      return;
    }

    if ($type eq 'VDC_h' && !$get_capacity) {
      error_debug( "$type-get-capacity");
      return;
    }

    my @data_wrapp = ( $type eq 'VDC_m' ) ? ( { %{$config}, %{ data_perf( $get, $type, $which ) } } ) : ({%{$config}, %{data_perf($get, $type, $which)}, %{data_perf($get_capacity,'VDCCAP', $which )}});

    return \@data_wrapp;
  }

  my $get = $call->('NOW');
  if (!$get) {
    error_debug("$type-NOW");
    return;
  }

  my %from_influx;
  foreach ( keys %{ $list_perf->{$type}{metric} } ) {
    $from_influx{$_} = $influx_data{$vdc_name}{$_};
  }

  my @data_wrapp = $which == 300 ? ( { %{$config}, %{ aggregated_metrics( \%from_influx, $type, $list_perf ) } } ) : ( { %{$config}, %{ aggregated_metrics( \%from_influx, $type, $list_perf ) }, %{ data_perf( $get, 'VDCCAP', $which ) } } );
  return \@data_wrapp;
}

sub VDC {
  my $type = 'VDC';

  my $conf = sorting($type);
  if (!$conf) {
    endJob("No data from VDC");
    return;
  }

  my $config = \%{ $conf->[0] };
  $vdc_name = $config->{name};
  $vdc_id   = $config->{id};

  $config->{name} = $vdc_alias;    ### our vdcname => vdc_alias
  $config->{name} =~ s/^-//;
  $config->{name_real} = $vdc_name;

  my $call = make_request($type);
  my $get = $call->('NOW');
  if (!$get) {
    endJob("No data for VDC");
    return;
  }

  my @data_wrapp = ( { %{$config}, %{ data( $get, $type ) } } );
  my $get_licence = $call->('LICENSE');
  if ($get_licence) {
    my $licence = data( $get_licence->{license_feature}[0], 'LICENCE' );
    $hs_vdc = 1 if $data_wrapp[0]{vdcBadDisks} && $data_wrapp[0]{vdcBadDisks} > 0 || $data_wrapp[0]{vdcBadNodes} && $data_wrapp[0]{vdcBadNodes} > 0;
    $data_wrapp[0]{vdcSerial} = $licence->{vdcSerial} if exists $licence->{vdcSerial};
    $data_wrapp[0]{vdcStatus} = $hs_vdc;
  } else {
    error_debug("$type-get")
  }

  return \@data_wrapp;
}

sub device_metric {
  my ($metric, $add, $process) = @_;

  my $device_metric = $metric;
  ## for ECS totals TSP is considered as IO
  $device_metric = 'io_rate' if $metric eq 'tps_rate';
  $device_metric =~ s/tps$/io/;

  if (exists $device_perf{$metric} && Xormon::isdigit($device_perf{$metric})) {
    $device_perf{$device_metric} = $process->() if Xormon::isdigit($add);
  } else {
    $device_perf{$device_metric} = $add;
  }
};

# extract required data for element from api request
sub data_perf {
  my ( $get, $type, $which ) = @_;
  my %result;

  foreach my $metric ( keys %{ $list_perf->{$type}{metric} } ) {
    my $api = $list_perf->{$type}{metric}{$metric}{api};
    next if !$api;    ### metric made by us

    my $name = $list_perf->{$type}{metric}{$metric}{name};
    my $value;

    if ( $which == 300 ) {    ### 300s interval VDC & NODE
      if ( $api eq 'transactionErrors' && exists $get->{$api}{all}[0]{Rate} ) {
        $value = $get->{$api}{'all'}[0]{'Rate'};
      } elsif ( ref($get->{$api}) eq 'ARRAY' && scalar @{$get->{$api}} && %{ $get->{$api}[0] } ) { ## ! without single data sample
        ### there are for some reason more then one data shot
        foreach ( keys %{ $get->{$api}[0] } ) {
          next if $_ eq 't';
          $value = exists $get->{$api}[0]{$_} ? $get->{$api}[0]{$_} : undef;
        }
      } else {
        # print Dumper $get->{$api};
        $value = undef;
      }
    } elsif ( $which == 3600 ) {    ### 1h interval
      if ( $type eq 'NS' || $type eq 'BKT' ){ $value = exists $get->{$api} ? $get->{$api} : undef; }
      elsif ( $type eq 'RG' || $type eq 'RGL' ){ $value = exists $get->{$api}[0]{Bandwidth} ? $get->{$api}[0]{Bandwidth} / 3600 : undef; }
      elsif ( $type eq 'VDCCAP' || $type eq 'NODECAP' ){ $value = exists $get->{$api}[0]{Space} ? $get->{$api}[0]{Space} : undef; }
      else { $value = exists $get->{$api}{Avg} ? $get->{$api}{Avg} : undef; }
    }

    $result{$name} = $value;
  }

  return aggregated_metrics(\%result, $type,);
}

sub aggregated_metrics {
  my ($data, $type) = @_;
    my $valid_data = sub {
    my $metrics = shift;
    foreach (@{$metrics}) {
      my $ok = Xormon::isdigit($data->{$_}) && exists $data->{$_} ? 1 : 0;

      return 0 if !$ok;
    }

    return 1;
  };

  if ( $type =~ m/^NODE_h|^VDC_h/ ) {
    my $a = $valid_data->(['memory_used', 'memory_used_percent']);
    my $b = Xormon::isdigit($data->{memory_used});
    my $ok = Xormon::isdigit($data->{memory_used}) && exists $data->{memory_used} ? 1 : 0;
    $data->{disk_data_rate} = $valid_data->(['disk_write', 'disk_read']) ? $data->{disk_write} + $data->{disk_read} : undef;
    $data->{disk_geo_data_rate} = $valid_data->(['disk_geo_write', 'disk_geo_read']) ? $data->{disk_geo_write} + $data->{disk_geo_read} : undef;
    $data->{disk_user_data_rate} = $valid_data->(['disk_user_write', 'disk_user_read']) ? $data->{disk_user_write} + $data->{disk_user_read} : undef;
    $data->{memory_free} = $valid_data->(['memory_used', 'memory_used_percent']) ? ($data->{memory_used} / $data->{memory_used_percent}) * (100 - $data->{memory_used_percent}) : undef;
  }

  if ( $type =~ m/^NODE_m|^VDC_m/ ) {
    my $a = $valid_data->(['memory_used', 'memory_used_percent']);
    my $ok = Xormon::isdigit($data->{memory_used}) && exists $data->{memory_used} ? 1 : 0;
    $data->{data_rate} = $valid_data->(['write', 'read']) ? $data->{write} + $data->{read} : undef;
    $data->{tps_rate} = $valid_data->(['write_tps', 'read_tps']) ? $data->{write_tps} + $data->{read_tps} : undef;
    $data->{resp_t} = Xormon::calculate_weighted_avg( [ { count => $data->{read_tps}, value => $data->{resp_t_r} }, { count => $data->{write_tps}, value => $data->{resp_t_w} } ] );

    if ($type eq 'VDC_m'){
      foreach (@{[qw( read write data_rate read_tps write_tps tps_rate )]}) {
        my $metric = sub { return $device_perf{$_} + $data->{$_}; };
        device_metric($_, $data->{$_}, $metric);
      }

      foreach (@{[qw( resp_t_r resp_t_w resp_t )]}) {
        my $metric = sub { return ($device_perf{$_} + $data->{$_}) / 2; };
        device_metric($_, $data->{$_}, $metric);
      }
    }
  }

  if ( $type =~ m/^NODECAP$|^VDCCAP$/) {
    ### capacity_used_percent
    $data->{capacity_used_percent} = $valid_data->(['capacity_total', 'capacity_used']) ? $data->{capacity_used} / $data->{capacity_total} * 100 : undef;

    if ($type eq "VDCCAP") {
      foreach (@{[qw(capacity_total capacity_used capacity_free)]}) {
        my $metric = sub { return $device_perf{$_} + $data->{$_}; };
        device_metric($_, $data->{$_}, $metric);
      }

      $device_perf{capacity_used_percent} = Xormon::isdigit($device_perf{capacity_used}) && exists $device_perf{capacity_used} && Xormon::isdigit($device_perf{capacity_total}) && exists $device_perf{capacity_total} ? $device_perf{capacity_used} / $device_perf{capacity_total} * 100 : undef;
    }
  }

  if ( $type =~ m/^NS|^BKT/ ) {
    $data->{data_rate} = $valid_data->(['write', 'read']) ? $data->{write} + $data->{read} : undef;

    ## rouding is called in type function
    return $data;
  }

  if ( $type =~ m/^RG/ ) {
    $data->{data_rate} = $valid_data->(['write', 'read']) ? $data->{write} + $data->{read} : undef;
  }

  return rounding( $data, $type, $list_perf );
}

sub data {
  my ( $get, $type ) = @_;
  my %result;

  foreach my $metric ( keys %{ $list_conf->{$type}{metric} } ) {
    next if !exists $list_conf->{$type}{metric}{$metric}{api};
    my $name = $list_conf->{$type}{metric}{$metric}{name};
    my $api  = $list_conf->{$type}{metric}{$metric}{api};

    if ( exists $list_conf->{$type}{metric}{$metric}{level} && exists $get->{$api}) {
      $result{$name} = $get->{$api}[0]{Count};
    } elsif (exists $get->{$api}) {
      $result{$name} = $get->{$api};
    } else {
      $result{$name} = undef;
    }
  }

  if ($type eq 'AL') {
    $result{alertTime} = time2str(str2time($result{alertTime}));
  }

  if ($type eq 'BKT' || $type eq 'NS') {
    ## free & usage
    if (exists $get->{hard_quota} && Xormon::isdigit($get->{hard_quota}) && $get->{hard_quota} ne -1 && exists $get->{size} && Xormon::isdigit($get->{size}) && $get->{size} ne -1) {
      $result{free} = $get->{hard_quota} - $get->{size};
      $result{usage} = $get->{size} / $get->{hard_quota} * 100;
    }
  }

  return rounding( \%result, $type, $list_conf );
}

sub rounding {
  my ( $data, $type, $list ) = @_;

  foreach ( keys %{$data} ) {
    my $value = \$data->{$_};
    my $unit  = $list->{$type}{metric}{$_}{unit};
    my $round = $list->{$type}{metric}{$_}{round};

    if (defined $$value && $$value eq "-1" || !defined $$value) {
      $$value = undef;
      next;
    }

    if ($unit) {
      # unit conversion
      if ( $unit eq "GiB") { $$value *= 2**30; }
      elsif ( $unit eq "TiB" ) { $$value *= 2**30; }
      elsif ( $unit eq "h2s" ) { $$value = $$value / 3600; }
    }

    # rounding
    $$value = sprintf( "%." . $round . "f", $$value ) if defined $round;
  }

  return $data;
}

sub sorting {
  my ($type, $get) = @_;
  my (undef, undef, $line) = caller;

  my (@confProp,$keyData);
  if ( exists $list_conf->{$type}{config} ) {
    foreach ( keys %{$list_conf->{$type}{config}} ) {
      push(@confProp, "$list_conf->{$type}{config}{$_}{api}-$list_conf->{$type}{config}{$_}{name}");
    }
  }

  my %api_props = (
    VDC => {
      api => 'VDC-CONF',
      key => 'vdc',
    },
    NODE => {
      api => 'NODE-VERSION',
      key => 'node',
    },
    RG => {
      api => 'RG-CONF',
      key => 'data_service_vpool',
    },
    NS => {
      api => 'NS-CONF',
      key => 'namespace',
    },
    RGL => {
      key => 'rgl',
    },
    BKT => {
      key => 'object_bucket',
    },
  );

  my $key = $api_props{$type}{key};

  return search($get, $key, \@confProp) if $type eq 'BKT' || $type eq 'RGL';

  my $api = $api_props{$type}{api};
  $get = make_request($type)->($api);

  return search($get, $key, \@confProp) if $get;
}

# return array of ids
sub search {
  my ( $get, $name, $what ) = @_;
  my @result;

  foreach ( @{ $get->{$name} } ) {
    my $local_vdc;    ### this RG is part of VDC for which is runnig Main()
    if ( $name ne "data_service_vpool" ) {    ### All expect RG
      ### find local a VDC for which is this Main() runnig => local == 1
      next if $name eq "vdc" && ( !$_->{'local'} || $_->{'local'} != 1 );
    } else {                                    ### only RG
      push(@rgids, {id => $_->{id}, name => $_->{name}});
      next if @{ $_->{varrayMappings} } <= 1;    ## RG contains more then one pool

      foreach my $pool ( @{ $_->{varrayMappings} } ) {
        ## is there any pool which is part of local vdc
        if ( $pool->{name} eq $vdc_id ) {
          $local_vdc = 1;
          last;
        }
      }

      next if !$local_vdc;
    }
    ### hold id, name, mapping for one item
    my %c_setting;

    foreach my $wanted ( @{$what} ) {
      my ( $api, $our ) = split( /-/, $wanted );
      $c_setting{$our} = $_->{$api};
    }

    ### hash of all items for subsystem, if id was found
    push( @result, \%c_setting ) if $c_setting{ ( split /-/, $what->[0] )[1] };
  }

  return \@result if @result;
}

sub version {
  my $data = make_request('VERSION')->('NODE-VERSION');

  return if !$data;

  foreach (@{$data->{node}}) {
    if ($_->{ip} eq $ip) {
      my ( $v1, $v2, $v3 ) = split( /\./, $_->{version} );
      return "$v1$v2$v3";
    }
  }

  my ( $v1, $v2, $v3 ) = split( /\./, $data->{node}[0]{version} );
  return "$v1$v2$v3";
}

sub set_cookie {
  my (undef, undef, $line) = caller;
  my $vdc = shift;
  logout() if $cookie;

  foreach (@{['node1', 'node2']}) {
    next if !$vdc->{$_};

    $ip = $vdc->{$_};
    my $cookie_new = login($vdc->{username}, $vdc->{password}, 1);
    return $cookie_new if $cookie_new;
  }
}

sub node_ip2vdc_id {
  my $client = shift;
  $cfg = shift;

  my %mapping;
  my $get_vdc_id = sub {
    my $vdc = shift;
    foreach (@{['node1', 'node2']}) {
      $ip = $vdc->{$_};
      $cookie = login($vdc->{username}, $vdc->{password}, 1);
      if (!$cookie) {
        $mapping{$vdc->{alias}}{$_} = undef;
        next
      }

      my $conf = sorting('VDC');
      if (!$conf) {
        $mapping{$vdc->{alias}}{$_} = undef;
        next;
      }

      $mapping{$vdc->{alias}}{$_} = lc(Xormon::create_hash_key($cfg->{hostcfg_id}. "VDC".$conf->[0]{id}));
    }
  };

  foreach (@{$cfg->{vdc}}) {
    $get_vdc_id->($_);
  }

  return \%mapping;
}

sub connection_test {
  my $client = shift;
  $cfg = shift;

  my $test = Test->new($client);

  my @variables = ("hostcfg_id", "hostalias");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }

  $test->addSection("Validation of variables", 1 , "");

  my @ports_state;
  my $code_global = 1;
  foreach my $vdc (@{$cfg->{vdc}}) {
    foreach (@{['node1', 'node2']}) {
      next if !$vdc->{$_};

      $ip = $vdc->{$_};
      my ($code, $text) = Xormon::conntest_tcp($ip,$port);
      $code_global = 0 if !$code;
      push(@ports_state, "$text\n");
    }
  }

  $test->addSection("API host network connection", $code_global, \@ports_state);

  return $test->stop() if !$code_global;

  foreach (@{$cfg->{vdc}}) {
    $cookie = set_cookie($_);
    if (!$cookie) {
      $test->addSection("API authorization $_->{alias}", 0, "Token NOT granted from $ip!");
      continue;
    }

    $test->addSection("API authorization $_->{alias}", 1, "Token granted from $ip");

    my $conf = sorting('VDC');
    if (!$conf) {
      $test->addSection("API data $_->{alias}", 0, "ERROR: Couldn't get configuration data");
      logout();
      continue;
    }

    my $get_licence = make_request('VDC')->('LICENSE');
    if (!$get_licence) {
      $test->addSection("API data $_->{alias}", 0, "ERROR: Couldn't get configuration data");
      logout();
      continue;
    }

    my $licence = data( $get_licence->{license_feature}[0], 'LICENCE' );
    $test->addSection("API data $_->{alias}", 1, [
      "Name  : $conf->[0]{name}\n",
      "Serial   : $licence->{vdcSerial}\n"
    ]);

    logout();
  }

  return $test->stop();
}

sub address {
  my ( $type, $parm1, $parm2, $interval ) = @_;

  my $url = "https://$ip:$port";

  if    ( $type eq 'LOGIN' )        { $url .= '/login'; }
  elsif ( $type eq 'LOGOUT' )       { $url .= '/logout'; }
  elsif ( $type eq "AL" )           { $url .= "/vdc/alerts?acknowledged=false"; }
  elsif ( $type eq 'NOW' )          { $url .= '/dashboard/zones/localzone?dataType=current'; }
  elsif ( $type eq 'VDC' )          { $url .= '/dashboard/zones/localzone'; }
  elsif ( $type eq 'VDC-CONF' )     { $url .= '/object/vdcs/vdc/list'; }
  elsif ( $type eq 'NODE-DATA' )    { $url .= '/dashboard/zones/localzone/nodes'; }
  elsif ( $type eq 'NOW-NODE' )     { $url .= '/dashboard/zones/localzone/nodes?dataType=current'; }
  elsif ( $type eq 'NODE-CONF' )    { $url .= "/vdc/$parm1/nodes"; }
  elsif ( $type eq 'NODE-VERSION' ) { $url .= '/vdc/nodes'; }
  elsif ( $type eq 'RG-CONF' )      { $url .= '/vdc/data-service/vpools'; }
  elsif ( $type eq 'RG-DATA' )      { $url .= '/dashboard/zones/localzone/replicationgroups'; }
  elsif ( $type eq 'RGL' )          { $url .= "/dashboard/replicationgroups/$parm1/rglinks"; }
  elsif ( $type eq 'NS-CONF' )      { $url .= '/object/namespaces'; }
  elsif ( $type eq 'NS-DATA' )      { $url .= "/object/billing/namespace/$parm1/sample"; }
  elsif ( $type eq 'BKT-CONF' )     { $url .= "/object/bucket?namespace=$parm1"; }
  elsif ( $type eq 'BKT-DATA' )     { $url .= "/object/billing/buckets/$parm1/$parm2/sample"; }
  elsif ( $type eq 'LICENSE' )      { $url .= '/license'; }
  elsif ( $type eq 'INFLUX' )       { $url .= '/flux/api/external/v2/query'; } ## always POST

  ### when ecs_debug is true
  if ( $ecs_debug && $type eq 'INFLUX' ) {
    $url .= ( !$parm2 ) ? "/$parm1" : "/$parm1/s";
  }

  return $url if !$interval;

  my $time_rounded2hour = set_time($interval);
  my $end   = $act_time - $time_rounded2hour;
  my $start = $end - $interval;

  my $dtType = 'historical';
  if ( $interval == 300 ) {
    my $categ  = 'traffic';
    return "$url?dataType=$dtType&startTime=$start&endTime=$end&interval=$interval&category=$categ";
  }

  # $interval == 3600
  if ( $type =~ m/^VDC$|^NODE-DATA$|^RG-DATA$/ ) {
    $url = "$url?dataType=$dtType&startTime=$start&endTime=$end&interval=$interval";
  } elsif ( $type =~ m/^BKT-DATA$|^NS-DATA$/ ) {
    $end   = time_form( $act_time - $time_rounded2hour - 3600 );
    $start = time_form( $act_time - $time_rounded2hour - 7200 );
    $url   = "$url?start_time=$start&end_time=$end";
  }

  return $url;
}

sub set_time {
  my ($sec, $min) = gmtime($act_time);
  return $sec + $min * 60;
}

sub time_form {
  my ($sec, $min, $hour, $mday, $mon, $year) = gmtime(shift);

  $year += 1900;
  $mon + 1 < 13 ? $mon += 1 : ( $mon = 1 );

  $mon  = sprintf("%02d", $mon);
  $mday = sprintf("%02d", $mday);
  $hour = sprintf("%02d", $hour);
  $min  = sprintf("%02d", $min);

  return "$year-$mon-$mday" . "T$hour:$min";
}

sub make_ua {
  return LWP::UserAgent->new(
    ssl_opts => {
      SSL_cipher_list => 'DEFAULT:!DH',
      verify_hostname => 0,
      SSL_verify_mode => 0
    }
  );
}

sub new_request {
  my $url = shift;
  my $method = shift || 'GET';

  return HTTP::Request->new($method, $url, [
    'Content-Type' => 'application/json',
    'Accept'       => 'application/json',
    'Connection'   => 'keep-alive'
  ]);
}

sub make_request {
  my $type = shift;
  my $method = shift || 'GET';

  if ($method eq 'GET') {
    return sub {
      my (undef, undef, $line) = caller;
      my ( $api_type, $parm1, $parm2, $interval, $print_err ) = @_;

      my $url = address($api_type, $parm1, $parm2, $interval);
      return request_get($url, $type, $line, $print_err, $api_type);
    };
  }

  return sub {
    my (undef, undef, $line) = caller;
    my ( $body, $print_err, $debug_type, $is_start ) = @_;

    return request_post($body, $type, $line, $print_err, $debug_type, $is_start );
  };
}

sub request_get {
  my ( $url, $type, $line, $print_err, $debug_type ) = @_;

  if ($ecs_debug && $debug) {
    my $url_to_f_name = $url;
    $url_to_f_name =~ s/https:\/\/$ip:4443\///;
    $url_to_f_name =~ s/\//-/g;
    if   ( $debug_type !~ m/^BKT-DATA|^NS-DATA/ ) { $url_to_f_name =~ s/startTime=\d*\&endTime=\d*\&//; }
    else                                    { $url_to_f_name =~ s/\?.*$//; }
    my $folder = ( $type !~ m/^NS/ && $type !~ m/^BKT/ ) ? "$vdc_alias" : "$cfg->{hostalias}";
    my $file = "./api/$folder/$url_to_f_name.json";

    if ( -e $file ) {
      my %data = %{ Xormon::read_json($file) };
      return \%data;
    }

    print ( "ERROR: Unable to get data from: $url_to_f_name: $file:" . __FILE__ . ":" . $line . "\n" );
    return 0;
  }

  return $api_data{$url} if exists $api_data{$url}; ## same api call has been done already

  my $ua = make_ua();
  my $req = new_request($url);
  $req->header( 'X-SDS-AUTH-TOKEN' => $cookie );

  my $res = $ua->request($req);
  if ($res->is_error) {
    api_error("Unable to get data type $type from: $url", $line, $res->content, $print_err);
    return;
  }

  my $data = decode_json($res->content);
  $api_data{$url} = $data;

  return $data;
}

sub request_post {
  my ($body, $type, $line, $print_err, $debug_type, $is_start ) = @_;
  my $url = address('INFLUX',$debug_type, $is_start );
  
  if ($ecs_debug && $debug) {
    my $url_to_f_name = $url;
    $url_to_f_name =~ s/https:\/\/$ip:4443\///;
    $url_to_f_name =~ s/\//-/g;

    my $file = "./api/$vdc_alias/$url_to_f_name.json";

    if ( -e $file ) {
      my %data = %{ Xormon::read_json($file) };
      return \%data;
    }

    print( "ERROR: Unable to get data from: $url_to_f_name :" . __FILE__ . ":" . $line . "\n" );
    return 0;
  }

  my $ua = make_ua();
  my $req = new_request($url, 'POST');
  $req->header( 'X-SDS-AUTH-TOKEN' => $cookie );

  my $en_json = encode_json($body);
  $req->content($en_json);

  my $res = $ua->request($req);
  if ($res->is_error) {
    api_error("Unable to post data type $type from: $url", $line, $res->content, $print_err);
    return;
  }

  return decode_json( $res->content );
}

sub login {
  my (undef, undef, $line) = caller;
  my ( $user, $pass, $print_err ) = ( shift, shift, shift );
  my $url = address('LOGIN');

  my $ua = make_ua();
  my $req = new_request($url);
  $req->authorization_basic( $user, $pass );

  my $res = $ua->request($req);
  return $res->headers->{"x-sds-auth-token"} if $res->is_success;

  api_error("Login to API $url failed", $line, $res->content, $print_err);
}

sub logout {
  my (undef, undef, $line) = caller;
  my $force = shift;

  my $url = address('LOGOUT');
  $url .= "?force=true" if $force;

  my $ua = make_ua();
  my $req = new_request($url);
  $req->header( 'X-SDS-AUTH-TOKEN' => $cookie );

  my $res = $ua->request($req);

  return if $res->is_success;

  api_error("Logout from API $url failed", $line, $res->content);
}

sub api_error {
  my ($msg, $line, $body, $print_error) = @_;

  return if $print_error || !$logger;

  error($msg, $line);
  $logger->error($body);
}

sub error {
  my (undef, $file, $call_line) = caller;
  my $msg = shift;
  my $line = shift || $call_line;

  $logger->error("$msg : $file : $line\n");
}

sub error_debug {
  return if !$debug;

  my (undef, $file, $call_line) = caller;
  my $type = shift;
  my $line = shift || $call_line;

  $logger->error("Too many missing $type information for $vdc_alias: $file : $line\n");
}

sub endJob {
  my (undef, $file, $call_line) = caller;

  my $msg = shift;
  my $line = shift || $call_line;
  error($msg, $line);

  exit;
}

sub list_conf {
  # ROW=apiname:name:second level of hash

  # row with hash(#) wouldn`t be in finall conf file
  # final order in conf row is determinate by order here
  # rounding just for h2( data) of category: NS, BKT
  # 'warning alerts' => { name => 'warning alerts', api => 'alertsNumUnackWarning', level => 'Count' },
  # 'error alerts' => { name => 'error alerts', api => 'alertsNumUnackError', level => 'Count' },
  # 'critical alerts' => { name => 'critical alerts', api => 'alertsNumUnackCritical', level => 'Count' }

  return {
    CD  => { names => ['ECS name'] },
    HSG => { names => ['VDC'] },
    HS  => {
      names => [
        'maintenance nodes',
        'maintenance disks',
        'bad nodes',
        'bad disks',
        'warning alerts',
        'error alerts',
        'critical alerts'
      ],
      metric => {
        'maintenance nodes' => { name => 'maintenance nodes', api => 'numMaintenanceNodes' },
        'maintenance disks' => { name => 'maintenance disks', api => 'numMaintenanceDisks' },
        'bad nodes'         => { name => 'bad nodes',         api => 'numBadNodes' },
        'bad disks'         => { name => 'bad disks',         api => 'numBadDisks' }
      }
    },
    AL => {
      names  => [qw( id alertSeverity alertTime alertText )],
      metric => {
        id            => { name => 'id',            api => 'id' },
        alertSeverity => { name => 'alertSeverity', api => 'severity' },
        alertTime     => { name => 'alertTime',     api => 'timestamp' },
        alertText     => { name => 'alertText',     api => 'description' }
      }
    },
    VDC => {
      metric => {
        vdcNumDisks   => { name => 'vdcNumDisks',   api => 'numDisks' },
        vdcGoodDisks  => { name => 'vdcGoodDisks',  api => 'numGoodDisks' },
        vdcBadDisks   => { name => 'vdcBadDisks',   api => 'numBadDisks' },
        vdcMaintDisks => { name => 'vdcMaintDisks', api => 'numMaintenanceDisks' },
        vdcNumNodes   => { name => 'vdcNumNodes',   api => 'numNodes' },
        vdcGoodNodes  => { name => 'vdcGoodNodes',  api => 'numGoodNodes' },
        vdcBadNodes   => { name => 'vdcBadNodes',   api => 'numBadNodes' },
        vdcMaintNodes => { name => 'vdcMaintNodes', api => 'numMaintenanceNodes' },
        vdcAlrtCrit   => { name => 'vdcAlrtCrit',   api => 'alertsNumUnackCritical', level => 'Count' },
        vdcAlrtErr    => { name => 'vdcAlrtErr',    api => 'alertsNumUnackError', level => 'Count' },
        vdcAlrtWarn   => { name => 'vdcAlrtWarn',   api => 'alertsNumUnackWarning', level => 'Count' }
      },
      config => {
        id => { name => 'id', api => 'vdcId'},
        name => { name => 'name', api => 'vdcName'},
      }
    },
    NODE => {
      metric => {
        id             => { name => 'id',             api => 'id' },
        name           => { name => 'name',           api => 'displayName' },
        nodeStatus     => { name => 'nodeStatus',     api => 'healthStatus' },
        nodeNumDisks   => { name => 'nodeNumDisks',   api => 'numDisks' },
        nodeGoodDisks  => { name => 'nodeGoodDisks',  api => 'numGoodDisks' },
        nodeBadDisks   => { name => 'nodeBadDisks',   api => 'numBadDisks' },
        nodeMaintDisks => { name => 'nodeMaintDisks', api => 'numMaintenanceDisks' },
        nodeSpoolId    => { name => 'nodeSpoolId',    api => 'storagePoolId' },
        nodeSpoolName  => { name => 'nodeSpoolName',  api => 'storagePoolName' }
      },
      config => {
        id => { name => 'id', api => 'nodeid'},
        name => { name => 'name', api => 'nodename'},
        ip => { name => 'ip', api => 'ip'},
        version => { name => 'version', api => 'version'},
      }
    },
    LICENCE => {
      names  => [qw( vdcSerial )],
      metric => { vdcSerial => { name => 'vdcSerial', api => 'serial' } }
    },
    RG => {
      metric => {},
      config => {
        id => { name => 'id', api => 'id'},
        name => { name => 'name', api => 'name'},
      }
    },
    RGL => {
      metric => { rgLnkStatus => { name => 'rgLnkStatus', api => 'rglinkStatus' } },
      config => {
        id => { name => 'id', api => 'id'},
        name => { name => 'name', api => 'zone2Name'},
      }
    },
    NS => {
      metric => {
        nsObjTotal => { name => 'nsObjTotal', api => 'total_objects', round => 0 },
        size     => { name => 'size',  api => 'total_size',    round => 0 },
        soft_quota   => { name => 'soft_quota',  round => 0 },
        hard_quota   => { name => 'hard_quota',  round => 0 },
        usage => { name => 'usage', round => 0 },
        free => { name => 'free', round => 0 }
      },
      config => {
        id => { name => 'id', api => 'id'},
        name => { name => 'name', api => 'name'},
        soft_quota => { name => 'soft_quota', api => 'notificationSize' },
        hard_quota => { name => 'hard_quota', api => 'blockSize'},
      }
    },
    BKT => {
      metric => {
        object_total => { name => 'object_total', api => 'total_objects', round => 0 },
        size     => { name => 'size',     api => 'total_size',    round => 0 },
        soft_quota   => { name => 'soft_quota',  round => 0 },
        hard_quota   => { name => 'hard_quota',  round => 0 },
        usage => { name => 'usage', round => 0 },
        free => { name => 'free', round => 0 }
      },
      config => {
        id => { name => 'id', api => 'id'},
        name => { name => 'name', api => 'name'},
        soft_quota => { name => 'soft_quota', api => 'notification_size' },
        hard_quota => { name => 'hard_quota', api => 'block_size' },
        owner => { name => 'owner', api => 'owner' },
        bktNs => { name => 'bktNs', api => 'namespace'},
        bktRG => { name => 'bktRG', api => 'vpool'},
      }
    }
  };
}

sub list_perf {
  return {
    device => {
      names => [qw(
        resp_t_r resp_t_w resp_t
        read write data_rate
        read_io write_io io_rate
        capacity_used capacity_free capacity_total capacity_used_percent
      )],
      metric => {
        capacity_used_percent => { name => 'capacity_used_percent', unit => '', round => 0 } # %
      }
    },
    VDC => {
      names => [qw(
        resp_t_r resp_t_w resp_t
        read write data_rate
        read_tps write_tps tps_rate error_rate_tps
        network_rx network_tx network_rate
        cpu_system
        capacity_used capacity_free capacity_total capacity_used_percent
        memory_used memory_free memory_used_percent
        network_util_rx network_util_tx network_util
        disk_read disk_write disk_data_rate
        disk_geo_read disk_geo_write disk_geo_data_rate
        disk_user_read disk_user_write disk_user_data_rate
      )]
    },
    VDC_m => {
      metric => {
        resp_t_r  => { name => 'resp_t_r',  api => 'transactionReadLatency',             unit => '',   round => 0 },
        resp_t_w  => { name => 'resp_t_w',  api => 'transactionWriteLatency',            unit => '',   round => 0 },
        read => { name => 'read', api => 'transactionReadBandwidth',           unit => '', round => 0 }, # kilobytes
        write => { name => 'write', api => 'transactionWriteBandwidth',          unit => '', round => 0 }, # kilobytes
        data_rate => { name => 'data_rate', unit => '', round => 0 }, # kilobytes
        read_tps  => { name => 'read_tps',  api => 'transactionReadTransactionsPerSec',  unit => '',   round => 0 },
        write_tps  => { name => 'write_tps',  api => 'transactionWriteTransactionsPerSec', unit => '',   round => 0 },
        tps_rate  => { name => 'tsp_rate', unit => '', round => 0 },
        error_rate_tps    => { name => 'error_rate_tps',    api => 'transactionErrors',                  unit => '',   round => 0 } # %
      }
    },
    VDC_h => {
      metric => {
        network_rate => { name => 'network_rate', api => 'nodeNicBandwidthAvgSummary',              unit => '', round => 0 }, # kilobytes
        network_rx  => { name => 'network_rx',  api => 'nodeNicReceivedBandwidthAvgSummary',      unit => '', round => 0 }, # kilobytes
        network_tx  => { name => 'network_tx',  api => 'nodeNicTransmittedBandwidthAvgSummary',   unit => '', round => 0 }, # kilobytes
        cpu_system    => { name => 'cpu_system',    api => 'nodeCpuUtilizationAvgSummary',            unit => '',   round => 0 },
        memory_used    => { name => 'memory_used',    api => 'nodeMemoryUtilizationBytesAvgSummary',    unit => '',  round => 0 }, # bytes
        memory_free    => { name => 'memory_free', unit => '',  round => 0 }, # bytes
        memory_used_percent => { name => 'memory_used_percent',    api => 'nodeMemoryUtilizationAvgSummary',         unit => '',   round => 2 },
        network_util => { name => 'network_util', api => 'nodeNicUtilizationAvgSummary',            unit => '',   round => 1 },
        network_util_rx  => { name => 'network_util_rx',  api => 'nodeNicReceivedUtilizationAvgSummary',    unit => '',   round => 1 },
        network_util_tx  => { name => 'network_util_tx',  api => 'nodeNicTransmittedUtilizationAvgSummary', unit => '',   round => 1 },
        disk_read  => { name => 'disk_read',  api => 'diskReadBandwidthTotalSummary',           unit => '', round => 0 }, # kilobytes
        disk_write  => { name => 'disk_write',  api => 'diskWriteBandwidthTotalSummary',          unit => '', round => 0 }, # kilobytes
        disk_data_rate => { name => 'disk_data_rate', api => 'diskWriteBandwidthTotalSummary',       unit => '', round => 0 }, # kilobytes
        disk_geo_read  => { name => 'disk_geo_read',  api => 'diskReadBandwidthGeoSummary',             unit => '', round => 0 }, # kilobytes
        disk_geo_write  => { name => 'disk_geo_write',  api => 'diskWriteBandwidthGeoSummary',            unit => '', round => 0 }, # kilobytes
        disk_geo_data_rate => { name => 'disk_geo_data_rate', unit => '', round => 0 }, # kilobytes
        disk_user_read  => { name => 'disk_user_read',  api => 'diskReadBandwidthUserSummary',            unit => '', round => 0 }, # kilobytes
        disk_user_write  => { name => 'disk_user_write',  api => 'diskWriteBandwidthUserSummary',           unit => '', round => 0 }, # kilobytes
        disk_user_data_rate => { name => 'disk_user_data_rate', unit => '', round => 0 }, # kilobytes
      }
    },
    VDCCAP => {
      metric => {
        capacity_used_percent => { name => 'capacity_used_percent', unit => '', round => 0 },
        capacity_used  => { name => 'capacity_used',  api => 'diskSpaceAllocatedCurrent', unit => '', round => 0 }, # bytes
        capacity_free  => { name => 'capacity_free',  api => 'diskSpaceFreeCurrent',      unit => '', round => 0 }, # bytes
        capacity_total => { name => 'capacity_total', api => 'diskSpaceTotalCurrent',     unit => '', round => 0 } # bytes
      }
    },
    NODE => {
      names => [qw(
        resp_t_r resp_t_w resp_t
        read write data_rate
        read_tps write_tps tps_rate error_rate_tps
        network_rx network_tx network_rate
        cpu_system
        memory_used memory_free memory_used_percent
        network_util_rx network_util_tx network_util
        disk_read disk_write disk_data_rate
        disk_geo_read disk_geo_write disk_geo_data_rate
        disk_user_read disk_user_write disk_user_data_rate
      )]
    },
    NODE_m => {
      metric => {
        resp_t_r  => { name => 'resp_t_r',  api => 'transactionReadLatency',             unit => '',   round => 0 },
        resp_t_w  => { name => 'resp_t_w',  api => 'transactionWriteLatency',            unit => '',   round => 0 },
        read => { name => 'read', api => 'transactionReadBandwidth',           unit => '', round => 0 }, # kilobytes
        write => { name => 'write', api => 'transactionWriteBandwidth',          unit => '', round => 0 }, # kilobytes
        read_tps  => { name => 'read_tps',  api => 'transactionReadTransactionsPerSec',  unit => '',   round => 0 },
        write_tps  => { name => 'write_tps',  api => 'transactionWriteTransactionsPerSec', unit => '',   round => 0 },
        error_rate_tps    => { name => 'error_rate_tps',    api => 'transactionErrors',                  unit => '',   round => 0 }
      }
    },
    NODE_h => {
      metric => {
        network_rate   => { name => 'network_rate',   api => 'nodeNicBandwidthSummary',              unit => '', round => 0 }, # kilobytes
        network_rx    => { name => 'network_rx',    api => 'nodeNicReceivedBandwidthSummary',      unit => '', round => 0 }, # kilobytes
        network_tx    => { name => 'network_tx',    api => 'nodeNicTransmittedBandwidthSummary',   unit => '', round => 0 }, # kilobytes
        network_util   => { name => 'network_util',   api => 'nodeNicUtilizationSummary',            unit => '',   round => 1 },
        network_util_rx    => { name => 'network_util_rx',    api => 'nodeNicReceivedUtilizationSummary',    unit => '',   round => 1 },
        network_util_tx    => { name => 'network_util_tx',    api => 'nodeNicTransmittedUtilizationSummary', unit => '',   round => 1 },
        memory_used      => { name => 'memory_used',      api => 'nodeMemoryUtilizationBytesSummary',    unit => '',  round => 0 }, #bytes
        memory_free    => { name => 'memory_free', unit => '',  round => 0 }, # bytes
        memory_used_percent      => { name => 'memory_used_percent',      api => 'nodeMemoryUtilizationSummary',         unit => '',   round => 0 },
        cpu_system      => { name => 'cpu_system',      api => 'nodeCpuUtilizationSummary',            unit => '',   round => 0 },
        disk_read => { name => 'disk_read', api => 'diskReadBandwidthTotalSummary',        unit => '', round => 0 }, # kilobytes
        disk_write => { name => 'disk_write', api => 'diskWriteBandwidthTotalSummary',       unit => '', round => 0 }, # kilobytes
        disk_data_rate => { name => 'disk_data_rate', api => 'diskWriteBandwidthTotalSummary',       unit => '', round => 0 }, # kilobytes
        disk_geo_read => { name => 'disk_geo_read', api => 'diskReadBandwidthGeoSummary',          unit => '', round => 0 }, # kilobytes
        disk_geo_write => { name => 'disk_geo_write', api => 'diskWriteBandwidthGeoSummary',         unit => '', round => 0 }, # kilobytes
        disk_geo_data_rate => { name => 'disk_geo_data_rate', unit => '', round => 0 }, # kilobytes
        disk_user_read => { name => 'disk_user_read', api => 'diskReadBandwidthUserSummary',         unit => '', round => 0 }, # kilobytes
        disk_user_write => { name => 'disk_user_write', api => 'diskWriteBandwidthUserSummary',        unit => '', round => 0 }, # kilobytes
        disk_user_data_rate => { name => 'disk_user_data_rate', unit => '', round => 0 }, # kilobytes
      }
    },
    NODECAP => {
      metric => {
        capacity_used_percent => { name => 'capacity_used_percent', unit => '', round => 0 },
        capacity_used    => { name => 'capacity_used',    api => 'diskSpaceAllocatedCurrent', unit => '', round => 0 }, # bytes
        capacity_free    => { name => 'capacity_free',    api => 'diskSpaceFreeCurrent',      unit => '', round => 0 }, # bytes
        capacity_total   => { name => 'capacity_total',   api => 'diskSpaceTotalCurrent',     unit => '', round => 0 }, # bytes
      }
    },
    RG => {
      names  => [qw(read write data_rate)],
      metric => {
        read => { name => 'read', api => 'replicationEgressTraffic',  unit => '', round => 0 }, # kilobytes
        write  => { name => 'write',  api => 'replicationIngressTraffic', unit => '', round => 0 }, # kilobytes
        data_rate  => { name => 'data_rate', unit => '', round => 0 } # kilobytes
      },
    },
    RGL => {
      names  => [qw(read write data_rate)],
      metric => {
        read => { name => 'read', api => 'replicationEgressTraffic',  unit => '', round => 0 }, # kilobytes
        write  => { name => 'write',  api => 'replicationIngressTraffic', unit => '', round => 0 }, # kilobytes
        data_rate  => { name => 'data_rate', unit => '', round => 0 } # kilobytes
      },
    },
    NS => {
      names  => [qw(read write data_rate object_created object_deleted object_total delta_size size soft_quota hard_quota capacity_free)],
      metric => {
        write   => { name => 'write',   api => 'ingress',         unit => 'h2s', round => 0 },
        read    => { name => 'read',    api => 'egress',          unit => 'h2s', round => 0 },
        data_rate   => { name => 'data_rate', unit => 'h2s', round => 0 },
        size      => { name => 'size',      api => 'total_size',      unit => 'GiB',   round => 0 },
        object_created => { name => 'object_created', api => 'objects_created', unit => '',   round => 0 },
        object_deleted => { name => 'object_deleted', api => 'objects_deleted', unit => '',   round => 0 },
        delta_size     => { name => 'delta_size',     api => 'bytes_delta',     unit => '',   round => 0 },
        object_total  => { name => 'object_total',  api => 'total_objects',   unit => '',   round => 0 },
        soft_quota    => { name => 'soft_quota', unit => 'GiB', round => 0 }, # gigabytes
        hard_quota    => { name => 'hard_quota', unit => 'GiB', round => 0 }, # gigabytes
        capacity_free    => { name => 'capacity_free', unit => 'GiB', round => 0 }, # gigabytes
      },
    },
    BKT => {
      names  => [qw(read write data_rate object_created object_deleted object_total delta_size size soft_quota hard_quota capacity_free)],
      metric => {
        write   => { name => 'write',   api => 'ingress',         unit => 'h2s', round => 0 },
        read   => { name => 'read',   api => 'egress',          unit => 'h2s', round => 0 },
        data_rate   => { name => 'data_rate', unit => 'h2s', round => 0 },
        size      => { name => 'size',      api => 'total_size',      unit => 'GiB',   round => 0 },
        object_created => { name => 'object_created', api => 'objects_created', unit => '',   round => 0 },
        object_deleted => { name => 'object_deleted', api => 'objects_deleted', unit => '',   round => 0 },
        delta_size     => { name => 'delta_size',     api => 'bytes_delta',     unit => '',   round => 0 },
        object_total  => { name => 'object_total',  api => 'total_objects',   unit => '',   round => 0 },
        soft_quota    => { name => 'soft_quota',    unit => 'GiB', round => 0 },
        hard_quota    => { name => 'hard_quota',    unit => 'GiB', round => 0 },
        capacity_free    => { name => 'capacity_free', unit => 'GiB', round => 0 }, # gigabytes
      },
    },
    influx => {
      db => {
        3600 => [
          { db    => 'monitoring_op',
            names => {
              cpu    => [qw( usage_quest usage_guest_nice usage_idle usage_iowait usage_irq usage_nice usage_softirq usage_steal usage_system usage_user )],
              disk   => [qw( used_percent used free total )],
              diskio => [qw( )],
              mem    => [qw( used used_percent )],
              net    => [qw( speed )]
            }
          },
          { db    => 'monitoring_main',
            names => {
              blob_IO_Statistics_data_read  => [qw()],
              blob_IO_Statistics_data_write => [qw()]
            }
          }
        ],
        300 => [
          { db    => 'monitoring_main',
            names => {
              statDataHead_performance_internal_error      => [qw( )],
              statDataHead_performance_internal_latency    => [qw()],
              statDataHead_performance_internal_throughput => [qw()]
            }
          },
          { db    => 'monitoring_op',
            names => { net => [qw( speed )] }
          }
        ]
      },
      naming => {
        vdc => {
          util_cpu           => 'cpu_system',
          bytes_sum_net      => 'network_rate',
          bytes_recv_net     => 'network_rx',
          bytes_sent_net     => 'network_tx',
          packets_sent_net   => 'read_tps',
          packets_recv_net   => 'write_tps',
          used_mem           => 'memory_used',
          util_net           => 'network_util',
          write_util_net     => 'network_util_rx',
          read_util_net      => 'network_util_tx',
          used_percent_mem   => 'memory_used_percent',
          read_bytes_diskio  => 'disk_read',
          write_bytes_diskio => 'disk_write',
          read_statDataHead_performance_internal_latency                         => 'resp_t_r',
          write_statDataHead_performance_internal_latency                        => 'resp_t_w',
          # error_statDataHead_performance_internal_error                          => 'error_rate_tps',
          total_read_requests_size_statDataHead_performance_internal_throughput  => 'read',
          total_write_requests_size_statDataHead_performance_internal_throughput => 'write',
          read_GEOTotal_blob_IO_Statistics_data_read                             => 'disk_geo_read',
          write_GEOTotal_blob_IO_Statistics_data_write                           => 'disk_geo_write',
          read_USERTotal_blob_IO_Statistics_data_read                            => 'disk_user_read',
          write_USERTotal_blob_IO_Statistics_data_write                          => 'disk_user_write',
          trErr_statDataHead_performance_internal_error                           => 'error_rate_tps'
        },
        node => {
          util_cpu           => 'cpu_system',
          write_bytes_diskio => 'disk_write',
          read_bytes_diskio  => 'disk_read',
          bytes_sum_net      => 'network_rate',
          bytes_recv_net     => 'network_rx',
          bytes_sent_net     => 'network_tx',
          write_util_net     => 'network_util_rx',
          read_util_net      => 'network_util_tx',
          util_net           => 'network_util',
          used_percent_mem   => 'memory_used_percent',
          used_mem           => 'memory_used',
          read_statDataHead_performance_internal_latency                         => 'resp_t_r',
          write_statDataHead_performance_internal_latency                        => 'resp_t_w',
          total_read_requests_size_statDataHead_performance_internal_throughput  => 'read',
          total_write_requests_size_statDataHead_performance_internal_throughput => 'write',
          packets_sent_net                                                       => 'read_tps',
          packets_recv_net                                                       => 'write_tps',
          # error_statDataHead_performance_internal_error                          => 'error_rate_tps',
          write_USERTotal_blob_IO_Statistics_data_write                          => 'disk_user_write',
          read_USERTotal_blob_IO_Statistics_data_read                            => 'disk_user_read',
          read_GEOTotal_blob_IO_Statistics_data_read                             => 'disk_geo_read',
          write_GEOTotal_blob_IO_Statistics_data_write                           => 'disk_geo_write',
          trErr_statDataHead_performance_internal_error                          => 'error_rate_tps'
        }
      }
    }
  };
}

1;
