package ESERIES;

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";

use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use Xormon;
use JSON qw(encode_json decode_json);
use Test;
use XormonLogger;

my %data;
my $xormon;
my $cfg;
my $logger;

my $stats_found     = 0;
my $connection_test = 0;
my @rows;

sub collect {
  $xormon = shift;
  $cfg    = shift;

  #
  # test main variables
  #
  unless ( exists $cfg->{'hostcfg_id'} )  { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")  && return "error"; }
  unless ( exists $cfg->{'hostalias'} )   { Xormon::error("Cannot continue, 'hostalias' is not configured!")   && return "error"; }
  $logger = XormonLogger->new( 'eseries', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost'} )     { $logger->error("Cannot continue, 'apiHost' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'apiPort'} )     { $logger->error("Cannot continue, 'apiPort' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'apiUsername'} ) { $logger->error("Cannot continue, 'apiUsername' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiPassword'} ) { $logger->error("Cannot continue, 'apiPassword' is not configured!") && return "error"; }
  $cfg->{'sample_rate'}   = 300;
  my $act_timestamp       = time();
  my $status              = ""; # status code
  my $last_perf_timestamp = "";

  #
  # get microservice data
  #
  my $service_data_in  = $xormon->getData();
  my $service_data_out = {};
  if ( exists $service_data_in->{last_conf_timestamp} ) {
    $service_data_out->{last_conf_timestamp} = $service_data_in->{last_conf_timestamp};
  }

  #
  # set stats time range
  #
  if ( exists $service_data_in->{last_perf_timestamp} && Xormon::isdigit($service_data_in->{last_perf_timestamp}) ) {
    if ( $service_data_in->{last_perf_timestamp} <= $act_timestamp ) {
      if (  ($act_timestamp - $service_data_in->{last_perf_timestamp} ) > 21600 ) {
        $logger->error( "Data was not be collected longer than 6 hours. Create only perf files for the last 1 hour. " . __FILE__ . ":" . __LINE__ );
        $cfg->{'sunix'}    = int( ( $act_timestamp  / $cfg->{'sample_rate'} ) * $cfg->{'sample_rate'}) - 3900; # last hour
      }
      elsif ( ($act_timestamp - $service_data_in->{last_perf_timestamp}) < $cfg->{'sample_rate'} ) {
        $logger->error( "Last perf file was created less than $cfg->{'sample_rate'} seconds ago. Do not run agent at this time! Exitting... " . __FILE__ . ":" . __LINE__ ) && return "successful";
      }
      else {
        #$cfg->{'sunix'} = $service_data_in->{last_perf_timestamp} - $cfg->{'sample_rate'}; # with this was created one additional perf file which was already created in the last run...
        $cfg->{'sunix'} = $service_data_in->{last_perf_timestamp};
        # historical-statistics does not provide statistics for DISKs in the first data sample
        # so statistics have to be collected with one more earlier sample which is always skipped
      }
    }
    else {
      $logger->error( "Something wrong, last_perf_timestamp is highter than act_timestamp! $service_data_in->{last_perf_timestamp} > $act_timestamp Exitting... " . __FILE__ . ":" . __LINE__ ) && return "error";
    }
  }
  else {
    $cfg->{'sunix'} = int( ( $act_timestamp  / $cfg->{'sample_rate'} ) * $cfg->{'sample_rate'}) - 3900; # last hour
  }
  $cfg->{'sunix_ms'} = $cfg->{'sunix'}."000";

  #
  # connection status
  #
  if ( get_system_cfg() ) {
    $xormon->isConnected(1);
  }
  else {
    $xormon->isConnected(0, "Connection failed!") && return "error";
  }

  #
  # collect perf data
  #
  get_perf();
  get_pool_cfg();
  get_volumes_cfg();     # volumes/thin-volumes, get capacity
  get_hosts_cfg();       # get cfg for perf data aggregation from volumes
  get_host_groups_cfg(); # get cfg for perf data aggregation from volumes
  mapping();             # from volumes to hosts and host-groups

  #
  # send perf data
  #
  if ( exists $data{'perf'} ) {
    my $idx = 0;
    foreach my $timestamp ( sort { $a <=> $b } keys( %{ $data{'perf'} } ) ) {
      if ( ! Xormon::isdigit($timestamp) ) { next; }

      $idx++;
      # historical-statistics does not provide statistics for DISKs in the first data sample
      # so statistics have to be collected with one more earlier sample which is always skipped
      if ( $idx == 1 ) { next; }

      my $perf_out = prepare_perf($timestamp);
      my $rc       = $xormon->saveJSON2($perf_out);
      $logger->info("save perf: $rc : ".localtime($timestamp)." : $timestamp");

      if ( $rc eq "201" ) {
        $status = "successful";
      }
      elsif ( $rc eq "401" ) {
        $status = "unauthorized";
        last;
      }
      else {
        $status = "error";
        last;
      }

      $last_perf_timestamp = $timestamp;
    }
  }

  #
  # perf data status
  #
  if ( Xormon::isdigit($last_perf_timestamp) ) {
    $service_data_out->{last_perf_timestamp} = $last_perf_timestamp;

    # check if there is current data
    if ( ( time() - $last_perf_timestamp ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: $last_perf_timestamp");
    }
    else {
      $xormon->hasData(1);
    }
  }
  else {
    $xormon->hasData(0, "no available stats");
  }
  # stop if data saving fails
  if ( $status ne 'successful' ) { return $status; }

  #
  # collect conf data, send architecture and conf data (only once per hour)
  #
  if ( ! exists $service_data_in->{last_conf_timestamp} || ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) || $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) {
    #$logger->info("CONF time...");
    $service_data_out->{last_conf_timestamp} = $act_timestamp;

    get_drives_cfg();
    get_ports_cfg();
    get_nodes_cfg();
    #get_workload_cfg();

    send_conf();
  }

  #
  # save microservice data
  #
  $xormon->saveData(\%{ $service_data_out });

  #
  # health status
  #
  health_status();

  return $status;
}

sub health_status {
  my @hs = ();
  my %hs_data;

  if ( exists $data{conf}{DEVICE}{status} ) {
    if ( $data{conf}{DEVICE}{status} eq "optimal" ) {
      push(@hs, { "item_id" => $cfg->{'hostcfg_id'}, "status" => "ok", "data" => "Storage array status is $data{conf}{DEVICE}{status}." });
    }
    else {
      # try to find out what is wrong
      my $result   = new_get_request("/devmgr/v2/storage-systems/$data{conf}{DEVICE}{id}/failures");
      my @failures = ();
      if ( defined $result && $result ne "0" && ref($result) eq "ARRAY" ) {
        foreach my $hash_ref ( @{$result} ) {
          while ( my ( $key, $value ) = each( %{$hash_ref} ) ) {
            if ( defined $key && $key ne '' && defined $value && $value ne '' ) {
              push( @failures,      "$key : $value" );

              push( @{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{failures} }, { "key" => $key, "value" => $value } );
            }
          }
        }
      }

      push(@hs, { "item_id" => $cfg->{'hostcfg_id'}, "status" => "error", "data" => "Storage array status is $data{conf}{DEVICE}{status}.".join(", ", @failures) });
    }

    # add HS data to DB to be able to show them in the HS view
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{status} = $data{conf}{DEVICE}{status};
  }
  else {
    push(@hs, { "item_id" => $cfg->{'hostcfg_id'}, "status" => "warning", "data" => "Health status for device $cfg->{hostalias} was not found!" });
    $logger->error( "Health status was not found! $!" . __FILE__ . ":" . __LINE__ );
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{status} = "Unknown";
  }

  $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{last_update_time} = localtime();

  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }

  # send statuses
  my $rc = $xormon->saveStatus(\@hs);
  $logger->info("save health status: $rc");

  # save HS data
  my $rc_hs = $xormon->saveConfJSON(\%hs_data,1);
  $logger->info("save health status table: $rc_hs");

  return 1;
}

sub connection_test {
  my $client = shift;
  $cfg       = shift;

  $connection_test = 1;
  $logger          = XormonLogger->new( 'eseries', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  my $test         = Test->new($client);

  #
  # test main variables
  #
  my @variables = ("hostcfg_id", "hostalias", "apiHost", "apiPort", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Device cfg", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Device cfg", 1 , "");

  #
  # set additional variables
  #
  $cfg->{'sample_rate'} = 300;
  my $act_timestamp     = time();
  $cfg->{'sunix'}       = int( ( $act_timestamp  / $cfg->{'sample_rate'} ) * $cfg->{'sample_rate'}) - 3900; # last hour
  $cfg->{'sunix_ms'}    = $cfg->{'sunix'}."000";

  #
  # test IP and port
  #
  my ( $code, $text ) = Xormon::conntest_tcp($cfg->{'apiHost'},$cfg->{'apiPort'});
  $test->addSection("API host network connection", $code, $text);
  if ( $code == 0 ) { return $test->stop(); }

  #
  # collect system cfg
  #
  my $result = get_system_cfg();
  if ( $result == 0 ) {
    push(@rows, "Connection failed!\n");
    $test->addSection("API authorization", 0 , \@rows);
    return $test->stop();
  }
  else {
    if ( exists $data{conf}{DEVICE}{id} )        { push(@rows, "storage-system id : $data{conf}{DEVICE}{id}\n"); }
    if ( exists $data{conf}{DEVICE}{fwVersion} ) { push(@rows, "fwVersion : $data{conf}{DEVICE}{fwVersion}\n"); }
    if ( exists $data{conf}{DEVICE}{status} )    { push(@rows, "status : $data{conf}{DEVICE}{status}\n"); }
    $test->addSection("API authorization", 1 , \@rows);
    @rows = ();
  }

  #
  # collect perf stats
  #
  $result = get_perf();
  if ( $result == 0 ) {
    push(@rows, "Cannot collect stats!\n");
    push(@rows, "Connection failed!!\n");
    $test->addSection("API data test", 0 , \@rows);
    return $test->stop();
  }
  elsif ( $stats_found == 0 ) {
    push(@rows, "Cannot find stats!\n");
    push(@rows, "Connection failed!!\n");
    $test->addSection("API data test", 0 , \@rows);
    return $test->stop();
  }
  else {
    push(@rows, "Stats found!\n");
    $test->addSection("API data test", 1 , \@rows);
  }

  return $test->stop();
}

sub send_conf {
  my @architecture;
  my %conf;

  #
  # DEVICE
  #
  # Architecture
  my %device_arch = (
    "item_id"    => $cfg->{'hostcfg_id'},
    "hostcfg_id" => $cfg->{'hostcfg_id'},
    "label"      => $cfg->{'hostalias'},
    "hw_type"    => "ESERIES",
    "subsystem"  => "device",
    "parents"    => []
  );
  push(@architecture, \%device_arch);
  # conf
  $conf{ $cfg->{'hostcfg_id'} }{ip} = $cfg->{'apiHost'};
  if ( exists $data{conf}{DEVICE}{chassisSerialNumber} ) { $conf{ $cfg->{'hostcfg_id'} }{serial}  = $data{conf}{DEVICE}{chassisSerialNumber}; }
  if ( exists $data{conf}{DEVICE}{fwVersion} )           { $conf{ $cfg->{'hostcfg_id'} }{version} = $data{conf}{DEVICE}{fwVersion}; }
  if ( exists $data{conf}{DEVICE}{model} )               { $conf{ $cfg->{'hostcfg_id'} }{model}   = $data{conf}{DEVICE}{model}; }
  if ( exists $data{conf}{DEVICE}{wwn} )                 { $conf{ $cfg->{'hostcfg_id'} }{wwn}     = Xormon::formatWWN($data{conf}{DEVICE}{wwn}); }
  # add node names
  my @nodes = ();
  if ( exists $data{conf}{NODE} ) {
    foreach my $id ( sort keys %{ $data{conf}{NODE} } ) {
      my $var = $data{conf}{NODE}{$id};
      if ( exists $var->{physicalLocation}{label} ) {
        push(@nodes, $var->{physicalLocation}{label});
      }
    }
  }
  if ( scalar(@nodes) > 0 ) { $conf{ $cfg->{'hostcfg_id'} }{node_name} = join(", ", @nodes); }

  #
  # VOLUMES
  #
  if ( exists $data{conf}{VOLUME} ) {
    foreach my $id ( sort keys %{ $data{conf}{VOLUME} } ) {
      my $var = $data{conf}{VOLUME}{$id};
      if ( exists $var->{label} ) {
        # Architecture
        my %arch = (
          "item_id"   => $id,
          "label"     => $var->{label},
          "hw_type"   => "ESERIES",
          "subsystem" => "volume",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents"   => [$cfg->{'hostcfg_id'}]
        );
        # conf
        #if ( exists $var->{status} )   { $conf{$id}{status}   = $var->{status}; }
        #if ( exists $var->{capacity} ) { $conf{$id}{capacity} = $var->{capacity}; }
        if ( exists $var->{volumeGroupRef} ) {
          #$conf{$id}{pool_id} = $var->{volumeGroupRef};
          # find pool_name
          #if ( exists $data{conf}{POOL}{ $var->{volumeGroupRef} }{label} ) {
          #  $conf{$id}{pool_name} = $data{conf}{POOL}{ $var->{volumeGroupRef} }{label};
          #}
          # add pool_id as a parent to Architecture
          push( @{ $arch{parents} }, $var->{volumeGroupRef} );
        }
        # WWN for mapping purposes
        if ( exists $var->{wwn} ) {
          #$conf{$id}{wwn} = Xormon::formatWWN($var->{wwn});
          $arch{wwn}      = Xormon::formatWWN($var->{wwn});
        }
        push(@architecture, \%arch);
        if ( exists $var->{volumeRef} ) { $conf{$id}{lun_id} = $var->{volumeRef}; }
      }
    }
  }

  #
  # POOLS
  #
  if ( exists $data{conf}{POOL} ) {
    foreach my $id ( sort keys %{ $data{conf}{POOL} } ) {
      my $var = $data{conf}{POOL}{$id};
      if ( exists $var->{label} ) {
        # Architecture
        my %arch = (
          "item_id"   => $id,
          "label"     => $var->{label},
          "hw_type"   => "ESERIES",
          "subsystem" => "pool",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents"   => [$cfg->{'hostcfg_id'}]
        );
        # conf
        #if ( exists $var->{worldWideName} )       { $conf{$id}{wwn}                 = $var->{worldWideName}; }
        #if ( exists $var->{driveMediaType} )      { $conf{$id}{driveMediaType}      = $var->{driveMediaType}; }
        #if ( exists $var->{drivePhysicalType} )   { $conf{$id}{drivePhysicalType}   = $var->{drivePhysicalType}; }
        #if ( exists $var->{raidLevel} )           { $conf{$id}{raidLevel}           = $var->{raidLevel}; }
        #if ( exists $var->{volumeGroupDataType} ) { $conf{$id}{volumeGroupDataType} = $var->{volumeGroupDataType}; }
        #if ( exists $var->{totalRaidedSpace} )    { $conf{$id}{totalRaidedSpace}    = $var->{totalRaidedSpace}; }
        if ( exists $var->{freeSpace} )           { $conf{$id}{capacity_free}       = $var->{freeSpace}; }
        if ( exists $var->{usedSpace} )           { $conf{$id}{capacity_used}       = $var->{usedSpace}; }
        if ( exists $conf{$id}{capacity_free} && Xormon::isdigit($conf{$id}{capacity_free}) && exists $conf{$id}{capacity_used} && Xormon::isdigit($conf{$id}{capacity_used}) ) {
          $conf{$id}{capacity_total} = $conf{$id}{capacity_free} + $conf{$id}{capacity_used};
        }
        push(@architecture, \%arch);
      }
    }
  }

  #
  # HOSTS
  #
  if ( exists $data{conf}{HOST} ) {
    foreach my $id ( sort keys %{ $data{conf}{HOST} } ) {
      my $var = $data{conf}{HOST}{$id};
      if ( exists $var->{label} ) {
        # Architecture
        my %arch = (
          "item_id"   => $id,
          "label"     => $var->{label},
          "hw_type"   => "ESERIES",
          "subsystem" => "host",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents"   => [$cfg->{'hostcfg_id'}]
        );
        if ( exists $var->{clusterRef} ) {
          if ( $var->{clusterRef} ne '0000000000000000000000000000000000000000' ) {
            # 0000000000000000000000000000000000000000 some strange HG id, it's most probably there only when HGs are not used
            push( @{ $arch{parents} }, $var->{clusterRef} ); # parent HOST-GROUP
          }
          # add volumes assigned to the parent's host group
          if ( exists $data{conf}{MAPPING}{'HOST-GROUP'}{ $var->{clusterRef} }{VOLUME} ) {
            foreach ( keys %{ $data{conf}{MAPPING}{'HOST-GROUP'}{ $var->{clusterRef} }{VOLUME} } ) {
              push( @{ $arch{children} }, $_ ); # children VOLUMES
            }
          }
        }
        if ( exists $data{conf}{MAPPING}{HOST}{$id}{VOLUME} ) {
          foreach ( keys %{ $data{conf}{MAPPING}{HOST}{$id}{VOLUME} } ) {
            push( @{ $arch{children} }, $_ ); # children VOLUMES
          }
        }
        push(@architecture, \%arch);
      }
    }
  }

  #
  # HOST-GROUPS
  #
  if ( exists $data{conf}{'HOST-GROUP'} ) {
    foreach my $id ( sort keys %{ $data{conf}{'HOST-GROUP'} } ) {
      my $var = $data{conf}{'HOST-GROUP'}{$id};
      if ( exists $var->{label} ) {
        # Architecture
        my %arch = (
          "item_id"   => $id,
          "label"     => $var->{label},
          "hw_type"   => "ESERIES",
          "subsystem" => "host_group",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents"   => [$cfg->{'hostcfg_id'}]
        );
        if ( exists $data{conf}{MAPPING}{'HOST-GROUP'}{$id}{VOLUME} ) {
          foreach ( keys %{ $data{conf}{MAPPING}{'HOST-GROUP'}{$id}{VOLUME} } ) {
            push( @{ $arch{children} }, $_ ); # children VOLUMES
          }
        }
        push(@architecture, \%arch);
      }
    }
  }

  #
  # DRIVES
  #
  if ( exists $data{conf}{DRIVE} ) {
    foreach my $id ( sort keys %{ $data{conf}{DRIVE} } ) {
      my $var = $data{conf}{DRIVE}{$id};
      if ( exists $var->{physicalLocation}{label} ) {
        # Architecture
        my %arch = (
          "item_id"   => $id,
          "label"     => $var->{physicalLocation}{label},
          "hw_type"   => "ESERIES",
          "subsystem" => "drive",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents"   => [$cfg->{'hostcfg_id'}]
        );
        # conf
        #if ( exists $var->{serialNumber} )                       { $conf{$id}{serial}           = $var->{serialNumber}; $conf{$id}{serial} =~ s/^\s+|\s+$//g; }
        #if ( exists $var->{status} )                             { $conf{$id}{status}           = $var->{status}; }
        #if ( exists $var->{physicalLocation}{slot} )             { $conf{$id}{slot}             = $var->{physicalLocation}{slot}; }
        #if ( exists $var->{physicalLocation}{locationPosition} ) { $conf{$id}{locationPosition} = $var->{physicalLocation}{locationPosition}; }
        push(@architecture, \%arch);
      }
    }
  }

  #
  # PORTS
  #
  if ( exists $data{conf}{PORT} ) {
    foreach my $id ( sort keys %{ $data{conf}{PORT} } ) {
      my $var           = $data{conf}{PORT}{$id};
      my $channelType   = ( exists $var->{channelType} )                                        ? $var->{channelType}                                        : "";
      my $controllerRef = ( exists $var->{controllerRef} )                                      ? $var->{controllerRef}                                      : "";
      my $interfaceType = ( exists $var->{ioInterfaceTypeData}->{interfaceType} )               ? $var->{ioInterfaceTypeData}->{interfaceType}               : "";
      my $node          = ( exists $data{conf}{NODE}{$controllerRef}{physicalLocation}{label} ) ? $data{conf}{NODE}{$controllerRef}{physicalLocation}{label} : "";
      my $type          = "";
      my $channel       = "";
      my $wwn           = "";
      if ( exists $var->{ioInterfaceTypeData} ) {
        foreach my $type_act ( keys %{ $var->{ioInterfaceTypeData} } ) {
          if ( ref($var->{ioInterfaceTypeData}->{$type_act}) eq "HASH" && exists $var->{ioInterfaceTypeData}->{$type_act}->{addressId} ) {
            $wwn = $var->{ioInterfaceTypeData}->{$type_act}->{addressId};
          }
          if ( ref($var->{ioInterfaceTypeData}->{$type_act}) eq "HASH" && exists $var->{ioInterfaceTypeData}->{$type_act}->{channel} ) {
            $type    = $type_act;
            $channel = $var->{ioInterfaceTypeData}->{$type}->{channel};
            last;
          }
        }
      }

      if ( defined $node && $node ne '' && defined $channelType && $channelType ne '' && defined $channel && $channel ne '' && defined $interfaceType && $interfaceType ne '' ) {
        my $port_type = $channelType;
        $port_type    =~ s/side//;
        my $label     = "$port_type-$interfaceType-$node$channel";
        # item_id must be created, bcs eseries provides the same uuid for all eseries devices like 070000000000000000000001 or 2202020000000000000000000000000000000000
        my $port_id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."port".$id) );
        # Architecture
        my %arch = (
          "item_id"   => $port_id,
          "label"     => $label,
          "hw_type"   => "ESERIES",
          "subsystem" => "port",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents"   => [$cfg->{'hostcfg_id'}]
        );
        # conf
        #$conf{$id}{channelType}   = $channelType;
        #$conf{$id}{interfaceType} = $interfaceType;
        #$conf{$id}{node_name}     = $node;
        #$conf{$id}{type}          = $type;
        #$conf{$id}{channel}       = $channel;
        #$conf{$id}{wwn}           = $wwn;

        # add node id as a parent
        if ( defined $controllerRef && $controllerRef ne '' ) {
          push( @{ $arch{parents} }, lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."node".$controllerRef) ) );
        }
        # WWN for mapping purposes
        if ( defined $wwn && $wwn ne '' ) {
          $conf{$id}{wwn} = Xormon::formatWWN($wwn);
          $arch{wwn}      = Xormon::formatWWN($wwn);
        }

        push(@architecture, \%arch);
      }
    }
  }

  #
  # NODES
  #
  if ( exists $data{conf}{NODE} ) {
    foreach my $id ( sort keys %{ $data{conf}{NODE} } ) {
      my $var = $data{conf}{NODE}{$id};
      if ( exists $var->{physicalLocation}{label} ) {
        # item_id must be created, bcs eseries provides the same uuid for all eseries devices like 070000000000000000000001 or 2202020000000000000000000000000000000000
        my $node_id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."node".$id) );
        # Architecture
        my %arch = (
          "item_id"   => $node_id,
          "label"     => $var->{physicalLocation}{label},
          "hw_type"   => "ESERIES",
          "subsystem" => "node",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents"   => [$cfg->{'hostcfg_id'}]
        );
        # conf
        #if ( exists $var->{serialNumber} ) { $conf{$id}{serial} = $var->{serialNumber}; $conf{$id}{serial} =~ s/^\s+|\s+$//g; }
        #if ( exists $var->{status} )       { $conf{$id}{status} = $var->{status}; }
        push(@architecture, \%arch);
      }
    }
  }

  #save architecture
  my $rc_arch = $xormon->saveArchitecture(\@architecture);
  $logger->info("save architecture: $rc_arch");

  #save configuration
  my $rc_conf = $xormon->saveConfJSON(\%conf);
  $logger->info("save conf: $rc_conf");

  #print "#################### Architecture\n";
  #print Dumper @architecture;
  #print "#################### CONF\n";
  #print Dumper \%conf;

  return 1;
}

sub mapping {
  #
  # mapping between volume -> host/host-group
  #
  if ( exists $data{conf}{VOLUME} ) {
    foreach ( keys %{ $data{conf}{VOLUME} } ) {
      my $vol = $data{conf}{VOLUME}{$_};
      if ( exists $vol->{id} && exists $vol->{label} && exists $vol->{listOfMappings} && ref($vol->{listOfMappings}) eq "ARRAY" ) {
        my $volume_id   = $vol->{id};
        my $volume_name = $vol->{label};
        foreach my $mapping ( @{ $vol->{listOfMappings} } ) {
          #print Dumper $mapping;
          if ( exists $mapping->{mapRef} ) { # targed id
            my $targed_id = $mapping->{mapRef};
            #print "$vol->{id} : $vol->{label} : $mapping->{mapRef}\n";
            if ( exists $data{conf}{HOST}{$targed_id} && exists $data{conf}{HOST}{$targed_id}{label} ) {
              my $target_name = $data{conf}{HOST}{$targed_id}{label};
              #print "    HOST : $data{conf}{HOST}{ $mapping->{mapRef} }{label}\n";
              $data{conf}{MAPPING}{VOLUME}{$volume_id}{label}            = $vol->{label};
              $data{conf}{MAPPING}{VOLUME}{$volume_id}{HOST}{$targed_id} = $target_name;
              $data{conf}{MAPPING}{HOST}{$targed_id}{label}              = $target_name;
              $data{conf}{MAPPING}{HOST}{$targed_id}{VOLUME}{$volume_id} = $volume_name;
            }
            if ( exists $data{conf}{'HOST-GROUP'}{$targed_id} && exists $data{conf}{'HOST-GROUP'}{$targed_id}{label} ) {
              my $target_name = $data{conf}{'HOST-GROUP'}{$targed_id}{label};
              #print "    HOST-GROUP : $data{conf}{'HOST-GROUP'}{ $mapping->{mapRef} }{label}\n";
              $data{conf}{MAPPING}{VOLUME}{$volume_id}{label}            = $vol->{label};
              $data{conf}{MAPPING}{VOLUME}{$volume_id}{'HOST-GROUP'}{$targed_id} = $target_name;
              $data{conf}{MAPPING}{'HOST-GROUP'}{$targed_id}{label}              = $target_name;
              $data{conf}{MAPPING}{'HOST-GROUP'}{$targed_id}{VOLUME}{$volume_id} = $volume_name;
            }
          }
        }
      }
    }
  }
  #print Dumper \%{ $data{conf}{MAPPING} };

  return 1;
}

sub get_workload_cfg {
  my $result = new_get_request("/devmgr/v2/storage-systems/$data{conf}{DEVICE}{id}/workloads");

  if ( defined $result && $result ne "0" && ref($result) eq "ARRAY" ) {
    foreach ( @{ $result } ) {
      if ( exists $_->{id} ) {
        $data{conf}{'WORKLOAD'}{ $_->{id} } = $_;
      }
    }
  }

  return 1;
}

sub get_pool_cfg {
  my $result = new_get_request("/devmgr/v2/storage-systems/$data{conf}{DEVICE}{id}/storage-pools");

  if ( defined $result && $result ne "0" && ref($result) eq "ARRAY" ) {
    foreach ( @{ $result } ) {
      if ( exists $_->{id} ) {
        $data{conf}{'POOL'}{ $_->{id} } = $_;
      }
    }
  }

  return 1;
}

sub get_ports_cfg {
  my $result = new_get_request("/devmgr/v2/storage-systems/$data{conf}{DEVICE}{id}/interfaces");

  if ( defined $result && $result ne "0" && ref($result) eq "ARRAY" ) {
    foreach ( @{ $result } ) {
      if ( exists $_->{interfaceRef} ) {
        $data{conf}{'PORT'}{ $_->{interfaceRef} } = $_;
      }
    }
  }

  return 1;
}

sub get_drives_cfg {
  my $result = new_get_request("/devmgr/v2/storage-systems/$data{conf}{DEVICE}{id}/drives");

  if ( defined $result && $result ne "0" && ref($result) eq "ARRAY" ) {
    foreach ( @{ $result } ) {
      if ( exists $_->{id} ) {
        $data{conf}{'DRIVE'}{ $_->{id} } = $_;
      }
    }
  }

  return 1;
}

sub get_nodes_cfg {
  my $result = new_get_request("/devmgr/v2/storage-systems/$data{conf}{DEVICE}{id}/controllers");

  if ( defined $result && $result ne "0" && ref($result) eq "ARRAY" ) {
    foreach ( @{ $result } ) {
      if ( exists $_->{id} ) {
        $data{conf}{'NODE'}{ $_->{id} } = $_;
      }
    }
  }

  return 1;
}

sub get_volumes_cfg {
  my $result = new_get_request("/devmgr/v2/storage-systems/$data{conf}{DEVICE}{id}/volumes");

  if ( defined $result && $result ne "0" && ref($result) eq "ARRAY" ) {
    foreach ( @{ $result } ) {
      if ( exists $_->{id} ) {
        $data{conf}{VOLUME}{ $_->{id} } = $_;
      }
    }
  }

  $result = new_get_request("/devmgr/v2/storage-systems/$data{conf}{DEVICE}{id}/thin-volumes");

  if ( defined $result && $result ne "0" && ref($result) eq "ARRAY" ) {
    foreach ( @{ $result } ) {
      if ( exists $_->{id} ) {
        $data{conf}{VOLUME}{ $_->{id} } = $_;
      }
    }
  }

  return 1;
}

sub get_host_groups_cfg {
  my $result = new_get_request("/devmgr/v2/storage-systems/$data{conf}{DEVICE}{id}/host-groups");

  if ( defined $result && $result ne "0" && ref($result) eq "ARRAY" ) {
    foreach ( @{ $result } ) {
      if ( exists $_->{id} ) {
        $data{conf}{'HOST-GROUP'}{ $_->{id} } = $_;
      }
    }
  }

  return 1;
}

sub get_hosts_cfg {
  my $result = new_get_request("/devmgr/v2/storage-systems/$data{conf}{DEVICE}{id}/hosts");

  if ( defined $result && $result ne "0" && ref($result) eq "ARRAY" ) {
    foreach ( @{ $result } ) {
      if ( exists $_->{id} ) {
        $data{conf}{HOST}{ $_->{id} } = $_;
      }
    }
  }

  return 1;
}

sub prepare_perf {
  my $timestamp = shift;

  my %data_out;

  #applicationStats
  #controllerStats
  #diskStats
  #interfaceStats
  #poolStats
  #systemStats
  #volumeStats
  #workloadStats

  if ( exists $cfg->{debug} && $cfg->{debug} eq "all_stats" ) {
    $logger->info("debug mode=$cfg->{debug} : ".localtime($timestamp)." : $timestamp\n");
    $logger->info("raw stats:\n");
    $logger->info(Dumper \%{ $data{'perf'}{$timestamp} });
    $logger->info("############################################\n");
  }

  # VOLUMES
  if ( exists $data{'perf'}{$timestamp}{volumeStats} && ref($data{'perf'}{$timestamp}{volumeStats}) eq "ARRAY" ) {
    foreach ( @{ $data{'perf'}{$timestamp}{volumeStats} } ) {
      if ( exists $_->{volumeId} ) {
        my $id = $_->{volumeId};

        if ( exists $_->{readThroughput}        && Xormon::isdigit($_->{readThroughput}) )        { $data_out{eseries}{volume}{$id}{$timestamp}{read}          = Xormon::returnBytes($_->{readThroughput},"MB"); }
        if ( exists $_->{readIOps}              && Xormon::isdigit($_->{readIOps}) )              { $data_out{eseries}{volume}{$id}{$timestamp}{read_io}       = $_->{readIOps}; }
        if ( exists $_->{readResponseTime}      && Xormon::isdigit($_->{readResponseTime}) )      { $data_out{eseries}{volume}{$id}{$timestamp}{resp_t_r}      = $_->{readResponseTime}; }
        if ( exists $_->{writeThroughput}       && Xormon::isdigit($_->{writeThroughput}) )       { $data_out{eseries}{volume}{$id}{$timestamp}{write}         = Xormon::returnBytes($_->{writeThroughput},"MB"); }
        if ( exists $_->{writeIOps}             && Xormon::isdigit($_->{writeIOps}) )             { $data_out{eseries}{volume}{$id}{$timestamp}{write_io}      = $_->{writeIOps}; }
        if ( exists $_->{writeResponseTime}     && Xormon::isdigit($_->{writeResponseTime}) )     { $data_out{eseries}{volume}{$id}{$timestamp}{resp_t_w}      = $_->{writeResponseTime}; }
        # we show r_cache_hit in %, so we have to use readCacheUtilization as a r_cache_hit and readHitBytes as a r_cache_usage
        if ( exists $_->{readHitBytes}          && Xormon::isdigit($_->{readHitBytes}) )          { $data_out{eseries}{volume}{$id}{$timestamp}{r_cache_usage} = $_->{readHitBytes}; }
        if ( exists $_->{readCacheUtilization}  && Xormon::isdigit($_->{readCacheUtilization}) )  { $data_out{eseries}{volume}{$id}{$timestamp}{r_cache_hit}   = $_->{readCacheUtilization}; }
        if ( exists $_->{writeHitBytes}         && Xormon::isdigit($_->{writeHitBytes}) )         { $data_out{eseries}{volume}{$id}{$timestamp}{w_cache_usage} = $_->{writeHitBytes}; }
        if ( exists $_->{writeCacheUtilization} && Xormon::isdigit($_->{writeCacheUtilization}) ) { $data_out{eseries}{volume}{$id}{$timestamp}{w_cache_hit}   = $_->{writeCacheUtilization}; }

        # calculate missing metrics
        if ( exists $data_out{eseries}{volume}{$id}{$timestamp}{read} && exists $data_out{eseries}{volume}{$id}{$timestamp}{write} ) {
          $data_out{eseries}{volume}{$id}{$timestamp}{data_rate} = $data_out{eseries}{volume}{$id}{$timestamp}{read} + $data_out{eseries}{volume}{$id}{$timestamp}{write};
        }
        if ( exists $data_out{eseries}{volume}{$id}{$timestamp}{read_io} && exists $data_out{eseries}{volume}{$id}{$timestamp}{write_io} ) {
          $data_out{eseries}{volume}{$id}{$timestamp}{io_rate} = $data_out{eseries}{volume}{$id}{$timestamp}{read_io} + $data_out{eseries}{volume}{$id}{$timestamp}{write_io};
          if ( exists $data_out{eseries}{volume}{$id}{$timestamp}{resp_t_r} && exists $data_out{eseries}{volume}{$id}{$timestamp}{resp_t_w} && $data_out{eseries}{volume}{$id}{$timestamp}{io_rate} > 0 ) {
            $data_out{eseries}{volume}{$id}{$timestamp}{resp_t} = ( ( $data_out{eseries}{volume}{$id}{$timestamp}{resp_t_r} * $data_out{eseries}{volume}{$id}{$timestamp}{read_io} ) + ( $data_out{eseries}{volume}{$id}{$timestamp}{resp_t_w} * $data_out{eseries}{volume}{$id}{$timestamp}{write_io} ) ) / $data_out{eseries}{volume}{$id}{$timestamp}{io_rate};
          }
        }
        # calculate block size
        if ( exists $data_out{eseries}{volume}{$id}{$timestamp}{io_rate} && Xormon::isdigit($data_out{eseries}{volume}{$id}{$timestamp}{io_rate}) && exists $data_out{eseries}{volume}{$id}{$timestamp}{data_rate} ) {
          if ( $data_out{eseries}{volume}{$id}{$timestamp}{io_rate} == 0 ) {
            $data_out{eseries}{volume}{$id}{$timestamp}{block_size} = 0;
          }
          else {
            $data_out{eseries}{volume}{$id}{$timestamp}{block_size} = Xormon::formatNumberByType("float4",$data_out{eseries}{volume}{$id}{$timestamp}{data_rate} / $data_out{eseries}{volume}{$id}{$timestamp}{io_rate});
          }
        }
        if ( exists $data_out{eseries}{volume}{$id}{$timestamp}{read_io} && Xormon::isdigit($data_out{eseries}{volume}{$id}{$timestamp}{read_io}) && exists $data_out{eseries}{volume}{$id}{$timestamp}{read} ) {
          if ( $data_out{eseries}{volume}{$id}{$timestamp}{read_io} == 0 ) {
            $data_out{eseries}{volume}{$id}{$timestamp}{block_size_r} = 0;
          }
          else {
            $data_out{eseries}{volume}{$id}{$timestamp}{block_size_r} = Xormon::formatNumberByType("float4",$data_out{eseries}{volume}{$id}{$timestamp}{read} / $data_out{eseries}{volume}{$id}{$timestamp}{read_io});
          }
        }
        if ( exists $data_out{eseries}{volume}{$id}{$timestamp}{write_io} && Xormon::isdigit($data_out{eseries}{volume}{$id}{$timestamp}{write_io}) && exists $data_out{eseries}{volume}{$id}{$timestamp}{write} ) {
          if ( $data_out{eseries}{volume}{$id}{$timestamp}{write_io} == 0 ) {
            $data_out{eseries}{volume}{$id}{$timestamp}{block_size_w} = 0;
          }
          else {
            $data_out{eseries}{volume}{$id}{$timestamp}{block_size_w} = Xormon::formatNumberByType("float4",$data_out{eseries}{volume}{$id}{$timestamp}{write} / $data_out{eseries}{volume}{$id}{$timestamp}{write_io});
          }
        }
        # add capacity
        if ( exists $data{conf}{VOLUME}{$id}{capacity} ) { $data_out{eseries}{volume}{$id}{$timestamp}{capacity_total} = $data{conf}{VOLUME}{$id}{capacity}; }
        #print "VOLUME: $id\n";
        #print Dumper \%{ $perf{$id} };
      }
    }
  }
  # POOLS
  if ( exists $data{'perf'}{$timestamp}{poolStats} && ref($data{'perf'}{$timestamp}{poolStats}) eq "ARRAY" ) {
    foreach ( @{ $data{'perf'}{$timestamp}{poolStats} } ) {
      if ( exists $_->{poolId} ) {
        my $id = $_->{poolId};

        if ( exists $_->{readThroughput}    && Xormon::isdigit($_->{readThroughput}) )    { $data_out{eseries}{pool}{$id}{$timestamp}{read}          = Xormon::returnBytes($_->{readThroughput},"MB"); }
        if ( exists $_->{readIOps}          && Xormon::isdigit($_->{readIOps}) )          { $data_out{eseries}{pool}{$id}{$timestamp}{read_io}       = $_->{readIOps}; }
        if ( exists $_->{readResponseTime}  && Xormon::isdigit($_->{readResponseTime}) )  { $data_out{eseries}{pool}{$id}{$timestamp}{resp_t_r}      = $_->{readResponseTime}; }
        if ( exists $_->{writeThroughput}   && Xormon::isdigit($_->{writeThroughput}) )   { $data_out{eseries}{pool}{$id}{$timestamp}{write}         = Xormon::returnBytes($_->{writeThroughput},"MB"); }
        if ( exists $_->{writeIOps}         && Xormon::isdigit($_->{writeIOps}) )         { $data_out{eseries}{pool}{$id}{$timestamp}{write_io}      = $_->{writeIOps}; }
        if ( exists $_->{writeResponseTime} && Xormon::isdigit($_->{writeResponseTime}) ) { $data_out{eseries}{pool}{$id}{$timestamp}{resp_t_w}      = $_->{writeResponseTime}; }

        # use capacity information from conf bcs these stats are missing in perf
        if ( exists $data{conf}{POOL}{$id}{freeSpace} && Xormon::isdigit($data{conf}{POOL}{$id}{freeSpace}) ) { $data_out{eseries}{pool}{$id}{$timestamp}{capacity_free} = $data{conf}{POOL}{$id}{freeSpace}; }
        if ( exists $data{conf}{POOL}{$id}{usedSpace} && Xormon::isdigit($data{conf}{POOL}{$id}{usedSpace}) ) { $data_out{eseries}{pool}{$id}{$timestamp}{capacity_used} = $data{conf}{POOL}{$id}{usedSpace}; }

        # calculate missing metrics
        if ( exists $data_out{eseries}{pool}{$id}{$timestamp}{read} && exists $data_out{eseries}{pool}{$id}{$timestamp}{write} ) {
          $data_out{eseries}{pool}{$id}{$timestamp}{data_rate} = $data_out{eseries}{pool}{$id}{$timestamp}{read} + $data_out{eseries}{pool}{$id}{$timestamp}{write};
        }
        if ( exists $data_out{eseries}{pool}{$id}{$timestamp}{read_io} && exists $data_out{eseries}{pool}{$id}{$timestamp}{write_io} ) {
          $data_out{eseries}{pool}{$id}{$timestamp}{io_rate} = $data_out{eseries}{pool}{$id}{$timestamp}{read_io} + $data_out{eseries}{pool}{$id}{$timestamp}{write_io};
          if ( exists $data_out{eseries}{pool}{$id}{$timestamp}{resp_t_r} && exists $data_out{eseries}{pool}{$id}{$timestamp}{resp_t_w} && $data_out{eseries}{pool}{$id}{$timestamp}{io_rate} > 0 ) {
            $data_out{eseries}{pool}{$id}{$timestamp}{resp_t} = ( ( $data_out{eseries}{pool}{$id}{$timestamp}{resp_t_r} * $data_out{eseries}{pool}{$id}{$timestamp}{read_io} ) + ( $data_out{eseries}{pool}{$id}{$timestamp}{resp_t_w} * $data_out{eseries}{pool}{$id}{$timestamp}{write_io} ) ) / $data_out{eseries}{pool}{$id}{$timestamp}{io_rate};
          }
        }
        if ( exists $data_out{eseries}{pool}{$id}{$timestamp}{capacity_free} && exists $data_out{eseries}{pool}{$id}{$timestamp}{capacity_used} ) {
          $data_out{eseries}{pool}{$id}{$timestamp}{capacity_total}        = $data_out{eseries}{pool}{$id}{$timestamp}{capacity_free} + $data_out{eseries}{pool}{$id}{$timestamp}{capacity_used};
          $data_out{eseries}{pool}{$id}{$timestamp}{capacity_used_percent} = $data_out{eseries}{pool}{$id}{$timestamp}{capacity_used} / ( $data_out{eseries}{pool}{$id}{$timestamp}{capacity_total} / 100 );
        }
        # calculate block size
        if ( exists $data_out{eseries}{pool}{$id}{$timestamp}{io_rate} && Xormon::isdigit($data_out{eseries}{pool}{$id}{$timestamp}{io_rate}) && $data_out{eseries}{pool}{$id}{$timestamp}{io_rate} != 0 && exists $data_out{eseries}{pool}{$id}{$timestamp}{data_rate} ) {
          $data_out{eseries}{pool}{$id}{$timestamp}{block_size} = Xormon::formatNumberByType("float4",$data_out{eseries}{pool}{$id}{$timestamp}{data_rate} / $data_out{eseries}{pool}{$id}{$timestamp}{io_rate});
        }
        if ( exists $data_out{eseries}{pool}{$id}{$timestamp}{read_io} && Xormon::isdigit($data_out{eseries}{pool}{$id}{$timestamp}{read_io}) && $data_out{eseries}{pool}{$id}{$timestamp}{read_io} != 0 && exists $data_out{eseries}{pool}{$id}{$timestamp}{read} ) {
          $data_out{eseries}{pool}{$id}{$timestamp}{block_size_r} = Xormon::formatNumberByType("float4",$data_out{eseries}{pool}{$id}{$timestamp}{read} / $data_out{eseries}{pool}{$id}{$timestamp}{read_io});
        }
        if ( exists $data_out{eseries}{pool}{$id}{$timestamp}{write_io} && Xormon::isdigit($data_out{eseries}{pool}{$id}{$timestamp}{write_io}) && $data_out{eseries}{pool}{$id}{$timestamp}{write_io} != 0 && exists $data_out{eseries}{pool}{$id}{$timestamp}{write} ) {
          $data_out{eseries}{pool}{$id}{$timestamp}{block_size_w} = Xormon::formatNumberByType("float4",$data_out{eseries}{pool}{$id}{$timestamp}{write} / $data_out{eseries}{pool}{$id}{$timestamp}{write_io});
        }
        #print "POOL: $id\n";
        #print Dumper \%{ $perf{$id} };
      }
    }
  }
  # NODES
  if ( exists $data{'perf'}{$timestamp}{controllerStats} && ref($data{'perf'}{$timestamp}{controllerStats}) eq "ARRAY" ) {
    foreach ( @{ $data{'perf'}{$timestamp}{controllerStats} } ) {
      if ( exists $_->{controllerId} ) {
        #my $id = $_->{controllerId};
        # item_id must be created, bcs eseries provides the same uuid for all eseries devices like 070000000000000000000001 or 2202020000000000000000000000000000000000
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."node".$_->{controllerId}) );

        if ( exists $_->{readThroughput}    && Xormon::isdigit($_->{readThroughput}) )    { $data_out{eseries}{node}{$id}{$timestamp}{read}       = Xormon::returnBytes($_->{readThroughput},"MB"); }
        if ( exists $_->{readIOps}          && Xormon::isdigit($_->{readIOps}) )          { $data_out{eseries}{node}{$id}{$timestamp}{read_io}    = $_->{readIOps}; }
        if ( exists $_->{readResponseTime}  && Xormon::isdigit($_->{readResponseTime}) )  { $data_out{eseries}{node}{$id}{$timestamp}{resp_t_r}   = $_->{readResponseTime}; }
        if ( exists $_->{writeThroughput}   && Xormon::isdigit($_->{writeThroughput}) )   { $data_out{eseries}{node}{$id}{$timestamp}{write}      = Xormon::returnBytes($_->{writeThroughput},"MB"); }
        if ( exists $_->{writeIOps}         && Xormon::isdigit($_->{writeIOps}) )         { $data_out{eseries}{node}{$id}{$timestamp}{write_io}   = $_->{writeIOps}; }
        if ( exists $_->{writeResponseTime} && Xormon::isdigit($_->{writeResponseTime}) ) { $data_out{eseries}{node}{$id}{$timestamp}{resp_t_w}   = $_->{writeResponseTime}; }
        if ( exists $_->{cpuAvgUtilization} && Xormon::isdigit($_->{cpuAvgUtilization}) ) { $data_out{eseries}{node}{$id}{$timestamp}{cpu_system} = $_->{cpuAvgUtilization}; }

        # calculate missing metrics
        if ( exists $data_out{eseries}{node}{$id}{$timestamp}{read} && exists $data_out{eseries}{node}{$id}{$timestamp}{write} ) {
          $data_out{eseries}{node}{$id}{$timestamp}{data_rate} = $data_out{eseries}{node}{$id}{$timestamp}{read} + $data_out{eseries}{node}{$id}{$timestamp}{write};
        }
        if ( exists $data_out{eseries}{node}{$id}{$timestamp}{read_io} && exists $data_out{eseries}{node}{$id}{$timestamp}{write_io} ) {
          $data_out{eseries}{node}{$id}{$timestamp}{io_rate} = $data_out{eseries}{node}{$id}{$timestamp}{read_io} + $data_out{eseries}{node}{$id}{$timestamp}{write_io};
          if ( exists $data_out{eseries}{node}{$id}{$timestamp}{resp_t_r} && exists $data_out{eseries}{node}{$id}{$timestamp}{resp_t_w} && $data_out{eseries}{node}{$id}{$timestamp}{io_rate} > 0 ) {
            $data_out{eseries}{node}{$id}{$timestamp}{resp_t} = ( ( $data_out{eseries}{node}{$id}{$timestamp}{resp_t_r} * $data_out{eseries}{node}{$id}{$timestamp}{read_io} ) + ( $data_out{eseries}{node}{$id}{$timestamp}{resp_t_w} * $data_out{eseries}{node}{$id}{$timestamp}{write_io} ) ) / $data_out{eseries}{node}{$id}{$timestamp}{io_rate};
          }
        }
        #print "NODE: $id\n";
        #print Dumper \%{ $perf{$id} };
      }
    }
  }
  # DRIVES
  if ( exists $data{'perf'}{$timestamp}{diskStats} && ref($data{'perf'}{$timestamp}{diskStats}) eq "ARRAY" ) {
    foreach ( @{ $data{'perf'}{$timestamp}{diskStats} } ) {
      if ( exists $_->{diskId} ) {
        my $id = $_->{diskId};

        if ( exists $_->{readThroughput}    && Xormon::isdigit($_->{readThroughput}) )    { $data_out{eseries}{drive}{$id}{$timestamp}{read}     = Xormon::returnBytes($_->{readThroughput},"MB"); }
        if ( exists $_->{readIOps}          && Xormon::isdigit($_->{readIOps}) )          { $data_out{eseries}{drive}{$id}{$timestamp}{read_io}  = $_->{readIOps}; }
        if ( exists $_->{readResponseTime}  && Xormon::isdigit($_->{readResponseTime}) )  { $data_out{eseries}{drive}{$id}{$timestamp}{resp_t_r} = $_->{readResponseTime}; }
        if ( exists $_->{writeThroughput}   && Xormon::isdigit($_->{writeThroughput}) )   { $data_out{eseries}{drive}{$id}{$timestamp}{write}    = Xormon::returnBytes($_->{writeThroughput},"MB"); }
        if ( exists $_->{writeIOps}         && Xormon::isdigit($_->{writeIOps}) )         { $data_out{eseries}{drive}{$id}{$timestamp}{write_io} = $_->{writeIOps}; }
        if ( exists $_->{writeResponseTime} && Xormon::isdigit($_->{writeResponseTime}) ) { $data_out{eseries}{drive}{$id}{$timestamp}{resp_t_w} = $_->{writeResponseTime}; }

        # calculate missing metrics
        if ( exists $data_out{eseries}{drive}{$id}{$timestamp}{read} && exists $data_out{eseries}{drive}{$id}{$timestamp}{write} ) {
          $data_out{eseries}{drive}{$id}{$timestamp}{data_rate} = $data_out{eseries}{drive}{$id}{$timestamp}{read} + $data_out{eseries}{drive}{$id}{$timestamp}{write};
        }
        if ( exists $data_out{eseries}{drive}{$id}{$timestamp}{read_io} && exists $data_out{eseries}{drive}{$id}{$timestamp}{write_io} ) {
          $data_out{eseries}{drive}{$id}{$timestamp}{io_rate} = $data_out{eseries}{drive}{$id}{$timestamp}{read_io} + $data_out{eseries}{drive}{$id}{$timestamp}{write_io};
          if ( exists $data_out{eseries}{drive}{$id}{$timestamp}{resp_t_r} && exists $data_out{eseries}{drive}{$id}{$timestamp}{resp_t_w} && $data_out{eseries}{drive}{$id}{$timestamp}{io_rate} > 0 ) {
            $data_out{eseries}{drive}{$id}{$timestamp}{resp_t} = ( ( $data_out{eseries}{drive}{$id}{$timestamp}{resp_t_r} * $data_out{eseries}{drive}{$id}{$timestamp}{read_io} ) + ( $data_out{eseries}{drive}{$id}{$timestamp}{resp_t_w} * $data_out{eseries}{drive}{$id}{$timestamp}{write_io} ) ) / $data_out{eseries}{drive}{$id}{$timestamp}{io_rate};
          }
        }
        #print "DRIVE: $id\n";
        #print Dumper \%{ $perf{$id} };
      }
    }
  }
  # PORTS
  if ( exists $data{'perf'}{$timestamp}{interfaceStats} && ref($data{'perf'}{$timestamp}{interfaceStats}) eq "ARRAY" ) {
    foreach ( @{ $data{'perf'}{$timestamp}{interfaceStats} } ) {
      if ( exists $_->{interfaceId} ) {
        #my $id = $_->{interfaceId};
        # item_id must be created, bcs eseries provides the same uuid for all eseries devices like 070000000000000000000001 or 2202020000000000000000000000000000000000
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."port".$_->{interfaceId}) );

        if ( exists $_->{readThroughput}     && Xormon::isdigit($_->{readThroughput}) )     { $data_out{eseries}{port}{$id}{$timestamp}{read}       = Xormon::returnBytes($_->{readThroughput},"MB"); }
        if ( exists $_->{readIOps}           && Xormon::isdigit($_->{readIOps}) )           { $data_out{eseries}{port}{$id}{$timestamp}{read_io}    = $_->{readIOps}; }
        if ( exists $_->{readResponseTime}   && Xormon::isdigit($_->{readResponseTime}) )   { $data_out{eseries}{port}{$id}{$timestamp}{resp_t_r}   = $_->{readResponseTime}; }
        if ( exists $_->{writeThroughput}    && Xormon::isdigit($_->{writeThroughput}) )    { $data_out{eseries}{port}{$id}{$timestamp}{write}      = Xormon::returnBytes($_->{writeThroughput},"MB"); }
        if ( exists $_->{writeIOps}          && Xormon::isdigit($_->{writeIOps}) )          { $data_out{eseries}{port}{$id}{$timestamp}{write_io}   = $_->{writeIOps}; }
        if ( exists $_->{writeResponseTime}  && Xormon::isdigit($_->{writeResponseTime}) )  { $data_out{eseries}{port}{$id}{$timestamp}{resp_t_w}   = $_->{writeResponseTime}; }
        if ( exists $_->{channelErrorCounts} && Xormon::isdigit($_->{channelErrorCounts}) ) { $data_out{eseries}{port}{$id}{$timestamp}{error_rate} = $_->{channelErrorCounts}; }

        # calculate missing metrics
        if ( exists $data_out{eseries}{port}{$id}{$timestamp}{read} && exists $data_out{eseries}{port}{$id}{$timestamp}{write} ) {
          $data_out{eseries}{port}{$id}{$timestamp}{data_rate} = $data_out{eseries}{port}{$id}{$timestamp}{read} + $data_out{eseries}{port}{$id}{$timestamp}{write};
        }
        if ( exists $data_out{eseries}{port}{$id}{$timestamp}{read_io} && exists $data_out{eseries}{port}{$id}{$timestamp}{write_io} ) {
          $data_out{eseries}{port}{$id}{$timestamp}{io_rate} = $data_out{eseries}{port}{$id}{$timestamp}{read_io} + $data_out{eseries}{port}{$id}{$timestamp}{write_io};
          if ( exists $data_out{eseries}{port}{$id}{$timestamp}{resp_t_r} && exists $data_out{eseries}{port}{$id}{$timestamp}{resp_t_w} && $data_out{eseries}{port}{$id}{$timestamp}{io_rate} > 0 ) {
            $data_out{eseries}{port}{$id}{$timestamp}{resp_t} = ( ( $data_out{eseries}{port}{$id}{$timestamp}{resp_t_r} * $data_out{eseries}{port}{$id}{$timestamp}{read_io} ) + ( $data_out{eseries}{port}{$id}{$timestamp}{resp_t_w} * $data_out{eseries}{port}{$id}{$timestamp}{write_io} ) ) / $data_out{eseries}{port}{$id}{$timestamp}{io_rate};
          }
        }
        #print "PORT: $id\n";
        #print Dumper \%{ $perf{$id} };
      }
    }
  }
  # DEVICE (aggregated from pools)
  if ( exists $data_out{eseries}{pool} ) {
    my $dev_var = {};
    my $idx     = 0;
    foreach my $pool_id ( keys %{ $data_out{eseries}{pool} } ) {
      if ( exists $data_out{eseries}{pool}{$pool_id}{$timestamp} ) {
        my $pool_var = $data_out{eseries}{pool}{$pool_id}{$timestamp};
        foreach my $metric ( keys %{ $pool_var } ) {
          if ( $metric =~ m/block_size/ )           { next; } # skip block_size metrics
          if ( $metric eq "capacity_used_percent" ) { next; }
          $idx++;
          if ( $metric eq "resp_t" ) {
            if ( exists $pool_var->{io_rate} && $pool_var->{io_rate} > 0 ) {
              if ( exists $dev_var->{$metric} ) {
                $dev_var->{$metric} += $pool_var->{$metric} * $pool_var->{io_rate};
              }
              else {
                $dev_var->{$metric} = $pool_var->{$metric} * $pool_var->{io_rate};
              }
            }
          }
          elsif ( $metric eq "resp_t_r" ) {
            if ( exists $pool_var->{read_io} && $pool_var->{read_io} > 0 ) {
              if ( exists $dev_var->{$metric} ) {
                $dev_var->{$metric} += $pool_var->{$metric} * $pool_var->{read_io};
              }
              else {
                $dev_var->{$metric} = $pool_var->{$metric} * $pool_var->{read_io};
              }
            }
          }
          elsif ( $metric eq "resp_t_w" ) {
            if ( exists $dev_var->{$metric} ) {
              $dev_var->{$metric} += $pool_var->{$metric} * $pool_var->{write_io};
            }
            else {
              $dev_var->{$metric} = $pool_var->{$metric} * $pool_var->{write_io};
            }
          }
          else {
            if ( exists $dev_var->{$metric} ) {
              $dev_var->{$metric} += $pool_var->{$metric};
            }
            else {
              $dev_var->{$metric} = $pool_var->{$metric};
            }
          }
        }
      }
    }
    if ( exists $dev_var->{resp_t} ) {
      if ( $dev_var->{io_rate} > 0 ) {
        $dev_var->{resp_t} = $dev_var->{resp_t}   / $dev_var->{io_rate};
      }
      else {
        $dev_var->{resp_t} = 0;
      }
    }
    if ( exists $dev_var->{resp_t_r} ) {
      if ( $dev_var->{read_io} > 0 ) {
        $dev_var->{resp_t_r} = $dev_var->{resp_t_r}   / $dev_var->{read_io};
      }
      else {
        $dev_var->{resp_t_r} = 0;
      }
    }
    if ( exists $dev_var->{resp_t_w} ) {
      if ( $dev_var->{write_io} > 0 ) {
        $dev_var->{resp_t_w} = $dev_var->{resp_t_w}   / $dev_var->{write_io};
      }
      else {
        $dev_var->{resp_t_w} = 0;
      }
    }
    if ( exists $dev_var->{capacity_total} && exists $dev_var->{capacity_used} ) {
      $dev_var->{capacity_used_percent} = $dev_var->{capacity_used} / ( $dev_var->{capacity_total} / 100 );
    }

    if ( $idx > 0 ) {
      $data_out{eseries}{device}{ $cfg->{'hostcfg_id'} }{$timestamp} = $dev_var;
    }
  }

  # HOSTS (aggregated from volumes)
  if ( exists $data{conf}{HOST} ) {
    foreach my $id ( sort keys %{ $data{conf}{HOST} } ) {
      my $volumes = {};
      my $vol_idx = 0;
      if ( exists $data{conf}{HOST}{$id}{clusterRef} ) {
        # add volumes assigned to the parent's host group
        my $host_group_id = $data{conf}{HOST}{$id}{clusterRef};
        if ( exists $data{conf}{MAPPING}{'HOST-GROUP'}{$host_group_id}{VOLUME} ) {
          foreach ( keys %{ $data{conf}{MAPPING}{'HOST-GROUP'}{$host_group_id}{VOLUME} } ) {
            $volumes->{$_} = $_;
            $vol_idx++;
          }
        }
      }
      if ( exists $data{conf}{MAPPING}{HOST}{$id}{VOLUME} ) {
        foreach ( keys %{ $data{conf}{MAPPING}{HOST}{$id}{VOLUME} } ) {
          $volumes->{$_} = $_;
          $vol_idx++;
        }
      }
      # aggregate data
      if ( $vol_idx > 0 ) {
        my $host_var = {};
        my $idx = 0;
        foreach my $vol_id ( %{ $volumes } ) {
          if ( exists $data_out{eseries}{volume}{$vol_id}{$timestamp} ) {
            my $vol_var = $data_out{eseries}{volume}{$vol_id}{$timestamp};
            foreach my $metric ( keys %{ $vol_var } ) {
              if ( $metric =~ m/cache/ )      { next; } # skip cache metrics
              if ( $metric =~ m/block_size/ ) { next; } # skip block_size metrics
              $idx++;
              if ( $metric eq "resp_t" ) {
                if ( exists $vol_var->{io_rate} && $vol_var->{io_rate} > 0 ) {
                  if ( exists $host_var->{$metric} ) {
                    $host_var->{$metric} += $vol_var->{$metric} * $vol_var->{io_rate};
                  }
                  else {
                    $host_var->{$metric} = $vol_var->{$metric} * $vol_var->{io_rate};
                  }
                }
              }
              elsif ( $metric eq "resp_t_r" ) {
                if ( exists $vol_var->{read_io} && $vol_var->{read_io} > 0 ) {
                  if ( exists $host_var->{$metric} ) {
                    $host_var->{$metric} += $vol_var->{$metric} * $vol_var->{read_io};
                  }
                  else {
                    $host_var->{$metric} = $vol_var->{$metric} * $vol_var->{read_io};
                  }
                }
              }
              elsif ( $metric eq "resp_t_w" ) {
                if ( exists $host_var->{$metric} ) {
                  $host_var->{$metric} += $vol_var->{$metric} * $vol_var->{write_io};
                }
                else {
                  $host_var->{$metric} = $vol_var->{$metric} * $vol_var->{write_io};
                }
              }
              else {
                if ( exists $host_var->{$metric} ) {
                  $host_var->{$metric} += $vol_var->{$metric};
                }
                else {
                  $host_var->{$metric} = $vol_var->{$metric};
                }
              }
            }
          }
        }
        if ( exists $host_var->{resp_t} ) {
          if ( $host_var->{io_rate} > 0 ) {
            $host_var->{resp_t} = $host_var->{resp_t}   / $host_var->{io_rate};
          }
          else {
            $host_var->{resp_t} = 0;
          }
        }
        if ( exists $host_var->{resp_t_r} ) {
          if ( $host_var->{read_io} > 0 ) {
            $host_var->{resp_t_r} = $host_var->{resp_t_r}   / $host_var->{read_io};
          }
          else {
            $host_var->{resp_t_r} = 0;
          }
        }
        if ( exists $host_var->{resp_t_w} ) {
          if ( $host_var->{write_io} > 0 ) {
            $host_var->{resp_t_w} = $host_var->{resp_t_w}   / $host_var->{write_io};
          }
          else {
            $host_var->{resp_t_w} = 0;
          }
        }

        if ( $idx > 0 ) {
          $data_out{eseries}{host}{$id}{$timestamp} = $host_var;
        }
      }
    }
  }

  #
  # HOST-GROUPS
  #
  if ( exists $data{conf}{'HOST-GROUP'} ) {
    foreach my $id ( sort keys %{ $data{conf}{'HOST-GROUP'} } ) {
      if ( exists $data{conf}{MAPPING}{'HOST-GROUP'}{$id}{VOLUME} ) {
        my $host_var = {};
        my $idx = 0;
        foreach my $vol_id ( keys %{ $data{conf}{MAPPING}{'HOST-GROUP'}{$id}{VOLUME} } ) {
          if ( exists $data_out{eseries}{volume}{$vol_id}{$timestamp} ) {
            my $vol_var = $data_out{eseries}{volume}{$vol_id}{$timestamp};
            foreach my $metric ( keys %{ $vol_var } ) {
              if ( $metric =~ m/cache/ )      { next; } # skip cache metrics
              if ( $metric =~ m/block_size/ ) { next; } # skip block_size metrics
              $idx++;
              if ( $metric eq "resp_t" ) {
                if ( exists $vol_var->{io_rate} && $vol_var->{io_rate} > 0 ) {
                  if ( exists $host_var->{$metric} ) {
                    $host_var->{$metric} += $vol_var->{$metric} * $vol_var->{io_rate};
                  }
                  else {
                    $host_var->{$metric} = $vol_var->{$metric} * $vol_var->{io_rate};
                  }
                }
              }
              elsif ( $metric eq "resp_t_r" ) {
                if ( exists $vol_var->{read_io} && $vol_var->{read_io} > 0 ) {
                  if ( exists $host_var->{$metric} ) {
                    $host_var->{$metric} += $vol_var->{$metric} * $vol_var->{read_io};
                  }
                  else {
                    $host_var->{$metric} = $vol_var->{$metric} * $vol_var->{read_io};
                  }
                }
              }
              elsif ( $metric eq "resp_t_w" ) {
                if ( exists $host_var->{$metric} ) {
                  $host_var->{$metric} += $vol_var->{$metric} * $vol_var->{write_io};
                }
                else {
                  $host_var->{$metric} = $vol_var->{$metric} * $vol_var->{write_io};
                }
              }
              else {
                if ( exists $host_var->{$metric} ) {
                  $host_var->{$metric} += $vol_var->{$metric};
                }
                else {
                  $host_var->{$metric} = $vol_var->{$metric};
                }
              }
            }
          }
        }
        if ( exists $host_var->{resp_t} ) {
          if ( $host_var->{io_rate} > 0 ) {
            $host_var->{resp_t} = $host_var->{resp_t}   / $host_var->{io_rate};
          }
          else {
            $host_var->{resp_t} = 0;
          }
        }
        if ( exists $host_var->{resp_t_r} ) {
          if ( $host_var->{read_io} > 0 ) {
            $host_var->{resp_t_r} = $host_var->{resp_t_r}   / $host_var->{read_io};
          }
          else {
            $host_var->{resp_t_r} = 0;
          }
        }
        if ( exists $host_var->{resp_t_w} ) {
          if ( $host_var->{write_io} > 0 ) {
            $host_var->{resp_t_w} = $host_var->{resp_t_w}   / $host_var->{write_io};
          }
          else {
            $host_var->{resp_t_w} = 0;
          }
        }

        if ( $idx > 0 ) {
          $data_out{eseries}{host_group}{$id}{$timestamp} = $host_var;
        }
      }
    }
  }

  if ( exists $cfg->{debug} && $cfg->{debug} eq "all_stats" ) {
    $logger->info("parsed stats:\n");
    $logger->info(Dumper \%data_out);
    $logger->info("############################################\n");
  }

  return \%data_out;
}

sub get_perf {
  my $result = new_get_request("/devmgr/v2/storage-systems/$data{conf}{DEVICE}{id}/historical-statistics?start=$cfg->{'sunix_ms'}");

  if ( defined $result && $result eq "0" ) {
    $logger->error( "WARNING: get_perf: API call 'historical-statistics' does not provide data!  $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  # parse data
  # change structure of provided stats, save it under statistic's timestamp
  foreach my $stat_type ( sort keys %{ $result } ) {
    #applicationStats
    #controllerStats
    #diskStats
    #interfaceStats
    #poolStats
    #systemStats
    #volumeStats
    #workloadStats
    if ( ref($result->{$stat_type}) eq "ARRAY" ) {
      foreach ( @{ $result->{$stat_type} } ) {
        if ( exists $_->{observedTimeInMS} && Xormon::isdigit( $_->{observedTimeInMS} ) ) {
          my $timestamp = int( $_->{observedTimeInMS} / 1000 ); # from ms to sec
          $timestamp    = int( $timestamp / 300 ) * 300;        # round timestamp to 5 mins samples

          push( @{ $data{perf}{$timestamp}{$stat_type} }, $_ );
          $stats_found++;
        }
        else {
          $logger->error( "WARNING: get_perf: Cannot save some stats, key observedTimeInMS does not exists!  $!" . __FILE__ . ":" . __LINE__ );
        }
      }
    }
  }

  return 1;
}

sub get_system_cfg {
  my $result = new_get_request("/devmgr/v2/storage-systems");

  if ( defined $result && $result eq "0" ) {
    $logger->error( "get_system_cfg failed! Cannot continue, exiting...  $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  # parse data
  if ( ref($result) eq "ARRAY" ) {
    my $dev_idx = 0;
    foreach ( @{ $result} ) {
      $data{conf}{DEVICE} = $_;
      $dev_idx++;
    }
    if ( $dev_idx > 1 ) {
      $logger->error( "WARNING: get_system_conf: API call 'storage-systems' provides more devices than expected...  $!" . __FILE__ . ":" . __LINE__ );
    }
  }

  if ( ! exists $data{conf}{DEVICE}{id} || $data{conf}{DEVICE}{id} eq "" ) {
    $logger->error( "get_system_cfg failed! Cannot continue, 'id' was not find, exiting...  $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  return 1;
}

sub new_get_request {
  my $url_path = shift;

  my $url     = "https://$cfg->{apiHost}:$cfg->{'apiPort'}" . $url_path;
  my $agent   = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( GET => $url );

  $request->header( 'Content-Type' => 'application/json' );
  $request->authorization_basic( $cfg->{'apiUsername'}, $cfg->{'apiPassword'} );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    my $res_con = $results->content;
    $logger->error("GET url      : $url");
    $logger->error("Request error: $st_line");
    $logger->error("Request error: $res_con");

    if ( $connection_test ) {
      push(@rows, "GET url      : $url\n");
      push(@rows, "Request error: $st_line\n");
      push(@rows, "Request error: $res_con\n");
    }

    return 0;
  }

  my $result = decode_json( $results->content );

  return $result;
}

1;
