use LWP;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use JSON;
use strict;
use warnings;
use Data::Dumper;
use Xormon;
use HCP;

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

# microservice port
my $DEFAULT_PORT = 3230;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);

my $server = IO::Socket->new(
  Domain => AF_INET,
  Type => SOCK_STREAM,
  Proto => 'tcp',
  LocalHost => '0.0.0.0',
  LocalPort => $port,
  ReusePort => 1,
  Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

 # waiting for a new client connection
while ($client = $server->accept()) {
  my $pid;
  unless ( defined( $pid = fork() ) ) {
    Xormon::error("Wait for fork");
    sleep 5;
  }

  if ($pid == 0) {
      # get information about a newly connected client
      my $client_address = $client->peerhost();
      my $client_port = $client->peerport();
      Xormon::log("Connection from $client_address:$client_port");

      # read up to 2048 characters from the connected client
      my $data = "";
      $client->recv($data, 2048);

      my $decoded = from_json($data);
      my $cfg     = $decoded->{data};

      Xormon::log("Received task: $decoded->{task} for Device: $cfg->{hostalias}");
      my $response = '';
      if ($decoded->{task} eq "fetch") {
        my $xormon = Xormon->new($client_address, $decoded->{jwt}, 'hcp', $cfg->{hostcfg_id}, $cfg->{hostalias});
      	$xormon->setDestinations($decoded->{'destinations'}) if defined $decoded->{'destinations'};
      	$xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

	      $client->send("Task successfuly started");
      	$client->shutdown(SHUT_WR);

	      my $status = HCP::collect($xormon,$cfg) || 'error';

        $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);
      	Xormon::log("Task [fetch] for $cfg->{hostalias} is done. Status: $status");
      } elsif ($decoded->{task} eq "perf") {
        my $xormon = Xormon->new($client_address, $decoded->{jwt}, 'hcp', $cfg->{hostcfg_id}, $cfg->{hostalias});
      	$xormon->setDestinations($decoded->{'destinations'}) if defined $decoded->{'destinations'};
      	$xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

	      $client->send("Task successfuly started");
      	$client->shutdown(SHUT_WR);

	      my $status = HCP::one_min_collect($xormon,$cfg) || 'error';

        $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);
      	Xormon::log("Task [perf] for $cfg->{hostalias} is done. Status: $status");
      } elsif ($decoded->{task} eq "test") {
        my $cfg      = $decoded->{data};
        my $response = HCP::connection_test($client, $cfg);

        $client->send($response);
        $client->shutdown(SHUT_WR);
      } else {
          $response = "undefined task";
          $client->send($response);
          $client->shutdown(SHUT_WR);
      }
      exit();
  } else {
      $client->autoflush(1);
  }
}

$server->close();
