package OCEANSTORPACIFICNAS;

use strict;
use warnings;
use Data::Dumper;
use JSON qw(encode_json decode_json);
use Date::Parse;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use POSIX ":sys_wait_h";
use LWP::UserAgent;
use File::Copy;
use File::Temp qw(tempfile);
use Test;
use XormonLogger;
use Xormon;
no warnings 'utf8';
my $logger;

my %data;
my $xormon;
my $cfg;
my $stats_found = 0;
my $data_directory = "$FindBin::Bin";
my $debug = 0;

my $storage_name = "";
my $active_minute = "";
my $debug_data = "";
my $debug_data_configuration = "";
my %performance_data;
my %configuration_data;
my $st_serial;
my @architecture = ();
my $service_data_out;
my $st_type = "oceanstorpacificnas";
my @rows_api_test = ();
my @rows_api_test_data = ();
my $protocol = "";
my $wrkdir = "";
my $service_data_in;
my $connection_test = 0;
my @health_status = ();
my %hosts;
my @hardware = ();
my $replication_capacity_used = "";
my @health_status_global = ();
my $token = "";
my $api_host = "";
my $port = 8088;
my %performance_statistics;
my %metric_inventory;
my $timestamp_global;
my $enviroment = 1;
my @pools = ();
my %nodes;
my %volumes;
my $device_id = "";
my $system_time = ();


sub collect {
  $xormon = shift;
  $cfg    = shift;

  #print Dumper \$cfg;
  #return "successful";

  #print Dumper \$cfg;
  $timestamp_global = get_global_timestamp();
  unless ( exists $cfg->{'hostcfg_id'} )        { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!") && return "error"; }
  $logger = XormonLogger->new( 'oceanstorpacificnas', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'node1Host'} )         { $logger->error("Cannot continue, 'node1Host' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiUsername'} )      { $logger->error("Cannot continue, 'apiUsername' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )      { $logger->error("Cannot continue, 'apiPassword' is not configured!") && return "error"; }
  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
  my $act_timestamp     = time();
  $service_data_in = $xormon->getData();
  #if (ref(\$service_data_in) eq "SCALAR"){
  #  $service_data_in = {};
  #}
  if ( exists $service_data_in->{last_conf_timestamp} ) {
    $service_data_out->{last_conf_timestamp} = $service_data_in->{last_conf_timestamp};
  }

  $storage_name = $cfg->{hostalias};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $st_serial = $cfg->{'uuid'};
  $api_host = $cfg->{'node1Host'};
  if (exists $service_data_in->{api_host} && $service_data_in->{api_host} ne $api_host){
    $api_host = $service_data_in->{api_host};
  }
  if (defined $cfg->{OCEANSTORPACIFICNAS_ENVIROMENT} && $cfg->{OCEANSTORPACIFICNAS_ENVIROMENT} eq "development"){
    $enviroment = 0;
  }
  data_structure();
  set_data_perf_list();
  ### login
  login();
  if (!defined $token || $token eq ""){
    if (defined $cfg->{node2Host}){
      if ($cfg->{node2Host} eq $api_host){
        $api_host = $cfg->{'node1Host'};
      }
      $api_host = $cfg->{'node2Host'};
      login();
      if (!defined $token || $token eq ""){
        $xormon->isConnected(0, "Connection failed!");
        $logger->error("Token is not defined") && return "error";
      }
    }
    else{
      $xormon->isConnected(0, "Connection failed!");
      $logger->error("Token or device_id is not defined") && return "error";
    }
  }
  ###
  ### check connection
  if (!check_connection()){
    logout();
    $xormon->isConnected(0, "Connection failed!");
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: error");
    return "error";
  }
  $service_data_out->{api_host} = $api_host;
  ###
  $xormon->isConnected(1);
  set_data_cluster();
  set_data_pool();
  set_data_node();
  set_data_drive();
  set_data_port_eth();
  set_data_device();
  #return "error";
  logout();
  delete_empty_metrics();
  calculate_missing_metrics();
  #print Dumper \%performance_data;
  my $rc = $xormon->saveJSON2(\%performance_data);
  my $status = get_status($rc, "saveJSON2 (performance)");

  ### host_cfg_data
  if (Xormon::isdigit($service_data_out->{last_perf_timestamp})){
    if ( ( time() -  $service_data_out->{last_perf_timestamp} ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: " . $service_data_out->{last_perf_timestamp});
    }
    else{
      $xormon->hasData(1);
    }
  }
  else{
    $xormon->hasData(0, "no available stats");
  }

  my $save_data = $xormon->saveData(\%{ $service_data_out });
  my $status_save_data = get_status($save_data, "SaveData");
  if (( ! exists $service_data_in->{last_conf_timestamp} ) || ( ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) ) || ( $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) || ( !-f "$debug_data_configuration/config-$st_serial.out" )){
    `touch "$debug_data_configuration/config-$st_serial.out"`;
    ### login
    login();
    if (!defined $token || $token eq "" ){
      if (defined $cfg->{node2Host}){
        if ($cfg->{node2Host} eq $api_host){
          $api_host = $cfg->{'node1Host'};
        }
        $api_host = $cfg->{'node2Host'};
        login();
        if (!defined $token || $token eq "" ){
          $logger->error("Token is not defined") && return "error";
        }
      }
      else{
        $logger->error("Token or device_id is not defined") && return "error";
      }
    }
    ###
    ### check connection
    if (!check_connection()){
      logout();
      $logger->info("Task for ".$cfg->{hostalias}." is done. Status: error");
      return "error";
    }
    set_data_cluster_conf();
    set_data_pool_conf();
    set_data_node_conf();
    set_data_port_eth_conf();
    set_data_drive_conf();
    set_data_device_conf();
    set_data_health_status();
    logout();
    my @types = ("pool", "cluster","node", "drive", "ethport", "device");
    foreach my $uid (keys %configuration_data){
      my $active = 0;
      foreach my $type (@types){
        if (defined $performance_data{$st_type}{$type}{$uid}){
          $active++;
          last;
        }
      }
      if ($active == 0){
        delete $configuration_data{$uid};
      }
    }
    $service_data_out->{last_conf_timestamp} = $act_timestamp;
    ($status, my $status_configuration, my $status_health) = set_architecture();
    if ($status eq "successful" && $status_configuration eq "successful" && $status_health eq "successful" ){
      my $save_data = $xormon->saveData(\%{ $service_data_out });
      my $status_save_data = get_status($save_data, "saveData");
    }
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
    return $status;
  }
  $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
  return $status;
}

sub set_data_cluster_conf {
  my $config_data = cmd("product", "GET",1,"product");
  my $section     = "cluster";
  if ( $config_data eq "" ) { return; }
  my $uid = Xormon::create_hash_key($st_serial.$section.$section);
  if ( ref($config_data) eq "HASH" ) {
    my $name          = $config_data->{data}->{cluster_name};
    my $version       = $config_data->{data}->{"version"};
    my $patch_version = $config_data->{data}->{"patch_version"};
    my $model_version = $config_data->{data}->{"product_model"};
    $configuration_data{$uid}{id}   = "cluster";
    $configuration_data{$uid}{TYPE}   = "cluster";
    if ( defined $name ) {
      $configuration_data{$uid}{label} = $name;
    }
    else{
      $configuration_data{$uid}{label} = "cluster";
    }
    if ( defined $version ) {
      $configuration_data{$uid}{version} = $version;
    }
    if ( defined $model_version ) {
      $configuration_data{$uid}{model} = $model_version;
    }
    $configuration_data{$uid}{serial_number} = $device_id;
  }
}


sub set_data_cluster{

  my $section     = "cluster";
  my $perf_json = create_perf_json(57347,"ALL", $performance_statistics{cluster});
  my $data = cmd("pms/performance_data", "GET", 1,"57347", $perf_json);
  if ( $data eq "" || !defined $data->{data} || ref( $data->{data} ) ne "ARRAY" ) { return; }
  my $uid = Xormon::create_hash_key($st_serial.$section.$section);
  my %inventory_data = get_calculate_performance_data( $data, $section, $section );
  if ( Xormon::isdigit( $inventory_data{timestamp} ) ) {
    $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}} = \%inventory_data;
    delete $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}}{timestamp};
  }
  else{
    delete $performance_data{$st_type}{$section}{$uid};
    return;
  }
  my $timestamp = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
  my @items = ( "object_read", "hdfs_read", "file_read" );
  $performance_data{$st_type}{$section}{$uid}{$timestamp}{read}      = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
  @items                                            = ( "object_write", "hdfs_write", "file_write" );
  $performance_data{$st_type}{$section}{$uid}{$timestamp}{write}     = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
  @items                                            = ( "object_data_rate", "hdfs_data_rate", "file_data_rate" );
  $performance_data{$st_type}{$section}{$uid}{$timestamp}{data_rate} = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
  @items                                           = ( "hdfs_read_io", "read_ops" );
  $performance_data{$st_type}{$section}{$uid}{$timestamp}{read_io}  = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
  @items                                           = ( "hdfs_write_io", "write_ops" );
  $performance_data{$st_type}{$section}{$uid}{$timestamp}{write_io} = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
  @items                                           = ( "hdfs_io_rate", "ops_rate" );
  $performance_data{$st_type}{$section}{$uid}{$timestamp}{io_rate}  = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
  #print Dumper \$performance_data{$st_type}{$section};
  
}



sub set_data_health_status{
  my $data = cmd("fms/alarms", "GET", 1, "alarm");
  #print Dumper \$data;
  if ( !defined $data || $data eq "" || ref($data->{data}) ne "ARRAY" ) {
    return;
  }

  my $message_global = "";
  my $status_global = "ok";
  my %items;
  $items{alarm_id} = "event";
  $items{create_time} = "time";
  $items{alarm_name} = "name";
  $items{alarmObjType} = "type";
  $items{severity} = "severity";
  $items{description} = "description";
  $items{suggestion} = "suggestion";

  foreach my $element ( @{$data->{data}} ) {
    if ( !defined $element->{alarm_id} || $element->{alarm_id} eq "") { next; }
    if ( defined $element->{alarmStatus} && $element->{alarmStatus} eq "2" ) {
      next;
    }
    my %data;
    foreach my $item (sort keys %items){
      if (defined $element->{$item}){
        if ($item eq "severity"){
          $data{$items{$item}} = get_level_text($element->{$item});
          if ($data{$items{$item}} =~ m/critical|major/){
            $status_global = "error";
            $message_global = $data{description};
          }
        }
        else{
          $data{$items{$item}} = $element->{$item};
        }
      }
      else{
        if ($item eq "severity"){
          $data{$items{$item}} = "unknown";
        }
        else{
          $data{$items{$item}} = "";
        }
      }
    }
    push(@health_status,\%data);
  }

  my %vm_status = (
    "item_id" => $st_serial,
    "status" => $status_global,
    "data" => $message_global
  );
  push(@health_status_global, \%vm_status);
  return;
}


sub set_data_device{
  my $type = "cluster";
  my $section = "device";
  my @items = ("read_io","write_io","read","write","io_rate", "data_rate", "nfs_resp_t", "nfs_resp_t_r", "nfs_resp_t_w");
  ### NFS response time?? to device as nfs_... or resp_t, resp_t_r, resp_t_w?
  foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$type}{$uid}}){
      foreach my $item (@items){
        my $item_new = $item;
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$timestamp}{$item})){
          if ($item =~ m/resp/){
            $item_new =~ s/nfs_//g;
          }
          $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item_new} = $performance_data{$st_type}{$type}{$uid}{$timestamp}{$item};
        }
      }
    }
  }
=pod
  @items = ("resp_t_r", "resp_t_w", "resp_t");
  foreach my $id (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$id}}){
      foreach my $item (@items){
        my $numerator = $performance_data{$st_type}{$section}{$id}{$timestamp}{"numerator_$item"};
        my $denominator = $performance_data{$st_type}{$section}{$id}{$timestamp}{"denominator_$item"};
        if (Xormon::isdigit($numerator) && Xormon::isdigit($denominator)){
          if ($denominator == 0){
            $performance_data{$st_type}{$section}{$id}{$timestamp}{$item} = 0;
          }
          else{
            $performance_data{$st_type}{$section}{$id}{$timestamp}{$item} = round($numerator / $denominator,3);
          }
        }
        if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{"numerator_$item"}){
          delete $performance_data{$st_type}{$section}{$id}{$timestamp}{"numerator_$item"};
        }
        if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{"denominator_$item"}){
          delete $performance_data{$st_type}{$section}{$id}{$timestamp}{"denominator_$item"};
        }
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};

=cut
  $type = "pool";
  $section = "device";
  @items = ("capacity_total","capacity_used","capacity_free");
  foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$type}{$uid}}){
      foreach my $item (@items){
        my $value = $performance_data{$st_type}{$type}{$uid}{$timestamp}{$item};
        if (Xormon::isdigit($value)){
          my $time_device = (keys %{$performance_data{$st_type}{$section}{$st_serial}})[0];
          if (!Xormon::isdigit($time_device)){
            $time_device = $timestamp;
          }
          if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$st_serial} && defined $performance_data{$st_type}{$section}{$st_serial}{$time_device} && defined $performance_data{$st_type}{$section}{$st_serial}{$time_device}{$item}){
            $performance_data{$st_type}{$section}{$st_serial}{$time_device}{$item} = $performance_data{$st_type}{$section}{$st_serial}{$time_device}{$item} + $value;
          }
          else{
            $performance_data{$st_type}{$section}{$st_serial}{$time_device}{$item} = $value;
          }
        }
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}

sub get_status {
  my $rc = shift;
  my $task = shift;
  if ($rc eq "201" || $rc eq "200") {
    $logger->info("Task for $task ".$cfg->{hostalias}." is done. Status: successful");
    return "successful";
  } elsif ($rc eq "401") {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: unauthorized");
    return "unauthorized";
  } else {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: error");
    return "error";
  }
}


sub set_data_pool_conf{
  my $section = "pool";
  my $config_data          = cmd("data_service/storagepool", "GET", 1, "storagepool");
  my %pools;
  if ( $config_data eq "" ) { return; }
  foreach my $element_id ( @{ $config_data->{storagePools} } ) {
    my $id  = $element_id->{storagePoolId};
    if ( !defined $id ) { next; }
    my $name = $element_id->{storagePoolName};
    if (!defined $name){$name = $id;}
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    $pools{$uid} = $id;
    $configuration_data{$uid}{id} = $id;
    $configuration_data{$uid}{label} = $name;
    $configuration_data{$uid}{TYPE} = $section;
    my %capacity = get_capacity_pool($element_id);
    foreach my $item (keys %capacity){
      $configuration_data{$uid}{$item} = $capacity{$item};
    }
  }
  #print Dumper \%configuration_data;
}
sub set_data_drive_conf{
  my $section     = "drive";
  my $param            = "";
  foreach my $pool_id (@pools) {
    $param = $param . "$pool_id,";
  }
  if ( length($param) > 0 ) {
    chop $param;
  }
  my $config_data = cmd("dsware/service/cluster/diskpool/queryNodeDiskInfo?diskPoolId=$param", "GET", 0, "disk");
  if ( $config_data eq "" ) {
    $param = "";
    foreach my $pool_id ( keys @pools ) {
      $param = $param . "diskPoolId=$pool_id&";
    }
    if ( length($param) > 0 ) {
      chop $param;
    }
    $config_data = cmd("dsware/service/cluster/diskpool/queryNodeDiskInfo?$param", "GET", 0, "disk");
  }
  if ( $config_data eq "" ) { return; }
  my @keys  = ( "diskSn", "diskSn",        "diskModel", "diskName", "diskSlot", "diskStatus", "diskType", "mediaCapacityForByte", "mediaUsedCapacityForByte", "sectorSize" );
  my @names = ( "id",     "serial_number", "model",     "label",     "slot",     "status",     "type",     "capacity_total",         "capacity_used",        "sector_size" );
  my %inventory_data;
  #print Dumper \$config_data->{nodeInfo};
  foreach my $node ( @{ $config_data->{nodeInfo} } ) {
    my $node_name = $node->{nodeName};
    if ( !defined $node->{mediaInfo} || ref( $node->{mediaInfo} ) ne "ARRAY" ) { next; }
    foreach my $element_id ( @{ $node->{mediaInfo} } ) {
      my $i = -1;
      foreach my $key (@keys) {
        $i++;
        if ( !defined $element_id->{ $keys[0] } || $element_id->{ $keys[0] } eq "" ) { next; }
        if ( defined $element_id->{$key} ) {
          if ( $key eq "wwn" ) {
            $element_id->{$key} =~ s/0x//g;
          }
          if ( $key eq "diskSn" && $names[$i] eq "id" ) {
            if ( defined $nodes{$node_name} ) {
              $inventory_data{$section}{ $element_id->{ $keys[0] } }{ $names[$i] } = "$nodes{$node_name}:$element_id->{$key}";
              next;
            }
          }
          if ( $key =~ m/diskStatus/ ) {
            $inventory_data{$section}{ $element_id->{ $keys[0] } }{ $names[$i] } = get_health_status_text( $element_id->{$key}, "disk" );
            next;
          }
          if ( $key =~ m/^type$/ ) {
            $inventory_data{$section}{ $element_id->{ $keys[0] } }{ $names[$i] } = get_type_text( $element_id->{$key} );
            next;
          }
          if ( $key =~ m/mediaCapacityForByte|mediaUsedCapacityForByte/ && Xormon::isdigit( $element_id->{$key} ) ) {
            $inventory_data{$section}{ $element_id->{ $keys[0] } }{ $names[$i] } = $element_id->{$key};
            next;
          }
          if ( $key eq "diskName" ) {
            if ( defined $nodes{$node_name} ) {
              $inventory_data{$section}{ $element_id->{ $keys[0] } }{ $names[$i] } = "$nodes{$node_name}:$element_id->{$key}";
            }
            else {
              $inventory_data{$section}{ $element_id->{ $keys[0] } }{ $names[$i] } = $element_id->{$key};
            }
            next;
          }
          $inventory_data{$section}{ $element_id->{ $keys[0] } }{ $names[$i] } = $element_id->{$key};
        }
        else {
          if ( $key eq "diskName" ) {
            if ( defined $nodes{$node_name} ) {
              $inventory_data{$section}{ $element_id->{ $keys[0] } }{ $names[$i] } = "$nodes{$node_name}:$element_id->{id}";
            }
            else {
              $inventory_data{$section}{ $element_id->{ $keys[0] } }{$key} = $element_id->{id};
            }
          }
        }
      }
    }
  }
  foreach my $serial (keys %{$inventory_data{$section}}){
    if (!defined $inventory_data{$section}{$serial}{id}){next;}
    my $uid = Xormon::create_hash_key($st_serial.$section.$inventory_data{$section}{$serial}{id});
    #print "$st_serial | $section | " . $inventory_data{$section}{$serial}{id} . "\n";
    foreach my $item (keys %{$inventory_data{$section}{$serial}}){
      $configuration_data{$uid}{$item} = $inventory_data{$section}{$serial}{$item};
      if (!defined $configuration_data{$uid}{TYPE}){
        $configuration_data{$uid}{TYPE} = $section;
      }
    }
  }
  #print Dumper \%configuration_data;
}

sub set_data_filesystem_conf{
  my $section = "fs";
  my $config_data          = cmd("Filesystem", "GET", 1, "Filesystem");
  if ( $config_data eq "" ) {
    $config_data = cmd("filesystem", "GET", 1, "filesystem");
  }
  if ( $config_data eq "" ) { return; }
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id  = $element_id->{ID};
    my $type = $element_id->{TYPE};
    my $health_status    = $element_id->{"HEALTHSTATUS"};
    my $running_status   = $element_id->{"RUNNINGSTATUS"};
    if ( !defined $id || !defined $type ) { next; }
    my $name = $element_id->{NAME};
    if (!defined $name){$name = $id;}
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    $configuration_data{$uid}{id} = $id;
    $configuration_data{$uid}{label} = $name;
    $configuration_data{$uid}{TYPE} = $section;
    my %capacity = get_capacity_section($element_id,$section);
    foreach my $item (keys %capacity){
      $configuration_data{$uid}{$item} = $capacity{$item};
    }
    if ( defined $health_status && $health_status ne "" ) {
      $health_status = get_health_status_text($health_status);
     $configuration_data{$uid}{"health_status"} = $health_status;
    }
    if ( defined $running_status && $running_status ne "" ) {
      $running_status = get_running_status_text($running_status);
      $configuration_data{$uid}{"running_status"} = $running_status;
    }
    #print Dumper \$configuration_data{$uid};
  }
  #print Dumper \%configuration_data;
}

sub sum_metrics{
  my $data_tmp = shift;
  my $uid = shift;
  my $section = shift;
  my %data = %{$data_tmp};
  foreach my $timestamp (keys %data){
    foreach my $metric (keys %{$data{$timestamp}}){
      if ($metric =~ m/capacity/){next;}
      if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$timestamp}{$metric}) ){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{$metric} = $performance_data{$st_type}{$section}{$uid}{$timestamp}{$metric} + $data{$timestamp}{$metric};
      }
      elsif ($metric =~ m/resp/){
        my $item = "io_rate";
        if ($metric =~ m/resp_t_r/){
          $item = "read_io";
        }
        elsif($metric =~ m/resp_t_w/){
          $item = "write_io";
        }
        if (Xormon::isdigit($data{$timestamp}{$item}) && Xormon::isdigit($data{$timestamp}{$metric})){
          if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$timestamp}{"numerator_$item"})){
            $performance_data{$st_type}{$section}{$uid}{$timestamp}{"numerator_$item"} += $data{$timestamp}{$item} * $data{$timestamp}{$metric};
          }
          else{
            $performance_data{$st_type}{$section}{$uid}{$timestamp}{"numerator_$item"} = $data{$timestamp}{$item} * $data{$timestamp}{$metric};
          }
        }
      }
      else{
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{$metric} = $data{$timestamp}{$metric};
      }
    }
  }
}

sub set_data_node_conf{
  my $section = "node";
  my $config_data          = cmd("servers", "GET", 1, "server");
  if ($config_data eq ""){return;}
  my @keys  = ( "id", "name", "status", "running_status", "software_version", "serial_number", "warranty_status", "model", "slot_number" );
  my @names = ( "id", "label", "status", "running_status", "software_version", "serial_number", "warranty_status", "model", "slot_number" );
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $i = -1;
    foreach my $key (@keys) {
      $i++;
      if ( !defined $element_id->{ $keys[0] } ) { next; }
      my $uid = Xormon::create_hash_key($st_serial.$section.$element_id->{ $keys[0]});
      if ( defined $element_id->{$key} ) {
        $configuration_data{$uid}{ $names[$i] } = $element_id->{$key};
        if ( $key eq "name" ) {
          $nodes{ $element_id->{$key} } = $element_id->{ $keys[0] };
        }
        if (!defined $configuration_data{$uid}{TYPE}){$configuration_data{$uid}{TYPE} = $section;}
      }
      else {
        if ( $key eq "name" ) {
          $configuration_data{$uid}{$names[$i]} = $element_id->{id};
        }
      }
    }
  }
  #print Dumper \%configuration_data;
}

sub set_data_port_eth{

  my $section     = "ethport";
  my $perf_json = create_perf_json(213,"ALL", $performance_statistics{ethport});
  my $data = cmd("pms/performance_data", "GET", 1,"213", $perf_json);
  if ( !defined $data || $data eq "" ) { return; }
  my %ports;
  foreach my $element_id ( @{ $data->{data} } ) {
    my $id_element = $element_id->{id};
    if ( !defined $id_element ) { next; }
    $ports{$id_element}{id} = $id_element;
  }
  foreach my $element_id ( keys %ports ) {
    my $uid = Xormon::create_hash_key($st_serial.$section.$element_id);
    if ( $data eq "" || !defined $data->{data} || ref( $data->{data} ) ne "ARRAY" ) { next; }
    my %inventory_data = get_calculate_performance_data( $data, $element_id, $section );
    if ( Xormon::isdigit( $inventory_data{timestamp} ) ) {
      $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}} = \%inventory_data;
      delete $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}}{timestamp};
    }
    else{
      delete $performance_data{$st_type}{$section}{$uid};
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}

sub set_data_pool {
  my $section     = "pool";
  my $config_data          = cmd("data_service/storagepool", "GET", 0, "storagepool");
  if ( $config_data eq "" ) { return; }
  foreach my $element_id ( @{ $config_data->{storagePools} } ) {
    my $id  = $element_id->{storagePoolId};
    if ( !defined $id) { next; }
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    #print "$uid | $st_serial | $section | $id\n";
    push(@pools,$id);
    my $timestamp = $system_time - 600;
    if (!$enviroment){
      $timestamp = time() - 600;
    }
    if (Xormon::isdigit($timestamp)){
      my %capacity = get_capacity_pool($element_id);
      foreach my $item (keys %capacity){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{$item} = $capacity{$item};
      }
    }
    else{
      delete $performance_data{$st_type}{$section}{$uid};
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}


sub set_data_drive {
  my $section     = "drive";
  my $param            = "";
  foreach my $pool_id (@pools) {
    $param = $param . "$pool_id,";
  }
  if ( length($param) > 0 ) {
    chop $param;
  }
  my $config_data = cmd("dsware/service/cluster/diskpool/queryNodeDiskInfo?diskPoolId=$param", "GET", 0, "disk");
  if ( $config_data eq "" ) {
    $param = "";
    foreach my $pool_id ( keys @pools ) {
      $param = $param . "diskPoolId=$pool_id&";
    }
    if ( length($param) > 0 ) {
      chop $param;
    }
    $config_data = cmd("dsware/service/cluster/diskpool/queryNodeDiskInfo?$param", "GET", 0, "disk");
  }
  if ( $config_data eq "" ) { return; }
  my $perf_json = create_perf_json(10,"ALL", $performance_statistics{drive});
  my $data = cmd("pms/performance_data", "GET", 1,"10", $perf_json);
  foreach my $element_id ( @{ $config_data->{nodeInfo} } ) {
    if ( !defined $element_id->{mediaInfo} || ref( $element_id->{mediaInfo} ) ne "ARRAY" ) { next; }
    foreach my $drive ( @{ $element_id->{mediaInfo} } ) {
      my $id_element = $drive->{diskSn};
      if ( !defined $id_element ) { next; }
      #my $uid = Xormon::create_hash_key($st_serial.$section.$id_element);
      my $total_capacity = $drive->{mediaCapacityForByte};
      my $used_capacity  = $drive->{mediaUsedCapacityForByte};
      my %inventory_data = get_calculate_performance_data( $data, $id_element, $section );
      my $keys             = keys %inventory_data;
      if ( $keys == 0 ) { next; }
      if ( Xormon::isdigit( $inventory_data{timestamp} ) && defined  $inventory_data{id} && $inventory_data{id} ne "") {
        my $uid = Xormon::create_hash_key($st_serial.$section.$inventory_data{id});
        $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}} = \%inventory_data;
        $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}}{capacity_total} = $total_capacity;
        $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}}{capacity_used} = $used_capacity;
        if (Xormon::isdigit($total_capacity) && Xormon::isdigit($used_capacity) && $total_capacity >= $used_capacity){
          $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}}{capacity_free} = $total_capacity - $used_capacity;
        }
        delete $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}}{id};
        delete $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}}{timestamp};
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}


sub set_data_node {
  my $section     = "node";
  my $config_data          = cmd("servers", "GET", 0, "server");
  if ( $config_data eq "" ) { return; }
  my $perf_json = create_perf_json(16385,"ALL", $performance_statistics{node});
  my $data = cmd("pms/performance_data", "GET", 1,"16385", $perf_json);
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id  = $element_id->{id};
    if ( !defined $id) { next; }
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    if ( $data eq "" || !defined $data->{data} || ref( $data->{data} ) ne "ARRAY" ) { next; }
    my %inventory_data = get_calculate_performance_data( $data, $id, $section );
    if ( Xormon::isdigit( $inventory_data{timestamp} ) ) {
      $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}} = \%inventory_data;
      delete $performance_data{$st_type}{$section}{$uid}{$inventory_data{timestamp}}{timestamp};      
    }
    else{
      delete $performance_data{$st_type}{$section}{$uid};
    }
    my $timestamp = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
    my @items = ( "object_read", "hdfs_read", "file_read" );
    $performance_data{$st_type}{$section}{$uid}{$timestamp}{read}      = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
    @items                                            = ( "object_write", "hdfs_write", "file_write" );
    $performance_data{$st_type}{$section}{$uid}{$timestamp}{write}     = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
    @items                                            = ( "object_data_rate", "hdfs_data_rate", "file_data_rate" );
    $performance_data{$st_type}{$section}{$uid}{$timestamp}{data_rate} = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
    @items                                           = ( "hdfs_read_io", "read_ops", "object_read_io" );
    $performance_data{$st_type}{$section}{$uid}{$timestamp}{read_io}  = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
    @items                                           = ( "hdfs_write_io", "write_ops", "object_write_io" );
    $performance_data{$st_type}{$section}{$uid}{$timestamp}{write_io} = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
    @items                                           = ( "hdfs_io_rate", "ops_rate", "object_io_rate" );
    $performance_data{$st_type}{$section}{$uid}{$timestamp}{io_rate}  = calculate_total_metric( $performance_data{$st_type}{$section}{$uid}{$timestamp}, \@items );
  }
  
  #print Dumper \$performance_data{$st_type}{$section};
}

sub set_data_port_eth_conf {
  my $section = "ethport";
  my $config_data          = cmd("dev/ports", "GET", 1, "port");
  if ($config_data eq ""){return;}
  my @keys = ( "id", "name", "mac", "status" );
  my @names = ( "id", "label", "mac", "status");
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $i = -1;
    foreach my $key (@keys) {
      $i++;
      if ( !defined $element_id->{ $keys[0] } ) { next; }
      my $uid = Xormon::create_hash_key($st_serial.$section.$element_id->{ $keys[0]});
      if ( defined $element_id->{$key} ) {
        $configuration_data{$uid}{ $names[$i] } = $element_id->{$key};
        if (!defined $configuration_data{$uid}{TYPE}){$configuration_data{$uid}{TYPE} = $section;}
        if ($key eq "name"){
          $configuration_data{$uid}{ $names[$i] } = $element_id->{id}; ### name is not unique
        }
      }
      else {
        if ( $key eq "name" ) {
          $configuration_data{$uid}{$names[$i]} = $element_id->{id};
        }
      }
    }
  }
  #print Dumper \%configuration_data;
}


sub get_capacity_pool{
  my $data = shift;
  my %data_out;
  my %convert;
  my %ratio_items;
  $convert{totalCapacity} = "capacity_total";
  $convert{usedCapacity} = "capacity_used";
  $convert{allocatedCapacity} = "capacity_allocated";
  $convert{compressionSaved} = "compression_saving";
  $convert{deduplicationSaved} = "dedupe_saving";
  #$convert{dedupCompressRatio} = "dedupe_compress_ratio";
  $convert{reductionInvolvedCapacity} = "reduction_involved_capacity_total";
  $convert{compressionRatio} = "compress_ratio";
  $convert{deduplicationRatio} = "dedupe_ratio";
  $convert{dataReductionRatio} = "data_reduction_ratio";

  #print Dumper \$data;
  foreach my $raw_item (keys %convert){
    if (Xormon::isdigit($data->{$raw_item})){
      my $multiple = 512;
      if ($raw_item =~ m/ratio/i){
        $multiple = 1;
      }
      $data_out{$convert{$raw_item}} = $multiple * $data->{$raw_item};
    }
  }
  if (Xormon::isdigit($data_out{capacity_total}) && Xormon::isdigit($data_out{capacity_used}) && $data_out{capacity_total} >=  $data_out{capacity_used}){
    $data_out{capacity_free} = $data_out{capacity_total} - $data_out{capacity_used};
  }
  ###
  return %data_out;
}


sub delete_empty_metrics{
  my $first_time = "";
  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        if ($first_time eq "" && Xormon::isdigit($time)){
          $first_time = $time;
        }
        if (Xormon::isdigit($first_time) && Xormon::isdigit($time) && $time < $first_time){
          $first_time = $time;
        }
        foreach my $item (keys %{$performance_data{$st_type}{$type}{$uid}{$time}}){
          if (!Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item}) || $performance_data{$st_type}{$type}{$uid}{$time}{$item} < 0){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
          if ($item =~ m/numerator|denominator/){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
        }
      }
      if (!keys %{$performance_data{$st_type}{$type}{$uid}}){
        delete $performance_data{$st_type}{$type}{$uid};
      }
    }
  }
  if (Xormon::isdigit($first_time)){
    $service_data_out->{last_perf_timestamp} = $first_time;
  }
  else{
    if (Xormon::isdigit($service_data_in->{last_perf_timestamp})){
      $service_data_out->{last_perf_timestamp} = $service_data_in->{last_perf_timestamp};
    }
  }
}

sub calculate_total_metric {
  my $data_tmp    = shift;
  my $metrics_tmp = shift;
  my %data        = %{$data_tmp};
  my @metrics     = @{$metrics_tmp};
  my $value       = "";
  foreach my $metric (@metrics) {
    if ( Xormon::isdigit( $data{$metric} ) ) {
      if ( Xormon::isdigit($value) ) {
        $value = $value + $data{$metric};
      }
      else {
        $value = $data{$metric};
      }
    }
  }
  return $value;
}


sub calculate_missing_metrics{

  my $item_total = "capacity_total";
  my $item_used = "capacity_used";
  my $item_used_percent = "capacity_used_percent";

  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        #capacity_used_percent
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_used})){
          if ($performance_data{$st_type}{$type}{$uid}{$time}{$item_total} == 0){
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = 0;
          }
          else{
            my $pom = ($performance_data{$st_type}{$type}{$uid}{$time}{$item_used} / $performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) * 100;
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = round($pom,2);
          }
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{data_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && $performance_data{$st_type}{$type}{$uid}{$time}{io_rate} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{data_rate} / $performance_data{$st_type}{$type}{$uid}{$time}{io_rate},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{read_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{read} / $performance_data{$st_type}{$type}{$uid}{$time}{read_io},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{write_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{write} / $performance_data{$st_type}{$type}{$uid}{$time}{write_io},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = 0;
        }
      }
    }
  }
}

sub logout{
  my $cmd = "aa/sessions";
  cmd($cmd,"DELETE",1,"close_session");
}

sub login{
  my $cmd =  "aa/sessions";
  my %hash_filter;
  $hash_filter{"user_name"} = $cfg->{apiUsername};
  $hash_filter{"password"} = $cfg->{apiPassword};
  $hash_filter{"scope"}    = "0";
  $hash_filter{"isEncrypt"} = "false";
  my $json_body = encode_json( \%hash_filter );
  my $data = cmd($cmd, "POST", 1, "sessions", $json_body);
  if ($data eq ""){return;}
  if (defined $data->{data}->{x_auth_token}){
    $token     = $data->{data}->{x_auth_token};
    $device_id = $data->{data}->{system_esn};
    $system_time = $data->{data}->{system_current_time};
    if ( Xormon::isdigit($system_time) ) {
      my $size = length($system_time);
      if ( $size != 10 ) {
        $system_time = time();
      } 
    }
    else {
      $system_time = time();
    }
  }
}

sub cmd{
  my $cmd = shift;
  my $type = shift;
  my $execute = shift;
  my $debug_file = shift;
  my $json_body = shift;
  my $perf_test = shift;
  if (!defined $perf_test){$perf_test = 0;}
  if (!defined $type || $type eq ""){$type = "GET";}
  if (!defined $execute || $execute eq ""){
    $execute = 1;
  }
  ### debug
  if (!$enviroment){
    $execute = 0;
  }
  if (!defined $debug_file){$debug_file = "";}
  if ($execute == 0 && -f "$debug_data_configuration/$debug_file.json" && $cmd !~ m/performance_data/){
    return Xormon::read_json("$debug_data_configuration/$debug_file.json");
  }
  elsif(-f "$debug_data/$debug_file.json" && $execute == 0){
    return Xormon::read_json("$debug_data/$debug_file.json");
  }
  elsif ($debug_file eq "close_session" && !$enviroment){return;}
  #print "$debug_data_configuration/$debug_file.json nemame\n\n\n";
  if (!$enviroment){
    print "$storage_name\n";
    print "$debug_data/$debug_file.json or $debug_data_configuration/$debug_file.json\n";
    return "";
  }
  my $url = "https://$api_host:$port/api/v2/$cmd";
  if ($cmd =~ m/dsware/){
    $url = "https://$api_host:$port/$cmd";
  }
  my $ua  = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( $type => $url );
  $req->header( 'Content-Type' => 'application/json' );
  $req->header( 'Accept'       => 'application/json' );
  $req->header( 'Connection'   => 'keep-alive' );
  $req->header( 'Accept-Language' => 'en' );
  if (defined $json_body && $json_body ne ""){
    $req->content($json_body);
  }
  if (defined $token && $token ne ""){
    $req->header( 'X-Auth-Token'   => $token );
  }
  my $res = $ua->request($req);
  #print $res->content;
  if ( ! $res->is_success ) {
    if (!$connection_test){
      $logger->error("API command $cmd failed");
    }
    else{
      push(@rows_api_test,"API command $cmd failed\n");
      push(@rows_api_test,$res->content . "\n");
      push(@rows_api_test_data,"API command $cmd failed\n");
      push(@rows_api_test_data,$res->content . "\n");
    }
    my $out = $res->content;
    $logger->error($out);
    return "";
  }
  my $data = decode_json( $res->content );
  if (defined $debug_file && $debug_file ne ""){
    if ($debug_file =~ m/_/){
      eval {Xormon::save_as_json("$debug_data/$debug_file.json",$data);};
    }
    else{
      eval {Xormon::save_as_json("$debug_data_configuration/$debug_file.json",$data);};
    }
    if ($@){
      $logger->error("Save data to file $debug_data/$debug_file.json failed");
    }
  }
  if (!defined $data->{data}){
    print Dumper \$data;
  }
  if ( defined $data->{data} && ref( $data->{data} ) ne "ARRAY" && $cmd =~ m/performance_data/) {
    my $error_description = $data->{result}->{description};
    my $error_code        = $data->{result}->{code};
    my $suggestion  = $data->{result}->{suggestion};
    if ( !defined $suggestion ) { $suggestion = ""; }
    if ( defined $perf_test && $perf_test eq "1" && defined $error_code ) {
      return $error_code;
    }
    if (!$connection_test){
      $logger->error("No data found for cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion");
    }
    else{
      push(@rows_api_test,"No data found for cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion\n");
      push(@rows_api_test_data,"No data found cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion\n");
    }
    return "";
  }
  if ( defined $data && defined $data->{result} && ref( $data->{result} ) ne "HASH" && Xormon::isdigit( $data->{result} ) && $data->{result} == 0 ) {
      return $data;
  }
  if ( defined $data && defined $data->{result} && ref( $data->{result} ) ne "HASH" && defined $data->{errorCode} && $data->{errorCode} != 0 ) {
    my $error_description = $data->{description};
    my $error_code        = $data->{errorCode};
    my $suggestion        = $data->{suggestion};
    if ( !defined $suggestion ) { $suggestion = ""; }
    if (!$connection_test){
      $logger->error("No data found for cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion");
    }
    else{
      push(@rows_api_test,"No data found for cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion\n");
      push(@rows_api_test_data,"No data found cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion\n");
    }
    return "";
  }
  if ( defined $data->{result}->{errorCode} && $data->{result}->{errorCode} != 0 ) {
    my $error_description = $data->{result}->{description};
    my $error_code        = $data->{result}->{errorCode};
    my $suggestion        = $data->{result}->{suggestion};
    if ( !defined $suggestion ) { $suggestion = ""; }
    if (!$connection_test){
      $logger->error("No data found for cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion");
    }
    else{
      push(@rows_api_test,"No data found for cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion\n");
      push(@rows_api_test_data,"No data found cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion\n");
    }
    return "";
  }
  if ( defined $data->{result}->{code} && $data->{result}->{code} != 0 ) {
    my $error_description = $data->{result}->{description};
    my $error_code        = $data->{result}->{code};
    my $suggestion        = $data->{result}->{suggestion};
    if ( !defined $suggestion ) { $suggestion = ""; }
    if (!$connection_test){
      $logger->error("No data found for cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion");
    }
    else{
      push(@rows_api_test,"No data found for cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion\n");
      push(@rows_api_test_data,"No data found cmd $cmd  description : $error_description error_code : $error_code suggestion : $suggestion\n");
    }
    return "";
  }
  return $data;
}

sub set_architecture{

  my %device_arch = (
    "item_id" => $cfg->{'uuid'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => $st_type,
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'uuid'}
  );
  #if (defined $cfg->{serial_number}){
  #  $device_arch{serial_number} = $cfg->{serial_number};
  #}
  #if (defined $cfg->{version}){
  #  $device_arch{version} = $cfg->{version};
  #}
  #if (defined $cfg->{model}){
  #  $device_arch{model} = $cfg->{model};
  #}
  push(@architecture, \%device_arch);
  #print Dumper \$configuration_data{$st_serial};
  #return;

  foreach my $uid (keys %configuration_data){
    my $name = $configuration_data{$uid}{label};
    my $type = $configuration_data{$uid}{TYPE};
    if (!defined $name || !defined $type){next;}
    my @children = ();
    my @parents = ();
    if (defined $configuration_data{$uid}{children}){
      @children = @{$configuration_data{$uid}{children}};
      ## delete children and parent from item_properties
      delete $configuration_data{$uid}{children};

    }
    if (defined $configuration_data{$uid}{parents}){
      @parents = @{$configuration_data{$uid}{parents}};
      delete $configuration_data{$uid}{parents};

    }
    #print Dumper \@children;
    #print Dumper \@parents;
    my %arch = (
      "item_id" => $uid,
      "label" => $name,
      "hw_type" => $st_type,
      "subsystem" => $type,
      "parents" => [$cfg->{'uuid'}]
      );
    if (@parents){
      push(@{$arch{parents}},@parents);
    }
    if (@children){
      push(@{$arch{children}},@children);
    }
    if ($type eq "volume"){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($uid);
    }
    if (($type eq "ethport" || $type eq "volume" ) && defined $configuration_data{$uid}{wwn}){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($configuration_data{$uid}{wwn});
    }
    #push(@{$arch{parents}},$cfg->{'uuid'});
    push(@architecture, \%arch);
    delete $configuration_data{$uid}{TYPE};

  }
  #print Dumper \@architecture;
  #foreach my $item (keys %{$data{conf}{device}}){
  #  $configuration_data{$st_serial}{$item} = $data{conf}{device}{$item};
  #}
  #print Dumper \$configuration_data{device}{$st_serial};

  #save configuration
  #$configuration_data{$cfg->{'uuid'}}{"label"} = $cfg->{'hostalias'};
  #event log
  $configuration_data{$cfg->{'uuid'}}{events} = encode_json(\@health_status);
  #$configuration_data{$cfg->{'uuid'}}{hardware} = encode_json(\@hardware);
  #print Dumper \$configuration_data{$cfg->{'uuid'}}{alerts};
  #print Dumper \@architecture;
  #print Dumper \%configuration_data;
  ### device information save
  #print Dumper \%configuration_data;
  my $rc = $xormon->saveConfJSON(\%configuration_data);
  my $status_configuration = get_status($rc, "saveConfJSON (configuration)");
  #$logger->info("Task for configuration ".$cfg->{hostalias}." is done. Status: $status");

  #save architecture
  #print Dumper \@architecture;

  $rc = $xormon->saveArchitecture(\@architecture);
  my $status_architecture = get_status($rc, "saveArchitecture");
  #$logger->info("Task for saveArchitecture ".$cfg->{hostalias}." is done. Status: $status");

  $rc = $xormon->saveStatus(\@health_status_global);
  my $status_health = get_status($rc, "health status");
  return ($status_architecture, $status_configuration, $status_health);
}

sub connection_test {
  my $client = shift;
  $cfg = shift;
  $connection_test = 1;
  if (defined $cfg->{OCEANSTORPACIFICNAS_ENVIROMENT} && $cfg->{OCEANSTORPACIFICNAS_ENVIROMENT} eq "development"){
    $enviroment = 0;
  }
  my $test = Test->new($client);
  my @variables = ("hostcfg_id", "hostalias", "node1Host", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  $logger = XormonLogger->new( 'oceanstorpacificnas', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  $storage_name = $cfg->{hostalias};
  $st_serial = $cfg->{hostcfg_id};
  data_structure();

  #### connection network test ####
  my $tcp_text = "";
  my ($code1, $text) = Xormon::conntest_tcp($cfg->{node1Host},$port);
  if (!$enviroment){
    $code1 = 1;
  }
  $tcp_text = $tcp_text . "$text\n";
  if (defined $cfg->{node2Host}){
    my ($code2, $text) = Xormon::conntest_tcp($cfg->{node2Host},$port);
    $tcp_text = $tcp_text . "$text\n";
    if ($code1 == 0 && $code2 == 0){
      $test->addSection("API host network connection", 0, $tcp_text);
      return $test->stop();
    }
    $test->addSection("API host network connection", 1, $tcp_text);
  }
  else{
    $test->addSection("API host network connection", $code1, $tcp_text);
    if ($code1 == 0){
      return $test->stop();
    }
  }
  ### API authorization
  $api_host = $cfg->{'node1Host'};
  login();
  if (!defined $token || $token eq "" ){
    if (defined $cfg->{node2Host}){
      if ($cfg->{node2Host} eq $api_host){
        $api_host = $cfg->{'node1Host'};
      }
      $api_host = $cfg->{'node2Host'};
      login();
      if (!defined $token || $token eq "" ){
        push(@rows_api_test, "Token is not defined\n");
        $test->addSection("API authorization", 0, \@rows_api_test);
        return $test->stop();
      }
    }
    else{
      push(@rows_api_test, "Token is not defined\n");
      $test->addSection("API authorization", 0, \@rows_api_test);
      return $test->stop();
    }
  }
  set_data_device_conf(1);
  if (%configuration_data){
    my %items;
    $items{version} = "Version";
    $items{model} = "Model";
    $items{node_name} = "Name";
    $items{serial} = "Serial";
    foreach my $item (keys %items){
      my $line;
      if (defined $configuration_data{$st_serial}{$item}){
        $line = $items{$item} . " : ". $configuration_data{$st_serial}{$item};
      }
      else{
        $line = $items{$item} . " : ";
      }
      push(@rows_api_test,"$line\n");
    }
    $test->addSection("API authorization", 1, \@rows_api_test);
  }
  else{
    logout();
    $test->addSection("API authorization", 0, \@rows_api_test);
    return $test->stop();
  }
  ###
  ### API test data
  set_data_perf_list();
  if (!check_connection()){
    $test->addSection("API test data", 0, \@rows_api_test_data);
    return $test->stop();
  }
  else{
    $test->addSection("API test data", 1, "Performance data ok");
  }
  ###

  return $test->stop();
  #return Xormon::conn_test($connection_test, \@rows_api_test);
}

sub data_structure {
  if (!-d "$data_directory/data"){
    mkdir "$data_directory/data" || $logger->error( "Cannot mkdir $data_directory/data: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial"){
    mkdir "$data_directory/data/$st_serial" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial/IOSTATS"){
    mkdir "$data_directory/data/$st_serial/IOSTATS" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial/IOSTATS: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }


  my $directory = "$data_directory/data/$st_serial/IOSTATS";
  if ( !-d $directory ) {
    mkdir $directory || $logger->error( "Cannot mkdir $directory: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if ( !-d "$directory/configuration" ) {
    mkdir "$directory/configuration" || $logger->error( "Cannot mkdir $directory/configuration: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  for ( my $i = 0; $i < 60; $i = $i + 5 ) {
    if ( $i < 10 ) {
      if ( !-d "$directory/0$i" ) {
        mkdir "$directory/0$i" || $logger->error( "Cannot mkdir $directory/0$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
    else {
      if ( !-d "$directory/$i" ) {
        mkdir "$directory/$i" || $logger->error( "Cannot mkdir $directory/$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
  } ## end for ( my $i = 0; $i < 60...)
  my @entry_time = split( " ", localtime() );
  ( undef, $active_minute, undef ) = split( ":", $entry_time[3] );
  if ($connection_test){
    $active_minute = 10;
  }
  #$active_minute = 10;
  if ( !-d "$directory/$active_minute" ) {
    mkdir "$directory/$active_minute" || $logger->error( "Cannot mkdir $directory/$active_minute: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  #$debug_data_performance = "$directory/$active_minute";
  #$debug_data = "$directory/configuration";
  $debug_data = "$directory/$active_minute";
  $debug_data_configuration = "$directory/configuration";
  #print "$debug_data\n";
  $wrkdir = "$data_directory/data/$st_serial";

} ## end sub data_structure

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

sub create_perf_json{
  my $object_type = shift;
  my $id          = shift;
  my $indicator   = shift;
  my $start       = $system_time - 900;
  my $end         = $system_time - 600;
  my %hash;
  my @object = ();
  my %hash_all;
  my @indicators = split( ",", $indicator );
  $hash{object_type} = $object_type;
  $hash{indicators}  = \@indicators;

  if ( defined $id && $id ne "ALL" ) {
    $hash{ids} = [$id];
  }
  push( @object, \%hash );
  $hash_all{begin_time} = $start;
  $hash_all{end_time}   = $end;
  $hash_all{objects}    = \@object;
  return encode_json( \%hash_all );

}

sub check_connection{
  my $data = cmd("cluster/product", "GET", 1, "product");
  if ( $data eq "" ) {
    if ($connection_test){
      push(@rows_api_test_data,"connection to $api_host on port $port has failed!\n");
    }
    else{
      $logger->error("connection to $api_host on port $port has failed!");
    }
    return 0;
  }
  my $perf_json = create_perf_json(16385,"ALL", $performance_statistics{node});
  $data = cmd("pms/performance_data", "GET", 1,"16385", $perf_json, 1);
  #print Dumper \$data;
  if ($data eq ""){
    if ($connection_test){
      push(@rows_api_test_data,"connection to $api_host on port $port has failed!\n");
    }
    else{
      $logger->error("connection to $api_host on port $port has failed!");
    }
    return 0;
  }
  if (!defined $data->{data} && $data eq "83890432"){
    return relogin();
  }
  return 1;
}

sub relogin{
  logout();
  if ($api_host ne $cfg->{'node2Host'}){
    $api_host = $cfg->{'node2Host'};
  }
  else{
    $api_host = $cfg->{'node1Host'};
  }
  $token = "";
  login();
  if (!defined $token || $token eq ""){
    return 0;
  }
  return 1;
}

sub set_data_perf_list{
  $performance_statistics{node} = "68,69,50001,50002,50003,50004,50005,50006,1201,1202,1200,40001,40002,40003,40004,40005,40006,30001,30002,30003,30004,30005,30006,30007,30008,30009,30011,30012,30013,30014,30015,30016,30017,30018";
  $performance_statistics{pool} = "123,124,811,25,28,22,432,430,428";
  $performance_statistics{drive} = "123,124,811,25,28,22,432,430,428";
  $performance_statistics{ethport} = "123,124,236,25,28,22";
  $performance_statistics{cluster} = "68,69,50001,50002,50003,1201,1202,1200,40001,40002,40003,40004,40005,40006,30001,30002,30003,30004,30005,30006,30007,30008,30009,30011,30012,30013,30014,30015,30016,30017,30018";

  $metric_inventory{124} = "write";
  $metric_inventory{123} = "read";
  $metric_inventory{25}  = "read_io";
  #$metric_inventory{27}  = "block_size_w";
  $metric_inventory{811} = "data_rate";
  $metric_inventory{28}  = "write_io";
  $metric_inventory{428} = "resp_t";
  $metric_inventory{430} = "resp_t_w";
  $metric_inventory{22}  = "io_rate";
  $metric_inventory{432} = "resp_t_r";
  #$metric_inventory{24}  = "block_size_r";
  $metric_inventory{68}    = "cpu_system";
  $metric_inventory{69}    = "cache_usage_percent";
  $metric_inventory{236} = "data_rate";

  #Filesystems stats
  $metric_inventory{30001} = "file_read";
  $metric_inventory{30002} = "file_write";
  $metric_inventory{30003} = "file_data_rate";

  $metric_inventory{1201} = "read_b";        #### storage statistics
  $metric_inventory{1202} = "write_b";
  $metric_inventory{1200} = "data_rate_b";

  $metric_inventory{30004} = "read_ops";
  $metric_inventory{30005} = "write_ops";
  $metric_inventory{30006} = "ops_rate";
  $metric_inventory{30011} = "nfs_resp_t";
  $metric_inventory{30013} = "nfs_resp_t_w";
  $metric_inventory{30015} = "nfs_resp_t_r";

  #Object stats ### cluster and nodes
  $metric_inventory{50001} = "object_read";
  $metric_inventory{50002} = "object_write";
  $metric_inventory{50003} = "object_data_rate";
  $metric_inventory{50004} = "object_read_io";
  $metric_inventory{50005} = "object_write_io";
  $metric_inventory{50006} = "object_io_rate";

  ## hdfs
  $metric_inventory{40001} = "hdfs_read";
  $metric_inventory{40002} = "hdfs_write";
  $metric_inventory{40003} = "hdfs_data_rate";
  $metric_inventory{40004} = "hdfs_read_io";
  $metric_inventory{40005} = "hdfs_write_io";
  $metric_inventory{40006} = "hdfs_io_rate";

}

sub get_health_status_text {
  my $number = shift;
  my $type   = shift;

  if ( $type eq "volume" ) {
    if ( $number == 1 ) {
      return "normal";
    }
    if ( $number == 2 ) {
      return "faulty";
    }
  }

  if ( $type eq "disk" ) {
    if ( $number == 0 ) {
      return "healthy";
    }
    if ( $number == 1 ) {
      return "faulty";
    }
    if ( $number == 2 ) {
      return "subhealthy";
    }
    if ( $number == 101 ) {
      return "The disk is removed from the storage pool";
    }
  }
  if ( $type eq "pool" ) {
    if ( $number == 0 ) {
      return "normal";
    }
    if ( $number == 1 ) {
      return "faulty";
    }
    if ( $number == 2 ) {
      return "write-protected";
    }
    if ( $number == 3 ) {
      return "stopped";
    }
    if ( $number == 4 ) {
      return "faulty&write-protected";
    }
    if ( $number == 5 ) {
      return "migrating data";
    }
    if ( $number == 7 ) {
      return "degraded";
    }
    if ( $number == 8 ) {
      return "rebuilding data";
    }
  }
  return $number;
}

sub get_running_status_text {
  my $number = shift;

  if ( $number == 27 ) {
    return "online";
  }
  if ( $number == 28 ) {
    return "offline";
  }
  if ( $number == 44 ) {
    return "rolling back";
  }
  return $number;
}


sub get_global_timestamp{
  my $act_timestamp = time();
  (my $sec,my $min,my $hour,my $mday,my $mon, my $year,my $wday,my $yday,my $isdst) = localtime();
  return $act_timestamp - $sec;

}

sub get_level_text {
  my $type = shift;
  if ( !defined $type ) { return "unknown"; }
  if ( $type == 2 )     { return "info"; }
  if ( $type == 3 )     { return "warning"; }
  if ( $type == 5 )     { return "major"; }
  if ( $type == 6 )     { return "critical"; }
  return "unknown";

}
sub get_type_text {
  my $type = shift;
  if ( $type eq "0" ) {
    return "common LUN";
  }
  if ( $type eq "1" ) {
    return "vVol LUN";
  }
  if ( $type eq "2" ) {
    return "PE LUN";
  }
  return $type;
}


sub set_data_device_conf{
  my $conn_test = shift;
  if (!defined $conn_test){$conn_test = 0;}

  my %items;
  $items{version} = "version";
  $items{patch_version} = "patch_version";
  $items{product_model} = "model";
  $items{cluster_name} = "node_name";

  my $section = "device";
  my $data = cmd("product", "GET", $conn_test, "product");
  #print Dumper \$data;
  if ( $data eq "" ) {
    return;
  }

  if (ref($data->{data}) eq "HASH"){
    foreach my $item (keys %items){
      if (defined $data->{data}->{$item}){
          $configuration_data{$st_serial}{$items{$item}} = $data->{data}->{$item};
      }
      else{
        $configuration_data{$st_serial}{$items{$item}} = "";
      }
    }
  }
  if (defined $configuration_data{$st_serial}{patch_version} && defined $configuration_data{$st_serial}{version}){
    $configuration_data{$st_serial}{version} = $configuration_data{$st_serial}{version} . " " . $configuration_data{$st_serial}{patch_version};
  }
  $configuration_data{$st_serial}{serial} = $device_id;

}

sub get_calculate_performance_data {
  my $data      = shift;
  my $id        = shift;
  my $section   = shift;
  my $timestamp = "";
  my %values;
  my %weighted_average_values;
  my %weighted_items;
  $weighted_items{io_rate}  = "resp_t";
  $weighted_items{read_io}  = "resp_t_r";
  $weighted_items{write_io} = "resp_t_w";
  my $id_correct = "";
  if ($data eq "" || !defined $data->{data}){
    return %values;
  }

  foreach my $element ( @{ $data->{data} } ) {
    my $id_actual = $element->{id};
    if ( !defined $id_actual ) { next; }
    my $tot_value = "";
    my $index     = 0;
    if ( !defined $element->{timestamp} || !defined $id_actual || !defined $element->{indicator} || !defined $element->{indicator_values} ) { next; }
    if ( $id_actual !~ m/:/ && $section !~ m/^drive|^ethport/ && $id ne $id_actual ) {
      next;
    }
    if ( $section =~ m/^ethport/ ) {
      if ( $id_actual ne $id ) { next; }
    }
    if ( $id_actual =~ m/:/ && $section =~ m/^drive/ ) {
      my ( $node, $uid ) = split( ":", $id_actual );
      if ( $uid ne $id ) { next; }
      $id_correct = $id_actual;
      #$id_correct =~ s/:/-/g;
    }
    if ( ref( $element->{timestamp} ) eq "ARRAY" ) {
      if ( defined @{ $element->{timestamp} }[-1] ) {
        $timestamp = @{ $element->{timestamp} }[-1];
        }
    }
    else {
      $timestamp = $element->{timestamp};
    }
    if ( ref( $element->{indicator_values} ) eq "ARRAY" ) {
      foreach my $val ( @{ $element->{indicator_values} } ) {
        if ( defined $metric_inventory{ $element->{indicator} } && $metric_inventory{ $element->{indicator} } =~ m/^read_io$|^resp_t_r$|^resp_t_w$|^resp_t$|write_io|io_rate|^read_ops$|^write_ops$|^ops_rate$|^nfs_resp_t/ ) {
          push( @{ $weighted_average_values{ $metric_inventory{ $element->{indicator} } } }, $val );
        }
        if ( Xormon::isdigit($val) && $val !~ m/^\./ ) {
          $index++;
          if ( Xormon::isdigit($tot_value) ) {
            $tot_value = $tot_value + $val;
          }
          else {
            $tot_value = $val;
          }
        }
      }
    }
    if ( Xormon::isdigit($tot_value) && defined $metric_inventory{ $element->{indicator} } ) {
      my $rounded = 0;
      if ( $metric_inventory{ $element->{indicator} } =~ m/resp/ ) {
        $rounded = 3;
      }
      if ($metric_inventory{ $element->{indicator} } =~ m/^read$|^write$|^data_rate$|^file_data_rate$|^file_read$|^file_write$|^object_read$|^object_write$|^object_data_rate$|^hdfs_data_rate$|^hdfs_read$|^hdfs_write$/){
        $tot_value = $tot_value * 1000;
      }
      $values{ $metric_inventory{ $element->{indicator} } } = round( $tot_value / $index, $rounded );
      $values{timestamp} = $timestamp;
    }
  }
  foreach my $item ( keys %weighted_items ) {
    my $sum_numerator   = 0;
    my $sum_denominator = 0;
    if ( defined $weighted_average_values{$item} && defined $weighted_average_values{ $weighted_items{$item} } ) {
      my $index = -1;
      foreach my $value ( @{ $weighted_average_values{$item} } ) {
        $index++;
        if ( Xormon::isdigit($value) && ( Xormon::isdigit( $weighted_average_values{ $weighted_items{$item} }[$index] ) && $weighted_average_values{ $weighted_items{$item} }[$index] !~ m/^\./ ) ) {
          $sum_numerator   = $sum_numerator + ( $value * $weighted_average_values{ $weighted_items{$item} }[$index] );
          $sum_denominator = $sum_denominator + $value;
        }
      }
    }
    else {
      next;
    }
    if ( $sum_denominator == 0 ) {
      $values{ $weighted_items{$item} } = 0;
    }
    else {
      $values{ $weighted_items{$item} } = round( $sum_numerator / $sum_denominator, 3 );
    }
  }
  if ( $section eq "drive" && $id_correct ne "" ) {
    $values{id} = $id_correct;
  }
  if (!$enviroment){
    $values{timestamp} = time() - 300;
  }
  foreach my $item ( keys %values ) {
    if ( $item =~ m/nfs_resp_t/ ) {
      $values{$item} = round( $values{$item} / 1000, 3 );
    }
  }

  return %values;
}

1;
