use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib '../../../perl-modules';
use OracleZFS;
use Xormon;
use File::Temp qw(tempfile);
use File::Copy;

use LWP::UserAgent();
use HTTP::Request;
use JSON qw(encode_json decode_json);
use XormonLogger;
use Test;

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);


my $time      = time();
my $datatime  = "";   #"2019-09-13T08:54:00Z"
my $starttime = "";#start_time($time);
my $endtime   = "";#end_time($time);
my $ft        = filetime();

my $version   = "";
my $storpath  = "api/storage";
my $analytics = "";
my $start     = "data?start=";
my $span      = "span=hour&granularity=minute";

my %stats;
my %pools;
my %disks;
my %devices;
my %igs;
my %datasets;
my %chassis;          #%chassis  = get_chassis();
my $cookie    = "";   #$cookie = login();
my $ip        = "";
my $nodename  = "";   #$nodename = get_nodename();
my $asn       = "";
my %device_conf;
my $skipped   = 0;
my $api_check = 0;

my $io = 0;
my $data_t = 0;
my $resp_t = 0;
my $used = 0;
my $free = 0;
my $total = 1;
my $replication = 0;
my $snapshot = 0;


my %protocol_unit = (
  fc    => "bytes",
  iscsi => "bytes",

  #  sftp   => "kilobytes",
);

my %stor = (
  "STOR2RRD_node" => [
    "cpu.utilization",       "cpu.utilization[cpu]", "nic.kilobytes",
    "nic.kilobytes[device]", "nic.kilobytes[direction]"
  ],

  #"net.kilobytes", "net.kilobytes[interface]", "net.kilobytes[direction]"],

  "STOR2RRD_iscsi" => [
    "iscsi.ops",            "iscsi.ops[op]",      "iscsi.ops[lun]",         "iscsi.ops[initiator]",
    "iscsi.bytes",          "iscsi.bytes[lun]",   "iscsi.bytes[initiator]", "iscsi.avglatency",
    "iscsi.avglatency[op]", "iscsi.ops[latency]", "iscsi.ops[latency]"
  ],

  #"iscsi.avglatency[pool]",

  "STOR2RRD_fc" => [
    "fc.ops",        "fc.ops[op]",          "fc.ops[lun]",   "fc.ops[initiator]", "fc.bytes",
    "fc.bytes[lun]", "fc.bytes[initiator]", "fc.avglatency", "fc.avglatency[op]", "fc.ops[latency]"
  ],

  "STOR2RRD_nfs" => [
    "nfs3.bytes",       "nfs3.bytes[op]", "nfs3.bytes[share]", "nfs4-1.bytes",
    "nfs4-1.bytes[op]", "nfs4-1.bytes[share]"
  ],

  #"nfs2.bytes", "nfs2.bytes[op]", "nfs2.bytes[share]",

  "STOR2RRD_smb" => [
    "smb2.bytes", "smb2.bytes[op]", "smb2.bytes[share]",
    "smb3.bytes", "smb3.bytes[op]", "smb3.bytes[share]"
  ],

  "STOR2RRD_disk" => [ "io.ops", "io.ops[op]", "io.ops[disk]", "io.bytes", "io.bytes[op]", "io.bytes[disk]" ],

  "STOR2RRD_arc" => [
    "arc.size",    "arc.size[component]", "arc.hitratio", "arc.accesses[hit/miss]",
    "arc.l2_size", "arc.l2_accesses[hit/miss]"
  ],
);

my $DEFAULT_PORT = 3208;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);
my $logger;
my $server = IO::Socket->new(
    Domain => AF_INET,
    Type => SOCK_STREAM,
    Proto => 'tcp',
    LocalHost => '0.0.0.0',
    LocalPort => $port,
    ReusePort => 1,
    Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

 # waiting for a new client connection
while ($client = $server->accept()) {
  my $pid;
  unless ( defined( $pid = fork() ) ) {
      Xormon::error("Wait for fork");
      sleep 5;
  }

  if ($pid == 0) {
    # get information about a newly connected client
    my $client_address = $client->peerhost();
    my $client_port = $client->peerport();
    Xormon::log("Connection from $client_address:$client_port");
    #received data template
    #
    #{
    #  "jwt"  : "<backendToken>",
    #  "task" : "<taskName>"
    #  "data" : {
    #    <configuration_from_hostcfg>
    #  }
    #}

    #read up to 2048 characters from the connected client
    my $data = "";
    $client->recv($data, 2048);

    my $decoded = from_json($data);
    my $cfg = $decoded->{data};
    $logger = XormonLogger->new( 'oraclezfs', $cfg->{hostcfg_id}, $cfg->{hostalias} );
    $logger->info("Connection from $client_address:$client_port");
    $logger->info("Received task: ".$decoded->{task});
    my $response = '';
    if ($decoded->{task} eq "fetch") {

      $client->send("Task successfuly started");
      $client->shutdown(SHUT_WR);

      my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'oraclezfs', $cfg->{hostcfg_id}, $cfg->{hostalias});

      $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

      ############################### AGENT SECTION ###############################

      # target destinations check
      if (defined $decoded->{'destinations'}) {
        $xormon->setDestinations($decoded->{'destinations'});
      }

      #print Dumper $cfg;
      # get data from technology
      my $dataToBackend = api2json($xormon, $cfg);
      #print Dumper $dataToBackend;

      # send data to backend
      my $rc = $xormon->saveJSON2($dataToBackend);

      ############################### END AGENT SECTION ###############################

      # status code
      my $status;
      if ($rc eq "201") {
        $status = "successful";
      } elsif ($rc eq "401") {
        $status = "unauthorized";
      } else {
        $status = "error";
      }

      $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);

      $logger->info("Task [$decoded->{task}] for ".$cfg->{hostalias}." is done");

      }
      elsif ($decoded->{task} eq "test") {
        $response = apitest($client, $cfg);

        $client->send($response);
        $client->shutdown(SHUT_WR);
      } else {
        $response = "undefined task";

        $client->send($response);
        $client->shutdown(SHUT_WR);
      }

    exit();

    } else {
      $client->autoflush(1);
    }
}





sub api2json {
  my $xormon = shift;
  my $data   = shift;

  my $mvc_dir      = $FindBin::Bin;
  my $username     = $data->{apiUsername};
  my $password     = $data->{apiPassword};
  my $ipone        = $data->{node1Host};
  my $iptwo        = $data->{node2Host};
  my $alias        = $data->{hostalias};
  my $hostcfg_id   = $data->{hostcfg_id};
  $starttime       = start_time($time);
  $endtime         = end_time($time);
  $time            = time();
  $ft              = filetime();
  my %weird_perf;
  my @versions  = ( "v2.0", "v1" );

  #>debug
  my $debug = 1;

  #>

  my $setup;

  my @ips = ( $ipone, $iptwo );

  #print Dumper \@ips;


  unless ( -d "$mvc_dir/data" ) {
    mkdir( "$mvc_dir/data", 0755 ) || warn( "Cannot mkdir $mvc_dir/data: $!" . __FILE__ . ":" . __LINE__ ) && exit;
  }
  my $checkup_file = "$mvc_dir/data/$hostcfg_id"."_checkup-zfs";
  if ( -e $checkup_file ) {
    my $modtime      = ( stat($checkup_file) )[9];
    my $timediff     = $time - $modtime;
    my $current_time = substr( $ft, -4 );
    if ( $timediff >= 86400 or $current_time eq "0000" ) {
      $setup = "setup";
      open my $fh, '>', $checkup_file;
      print $fh "1\n";
      close $fh;
    }
  }
  elsif ( !-e $checkup_file ) {
    $setup = "setup";
    open my $fh, '>', $checkup_file;
    print $fh "1\n";
    close $fh;
  }

  my $connected = 0;

  foreach my $currentip (@ips) {
    next if (!defined $currentip);
    $ip = "$currentip".":215";
    if ( $ip eq ":215" ) {
      $connected = Test::connTest($currentip, "215");
      next;
    }
    $connected = 1;
    foreach my $c_version (@versions) {
      $version = $c_version;
      my $message = lc( login($username, $password, 1) );
      if ( $message ne "not found" ) {
        $api_check = 1;
        last;
      }
    }
    $datatime  = data_time($time);
    $analytics = "api/analytics/$version/datasets";

    if ( $api_check == 0 ) {
      $logger->error("Probably no REST API support. IP: $ip");
      next;
    }

    #>debug
    if ( $debug == 1 ) {
      $logger->info("IP: $ip  Data collection start " . timestamp( time() ) . "\n");
    }

    #>
    $cookie = login($username,$password);
    if ( !defined $cookie || $cookie eq "" ) {
      if ( $skipped == 0 ) {
        $skipped++;
        $logger->error("Cookie is not defined IP: $ip ");
        next;
      }
      else {
        $logger->error("ERROR: Cookie is not defined! IP: $ip " . __FILE__ . ":" . __LINE__ ) && exit 1;
      }
    }
    else {
      #print "logged in \n";
    }

    $nodename = get_nodename();
    %chassis  = get_chassis();

    #my %interfaces = get_interfaces();
    #add_interfacedata();
    ##interface_level_statistics();

    if ( defined $setup and $setup eq "setup" ) {

      #>debug
      if ( $debug == 1 ) {
        $logger->info("Setup start " . timestamp( time() ) . "\n");
        $logger->info("Getting datasets start " . timestamp( time() ) . "\n");
      }

      #>
      %datasets = get_datasets();

      #>debug
      if ( $debug == 1 ) {
        $logger->info( "Getting datasets end " . timestamp( time() ) . "\n");
        $logger->info( "Adding worksheets " . timestamp( time() ) . "\n");
      }

      #>
      add_worksheets();

      #>debug
      if ( $debug == 1 ) {
        $logger->info("Worksheets added " . timestamp( time() ) . "\n");
        $logger->info("Adding and activating datasets " . timestamp( time() ) . "\n");
      }

      #>
      #add_byservices();
      add_therest();

      #>debug
      if ( $debug == 1 ) {
        $logger->info("Datasets added and activated " . timestamp( time() ) . "\nSetup done \n");
      }

      #>
    }

    #>debug
    if ( $debug == 1 ) {
      $logger->info("Initializing data start " . timestamp( time() ) . "\n");
    }

    #>
    get_poolinfo();
    get_listofdisks( \%pools );

    #api call takes ages
    get_diskserial();
    get_diskdata();
    get_devices();
    get_initiatorgroups();
    add_initiatordata();

    #>debug
    if ( $debug == 1 ) {
      $logger->info("Initializing data end " . timestamp( time() ) . "\n");
    }

    #>
    #print "$datatime\n";
    #print "$starttime\n";
    #print "$endtime\n";
    #print Dumper \%pools;
    #print Dumper \%disks;
    #print Dumper \%devices;
    #print Dumper \%stor;
    #>debug
    if ( $debug == 1 ) {
      $logger->info("Getting statistics start " . timestamp( time() ) . "\n");
    }

    #>
    filesystemcap_level_statistics();
    projectcap_level_statistics();
    initiator_level_statistics();
    poolcap_level_statistics();
    l2cache_level_statistics();
    luncap_level_statistics();
    cache_level_statistics();
    node_level_statistics();
    disk_level_statistics();
    port_level_statistics();
    cpu_level_statistics();

    #>debug
    if ( $debug == 1 ) {
      $logger->info("Getting statistics end " . timestamp( time() ) . "\n");
    }

    #>
    logout();

    #>debug
    if ( $debug == 1 ) {
      $logger->info("IP: $ip  Data collection end " . timestamp( time() ) . "\n");
    }

    #>

    #warn Dumper get_pools_aggr();
    #warn Dumper get_luns_aggr();
    #warn Dumper get_filesystems_aggr();
    #warn Dumper get_projects_aggr();

    #warn Dumper \%pools;
    #warn Dumper \%disks;
    #warn Dumper \%devices;
    #warn Dumper \%datasets;
    #warn Dumper \%chassis;

    undef %pools;
    undef %disks;
    undef %devices;
    undef %datasets;
    undef %chassis;
    $ip        = "";
    $asn       = "";
    $cookie    = "";
    $version   = "";
    $datatime  = "";
    $nodename  = "";
    $api_check = "";
    $analytics = "";
  }

  if ($connected == 1) {
    $xormon->isConnected(1);
  } else {
    $xormon->isConnected(0, "TCP connection to one of the nodes has failed!");
    return ();
  }

  my %subs_from_storname = (
    'Node Level Statistics' => "NODE",
    'Port Level Statistics' => "PORT",
    'Volume Level Statistics' => "VOLUME",
    'Pool Level Statistics' => "POOL",
    'Cache Level Statistics' => "CACHE",
    'Project Level Statistics' => "PROJECT",
    'L2Cache Level Statistics' => "L2CACHE",
    'CPU Level Statistics' => "CPU",
    'Host Level Statistics' => "HOST",
    'Disk Level Statistics' => "DRIVE",
    'Filesystem Level Statistics' => "FS",
  );

  my %standardize = (
    'CPU' => {
      'cpu_utilization' => 'sys'
    },
    'PORT' => {
      'port_data(kbs)' => 'data_rate'
    },
    'DRIVE' => {
      'data_write' => 'write',
      'disk_iops' => 'io_rate',
      'data_read' => 'read',
      'io_read' => 'read_io',
      'io_write' => 'write_io',
      'disk_data' => 'data_rate'
    },
    'CACHE' => {
#      'metadata_misses' => '5',
#      'data_misses' => '7',
#      'prefetched_data_hits' => '',
#      'arc_hitratio' => '193',
#      'metadata_hits' => '8',
#      'data_hits' => '1984',
#      'arc_size' => '99',
#      'prefetched_metadata_hits' => '',
#      'ARC_other' => '321',
#      'ARC_headers' => '20',
#      'ARC_data' => '350'
    },
    'VOLUME' => {
#      'used_percentage' => '0.2',
      'io_rate' => 'io_rate',
      'data_rate' => 'data_rate',
      'iscsi_ops' => 'io_rate',
      'capacity' => 'capacity_used',
      'iscsi_kilobytes' => 'data_rate',
#      'fc_ops' => '0',
#      'fc_kilobytes' => '0',
    },
    'POOL' => {
      'io_rate' => 'io_rate',
      'data_rate' => 'data_rate',
      'iscsi_ops' => 'iscsi_ops',
      'iscsi_kilobytes' => 'iscsi_kilobytes',
      'free_capacity' => 'capacity_free',
      'used_capacity' => 'capacity_used',
#      'snapshots' => '',
      'UserDataStored' => 'capacity_data',
      'subscription' => 'virtual_capacity_total',
#      'used_percentage' => '52.7',
#      'usage_total' => '5538149376',
#      'usage_metaused' => '0',
#      'fc_kilobytes' => '0',
#      'smb3_kilobytes' => '0',
#      'compression' => '1',
#      'subscription_percentage' => '51.1',
#      'smb2_kilobytes' => '0',
#      'dedupsize' => '0',
#      'used_capacity' => '5538149376',
#      'usage_child_reservation' => '0',
#      'total' => '10670309376',
#      'usage_replication' => '0',
#      'dedupratio' => '100',
#      'nfs4-1_kilobytes' => '0',
#      'available' => '4964449792',
#      'fc_ops' => '0',
#      'usage_metasize' => '0',
#      'nfs3_kilobytes' => '0',
#      'usage_reservation' => '0'
    },
    'NODE' => {
        'io_rate' => 'io_rate',
        'data_rate' => 'data_rate',
        'iscsi_latency_write' => 'iscsi_resp_t_w',
        'iscsi_latency_read' => 'iscsi_resp_t_r',
#        'iscsi_io_write' => 'write_io',
        'iscsi_latency' => 'iscsi_resp_t',
        'iscsi_data' => 'iscsi_data_rate',
        'iscsi_io_write' => 'iscsi_write_io',
        'fc_latency_read' => 'fc_resp_t_r',
        'fc_latency_write' => 'fc_resp_t_w',
#
        'smb_data' => 'smb_data_rate',
#        'fc_latency' => 'fc_',
        'fc_data' => 'fc_data_rate',
        'fc_io' => 'fc_io_rate',
        'nfs_data' => 'nfs_data_rate',
#        'iscsi_data' => 'data_rate',
#
#        'smb_data_write' => '0',
#        'smb_data_read' => '0',
#        'fc_latency' => '0.000',
#        'fc_io_read' => '0',
#        'fc_io_write' => '0',
#        'nfs_data_read' => '0'
#        'nfs_data_write' => '0',
#        'cpu_utilization' => '52',
#        'device_in' => '61',
#        'device_out' => '232',
    }
  );
  my %perf;
  my %wip_perf;
  my @architecture;
  my $parent_id = OracleZFS::md5_string("$hostcfg_id-$alias");
  push(@architecture, create_arc_hash($parent_id, $alias, "DEVICE", [], $hostcfg_id));
  #warn '$io,$data_t,$resp_t,$used,$free,$total,$replication,$snapshot';
  #warn "$io,$data_t,$resp_t,$used,$free,$total,$replication,$snapshot";
  $perf{oraclezfs}{DEVICE}{$parent_id}{$endtime}{io_rate} = $io;
  $perf{oraclezfs}{DEVICE}{$parent_id}{$endtime}{data_rate} = $data_t;
  $perf{oraclezfs}{DEVICE}{$parent_id}{$endtime}{resp_t} = $resp_t;
  $perf{oraclezfs}{DEVICE}{$parent_id}{$endtime}{capacity_used} = $used;
  $perf{oraclezfs}{DEVICE}{$parent_id}{$endtime}{capacity_free} = $free;
  $perf{oraclezfs}{DEVICE}{$parent_id}{$endtime}{capacity_used_percent} = $used / ($total / 100);
  $perf{oraclezfs}{DEVICE}{$parent_id}{$endtime}{capacity_total} = $total;
  $perf{oraclezfs}{DEVICE}{$parent_id}{$endtime}{replication_capacity_used} = $replication;
  $perf{oraclezfs}{DEVICE}{$parent_id}{$endtime}{snapshot_capacity_used} = $snapshot;

  for my $subsystem (keys %stats){
    my $act_subsystem = $subs_from_storname{$subsystem};
    next if (! defined $stats{$subsystem}{header} or ! defined $stats{$subsystem}{body});
    my $data = $stats{$subsystem}{body};
    my @header = split(/,/, $stats{$subsystem}{header});
    next if ($subsystem eq 'L2Cache Level Statistics' or $subsystem eq 'Project Level Statistics');
    #print Dumper \@data;
    #print Dumper \@header;
    my $counter = 1;
    foreach my $line (@{$data}){
      my @sp = split(/,/, $line);
      #print Dumper @sp;
      for my $i (0 .. $#header){

        my $stand_name = defined $standardize{$act_subsystem}{$header[$i]} ? $standardize{$act_subsystem}{$header[$i]} : $header[$i];

        $wip_perf{oraclezfs}{$act_subsystem}{$counter}{$stand_name} = $sp[$i];
      }
      my $id = $wip_perf{oraclezfs}{$act_subsystem}{$counter}{id};
      my $new_id = OracleZFS::md5_string("$id-$alias");
      my $ntime = $wip_perf{oraclezfs}{$act_subsystem}{$counter}{time};


      $perf{oraclezfs}{$act_subsystem}{$new_id}{$ntime} = $wip_perf{oraclezfs}{$act_subsystem}{$counter};

      my $label = defined $perf{oraclezfs}{$act_subsystem}{$new_id}{$ntime}{name} ? $perf{oraclezfs}{$act_subsystem}{$new_id}{$ntime}{name}: "templabel";
      delete $perf{oraclezfs}{$act_subsystem}{$new_id}{$ntime}{id};
      delete $perf{oraclezfs}{$act_subsystem}{$new_id}{$ntime}{time};
      delete $perf{oraclezfs}{$act_subsystem}{$new_id}{$ntime}{name};

      push(@architecture, create_arc_hash($new_id, $label, $act_subsystem, [$parent_id], $hostcfg_id, $id));
      $counter++;
    }

  }

  if (defined $perf{oraclezfs}{POOL}) {
    $xormon->hasData(1);
  } else {
    $xormon->hasData(0, "Unable to get data")
  }

  #write_out($mvc_folder, $alias);

  #save configuration
  my %new_device_info = (
    $parent_id => %device_conf
  );

  $xormon->saveConfJSON( \%new_device_info, 1 );
  #print Dumper \@architecture;
  $xormon->saveArchitecture(\@architecture);

  return \%perf;
}

sub create_arc_hash {
  my $item_id    = shift;
  my $label      = shift;
  my $subsystem  = shift;
  my @parents    = shift;
  my $hostcfg_id = shift;
  my $wwn        = shift;
  my $hw_type = "ORACLEZFS";

  my %arch = (
    "item_id" => $item_id,
    "label" => $label,
    "hw_type" => $hw_type,
    "subsystem" => $subsystem,
    "parents" => @parents,
    "hostcfg_id" => $hostcfg_id,
  );
  if ($subsystem eq "VOLUME"){
     $arch{wwn} = $wwn;
  }

  return \%arch;
}

sub add_byservices {
  my $url  = "https://$ip/api/service/$version/services";
  my $data = get_data($url);

  if ( $data eq "" ) {
    $logger->error("Couldn't get services");
    return;
  }

  my @arr = @{ $data->{services} };
  for my $i ( 0 .. $#arr ) {
    if ( $arr[$i]->{name} eq "iscsi"
      or $arr[$i]->{name} eq "nfs"
      or $arr[$i]->{name} eq "smb" )
    {
      # or $arr[$i]->{name} eq "sftp"
      if ( $arr[$i]->{'<status>'} eq "online" ) {
        next if (!defined $arr[$i]->{name});
        add_statistic( $arr[$i]->{name} );
      }
    }
  }
}

sub add_therest {
  my @arr = ( "node", "disk", "fc", "arc" );
  foreach my $stat (@arr) {
    add_statistic($stat);
  }
}

sub get_datasets {
  my $url  = "https://$ip/api/analytics/$version/datasets";
  my $data = get_data($url);
  my %ds;
  if ( $data eq "" ) { $logger->error("Couldnt get datasets"); return; }

  my @arr = @{ $data->{datasets} };
  for my $i ( 0 .. $#arr ) {
    $ds{ $arr[$i]->{name} } = {};
  }
  return %ds;
}

sub add_statistic {
  my $url       = "https://$ip/api/analytics/$version/datasets";
  my $foo       = shift;
  my $worksheet = "STOR2RRD_" . "$foo";
  my @arr       = @{ $stor{$worksheet} };
  my $uuid      = pop @arr;

  for my $i ( 0 .. $#arr ) {
    if ( !exists $datasets{ $arr[$i] } ) {
      my $fullstat = { statistic => $arr[$i] };
      post_data( $url, $fullstat ); 
    }
    my $bool = stat_to_worksheet( $arr[$i], $worksheet, $uuid );
    my $purl = "https://$ip/api/analytics/$version/datasets/$arr[$i]/data";
    if ( $bool eq "0" ) {
      put_data($purl);
    }
  }
}

sub stat_to_worksheet {
  my $stat      = shift;
  my $worksheet = shift;
  my $uuid      = shift;
  my $foo       = { name => $stat };#, seconds => 600 };
  my $url       = "https://$ip/api/analytics/$version/worksheets/$uuid/datasets";
  my $data      = get_data($url);
  my $bool      = "1";

  if ( $data eq "" ) { $logger->error("Couldnt get worksheet datasets"); return; }

  if ( !defined $data->{datasets} ) {
    $bool = "0";
    post_data( $url, $foo );
    return $bool;
  }
  my @arr     = @{ $data->{datasets} };
  my @storarr = @{ $stor{$worksheet} };
  if ( $#arr < $#storarr - 1 ) {
    $bool = "0";
    for my $i ( 0 .. $#arr ) {
      if ( $arr[$i]->{name} eq $stat ) {
        $bool = "1";
        last;
      }
    }
    if ( $bool eq "0" ) {
      post_data( $url, $foo );
    }
  }
  return $bool;
}

sub add_worksheets {
  my $url  = "https://$ip/api/analytics/$version/worksheets";
  my $data = get_data($url);
  if ( $data eq "" ) { $logger->error("Couldnt get worksheets"); return; }

  my @arr = @{ $data->{worksheets} };
  my %currentsheets;
  for my $i ( 0 .. $#arr ) {
    $currentsheets{ $arr[$i]->{name} } = {};
  }

  for my $worksheet ( keys %stor ) {
    if ( !exists $currentsheets{$worksheet} ) {
      my $foo = { name => $worksheet };
      post_data( $url, $foo );
    }
  }
  my $dat = get_data($url);
  if ( $dat eq "" ) { return; }
  my @wsarr = @{ $dat->{worksheets} };
  for my $k ( 0 .. $#wsarr ) {
    push( @{ $stor{ $arr[$k]{name} } }, $arr[$k]->{uuid} );
  }

  #print Dumper \%stor;
}

sub write_out {
  my $mvc_dir = shift;
  my $alias = shift;
  my $file_tmp = "$mvc_dir/data/$alias/$alias" . "_oraclezfsperf_" . $ft . ".out";

  #my $file_tmp = "oraclezfs-perf.txt";
  open( DATA, ">$file_tmp" ) or $logger->error("Cannot open file: $file_tmp : $!") && exit 1;
  for my $key ( keys %stats ) {
    if ( defined $stats{$key}{body}[0] ) {
      print DATA "$key\n";
      print DATA "$stats{$key}{header}\n";
      my @arr = @{ $stats{$key}{body} };
      for my $i ( 0 .. $#arr ) {
        print DATA "$arr[$i]\n";
      }
      print DATA "\n";
    }
  }
  close DATA;
}

sub poolcap_level_statistics {
  my %foo         = get_pools_aggr();
  my $header      = "id,time,name,fc_data_rate,fc_ops_rate,date_rate,io_rate,nfs3_data_rate,nfs4_data_rate,smb2_data_rate,smb3_data_rate,capacity,capacity_free,capacity_used,capacity_data,capacity_used_percent,snapshot_capacity_used,virtual_capacity_total,capacity_oversubscr_percentage,replication_capacity_used,capacity_available,usage_metasize,compression,usage_child_reservation,dedupsize,usage_reservation,usage_total,dedupratio,usage_metaused,total";
  my $perfhr      = "fc_bytes,fc_ops,iscsi_bytes,iscsi_ops,nfs3_bytes,nfs4-1_bytes,smb2_bytes,smb3_bytes";
  my @perfheaders = split( ",", $perfhr );
  my $name        = "Pool Level Statistics";
  if ( !defined $stats{$name}{body} ) {
    $stats{$name}{body} = [];
  }
  for my $pool ( keys %pools ) {
    my $body = "$pools{$pool}{asn}" . "-" . "$pool" . ",$endtime,$pool";
    foreach my $phead (@perfheaders) {
      if ( defined $foo{$pool}{$phead} ) {
        $body .= ",$foo{$pool}{$phead}";
      }
      else {
        $body .= ",";
      }
    }
    $body .= ",$pools{$pool}{data}{size},$pools{$pool}{data}{free},$pools{$pool}{data}{used},$pools{$pool}{data}{data},$pools{$pool}{data}{used_percentage},$pools{$pool}{data}{snapshots},$pools{$pool}{data}{subscription},$pools{$pool}{data}{subscription_percentage},$pools{$pool}{data}{usage_replication},$pools{$pool}{data}{available},$pools{$pool}{data}{usage_metasize},$pools{$pool}{data}{compression},$pools{$pool}{data}{usage_child_reservation},$pools{$pool}{data}{dedupsize},$pools{$pool}{data}{usage_reservation},$pools{$pool}{data}{usage_total},$pools{$pool}{data}{dedupratio},$pools{$pool}{data}{usage_metaused},$pools{$pool}{data}{total}";
    if ( !defined $stats{$name}{header} ) {
      $stats{$name}{header} = $header;
    }
    push( @{ $stats{$name}{body} }, $body );

    $used         += $pools{$pool}{data}{data};
    $free         += $pools{$pool}{data}{free};
    $total        += $pools{$pool}{data}{size};
    $replication  += $pools{$pool}{data}{usage_replication};
    $snapshot     += $pools{$pool}{data}{snapshots};
  }
}

sub luncap_level_statistics {
  my %lunaggr     = get_luns_aggr();
  #thin,status,encryption,reduction,dedup,compression,exported,
  my $header      = "id,time,name,fc_data_rate,fc_ops_rate,data_rate,io_rate,capacity_total,capacity_used_percent,capacity_data,capacity_used,snapshot_capacity_used";
  my $name        = "Volume Level Statistics";
  my $perfhr      = "fc_bytes,fc_ops,iscsi_bytes,iscsi_ops";
  my @perfheaders = split( ",", $perfhr );
  if ( !defined $stats{$name}{body} ) {
    $stats{$name}{body} = [];
  }
  for my $pool ( keys %pools ) {
    for my $project ( keys %{ $pools{$pool}{projects} } ) {
      for my $lun ( keys %{ $pools{$pool}{projects}{$project}{luns} } ) {
        my %foo  = %{ $pools{$pool}{projects}{$project}{luns} };
        my $body = "$foo{$lun}{uuid},$endtime,$lun";
        foreach my $phead (@perfheaders) {
          if ( defined $lunaggr{$lun}{$phead} ) {
            $body .= ",$lunaggr{$lun}{$phead}";
          }
          else {
            $body .= ",";
          }
        }
        #$foo{$lun}{thin},status,$foo{$lun}{reduction},$foo{$lun}{dedup},$foo{$lun}{encryption},$foo{$lun}{compression},$foo{$lun}{exported},
        $body .= ",$foo{$lun}{size},$foo{$lun}{used_percentage},$foo{$lun}{data},$foo{$lun}{used},$foo{$lun}{snapshots}";
        if ( !defined $stats{$name}{header} ) {
          $stats{$name}{header} = $header;
        }
        push( @{ $stats{$name}{body} }, $body );
      }
    }
  }
}

sub filesystemcap_level_statistics {
  my %shareaggr   = get_filesystems_aggr();
  my $header      = "id,time,name,nfs3_data_rate,nfs4_data_rate,smb2_data_rate,smb3_data_rate,capacity_quota,capacity_data,encryption,reduction,compression,capacity_used,snapshot_capacity_used";
  my $perfhr      = "nfs3_bytes,nfs4-1_bytes,smb2_bytes,smb3_bytes";
  my $caphr       = "quota,data,encryption,reduction,compression,used,snapshots";
  my @perfheaders = split( ",", $perfhr );
  my @capheaders  = split( ",", $caphr );
  my $name        = "Filesystem Level Statistics";
  if ( !defined $stats{$name}{body} ) {
    $stats{$name}{body} = [];
  }
  for my $pool ( keys %pools ) {
    for my $project ( keys %{ $pools{$pool}{projects} } ) {
      for my $share ( keys %{ $pools{$pool}{projects}{$project}{filesystems} } ) {
        my %foo  = %{ $pools{$pool}{projects}{$project}{filesystems} };
        my $body = "$foo{$share}{id},$endtime,$share";
        foreach my $phead (@perfheaders) {
          if ( defined $shareaggr{$share}{$phead} ) {
            $body .= ",$shareaggr{$share}{$phead}";
          }
          else {
            $body .= ",";
          }
        }
        foreach my $chead (@capheaders) {
          if ( defined $foo{$share}{$chead} ) {
            $body .= ",$foo{$share}{$chead}";
          }
          else {
            $body .= ",";
          }
        }
        if ( !$stats{$name}{header} ) {
          $stats{$name}{header} = $header;
        }
        push( @{ $stats{$name}->{body} }, $body );
      }
    }
  }
}

sub projectcap_level_statistics {
  my %foo     = get_projects_aggr();
  my $header  = "id,time,name,capacity_used,capacity_data,snapshot_capacity_used,fc_data_rate,fc_ops,data_rate,io_rate,smb2_data_rate,smb3_data_rate,nfs3_data_rate,nfs4_data_rate";
  my $hr      = "fc_bytes,fc_ops,iscsi_bytes,iscsi_ops,smb2_bytes,smb3_bytes,nfs3_bytes,nfs4-1_bytes";
  my @headers = split( ",", $hr );
  my $name    = "Project Level Statistics";
  if ( !defined $stats{$name}{body} ) {
    $stats{$name}{body} = [];
  }
  for my $pool ( keys %pools ) {
    for my $project ( keys %{ $pools{$pool}{projects} } ) {
      my %proj = %{ $pools{$pool}{projects}{$project} };
      my $body = "$proj{id},$endtime,$project,$proj{used},$proj{data},$proj{snapshot}";
      foreach my $head (@headers) {
        if ( defined $foo{$project}{$head} ) {
          $body .= ",$foo{$project}{$head}";
        }
        else {
          $body .= ",";
        }
      }
      push( @{ $stats{$name}{body} }, $body );
    }
  }
  $stats{$name}{header} = $header;
}

sub node_level_statistics {
  my %node = (
    get_cpu(), get_nicinout(),
    get_rw("fc"),    get_bytes("fc"),    get_rw( "fc",    "avglatency" ),
    get_rw("iscsi"), get_bytes("iscsi"), get_rw( "iscsi", "avglatency" ),
    get_rw( "nfs3", "bytes" ), get_rw( "nfs4-1", "bytes" ),
    get_rw( "smb2", "bytes" ), get_rw( "smb3",   "bytes" ),
    get_latency("fc"), get_latency("iscsi")
  );

  #print Dumper \%node;
  my $header  = "id,time,name,cpu_utilization,device_in,device_out,fc_io_read,fc_io_write,fc_data,fc_latency_read,fc_latency_write,iscsi_io_read,iscsi_io_write,iscsi_data,iscsi_latency_read,iscsi_latency_write,nfs_data_read,nfs_data_write,smb_data_read,smb_data_write,fc_latency,iscsi_latency,nfs_data,smb_data,fc_io_rate,iscsi_io_rate,data_rate,io_rate,read,write,io_read,io_write";
  my $hr      = "cpu_utilization,nic_data_in,nic_data_out,fc_io_read,fc_io_write,fc_bytes,fc_latency_read,fc_latency_write,iscsi_io_read,iscsi_io_write,iscsi_bytes,iscsi_latency_read,iscsi_latency_write";
  my $body    = "$asn,$endtime,$nodename";
  my $name    = "Node Level Statistics";
  my @headers = split( ",", $hr );
  if ( !defined $stats{$name}{body} ) {
    $stats{$name}{body} = [];
  }
  foreach my $head (@headers) {
    if ( defined $node{$head} ) {
      $body .= ",$node{$head}";
    }
    else {
      $body .= ",";
    }
  }
  #HERE
  my $nfs3dr     = defined $node{"nfs3_data_read"}    ? $node{"nfs3_data_read"}    : 0;
  my $nfs4dr     = defined $node{"nfs4-1_data_read"}  ? $node{"nfs4-1_data_read"}  : 0;
  my $nfs3dw     = defined $node{"nfs3_data_write"}   ? $node{"nfs3_data_write"}   : 0;
  my $nfs4dw     = defined $node{"nfs4-1_data_write"} ? $node{"nfs4-1_data_write"} : 0;
  my $snb2dr     = defined $node{"smb2_data_read"}    ? $node{"smb2_data_read"}    : 0;
  my $snb2dw     = defined $node{"smb2_data_write"}   ? $node{"smb2_data_write"}   : 0;
  my $snb3dr     = defined $node{"smb3_data_read"}    ? $node{"smb3_data_read"}    : 0;
  my $snb3dw     = defined $node{"smb3_data_write"}   ? $node{"smb3_data_write"}   : 0;
  my $fc_io_r    = defined $node{"fc_io_read"}        ? $node{"fc_io_read"}        : 0;
  my $fc_io_w    = defined $node{"fc_io_write"}       ? $node{"fc_io_write"}       : 0;
  my $iscsi_io_r = defined $node{"iscsi_io_read"}     ? $node{"iscsi_io_read"}     : 0;
  my $iscsi_io_w = defined $node{"iscsi_io_write"}    ? $node{"iscsi_io_write"}    : 0;
  my $iscsi_data = defined $node{"iscsi_data"}        ? $node{"iscsi_data"}        : 0;
  my $fc_data    = defined $node{"fc_data"}           ? $node{"fc_data"}           : 0;
  my $iscsi_resp = defined $node{"iscsi_latency"}     ? $node{"iscsi_latency"}     : 0;
  my $fc_resp    = defined $node{"fc_latency"}        ? $node{"fc_latency"}        : 0;

  my $nfs_read    = $nfs3dr + $nfs4dr;
  my $nfs_write   = $nfs3dw + $nfs4dw;
  my $nfs_total   = $nfs_read + $nfs_write;
  my $smb_read    = $snb2dr + $snb3dr;
  my $smb_write   = $snb2dw + $snb3dw;
  my $smb_total   = $smb_read + $smb_write;
  my $fc_total    = $fc_io_r + $fc_io_w;
  my $iscsi_total = $iscsi_io_r + $iscsi_io_w;
  my $io_rate_r   = $fc_io_r + $iscsi_io_r;
  my $io_rate_w   = $fc_io_w + $iscsi_io_w;
  my $io_rate     = $fc_total + $iscsi_total;
  my $data_rate   = $fc_data + $iscsi_data + $smb_total + $nfs_total;
  my $read        = $smb_read + $nfs_read;
  my $write       = $smb_write + $nfs_write;
  my $resp_total  = $iscsi_resp + $fc_resp;

  my $fc_l      = 0;
  if ( $node{fc_latency} ) {
    $fc_l = $node{fc_latency};
  }
  my $isc_l = 0;
  if ( $node{iscsi_latency} ) {
    $isc_l = $node{iscsi_latency};
  }
  $body .= ",$nfs_read,$nfs_write,$smb_read,$smb_write,$fc_l,$isc_l,$nfs_total,$smb_total,$fc_total,$iscsi_total,$data_rate,$io_rate,$read,$write,$io_rate_r,$io_rate_w";
  $stats{$name}->{header} = $header;
  push( @{ $stats{$name}->{body} }, $body );

  $io      += $io_rate ;
  $data_t  += $data_rate;
  $resp_t  += $resp_total;
}

sub disk_level_statistics {
  my %rw      = ( get_rw("io"), get_rw( "io", "bytes" ) );
  my $name    = "Disk Level Statistics";
  my $header  = "id,time,name,disk_iops,disk_data,data_read,data_write,io_read,io_write";
  my $hr      = "io_rate_read,io_rate_write,data_rate_read,date_rate_write";
  my @headers = split( ",", $hr );
  if ( !defined $stats{$name}{body} ) {
    $stats{$name}{body} = [];
  }
  for my $disk ( keys %disks ) {
    for my $pool ( keys %pools ) {
      if ( exists $pools{$pool}{disks}{$disk} ) {
        my $disk_iops = "";
        if ( defined $disks{$disk}->{'disk_iops'} ) {
          $disk_iops = $disks{$disk}->{'disk_iops'};
        }
        my $disk_data = "";
        if ( defined $disks{$disk}->{'disk_data'} ) {
          $disk_data = $disks{$disk}->{'disk_data'};
        }
        my $body = "$disks{$disk}{serial_number},$endtime,$disk,$disk_iops,$disk_data";
        foreach my $head (@headers) {
          if ( defined $rw{$head} ) {
            $body .= ",$rw{$head}";
          }
          else {
            $body .= ",";
          }
        }
        if ( !defined $stats{$name}{header} ) {
          $stats{$name}{header} = $header;
        }
        push( @{ $stats{$name}->{body} }, $body );
      }
    }
  }
}

sub initiator_level_statistics {
  my $name    = "Host Level Statistics";
  my $header  = "id,time,name,fc_kilobytes,fc_ops,iscsi_kilobytes,iscsi_ops";
  my $hr      = "fc_bytes,fc_ops,iscsi_bytes,iscsi_ops";
  my @headers = split( ",", $hr );
  $stats{$name}{body} = [];
  #print Dumper \%igs;
  for my $key ( keys %igs ) {
    my $id   = "$asn" . "-" . "$key";
    my $body = "$id,$endtime,$key";

    foreach my $head (@headers) {
      my $current_position_data = 0;
      for my $akey ( keys %{ $igs{$key} } ) {
        if ( $akey eq "node" ) { next; }
        if ( $igs{$key}{$akey}{$head} ) {
          $current_position_data += $igs{$key}{$akey}{$head};
        }
      }
      $body .= ",$current_position_data";
    }
    if ( !defined $stats{$name}{header} ) {
      $stats{$name}{header} = $header;
    }
    push( @{ $stats{$name}->{body} }, $body );
  }
}

sub port_level_statistics {
  my %devices = get_netdevice();
  my $header  = "id,time,name,data_rate";
  my $name    = "Port Level Statistics";
  if ( !defined $stats{$name}{body} ) {
    $stats{$name}{body} = [];
  }
  $stats{$name}{header} = $header;
  for my $key ( keys %devices ) {
    if ( $key ne "summary_port" ) {
      my $nic_k = 0;
      if ( $devices{$key}{nic_kilobytes} ) {
        $nic_k = $devices{$key}{nic_kilobytes};
      }
      my $body = "$key,$endtime,$key,$nic_k";
      push( @{ $stats{$name}->{body} }, $body );
    }
  }
}

sub cache_level_statistics {
  my $name    = "Cache Level Statistics";
  my $header  = "id,time,prefetched_data_hits,prefetched_metadata_hits,data_misses,metadata_misses,metadata_hits,ARC_headers,ARC_other,ARC_data,arc_size,arc_hitratio,data_hits";
  my $hr      = "prefetched_data_hits,prefetched_metadata_hits,data_misses,metadata_misses,ARC_headers,ARC_other,ARC_data,arc_size,arc_hitratio,data_hits,metadata_hits";
  my @headers = split( ",", $hr );
  my %cache   = ( get_cachesize(), get_cachehitratio(), get_cache() );
  if ( !defined $stats{$name}{body} ) {
    $stats{$name}{body} = [];
  }
  my $body = "$asn" . "_cache,$endtime";
  $stats{$name}{header} = $header;
  foreach my $head (@headers) {
    if ( defined $cache{$head} ) {
      $body .= ",$cache{$head}";
    }
    else {
      $body .= ",";
    }
  }
  push( @{ $stats{$name}->{body} }, $body );
}

sub l2cache_level_statistics {
  my $name   = "L2Cache Level Statistics";
  my $header = "id,time,l2arc_size,misses,hits";
  my %cache  = ( get_l2cachesize(), get_l2cache_access() );
  if ( !defined $stats{$name}{body} ) {
    $stats{$name}{body} = [];
  }
  my $body = "$asn" . "_l2cache,$endtime,$cache{arc_l2_size},$cache{access_io_misses},$cache{access_io_hits}";
  push( @{ $stats{$name}{body} }, $body );
  $stats{$name}{header} = $header;
}

sub cpu_level_statistics {
  my %cpus;
  my $url  = "https://$ip/$analytics/cpu.utilization[cpu]/data?start=$datatime&span=hour&granularity=minute";
  my $data = get_data($url);

  if ( $data eq "" ) { return; }
  %cpus = get_keyvaluedata( "cpu", "utilization", "[cpu]" );
  my $name   = "CPU Level Statistics";
  my $header = "id,time,name,cpu_utilization";
  $stats{$name}{header} = $header;
  if ( !defined $stats{$name}{body} ) {
    $stats{$name}{body} = [];
  }
  for my $cpu ( keys %cpus ) {
    if ( $cpu ne "summary_port" ) {
      my $body = "$asn" . "-cpu-$cpu" . ",$endtime,cpu-$cpu,$cpus{$cpu}{cpu_utilization}";
      push( @{ $stats{$name}{body} }, $body );
    }
  }
}

sub get_poolinfo {
  add_projects();
  add_luns();
  add_filesystems();
  add_poolsdata();
  add_filesystems_aggr();
  add_luns_aggr();
}

sub add_filesystems_aggr {
  my %fsa;
  my @whats      = ( "nfs3",  "nfs4-1", "smb2",  "smb3" );
  my @operations = ( "bytes", "bytes",  "bytes", "bytes" );
  for my $i ( 0 .. 3 ) {
    my %shareaggr = get_keyvaluedata( $whats[$i], $operations[$i], "[share]" );
    for my $key ( keys %shareaggr ) {
      for my $akey ( keys %{ $shareaggr{$key} } ) {
        $fsa{$key}->{$akey} = $shareaggr{$key}->{$akey};
      }
    }
  }
  add_pdata( \%fsa, "filesystems" );
}

sub add_luns_aggr {
  my %lns;
  my @whats      = ( "iscsi", "fc",  "iscsi", "fc" );
  my @operations = ( "ops",   "ops", "bytes", "bytes" );
  for my $i ( 0 .. 3 ) {
    my %lunaggr = get_keyvaluedata( $whats[$i], $operations[$i], "[lun]" );
    for my $key ( keys %lunaggr ) {
      for my $akey ( keys %{ $lunaggr{$key} } ) {
        $lns{$key}->{$akey} = $lunaggr{$key}->{$akey};
      }
    }
  }
  add_pdata( \%lns, "luns" );
}

sub add_pdata {
  my $hathash  = shift;
  my %whathash = %{$hathash};
  my $what     = shift;
  for my $pool ( keys %pools ) {
    if ( defined $pools{$pool}->{projects} ) {
      for my $project ( keys %{ $pools{$pool}->{projects} } ) {
        next if ( !defined $pools{$pool}->{projects}->{$project}->{$what} );
        for my $share ( keys %{ $pools{$pool}->{projects}->{$project}->{$what} } ) {
          for my $key ( keys %whathash ) {
            for my $akey ( keys %{ $whathash{$key} } ) {
              if ( $key eq "summary_port" and $whathash{$key}->{$akey} eq 0 ) {
                my $foo = basename( $akey, "__" );
                $pools{$pool}->{projects}->{$project}->{$what}->{$share}->{p_data}->{$foo} = 0;
              }
              if ( $share eq $key and $whathash{$key}->{$akey} ne 0 ) {
                $pools{$pool}->{projects}->{$project}->{$what}->{$share}->{p_data}->{$akey} = $whathash{$key}->{$akey};
              }
            }
          }
        }
      }
    }
  }
}

# returns sum of all LUN sizes #
sub add_luns {
  for my $pool ( keys %pools ) {
    my $subs = 0;
    for my $project ( keys %{ $pools{$pool}{projects} } ) {
      my $url  = "https://$ip/$storpath/$version/pools/$pool/projects/$project/luns";
      my $data = get_data($url);

      if ( $data eq "" ) { return; }

      my @arr = @{ $data->{luns} };
      my %volsize;
      for my $i ( 0 .. $#arr ) {
        $volsize{ $arr[$i]->{name} }->{data}            =  $arr[$i]->{space_data};
        $volsize{ $arr[$i]->{name} }->{size}            =  $arr[$i]->{volsize};
        $volsize{ $arr[$i]->{name} }->{used}            =  $arr[$i]->{space_total};
        $volsize{ $arr[$i]->{name} }->{snapshots}       =  $arr[$i]->{space_snapshots};
        $volsize{ $arr[$i]->{name} }->{used_percentage} = sprintf( "%.1f", $arr[$i]->{space_total} / ( $arr[$i]->{volsize} / 100 ) );
        $volsize{ $arr[$i]->{name} }->{compression}     = $arr[$i]->{compression};
        $volsize{ $arr[$i]->{name} }->{encryption}      = $arr[$i]->{encryption};
        $volsize{ $arr[$i]->{name} }->{reduction}       = $arr[$i]->{compressratio};
        $volsize{ $arr[$i]->{name} }->{uuid}            = $arr[$i]->{lunguid};
        $volsize{ $arr[$i]->{name} }->{exported}        = jsonbool_totext( $arr[$i]->{exported} );
        $volsize{ $arr[$i]->{name} }->{thin}            = jsonbool_totext( $arr[$i]->{sparse} );
        $volsize{ $arr[$i]->{name} }->{dedup}           = jsonbool_totext( $arr[$i]->{dedup} );
        $volsize{ $arr[$i]->{name} }->{targetgroup}     = $arr[$i]->{targetgroup};
        $volsize{ $arr[$i]->{name} }->{initiatorgroup}  = $arr[$i]->{initiatorgroup};
        $volsize{ $arr[$i]->{name} }->{lun_id}          = $arr[$i]->{lunguid};
        $subs += $arr[$i]->{volsize};
      }
      $pools{$pool}->{subscription} = $subs;
      $pools{$pool}->{projects}->{$project}->{luns} = {%volsize};
    }
  }
}

sub add_filesystems {

  for my $pool ( keys %pools ) {
    my $subs = 0;
    for my $project ( keys %{ $pools{$pool}{projects} } ) {
      my $url  = "https://$ip/$storpath/$version/pools/$pool/projects/$project/filesystems";
      my $data = get_data($url);

      if ( $data eq "" ) { return; }

      my @arr = @{ $data->{filesystems} };
      my %volsize;
      for my $i ( 0 .. $#arr ) {
        $volsize{ $arr[$i]->{name} }->{compression} = $arr[$i]->{compression};
        $volsize{ $arr[$i]->{name} }->{encryption}  = $arr[$i]->{encryption};
        $volsize{ $arr[$i]->{name} }->{reduction}   = $arr[$i]->{compressratio};
        $volsize{ $arr[$i]->{name} }->{snapshots}   = $arr[$i]->{space_snapshots};
        $volsize{ $arr[$i]->{name} }->{quota}       = $arr[$i]->{quota};
        $volsize{ $arr[$i]->{name} }->{used}        = $arr[$i]->{space_total};
        $volsize{ $arr[$i]->{name} }->{data}        = $arr[$i]->{space_data};
        $volsize{ $arr[$i]->{name} }->{id}          = $arr[$i]->{id};
      }
      $pools{$pool}->{projects}->{$project}->{filesystems} = {%volsize};
    }
  }

  #print Dumper \%pools;
  #return %volsize;
}

# returns array of pool names #
sub add_projects {
  my $url  = "https://$ip/$storpath/$version/pools";
  my $data = get_data($url);
  my %poolnames;

  if ( $data eq "" ) { return; }

  my @arr = @{ @$data{pools} };
  for my $i ( 0 .. $#arr ) {
    my $name = $arr[$i]->{name};
    if ( $arr[$i]->{status} eq "exported" ) {
      next;
    }
    my $url         = "https://$ip/$storpath/$version/pools/$name/projects";
    my $projectdata = get_data($url);
    if ( $projectdata eq "" ) { return; }
    my @projects = @{ @$projectdata{projects} };
    my %projectnames;
    for my $j ( 0 .. $#projects ) {
      my %projectitems;
      $projectitems{snapshot}                = $projects[$j]->{space_snapshots};
      $projectitems{used}                    = $projects[$j]->{space_total};
      $projectitems{data}                    = $projects[$j]->{space_data};
      $projectitems{id}                      = $projects[$j]->{id};
      $projectnames{ $projects[$j]->{name} } = {%projectitems};
    }
    $pools{$name}->{projects} = {%projectnames};
  }
}

# returns pools info (CAPACITY->Pool) #
sub add_poolsdata {
  my %poolsarr;
  my $k    = 0;
  my $url  = "https://$ip/$storpath/$version/pools";
  my $data = get_data($url);

  if ( $data eq "" ) { return; }

  foreach my $pool ( @{ $data->{pools} } ) {
    if ( $pool->{status} eq "exported" ) {
      next;
    }
    my %table;
    my $asn = "";
    $asn                            = $pool->{asn};
    $table{free}                    = $pool->{usage}->{free};
    $table{size}                    = $table{free} + $pool->{usage}->{used};
    $table{data}                    = $pool->{usage}->{usage_data};
    $table{usage_replication}       = $pool->{usage}->{usage_replication};
    $table{available}               = $pool->{usage}->{available};
    $table{usage_metasize}          = $pool->{usage}->{usage_metasize};
    $table{compression}             = $pool->{usage}->{compression};
    $table{usage_child_reservation} = $pool->{usage}->{usage_child_reservation};
    $table{dedupsize}               = $pool->{usage}->{dedupsize};
    $table{usage_reservation}       = $pool->{usage}->{usage_reservation};
    $table{usage_total}             = $pool->{usage}->{usage_total};
    $table{dedupratio}              = $pool->{usage}->{dedupratio};
    $table{usage_metaused}          = $pool->{usage}->{usage_metaused};
    $table{total}                   = $pool->{usage}->{total};
    $table{snapshots}               = $pool->{usage}->{usage_snapshots};
    $table{used}                    = $pool->{usage}->{usage_total};
    my $tbl_size = $table{size} / 100;

    if ( !$tbl_size or $tbl_size eq "" or $tbl_size == 0 ) {
      $tbl_size = 1;
    }
    $table{used_percentage}         = sprintf( "%.1f", $table{used} / ($tbl_size) );
    $table{subscription}            = $pools{ $pool->{name} }{subscription};
    $table{subscription_percentage} = sprintf( "%.1f", ( $pools{ $pool->{name} }{subscription} / ($tbl_size) ) );
    $table{used}                    = $table{used};
    $table{free}                    = $table{free};
    $table{size}                    = $table{size};
    $table{data}                    = $table{data};
    $table{snapshots}               = $table{snapshots};
    $k++;
    $pools{ $pool->{name} }->{data} = {%table};
    $pools{ $pool->{name} }->{asn}  = $asn;
  }
}

sub get_filesystems_aggr {
  my %proj;
  for my $pool ( keys %pools ) {
    if ( defined $pools{$pool}->{projects} ) {
      for my $project ( keys %{ $pools{$pool}->{projects} } ) {
        if ( defined $pools{$pool}{projects}{$project}{filesystems} ) {
          for my $share ( keys %{ $pools{$pool}{projects}{$project}{filesystems} } ) {
            for my $pdata ( keys %{ $pools{$pool}{projects}{$project}{filesystems}{$share}{p_data} } ) {
              $proj{$share}{$pdata} += $pools{$pool}{projects}{$project}{filesystems}{$share}{p_data}{$pdata};
              $proj{$share}{id} = $pools{$pool}{projects}{$project}{filesystems}{$share}{id};
            }
          }
        }
      }
    }
  }

  return %proj;
}

sub get_luns_aggr {
  my %proj;
  for my $pool ( keys %pools ) {
    if ( defined $pools{$pool}->{projects} ) {
      for my $project ( keys %{ $pools{$pool}{projects} } ) {
        if ( defined $pools{$pool}{projects}{$project}{luns} ) {
          for my $lun ( keys %{ $pools{$pool}{projects}{$project}{luns} } ) {
            for my $pdata ( keys %{ $pools{$pool}{projects}{$project}{luns}{$lun}{p_data} } ) {
              $proj{$lun}{$pdata} += $pools{$pool}{projects}{$project}{luns}{$lun}{p_data}{$pdata};
              $proj{$lun}{uuid} = $pools{$pool}{projects}{$project}{luns}{$lun}{uuid};
            }
          }
        }
      }
    }
  }

  return %proj;
}

sub get_projects_aggr {
  my %proj;
  for my $pool ( keys %pools ) {
    my $lunbool   = 0;
    my $sharebool = 0;
    if ( defined $pools{$pool}->{projects} ) {
      for my $project ( keys %{ $pools{$pool}->{projects} } ) {
        $proj{$project}{id} = $pools{$pool}{projects}{$project}{id};
        if ( %{ $pools{$pool}{projects}{$project}{luns} } ) {
          for my $lun ( keys %{ $pools{$pool}->{projects}->{$project}->{luns} } ) {
            if ( defined $pools{$pool}{projects}{$project}{luns}{$lun}{p_data} ) {
              $lunbool = 1;
              for my $pdata ( keys %{ $pools{$pool}{projects}{$project}{luns}{$lun}{p_data} } ) {
                $proj{$project}{$pdata} += $pools{$pool}{projects}{$project}{luns}{$lun}{p_data}{$pdata};
              }
            }
          }
        }
        if ( $lunbool == 0 ) {
          if ( !defined $proj{$pool}{iscsi_bytes} ) {
            $proj{$pool}{iscsi_bytes} = 0;
          }
          if ( !defined $proj{$pool}{iscsi_ops} ) {
            $proj{$pool}{iscsi_ops} = 0;
          }
          if ( !defined $proj{$pool}{fc_bytes} ) {
            $proj{$pool}{fc_bytes} = 0;
          }
          if ( !defined $proj{$pool}{fc_ops} ) {
            $proj{$pool}{fc_ops} = 0;
          }
        }
        if ( %{ $pools{$pool}{projects}{$project}{filesystems} } ) {
          for my $share ( keys %{ $pools{$pool}{projects}{$project}{filesystems} } ) {
            if ( defined $pools{$pool}{projects}{$project}{filesystems}{$share}{p_data} ) {
              for my $pdata ( keys %{ $pools{$pool}{projects}{$project}{filesystems}{$share}{p_data} } ) {
                $sharebool = 1;
                $proj{$project}{$pdata} += $pools{$pool}{projects}{$project}{filesystems}{$share}{p_data}{$pdata};
              }
            }
          }
        }
        if ( $sharebool == 0 ) {

          #$proj{$pool}{sftp_kilobytes} = 0;
          if ( !defined $proj{$pool}{'nfs4-1_bytes'} ) {
            $proj{$pool}{'nfs4-1_bytes'} = 0;
          }
          if ( !defined $proj{$pool}{'nfs3_bytes'} ) {
            $proj{$pool}{'nfs3_bytes'} = 0;
          }
          if ( !defined $proj{$pool}{'smb2_bytes'} ) {
            $proj{$pool}{'smb2_bytes'} = 0;
          }
          if ( !defined $proj{$pool}{'smb3_bytes'} ) {
            $proj{$pool}{'smb3_bytes'} = 0;
          }
        }
      }
    }
    $proj{$pool}{'io_rate'}   = $proj{$pool}{'iscsi_ops'} + $proj{$pool}{'fc_ops'};
    $proj{$pool}{'data_rate'} = $proj{$pool}{'iscsi_bytes'} + $proj{$pool}{'fc_bytes'};
  }

  return %proj;
}

sub get_pools_aggr {
  my %proj;
  for my $pool ( keys %pools ) {
    if ( defined $pools{$pool}->{projects} ) {
      for my $project ( keys %{ $pools{$pool}->{projects} } ) {
        my $lunbool   = 0;
        my $sharebool = 0;

        if ( defined $pools{$pool}{projects}->{$project}->{luns} ) {
          for my $lun ( keys %{ $pools{$pool}->{projects}->{$project}->{luns} } ) {
            if ( defined $pools{$pool}{projects}{$project}{luns}{$lun}{p_data} ) {
              $lunbool = 1;
              for my $pdata ( keys %{ $pools{$pool}{projects}{$project}{luns}{$lun}{p_data} } ) {
                $proj{$pool}{$pdata} += $pools{$pool}{projects}{$project}{luns}{$lun}{p_data}{$pdata};
              }
            }
          }
        }
        if ( $lunbool == 0 ) {
          if ( !defined $proj{$pool}{iscsi_bytes} ) {
            $proj{$pool}{iscsi_bytes} = 0;
          }
          if ( !defined $proj{$pool}{iscsi_ops} ) {
            $proj{$pool}{iscsi_ops} = 0;
          }
          if ( !defined $proj{$pool}{fc_bytes} ) {
            $proj{$pool}{fc_bytes} = 0;
          }
          if ( !defined $proj{$pool}{fc_ops} ) {
            $proj{$pool}{fc_ops} = 0;
          }
        }

        if ( defined $pools{$pool}{projects}{$project}{filesystems} ) {
          for my $share ( keys %{ $pools{$pool}{projects}{$project}{filesystems} } ) {
            if ( defined $pools{$pool}{projects}{$project}{filesystems}{$share}{p_data} ) {
              $sharebool = 1;
              for my $pdata ( keys %{ $pools{$pool}{projects}{$project}{filesystems}{$share}{p_data} } ) {
                $proj{$pool}{$pdata} += $pools{$pool}{projects}{$project}{filesystems}{$share}{p_data}{$pdata};
              }
            }
          }
        }
        if ( $sharebool == 0 ) {

          #$proj{$pool}{sftp_kilobytes} = "";
          if ( !defined $proj{$pool}{'nfs4-1_bytes'} ) {
            $proj{$pool}{'nfs4-1_bytes'} = 0;
          }
          if ( !defined $proj{$pool}{'nfs3_bytes'} ) {
            $proj{$pool}{'nfs3_bytes'} = 0;
          }
          if ( !defined $proj{$pool}{'smb2_bytes'} ) {
            $proj{$pool}{'smb2_bytes'} = 0;
          }
          if ( !defined $proj{$pool}{'smb3_bytes'} ) {
            $proj{$pool}{'smb3_bytes'} = 0;
          }
        }
      }
    }
  }

  return %proj;
}

sub get_cachesize {
  my $url  = "https://$ip/$analytics/arc.size/data?start=$datatime&span=hour&granularity=minute";
  my $data = get_data($url);

  if ( $data eq "" ) { return; }

  my %a = work_valuesonly( $data, "arc", "size" );
  return %a;
}

sub get_cachehitratio {
  my $url  = "https://$ip/$analytics/arc.hitratio/data?start=$datatime&span=hour&granularity=minute";
  my $data = get_data($url);

  if ( $data eq "" ) { return; }

  my %a = work_valuesonly( $data, "arc", "hitratio" );
  return %a;
}

sub get_l2cachesize {
  my $url  = "https://$ip/$analytics/arc.l2_size/data?start=$datatime&span=hour&granularity=minute";
  my $data = get_data($url);

  if ( $data eq "" ) { return; }

  my %a = work_valuesonly( $data, "arc", "l2_size" );
  return %a;
}

sub get_l2cache_access {
  my $url  = "https://$ip/$analytics/arc.l2_accesses[hit/miss]/data?start=$datatime&span=hour&granularity=minute";
  my $data = get_data($url);

  if ( $data eq "" ) { return; }

  my %a = work_rwlikedata( $data, "access", "kilobytes", "mh" );
  return %a;
}

sub get_cache {
  my %lns;
  my %a = ( get_keyvaluedata( "arc", "size", "[component]" ), get_keyvaluedata( "arc", "accesses", "[hit/miss]" ) );

  for my $key ( keys %a ) {
    for my $akey ( keys %{ $a{$key} } ) {
      if ( $key ne "summary_port" ) {
        my $k = $key;
        $key =~ s/ /_/g;
        $lns{$key} = $a{$k}{$akey};
      }
    }
  }
  return %lns;
}

# calculates iops for every disk stored in  global hash disks #
sub get_diskdata {
  my %disk_data = get_keyvaluedata( "io", "bytes", "[disk]" );
  my %disk_ops  = get_keyvaluedata( "io", "ops",   "[disk]" );

  for my $key ( keys %disks ) {
    if ( exists $disk_data{$key} ) {
      $disks{$key}->{disk_data} = $disk_data{$key}{io_bytes};
    }
    else {
      $disks{$key}->{disk_data} = 0;
    }
    if ( exists $disk_ops{$key} ) {
      $disks{$key}->{disk_iops} = $disk_ops{$key}{io_ops};
    }
    else {
      $disks{$key}->{disk_iops} = 0;
    }
  }
}

# return read and write operations works for iSCSI(NODE->iSCSI) and IO(DISK->IO) #
sub get_rw {
  my $what   = shift;
  my $option = shift;
  my $foo    = "";
  my %rwops;

  my $url = "";
  if ( !defined $option ) {
    $option = "ops";
  }
  $foo = "$what" . ".$option";

  $url = "https://$ip/$analytics/$foo" . "[op]/data?start=$datatime&span=hour&granularity=minute";

  my $data = get_data($url);
  if ( $data eq "" ) { return; }

  return work_rwlikedata( $data, $what, $option );
}

sub work_rwlikedata {
  my $data   = shift;
  my $what   = shift;
  my $option = shift;
  my $type   = shift;
  my %rwops;
  my $tp         = "";
  my $denom      = 1;
  my $toval      = 0;
  my $ttval      = 0;
  my $type_one   = "fail";
  my $type_two   = "fail";
  my $data_count = 0;

  if ( $option and $option eq "bytes" ) {
    $tp = "_data";
  }
  elsif ( $option and $option eq "avglatency" ) {
    $denom = 1000;
    $tp    = "_latency";
  }
  else {
    $tp = "_io";
  }
  if ( !defined @$data{"data"} ) {
    return;
  }
  my @arr = @{ @$data{"data"} };
  @arr = @arr[ $#arr - 4 .. $#arr ];

  if ( !defined $type or $type eq "rw" ) {
    $type_one = "read";
    $type_two = "write";
  }
  elsif ( $type eq "mh" ) {
    $type_one = "hits";
    $type_two = "misses";
  }
  else {
    $type_one = "in";
    $type_two = "out";
  }

  if ( $type_one eq "fail" ) { $logger->error("Key type fail"); }

  for my $i ( 0 .. $#arr ) {
    my $neco = $arr[$i]{data}->{value};
    if ( !defined $neco or $neco eq "" ) {
      $neco = 0;
    }
    if ( $neco eq 0 ) { $data_count++; }

    else {
      if ( lc( $arr[$i]{data}->{data}->[0]->{key} ) eq $type_one ) {
        $toval += $arr[$i]{data}->{data}->[0]->{value};

        if ( !defined $arr[$i]{data}->{data}->[1]->{value} ) {
          $ttval += 0;
        }
        else {
          $ttval += $arr[$i]{data}->{data}->[1]->{value};
        }
        $data_count++;
      }
      else {
        if ( !defined $arr[$i]{data}->{data}->[1]->{value} ) {
          $toval += 0;
        }
        else {
          $toval += $arr[$i]{data}->{data}->[1]->{value};
        }
        $ttval += $arr[$i]{data}->{data}->[0]->{value};
        $data_count++;
      }
    }
  }
  if ( $option and $option eq "avglatency" ) {
    $rwops{ "$what" . $tp . "_" . "$type_one" } = sprintf( "%.3f", ( $toval / $data_count ) / $denom );
    $rwops{ "$what" . $tp . "_" . "$type_two" } = sprintf( "%.3f", ( $ttval / $data_count ) / $denom );
  }
  else {
    $rwops{ "$what" . $tp . "_" . "$type_one" } = sprintf( "%.f", ( $toval / $data_count ) / $denom );
    $rwops{ "$what" . $tp . "_" . "$type_two" } = sprintf( "%.f", ( $ttval / $data_count ) / $denom );
  }

  # print Dumper \%rwops;
  return %rwops;
}

sub get_keyvaluedata {
  my $what     = shift;
  my $option   = shift;
  my $brackets = shift;
  my %rwops;
  my $foo = "";

  my $url = "";
  if ( !defined $option ) {
    $option = "ops";
  }
  $foo = "$what" . ".$option" . "$brackets";
  $url = "https://$ip/$analytics/$foo/data?start=$datatime&span=hour&granularity=minute";
  my $data = get_data($url);
  if ( $data eq "" ) { return; }
  return work_keyvaluedata( $data, $what, $option );
}

sub work_keyvaluedata {
  my $data   = shift;
  my $what   = shift;
  my $option = shift;
  my $type   = shift;
  my %lunstat;
  my $denom = 1;
  my $check = 0;

  if ( $option eq "bytes" ) {
    $denom = 1;
  }
  if ( !defined @$data{"data"} ) {
    return;
  }
  my @arr = @{ @$data{data} };
  @arr = @arr[ $#arr - 4 .. $#arr ];
  for my $i ( 0 .. $#arr ) {
    next if ( !@arr or !defined $arr[$i]->{data}->{data} );
    my @innerarr = @{ $arr[$i]->{data}->{data} };
    for my $k ( 0 .. $#innerarr ) {
      if ( defined $innerarr[$k]->{key} ) {
        $lunstat{ basename( $innerarr[$k]->{key}, '/' ) }->{ $what . "_" . $option } += $innerarr[$k]->{value};
        $check++;
      }
    }
  }

  #print Dumper \%lunstat;
  #print $checker;
  for my $key ( keys %lunstat ) {
    $lunstat{$key}->{ $what . "_" . $option } = sprintf( "%.f", ( ( $lunstat{$key}->{ $what . "_" . $option } / 5 ) / $denom ) );
    $lunstat{summary_port}->{ "summary__" . $what . "_" . $option } += $lunstat{$key}->{ $what . "_" . $option };
  }
  if ( $check == 0 ) {
    $lunstat{summary_port}->{ "summary__" . $what . "_" . $option } = 0;
  }

  return %lunstat;
}

# returns iops by protocol (NODE->IO) #
sub get_iops {
  my $protocol = shift;
  my $url      = "https://$ip/$analytics/$protocol.ops/data?start=$datatime&span=hour&granularity=minute";
  my $data     = get_data($url);

  if ( $data eq "" ) { return; }

  return work_valuesonly( $data, "$protocol", "iops" );
}

# returns latency of a protocol #
sub get_latency {
  my $protocol = shift;
  my $url      = "";
  my $data     = "";
  $url  = "https://$ip/$analytics/$protocol.avglatency/data?start=$datatime&span=hour&granularity=minute";
  $data = get_data( $url, "dontcatch" );
  if ( $data eq "" ) {
    $url  = "https://$ip/$analytics/$protocol.ops[latency]/data?start=$datatime&span=hour&granularity=minute";
    $data = get_data($url);
    if ( $data eq "" ) { return; }

    return calculate_latency( $data, "$protocol" );
  }

  return work_valuesonly( $data, "$protocol", "latency" );
}

sub get_bytes {
  my $protocol = shift;
  my $url      = "https://$ip/$analytics/$protocol.bytes/data?start=$datatime&span=hour&granularity=minute";
  my $data     = get_data($url);

  if ( $data eq "" ) { return; }

  return work_valuesonly( $data, "$protocol", "bytes" );
}

# returns cpu load (NODE->Cpu) #
sub get_cpu {
  my $url  = "https://$ip/$analytics/cpu.utilization/data?start=$datatime&span=hour&granularity=minute";
  my $data = get_data($url);

  if ( $data eq "" ) { return; }

  #print Dumper \%cpu;
  return work_valuesonly( $data, "cpu", "utilization" );
}

sub calculate_latency {
  my $data   = shift;
  my $what   = shift;
  my $option = shift;
  my %values;
  my $value      = 0;
  my $data_count = 0;

  if ( !defined @$data{data} ) {
    return;
  }
  my @arr = @{ @$data{data} };
  @arr = @arr[ $#arr - 4 .. $#arr ];
  for my $i ( 0 .. $#arr ) {
    my $sample_sum       = 0;
    my $sample_value_sum = 0;
    if ( $arr[$i]->{data}->{data} ) {
      my @lat_break = @{ $arr[$i]->{data}->{data} };
      for my $k ( 0 .. $#lat_break ) {
        $sample_value_sum += $lat_break[$k]->{value};
        $sample_sum       += $lat_break[$k]->{key} * $lat_break[$k]->{value};
      }
      if ( $sample_value_sum == 0 ) {
        $sample_value_sum = 1;
      }
      $value += ( $sample_sum / $sample_value_sum );
    }
    $data_count++;
  }
  $values{ "$what" . "_" . "latency" } = sprintf( "%.3f", ( ( $value / $data_count ) / 1000 ) );
  return %values;
}

sub work_valuesonly {
  my $data   = shift;
  my $what   = shift;
  my $option = shift;
  my $unit   = shift;
  my %values;
  my $value;
  my $denom      = 1;
  my $data_count = 0;

  if ( defined $unit and $unit eq "bytes" ) {
    $denom = 1;
  }
  if ( !@$data{data} or ref( @$data{data} ) ne "ARRAY" ) {
    return;
  }
  my @arr = @{ @$data{data} };
  @arr = @arr[ $#arr - 4 .. $#arr ];
  for my $i ( 0 .. $#arr ) {
    if ( $arr[$i]->{data}->{value} ) {
      $value += $arr[$i]->{data}->{value};
    }
    else {
      $value += 0;
    }
    $data_count++;
  }
  if ( $option eq "avglatency" ) {
    $values{ "$what" . "_" . "$option" } = sprintf( "%.3f", ( ( $value / $data_count ) / $denom ) );
  }
  elsif ( $option eq "latency" ) {
    $values{ "$what" . "_" . "$option" } = sprintf( "%.3f", ( ( $value / $data_count ) / 1000 ) );
  }
  else {
    $values{ "$what" . "_" . "$option" } = sprintf( "%.f", ( ( $value / $data_count ) / $denom ) );
  }
  return %values;
}

# returns data by protocol and specified unit of data (NODE->Data>FCP/iSCSI/FTP) #
sub get_protocoldata {
  my $protocol = shift;
  my $unit     = $protocol_unit{$protocol};
  my $url      = "https://$ip/$analytics/$protocol.$unit/data?start=$datatime&span=hour&granularity=minute";
  my $data     = get_data($url);

  if ( $data eq "" ) { return; }

  return work_valuesonly( $data, $protocol, "data", $unit );
}

# returns network in and out trafic on nic port (NODE->Data>'Network IN/OUT') #
sub get_nicinout {
  my %netinout;

  my $url  = "https://$ip/$analytics/nic.kilobytes[direction]/data?start=$datatime&span=hour&granularity=minute";
  my $data = get_data($url);

  if ( $data eq "" ) { return; }

  return work_rwlikedata( $data, "nic", "bytes", "in" );
}

sub get_netinout {
  my %netinout;

  my $url  = "https://$ip/$analytics/net.kilobytes[direction]/data?start=$datatime&span=hour&granularity=minute";
  my $data = get_data($url);

  if ( $data eq "" ) { return; }

  return work_rwlikedata( $data, "net", "kilobytes", "in" );
}

# return data rate by device (NODE->Data>'Network Device') NOTDONE #
sub get_netdevice {
  my $url  = "https://$ip/$analytics/nic.kilobytes[device]/data?start=$datatime&span=hour&granularity=minute";
  my $data = get_data($url);

  if ( $data eq "" ) { return; }

  my %nic = work_keyvaluedata( $data, "nic", "kilobytes" );
  for my $key ( keys %nic ) {
    for my $nkey ( keys %{ $nic{$key} } ) {
      $devices{$key}->{$nkey} = $nic{$key}->{$nkey};
    }
  }

  #print Dumper \%devices;
  return %devices;
}

sub get_devices {
  my $url  = "https://$ip/api/network/$version/devices";
  my $data = get_data($url);

  if ( $data eq "" ) { return; }
  my @arr = @{ $data->{devices} };
  for my $i ( 0 .. $#arr ) {
    $devices{ $arr[$i]{device} }->{factory_mac} = $arr[$i]{factory_mac};
  }
}

sub add_initiatordata {
  my %fsa;
  my @whats      = ( "iscsi", "iscsi", "fc",  "fc" );
  my @operations = ( "ops",   "bytes", "ops", "bytes" );
  for my $i ( 0 .. $#whats ) {
    my %shareaggr = get_keyvaluedata( $whats[$i], $operations[$i], "[initiator]" );
    for my $key ( keys %shareaggr ) {
      for my $akey ( keys %{ $shareaggr{$key} } ) {
        $fsa{$key}->{$akey} = $shareaggr{$key}->{$akey};
      }
    }
  }
  for my $group ( keys %igs ) {
    if ( defined $igs{$group} ) {
      for my $initiator ( keys %{ $igs{$group} } ) {
        if ( $initiator eq "node" ) {
          next;
        }
        for my $key ( keys %fsa ) {
          for my $akey ( keys %{ $fsa{$key} } ) {
            if ( $key eq "summary_port" and $fsa{$key}->{$akey} eq 0 ) {
              my $foo = basename( $akey, "__" );
              if ( !defined $igs{$group}{$initiator}{$foo} ) {
                $igs{$group}{$initiator}{$foo} = 0;
              }
            }
            if ( $igs{$group}{$initiator}{alias} eq $key and $fsa{$key}->{$akey} ne 0 ) {
              if ( !$igs{$group}{$initiator}{$akey} ) {
                $igs{$group}{$initiator}{$akey} = $fsa{$key}->{$akey};
              }
              else {
                $igs{$group}{$initiator}{$akey} += $fsa{$key}->{$akey};
              }
            }
          }
        }
      }
    }
  }

  #print Dumper \%fsa;
  #print Dumper \%igs;
}

sub get_initiatorgroups {
  my @protocols = ( "iscsi", "fc" );
  for my $i ( 0 .. $#protocols ) {
    my $url  = "https://$ip/api/san/$version/$protocols[$i]/initiator-groups";
    my $data = get_data($url);
    if ( $data eq "" ) { return; }
    if ( @{ $data->{groups} } ) {
      my @arr = @{ $data->{groups} };
      for my $j ( 0 .. $#arr ) {
        my %helparr;

        #print Dumper $arr[$j]->{initiators};
        foreach my $initiator ( @{ $arr[$j]->{initiators} } ) {
          if ( !exists $igs{ $arr[$j]{name} }{$initiator} ) {
            $igs{ $arr[$j]{name} }{$initiator} = {};
          }
        }
      }
    }
    my $uurl  = "https://$ip/api/san/$version/$protocols[$i]/initiators";
    my $ddata = get_data($uurl);
    if ( $ddata eq "" ) { return; }
    if ( @{ $ddata->{initiators} } ) {
      my @arra = @{ $ddata->{initiators} };
      for my $i ( 0 .. $#arra ) {
        for my $ingroup ( keys %igs ) {
          for my $initiator ( keys %{ $igs{$ingroup} } ) {
            if ( $arra[$i]->{initiator} eq $initiator ) {
              $igs{$ingroup}{$initiator}{alias} = $arra[$i]->{alias};
              if ( !$igs{$ingroup}{node} ) {
                $igs{$ingroup}{node} = "$nodename ";
              }
              else {
                if ( index( $igs{$ingroup}{node}, $nodename ) == -1 ) {
                  $igs{$ingroup}{node} .= "$nodename";
                }
              }
            }
          }
        }
      }
    }
  }
}

# adds disk serial number to %disks #
sub get_diskserial {
  for my $key ( keys %chassis ) {
    my $url  = "https://$ip/api/hardware/$version/chassis/$key/disk";
    my $data = get_data($url);

    if ( $data eq "" ) { return; }

    my @dt = @{ $data->{disk} };
    for my $i ( 0 .. $#dt ) {
      if ( $dt[$i]->{present} == 1 and $dt[$i]->{serial} ) {
        $disks{ $dt[$i]->{label} }->{serial_number} = $dt[$i]->{serial};
      }
    }
  }
}

# returns hash of all disks from given array of pools #
sub get_listofdisks {
  my @arr;
  my $lop         = shift;
  my %listofpools = %$lop;
  for my $key ( keys %listofpools ) {
    my %listofdisk;
    my $url  = "https://$ip/$storpath/$version/pools/$key";
    my $data = get_data($url);

    if ( $data eq "" ) { return; }

    @arr = @{ $data->{pool}->{vdev} };
    for my $j ( 0 .. $#arr ) {
      if ( $data->{pool}->{status} eq "exported" ) {
        next;
      }
      if ( !defined( $arr[$j]->{children} ) ) {
        if ( $arr[$j]->{label} ) {
          $disks{ $arr[$j]->{label} }{disk_iops}      = 0;
          $listofdisk{ $arr[$j]->{label} }{disk_iops} = 0;
        }
      }
      else {
        my @childrenarr = @{ $arr[$j]->{children} };
        for my $k ( 0 .. $#childrenarr ) {
          if ( $childrenarr[$k]->{label} ) {
            $disks{ $childrenarr[$k]->{label} }{disk_iops}      = 0;
            $listofdisk{ $childrenarr[$k]->{label} }{disk_iops} = 0;
          }
        }
      }
    }

    $pools{$key}->{disks} = {%listofdisk};
  }
}

sub get_nodename {
  my $url  = "https://$ip/api/system/$version/version";
  my $node = get_data($url);

  $asn = $node->{version}->{asn};
  $device_conf{model} = $node->{version}{'product'};
  $device_conf{version} = $node->{version}{'version'};

  return $node->{version}->{nodename};
}

sub get_chassis {
  my $url  = "https://$ip/api/hardware/$version/chassis";
  my $data = get_data($url);
  my %chas;

  if ( $data eq "" ) { return; }

  my @arr = @{ $data->{chassis} };
  for my $i ( 0 .. $#arr ) {
    my $chasnumber = basename( $arr[$i]{href}, "/" );
    $chas{$chasnumber}{serial} = $arr[$i]{serial};
  }
  return %chas;
}

# universal sub that returns json data using session #
sub get_data {
  my $url         = shift;
  my $error_catch = "";
  $error_catch = shift;
  #warn "GET ME THE URL: $url";
  my $ua = LWP::UserAgent->new(
    ssl_opts => {
      SSL_cipher_list => 'DEFAULT:!DH',
      verify_hostname => 0,
      SSL_verify_mode => 0
    }
  );
  my $req = HTTP::Request->new( GET => $url );
  $req->header( 'Content-Type'   => 'application/json' );
  $req->header( 'Accept'         => 'application/json' );
  $req->header( 'Connection'     => 'keep-alive' );
  $req->header( 'X-Auth-Session' => $cookie );

  my $res = $ua->request($req);
  if ( !$res->is_success ) {
    if ( $error_catch and $error_catch ne "dontcatch" ) {
      $logger->error("ERROR:Cannot get data \n");
      $logger->error( $res->content );
    }
    return "";
  }
  else {
    my $data = decode_json( $res->content );
    #warn "$url AND ERR ".$res->content."\n\n";
    return $data;
  }
}

sub post_data {
  my $url     = shift;
  my $content = shift;
  my $header  = [ 'Content-Type' => 'application/json; charset=UTF-8' ];
  my $ua      = LWP::UserAgent->new(
    ssl_opts => {
      SSL_cipher_list => 'DEFAULT:!DH',
      verify_hostname => 0,
      SSL_verify_mode => 0
    }
  );
  my $req = HTTP::Request->new( POST => $url );

  $req->header( 'Content-Type'   => 'application/json' );
  $req->header( 'X-Auth-Session' => $cookie );
  $req->content( encode_json($content) );

  my $res = $ua->request($req);
  if ( !defined $res->is_success ) {
    $logger->error("ERROR:POST failed \n");
    $logger->error( $res->content );
    return "";
  }
  else {
    #print "\n POSTED \n";
    #print Dumper $res;
    return "posted";
  }
}

sub put_data {
  my $url     = shift;
  my $content = shift;
  my $header  = [ 'Content-Type' => 'application/json; charset=UTF-8' ];
  my $ua      = LWP::UserAgent->new(
    ssl_opts => {
      SSL_cipher_list => 'DEFAULT:!DH',
      verify_hostname => 0,
      SSL_verify_mode => 0
    }
  );
  my $req = HTTP::Request->new( PUT => $url );

  $req->header( 'Content-Type'   => 'application/json' );
  $req->header( 'X-Auth-Session' => $cookie );

  my $res = $ua->request($req);
  if ( !defined $res->is_success ) {
    $logger->error("ERROR:PUT failed \n");
    $logger->error( $res->content );
    return "";
  }
  else {
    return "put";
  }
}

# logs in with credentials and returns session if successful #
sub login {
  my $username = shift;
  my $password = shift;
  my $check = shift;
  my $url   = "https://$ip/api/access/$version";
  my $ua    = LWP::UserAgent->new(
    ssl_opts => {
      SSL_cipher_list => 'DEFAULT:!DH',
      verify_hostname => 0,
      SSL_verify_mode => 0
    }
  );
  my $req = HTTP::Request->new( POST => $url );
  $req->header( 'Content-Type' => 'application/json' );
  $req->header( 'Accept'       => 'application/json' );
  $req->header( 'Connection'   => 'keep-alive' );
  $req->authorization_basic( $username, $password );
  my $res = $ua->request($req);

  if ( !$res->is_success ) {

    if ($check) {
      return $res->message;
    }
    else {
      $logger->error("ERROR: Login to API failed\n");
      $logger->error( $res->content );
      return "";
    }
  }
  else {
    my $cookie = $res->headers->{"x-auth-session"};
    return $cookie;
  }
}

# logs out and closes session access #
sub logout {
  my $url = "https://$ip/api/access/$version";
  my $ua  = LWP::UserAgent->new(
    ssl_opts => {
      SSL_cipher_list => 'DEFAULT:!DH',
      verify_hostname => 0,
      SSL_verify_mode => 0
    }
  );
  my $req = HTTP::Request->new( DELETE => $url );
  $req->header( 'Content-Type'   => 'application/json' );
  $req->header( 'Accept'         => 'application/json' );
  $req->header( 'Connection'     => 'keep-alive' );
  $req->header( 'X-Auth-Session' => $cookie );

  my $res = $ua->request($req);
  if ( !$res->is_success ) {
    $logger->error("ERROR: Logout to API failed\n");
    $logger->error( $res->content );
    return "";
  }
  else {
    #print "\nlogged out\n";
  }
}

sub jsonbool_totext {
  my $bool = shift;
  if ( $bool == 1 ) {
    return "true";
  }
  elsif ( $bool == 0 ) {
    return "false";
  }
  else {
    return "";
    $logger->error( "Variable is not JSON boolean value" . __FILE__ . ":" . __LINE__ );
  }
}

# generates time for data extraction #
sub data_time {
  my $time = shift;
  my ( $sec, $min, $hour, $mday, $mon, $year ) = ( gmtime( $time - 3600 ) );
  my $dt = gmtime2utc( $sec, $min, $hour, $mday, $mon, $year );
  if ( $version eq "v1" ) {
    $dt =~ s/-//g;
    $dt =~ s/Z//g;
  }
  return $dt;
}

# generates pseudo start of data retrieval interval #
sub start_time {
  my $time = shift;
  my ( $sec, $min, $hour, $mday, $mon, $year ) = ( gmtime( $time - 300 ) );
  return gmtime2utc( $sec, $min, $hour, $mday, $mon, $year );
}

# generates pseudo end of data retrieval interval #
sub end_time {
  return time();
}

sub timestamp {
  my $time = shift;
  my ( $sec, $min, $hour, $mday, $mon, $year ) = ( gmtime($time) );
  return gmtime2utc( $sec, $min, $hour, $mday, $mon, $year, 1 );
}

# transforms given gmtime to UTC #
sub gmtime2utc {
  my ( $sec, $min, $hour, $mday, $mon, $year, $secbool ) = @_;
  $year += 1900;
  $mon  += 1;
  $mon  = sprintf( "%02d", $mon );
  $mday = sprintf( "%02d", $mday );
  $hour = sprintf( "%02d", $hour );
  $min  = sprintf( "%02d", $min );
  if ( defined $secbool and $secbool == 1 ) {
    return "$year-$mon-$mday" . "T" . "$hour:$min:$sec" . "Z";
  }
  else {
    return "$year-$mon-$mday" . "T" . "$hour:$min:00Z";
  }
}

sub filetime {
  my ( $sec, $min, $hour, $mday, $mon, $year ) = ( localtime($time) );
  my $utctime = gmtime2utc( $sec, $min, $hour, $mday, $mon, $year );
  $utctime = substr( $utctime, 0, -4 );
  $utctime =~ s/-//g;
  $utctime =~ s/://g;
  $utctime =~ s/T/_/g;
  return $utctime;
}

# returns file name  from file path #
sub basename {
  my $full      = shift;
  my $separator = shift;
  my $out       = "";

  #my $length = length($full);
  if ( defined $separator and defined $full and index( $full, $separator ) != -1 ) {
    $out = substr( $full, length($full) - index( reverse($full), $separator ), length($full) );
    return $out;
  }
  return $full;
}

sub to_gib {
  my $value = shift;
  my $GiB   = 1073741824;
  return sprintf( "%.1f", ( $value / $GiB ) );
}

sub to_tib {
  my $value = shift;
  my $GiB   = 1073741824;
  return sprintf( "%.3f", ( ( $value / $GiB ) / 1024 ) );
}

sub apitest {
  my $client = shift;
  my $data   = shift;

  my $mvc_dir      = $FindBin::Bin;
  my $username     = $data->{apiUsername};
  my $password     = $data->{apiPassword};
  my $ipone        = $data->{node1Host};
  my $iptwo        = $data->{node2Host};
  my $alias        = $data->{hostalias};
  my $hostcfg_id   = $data->{hostcfg_id};
  $starttime       = start_time($time);
  $endtime         = end_time($time);
  $time            = time();
  $ft              = filetime();
  my %weird_perf;
  my @versions  = ( "v2.0", "v1" );

  #>debug
  my $debug = 0;

  #>

  my $setup;

  my @ips = ( $ipone, $iptwo );


my $http      = "https";

my $analytics = "";
my $start     = "data?start";
my $span      = "span=hour&granularity=minute";
my $datatime  = "";
my $skipped   = 0;
my $api_check = 0;
my $test = Test->new($client);
my @rows;

foreach my $currentip (@ips) {
  $ip = "$currentip".":215";
  if ( $ip eq ":215" ) {
    next;
  }

  my ($code, $text) = Xormon::conntest_tcp($currentip, "215");
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  foreach my $c_version (@versions) {
    $version = $c_version;
    my $message = lc( login($username,$password,1) );
    if ( $message ne "not found" ) {
      $api_check = 1;
      last;
    }
  }
  $datatime  = data_time($time);
  $analytics = "api/analytics/$version/datasets";

  if ( $api_check == 0 ) {
    push(@rows, "Check your appliance for REST API support, please.");
    $test->addSection("API versions test", 0, \@rows);
    #return $test->stop();
    next;
  }
  $cookie = login($username,$password);
  if ( !defined $cookie || $cookie eq "" ) {
    if ( $skipped == 0 ) {
      $skipped++;
      push(@rows, "Cookie is not defined IP: $ip ");
      next;
    }
    else {
      push(@rows, "Cookie is not defined IP: $ip ");
      $test->addSection("Device connection ", 0 , \@rows);
      #return $test->stop();
    }
  }
  my $curl     = "$http://$ip/api/system/$version/version";
  my $confdata = get_data($curl);
  if ( !defined $confdata || $confdata eq "" ) {
    push(@rows, "Couldn't get configuration data.");
    $test->addSection("Configuration data test", 0 , \@rows);
    logout();
    return $test->stop();
  }

  else {
    my %foo     = %{ $confdata->{version} };
    my $release = "";
    $release = $foo{ak_release};
    push(@rows, "Name      : $foo{nodename}\n");
    push(@rows, "Product   : $foo{product}\n");
    push(@rows, "Model     : $foo{mkt_product}\n");
    push(@rows, "Release   : $release\n");
    push(@rows, "Version   : $foo{version}\n");
    push(@rows, "Boot time : $foo{boot_time}\n");
    $test->addSection("Configuration data test", 1 , \@rows);
  }
    undef @rows;
    my $purl     = "$http://$ip/$analytics/cpu.utilization/$start=$datatime&$span";
    my $perfdata = get_data($purl);

    if ( !defined $perfdata || $perfdata eq "" ) {
      push(@rows, "Couldn't get performance data.");
      $test->addSection("Performance data test", 0 , \@rows);
      logout();
      return $test->stop();
    }
    else {
      push(@rows, "Performance: OK");
      $test->addSection("Performance data test", 1, \@rows);
    }
  logout();
  $ip        = "";
  $cookie    = "";
  $api_check = "";
  $version   = "";
  $analytics = "";
  $datatime  = "";
}
  return $test->stop();
}

$server->close();
