package PostgreSQL;

use strict;
use warnings;

use JSON;
use Data::Dumper;
use Digest::MD5 qw(md5 md5_hex md5_base64);



sub get_counters {
  my %counters = (
      "checkpoints_timed" => 1,
      "buffers_backend" => 1,
      "buffers_checkpoint" => 1,
      "buffers_clean" => 1,
      "checkpoints_req" => 1,
      "tup_fetched" => 1,
      "xact_commit" => 1,
      "xact_rollback" => 1,
      "blks_hit" => 1,
      "deadlocks" => 1,
      "blks_read" => 1,
      "temp_files" => 1,
      "temp_bytes" => 1,
      "tup_deleted" => 1,
      "tup_updated" => 1,
      "tup_returned" => 1,
      "tup_inserted" => 1
  );

  return \%counters;
}

sub isdigit {
  my $digit = shift;

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  #$digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {
    # is a number
    return 1;
  }

  return 0;
} ## end sub isdigit

sub basename {
  my $full      = shift;
  my $separator = shift;
  my $out = "";
  #my $length = length($full);
  if(defined $separator and defined $full and  index($full, $separator) != -1){
    $out = substr($full, length($full) - index(reverse($full), $separator), length($full));
    return $out;
  }
  return $full;
}

sub md5_string {
  my $data = shift;
  my $out = md5_hex($data);

  return $out;
}

sub conf_files{
  my $wrkdir = shift;
  my $type = shift;
  my $alias = shift;
  my $host_url = shift;

  #$type = remove_subs($type);

  my @conf_files;

  if($type =~ /configuration_Cluster/){
    @conf_files = (get_dir("_main", $alias));
  }
  elsif($type =~ m/configuration/){
    @conf_files = (" ",get_dir("_relations", $alias, $host_url));
  }
  return \@conf_files;
}

sub get_items {
  my %params = %{shift()};
  my @result;

  my $arc = get_arc();
  if ( defined $params{item_type} or $arc == 0 or !defined $arc->{$params{item_type}} ) {
    my @empty = [];
    return \@empty;
  }

  my %arc = %{$arc};
  if($params{item_type} eq "hostnames"){
    @result = keys %{$arc{$params{item_type}}};
  }elsif($params{item_type} eq "dbs"){
    for my $hostname (keys %{$arc{hostnames}}){
      push(@result, keys %{$arc{hostnames}{$hostname}{"_$params{item_type}"}});
    }
  }
  
  return \@result;
}

1;

