package CMC_COLLECT;

use strict;
use warnings;

use HTTP::Request;
use JSON;
use Data::Dumper;

use CMC_COMMON;

my ($portal_url, $CMC_client_id, $CMC_client_secret, $proxy);
my $console_name;

my %usage_pools;

my %credit_info = ();


# API limit:
# All API calls are automatically rate limited to a maximum of 10 calls per second.
my $request_cycle_counter = 1;


my $DEBUG     = 1;
my $USE_V2    = 0;

my (%data, %data_timed, %os_usage_per_system);
my ($data_timed_ref, $os_usage_per_system_ref, $data_ref);

$data_ref = \%data;
$data_timed_ref = \%data_timed;
$os_usage_per_system_ref = \%os_usage_per_system;

#-----------------------------------------------------------------------------------
# TIME HANDLING
# OUT: $StartTS, $EndTS
#-----------------------------------------------------------------------------------
my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst );

my $use_time = time();

my $Frequency = "Minute";
my $collection_delay  = 1800;

my $interval          = 2600;

my $start_time = $use_time - $interval - $collection_delay;
( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = gmtime($start_time);

my $StartTS = sprintf( "%4d-%02d-%02dT%02d:%02d:00Z", $year + 1900, $month + 1, $day, $hour, $min );

my $end_time = $use_time - $collection_delay;
( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = gmtime($end_time);

my $EndTS = sprintf( "%4d-%02d-%02dT%02d:%02d:00Z", $year + 1900, $month + 1, $day, $hour, $min );


my %os_stats = (
  "Base" => {

    #"BaseMemory" =>  "base_memory",

    "BaseCores" => {
      "BaseAnyOSCores"      => "base_core_any_os",
      "BaseLinuxVIOSCores"  => "base_core_linuxvios",

      "BaseRHELCoreOSCores" => "base_core_rhcos",
      "BaseRHELCores"       => "base_core_rhel",
      "BaseSLESCores"       => "base_core_sles",
      "BaseAIXCores"        => "base_core_aix",
      "BaseIBMiCores"       => "base_core_ibmi",
    },
  },
  "Usage" => {
    "AverageCoreUsage" => {
      "AIX"         => "core_aix",
      "IBMi"        => "core_ibmi",

      "RHELCoreOS"  => "core_rhelcoreos",
      "RHEL"        => "core_rhel",
      "SLES"        => "core_sles",
      "OtherLinux"  => "core_other_linux",
      "VIOS"        => "core_vios",

      "Total"       => "core_total",
    },
    "AverageMemoryUsage" => {
      "AIX"             => "mem_aix",
      "IBMi"            => "mem_ibmi",

      "RHELCoreOS"      => "mem_rhelcoreos",
      "RHEL"            => "mem_rhel",
      "SLES"            => "mem_sles",
      "OtherLinux"      => "mem_other_linux",
      "VIOS"            => "mem_vios",

      "SystemOther"     => "mem_system_other",

      "Total"           => "mem_total",
    },
  }

);

my $GIBI_BYTE = 1024*1024*1024;


my %conf_multiplier = (
  'Pools' => {
    'mem_available' => $GIBI_BYTE,
    'mem_installed' => $GIBI_BYTE,
    'base_memory'   => $GIBI_BYTE,
  },
  'Systems' => {
    'mem_available' => $GIBI_BYTE,
    'mem_installed' => $GIBI_BYTE,
    'base_memory'   => $GIBI_BYTE,
  },
);

my $os_stats_ref = \%os_stats;
my $url_management_console;
my %management_console_data;

#---------------------------------------------------------------------------------------------------------
# scheme of configuration hash %data
#---------------------------------------------------------------------------------------------------------
#              => Pools       => *PoolID        => Name           => value
#                                               => Configuration  => *Name => value
#                                               => Systems        => *UUID => Name         => value
#                                               => Partitions     => *UUID => Name         => value
#
#              => Tags        => *TagID         => Name           => value
#                                               => Configuration  => *Name => value
#                                               => Systems        => *UUID => Name => value
#                                               => Partitions     => *UUID => Name => value
#
#              => Servers     => *ServerUUID    => Name           => value
#                                               => Configuration  => *Name => value
#                                               => Pools          => *PoolID       => Name => value 
#
#              => Partitions  => *PartitionUUID => Name           => value
#                                               => Configuration  => *Name => value
#                                               => Pools          => *PoolID       => Name => value  
#
#              => HMCs        => *HMCUUID       => Name           => value
#                                               => Configuration  => *Name => value
#                                               => Systems        => *UUID => Name => value
#                                               => Partitions     => *UUID => Name => value
#---------------------------------------------------------------------------------------------------------

#---------------------------------------------------------------------------------------------------------
# scheme of performance hash %data_timed
#---------------------------------------------------------------------------------------------------------
#              => Pools       => *PoolID        => Metrics        => *time   => *MetricName  => value
#
#              => Servers     => *ServerUUID    => Metrics        => *time   => *MetricName  => value     
#---------------------------------------------------------------------------------------------------------
my %managed_hmc_data;
my $url_managed_system;
my $url_usage_pools;

sub set_time {
  print "CMC: SETTING TIME \n";
  $use_time = time();

  $start_time = $use_time - $interval - $collection_delay;
  ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = gmtime($start_time);

  $StartTS = sprintf( "%4d-%02d-%02dT%02d:%02d:00Z", $year + 1900, $month + 1, $day, $hour, $min );

  $end_time = $use_time - $collection_delay;
  ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = gmtime($end_time);

  $EndTS = sprintf( "%4d-%02d-%02dT%02d:%02d:00Z", $year + 1900, $month + 1, $day, $hour, $min );
  print "CMC: TIME: $use_time  \n";
  print "CMC: DELAY [sec]: $collection_delay  \n";
  print "CMC: StartTS:  $StartTS \n";
  print "CMC: EndTS:    $EndTS \n";


}

sub collect {
  $portal_url         = shift;
  $CMC_client_id      = shift;
  $CMC_client_secret  = shift;
  $proxy              = shift;

  $console_name = $portal_url;

  set_time();

  #-----------------------------------------------------------------------------------
  # :1: INVENTORY TAGS
  #-----------------------------------------------------------------------------------
  # GET /ep/inventory/tags 
  # https://<portal-url>/api/public/v1/ep/inventory/tags/{tag_name}
  #-----------------------------------------------------------------------------------
  print_debug_message("PROCESSING INVENTORY TAGS");

  my $url_inventory = "https://${portal_url}/api/public/v1/ep/inventory/tags";

  my %inventory_tags = general_hash_request("GET", $url_inventory);

  ($data_ref) = process_inventory_tags($data_ref, $os_stats_ref, \%inventory_tags, $url_inventory);

  %inventory_tags = ();

  %data = %{$data_ref};

  #-----------------------------------------------------------------------------------
  # :2: USAGE POOLS
  #-----------------------------------------------------------------------------------
  # GET ep/usage/pools 
  # https://<portal-url>/api/public/v1/ep/usage/pools/{pool_name}
  #   GET POOL NAMES: The available pool names are returned if you do not specify a pool name. 
  #-----------------------------------------------------------------------------------
  #  QUERY PARAMETERS:
  # StartTS: yyyy-MM-ddTHH:mm:ssZ 
  # EndTS: yyyy-MM-ddTHH:mm:ssZ
  # Frequency:  Minute  Hourly  Daily  Weekly  Monthly  	
  #-----------------------------------------------------------------------------------
  print_debug_message("PROCESSING USAGE POOLS");

  $url_usage_pools = "https://${portal_url}/api/public/v1/ep/usage/pools";

  %usage_pools = general_hash_request("GET", timify_query($url_usage_pools));

  process_usage_pools();

  check_tagging();

  #-----------------------------------------------------------------------------------
  # :3: USAGE TAGS
  #-----------------------------------------------------------------------------------
  # GET ep/usage/tags 
  # https://<portal-url>/api/public/v1/ep/usage/tags/{tag_name}
  #-----------------------------------------------------------------------------------
  #  QUERY PARAMETERS: 
  # StartTS: yyyy-MM-ddTHH:mm:ssZ 
  # EndTS: yyyy-MM-ddTHH:mm:ssZ
  # Frequency:  Minute  Hourly  Daily  Weekly  Monthly  	
  #-----------------------------------------------------------------------------------
  print_debug_message("PROCESSING USAGE TAGS");

  my $url_usage_tags = "https://${portal_url}/api/public/v1/ep/usage/tags";

  if ( $USE_V2 ) {
    $url_usage_tags = "https://${portal_url}/api/public/v2/ep/usage/tags";
    # TODO: IBMi = sum(IBMiP30, IBMiP20, IBMiP10)
  }

  my %usage_tags = general_hash_request("GET", timify_query($url_usage_tags));

  ($data_timed_ref, $os_usage_per_system_ref, $data_ref) = process_usage_tags(\%os_usage_per_system, \%data_timed, \%data, \%usage_tags, $url_usage_tags);

  %usage_tags = ();

  # ! CHANGE THAT:
  %data                 = %{$data_ref};
  %data_timed           = %{$data_timed_ref};
  %os_usage_per_system  = %{$os_usage_per_system_ref};



  #print Dumper %os_usage_per_system;
  #-----------------------------------------------------------------------------------
  # :4: MANAGED SYSTEM
  #-----------------------------------------------------------------------------------
  print_debug_message("PROCESSING MANAGED SYSTEM");

  $url_managed_system = "https://${portal_url}/api/public/v1/inventory/cmc/ManagedSystem";

  %managed_hmc_data = general_hash_request("GET", $url_managed_system);

  process_managed_system_data();

  #-----------------------------------------------------------------------------------
  # :5: MANAGEMENT CONSOLE
  #-----------------------------------------------------------------------------------
  print_debug_message("PROCESSING MANAGEMENT CONSOLE");

  $url_management_console = "https://${portal_url}/api/public/v1/inventory/cmc/ManagementConsole";

  %managed_hmc_data = general_hash_request("GET", $url_management_console);

  process_management_console_data();

  #-----------------------------------------------------------------------------------
  # :6: CREDIT USAGE
  #-----------------------------------------------------------------------------------
  print_debug_message("PROCESSING CREDIT USAGE");

  collect_full_range_credits();
  round_and_multiply_data();

  return (\%data, \%data_timed, \%credit_info);
}




#-----------------------------------------------------------------------------------
# API rate limit
#-----------------------------------------------------------------------------------
# As script collects v2 samples, it is important to take into account CMC API request limit.
# NOTE: one user pointed out, that perf request took more than a minute.

sub check_limit_to_sleep {
  $request_cycle_counter++;

  print_debug_message("LIMIT CHECK: $request_cycle_counter ");

  my $possible_waiting_interval = 1;

  if ( $request_cycle_counter >= 10 ) {
    print_debug_message( "WAITING: $possible_waiting_interval");
    sleep($possible_waiting_interval);

    $request_cycle_counter = 1;
  }

}

#-----------------------------------------------------------------------------------
# REQUEST
# All API calls are automatically rate limited to a maximum of 10 calls per second.
#-----------------------------------------------------------------------------------
sub general_hash_request {
  my $method  = shift;
  my $url     = shift;
  
  my $loc_console_name      = shift || $console_name;
  my $loc_CMC_client_id     = shift || $CMC_client_id;
  my $loc_CMC_client_secret = shift || $CMC_client_secret;
  my $loc_proxy             = shift || $proxy;

  my %decoded_json;

  # Api rate limit
  check_limit_to_sleep();

  print_debug_message("$method $url");

  # HTTP request (user) OR read saved JSON (development)
  if ( CMC_COMMON::use_api_data($loc_console_name) ) {

    my $user_agent    = LWP::UserAgent->new( 
      ssl_opts => { 
        SSL_cipher_list => 'DEFAULT:!DH',
        verify_hostname => 0, 
        SSL_verify_mode => 0
      } 
    );

    if ( $loc_proxy ){ # GLOBAL variable $loc_proxy
      # expected proxy format: http://host:port
      #print "\n proxy:-${loc_proxy}- \n";
      $user_agent->proxy( ['http', 'https', 'ftp'] => $loc_proxy );
    }

    my $request = HTTP::Request->new( $method => $url );

    $request->header( 'X-CMC-Client-Id'     => "$loc_CMC_client_id" );
    $request->header( 'X-CMC-Client-Secret' => "$loc_CMC_client_secret" );
    $request->header( 'Accept'              => 'application/json' );

    my $result = $user_agent->request($request);
 
    eval{
      %decoded_json = %{decode_json($result->{'_content'})};
    };
    if($@){
      my $error_message = "";
      $error_message .= "PROBLEM OCCURED during decode_json HASH with url $url!" ;
      $error_message .= "\n --- RESULT->_content --- \n $result->{'_content'} \n";
      print "$error_message";
      print Dumper $result->{'_headers'};

      error( "$error_message ");
      return ()
    }

  }
  else {
    # DEVELOPMENT: LOAD JSON DATA
    my $development_data_dir = CMC_COMMON::get_development_data_dir();

    my $file_name = match_url_to_file($url); # returns basic filename (e.g. usage_tags.json)

    my $path_to_read = "${development_data_dir}/${loc_console_name}/CMC-${loc_console_name}-${file_name}";

    print "FAKE DATA LOAD: \n\n";
    print("$path_to_read\n");

    if ( ! -f $path_to_read ) {
      # in case of older dataset
      print_debug_message("WRONG PATH: $path_to_read");
      $path_to_read = "${development_data_dir}/${loc_console_name}/${file_name}";
    }

    eval{
      %decoded_json = %{decode_json(file_to_string($path_to_read))};
    };

    if($@){
      error( "UNKNOWN file::>$file_name<::elif");
      print "$@";

      %decoded_json = ();
      #return ()
    }

  }

  #if ( $SAVE_RESPONSES ) {
  #  # DEVELOPMENT: SAVE RESPONSE JSON TO TMP
  #  my $file_name = match_url_to_file($url); # returns basic filename (e.g. usage_tags.json)
  #
  #  my $response_path = "$lpar2rrd_dir/tmp/CMC-${console_name}-${file_name}";
  #
  #  print_debug_message("Saving response.");
  #
  #  my $json      = JSON->new->utf8;
  #  my $json_data = $json->encode(\%decoded_json);
  #  write_to_file($response_path, $json_data);
  #}

  return %decoded_json;
}



sub match_url_to_file {
  my $url_called = shift;

  my %translation_marks = (
    'api/public/v1/ep/inventory/tags'               => 'inventory_tags',
    'api/public/v1/ep/usage/pools'                  => 'usage_pools',
    'api/public/v1/ep/usage/tags'                   => 'usage_tags',
    'api/public/v1/inventory/cmc/ManagedSystem'     => 'managed_system',
    'api/public/v1/inventory/cmc/ManagementConsole' => 'management_console',
    'api/public/v1/cm/usage/ManagedSystem'          => 'cm_managed_system',

    'api/public/v2/ep/inventory/tags'               => 'v2_inventory_tags',
    'api/public/v2/ep/usage/pools'                  => 'v2_usage_pools',
    'api/public/v2/ep/usage/tags'                   => 'v2_usage_tags',
    'api/public/v2/inventory/cmc/ManagedSystem'     => 'v2_managed_system',
    'api/public/v2/inventory/cmc/ManagementConsole' => 'v2_management_console',
    'api/public/v2/cm/usage/ManagedSystem'          => 'v2_cm_managed_system',
  );
  
  my $file_name;
  
  for my $mark ( keys %translation_marks ) {
    if ( $url_called =~ /$mark/ ){
      $file_name = $translation_marks{$mark};
      last;
    }
  }

  # Minute is used as default = .json
  my %frequency_marks = (
    '=Daily'   => '_daily',
    '=Weekly'  => '_weekly',
    '=Monthly' => '_monthly',
  );

  for my $mark ( keys %frequency_marks ) {
    if ( $url_called =~ /$mark/ ){
      $file_name .= $frequency_marks{$mark};
      last;
    }
  }

  $file_name .= '.json';
  
  return $file_name;
}

#-----------------------------------------------------------------------------------
# DEBUG AND ERROR
#-----------------------------------------------------------------------------------
sub print_debug_message {
  # FORMATE:
  # time - triggering function - message
  if ( $DEBUG ) {
    my $message = shift;
    my $debug_time = localtime();
    my ($sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst) = localtime();

    $debug_time = sprintf( "%4d-%02d-%02d %02d:%02d:%02d", $year + 1900, $month + 1, $day, $hour, $min, $sec );
  
    my $function_message = (caller(1))[3] || "MAIN";

    if ( $function_message eq "MAIN" ) {
      $debug_time = "\n$debug_time";
    }

    $function_message =~ s/main:://;

    print ("${debug_time}: $function_message - $message \n");
  }
}

sub error {
  my $e_message = shift;

  print("$e_message \n");
}


sub time_budget_query {
  my $ux_end_time = shift;
  my $interval    = shift;

  print_debug_message("Interval: $interval");

  # Minute Hourly Daily Weekly Monthly

  # Minutes, for a maximum of 60 minutes
  # Hours, for a maximum of a week of time
  # Days, for a maximum of six months duration
  # Weeks, for a maximum of two years
  # Months, for a maximum of ten years

  my %interval_backtime = (
    "Daily"   => 35 *24*3600,
    "Weekly"  => 20 *7*24*3600,
    "Monthly" => 20 *31*24*3600,
  );
  my ($sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst) = gmtime($ux_end_time);

  my $EndTS = sprintf( "%4d-%02d-%02dT%02d:%02d:00Z", $year + 1900, $month + 1, $day, $hour, 0 );
  my $time_start_ux = $ux_end_time - int($interval_backtime{$interval});
  ($sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst) = gmtime($time_start_ux);
  
  my $StartTS;# = sprintf( "%4d-%02d-%02dT%02d:%02d:00Z", $year + 1900, $month + 1, $day, $hour, $min );

  if ( $interval eq "Daily" ) {
    $StartTS = sprintf( "%4d-%02d-%02dT%02d:%02d:00Z", $year + 1900, $month + 1, $day, 0, 0 );
  }
  elsif ( $interval eq "Weekly" ) {
    $StartTS = sprintf( "%4d-%02d-%02dT%02d:%02d:00Z", $year + 1900, $month + 1, $day, 0, 0 );
  }
  elsif ( $interval eq "Monthly" ) {
    $StartTS = sprintf( "%4d-%02d-%02dT%02d:%02d:00Z", $year + 1900, $month + 1, 1, 0, 0 );
  }

  my $query_r = "EndTS=${EndTS}&Frequency=${interval}&StartTS=${StartTS}";
  print_debug_message("QUERY: $query_r");

  return $query_r;
}


sub process_inventory_tags {
  my %data            = %{$_[0]};
  my %os_stats        = %{$_[1]};
  my %inventory_tags  = %{$_[2]};
  my $url_inventory   = $_[3];

  my $loc_query;
  print_debug_message("");
  #print Dumper %inventory_tags;


  if (defined $inventory_tags{Tags}){

    for (my $i=0; $i<scalar(@{$inventory_tags{Tags}}); $i++){
      # Partitions => [], Systems => []
      my $tag_id = $inventory_tags{Tags}[$i]{ID};
      my $tag_name = $inventory_tags{Tags}[$i]{Name};

      $data{Tags}{$tag_id}{Name} = $tag_name;

      ## TAGGED PARTITIONS
      #for (my $j=0; $j<scalar(@{$inventory_tags{Tags}[$i]{Partitions}}); $j++){
      #  my %subhash = %{$inventory_tags{Tags}[$i]{Partitions}[$j]};
      #
      #  $UUID                                              = $subhash{UUID};
      #
      #  my $pool_id = $subhash{PoolID};
      #  $data{Pools}{$pool_id}{Name}      = $subhash{PoolName};
      #
      #  $data{Pools}{$pool_id}{Partitions}{$UUID}{Name} = $subhash{Name};
      #  $data{Tags}{$tag_id}{Partitions}{$UUID}{Name}   = $subhash{Name};
      #  $data{Partitions}{$UUID}{Name}                  = $subhash{Name};
      #
      #  $data{Partitions}{$UUID}{Metrics}{proc_available}  = $subhash{ProcessorConfiguration}{AvailableProcessorUnits};
      #  $data{Partitions}{$UUID}{Metrics}{proc_installed}  = $subhash{ProcessorConfiguration}{InstalledProcessorUnits};
      #  $data{Partitions}{$UUID}{Metrics}{mem_available}   = $subhash{MemoryConfiguration}{AvailableMemory};
      #  $data{Partitions}{$UUID}{Metrics}{mem_installed}   = $subhash{MemoryConfiguration}{InstalledMemory};
      # 
      #  $data{Partitions}{$UUID}{Pools}{$pool_id}{Name}    = $subhash{PoolName};
      #  
      #}

      # TAGGED SYSTEMS
      if (defined $inventory_tags{Tags}[$i]{Systems} && scalar @{$inventory_tags{Tags}[$i]{Systems}}){
        for (my $j=0; $j<scalar(@{$inventory_tags{Tags}[$i]{Systems}}); $j++){
          my %subhash = %{$inventory_tags{Tags}[$i]{Systems}[$j]};
          #print Dumper %subhash;
          my $UUID                                         = $subhash{UUID};
          # ONLY SYSTEMS IN POOL
          # possibly could be ""
          if (defined $subhash{PoolID} && $subhash{PoolID} && $subhash{PoolName}){
            my $pool_id = $subhash{PoolID};

            $data{Pools}{$pool_id}{Name}                    = $subhash{PoolName};
            $data{Systems}{$UUID}{Name}                     = $subhash{Name};
            $data{Systems}{$UUID}{Configuration}{pool_name} = $subhash{PoolName};

            $data{Pools}{$pool_id}{Systems}{$UUID}{Name} = $subhash{Name};
            $data{Systems}{$UUID}{Pools}{$pool_id}{Name} = $subhash{PoolName};

            $data{Tags}{$tag_id}{Systems}{$UUID}{Name} = $subhash{Name};
            $data{Systems}{$UUID}{Tags}{$tag_id}{Name} = $tag_name;

            $data{Systems}{$UUID}{Configuration}{proc_available}           = $subhash{ProcessorConfiguration}{AvailableProcessorUnits};
            $data{Systems}{$UUID}{Configuration}{proc_installed}           = $subhash{ProcessorConfiguration}{InstalledProcessorUnits};

            $data{Systems}{$UUID}{Configuration}{mem_available}            = $subhash{MemoryConfiguration}{AvailableMemory};
            $data{Systems}{$UUID}{Configuration}{mem_installed}            = $subhash{MemoryConfiguration}{InstalledMemory};

            # Probably API bug:
            # Problem in tagged systems: some of them are under 2 tags, 
            # while only one has non 0 value, while the other (same) one has 0 base memory
            if ( ! defined $data{Systems}{$UUID}{Configuration}{base_memory} ||  ! $data{Systems}{$UUID}{Configuration}{base_memory} ) {
                $data{Systems}{$UUID}{Configuration}{base_memory} = $subhash{BaseMemory};
            }

            $data{Systems}{$UUID}{Configuration}{base_anyoscores}          = $subhash{BaseCores}{BaseAnyOSCores} + $subhash{BaseCores}{BaseLinuxVIOSCores};

            $data{Systems}{$UUID}{Configuration}{State}  = $subhash{State};
            $data{Systems}{$UUID}{Configuration}{NumberOfLPARs}  = $subhash{NumberOfLPARs};
            for my $os (keys %{$os_stats{Base}{BaseCores}}){
              my $os_metric_name = $os_stats{Base}{BaseCores}{$os};
              $data{Systems}{$UUID}{Configuration}{$os_metric_name} = $subhash{BaseCores}{$os} || 0;
            }

          }
        }
      }
      else{
        $loc_query = $url_inventory;
        # Unimportant message + problem with default Control Plane Node tag
        #error("No tagged systems in tag $tag_name from $loc_query ");
      }

    }
  }
  else{
    $loc_query = $url_inventory;
    error("No tag data from $loc_query ");
  }

  # SUM of os bases to pool base
  for my $pool_id (keys %{$data{Pools}}) {
    # Prepare base configuration of pools
    for my $os_metric_name (keys %{$os_stats{Base}{BaseCores}}){
      $data{Pools}{$pool_id}{Configuration}{$os_metric_name} = 0;
    }


    for my $UUID (keys %{$data{Pools}{$pool_id}{Systems}}) {

      for my $os_metric_name (keys %{$os_stats{Base}{BaseCores}}){
        my $os_metric_name = $os_stats{Base}{BaseCores}{$os_metric_name};
        $data{Pools}{$pool_id}{Configuration}{$os_metric_name} += $data{Systems}{$UUID}{Configuration}{$os_metric_name};
      }

    }

  }
  return (\%data);
} # process_inventory_tags

my $loc_query;

sub process_usage_tags {
  print_debug_message("");

  my %os_usage_per_system = %{$_[0]};
  my %data_timed          = %{$_[1]};
  my %data                = %{$_[2]};
  my %usage_tags          = %{$_[3]};
  my $url_usage_tags      = $_[4];
  my $UUID;
  # Cycle fills timestamped data from servers response
  if ( defined $usage_tags{Tags} ){
    if ( !scalar(@{$usage_tags{Tags}}) ){

      $loc_query = timify_query($url_usage_tags);
      error( "No tag data from $loc_query ");

    }

    for (my $i=0; $i<scalar(@{$usage_tags{Tags}}); $i++){
      # Partitions => [], Systems => []
      my $tag_id = $usage_tags{Tags}[$i]{ID};
      my $tag_name = $usage_tags{Tags}[$i]{Name};

      # SYSTEMS
      if (defined $usage_tags{Tags}[$i]{SystemsUsage}){

        for (my $j=0; $j<scalar(@{$usage_tags{Tags}[$i]{SystemsUsage}{Systems}}); $j++){

          my %subhash = %{$usage_tags{Tags}[$i]{SystemsUsage}{Systems}[$j]};
          $UUID = $subhash{UUID};

          if (defined $data{Systems}{$UUID} && defined $subhash{Usage}{Usage}){
            for (my $k=0; $k<scalar(@{$subhash{Usage}{Usage}}); $k++){ 
              my %system_usage = %{$subhash{Usage}{Usage}[$k]};

              my $time_start = $system_usage{StartTime};
              my $timestamp = time2unix($time_start);

              for my $metric_group (keys %{$os_stats{Usage}}){
                for my $os (keys %{$os_stats{Usage}{$metric_group}}){
                  my $os_metric_name = $os_stats{Usage}{$metric_group}{$os};

                  $os_usage_per_system{Systems_OS}{$UUID}{Metrics}{$timestamp}{$os_metric_name} = $system_usage{$metric_group}{$os};
                  $data_timed{Systems}{$UUID}{Metrics}{$timestamp}{$os_metric_name} = $system_usage{$metric_group}{$os};
                  
                  if ( ( $USE_V2 ) && ( $os eq "IBMi" ) ) {
                    my $IBMiP10 = $system_usage{$metric_group}{"IBMiP10"} || 0;
                    my $IBMiP20 = $system_usage{$metric_group}{"IBMiP20"} || 0;
                    my $IBMiP30 = $system_usage{$metric_group}{"IBMiP30"} || 0;
                    $os_usage_per_system{Systems_OS}{$UUID}{Metrics}{$timestamp}{$os_metric_name} = $IBMiP10 + $IBMiP20 + $IBMiP30;
                    $data_timed{Systems}{$UUID}{Metrics}{$timestamp}{$os_metric_name} = $IBMiP10 + $IBMiP20 + $IBMiP30;
                  }
                }
              }

              for my $os (keys %{$os_stats{Base}{BaseCores}}){
                my $os_metric_name = $os_stats{Base}{BaseCores}{$os};
                $os_usage_per_system{Systems_OS}{$UUID}{Metrics}{$timestamp}{$os_metric_name} = $data{Systems}{$UUID}{Configuration}{$os_metric_name};
                $data_timed{Systems}{$UUID}{Metrics}{$timestamp}{$os_metric_name} = $data{Systems}{$UUID}{Configuration}{$os_metric_name};
              }

              $data_timed{Systems}{$UUID}{Metrics}{$timestamp}{utilizedProcUnits} = $system_usage{AverageCoreUsage}{Total};
              $data_timed{Systems}{$UUID}{Metrics}{$timestamp}{proc_installed}    = $data{Systems}{$UUID}{Configuration}{proc_installed};
              $data_timed{Systems}{$UUID}{Metrics}{$timestamp}{base_anyoscores}   = $data{Systems}{$UUID}{Configuration}{base_anyoscores}; 
              $data_timed{Systems}{$UUID}{Metrics}{$timestamp}{mem_available}     = $data{Systems}{$UUID}{Configuration}{mem_available};
              $data_timed{Systems}{$UUID}{Metrics}{$timestamp}{mem_installed}     = $data{Systems}{$UUID}{Configuration}{mem_installed};
              $data_timed{Systems}{$UUID}{Metrics}{$timestamp}{base_memory}       = $data{Systems}{$UUID}{Configuration}{base_memory};
            }
          }
        }
      }
    }
  }
  else{
    $loc_query = timify_query($url_usage_tags);
    error( "No tag data from $loc_query ");
  }

  return (\%data_timed, \%os_usage_per_system, \%data);
} # process_usage_tags

sub process_usage_pools {
  # From Frequency=Hourly: collect configuration
  # From Frequency=Minute: collect performance
  if (defined $usage_pools{Pools}){

    # NO DATA CATCH
    if (!scalar(@{$usage_pools{Pools}})){
      $loc_query = timify_query($url_usage_pools);
      error("No pool data from $loc_query ");
    }

    for (my $i=0; $i<scalar(@{$usage_pools{Pools}}); $i++){
      my $ID = $usage_pools{Pools}[$i]{PoolID};
      # possibly could be ""
      if ($ID){ 
        $data{Pools}{$ID}{Name} = $usage_pools{Pools}[$i]{PoolName};
        $data{Pools}{$ID}{Configuration}{CurrentRemainingCreditBalance} = $usage_pools{Pools}[$i]{CurrentRemainingCreditBalance}; 
      }
    }

  }
  else{
    $loc_query = timify_query($url_usage_pools);
    error("No pool data from $loc_query ");
  }

  # TIMED DATA CYCLE: 
  # Load timed data from timed queries
  if (defined $usage_pools{Pools}){
    # ERROR CATCH OF USAGE/POOLS IN UPPER CYCLE
    for (my $i=0; $i<scalar(@{$usage_pools{Pools}}); $i++){
      my $ID = $usage_pools{Pools}[$i]{PoolID};
      # possibly could be ""
      if ($ID){ 

        if (defined $usage_pools{Pools}[$i]{Usage}{Usage}){

          for (my $j=0; $j<scalar(@{$usage_pools{Pools}[$i]{Usage}{Usage}}); $j++){ 
            my %poolBox = %{$usage_pools{Pools}[$i]{Usage}{Usage}[$j]};

            my $StartTime = $poolBox{StartTime};
            my $unixStartTime = time2unix($StartTime);

            # !!! CHANGE next part: write out all needed information
            # memory minutes
            for my $lpar (keys %{$poolBox{MemoryMinutes}}){
              my $lc_lpar = lc($lpar);
              $data_timed{Pools}{$ID}{Metrics}{$unixStartTime}{"mm_$lc_lpar"} = $poolBox{MemoryMinutes}{$lpar};
            }

            # core minutes
            for my $lpar (keys %{$poolBox{CoreMinutes}}){
              my $lc_lpar = lc($lpar);
              $data_timed{Pools}{$ID}{Metrics}{$unixStartTime}{"cm_$lc_lpar"} = $poolBox{CoreMinutes}{$lpar};
            }

            # core metered minutes
            for my $lpar (keys %{$poolBox{CoreMeteredMinutes}}){
              my $lc_lpar = lc($lpar);
              $data_timed{Pools}{$ID}{Metrics}{$unixStartTime}{"cmm_$lc_lpar"} = $poolBox{CoreMeteredMinutes}{$lpar};
            }

            # core metered credits
            for my $lpar (keys %{$poolBox{CoreMeteredCredits}}){
              my $lc_lpar = lc($lpar);
              $data_timed{Pools}{$ID}{Metrics}{$unixStartTime}{"cmc_$lc_lpar"} = $poolBox{CoreMeteredCredits}{$lpar};
              #warn "\nCoreMeteredCredits: $lpar: ";
              #warn $poolBox{CoreMeteredCredits}{$lpar};
            }

            $data_timed{Pools}{$ID}{Metrics}{$unixStartTime}{mm_minutes} = $poolBox{MemoryMeteredMinutes};
            $data_timed{Pools}{$ID}{Metrics}{$unixStartTime}{mm_credits} = $poolBox{MemoryMeteredCredits};

            my @pool_total_credit = (
              'cmc_aix',
              'cmc_ibmi',
              'cmc_rhelcoreos',
              'cmc_rhel',
              'cmc_sles',
              'cmc_linuxvios',
              'mm_credits'
            );

            my $credit_sum = 0;
            my $undef_coutner = 0;

            for my $metric_to_sum (@pool_total_credit){
              $credit_sum += $data_timed{Pools}{$ID}{Metrics}{$unixStartTime}{$metric_to_sum};
            }
            #if ($undef_counter eq scalar(@pool_total_credit)){
            #  #undef
            #  $credit_sum = $data_timed{Pools}{$ID}{Metrics}{$unixStartTime}{cmc_aix};
            #}
            #print "$unixStartTime $credit_sum \n";
            my %translate_to_reserve = (
              "base_core_any_os"    =>  "reserve_3",  
              "base_core_linuxvios" =>  "reserve6",
              "base_core_rhcos"     =>  "reserve8",
              "base_core_rhel"      =>  "reserve_5",
              "base_core_sles"      =>  "reserve10",
              "base_core_aix"       =>  "reserve2",
              "base_core_ibmi"      =>  "reserve4",
            );

            for my $os_metric (keys %translate_to_reserve) {
              my $reserve_metric = $translate_to_reserve{$os_metric};
              $data_timed{Pools}{$ID}{Metrics}{$unixStartTime}{$reserve_metric} = $data{Pools}{$ID}{Configuration}{$os_metric};
              $data_timed{Pools}{$ID}{Metrics}{$unixStartTime}{$os_metric} = $data{Pools}{$ID}{Configuration}{$os_metric};
            }

            $data_timed{Pools}{$ID}{Metrics}{$unixStartTime}{reserve_1} = $credit_sum;
          }
        }
        else{
          $loc_query = timify_query($url_usage_pools);
          error("No usage pool data for poolID $ID from $loc_query ");
        }

      }
    }
  }
  # ERROR CATCH OF USAGE/POOLS IN UPPER CYCLE

} # process_usage_pools

sub process_managed_system_data {
  # work with %managed_hmc_data
  if (defined $managed_hmc_data{ManagedSystems}){
    for (my $i = 0; $i<scalar(@{$managed_hmc_data{ManagedSystems}}); $i++){
      my %one_server_data = %{$managed_hmc_data{ManagedSystems}[$i]};
      #print Dumper \%one_server_data; 
      my $UUID = $one_server_data{UUID};
      #only servers in pools
      if ($data{Systems}{$UUID}){
        $data{Systems}{$UUID}{Name} = $one_server_data{Name};
      
       # # PERFORMANCE FOR RRD
       # $data{Systems}{$UUID}{Metrics}{proc_installed} = $one_server_data{ProcessorConfiguration}{InstalledProcessorUnits};
       # $data{Systems}{$UUID}{Metrics}{proc_available} = $one_server_data{ProcessorConfiguration}{AvailableProcessorUnits};
       # $data{Systems}{$UUID}{Metrics}{mem_installed}  = $one_server_data{MemoryConfiguration}{InstalledMemory};
       # $data{Systems}{$UUID}{Metrics}{mem_available}  = $one_server_data{MemoryConfiguration}{AvailableMemory};
      
        # CONFIGURATION 
        $data{Systems}{$UUID}{Configuration}{proc_installed} = $one_server_data{ProcessorConfiguration}{InstalledProcessorUnits};
        $data{Systems}{$UUID}{Configuration}{proc_available} = $one_server_data{ProcessorConfiguration}{AvailableProcessorUnits};
        $data{Systems}{$UUID}{Configuration}{mem_installed}  = $one_server_data{MemoryConfiguration}{InstalledMemory};
        $data{Systems}{$UUID}{Configuration}{mem_available}  = $one_server_data{MemoryConfiguration}{AvailableMemory};


        $data{Systems}{$UUID}{Configuration}{State}  = $one_server_data{State};
        $data{Systems}{$UUID}{Configuration}{NumberOfLPARs}  = $one_server_data{NumberOfLPARs} || 0;
        $data{Systems}{$UUID}{Configuration}{NumberOfVIOSs}  = $one_server_data{NumberOfVIOSs} || 0;
        $data{Systems}{$UUID}{Configuration}{number_of_partitions}  = $one_server_data{NumberOfVIOSs} + $one_server_data{NumberOfLPARs};
        $data{Systems}{$UUID}{Configuration}{SystemFirmware}  = $one_server_data{SystemFirmware};

      }
    }
  }
  else{
    $loc_query = $url_managed_system;
    error( "No server data from $loc_query ");
  }
  #print Dumper \%data;
  # add configuration to every entry in timed data
  for my $pool_id (keys %{$data{Pools}}){
    my @configuration_to_sum = (
      'proc_available', 'proc_installed', 'mem_available', 
      'mem_installed', 'base_anyoscores', 'base_memory',
      'NumberOfLPARs', 'NumberOfVIOSs', 'number_of_partitions'
    );

    for my $confitem (@configuration_to_sum){

      $data{Pools}{$pool_id}{Configuration}{$confitem} = 0; 

      for my $UUID (keys %{$data{Pools}{$pool_id}{Systems}}){
        if ( defined $data{Systems}{$UUID}{Configuration}{$confitem} ){
          $data{Pools}{$pool_id}{Configuration}{$confitem}  += $data{Systems}{$UUID}{Configuration}{$confitem};
        }
      }
      for my $UUID (keys %{$data{Pools}{$pool_id}{Systems}}){
        if (! defined $data{Systems}{$UUID}{Configuration}{$confitem}){
          $data{Pools}{$pool_id}{Configuration}{$confitem} = '';
        }
      }
    }
  }
} # process_managed_system_data

sub process_management_console_data {
  if (defined $managed_hmc_data{HMCs} && scalar(@{$managed_hmc_data{HMCs}})){

    for (my $i = 0; $i<scalar(@{$managed_hmc_data{HMCs}}); $i++){
      my %one_hmc_data = %{$managed_hmc_data{HMCs}[$i]};

      my $hmc_uuid = $one_hmc_data{UUID};

      $data{HMCs}{$hmc_uuid}{Name} = $managed_hmc_data{HMCs}[$i]{Name};
      $data{HMCs}{$hmc_uuid}{Configuration}{UVMID} = $managed_hmc_data{HMCs}[$i]{UVMID};

      for (my $j = 0; $j<scalar(@{$one_hmc_data{ManagedSystems}}); $j++){
        my $UUID = $one_hmc_data{ManagedSystems}[$j]{UUID};

        if ($data{Systems}{$UUID}){ 
          $data{Systems}{$UUID}{HMCs}{$hmc_uuid} = $one_hmc_data{Name};
          $data{HMCs}{$hmc_uuid}{Systems}{$UUID} = $one_hmc_data{ManagedSystems}[$j]{Name};

          if ( !defined $data{Systems}{$UUID}{Configuration}{hmc_names} || ! $data{Systems}{$UUID}{Configuration}{hmc_names} ) {
            $data{Systems}{$UUID}{Configuration}{hmc_names} = $one_hmc_data{Name};
          }
          else {
            $data{Systems}{$UUID}{Configuration}{hmc_names} .= " ".$one_hmc_data{Name};
          }
        }
      }
    }
  }
  else{
    $loc_query = $url_management_console;
    error( "No HMC console data from $loc_query ");
  }
} # process_management_console_data

sub round_and_multiply_data {

  my %rounding_rules = (
    "Systems" => {
      'mem_available' => 0,
      'mem_installed' => 0,
      'base_memory'   => 0,
    },
    "Pools" => {
      'mem_available' => 0,
      'mem_installed' => 0,
      'base_memory'   => 0,
      'CurrentRemainingCreditBalance' => 2,
    }
  
  );
  my %multipliers = (
    'Pools' => {
      'mem_available' => $GIBI_BYTE,
      'mem_installed' => $GIBI_BYTE,
      'base_memory'   => $GIBI_BYTE,
      'CurrentRemainingCreditBalance' => 1,
    },
    'Systems' => {
      'mem_available' => $GIBI_BYTE,
      'mem_installed' => $GIBI_BYTE,
      'base_memory'   => $GIBI_BYTE,
    },
  );

  for my $section (keys %rounding_rules){
    for my $id (keys %{$data{$section}}){
      for my $conf_name (keys %{$multipliers{$section}}){
        if ( ! defined $data{$section}{$id}{Configuration}{$conf_name} ) {
          print " UNKNOWN $conf_name \n";
        }
        my $multiplied_value = $data{$section}{$id}{Configuration}{$conf_name} * $multipliers{$section}{$conf_name};

        my $round_to = $rounding_rules{$section}{$conf_name};

        my $rounded_value = (sprintf "%.${round_to}f",$multiplied_value);

        $data{$section}{$id}{Configuration}{$conf_name} = $rounded_value;
      }
    }
  }
}

sub check_tagging {
  my @tagged_pools = keys %{$data{Pools}};
  # POOL TAGGING INTEGRITY CHECK
  my @all_pools = keys %{$data{Pools}};
  
  my %pool_check;
  for my $some_pool (@all_pools){
    $pool_check{$some_pool} = 0;
  }
  for my $tagged_pool (@tagged_pools){
    $pool_check{$tagged_pool} = 1;
  }
  
  # 2: Pool tagging check
  my @untagged_pools;
  my $tagging_problem = 0;
  for my $some_pool (keys %pool_check){
    if ($some_pool){
      if (! $pool_check{$some_pool}){
        my $p_name = $data{Pools}{$some_pool}{Name};
        $tagging_problem = 1;
        push (@untagged_pools, $p_name);
      }
    }
  }
  
  if ($tagging_problem){
    print "UNTAGGED POOLS: @untagged_pools";
    error("UNTAGGED POOLS: @untagged_pools ");
  }
}


sub make_pool_budget_data {
  my $console             = shift;
  my $datahash_reference  = shift;
  my $interval            = shift; # hourly/daily/weekly/monthly

  # Larger intervals must be collected: credit values are generally small + only 2 decimal places 
  # sum of 24 hourly != daily
  print_debug_message("MAKING POOL BUDGET");

  my %usage_pools = %{$datahash_reference};


  if (defined $usage_pools{Pools}){
    # ERROR CATCH OF USAGE/POOLS IN UPPER CYCLE
    for (my $i=0; $i<scalar(@{$usage_pools{Pools}}); $i++){
      my $ID = $usage_pools{Pools}[$i]{PoolID};
      # possibly could be ""
      if ($ID){
        my %data_timed_credit = ();

        if (defined $usage_pools{Pools}[$i]{Usage}{Usage}){
          #my %max_size = (
          #   "Daily" => 14,
          #   "Weekly" => 12,
          #   "Monthly" => 12
          #);


          for (my $j=0; $j<scalar(@{$usage_pools{Pools}[$i]{Usage}{Usage}}); $j++){ 
            my %poolBox = %{$usage_pools{Pools}[$i]{Usage}{Usage}[$j]};

            my $StartTime = $poolBox{StartTime};
            my $unixStartTime = time2unix($StartTime);

            if ( $interval eq "Daily" ) {
              $StartTime =~ s/:..\..*//;
              $StartTime =~ s/T/ /;

            }
            elsif ( $interval eq "Weekly" ) {
              $StartTime =~ s/T.*//;
            }
            elsif ( $interval eq "Monthly" ) {
              $StartTime =~ s/T.*//;
            }

            ## memory minutes
            #for my $lpar (keys %{$poolBox{MemoryMinutes}}){
            #  my $lc_lpar = lc($lpar);
            #  #$data_timed_credit{$StartTime}{"mm_$lc_lpar"} = $poolBox{MemoryMinutes}{$lpar};
            #}

            # core metered minutes
            for my $lpar (keys %{$poolBox{CoreMeteredMinutes}}){
              $data_timed_credit{$StartTime}{CoreMeteredMinutes}{$lpar} = $poolBox{CoreMeteredMinutes}{$lpar};
            }

            my @collectibles = (
              "LinuxVIOS",
              "AnyOS",
              "RHEL",
              "SLES",
              "RHELCoreOS",
              "Total",
              "IBMi",
              "AIX",
            );

            # core metered credits
            #for my $lpar (keys %{$poolBox{CoreMeteredCredits}}){
            for my $lpar (@collectibles){
              if ( defined $poolBox{CoreMeteredCredits}{$lpar} ) {
                $data_timed_credit{$StartTime}{"CoreMeteredCredits"}{$lpar} = $poolBox{CoreMeteredCredits}{$lpar} ;
              }
            }

            $data_timed_credit{$StartTime}{MemoryMeteredMinutes} = $poolBox{MemoryMeteredMinutes};
            $data_timed_credit{$StartTime}{MemoryMeteredCredits} = $poolBox{MemoryMeteredCredits};

          }
        }
        else{
          #$loc_query = timify_query($url_usage_pools);
          #error("No usage pool data for poolID $ID from $loc_query ");
        }

        my $encoded_credits = encode_json(\%data_timed_credit);

        $credit_info{$ID}{$interval} = $encoded_credits;
      }
    }
  }

} # make_pool_budget_data


sub timify_query {
  my $base_url  = shift;
  my $frequency = shift || "Minute";

  $base_url .= "?EndTS=${EndTS}&Frequency=${frequency}&StartTS=${StartTS}";

  return $base_url;
}

sub collect_full_range_credits {
  my $url_usage_pools = "https://${portal_url}/api/public/v1/ep/usage/pools";

  my %usage_pools;
  #my @frequencies = ("Minute", "Hourly", "Daily", "Weekly", "Monthly");
  my @frequencies = ("Daily", "Weekly", "Monthly");


  for my $frequency (@frequencies) {
    my $use_time = time();
    my $query = time_budget_query($use_time, $frequency);
    my $timed_url = "${url_usage_pools}?${query}";

    %usage_pools = general_hash_request("GET", $timed_url);

    make_pool_budget_data($console_name, \%usage_pools, $frequency);

  }



}

# general subroutine to load files
sub file_to_string {
  my $filename = shift;
  my $json;
  
  print_debug_message("READING FILE: ${filename} ");
  open(FH, '<', $filename) || print( " Can't open: $filename : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  
  while(<FH>){
     $json .= $_;
  }

  close(FH);
  return $json;
}

sub time2unix {
  # CMC format
  # 2023-04-27T12:00:00.000Z  
  # >> 
  # UNIX format
  # 1682589600
  my $time_string = shift;
  use Time::Local;
  #print_debug_message("$time_string");

  my $unix_time;
  
  if ($time_string =~ /(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})/){
    $unix_time = timegm(0,$5,$4,$3,$2-1,$1);
  }
  
  #$unix_time += 1123200 - 43200 + 86400 ;
  return $unix_time;
}

1;
