package Power;

use strict;
use warnings;

use HTTP::Request::Common;
use LWP;
use Data::Dumper;
use JSON;
use POSIX qw(strftime ceil);
use Date::Parse;
use Time::Local;
use JSON qw(decode_json);
use Xormon;

use XormonLogger;


require "xml.pl";

my $hw_type = 'power';

my $DEFAULT_TIMEOUT           = 120;
my $DEFAULT_REQ_CHECK_LIMIT   = 5;
my $DEBUG_HMC_RESPONSE        = 0; # print REST API response
my $SAVE_HMC_RESPONSE_SAMPLE  = 0;

# For debug purpouses, NG can simulate a big environment by duplicating the current environment LPARs. If set to 100, it will create 100 copies of all current LPARs.
sub lpar_multiplicator {
  my $multiplicator = 1;
  return $multiplicator;
}

# TODO: more parents to arch
# TODO: logging servers automatically
# TODO NOTE: more about current state to logs
# TODO NOTE: check+rework usage of local/global variables
# TODO NOTE: check sessionwork - DB state, logouts ...
# TODO: encode password resulting in HMC error

# Typically Memory is not reported in bytes but in MiBi bytes. So these are the values to convert the values reported by the API.
my $convertValues = {
  'ConfigurableSystemMemory' => 1048576,
  'ConfiguredMirroredMemory' => 1048576,
  'CurrentAvailableMirroredMemory' => 1048576,
  'CurrentAvailableSystemMemory' => 1048576,
  'CurrentMirroredMemory' => 1048576,
  'DeconfiguredSystemMemory' => 1048576,
  'InstalledSystemMemory' => 1048576,
  'MemoryUsedByHypervisor' => 1048576,
  'PendingAvailableSystemMemory' => 1048576,
  'assignedMem' => 1048576,
  'configurableMem' => 1048576,
  'assignedMemToLpars' => 1048576,
  'availableMem' => 1048576,
  'totalMem' => 1048576,
  'mappedIOMemToLpars' => 1048576,
  'totalIOMem' => 1048576,
  'assignedMemToLpars' => 1048576,
  'assignedMemToSysFirmware' => 1048576,
  'logicalMem' => 1048576,
  'utilizedMem' => 1048576,
  'backedPhysicalMem' => 1048576,
  'virtualPersistentMem' => 1048576,
};

# note: usage
#list of available subsystems in the Power environment
sub available_subsystems (){  
  return [
    { subsystem => 'server', label => 'Server' },
    { subsystem => 'vm', label => 'LPAR' },
    { subsystem => 'pool', label => 'Shared CPU Pool' },
    # { subsystem => 'mempool', label => 'Memory Pool' },
    { subsystem => 'sea', label => 'SEA' },
    { subsystem => 'fiber', label => 'SAN' },
    { subsystem => 'generic', label => 'LAN' },
    { subsystem => 'genericv', label => 'vLAN' },
    { subsystem => 'genericvirtual', label => 'vSCSI' },
    { subsystem => 'genericphysical', label => 'SAS' },
    { subsystem => 'hea', label => 'HEA' },
    { subsystem => 'sharedstoragepool', label => 'Storage Pool' },
    { subsystem => 'virtualethernet', label => 'Virtual Ethernet' },
    { subsystem => 'sriovlogical', label => 'SR-IOV Logical' },
    { subsystem => 'sriovphysical', label => 'SR-IOV Physical' },
    { subsystem => 'physicalvolume', label => 'Volume' },
    { subsystem => 'trunk', label => 'Trunk' },
    { subsystem => 'backingsea', label => 'Backing SEA' },
    { subsystem => 'freelinkaggregation', label => 'Link Aggregation' },
    { subsystem => 'ioadapter', label => 'IO Adapter' },
    { subsystem => 'iobus', label => 'IO Bus' },
    { subsystem => 'ioslot', label => 'IO Slot' },
    { subsystem => 'profileioslot', label => 'Profile IO Slot' },
    { subsystem => 'virtualfibrenpiv', label => 'NPIV' },

  ];
}

sub checkInputParams {
  for (@_){
    if (!defined $_){
      warn "test: $_\n";
      return 1;
    } 
  }
  return 0;
}

#get performance data from a Managed system
sub ManagedSystemEnergyToData {
  my ($self, $data_out, $sample, $server_uid) = @_;


  $self->{'logger'}->info("ManagedSystemEnergyToData");

  my $link  = $sample->{'link'};
  my $energyJsonContent = $self->getJsonContent ($link);
  for (@{ $energyJsonContent->{'systemUtil'}{'utilSamples'} }){
    my $serverSample = $_;
    my $timeStamp = str2time ( $serverSample->{'sampleInfo'}{'timeStamp'} );
    my $subsystem = 'server';
    my $energyUtil = $serverSample->{'energyUtil'};

    # watts
    my $powerUtil = $energyUtil->{'powerUtil'};
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'powerReading'} = defined $powerUtil->{'powerReading'}[0] ? $powerUtil->{'powerReading'}[0] : undef;

    ##
    ## note: not used, comment out
    ##
    ## TODO: temperature
    #my $thermalUtil = $energyUtil->{'thermalUtil'};
    #
    #foreach (@{$thermalUtil->{'inletTemperatures'}}){
    #  next;
    #  my $inlet = $_;
    #  my $inlet_uid = $server_uid.'-'.$inlet->{'entityId'};
    #  my $inlet_label = $inlet->{'entityInstance'};
    #  $data_out->{$hw_type}{'inlet'}{$inlet_uid}{$timeStamp}{'temperatureReading'} = defined $inlet->{'temperatureReading'}[0] ? $inlet->{'temperatureReading'}[0] : undef;
    #  if (!defined $self->{'architecture_duplicates_check'}->{$inlet_uid}){
    #    #architecture inlet
    #    my $label = $inlet_label ? $inlet_label : $inlet_uid;
    #    my %inlet_arch = (
    #        'item_id' => $inlet_uid,
    #        'label' => $inlet_label,
    #        'hw_type' => $hw_type,
    #        'subsystem' => 'inlet',
    #        'parents' => [$server_uid],
    #        'hostcfg_id' => $self->{'hostcfg_id'}
    #    );
    #    $self->{'architecture_duplicates_check'}->{$inlet_uid} = 1;
    #    push(@{$self->{'architecture'}->{'inlet'}}, \%inlet_arch);
    #  }
    #}
    #foreach (@{$thermalUtil->{'baseboardTemperatures'}}){
    #  next;
    #  my $baseboard = $_;
    #  my $baseboard_uid = $server_uid.'-'.$baseboard->{'entityId'};
    #  my $baseboard_label = $baseboard->{'entityInstance'};
    #  $data_out->{$hw_type}{'baseboard'}{$baseboard_uid}{$timeStamp}{'temperatureReading'} = defined $baseboard->{'temperatureReading'}[0] ? $baseboard->{'temperatureReading'}[0] : undef;
    #  if (!defined $self->{'architecture_duplicates_check'}->{$baseboard_uid}){
    #    #architecture baseboard
    #    my $label = $baseboard_label ? $baseboard_label : $baseboard_uid;
    #    my %baseboard_arch = (
    #        'item_id' => $baseboard_uid,
    #        'label' => $baseboard_label,
    #        'hw_type' => $hw_type,
    #        'subsystem' => 'baseboard',
    #        'parents' => [$server_uid],
    #        'hostcfg_id' => $self->{'hostcfg_id'}
    #    );
    #    $self->{'architecture_duplicates_check'}->{$baseboard_uid} = 1;
    #    push(@{$self->{'architecture'}->{'baseboard'}}, \%baseboard_arch);
    #  }
    #}
    #foreach (@{$thermalUtil->{'cpuTemperatures'}}){
    #  next;
    #  my $cpu = $_;
    #  my $cpu_uid = $server_uid.'-'.$cpu->{'entityId'};
    #  my $cpu_label = $cpu->{'entityInstance'};
    #  $data_out->{$hw_type}{'cpu'}{$cpu_uid}{$timeStamp}{'temperatureReading'} = defined $cpu->{'temperatureReading'}[0] ? $cpu->{'temperatureReading'}[0] : undef;
    #  if (!defined $self->{'architecture_duplicates_check'}->{$cpu_uid}){
    #    #architecture cpu
    #    my $label = $cpu_label ? $cpu_label : $cpu_uid;
    #    my %cpu_arch = (
    #        'item_id' => $cpu_uid,
    #        'label' => $cpu_label,
    #        'hw_type' => $hw_type,
    #        'subsystem' => 'cpu',
    #        'parents' => [$server_uid],
    #        'hostcfg_id' => $self->{'hostcfg_id'}
    #    );
    #    $self->{'architecture_duplicates_check'}->{$cpu_uid} = 1;
    #    push(@{$self->{'architecture'}->{'cpu'}}, \%cpu_arch);
    #  }
    #}    
    #$data_out->{$hw_type}{cpu}{$cpu_uid}{$timeStamp}{'metric'} = defined $powerUtil->{'metric'}[0] ? $powerUtil->{'metric'}[0] : undef;
  }
  return 0;
}

#get performance data from a Managed system
sub ManagedSystemPerformanceToData {
  my ($self, $data_out, $sample, $server_uid, $sharedProcessorPools, $cpu_utilization_coresTotalLparsForThisServer) = @_;

  if (checkInputParams($sample, $server_uid, $sharedProcessorPools)){
    $self->{'logger'}->error("ManagedSystemPerformanceToData Params error $sample, $server_uid, $sharedProcessorPools");
    return {};
  } else {
    $self->{'logger'}->debug("ManagedSystemPerformanceToData Params seem ok, $sample, $server_uid, $sharedProcessorPools");
  }

  my $link  = $sample->{'link'};
  my $processedJsonContent = $self->getJsonContent ($link);

  my $bundle_size = 100;
  
  if (!%{$processedJsonContent}){
    $self->{'logger'}->error("No SERVERs processed metrics JSONs found for $server_uid");
  }

  for (@{ $processedJsonContent->{'systemUtil'}{'utilSamples'} }){
    my $serverSample = $_;
    my $timeStamp = str2time ( $serverSample->{'sampleInfo'}{'timeStamp'} );
    my $subsystem = 'vm';
    
    #ManagedSystem Performnace
    my $serverUtil = $serverSample->{'serverUtil'};
    my $systemFirmwareUtil = $serverSample->{'systemFirmwareUtil'};
    my $serverProcessor = $serverUtil->{'processor'};
    my $serverMemory = $serverUtil->{'memory'};
    my $serverSharedMemoryPool = $serverUtil->{'sharedMemoryPool'};
    my $serverSharedProcessorPool = $serverUtil->{'sharedProcessorPool'};
    my $serverNetwork = $serverUtil->{'network'};
    my $serverPhysicalProcessorPool = $serverUtil->{'physicalProcessorPool'};

    #ViosUtil Performance
    my $viosUtil = $serverSample->{'viosUtil'};
    for (@{$viosUtil}){
      my $vios = $_;
      my $vios_uid = $vios->{'uuid'};
      my $vios_label = $vios->{'name'};
      my $proc = $vios->{'processor'};

      my $mem = $vios->{'memory'};
      my $stor = $vios->{'storage'};
      my $netw = $vios->{'network'};

      if (!defined $vios_uid){
        $self->{'logger'}->error("vios_uid not defined for $vios_label on $server_uid.");
        next;
      }

      
      # Cores assigned to the VM.
      # Tricky one is the currentVirtualProcessors - if the currentVirtualProcessor is not defined (or 0), then the vios is dedicated, so the entitledProcUnits = currentVirtualProcessors
      $subsystem = 'vm';
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'} = defined $proc->{'currentVirtualProcessors'}[0] && $proc->{'currentVirtualProcessors'}[0] > 0 ? sprintf("%d", $proc->{'currentVirtualProcessors'}[0]) : sprintf("%d", $proc->{'entitledProcUnits'}[0]);
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'maxVirtualProcessors'} = defined $proc->{'maxVirtualProcessors'}[0] ? sprintf("%d", $proc->{'maxVirtualProcessors'}[0]) : undef;
      
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'entitledProcUnits'} = defined $proc->{'entitledProcUnits'}[0] ? $proc->{'entitledProcUnits'}[0] : undef;
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_cores'} = defined $proc->{'utilizedProcUnits'}[0] ? $proc->{'utilizedProcUnits'}[0] : undef;
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'maxProcUnits'} = defined $proc->{'maxProcUnits'}[0] ? $proc->{'maxProcUnits'}[0] : undef;
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'logicalMem'} = defined $mem->{'assignedMem'}[0] ? sprintf("%d", $mem->{'assignedMem'}[0] * $convertValues->{'assignedMem'}) : undef;
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'backedPhysicalMem'} = defined $mem->{'backedPhysicalMem'}[0] ? sprintf("%d", $mem->{'backedPhysicalMem'}[0]* $convertValues->{'backedPhysicalMem'}) : undef;
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'virtualPersistentMem'} = defined $mem->{'virtualPersistentMem'}[0] ? sprintf("%d", $mem->{'virtualPersistentMem'}[0]* $convertValues->{'virtualPersistentMem'}) : undef;
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'utilizedMem'} = defined $mem->{'utilizedMem'}[0] ? sprintf("%d", $mem->{'utilizedMem'}[0] * $convertValues->{'utilizedMem'}) : undef;
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'assignedMem'} = defined $mem->{'assignedMem'}[0] ? sprintf("%d", $mem->{'assignedMem'}[0] * $convertValues->{'assignedMem'}) : undef;

      # 1. try: usage = capped + uncapped - idle
      #   - if usage < 0 (<=> unrealistic (?) idle)  => set usage = undef - That indicated some problem with returned idle
      #       - condition: dedicated LPAR + kvm + VM x86 win XP
      # 2. If response includes utilizedProcUnitsDeductIdle, use it as usage regardless of the previous step!
      #
      # 3. sum usage to server sum, if undefined usage, add 0
      #   - TODO: if all lpar usages are undef, then sum = undef
      if (  defined $proc->{'utilizedCappedProcUnits'}[0] &&
            defined $proc->{'utilizedUncappedProcUnits'}[0] &&
            defined $proc->{'idleProcUnits'}[0]) {

        # just for clarity, TODO: remove declaration
        my $cpu_util_capped   = $proc->{'utilizedCappedProcUnits'}[0];
        my $cpu_util_uncapped = $proc->{'utilizedUncappedProcUnits'}[0];
        my $cpu_idle          = $proc->{'idleProcUnits'}[0];

        my $cpu_physical = $cpu_util_capped + $cpu_util_uncapped;
        my $cpu_usage    = $cpu_util_capped + $cpu_util_uncapped - $cpu_idle;

        if ( $cpu_usage < 0 ) {
            $cpu_usage = undef;
        }

        $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_cores'}    = $cpu_physical;
        $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} = $cpu_usage;
      }
      else {
        #rest_api_log("PROBLEM: unexpected metrics");

        if ( defined $proc->{'utilizedProcUnits'}[0] && defined $proc->{'utilizedProcUnitsDeductIdle'}[0] ) {
          $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_cores'}    = $proc->{'utilizedProcUnits'}[0];
          $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} = $proc->{'utilizedProcUnitsDeductIdle'}[0];
        }
        elsif ( defined $proc->{'utilizedProcUnits'}[0] ) {
          $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_cores'}    = $proc->{'utilizedProcUnits'}[0];
          $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} = undef;
        }
        else {
          # NOT DONE!
          # TODO: all API possibilities
          #rest_api_log("PROBLEM: expected metrics NOT FOUND!");

          $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_cores'}    = undef;
          $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} = undef;
        }

      }

      # If REST API returns Usage, use it
      # there might be some problem with idle values
      if ( defined $proc->{'utilizedProcUnitsDeductIdle'}[0] ) {
        $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} = $proc->{'utilizedProcUnitsDeductIdle'}[0];
      }


      # warning - use only if there is no utilizedProcUnitsDeductIdle for the server!!
      if ( defined $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} ) {
        # do not devalue whole server stats if one lpar returns idle > capped + uncapped
        $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_utilization_coresTotalLparsForThisServer'} += $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'};
        $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_utilization_cores'} += $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'};
      }
      else {
        $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_utilization_coresTotalLparsForThisServer'} += 0;
        $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_utilization_cores'} += 0;
      }
      
      $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_physical_cores'} += $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_cores'};
      
      if (defined $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'idleProcUnits'} ){
        $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'idle'} += $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'idleProcUnits'};
      }
      else {
        $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'idle'} += 0;
      }
      
      #create these metric for xormon
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_percent'} = (defined $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'} && $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_cores'} / $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'})) : undef;
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'entitledProcUnitsPercent'} = (defined $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'entitledProcUnits'} && $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'entitledProcUnits'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_cores'} / $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'entitledProcUnits'})) : undef;

      # MAX
      $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_percent'} = (defined $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} && defined $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'} && $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} / $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'})) : undef;
      #$data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_cores_max'} = defined $proc->{'utilizedProcUnits'}[0] ? $proc->{'utilizedProcUnits'}[0] : undef;
      #$data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_percent_max'} = (defined $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'} && $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_physical_cores'} / $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'})) : undef;    
      #$data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_percent_max'} = (defined $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} && defined $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'} && $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} / $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'currentVirtualProcessors'})) : undef;
      #$data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores_max'} = defined $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} ? $data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'cpu_utilization_cores'} : undef;
        

      # Do not store there metrics for now. it can be useful in future
      #$data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'utilizedCappedProcUnits'} = $proc->{'utilizedCappedProcUnits'}[0];
      #$data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'utilizedUncappedProcUnits'} = $proc->{'utilizedUncappedProcUnits'}[0];
      #$data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'idleProcUnits'} = $proc->{'idleProcUnits'}[0];
      #$data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'donatedProcUnits'} = $proc->{'donatedProcUnits'}[0];
      #$data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'timeSpentWaitingForDispatch'} = $proc->{'timeSpentWaitingForDispatch'}[0];
      #$data_out->{$hw_type}{$subsystem}{$vios_uid}{$timeStamp}{'timePerInstructionExecution'} = $proc->{'timePerInstructionExecution'}[0];
      
      # sea
      $subsystem = 'sea';
      for (@{ $netw->{'sharedAdapters'} }){
        my $sea_adapter = $_;
        #my $sea_adapter_uid = $subsystem.'-'.$sea_adapter->{'physicalLocation'}.'-'.$sea_adapter->{'id'};

        if ( !defined $sea_adapter->{'physicalLocation'} ) {
          $self->{'logger'}->info("Undefined ID [sea]: physicalLocation");
          next;
        }
        my $sea_adapter_uid = $sea_adapter->{'physicalLocation'};

        $data_out->{$hw_type}{$subsystem}{$sea_adapter_uid}{$timeStamp}{'sentBytes'} = sprintf("%d", $sea_adapter->{'sentBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$sea_adapter_uid}{$timeStamp}{'receivedBytes'} = sprintf("%d", $sea_adapter->{'receivedBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$sea_adapter_uid}{$timeStamp}{'sentPackets'} = sprintf("%d", $sea_adapter->{'sentPackets'}[0]);
        $data_out->{$hw_type}{$subsystem}{$sea_adapter_uid}{$timeStamp}{'receivedPackets'} = sprintf("%d", $sea_adapter->{'receivedPackets'}[0]);
        #$data_out->{$hw_type}{$subsystem}{$sea_adapter_uid}{$timeStamp}{'id'} = $sea_adapter->{'id'};
        #$data_out->{$hw_type}{$subsystem}{$sea_adapter_uid}{$timeStamp}{'type'} = $sea_adapter->{'type'};
        #$data_out->{$hw_type}{$subsystem}{$sea_adapter_uid}{$timeStamp}{'transferredBytes'} = $sea_adapter->{'transferredBytes'}[0];
        #$data_out->{$hw_type}{$subsystem}{$sea_adapter_uid}{$timeStamp}{'droppedPackets'} = $sea_adapter->{'droppedPackets'}[0];

        #architecture sea
        if (!defined $self->{'architecture_duplicates_check'}->{$sea_adapter_uid}){
          my $label = $sea_adapter->{'id'} ? $sea_adapter->{'id'} : $sea_adapter_uid;
          my (@physLocParts) = split('\.', $sea_adapter->{'physicalLocation'});
          $label    = $physLocParts[-1] ? "$vios_label $label" : $label;

          my %sea_arch = (
              'item_id'     => $sea_adapter_uid,
              'label'       => $label,
              'hw_type'     => $hw_type,
              'subsystem'   => $subsystem,
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'}
          );
          push(@{$self->{'architecture'}->{$subsystem}}, \%sea_arch);
          $self->{'architecture_duplicates_check'}->{$sea_adapter_uid} = 1;

        }
      }

      # genericAdapters
      $subsystem = 'generic';
      for (@{ $netw->{'genericAdapters'} }){
        my $generic_adapter = $_;

        if ( !defined $generic_adapter->{'physicalLocation'} ) {
          $self->{'logger'}->info("Undefined ID [generic/virtual]: physicalLocation");
          next;
        }
        my $generic_adapter_uid = $generic_adapter->{'physicalLocation'};

        $subsystem = $generic_adapter->{'type'} eq 'virtual' ? 'genericv' : 'generic';

        if ($subsystem eq 'genericv'){
          # Generic adapter of type "virtual" has the same UID as HEA... Need to be implemented in other way. Skip them for now as they break the item_properties table (same uid as HEA)
          $generic_adapter_uid .= '-virtual';
        }
        #my $generic_adapter_uid = $subsystem.'-'.$generic_adapter->{'physicalLocation'}.'-'.$generic_adapter->{'id'};
        #print "debug generic : $generic_adapter_uid\n";
        $data_out->{$hw_type}{$subsystem}{$generic_adapter_uid}{$timeStamp}{'sentBytes'} = sprintf("%d", $generic_adapter->{'sentBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$generic_adapter_uid}{$timeStamp}{'receivedBytes'} = sprintf("%d", $generic_adapter->{'receivedBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$generic_adapter_uid}{$timeStamp}{'sentPackets'} = sprintf("%d", $generic_adapter->{'sentPackets'}[0]);
        $data_out->{$hw_type}{$subsystem}{$generic_adapter_uid}{$timeStamp}{'receivedPackets'} = sprintf("%d", $generic_adapter->{'receivedPackets'}[0]);
        #$data_out->{$hw_type}{$subsystem}{$generic_adapter_uid}{$timeStamp}{'id'} = $generic_adapter->{'id'};
        #$data_out->{$hw_type}{$subsystem}{$generic_adapter_uid}{$timeStamp}{'type'} = $generic_adapter->{'type'};
        #$data_out->{$hw_type}{$subsystem}{$generic_adapter_uid}{$timeStamp}{'transferredBytes'} = $generic_adapter->{'transferredBytes'}[0];
        #$data_out->{$hw_type}{$subsystem}{$generic_adapter_uid}{$timeStamp}{'droppedPackets'} = $generic_adapter->{'droppedPackets'}[0];

        #architecture generic adapters
        if (!defined $self->{'architecture_duplicates_check'}->{$generic_adapter_uid}){
          my $label = $generic_adapter->{'id'} ? $generic_adapter->{'id'} : $generic_adapter_uid;
          my (@physLoc) = split('\.', $generic_adapter->{'physicalLocation'});
          $label    = $physLoc[-1] ? "$vios_label $label" : $label;
          my %generic_arch = (
              'item_id'     => $generic_adapter_uid,
              'label'       => $label,
              'hw_type'     => $hw_type,
              'subsystem'   => $subsystem,
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'}
          );
          push(@{$self->{'architecture'}->{$subsystem}}, \%generic_arch);
          $self->{'architecture_duplicates_check'}->{$generic_adapter_uid} = 1;        
        }
      }

      # sriovLogicalPorts
      $subsystem = 'sriovlogical';
      for (@{ $netw->{'sriovLogicalPorts'} }){
        my $port = $_;

        if ( !defined $port->{'physicalLocation'} ) {
          $self->{'logger'}->info("Undefined ID [sriovlogical]: physicalLocation");
          next;
        }
        my $port_uid = $port->{'physicalLocation'};

        #my $port_uid = $subsystem.'-'.$port->{'physicalLocation'}.'-'.$port->{'id'};
        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'sentBytes'} = sprintf("%d", $port->{'sentBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'receivedBytes'} = sprintf("%d", $port->{'receivedBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'sentPackets'} = sprintf("%d", $port->{'sentPackets'}[0]);
        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'receivedPackets'} = sprintf("%d", $port->{'receivedPackets'}[0]);
        #architecture sriov logical ports
        if (!defined $self->{'architecture_duplicates_check'}->{$port_uid}){
          my $label = $port->{'id'} ? $port->{'id'} : $port_uid;
          my (@physLoc) = split('\.', $port_uid);
          $label    = $physLoc[-1] ? "$vios_label $label" : $label;
          my %arch = (
              'item_id'     => $port_uid,
              'label'       => $label,
              'hw_type'     => $hw_type,
              'subsystem'   => $subsystem,
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'}
          );
          push(@{$self->{'architecture'}->{$subsystem}}, \%arch);
          $self->{'architecture_duplicates_check'}->{$port_uid} = 1;

        }
      }
      # virtualEthernetAdapters
      $subsystem = 'virtualethernet';
      for (@{ $netw->{'virtualEthernetAdapters'} }){
        my $adapter = $_;

        if ( !defined $adapter->{'physicalLocation'} ) {
          $self->{'logger'}->info("Undefined ID [virtualethernet]: physicalLocation");
          next;
        }
        my $adapter_uid = $adapter->{'physicalLocation'};

        #my $adapter_uid = $subsystem.'-'.$adapter->{'physicalLocation'}.'-'.$adapter->{'id'};
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'sentBytes'} = sprintf("%d", $adapter->{'sentBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'receivedBytes'} = sprintf("%d", $adapter->{'receivedBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'sentPackets'} = sprintf("%d", $adapter->{'sentPackets'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'receivedPackets'} = sprintf("%d", $adapter->{'receivedPackets'}[0]);

        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'sentPhysicalBytes'} = sprintf("%d", $adapter->{'sentPhysicalBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'receivedPhysicalBytes'} = sprintf("%d", $adapter->{'receivedPhysicalBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'sentPhysicalPackets'} = sprintf("%d", $adapter->{'sentPhysicalPackets'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'receivedPhysicalPackets'} = sprintf("%d", $adapter->{'receivedPhysicalPackets'}[0]);

        #architecture virtual ethernet adapters
        if (!defined $self->{'architecture_duplicates_check'}->{$adapter_uid}){
          my (@physLoc) = split('\.', $adapter->{'physicalLocation'});
          my $label = $physLoc[-1] ? $physLoc[-1] : $adapter_uid;
          my %arch = (
              'item_id'     => $adapter_uid,
              'label'       => $label,
              'hw_type'     => $hw_type,
              'subsystem'   => $subsystem,
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'}
          );
          push(@{$self->{'architecture'}->{$subsystem}}, \%arch);
          $self->{'architecture_duplicates_check'}->{$adapter_uid} = 1;

        }
      }

      #
      # Fiber Channel Adapters
      #
      $subsystem = 'fiber';
      for (@{ $stor->{'fiberChannelAdapters'} }){
        my $adapter = $_;

        #print "\n\n------------------------------\nfiberChannelAdapters\n";
        #print Dumper $adapter;

        if ( !defined $adapter->{'physicalLocation'} ) {
          $self->{'logger'}->info("Undefined ID [fiber]: physicalLocation");
          next;
        }
        my $adapter_uid = $adapter->{'physicalLocation'};

        #my $adapter_uid = $subsystem.'-'.$adapter->{'physicalLocation'}.'-'.$adapter->{'id'};
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'readBytes'} = sprintf("%d", $adapter->{'readBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'writeBytes'} = sprintf("%d", $adapter->{'writeBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'numOfReads'} = sprintf("%d", $adapter->{'numOfReads'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'numOfWrites'} = sprintf("%d", $adapter->{'numOfWrites'}[0]);
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'id'} = $adapter->{'id'};

        #architecture fiber adapters
        if (!defined $self->{'architecture_duplicates_check'}->{$adapter_uid}){
          my (@physLoc) = split ('\.', $adapter->{'physicalLocation'});
          my $label = $adapter->{'id'} ? $adapter->{'id'} : $adapter_uid;
          $label    = $physLoc[-1] ? "$vios_label $label" : $label;

          #print "FIBER CHANNEL ADAPTER - UID: $adapter_uid, LABEL: $label";
          my %adapter_arch = (
              'item_id'     => $adapter_uid,
              'label'       => $label,
              'hw_type'     => $hw_type,
              'subsystem'   => $subsystem,
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'}
          );

          push(@{$self->{'architecture'}->{$subsystem}}, \%adapter_arch);
          $self->{'architecture_duplicates_check'}->{$adapter_uid} = 1;

        }
      }

      # genericPhysicalAdapters
      $subsystem = 'genericphysical';
      for (@{ $stor->{'genericPhysicalAdapters'} }){
        my $adapter = $_;

        if ( !defined $adapter->{'physicalLocation'} ) {
          $self->{'logger'}->info("Undefined ID [genericphysical]: physicalLocation");
          next;
        }
        my $adapter_uid = $adapter->{'physicalLocation'};

        #my $adapter_uid = $subsystem.'-'.$adapter->{'physicalLocation'}.'-'.$adapter->{'id'};
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'readBytes'} = sprintf("%d", $adapter->{'readBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'writeBytes'} = sprintf("%d", $adapter->{'writeBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'numOfReads'} = sprintf("%d", $adapter->{'numOfReads'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'numOfWrites'} = sprintf("%d", $adapter->{'numOfWrites'}[0]);
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'id'} = $adapter->{'id'};
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'type'} = $adapter->{'type'};
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'transmittedBytes'} = $adapter->{'transmittedBytes'}[0];
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'runningSpeed'} = $adapter->{'runningSpeed'}[0];
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'wwpn'} = $adapter->{'wwpn'};
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'numOfPorts'} = $adapter->{'numOfPorts'};

        #architecture fiber adapters
        if (!defined $self->{'architecture_duplicates_check'}->{$adapter_uid}){
          my $label = $adapter->{'id'} ? $adapter->{'id'} : $adapter_uid;
          my (@physLoc) = split('\.', $adapter->{'physicalLocation'});
          $label    = $physLoc[-1] ? "$vios_label $label" : $label;
          my %adapter_arch = (
              'item_id'     => $adapter_uid,
              'label'       => $label,
              'hw_type'     => $hw_type,
              'subsystem'   => $subsystem,
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'}
          );
          push(@{$self->{'architecture'}{$subsystem}}, \%adapter_arch);
          $self->{'architecture_duplicates_check'}->{$adapter_uid} = 1;
        }
      }

      # genericVirtualAdapters
      $subsystem = 'genericvirtual';
      for (@{ $stor->{'genericVirtualAdapters'} }){
        my $adapter = $_;
        #my $adapter_uid = $subsystem.'-'.$adapter->{'physicalLocation'}.'-'.$adapter->{'id'};

        if ( !defined $adapter->{'physicalLocation'} ) {
          $self->{'logger'}->info("Undefined ID [genericvirtual]: physicalLocation");
          next;
        }
        my $adapter_uid = $adapter->{'physicalLocation'};

        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'readBytes'} = sprintf("%d", $adapter->{'readBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'writeBytes'} = sprintf("%d", $adapter->{'writeBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'numOfReads'} = sprintf("%d", $adapter->{'numOfReads'}[0]);
        $data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'numOfWrites'} = sprintf("%d", $adapter->{'numOfWrites'}[0]);
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'id'} = $adapter->{'id'};
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'type'} = $adapter->{'type'};
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'transmittedBytes'} = $adapter->{'transmittedBytes'}[0];
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'runningSpeed'} = $adapter->{'runningSpeed'}[0];
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'wwpn'} = $adapter->{'wwpn'};
        #$data_out->{$hw_type}{$subsystem}{$adapter_uid}{$timeStamp}{'numOfPorts'} = $adapter->{'numOfPorts'};

        #architecture fiber adapters
        if (!defined $self->{'architecture_duplicates_check'}->{$adapter_uid}){
          my $label = $adapter->{'id'} ? $adapter->{'id'} : $adapter_uid;
          my (@physLocParts) = split('\.', $adapter->{'physicalLocation'});
          $label    = $physLocParts[-1] ? "$vios_label $label" : $label;
          my %adapter_arch = (
              'item_id'     => $adapter_uid,
              'label'       => $label,
              'hw_type'     => $hw_type,
              'subsystem'   => $subsystem,
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'}
          );
          push(@{$self->{'architecture'}{$subsystem}}, \%adapter_arch);
          $self->{'architecture_duplicates_check'}->{$adapter_uid} = 1;

        }
      }
      # shared storage pools
      $subsystem = 'sharedstoragepool';
      for (@{ $stor->{'sharedStoragePools'} }){
        my $pool = $_;
        #my $pool_uid = $subsystem.'-'.$pool->{'id'};

        if ( !defined $pool->{'id'} ) {
          $self->{'logger'}->info("Undefined ID [sharedstoragepool]: id");
          next;
        }
        my $pool_uid = $pool->{'id'};

        $data_out->{$hw_type}{$subsystem}{$pool_uid}{$timeStamp}{'numOfReads'} = sprintf("%d", $pool->{'numOfReads'}[0]);
        $data_out->{$hw_type}{$subsystem}{$pool_uid}{$timeStamp}{'numOfWrites'} = sprintf("%d", $pool->{'numOfWrites'}[0]);
        $data_out->{$hw_type}{$subsystem}{$pool_uid}{$timeStamp}{'readBytes'} = sprintf("%d", $pool->{'readBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$pool_uid}{$timeStamp}{'writeBytes'} = sprintf("%d", $pool->{'writeBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$pool_uid}{$timeStamp}{'usedSpace'} = sprintf("%d", $pool->{'usedSpace'}[0]);
        $data_out->{$hw_type}{$subsystem}{$pool_uid}{$timeStamp}{'totalSpace'} = sprintf("%d", $pool->{'totalSpace'}[0]);
        #$data_out->{$hw_type}{$subsystem}{$pool_uid}{$timeStamp}{'id'} = $pool->{'id'};

        #architecture shared storage pools
        if (!defined $self->{'architecture_duplicates_check'}->{$pool_uid}){
          my %pool_arch = (
              'item_id'     => $pool_uid,
              'label'       => $pool_uid,
              'hw_type'     => $hw_type,
              'subsystem'   => $subsystem,
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'}
          );
          push(@{$self->{'architecture'}{$subsystem}}, \%pool_arch);
          $self->{'architecture_duplicates_check'}->{$pool_uid} = 1;
        }
      }
      $subsystem=undef;
    }
    $subsystem = 'server';
    # Physical Processor, these are metrics from API that we do not use now. It might be enabled in future.
    # NOTE: why is utilizedProcUnitsPhysicalProcessor duplicated as cpu_physical_cores?
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'assignedProcUnitsPhysicalProcessor'} = defined $serverPhysicalProcessorPool->{'assignedProcUnits'}[0] ? $serverPhysicalProcessorPool->{'assignedProcUnits'}[0] : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'utilizedProcUnitsPhysicalProcessor'} = defined $serverPhysicalProcessorPool->{'utilizedProcUnits'}[0] ? $serverPhysicalProcessorPool->{'utilizedProcUnits'}[0] : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'availableProcUnitsPhysicalProcessor'} = defined $serverPhysicalProcessorPool->{'availableProcUnits'}[0] ? $serverPhysicalProcessorPool->{'availableProcUnits'}[0] : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'configuredProcUnitsPhysicalProcessor'} = defined $serverPhysicalProcessorPool->{'configuredProcUnits'}[0] ? $serverPhysicalProcessorPool->{'configuredProcUnits'}[0] : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'borrowedProcUnitsPhysicalProcessor'} = defined $serverPhysicalProcessorPool->{'borrowedProcUnits'}[0] ? $serverPhysicalProcessorPool->{'borrowedProcUnits'}[0] : undef;

    # new configured
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'assignedMinusBorrowed'} = (defined $serverPhysicalProcessorPool->{'assignedProcUnits'}[0] && defined $serverPhysicalProcessorPool->{'borrowedProcUnits'}[0])? $serverPhysicalProcessorPool->{'assignedProcUnits'}[0]-$serverPhysicalProcessorPool->{'borrowedProcUnits'}[0] : undef;
    
    # Server Processor
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'systemFirmwareUtilUtilizedProcUnits'} = defined $systemFirmwareUtil->{'utilizedProcUnits'}[0] ? $systemFirmwareUtil->{'utilizedProcUnits'}[0] : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'} = defined $serverProcessor->{'utilizedProcUnits'}[0] ? $serverProcessor->{'utilizedProcUnits'}[0] : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} = defined $serverProcessor->{'utilizedProcUnitsDeductIdle'}[0] ? $serverProcessor->{'utilizedProcUnitsDeductIdle'}[0] : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'configurableProcUnits'} = defined $serverProcessor->{'configurableProcUnits'}[0] ? $serverProcessor->{'configurableProcUnits'}[0] : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'availableProcUnits'} = defined $serverProcessor->{'availableProcUnits'}[0] ? $serverProcessor->{'availableProcUnits'}[0] : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'totalProcUnits'} = defined $serverProcessor->{'totalProcUnits'}[0] ? $serverProcessor->{'totalProcUnits'}[0] : undef;
    # NOTE
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'assignedProcUnits'} = defined $serverProcessor->{'configurableProcUnits'}[0] ? $serverProcessor->{'configurableProcUnits'}[0] - $serverProcessor->{'availableProcUnits'}[0] : undef;


    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} = $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_utilization_cores'};
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'}    = $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_physical_cores'};

    # Prioritize API response
    if ( defined $serverProcessor->{'utilizedProcUnitsDeductIdle'}[0] ) {
      $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} = $serverProcessor->{'utilizedProcUnitsDeductIdle'}[0];
    }

    # Never negative (make more global?)
    if ( $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'} < 0 ) {
        $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'} = undef;
    }

    if ( $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} < 0 ) {
        $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} = undef;
    }

    # Never above limit (chosen limiting metric: configurableProcUnits)
    if ( defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'} &&
         defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'configurableProcUnits'} &&
         $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'} > $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'configurableProcUnits'} 
        ) {
        $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'} = undef;
    }

    if ( defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} &&
         defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'configurableProcUnits'} &&
         $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} > $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'configurableProcUnits'} 
        ) {
        $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} = undef;
    }

    if ( defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'utilizedProcUnitsPhysicalProcessor'} &&
         defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'configurableProcUnits'} &&
         $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'utilizedProcUnitsPhysicalProcessor'} > $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'configurableProcUnits'} 
        ) {
        $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'utilizedProcUnitsPhysicalProcessor'} = undef;
    }
    


    # create these metrics manually and store to DB. They are not present in the HMC API.
    #$data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'assignedProcUnitsPercent'} = defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'assignedProcUnits'} ? sprintf("%d", 100 * ( $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'} / $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'assignedProcUnits'} ) ) : undef;
    # NOTE: division to % is CPU_metric/configurable, not CPU_metric/configurable
    if ( defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'} ) {
        $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_percent'} = defined $serverProcessor->{'configurableProcUnits'}[0] ? sprintf("%d", 100 * ( $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'} / $serverProcessor->{'configurableProcUnits'}[0] ) ) : undef;
    }
    else {
        $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_percent'} = undef;
    }

    if ( defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} ) {
        $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_percent'} = defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} ? sprintf("%d", 100 * ( $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} / $serverProcessor->{'configurableProcUnits'}[0] ) ) : undef;
    }
    else {
        $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_percent'} = undef;
    }

    # Server Memory
    # convert MiB to bytes to be stored in DB.
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'systemFirmwareUtilAssignedMem'} = defined $systemFirmwareUtil->{'assignedMem'}[0] ? sprintf("%d", $systemFirmwareUtil->{'assignedMem'}[0] * $convertValues->{'assignedMem'}) : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'configurableMem'} = defined $serverMemory->{'configurableMem'}[0] ? sprintf("%d", $serverMemory->{'configurableMem'}[0] * $convertValues->{'configurableMem'}) : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'assignedMemToLpars'} = defined $serverMemory->{'assignedMemToLpars'}[0] ? sprintf("%d", $serverMemory->{'assignedMemToLpars'}[0] * $convertValues->{'assignedMemToLpars'}) : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'availableMem'} = defined $serverMemory->{'availableMem'}[0] ? sprintf("%d", $serverMemory->{'availableMem'}[0] * $convertValues->{'availableMem'}) : undef;
    $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'totalMem'} = defined $serverMemory->{'totalMem'}[0] ? sprintf("%d", $serverMemory->{'totalMem'}[0] * $convertValues->{'totalMem'}) : undef;

    # MAX 
    #$data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores_max'} = defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'} ? $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_cores'} : undef;
    #$data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_physical_percent_max'} = defined $serverProcessor->{'totalProcUnits'}[0] ? sprintf("%d", 100 * ( $serverProcessor->{'utilizedProcUnits'}[0] / $serverProcessor->{'totalProcUnits'}[0] ) ) : undef;
    #$data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'totalProcUnitsMax'} = defined $serverProcessor->{'totalProcUnits'}[0] ? $serverProcessor->{'totalProcUnits'}[0] : undef;
    #$data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores_max'} = defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} ? $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} : undef;
    #$data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_percent_max'} = defined $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} ? sprintf("%d", 100 * ( $data_out->{$hw_type}{$subsystem}{$server_uid}{$timeStamp}{'cpu_utilization_cores'} / $serverProcessor->{'totalProcUnits'}[0] ) ) : undef;
    
    $subsystem = 'hea';
    for (@{$serverNetwork->{'headapters'}}){
      my $adapter = $_;
      for (@{$adapter->{'physicalPorts'}}){
        my $port = $_;

        if (  !defined $port->{'physicalLocation'} ) {
            $self->{'logger'}->info("Undefined ID [$subsystem]: physicalLocation");
            next;
        }

        my $port_uid = $port->{'physicalLocation'};
        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'sentBytes'} = sprintf("%d", $port->{'sentBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'receivedBytes'} = sprintf("%d", $port->{'receivedBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'sentPackets'} = sprintf("%d", $port->{'sentPackets'}[0]);
        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'receivedPackets'} = sprintf("%d", $port->{'receivedPackets'}[0]);
        #$data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'id'} = $port->{'id'};
        #$data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'type'} = $port->{'type'};

        #architecture hea
        if (!defined $self->{'architecture_duplicates_check'}->{$port_uid}){
          my $label = $port_uid;
          my (@physLoc) = split('\.', $port->{'physicalLocation'});
          $label    = $physLoc[-1] ? "$label" : $label;
          my %port_arch = (
              'item_id'     => $port_uid,
              'label'       => $label,
              'hw_type'     => $hw_type,
              'subsystem'   => $subsystem,
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'}
          );
          push(@{$self->{'architecture'}->{$subsystem}}, \%port_arch);
          $self->{'architecture_duplicates_check'}->{$port_uid} = 1;
        }
      }
    }
    $subsystem = 'sriovphysical';
    for (@{$serverNetwork->{'sriovAdapters'}}){
      my $adapter = $_;
      for (@{$adapter->{'physicalPorts'}}){
        my $port = $_;

        if (  !defined $port->{'physicalLocation'} ) {
            $self->{'logger'}->info("Undefined ID [$subsystem]: physicalLocation");
            next;
        }
        my $port_uid = $port->{'physicalLocation'};

        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'sentBytes'} = sprintf("%d", $port->{'sentBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'receivedBytes'} = sprintf("%d", $port->{'receivedBytes'}[0]);
        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'sentPackets'} = sprintf("%d", $port->{'sentPackets'}[0]);
        $data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'receivedPackets'} = sprintf("%d", $port->{'receivedPackets'}[0]);
        #$data_out->{$hw_type}{$subsystem}{$port_uid}{$timeStamp}{'id'} = $port->{'id'};

        #architecture sriov physical
        if (!defined $self->{'architecture_duplicates_check'}->{$port_uid}){
          my %port_arch = (
              'item_id'     => $port_uid,
              'label'       => $port_uid,
              'hw_type'     => $hw_type,
              'subsystem'   => $subsystem,
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'}
          );
          push(@{$self->{'architecture'}->{$subsystem}}, \%port_arch);
          $self->{'architecture_duplicates_check'}->{$port_uid} = 1;
        }
      }
    }

    # Server Shared Processor Pool
    $subsystem = 'pool';
    for (@{$serverSharedProcessorPool}){ 
      my $spp = $_;
      my $PoolID  = $spp->{'id'};
      my $pool = $sharedProcessorPools->{$server_uid}{'SharedProcessorPool'}{$PoolID};
      my $PoolUID = defined $pool->{'uid'} ? $pool->{'uid'} : undef;
      my $PoolName = defined $pool->{'label'} ? $pool->{'label'} : undef;

      if (!defined $PoolUID){
        $self->{'logger'}->error("PoolUID not defined for Shared Processor Pool $PoolID on $server_uid. Skipping Shared Processor Pool $PoolID.");
        next;
      }

      #shared processor pool conf_out    
      #$data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'id'} = defined $spp->{'id'} ? $spp->{'id'} : undef;
      $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'configuredProcUnits'} = defined $spp->{'configuredProcUnits'}[0] ? $spp->{'configuredProcUnits'}[0] : undef;
      $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'borrowedProcUnits'} = defined $spp->{'borrowedProcUnits'}[0] ? $spp->{'borrowedProcUnits'}[0] : undef;
      $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'assignedProcUnits'} = defined $spp->{'assignedProcUnits'}[0] ? $spp->{'assignedProcUnits'}[0] : undef;
      $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'utilizedProcUnits'} = defined $spp->{'utilizedProcUnits'}[0] ? $spp->{'utilizedProcUnits'}[0] : undef;
      $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'availableProcUnits'} = defined $spp->{'availableProcUnits'}[0] ? $spp->{'availableProcUnits'}[0] : undef;
      $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'totalUtilizedProcUnitsPercent'} = (defined $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'utilizedProcUnits'} && $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'assignedProcUnits'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'utilizedProcUnits'} / $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'assignedProcUnits'})): undef;

      #MAX 
      #$data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'utilizedProcUnitsMax'} = defined $spp->{'utilizedProcUnits'}[0] ? $spp->{'utilizedProcUnits'}[0] : undef;
      #$data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'totalUtilizedProcUnitsPercentMax'} = (defined $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'utilizedProcUnits'} && $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'assignedProcUnits'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'utilizedProcUnits'} / $data_out->{$hw_type}{$subsystem}{$PoolUID}{$timeStamp}{'assignedProcUnits'})): undef;

      if (!defined $self->{'architecture_duplicates_check'}->{$PoolUID}){
        #architecture shared processor pool
        my $label = $PoolName ? $PoolName : $PoolUID;
        my %spp_arch = (
            'item_id'     => $PoolUID,
            'label'       => $label,
            'hw_type'     => $hw_type,
            'subsystem'   => $subsystem,
            'parents'     => [$server_uid],
            'hostcfg_id'  => $self->{'hostcfg_id'}
        );
        $self->{'architecture_duplicates_check'}->{$PoolUID} = 1;
        push(@{$self->{'architecture'}->{$subsystem}}, \%spp_arch);
      }
    }
    $subsystem = undef;
  }

  # Send data to backend - if there are too many samples, Xormon.pm will split them into multiple data bundles
  # $self->{'xormon'}->saveJSON2($data_out) if scalar keys %{$data_out} > 0;
  my %data;
  if (defined $processedJsonContent->{'systemUtil'}{'utilSamples'}->[-1]){
    #%data = ('LastFetchTimeServer' => str2time ($processedJsonContent->{'systemUtil'}{'utilSamples'}->[-1]->{'sampleInfo'}{'timeStamp'}));
    $self->{'microservice_data_into_db'}{'LastFetchTimeServer'} = str2time($processedJsonContent->{'systemUtil'}{'utilSamples'}->[-1]->{'sampleInfo'}{'timeStamp'})
  } else {
    $self->{'logger'}->error("No samples for processed metrics JSONs found for $server_uid");
    #%data = {};
    $self->{'microservice_data_into_db'}{'LastFetchTimeServer'} = undef;
  }
  #my $resp = $xormon->saveData("LastFetchTimeServer", $microservice_data_into_db);
  
  return 0;
}

sub LparPerformanceToData {
  my ($self, $data_out, $sample, $server_uid, $cpu_utilization_coresTotalLparsForThisServer) = @_;

  if (checkInputParams($self, $sample, $server_uid)){
    $self->{'logger'}->error("LparPerformanceToData Params error, $self, $sample, $server_uid");
    return {};
  }
  else {
    $self->{'logger'}->debug("LparPerformanceToData Params seem ok, $self, $sample, $server_uid");
  }

  my $link  = $sample->{'link'};
  my $sample2 = $self->getServerProcessedSample($link);
  my $processedJsonLink = $sample2->{'link'};
  my $processedJsonContent = $self->getJsonContent ($processedJsonLink);
  my $utilSamples = $processedJsonContent->{'systemUtil'}{'utilSamples'};

  my $subsystem = 'vm';

  if (!%{$processedJsonContent}){
    $self->{'logger'}->error("No LPARs processed metrics JSONs found for $server_uid");
  }

  #send data in bundles of at least 100 samples
  my $bundle_size = 100;

  my $data_out_index = 0;

  for (my $i=0; $i<$self->lpar_multiplicator(); $i++){
    for my $sample (@{$utilSamples}){

      my $timeStamp = str2time ( $sample->{'sampleInfo'}{'timeStamp'} );   

      #lpar performance data
      # convert MiB to bytes to be stored in DB.
      my $lparsUtil = $sample->{'lparsUtil'};

      for my $lpar (@{$lparsUtil}){
        #print Dumper $_;# if ($server_uid eq "c2a76391-12bc-37d2-85b0-130b2d0953c0");

        my $proc = $lpar->{'processor'};
        my $mem = $lpar->{'memory'};
        my $lpar_uid = $lpar->{'uuid'};
        my $lpar_label = $lpar->{'name'};

        my $save_name = "utilsSample-$server_uid-$lpar_label";
        if ( defined $self->{'OPTIONS'}{'SAVE_HMC_RESPONSE_SAMPLE'} && $self->{'OPTIONS'}{'SAVE_HMC_RESPONSE_SAMPLE'} && ! defined $self->{'SAMPLE_SAVED'}{$save_name} ) {
          $self->{'SAMPLE_SAVED'}{$save_name} = 1;
          $self->{'logger'}->info("Saving response sample $save_name");
          Xormon::keepLastResponse($utilSamples, "power", $self->{hostcfg_id}, "$save_name");
        }

        if (!defined $lpar_uid){
          $self->{'logger'}->error("lpar_uid not defined for $lpar_label on $server_uid.");
          next;
        }
        if ($i>0){
          $lpar_uid = "$lpar_uid$i";
          $lpar_label = "$lpar_label-copy-$i";
        }

        $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'maxVirtualProcessors'} = $proc->{'maxVirtualProcessors'}[0] ? sprintf("%d", $proc->{'maxVirtualProcessors'}[0]) : undef;
        $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} = $proc->{'currentVirtualProcessors'}[0] > 0 ? sprintf("%d", $proc->{'currentVirtualProcessors'}[0]) : sprintf("%d", $proc->{'entitledProcUnits'}[0]);
        $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'entitledProcUnits'} = defined $proc->{'entitledProcUnits'}[0] ? $proc->{'entitledProcUnits'}[0] : undef;
        $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'maxProcUnits'} = defined $proc->{'maxProcUnits'}[0] ? $proc->{'maxProcUnits'}[0] : undef;
        $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'logicalMem'} = defined $mem->{'logicalMem'}[0] ? sprintf("%d", $mem->{'logicalMem'}[0] * $convertValues->{'logicalMem'}) : undef;
        $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'backedPhysicalMem'} = defined $mem->{'backedPhysicalMem'}[0] ? sprintf("%d", $mem->{'backedPhysicalMem'}[0]* $convertValues->{'backedPhysicalMem'}) : undef;
        $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'virtualPersistentMem'} = defined $mem->{'virtualPersistentMem'}[0] ? sprintf("%d", $mem->{'virtualPersistentMem'}[0]* $convertValues->{'virtualPersistentMem'}) : undef;

        # 1. try: usage = capped + uncapped - idle
        #   - if usage < 0 (<=> unrealistic (?) idle)  => set usage = undef - That indicated some problem with returned idle
        #       - condition: dedicated LPAR + kvm + VM x86 win XP
        # 2. If response includes utilizedProcUnitsDeductIdle, use it as usage regardless of the previous step!
        #
        # 3. sum usage to server sum, if undefined usage, add 0
        #   - TODO: if all lpar usages are undef, then sum = undef
        if (  defined $proc->{'utilizedCappedProcUnits'}[0] &&
              defined $proc->{'utilizedUncappedProcUnits'}[0] &&
              defined $proc->{'idleProcUnits'}[0]) {

          # just for clarity
          my $cpu_util_capped   = $proc->{'utilizedCappedProcUnits'}[0];
          my $cpu_util_uncapped = $proc->{'utilizedUncappedProcUnits'}[0];
          my $cpu_idle          = $proc->{'idleProcUnits'}[0];

          my $cpu_physical = $cpu_util_capped + $cpu_util_uncapped;
          my $cpu_usage    = $cpu_util_capped + $cpu_util_uncapped - $cpu_idle;

          if ( $cpu_usage < 0 ) {
            $cpu_usage = undef;
          }

          $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores'}    = $cpu_physical;
          $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} = $cpu_usage;
        }
        else {
          #rest_api_log("PROBLEM: unexpected metrics");

          if ( defined $proc->{'utilizedProcUnits'}[0] && defined $proc->{'utilizedProcUnitsDeductIdle'}[0] ) {
            $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores'}    = $proc->{'utilizedProcUnits'}[0];
            $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} = $proc->{'utilizedProcUnitsDeductIdle'}[0];
          }
          elsif ( defined $proc->{'utilizedProcUnits'}[0] ) {
            $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores'}    = $proc->{'utilizedProcUnits'}[0];
            $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} = undef;
          }
          else {
            # NOT DONE!
            # TODO: all API possibilities
            #rest_api_log("PROBLEM: expected metrics NOT FOUND!");

            $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores'}    = undef;
            $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} = undef;
          }

        }

        # if API returns usage, use it
        if ( defined $proc->{'utilizedProcUnitsDeductIdle'}[0] ) {
          $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} = $proc->{'utilizedProcUnitsDeductIdle'}[0];
        }

        ##another quite a bit tricky part. Some lpars are reporting utilizedProcUnits as 100 % (1), these reports aleso utilizedProcUnitsDeductIdle (e.g. dedicated lpars). Use this as CPU if defined... I dont know.
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores'} = defined $proc->{'utilizedProcUnits'}[0] ? $proc->{'utilizedProcUnits'}[0] : undef;
        #
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} = defined $proc->{'utilizedProcUnitsDeductIdle'}[0] ? $proc->{'utilizedProcUnitsDeductIdle'}[0] : undef;
        
        $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'idleProcUnits'} = $proc->{'idleProcUnits'}[0];

        if ( defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} ) {
          $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_utilization_coresTotalLparsForThisServer'} += $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'};
          $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_utilization_cores'} += $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'};
        }
        else {
          $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_utilization_coresTotalLparsForThisServer'} += 0;
          $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_utilization_cores'} += 0;
        }
        
        $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'cpu_physical_cores'}    += $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores'};

        if (defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'idleProcUnits'} ){
          $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'idle'} += $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'idleProcUnits'};
        }
        else {
          $cpu_utilization_coresTotalLparsForThisServer->{$server_uid}{$timeStamp}{'idle'} += 0;
        }
        #print "add to $server_uid value of $lpar_label ($timeStamp) = $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'}\n";

        #create these metric for xormon
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'totalProcUnitsPercent'} = (defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} && $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores'} / $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'})) : undef;
        $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'entitledProcUnitsPercent'} = (defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'entitledProcUnits'} && $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'entitledProcUnits'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores'} / $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'entitledProcUnits'})) : undef;
        $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_percent'} = (defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} && $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores'} / $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'})) : undef;
        $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_percent'} = (defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} && defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} && $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} / $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'})) : undef;
      
        # MAX
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores_max'} = defined $proc->{'utilizedProcUnits'}[0] ? $proc->{'utilizedProcUnits'}[0] : undef;
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_percent_max'} = (defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} && $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores'} / $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'})) : undef;
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_percent_max'} = (defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} && defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} && $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} / $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'})) : undef;
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores_max'} = defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} ? $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_utilization_cores'} : undef;

        
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'totalProcUnitsPercentMax'} = (defined $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} && $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'} != 0) ? sprintf("%d", (100 * $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'cpu_physical_cores'} / $data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'currentVirtualProcessors'})) : undef;
        
        # Do not store there metrics for now. it can be useful in future
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'utilizedCappedProcUnits'} = $proc->{'utilizedCappedProcUnits'}[0];
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'utilizedUncappedProcUnits'} = $proc->{'utilizedUncappedProcUnits'}[0];
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'donatedProcUnits'} = $proc->{'donatedProcUnits'}[0];
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'timeSpentWaitingForDispatch'} = $proc->{'timeSpentWaitingForDispatch'}[0];
        #$data_out->{$hw_type}{$subsystem}{$lpar_uid}{$timeStamp}{'timePerInstructionExecution'} = $proc->{'timePerInstructionExecution'}[0];
      }
    }
  }
  # Send data to backend - if there are too many samples, Xormon.pm will split them into multiple data bundles
  
  #$self->{'xormon'}->saveJSON2($data_out) if scalar keys %{$data_out} > 0;
  my %data;
  if (defined $utilSamples->[-1]){
    #%data = ('LastFetchTimeLpar' => str2time($utilSamples->[-1]->{'sampleInfo'}{'timeStamp'}));
    $self->{'microservice_data_into_db'}->{'LastFetchTimeLpar'} = str2time($utilSamples->[-1]->{'sampleInfo'}{'timeStamp'})
  } else {
    $self->{'logger'}->error("No samples for xx JSONs found for $server_uid");
    #%data = {};
    $self->{'microservice_data_into_db'}->{'LastFetchTimeLpar'} = undef;
  }
  #my $resp = $xormon->saveData("LastFetchTimeLpar", $microservice_data_into_db);

  return 0;
}

sub getManagedSystemConfigurationDetail {
  my ($self, $conf_out, $configurationMetricsServer) = @_;

  if (checkInputParams($self, $conf_out, $configurationMetricsServer)){
    $self->{'logger'}->error("getManagedSystemConfigurationDetail Params error $self, $conf_out, $configurationMetricsServer");
    return {};
  } else {
    $self->{'logger'}->debug("getManagedSystemConfigurationDetail Params seem ok $self, $conf_out, $configurationMetricsServer");
  }

  my $server_uid = defined $configurationMetricsServer->{'Metadata'}{'Atom'}{'AtomID'} ? $configurationMetricsServer->{'Metadata'}{'Atom'}{'AtomID'} : undef;
  my $server_name = defined $configurationMetricsServer->{'SystemName'}{'content'} ? $configurationMetricsServer->{'SystemName'}{'content'} : undef;

  if (!defined $server_uid){
    $self->{'logger'}->error("error in getManagedSystemConfigurationDetail, not defined server uid");
    return 1;
  }

  my $different_units = {
    'CurrentMemory' => 'MiB',
    'MinimumMemory' => 'MiB',
    'MaximumMemory' => 'MiB',
    'CurrentAvailableSystemMemory' => 'MiB',
    'CurrentMinimumMemory' => 'MiB',
    'CurrentMaximumMemory' => 'MiB',
    'DesiredMemory' => 'MiB',
    'RuntimeMemory' => 'MiB',
    'RuntimeMinimumMemory' => 'MiB'
  };

  my $subsystem = 'server';
  for (keys %{ $configurationMetricsServer }){
    my $metric = $_;
    eval {
        if ($metric eq 'SystemTime'){
          $conf_out->{$subsystem}{$server_uid}{$metric} = $configurationMetricsServer->{$metric}{'content'} / 1000  if ( ref($configurationMetricsServer->{$metric}) eq 'HASH' && defined $configurationMetricsServer->{$metric}{'content'});
        } else {
          $conf_out->{$subsystem}{$server_uid}{$metric} = $configurationMetricsServer->{$metric}{'content'}  if ( ref($configurationMetricsServer->{$metric}) eq 'HASH' && defined $configurationMetricsServer->{$metric}{'content'});
        }
    };
  }

  my $details = [ 
    'AssociatedSystemProcessorConfiguration', 
    'AssociatedIPLConfiguration',
    'AssociatedIPLConfiguration' ,
    'AssociatedLogicalPartitions' ,
    'AssociatedReservedStorageDevicePool' ,
    'AssociatedSystemCapabilities' ,
    'AssociatedSystemIOConfiguration' ,
    'AssociatedSystemMemoryConfiguration' ,
    'AssociatedSystemProcessorConfiguration' ,
    'AssociatedSystemSecurity' ,
    'AssociatedVirtualIOServers' ,
    'EnergyManagementConfiguration' ,
    'MachineTypeModelAndSerialNumber' ,
    'MergedReferenceCode' ,
    'Metadata' ,
    'ReferenceCode' ,
    'ServicePartition' ,
    'SystemMigrationInformation', 
  ];

  # server details
  for ( @{ $details } ){
    my $detail = $_;
    if ($detail eq 'AssociatedSystemIOConfiguration'){
      #print "Detail: $detail\n";
      #print Dumper $configurationMetricsServer->{$detail};

      my $IOAdapters = $configurationMetricsServer->{$detail}{'IOAdapters'};
      my $IOBuses = $configurationMetricsServer->{$detail}{'IOBuses'};
      my $IOSlots = $configurationMetricsServer->{$detail}{'IOSlots'};

      for (@{to_array($IOAdapters->{'IOAdapterChoice'})}){
        my $adapter = $_->{'IOAdapter'};

        if ( ! defined $adapter->{'AdapterID'}{'content'}) {
            $self->{'logger'}->info("Undefined ID [ioadapter]: AdapterID");
            next;
        }
        my $adapter_uid = "$server_uid-$adapter->{'AdapterID'}{'content'}";

        my $adapter_name = $adapter->{'DeviceName'}{'content'};
        if (!defined $self->{'architecture_duplicates_check'}->{$adapter_uid}){
          my %arch = (
              'item_id'     => $adapter_uid,
              'label'       => $adapter_name,
              'hw_type'     => $hw_type,
              'subsystem'   => 'ioadapter',
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'},
          );
          push(@{$self->{'architecture'}{'ioadapter'}}, \%arch);
          $self->{'architecture_duplicates_check'}->{$adapter_uid} = 1;
        }
        for (keys %{$adapter}){
          eval {
            $conf_out->{'ioadapter'}{$adapter_uid}{$_} = $adapter->{$_}{'content'} if (defined $adapter->{$_}{'content'});
          }
        }
      }

      for (@{to_array($IOSlots->{'IOSlot'})}){
        my $slot = $_;

        if ( ! defined $slot->{'PCAdapterID'}{'content'} || ! defined $slot->{'RelatedIOAdapter'}{'IOAdapter'}{'AdapterID'}{'content'} ) {
            $self->{'logger'}->info("Undefined ID [ioslot]: PCAdapterID OR RelatedIOAdapter");
            next;
        }
        my $slot_uid = "$server_uid-$slot->{'PCAdapterID'}{'content'}-$slot->{'RelatedIOAdapter'}{'IOAdapter'}{'AdapterID'}{'content'}";

        my $slot_name = $slot->{'IOUnitPhysicalLocation'}{'content'};
        if (!defined $self->{'architecture_duplicates_check'}->{$slot_uid}){
          my %arch = (
              'item_id'     => $slot_uid,
              'label'       => $slot_name,
              'hw_type'     => $hw_type,
              'subsystem'   => 'ioslot',
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'},
          );
          push(@{$self->{'architecture'}{'ioslot'}}, \%arch);
          $self->{'architecture_duplicates_check'}->{$slot_uid} = 1;
        }
        for (keys %{$slot}){
          eval {
            $conf_out->{'ioslot'}{$slot_uid}{$_} = $slot->{$_}{'content'} if (defined $slot->{$_}{'content'});
          }
        }
      }

      for (@{ to_array($IOBuses->{'IOBus'}) }){
        my $bus = $_;

        if (!defined $bus->{'IOBusID'}{'content'} ) {
            $self->{'logger'}->info("Undefined ID [iobus]: IOBusID");
            next;
        }

        my $bus_uid = "$server_uid-$bus->{'IOBusID'}{'content'}";

        my $bus_name = $bus->{'BackplanePhysicalLocation'}{'content'};
        if (!defined $self->{'architecture_duplicates_check'}->{$bus_uid}){
          my %arch = (
              'item_id'     => $bus_uid,
              'label'       => $bus_name,
              'hw_type'     => $hw_type,
              'subsystem'   => 'iobus',
              'parents'     => [ $server_uid ],
              'hostcfg_id'  => $self->{'hostcfg_id'},
          );
          push(@{$self->{'architecture'}{'iobus'}}, \%arch);
          $self->{'architecture_duplicates_check'}->{$bus_uid} = 1;
        }
        for (keys %{$bus}){
          eval {
            $conf_out->{'iobus'}{$bus_uid}{$_} = $bus->{$_}{'content'} if (defined $bus->{$_}{'content'});
          }
        }
      }
    }
    for (keys %{ $configurationMetricsServer->{$detail} }){
        my $metric = $_;
        my $value = $configurationMetricsServer->{$detail}{$metric}{'content'} if (ref ($configurationMetricsServer->{$detail}{$metric}) eq 'HASH' && defined $configurationMetricsServer->{$detail}{$metric}{'content'});
        if (defined $convertValues->{$metric} ){
          $value *= $convertValues->{$metric};
        }
        $conf_out->{$subsystem}{$server_uid}{$metric} = $value if ( defined $value );
    }
  }

  # lpar details
  $self->{'logger'}->info("Fetch Configuration data LPAR ($server_name)");
  my $configurationMetricsLpars = $self->getLparsConfiguration($server_uid);
  my $lpars_array;
  if (defined $configurationMetricsLpars->{'id'}){
    my $lpar_uid = $configurationMetricsLpars->{'id'};
    my $lpar = $configurationMetricsLpars->{'content'}{'LogicalPartition:LogicalPartition'};
    $lpars_array->{$lpar_uid} = $lpar;

  } else {
    for (keys %{ $configurationMetricsLpars }){
      my $lpar_uid = $_;
      my $v = $configurationMetricsLpars->{$lpar_uid}{'content'}{'LogicalPartition:LogicalPartition'};
      $lpars_array->{$lpar_uid} = $v;
    }
  }

  #============================================================================================
  # LPAR
  #============================================================================================
  for (keys %{ $lpars_array }){

    my $lpar_uid = $_;


    my $lpar = $lpars_array->{$lpar_uid};
    
    #print "\n\n------------------------------\nlpar\n";

    #print Dumper $lpar;

    #health status lpar
    my $partitionState = $lpar->{'PartitionState'}{'content'} ? $lpar->{'PartitionState'}{'content'} : 'unknown';
    my $health_status;

    if ($partitionState eq 'running' || $partitionState eq 'not activated'){
        $health_status = 'ok';
    } else {
        $health_status = 'warning';
    }

    my %lpar_status = (
        'item_id' => $lpar_uid,
        'status' => $health_status,
        'updated' => "".localtime()."",
        'data' => $partitionState
    );
    push @{$self->{'status'}}, \%lpar_status;
    
    for (keys %{ $lpar }){
        my $metric = $_;
        eval {
          if (defined $different_units->{$metric}){
            $conf_out->{'vm'}{$lpar_uid}{$metric} = Xormon::returnBytes($lpar->{$metric}{'content'}, $different_units->{$metric}) if ( ref($lpar->{$metric}) eq 'HASH' && defined $lpar->{$metric}{'content'});
          }
          else {
            $conf_out->{'vm'}{$lpar_uid}{$metric} = $lpar->{$metric}{'content'}  if ( ref($lpar->{$metric}) eq 'HASH' && defined $lpar->{$metric}{'content'});
          }
        };
    }

    if ( defined $conf_out->{'vm'}{$lpar_uid}{'OperatingSystemVersion'} ) {
        # Reason: shorten
        # IBM i Licensed Internal Code 7.2.0 410 9
        # to
        # IBM i 7.2.0 410 9
        $conf_out->{'vm'}{$lpar_uid}{'OperatingSystemVersion'} =~ s/Licensed Internal Code\s*//;
    }
  
    my $details = [ 
      'AssociatedManagedSystem',
      'AssociatedPartitionProfile',
      'ClientNetworkAdapters',
      'HostEthernetAdapterLogicalPorts',
      'Metadata',
      'PartitionCapabilities',
      'PartitionIOConfiguration',
      'PartitionMemoryConfiguration',
      'PartitionProcessorConfiguration',
      'PartitionProfiles',
      'PrimaryPagingServicePartition',
      'ProcessorPool',
      'ReferenceCode',
      'VirtualSCSIClientAdapters',
    ];
    my $details2 = [
      'CurrentSharedProcessorConfiguration',
      'SharedProcessorConfiguration',
      'DedicatedProcessorConfiguration',
      'CurrentDedicatedProcessorConfiguration',
      'TaggedIO',
      'CurrentPagingServicePartition',
      'PrimaryPagingServicePartition',

    ];
    for ( @{ $details } ){
      my $detail = $_;
      for (keys %{ $lpar->{$detail} }){
        my $metric = $_;
        my $value = $lpar->{$detail}{$metric}{'content'} if (ref ($lpar->{$detail}{$metric}) eq 'HASH' && defined $lpar->{$detail}{$metric}{'content'});
        if (defined $different_units->{$metric}){
          $conf_out->{'vm'}{$lpar_uid}{$metric} = Xormon::returnBytes($value, $different_units->{$metric})  if (defined $value );
        } else {
          $conf_out->{'vm'}{$lpar_uid}{$metric} = $value if (defined $value );
        }
        for ( @{ $details2 } ){
          my $detail2 = $_;
          for (keys %{ $lpar->{$detail}{$detail2} }){
            my $metric2 = $_;
            my $value2 = $lpar->{$detail}{$detail2}{$metric2}{'content'} if (ref ($lpar->{$detail}{$detail2}{$metric2}) eq 'HASH' && defined $lpar->{$detail}{$detail2}{$metric2}{'content'});
            if (defined $different_units->{$metric2}){
              $conf_out->{'vm'}{$lpar_uid}{$metric2} = Xormon::returnBytes($value2, $different_units->{$metric2})  if (defined $value2 );
            } else {
              $conf_out->{'vm'}{$lpar_uid}{$metric2} = $value2 if (defined $value2 );
            }
          }
        }
      }
    }
  }

  #============================================================================================
  # VIOS
  #============================================================================================
  # vios details (vioses, adapter, storage, target, trunk, backing sea, physical volumes, )
  $self->{'logger'}->info("Fetch Configuration data VIOS ($server_name)");
  
  my $configurationMetricsVioses = $self->getViosesConfiguration($server_uid);
  my $vioses_array;
  if (defined $configurationMetricsVioses->{'id'}){
    my $vios_uid = $configurationMetricsVioses->{'id'};
    my $vios = $configurationMetricsVioses->{'content'}{'VirtualIOServer:VirtualIOServer'};
    $vioses_array->{$vios_uid} = $vios;

  } else {
    for (keys %{ $configurationMetricsVioses }){
      my $vios_uid = $_;
      my $vios = $configurationMetricsVioses->{$vios_uid}{'content'}{'VirtualIOServer:VirtualIOServer'};
      $vioses_array->{$vios_uid} = $vios;
    }
  }

  for (keys %{ $vioses_array }){
    my $vios_uid = $_;
    my $vios = $vioses_array->{$vios_uid};

    # VIOS: HEALTH STATUS
    my $health_status;
    my $partitionState = $vios->{'PartitionState'}{'content'} ? $vios->{'PartitionState'}{'content'} : 'unknown';

    if ($partitionState eq 'running' || $partitionState eq 'not activated'){
        $health_status = 'ok';
    } else {
        $health_status = 'warning';
    }

    my %vios_status = (
        'item_id' => $vios_uid,
        'status' => $health_status,
        'updated' => "".localtime()."",
        'data' => $partitionState
    );
    push @{$self->{'status'}}, \%vios_status;

    # FREELINK AGGREGATION
    my $subsystem = 'freelinkaggregation';

    my $ioadapterchoice = to_array($vios->{'FreeIOAdaptersForLinkAggregation'}{'IOAdapterChoice'});

    for (@{$ioadapterchoice}){
      my $adapter = $_->{'IOAdapter'};

      if (!defined $adapter->{'UniqueDeviceID'}{'content'}){
        $self->{'logger'}->info("Undefined ID [freelinkaggregation]: UniqueDeviceID");
        next;
      }
      my $adapter_uid = $adapter->{'UniqueDeviceID'}{'content'};

      my $adapter_name = $adapter->{'DeviceName'}{'content'};
      if (!defined $self->{'architecture_duplicates_check'}->{$adapter_uid}){
        my %arch = (
            'item_id'     => $adapter_uid,
            'label'       => $adapter_name,
            'hw_type'     => $hw_type,
            'subsystem'   => $subsystem,
            'parents'     => [ $server_uid ],
            'hostcfg_id'  => $self->{'hostcfg_id'},
        );
        push(@{$self->{'architecture'}{$subsystem}}, \%arch);
        $self->{'architecture_duplicates_check'}->{$adapter_uid} = 1;
      }

      for (keys %{$adapter}){
        eval {
          $conf_out->{$subsystem}{$adapter_uid}{$_} = $adapter->{$_}{'content'} if (defined $adapter->{$_}{'content'});
        }
      }
    }

    # Fibre Channel Mapping, do not use now, needs to be tested first
    my $fc_mappings = to_array($vios->{'VirtualFibreChannelMappings'}{'VirtualFibreChannelMapping'});

    #====================================================================================================
    # NPIV
    # VIRTUAL FIBRE 
    #====================================================================================================
    $subsystem = 'virtualfibrenpiv';

    for (@{ $fc_mappings }){
      my $mapping = $_;

      #print "\n\n------------------------------\nvirtualfibrenpiv\n";
      #print Dumper $mapping;

      my $server = $mapping->{'ServerAdapter'};
      my $client = $mapping->{'ClientAdapter'};
      my $associated_lpar = $mapping->{'AssociatedLogicalPartition'};
      my $port = $mapping->{'Port'};

      my $fcs_alias_server = $server->{'MapPort'}{'content'};

      if (!defined $server->{'MapPort'}{'content'}){
        $self->{'logger'}->info("Undefined ID [virtualfibrenpiv]: MapPort");
        next;
      }
      my $server_adapter_uid = $server->{'LocationCode'}{'content'};

      my $npiv_uid = "$server_adapter_uid-$fcs_alias_server";

      my $npiv_label = $vios->{PartitionName}{content} || "";
      $npiv_label .= " $fcs_alias_server";

      #print "NPIV LABEL: $npiv_label\n";
      if (!defined $self->{'architecture_duplicates_check'}->{$npiv_uid}){
        my %arch = (
            'item_id'     => $npiv_uid,
            'label'       => $npiv_label,
            'hw_type'     => $hw_type,
            'subsystem'   => $subsystem,
            'parents'     => [ $server_uid ],
            'hostcfg_id'  => $self->{'hostcfg_id'},

            'wwn' => $port->{'WWPN'}{'content'},

        );
        push(@{$self->{'architecture'}{$subsystem}}, \%arch);
        $self->{'architecture_duplicates_check'}->{$npiv_uid} = 1;
      }

      $conf_out->{$subsystem}{$npiv_uid}{'viosName'} = $vios->{PartitionName}{content} || "";

      #server adapter
      for (keys %{$server}){
        my $metric = $_;
        eval{
          $conf_out->{$subsystem}{$npiv_uid}{$metric} = $server->{$metric}{'content'} if (defined $server->{$metric}{'content'});
        }
      }

      #client adapter
     # my $client_adapter_uid = $client->{'LocationCode'}{'content'};
     
     for (keys %{$client}){
       my $metric = $_;
       eval{
          if (defined $conf_out->{$subsystem}{$npiv_uid}{$metric}){
            #print "pro $npiv_uid tohle uz mam: $metric \n";
          } else {
            $conf_out->{$subsystem}{$npiv_uid}{$metric} = $client->{$metric}{'content'} if (defined $client->{$metric}{'content'});
          }
       }
     }

      #port
     my $port_uid = $port->{'UniqueDeviceID'}{'content'};
     for (keys %{$port}){
       my $metric = $_;
       eval{
         if (defined $conf_out->{$subsystem}{$npiv_uid}{$metric}){
            #print "pro $npiv_uid tohle uz mam: $metric \n";
          } else {
            #$conf_out->{$subsystem}{$npiv_uid}{$metric} = $client->{$metric}{'content'} if (defined $client->{$metric}{'content'});
            $conf_out->{$subsystem}{$npiv_uid}{$metric} = $port->{$metric}{'content'} if (defined $port->{$metric}{'content'});
          }
       }
     }

      if (defined $associated_lpar->{'href'}){
        my $associated_lpar_uid = $associated_lpar->{'href'};
        $associated_lpar_uid =~ s/^.*\///g;
        $conf_out->{$subsystem}{$npiv_uid}{'PartitionName'} = $conf_out->{'vm'}{$associated_lpar_uid}{'PartitionName'};
        $conf_out->{$subsystem}{$npiv_uid}{'AssociatedLogicalPartition'} = $associated_lpar_uid;
      }
    }

    $subsystem = 'genericvirtual';
    my $mappings = to_array($vios->{'VirtualSCSIMappings'}{'VirtualSCSIMapping'});


    for (@{ $mappings }){
      my $mapping = $_;
      my $server = $mapping->{'ServerAdapter'};
      my $client = $mapping->{'ClientAdapter'};
      my $target = $mapping->{'TargetDevice'};
      my $storage = $mapping->{'Storage'};
      my $associated_lpar = $mapping->{'AssociatedLogicalPartition'};


      #server adapter
      my $server_adapter_uid = $server->{'LocationCode'}{'content'};
      if (defined $associated_lpar->{'href'}){
        my $associated_lpar_uid = $associated_lpar->{'href'};
        $associated_lpar_uid =~ s/^.*\///g;
        $conf_out->{$subsystem}{$server_adapter_uid}{'PartitionName'} = $conf_out->{'vm'}{$associated_lpar_uid}{'PartitionName'};
        $conf_out->{$subsystem}{$server_adapter_uid}{'AssociatedLogicalPartition'} = $associated_lpar_uid;
      }
      for (keys %{$server}){
        my $metric = $_;
        eval{
          $conf_out->{$subsystem}{$server_adapter_uid}{$metric} = $server->{$metric}{'content'} if (defined $server->{$metric}{'content'});
        }
      }

      #client adapter
      my $client_adapter_uid = $client->{'LocationCode'}{'content'};
      for (keys %{$client}){
        my $metric = $_;
        eval{
          $conf_out->{$subsystem}{$server_adapter_uid}{$metric} = $client->{$metric}{'content'} if (defined $client->{$metric}{'content'});
        }
      }

      #storage
      for (keys %{$storage}){
        my $type = $_;
        eval{
          my $storage_uid = $storage->{$type}{'UniqueDeviceID'}{'content'};

          for (keys %{$storage->{$type}}){
            if (!defined $storage_uid) { next; }
            my $metric = defined $_ ? $_ : 'undefined';
            if ( $metric eq 'DiskCapacity'){
              $conf_out->{$subsystem}{$server_adapter_uid}{$metric} = Xormon::returnBytes($storage->{$type}{$metric}{'content'}, 'GiB') if (defined $storage->{$type}{$metric}{'content'});
            }
            else {
              $conf_out->{$subsystem}{$server_adapter_uid}{$metric} = $storage->{$type}{$metric}{'content'} if (defined $storage->{$type}{$metric}{'content'});
            }
          }
        }
      }

      #target
      for (keys %{$target}){
        my $type = $_;
        eval{
          my $target_uid = $target->{$type}{'UniqueDeviceID'}{'content'};
          for (keys %{$target->{$type}}){
            if (!defined $target_uid) { next; }
            my $metric = $_;
            $conf_out->{$subsystem}{$server_adapter_uid}{$metric} = $target->{$type}{$metric}{'content'} if (defined $target->{$type}{$metric}{'content'});
          }
        }
      }
      if (defined $associated_lpar->{'href'}){
        my $associated_lpar_uid = $associated_lpar->{'href'};
        $associated_lpar_uid =~ s/^.*\///g;
        $conf_out->{$subsystem}{$server_adapter_uid}{'PartitionName'} = $conf_out->{'vm'}{$associated_lpar_uid}{'PartitionName'};
        $conf_out->{$subsystem}{$server_adapter_uid}{'AssociatedLogicalPartition'} = $associated_lpar_uid;
      }
    }

    #my $trunks = ref($vios->{'TrunkAdapters'}{'TrunkAdapter'}) eq "ARRAY" ? $vios->{'TrunkAdapters'}{'TrunkAdapter'} : [ $vios->{'TrunkAdapters'}{'TrunkAdapter'} ] ;
    my $trunks = to_array($vios->{'TrunkAdapters'}{'TrunkAdapter'});

    $subsystem = 'trunk';

    for (@{ $trunks }){
      my $trunk = $_;

      if ( !defined $trunk->{'LocationCode'}{'content'} ) {
        $self->{'logger'}->info("Undefined ID [$subsystem]: LocationCode");
        next;
      }
      my $trunk_uid = $trunk->{'LocationCode'}{'content'};

      #architecture trunk adapter
      if (!defined $self->{'architecture_duplicates_check'}->{$trunk_uid}){
        my %arch = (
            'item_id'     => $trunk_uid,
            'label'       => $trunk_uid,
            'hw_type'     => $hw_type,
            'subsystem'   => $subsystem,
            'parents'     => [ $server_uid ],
            'hostcfg_id'  => $self->{'hostcfg_id'},
        );
        push(@{$self->{'architecture'}{$subsystem}}, \%arch);
        $self->{'architecture_duplicates_check'}->{$trunk_uid} = 1;
      }
      for (keys %{ $trunk }){
        my $metric = $_;
        eval {
          if ($metric eq "LocalPartitionID"){
            for (keys %{ $vioses_array }){
              my $lpar_uid = $_;
              my $lpar = $vioses_array->{$lpar_uid};
              my $lpar_name = $lpar->{'PartitionName'}{'content'};
              my $lpar_id = $lpar->{'PartitionID'}{'content'};
              if ($trunk->{$metric}{'content'} == $lpar_id){
                $conf_out->{$subsystem}{$trunk_uid}{LocalPartitionLabel} = $lpar_name ? $lpar_name : undef;
                last;
              }
            }
          } else {
            $conf_out->{$subsystem}{$trunk_uid}{$metric} = $trunk->{$metric}{'content'} if ( ref($trunk->{$metric}) eq 'HASH' && defined $trunk->{$metric}{'content'});
          }

        };
      }
    }

    $subsystem = 'backingsea';
    my $FreeEthenetBackingDevicesForSEA = to_array($vios->{'FreeEthenetBackingDevicesForSEA'}{'IOAdapterChoice'});
    for (@{ $FreeEthenetBackingDevicesForSEA }){
      my $IOAdapter = $_;
      my $EthernetBackingDevice = $IOAdapter->{'EthernetBackingDevice'};

      if ( !defined $EthernetBackingDevice->{'PhysicalLocation'}{'content'} ){
        $self->{'logger'}->info("Undefined ID [$subsystem]: PhysicalLocation");
        next;
      }
      my $device_uid = $EthernetBackingDevice->{'PhysicalLocation'}{'content'};

      #architecture backing for sea
      if (!defined $self->{'architecture_duplicates_check'}->{$device_uid}){
        my %arch = (
            'item_id'     => $device_uid,
            'label'       => $EthernetBackingDevice->{'DeviceName'}{'content'},
            'hw_type'     => $hw_type,
            'subsystem'   => $subsystem,
            'parents'     => [ $server_uid ],
            'hostcfg_id'  => $self->{'hostcfg_id'},
        );
        push(@{$self->{'architecture'}{$subsystem}}, \%arch);
        $self->{'architecture_duplicates_check'}->{$device_uid} = 1;
      }
      for (keys %{ $EthernetBackingDevice }){
        my $metric = $_;
        eval {
            $conf_out->{$subsystem}{$device_uid}{$metric} = $EthernetBackingDevice->{$metric}{'content'} if ( ref($EthernetBackingDevice->{$metric}) eq 'HASH' && defined $EthernetBackingDevice->{$metric}{'content'});
        };
        for (keys %{ $EthernetBackingDevice->{'IPInterface'} }){
          my $metric2  = $_;
          eval {
            $conf_out->{$subsystem}{$device_uid}{$metric2} = $EthernetBackingDevice->{'IPInterface'}{$metric2}{'content'} if ( ref($EthernetBackingDevice->{'IPInterface'}{$metric2}) eq 'HASH' && defined $EthernetBackingDevice->{'IPInterface'}{$metric2}{'content'});
          };
        }
      }
    }

    my $PhysicalVolumes = to_array($vios->{'PhysicalVolumes'}{'PhysicalVolume'});

    $subsystem = 'physicalvolume';
    for (@{ $PhysicalVolumes }){
      my $PhysicalVolume = $_;

      if ( !defined $PhysicalVolume->{'VolumeUniqueID'}{'content'} ) {
        $self->{'logger'}->info("Undefined ID [$subsystem]: VolumeUniqueID");
        next;
      }
      my $device_uid = $PhysicalVolume->{'VolumeUniqueID'}{'content'};

      #architecture physical volumes
      if (!defined $self->{'architecture_duplicates_check'}->{$device_uid}){
        my %arch = (
            'item_id'     => $device_uid,
            'label'       => $PhysicalVolume->{'VolumeName'}{'content'},
            'hw_type'     => $hw_type,
            'subsystem'   => $subsystem,
            'parents'     => [ $server_uid ],
            'hostcfg_id'  => $self->{'hostcfg_id'},
        );
        push(@{$self->{'architecture'}{$subsystem}}, \%arch);
        $self->{'architecture_duplicates_check'}->{$device_uid} = 1;
      }
      for (keys %{ $PhysicalVolume }){
        my $metric = $_;
        eval {

            if ( $metric eq 'VolumeCapacity'){
              $conf_out->{$subsystem}{$device_uid}{$metric} = Xormon::returnBytes($PhysicalVolume->{$metric}{'content'}, 'MiB') if (defined $PhysicalVolume->{$metric}{'content'});
            } else {
              $conf_out->{$subsystem}{$device_uid}{$metric} = $PhysicalVolume->{$metric}{'content'} if ( ref($PhysicalVolume->{$metric}) eq 'HASH' && defined $PhysicalVolume->{$metric}{'content'});
            }
        };
      }
    }

    my $ProfileIOSlots = to_array($vios->{'PartitionIOConfiguration'}{'ProfileIOSlots'}{'ProfileIOSlot'});
    $subsystem = 'profileioslot';
    for (@{ $ProfileIOSlots }){
      my $slot = $_->{'AssociatedIOSlot'};

      $subsystem = 'profileioslot';

      if ( !defined $slot->{'SlotDynamicReconfigurationConnectorName'}{'content'} ) {
        $self->{'logger'}->info("Undefined ID [$subsystem]: SlotDynamicReconfigurationConnectorName");
        next;
      }
      if ( !defined $slot->{'PCAdapterID'}{'content'} ) {
        $self->{'logger'}->info("Undefined ID [$subsystem]: PCAdapterID");
        next;
      }
      if ( !defined $slot->{'SlotDynamicReconfigurationConnectorIndex'}{'content'} ) {
        $self->{'logger'}->info("Undefined ID [$subsystem]: SlotDynamicReconfigurationConnectorIndex");
        next;
      }

      my $slot_uid = "$slot->{'SlotDynamicReconfigurationConnectorName'}{'content'}-$slot->{'PCAdapterID'}{'content'}-$slot->{'SlotDynamicReconfigurationConnectorIndex'}{'content'}";


      #architecture profileioslot
      if (!defined $self->{'architecture_duplicates_check'}->{$slot_uid}){
        my %arch = (
            'item_id'     => $slot_uid,
            'label'       => $slot->{'IOUnitPhysicalLocation'}{'content'},
            'hw_type'     => $hw_type,
            'subsystem'   => $subsystem,
            'parents'     => [ $server_uid ],
            'hostcfg_id'  => $self->{'hostcfg_id'},
        );
        push(@{$self->{'architecture'}{$subsystem}}, \%arch);
        $self->{'architecture_duplicates_check'}->{$slot_uid} = 1;
      }

      for (keys %{ $slot }){
        my $metric = $_;
        eval {
              $conf_out->{$subsystem}{$slot_uid}{$metric} = $slot->{$metric}{'content'} if ( ref($slot->{$metric}) eq 'HASH' && defined $slot->{$metric}{'content'});
        };
      }

    }
      
    for (keys %{ $vios }){
        my $metric = $_;
        eval {
            $conf_out->{'vm'}{$vios_uid}{$metric} = $vios->{$metric}{'content'} if ( ref($vios->{$metric}) eq 'HASH' && defined $vios->{$metric}{'content'});
        };
    }

    if ( defined $conf_out->{'vm'}{$vios_uid}{'OperatingSystemVersion'} ) {
        # (VIOS might not have this)
        # Reason: shorten
        # IBM i Licensed Internal Code 7.2.0 410 9
        # to
        # IBM i 7.2.0 410 9
        $conf_out->{'vm'}{$vios_uid}{'OperatingSystemVersion'} =~ s/Licensed Internal Code\s*//;
    }

    my $details = [ 
      'AssociatedManagedSystem',
      'AssociatedPartitionProfile',
      'ClientNetworkAdapters',
      'HostEthernetAdapterLogicalPorts',
      'Metadata',
      'PartitionCapabilities',
      'PartitionIOConfiguration',
      'PartitionMemoryConfiguration',
      'PartitionProcessorConfiguration',
      'PartitionProfiles',
      'PrimaryPagingServicePartition',
      'ProcessorPool',
      'ReferenceCode',
      'VirtualSCSIClientAdapters',
    ];

    my $details2 = [
      'CurrentSharedProcessorConfiguration',
      'SharedProcessorConfiguration',
      'DedicatedProcessorConfiguration',
      'CurrentDedicatedProcessorConfiguration',
      'TaggedIO',
      'CurrentPagingServicePartition',
      'PrimaryPagingServicePartition',
    ];

    for ( @{ $details } ){
      my $detail = $_;
      for (keys %{ $vios->{$detail} }){
        my $metric = $_;
        my $value = $vios->{$detail}{$metric}{'content'} if (ref ($vios->{$detail}{$metric}) eq 'HASH' && defined $vios->{$detail}{$metric}{'content'});
        if (defined $different_units->{$metric}){
          $conf_out->{'vm'}{$vios_uid}{$metric} = Xormon::returnBytes($value, $different_units->{$metric})  if (defined $value );
        } else {
          $conf_out->{'vm'}{$vios_uid}{$metric} = $value if (defined $value );
        }
        for ( @{ $details2 } ){
          my $detail2 = $_;
          for (keys %{ $vios->{$detail}{$detail2} }){
            my $metric2 = $_;
            my $value2 = $vios->{$detail}{$detail2}{$metric2}{'content'} if (ref ($vios->{$detail}{$detail2}{$metric2}) eq 'HASH' && defined $vios->{$detail}{$detail2}{$metric2}{'content'});
            if (defined $different_units->{$metric2}){
              $conf_out->{'vm'}{$vios_uid}{$metric2} = Xormon::returnBytes($value2, $different_units->{$metric2}) if (defined $value2 );
            } else {
              $conf_out->{'vm'}{$vios_uid}{$metric2} = $value2 if (defined $value2 );
            }

          }
        }
      }
    }
  }
  return 0;
}

sub getServerSharedMemoryPool {
  my ($self, $uid) = @_;
  if (checkInputParams($self, $uid)){
    $self->{'logger'}->error("getServerSharedMemoryPool Params error  $self, $uid");
    return {};
  } else {
    $self->{'logger'}->debug("getServerSharedMemoryPool Params seem ok $self, $uid");
  }
  my $url = "/rest/api/uom/ManagedSystem/$uid/SharedMemoryPool";
  my $resp = $self->apiRequest('GET', 'application/xml', $url);

  if (!defined $resp->{'entry'}{'content'}{'SharedMemoryPool:SharedMemoryPool'}){
    #$self->{'logger'}->error("Shared Memory Pool not found for $uid");
    #$self->{'logger'}->error($url);
    return [];
  }

  my $smp = $resp->{'entry'}{'content'}{'SharedMemoryPool:SharedMemoryPool'};
  my $arr_out = [];
  if (ref ($smp) eq 'HASH' ){
    push @{ $arr_out }, $smp;
  } elsif (ref ($smp) eq 'HASH' ){
    $arr_out = $smp;
  }
  return $arr_out ? $arr_out : [];
}

sub getServerSharedProcessorPool {
  my ($self, $uid) = @_;
  if (checkInputParams($self, $uid)){
    $self->{'logger'}->error("getServerSharedProcessorPool Params error $self, $uid");
    return [];
  } else {
    $self->{'logger'}->debug("getServerSharedProcessorPool Params seem ok $self, $uid");
  }
  my $url = "/rest/api/uom/ManagedSystem/$uid/SharedProcessorPool";
  my $resp = $self->apiRequest('GET', 'application/xml', $url);

  my $out = [];
  for (keys %{ $resp->{'entry'} }){
    my $PoolUID = $_;
    my $pool = $resp->{'entry'}{$PoolUID};
    my $pool_det = $pool->{'content'}{'SharedProcessorPool:SharedProcessorPool'};
    push @{$out}, $pool_det;
  }
  return $out ? $out : [];
}

sub getServerSharedProcessorPoolDetail {
  my ($self, $uid, $pool_uid) = @_;
  if (checkInputParams($self, $uid, $pool_uid)){
    $self->{'logger'}->error("getServerSharedProcessorPoolDetail Params error $self, $uid, $pool_uid");
    return {};
  } else {
    $self->{'logger'}->debug("getServerSharedProcessorPoolDetail Params seem ok $self, $uid, $pool_uid");
  }
  my $url = "/rest/api/uom/ManagedSystem/$uid/SharedProcessorPool/$pool_uid";
  my $resp = $self->apiRequest('GET', 'application/xml', $url);
  my $detail = $resp->{'content'}{'SharedProcessorPool:SharedProcessorPool'};
  return $detail ? $detail : {};
}

sub getHmcs {
  my ($self) = @_;
  if (checkInputParams($self)){
    $self->{'logger'}->error("getHmcs Params error $self");
    return [];
  } else {
    $self->{'logger'}->debug("getHmcs Params seem ok $self");
  }
  my $url = "/rest/api/uom/ManagementConsole";
  my $resp = $self->apiRequest('GET', 'application/xml', $url);
  my $hmcs = $resp->{'entry'}{'content'}{'ManagementConsole:ManagementConsole'} ? $resp->{'entry'}{'content'}{'ManagementConsole:ManagementConsole'} : [];
  if (ref($hmcs) eq 'HASH'){
    $hmcs = [ $hmcs ];
  }
  return $hmcs ? $hmcs : [];
}

sub getSSPs {
  my ($self) = @_;
  if (checkInputParams($self)){
    $self->{'logger'}->error("getServers Params error $self");
    return [];
  } else {
    $self->{'logger'}->debug("getServers Params seem ok $self");
  }
  my $url = "/rest/api/pcm/preferences/SSP";
  my $resp = $self->apiRequest('GET', 'application/xml', $url);
  my $out = [];

  return $resp;

  my $servers = $resp->{'entry'}{'content'}{'ManagementConsolePcmPreference:ManagementConsolePcmPreference'}{'ManagedSystemPcmPreference'} ? $resp->{'entry'}{'content'}{'ManagementConsolePcmPreference:ManagementConsolePcmPreference'}{'ManagedSystemPcmPreference'} : [];
  
  if (ref($servers) eq 'HASH'){
    $servers = [ $servers ];
  }
  if (scalar @{$servers} <= 0){
    $self->{'logger'}->error ("Couldn't get any ManagedSystems from rest/api/pcm/preferences at ".__FILE__.":".__LINE__);
    return [];
  }

  # To eliminate getting any information from powered off managed systems, check its status and if it is not operating, skip it and do not include in the response
  for (@{$servers}){
    my $managed_system_link = $_->{'AssociatedManagedSystem'}{'href'};
    my $resp = $self->apiRequest('GET', 'application/xml', $managed_system_link);
    my $server_state = $resp->{'content'}{'ManagedSystem:ManagedSystem'}{'State'}{'content'} ? $resp->{'content'}{'ManagedSystem:ManagedSystem'}{'State'}{'content'} : 'unknown';
    push @{$out}, $_;
    $self->{'logger'}->info("$_->{'SystemName'}{'content'} is in $server_state state");

  }
  return $out ? $out : [];
}

sub getServers {
  my ($self) = @_;
  if (checkInputParams($self)){
    $self->{'logger'}->error("getServers Params error $self");
    return [];
  } else {
    $self->{'logger'}->debug("getServers Params seem ok $self");
  }
  my $url = "/rest/api/pcm/preferences";
  my $resp = $self->apiRequest('GET', 'application/xml', $url);
  my $out = [];
  my $servers = $resp->{'entry'}{'content'}{'ManagementConsolePcmPreference:ManagementConsolePcmPreference'}{'ManagedSystemPcmPreference'} ? $resp->{'entry'}{'content'}{'ManagementConsolePcmPreference:ManagementConsolePcmPreference'}{'ManagedSystemPcmPreference'} : [];
  
  if (ref($servers) eq 'HASH'){
    $servers = [ $servers ];
  }
  if (scalar @{$servers} <= 0){
    $self->{'logger'}->error ("Couldn't get any ManagedSystems from rest/api/pcm/preferences at ".__FILE__.":".__LINE__);
    return [];
  }

  # To eliminate getting any information from powered off managed systems, check its status and if it is not operating, skip it and do not include in the response
  for (@{$servers}){
    my $managed_system_link = $_->{'AssociatedManagedSystem'}{'href'};
    my $resp = $self->apiRequest('GET', 'application/xml', $managed_system_link);
    my $server_state = $resp->{'content'}{'ManagedSystem:ManagedSystem'}{'State'}{'content'} ? $resp->{'content'}{'ManagedSystem:ManagedSystem'}{'State'}{'content'} : 'unknown';
    push @{$out}, $_;
    $self->{'logger'}->info("$_->{'SystemName'}{'content'} is in $server_state state");

  }
  return $out ? $out : [];
}


sub getLpars {
  my ($self) = @_;

  if (checkInputParams($self)){
    $self->{'logger'}->error("getLpars Params error $self");
    return [];
  } else {
    $self->{'logger'}->debug("getLpars Params seem ok $self");
  }

  my $out = [];
  my $url = "/rest/api/uom/LogicalPartition";

  my $resp = $self->apiRequest('GET', 'application/xml', $url);

  # If there is only one lpar
  if (defined $resp->{'entry'}{'id'}){
    my $lpar = $resp->{'entry'}{'content'}{'LogicalPartition:LogicalPartition'};
    $out = [ $lpar ];
  } else { # If there are at least two lpars
    for (keys %{$resp->{'entry'}}){
      my $lpar_uid = $_;
      my $lpar = $resp->{'entry'}{$lpar_uid}{'content'}{'LogicalPartition:LogicalPartition'};
      push @{$out}, $lpar;
    }
  }
  return $out ? $out : [];
}

sub getVioses {
  my ($self) = @_;

  if (checkInputParams($self)){
    $self->{'logger'}->error("getVioses Params error $self");
    return [];
  }
  else {
    $self->{'logger'}->debug("getVioses Params seem ok $self");
  }

  my $out = [];
  my $url = "/rest/api/uom/VirtualIOServer";
  my $resp = $self->apiRequest('GET', 'application/xml', $url);

  # If there is only one vios
  if (defined $resp->{'entry'}{'id'}){
    my $vios = $resp->{'entry'}{'content'}{'VirtualIOServer:VirtualIOServer'};
    $out = [ $vios ];
  }
  else { # If there are at least two vioses
    for (keys %{$resp->{'entry'}}){
      my $vios_uid = $_;
      my $vios = $resp->{'entry'}{$vios_uid}{'content'}{'VirtualIOServer:VirtualIOServer'};
      push @{$out}, $vios;
    }
  }
  return $out ? $out : [];
}

sub getManagedSystemConfiguration {
  my ($self, $uid) = @_;

  if (checkInputParams($self, $uid)){
    $self->{'logger'}->error("getManagedSystemConfiguration Params error $self, $uid");
    return {};
  }
  else {
    $self->{'logger'}->debug("getManagedSystemConfiguration Params seem ok $self, $uid");
  }

  my $url = "/rest/api/uom/ManagedSystem/$uid";
  my $resp = $self->apiRequest('GET', 'application/xml', $url);
  return $resp->{'content'}{'ManagedSystem:ManagedSystem'} ? $resp->{'content'}{'ManagedSystem:ManagedSystem'} : {};
}

sub getLparsConfiguration {
  my ($self, $uid) = @_;
  if (checkInputParams($self, $uid)){
    $self->{'logger'}->error("getLparsConfiguration Params error $self, $uid");
    return {};
  } else {
    $self->{'logger'}->debug("getLparsConfiguration Params seem ok $self, $uid");
  }
  my $url = "/rest/api/uom/ManagedSystem/$uid/LogicalPartition";
  my $resp = $self->apiRequest('GET', 'application/xml', $url);
  return $resp->{'entry'} ? $resp->{'entry'} : {};
}

sub getViosesConfiguration {
  my ($self, $uid) = @_;

  if (checkInputParams($self, $uid)){
    $self->{'logger'}->error("getViosesConfiguration Params error $self, $uid");
    return {};
  }
  else {
    $self->{'logger'}->debug("getViosesConfiguration Params seem ok $self, $uid");
  }

  my $url = "/rest/api/uom/ManagedSystem/$uid/VirtualIOServer";
  my $resp = $self->apiRequest('GET', 'application/xml', $url);

  return $resp->{'entry'} ? $resp->{'entry'} : {};
}

# uid - managed system uid
# start/end unix timestamp of where to start/end getting perf data
sub getServerProcessed {
  my ($self, $uid, $start, $end, $type) = @_;

  if (checkInputParams($self, $uid, $start)){
    $self->{'logger'}->error("getServerProcessed Params error, $self, $uid, $start");
    return {};
  }
  else {
    $self->{'logger'}->debug("getServerProcessed Params seem ok $self, $uid, $start");
  }

  my $url = add_start_end("/rest/api/pcm/ManagedSystem/$uid/ProcessedMetrics", $start, $end);
  $url = $type ? "$url&Type=$type" : $url;

  my $resp = $self->apiRequest('GET', 'application/xml', $url);
  my $out = {};

  if (defined $resp->{'entry'}{'id'}){
    $out->{'only_one_uuid'} = $resp->{'entry'};
  }
  else {
    $out = $resp->{'entry'};
  }

  return $out ? $out : {};
}

sub getServerProcessedSample {
  my ($self, $link) = @_;
  if (checkInputParams($self, $link)){
    $self->{'logger'}->error("getServerProcessedSample Params error  $self, $link");
    return {};
  } else {
    $self->{'logger'}->debug("getServerProcessedSample Params seem ok $self, $link");
  }
  my $url = $link->{'href'};
  my $resp = $self->apiRequest('GET', $link->{'type'}, $url);
  return $resp->{'entry'} ? $resp->{'entry'} : {};  
}

sub getServerAggregated {
  my ($self, $uid, $start, $end) = @_;
  if (checkInputParams($self, $uid, $start, $end)){
    $self->{'logger'}->error("getServerAggregated Params error $self, $uid, $start, $end");
    return {};
  } else {
    $self->{'logger'}->debug("getServerAggregated Params seem ok $self, $uid, $start, $end");
  }
  my $url = add_start_end("/rest/api/pcm/ManagedSystem/$uid/AggregatedMetrics", $start, $end);
  my $resp = $self->apiRequest('GET', 'application/xml', $url);

  my $out = {};
  if (defined $resp->{'entry'}{'id'}){
    $out->{'only_one_uuid'} = $resp->{'entry'};
  } else {
    $out = $resp->{'entry'};
  }
  return $out ? $out : {};
}

sub getServerLTM {
  my ($self, $uid, $start, $end) = @_;
  if (checkInputParams($self, $uid, $start, $end)){
    $self->{'logger'}->error("getServerLTM Params error $self, $uid, $start, $end");
    return {};
  } else {
    $self->{'logger'}->debug("getServerLTM Params seem ok $self, $uid, $start, $end");
  }
  my $url = add_start_end("/rest/api/pcm/ManagedSystem/$uid/RawMetrics/LongTermMonitor", $start, $end);
  my $resp = $self->apiRequest('GET', 'application/xml', $url);
  return $resp->{'entry'} ? $resp->{'entry'} : {};
}

sub getEvents {
  my ($self) = @_;
  if (checkInputParams($self)){
    $self->{'logger'}->error("getEvents Params error $self");
    return {};
  } else {
    $self->{'logger'}->debug("getEvents Params seem ok $self");
  }
  my $url = "/rest/api/uom/Event";
  my $resp = [];
  $resp = $self->apiRequest('GET', 'application/xml', $url);
  my $events = $resp->{'entry'}{'content'}{'Event:Event'} ? $resp->{'entry'}{'content'}{'Event:Event'} : [];
  if (ref($events) eq 'HASH'){
    $events = [ $events ];
  }
  return $events ? $events : [];
}

sub getOperations {
  my ($self, $uid) = @_;
  if (checkInputParams($self, $uid)){
    $self->{'logger'}->error("getEvents Params error, $self, $uid");
    return {};
  } else {
    $self->{'logger'}->debug("getEvents Params seem ok $self, $uid");
  }
  my $url = "/rest/api/uom/$uid/operations";
  my $resp = $self->apiRequest('GET', 'application/xml', $url);
  my $operations = $resp->{'entry'}{'content'}{'OperationSet:OperationSet'}{'DefinedOperations'}{'Operation'};
  if (ref($operations) eq 'HASH'){
    $operations = [ $operations ];
  }
  return $operations ? $operations : [];
}

# Expects : [string]  - URL to a JSON file
# Returns : [hash]    - Decoded JSON as a hash or array
sub getJsonContent {
  my ($self, $link) = @_;
  if (checkInputParams($self, $link)){
    $self->{'logger'}->error("getEvents Params error $self, $link");
    return {};
  } else {
    $self->{'logger'}->debug("getEvents Params seem ok $self, $link");
  }
  if (!defined $link->{'href'} || $link->{'href'} eq "" ) {
    $self->{'logger'}->error("Cannot get the JSON content. The JSON file link is an empty string.");
    return {};
  }
  my $url = $link->{'href'};
  my $resp = $self->apiRequest('GET', $link->{'type'}, $url);
  return $resp ? $resp : {};
}

# Expects: url[string], *start[unixtime], *end [unixtime], *offset_min [number of minutes]
#add the timestamps start/end to the HMC REST API url
sub add_start_end {
  my $offset_min = 1;
  my ($url, $start, $end, $type) = @_;

  $start = $start ? strftime('%Y-%m-%dT%H:%M:%S%z', localtime($start - $offset_min  * 60 )) : undef;
  $end = $end ? strftime('%Y-%m-%dT%H:%M:%S%z', localtime($end)) : undef;

  $url = "$url?";
  $url = $start ? $url . 'StartTS=' . $start . "&" : $url;
  $url = $end   ? $url . 'EndTS='   . $end   . '&' : $url;
  $url =~ s/.$//g;
  return $url;
}

# Use regex to get rid of e.g. https://127.0.0.1:12443 in the URL. Subroutine apiRequest expects e.g. "/rest/api/pcm/prefernces" and adds e.g. "https://hmc_ip:12443" at the begginning.
# This is needed as some versions of HMC put localhost or 127.0.0.1 to the URL for some reason. Switch to the valid HMC IP.
sub chop_link {
  my $link = shift;
  $link =~ s/^.*rest\/api/\/rest\/api/g;
  return $link;
}

sub to_array {
  my $in = shift;
  my $array = ref($in) eq "ARRAY" ? $in : [ $in ]; #if it is not an array, make it an array
  return $array;
}

# HMC Requests as Login, API Request, Logout
# accepts params : (method, type, url) and returns output in hash
sub apiRequest {
  my ($self, $method, $type, $url) = @_;

  if (checkInputParams($self, $method, $type, $url)){
    $self->{'logger'}->error("API Request : Params error $self, $method, $type, $url");
    return {};
  }
  else {
    $self->{'logger'}->debug("API Request : Params seem ok $self, $method, $type, $url");
  }

  if (!defined $self->{numOfRequests}) {
    $self->{numOfRequests} = 0;
  }
  else {
    $self->{numOfRequests}++;
  }

  my $request_timeout = $self->{'OPTIONS'}{'DEFAULT_TIMEOUT'} || $DEFAULT_TIMEOUT;

  my $ua = LWP::UserAgent->new(
    timeout => $request_timeout,
    ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 },
  );

  $url = chop_link($url);

  my $req_url = $self->{'protocol'}."://".$self->{'host'}.":".$self->{'port'}. $url;
  my $req = HTTP::Request->new( $method, $req_url );

  $req->content_type( $type );
  $req->header( 'Accept'=>'*/*' );
  $req->header( 'X-API-Session' => $self->{'APISession'} );

  my $request_start_time = time;

  my $response = $ua->request($req);

  if ( defined $self->{'OPTIONS'}{'DEBUG_HMC_RESPONSE'} && $self->{'OPTIONS'}{'DEBUG_HMC_RESPONSE'} ) {
    $self->print_anonymized_response($url, $response);
  }

  my $request_duration = time - $request_start_time;

  if ($request_duration > $DEFAULT_REQ_CHECK_LIMIT) {
    $self->{'logger'}->error("API Request : Request $req_url took $request_duration seconds.\n");
  }
  else {
    $self->{'logger'}->debug("API Request : Request $req_url took $request_duration seconds.\n");
  }

  if ( ! $response->is_success ){
    $self->{'logger'}->error("Response is not success at " . $req_url . "\n" );
    #warn Dumper $req;
    return {};
  }

  my $content = {};

  if ($type eq 'application/xml' || $type eq 'application/atom+xml'){
    eval {
      if ($response->content eq ''){ 
        #$self->{'logger'}->error("Cannot get content of XML file at " . $req_url );
        return {};
      }
      my $xmlin_start_time = time;

      $content = XMLin( $response->content );

      my $xmlin_duration = time - $xmlin_start_time;
      $self->{'logger'}->debug("API Request : XMLin $req_url took $xmlin_duration seconds.\n");
    };
    if ($@){
      $self->{'logger'}->error("Eval Error: $@ : Cannot get XML content of file at " . $req_url );
      $self->{'logger'}->error("$content");
    }
  }
  elsif ($type eq 'application/vnd.ibm.powervm.pcm.json' || $type eq 'application/json') {
      my $decodejson_start_time = time;
      my $decodejson_duration;
      eval {
        $content = (decode_json ($response->content));
        $decodejson_duration = time - $decodejson_start_time;
      };
      if ($@){
        $self->{'logger'}->error("Eval Error: $@ : Cannot decode JSON file at " . $req_url);
        $self->{'logger'}->error("$content");
      }
      $self->{'logger'}->debug("API Request : DecodeJSON $req_url took $decodejson_duration seconds.\n");
  }
  else {
    $self->{'logger'}->error("Unsupported content type: $type");
  }

  if (defined $content->{'HTTPStatus'}{'content'}){
    $self->{'logger'}->error("" . $req_url );
    $self->{'logger'}->error("$content->{'HTTPStatus'}{'content'} | $content->{'ReasonCode'}{'content'} : $content->{'Message'}{'content'}");    
  }

  return $content ? $content : {};
}

sub check_db_token_and_logout_if_necessary {
    my ($self) = @_;
    # If there is a SessionToken in the database
    if (defined $self->{'microservice_data'}){
        # It has been logged out already, ignore
        if ($self->{'microservice_data'}{'SessionToken'} eq "session_logged_out" || $self->{'microservice_data'}{'SessionToken'} eq 'invalid_session_token'){
         $self->{'logger'}->info("SessionToken in DB is \"$self->{'microservice_data'}{'SessionToken'}\", continue and get a new one");
        }
        # It is actually a valid SessionToken (probably remained there when microservice forcibly shutdown), do not use it but logout the session
        elsif ($self->{'microservice_data'}{'SessionToken_created'} && (time - $self->{'microservice_data'}{'SessionToken_created'} > 1800)) {
            my $rc_logout = 0;
            $rc_logout = $self->logout($self->{'microservice_data'}{'SessionToken'});
            #Just check if the previous session stored in DB was logged out properly
            if ($rc_logout == 401){
             $self->{'logger'}->info("Session in DB not valid anymore, continue and get a new one");
            } else {
             $self->{'logger'}->info("Found old valid SessionToken in Xormon database. Logged out to prevent stacking sessions");
            }
        }
    }

    return 0;
}

#
# SESSION: login request
#
#   $self->{'APISession'}
#
# TODO: change return value - now: 1 means failure
sub login_to_hmc {
  my ($self) = @_;

  $self->{'logger'}->info("Getting session");

  my $connected = Test::connTest($self->{'host'}, $self->{'port'});
  if (!$connected == 1 && $self->{'dual_hmc'}) {
    $connected = Test::connTest($self->{'second_host'}, $self->{'port'});
    if ($connected == 1) {
      $self->{'host'} = $self->{'second_host'};
    }
  }

  my $url = $self->{'protocol'}."://".$self->{'host'}.":".$self->{'port'}."/rest/api/web/Logon";
  my $session = 'invalid_session_token';

  if (!defined $self->{'username'} || !defined $self->{'password'} ){
    $self->{'logger'}->error("Not defined username or password");
    $self->{'APISession'} = $session;
    return 1;
  }

  my $request_timeout = $self->{'OPTIONS'}{'DEFAULT_TIMEOUT'} || $DEFAULT_TIMEOUT;
  
  my $ua = LWP::UserAgent->new(
    timeout => $request_timeout,
    ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 },
  );

  my $request_body = <<_REQUEST_;
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<LogonRequest xmlns="http://www.ibm.com/xmlns/systems/power/firmware/web/mc/2012_10/" schemaVersion="V1_0">
  <UserID kb="CUR" kxe="false">$self->{'username'}</UserID>
  <Password kb="CUR" kxe="false">$self->{'password'}</Password>
</LogonRequest>
_REQUEST_

  my $req = HTTP::Request->new( 'PUT' => $url );
  $req->content_type('application/vnd.ibm.powervm.web+xml');
  $req->content_length( length($request_body) );
  $req->header( 'Accept' => '*/*' );
  $req->content($request_body);
  my $response = $ua->request($req);

  if ( defined $self->{'OPTIONS'}{'DEBUG_HMC_RESPONSE'} && $self->{'OPTIONS'}{'DEBUG_HMC_RESPONSE'} ) {
    $self->print_anonymized_response($url, $response);
  }

  my $message = "Login $self->{'username'} $response->{'_msg'} ($response->{'_rc'}) at $url";

  my $ref = {};

  eval {
    $ref = XMLin( $response->content );
  };
  if ($@) {
    $self->{'logger'}->error("XMLin error:");
    $self->{'logger'}->error( $@ );
  }

  if ($response->is_success){
    $session = defined $ref->{'X-API-Session'}{'content'} ? $ref->{'X-API-Session'}{'content'} : 'invalid_session_token';
    $self->{'logger'}->info("". $message);
  }
  else {
    my $err = defined $ref->{'content'}{'HttpErrorResponse:HttpErrorResponse'} ? $ref->{'content'}{'HttpErrorResponse:HttpErrorResponse'} : undef;
    if (defined $err){
      $self->{'logger'}->error("$err->{'HTTPStatus'}{'content'}:$err->{'Message'}{'content'}, $err->{'ReasonCode'}{'content'}");
    } else {
      $self->{'logger'}->error("Unknown error in login");
    }
  }
  
  $self->{'APISession'} = $session ? $session : 'invalid_session_token';

  if (defined $session && $session eq 'invalid_session_token'){
    return 1;
  } else {
    return 0;
  }
}

#logout one hmc api session
sub logout {
  my ($self, $session) = @_;

  my $request_timeout = $self->{'OPTIONS'}{'DEFAULT_TIMEOUT'} || $DEFAULT_TIMEOUT;

  my $ua = LWP::UserAgent->new(
    timeout => $request_timeout,
    ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 },
  );
  my $url = $self->{'protocol'}."://".$self->{'host'}.":".$self->{'port'}."/rest/api/web/Logon";
  my $req_del = HTTP::Request->new( "DELETE" => $url );
  $req_del->header( 'X-API-Session' => $session );
  my $data = $ua->request($req_del);

  if ( defined $self->{'OPTIONS'}{'DEBUG_HMC_RESPONSE'} && $self->{'OPTIONS'}{'DEBUG_HMC_RESPONSE'} ) {
    $self->print_anonymized_response($url, $data);
  }

  return $data->{'_rc'};
}


sub print_anonymized_response {
  my($self, $url_loc, $resp_loc) = @_;

  $self->{'logger'}->info("DEBUG_HMC_RESPONSE $url_loc\n");
  $self->{'logger'}->info("DEBUG_HMC_RESPONSE active - printing received message\n");

  # Anonymize
  if ( defined $resp_loc->{_request} ) {
    $resp_loc->{_request} = {};
  }

  print Dumper $resp_loc;

}

#
# Power Init
#
#
#   logger - XormonLogger
#
#   LATER:
#     xormon
#     microservice_data
#     microservice_data_into_db
#
# TODO: enable DEFAULT_REQ_CHECK_LIMIT
# TODO: compose all additional data of $power here
#
sub new {
  my($self, $cfg) = @_;

  my $power = {};

  for my $cfg_item ( keys %{$cfg} ) {
    $power->{$cfg_item} = $cfg->{$cfg_item};
  }

  $power->{'logger'} = XormonLogger->new($hw_type, $cfg->{'hostcfg_id'}, $cfg->{'hostalias'}, 1, $cfg->{'debug'});

  $power->{'OPTIONS'}{'DEFAULT_TIMEOUT'}          = $cfg->{'DEFAULT_TIMEOUT'}           || $DEFAULT_TIMEOUT           || 120;
  $power->{'OPTIONS'}{'DEFAULT_REQ_CHECK_LIMIT'}  = $cfg->{'DEFAULT_REQ_CHECK_LIMIT'}   || $DEFAULT_REQ_CHECK_LIMIT   || 5;
  $power->{'OPTIONS'}{'DEBUG_HMC_RESPONSE'}       = $cfg->{'DEBUG_HMC_RESPONSE'}        || $DEBUG_HMC_RESPONSE        || 0;
  $power->{'OPTIONS'}{'SAVE_HMC_RESPONSE_SAMPLE'} = $cfg->{'SAVE_HMC_RESPONSE_SAMPLE'}  || $SAVE_HMC_RESPONSE_SAMPLE  || 0;
  $power->{'HMC_CONNECTION_SAVED'} = 0;

  return (bless $power);
}

1;
