use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use Xormon;
use Digest::MD5 qw(md5 md5_hex md5_base64);

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

# microservice port
my $port = 3110; #TODO change this later

my $server = IO::Socket->new(
    Domain => AF_INET,
    Type => SOCK_STREAM,
    Proto => 'tcp',
    LocalHost => '0.0.0.0',
    LocalPort => $port,
    ReusePort => 1,
    Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

 # waiting for a new client connection
while ($client = $server->accept()) {

    my $pid;
    unless ( defined( $pid = fork() ) ) {
        Xormon::error("Wait for fork");
        sleep 5;
    }

    if ($pid == 0) {

        # get information about a newly connected client
        my $client_address = $client->peerhost();
        my $client_port = $client->peerport();
        Xormon::log("Connection from $client_address:$client_port");

        # received data template
        #
        # {
        #   "jwt"  : "<backendToken>",
        #   "task" : "<taskName>"
        #   "data" : {
        #     <configuration_from_hostcfg>
        #   }
        # }

        # read up to 2048 characters from the connected client
        my $data = "";
        $client->recv($data, 2048);

        my $decoded = from_json($data);

        Xormon::log("Received task: ".$decoded->{task});
        my $response = '';
        if ($decoded->{task} eq "fetch") {

            $client->send("Task successfuly started");
            $client->shutdown(SHUT_WR);

            my $xormon = Xormon->new("$client_address:3000", $decoded->{jwt}, 'random-data');
            my $cfg = $decoded->{data};
            $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

            my %perf;
            if ( $cfg->{type} eq "cpu" ) {

              %perf = get_data_cpu($cfg, $xormon);

            }

            # send data to backend

            Xormon::log("Task [$decoded->{task}] is done");
            my $status = "successful";
            $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);

        } elsif ($decoded->{task} eq "test") {

            $response = apitest();

            $client->send($response);
            $client->shutdown(SHUT_WR);

        } else {
            $response = "undefined task";

            $client->send($response);
            $client->shutdown(SHUT_WR);

        }

        exit();

    } else {
        $client->autoflush(1);
    }
}

sub get_data_cpu {

  my $cfg = shift;
  my $xormon = shift;

  my %perf;
  my @architecture;
  my $cnt = 0;

  for my $item (1..$cfg->{items}) {
    #Architecture
    my %random_data_arch = (
        "item_id" => md5_hex('item'.$item),
        "label" => "Random $item",
        "hw_type" => "RANDOM",
        "subsystem" => "DATA",
        "parents" => []
    );
    push(@architecture, \%random_data_arch);
  }
  $xormon->saveArchitecture(\@architecture);

  for my $item (1..$cfg->{items}) {
    for(my $i = $cfg->{start}; $i <= $cfg->{end}; $i += $cfg->{interval}){
      $cnt++;

      my ($sec,$min,$hour,$day,$month,$year) = localtime($i);

      if ( $hour < 4 ) {
        $perf{'random'}{'data'}{md5_hex('item'.$item)}{$i}{'cpu'} = sprintf("%.0f", rand(50))
      }
      elsif ( $hour > 20 ) {
        $perf{'random'}{'data'}{md5_hex('item'.$item)}{$i}{'cpu'} = sprintf("%.0f", rand(50) + rand(49));
      }
      else {
        $perf{'random'}{'data'}{md5_hex('item'.$item)}{$i}{'cpu'} = sprintf("%.0f", rand(99));
      }

      if ( $cnt >= 10000 ) {
        $cnt = 0;

        saveToBe(\%perf, $xormon);
        %perf = ();
      }

    }
    Xormon::log("Saved item no. $item");
    saveToBe(\%perf, $xormon);
    %perf = ();
  }
  saveToBe(\%perf, $xormon);

  #save architecture

  return %perf;

}

sub saveToBe {

  my $perf_ref = shift;
  my $xormon   = shift;

  my %perf = %{$perf_ref};
  my $rc = $xormon->saveJSON2(\%perf);

  ############################### END AGENT SECTION ###############################

  # status code
  my $status;
  if ($rc eq "201") {
      $status = "successful";
  } elsif ($rc eq "401") {
      $status = "unauthorized";
  } else {
      $status = "error";
  }
}

sub api2csv {
	my $data = shift;

    my $time1 = time();
    my $time2 = $time1-15;
    my $time3 = $time2-30;
    my $time4 = $time3-45;

    return "uuid;timestamp;data1;data2;data3;data4;data5;data6;data7;data8;
f6969d67-453d-4337-970c-ccc72f5d8c43;$time1;1;2;3;4;5;6;7;8;
f6969d67-453d-4337-970c-ccc72f5d8c43;$time2;2;3;2;4;5;6;7;8;
f6969d67-453d-4337-970c-ccc72f5d8c43;$time3;2;3;3;4;5;6;7;8;
f6969d67-453d-4337-970c-ccc72f5d8c43;$time4;5;4;1;4;5;6;7;8;"

}

sub apitest {
    my $data = shift;

    my @rows;
    push(@rows, "Number of nodes: 10");
    push(@rows, "Number of VMs: 65");
    return Xormon::conn_test(1, \@rows);
}

$server->close();
