from operator import eq
from kafkian import Consumer
from kafkian.serde.serialization import AvroSerializer, AvroStringKeySerializer, SubjectNameStrategy
from kafkian.serde.deserialization import AvroDeserializer, Deserializer
import json
import os
from datetime import datetime

ip          = os.getenv('KAFKA_IP')
schema_port = os.getenv('KAFKA_SCHEMA_PORT')
broker_port = os.getenv('KAFKA_BROKER_PORT')
dir         = os.getenv('KAFKA_DIR')
debug       = os.getenv('KAFKA_DEBUG')
conn_test   = os.getenv('KAFKA_CONNECTION_TEST')

auto_commit = "True"
if conn_test == "1":
  auto_commit = "False"


if debug == "1":
    print("KAFKA_IP              : " + ip)
    print("KAFKA_SCHEMA_PORT     : " + schema_port)
    print("KAFKA_BROKER_PORT     : " + broker_port)
    print("KAFKA_DIR             : " + dir)
    print("KAFKA_DEBUG           : " + debug)
    print("KAFKA_CONNECTION_TEST : " + str(conn_test))
    print("auto_commit           : " + auto_commit)

CONSUMER_CONFIG = {
    'bootstrap.servers': f'{ip}:{broker_port}',
    'default.topic.config': {
        'auto.offset.reset': 'earliest',
        'enable.auto.commit': auto_commit,
        #'auto.offset.reset': 'latest',
        #'consume.callback.max.messages': '2',
    },
    'group.id': 'notifications',
    #'topic.metadata.refresh.interval.ms': '-1',
    #'connections.max.idle.ms': '2000',
    #'heartbeat.interval.ms': '2000',
    #'session.timeout.ms': '45000',
    #'max.poll.interval.ms': '50000',
}

consumer = Consumer(
    CONSUMER_CONFIG,
    #topics=['flowmon_metrics_switch_flow', 'heartbeat_metrics_switch', 'perfmon_metrics_ethport', 'perfmon_metrics_extn_circuit', 'perfmon_metrics_extn_tunnel', 'perfmon_metrics_fcport', 'perfmon_metrics_switch'],
    topics=['perfmon_metrics_fcport', 'perfmon_metrics_switch'],
    key_deserializer=AvroDeserializer(schema_registry_url=f'http://{ip}:{schema_port}'),
    value_deserializer=AvroDeserializer(schema_registry_url=f'http://{ip}:{schema_port}'),
)

for message in consumer:
    if message:
        json_data     = dict(message.value)
        json_key_data = dict(message.key)
        id            = (json_key_data["key"])
        output_file   = dir + "/" + ip + "-" + str(message.timestamp) + "-" + str(message.topic) + "-" + id + ".json"

        f = open(output_file + "-tmp", "x")
        #f.write(printRecord)
        f.write(json.dumps(json_data))
        f.close()
        os.rename(output_file + "-tmp", output_file)

        # save last stats for connection test
        last_stats_file = dir + "/../" + ip + "-last-stats.json"
        f = open(last_stats_file, "w")
        f.write(json.dumps(json_data))
        f.close()

        if conn_test == "1":
            print(now.strftime("%Y-%m-%d %H:%M:%S") + " : consumer._close : conn_test=" + str(conn_test))
            consumer._close()

        if debug == "1":
            now = datetime.now()

            print("##########################################################")
            print("act time          : " + now.strftime("%Y-%m-%d %H:%M:%S"))
            print("output file       : " + output_file)
            print("message.timestamp : " + str(message.timestamp))
            print("message.topic     : " + str(message.topic))
            print("message.partition : " + str(message.partition))
            print("message.offset    : " + str(message.offset))
            print("message.headers   : " + str(message.headers))
            print("message.key       : " + json.dumps(json_key_data))
            print("message.value     : " + json.dumps(json_data))

        #consumer.commit()
        #consumer._close()

    #if message.error:
    #    print(now.strftime("%Y-%m-%d %H:%M:%S") + " : consumer._close : an error has been found : message.error=" + message.error)
    #    consumer._close()
