package STORAGEGRID;

use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Request;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Data::Dumper;
use Xormon;
use JSON qw(encode_json decode_json);
use HTTP::Date;
use Date::Parse;
use Test;
use XormonLogger;
#use POSIX qw( strftime );

my %data;
my $xormon;
my $cfg;
my $logger;
my @gsites;

sub collect_data {
  $xormon = shift;
  $cfg    = shift;
  @gsites = ();
  $cfg->{'timestamp'} = time();
  $cfg->{'gtime'}     = gmtime($cfg->{'timestamp'});
  $cfg->{'gtime'} =~ s/ {2,}/ /g;

  # test main variables
  #
  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")    && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!")     && return "error"; }
  $logger = XormonLogger->new( 'storagegrid', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost'} )       { $logger->error("Cannot continue, 'host' is not configured!")          && return "error"; }
  unless ( exists $cfg->{'apiUsername'} )   { $logger->error("Cannot continue, 'apiUsername' is not configured!")   && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )   { $logger->error("Cannot continue, 'apiPassword' is not configured!")   && return "error"; }

  $cfg->{'token'} = get_api_token("https://$cfg->{'apiHost'}/api/v3/authorize");
  if ( $cfg->{'token'} eq "0" ) {
      $xormon->isConnected(0, "login failed cant login to api");
      Xormon::error("Login failed : Cant' login to API") && return 0;
  }

  $xormon->isConnected(1);

  ( $cfg->{'nameGrid'}, @gsites ) = create_conf();

  ( $cfg->{'startTime'}, $cfg->{'endTime'}, $cfg->{'pre_start_time'} ) = set_time();
  create_perf();
  print_perf();
  print_conf();
  health_status();
  logout("https://$cfg->{'apiHost'}/api/v3/authorize");

  return 1;
}

sub create_conf {
  my $return = "";
  ##################
  #         TOPOLOGY
  ##################
  $return = new_get_request("https://$cfg->{'apiHost'}/api/v3/grid/health/topology?depth=subcomponent");

  #print Dumper $return;
  my $name      = "";
  my $grid_name = "";
  my $vdc_name  = "";
  my @sites;
  if ( $return ne "0" ) {
    if ( defined $return->{'data'}->{'uniqueName'} && $return->{'data'}->{'uniqueName'} ne "" ) {
      $grid_name = $return->{'data'}->{'uniqueName'};
      $data{'conf'}{'grid'}{$grid_name}{'name'} = $return->{'data'}->{'uniqueName'};
      if ( defined $return->{'data'}->{'id'} && $return->{'data'}->{'id'} ne "" ) {
        $data{'conf'}{'grid'}{$grid_name}{id} = $return->{'data'}->{'id'};
      }
      if ( defined $return->{'data'}->{'state'} && $return->{'data'}->{'state'} ) {
        $data{'conf'}{'grid'}{$grid_name}{'state'} = $return->{'data'}->{'state'};
      }

      #1 child
      if ( defined $return->{'data'}->{'children'} && ref( $return->{'data'}->{'children'} ) eq "ARRAY" ) {
        foreach my $children ( @{ $return->{'data'}->{'children'} } ) {
          if ( defined $children->{'uniqueName'} && $children->{'uniqueName'} ne "" ) {
            $vdc_name = $children->{'uniqueName'};
            push( @sites, $vdc_name );
            $data{'conf'}{'vdc'}{$vdc_name}{'name'} = $children->{'uniqueName'};
            if ( defined $children->{'id'} && $children->{'id'} ne "" ) {
              $data{'conf'}{'vdc'}{$vdc_name}{'id'} = $children->{'id'};
            }
            if ( defined $children->{'state'} && $children->{'state'} ne "" ) {
              $data{'conf'}{'vdc'}{$vdc_name}{'state'} = $children->{'state'};
            }

            #2 child
            if ( defined $children->{'children'} && ref( $children->{'children'} ) eq "ARRAY" ) {
              foreach my $children2 ( @{ $children->{'children'} } ) {
                if ( defined $children2->{'uniqueName'} && $children2->{'uniqueName'} ne "" ) {
                  $name = $children2->{'uniqueName'};
                  $data{'conf'}{'node'}{$name}{'name'} = $children2->{'uniqueName'};
                  if ( defined $children2->{'id'} && $children2->{'id'} ne "" ) {
                    $data{'conf'}{'node'}{$name}{'id'} = $children2->{'id'};
                  }
                  if ( defined $children2->{'state'} && $children2->{'state'} ne "" ) {
                    $data{'conf'}{'node'}{$name}{'state'} = $children2->{'state'};
                  }
                  if ( defined $name && $name ne "" ) {
                    $data{'conf'}{'node'}{$name}{vdcName} = $vdc_name;
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  ##################
  #  Product Version
  ##################
  $return = new_get_request("https://$cfg->{'apiHost'}/api/v3/grid/config/product-version");
  if ( $return ne "0" ) {
    if ( defined $return->{'apiVersion'} && $return->{'apiVersion'} ne "" ) {
      $data{'conf'}{'global'}{apiVersion} = $return->{'apiVersion'};
    }
    if ( defined $return->{'data'}->{'productVersion'} && $return->{'data'}->{'productVersion'} ne "" ) {
      $data{'conf'}{'global'}{productVersion} = $return->{'data'}->{'productVersion'};
    }
  }

  return ( $grid_name, @sites );
}    #end create_conf

sub create_perf {

  #########
  #   NODE
  #########
  foreach my $node ( keys( %{ $data{'conf'}{'node'} } ) ) {
    my $type = "node";

    #Memory - USED%
    get_data( "$type", "memory", "used_prct", "$node", "https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=(node_memory_MemTotal_bytes%7Binstance%3D%22$node%22%7D%20-%20node_memory_MemAvailable_bytes%7Binstance%3D%22$node%22%7D)%20%2F%20node_memory_MemTotal_bytes%7Binstance%3D%22$node%22%7D&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=5m" );

    #Memory - USED
    get_data( "$type", "memory", "used", "$node", "https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=node_memory_MemTotal_bytes%7Binstance%3D%22$node%22%7D%20-%20node_memory_MemAvailable_bytes%7Binstance%3D%22$node%22%7D&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=5m" );

    #Memory - Cached
    #get_data("$type","memory","cached","$node","https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=node_memory_Cached_bytes%7Binstance%3D%22$node%22%7D&start=$cfg->{'endTime'}&end=$cfg->{'endTime'}&step=5m");
    #Memory - Buffers
    #get_data("$type","memory","buffers","$node","https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=node_memory_Buffers_bytes%7Binstance%3D%22$node%22%7D&start=$cfg->{'endTime'}&end=$cfg->{'endTime'}&step=5m");
    #Memory - Free
    get_data( "$type", "memory", "free", "$node", "https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=node_memory_MemFree_bytes%7Binstance%3D%22$node%22%7D&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=5m" );

    #Memory - Slab
    #get_data("$type","memory","slab","$node","https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=node_memory_Slab_bytes%7Binstance%3D%22$node%22%7D&start=$cfg->{'endTime'}&end=$cfg->{'endTime'}&step=5m");
    #Memory - Total
    get_data( "$type", "memory", "total", "$node", "https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=node_memory_MemTotal_bytes%7Binstance%3D%22$node%22%7D&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=5m" );

    #cpu_utilization
    get_data( "$type", "cpu", "utilization", $node, "https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=sum%20(sum%20by%20(mode)(irate(node_cpu_seconds_total%7Binstance%3D%22$node%22%2Cmode!%3D%27idle%27%7D%5B5m%5D))%20*%20100%20%2F%20count%20by%20(mode)(node_cpu_seconds_total%7Binstance%3D%22$node%22%2Cmode!%3D%27idle%27%7D))&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

    #Load_Balancer
    # Sent
    get_data( "$type", "load_balancer", "sent", $node, "https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=8%20*%20sum%20(rate(storagegrid_private_load_balancer_storage_tx_bytes%7Binstance%3D%22$node%22%7D%5B5m%5D))&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

    # Received
    get_data( "$type", "load_balancer", "received", $node, "https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=8%20*%20sum%20(rate(storagegrid_private_load_balancer_storage_rx_bytes%7Binstance%3D%22$node%22%7D%5B5m%5D))&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

    #Incomming Request Rate
    get_data( "$type", "load_balancer", "in_req_rate", $node, "https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=sum%20(rate(storagegrid_private_load_balancer_storage_request_incoming_count%7Binstance%3D%22$node%22%7D%5B3m%5D))%20%3E%200&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

    #Error response rate
    get_data( "$type", "load_balancer", "err_resp_rate", $node, "https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=sum%20by%20(code)%20(rate(storagegrid_private_load_balancer_storage_request_count%7Bcode%3D~%22%5B45%5D.*%22%2C%20instance%3D%22$node%22%7D%5B3m%5D))%20%3E%200&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );
  }

  ##############################
  # Network - For all nodes
  ##############################
  #Received rate
  get_data_rate( "network", "received_rate", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_network_received_bytes%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

  #Received rate
  get_data_rate( "network", "sent_rate", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_network_transmitted_bytes%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

  ################
  # Interface
  #################

  # Receive
  ###########

  #Bytes
  get_data_rate( "interface", "received_bytes", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28node_network_receive_bytes_total%5B6m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #Dropped
  get_data_rate( "interface", "received_drop", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28node_network_receive_drop_total%5B6m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #Errors
  get_data_rate( "interface", "received_errs", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28node_network_receive_errs_total%5B6m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  # Transmit
  ############

  #Bytes
  get_data_rate( "interface", "transmit_bytes", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28node_network_transmit_bytes_total%5B6m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #Dropped
  get_data_rate( "interface", "transmit_drop", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28node_network_transmit_drop_total%5B6m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #Errors
  get_data_rate( "interface", "transmit_errs", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28node_network_transmit_errs_total%5B6m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  ################################
  # CPU - #For all nodes all stats
  ################################
  cpu_data( "cpu", "https://$cfg->{'apiHost'}/api/datasources/proxy/1/api/v1/query_range?query=node_cpu_seconds_total&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=5m&timeout=120s" );

  ###############################
  # Capacity
  ###############################

  #Total
  #############
  get_data_rate( "data_total", "total", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=storagegrid_storage_utilization_data_bytes&start=$cfg->{'endTime'}&end=$cfg->{'endTime'}&step=5m" );

  #Data
  ############

  #total
  get_data_rate( "data", "total_cap", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=storagegrid_storage_utilization_total_space_bytes&start=$cfg->{'endTime'}&end=$cfg->{'endTime'}&step=5m" );

  #Used
  get_data_rate( "data", "free_cap", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=storagegrid_storage_utilization_usable_space_bytes&start=$cfg->{'endTime'}&end=$cfg->{'endTime'}&step=5m" );

  #Metadata
  ############

  #Usable
  get_data_rate( "metadata", "allowed_bytes", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=storagegrid_storage_utilization_metadata_allowed_bytes&start=$cfg->{'endTime'}&end=$cfg->{'endTime'}&step=5m" );

  #Used
  get_data_rate( "metadata", "used_bytes", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=storagegrid_storage_utilization_metadata_bytes&start=$cfg->{'endTime'}&end=$cfg->{'endTime'}&step=5m" );

  #############################
  # Protocol
  #############################

  #S3
  ##########

  #ingested bytes
  get_data_rate( "s3", "ingested_bytes", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_s3_data_transfers_bytes_ingested%5B6m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #retrieved bytes
  get_data_rate( "s3", "retrieved_bytes", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_s3_data_transfers_bytes_retrieved%5B6m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #operations successful
  get_data_rate( "s3", "operations_successful", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_s3_operations_successful%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #operations successful
  get_data_rate( "s3", "operations_failed", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_s3_operations_failed%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #operations unauthorized
  get_data_rate( "s3", "operations_unauthorized", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_s3_operations_unauthorized%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #Swift
  ########

  #ingested bytes
  get_data_rate( "swift", "ingested_bytes", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_swift_data_transfers_bytes_ingested%5B6m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #retrieved bytes
  get_data_rate( "swift", "retrieved_bytes", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_swift_data_transfers_bytes_retrieved%5B6m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #operations successful
  get_data_rate( "swift", "operations_successful", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_swift_operations_successful%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #operations successful
  get_data_rate( "swift", "operations_failed", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_swift_operations_failed%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #operations unauthorized
  get_data_rate( "swift", "operations_unauthorized", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_swift_operations_unauthorized%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  ########################
  # Disk
  ########################

  #IO read - total before 5 mins
  get_data_rate_before( "disk", "io_read_total", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=node_disk_reads_completed_total&start=$cfg->{'pre_start_time'}&end=$cfg->{'startTime'}&step=300" );

  #IO write - total before 5 mins
  get_data_rate_before( "disk", "io_write_total", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=node_disk_writes_completed_total&start=$cfg->{'pre_start_time'}&end=$cfg->{'startTime'}&step=300" );

  #read time - total before 5 mins
  get_data_rate_before( "disk", "read_time", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=node_disk_read_time_seconds_total&start=$cfg->{'pre_start_time'}&end=$cfg->{'startTime'}&step=300" );

  #write time - total before 5 mins
  get_data_rate_before( "disk", "write_time", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=node_disk_write_time_seconds_total&start=$cfg->{'pre_start_time'}&end=$cfg->{'startTime'}&step=300" );

  #IO TOTAL ONLY FOR RESPONSE TIME
  #IO read - now 5 mins
  get_data_rate( "disk", "io_read_total", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=node_disk_reads_completed_total&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #IO write - now 5 mins
  get_data_rate( "disk", "io_write_total", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=node_disk_writes_completed_total&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #data read
  get_data_rate( "disk", "data_read", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28node_disk_read_bytes_total%5B3m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

  #data write
  get_data_rate( "disk", "data_write", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28node_disk_written_bytes_total%5B3m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

  #IO read
  get_data_rate( "disk", "io_read", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28node_disk_reads_completed_total%5B3m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

  #IO write
  get_data_rate( "disk", "io_write", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28node_disk_writes_completed_total%5B3m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

  #read time
  get_data_rate( "disk", "read_time", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=node_disk_read_time_seconds_total&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #write time
  get_data_rate( "disk", "write_time", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=node_disk_write_time_seconds_total&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

  #######################
  # ILM
  #######################

  # Awaiting/Queued - All
  get_data_rate( "ilm", "awaiting_all", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_ilm_awaiting_total_objects%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

  # Awaiting/Queued - Client
  get_data_rate( "ilm", "awaiting_client", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_ilm_awaiting_client_objects%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

  # Awaiting/Queued - Background
  get_data_rate( "ilm", "awaiting_background", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_ilm_awaiting_background_objects%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

  # Awaiting/Queued - Evaluation Rate
  get_data_rate( "ilm", "awaiting_evaluation_rate", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_ilm_awaiting_client_evaluation_objects_per_second%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

  # Scan Rate
  get_data_rate( "ilm", "scan_rate", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_ilm_scan_objects_per_second%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30" );

  #######################
  # Load Balancer
  #######################

  # Sent
  #get_data_rate("load_balancer","sent","https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_private_load_balancer_storage_tx_bytes%5B5m%5D%29&start=$cfg->{'endTime'}&end=$cfg->{'endTime'}&step=5m");
  # Received
  #get_data_rate("load_balancer","received","https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_private_load_balancer_storage_rx_bytes%5B5m%5D%29&start=$cfg->{'endTime'}&end=$cfg->{'endTime'}&step=5m");

  #Incomming Request Rate
  #get_data_rate("load_balancer","in_req_rate","https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_private_load_balancer_storage_request_incoming_count%5B3m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30");
  #Error response rate
  #get_data_rate("load_balancer","err_resp_rate","https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28storagegrid_private_load_balancer_storage_request_count%5B2m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=30");

  #Load
  get_data_rate( "load", "load", "https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=avg_over_time%28node_load5%5B5m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300" );

}    #end create_perf

sub cpu_data {
  my $stat    = shift;
  my $url     = shift;
  my $return  = "";
  my $type    = "node";
  my $metrics = "";
  my $node    = "";
  my $result  = "";
  $return = new_get_request($url);

  if ( $return ne "0" ) {
    if ( defined $return->{'data'}->{'result'} && ref( $return->{'data'}->{'result'} ) eq "ARRAY" ) {
      foreach my $hash ( @{ $return->{'data'}->{'result'} } ) {
        if ( defined $hash->{'metric'}->{'instance'} && $hash->{'metric'}->{'instance'} ne "" ) {
          $node = $hash->{'metric'}->{'instance'};
        }
        if ( defined $hash->{'metric'}->{'mode'} && $hash->{'metric'}->{'mode'} ne "" ) {
          $metrics = $hash->{'metric'}->{'mode'};
        }

        #Calculation
        if ( defined $hash->{'values'}->[0]->[1] && Xormon::isdigit( $hash->{'values'}->[0]->[1] ) && defined $hash->{'values'}->[1]->[1] && Xormon::isdigit( $hash->{'values'}->[1]->[1] ) ) {
          my $diff = ( $hash->{'values'}->[1]->[1] - $hash->{'values'}->[0]->[1] );
          $result = $diff / 300 * 100;
          if ( $stat ne "" && $metrics ne "" && $node ne "" && $result ne "" && Xormon::isdigit($result) ) {
            $data{'perf'}{$type}{$node}{$stat}{$metrics} = sprintf( '%.2f', $result );
          }
        }
        else {
          $logger->error("$stat data not defined \n");
        }
      }
    }
  }
}

sub get_data {
  my $type    = shift;    #node
  my $stat    = shift;    #network
  my $metrics = shift;    #read
  my $node    = shift;    #sg-01w
  my $url     = shift;
  my $return  = "";
  my $exp     = "";
  $return = new_get_request($url);

  if ( $return ne "0" ) {
    if ( defined $return->{'data'}->{'result'} && ref( $return->{'data'}->{'result'} ) eq "ARRAY" ) {

      #$timestamp = $return->{'data'}->{'result'}->[0]->{'values'}->[0]->[0];
      foreach my $result ( @{ $return->{'data'}->{'result'} } ) {
        if ( defined $result->{'values'} && ref( $result->{'values'} ) eq "ARRAY" ) {
          my $poc = 0;
          foreach my $values ( @{ $result->{'values'} } ) {
            if ( Xormon::isdigit($exp) ) {
              $poc++;
              $exp += $values->[1];
            }
            else {
              $poc++;
              $exp = $values->[1];
            }
          }
          $exp = $exp / $poc;
          $data{'perf'}{$type}{$node}{$stat}{$metrics} = sprintf( '%.2f', $exp );
        }
      }
    }
  }
}

sub get_data_rate {
  my $type    = "node";    #node
  my $stat    = shift;     #network
  my $metrics = shift;     #read
  my $url     = shift;
  my $node    = "";        #sg-01w
  my $result  = "";
  my $device  = "";
  my $return  = "";
  my $vysl    = "";
  $return = new_get_request($url);

  if ( $return ne "0" ) {
    if ( defined $return->{'data'}->{'result'} && ref( $return->{'data'}->{'result'} ) eq "ARRAY" ) {
      foreach my $hash ( @{ $return->{'data'}->{'result'} } ) {
        if ( defined $hash->{'metric'}->{'instance'} && $hash->{'metric'}->{'instance'} ne "" ) {
          $node = $hash->{'metric'}->{'instance'};
        }
        if ( defined $hash->{'metric'}->{'device'} && $hash->{'metric'}->{'device'} ne "" ) {
          $device = $hash->{'metric'}->{'device'};
        }
        if ( defined $hash->{'values'} && ref( $hash->{'values'} ) eq "ARRAY" ) {
          my $poc    = 0;
          my $result = "";
          foreach my $values ( @{ $hash->{'values'} } ) {
            if ( Xormon::isdigit($result) ) {
              $poc++;
              $result += $values->[1];
            }
            else {
              $poc++;
              $result = $values->[1];
            }
          }
          $vysl = $result / $poc;
        }
        if ( $node ne "" && $vysl ne "" && $device ne "" ) {
          $data{'perf'}{$type}{$node}{$stat}{$device}{$metrics} = sprintf( '%.2f', $vysl );
        }
        elsif ( $node ne "" && $vysl ne "" ) {
          $data{'perf'}{$type}{$node}{$stat}{$metrics} = sprintf( '%.2f', $vysl );
        }
      }
    }
  }
}

sub get_data_rate_before {
  my $type    = "node";    #node
  my $stat    = shift;     #network
  my $metrics = shift;     #read
  my $url     = shift;
  my $node    = "";        #sg-01w
  my $result  = "";
  my $device  = "";
  my $return  = "";
  my $vysl    = "";
  $return = new_get_request($url);

  if ( $return ne "0" ) {
    if ( defined $return->{'data'}->{'result'} && ref( $return->{'data'}->{'result'} ) eq "ARRAY" ) {
      foreach my $hash ( @{ $return->{'data'}->{'result'} } ) {
        if ( defined $hash->{'metric'}->{'instance'} && $hash->{'metric'}->{'instance'} ne "" ) {
          $node = $hash->{'metric'}->{'instance'};
        }
        if ( defined $hash->{'metric'}->{'device'} && $hash->{'metric'}->{'device'} ne "" ) {
          $device = $hash->{'metric'}->{'device'};
        }
        if ( defined $hash->{'values'} && ref( $hash->{'values'} ) eq "ARRAY" ) {
          my $poc    = 0;
          my $result = "";
          foreach my $values ( @{ $hash->{'values'} } ) {
            if ( Xormon::isdigit($result) ) {
              $poc++;
              $result += $values->[1];
            }
            else {
              $poc++;
              $result = $values->[1];
            }
          }
          $vysl = $result / $poc;
        }
        if ( $node ne "" && $vysl ne "" && $device ne "" ) {
          $data{'perf'}{'before'}{$type}{$node}{$stat}{$device}{$metrics} = sprintf( '%.2f', $vysl );
        }
        elsif ( $node ne "" && $vysl ne "" ) {
          $data{'perf'}{'before'}{$type}{$node}{$stat}{$metrics} = sprintf( '%.2f', $vysl );
        }
      }
    }
  }
}

sub print_conf {
  my @architecture;
  my $st_serial = $cfg->{'hostcfg_id'};
  my %conf;

  $conf{ $cfg->{'hostcfg_id'} }{'node_name'} = $cfg->{'nameGrid'};
  $conf{ $cfg->{'hostcfg_id'} }{'model'} = "StorageGRID";
  if ( exists $data{'conf'}{'global'}{'apiVersion'} ) {
    $conf{ $cfg->{'hostcfg_id'} }{'apiVersion'} = $data{'conf'}{'global'}{'apiVersion'};
  }
  if ( defined $data{'conf'}{'global'}{'productVersion'} && $data{'conf'}{'global'}{'productVersion'} ne "" ) {
    $conf{ $cfg->{'hostcfg_id'} }{'version'} = $data{'conf'}{'global'}{'productVersion'};
  }
  if ( defined $data{'conf'}{'grid'}{$cfg->{'nameGrid'}}{'id'} && $data{'conf'}{'grid'}{$cfg->{'nameGrid'}}{'id'} ne "" ) {
    $conf{ $cfg->{'hostcfg_id'} }{'grid_id'} = $data{'conf'}{'grid'}{$cfg->{'nameGrid'}}{'id'};
  }
  $conf{ $cfg->{'hostcfg_id'} }{'ip'} = $cfg->{'apiHost'};

  #SYSTEM STORAGE CONFIGURATION";
  my %device_arch = (
    "item_id" => $cfg->{'hostcfg_id'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => "storagegrid",
    "subsystem" => "device",
    "hostcfg_id" => "$cfg->{'hostcfg_id'}",
    "parents" => []
  );

  push(@architecture, \%device_arch);

  #Site
  #print CSITE "\nSite Level Configuration\n";
  #print CSITE "--------------------------\n";
  #print CSITE "name,id,capacity_used,capacity_total,capacity_free,metadata_used,metadata_total,metadata_free,data_used\n";
  if ( @gsites && @gsites ne "" ) {
    # print Dumper $cfg->{'sites'};
    foreach my $sit (@gsites) {
      my $cap_used   = "";
      my $cap_total  = "";
      my $cap_free   = "";
      my $meta_used  = "";
      my $meta_total = "";
      my $meta_free  = "";
      my $data_used  = "";
      my $sit_id     = "";
      if ( defined $data{'conf'}{'vdc'}{$sit}{'id'} && $data{'conf'}{'vdc'}{$sit}{'id'} ne "" ) {
        $sit_id = $data{'conf'}{'vdc'}{$sit}{'id'};
      }
      if ( defined $data{'conf'}{'vdc'}{$sit}{'capacity'}{'used'} && $data{'conf'}{'vdc'}{$sit}{'capacity'}{'used'} ne "" ) {
        $cap_used = $data{'conf'}{'vdc'}{$sit}{'capacity'}{'used'};
      }
      if ( defined $data{'conf'}{'vdc'}{$sit}{'capacity'}{'total'} && $data{'conf'}{'vdc'}{$sit}{'capacity'}{'total'} ne "" ) {
        $cap_total = $data{'conf'}{'vdc'}{$sit}{'capacity'}{'total'};
      }
      if ( defined $data{'conf'}{'vdc'}{$sit}{'capacity'}{'free'} && $data{'conf'}{'vdc'}{$sit}{'capacity'}{'free'} ne "" ) {
        $cap_free = $data{'conf'}{'vdc'}{$sit}{'capacity'}{'free'};
      }
      if ( defined $data{'conf'}{'vdc'}{$sit}{'metadata'}{'used'} && $data{'conf'}{'vdc'}{$sit}{'metadata'}{'used'} ne "" ) {
        $meta_used = $data{'conf'}{'vdc'}{$sit}{'metadata'}{'used'};
      }
      if ( defined $data{'conf'}{'vdc'}{$sit}{'metadata'}{'allowed'} && $data{'conf'}{'vdc'}{$sit}{'metadata'}{'allowed'} ne "" ) {
        $meta_total = $data{'conf'}{'vdc'}{$sit}{'metadata'}{'allowed'};
      }
      if ( defined $data{'conf'}{'vdc'}{$sit}{'metadata'}{'free'} && $data{'conf'}{'vdc'}{$sit}{'metadata'}{'free'} ne "" ) {
        $meta_free = $data{'conf'}{'vdc'}{$sit}{'metadata'}{'free'};
      }
      if ( defined $data{'conf'}{'vdc'}{$sit}{'data'}{'used'} && $data{'conf'}{'vdc'}{$sit}{'data'}{'used'} ne "" ) {
        $data_used = $data{'conf'}{'vdc'}{$sit}{'data'}{'used'};
      }

      #SITE STORAGE CONFIGURATION";
      my $hash_key = Xormon::create_hash_key($st_serial."SITE".$sit_id); $hash_key = lc($hash_key);
      my %device_arch = (
        "item_id" => $hash_key,
        "label" => $sit,
        "hw_type" => "storagegrid",
        "subsystem" => "site",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'}],
      );

      $conf{$hash_key}{'capacity_used'} = $cap_used;
      $conf{$hash_key}{'capacity_total'} = $cap_total;
      $conf{$hash_key}{'capacity_free'} = $cap_free;

      push(@architecture, \%device_arch);


      #print CSITE "$sit,$sit_id,$cap_used,$cap_total,$cap_free,$meta_used,$meta_total,$meta_free,$data_used\n";
    }
  }
  #close(CSITE);
  if ( @gsites && @gsites ne "" ) {
    foreach my $sites (@gsites) {
      my $out_disk  = "";
      my $out_ports = "";
      my $sit_id   = "";
      my $name = "";
      if ( exists $data{'conf'}{'vdc'}{$sites}{'id'} && $data{'conf'}{'vdc'}{$sites}{'id'} ne "" ) {
        $sit_id = $data{'conf'}{'vdc'}{$sites}{'id'};
      }
      if ( exists $data{'conf'}{'vdc'}{$sites}{'name'} && $data{'conf'}{'vdc'}{$sites}{'name'} ne "" ) {
        $name = $data{'conf'}{'vdc'}{$sites}{'name'};
      }
      #print CONF "Configuration Data\n";
      #print CONF "-------------------\n";
      #print CONF "Api Version     : $api\n";
      #print CONF "Product Version : $product\n";
      #print CONF "Storage Name    : $storage_name\n";
      #print CONF "Grid Name       : $cfg->{'nameGrid'}\n";
      #print CONF "Grid id         : $grid_id\n";
      #print CONF "Site name       : $name\n";
      #print CONF "Site id         : $id\n";

      #print CONF "\n";
      #print CONF "Node Level Configuration\n";
      #print CONF "------------------------\n";
      #print CONF "id,name,capacity_used,capacity_total,capacity_free,metadata_used,metadata_total,metadata_free,data_used\n";
      if ( exists $data{'conf'}{'node'} && $data{'conf'}{'node'} ne "" ) {
        foreach my $nodes ( keys( %{ $data{'conf'}{'node'} } ) ) {
          my $site_key = Xormon::create_hash_key($st_serial."SITE".$sit_id); $site_key = lc($site_key);
          if ( $data{'conf'}{'node'}{$nodes}{'vdcName'} ne "$sites" ) {
            next;
          }
          my $id         = "";
          my $cap_used   = "";
          my $cap_total  = "";
          my $cap_free   = "";
          my $meta_used  = "";
          my $meta_total = "";
          my $meta_free  = "";
          my $data_used  = "";

          if ( exists $data{'conf'}{'node'}{$nodes}{'id'} && $data{'conf'}{'node'}{$nodes}{'id'} ne "" ) {
            $id = $data{'conf'}{'node'}{$nodes}{'id'};
          }
          if ( defined $data{'conf'}{'node'}{$nodes}{'capacity'}{'used'} && $data{'conf'}{'node'}{$nodes}{'capacity'}{'used'} ne "" ) {
            $cap_used = $data{'conf'}{'node'}{$nodes}{'capacity'}{'used'};
          }
          if ( defined $data{'conf'}{'node'}{$nodes}{'capacity'}{'total'} && $data{'conf'}{'node'}{$nodes}{'capacity'}{'total'} ne "" ) {
            $cap_total = $data{'conf'}{'node'}{$nodes}{'capacity'}{'total'};
          }
          if ( defined $data{'conf'}{'node'}{$nodes}{'capacity'}{'free'} && $data{'conf'}{'node'}{$nodes}{'capacity'}{'free'} ne "" ) {
            $cap_free = $data{'conf'}{'node'}{$nodes}{'capacity'}{'free'};
          }
          if ( defined $data{'conf'}{'node'}{$nodes}{'metadata'}{'used'} && $data{'conf'}{'node'}{$nodes}{'metadata'}{'used'} ne "" ) {
            $meta_used = $data{'conf'}{'node'}{$nodes}{'metadata'}{'used'};
          }
          if ( defined $data{'conf'}{'node'}{$nodes}{'metadata'}{'allowed'} && $data{'conf'}{'node'}{$nodes}{'metadata'}{'allowed'} ne "" ) {
            $meta_total = $data{'conf'}{'node'}{$nodes}{'metadata'}{'allowed'};
          }
          if ( defined $data{'conf'}{'node'}{$nodes}{'metadata'}{'free'} && $data{'conf'}{'node'}{$nodes}{'metadata'}{'free'} ne "" ) {
            $meta_free = $data{'conf'}{'node'}{$nodes}{'metadata'}{'free'};
          }
          if ( defined $data{'conf'}{'node'}{$nodes}{'data'}{'used'} && $data{'conf'}{'node'}{$nodes}{'data'}{'used'} ne "" ) {
            $data_used = $data{'conf'}{'node'}{$nodes}{'data'}{'used'};
          }

          #DISK
          if ( defined $data{'perf'}{'node'}{$nodes}{'disk'} && $data{'perf'}{'node'}{$nodes}{'disk'} ne "" ) {
            foreach my $disk ( keys( %{ $data{'perf'}{'node'}{$nodes}{'disk'} } ) ) {
              my $my_disk_name = "$nodes-$disk";
              #DISK STORAGE CONFIGURATION";
              my $hash_key = Xormon::create_hash_key($st_serial."DISK".$my_disk_name); $hash_key = lc($hash_key);
              my %drive_arch = (
                "item_id" => $hash_key,
                "label" => $my_disk_name,
                "hw_type" => "storagegrid",
                "subsystem" => "drive",
                "hostcfg_id" => "$cfg->{'hostcfg_id'}",
                "parents" => [$cfg->{'hostcfg_id'},$site_key],
              );

              push(@architecture, \%drive_arch);
            }
          }

          #PORT
          if ( defined $data{'perf'}{'node'}{$nodes}{'interface'} && $data{'perf'}{'node'}{$nodes}{'interface'} ne "" ) {
            foreach my $interface ( keys( %{ $data{'perf'}{'node'}{$nodes}{'interface'} } ) ) {
              my $my_interface_name = "$nodes-$interface";
              $out_ports .= "$my_interface_name,$my_interface_name\n";

              #PORT STORAGE CONFIGURATION";
              my $hash_key = Xormon::create_hash_key($st_serial."PORT".$my_interface_name); $hash_key = lc($hash_key);
              my %device_arch = (
                "item_id" => $hash_key,
                "label" => "$my_interface_name",
                "hw_type" => "storagegrid",
                "subsystem" => "port",
                "hostcfg_id" => "$cfg->{'hostcfg_id'}",
                "parents" => [$cfg->{'hostcfg_id'},$site_key],
              );

              push(@architecture, \%device_arch);
            }
          }
          #NODE STORAGE CONFIGURATION";
          my $hash_key = Xormon::create_hash_key($st_serial."NODE".$id); $hash_key = lc($hash_key);
          my %device_arch = (
            "item_id" => $hash_key,
            "label" => "$nodes",
            "hw_type" => "storagegrid",
            "subsystem" => "node",
            "hostcfg_id" => "$cfg->{'hostcfg_id'}",
            "parents" => [$cfg->{'hostcfg_id'},$site_key],
          );

          push(@architecture, \%device_arch);

          $conf{$hash_key}{'capacity_used'} = $cap_used;
          $conf{$hash_key}{'capacity_total'} = $cap_total;
          $conf{$hash_key}{'capacity_free'} = $cap_free;

          #print CONF "$id,$nodes,$cap_used,$cap_total,$cap_free,$meta_used,$meta_total,$meta_free,$data_used\n";
        }
      }
      #print CONF "\n";
      #print CONF "Disk Level Configuration\n";
      #print CONF "-------------------------\n";
      #print CONF "id,name\n";
      #print CONF "$out_disk";

      #print CONF "\n";
      #print CONF "Port Level Configuration\n";
      #print CONF "------------------------\n";
      #print CONF "id,name\n";
      #print CONF "$out_ports";

    }
  }
  #save configuration
  $xormon->saveConfJSON(\%conf);

  #save architecture
  #print "ARCHITECTURE\n";
  #print Dumper @architecture;
  $xormon->saveArchitecture(\@architecture);
  #close(CONF);
}

sub print_perf {
  my $st_serial = $cfg->{'hostcfg_id'};
  my $data_out = "";
  my $site_out = "";
  my $rc;
  my $first_time       = 0;

  my $status;

  my $device_cap_total = "";
  my $device_cap_used = "";
  my $device_cap_free = "";
  my $device_used_pct = "";

  if (@gsites) {
    $xormon->hasData(1); ## it has some sites
  }
  else {
    $xormon->hasData(0, "No sites found");
  }

  foreach my $sites (@gsites) {
    my ( $data_nodes, $metadata_nodes, $cpu_nodes, $load_nodes, $mem_nodes, $network_nodes, $s3_nodes, $swift_nodes, $ilm_nodes, $lb_nodes, $data_total_nodes ) = (0) x 11;
    my ( $node_out, $disk_out, $port_out) = ("") x 3;
    foreach my $nodes ( keys( %{ $data{'perf'}{'node'} } ) ) {
      if ( $data{'conf'}{'node'}{$nodes}{'vdcName'} ne "$sites" ) {
        next;
      }
      my $network_receive_error  = "";
      my $network_transmit_error = "";
      my $used_cap        = "";
      my $metadata_free_bytes    = "";

      #DISK
      if ( defined $data{'perf'}{'node'}{$nodes}{'disk'} && $data{'perf'}{'node'}{$nodes}{'disk'} ne "" ) {
        foreach my $disk ( keys( %{ $data{'perf'}{'node'}{$nodes}{'disk'} } ) ) {
          my ( $disk_write_time, $disk_data_write, $disk_io_write, $disk_data_read, $disk_read_time, $disk_io_read, $disk_read_response_time, $disk_write_response_time, $disk_io_rate, $disk_data_rate ) = ("") x 10;
          my $my_disk_name           = "$nodes-$disk";
          my $disk_write_time_now    = "";
          my $disk_write_time_before = "";
          if ( exists $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'write_time'} && $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'write_time'} ne "" ) {
            $disk_write_time_now = sprintf( '%.2f', ( $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'write_time'} ) );

            if ( defined $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'write_time'} && $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'write_time'} ne "" ) {
              $disk_write_time_before = sprintf( '%.2f', ( $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'write_time'} ) );
              my $disk_write_time_disc = ( $disk_write_time_now - $disk_write_time_before );
              if ( $disk_write_time_disc < 0 ) {
                $disk_write_time_disc = ( $disk_write_time_disc * -1 );
              }

              #FINAL FOR PERF AND RESPONSE TIME
              $disk_write_time = ( $disk_write_time_disc * 1000 );
            }
          }
          if ( exists $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'data_write'} && $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'data_write'} ne "" ) {
            $disk_data_write = $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'data_write'};
            $disk_data_rate = $disk_data_write;
          }
          if ( exists $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_write'} && $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_write'} ne "" ) {
            $disk_io_write = sprintf( '%.2f', ( $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_write'} ) );
            $disk_io_rate = $disk_io_write;
          }
          if ( exists $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'data_read'} && $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'data_read'} ne "" ) {
            $disk_data_read = $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'data_read'};
            if (Xormon::isdigit($disk_data_rate)){
              $disk_data_rate += $disk_data_read;
            }
            else{
              $disk_data_rate = $disk_data_read;
            }
          }
          my $disk_read_time_now    = "";
          my $disk_read_time_before = "";
          if ( exists $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'read_time'} && $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'read_time'} ne "" ) {
            $disk_read_time_now = sprintf( '%.2f', ( $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'read_time'} ) );

            if ( defined $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'read_time'} && $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'read_time'} ne "" ) {
              $disk_read_time_before = sprintf( '%.2f', ( $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'read_time'} ) );
              my $disk_read_time_disc = ( $disk_read_time_now - $disk_read_time_before );
              if ( $disk_read_time_disc < 0 ) {
                $disk_read_time_disc = ( $disk_read_time_disc * -1 );
              }

              #FINAL FOR PERF AND RESPONSE TIME
              $disk_read_time = ( $disk_read_time_disc * 1000 );
            }
          }
          if ( exists $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_read'} && $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_read'} ne "" ) {
            $disk_io_read = sprintf( '%.2f', ( $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_read'} ) );
            if (Xormon::isdigit($disk_io_rate)){
              $disk_io_rate += $disk_io_read;
            }
            else{
              $disk_io_rate = $disk_io_read;
            }
          }

          ###IO TOTAL FOR RESPONSE TIME
          ###READ
          my $disk_io_read_total  = "";
          my $disk_io_read_now    = "";
          my $disk_io_read_before = "";
          if ( defined $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_read_total'} && $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_read_total'} ne "" ) {
            $disk_io_read_now = $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_read_total'};
            if ( defined $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'io_read_total'} && $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'io_read_total'} ne "" ) {
              $disk_io_read_before = $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'io_read_total'};
            }
            if ( Xormon::isdigit($disk_io_read_now) && Xormon::isdigit($disk_io_read_before) ) {
              $disk_io_read_total = ( $disk_io_read_now - $disk_io_read_before );
            }
          }
          if ( Xormon::isdigit($disk_read_time) && Xormon::isdigit($disk_io_read_total) && $disk_io_read_total != 0 ) {
            $disk_read_response_time = $disk_read_time / $disk_io_read_total;
          }

          ####WRITE
          my $disk_io_write_total  = "";
          my $disk_io_write_now    = "";
          my $disk_io_write_before = "";
          if ( defined $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_write_total'} && $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_write_total'} ne "" ) {
            $disk_io_write_now = $data{'perf'}{'node'}{$nodes}{'disk'}{$disk}{'io_write_total'};
            if ( defined $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'io_write_total'} && $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'io_write_total'} ne "" ) {
              $disk_io_write_before = $data{'perf'}{'before'}{'node'}{$nodes}{'disk'}{$disk}{'io_write_total'};
            }
            if ( Xormon::isdigit($disk_io_write_now) && Xormon::isdigit($disk_io_write_before) ) {
              $disk_io_write_total = ( $disk_io_write_now - $disk_io_write_before );
            }
          }
          if ( Xormon::isdigit($disk_write_time) && Xormon::isdigit($disk_io_write_total) && $disk_io_write_total != 0 ) {
            $disk_write_response_time = $disk_write_time / $disk_io_write_total;
          }

          my $hash_key = Xormon::create_hash_key($st_serial."DISK".$my_disk_name); $hash_key = lc($hash_key);
          $disk_out .= "$hash_key;$cfg->{'timestamp'};$disk_data_write;$disk_io_write;$disk_data_read;$disk_io_read;$disk_read_response_time;$disk_write_response_time;$disk_io_rate;$disk_data_rate\n";
        }
      }

      #PORT
      if ( defined $data{'perf'}{'node'}{$nodes}{'interface'} && $data{'perf'}{'node'}{$nodes}{'interface'} ne "" ) {
        foreach my $interface ( keys( %{ $data{'perf'}{'node'}{$nodes}{'interface'} } ) ) {
          my ( $interface_transmit_errs, $interface_received_drop, $interface_transmit_drop, $interface_received_bytes, $interface_transmit_bytes, $interface_received_errs, $data_rate, $error_rate, $drop_rate ) = ("") x 9;
          my $my_interface_name = "$nodes-$interface";
          if ( exists $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'transmit_errs'} && $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'transmit_errs'} ne "" ) {
            $interface_transmit_errs = sprintf( '%.2f', ( $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'transmit_errs'} ) );
            if ( Xormon::isdigit($network_transmit_error) && Xormon::isdigit($interface_transmit_errs) ) {
              $network_transmit_error += $interface_transmit_errs;
            }
            else {
              $network_transmit_error = $interface_transmit_errs;
            }
            $error_rate = $interface_transmit_errs;
          }
          if ( exists $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'received_drop'} && $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'received_drop'} ne "" ) {
            $interface_received_drop = sprintf( '%.2f', ( $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'received_drop'} ) );
            $drop_rate = $interface_received_drop;
          }
          if ( exists $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'transmit_drop'} && $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'transmit_drop'} ne "" ) {
            $interface_transmit_drop = sprintf( '%.2f', ( $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'transmit_drop'} ) );
            if (Xormon::isdigit($drop_rate)){
              $drop_rate += $interface_transmit_drop;
            }
            else{
              $drop_rate = $interface_transmit_drop;
            }
          }
          if ( exists $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'received_bytes'} && $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'received_bytes'} ne "" ) {
            $interface_received_bytes = sprintf( '%.2f', $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'received_bytes'} );
            $data_rate = $interface_received_bytes;
          }
          if ( exists $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'transmit_bytes'} && $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'transmit_bytes'} ne "" ) {
            $interface_transmit_bytes = sprintf( '%.2f', $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'transmit_bytes'} );
            if (Xormon::isdigit($data_rate)){
              $data_rate += $interface_transmit_bytes;
            }
            else{
              $data_rate = $interface_transmit_bytes;
            }
          }
          if ( exists $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'received_errs'} && $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'received_errs'} ne "" ) {
            $interface_received_errs = $data{'perf'}{'node'}{$nodes}{'interface'}{$interface}{'received_errs'};
            if ( Xormon::isdigit($network_receive_error) && Xormon::isdigit($interface_received_errs) ) {
              $network_receive_error += $interface_received_errs;
            }
            else {
              $network_receive_error = $interface_received_errs;
            }
            if (Xormon::isdigit($error_rate)){
              $error_rate += $interface_received_errs;
            }
            else{
              $error_rate = $interface_received_errs;
            }
          }
          my $hash_key = Xormon::create_hash_key($st_serial."PORT".$my_interface_name); $hash_key = lc($hash_key);
          $port_out .= "$hash_key;$cfg->{'timestamp'};$interface_transmit_errs;$interface_received_drop;$interface_transmit_drop;$interface_received_bytes;$interface_transmit_bytes;$interface_received_errs;$data_rate;$error_rate;$drop_rate\n";

          #print PERF "$my_interface_name;$interface;$interface_transmit_errs;$interface_received_drop;$interface_transmit_drop;$interface_received_bytes;$interface_transmit_bytes;$interface_received_errs\n";
        }
      }

      #NODE
      my ( $s3_total, $s3_ingested_bytes, $s3_operations_failed, $s3_operations_successful, $s3_retrieved_bytes, $s3_operations_unauthorized, $network_received_rate, $network_sent_rate, $net_rate, $metadata_allowed_bytes, $metadata_used_bytes, $total_cap, $free_cap, $capacity_used_percent, $ilm_awaiting_client, $ilm_scan_rate, $ilm_awaiting_background, $ilm_awaiting_evaluation_rate, $ilm_awaiting_all, $swift_operations_failed, $swift_operations_successful, $swift_ingested_bytes, $swift_operations_unauthorized, $swift_retrieved_bytes, $memory_util, $memory_used_prct, $memory_used, $memory_free, $memory_total, $cpu_utilization, $cpu_system, $cpu_iowait, $cpu_user, $cpu_steal, $cpu_softirq, $cpu_nice, $cpu_irq, $cpu_idle, $lb_total, $load_balancer_sent, $load_balancer_in_req_rate, $load_balancer_err_resp_rate, $load_balancer_received, $load, $data_totals, $node_error_rate ) = ("") x 46;
      my $node_id = "";
      if ( exists $data{'conf'}{'node'}{$nodes}{'id'} && $data{'conf'}{'node'}{$nodes}{'id'} ne "" ) {
        $node_id = $data{'conf'}{'node'}{$nodes}{'id'};
      }

      #S3
      if ( defined $data{'perf'}{'node'}{$nodes}{'s3'} && $data{'perf'}{'node'}{$nodes}{'s3'} ne "" ) {
        $s3_nodes++;
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'s3'}{'ingested_bytes'} && $data{'perf'}{'node'}{$nodes}{'s3'}{'ingested_bytes'} ne "" ) {
        $s3_ingested_bytes = $data{'perf'}{'node'}{$nodes}{'s3'}{'ingested_bytes'};
        if (Xormon::isdigit($s3_ingested_bytes)){
          $s3_total = $s3_ingested_bytes;
        }

        #for site totals
        data_for_site( "$sites", "s3", "ingest", "$s3_ingested_bytes" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'s3'}{'operations_failed'} && $data{'perf'}{'node'}{$nodes}{'s3'}{'operations_failed'} ne "" ) {
        $s3_operations_failed = sprintf( '%.2f', ( $data{'perf'}{'node'}{$nodes}{'s3'}{'operations_failed'} ) );

        #for site totals
        data_for_site( "$sites", "s3", "failed", "$s3_operations_failed" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'s3'}{'operations_successful'} && $data{'perf'}{'node'}{$nodes}{'s3'}{'operations_successful'} ne "" ) {
        $s3_operations_successful = sprintf( '%.2f', ( $data{'perf'}{'node'}{$nodes}{'s3'}{'operations_successful'} ) );

        #for site totals
        data_for_site( "$sites", "s3", "successful", "$s3_operations_successful" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'s3'}{'retrieved_bytes'} && $data{'perf'}{'node'}{$nodes}{'s3'}{'retrieved_bytes'} ne "" ) {
        $s3_retrieved_bytes = $data{'perf'}{'node'}{$nodes}{'s3'}{'retrieved_bytes'};
        if ( Xormon::isdigit($s3_total) ) {
          $s3_total += $s3_retrieved_bytes;
        }
        else {
          $s3_total = $s3_retrieved_bytes;
        }

        #for site totals
        data_for_site( "$sites", "s3", "retrieve", "$s3_retrieved_bytes" );
        data_for_site( "$sites", "s3", "total", "$s3_total" );

      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'s3'}{'operations_unauthorized'} && $data{'perf'}{'node'}{$nodes}{'s3'}{'operations_unauthorized'} ne "" ) {
        $s3_operations_unauthorized = sprintf( '%.2f', ( $data{'perf'}{'node'}{$nodes}{'s3'}{'operations_unauthorized'} ) );

        #for site totals
        data_for_site( "$sites", "s3", "unauthorized", "$s3_operations_unauthorized" );
      }

      #Network
      if ( defined $data{'perf'}{'node'}{$nodes}{'network'} && $data{'perf'}{'node'}{$nodes}{'network'} ne "" ) {
        $network_nodes++;
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'network'}{'received_rate'} && $data{'perf'}{'node'}{$nodes}{'network'}{'received_rate'} ne "" ) {
        $network_received_rate = $data{'perf'}{'node'}{$nodes}{'network'}{'received_rate'};
        $net_rate = $network_received_rate;

        #for site totals
        data_for_site( "$sites", "network", "rx", "$network_received_rate" );
      }

      if ( exists $data{'perf'}{'node'}{$nodes}{'network'}{'sent_rate'} && $data{'perf'}{'node'}{$nodes}{'network'}{'sent_rate'} ne "" ) {
        $network_sent_rate = $data{'perf'}{'node'}{$nodes}{'network'}{'sent_rate'};
        if (Xormon::isdigit($net_rate)){
          $net_rate += $network_sent_rate;
        }
        else{
          $net_rate = $network_sent_rate;
        }

        #for site totals
        data_for_site( "$sites", "network", "tx", "$network_sent_rate" );
        data_for_site( "$sites", "network", "net_rate", "$net_rate" );
      }

      #Totals
      if ( defined $data{'perf'}{'node'}{$nodes}{'data_total'} && $data{'perf'}{'node'}{$nodes}{'data_total'} ne "" ) {
        $data_total_nodes++;

        if ( defined $data{'perf'}{'node'}{$nodes}{'data_total'}{'total'} && $data{'perf'}{'node'}{$nodes}{'data_total'}{'total'} ne "" ) {
          $data_totals = $data{'perf'}{'node'}{$nodes}{'data_total'}{'total'};
          data_for_site( "$sites", "data_total", "total", "$data_totals" );
        }
      }

      #Metadata
      if ( defined $data{'perf'}{'node'}{$nodes}{'metadata'} && $data{'perf'}{'node'}{$nodes}{'metadata'} ne "" ) {
        $metadata_nodes++;
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'metadata'}{'allowed_bytes'} && $data{'perf'}{'node'}{$nodes}{'metadata'}{'allowed_bytes'} ne "" ) {
        $metadata_allowed_bytes = $data{'perf'}{'node'}{$nodes}{'metadata'}{'allowed_bytes'};
        data_for_site( "$sites", "metadata", "total", "$metadata_allowed_bytes" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'metadata'}{'used_bytes'} && $data{'perf'}{'node'}{$nodes}{'metadata'}{'used_bytes'} ne "" ) {
        $metadata_used_bytes = $data{'perf'}{'node'}{$nodes}{'metadata'}{'used_bytes'};
        data_for_site( "$sites", "metadata", "used", "$metadata_used_bytes" );
      }
      if ( Xormon::isdigit($metadata_allowed_bytes) && Xormon::isdigit($metadata_used_bytes) ) {
        $metadata_free_bytes = $metadata_allowed_bytes - $metadata_used_bytes;
        data_for_site( "$sites", "metadata", "free", "$metadata_free_bytes" );
      }

      #Data
      if ( defined $data{'perf'}{'node'}{$nodes}{'data'} && $data{'perf'}{'node'}{$nodes}{'data'} ne "" ) {
        $data_nodes++;
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'data'}{'total_cap'} && $data{'perf'}{'node'}{$nodes}{'data'}{'total_cap'} ne "" ) {
        $total_cap = $data{'perf'}{'node'}{$nodes}{'data'}{'total_cap'};
        data_for_site( "$sites", "data", "total", "$total_cap" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'data'}{'free_cap'} && $data{'perf'}{'node'}{$nodes}{'data'}{'free_cap'} ne "" ) {
        $free_cap = $data{'perf'}{'node'}{$nodes}{'data'}{'free_cap'};
        data_for_site( "$sites", "data", "free", "$free_cap" );
      }
      if ( Xormon::isdigit($total_cap) && Xormon::isdigit($free_cap) ) {
        $used_cap = ( $total_cap - $free_cap );
        $capacity_used_percent = Xormon::formatNumberByType("float4", $used_cap / ($total_cap / 100) );
        data_for_site( "$sites", "data", "used", "$used_cap" );
      }

      #ilm
      if ( defined $data{'perf'}{'node'}{$nodes}{'ilm'} && $data{'perf'}{'node'}{$nodes}{'ilm'} ne "" ) {
        $ilm_nodes++;
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_client'} && $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_client'} ne "" ) {
        $ilm_awaiting_client = $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_client'};
        data_for_site( "$sites", "ilm", "awaiting_client", "$ilm_awaiting_client" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'ilm'}{'scan_rate'} && $data{'perf'}{'node'}{$nodes}{'ilm'}{'scan_rate'} ne "" ) {
        $ilm_scan_rate = $data{'perf'}{'node'}{$nodes}{'ilm'}{'scan_rate'};
        data_for_site( "$sites", "ilm", "scan_rate", "$ilm_scan_rate" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_background'} && $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_background'} ne "" ) {
        $ilm_awaiting_background = $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_background'};
        data_for_site( "$sites", "ilm", "awaiting_background", "$ilm_awaiting_background" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_evaluation_rate'} && $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_evaluation_rate'} ne "" ) {
        $ilm_awaiting_evaluation_rate = $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_evaluation_rate'};
        data_for_site( "$sites", "ilm", "eval_rate", "$ilm_awaiting_evaluation_rate" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_all'} && $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_all'} ne "" ) {
        $ilm_awaiting_all = $data{'perf'}{'node'}{$nodes}{'ilm'}{'awaiting_all'};
        data_for_site( "$sites", "ilm", "awaiting_all", "$ilm_awaiting_all" );
      }

      #swift
      if ( defined $data{'perf'}{'node'}{$nodes}{'swift'} && $data{'perf'}{'node'}{$nodes}{'swift'} ne "" ) {
        $swift_nodes++;
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'swift'}{'operations_failed'} && $data{'perf'}{'node'}{$nodes}{'swift'}{'operations_failed'} ne "" ) {
        $swift_operations_failed = $data{'perf'}{'node'}{$nodes}{'swift'}{'operations_failed'};

        #for site totals
        data_for_site( "$sites", "swift", "failed", "$swift_operations_failed" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'swift'}{'operations_successful'} && $data{'perf'}{'node'}{$nodes}{'swift'}{'operations_successful'} ne "" ) {
        $swift_operations_successful = $data{'perf'}{'node'}{$nodes}{'swift'}{'operations_successful'};

        #for site totals
        data_for_site( "$sites", "swift", "successful", "$swift_operations_successful" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'swift'}{'ingested_bytes'} && $data{'perf'}{'node'}{$nodes}{'swift'}{'ingested_bytes'} ne "" ) {
        $swift_ingested_bytes = $data{'perf'}{'node'}{$nodes}{'swift'}{'ingested_bytes'};

        #for site totals
        data_for_site( "$sites", "swift", "ingest", "$swift_ingested_bytes" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'swift'}{'operations_unauthorized'} && $data{'perf'}{'node'}{$nodes}{'swift'}{'operations_unauthorized'} ne "" ) {
        $swift_operations_unauthorized = $data{'perf'}{'node'}{$nodes}{'swift'}{'operations_unauthorized'};

        #for site totals
        data_for_site( "$sites", "swift", "unauthorized", "$swift_operations_unauthorized" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'swift'}{'retrieved_bytes'} && $data{'perf'}{'node'}{$nodes}{'swift'}{'retrieved_bytes'} ne "" ) {
        $swift_retrieved_bytes = $data{'perf'}{'node'}{$nodes}{'swift'}{'retrieved_bytes'};

        #for site totals
        data_for_site( "$sites", "swift", "retrieve", "$swift_retrieved_bytes" );
      }

      #memory
      if ( defined $data{'perf'}{'node'}{$nodes}{'memory'} && $data{'perf'}{'node'}{$nodes}{'memory'} ne "" ) {
        $mem_nodes++;
      }
      # memory util is calculated from total and used
      # if ( exists $data{'perf'}{'node'}{$nodes}{'memory'}{'used_prct'} && $data{'perf'}{'node'}{$nodes}{'memory'}{'used_prct'} ne "" ) {
      #   $memory_used_prct = $data{'perf'}{'node'}{$nodes}{'memory'}{'used_prct'};
      # }
      if ( exists $data{'perf'}{'node'}{$nodes}{'memory'}{'used'} && $data{'perf'}{'node'}{$nodes}{'memory'}{'used'} ne "" ) {
        $memory_used = $data{'perf'}{'node'}{$nodes}{'memory'}{'used'};
        data_for_site( "$sites", "memory", "used", "$memory_used" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'memory'}{'total'} && $data{'perf'}{'node'}{$nodes}{'memory'}{'total'} ne "" ) {
        $memory_total = $data{'perf'}{'node'}{$nodes}{'memory'}{'total'};
        data_for_site( "$sites", "memory", "total", "$memory_total" );
      }
      if ( Xormon::isdigit($memory_used) && Xormon::isdigit($memory_total) && $memory_total != 0 ) {
        $memory_util = sprintf( '%.2f', ( $memory_used / ( $memory_total / 100 ) ) );
        data_for_site( "$sites", "memory", "utilization", "$memory_util" );
      }
      # using difference from total because used and free did not get total right
      if ( Xormon::isdigit($memory_total) && Xormon::isdigit($memory_used) ) {
        $memory_free = $memory_total - $memory_used;
        data_for_site( "$sites", "memory", "free", "$memory_free" );
      }

      #cpu
      if ( defined $data{'perf'}{'node'}{$nodes}{'cpu'} && $data{'perf'}{'node'}{$nodes}{'cpu'} ne "" ) {
        $cpu_nodes++;
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'cpu'}{'utilization'} && $data{'perf'}{'node'}{$nodes}{'cpu'}{'utilization'} ne "" ) {
        $cpu_utilization = $data{'perf'}{'node'}{$nodes}{'cpu'}{'utilization'};
        data_for_site( "$sites", "cpu", "utilization", "$cpu_utilization" );
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'cpu'}{'system'} && $data{'perf'}{'node'}{$nodes}{'cpu'}{'system'} ne "" ) {
        $cpu_system = $data{'perf'}{'node'}{$nodes}{'cpu'}{'system'};
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'cpu'}{'iowait'} && $data{'perf'}{'node'}{$nodes}{'cpu'}{'iowait'} ne "" ) {
        $cpu_iowait = $data{'perf'}{'node'}{$nodes}{'cpu'}{'iowait'};
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'cpu'}{'user'} && $data{'perf'}{'node'}{$nodes}{'cpu'}{'user'} ne "" ) {
        $cpu_user = $data{'perf'}{'node'}{$nodes}{'cpu'}{'user'};
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'cpu'}{'steal'} && $data{'perf'}{'node'}{$nodes}{'cpu'}{'steal'} ne "" ) {
        $cpu_steal = $data{'perf'}{'node'}{$nodes}{'cpu'}{'steal'};
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'cpu'}{'softirq'} && $data{'perf'}{'node'}{$nodes}{'cpu'}{'softirq'} ne "" ) {
        $cpu_softirq = $data{'perf'}{'node'}{$nodes}{'cpu'}{'softirq'};
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'cpu'}{'nice'} && $data{'perf'}{'node'}{$nodes}{'cpu'}{'nice'} ne "" ) {
        $cpu_nice = $data{'perf'}{'node'}{$nodes}{'cpu'}{'nice'};
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'cpu'}{'irq'} && $data{'perf'}{'node'}{$nodes}{'cpu'}{'irq'} ne "" ) {
        $cpu_irq = $data{'perf'}{'node'}{$nodes}{'cpu'}{'irq'};
      }
      if ( exists $data{'perf'}{'node'}{$nodes}{'cpu'}{'idle'} && $data{'perf'}{'node'}{$nodes}{'cpu'}{'idle'} ne "" ) {
        $cpu_idle = $data{'perf'}{'node'}{$nodes}{'cpu'}{'idle'};
      }

      #load_balancer
      if ( defined $data{'perf'}{'node'}{$nodes}{'load_balancer'} && $data{'perf'}{'node'}{$nodes}{'load_balancer'} ne "" ) {
        $lb_nodes++;
      }
      if ( defined $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'sent'} && $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'sent'} ne "" ) {
        $load_balancer_sent = $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'sent'};
        data_for_site( "$sites", "load_balancer", "tx", "$load_balancer_sent" );
      }
      if ( defined $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'in_req_rate'} && $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'in_req_rate'} ne "" ) {
        $load_balancer_in_req_rate = $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'in_req_rate'};
        data_for_site( "$sites", "load_balancer", "request_rate", "$load_balancer_in_req_rate" );
      }
      if ( defined $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'err_resp_rate'} && $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'err_resp_rate'} ne "" ) {
        $load_balancer_err_resp_rate = $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'err_resp_rate'};
        data_for_site( "$sites", "load_balancer", "error_rate", "$load_balancer_err_resp_rate" );
      }
      if ( defined $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'received'} && $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'received'} ne "" ) {
        $load_balancer_received = $data{'perf'}{'node'}{$nodes}{'load_balancer'}{'received'};
        data_for_site( "$sites", "load_balancer", "rx", "$load_balancer_received" );
      }

      #load
      if ( defined $data{'perf'}{'node'}{$nodes}{'load'} && $data{'perf'}{'node'}{$nodes}{'load'} ne "" ) {
        $load_nodes++;
      }
      if ( defined $data{'perf'}{'node'}{$nodes}{'load'}{'load'} && $data{'perf'}{'node'}{$nodes}{'load'}{'load'} ne "" ) {
        $load = $data{'perf'}{'node'}{$nodes}{'load'}{'load'};
        data_for_site( "$sites", "load", "load", "$load" );
      }


      if (Xormon::isdigit($network_receive_error) && Xormon::isdigit($network_transmit_error)) {
        $node_error_rate = $network_receive_error + $network_transmit_error;
      }

      if ( !defined $network_receive_error ) {
        my $network_receive_error = "";
      }
      if ( !defined $network_transmit_error ) {
        my $network_transmit_error = "";
      }
      if ( !defined $metadata_free_bytes ) {
        my $metadata_free_bytes = "";
      }
      if ( !defined $used_cap ) {
        my $used_cap = "";
      }

      # #Capacity for Conf File
      # $data{'conf'}{'node'}{$nodes}{'metadata'}{'allowed'} = $metadata_allowed_bytes;
      # $data{'conf'}{'node'}{$nodes}{'metadata'}{'used'}    = $metadata_used_bytes;
      # $data{'conf'}{'node'}{$nodes}{'metadata'}{'free'}    = $metadata_free_bytes;
      # $data{'conf'}{'node'}{$nodes}{'capacity'}{'total'}   = $total_cap;
      # $data{'conf'}{'node'}{$nodes}{'capacity'}{'used'}    = $free_cap;
      # $data{'conf'}{'node'}{$nodes}{'capacity'}{'free'}    = $used_cap;
      # $data{'conf'}{'node'}{$nodes}{'data'}{'used'}        = $data_totals;

      my $hash_key = Xormon::create_hash_key($st_serial."NODE".$node_id); $hash_key = lc($hash_key);
      $node_out .= "$hash_key;$cfg->{'timestamp'};$s3_total;$s3_ingested_bytes;$s3_operations_failed;$s3_operations_successful;$s3_retrieved_bytes;$s3_operations_unauthorized;$network_received_rate;$network_sent_rate;$net_rate;$network_receive_error;$network_transmit_error;$node_error_rate;$metadata_allowed_bytes;$metadata_used_bytes;$metadata_free_bytes;$total_cap;$used_cap;$free_cap;$capacity_used_percent;$ilm_awaiting_client;$ilm_scan_rate;$ilm_awaiting_background;$ilm_awaiting_evaluation_rate;$ilm_awaiting_all;$swift_operations_failed;$swift_operations_successful;$swift_ingested_bytes;$swift_operations_unauthorized;$swift_retrieved_bytes;$memory_util;$memory_used;$memory_free;$memory_total;$cpu_utilization;$cpu_system;$cpu_iowait;$cpu_user;$cpu_steal;$cpu_softirq;$cpu_nice;$cpu_irq;$cpu_idle;$lb_total;$load_balancer_sent;$load_balancer_in_req_rate;$load_balancer_err_resp_rate;$load_balancer_received;$data_totals\n";

    }

    #DISK
    $data_out = "";
    $data_out .= "id;timestamp;write;write_io;read;read_io;resp_t_r;resp_t_w;io_rate;data_rate\n";
    $data_out .= "$disk_out";

    #Save Disk
    $rc = $xormon->saveCSV($data_out,"storagegrid","drive");
    #status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }

    #PORT
    $data_out = "";
    $data_out .= "id;timestamp;write_error;read_drop;write_drop;read;write;read_error;data_rate;error_rate;drop_rate\n";
    $data_out .= "$port_out";

    #Save port
    $rc = $xormon->saveCSV($data_out,"storagegrid","port");
    #status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }

    #NODE
    $data_out = "";
    $data_out .= "id;timestamp;s3_total;s3_ingested;s3_ops_failed;s3_ops_successful;s3_retrieved;s3_ops_unauthorized;network_rx;network_tx;network_rate;read_error;write_error;error_rate;metadata_capacity_total;metadata_capacity_used;metadata_capacity_free;capacity_total;capacity_used;capacity_free;capacity_used_percent;ilm_queue_client;ilm_scan_rate;ilm_queue_background;ilm_queue_eval_rate;ilm_queue_all;swift_ops_failed;swift_ops_successful;swift_ingested;swift_ops_unauthorized;swift_retrieved;memory_used_percent;memory_used;memory_free;memory_total;cpu_busy;cpu_system;cpu_wait;cpu_user;cpu_steal;cpu_softirq;cpu_nice;cpu_irq;cpu_idle;lb_total;lb_tx;lb_request_rate;lb_error_rate;lb_rx;user_data_used\n";
    $data_out .= "$node_out";

    #Save node
    $rc = $xormon->saveCSV($data_out,"storagegrid","node");
    #status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }

    #SITE TOTALS

    my ( $cap_data_used, $cap_data_free, $cap_data_prct, $cap_metadata_used, $cap_metadata_free, $cpu_prct, $load_5m, $mem_util, $net_rx, $net_tx, $net_rate, $s3_ingest, $s3_retrieve, $s3_successful, $s3_failed, $s3_unauthorized, $s3_total, $swift_ingest, $swift_retrieve, $swift_successful, $swift_failed, $swift_unauthorized, $ilm_queue_all, $ilm_queue_client, $ilm_queue_background, $ilm_eval_rate, $ilm_scan_rate, $lb_rx, $lb_tx, $lb_request_rate, $lb_error_rate, $data_totals, $cap_data_total, $cap_metadata_total ) = ("") x 34;

    #FINAL SITE DATA
    my $site_id = "";
    if ( exists $data{'conf'}{'vdc'}{$sites}{'id'} && $data{'conf'}{'vdc'}{$sites}{'id'} ne "" ) {
      $site_id = $data{'conf'}{'vdc'}{$sites}{'id'};
    }

    #Totals
    if ( defined $data{'perf'}{'site'}{'sum'}{$sites}{'data_total'}{'total'} && $data{'perf'}{'site'}{'sum'}{$sites}{'data_total'}{'total'} ne "" ) {
      $data_totals = $data{'perf'}{'site'}{'sum'}{$sites}{'data_total'}{'total'};
    }

    #data
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'data'}{'total'} ) {
      $cap_data_total = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'data'}{'total'} ) );
      if (Xormon::isdigit($device_cap_total)){
        $device_cap_total += $cap_data_total;
      }
      else{
        $device_cap_total = $cap_data_total;
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'data'}{'used'} ) {
      $cap_data_used = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'data'}{'used'} ) );
      if (Xormon::isdigit($device_cap_used)){
        $device_cap_used += $cap_data_used;
      }
      else{
        $device_cap_used = $cap_data_used;
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'data'}{'free'} ) {
      $cap_data_free = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'data'}{'free'} ) );
      if (Xormon::isdigit($device_cap_free)){
        $device_cap_free += $cap_data_free;
      }
      else{
        $device_cap_free = $cap_data_free;
      }
    }
    if ( Xormon::isdigit($cap_data_used) && Xormon::isdigit($cap_data_free) ) {
      $cap_data_prct = sprintf( '%.2f', ( ( $cap_data_used / ( $cap_data_used + $cap_data_free ) ) * 100 ) );
    }

    #metadata
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'metadata'}{'total'} ) {
      $cap_metadata_total = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'metadata'}{'total'} ) );
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'metadata'}{'used'} ) {
      $cap_metadata_used = ( $data{'perf'}{'site'}{'sum'}{$sites}{'metadata'}{'used'} );
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'metadata'}{'free'} ) {
      $cap_metadata_free = ( $data{'perf'}{'site'}{'sum'}{$sites}{'metadata'}{'free'} );
    }

    #cpu
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'cpu'}{'utilization'} ) {
      if ( $cpu_nodes != 0 ) {
        $cpu_prct = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'cpu'}{'utilization'} / $cpu_nodes ) );
      }
    }

    #load
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'load'}{'load'} ) {
      if ( $load_nodes != 0 ) {
        $load_5m = ( $data{'perf'}{'site'}{'sum'}{$sites}{'load'}{'load'} / $load_nodes );
      }
    }

    #mem
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'memory'}{'utilization'} ) {
      if ( $mem_nodes != 0 ) {
        $mem_util = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'memory'}{'utilization'} / $mem_nodes ) );
      }
    }

    #network
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'network'}{'tx'} ) {
      if ( $network_nodes != 0 ) {
        $net_tx = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'network'}{'tx'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'network'}{'rx'} ) {
      if ( $network_nodes != 0 ) {
        $net_rx = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'network'}{'rx'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'network'}{'net_rate'} ) {
      if ( $network_nodes != 0 ) {
        $net_rate = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'network'}{'net_rate'} ) );
      }
    }

    #S3
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'unauthorized'} ) {
      if ( $s3_nodes != 0 ) {
        $s3_unauthorized = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'unauthorized'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'retrieve'} ) {
      if ( $s3_nodes != 0 ) {
        $s3_retrieve = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'retrieve'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'ingest'} ) {
      if ( $s3_nodes != 0 ) {
        $s3_ingest = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'ingest'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'successful'} ) {
      if ( $s3_nodes != 0 ) {
        $s3_successful = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'successful'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'failed'} ) {
      if ( $s3_nodes != 0 ) {
        $s3_failed = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'failed'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'total'} ) {
      if ( $s3_nodes != 0 ) {
        $s3_total = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'s3'}{'total'} ) );
      }
    }

    #SWIFT
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'swift'}{'unauthorized'} ) {
      if ( $swift_nodes != 0 ) {
        $swift_unauthorized = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'swift'}{'unauthorized'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'swift'}{'retrieve'} ) {
      if ( $swift_nodes != 0 ) {
        $swift_retrieve = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'swift'}{'retrieve'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'swift'}{'ingest'} ) {
      if ( $swift_nodes != 0 ) {
        $swift_ingest = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'swift'}{'ingest'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'swift'}{'successful'} ) {
      if ( $swift_nodes != 0 ) {
        $swift_successful = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'swift'}{'successful'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'swift'}{'failed'} ) {
      if ( $swift_nodes != 0 ) {
        $swift_failed = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'swift'}{'failed'} ) );
      }
    }

    #ILM
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'ilm'}{'awaiting_all'} ) {
      if ( $ilm_nodes != 0 ) {
        $ilm_queue_all = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'ilm'}{'awaiting_all'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'ilm'}{'awaiting_client'} ) {
      if ( $ilm_nodes != 0 ) {
        $ilm_queue_client = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'ilm'}{'awaiting_client'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'ilm'}{'awaiting_background'} ) {
      if ( $ilm_nodes != 0 ) {
        $ilm_queue_background = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'ilm'}{'awaiting_background'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'ilm'}{'eval_rate'} ) {
      if ( $ilm_nodes != 0 ) {
        $ilm_eval_rate = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'ilm'}{'eval_rate'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'ilm'}{'scan_rate'} ) {
      if ( $ilm_nodes != 0 ) {
        $ilm_scan_rate = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'ilm'}{'scan_rate'} ) );
      }
    }

    #LOAD BALANCER $lb_rx,$lb_tx,$lb_request_rate,$lb_error_rate
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'load_balancer'}{'rx'} ) {
      if ( $lb_nodes != 0 ) {
        $lb_rx = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'load_balancer'}{'rx'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'load_balancer'}{'tx'} ) {
      if ( $lb_nodes != 0 ) {
        $lb_tx = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'load_balancer'}{'tx'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'load_balancer'}{'error_rate'} ) {
      if ( $lb_nodes != 0 ) {
        $lb_error_rate = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'load_balancer'}{'error_rate'} ) );
      }
    }
    if ( exists $data{'perf'}{'site'}{'sum'}{$sites}{'load_balancer'}{'request_rate'} ) {
      if ( $lb_nodes != 0 ) {
        $lb_request_rate = sprintf( '%.2f', ( $data{'perf'}{'site'}{'sum'}{$sites}{'load_balancer'}{'request_rate'} ) );
      }
    }

    # #Capacity for Conf File
    # $data{'conf'}{'vdc'}{$sites}{'metadata'}{'allowed'} = $cap_metadata_total;
    # $data{'conf'}{'vdc'}{$sites}{'metadata'}{'used'}    = $cap_metadata_used;
    # $data{'conf'}{'vdc'}{$sites}{'metadata'}{'free'}    = $cap_metadata_free;
    # $data{'conf'}{'vdc'}{$sites}{'capacity'}{'total'}   = $cap_data_total;
    # $data{'conf'}{'vdc'}{$sites}{'capacity'}{'used'}    = $cap_data_free;
    # $data{'conf'}{'vdc'}{$sites}{'capacity'}{'free'}    = $cap_data_used;
    # $data{'conf'}{'vdc'}{$sites}{'data'}{'used'}        = $data_totals;

    if ( $first_time == 0 ) {
      #print SITE "Site Level Statistics\n";
      #print SITE "------------------------\n";
      $site_out .= "id;timestamp;capacity_free;capacity_used;capacity_used_percent;metadata_capacity_used;metadata_capacity_free;cpu_busy;memory_used_percent;network_rx;network_tx;network_rate;s3_ingested;s3_retrieved;s3_ops_successful;s3_ops_failed;s3_ops_unauthorized;s3_total;swift_ingested;swift_retrieved;swift_ops_successful;swift_ops_failed;swift_ops_unauthorized;ilm_queue_all;ilm_queue_client;ilm_queue_background;ilm_queue_eval_rate;ilm_scan_rate;lb_rx;lb_tx;lb_request_rate;lb_error_rate;user_data_used;capacity_total;metadata_capacity_total\n";
    }
    my $hash_key = Xormon::create_hash_key($st_serial."SITE".$site_id); $hash_key = lc($hash_key);
    $site_out .= "$hash_key;$cfg->{'timestamp'};$cap_data_free;$cap_data_used;$cap_data_prct;$cap_metadata_used;$cap_metadata_free;$cpu_prct;$mem_util;$net_rx;$net_tx;$net_rate;$s3_ingest;$s3_retrieve;$s3_successful;$s3_failed;$s3_unauthorized;$s3_total;$swift_ingest;$swift_retrieve;$swift_successful;$swift_failed;$swift_unauthorized;$ilm_queue_all;$ilm_queue_client;$ilm_queue_background;$ilm_eval_rate;$ilm_scan_rate;$lb_rx;$lb_tx;$lb_request_rate;$lb_error_rate;$data_totals;$cap_data_total;$cap_metadata_total\n";
    $first_time++;
  }

  if ( Xormon::isdigit($device_cap_total) && Xormon::isdigit($device_cap_used) && ($device_cap_total != 0) ){
    $device_used_pct = Xormon::formatNumberByType("float4", $device_cap_used / ($device_cap_total / 100) );
  }

  #SITE
  $data_out = "";
  $data_out .= "$site_out";

  #Save site
  $rc = $xormon->saveCSV($data_out,"storagegrid","site");
  #status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }

  my $device_out = "id;timestamp;capacity_total;capacity_used;capacity_free;capacity_used_percent\n";
  $device_out .= "$cfg->{'hostcfg_id'};$cfg->{'timestamp'};$device_cap_total;$device_cap_used;$device_cap_free;$device_used_pct\n";

  $rc = $xormon->saveCSV($device_out,"storagegrid","device");
  #status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
}

sub get_api_token {
  my $url = shift;
  if ( $ENV{STORAGEGRID} ){
    print "URL : $url\n";
  }

  my $agent   = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( POST => $url );

  $request->header( 'Content-Type' => 'application/json', 'Accept' => 'application/json' );

  my %json_body = (
    "username" => "$cfg->{'apiUsername'}",
    "password" => "$cfg->{'apiPassword'}",
    "cookie" => "true",
    "csrfToken" => "true",
  );

  if ( $ENV{STORAGEGRID} ){
    print "JSON BODY\n";
    print Dumper \%json_body;
  }

  my $json_con = encode_json( \%json_body );

  $request->content($json_con);
  my $results = $agent->request($request);

  if ( $ENV{STORAGEGRID} ){
    print "\nRESULTS\n";
    print "##########\n";
    print Dumper $results;
    print "##########\n";
    print "END RESULT\n";
  }

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    $logger->error("Request error: $st_line");
    $logger->error("Post url     : $url");
    return 0;
  }
  else {
    if ( defined $results->content && $results->content ne "" ) {
      my $result = decode_json( $results->content );
      if ( defined $result->{'data'} && $result->{'data'} ne "" ) {
        my $cookie = $result->{'data'};
        chomp($cookie);
        return $cookie;
      }
      else {
        $logger->error("Cookie not found! Login failed! access_token not found!") && return "0";
      }
    }
    else {
      $logger->error("Cookie not found! Login failed!") && return "0";
    }
  }
  return 0;
}

sub new_get_request {
  my $url    = shift;
  my $result = "";

  my $agent   = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( GET => $url );
  $request->header(
    'Accept'        => 'application/json',
    'Authorization' => 'Bearer ' . $cfg->{'token'} . ''
  );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    $logger->error("GET url           : $url");
    $logger->error("Request error     : $st_line");

    return "0";
  }

  if ( defined $results->content && $results->content ne "" ) {
    my $result = decode_json( $results->content );

    #print "###############################\n";
    #print "$url\n";
    #print Dumper $result;
    return $result;
  }

  return "0";
}

sub set_time {
  my @t = split( / /, $cfg->{'gtime'} );
  my ( undef, undef, $sec ) = split( ":", $t[3] );
  if ( $sec =~ m/^0/ ) {
    $sec =~ s/0//;
  }
  my $end_time = str2time( join( " ", @t ) );

  #my $diff = $ltime - $end_time;
  $end_time = $end_time - $sec - 1;
  my $start_time        = $end_time - 300;
  my $start_time_before = $start_time - 300;                  #5 mins back
  my $start_utc         = get_time_utc($start_time);
  my $end_utc           = get_time_utc($end_time);
  my $before            = get_time_utc($start_time_before);
  return ( "$start_utc", "$end_utc", "$before" );
}

sub get_time_utc {
  my $timestamp_end = shift;
  my @time          = localtime($timestamp_end);
  my $time_human    = localtime($timestamp_end);
  my $sec           = $time[0];
  my $minute        = $time[1];
  my $hour          = $time[2];
  my $day           = $time[3];
  my $month         = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;
  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }
  my $time_end = "$year-$month-$day" . "T" . "$hour:$minute:$sec" . "Z";
  return $time_end;
}

sub data_for_site {
  my $sites   = shift;
  my $stats   = shift;
  my $metrics = shift;
  my $value   = shift;
  if ( Xormon::isdigit( $data{'perf'}{'site'}{'sum'}{$sites}{$stats}{$metrics} ) ) {
    $data{'perf'}{'site'}{'sum'}{$sites}{$stats}{$metrics} += $value;
  }
  else {
    $data{'perf'}{'site'}{'sum'}{$sites}{$stats}{$metrics} = $value;
  }
}

sub health_status {
  my $act_timestamp  = time();
  my $return        = "";
  my @error_message = ();

  my %hs_data;
  my @hs              = ();
  my $main_state      = "";

  ###Health
  $return = new_get_request("https://$cfg->{'apiHost'}/api/v3/grid/health");
  if ( $return ne "0" ) {
    $main_state = "ok";
    if ( defined $return->{'data'}->{'alarms'} && $return->{'data'}->{'alarms'} ne "" ) {
      foreach my $alarms ( keys( %{ $return->{'data'}->{'alarms'} } ) ) {
        if ( defined $return->{'data'}->{'alarms'}->{$alarms} && $return->{'data'}->{'alarms'}->{$alarms} ne "" ) {
          $data{'health'}{'alarms'}{$alarms} = $return->{'data'}->{'alarms'}->{$alarms};
        }
      }
      foreach my $alerts ( keys( %{ $return->{'data'}->{'alerts'} } ) ) {
        if ( defined $return->{'data'}->{'alerts'}->{$alerts} && $return->{'data'}->{'alerts'}->{$alerts} ne "" ) {
          $data{'health'}{'alerts'}{$alerts} = $return->{'data'}->{'alerts'}->{$alerts};
        }
      }
      foreach my $nodes ( keys( %{ $return->{'data'}->{'nodes'} } ) ) {
        if ( defined $return->{'data'}->{'nodes'}->{$nodes} && $return->{'data'}->{'nodes'}->{$nodes} ne "" ) {
          $data{'health'}{'nodes'}{$nodes} = $return->{'data'}->{'nodes'}->{$nodes};
        }
      }
    }
  }

  ## ALARMS
  my $major       = "";
  my $critical    = "";
  my $notice      = "";
  my $minor       = "";

  if ( defined $data{'health'}{'alarms'}{'major'} && $data{'health'}{'alarms'}{'major'} ne "" ) {
    $major = $data{'health'}{'alarms'}{'major'};
  }
  if ( defined $data{'health'}{'alarms'}{'critical'} && $data{'health'}{'alarms'}{'critical'} ne "" ) {
    $critical = $data{'health'}{'alarms'}{'critical'};
  }
  if ( defined $data{'health'}{'alarms'}{'notice'} && $data{'health'}{'alarms'}{'notice'} ne "" ) {
    $notice = $data{'health'}{'alarms'}{'notice'};
  }
  if ( defined $data{'health'}{'alarms'}{'minor'} && $data{'health'}{'alarms'}{'minor'} ne "" ) {
    $minor = $data{'health'}{'alarms'}{'minor'};
  }
  if ( $major ne "" && $major != 0 ) {
    $main_state = "error";
  }
  if ( $critical ne "" && $critical != 0 ) {
    $main_state = "error";
  }
  my %alarm;
  $alarm{major}    = $major;
  $alarm{critical} = $critical;
  $alarm{notice}   = $notice;
  $alarm{minor}    = $minor;
  push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{alarms} }, \%alarm );

  ## ALERTS COUNT TABLE
  my $major_al    = "";
  my $critical_al = "";
  my $minor_al    = "";

  if ( defined $data{'health'}{'alerts'}{'major'} && $data{'health'}{'alerts'}{'major'} ne "" ) {
    $major_al = $data{'health'}{'alerts'}{'major'};
  }
  if ( defined $data{'health'}{'alerts'}{'critical'} && $data{'health'}{'alerts'}{'critical'} ne "" ) {
    $critical_al = $data{'health'}{'alerts'}{'critical'};
  }
  if ( defined $data{'health'}{'alerts'}{'minor'} && $data{'health'}{'alerts'}{'minor'} ne "" ) {
    $minor_al = $data{'health'}{'alerts'}{'minor'};
  }
  if ( $major_al ne "" && $major_al != 0 ) {
    $main_state = "error";
  }
  if ( $critical_al ne "" && $critical_al != 0 ) {
    $main_state = "error";
  }
  my %alert_count;
  $alert_count{major}    = $major_al;
  $alert_count{critical} = $critical_al;
  $alert_count{minor}    = $minor_al;
  push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{alerts} }, \%alert_count );

  ## ALERTING TABLE
  $return = new_get_request("https://$cfg->{'apiHost'}/api/v3/grid/alerts?include=active");

  if ( defined $return->{'data'} && ref( $return->{'data'} ) eq "ARRAY" ) {
    foreach my $all_alerts ( @{ $return->{'data'} } ) {
      my $site_al      = "";
      my $instance_al  = "";
      my $severity_al  = "";
      my $alertname_al = "";
      my $time_al      = "";
      my $id           = "";
      if ( defined $all_alerts->{'id'} && $all_alerts->{'id'} ne "" ) {
        $id = $all_alerts->{'id'};
        if ( defined $all_alerts->{'labels'}->{'site_name'} && $all_alerts->{'labels'}->{'site_name'} ne "" ) {
          $site_al = $all_alerts->{'labels'}->{'site_name'};
        }
        if ( defined $all_alerts->{'labels'}->{'instance'} && $all_alerts->{'labels'}->{'instance'} ne "" ) {
          $instance_al = $all_alerts->{'labels'}->{'instance'};
        }
        if ( defined $all_alerts->{'labels'}->{'severity'} && $all_alerts->{'labels'}->{'severity'} ne "" ) {
          $severity_al = $all_alerts->{'labels'}->{'severity'};
        }
        if ( defined $all_alerts->{'labels'}->{'alertname'} && $all_alerts->{'labels'}->{'alertname'} ne "" ) {
          $alertname_al = $all_alerts->{'labels'}->{'alertname'};
        }
        if ( defined $all_alerts->{'startsAt'} && $all_alerts->{'startsAt'} ne "" ) {
          $time_al = $all_alerts->{'startsAt'};
        }
        my %alert;
        $alert{id}       = $id;
        $alert{site}     = $site_al;
        $alert{instance} = $instance_al;
        $alert{time}     = $time_al;
        $alert{severity} = $severity_al;
        $alert{name}     = $alertname_al;
        push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{alerting} }, \%alert );
      }
    }
  }

  ## GRID TABLE
  foreach my $grid_name ( keys( %{ $data{'conf'}{'grid'} } ) ) {
    my $grid_state = "";
    if ( defined $data{'conf'}{'grid'}{$grid_name}{'state'} && $data{'conf'}{'grid'}{$grid_name}{'state'} ne "" ) {
      $grid_state = $data{'conf'}{'grid'}{$grid_name}{'state'};
    }
    my %grid;
    $grid{name}   = $grid_name;
    $grid{status} = $grid_state;
    push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{grid} }, \%grid );
  }

  ## NODES TABLE
  my $connected  = "";
  my $down       = "";
  my $uknow      = "";

  if ( defined $data{'health'}{'nodes'}{'connected'} && $data{'health'}{'nodes'}{'connected'} ne "" ) {
    $connected = $data{'health'}{'nodes'}{'connected'};
  }
  if ( defined $data{'health'}{'nodes'}{'administratively-down'} && $data{'health'}{'nodes'}{'administratively-down'} ne "" ) {
    $down = $data{'health'}{'nodes'}{'administratively-down'};
  }
  if ( defined $data{'health'}{'nodes'}{'unknown'} && $data{'health'}{'nodes'}{'unknown'} ne "" ) {
    $uknow = $data{'health'}{'nodes'}{'unknown'};
  }
  if ( $uknow ne "" && $uknow != 0 ) {
    $main_state = "error";
  }
  my %node_count;
  $node_count{connected}               = $connected;
  $node_count{'administratively_down'} = $down;
  $node_count{unknown}                 = $uknow;
  push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{node_count} }, \%node_count );

  ## SITE TABLE
  my $table_site = "";
  foreach my $site_name ( keys( %{ $data{'conf'}{'vdc'} } ) ) {
    my $site_state = "";
    if ( defined $data{'conf'}{'vdc'}{$site_name}{'state'} && $data{'conf'}{'vdc'}{$site_name}{'state'} ne "" ) {
      $site_state = $data{'conf'}{'vdc'}{$site_name}{'state'};
    }
    my %site;
    $site{name}   = $site_name;
    $site{status} = $site_state;
    push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{sites} }, \%site );
    foreach my $nodes_name ( keys( %{ $data{'conf'}{'node'} } ) ) {
      if ( $data{'conf'}{'node'}{$nodes_name}{'vdcName'} ne "$site_name" ) { next; }
      my $node_state = "";
      if ( $data{'conf'}{'node'}{$nodes_name}{'state'} && $data{'conf'}{'node'}{$nodes_name}{'state'} ne "" ) {
        $node_state = $data{'conf'}{'node'}{$nodes_name}{'state'};
      }
      my %node;
      $node{name}      = $nodes_name;
      $node{site_name} = $site_name;
      $node{status}    = $node_state;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{node} }, \%node );
    }
  }

  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }
  # save events
  my $rc_hs = $xormon->saveConfJSON(\%hs_data, 1);

  my %main_hs = (
      "item_id" => $cfg->{'hostcfg_id'},
      "status" => $main_state,
      "data" => join("\n", @error_message),
  );
  push(@hs, \%main_hs);

  $xormon->saveStatus(\@hs);

}

sub logout {
  my $url    = shift;

  my $agent   = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( DELETE => $url );
  $request->header(
    'Accept'        => 'application/json',
  );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    $logger->error("GET url           : $url");
    $logger->error("Request error     : $st_line");

    return 0;
  }

  return 1;
}
sub connection_test {
  my $client = shift;
  $cfg = shift;

  $logger = XormonLogger->new( 'storagegrid', $cfg->{hostcfg_id}, $cfg->{hostalias} );

  my $error = 1;
  my @rows;
  my %data;

  my $test = Test->new($client);

  $cfg->{'timestamp'} = time();
  $cfg->{'gtime'}     = gmtime($cfg->{'timestamp'});
  $cfg->{'gtime'} =~ s/ {2,}/ /g;

  my @variables = ("hostcfg_id", "hostalias", "apiHost", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  #### connection network test ####
  my ($code, $text) = Xormon::conntest_tcp($cfg->{'apiHost'},'443');
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  $cfg->{'token'} = get_api_token("https://$cfg->{'apiHost'}/api/v3/authorize");
  if ( $cfg->{'token'} eq "0" ) {
    $test->addSection("API authorization", 0, "Login failed : Cant' login to API");
    return $test->stop();
  }
  else {
    $test->addSection("API authorization", 1, "Login success : Token granted from API");
  }

  ( $cfg->{'nameGrid'}, @gsites ) = create_conf();

  ( $cfg->{'startTime'}, $cfg->{'endTime'}, $cfg->{'pre_start_time'} ) = set_time();
  my $api     = "";
  my $product = "";
  my $storage = "";
  my $return = new_get_request("https://$cfg->{'apiHost'}/api/v3/grid/config/product-version");
  if ( $return ne "0" ) {
    if ( defined $return->{'apiVersion'} && $return->{'apiVersion'} ne "" ) {
      $api = $return->{'apiVersion'};
    }
    if ( defined $return->{'data'}->{'productVersion'} && $return->{'data'}->{'productVersion'} ne "" ) {
      $product = $return->{'data'}->{'productVersion'};
    }
    push(@rows, "IP : $cfg->{'apiHost'}\n");
    push(@rows, "Api Version : $api\n");
    push(@rows, "Product Version : $product\n");
    $test->addSection("System information", 1 , \@rows);
  }
  else {
    $test->addSection("System information", 0, "System information missing!");
    return $test->stop();
  }

  $return = new_get_request("https://$cfg->{'apiHost'}/api/v3/grid/metric-query-range?query=rate%28node_network_receive_bytes_total%5B6m%5D%29&start=$cfg->{'startTime'}&end=$cfg->{'endTime'}&step=300");
  if ( $return ne "0" ) {
    if ( defined $return->{'data'}->{'result'} && ref( $return->{'data'}->{'result'} ) eq "ARRAY" ) {
      $test->addSection("Performance data", 1 , "Performance data OK");
    }
    else {
      $test->addSection("Performance data", 0 , "Performance data missing!");
    }
  }
  else {
    $test->addSection("Performance data", 0 , "Performance data missing!");
  }
  logout("https://$cfg->{'apiHost'}/api/v3/authorize");
  return $test->stop();
}
1;
