package SNMPHandler;
use lib '../../../perl-modules/'; #SNMP_lib
use SNMP_lib;
use Data::Dumper;

sub new {
    my ($class, $cfg, $logger, $conn_test) = @_;

    SNMP_lib::initialize_snmp($cfg);    
    SNMP_lib::initialize_mibs('SYNOLOGY');   

    # Default variables for SYNOLOGY
    my %params = ( Timeout => 50000000 );

    # If Test only 10sec for response
    $params{Timeout} = 1000000 if $conn_test;   

    # Check if exist ContextName from configuration
    if ( exists $cfg->{snmpContextName} && $cfg->{snmpContextName} ne '' ) { 
        $params{Context} = $cfg->{snmpContextName};
    }    
    # Init new session
    my $sess = SNMP_lib::new_session(\%params);

    # if ( exists $cfg->{'snmp_request_sleep'} && Xormon::isdigit($cfg->{'snmp_request_sleep'}) ) {
    #     $snmp_request_sleep = $cfg->{'snmp_request_sleep'};
    #     $logger->info("snmp_request_sleep=$snmp_request_sleep") if defined $logger;
    # }   
   
    my $self = {
        cfg => $cfg,
        sess => $sess,
        data => {},
        logger => $logger,
        conn_test => $conn_test     
    };    

    bless $self, $class;
    return $self;
}

# Public
sub collect_synology {
    my ($self, $section, $mib, $metric) = @_;    
    
    $self->{data} = {} unless ref($self->{data}) eq 'HASH';
    # When testing the connection, there are different return values: a status with a flag of 
    # 0 if the test fails, and 1 if it passes. In the case of status 0, the return value is 
    # extended with the reason for the error.
    if($self->{conn_test}) {
        return SNMP_lib::collect_synology($self->{data}, $self->{sess}, $section, $mib, $metric);  
    }

    # normaly request returning data in hash
    $self->{data} = SNMP_lib::collect_synology($self->{data}, $self->{sess}, $section, $mib, $metric);       
  
    # This make some bugs while connect synology
    #if ( $self->{snmp_request_sleep} ) { sleep($self->{snmp_request_sleep}); }    
    
    return 1;
}

# Public
sub check_connection {
    my ($self) = @_;

    my ($conn_status, $reason) = $self->collect_synology('configuration', 'sysName', '');
    
    if (!$conn_status) {
        $self->{logger}->error("Connection check failed: $reason") if defined $self->{logger};
    }

    return ($conn_status, $reason);    
}

# Public
sub build_snmpwalk_cmd {
    my ($self) = @_;
    return SNMP_lib::build_snmpwalk_cmd($self->{sess});
}

1;
