package TestTask;

use strict;
use warnings;

use parent 'Task';

use Test;
use SNMP_lib;
use SYNOLOGY;

# TestTask class to handle 'test' tasks
# This class extends the Task base class

# The execute method for the TestTask
# It handles the specific logic for testing connections
sub execute {
    my ($self) = @_;
    my $cfg = $self->{data}->{data};       
    my $test = Test->new($self->{client});  
  
    $self->log("Task '$self->{data}->{task}' for " . $self->{data}->{data}->{hostalias} . " successfuly started.");

    # Test IP and port
    my $port = (exists $cfg->{'snmpPort'} && Xormon::isdigit($cfg->{'snmpPort'}) && $cfg->{'snmpPort'} < 65536)
                ? $cfg->{'snmpPort'} 
                : 161; # default 161
    my ( $code, $text ) = Xormon::conntest_udp($cfg->{snmpHost}, $port);
    $test->addSection("Network connection", $code, $text);

    if ($code == 0) { 
        # Log
        $self->log("Task '$self->{data}->{task}' for " . $self->{data}->{data}->{hostalias} . " is done.");
        return $test->stop(); 
    }  

    # SNMP connection test
    my ($status, $reason, $snmpwalk) = SYNOLOGY::snmp_connection_test($cfg);

    my $response = "";
    my @rows     = ();
 
    if ($status) { # ok=1
        if (defined $snmpwalk && defined $reason) {
            push(@rows, "$snmpwalk\n"); # cli snmpwalk
            push(@rows, "$reason\n");   # $data{configuration}{sysName}
        }
        $test->addSection("SNMP connection", 1, \@rows);
    } else { # nok=0
        if (defined $reason) {
            push(@rows, "$reason\n");
        }
        if (defined $snmpwalk) {
            push(@rows, "\n");
            push(@rows, "This snmpwalk command must work from the command line:\n");
            push(@rows, "$snmpwalk\n");
        }           
       
        $test->addSection("SNMP connection", 0, \@rows);   
    }
    
    # STOP test
    $test->stop();   
    
    # Unnecessary $test stop, do same
    $self->shutdown();  
    
    # Log
    $self->log("Task '$self->{data}->{task}' for " . $self->{data}->{data}->{hostalias} . " is done.");
}

1;
