package VMAX;

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";

use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use Xormon;
use JSON qw(encode_json decode_json);
use Test;
use XormonLogger;

my %data;
my $xormon;
my $cfg;
my $unisphere_version;
my $eunix_ms;
my $sunix_ms;
my $url_paths;
my $debug           = 1;
my $critical_alerts = 1;
my $connection_test = 0;
my @rows;
my $logger;

###########################################
# TODO:
# perf_diskgroup -> exclude io metrics (DiskReads and DiskWrites) for the whole day to avoid the first request failing every 5 minutes
# send_conf      -> add mapping between slo -> volumes (SGs), host -> volumes (SGs)
# prepare_perf   -> add CachePartition metrics to device stats
# healts_status  -> disable critical alerts ($critical_alerts = 0)
###########################################
#my $inputdir = "/home/xormon/nextgen/server-nest/microservices/vmax";
#my $debugdir = "$inputdir/tmp/VMAX2NG-debug";
###########################################

sub collect {
  $xormon = shift;
  $cfg    = shift;

  #
  # test main variables
  #
  unless ( exists $cfg->{'hostcfg_id'} )  { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")  && return "error"; }
  unless ( exists $cfg->{'hostalias'} )   { Xormon::error("Cannot continue, 'hostalias' is not configured!")   && return "error"; }
  $logger = XormonLogger->new( 'vmax', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost'} )     { $logger->error("Cannot continue, 'apiHost' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'apiPort'} )     { $logger->error("Cannot continue, 'apiPort' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'apiUsername'} ) { $logger->error("Cannot continue, 'apiUsername' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiPassword'} ) { $logger->error("Cannot continue, 'apiPassword' is not configured!") && return "error"; }
  unless ( exists $cfg->{'symmetrix'} )   { $logger->error("Cannot continue, 'symmetrix' is not configured!")   && return "error"; }
  $cfg->{'sample_rate'}   = 300;
  my $act_timestamp       = time();
  my $status              = "error"; # status code
  my $last_perf_timestamp = "";
  $eunix_ms               = int( ( time() - 300 ) / 300 ) * 300 * 1000; # 5 minutes back in ms
  $sunix_ms               = $eunix_ms - ( 3600 * 24 * 1000);            # default for the fist run is one day back

  #
  # get microservice data
  #
  my $service_data_in  = $xormon->getData();
  my $service_data_out = {};
  if ( exists $service_data_in->{last_conf_timestamp} ) {
    $service_data_out->{last_conf_timestamp} = $service_data_in->{last_conf_timestamp};
  }
  if ( exists $service_data_in->{last_perf_timestamp} && Xormon::isdigit($service_data_in->{last_perf_timestamp}) ) {
    if ( ($act_timestamp - ( int($service_data_in->{last_perf_timestamp} / 1000) )) < 86400 ) { # max is one day back
      $sunix_ms = $service_data_in->{last_perf_timestamp};
    }
  }

  #
  # get Unisphere version
  #
  get_unisphere_version();

  #
  # connection status
  #
  if ( defined $unisphere_version ) {
    $xormon->isConnected(1);
  }
  else {
    $xormon->isConnected(0, "Unisphere version has not been found!");
    $xormon->hasData(0, "Connection issue, cannot collect stats!");
    $logger->error( "Cannot continue, Unisphere version has not been found!" . __FILE__ . ":" . __LINE__ ) && return $status;
  }

  #
  # set URL paths
  #
  set_url_paths();

  #
  # perf data
  #
  get_perf();
  if ( exists $data{perf} ) {
    foreach my $timestamp_ms ( sort { $a <=> $b } keys %{ $data{perf} } ) {
      unless ( $timestamp_ms > $sunix_ms ) { next; } # save only new stats

      my $perf_out = prepare_perf($timestamp_ms);
      my $rc       = $xormon->saveJSON2($perf_out);
      $logger->info("save perf: $timestamp_ms: ".localtime($timestamp_ms/1000).": $rc");
      $service_data_out->{last_perf_timestamp} = $timestamp_ms;

      if ( $rc eq "201" ) {
        $status = "successful";
      }
      elsif ( $rc eq "401" ) {
        $status = "unauthorized";
        last;
      }
      else {
        $status = "error";
        last;
      }
    }
  }

  #
  # perf data status
  #
  if ( $status eq "successful" ) {
    $xormon->hasData(1);
  }
  else {
    $xormon->hasData(0, "no available stats: $status");
  }

  #
  # collect conf data, send architecture and conf data (only once per hour)
  #
  if ( ! exists $service_data_in->{last_conf_timestamp} || ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) || $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) {
    $service_data_out->{last_conf_timestamp} = $act_timestamp;

    #if ( defined $data{'ThinTier'} && $data{'ThinTier'} ne '' ) {
    #  conf_tierbystoragegroup();
    #}
    #unless ( exists $ENV{VMAX_DISABLE_SRDF} && $ENV{VMAX_DISABLE_SRDF} eq "1" ) {
    #  conf_srdf();
    #}

    ################################################
    #TODO
    # Volume (TDEV) capacity
    #unless ( exists $ENV{VMAX_DISABLE_VOLUMES_CAP} && $ENV{VMAX_DISABLE_VOLUMES_CAP} eq "1" ) {
    #  cap_volume();
    #}
    #conf_host();
    conf_feport();

    get_device_conf();
    send_conf();
    ################################################
  }

  #
  # Health status (Unisphere V9 and V10 only)
  #
  health_status();

  #
  # save microservice data
  #
  $xormon->saveData(\%{ $service_data_out });

  return $status;
}

sub conf_feport {
  #if ( $univmax_version =~ m/^V10/ ) { return 1; } # deprecated in V10 # update, it works with V10.1.0.1, but here is "system" intead of "sloprovisioning"
  my $fe_port_path = $url_paths->{provisioning};

  if ( $unisphere_version =~ m/^V10/ ) {
    $fe_port_path = $url_paths->{system};
  }

  if ( exists $data{conf}{FEPort} ) {
    foreach my $director_id ( keys( %{ $data{conf}{'FEPort'} } ) ) {
      if ( defined $data{conf}{'FEPort'}{$director_id} && $data{conf}{'FEPort'}{$director_id} ne '' ) {
        foreach my $port_id ( keys( %{ $data{conf}{'FEPort'}{$director_id} } ) ) {
          my $result = new_get_request("$fe_port_path/symmetrix/$cfg->{'symmetrix'}/director/$director_id/port/$port_id");

          unless ( defined $result ) {
            if ( $url_paths->{sloprovisioning} =~ m/^90/ ) {
              $result = new_get_request("$url_paths->{sloprovisioning}/symmetrix/$cfg->{'symmetrix'}/director/$director_id/port/$port_id");
            }
            else {
              $result = new_get_request("$url_paths->{sloprovisioning}/symmetrix/$cfg->{'symmetrix'}/director/$director_id/port/$port_id");
            }
          }

          if ( defined $result ) {
            if ( exists $result->{'symmetrixPort'} ) {
              if ( ref( $result->{'symmetrixPort'} ) eq "ARRAY" ) {
                foreach ( @{ $result->{'symmetrixPort'} } ) {
                  if ( defined $_->{'type'} && $_->{'type'} ne '' ) {
                    $data{conf}{'FEPort'}{$director_id}{$port_id}{'type'} = $_->{'type'};
                  }
                  if ( defined $_->{'identifier'} && $_->{'identifier'} ne '' ) {
                    $data{conf}{'FEPort'}{$director_id}{$port_id}{'identifier'} = $_->{'identifier'};
                  }
                  if ( defined $_->{'num_of_mapped_vols'} && $_->{'num_of_mapped_vols'} ne '' ) {
                    $data{conf}{'FEPort'}{$director_id}{$port_id}{'num_of_mapped_vols'} = $_->{'num_of_mapped_vols'};
                  }
                }
              }
              else {
                if ( exists $result->{'symmetrixPort'}{type} )               { $data{conf}{'FEPort'}{$director_id}{$port_id}{'type'}               = $result->{'symmetrixPort'}{type}; }
                if ( exists $result->{'symmetrixPort'}{identifier} )         { $data{conf}{'FEPort'}{$director_id}{$port_id}{'identifier'}         = $result->{'symmetrixPort'}{identifier}; }
                if ( exists $result->{'symmetrixPort'}{num_of_mapped_vols} ) { $data{conf}{'FEPort'}{$director_id}{$port_id}{'num_of_mapped_vols'} = $result->{'symmetrixPort'}{num_of_mapped_vols}; }
              }
            }
          }
        }
      }
    }
  }

  return 1;
}

sub connection_test {
  my $client = shift;
  $cfg       = shift;

  $connection_test = 1;
  my $test         = Test->new($client);

  #
  # test main variables
  #
  my @variables = ("hostcfg_id", "hostalias", "apiHost", "apiPort", "apiUsername", "apiPassword", "symmetrix");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Device cfg", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Device cfg", 1 , "");

  $logger = XormonLogger->new( 'vmax', $cfg->{hostcfg_id}, $cfg->{hostalias} );

  #
  # test IP and port
  #
  my ( $code, $text ) = Xormon::conntest_tcp($cfg->{'apiHost'},$cfg->{'apiPort'});
  $test->addSection("API host network connection", $code, $text);
  if ( $code == 0 ) { return $test->stop(); }

  #
  # get Unisphere version
  #
  get_unisphere_version();
  if ( defined $unisphere_version && $unisphere_version ne '' ) {
    push(@rows, "Unisphere IP     : $cfg->{'apiHost'}\n");
    push(@rows, "user             : $cfg->{'apiUsername'}\n");
    push(@rows, "Symmetrix_ID     : $cfg->{'symmetrix'}\n");
    push(@rows, "Unisphere version: $unisphere_version\n");

    $test->addSection("API authorization", 1 , \@rows);
    @rows = ();
  }
  else {
    push(@rows, "Unisphere version has not been found!\n");
    push(@rows, "Connection failed!\n");
    $test->addSection("API authorization", 0 , \@rows);
    return $test->stop();
  }

  #
  # collect stats
  #
  my $result      = new_get_request("performance/Array/keys");
  my @metrics     = ( "HostMBs", "HostMBReads", "HostMBWritten", "HostIOs", "HostReads", "HostWrites", "ReadResponseTime", "WriteResponseTime" );
  my $stats_found = 0;
  $eunix_ms       = int( ( time() - 300 ) / 300 ) * 300 * 1000; # 5 minutes back in ms
  $sunix_ms       = $eunix_ms - ( 3600 * 24 * 1000);            # default for the fist run is one day back

  my @available_symmetrix_ids = ();
  my $symmetrix_found         = 0;
  if ( defined $result && exists $result->{arrayInfo} && ref($result->{arrayInfo}) eq "ARRAY" ) {
    foreach ( @{ $result->{arrayInfo} } ) {
      unless ( exists $_->{symmetrixId} && exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      push(@available_symmetrix_ids, $_->{symmetrixId});

      if ( $_->{symmetrixId} eq $cfg->{'symmetrix'} ) {
        $symmetrix_found++
      }
      else {
        next;
      }
      #$data{conf}{Array}{ $cfg->{symmetrix} }{id} = $cfg->{symmetrix};

      ($sunix_ms,$eunix_ms) = set_timerange("perf_array",$sunix_ms,$eunix_ms,$_);

      my $params;
      $params->{startDate}   = $sunix_ms;
      $params->{endDate}     = $eunix_ms;
      $params->{symmetrixId} = $cfg->{'symmetrix'};
      $params->{metrics}     = [@metrics];

      push(@rows, "Available Array stats:\n");
      push(@rows, "firstAvailableDate   : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) )."\n");
      push(@rows, "lastAvailableDate    : $_->{'lastAvailableDate'} :  ".localtime( int($_->{'lastAvailableDate'}/1000) )."\n");
      push(@rows, "act_time             : ".localtime()."\n");

      my $stats = new_post_request("performance/Array/metrics",$params,1);

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            #$data{perf}{ $values->{timestamp} }{Array}{ $cfg->{symmetrix} } = $values;
            $stats_found++;
          }
        }
      }
      else {
        push(@rows, "Array stats have not been collected!\n");
        $test->addSection("API data test", 0 , \@rows);
        return $test->stop();
      }
    }
  }
  else {
    push(@rows, "Array keys have not been found!\n");
    push(@rows, "Try this URL in your web browser with \"$cfg->{apiUsername}\" user authorization!\n");
    push(@rows, "You should see configured symmetrix ID!\n");
    push(@rows, "https://$cfg->{'apiHost'}:$cfg->{'apiPort'}/univmax/restapi/performance/Array/keys\n");
    $test->addSection("API data test", 0 , \@rows);
    return $test->stop();
  }

  if ( $stats_found > 0 ) {
    push(@rows, "Stats found!\n");
    $test->addSection("API data test", 1 , \@rows);
  }
  elsif ( $symmetrix_found == 0 ) {
    push(@rows, "Configured symmetrixId \"$cfg->{'symmetrix'}\" has not been found!\n");
    push(@rows, "Available symmetrix IDs: ".join(", ", @available_symmetrix_ids)."\n");
    $test->addSection("API data test", 0 , \@rows);
    return $test->stop();
  }
  else {
    push(@rows, "Array stats have not been found!\n");
    $test->addSection("API data test", 0 , \@rows);
    return $test->stop();
  }

  return $test->stop();
}

sub health_status {
  my $main_status   = "ok";
  my @error_message = ();
  my @hs            = ();
  my %hs_data;

  #
  # collect data
  #
  my $result = "";
  if ( $unisphere_version =~ m/^V10/ ) {
    $result = new_get_request("100/system/symmetrix/$cfg->{'symmetrix'}/health");
  }
  elsif ( $unisphere_version =~ m/^V9/ ) {
    $result = new_get_request("90/system/symmetrix/$cfg->{'symmetrix'}/health");
  }
  else {
    return 1; # not supported version
  }

  unless ( defined $result ) { $logger->error( "Health status has not been found!" . __FILE__ . ":" . __LINE__ ) && return 0; }

  if ( exists $result->{health_score_metric} && ref( $result->{health_score_metric} ) eq "ARRAY" ) {
    foreach ( @{ $result->{health_score_metric} } ) {

      my $section = ( exists $_->{metric} ) ? $_->{metric} : "";
      my $time    = "NA";

      if ( exists $_->{data_date} && Xormon::isdigit( $_->{data_date} ) ) {
        $time = localtime( int( $_->{data_date} / 1000 ) );    # at firt convert timestamp from ms to sec.
      }

      if ( exists $_->{instance_metrics} && ref( $_->{instance_metrics} ) eq "ARRAY" ) {
        foreach my $val ( @{ $_->{instance_metrics} } ) {

          if ( exists $val->{health_score_instance_metric} && ref( $val->{health_score_instance_metric} ) eq "ARRAY" ) {
            foreach my $value ( @{ $val->{health_score_instance_metric} } ) {

              my $metric_name     = ( exists $value->{metric_name} )     ? $value->{metric_name}     : "";
              my $metric_category = ( exists $value->{metric_category} ) ? $value->{metric_category} : "";
              my $instance_name   = ( exists $value->{instance_name} )   ? $value->{instance_name}   : "";
              my $severity        = ( exists $value->{severity} )        ? $value->{severity}        : "";

              #
              # Global Health status part 2
              #
              if ( $section eq "CONFIGURATION" && ( $severity eq "Fatal" || ( $severity eq "Critical" && $critical_alerts == 1 ) ) ) {

                $main_status = "error";
                push( @error_message, "$time : $severity : $metric_name : $metric_category : $instance_name" );
              }

              # add HS data to DB to be able to show them in the HS view
              push( @{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} }, { "time" => $time, "severity" => $severity, "type" => $metric_name, "category" => $metric_category, "description" => $instance_name } );
            }
          }
        }
      }
    }
  }
  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }

  push(@hs, { "item_id" => $cfg->{'hostcfg_id'}, "status" => $main_status, "data" => join("\n", @error_message) });

  # send statuses
  my $rc = $xormon->saveStatus(\@hs);
  $logger->info("save health status: $rc");

  # save HS data
  my $rc_hs = $xormon->saveConfJSON(\%hs_data,1);
  $logger->info("save health status table: $rc_hs");

  return 1;
}

sub send_conf {
  my @architecture;
  my %conf;

  #
  # DEVICE
  #
  # Architecture
  my %device_arch = (
    "item_id"    => $cfg->{'hostcfg_id'},
    "hostcfg_id" => $cfg->{'hostcfg_id'},
    "label"      => $cfg->{'hostalias'},
    "hw_type"    => "vmax",
    "subsystem"  => "device",
    "parents"    => []
  );
  push(@architecture, \%device_arch);
  # conf
  $conf{ $cfg->{'hostcfg_id'} }{symmetrix_id} = $cfg->{'symmetrix'};
  $conf{ $cfg->{'hostcfg_id'} }{serial}       = $cfg->{'symmetrix'};
  $conf{ $cfg->{'hostcfg_id'} }{version}      = $unisphere_version;
  $conf{ $cfg->{'hostcfg_id'} }{ip}           = $cfg->{'apiHost'};
  if ( exists $data{conf}{DEVICE}{model}     && $data{conf}{DEVICE}{model}     ne '' ) { $conf{ $cfg->{'hostcfg_id'} }{model}     = $data{conf}{DEVICE}{model}; }
  if ( exists $data{conf}{DEVICE}{ucode}     && $data{conf}{DEVICE}{ucode}     ne '' ) { $conf{ $cfg->{'hostcfg_id'} }{ucode}     = $data{conf}{DEVICE}{ucode}; }
  if ( exists $data{conf}{DEVICE}{microcode} && $data{conf}{DEVICE}{microcode} ne '' ) { $conf{ $cfg->{'hostcfg_id'} }{microcode} = $data{conf}{DEVICE}{microcode}; } # V10

  #
  # SRP
  #
  if ( exists $data{conf}{SRP} ) {
    foreach my $label ( sort keys %{ $data{conf}{SRP} } ) {
      my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."pool".$label) );
      # Architecture
      my %arch = (
        "item_id"   => $id,
        "label"     => $label,
        "hw_type"   => "vmax",
        "subsystem" => "pool",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents"   => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
      # conf
      my $val = $data{conf}{SRP}{$label};
      #if ( exists $val->{last_stats}{OverallEfficiencyRatio} ) { $conf{$id}{overall_efficiency_ratio} = $val->{last_stats}{OverallEfficiencyRatio}; }
      #if ( exists $val->{overall_efficiency_ratio_to_one} )    { $conf{$id}{overall_efficiency_ratio} = $val->{overall_efficiency_ratio_to_one}; }
      #if ( exists $val->{data_reduction_ratio_to_one} )        { $conf{$id}{data_reduction_ratio}     = $val->{data_reduction_ratio_to_one}; }
      #if ( exists $val->{savings_tb} )                         { $conf{$id}{savings_tb}               = $val->{savings_tb}; }
      #if ( exists $val->{reducing_data_percent} )              { $conf{$id}{reducing_data_prct}       = $val->{reducing_data_percent}; }
      #if ( exists $val->{physical_capacity} ) {
      #  if ( exists $val->{physical_capacity}{total_tb} ) { $conf{$id}{physical_capacity_total} = Xormon::returnBytes($val->{physical_capacity}{total_tb},"TB"); }
      #  if ( exists $val->{physical_capacity}{used_tb} )  { $conf{$id}{physical_capacity_used}  = Xormon::returnBytes($val->{physical_capacity}{used_tb},"TB"); }
      #  if ( exists $val->{physical_capacity}{free_tb} )  { $conf{$id}{physical_capacity_free}  = Xormon::returnBytes($val->{physical_capacity}{free_tb},"TB"); }
      #}
      # capacity_total
      if ( exists $val->{last_stats}{TotalSRPCapacity} ) { # Total SRP Capacity (GB).
        $conf{$id}{capacity_total} = Xormon::returnBytes($val->{last_stats}{TotalSRPCapacity},"GB");
      }
      elsif ( exists $val->{total_usable_cap_gb} ) { # V8.1.2.3 does not support TotalSRPCapacity and UsedSRPCapacity
        $conf{$id}{capacity_total} = Xormon::returnBytes($val->{total_usable_cap_gb},"GB");
      }
      elsif ( exists $val->{usable_total_tb} ) { # V9
        $conf{$id}{capacity_total} = Xormon::returnBytes($val->{usable_total_tb},"TB");
      }
      elsif ( exists $val->{total_tb} ) { # V10
        $conf{$id}{capacity_total} = Xormon::returnBytes($val->{total_tb},"TB");
      }
      # capacity_used
      if ( exists $val->{last_stats}{UsedSRPCapacity} ) { # Used SRP Capacity (GB).
        $conf{$id}{capacity_used} = Xormon::returnBytes($val->{last_stats}{UsedSRPCapacity},"GB");
      }
      elsif ( exists $val->{usable_used_tb} ) { # V9
        $conf{$id}{capacity_used} = Xormon::returnBytes($val->{usable_used_tb},"TB");
      }
      elsif ( exists $val->{used_tb} ) { # V10
        $conf{$id}{capacity_used} = Xormon::returnBytes($val->{used_tb},"TB");
      }
      if ( exists $cfg->{FISERV} && $cfg->{FISERV} eq "1" && exists $val->{usable_used_tb} ) { # customer modification
        $conf{$id}{capacity_used} = Xormon::returnBytes($val->{usable_used_tb},"TB");
      }
      if ( exists $cfg->{VMAX_CAP_USED} && $cfg->{VMAX_CAP_USED} eq "2" && exists $val->{usable_used_tb} ) { # modification for other customers
        $conf{$id}{capacity_used} = Xormon::returnBytes($val->{usable_used_tb},"TB");
      }
      # capacity_free
      if ( exists $conf{$id}{capacity_total} && Xormon::isdigit($conf{$id}{capacity_total}) && exists $conf{$id}{capacity_used} && Xormon::isdigit($conf{$id}{capacity_used}) ) {
        $conf{$id}{capacity_free} = $conf{$id}{capacity_total} - $conf{$id}{capacity_used};
      }
      # subscription
      if ( exists $val->{total_subscribed_cap_gb} ) {
        $conf{$id}{virtual_capacity_total} = Xormon::returnBytes($val->{total_subscribed_cap_gb},"GB");
        if ( exists $conf{$id}{capacity_total} && Xormon::isdigit($conf{$id}{capacity_total}) ) {
          $conf{$id}{subscribed_prct} = int( $conf{$id}{virtual_capacity_total} / ( $conf{$id}{capacity_total} / 100 ) );
        }
      }
      elsif ( exists $val->{subscribed_total_tb} ) { # V9
        $conf{$id}{virtual_capacity_total} = Xormon::returnBytes($val->{subscribed_total_tb},"TB");
        if ( exists $conf{$id}{capacity_total} && Xormon::isdigit($conf{$id}{capacity_total}) ) {
          $conf{$id}{subscribed_prct} = int( $conf{$id}{virtual_capacity_total} / ( $conf{$id}{capacity_total} / 100 ) );
        }
      }
      elsif ( exists $val->{provisioned_tb} && exists $val->{provisioned_percent} ) { # V10
        $conf{$id}{virtual_capacity_total} = Xormon::returnBytes($val->{provisioned_tb},"TB");
        $conf{$id}{subscribed_prct}        = int( $val->{provisioned_percent} );
      }
    }
  }

  #
  # StorageGroup (volume)
  #
  if ( exists $data{conf}{StorageGroup} ) {
    foreach my $label ( sort keys %{ $data{conf}{StorageGroup} } ) {
      my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."volume".$label) );
      # Architecture
      my %arch = (
        "item_id"    => $id,
        "label"      => $label,
        "hw_type"    => "vmax",
        "subsystem"  => "volume",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents"    => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
      # conf
      my $val = $data{conf}{StorageGroup}{$label};
      if ( exists $val->{last_stats}{AllocatedCapacity} ) { $conf{$id}{capacity_total} = Xormon::returnBytes($val->{last_stats}{AllocatedCapacity},"GB"); }
      $conf{$id}{lun_id} = $label;
      #TODO: calculate cap from assigned TDEVs
    }
  }

  #
  # ThinPool (pool, only when SRP doesn't exist)
  #
  if ( ! exists $data{conf}{SRP} && exists $data{conf}{ThinPool} ) {
    foreach my $label ( sort keys %{ $data{conf}{ThinPool} } ) {
      my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."pool".$label) );
      # Architecture
      my %arch = (
        "item_id"    => $id,
        "label"      => $label,
        "hw_type"    => "vmax",
        "subsystem"  => "pool",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents"    => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
      # conf
    }
  }

  #
  # ThinTier (tier, not supported in Unisphere V10)
  #
  if ( exists $data{conf}{ThinTier} ) {
    foreach my $label ( sort keys %{ $data{conf}{ThinTier} } ) {
      my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."tier".$label) );
      # Architecture
      my %arch = (
        "item_id"    => $id,
        "label"      => $label,
        "hw_type"    => "vmax",
        "subsystem"  => "tier",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents"    => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
      # conf
    }
  }

  #
  # FEDirector
  #
  if ( exists $data{conf}{FEDirector} ) {
    foreach my $label ( sort keys %{ $data{conf}{FEDirector} } ) {
      my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."fedirector".$label) );
      # Architecture
      my %arch = (
        "item_id"    => $id,
        "label"      => $label,
        "hw_type"    => "vmax",
        "subsystem"  => "fedirector",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents"    => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
      # conf
    }
  }

  #
  # BEDirector
  #
  if ( exists $data{conf}{BEDirector} ) {
    foreach my $label ( sort keys %{ $data{conf}{BEDirector} } ) {
      my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."bedirector".$label) );
      # Architecture
      my %arch = (
        "item_id"    => $id,
        "label"      => $label,
        "hw_type"    => "vmax",
        "subsystem"  => "bedirector",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents"    => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
      # conf
    }
  }

  #
  # RDFDirector
  #
  if ( exists $data{conf}{RDFDirector} ) {
    foreach my $label ( sort keys %{ $data{conf}{RDFDirector} } ) {
      my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."rdfdirector".$label) );
      # Architecture
      my %arch = (
        "item_id"    => $id,
        "label"      => $label,
        "hw_type"    => "vmax",
        "subsystem"  => "rdfdirector",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents"    => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
      # conf
    }
  }

  #
  # FEPort
  #
  if ( exists $data{conf}{FEPort} ) {
    foreach my $director ( sort keys %{ $data{conf}{FEPort} } ) {
      foreach my $label ( sort keys %{ $data{conf}{FEPort}{$director} } ) {
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."feport"."$director-$label") );
        # Architecture
        my %arch = (
          "item_id"    => $id,
          "label"      => "$director-$label",
          "hw_type"    => "vmax",
          "subsystem"  => "feport",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents"    => [$cfg->{'hostcfg_id'}]
        );
        # conf
        if ( exists $data{conf}{FEPort}{$director}{$label}{identifier} ) {
          $arch{wwn} = Xormon::formatWWN($data{conf}{FEPort}{$director}{$label}{identifier});
          $conf{$id}{wwn} = $data{conf}{FEPort}{$director}{$label}{identifier};
        }
        push(@architecture, \%arch);
      }
    }
  }

  #
  # BEPort
  #
  if ( exists $data{conf}{BEPort} ) {
    foreach my $director ( sort keys %{ $data{conf}{BEPort} } ) {
      foreach my $label ( sort keys %{ $data{conf}{BEPort}{$director} } ) {
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."beport"."$director-$label") );
        # Architecture
        my %arch = (
          "item_id"    => $id,
          "label"      => "$director-$label",
          "hw_type"    => "vmax",
          "subsystem"  => "beport",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents"    => [$cfg->{'hostcfg_id'}]
        );
        push(@architecture, \%arch);
        # conf
      }
    }
  }

  #
  # RDFPort
  #
  if ( exists $data{conf}{RDFPort} ) {
    foreach my $director ( sort keys %{ $data{conf}{RDFPort} } ) {
      foreach my $label ( sort keys %{ $data{conf}{RDFPort}{$director} } ) {
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."rdfport"."$director-$label") );
        # Architecture
        my %arch = (
          "item_id"    => $id,
          "label"      => "$director-$label",
          "hw_type"    => "vmax",
          "subsystem"  => "rdfport",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents"    => [$cfg->{'hostcfg_id'}]
        );
        push(@architecture, \%arch);
        # conf
      }
    }
  }

  #
  # DiskGroup
  #
  if ( exists $data{conf}{DiskGroup} ) {
    foreach my $label ( sort keys %{ $data{conf}{DiskGroup} } ) {
      my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."drive".$label) );
      # Architecture
      my %arch = (
        "item_id"    => $id,
        "label"      => $label,
        "hw_type"    => "vmax",
        "subsystem"  => "drive",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents"    => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
      # conf
      my $val = $data{conf}{StorageGroup}{$label};
      if ( exists $val->{last_stats}{TotalCapacity} ) { $conf{$id}{capacity_total} = Xormon::returnBytes($val->{last_stats}{TotalCapacity},"GB"); }
      if ( exists $val->{last_stats}{UsedCapacity} )  { $conf{$id}{capacity_used}  = Xormon::returnBytes($val->{last_stats}{UsedCapacity},"GB"); }
      if ( exists $conf{$id}{capacity_total} && exists $conf{$id}{capacity_used} ) {
        $conf{$id}{capacity_free} = $conf{$id}{capacity_total} - $conf{$id}{capacity_used};
      }
    }
  }

  #
  # SLO
  #
  if ( exists $data{conf}{SLO} ) {
    foreach my $label ( sort keys %{ $data{conf}{SLO} } ) {
      my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."slo".$label) );
      # Architecture
      my %arch = (
        "item_id"    => $id,
        "label"      => $label,
        "hw_type"    => "vmax",
        "subsystem"  => "slo",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents"    => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
      # conf
    }
  }

  #
  # Host
  #
  if ( exists $data{conf}{Host} ) {
    foreach my $label ( sort keys %{ $data{conf}{Host} } ) {
      my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."host".$label) );
      # Architecture
      my %arch = (
        "item_id"    => $id,
        "label"      => $label,
        "hw_type"    => "vmax",
        "subsystem"  => "host",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents"    => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
      # conf
    }
  }

  #save architecture
  my $rc_arch = $xormon->saveArchitecture(\@architecture);
  $logger->info("save architecture: $rc_arch");

  #save configuration
  my $rc_conf = $xormon->saveConfJSON(\%conf);
  $logger->info("save conf: $rc_conf");

  #print "#################### Architecture\n";
  #print Dumper @architecture;
  #print "#################### CONF\n";
  #print Dumper \%conf;

  return 1;
}

sub get_device_conf {
  my $result = new_get_request("$url_paths->{system}/symmetrix/$cfg->{'symmetrix'}");

  if ( defined $result ) {
    if ( exists $result->{'symmetrix'} && ref($result->{'symmetrix'}) eq "ARRAY" ) {
      foreach ( @{ $result->{'symmetrix'} } ) {
        if ( defined $_->{'symmetrixId'} && $_->{'symmetrixId'} eq $cfg->{'symmetrix'} ) {
          $data{conf}{DEVICE} = $_;
        }
      }
    }
    elsif ( exists $result->{model} || $result->{microcode} ) { # V10
      $data{conf}{DEVICE} = $result;
    }
  }

  return 1;
}

sub prepare_perf {
  my $timestamp_ms = shift;
  my $timestamp    = int($timestamp_ms / 1000); # from ms to sec
  my $ref          = $data{perf}{$timestamp_ms};

  my %data_out;

  #
  # SRP (pool)
  #
  if ( exists $ref->{SRP} ) {
    foreach my $label ( sort keys %{ $ref->{SRP} } ) {
      my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."pool".$label) );
      my $perf = $ref->{SRP}{$label};
      my $conf = $data{conf}{SRP}{$label};

      if ( exists $perf->{HostIOs} )           { $data_out{vmax}{pool}{$id}{$timestamp}{io_rate}     = Xormon::formatNumberByType("int",$perf->{HostIOs}); }
      if ( exists $perf->{HostReads} )         { $data_out{vmax}{pool}{$id}{$timestamp}{read_io}     = Xormon::formatNumberByType("int",$perf->{HostReads}); }
      if ( exists $perf->{HostWrites} )        { $data_out{vmax}{pool}{$id}{$timestamp}{write_io}    = Xormon::formatNumberByType("int",$perf->{HostWrites}); }
      if ( exists $perf->{HostMBs} )           { $data_out{vmax}{pool}{$id}{$timestamp}{data_rate}   = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBs},"MB")); }
      if ( exists $perf->{HostMBReads} )       { $data_out{vmax}{pool}{$id}{$timestamp}{read}        = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBReads},"MB")); }
      if ( exists $perf->{HostMBWritten} )     { $data_out{vmax}{pool}{$id}{$timestamp}{write}       = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBWritten},"MB")); }
      if ( exists $perf->{BEReqs} )            { $data_out{vmax}{pool}{$id}{$timestamp}{io_rate_b}   = Xormon::formatNumberByType("int",$perf->{BEReqs}); }
      if ( exists $perf->{BEReadReqs} )        { $data_out{vmax}{pool}{$id}{$timestamp}{read_io_b}   = Xormon::formatNumberByType("int",$perf->{BEReadReqs}); }
      if ( exists $perf->{BEWriteReqs} )       { $data_out{vmax}{pool}{$id}{$timestamp}{write_io_b}  = Xormon::formatNumberByType("int",$perf->{BEWriteReqs}); }
      if ( exists $perf->{BEMBTransferred} )   { $data_out{vmax}{pool}{$id}{$timestamp}{data_rate_b} = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{BEMBTransferred},"MB")); }
      if ( exists $perf->{BEMBReads} )         { $data_out{vmax}{pool}{$id}{$timestamp}{read_b}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{BEMBReads},"MB")); }
      if ( exists $perf->{BEMBWritten} )       { $data_out{vmax}{pool}{$id}{$timestamp}{write_b}     = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{BEMBWritten},"MB")); }
      if ( exists $perf->{ResponseTime} )      { $data_out{vmax}{pool}{$id}{$timestamp}{resp_t}      = Xormon::formatNumberByType("float4",$perf->{ResponseTime}); }
      if ( exists $perf->{ReadResponseTime} )  { $data_out{vmax}{pool}{$id}{$timestamp}{resp_t_r}    = Xormon::formatNumberByType("float4",$perf->{ReadResponseTime}); }
      if ( exists $perf->{WriteResponseTime} ) { $data_out{vmax}{pool}{$id}{$timestamp}{resp_t_w}    = Xormon::formatNumberByType("float4",$perf->{WriteResponseTime}); }
      # capacity_total
      if ( exists $perf->{TotalSRPCapacity} ) { # Total SRP Capacity (GB).
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total} = Xormon::returnBytes($perf->{TotalSRPCapacity},"GB");
      }
      elsif ( exists $conf->{total_usable_cap_gb} ) { # V8.1.2.3 does not support TotalSRPCapacity and UsedSRPCapacity
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total} = Xormon::returnBytes($conf->{total_usable_cap_gb},"GB");
      }
      elsif ( exists $conf->{usable_total_tb} ) { # V9
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total} = Xormon::returnBytes($conf->{usable_total_tb},"TB");
      }
      elsif ( exists $conf->{total_tb} ) { # V10
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total} = Xormon::returnBytes($conf->{total_tb},"TB");
      }
      # capacity_used
      if ( exists $perf->{UsedSRPCapacity} ) { # Used SRP Capacity (GB).
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_used} = Xormon::returnBytes($perf->{UsedSRPCapacity},"GB");
      }
      elsif ( exists $conf->{usable_used_tb} ) { # V9
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_used} = Xormon::returnBytes($conf->{usable_used_tb},"TB");
      }
      elsif ( exists $conf->{used_tb} ) { # V10
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_used} = Xormon::returnBytes($conf->{used_tb},"TB");
      }
      if ( exists $cfg->{FISERV} && $cfg->{FISERV} eq "1" && exists $conf->{usable_used_tb} ) { # customer modification
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_used} = Xormon::returnBytes($conf->{usable_used_tb},"TB");
      }
      if ( exists $cfg->{VMAX_CAP_USED} && $cfg->{VMAX_CAP_USED} eq "2" && exists $conf->{usable_used_tb} ) { # modification for other customers
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_used} = Xormon::returnBytes($conf->{usable_used_tb},"TB");
      }
      # capacity_free
      if ( exists $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total} && Xormon::isdigit($data_out{vmax}{pool}{$id}{$timestamp}{capacity_total}) && exists $data_out{vmax}{pool}{$id}{$timestamp}{capacity_used} && Xormon::isdigit($data_out{vmax}{pool}{$id}{$timestamp}{capacity_used}) ) {
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_free}         = $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total} - $data_out{vmax}{pool}{$id}{$timestamp}{capacity_used};
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_used_percent} = $data_out{vmax}{pool}{$id}{$timestamp}{capacity_used} / ( $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total} / 100 );
      }
      # subscription/virtual capacity
      if ( exists $conf->{total_subscribed_cap_gb} ) {
        $data_out{vmax}{pool}{$id}{$timestamp}{virtual_capacity_total} = Xormon::returnBytes($conf->{total_subscribed_cap_gb},"GB");
      }
      elsif ( exists $conf->{subscribed_total_tb} ) { # V9
        $data_out{vmax}{pool}{$id}{$timestamp}{virtual_capacity_total} = Xormon::returnBytes($conf->{subscribed_total_tb},"TB");
      }
      elsif ( exists $conf->{provisioned_tb} && exists $conf->{provisioned_percent} ) { # V10
        $data_out{vmax}{pool}{$id}{$timestamp}{virtual_capacity_total} = Xormon::returnBytes($conf->{provisioned_tb},"TB");
        #$data_out{vmax}{pool}{$id}{$timestamp}{subscribed_prct}        = int( $conf->{provisioned_percent} );
      }
      # oversubscription capacity
      if ( exists $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total} && Xormon::isdigit($data_out{vmax}{pool}{$id}{$timestamp}{capacity_total}) && exists $data_out{vmax}{pool}{$id}{$timestamp}{virtual_capacity_total} && Xormon::isdigit($data_out{vmax}{pool}{$id}{$timestamp}{virtual_capacity_total}) ) {
        if ( $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total} < $data_out{vmax}{pool}{$id}{$timestamp}{virtual_capacity_total} ) {
          $data_out{vmax}{pool}{$id}{$timestamp}{capacity_oversubscr}         = $data_out{vmax}{pool}{$id}{$timestamp}{virtual_capacity_total} - $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total};
          $data_out{vmax}{pool}{$id}{$timestamp}{capacity_oversubscr_percent} = int( $data_out{vmax}{pool}{$id}{$timestamp}{virtual_capacity_total} / $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total} ) * 100;
        }
        else {
          $data_out{vmax}{pool}{$id}{$timestamp}{capacity_oversubscr}         = 0;
          $data_out{vmax}{pool}{$id}{$timestamp}{capacity_oversubscr_percent} = 0;
        }
      }
      # calculate block size
      my $data_act = $data_out{vmax}{pool}{$id}{$timestamp};
      if ( exists $data_act->{io_rate} && Xormon::isdigit($data_act->{io_rate}) && exists $data_act->{data_rate} ) {
        if ( $data_act->{io_rate} == 0 ) {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size} = 0;
        }
        else {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size} = Xormon::formatNumberByType("float4",$data_act->{data_rate} / $data_act->{io_rate});
        }
      }
      if ( exists $data_act->{read_io} && Xormon::isdigit($data_act->{read_io}) && exists $data_act->{read} ) {
        if ( $data_act->{read_io} == 0 ) {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size_r} = 0;
        }
        else {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size_r} = Xormon::formatNumberByType("float4",$data_act->{read} / $data_act->{read_io});
        }
      }
      if ( exists $data_act->{write_io} && Xormon::isdigit($data_act->{write_io}) && exists $data_act->{write} ) {
        if ( $data_act->{write_io} == 0 ) {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size_w} = 0;
        }
        else {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size_w} = Xormon::formatNumberByType("float4",$data_act->{write} / $data_act->{write_io});
        }
      }
    }
  }

  #
  # ThinPool (pool, only when SRP doesn't exist)
  #
  if ( exists $ref->{ThinPool} ) {
    foreach my $label ( sort keys %{ $ref->{ThinPool} } ) {
      my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."pool".$label) );
      my $perf = $ref->{ThinPool}{$label};
      my $conf = $data{conf}{ThinPool}{$label};

      if ( exists $perf->{HostIOs} )           { $data_out{vmax}{pool}{$id}{$timestamp}{io_rate}        = Xormon::formatNumberByType("int",$perf->{HostIOs}); }
      if ( exists $perf->{HostReads} )         { $data_out{vmax}{pool}{$id}{$timestamp}{read_io}        = Xormon::formatNumberByType("int",$perf->{HostReads}); }
      if ( exists $perf->{HostWrites} )        { $data_out{vmax}{pool}{$id}{$timestamp}{write_io}       = Xormon::formatNumberByType("int",$perf->{HostWrites}); }
      if ( exists $perf->{HostMBs} )           { $data_out{vmax}{pool}{$id}{$timestamp}{data_rate}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBs},"MB")); }
      if ( exists $perf->{HostMBReads} )       { $data_out{vmax}{pool}{$id}{$timestamp}{read}           = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBReads},"MB")); }
      if ( exists $perf->{HostMBWritten} )     { $data_out{vmax}{pool}{$id}{$timestamp}{write}          = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBWritten},"MB")); }
      if ( exists $perf->{ReadResponseTime} )  { $data_out{vmax}{pool}{$id}{$timestamp}{resp_t_r}       = Xormon::formatNumberByType("float4",$perf->{ReadResponseTime}); }
      if ( exists $perf->{WriteResponseTime} ) { $data_out{vmax}{pool}{$id}{$timestamp}{resp_t_w}       = Xormon::formatNumberByType("float4",$perf->{WriteResponseTime}); }
      if ( exists $perf->{BEMBTransferred} )   { $data_out{vmax}{pool}{$id}{$timestamp}{data_rate_b}    = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{BEMBTransferred},"MB")); }
      if ( exists $perf->{BEMBReads} )         { $data_out{vmax}{pool}{$id}{$timestamp}{read_b}         = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{BEMBReads},"MB")); }
      if ( exists $perf->{BEMBWritten} )       { $data_out{vmax}{pool}{$id}{$timestamp}{write_b}        = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{BEMBWritten},"MB")); }
      if ( exists $perf->{BEReqs} )            { $data_out{vmax}{pool}{$id}{$timestamp}{io_rate_b}      = Xormon::formatNumberByType("int",$perf->{BEReqs}); }
      if ( exists $perf->{BEReadReqs} )        { $data_out{vmax}{pool}{$id}{$timestamp}{read_io_b}      = Xormon::formatNumberByType("int",$perf->{BEReadReqs}); }
      if ( exists $perf->{BEWriteReqs} )       { $data_out{vmax}{pool}{$id}{$timestamp}{write_io_b}     = Xormon::formatNumberByType("int",$perf->{BEWriteReqs}); }
      if ( exists $perf->{TotalPoolCapacity} ) { $data_out{vmax}{pool}{$id}{$timestamp}{capacity_total} = Xormon::returnBytes($perf->{TotalPoolCapacity},"GB"); }
      if ( exists $perf->{UsedPoolCapacity} )  { $data_out{vmax}{pool}{$id}{$timestamp}{capacity_used}  = Xormon::returnBytes($perf->{UsedPoolCapacity},"GB"); }
      # resp_t
      my $data_act = $data_out{vmax}{pool}{$id}{$timestamp};
      if ( exists $data_act->{resp_t_r} && Xormon::isdigit($data_act->{resp_t_r}) && exists $data_act->{read_io} && Xormon::isdigit($data_act->{read_io}) && exists $data_act->{resp_t_w} && Xormon::isdigit($data_act->{resp_t_w}) && exists $data_act->{write_io} && Xormon::isdigit($data_act->{write_io}) ) {
        my $resp_t = Xormon::calculate_weighted_avg( [ { count => $data_act->{read_io}, value => $data_act->{resp_t_r} }, { count => $data_act->{write_io}, value => $data_act->{resp_t_w} } ] );
        if ( defined $resp_t ) { $data_out{vmax}{pool}{$id}{$timestamp}{resp_t} = Xormon::formatNumberByType("float4",$resp_t); }
      }
      # capacity_free
      if ( exists $data_act->{capacity_total} && Xormon::isdigit($data_act->{capacity_total}) && exists $data_act->{capacity_used} && Xormon::isdigit($data_act->{capacity_used}) ) {
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_free}         = $data_act->{capacity_total} - $data_act->{capacity_used};
        $data_out{vmax}{pool}{$id}{$timestamp}{capacity_used_percent} = $data_act->{capacity_used} / ( $data_act->{capacity_total} / 100 );
      }
      # calculate block size
      if ( exists $data_act->{io_rate} && Xormon::isdigit($data_act->{io_rate}) && exists $data_act->{data_rate} ) {
        if ( $data_act->{io_rate} == 0 ) {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size} = 0;
        }
        else {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size} = Xormon::formatNumberByType("float4",$data_act->{data_rate} / $data_act->{io_rate});
        }
      }
      if ( exists $data_act->{read_io} && Xormon::isdigit($data_act->{read_io}) && exists $data_act->{read} ) {
        if ( $data_act->{read_io} == 0 ) {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size_r} = 0;
        }
        else {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size_r} = Xormon::formatNumberByType("float4",$data_act->{read} / $data_act->{read_io});
        }
      }
      if ( exists $data_act->{write_io} && Xormon::isdigit($data_act->{write_io}) && exists $data_act->{write} ) {
        if ( $data_act->{write_io} == 0 ) {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size_w} = 0;
        }
        else {
          $data_out{vmax}{pool}{$id}{$timestamp}{block_size_w} = Xormon::formatNumberByType("float4",$data_act->{write} / $data_act->{write_io});
        }
      }
      # percent_subscription
      if ( exists $conf->{percent_subscription} && Xormon::isdigit($conf->{percent_subscription}) ) {
        $data_out{vmax}{pool}{$id}{$timestamp}{subscribed_prct}        = int( $conf->{percent_subscription} );

        if ( $data_act->{capacity_total} && Xormon::isdigit($data_act->{capacity_total}) ) {
          $data_out{vmax}{pool}{$id}{$timestamp}{virtual_capacity_total} = int( ( $data_act->{capacity_total} / 100 ) * $conf->{percent_subscription} );
        }
      }
    }
  }

  #
  # DEVICE (must be after pool sections, because we need capacity stats from pools)
  #
  if ( exists $ref->{Array} ) {
    foreach my $label ( sort keys %{ $ref->{Array} } ) {
      my $id   = $cfg->{'hostcfg_id'};
      my $perf = $ref->{Array}{$label};

      if ( exists $perf->{HostMBs} )           { $data_out{vmax}{device}{$id}{$timestamp}{data_rate} = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBs},"MB")); }
      if ( exists $perf->{HostMBReads} )       { $data_out{vmax}{device}{$id}{$timestamp}{read}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBReads},"MB")); }
      if ( exists $perf->{HostMBWritten} )     { $data_out{vmax}{device}{$id}{$timestamp}{write}     = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBWritten},"MB")); }
      if ( exists $perf->{HostIOs} )           { $data_out{vmax}{device}{$id}{$timestamp}{io_rate}   = Xormon::formatNumberByType("int",$perf->{HostIOs}); }
      if ( exists $perf->{HostReads} )         { $data_out{vmax}{device}{$id}{$timestamp}{read_io}   = Xormon::formatNumberByType("int",$perf->{HostReads}); }
      if ( exists $perf->{HostWrites} )        { $data_out{vmax}{device}{$id}{$timestamp}{write_io}  = Xormon::formatNumberByType("int",$perf->{HostWrites}); }
      if ( exists $perf->{ReadResponseTime} )  { $data_out{vmax}{device}{$id}{$timestamp}{resp_t_r}  = Xormon::formatNumberByType("float4",$perf->{ReadResponseTime}); }
      if ( exists $perf->{WriteResponseTime} ) { $data_out{vmax}{device}{$id}{$timestamp}{resp_t_w}  = Xormon::formatNumberByType("float4",$perf->{WriteResponseTime}); }
      # resp_t
      my $data_act = $data_out{vmax}{device}{$id}{$timestamp};
      if ( exists $data_act->{resp_t_r} && Xormon::isdigit($data_act->{resp_t_r}) && exists $data_act->{read_io} && Xormon::isdigit($data_act->{read_io}) && exists $data_act->{resp_t_w} && Xormon::isdigit($data_act->{resp_t_w}) && exists $data_act->{write_io} && Xormon::isdigit($data_act->{write_io}) ) {
        my $resp_t = Xormon::calculate_weighted_avg( [ { count => $data_act->{read_io}, value => $data_act->{resp_t_r} }, { count => $data_act->{write_io}, value => $data_act->{resp_t_w} } ] );
        if ( defined $resp_t ) { $data_out{vmax}{device}{$id}{$timestamp}{resp_t} = Xormon::formatNumberByType("float4",$resp_t); }
      }
      # add capacity from SRP or ThinPools
      if ( exists $data_out{vmax}{pool} ) {
        my $pool_idx = 0;
        foreach my $pool_id ( sort keys %{ $data_out{vmax}{pool} } ) {
          $pool_idx++;
          foreach my $timestamp_tmp ( sort keys %{ $data_out{vmax}{pool}{$pool_id} } ) {
            my $pool_stats = $data_out{vmax}{pool}{$pool_id}{$timestamp_tmp};
            if ( exists $pool_stats->{capacity_total} && Xormon::isdigit($pool_stats->{capacity_total}) ) {
              if ( exists $data_out{vmax}{device}{$id}{$timestamp}{capacity_total} ) {
                $data_out{vmax}{device}{$id}{$timestamp}{capacity_total} += $pool_stats->{capacity_total};
              }
              else {
                $data_out{vmax}{device}{$id}{$timestamp}{capacity_total} = $pool_stats->{capacity_total};
              }
            }
            if ( exists $pool_stats->{capacity_free} && Xormon::isdigit($pool_stats->{capacity_free}) ) {
              if ( exists $data_out{vmax}{device}{$id}{$timestamp}{capacity_free} ) {
                $data_out{vmax}{device}{$id}{$timestamp}{capacity_free} += $pool_stats->{capacity_free};
              }
              else {
                $data_out{vmax}{device}{$id}{$timestamp}{capacity_free} = $pool_stats->{capacity_free};
              }
            }
            if ( exists $pool_stats->{capacity_used} && Xormon::isdigit($pool_stats->{capacity_used}) ) {
              if ( exists $data_out{vmax}{device}{$id}{$timestamp}{capacity_used} ) {
                $data_out{vmax}{device}{$id}{$timestamp}{capacity_used} += $pool_stats->{capacity_used};
              }
              else {
                $data_out{vmax}{device}{$id}{$timestamp}{capacity_used} = $pool_stats->{capacity_used};
              }
            }
            if ( exists $pool_stats->{virtual_capacity_total} && Xormon::isdigit($pool_stats->{virtual_capacity_total}) ) {
              if ( exists $data_out{vmax}{device}{$id}{$timestamp}{virtual_capacity_total} ) {
                $data_out{vmax}{device}{$id}{$timestamp}{virtual_capacity_total} += $pool_stats->{virtual_capacity_total};
              }
              else {
                $data_out{vmax}{device}{$id}{$timestamp}{virtual_capacity_total} = $pool_stats->{virtual_capacity_total};
              }
            }
            if ( exists $pool_stats->{subscribed_prct} && Xormon::isdigit($pool_stats->{subscribed_prct}) ) {
              if ( exists $data_out{vmax}{device}{$id}{$timestamp}{subscribed_prct} ) {
                $data_out{vmax}{device}{$id}{$timestamp}{subscribed_prct} += $pool_stats->{subscribed_prct};
              }
              else {
                $data_out{vmax}{device}{$id}{$timestamp}{subscribed_prct} = $pool_stats->{subscribed_prct};
              }
            }
          }
        }
        # subscribed_prct must be avg
        if ( exists $data_out{vmax}{device}{$id}{$timestamp}{subscribed_prct} && $pool_idx > 1 ) {
          $data_out{vmax}{device}{$id}{$timestamp}{subscribed_prct} = $data_out{vmax}{device}{$id}{$timestamp}{subscribed_prct} / $pool_idx;
        }
        # capacity_used_percent
        if ( exists $data_out{vmax}{device}{$id}{$timestamp}{capacity_total} && Xormon::isdigit($data_out{vmax}{device}{$id}{$timestamp}{capacity_total}) && exists $data_out{vmax}{device}{$id}{$timestamp}{capacity_used} && Xormon::isdigit($data_out{vmax}{device}{$id}{$timestamp}{capacity_used}) ) {
          $data_out{vmax}{device}{$id}{$timestamp}{capacity_used_percent} = $data_out{vmax}{device}{$id}{$timestamp}{capacity_used} / ( $data_out{vmax}{device}{$id}{$timestamp}{capacity_total} / 100 );
        }
      }
    }
  }

  #
  # StorageGroup (volume)
  #
  if ( exists $ref->{StorageGroup} ) {
    foreach my $label ( sort keys %{ $ref->{StorageGroup} } ) {
      my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."volume".$label) );
      my $perf = $ref->{StorageGroup}{$label};
      my $conf = $data{conf}{StorageGroup}{$label};

      if ( exists $perf->{HostIOs} )           { $data_out{vmax}{volume}{$id}{$timestamp}{io_rate}        = Xormon::formatNumberByType("int",$perf->{HostIOs}); }
      if ( exists $perf->{HostReads} )         { $data_out{vmax}{volume}{$id}{$timestamp}{read_io}        = Xormon::formatNumberByType("int",$perf->{HostReads}); }
      if ( exists $perf->{HostWrites} )        { $data_out{vmax}{volume}{$id}{$timestamp}{write_io}       = Xormon::formatNumberByType("int",$perf->{HostWrites}); }
      if ( exists $perf->{HostMBs} )           { $data_out{vmax}{volume}{$id}{$timestamp}{data_rate}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBs},"MB")); }
      if ( exists $perf->{HostMBReads} )       { $data_out{vmax}{volume}{$id}{$timestamp}{read}           = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBReads},"MB")); }
      if ( exists $perf->{HostMBWritten} )     { $data_out{vmax}{volume}{$id}{$timestamp}{write}          = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBWritten},"MB")); }
      if ( exists $perf->{ResponseTime} )      { $data_out{vmax}{volume}{$id}{$timestamp}{resp_t}         = Xormon::formatNumberByType("float4",$perf->{ResponseTime}); }
      if ( exists $perf->{ReadResponseTime} )  { $data_out{vmax}{volume}{$id}{$timestamp}{resp_t_r}       = Xormon::formatNumberByType("float4",$perf->{ReadResponseTime}); }
      if ( exists $perf->{WriteResponseTime} ) { $data_out{vmax}{volume}{$id}{$timestamp}{resp_t_w}       = Xormon::formatNumberByType("float4",$perf->{WriteResponseTime}); }
      if ( exists $perf->{AllocatedCapacity} ) { $data_out{vmax}{volume}{$id}{$timestamp}{capacity_total} = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{AllocatedCapacity},"GB")); }
      if ( exists $perf->{PercentHit} )        { $data_out{vmax}{volume}{$id}{$timestamp}{cache_hit}      = Xormon::formatNumberByType("int",$perf->{PercentHit}); }
      if ( exists $perf->{PercentReadHit} )    { $data_out{vmax}{volume}{$id}{$timestamp}{r_cache_hit}    = Xormon::formatNumberByType("int",$perf->{PercentReadHit}); }
      if ( exists $perf->{PercentWriteHit} )   { $data_out{vmax}{volume}{$id}{$timestamp}{w_cache_hit}    = Xormon::formatNumberByType("int",$perf->{PercentWriteHit}); }
      if ( exists $perf->{RdfResponseTime} )   { $data_out{vmax}{volume}{$id}{$timestamp}{rdf_resp_t}     = Xormon::formatNumberByType("float4",$perf->{RdfResponseTime}); }
      if ( exists $perf->{RdfReads} )          { $data_out{vmax}{volume}{$id}{$timestamp}{rdf_read_io}    = Xormon::formatNumberByType("int",$perf->{RdfReads}); }
      if ( exists $perf->{RdfWrites} )         { $data_out{vmax}{volume}{$id}{$timestamp}{rdf_write_io}   = Xormon::formatNumberByType("int",$perf->{RdfWrites}); }
      # calculate block size
      my $data_act = $data_out{vmax}{volume}{$id}{$timestamp};
      if ( exists $data_act->{io_rate} && Xormon::isdigit($data_act->{io_rate}) && exists $data_act->{data_rate} ) {
        if ( $data_act->{io_rate} == 0 ) {
          $data_out{vmax}{volume}{$id}{$timestamp}{block_size} = 0;
        }
        else {
          $data_out{vmax}{volume}{$id}{$timestamp}{block_size} = Xormon::formatNumberByType("float4",$data_act->{data_rate} / $data_act->{io_rate});
        }
      }
      if ( exists $data_act->{read_io} && Xormon::isdigit($data_act->{read_io}) && exists $data_act->{read} ) {
        if ( $data_act->{read_io} == 0 ) {
          $data_out{vmax}{volume}{$id}{$timestamp}{block_size_r} = 0;
        }
        else {
          $data_out{vmax}{volume}{$id}{$timestamp}{block_size_r} = Xormon::formatNumberByType("float4",$data_act->{read} / $data_act->{read_io});
        }
      }
      if ( exists $data_act->{write_io} && Xormon::isdigit($data_act->{write_io}) && exists $data_act->{write} ) {
        if ( $data_act->{write_io} == 0 ) {
          $data_out{vmax}{volume}{$id}{$timestamp}{block_size_w} = 0;
        }
        else {
          $data_out{vmax}{volume}{$id}{$timestamp}{block_size_w} = Xormon::formatNumberByType("float4",$data_act->{write} / $data_act->{write_io});
        }
      }
      # rdf_io_rate
      if ( exists $data_act->{rdf_read_io} && Xormon::isdigit($data_act->{rdf_read_io}) && exists $data_act->{rdf_write_io} && Xormon::isdigit($data_act->{rdf_write_io}) ) {
        $data_out{vmax}{volume}{$id}{$timestamp}{rdf_io_rate} = Xormon::formatNumberByType("int",$data_act->{rdf_read_io}+$data_act->{rdf_write_io});
      }
    }
  }

  #
  # ThinTier (tier, not supported in Unisphere V10)
  #
  if ( exists $ref->{ThinTier} ) {
    foreach my $label ( sort keys %{ $ref->{ThinTier} } ) {
      my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."tier".$label) );
      my $perf = $ref->{ThinTier}{$label};
      my $conf = $data{conf}{ThinTier}{$label};

      if ( exists $perf->{HostMBs} )           { $data_out{vmax}{tier}{$id}{$timestamp}{data_rate}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBs},"MB")); }
      if ( exists $perf->{HostMBReads} )       { $data_out{vmax}{tier}{$id}{$timestamp}{read}           = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBReads},"MB")); }
      if ( exists $perf->{HostMBWritten} )     { $data_out{vmax}{tier}{$id}{$timestamp}{write}          = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBWritten},"MB")); }
      if ( exists $perf->{HostIOs} )           { $data_out{vmax}{tier}{$id}{$timestamp}{io_rate}        = Xormon::formatNumberByType("int",$perf->{HostIOs}); }
      if ( exists $perf->{HostReads} )         { $data_out{vmax}{tier}{$id}{$timestamp}{read_io}        = Xormon::formatNumberByType("int",$perf->{HostReads}); }
      if ( exists $perf->{HostWrites} )        { $data_out{vmax}{tier}{$id}{$timestamp}{write_io}       = Xormon::formatNumberByType("int",$perf->{HostWrites}); }
      if ( exists $perf->{ReadResponseTime} )  { $data_out{vmax}{tier}{$id}{$timestamp}{resp_t_r}       = Xormon::formatNumberByType("float4",$perf->{ReadResponseTime}); }
      if ( exists $perf->{WriteResponseTime} ) { $data_out{vmax}{tier}{$id}{$timestamp}{resp_t_w}       = Xormon::formatNumberByType("float4",$perf->{WriteResponseTime}); }
      if ( exists $perf->{PercentReadHit} )    { $data_out{vmax}{tier}{$id}{$timestamp}{r_cache_hit}    = Xormon::formatNumberByType("int",$perf->{PercentReadHit}); }
      if ( exists $perf->{PercentWriteHit} )   { $data_out{vmax}{tier}{$id}{$timestamp}{w_cache_hit}    = Xormon::formatNumberByType("int",$perf->{PercentWriteHit}); }
      if ( exists $perf->{BEMBTransferred} )   { $data_out{vmax}{tier}{$id}{$timestamp}{data_rate_b}    = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{BEMBTransferred},"MB")); }
      if ( exists $perf->{BEMBReads} )         { $data_out{vmax}{tier}{$id}{$timestamp}{read_b}         = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{BEMBReads},"MB")); }
      if ( exists $perf->{BEMBWritten} )       { $data_out{vmax}{tier}{$id}{$timestamp}{write_b}        = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{BEMBWritten},"MB")); }
      if ( exists $perf->{BEReqs} )            { $data_out{vmax}{tier}{$id}{$timestamp}{io_rate_b}      = Xormon::formatNumberByType("int",$perf->{BEReqs}); }
      if ( exists $perf->{BEReadReqs} )        { $data_out{vmax}{tier}{$id}{$timestamp}{read_io_b}      = Xormon::formatNumberByType("int",$perf->{BEReadReqs}); }
      if ( exists $perf->{BEWriteReqs} )       { $data_out{vmax}{tier}{$id}{$timestamp}{write_io_b}     = Xormon::formatNumberByType("int",$perf->{BEWriteReqs}); }
      if ( exists $perf->{TotalPoolCapacity} ) { $data_out{vmax}{tier}{$id}{$timestamp}{capacity_total} = Xormon::returnBytes($perf->{TotaltierCapacity},"GB"); }
      if ( exists $perf->{UsedPoolCapacity} )  { $data_out{vmax}{tier}{$id}{$timestamp}{capacity_used}  = Xormon::returnBytes($perf->{UsedtierCapacity},"GB"); }
      my $data_act = $data_out{vmax}{tier}{$id}{$timestamp};
      # capacity_free
      if ( exists $data_act->{capacity_total} && Xormon::isdigit($data_act->{capacity_total}) && exists $data_act->{capacity_used} && Xormon::isdigit($data_act->{capacity_used}) ) {
        $data_out{vmax}{tier}{$id}{$timestamp}{capacity_free}         = $data_act->{capacity_total} - $data_act->{capacity_used};
        $data_out{vmax}{tier}{$id}{$timestamp}{capacity_used_percent} = $data_act->{capacity_used} / ( $data_act->{capacity_total} / 100 );
      }
      # resp_t
      if ( exists $data_act->{resp_t_r} && Xormon::isdigit($data_act->{resp_t_r}) && exists $data_act->{read_io} && Xormon::isdigit($data_act->{read_io}) && exists $data_act->{resp_t_w} && Xormon::isdigit($data_act->{resp_t_w}) && exists $data_act->{write_io} && Xormon::isdigit($data_act->{write_io}) ) {
        my $resp_t = Xormon::calculate_weighted_avg( [ { count => $data_act->{read_io}, value => $data_act->{resp_t_r} }, { count => $data_act->{write_io}, value => $data_act->{resp_t_w} } ] );
        if ( defined $resp_t ) { $data_out{vmax}{tier}{$id}{$timestamp}{resp_t} = Xormon::formatNumberByType("float4",$resp_t); }
      }
    }
  }

  #
  # FEDirector
  #
  if ( exists $ref->{FEDirector} ) {
    foreach my $label ( sort keys %{ $ref->{FEDirector} } ) {
      my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."fedirector".$label) );
      my $perf = $ref->{FEDirector}{$label};
      my $conf = $data{conf}{FEDirector}{$label};

      if ( exists $perf->{HostMBs} )            { $data_out{vmax}{fedirector}{$id}{$timestamp}{data_rate}   = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBs},"MB")); }
      if ( exists $perf->{Reqs} )               { $data_out{vmax}{fedirector}{$id}{$timestamp}{io_rate}     = Xormon::formatNumberByType("int",$perf->{Reqs}); }
      if ( exists $perf->{ReadReqs} )           { $data_out{vmax}{fedirector}{$id}{$timestamp}{read_io}     = Xormon::formatNumberByType("int",$perf->{ReadReqs}); }
      if ( exists $perf->{WriteReqs} )          { $data_out{vmax}{fedirector}{$id}{$timestamp}{write_io}    = Xormon::formatNumberByType("int",$perf->{WriteReqs}); }
      if ( exists $perf->{ReadResponseTime} )   { $data_out{vmax}{fedirector}{$id}{$timestamp}{resp_t_r}    = Xormon::formatNumberByType("float4",$perf->{ReadResponseTime}); }
      if ( exists $perf->{WriteResponseTime} )  { $data_out{vmax}{fedirector}{$id}{$timestamp}{resp_t_w}    = Xormon::formatNumberByType("float4",$perf->{WriteResponseTime}); }
      if ( exists $perf->{PercentBusy} )        { $data_out{vmax}{fedirector}{$id}{$timestamp}{cpu_busy}    = Xormon::formatNumberByType("int",$perf->{PercentBusy}); }
      if ( exists $perf->{PercentHitReqs} )     { $data_out{vmax}{fedirector}{$id}{$timestamp}{cache_hit}   = Xormon::formatNumberByType("int",$perf->{PercentHitReqs}); }
      if ( exists $perf->{PercentReadReqHit} )  { $data_out{vmax}{fedirector}{$id}{$timestamp}{r_cache_hit} = Xormon::formatNumberByType("int",$perf->{PercentReadReqHit}); }
      if ( exists $perf->{PercentWriteReqHit} ) { $data_out{vmax}{fedirector}{$id}{$timestamp}{w_cache_hit} = Xormon::formatNumberByType("int",$perf->{PercentWriteReqHit}); }
      my $data_act = $data_out{vmax}{fedirector}{$id}{$timestamp};
      # resp_t
      if ( exists $data_act->{resp_t_r} && Xormon::isdigit($data_act->{resp_t_r}) && exists $data_act->{read_io} && Xormon::isdigit($data_act->{read_io}) && exists $data_act->{resp_t_w} && Xormon::isdigit($data_act->{resp_t_w}) && exists $data_act->{write_io} && Xormon::isdigit($data_act->{write_io}) ) {
        my $resp_t = Xormon::calculate_weighted_avg( [ { count => $data_act->{read_io}, value => $data_act->{resp_t_r} }, { count => $data_act->{write_io}, value => $data_act->{resp_t_w} } ] );
        if ( defined $resp_t ) { $data_out{vmax}{fedirector}{$id}{$timestamp}{resp_t} = Xormon::formatNumberByType("float4",$resp_t); }
      }
    }
  }

  #
  # BEDirector
  #
  if ( exists $ref->{BEDirector} ) {
    foreach my $label ( sort keys %{ $ref->{BEDirector} } ) {
      my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."bedirector".$label) );
      my $perf = $ref->{BEDirector}{$label};
      my $conf = $data{conf}{BEDirector}{$label};

      if ( exists $perf->{MBs} )               { $data_out{vmax}{bedirector}{$id}{$timestamp}{data_rate} = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBs},"MB")); }
      if ( exists $perf->{MBRead} )            { $data_out{vmax}{bedirector}{$id}{$timestamp}{read}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBRead},"MB")); }
      if ( exists $perf->{MBWritten} )         { $data_out{vmax}{bedirector}{$id}{$timestamp}{write}     = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBWritten},"MB")); }
      if ( exists $perf->{IOs} )               { $data_out{vmax}{bedirector}{$id}{$timestamp}{io_rate}   = Xormon::formatNumberByType("int",$perf->{IOs}); }
      if ( exists $perf->{ReadReqs} )          { $data_out{vmax}{bedirector}{$id}{$timestamp}{read_io}   = Xormon::formatNumberByType("int",$perf->{ReadReqs}); }
      if ( exists $perf->{WriteReqs} )         { $data_out{vmax}{bedirector}{$id}{$timestamp}{write_io}  = Xormon::formatNumberByType("int",$perf->{WriteReqs}); }
      if ( exists $perf->{PercentBusy} )       { $data_out{vmax}{bedirector}{$id}{$timestamp}{cpu_busy}  = Xormon::formatNumberByType("int",$perf->{PercentBusy}); }
    }
  }

  #
  # RDFDirector
  #
  if ( exists $ref->{RDFDirector} ) {
    foreach my $label ( sort keys %{ $ref->{RDFDirector} } ) {
      my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."rdfdirector".$label) );
      my $perf = $ref->{RDFDirector}{$label};
      my $conf = $data{conf}{RDFDirector}{$label};

      if ( exists $perf->{MBSentAndReceived} ) { $data_out{vmax}{rdfdirector}{$id}{$timestamp}{data_rate}  = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBSentAndReceivedtMBs},"MB")); }
      if ( exists $perf->{MBRead} )            { $data_out{vmax}{rdfdirector}{$id}{$timestamp}{read}       = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBRead},"MB")); }
      if ( exists $perf->{MBWritten} )         { $data_out{vmax}{rdfdirector}{$id}{$timestamp}{write}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBWritten},"MB")); }
      if ( exists $perf->{IOs} )               { $data_out{vmax}{rdfdirector}{$id}{$timestamp}{io_rate}    = Xormon::formatNumberByType("int",$perf->{IOs}); }
      if ( exists $perf->{ReadReqs} )          { $data_out{vmax}{rdfdirector}{$id}{$timestamp}{read_io}    = Xormon::formatNumberByType("int",$perf->{ReadReqs}); }
      if ( exists $perf->{WriteReqs} )         { $data_out{vmax}{rdfdirector}{$id}{$timestamp}{write_io}   = Xormon::formatNumberByType("int",$perf->{WriteReqs}); }
      if ( exists $perf->{PercentBusy} )       { $data_out{vmax}{rdfdirector}{$id}{$timestamp}{cpu_busy}   = Xormon::formatNumberByType("int",$perf->{PercentBusy}); }
    }
  }

  #
  # FEPort
  #
  if ( exists $ref->{FEPort} ) {
    foreach my $director ( sort keys %{ $ref->{FEPort} } ) {
      foreach my $label ( sort keys %{ $ref->{FEPort}{$director} } ) {
        my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."feport"."$director-$label") );
        my $perf = $ref->{FEPort}{$director}{$label};
        my $conf = $data{conf}{FEPort}{$director}{$label};

        if ( exists $perf->{MBs} )               { $data_out{vmax}{feport}{$id}{$timestamp}{data_rate}  = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBs},"MB")); }
        if ( exists $perf->{MBRead} )            { $data_out{vmax}{feport}{$id}{$timestamp}{read}       = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBRead},"MB")); }
        if ( exists $perf->{MBWritten} )         { $data_out{vmax}{feport}{$id}{$timestamp}{write}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBWritten},"MB")); }
        if ( exists $perf->{IOs} )               { $data_out{vmax}{feport}{$id}{$timestamp}{io_rate}    = Xormon::formatNumberByType("int",$perf->{IOs}); }
        if ( exists $perf->{Reads} )             { $data_out{vmax}{feport}{$id}{$timestamp}{read_io}    = Xormon::formatNumberByType("int",$perf->{Reads}); }
        if ( exists $perf->{Writes} )            { $data_out{vmax}{feport}{$id}{$timestamp}{write_io}   = Xormon::formatNumberByType("int",$perf->{Writes}); }
        if ( exists $perf->{ResponseTime} )      { $data_out{vmax}{feport}{$id}{$timestamp}{resp_t}     = Xormon::formatNumberByType("float4",$perf->{ResponseTime}); }
        if ( exists $perf->{ReadResponseTime} )  { $data_out{vmax}{feport}{$id}{$timestamp}{resp_t_r}   = Xormon::formatNumberByType("float4",$perf->{ReadResponseTime}); }
        if ( exists $perf->{WriteResponseTime} ) { $data_out{vmax}{feport}{$id}{$timestamp}{resp_t_w}   = Xormon::formatNumberByType("float4",$perf->{WriteResponseTime}); }
        if ( exists $perf->{PercentBusy} )       { $data_out{vmax}{feport}{$id}{$timestamp}{cpu_busy}   = Xormon::formatNumberByType("int",$perf->{PercentBusy}); }
      }
    }
  }

  #
  # BEPort
  #
  if ( exists $ref->{BEPort} ) {
    foreach my $director ( sort keys %{ $ref->{BEPort} } ) {
      foreach my $label ( sort keys %{ $ref->{BEPort}{$director} } ) {
        my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."beport"."$director-$label") );
        my $perf = $ref->{BEPort}{$director}{$label};
        my $conf = $data{conf}{BEPort}{$director}{$label};

        if ( exists $perf->{MBs} )               { $data_out{vmax}{beport}{$id}{$timestamp}{data_rate}  = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBs},"MB")); }
        if ( exists $perf->{MBRead} )            { $data_out{vmax}{beport}{$id}{$timestamp}{read}       = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBRead},"MB")); }
        if ( exists $perf->{MBWritten} )         { $data_out{vmax}{beport}{$id}{$timestamp}{write}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBWritten},"MB")); }
        if ( exists $perf->{IOs} )               { $data_out{vmax}{beport}{$id}{$timestamp}{io_rate}    = Xormon::formatNumberByType("int",$perf->{IOs}); }
        if ( exists $perf->{Reads} )             { $data_out{vmax}{beport}{$id}{$timestamp}{read_io}    = Xormon::formatNumberByType("int",$perf->{Reads}); }
        if ( exists $perf->{Writes} )            { $data_out{vmax}{beport}{$id}{$timestamp}{write_io}   = Xormon::formatNumberByType("int",$perf->{Writes}); }
        if ( exists $perf->{PercentBusy} )       { $data_out{vmax}{beport}{$id}{$timestamp}{cpu_busy}   = Xormon::formatNumberByType("int",$perf->{PercentBusy}); }
      }
    }
  }

  #
  # RDFPort
  #
  if ( exists $ref->{RDFPort} ) {
    foreach my $director ( sort keys %{ $ref->{RDFPort} } ) {
      foreach my $label ( sort keys %{ $ref->{RDFPort}{$director} } ) {
        my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."rdfport"."$director-$label") );
        my $perf = $ref->{RDFPort}{$director}{$label};
        my $conf = $data{conf}{RDFPort}{$director}{$label};

        if ( exists $perf->{MBs} )               { $data_out{vmax}{rdfport}{$id}{$timestamp}{data_rate}  = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBs},"MB")); }
        if ( exists $perf->{MBRead} )            { $data_out{vmax}{rdfport}{$id}{$timestamp}{read}       = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBRead},"MB")); }
        if ( exists $perf->{MBWritten} )         { $data_out{vmax}{rdfport}{$id}{$timestamp}{write}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBWritten},"MB")); }
        if ( exists $perf->{IOs} )               { $data_out{vmax}{rdfport}{$id}{$timestamp}{io_rate}    = Xormon::formatNumberByType("int",$perf->{IOs}); }
        if ( exists $perf->{Reads} )             { $data_out{vmax}{rdfport}{$id}{$timestamp}{read_io}    = Xormon::formatNumberByType("int",$perf->{Reads}); }
        if ( exists $perf->{Writes} )            { $data_out{vmax}{rdfport}{$id}{$timestamp}{write_io}   = Xormon::formatNumberByType("int",$perf->{Writes}); }
        if ( exists $perf->{PercentBusy} )       { $data_out{vmax}{rdfport}{$id}{$timestamp}{cpu_busy}   = Xormon::formatNumberByType("int",$perf->{PercentBusy}); }
      }
    }
  }

  #
  # DiskGroup
  #
  if ( exists $ref->{DiskGroup} ) {
    foreach my $label ( sort keys %{ $ref->{DiskGroup} } ) {
      my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."drive".$label) );
      my $perf = $ref->{DiskGroup}{$label};
      my $conf = $data{conf}{DiskGroup}{$label};

      if ( exists $perf->{MBs} )               { $data_out{vmax}{drive}{$id}{$timestamp}{data_rate}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBs},"MB")); }
      if ( exists $perf->{MBReads} )           { $data_out{vmax}{drive}{$id}{$timestamp}{read}           = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBReads},"MB")); }
      if ( exists $perf->{MBWritten} )         { $data_out{vmax}{drive}{$id}{$timestamp}{write}          = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBWritten},"MB")); }
      if ( exists $perf->{IOs} )               { $data_out{vmax}{drive}{$id}{$timestamp}{io_rate}        = Xormon::formatNumberByType("int",$perf->{IOs}); }
      if ( exists $perf->{DiskReads} )         { $data_out{vmax}{drive}{$id}{$timestamp}{read_io}        = Xormon::formatNumberByType("int",$perf->{DiskReads}); }
      if ( exists $perf->{DiskWrites} )        { $data_out{vmax}{drive}{$id}{$timestamp}{write_io}       = Xormon::formatNumberByType("int",$perf->{DiskWrites}); }
      if ( exists $perf->{ResponseTime} )      { $data_out{vmax}{drive}{$id}{$timestamp}{resp_t}         = Xormon::formatNumberByType("float4",$perf->{ResponseTime}); }
      if ( exists $perf->{ReadResponseTime} )  { $data_out{vmax}{drive}{$id}{$timestamp}{resp_t_r}       = Xormon::formatNumberByType("float4",$perf->{ReadResponseTime}); }
      if ( exists $perf->{WriteResponseTime} ) { $data_out{vmax}{drive}{$id}{$timestamp}{resp_t_w}       = Xormon::formatNumberByType("float4",$perf->{WriteResponseTime}); }
      if ( exists $perf->{PercentBusy} )       { $data_out{vmax}{drive}{$id}{$timestamp}{cpu_busy}       = Xormon::formatNumberByType("int",$perf->{PercentBusy}); }
      if ( exists $perf->{TotalCapacity} )     { $data_out{vmax}{drive}{$id}{$timestamp}{capacity_total} = Xormon::returnBytes($perf->{TotalCapacity},"GB"); }
      if ( exists $perf->{UsedCapacity} )      { $data_out{vmax}{drive}{$id}{$timestamp}{capacity_used}  = Xormon::returnBytes($perf->{UsedCapacity},"GB"); }
      my $data_act = $data_out{vmax}{drive}{$id}{$timestamp};
      # capacity_free
      if ( exists $data_act->{capacity_total} && Xormon::isdigit($data_act->{capacity_total}) && exists $data_act->{capacity_used} && Xormon::isdigit($data_act->{capacity_used}) ) {
        $data_out{vmax}{drive}{$id}{$timestamp}{capacity_free}        = $data_act->{capacity_total} - $data_act->{capacity_used};
        $data_out{vmax}{tier}{$id}{$timestamp}{capacity_used_percent} = $data_act->{capacity_used} / ( $data_act->{capacity_total} / 100 );
      }
    }
  }

  #
  # SLO (aggregated from assigned StorageGroups)
  #
  if ( exists $data{conf}{'SLO'} ) {
    foreach my $label ( sort keys %{ $data{conf}{'SLO'} } ) {
      if ( exists $data{conf}{'SLO'}{$label}{storagegroups} ) {
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."slo".$label) );

        my $values2avg;
        my $values2weighted_avg;
        foreach my $sg_label ( sort keys %{ $data{conf}{'SLO'}{$label}{storagegroups} } ) {
          my $sg_id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."volume".$sg_label) );
          if ( exists $data_out{vmax}{volume}{$sg_id}{$timestamp} ) {
            my $sg_perf = $data_out{vmax}{volume}{$sg_id}{$timestamp};

            if ( exists $sg_perf->{io_rate} && Xormon::isdigit($sg_perf->{io_rate}) ) {
              if ( exists $data_out{vmax}{slo}{$id}{$timestamp}{io_rate} ) {
                $data_out{vmax}{slo}{$id}{$timestamp}{io_rate} += $sg_perf->{io_rate};
              }
              else {
                $data_out{vmax}{slo}{$id}{$timestamp}{io_rate} = $sg_perf->{io_rate};
              }
            }
            if ( exists $sg_perf->{read_io} && Xormon::isdigit($sg_perf->{read_io}) ) {
              if ( exists $data_out{vmax}{slo}{$id}{$timestamp}{read_io} ) {
                $data_out{vmax}{slo}{$id}{$timestamp}{read_io} += $sg_perf->{read_io};
              }
              else {
                $data_out{vmax}{slo}{$id}{$timestamp}{read_io} = $sg_perf->{read_io};
              }
            }
            if ( exists $sg_perf->{write_io} && Xormon::isdigit($sg_perf->{write_io}) ) {
              if ( exists $data_out{vmax}{slo}{$id}{$timestamp}{write_io} ) {
                $data_out{vmax}{slo}{$id}{$timestamp}{write_io} += $sg_perf->{write_io};
              }
              else {
                $data_out{vmax}{slo}{$id}{$timestamp}{write_io} = $sg_perf->{write_io};
              }
            }
            if ( exists $sg_perf->{data_rate} && Xormon::isdigit($sg_perf->{data_rate}) ) {
              if ( exists $data_out{vmax}{slo}{$id}{$timestamp}{data_rate} ) {
                $data_out{vmax}{slo}{$id}{$timestamp}{data_rate} += $sg_perf->{data_rate};
              }
              else {
                $data_out{vmax}{slo}{$id}{$timestamp}{data_rate} = $sg_perf->{data_rate};
              }
            }
            if ( exists $sg_perf->{read} && Xormon::isdigit($sg_perf->{read}) ) {
              if ( exists $data_out{vmax}{slo}{$id}{$timestamp}{read} ) {
                $data_out{vmax}{slo}{$id}{$timestamp}{read} += $sg_perf->{read};
              }
              else {
                $data_out{vmax}{slo}{$id}{$timestamp}{read} = $sg_perf->{read};
              }
            }
            if ( exists $sg_perf->{write} && Xormon::isdigit($sg_perf->{write}) ) {
              if ( exists $data_out{vmax}{slo}{$id}{$timestamp}{write} ) {
                $data_out{vmax}{slo}{$id}{$timestamp}{write} += $sg_perf->{write};
              }
              else {
                $data_out{vmax}{slo}{$id}{$timestamp}{write} = $sg_perf->{write};
              }
            }
            if ( exists $sg_perf->{capacity_total} && Xormon::isdigit($sg_perf->{capacity_total}) ) {
              if ( exists $data_out{vmax}{slo}{$id}{$timestamp}{capacity_total} ) {
                $data_out{vmax}{slo}{$id}{$timestamp}{capacity_total} += $sg_perf->{capacity_total};
              }
              else {
                $data_out{vmax}{slo}{$id}{$timestamp}{capacity_total} = $sg_perf->{capacity_total};
              }
            }
            # avg
            if ( exists $sg_perf->{cache_hit} && Xormon::isdigit($sg_perf->{cache_hit}) ) {
              push(@{ $values2avg->{cache_hit} }, $sg_perf->{cache_hit});
            }
            if ( exists $sg_perf->{r_cache_hit} && Xormon::isdigit($sg_perf->{r_cache_hit}) ) {
              push(@{ $values2avg->{r_cache_hit} }, $sg_perf->{r_cache_hit});
            }
            if ( exists $sg_perf->{w_cache_hit} && Xormon::isdigit($sg_perf->{w_cache_hit}) ) {
              push(@{ $values2avg->{w_cache_hit} }, $sg_perf->{w_cache_hit});
            }
            # weighted_avg
            if ( exists $sg_perf->{resp_t} && Xormon::isdigit($sg_perf->{resp_t}) && exists $sg_perf->{io_rate} && Xormon::isdigit($sg_perf->{io_rate}) && $sg_perf->{io_rate} != 0 ) {
              push(@{ $values2weighted_avg->{resp_t} }, { count => $sg_perf->{io_rate}, value => $sg_perf->{resp_t} } );
            }
            if ( exists $sg_perf->{resp_t_r} && Xormon::isdigit($sg_perf->{resp_t_r}) && exists $sg_perf->{read_io} && Xormon::isdigit($sg_perf->{read_io}) && $sg_perf->{read_io} != 0 ) {
              push(@{ $values2weighted_avg->{resp_t_r} }, { count => $sg_perf->{read_io}, value => $sg_perf->{resp_t_r} } );
            }
            if ( exists $sg_perf->{resp_t_w} && Xormon::isdigit($sg_perf->{resp_t_w}) && exists $sg_perf->{write_io} && Xormon::isdigit($sg_perf->{write_io}) && $sg_perf->{write_io} != 0 ) {
              push(@{ $values2weighted_avg->{resp_t_w} }, { count => $sg_perf->{write_io}, value => $sg_perf->{resp_t_w} } );
            }
          }
        }
        # avg
        foreach my $metric ( sort keys %{ $values2avg } ) {
          my $value = Xormon::calculate_avg($values2avg->{ $metric });
          if ( defined $value ) {
            $data_out{vmax}{slo}{$id}{$timestamp}{$metric} = $value;
          }
        }
        # weighted_avg
        foreach my $metric ( sort keys %{ $values2weighted_avg } ) {
          my $value = Xormon::calculate_weighted_avg($values2weighted_avg->{ $metric });
          if ( defined $value ) {
            $data_out{vmax}{slo}{$id}{$timestamp}{$metric} = $value;
          }
        }
      }
    }
  }

  #
  # Host
  #
  if ( exists $ref->{Host} ) {
    foreach my $label ( sort keys %{ $ref->{Host} } ) {
      my $id   = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."host".$label) );
      my $perf = $ref->{Host}{$label};
      my $conf = $data{conf}{Host}{$label};

      if ( exists $perf->{MBs} )               { $data_out{vmax}{host}{$id}{$timestamp}{data_rate} = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{MBs},"MB")); }
      if ( exists $perf->{HostMBReads} )       { $data_out{vmax}{host}{$id}{$timestamp}{read}      = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBReads},"MB")); }
      if ( exists $perf->{HostMBWrites} )      { $data_out{vmax}{host}{$id}{$timestamp}{write}     = Xormon::formatNumberByType("int",Xormon::returnBytes($perf->{HostMBWrites},"MB")); }
      if ( exists $perf->{HostIOs} )           { $data_out{vmax}{host}{$id}{$timestamp}{io_rate}   = Xormon::formatNumberByType("int",$perf->{HostIOs}); }
      if ( exists $perf->{Reads} )             { $data_out{vmax}{host}{$id}{$timestamp}{read_io}   = Xormon::formatNumberByType("int",$perf->{Reads}); }
      if ( exists $perf->{Writes} )            { $data_out{vmax}{host}{$id}{$timestamp}{write_io}  = Xormon::formatNumberByType("int",$perf->{Writes}); }
      if ( exists $perf->{ResponseTime} )      { $data_out{vmax}{host}{$id}{$timestamp}{resp_t}    = Xormon::formatNumberByType("float4",$perf->{ResponseTime}); }
      if ( exists $perf->{ReadResponseTime} )  { $data_out{vmax}{host}{$id}{$timestamp}{resp_t_r}  = Xormon::formatNumberByType("float4",$perf->{ReadResponseTime}); }
      if ( exists $perf->{WriteResponseTime} ) { $data_out{vmax}{host}{$id}{$timestamp}{resp_t_w}  = Xormon::formatNumberByType("float4",$perf->{WriteResponseTime}); }
    }
  }



  return \%data_out;
}

sub get_perf {
  if ( $debug  ) {
    message("get_perf sunix_ms  : $sunix_ms : ".localtime(int($sunix_ms/1000)));
    message("get_perf eunix_ms  : $eunix_ms : ".localtime(int($eunix_ms/1000)));
  }

  perf_array();
  perf_storagegroup();
  perf_srp();
  perf_thinpool(); # only when SRP doesn't exist
  perf_thintier(); # not supported in Unisphere V10
  perf_fedirector();
  perf_bedirector();
  perf_rdfdirector();
  perf_diskgroup();
  #perf_cachepartition(); # not supported yet
  perf_beport();
  perf_feport();
  perf_rdfport();
  perf_host();
  perf_slo(); # stats for SLO are aggregated from the assigned StorageGroups

  return 1;
}

sub set_timerange {
  my $type   = shift;
  my $sunix  = shift; # in ms
  my $eunix  = shift; # in ms
  my $params = shift;

  if ( $debug == 2 ) {
    message("set_timerange      : $type: sunix before=$sunix : ".localtime(int($sunix/1000)));
    message("set_timerange      : $type: eunix before=$eunix : ".localtime(int($eunix/1000)));
  }

  if ( $eunix < $params->{'firstAvailableDate'} || $eunix > $params->{'lastAvailableDate'} ) {
    $eunix = $params->{'lastAvailableDate'} - 300000; # 5 minutes back, just to be sure there will be data for all items
    $logger->error( "set_timerange: $type: adjust eunix to $eunix: ".localtime(int($eunix/1000)) . __FILE__ . ":" . __LINE__ );
  }
  if ( $sunix < $params->{'firstAvailableDate'} ) {
    $sunix = $params->{'firstAvailableDate'};
    $logger->error( "set_timerange: $type: adjust sunix to $sunix: ".localtime(int($sunix/1000)) . __FILE__ . ":" . __LINE__ );
  }
  if ( $sunix > $params->{'lastAvailableDate'} ) {
    $sunix = $params->{'lastAvailableDate'} - 900000;
    $logger->error( "set_timerange: $type: adjust sunix to $sunix: ".localtime(int($sunix/1000)) . __FILE__ . ":" . __LINE__ );
  }
  if ( ( $eunix - $sunix ) > ( 3600 * 24 * 1000 ) ) {
    $sunix = $eunix - (3600 * 24 * 1000); # max a day back
    $logger->error( "set_timerange: $type: adjust sunix to $sunix: ".localtime(int($sunix/1000)) . __FILE__ . ":" . __LINE__ );
  }

  if ( $debug == 2 ) {
    message("set_timerange      : $type: sunix after=$sunix : ".localtime(int($sunix/1000)));
    message("set_timerange      : $type: eunix after=$eunix : ".localtime(int($eunix/1000)));
  }

  return ($sunix,$eunix);
}

sub perf_slo {
  #
  # SLO cfg (find SLO and assigned StorageGroups)
  #
  my $result = new_get_request("$url_paths->{sloprovisioning}/symmetrix/$cfg->{'symmetrix'}/slo");

  if ( defined $result && exists $result->{sloId} && ref($result->{sloId}) eq "ARRAY" ) {
    foreach my $slo_id ( @{ $result->{sloId} } ) {
      my $conf = new_get_request("$url_paths->{sloprovisioning}/symmetrix/$cfg->{'symmetrix'}/slo/$slo_id");

      if ( defined $conf ) {
        if ( exists $conf->{slo} && ref($conf->{slo}) eq "ARRAY" ) {
          foreach ( @{ $conf->{slo} } ) {
            if ( exists $_->{'storageGroupId'} && ref($_->{'storageGroupId'}) eq "ARRAY" ) {
              foreach my $sg_id ( @{ $_->{'storageGroupId'} } ) {

                $data{conf}{'SLO'}{$slo_id}{'storagegroups'}{$sg_id} = $sg_id;
              }
            }
          }
        }
        elsif ( exists $conf->{storageGroupId} && ref($conf->{storageGroupId}) eq "ARRAY" ) {
          foreach my $sg_id ( @{ $conf->{'storageGroupId'} } ) {

            $data{conf}{'SLO'}{$slo_id}{'storagegroups'}{$sg_id} = $sg_id;
          }
        }
      }
    }
  }

  # SLO perf (aggregated from assigned StorageGroups, this will be in the "prepare_perf" function)

  return 1;
}

sub perf_host {
  #
  # set metrics
  #
  my @metrics = ( "HostIOs", "HostMBReads", "HostMBWrites", "Reads", "ResponseTime", "ReadResponseTime", "WriteResponseTime", "Writes", "MBs" );

  #
  # collect data
  #
  my $params;
  $params->{symmetrixId} = $cfg->{'symmetrix'};

  # this doesn't work
  #my $result;
  #if ( $unisphere_version =~ m/^V9|^V10/ ) {
  #  $result = new_post_request("performance/Host/keys",$params,1);
  #}
  #else {
  #  $result = new_post_request("81/performance/Host/keys",$params,1);
  #}

  # must be used like here
  my $result = new_get_request("$url_paths->{sloprovisioning}/symmetrix/$cfg->{'symmetrix'}/host");

  unless ( defined $result ) { $result = new_get_request("$url_paths->{provisioning}/symmetrix/$cfg->{'symmetrix'}/host"); }

  if ( defined $result && exists $result->{hostId} && ref($result->{hostId}) eq "ARRAY" ) {
    foreach my $host_id ( @{ $result->{hostId} } ) {
      #unless ( exists $_->{hostId} )                                                   { next; }
      #unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      $data{conf}{Host}{ $host_id }{id} = $host_id;

      #($sunix_ms,$eunix_ms) = set_timerange("perf_host",$sunix_ms,$eunix_ms,$_);

      # use already discovered sunix and eunix (only in this case)
      $params->{startDate} = $sunix_ms;
      $params->{endDate}   = $eunix_ms;
      $params->{metrics}   = [@metrics];
      $params->{hostId}    = $host_id;

      if ( $debug == 2 ) {
        message("perf_host          : $params->{hostId}");
        message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
        message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
        message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
        message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
        message("metrics            : ".join(", ",@metrics));
      }

      my $stats;
      if ( $unisphere_version =~ m/^V9|^V10/ ) {
        $stats = new_post_request("performance/Host/metrics",$params,1);
      }
      else {
        $stats = new_post_request("81/performance/Host/metrics",$params,1);
      }

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            $data{perf}{ $values->{timestamp} }{Host}{ $params->{hostId} } = $values;
          }
        }
      }
      else {
        $logger->error( "Host $params->{hostId} stats not found!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }
  else {
    $logger->error( "Host keys not found!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub perf_rdfport {
  #
  # set metrics
  #
  my @metrics = ( "Reads", "Writes", "IOs", "MBRead", "MBWritten", "MBs", "PercentBusy" );

  #
  # collect data
  #
  if ( exists $data{conf}{'RDFDirector'} ) {
    foreach my $director_id ( keys( %{ $data{conf}{'RDFDirector'} } ) ) {
      my $params;
      $params->{symmetrixId} = $cfg->{'symmetrix'};
      $params->{directorId}  = $director_id;
      my $result             = new_post_request("performance/RDFPort/keys",$params,1);

      if ( defined $result && exists $result->{rdfPortInfo} && ref($result->{rdfPortInfo}) eq "ARRAY" ) {
        foreach ( @{ $result->{rdfPortInfo} } ) {
          unless ( exists $_->{portId} )                                                   { next; }
          unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

          $data{conf}{RDFPort}{$director_id}{ $_->{portId} }{id} = $_->{portId};

          ($sunix_ms,$eunix_ms) = set_timerange("perf_rdfport",$sunix_ms,$eunix_ms,$_);

          $params->{startDate} = $sunix_ms;
          $params->{endDate}   = $eunix_ms;
          $params->{metrics}   = [@metrics];
          $params->{portId}    = $_->{portId};

          if ( $debug == 2 ) {
            message("perf_rdfport       : $params->{portId}");
            message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
            message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
            message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
            message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
            message("metrics            : ".join(", ",@metrics));
          }

          my $stats = new_post_request("performance/RDFPort/metrics",$params,1);

          #
          # parse stats
          #
          if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
            foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
              if ( exists $values->{timestamp} ) {
                $data{perf}{ $values->{timestamp} }{RDFPort}{$director_id}{ $params->{portId} } = $values;
              }
            }
          }
          else {
            $logger->error( "RDFPort $params->{portId} stats not found!" . __FILE__ . ":" . __LINE__ );
          }
        }
      }
      else {
        $logger->error( "RDFPort keys for Director $director_id not found!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }

  return 1;
}

sub perf_feport {
  #
  # set metrics
  #
  my @metrics = ( "Reads", "Writes", "IOs", "MBRead", "MBWritten", "MBs", "PercentBusy", "ResponseTime", "ReadResponseTime", "WriteResponseTime" );

  #
  # collect data
  #
  if ( exists $data{conf}{'FEDirector'} ) {
    foreach my $director_id ( keys( %{ $data{conf}{'FEDirector'} } ) ) {
      my $params;
      $params->{symmetrixId} = $cfg->{'symmetrix'};
      $params->{directorId}  = $director_id;
      my $result             = new_post_request("performance/FEPort/keys",$params,1);

      if ( defined $result && exists $result->{fePortInfo} && ref($result->{fePortInfo}) eq "ARRAY" ) {
        foreach ( @{ $result->{fePortInfo} } ) {
          unless ( exists $_->{portId} )                                                   { next; }
          unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

          $data{conf}{FEPort}{$director_id}{ $_->{portId} }{id} = $_->{portId};

          ($sunix_ms,$eunix_ms) = set_timerange("perf_feport",$sunix_ms,$eunix_ms,$_);

          $params->{startDate} = $sunix_ms;
          $params->{endDate}   = $eunix_ms;
          $params->{metrics}   = [@metrics];
          $params->{portId}    = $_->{portId};

          if ( $debug == 2 ) {
            message("perf_feport        : $params->{portId}");
            message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
            message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
            message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
            message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
            message("metrics            : ".join(", ",@metrics));
          }

          my $stats = new_post_request("performance/FEPort/metrics",$params,1);

          #
          # parse stats
          #
          if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
            foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
              if ( exists $values->{timestamp} ) {
                $data{perf}{ $values->{timestamp} }{FEPort}{$director_id}{ $params->{portId} } = $values;
              }
            }
          }
          else {
            $logger->error( "FEPort $params->{portId} stats not found!" . __FILE__ . ":" . __LINE__ );
          }
        }
      }
      else {
        $logger->error( "FEPort keys for Director $director_id not found!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }

  return 1;
}

sub perf_beport {
  #
  # set metrics
  #
  my @metrics = ( "Reads", "Writes", "IOs", "MBRead", "MBWritten", "MBs", "PercentBusy" );

  #
  # collect data
  #
  if ( exists $data{conf}{'BEDirector'} ) {
    foreach my $director_id ( keys( %{ $data{conf}{'BEDirector'} } ) ) {
      my $params;
      $params->{symmetrixId} = $cfg->{'symmetrix'};
      $params->{directorId}  = $director_id;
      my $result             = new_post_request("performance/BEPort/keys",$params,1);

      if ( defined $result && exists $result->{bePortInfo} && ref($result->{bePortInfo}) eq "ARRAY" ) {
        foreach ( @{ $result->{bePortInfo} } ) {
          unless ( exists $_->{portId} )                                                   { next; }
          unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

          $data{conf}{BEPort}{$director_id}{ $_->{portId} }{id} = $_->{portId};

          ($sunix_ms,$eunix_ms) = set_timerange("perf_beport",$sunix_ms,$eunix_ms,$_);

          $params->{startDate} = $sunix_ms;
          $params->{endDate}   = $eunix_ms;
          $params->{metrics}   = [@metrics];
          $params->{portId}    = $_->{portId};

          if ( $debug == 2 ) {
            message("perf_beport        : $params->{portId}");
            message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
            message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
            message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
            message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
            message("metrics            : ".join(", ",@metrics));
          }

          my $stats = new_post_request("performance/BEPort/metrics",$params,1);

          #
          # parse stats
          #
          if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
            foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
              if ( exists $values->{timestamp} ) {
                $data{perf}{ $values->{timestamp} }{BEPort}{$director_id}{ $params->{portId} } = $values;
              }
            }
          }
          else {
            $logger->error( "BEPort $params->{portId} stats not found!" . __FILE__ . ":" . __LINE__ );
          }
        }
      }
      else {
        $logger->error( "BEPort keys for Director $director_id not found!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }

  return 1;
}

sub perf_cachepartition {
  #
  # set metrics
  #
  my @metrics = ( "HostIOs", "MB_RATE", "PercentWPUtilization", "PercentHit", "PercentCacheUsed", "WPCount", "SystemMaxWPLimit" );

  #
  # collect data
  #
  my $params;
  $params->{symmetrixId} = $cfg->{'symmetrix'};
  my $result             = new_post_request("performance/CachePartition/keys",$params,1);

  if ( defined $result && exists $result->{cachePartitionInfo} && ref($result->{cachePartitionInfo}) eq "ARRAY" ) {
    foreach ( @{ $result->{cachePartitionInfo} } ) {
      unless ( exists $_->{cachePartitionId} )                                         { next; }
      unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      $data{conf}{CachePartition}{ $_->{cachePartitionId} }{id} = $_->{cachePartitionId};

      ($sunix_ms,$eunix_ms) = set_timerange("perf_cachepartition",$sunix_ms,$eunix_ms,$_);

      $params->{startDate}   = $sunix_ms;
      $params->{endDate}     = $eunix_ms;
      $params->{metrics}     = [@metrics];
      $params->{cachePartitionId} = $_->{cachePartitionId};

      if ( $debug == 2 ) {
        message("perf_cachepartition: $params->{cachePartitionId}");
        message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
        message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
        message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
        message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
        message("metrics            : ".join(", ",@metrics));
      }

      my $stats = new_post_request("performance/CachePartition/metrics",$params,1);

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            $data{perf}{ $values->{timestamp} }{CachePartition}{ $params->{cachePartitionId} } = $values;
          }
        }
      }
      else {
        $logger->error( "CachePartition $params->{cachePartitionId} stats not found!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }
  else {
    $logger->error( "CachePartition keys not found!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub perf_diskgroup {
  #
  # set metrics
  #
  my @metrics = ( "IOs", "MBs", "MBReads", "MBWritten", "PercentBusy", "DiskReads", "ReadResponseTime", "ResponseTime", "WriteResponseTime", "UsedCapacity", "DiskWrites", "TotalCapacity" );

  #
  # collect data
  #
  my $params;
  $params->{symmetrixId} = $cfg->{'symmetrix'};
  my $result             = new_post_request("performance/DiskGroup/keys",$params,1);

  if ( defined $result && exists $result->{diskGroupInfo} && ref($result->{diskGroupInfo}) eq "ARRAY" ) {
    foreach ( @{ $result->{diskGroupInfo} } ) {
      unless ( exists $_->{diskGroupId} )                                              { next; }
      unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      $data{conf}{DiskGroup}{ $_->{diskGroupId} }{id} = $_->{diskGroupId};

      ($sunix_ms,$eunix_ms) = set_timerange("perf_diskgroup",$sunix_ms,$eunix_ms,$_);

      $params->{startDate}   = $sunix_ms;
      $params->{endDate}     = $eunix_ms;
      $params->{metrics}     = [@metrics];
      $params->{diskGroupId} = $_->{diskGroupId};

      if ( $debug == 2 ) {
        message("perf_diskgroup     : $params->{diskGroupId}");
        message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
        message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
        message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
        message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
        message("metrics            : ".join(", ",@metrics));
      }

      my $stats = new_post_request("performance/DiskGroup/metrics",$params,1);

      unless ( defined $stats ) {
        # exclude io metrics (DiskReads and DiskWrites)
        @metrics           = ( "IOs", "MBs", "MBReads", "MBWritten", "PercentBusy", "ReadResponseTime", "ResponseTime", "WriteResponseTime", "UsedCapacity", "TotalCapacity" );
        $params->{metrics} = [@metrics];
        $stats             = new_post_request("performance/DiskGroup/metrics",$params,1);
      }

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            $data{perf}{ $values->{timestamp} }{DiskGroup}{ $params->{diskGroupId} } = $values;
            $data{conf}{DiskGroup}{ $params->{diskGroupId} }{last_stats}             = $values;
          }
        }
      }
      else {
        $logger->error( "DiskGroup $params->{diskGroupId} stats not found!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }
  else {
    $logger->error( "DiskGroup keys not found!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub perf_rdfdirector {
  #
  # set metrics
  #
  my @metrics = ( "IOs", "MBSentAndReceived", "MBRead", "MBWritten", "PercentBusy", "ReadReqs", "WriteReqs" );

  #
  # collect data
  #
  my $params;
  $params->{symmetrixId} = $cfg->{'symmetrix'};
  my $result             = new_post_request("performance/RDFDirector/keys",$params,1);

  if ( defined $result && exists $result->{rdfDirectorInfo} && ref($result->{rdfDirectorInfo}) eq "ARRAY" ) {
    foreach ( @{ $result->{rdfDirectorInfo} } ) {
      unless ( exists $_->{directorId} )                                               { next; }
      unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      $data{conf}{RDFDirector}{ $_->{directorId} }{id} = $_->{directorId};

      ($sunix_ms,$eunix_ms) = set_timerange("perf_rdfdirector",$sunix_ms,$eunix_ms,$_);

      $params->{startDate}  = $sunix_ms;
      $params->{endDate}    = $eunix_ms;
      $params->{metrics}    = [@metrics];
      $params->{directorId} = $_->{directorId};

      if ( $debug == 2 ) {
        message("perf_rdfdirector   : $params->{directorId}");
        message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
        message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
        message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
        message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
        message("metrics            : ".join(", ",@metrics));
      }

      my $stats = new_post_request("performance/RDFDirector/metrics",$params,1);

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            $data{perf}{ $values->{timestamp} }{RDFDirector}{ $params->{directorId} } = $values;
          }
        }
      }
      else {
        $logger->error( "RDFDirector $params->{directorId} stats not found!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }
  else {
    $logger->error( "RDFDirector keys not found!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub perf_bedirector {
  #
  # set metrics
  #
  my @metrics = ( "IOs", "MBs", "MBRead", "MBWritten", "PercentBusy", "ReadReqs", "WriteReqs" );

  #
  # collect data
  #
  my $params;
  $params->{symmetrixId} = $cfg->{'symmetrix'};
  my $result             = new_post_request("performance/BEDirector/keys",$params,1);

  if ( defined $result && exists $result->{beDirectorInfo} && ref($result->{beDirectorInfo}) eq "ARRAY" ) {
    foreach ( @{ $result->{beDirectorInfo} } ) {
      unless ( exists $_->{directorId} )                                               { next; }
      unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      $data{conf}{BEDirector}{ $_->{directorId} }{id} = $_->{directorId};

      ($sunix_ms,$eunix_ms) = set_timerange("perf_bedirector",$sunix_ms,$eunix_ms,$_);

      $params->{startDate}  = $sunix_ms;
      $params->{endDate}    = $eunix_ms;
      $params->{metrics}    = [@metrics];
      $params->{directorId} = $_->{directorId};

      if ( $debug == 2 ) {
        message("perf_bedirector    : $params->{directorId}");
        message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
        message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
        message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
        message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
        message("metrics            : ".join(", ",@metrics));
      }

      my $stats = new_post_request("performance/BEDirector/metrics",$params,1);

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            $data{perf}{ $values->{timestamp} }{BEDirector}{ $params->{directorId} } = $values;
          }
        }
      }
      else {
        $logger->error( "BEDirector $params->{directorId} stats not found!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }
  else {
    $logger->error( "BEDirector keys not found!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub perf_fedirector {
  #
  # set metrics
  #
  my @metrics = ( "HostMBs", "Reqs", "ReadReqs", "WriteReqs", "ReadResponseTime", "WriteResponseTime", "PercentHitReqs", "PercentReadReqHit", "PercentWriteReqHit", "PercentBusy" );

  #
  # collect data
  #
  my $params;
  $params->{symmetrixId} = $cfg->{'symmetrix'};
  my $result             = new_post_request("performance/FEDirector/keys",$params,1);

  if ( defined $result && exists $result->{feDirectorInfo} && ref($result->{feDirectorInfo}) eq "ARRAY" ) {
    foreach ( @{ $result->{feDirectorInfo} } ) {
      unless ( exists $_->{directorId} )                                               { next; }
      unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      $data{conf}{FEDirector}{ $_->{directorId} }{id} = $_->{directorId};

      ($sunix_ms,$eunix_ms) = set_timerange("perf_fedirector",$sunix_ms,$eunix_ms,$_);

      $params->{startDate}  = $sunix_ms;
      $params->{endDate}    = $eunix_ms;
      $params->{metrics}    = [@metrics];
      $params->{directorId} = $_->{directorId};

      if ( $debug == 2 ) {
        message("perf_fedirector    : $params->{directorId}");
        message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
        message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
        message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
        message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
        message("metrics            : ".join(", ",@metrics));
      }

      my $stats = new_post_request("performance/FEDirector/metrics",$params,1);

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            $data{perf}{ $values->{timestamp} }{FEDirector}{ $params->{directorId} } = $values;
          }
        }
      }
      else {
        $logger->error( "FEDirector $params->{directorId} stats not found!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }
  else {
    $logger->error( "FEDirector keys not found!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub perf_thintier {
  if ( $unisphere_version =~ m/^V10/ ) { return 1; } # Unisphere 10 doesn't support this

  #
  # set metrics
  #
  my @metrics = ( "HostIOs", "HostReads", "HostWrites", "HostMBs", "HostMBReads", "HostMBWritten", "ReadResponseTime", "WriteResponseTime", "PercentReadHit", "PercentWriteHit", "BEMBRead", "BEMBTransferred", "BEMBWritten", "BEReqs", "BEReadReqs", "BEWriteReqs", "TotalPoolCapacity", "UsedPoolCapacity" );

  #
  # collect data
  #
  my $params;
  $params->{symmetrixId} = $cfg->{'symmetrix'};
  my $result             = new_post_request("performance/ThinTier/keys",$params,1);

  if ( defined $result && exists $result->{tierInfo} && ref($result->{tierInfo}) eq "ARRAY" ) {
    foreach ( @{ $result->{tierInfo} } ) {
      unless ( exists $_->{tierId} )                                                   { next; }
      unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      $data{conf}{ThinTier}{ $_->{tierId} }{id} = $_->{tierId};

      ($sunix_ms,$eunix_ms) = set_timerange("perf_thintier",$sunix_ms,$eunix_ms,$_);

      $params->{startDate} = $sunix_ms;
      $params->{endDate}   = $eunix_ms;
      $params->{metrics}   = [@metrics];
      $params->{tierId}    = $_->{tierId};

      if ( $debug == 2 ) {
        message("perf_thintier      : $params->{tierId}");
        message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
        message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
        message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
        message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
        message("metrics            : ".join(", ",@metrics));
      }

      my $stats = new_post_request("performance/ThinTier/metrics",$params,1);

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            $data{perf}{ $values->{timestamp} }{ThinTier}{ $params->{tierId} } = $values;
          }
        }
      }
      else {
        $logger->error( "ThinTier $params->{tierId} stats not found!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }
  else {
    $logger->error( "ThinTier keys not found!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub perf_srp {
  #
  # set metrics
  #
  my @metrics = ( "HostIOs", "HostReads", "HostWrites", "HostMBs", "HostMBReads", "HostMBWritten", "BEReqs", "BEReadReqs", "BEWriteReqs", "BEMBTransferred", "BEMBReads", "BEMBWritten", "ResponseTime", "ReadResponseTime", "WriteResponseTime" );
  if ( $unisphere_version =~ m/^V9|^V10/ ) { # Unisphere version 8.1.2.3 do not support capacity metrics
    @metrics = ( "HostIOs", "HostReads", "HostWrites", "HostMBs", "HostMBReads", "HostMBWritten", "BEReqs", "BEReadReqs", "BEWriteReqs", "BEMBTransferred", "BEMBReads", "BEMBWritten", "ResponseTime", "ReadResponseTime", "WriteResponseTime", "TotalSRPCapacity", "UsedSRPCapacity", "OverallEfficiencyRatio" );
  }

  #
  # collect data
  #
  my $params;
  $params->{symmetrixId} = $cfg->{'symmetrix'};
  my $result             = new_post_request("performance/SRP/keys",$params,1);

  if ( defined $result && exists $result->{srpInfo} && ref($result->{srpInfo}) eq "ARRAY" ) {
    foreach ( @{ $result->{srpInfo} } ) {
      unless ( exists $_->{srpId} )                                                    { next; }
      unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      $data{conf}{SRP}{ $_->{srpId} }{id} = $_->{srpId};

      ($sunix_ms,$eunix_ms) = set_timerange("perf_srp",$sunix_ms,$eunix_ms,$_);

      $params->{startDate} = $sunix_ms;
      $params->{endDate}   = $eunix_ms;
      $params->{metrics}   = [@metrics];
      $params->{srpId}     = $_->{srpId};

      if ( $debug == 2 ) {
        message("perf_srp           : $params->{srpId}");
        message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
        message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
        message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
        message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
        message("metrics            : ".join(", ",@metrics));
      }

      my $stats = new_post_request("performance/SRP/metrics",$params,1);

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            $data{perf}{ $values->{timestamp} }{SRP}{ $params->{srpId} } = $values;
            $data{conf}{SRP}{ $_->{srpId} }{last_stats}                  = $values;
          }
        }
      }
      else {
        $logger->error( "SRP $params->{srpId} stats not found!" . __FILE__ . ":" . __LINE__ );
      }

      #
      # collect conf
      #
      my $conf_result = new_get_request("$url_paths->{sloprovisioning}/symmetrix/$cfg->{'symmetrix'}/srp/$_->{srpId}");

      if ( defined $conf_result ) {
        if ( exists $conf_result->{srp} && ref($conf_result->{srp}) eq "ARRAY" ) {
          foreach my $var ( @{ $conf_result->{srp} } ) {
            if ( exists $var->{'total_usable_cap_gb'}     && Xormon::isdigit( $var->{'total_usable_cap_gb'} ) )     { $data{conf}{'SRP'}{$_->{srpId}}{'total_usable_cap_gb'}     = $var->{'total_usable_cap_gb'}; }
            if ( exists $var->{'total_allocated_cap_gb'}  && Xormon::isdigit( $var->{'total_allocated_cap_gb'} ) )  { $data{conf}{'SRP'}{$_->{srpId}}{'total_allocated_cap_gb'}  = $var->{'total_allocated_cap_gb'}; }
            if ( exists $var->{'total_subscribed_cap_gb'} && Xormon::isdigit( $var->{'total_subscribed_cap_gb'} ) ) { $data{conf}{'SRP'}{$_->{srpId}}{'total_subscribed_cap_gb'} = $var->{'total_subscribed_cap_gb'}; }
          }
        }
        elsif ( exists $conf_result->{srp_capacity} ) { # V9
          my $var = $conf_result->{srp_capacity};
          if ( exists $var->{subscribed_total_tb} && Xormon::isdigit( $var->{subscribed_total_tb} ) ) { $data{conf}{'SRP'}{$_->{srpId}}{'subscribed_total_tb'} = $var->{subscribed_total_tb}; }
          if ( exists $var->{usable_total_tb}     && Xormon::isdigit( $var->{usable_total_tb} ) )     { $data{conf}{'SRP'}{$_->{srpId}}{'usable_total_tb'}     = $var->{usable_total_tb}; }
          if ( exists $var->{usable_used_tb}      && Xormon::isdigit( $var->{usable_used_tb} ) )      { $data{conf}{'SRP'}{$_->{srpId}}{'usable_used_tb'}      = $var->{usable_used_tb}; }
        }
        elsif ( exists $conf_result->{fba_srp_capacity} ) { # V10
          my $var = $conf_result->{fba_srp_capacity};
          if ( exists $var->{effective} ) {
            if ( exists $var->{effective}->{total_tb} ) { $data{conf}{'SRP'}{$_->{srpId}}{'total_tb'} = $var->{effective}->{total_tb}; }
            if ( exists $var->{effective}->{used_tb} )  { $data{conf}{'SRP'}{$_->{srpId}}{'used_tb'}  = $var->{effective}->{used_tb}; }
            if ( exists $var->{effective}->{free_tb} )  { $data{conf}{'SRP'}{$_->{srpId}}{'free_tb'}  = $var->{effective}->{free_tb}; }

            if ( exists $var->{effective}->{physical_capacity} ) {
              if ( exists $var->{effective}->{physical_capacity}->{total_tb} ) { $data{conf}{'SRP'}{$_->{srpId}}{physical_capacity}{'total_tb'} = $var->{effective}->{physical_capacity}->{total_tb}; }
              if ( exists $var->{effective}->{physical_capacity}->{used_tb} )  { $data{conf}{'SRP'}{$_->{srpId}}{physical_capacity}{'used_tb'}  = $var->{effective}->{physical_capacity}->{used_tb}; }
              if ( exists $var->{effective}->{physical_capacity}->{free_tb} )  { $data{conf}{'SRP'}{$_->{srpId}}{physical_capacity}{'free_tb'}  = $var->{effective}->{physical_capacity}->{free_tb}; }
            }
          }
          if ( exists $var->{provisioned} ) {
            if ( exists $var->{provisioned}->{provisioned_tb} )      { $data{conf}{'SRP'}{$_->{srpId}}{'provisioned_tb'}      = $var->{provisioned}->{provisioned_tb}; }
            if ( exists $var->{provisioned}->{provisioned_percent} ) { $data{conf}{'SRP'}{$_->{srpId}}{'provisioned_percent'} = $var->{provisioned}->{provisioned_percent}; }
          }
          if ( exists $var->{data_reduction} ) {
            if ( exists $var->{data_reduction}->{savings_tb} )            { $data{conf}{'SRP'}{$_->{srpId}}{'savings_tb'}            = $var->{data_reduction}->{savings_tb}; }
            if ( exists $var->{data_reduction}->{reducing_data_percent} ) { $data{conf}{'SRP'}{$_->{srpId}}{'reducing_data_percent'} = $var->{data_reduction}->{reducing_data_percent}; }
          }
        }

        # add data_reduction_ratio_to_one and overall_efficiency_ratio_to_one
        if ( exists $conf_result->{srp_efficiency}->{data_reduction_ratio_to_one} )     { $data{conf}{'SRP'}{$_->{srpId}}{data_reduction_ratio_to_one}     = $conf_result->{srp_efficiency}->{data_reduction_ratio_to_one}; }
        if ( exists $conf_result->{srp_efficiency}->{overall_efficiency_ratio_to_one} ) { $data{conf}{'SRP'}{$_->{srpId}}{overall_efficiency_ratio_to_one} = $conf_result->{srp_efficiency}->{overall_efficiency_ratio_to_one}; }
      }
    }
  }
  else {
    $logger->error( "SRP keys not found!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub perf_thinpool {
  if ( exists $data{conf}{SRP} ) { return 1; } # use SRP instead of ThinPools

  #
  # set metrics
  #
  my @metrics = ( "HostIOs", "HostReads", "HostWrites", "HostMBs", "HostMBReads", "HostMBWritten", "ReadResponseTime", "WriteResponseTime", "BEMBTransferred", "BEMBRead", "BEMBWritten", "BEReqs", "BEReadReqs", "BEWriteReqs", "TotalPoolCapacity", "UsedPoolCapacity" );

  #
  # collect stats
  #
  my $params;
  $params->{symmetrixId} = $cfg->{'symmetrix'};
  my $result             = new_post_request("performance/ThinPool/keys",$params,1);

  if ( defined $result && exists $result->{poolInfo} && ref($result->{poolInfo}) eq "ARRAY" ) {
    foreach ( @{ $result->{poolInfo} } ) {
      unless ( exists $_->{poolId} )                                                   { next; }
      unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      $data{conf}{'ThinPool'}{ $_->{poolId} }{id} = $_->{poolId};

      ($sunix_ms,$eunix_ms) = set_timerange("perf_thinpool",$sunix_ms,$eunix_ms,$_);

      $params->{startDate} = $sunix_ms;
      $params->{endDate}   = $eunix_ms;
      $params->{metrics}   = [@metrics];
      $params->{poolId}    = $_->{poolId};

      if ( $debug == 2 ) {
        message("perf_thinpool      : $params->{poolId}");
        message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
        message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
        message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
        message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
        message("metrics            : ".join(", ",@metrics));
      }

      my $stats = new_post_request("performance/ThinPool/metrics",$params,1);

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            $data{perf}{ $values->{timestamp} }{ThinPool}{ $params->{poolId} } = $values;
          }
        }
      }
      else {
        $logger->error( "ThinPool $params->{poolId} stats not found!" . __FILE__ . ":" . __LINE__ );
      }

      if ( $unisphere_version =~ m/^V10/ ) { next; } # Unisphere 10 doesn't support following endpoint

      #
      # collect conf
      #
      my $conf_result = new_get_request("$url_paths->{provisioning}/symmetrix/$cfg->{'symmetrix'}/thinpool/$_->{poolId}");

      if ( defined $conf_result && exists $conf_result->{'thinPool'} ) {
        foreach my $pool_conf ( @{ $conf_result->{'thinPool'} } ) {
          if ( defined $pool_conf->{'percent_subscription'} && $pool_conf->{'percent_subscription'} ne '' ) {
            $data{conf}{'ThinPool'}{$_->{poolId}}{'percent_subscription'} = $pool_conf->{'percent_subscription'};
          }
        }
      }

    }
  }
  else {
    $logger->error( "ThinPool keys not found!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub perf_storagegroup {
  #
  # set metrics
  #
  my @metrics = ( "AllocatedCapacity", "HostIOs", "HostMBs", "HostMBReads", "HostMBWritten", "PercentHit", "PercentReadHit", "PercentWriteHit", "HostReads", "HostWrites", "ResponseTime", "ReadResponseTime", "WriteResponseTime", "RdfResponseTime", "RdfReads", "RdfWrites" );

  #
  # collect stats
  #
  my $params;
  $params->{symmetrixId} = $cfg->{'symmetrix'};
  my $result             = new_post_request("performance/StorageGroup/keys",$params,1);

  if ( defined $result && exists $result->{storageGroupInfo} && ref($result->{storageGroupInfo}) eq "ARRAY" ) {
    foreach ( @{ $result->{storageGroupInfo} } ) {
      unless ( exists $_->{storageGroupId} )                                           { next; }
      unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      $data{conf}{StorageGroup}{ $_->{storageGroupId} }{id} = $_->{storageGroupId};

      ($sunix_ms,$eunix_ms) = set_timerange("perf_storagegroup",$sunix_ms,$eunix_ms,$_);

      $params->{startDate}      = $sunix_ms;
      $params->{endDate}        = $eunix_ms;
      $params->{metrics}        = [@metrics];
      $params->{storageGroupId} = $_->{storageGroupId};

      if ( $debug == 2 ) {
        message("perf_storagegroup  : $params->{storageGroupId}");
        message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
        message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
        message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
        message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
        message("metrics            : ".join(", ",@metrics));
      }

      my $stats = new_post_request("performance/StorageGroup/metrics",$params,1);

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            $data{perf}{ $values->{timestamp} }{StorageGroup}{ $params->{storageGroupId} } = $values;
            $data{conf}{StorageGroup}{ $params->{storageGroupId} }{last_stats}             = $values;
          }
        }
      }
      else {
        $logger->error( "StorageGroup $params->{storageGroupId} stats not found!" . __FILE__ . ":" . __LINE__ );
      }

    }
  }
  else {
    $logger->error( "StorageGroup keys not found!" . __FILE__ . ":" . __LINE__ );
  }


  return 1;
}

sub perf_array {
  #
  # set metrics
  #
  my @metrics = ( "HostMBs", "HostMBReads", "HostMBWritten", "HostIOs", "HostReads", "HostWrites", "ReadResponseTime", "WriteResponseTime" );

  #
  # collect stats
  #
  my $result = new_get_request("performance/Array/keys");

  if ( defined $result && exists $result->{arrayInfo} && ref($result->{arrayInfo}) eq "ARRAY" ) {
    foreach ( @{ $result->{arrayInfo} } ) {
      unless ( exists $_->{symmetrixId} && $_->{symmetrixId} eq $cfg->{'symmetrix'} )  { next; }
      unless ( exists $_->{'firstAvailableDate'} && exists $_->{'lastAvailableDate'} ) { next; }

      $data{conf}{Array}{ $cfg->{symmetrix} }{id} = $cfg->{symmetrix};

      ($sunix_ms,$eunix_ms) = set_timerange("perf_array",$sunix_ms,$eunix_ms,$_);

      my $params;
      #$params->{startDate}   = $eunix_ms - 900000; # it is used like here in stor2rrd, don't know why...
      #$params->{endDate}     = $eunix_ms - 600000; # it is used like here in stor2rrd, don't know why...
      $params->{startDate}   = $sunix_ms;
      $params->{endDate}     = $eunix_ms;
      $params->{symmetrixId} = $cfg->{'symmetrix'};
      $params->{metrics}     = [@metrics];

      if ( $debug == 2 ) {
        message("perf_array         : $params->{symmetrixId}");
        message("startDate          : $params->{startDate} : "      .localtime( int($params->{startDate}/1000) ));
        message("endDate            : $params->{endDate} : "        .localtime( int($params->{endDate}/1000) ));
        message("firstAvailableDate : $_->{'firstAvailableDate'} : ".localtime( int($_->{'firstAvailableDate'}/1000) ));
        message("lastAvailableDate  : $_->{'lastAvailableDate'} : " .localtime( int($_->{'lastAvailableDate'}/1000) ));
        message("metrics            : ".join(", ",@metrics));
      }

      my $stats = new_post_request("performance/Array/metrics",$params,1);

      #
      # parse stats
      #
      if ( defined $stats && exists $stats->{'resultList'}{'result'} && ref($stats->{'resultList'}{'result'}) eq "ARRAY" ) {
        foreach my $values ( @{ $stats->{'resultList'}{'result'} } ) {
          if ( exists $values->{timestamp} ) {
            $data{perf}{ $values->{timestamp} }{Array}{ $cfg->{symmetrix} } = $values;
          }
        }
      }
      else {
        $logger->error( "Array stats not found!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }
  else {
    $logger->error( "Array keys not found!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub get_unisphere_version {
  my $result = new_get_request("system/version");

  if ( defined $result && exists $result->{version} ) {
    $unisphere_version = $result->{version};
  }
  else {
    # try newer namespace yet
    $result = new_get_request("90/system/version"); # V9

    if ( defined $result && exists $result->{version} ) {
      $unisphere_version = $result->{version};
    }
  }

  return 1;
}

sub set_url_paths {
  $url_paths->{provisioning}    = "provisioning";
  $url_paths->{sloprovisioning} = "sloprovisioning";
  $url_paths->{system}          = "system";
  $url_paths->{replication}     = "83/replication";

  if ( $unisphere_version =~ m/^V9|^V10/ ) {
    $url_paths->{provisioning}    = "90/provisioning";
    $url_paths->{sloprovisioning} = "90/sloprovisioning";

    if ( $unisphere_version =~ m/^V9\.1|^V9\.2/ ) {
      $url_paths->{provisioning}    = "91/provisioning";
      $url_paths->{sloprovisioning} = "91/sloprovisioning";
    }
    elsif ( $unisphere_version =~ m/^V10/ ) {
      $url_paths->{provisioning}    = "100/provisioning";
      $url_paths->{sloprovisioning} = "100/sloprovisioning";
      $url_paths->{system}          = "100/system";
      $url_paths->{replication}     = "100/replication";
    }
  }

  return 1;
}

sub new_post_request {
  my $namespace   = shift;
  my $body        = shift;
  my $print_error = shift;

  my $url     = "https://$cfg->{'apiHost'}:$cfg->{'apiPort'}/univmax/restapi/" . $namespace;
  my $agent   = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( POST => $url );

  $request->header( 'Content-Type' => 'application/json' );
  $request->authorization_basic( $cfg->{'apiUsername'}, $cfg->{'apiPassword'} );
  $request->content( encode_json($body) );

  if ( $debug ) {
    #my $filename = lc( Xormon::create_hash_key($url) );
    #my $outfile  = "$debugdir/$filename.json";
    #if ( -f $outfile ) {
    #  #print "$url : $outfile : exists!\n";
    #  return Xormon::read_json($outfile);
    #}
    #else {
    #  message("$url : $outfile : NOT exists!");
    #  return undef;
    #}
  }

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    if ( defined $print_error && $print_error eq "1" ) {
      my $st_line = $results->status_line;
      my $res_con = $results->content;
      $logger->error("POST url     : $url");
      $logger->error("Request error: $st_line");
      $logger->error("Request error: $res_con");
      $logger->error("JSON body    : ".encode_json($body));

      if ( $connection_test ) {
        push(@rows, "POST url     : $url\n");
        push(@rows, "Request error: $st_line\n");
        push(@rows, "Request error: $res_con\n");
        push(@rows, "JSON body    : ".encode_json($body)."\n");
      }
    }

    return undef;
  }

  if ( defined $results->content && $results->content ne '' ) {
    my $result = {};
    eval { $result = decode_json( $results->content ); 1 };

    return $result;
  }

  return undef;
}

sub new_get_request {
  my $namespace = shift;

  my $url     = "https://$cfg->{'apiHost'}:$cfg->{'apiPort'}/univmax/restapi/" . $namespace;
  my $agent   = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( GET => $url );

  $request->header( 'Content-Type' => 'application/json' );
  $request->authorization_basic( $cfg->{'apiUsername'}, $cfg->{'apiPassword'} );

  if ( $debug ) {
    #my $filename = lc( Xormon::create_hash_key($url) );
    #my $outfile  = "$debugdir/$filename.json";
    #if ( -f $outfile ) {
    #  #print "$url : $outfile : exists!\n";
    #  return Xormon::read_json($outfile);
    #}
    #else {
    #  message("$url : $outfile : NOT exists!");
    #  return undef;
    #}
  }

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    my $res_con = $results->content;
    $logger->error("GET url      : $url");
    $logger->error("Request error: $st_line");
    $logger->error("Request error: $res_con");

    if ( $connection_test ) {
      push(@rows, "GET url      : $url\n");
      push(@rows, "Request error: $st_line\n");
      push(@rows, "Request error: $res_con\n");
    }

    return undef;
  }

  if ( defined $results->content && $results->content ne '' ) {
    my $result = {};
    eval { $result = decode_json( $results->content ); 1 };

    return $result;
  }

  return undef;
}

sub message {
  my $text = shift;

  if ( defined $text ) { print localtime()." : $text\n"; }

  return 1;
}

1;
