use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;                
use lib "$FindBin::Bin/"; # use lib in this folder
use Xormon;
use VmwareX;
#use lib '/home/xormon/nextgen/server-nest/microservices/vmware';
use lib "../../../server-nest/microservices/vmware";
use lib "../../../server-nest/microservices/vmware/lpar2rrd/bin";
use File::Copy;
use lib "../../../perl-modules/vmware-lib";

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

my $DEFAULT_PORT = 3107;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);

my $server = IO::Socket->new(
    Domain => AF_INET,
    Type => SOCK_STREAM,
    Proto => 'tcp',
    LocalHost => '0.0.0.0',
    LocalPort => $port,
    ReusePort => 1,
    Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

my %result_hash = ();

 # waiting for a new client connection
while ($client = $server->accept()) {

  my $pid;
  unless ( defined( $pid = fork() ) ) {
    Xormon::error("Wait for fork");
    sleep 5;
  }

  if ($pid == 0) {

    # get information about a newly connected client
    my $client_address = $client->peerhost();
    my $client_port = $client->peerport();
    Xormon::log("Connection from $client_address:$client_port");

    # received data template 
    #
    # { 
    #   "jwt"  : "<backendToken>",
    #   "task" : "<taskName>"
    #   "data" : {
    #     <configuration_from_hostcfg>
    #   }
    # }

    # read up to 2048 characters from the connected client
    my $data = "";
    $client->recv($data, 2048);

    my $decoded = from_json($data);
        
    Xormon::log("Received task: ".$decoded->{task});
    my $response = '';

    # my %result_hash = (); # is global var

    if ($decoded->{task} eq "fetch") {

      $client->send("Task $decoded->{task} successfuly started");
      $client->shutdown(SHUT_WR);

      # print Dumper ("81 xormon",$xormon);

      my $cfg = $decoded->{data};
      my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'vmware', $cfg->{hostcfg_id}, $cfg->{hostalias});

      $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

      ############################### AGENT SECTION ###############################

      # target destinations check
      if (defined $decoded->{'destinations'}) {
        $xormon->setDestinations($decoded->{'destinations'});
      }
            
      $ENV{VI_PASSWORD}   = $cfg->{password};
      $ENV{VI_USERNAME}   = $cfg->{username};
	  if (! exists $cfg->{proto}) {
		$ENV{VI_PROTOCOL} = 'https'
	  }
	  else {
        $ENV{VI_PROTOCOL}   = $cfg->{proto};
	  }
      $ENV{VI_PORTNUMBER} = $cfg->{port};
      $ENV{VI_SERVER}     = $cfg->{host};
      $ENV{HOSTCFG_ID}    = $cfg->{hostcfg_id};
      $ENV{HOSTALIAS}     = $cfg->{hostalias};
      Xormon::log( "--------- api2csv cfg ------- $ENV{HOSTALIAS} $ENV{VI_SERVER} $ENV{HOSTCFG_ID}\n");

      # push architecture from last run
      VmwareX::vmware_json2ng($xormon, $cfg->{host});
            
      # next 2 lines only for debug, only arch file by file from tmp/arch_$cfg->{host}... has been sent
      # $xormon->endJob($decoded->{task}, $cfg->{hostalias}, "199", $$);
      # exit();

      my $pwd = `pwd`;
	  chomp $pwd;
	  my $vmware_log_dir = "$pwd"."/../../logs";
      # print "PWD = $pwd\n";
      my $vmware_scr = "$pwd/lpar2rrd/load_vmware.sh";
      my $vmware_out = "$vmware_log_dir/vmware_load_$ENV{VI_SERVER}.tmp"; # not $client_port
	  move "$vmware_log_dir/vmware_load_$ENV{VI_SERVER}.out","$vmware_log_dir/vmware_load_$ENV{VI_SERVER}.prev";
	  copy "$vmware_log_dir/vmware_load_$ENV{VI_SERVER}.tmp","$vmware_log_dir/vmware_load_$ENV{VI_SERVER}.out";

      # following 2 lines you can use as a test
  	  # my $conn_vmware = `perl $pwd/lpar2rrd/vmware-lib/apps/connect.pl --server $ENV{VI_SERVER} --username '$ENV{VI_USERNAME}' --password $ENV{VI_PASSWORD} --protocol $ENV{VI_PROTOCOL} 2>&1`;
      # print "106 \$ENV{VI_SERVER} $ENV{VI_SERVER} \$ENV{VI_USERNAME} $ENV{VI_USERNAME} \$ENV{VI_PASSWORD} $ENV{VI_PASSWORD} \$ENV{VI_PORTNUMBER} $ENV{VI_PORTNUMBER} \$ENV{VI_PROTOCOL} $ENV{VI_PROTOCOL} \$conn_vmware = ,$conn_vmware,\n";

      # get data from technology
      #
      ### run another perl script
      # `/home/xormon/nextgen/server-nest/microservices/vmware/lpar2rrd/load_vmware.sh > /home/xormon/nextgen/server-nest/microservices/vmware/lpar2rrd/load_vmware.out 2>&1`;
	  print "126 ".localtime()." pid= $$\n";
	  if (!-f $vmware_scr) {
        Xormon::error("File \$vmware_scr $vmware_scr does not exist!"); 
	  }
      if (!-f $vmware_out) {
        Xormon::error("File \$vmware_out $vmware_out does not exist!");
      }
      Xormon::log("Calling cmd system ($vmware_scr > $vmware_out 2>&1)");
      #my $vmw_res = `"$vmware_scr" > "$vmware_out" 2>&1`;
	  # system(`"$vmware_scr" > "$vmware_out" 2>&1`);
	  system("$vmware_scr > $vmware_out 2>&1");
	  print "142 result=,$!, at ".localtime()." pid= $$\n"; 
	  # print "128 \$vmw_res ,$vmw_res, ".localtime()."\n";
			 
      %result_hash = ();
      my $dataToBackend = api2csv($xormon,$cfg);
      if ($dataToBackend eq "99") {
        $xormon->hasData(1);
		$xormon->isConnected(1);
	  }
	  else {
        $xormon->hasData(0);
        $xormon->isConnected(0);
	  }

      # print Dumper ($result_hash{ESXI});
      # print Dumper (\%result_hash);

      # testing 
      # sleep(30);

      # send data to backend
      # status code
      my $status;

      my $rc = "not defined";
      $rc = $xormon->saveCSV($result_hash{ESXI},"vmware","esxi") if (defined $result_hash{ESXI});
      $status .= "ESXI $rc, ";
      
	  $rc = "not defined";
      $rc = $xormon->saveCSV($result_hash{VM},"vmware","vm") if (defined $result_hash{VM});
      $status .= "VM $rc, ";
      
	  $rc = "not defined";
      $rc = $xormon->saveCSV($result_hash{RESGROUP},"vmware","respool") if (defined $result_hash{RESGROUP});
      $status .= "RESGROUP $rc, ";
      
	  $rc = "not defined";
      $rc = $xormon->saveCSV($result_hash{CLUSTER},"vmware","cluster") if (defined $result_hash{CLUSTER});
      $status .= "CLUSTER $rc, ";

      $rc = "not defined";
      $rc = $xormon->saveCSV($result_hash{DATASTORE},"vmware","datastore") if (defined $result_hash{DATASTORE});
      $status .= "DATASTORE $rc, ";

      $rc = "not defined";
      $rc = $xormon->saveCSV($result_hash{DATASTORE_VM},"vmware","datastore_vm") if (defined $result_hash{DATASTORE_VM});
      $status .= "DATASTORE_VM $rc, ";

      $rc = "not defined";
      $rc = $xormon->saveCSV($result_hash{ESXI_VM},"vmware","esxi_vm") if (defined $result_hash{ESXI_VM});
      $status .= "ESXI_VM $rc, ";

      my $status_all = $status;
      ############################### END AGENT SECTION ###############################
      # $status = "successful";
      # status code
      # my $status;
      # if ($rc eq "201") {
      #   $status = "successful";
      # } elsif ($rc eq "401") {
      # $status = "unauthorized";
      # } else {
      # $status = "error";
      # }

      # VmwareX::vmware_json2ng($xormon); # do it before vmware load

      $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);
      Xormon::log("Task [$decoded->{task}] for ".$cfg->{hostalias}." is done with status $status_all\n-----------------------------------");

    } elsif ($decoded->{task} eq "test") {

        my $cfg = $decoded->{data};
        $response = apitest($cfg);

        $client->send($response);
        $client->shutdown(SHUT_WR);

    } else {
        $response = "undefined task";

        $client->send($response);
        $client->shutdown(SHUT_WR);

    }

    exit();

  } else {
      $client->autoflush(1);    
  }
}

sub api2csv {
  my $xormon = shift;
  my $cfg = shift;

  #Xormon::log( "--------- api2csv data -------\n");
  # print Dumper \$cfg;

  #my $host_name = "10.22.11.10";
  my $host_name = $cfg->{host};

  my $exit_value = VmwareX::push_data_from_timed_files($xormon,$host_name,\%result_hash);
  print "sub: push_data_from_timed_files has \$exit_value $exit_value\n";
  return $exit_value;
}

sub apitest {
    my $cfg = shift;
    my @rows;

    my %result_con = connTest( $cfg->{host}, $cfg->{port} );
	push @rows, $result_con{msg};
	my $return_status = $result_con{status};

#    if ( $result{status} eq "1" ) {
#      $msg .= "$result{msg} </br>";
#    }
#    elsif ( $result{status} eq "0" ) {
#      $err_msg .= "$result{msg} </br>";
#    }
    my %result_auth = authorization_test ($cfg);
	push @rows, $result_auth{msg};
	$return_status = $result_auth{status};

    my %result_data = vmware_data_test ($cfg);
	push @rows, $result_data{msg};
	$return_status = $result_data{status};

    return Xormon::conn_test($return_status, \@rows);
}

sub authorization_test {
  my $cfg = shift;
  my %result;

  my $password = $cfg->{password};
  my $username = $cfg->{username};
  my $proto    = 'https';
  if ( exists $cfg->{proto}) {
    $proto     = $cfg->{proto};
  }
  my $port     = $cfg->{port};
  my $host     = $cfg->{host};

  my $pwd = `pwd`;
  chomp $pwd;

  my $conn;
  my @ainc = @INC; 
  eval {
    local $SIG{ALRM} = sub { die 'Timed Out'; };
    alarm 5;
    $conn = `perl $pwd/../../../perl-modules/vmware-lib/apps/connect.pl --server '$host' --username '$username' --password '$password' --portnumber '$port' --protocol '$proto' 2>&1`;
 #$conn = `perl /home/xormon/nextgen/perl-modules/vmware-lib/apps/connect.pl --server '$host' --username '$username' --password '$password' --portnumber '$port' --protocol '$proto' 2>&1`;
# $conn = `perl /home/xormon/nextgen/server-nest/microservices/vmware/lpar2rrd/vmware-lib/apps/connect.pl --server '$host' --username '$username' --password '$password' --portnumber '$port' --protocol '$proto' 2>&1`;
    alarm 0;
  };
  alarm 0;    # race condition protection
  if ( $@ && $@ =~ /Timed Out/ ) {
	$result{status} = 0;
    $result{msg} = "$@ API login to $host timed out after 5 seconds!";
  }
  elsif ( $conn =~ "Connection Successful" ) {
	$result{status} = 1;
    # $result{msg} = "API authorization to $host($username) is OK.@ainc,";
	$result{msg} = "API authorization to $host($username) is OK.";
  }
  else {
    if ( $conn =~ "Server version unavailable" ) {
      my $outext = `perl $pwd/lpar2rrd/bin/perl_modules_check.pl`;
      if ( $outext =~ "Perl module has not been found" ) {
        # $outext =~ s/\n/<br>\n/g;
		$result{status} = 0;
        $result{msg} = "$conn\n$outext\n";
      }
      else {
		$result{status} = 0;
        $result{msg} = "API authorization to $host has failed!\n$conn\n";
      }
    }
    else {
	  $result{status} = 0;
      $result{msg} = "API authorization to $host has failed!<br>$conn\n";
    }
  }
  return %result;
}

sub connTest {
  my $host = shift;
  my $port = shift;
  my %result;

  use IO::Socket::INET;
  my $sock;

  eval {
    local $SIG{ALRM} = sub { die 'Timed Out'; };
    alarm 10;
    $sock = new IO::Socket::INET(
      PeerAddr => $host,
      PeerPort => $port,
      Proto    => 'tcp',
      Timeout  => 3
    );

    alarm 0;
  };
  alarm 0;    # race condition protection
  if ( $@ && $@ =~ /Timed Out/ ) {
    $result{status} = 0;
    $result{msg}    = "TCP connection to $host:$port timed out after 10 seconds!";
  }
  elsif ($sock) {
    $result{status} = 1;
    $result{msg}    = "TCP connection to $host:$port is OK.";
  }
  else {
    $result{status} = 0;
    $result{msg}    = "TCP connection to $host:$port has failed! Open it on the firewall.";
  }
  return %result;
}

sub vmware_data_test {
  my $cfg = shift;
  my %result;

  my $password = $cfg->{password};
  my $username = $cfg->{username};
  my $proto    = 'https';
  if ( exists $cfg->{proto}) {
    $proto     = $cfg->{proto};
  }
  my $port     = $cfg->{port};
  my $host     = $cfg->{host};

  my $pwd = `pwd`;
  chomp $pwd;
# push @INC, "$pwd/../../../perl-modules/";
# push @INC, "/home/xormon/nextgen/perl-modules/vmware-lib";
  my $conn     = "OK";

  # save STDOUT and redirect actual one to /dev/null,
  # Opts::validate() can print on stdout some garbage what leds to incorect answer to front-end
  open( my $oldout, ">&STDOUT" );
  open( LOG,        ">/dev/null" );
  *STDOUT = *LOG;

  eval {
    local $SIG{ALRM} = sub { die 'Timed Out'; };
    alarm 20;

    require VMware::VIRuntime;

    # easy test if at least one VM and one HostSystem presented
    $ENV{VI_SERVER}    = "$host";
    $ENV{VI_USERNAME}  = "$username";
    #$ENV{VI_CREDSTORE} = "$credstore";
    $ENV{VI_PASSWORD}  = $cfg->{password};
    $ENV{VI_PROTOCOL}  = $proto;
    $ENV{VI_PORTNUMBER} = $cfg->{port};
	
    Opts::parse();
    Opts::validate();
    Util::connect();

    my $obj_views = Vim::find_entity_views( view_type => 'VirtualMachine', properties => ['name'] );
    if ( !defined $obj_views || $obj_views eq "" || !defined @$obj_views[0] ) {
      $conn = "not presented any VirtualMachine - is there Read-only permission for all Vmware folders?";
    }
    $obj_views = Vim::find_entity_views( view_type => 'HostSystem', properties => ['name'] );
    if ( !defined $obj_views || $obj_views eq "" || !defined @$obj_views[0] ) {
      $conn = "not presented any HostSystem - is there Read-only permission for all Vmware folders?";
    }
    Util::disconnect();
    alarm 0;

    # recovery of STDOUT
    close(LOG);
    open( STDOUT, ">&", $oldout ) or die "Can't dup \$oldout: $!";

    if ( $conn =~ "not presented" ) {
      $result{status} = 0;
	  $result{msg} = "API data test:$conn";
    }
    else {
	  $result{status} = 1;
      $result{msg} = "API data test:$conn";
    }
  };
  alarm 0;    # race condition protection

  if ($@) {

    # recovery of STDOUT
    close(LOG);
    open( STDOUT, ">&", $oldout ) or die "Can't dup \$oldout: $!";
    if ( $@ =~ /Timed Out/ ) {
      $result{status} = 0;
      $result{msg} = "API data test timed out after 20 seconds!";
    }
    else {
      if ( $@ =~ /stty failed/ ) {
		$result{status} = 0;
		$result{msg} = "API data test failed: Most probably you have more entries of that vCenter in Configuration. Edit it and remove all except one and try it again";
      }
      else {
		$result{status} = 0;
		$result{msg} = "API data test failed: $@";
      }
    }
  }
  return %result;
}

$server->close();
