package VSPG;

use strict;
use warnings;

use Data::Dumper;
use JSON qw(encode_json decode_json);
use Date::Parse;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Xormon;
use POSIX ":sys_wait_h";
use LWP::UserAgent;
use File::Copy;
use File::Temp qw(tempfile);
use Test;
use XormonLogger;
my $logger;


my %data;
my $xormon;
my $cfg;
my $data_directory = "$FindBin::Bin";
my $debug = 0;

my $storage_name = "";
my $active_minute = "";
my $debug_data = "";
my %performance_data;
my %configuration_data;
my $st_serial;
my @architecture = ();
my $service_data_out;
my $st_type = "vspg";
my @rows_api_test = ();
my @rows_api_test_data = ();
my @data_test = ();
my $protocol = "";
my $wrkdir = "";
my %header_section;
my $service_data_in;
my $connection_test = 0;
my @health_status = ();
my @hs_global = ();
my $api_port = 80;
my $api_ip = "";
my $token = "";
my @ips = ();
my $session_id = "";
my $vsp_clidir;
my $vspg_sample = "-0005:";
my $performance_data_store = "";
my %volumes;
my $index_data = 6;
my %caches;
my %volumes_gad;
my @cache_type = ();
my $export_tool_port = $ENV{VSPG_EXPORT_TOOL_PORT};
my $sample_rate_global = 5;
my $export_tool_version = 1;
my $offset_time = "";
my $enviroment = 1;
my $timestamp_global;
my $prefix_uuid_volume = "";
my $device_uid_prefix = "";
my $path = $ENV{PATH};
my $run_unix      = "./runUnix.sh";
if ( -f "/usr/bin/bash" || -l "/usr/bin/bash" ) {
  $run_unix = "bash runUnix.sh";
}

sub get_time_range_export_tool1{
  use POSIX qw(ceil);
  if (!Xormon::isdigit($service_data_in->{last_perf_timestamp})){
    return "-0005:";
  }
  my $diff = abs(time() - $service_data_in->{last_perf_timestamp});
  if ( $diff > 500 && $diff < 18500 ) {
    my $secs        = $diff;
    my $sample_rate = get_sample_rate();
    my $count_data = ceil( $secs / $sample_rate );
    if ( $count_data == 0 ) {
      $count_data  = 1;
      return "-0005:";
    }
    my $days = int( $secs / ( 24 * 60 * 60 ) );
    $secs -= ( $days * 24 * 60 * 60 );
    my $hours = int( $secs / 3600 );
    $secs -= ( $hours * 3600 );
    my $mins = int( $secs / 60 );
    $secs -= ( $mins * 60 );
    if ( $mins < 10 ) {
      $mins = "0$mins";
    }
    if ( $hours < 10 ) {
      $hours = "0$hours";
    }
    my $vspg_sample_prev = $vspg_sample;
    return "-$hours$mins:";
  }
  return "-0005:";
}

sub get_sample_rate {
  my $file_port_iops = "$performance_data_store/Port_dat/Port_IOPS.csv";
  if ( -f $file_port_iops ) {
    my @data = get_array_data($file_port_iops);
    foreach my $line (@data) {
      chomp $line;
      if ( $line =~ m/sampling rate :/ ) {
        my ( undef, $sample ) = split( " : ", $line );
        if ( Xormon::isdigit($sample) ) {
          return $sample * 60;
        }
      }
    }
  }
  return 300;
}


sub collect {
  $xormon = shift;
  $cfg    = shift;

  #print Dumper \$cfg;
  #'hcmPort' => 23451,
  #'device' => 'HITACHI__VSP_G',
  #'apiPassword' => 'stor2rrd',
  #'currency' => 'USD',
  #'hcmProto' => 'https',
  #'svpHost' => 'localhost',
  #'hostalias' => 'Hitachi-VSPG',
  #'exportRadio' => 'rest',
  #'apiUsername' => 'stor2rrd',
  #'created' => '1689765069968',
  #'node1Host' => 'localhost',
  #'hostcfg_id' => '22b69e8f-c0d9-4d0c-b0ef-00b50f76c923',
  #'updated' => undef,
  #'restProto' => 'https',
  #'node2Host' => 'localhost'
  #print Dumper \$cfg;
  #return "error";

  my @variables = ();
  $timestamp_global = get_global_timestamp();
  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!") && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!") && return "error"; }
  $logger = XormonLogger->new( 'vspg', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'node1Host'} )          { $logger->error("Cannot continue, 'node1Host' is not configured!") && return "error"; }
  #unless ( exists $cfg->{'node2Host'} )          { $logger->error("Cannot continue, 'node2Host' is not configured!") && return "error"; }
  unless ( exists $cfg->{'restProto'} )      { $logger->error("Cannot continue, 'restProto' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiUsername'} )      { $logger->error("Cannot continue, 'apiUsername' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )      { $logger->error("Cannot continue, 'apiPassword' is not configured!") && return "error"; }
  unless ( exists $cfg->{'svpHost'} )      { $logger->error("Cannot continue, 'svpHost' is not configured!") && return "error"; }
  unless ( exists $cfg->{'exportRadio'} )      { $logger->error("Cannot continue, 'exportRadio' is not configured!") && return "error"; }
  if ($cfg->{'exportRadio'} eq "manager"){
    unless ( exists $cfg->{'hcmHost'} )      { $logger->error("Cannot continue, 'hcmHost' is not configured!") && return "error"; }
    unless ( exists $cfg->{'hcmPort'} )      { $logger->error("Cannot continue, 'hcmPort' is not configured!") && return "error"; }
    unless ( exists $cfg->{'hcmProto'} )     { $logger->error("Cannot continue, 'hcmProto' is not configured!") && return "error"; }
    unless ( exists $cfg->{'hcmDeviceId'} )  { $logger->error("Cannot continue, 'hcmDeviceId' is not configured!") && return "error"; }

  }
  if (defined $cfg->{VSPG_ENVIROMENT} && $cfg->{VSPG_ENVIROMENT} eq "development"){
    $enviroment = 0;
  }
  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
  if (defined $cfg->{exportToolHome}){
    $vsp_clidir = $cfg->{exportToolHome};
  }
  else{
    $vsp_clidir = "/opt/hds";
  }
  if (defined $cfg->{device_uuid}){
    $device_uid_prefix = $cfg->{device_uuid};
  }

  #print Dumper \$cfg;

  my $act_timestamp     = time();
  #$service_data_in = $xormon->getData($cfg->{'hostalias'});
  $service_data_in  = $xormon->getData();
  if ( exists $service_data_in->{last_conf_timestamp} ) {
    $service_data_out->{last_conf_timestamp} = $service_data_in->{last_conf_timestamp};
  }
  if (defined $service_data_in->{host_api} && $service_data_in->{host_api} ne ""){
    push(@ips,$service_data_in->{host_api});
  }
  #print $service_data_in->{data}{value}{last_perf_timestamp} . "\n";
  $storage_name = $cfg->{hostalias};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $st_serial = $cfg->{'uuid'};
  $protocol = $cfg->{'restProto'};
  if ($protocol eq "https"){
    $api_port = 443;
  }
  if (!grep {/^$cfg->{'node1Host'}$/} @ips){
    push(@ips,$cfg->{'node1Host'});
  }
  if (defined $cfg->{'node2Host'} && $cfg->{'node2Host'} ne "" && !grep {/^$cfg->{'node2Host'}$/} @ips){
    push(@ips,$cfg->{'node2Host'});
  }
  if (!grep {/^$cfg->{'svpHost'}$/} @ips){
    push(@ips,$cfg->{'svpHost'});
  }
  if ($cfg->{'exportRadio'} eq "manager"){
    @ips = ();
    push(@ips,$cfg->{"hcmHost"});
  }

  data_structure();
  if (!defined $vsp_clidir){
    $vsp_clidir = "/opt/hds";
  }
  #### delete log files if files are older than 3 days
  delete_files("$data_directory/data/$st_serial/log");
  ####

  ### device_id is exist
  if (!check_device_id()){
    $xormon->isConnected(0, "Connection failed!");
    $logger->error("Device_id $cfg->{'hcmDeviceId'} not found") && return "error";
  }
  ###

  #login
  foreach my $ip (@ips){
    $api_ip = $ip;
    set_token_and_session_id();
    if (defined $token && $token ne "" && defined $session_id && $session_id ne ""){last;}
  }
  if (!defined $token || $token eq "" || !defined $session_id || $session_id eq ""){
    $xormon->isConnected(0, "Connection failed!");
    $logger->error("Session id or token is not defined!") && return "error";
  }
  $xormon->isConnected(1);
  $offset_time = get_offset_time();
  set_offset_time();
  cmd_api("v1/objects/sessions/$session_id", "DELETE", "v1/objects/sessions",1, "", 0); ## logout export may take several minutes
  if (!get_performance_statistics()){
    #cmd_api("v1/objects/sessions/$session_id", "DELETE", "v1/objects/sessions",1, "", 0);
    if (Xormon::isdigit($service_data_in->{last_perf_timestamp})){
      if ( ( time() -  $service_data_in->{last_perf_timestamp} ) > 10800 ) {
        $xormon->hasData(0, "last available stats are older than 3 hours: " . $service_data_in->{last_perf_timestamp});
      }
      else{
        $xormon->hasData(1);
      }
    }
    else{
      $xormon->hasData(0, "no available stats");
    }
    foreach my $line (@data_test){
      chomp $line;
      if ($line =~ m/sun.security.x509/i){
        $logger->error("Java security error (SSL / X509). Install a version of java older than 16. We recommend java 11.");
        last;
      }
    }
    $logger->error("Data collection (performance) failed") && return "error";
  }
  set_token_and_session_id(); 
  if (!$enviroment && -f "$data_directory/data/$st_serial/index_data.txt"){
    my @data_index = get_array_data("$data_directory/data/$st_serial/index_data.txt");
    if (defined $data_index[0]){
      $index_data = $data_index[0];
    }
  }
  #$logger->info("Data start: timestamp global = $timestamp_global\n");
  if ($export_tool_version == 1){
    set_data_port();
    set_data_volume();
    set_data_cpu();
    set_data_cache();
    set_data_raid_group();
    set_data_host();
    set_data_pool();
    set_data_node();
    set_data_tier();
    set_data_device();
    #$logger->info("timestamp data for port cf1d260a8d is " .  (keys %{$performance_data{$st_type}{port}{cf1d260a8d}})[0] . "\n");
    #print Dumper \$performance_data{$st_type}{device};
  }
  else{
    if (!$enviroment && $index_data > 6){
      $index_data = 0;
    }
    set_data_port2();
    set_data_volume2();
    set_data_cpu2();
    set_data_cache2();
    set_data_raid_group2();
    set_data_host2();
    set_data_pool();
    set_data_node();
    set_data_tier();
    set_data_device();

  }

  undef %volumes;

  #print Dumper \%configuration_data;
  #print "$token & $session_id\n";
  delete_empty_metrics();
  calculate_missing_metrics();
  #print Dumper \%performance_data;
  #print "koncime\n";
  if (!$enviroment){
    $index_data++;
    write_to_file(($index_data),"$data_directory/data/$st_serial/index_data.txt");
  }
  #return "successful";

  #
  my $rc = $xormon->saveJSON2(\%performance_data);
  #print Dumper \$performance_data{device};
  my $status = get_status($rc, "saveJSON2 (performance)");

  ### host_cfg_data
  if (Xormon::isdigit($service_data_out->{last_perf_timestamp})){
    if ( ( time() -  $service_data_out->{last_perf_timestamp} ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: " . $service_data_out->{last_perf_timestamp});
    }
    else{
      $xormon->hasData(1);
    }
  }
  else{
    $xormon->hasData(0, "no available stats");
  }

  #return $status;
  #$logger->info("Task performance for ".$cfg->{hostalias}." is done. Status: $status");
  #print Dumper $service_data_out;
  my $save_data = $xormon->saveData(\%{ $service_data_out });
  my $status_save_data = get_status($save_data, "SaveData");

  cmd_api("v1/objects/sessions/$session_id", "DELETE", "v1/objects/sessions",1, "", 0);
  #### architecture

  #configuration
  if (( ! exists $service_data_in->{last_conf_timestamp} ) || ( ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) ) || ( $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) || ( !-f "$debug_data/config-$st_serial.out" )){
    `touch "$debug_data/config-$st_serial.out"`;
    foreach my $ip (@ips){
      $api_ip = $ip;
      set_token_and_session_id();
      if (defined $token && $token ne "" && defined $session_id && $session_id ne ""){last;}
    }
    if (!defined $token || $token eq "" || !defined $session_id || $session_id eq ""){
       $logger->error("Session id or token is not defined!") && return "error";
    }
    #print Dumper \%configuration_data;
    set_data_port_conf();
    set_data_volume_conf();
    set_data_pool_conf();
    set_data_node_conf();
    set_data_host_conf();
    set_data_device_conf();
    set_data_cpu_conf();
    set_data_cache_type_conf();
    set_data_raid_group_conf();
    set_data_tier_conf();

    ### health status
    set_data_health_status();
    ###



    cmd_api("v1/objects/sessions/$session_id", "DELETE", "v1/objects/sessions",1, "", 0);
    my @types = ("pool", "volume", "port", "node", "host", "cpu", "cache", "cpu_type", "raid_group", "tier", "device");
    foreach my $uid (keys %configuration_data){
      my $active = 0;
      foreach my $type (@types){
        if (defined $performance_data{$st_type}{$type}{$uid}){
          $active++;
          last;
        }
        if ($type eq "cpu_type" && defined $configuration_data{$uid}{TYPE} && $configuration_data{$uid}{TYPE} eq $type){
          $active++;
          last;
        }
      }
      if ($active == 0){
        #print Dumper \$configuration_data{$uid};
	#print "mazeme prvek $uid nema perf data\n";
        delete $configuration_data{$uid};
        #print "mazeme prvek $uid\n";
      }
    }
    #print Dumper \%configuration_data;
    #print Dumper $performance_data{$st_type}{pool};
    $service_data_out->{last_conf_timestamp} = $act_timestamp;
    ### health status
    #set_data_health_status();
    ###
    ($status, my $status_configuration, my $status_health) = set_architecture();
    #$xormon->saveConfJSON(\%configuration_data);
    if ($status eq "successful" && $status_configuration eq "successful" && $status_health eq "successful" ){
      my $save_data = $xormon->saveData(\%{ $service_data_out });
      my $status_save_data = get_status($save_data, "saveData");
    }
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
    return $status;
    #$logger->info("Task SaveData for ".$cfg->{hostalias}." is done. Status: $status_save_data");
  }
  $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
  return $status;
}

sub set_data_health_status{
  my $message_global = "";
  my $status_global = "ok";
  my $cmd = "v1/objects/components/instance";
  my $data = cmd_api($cmd, "GET", "components", 1, "");
  if ($data eq ""){
    $cmd = "v1/services/storage-cache-service/actions/refresh/invoke";
    my $refresh = cmd_api($cmd, "POST", "invoke", 1, "");
    $cmd  = "v1/objects/components/instance?componentOption=class";
    $data = cmd_api($cmd, "GET", "components_class", 1, "");
  }
  if ( $data ne "" && !defined $data->{bkmfs} && !defined $data->{cacheMemories} && !defined $data->{driveBoxes} && !defined $data->{fans} && defined $data->{ctls} ) {
    if ( ref( $data->{ctls} ) eq "ARRAY" ) {
      foreach my $node ( @{ $data->{ctls} } ) {
        my $local  = $node->{location};
        my $status = $node->{status};
        if ( !defined $local || !defined $status ) { next; }
        if ( $status ne "Normal" ) {
          $status_global = "error";
          $message_global = "Storage System $local $status";
          my %data;
          $data{section} = "Storage System";
          $data{"component_unit"} = "$local";
          $data{"status"} = $status;
          push(@health_status,\%data);
        }
      }
      if ($status_global ne "error"){
        my %data;
        $data{section} = "Storage System";
        $data{"component_unit"} = "Controllers";
        $data{"status"} = "Normal";
        push(@health_status,\%data);
      }
    }
    $cmd = "v1/services/storage-cache-service/actions/refresh/invoke";
    my $refresh = cmd_api($cmd, "POST", "invoke", 1, "");
    $cmd  = "v1/objects/components/instance?componentOption=class";


    $data = cmd_api($cmd, "GET", "components_class", 1, "");
  }
  if ( $data eq "" ) {
    my %vm_status = (
    "item_id" => $st_serial,
    "status" => $status_global,
    "data" => $message_global
    );
    push(@hs_global, \%vm_status);
    return;
  }
  #$data = ();
  my $active_alert_drive         = 0;
  my $active_alert_ps            = 0;
  my $active_alert_enc           = 0;
  my $active_alert_fan           = 0;
  my $active_alert_backup_module = 0;
  my $active_alert_battery       = 0;
  my $active_alert_cache         = 0;
  my $active_alert_channel_board = 0;
  my $active_alert_node          = 0;
  my $active_alert_disk_board    = 0;
  my $active_alert_power_supply  = 0;
  my $active_alert_sfp           = 0;
  if ( $cmd =~ m/componentOption/ ) {
    $active_alert_drive         = -1;
    $active_alert_ps            = -1;
    $active_alert_enc           = -1;
    $active_alert_fan           = -1;
    $active_alert_backup_module = -1;
    $active_alert_battery       = -1;
    $active_alert_cache         = -1;
    $active_alert_channel_board = -1;
    $active_alert_node          = -1;
    $active_alert_disk_board    = -1;
    $active_alert_power_supply  = -1;
    $active_alert_sfp           = -1;
    my %convert_component_name;
    $convert_component_name{powerSupply}   = "Power supplies";
    $convert_component_name{"dkcps"}       = "Power supplies";
    $convert_component_name{"fan"}         = "Fans";
    $convert_component_name{"battery"}     = "Batteries";
    $convert_component_name{"cacheMemory"} = "Cache";
    $convert_component_name{"bkmf"}        = "Backup Modules";
    foreach my $section ( keys %{$data} ) {
      foreach my $component ( keys %{ $data->{$section} } ) {
        if ( $section =~ m/driveBoxSummary/ ) {
          my $component_new = $component;
          $component_new =~ s/Status//g;
          if ( defined $convert_component_name{$component_new} ) {
            $component_new = $convert_component_name{$component_new};
          }
          my $status = $data->{$section}->{$component};
          if ( $status =~ m/Service|Serious|Acute/ ) {
            $status_global = "error";
            $message_global = "Disk Unit $component_new $status";

          }
          if ($status =~ m/Moderate/ && $status_global eq "ok"){
            $status_global = "warning";
            $message_global = "Disk Unit $component_new $status";
          }
          my %data;
          $data{section} = "Disk Unit";
          $data{"component_unit"} = $component_new;
          $data{"status"} = $status;
          push(@health_status,\%data);
        }
        else {
          my $section_new = $section;
          $section_new =~ s/Summary//g;
          if ( defined $convert_component_name{$section_new} ) {
            $section_new = $convert_component_name{$section_new};
          }
          if ( $component =~ m/status/i ) {
            my $status = $data->{$section}->{$component};
            if ( $status =~ m/Service|Serious|Acute/ ) {
              $status_global = "error";
              $message_global = "Storage Unit $section_new $status";

            }
            if ($status =~ m/Moderate/ && $status_global eq "ok"){
              $status_global = "warning";
              $message_global = "Storage Unit $section_new $status";
            }
            my %data;
            $data{section} = "Storage Unit";
            $data{"component_unit"} = $section_new;
            $data{"status"} = $status;
            push(@health_status,\%data);
          }
        }
      }
    }
  }
  if ( defined $data->{driveBoxes} && ref( $data->{driveBoxes} ) eq "ARRAY" ) {
    foreach my $drives ( @{ $data->{driveBoxes} } ) {
      if ( defined $drives->{drives} && ref( $drives->{drives} ) eq "ARRAY" ) {
        foreach my $drive ( @{ $drives->{drives} } ) {
          my $local  = $drive->{location};
          my $status = $drive->{status};
          if ( !defined $local || !defined $status ) { next; }
          if ( $status eq "Failed" || $status =~ m/Warning|incomplete/ ) {
            $status_global = "error";
            $message_global = "Disk Unit $local $status";
            $active_alert_drive++;
            my %data;
            $data{section} = "Disk Unit";
            $data{"component_unit"} = $local;
            $data{"status"} = $status;
            push(@health_status,\%data);
          }
        }
      }
      if ( defined $drives->{dbps} && ref( $drives->{dbps} ) eq "ARRAY" ) {
        foreach my $drive ( @{ $drives->{dbps} } ) {
          my $local  = $drive->{location};
          my $status = $drive->{status};
          if ( !defined $local || !defined $status ) { next; }
          if ( $status ne "Normal" ) {
            my %data;
            $data{section} = "Disk Unit";
            $data{"component_unit"} = $local;
            $data{"status"} = $status;
            push(@health_status,\%data);
            $active_alert_ps++;
            $status_global = "error";
            $message_global = "Disk Unit $local $status";
          }
        }
      }
      else {
        $active_alert_ps = -1;
      }
      if ( defined $drives->{enc} && ref( $drives->{enc} ) eq "ARRAY" ) {
        foreach my $drive ( @{ $drives->{enc} } ) {
          my $local  = $drive->{location};
          my $status = $drive->{status};
          if ( !defined $local || !defined $status ) { next; }
          if ( $status ne "Normal" ) {
            $active_alert_enc++;
            my %data;
            $data{section} = "Disk Unit";
            $data{"component_unit"} = $local;
            $data{"status"} = $status;
            push(@health_status,\%data);
            $status_global = "error";
            $message_global = "Disk Unit $local $status";
          }
        }
      }
      else {
        $active_alert_enc = -1;
      }
    }
  }
  elsif ( $cmd !~ m/componentOption/ ) {
    $active_alert_ps  = -1;
    $active_alert_enc = -1;
    #foreach my $disk ( keys %disks_status ) {
    #  push( @datas,         "<tr><td>Disk Unit</td><td>$disk</td><td>$disks_status{$disk}</td></tr>\n" );
    #  push( @error_message, "Disk Unit;$disk;$disks_status{$disk}" );
    #  $active_alert_drive++;
    #  $error_health++;
    #}
  }
  if ( defined $data->{fans} && ref( $data->{fans} ) eq "ARRAY" ) {
    foreach my $fan ( @{ $data->{fans} } ) {
      my $local  = $fan->{location};
      my $status = $fan->{status};
      if ( !defined $local || !defined $status ) { next; }
      if ( $status ne "Normal" ) {
        $active_alert_fan++;
        my %data;
        $data{section} = "Storage System";
        $data{"component_unit"} = $local;
        $data{"status"} = $status;
        push(@health_status,\%data);
        $status_global = "error";
        $message_global = "Storage System $local $status";
      }
    }
  }
  else {
    $active_alert_fan = -1;
  }
  if ( defined $data->{bkmfs} && ref( $data->{bkmfs} ) eq "ARRAY" ) {
    foreach my $bac ( @{ $data->{bkmfs} } ) {
      my $local  = $bac->{location};
      my $status = $bac->{status};
      if ( !defined $local || !defined $status ) { next; }
      if ( $status ne "Normal" ) {
        $active_alert_backup_module++;
        my %data;
        $data{section} = "Storage System";
        $data{"component_unit"} = $local;
        $data{"status"} = $status;
        push(@health_status,\%data);
        $status_global = "error";
        $message_global = "Storage System $local $status";
      }
      if ( defined $bac->{batteries} && ref( $bac->{batteries} ) eq "ARRAY" ) {
        foreach my $battery ( @{ $bac->{batteries} } ) {
          my $local  = $battery->{location};
          my $status = $battery->{status};
          if ( !defined $local || !defined $status ) { next; }
          if ( $status ne "Normal" ) {
            $active_alert_battery++;
            my %data;
            $data{section} = "Storage System";
            $data{"component_unit"} = $local;
            $data{"status"} = $status;
            push(@health_status,\%data);
            $status_global = "error";
            $message_global = "Storage System $local $status";
          }
        }
      }
      else {
        $active_alert_battery = -1;
      }
    }
  }
  else {
    $active_alert_backup_module = -1;
    $active_alert_battery       = -1;
  }
  if ( defined $data->{cacheMemories} && ref( $data->{cacheMemories} ) eq "ARRAY" ) {
    foreach my $cache ( @{ $data->{cacheMemories} } ) {
      my $local  = $cache->{location};
      my $status = $cache->{status};
      if ( !defined $local || !defined $status ) { next; }
      if ( $status ne "Normal" ) {
        $active_alert_cache++;
        my %data;
        $data{section} = "Storage System";
        $data{"component_unit"} = $local;
        $data{"status"} = $status;
        push(@health_status,\%data);
        $status_global = "error";
        $message_global = "Storage System $local $status";
      }
    }
  }
  else {
    $active_alert_cache = -1;
  }

  if ( defined $data->{chbs} && ref( $data->{chbs} ) eq "ARRAY" ) {
    foreach my $channel ( @{ $data->{chbs} } ) {
      my $local  = $channel->{location};
      my $status = $channel->{status};
      if ( !defined $local || !defined $status ) { next; }
      if ( $status ne "Normal" ) {
        $active_alert_channel_board++;
        my %data;
        $data{section} = "Storage System";
        $data{"component_unit"} = $local;
        $data{"status"} = $status;
        push(@health_status,\%data);
        $status_global = "error";
        $message_global = "Storage System $local $status";
      }
    }
  }
  else {
    $active_alert_channel_board = -1;
  }
  if ( defined $data->{ctls} && ref( $data->{ctls} ) eq "ARRAY" ) {
    foreach my $node ( @{ $data->{ctls} } ) {
      my $local  = $node->{location};
      my $status = $node->{status};
      if ( !defined $local || !defined $status ) { next; }
      if ( $status ne "Normal" ) {
        $active_alert_node++;
        my %data;
        $data{section} = "Storage System";
        $data{"component_unit"} = $local;
        $data{"status"} = $status;
        push(@health_status,\%data);
        $status_global = "error";
        $message_global = "Storage System $local $status";
      }
    }
  }
  else {
    $active_alert_node = -1;
  }
  if ( defined $data->{dkbs} && ref( $data->{dkbs} ) eq "ARRAY" ) {
    foreach my $disk ( @{ $data->{dkbs} } ) {
      my $local  = $disk->{location};
      my $status = $disk->{status};
      if ( !defined $local || !defined $status ) { next; }
      if ( $status ne "Normal" ) {
        $active_alert_disk_board++;
        my %data;
        $data{section} = "Storage System";
        $data{"component_unit"} = $local;
        $data{"status"} = $status;
        push(@health_status,\%data);
        $status_global = "error";
        $message_global = "Storage System $local $status";
      }
    }
  }
  else {
    $active_alert_disk_board = -1;
  }
  if ( defined $data->{dkcpss} && ref( $data->{dkcpss} ) eq "ARRAY" ) {
    foreach my $disk ( @{ $data->{dkcpss} } ) {
      my $local  = $disk->{location};
      my $status = $disk->{status};
      if ( !defined $local || !defined $status ) { next; }
      if ( $status ne "Normal" ) {
        $active_alert_power_supply++;
        my %data;
        $data{section} = "Storage System";
        $data{"component_unit"} = $local;
        $data{"status"} = $status;
        push(@health_status,\%data);
        $status_global = "error";
        $message_global = "Storage System $local $status";
      }
    }
  }
  else {
    $active_alert_power_supply = -1;
  }
  if ( defined $data->{sfps} && ref( $data->{sfps} ) eq "ARRAY" ) {
    foreach my $sfp ( @{ $data->{sfps} } ) {
      my $local  = $sfp->{location};
      my $status = $sfp->{status};
      if ( !defined $local || !defined $status ) { next; }
      if ( $status ne "Normal" ) {
        $active_alert_sfp++;
        my %data;
        $data{section} = "Storage System";
        $data{"component_unit"} = $local;
        $data{"status"} = $status;
        push(@health_status,\%data);
        $status_global = "error";
        $message_global = "Storage System $local $status";
      }
    }
  }
  else {
    $active_alert_sfp = -1;
  }
  if ( $active_alert_drive == 0 ) {
    my %data;
    $data{section} = "Disk Unit";
    $data{"component_unit"} = "Drives";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  if ( $active_alert_ps == 0 ) {
    my %data;
    $data{section} = "Disk Unit";
    $data{"component_unit"} = "Power supplies";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  if ( $active_alert_enc == 0 ) {
    my %data;
    $data{section} = "Disk Unit";
    $data{"component_unit"} = "ENC";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  if ( $active_alert_fan == 0 ) {
    my %data;
    $data{section} = "Storage System";
    $data{"component_unit"} = "Fans";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  if ( $active_alert_backup_module == 0 ) {
    my %data;
    $data{section} = "Storage System";
    $data{"component_unit"} = "Backup Modules";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  if ( $active_alert_battery == 0 ) {
    my %data;
    $data{section} = "Storage System";
    $data{"component_unit"} = "Batteries";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  if ( $active_alert_cache == 0 ) {
    my %data;
    $data{section} = "Storage System";
    $data{"component_unit"} = "Cache";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  if ( $active_alert_channel_board == 0 ) {
    my %data;
    $data{section} = "Storage System";
    $data{"component_unit"} = "Channel Board";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  if ( $active_alert_node == 0 ) {
    my %data;
    $data{section} = "Storage System";
    $data{"component_unit"} = "Controllers";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  if ( $active_alert_disk_board == 0 ) {
    my %data;
    $data{section} = "Storage System";
    $data{"component_unit"} = "Disk Board";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  if ( $active_alert_power_supply == 0 ) {
    my %data;
    $data{section} = "Storage System";
    $data{"component_unit"} = "Power supplies";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  if ( $active_alert_sfp == 0 ) {
    my %data;
    $data{section} = "Storage System";
    $data{"component_unit"} = "SFP";
    $data{"status"} = "Normal";
    push(@health_status,\%data);
  }
  my %vm_status = (
    "item_id" => $st_serial,
    "status" => $status_global,
    "data" => $message_global
  );
  push(@hs_global, \%vm_status);
  #my $rc = $xormon->saveStatus(\@hs_global);
  #my $status = get_status($rc, "health status");
  #$logger->info("Task healh status ".$cfg->{hostalias}." is done. Status: $status");
  return;
}


sub get_timezone{
  my $cmd = "v1/objects/date-times/instance";
  my $data  = cmd_api($cmd, "GET", "timezone", 1, "");
  if (!defined $data || $data eq ""){return "";}
  if (!defined $data->{"timeZoneId"}){return "";}
  return $data->{"timeZoneId"};

}
sub get_offset_time{
  my $timezone_act = get_timezone();
  if ($timezone_act eq ""){return "";}
  my $data = `export TZ=":$timezone_act";date +%z;echo \$?`;
  my ($offset, $code) = split("\n",$data);
  if ($code == 0){
    return $offset;
  }

  my $cmd = "v1/objects/time-zones";
  $data  = cmd_api($cmd, "GET", "timezones", 1, "");
  if ( $data ne "" && defined $data->{data} && ref( $data->{data} ) eq "ARRAY" ) {
    foreach my $timezone (@{$data->{data}}){
      if (defined $timezone->{"timeZoneId"} && $timezone->{"timeZoneId"} eq $timezone_act && defined $timezone->{"timeZone"}){
        my $offset_time = $timezone->{"timeZone"};
        $offset_time =~ s/\(//g;
        $offset_time =~ s/\)//g;
        $offset_time =~ s/UTC//g;
        $offset_time =~ s/://g;
        return $offset_time;
      }
    }
  }
  return "";
}

sub set_data_device_conf{
  my $cmd          = "v1/objects/storages";
  my $data         = cmd_api($cmd, "GET", "storages", 1, "");
  if ( $data ne "" && defined $data->{data} && ref( $data->{data} ) eq "ARRAY" ) {
    foreach my $element ( @{ $data->{data} } ) {
      my $model     = $element->{model};
      my $ipa       = $element->{ctl1Ip};
      my $ipb       = $element->{ctl2Ip};
      my $serial    = $element->{serialNumber};
      my $device_id = $element->{storageDeviceId};
      if ( $cfg->{'exportRadio'} eq "manager" && $device_id ne $cfg->{'hcmDeviceId'}) { next; }
      if ( !defined $model )     { $model     = ""; }
      if ( !defined $ipa )       { $ipa       = ""; }
      if ( !defined $ipb )       { $ipb       = ""; }
      if ( !defined $serial )    { $serial    = ""; }
      if ( !defined $device_id ) { $device_id = ""; }
      $cfg->{serial} = $serial;
      if ($ipa eq ""){
        $ipa = $cfg->{node1Host};
      }
      if ($ipb eq "" && defined $cfg->{node2Host}){
        $ipb = $cfg->{node2Host};
      }
      $cfg->{"ctla"} = $ipa;
      $cfg->{"ctlb"} = $ipb;
      $cfg->{"device_id"} = $device_id;
      $cfg->{"machine_name"} = $storage_name;
      $cfg->{"model"} = $model;
      my $cmd                 = "v1/objects/storages/$device_id";
      my $export_tool_version = cmd_api($cmd, "GET", "storages-$device_id", 1,"");

      if ( $export_tool_version ne "" && defined $export_tool_version->{dkcMicroVersion} ) {
        $cfg->{"version"} = $export_tool_version->{dkcMicroVersion};
      }
    }
  }
  $cmd  = "v1/objects/snmp-settings/instance";
  $data = cmd_api($cmd, "GET", "instance", 1, "");
  if ( $data ne "" && defined $data->{systemGroupInformation} && ref( $data->{systemGroupInformation} ) eq "HASH" ) {
    my $storage_name = $data->{systemGroupInformation}->{storageSystemName};
    if (defined $storage_name){
      $cfg->{"machine_name"} = $storage_name;
    }
  }
  #print Dumper \$cfg;
}

sub set_data_device{
  my $type = "pool";
  my $section = "device";
  my @items = ("read_io","write_io","read","write", "resp_t_r" ,"resp_t_w","io_rate", "data_rate", "resp_t","capacity_total", "capacity_used", "capacity_free", "virtual_capacity_total");
  foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$type}{$uid}}){
      foreach my $item (@items){
        my $value = $performance_data{$st_type}{$type}{$uid}{$timestamp}{$item};
        my $io_rate = $performance_data{$st_type}{$type}{$uid}{$timestamp}{io_rate};
        if ($item eq "resp_t_r"){
          if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$timestamp}{read_io})){
            $io_rate = $performance_data{$st_type}{$type}{$uid}{$timestamp}{read_io};
          }
          else{
            undef $io_rate;
          }
        }
        if ($item eq "resp_t_w"){
          if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$timestamp}{write_io})){
            $io_rate = $performance_data{$st_type}{$type}{$uid}{$timestamp}{write_io};
          }
          else{
            undef $io_rate;
          }
        }
        if (Xormon::isdigit($value)){
          if ($item =~ m/resp/){
            if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$st_serial} && defined $performance_data{$st_type}{$section}{$st_serial}{$timestamp} && defined $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{"numerator_$item"}){
              if (Xormon::isdigit($io_rate)){
                $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{"numerator_$item"} = $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{"numerator_$item"} + ($value * $io_rate);
                $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{"denominator_$item"} = $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{"denominator_$item"} + $io_rate;
              }
            }
            else{
              if (Xormon::isdigit($io_rate)){
                $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{"numerator_$item"} = $value * $io_rate;
                $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{"denominator_$item"} = $io_rate;
              }
            }
            next;
          }
          if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$st_serial} && defined $performance_data{$st_type}{$section}{$st_serial}{$timestamp} && defined $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item}){
            $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item} = $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item} + $value;
          }
          else{
            $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item} = $value;
          }
        }
      }
    }
  }
  @items = ("resp_t_r", "resp_t_w", "resp_t");
  foreach my $id (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$id}}){
      foreach my $item (@items){
        my $numerator = $performance_data{$st_type}{$section}{$id}{$timestamp}{"numerator_$item"};
        my $denominator = $performance_data{$st_type}{$section}{$id}{$timestamp}{"denominator_$item"};
        if (Xormon::isdigit($numerator) && Xormon::isdigit($denominator)){
          if ($denominator == 0){
            $performance_data{$st_type}{$section}{$id}{$timestamp}{$item} = 0;
          }
          else{
            $performance_data{$st_type}{$section}{$id}{$timestamp}{$item} = round($numerator / $denominator,3);
          }
        }
        if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{"numerator_$item"}){
          delete $performance_data{$st_type}{$section}{$id}{$timestamp}{"numerator_$item"};
        }
        if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{"denominator_$item"}){
          delete $performance_data{$st_type}{$section}{$id}{$timestamp}{"denominator_$item"};
        }
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}

sub get_status {
  my $rc = shift;
  my $task = shift;
  if ($rc eq "201" || $rc eq "200") {
    $logger->info("Task for $task ".$cfg->{hostalias}." is done. Status: successful ($rc)");
    return "successful";
  } elsif ($rc eq "401") {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: unauthorized ($rc)");
    return "unauthorized";
  } else {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: error ($rc)");
    return "error";
  }
}

sub set_data_node_conf{
  my $section = "node";
  my $cmd = "v1/objects/components/instance";
  my $data = cmd_api($cmd,"GET","nodes",1,"");
  if ($data eq "" || ref($data->{ctls}) ne "ARRAY"){return;}
  foreach my $node (@{$data->{ctls}}){
    my $local       = $node->{location};
    if (!defined $local || $local eq "" ){next;}
    my $uid = Xormon::create_hash_key($st_serial.$section.$local);
    $configuration_data{$uid}{id} = $local;
    $configuration_data{$uid}{label} = $local;
    $configuration_data{$uid}{TYPE} = $section;

  }
  #print Dumper \$performance_data{$st_type}{$section};
}


sub set_data_pool_conf{

  my $section     = "pool";
  my $cmd = "v1/objects/pools";
  my $data = cmd_api($cmd,"GET","pools",1, "");
  my %data = get_data_pool_capacity(1);
  foreach my $pool (keys %data){
    foreach my $item (keys %{$data{$pool}}){
      $configuration_data{$pool}{$item} = $data{$pool}{$item};
    }
    #print Dumper \$configuration_data{$pool};
  }
  my $pool_uid = Xormon::create_hash_key($st_serial.$section."-1");
  if (defined $performance_data{$st_type}{$section}{$pool_uid}){
    $configuration_data{$pool_uid}{id} = -1;
    $configuration_data{$pool_uid}{label} = "GAD_unknown";
    $configuration_data{$pool_uid}{TYPE} = $section;
  }

}


sub set_data_host_conf{
  my $section = "host";
  my %host_groups;
  my $cmd        = "v1/objects/host-groups";
  my $data       = cmd_api($cmd,"GET","host_groups",1,"");
  my $count_query = 0;
  if ( $data eq "" || ref( $data->{data} ) ne "ARRAY" ) {return;}
  foreach my $host_group ( @{ $data->{data} } ) {
    if ( defined $host_group->{hostGroupId} ) {
      my ( $port, $id ) = split( ",", $host_group->{hostGroupId} );
      if ( defined $port && defined $id ) {
        my $hosts = cmd_api("v1/objects/host-wwns?portId=$port&hostGroupNumber=$id", "GET", "v1/objects/host-wwns--portId-$port-hostGroupNumber-$id", 1, "" );
        $count_query++;
        if ($count_query == 60){
          cmd_api("v1/objects/sessions/$session_id", "DELETE", "v1/objects/sessions",1, "", 0);
          set_token_and_session_id();
          $count_query = 0;
          ## logout and login
        }
        if ($hosts eq "" || ref( $hosts->{data} ) ne "ARRAY"){next;}
        my $host_group_alias = 0;
        foreach my $host (@{$hosts->{data}}){
          my $host_group_name = $host->{hostGroupName};
          my $host_wwn = $host->{hostWwn};
          my $host_name = $host->{wwnNickname};
          if (!defined $host_group_name || $host_group_name eq "" || !defined $host_wwn || $host_wwn eq ""){next;}
          
          $host_wwn = uc($host_wwn);
          if (!defined $host_name || $host_name eq "" || $host_name eq "-"){
            $host_name = $host_group_name;
            $host_group_alias = 1;
          }
          
          my $uid = Xormon::create_hash_key($st_serial.$section.$host_wwn);
          if (defined $configuration_data{$uid} && $configuration_data{$uid}{host_group_alias} == 0 && $host_group_alias == 1){
            $host_name = $configuration_data{$uid}{label}; # fix a bug (host group name must not overwrite host name)
            $host_group_alias = 0;
          }
          $configuration_data{$uid}{wwn} = $host_wwn;
          $configuration_data{$uid}{label} = $host_name;
          $configuration_data{$uid}{host_group_name} = $host_group_name;
          $configuration_data{$uid}{TYPE} = $section;
          $configuration_data{$uid}{host_group_alias} = $host_group_alias;
        }
        #print Dumper \$hosts;
      }
    }
  }
  #print Dumper \%volumes;
  foreach my $volume (keys %volumes){
    if (!defined $configuration_data{$volume} || !defined $performance_data{$st_type}{volume}{$volume}){
      next;
    }
    foreach my $hs_group (@{$volumes{$volume}{"host_group"}}){
      foreach my $uid (keys %configuration_data){
        if (!defined $configuration_data{$uid}{host_group_name}){next;}
        if ($hs_group eq $configuration_data{$uid}{host_group_name} && !grep( /^$volume$/, @{$configuration_data{$uid}{children}})){
          push(@{$configuration_data{$uid}{children}},$volume);
        }
      }
    }
  }
  #print Dumper \%configuration_data;
}

sub set_data_volume_conf{
  my $section = "volume";
  my $data    = cmd_api("v1/objects/ldevs?count=16384&ldevOption=defined", "GET", "ldevs", 1, "");
  if ($data eq "" || ref($data->{data}) ne "ARRAY"){return;}

  ### wwn volume
  if ( $prefix_uuid_volume ne "" ) {
    chop($prefix_uuid_volume);
    chop($prefix_uuid_volume);
  }
  if ($data eq ""){
    $data = undef;
    $data->{data} = ();
  }
  foreach my $volume ( @{ $data->{data} } ) {
    my $type = $volume->{emulationType};
    if ( defined $type && $type eq "NOT DEFINED" ) { next; }
    my $id = $volume->{ldevId};
    if ( !defined $id ) { next; }
    my $pool_volume = $volume->{attributes};
    my $hex         = sprintf( "0x%X", $id );
    $hex =~ s/x//;
    my $len = length($hex);
    if ( $len < 6 ) {
      my $add_null = 6 - $len;
      for ( my $i = 0; $i < $add_null; $i++ ) {
        $hex = "0$hex";
      }
    }
    my @chars = split( "", $hex );
    $hex = "$chars[0]$chars[1]:$chars[2]$chars[3]:$chars[4]$chars[5]";
    $id  = $hex;

    my $uuid  = $id;
    my $uuid2 = $id;
    $uuid  =~ s/://g;
    $uuid2 =~ s/://g;
    $uuid  =~ s/^0*//g;
    if ( $uuid eq "" ) {
      $uuid = "0";
    }
    $uuid = uc( $prefix_uuid_volume . $uuid );
    if ( defined $device_uid_prefix && $device_uid_prefix ne "" ) {
      $uuid = uc( $device_uid_prefix . $uuid2 );
    }

    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    my $pool_id = $volume->{poolId};
    if ( !defined $pool_id || $pool_id eq "") { $pool_id = "-1"; }
    my $capacity     = $volume->{blockCapacity};
    my $label = $volume->{label};
    if (!defined $label || $label eq ""){$label = $id;}
    if ($id ne $label){
      $label = "$id $label";
    }
    my $raw_capacity = "";
    if ( !Xormon::isdigit($capacity) ) { $capacity = ""; }
    else {
      $raw_capacity = $capacity;
      $capacity     = ( $capacity * 512 );
    }
    my $used_capacity = $volume->{numOfUsedBlock};
    if ( !Xormon::isdigit($used_capacity) ) { $used_capacity = ""; }
    else {
      $used_capacity = ( $used_capacity * 512 );
    }
    my $ports = $volume->{ports};
    if (defined $ports && ref($ports) eq "ARRAY" ){
      foreach my $host_group (@{$ports}){
        my $name = $host_group->{hostGroupName};
        if (defined $name && $name ne ""){
          push(@{$volumes{$uid}{host_group}},$name);
        }
      }
    }
    ####
    $configuration_data{$uid}{label} = $label;
    $configuration_data{$uid}{capacity_used} = $used_capacity;
    $configuration_data{$uid}{capacity_total} = $capacity;
    $configuration_data{$uid}{id} = $id;
    $configuration_data{$uid}{TYPE} = $section;
    $configuration_data{$uid}{wwn} = $uuid;
    $configuration_data{$uid}{"lun_id"} = $uuid;
    my $pool_uid = Xormon::create_hash_key($st_serial."pool".$pool_id);
    if (defined $performance_data{$st_type}{pool}{$pool_uid}){
      push(@{$configuration_data{$uid}{parents}},$pool_uid);
      $configuration_data{$uid}{pool_id} = $pool_id;
    }
    if (Xormon::isdigit($used_capacity) && Xormon::isdigit($capacity)){
      $configuration_data{$uid}{capacity_free} = $capacity - $used_capacity;
      if ($configuration_data{$uid}{capacity_free} < 0){
        $configuration_data{$uid}{capacity_free} = 0;
        $used_capacity = $capacity;
      }
    }
    ####
  }
  ### GAD disks -> volumes is not in volumes configuration
  my $pool_uid = Xormon::create_hash_key($st_serial."pool"."-1");
  foreach my $volume (keys %{$performance_data{$st_type}{$section}}){
    if (!defined $configuration_data{$volume} && defined $volumes_gad{$volume}){
      $configuration_data{$volume}{TYPE} = $section;
      $configuration_data{$volume}{id} = $volumes_gad{$volume}{id};
      $configuration_data{$volume}{label} = $volumes_gad{$volume}{id};
      $configuration_data{$volume}{wwn} = $volume;
      if (defined $performance_data{$st_type}{pool}{$pool_uid}){
        $configuration_data{$volume}{pool_id} = -1;
        push(@{$configuration_data{$volume}{parents}},$pool_uid);
      }
    }
  }
  #print Dumper \%configuration_data;
}

sub set_data_node{
  my $section = "node";
  my $section_port = "port";
  my $cmd = "v1/objects/components/instance";
  my $data = cmd_api($cmd,"GET","nodes",0,"");
  if ($data eq "" || ref($data->{ctls}) ne "ARRAY"){return;}
  foreach my $node (@{$data->{ctls}}){
    my $local       = $node->{location};
    my $temperature = $node->{temperature};
    if (!defined $local || $local eq "" || !Xormon::isdigit($temperature)){next;}
    my $uid = Xormon::create_hash_key($st_serial.$section.$local);
    foreach my $port_id (keys %{$performance_data{$st_type}{$section_port}}){
      foreach my $timestamp (keys %{$performance_data{$st_type}{$section_port}{$port_id}}){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{"temperature"} = $temperature;
      }
      last;
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}

sub get_data_tier_raw_capacity{
  my $data = shift;
  my %data;
  my %convert_metrics;
  $convert_metrics{DATA} = "raw_data_capacity_used";
  $convert_metrics{SPARE} = "spare_capacity_total";
  $convert_metrics{FREE} = "raw_capacity_free";
  $convert_metrics{USED} = "raw_capacity_used";
  $convert_metrics{TOTAL} = "raw_capacity_total";
  my %disk_check;
  if ($data eq ""){
    $data = undef;
    $data->{data} = ();
  }
  foreach my $drive ( @{ $data->{data} } ) {
      my $type   = $drive->{driveTypeName};
      my $usage  = $drive->{usageType};
      my $group  = $drive->{parityGroupId};
      my $speed  = $drive->{driveSpeed};
      if (!defined $speed){$speed = "";}
      my $group_uid;
      if (defined $group ){
        $group_uid = Xormon::create_hash_key($st_serial."raid_group".$group);
      }
      #my $status = $drive->{status};
      #my $id     = $drive->{driveLocationId};
      my $cap    = $drive->{totalCapacity};
      if ( !defined $type || $type eq "" || !defined $usage || $usage eq "" || !Xormon::isdigit($cap)) { next; }
      $cap = $cap * 1000000000;
      $type = convert_type($type,$speed);
      if ($type eq " "){next;}
      if (defined $convert_metrics{$usage}){
        $usage = $convert_metrics{$usage};
      }
      if (Xormon::isdigit($data{$type}{$usage})){
        $data{$type}{$usage} += $cap;
      }
      else{
        $data{$type}{$usage} = $cap;
      }
      if (defined $group_uid && !defined $disk_check{$group_uid} && defined $performance_data{$st_type}{"raid_group"}{$group_uid}){
        foreach my $timestamp (keys %{$performance_data{$st_type}{"raid_group"}{$group_uid}}){
          my $count_disk = $performance_data{$st_type}{"raid_group"}{$group_uid}{$timestamp}{count_disk};
          if (defined $count_disk){
            if (!grep( /^$group_uid$/, @{$data{$type}{GROUPS}} )){
                push(@{$data{$type}{GROUPS}},$group_uid);
            }
            if (Xormon::isdigit($data{$type}{"physical_capacity_total"})){
              $data{$type}{"physical_capacity_total"} = $data{$type}{"physical_capacity_total"} + ($cap * $count_disk);
              $disk_check{$group_uid} = $group_uid;
              last;
            }
            else{
              $data{$type}{"physical_capacity_total"} = $cap * $count_disk;
              $disk_check{$group_uid} = $group_uid;
              last;
            }
          }
        }
      }
      #if (defined $group && $group ne ""){
      #  if ( !grep { $group eq $_ } @{ $data{$type}{"GROUPS"} } ) {
      #    push( @{ $data{$type}{"GROUPS"} }, $group );
      #  }
      #}
      #if ( defined $id && defined $status && $status !~ m/NML/ ) {
      #  my $state = get_status_drive($status);
      #  $disks_status{$id} = $state;
      #}
    }
    #print Dumper \%data;
    my @keys = ("raw_data_capacity_used", "raw_capacity_free", "spare_capacity_total");
    foreach my $type (keys %data){
      foreach my $key (@keys){
        if (!Xormon::isdigit($data{$type}{$key})){
          $data{$type}{$key} = 0;
        }
        if (Xormon::isdigit($data{$type}{"raw_capacity_total"})){
          $data{$type}{"raw_capacity_total"} += $data{$type}{$key};
        }
        else{
          $data{$type}{"raw_capacity_total"} = $data{$type}{$key};
        }
      }
      if (Xormon::isdigit($data{$type}{"raw_capacity_total"})){
        $data{$type}{"raw_capacity_used"} = $data{$type}{"raw_capacity_total"} - $data{$type}{"raw_capacity_free"};
      }
    }

    #print Dumper \%data;
    return %data;

}

sub convert_type{
  my $type = shift;
  my $speed = shift;
  if ( $type =~ m/SAS/ && defined $speed && $speed =~ m/10000/ ) { $type = "SAS(10K)"; }
  if ( $type =~ m/SAS/ && defined $speed && $speed =~ m/15000/ ) { $type = "SAS(15K)"; }
  if ( $type =~ m/SAS/ && defined $speed && $speed =~ m/7200/ )  { $type = "SAS(NL)"; }
  return $type;

}

sub get_data_tier_format_capacity{
  my $data_tmp = shift;
  my %data = %{$data_tmp};
  my $section = "raid_group";
  my $section_tier = "tier";
  my %data_out;

  my %pool_volume = get_pool_volume();

  foreach my $uid (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$uid}}){
      my $capacity_total = $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_total};
      my $type = $performance_data{$st_type}{$section}{$uid}{$timestamp}{type};
      if (!defined $type || !Xormon::isdigit($capacity_total)){next;}
      if (!defined $type || $type eq "SAS"){
        foreach my $tier (keys %data){
          if (defined $data{$tier}{GROUPS} && grep( /^$uid$/, @{$data{$tier}{GROUPS}} )){
            $type = $tier;
            last;
          }
        }
      }
      if (defined $data{$type}){
        if (Xormon::isdigit($data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_total})){
          $data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_total} += $capacity_total;
        }
        else{
          $data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_total} = $capacity_total;
        }
        foreach my $pool_volume_uid (keys %pool_volume){
          if (!defined $pool_volume{$pool_volume_uid}{raid_group}){next;}
          if (!Xormon::isdigit($pool_volume{$pool_volume_uid}{"capacity_total"})){next;}
          if (grep( /^$uid$/, @{$pool_volume{$pool_volume_uid}{raid_group}} )){
            if (Xormon::isdigit($data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_used})){
              $data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_used} += $pool_volume{$pool_volume_uid}{"capacity_total"};
            }
            else{
              $data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_used} = $pool_volume{$pool_volume_uid}{"capacity_total"};
            }
          }
        }
      }
      if (Xormon::isdigit($data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_total}) && Xormon::isdigit($data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_used})){
        my $format_capacity_free = $data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_total} - $data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_used};
        if ($format_capacity_free < 0){
          $data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_used} = $data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_total};
          $data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_free} = 0;
        }
        else{
          $data_out{$st_type}{$section_tier}{$type}{$timestamp}{capacity_free} = $format_capacity_free;
        }
      }
    }
  }
  #print Dumper \%pool_volume;


  return %data_out;
}

sub get_pool_volume{

  my $data    = cmd_api("v1/objects/ldevs?count=16384&ldevOption=defined", "GET", "ldevs", 0, "");
  my %data;
  my @data_pool_volume_table = ();
  my @data_hdp_volume_table = ();
  if ( $data ne "" && defined $data->{data} && ref( $data->{data} ) eq "ARRAY" ) {
    foreach my $volume ( @{ $data->{data} } ) {
      my $pool_volume_active = 0;
      my $hdp_volume_active = 0;
      my %data_hdp;
      my $type = $volume->{emulationType};
      if ( defined $type && $type eq "NOT DEFINED" ) { next; }
      my $id = $volume->{ldevId};
      if ( !defined $id ) { next; }
      my $attributes = "";
      #print Dumper \$volume;
      my $pool_volume = $volume->{attributes};
      my $raid_group = $volume->{parityGroupIds};
      if (defined $pool_volume && ref($pool_volume) eq "ARRAY"){
        foreach my $atribute (@{$pool_volume}){
          if ($attributes ne ""){
            $attributes = $attributes . " " . $atribute;
          }
          else{
            $attributes = $atribute;
          }
          if ($atribute eq "POOL"){
            $pool_volume_active = 1;
            #last;
          }
          if ($atribute eq "HDP"){
            $hdp_volume_active = 1;
          }
        }
      }
      if (defined $pool_volume && ref($pool_volume) ne "ARRAY"){
        if ($pool_volume eq "POOL"){
          $pool_volume_active = 1;
        }
      }
      if (!($pool_volume_active || $hdp_volume_active)){next;}
      #print Dumper \$volume;
      my $uid = get_volume_uid($id);
      my $capacity     = $volume->{blockCapacity};
      my $label = $volume->{label};
      my $pool_id;
      if (defined $volume->{composingPoolId}){
        $pool_id = $volume->{composingPoolId};
      }
      else{
        $pool_id = $volume->{poolId};
      }
      my $drive_type = $volume->{driveType};
      my $raid_type = $volume->{raidType};
      my $raid_level = $volume->{raidLevel};
      my $pool_snap = $volume->{snapshotPoolId};
      my $used = $volume->{numOfUsedBlock};
      my $snap_used = $volume->{usedTotalCapacityForSnapshot};
      if (!Xormon::isdigit($used)){
        $used = "";
      }
      else{
        $used = $used * 512;
      }
      if (!Xormon::isdigit($snap_used)){
        $snap_used = "";
      }
      else{
        $snap_used = $snap_used * (1024 * 1024 * 1024);
      }

      if (!defined $pool_id){$pool_id = "";}
      else{
        my $pool_uid = Xormon::create_hash_key($st_serial."pool".$pool_id);
        if (defined $configuration_data{$pool_uid} && $configuration_data{$pool_uid}{label}){
          $pool_id = $configuration_data{$pool_uid}{label};
        }
      }
      if (!defined $pool_snap){$pool_snap = "";}
      else{
        my $pool_snap_uid = Xormon::create_hash_key($st_serial."pool".$pool_snap);
        if (defined $configuration_data{$pool_snap_uid} && $configuration_data{$pool_snap_uid}{label}){
          $pool_snap = $configuration_data{$pool_snap_uid}{label};
        }
      }
      if (!defined $drive_type){$drive_type = "";}
      if (!defined $raid_level){$raid_level = "";}
      if (!Xormon::isdigit($capacity)){
        $capacity = "";
      }
      else{$capacity = $capacity * 512;}
      if ($pool_volume_active){
        $data{$uid}{"capacity_total"} = $capacity;
        $data{$uid}{id} = get_volume_id($id);
      }
      if (!defined $label ){$label = "";}
      else{
        if ($pool_volume_active){
          $label =~ s/$data{$uid}{id} //g;
        }
        if ($hdp_volume_active){
          $data_hdp{$uid}{id} = get_volume_id($id);
          $label =~ s/$data_hdp{$uid}{id} //g;
        }
      }
      if ($pool_volume_active){
        $data{$uid}{label} = $label;
        $data{$uid}{pool} = $pool_id;
        $data{$uid}{drive_type} = $drive_type;
        $data{$uid}{raid_level} = $raid_level;
        $data{$uid}{raid_type} = $raid_type;
        $data{$uid}{attributes} = $attributes;
      }

      if (defined $raid_group){
        foreach my $rg (@{$raid_group}){
          if (defined $data{$uid}{raid_groups}){
            if ($pool_volume_active){
              $data{$uid}{raid_groups} = $data{$uid}{raid_groups} . " " . $rg;
            }
          }
          else{
            if ($pool_volume_active){
              $data{$uid}{raid_groups} = $rg;
            }
          }
          my $rg_uid = Xormon::create_hash_key($st_serial."raid_group".$rg);
          if ($pool_volume_active){
            push(@{$data{$uid}{raid_group}},$rg_uid);
          }
        }
      }
      if ($hdp_volume_active){
        $data_hdp{$uid}{label} = $label;
        $data_hdp{$uid}{pool} = $pool_id;
        $data_hdp{$uid}{snap_pool} = $pool_snap;
        $data_hdp{$uid}{"capacity_total"} = $capacity;
        $data_hdp{$uid}{"capacity_used"} = $used;
        $data_hdp{$uid}{"snapshot_capacity_used"} = $snap_used;
        $data_hdp{$uid}{attributes} = $attributes;
        $data_hdp{$uid}{copy_of} = ""; ### then snapshot section
        if (Xormon::isdigit($capacity) && Xormon::isdigit($used)){
          $data_hdp{$uid}{"capacity_free"} = $capacity - $used;
          if ($capacity > 0){
            my $capacity_used_percent = round(($used / $capacity) * 100,0);
            $data_hdp{$uid}{"capacity_used_percent"} = $capacity_used_percent;
          }
        }
        else{
          $data_hdp{$uid}{"capacity_free"} = "";
          $data_hdp{$uid}{"capacity_used_percent"} = "";
        }
        #print Dumper \%data_hdp;
        push(@data_hdp_volume_table,$data_hdp{$uid});
      }
      if ($pool_volume_active){
        push(@data_pool_volume_table,$data{$uid});
      }
    }
  }
  $configuration_data{$cfg->{'uuid'}}{pool_volume} = encode_json(\@data_pool_volume_table);
  $configuration_data{$cfg->{'uuid'}}{hdp_volume} = encode_json(\@data_hdp_volume_table);

  return %data;
}

sub get_volume_id{
  my $id = shift;
  my $section = "volume";
  my $hex         = sprintf( "0x%X", $id );
  $hex =~ s/x//;
  my $len = length($hex);
  if ( $len < 6 ) {
    my $add_null = 6 - $len;
    for ( my $i = 0; $i < $add_null; $i++ ) {
      $hex = "0$hex";
    }
  }
  my @chars = split( "", $hex );
  $hex = "$chars[0]$chars[1]:$chars[2]$chars[3]:$chars[4]$chars[5]";
  return $hex;
}

sub get_volume_uid{
  my $id = shift;
  my $section = "volume";
  my $hex         = sprintf( "0x%X", $id );
  $hex =~ s/x//;
  my $len = length($hex);
  if ( $len < 6 ) {
    my $add_null = 6 - $len;
    for ( my $i = 0; $i < $add_null; $i++ ) {
      $hex = "0$hex";
    }
  }
  my @chars = split( "", $hex );
  $hex = "$chars[0]$chars[1]:$chars[2]$chars[3]:$chars[4]$chars[5]";
  my $uid = Xormon::create_hash_key($st_serial.$section.$hex);
  return $uid;
}

sub get_data_disks{
  my $data = shift;
  my @data_out;
  my @data_drive_types_out;
  my @search_items = ( "driveLocationId", "driveTypeName", "driveSpeed", "driveType", "parityGroupId", "totalCapacity", "usageType" );
  my @items_convert = ("location_id", "type", "rpm", "code", "parity_group", "capacity_total", "usage");
  my %drives_types;
  if ($data eq ""){
    $data = undef;
    $data->{data} = ();
  }
  foreach my $drive ( @{ $data->{data} } ) {
    my %data;
    my $index = -1;
    foreach my $item (@search_items){
      $index++;
      if (!defined $drive->{$item} ){
        $data{$items_convert[$index]} = "";
      }
      else{
        if ($item eq "totalCapacity" && Xormon::isdigit($drive->{$item})){
          $data{$items_convert[$index]} = $drive->{$item} * 1000000000;
        }
        else{
          $data{$items_convert[$index]} = $drive->{$item};
        }
      }
    }
    push(@data_out,\%data);
  }
  foreach my $element (@data_out){
    my $type = $element->{type};
    my $rpm = $element->{rpm};
    my $capacity = $element->{"capacity_total"};
    my $usage = $element->{usage};
    if ($usage eq ""){next;}
    $usage = lc($usage);
    if (defined $drives_types{"$type : $rpm : $capacity"}){
      if (Xormon::isdigit($drives_types{"$type : $rpm : $capacity"}{total_capacity})){
        $drives_types{"$type : $rpm : $capacity"}{total_capacity} += $capacity;
      }
      else{
        $drives_types{"$type : $rpm : $capacity"}{total_capacity} = $capacity;
      }
      if (Xormon::isdigit($drives_types{"$type : $rpm : $capacity"}{$usage})){
        $drives_types{"$type : $rpm : $capacity"}{$usage} += $capacity;
      }
      else{
        $drives_types{"$type : $rpm : $capacity"}{$usage} = $capacity;
      }
    }
    else{
      $drives_types{"$type : $rpm : $capacity"}{drive_type} = $type;
      $drives_types{"$type : $rpm : $capacity"}{rmp} = $rpm;
      $drives_types{"$type : $rpm : $capacity"}{drive_size} = $capacity;
      $drives_types{"$type : $rpm : $capacity"}{total_capacity} = $capacity;
      $drives_types{"$type : $rpm : $capacity"}{$usage} = $capacity;
    }
  }
  #print Dumper \%drives_types;
  #return @data_out;
  foreach my $key (keys %drives_types){
    my $drive_type = $drives_types{$key}{"drive_type"};
    my $rpm        = $drives_types{$key}{"rpm"};
    my $drive_size = $drives_types{$key}{"drive_size"};
    my $data       = $drives_types{$key}{data};
    my $spare      = $drives_types{$key}{spare};
    my $total      = $drives_types{$key}{"total_capacity"};
    my $free = "";
    if ( !defined $drive_type ) { $drive_type = ""; }
    if ( !defined $rpm )        { $rpm        = ""; }
    if ( !defined $drive_size ) { $drive_size = ""; }
    if ( !defined $data )       { $data       = ""; }
    if ( !defined $spare )      { $spare      = ""; }
    if ( !defined $total )      { $total      = ""; }
    if ( Xormon::isdigit($total) ) {
      if ( Xormon::isdigit($data) ) {
        $free = $total - $data;
        if ( Xormon::isdigit($spare) ) {
          $free = $free - $spare;
        }
      }
      elsif ( Xormon::isdigit($spare) ) {
        $free = $total - $spare;
      }
      else {
        $free = $total;
      }
    }
    my %data;
    $data{"drive_type"} = $drive_type;
    $data{rpm} = $rpm;
    $data{drive_size_capacity_total} = $drive_size;
    $data{capacity_total} = $total;
    $data{data} = $data;
    $data{spare} = $spare;
    $data{free} = $free;
    push(@data_drive_types_out,\%data);
  }
  #print Dumper \@data_drive_types_out;
  return (\@data_out, \@data_drive_types_out);
  #print Dumper \@data_out;

}


sub set_data_tier_conf{
  my $section = "tier";
  my $cmd = "v1/objects/drives";
  my %convert_metrics;
  my $data = cmd_api($cmd,"GET","drives",1, "");
  if ( $data eq "" || !defined $data->{data} || ref( $data->{data} ) ne "ARRAY" ) {return;}
  my %data_raw = get_data_tier_raw_capacity($data);
  my %data_format = get_data_tier_format_capacity(\%data_raw);
  foreach my $tier (keys %data_raw){
    my $uid = Xormon::create_hash_key($st_serial.$section.$tier);
    foreach my $item (keys %{$data_raw{$tier}}){
      if ($item eq "GROUPS"){next;}
      $configuration_data{$uid}{$item} = $data_raw{$tier}{$item};
    }
    $configuration_data{$uid}{id} = $tier;
    $configuration_data{$uid}{label} = $tier;
    $configuration_data{$uid}{TYPE} = $section;

  }
  foreach my $tier (keys %{$data_format{$st_type}{$section}}){
    my $uid = Xormon::create_hash_key($st_serial.$section.$tier);
    if (!defined $configuration_data{$uid}){next;}
    foreach my $timestamp (keys %{$data_format{$st_type}{$section}{$tier}}){
      foreach my $item (keys %{$data_format{$st_type}{$section}{$tier}{$timestamp}}){
        $configuration_data{$uid}{$item} = $data_format{$st_type}{$section}{$tier}{$timestamp}{$item};
      }
      last;
    }
  }

  ### tables
  my ($disks, $drive_types) = get_data_disks($data);
  #print Dumper \@drive_types;
  $configuration_data{$cfg->{'uuid'}}{disks} = encode_json($disks);
  $configuration_data{$cfg->{'uuid'}}{drive_type} = encode_json($drive_types);




  #print Dumper \$configuration_data{e792bf3a60};
  #delete $configuration_data{e792bf3a60};
}

sub set_data_tier{

  my $section = "tier";
  my $cmd = "v1/objects/drives";
  my %convert_metrics;
  my $data = cmd_api($cmd,"GET","drives",0, "");
  if ( $data eq "" || !defined $data->{data} || ref( $data->{data} ) ne "ARRAY" ) {return;}
  my %data_raw = get_data_tier_raw_capacity($data);
  #print Dumper \%data_raw;
  my %data_format = get_data_tier_format_capacity(\%data_raw);
  #print Dumper \%data_format;
  foreach my $tier (keys %{$data_format{$st_type}{$section}}){
    my $uid = Xormon::create_hash_key($st_serial.$section.$tier);
    foreach my $timestamp (keys %{$data_format{$st_type}{$section}{$tier}}){
      foreach my $item (keys %{$data_format{$st_type}{$section}{$tier}{$timestamp}}){
        if ($item eq "GROUPS"){next;}
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{$item} = $data_format{$st_type}{$section}{$tier}{$timestamp}{$item};
      }
    }
  }
  foreach my $tier (keys %data_raw){
    my $uid = Xormon::create_hash_key($st_serial.$section.$tier);
    if (!defined $performance_data{$st_type}{$section}{$uid}){next;}
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$uid}}){
      foreach my $item (keys %{$data_raw{$tier}}){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{$item} = $data_raw{$tier}{$item};
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
  #delete $performance_data{$st_type}{$section};


  #print Dumper \%data_format;
  #print Dumper \%data_raw;
  #foreach my

}

sub get_percent_from_ratio{
  #% = ( ( ratio - 1 ) * 100 ) / ratio
  my $item = shift;
  my $data_tmp = shift;
  my %data = %{$data_tmp};
  my $ratio = $data{$item};
  my $percent = "";
  if (Xormon::isdigit($ratio) && $ratio > 0){
    $percent = round((($ratio - 1) * 100) / $ratio,0);
  }
  return $percent;
}

sub get_data_pool_capacity{
  my $run = shift;
  my $section = "pool";
  my $cmd = "v1/objects/pools";
  my $data = cmd_api($cmd,"GET","pools",$run, "");
  my %convert_metrics;
  my %data;
  $convert_metrics{totalPoolCapacity} = "capacity_total";
  $convert_metrics{availableVolumeCapacity} = "capacity_free";
  $convert_metrics{totalLocatedCapacity} = "virtual_capacity_total";
  $convert_metrics{totalPhysicalCapacity} = "physical_capacity_total";
  $convert_metrics{availablePhysicalVolumeCapacity} = "physical_capacity_free";
  $convert_metrics{dataReductionAccelerateCompRate} = "fmc_compression_saving_ratio";
  $convert_metrics{snapshotUsedCapacity} = "format_snapshot_capacity_used";
  $convert_metrics{totalReservedCapacity} = "format_reserved_capacity_used";
  $convert_metrics{capacitiesExcludingSystemData}{compressedCapacity} = "dkc_compression_saving";
  $convert_metrics{capacitiesExcludingSystemData}{dedupedCapacity} = "dkc_dedupe_saving";
  $convert_metrics{capacitiesExcludingSystemData}{reclaimedCapacity} = "dkc_zpr_saving";
  if ($run == 1){
    $convert_metrics{poolId} = "id";
    $convert_metrics{poolName} = "label";
  }
  if ($data eq "" || ref($data->{data}) ne "ARRAY"){return;}
  foreach my $pool (@{$data->{data}}){
    my $pool_id = $pool->{poolId};
    if (!defined $pool_id || $pool_id eq ""){next;}
    my $pool_uid = Xormon::create_hash_key($st_serial.$section.$pool_id);
    if (!defined $performance_data{$st_type}{$section}{$pool_uid}){next;}
    foreach my $item (keys %convert_metrics){
      if (defined $pool->{$item} && ref($pool->{$item}) ne "HASH" && $item =~ m/poolId|poolName/){
        $data{$pool_uid}{$convert_metrics{$item}} = $pool->{$item};
        next;
      }
      if (defined $pool->{$item} && ref($pool->{$item}) ne "HASH" && Xormon::isdigit($pool->{$item})){
        my $multiplier = 1048576;
        if ($convert_metrics{$item} =~ m/ratio/){
          $multiplier = 1;
        }
        if ($convert_metrics{$item} =~ m/saving/){
          $multiplier = 512;
        }
        $data{$pool_uid}{$convert_metrics{$item}} = $pool->{$item} * 1048576;
        next;
      }
      if (ref($pool->{$item}) eq "HASH"){
        foreach my $item2 (keys %{$convert_metrics{$item}}){
          if (Xormon::isdigit($pool->{$item}->{$item2})){
            $data{$pool_uid}{$convert_metrics{$item}{$item2}} = $pool->{$item}->{$item2} * 512;
          }
        }
      }
    }
  }
  undef %convert_metrics;
  $convert_metrics{fmcPoolVolumesCapacitySaving} = "fmc_compression_saving";

  $cmd = "v1/objects/pools?detailInfoType=FMC";
  $data = cmd_api($cmd,"GET","pools-fmc",$run, "");
  if ($data eq ""){
    $data = undef;
    $data->{data} = ();
  }
  foreach my $pool (@{$data->{data}}){
    my $pool_id = $pool->{poolId};
    if (!defined $pool_id || $pool_id eq ""){next;}
    my $pool_uid = Xormon::create_hash_key($st_serial.$section.$pool_id);
    if (!defined $performance_data{$st_type}{$section}{$pool_uid}){next;}
    foreach my $item (keys %convert_metrics){
      if (defined $pool->{$item} && ref($pool->{$item}) ne "HASH" && Xormon::isdigit($pool->{$item})){
        $data{$pool_uid}{$convert_metrics{$item}} = $pool->{$item} * 512;
        next;
      }
    }
  }

  undef %convert_metrics;
  $convert_metrics{efficiency}{dedupeAndCompression}{compressionRatio} = "dkc_compression_saving_ratio";
  $convert_metrics{efficiency}{dedupeAndCompression}{dedupeRatio} = "dkc_dedupe_saving_ratio";
  $convert_metrics{efficiency}{dedupeAndCompression}{reclaimRatio} = "dkc_zpr_saving_ratio";


  if ($run){
    $convert_metrics{efficiency}{dedupeAndCompression}{totalRatio} = "dkc_total_ratio";
    $convert_metrics{efficiency}{totalRatio} = "total_ratio";
    $convert_metrics{efficiency}{snapshotRatio} = "snapshot_ratio";
    $convert_metrics{efficiency}{compressionRatio} = "compression_ratio";
    $convert_metrics{efficiency}{acceleratedCompression}{totalRatio} = "accelerated_compression_total_ratio";
    $convert_metrics{efficiency}{acceleratedCompression}{reclaimRatio} = "accelerated_compression_reclaim_ratio";
    $convert_metrics{efficiency}{acceleratedCompression}{compressionRatio} = "accelerated_compression_ratio";
  }


  $cmd = "v1/objects/pools?detailInfoType=efficiency";
  $data = cmd_api($cmd,"GET","pools-efficiency",$run, "");
  if ($data eq ""){
    $data = undef;
    $data->{data} = ();
  }
  foreach my $pool (@{$data->{data}}){
    my $pool_id = $pool->{poolId};
    if (!defined $pool_id || $pool_id eq ""){next;}
    my $pool_uid = Xormon::create_hash_key($st_serial.$section.$pool_id);
    if (!defined $performance_data{$st_type}{$section}{$pool_uid}){next;}
    foreach my $item (keys %convert_metrics){
      if (defined $pool->{$item} && ref($pool->{$item}) ne "HASH" && Xormon::isdigit($pool->{$item})){
        $data{$pool_uid}{$convert_metrics{$item}} = $pool->{$item};
        next;
      }
      if (ref($pool->{$item}) eq "HASH"){
        foreach my $item2 (keys %{$convert_metrics{$item}}){
          if (ref($pool->{$item}->{$item2}) eq "HASH"){
            foreach my $item3 (keys %{$convert_metrics{$item}{$item2}}){
              if (Xormon::isdigit($pool->{$item}->{$item2}->{$item3})){
                $data{$pool_uid}{$convert_metrics{$item}{$item2}{$item3}} = $pool->{$item}->{$item2}->{$item3};
              }
            }
          }
          else{
            if (Xormon::isdigit($pool->{$item}->{$item2})){
              $data{$pool_uid}{$convert_metrics{$item}{$item2}} = $pool->{$item}->{$item2};
              next;
            }
          }
        }
      }
    }
  }

  foreach my $id (keys %data){
    $data{$id}{"capacity_used"} = get_difference_of_values("capacity_total", "capacity_free", $data{$id});
    $data{$id}{"physical_capacity_used"} = get_difference_of_values("physical_capacity_total", "physical_capacity_free", $data{$id});
    $data{$id}{"format_data_capacity_used"} = get_difference_of_values("capacity_used", "format_snapshot_capacity_used", $data{$id});
    $data{$id}{"format_data_capacity_used"} = get_difference_of_values("format_data_capacity_used", "format_reserved_capacity_used", $data{$id});
    $data{$id}{"capacity_used_percent"} = get_percent_used($data{$id},"capacity_total", "capacity_used");
    if ($run){
      $data{$id}{TYPE} = $section;
      $data{$id}{"physical_capacity_used_percent"} = get_percent_used($data{$id},"physical_capacity_total", "physical_capacity_used");
    }
    foreach my $item (keys %{$data{$id}}){
      if ($item =~ m/_ratio$/ && $item =~ m/_saving_ratio/){
        my $new_item = $item;
        $new_item =~ s/_saving_ratio/_percent/g;
        $data{$id}{$new_item} = get_percent_from_ratio($item,$data{$id});
        if (!$run){
          delete $data{$id}{$item};
        }
      }
    }
  }
  foreach my $id (keys %data){
    if ( Xormon::isdigit($data{$id}{capacity_total}) && Xormon::isdigit($data{$id}{virtual_capacity_total})){
      $data{$id}{"capacity_oversubscr"} = $data{$id}{"virtual_capacity_total"} - $data{$id}{"capacity_total"};
      if ($data{$id}{"capacity_oversubscr"} < 0){$data{$id}{"capacity_oversubscr"} = 0; $data{$id}{"capacity_oversubscr_percent"} = 0;}
      elsif ($data{$id}{"capacity_total"} != 0){
        $data{$id}{"capacity_oversubscr_percent"} = round(($data{$id}{"virtual_capacity_total"} / $data{$id}{"capacity_total"}) * 100,0);
      }
    }
  }
  ##print Dumper \%data;
  return %data;
}

sub get_percent_used{
  my $data_tmp = shift;
  my %data = %{$data_tmp};
  my $item1 = shift;
  my $item2 = shift;

  if (Xormon::isdigit($data{$item1}) && Xormon::isdigit($data{$item2}) && $data{$item1} > 0){
    return round(($data{$item2} / $data{$item1}) * 100,0);
  }
  return 0;

}

sub set_data_pool {

  #### pool stastistics from volumes
  my $section_volume = "volume";
  my $section     = "pool";
  my @items = ("io_rate","read_io","write_io","read","write","data_rate","resp_t_r","resp_t_w","resp_t");
  foreach my $uid (keys %{$performance_data{$st_type}{$section_volume}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section_volume}{$uid}}){
      foreach my $item (@items){
        my $value = $performance_data{$st_type}{$section_volume}{$uid}{$timestamp}{$item};
        my $io_rate = $performance_data{$st_type}{$section_volume}{$uid}{$timestamp}{io_rate};
        if ($item eq "resp_t_r"){
          if (Xormon::isdigit($performance_data{$st_type}{$section_volume}{$uid}{$timestamp}{read_io})){
            $io_rate = $performance_data{$st_type}{$section_volume}{$uid}{$timestamp}{read_io};
          }
          else{
            undef $io_rate;
          }
        }
        if ($item eq "resp_t_w"){
          if (Xormon::isdigit($performance_data{$st_type}{$section_volume}{$uid}{$timestamp}{write_io})){
            $io_rate = $performance_data{$st_type}{$section_volume}{$uid}{$timestamp}{write_io};
          }
          else{
            undef $io_rate;
          }
        }
        if (Xormon::isdigit($value)){
          if (defined $volumes{$uid} && defined $volumes{$uid}{pool}){
            my $pool_id = $volumes{$uid}{pool};
            my $pool_uid = Xormon::create_hash_key($st_serial.$section.$pool_id);
            if ($item =~ m/resp/){
              if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$pool_uid} && defined $performance_data{$st_type}{$section}{$pool_uid}{$timestamp} && defined $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{"numerator_$item"}){
                if (Xormon::isdigit($io_rate)){
                  $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{"numerator_$item"} = $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{"numerator_$item"} + ($value * $io_rate);
                  $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{"denominator_$item"} = $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{"denominator_$item"} + $io_rate;
                }
              }
              else{
                if (Xormon::isdigit($io_rate)){
                  $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{"numerator_$item"} = $value * $io_rate;
                  $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{"denominator_$item"} = $io_rate;
                }
              }
              next;
            }
            if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$pool_uid} && defined $performance_data{$st_type}{$section}{$pool_uid}{$timestamp} && defined $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{$item}){
              $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{$item} = $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{$item} + $value;
            }
            else{
              $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{$item} = $value;
            }
          }
        }
      }
    }
  }
  @items = ("resp_t_r", "resp_t_w", "resp_t");
  foreach my $id (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$id}}){
      foreach my $item (@items){
        my $numerator = $performance_data{$st_type}{$section}{$id}{$timestamp}{"numerator_$item"};
        my $denominator = $performance_data{$st_type}{$section}{$id}{$timestamp}{"denominator_$item"};
        if (Xormon::isdigit($numerator) && Xormon::isdigit($denominator)){
          if ($denominator == 0){
            $performance_data{$st_type}{$section}{$id}{$timestamp}{$item} = 0;
          }
          else{
            $performance_data{$st_type}{$section}{$id}{$timestamp}{$item} = round($numerator / $denominator,3);
          }
        }
        if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{"numerator_$item"}){
          delete $performance_data{$st_type}{$section}{$id}{$timestamp}{"numerator_$item"};
        }
        if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{"denominator_$item"}){
          delete $performance_data{$st_type}{$section}{$id}{$timestamp}{"denominator_$item"};
        }
      }
    }
  }
  ####
  #print Dumper \$performance_data{$st_type}{$section};
  my %data = get_data_pool_capacity(0);
  #print Dumper \%data;
  foreach my $pool_uid (keys %data){
    foreach my $item (keys %{$data{$pool_uid}}){
      foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$pool_uid}}){
        $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{$item} = $data{$pool_uid}{$item};
      }
    }
  }

  ###

  ### delete pools if no load
  @items = ("capacity_total", "capacity_free", "capacity_used", "io_rate", "data_rate");
  my %pools;
  foreach my $pool_uid (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$pool_uid}}){
      foreach my $item (@items){
        if (Xormon::isdigit($performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{$item}) && $performance_data{$st_type}{$section}{$pool_uid}{$timestamp}{$item} > 0){
          $pools{$pool_uid}{$timestamp}{$item} = 1;
          last;
        }
      }
    }
    if (!defined $pools{$pool_uid}){
      delete $performance_data{$st_type}{$section}{$pool_uid};
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}


sub set_data_cpu{
  my $section     = "cpu";
  my %datas;
  $datas{"PHY_MP.csv"} = "cpu_system";
  $datas{"PHY_Short_MP.csv"} = "cpu_system";
  $datas{"PHY_Short_DRR.csv"} = "cpu_system";
  foreach my $file (keys %datas){
    my $filename = "$performance_data_store/PhyProc_dat/$file";
    if (-f $filename){
      set_data($filename,$section,$datas{$file});
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}

sub set_data_cpu2{
  my $section = "cpu";
  my %datas;
  $datas{"MP/Utilization_Rate"} = "cpu_system";
  $datas{"Cache/CLPR/Write_Pending_Rate"} = "write_pending_percent";
  $datas{"Cache/CLPR/Usage_Rate"} = "cache_usage_percent";
  $datas{"Cache/CLPR/Used_Size"} = "cache_allocated_size";

  foreach my $file (keys %datas){
    my $filename = "$performance_data_store/$file.csv";
    if (-f $filename){
      set_data2($filename,$section,$datas{$file});
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};

}

sub set_data_cpu_conf{
  foreach my $cpu (keys %caches){
    if (defined $configuration_data{$cpu}){
      push(@{$configuration_data{$cpu}{parents}}, $caches{$cpu});
    }
  }
}

sub set_data_cache_type_conf{
  my $section = "cpu_type";
  #print Dumper \@cache_type;
  if ($export_tool_version == 2){
    foreach my $cache_type (@cache_type){
      my $uid = Xormon::create_hash_key($st_serial."cache".$cache_type);
      if (!defined $configuration_data{$uid} || !defined $configuration_data{$uid}{label}){next;}
      my ($label, undef) = split("-",$cache_type);
      if (!defined $label || $label eq ""){next;}
      my $uid_cpu_type = Xormon::create_hash_key($st_serial.$section.$label);
      $configuration_data{$uid_cpu_type}{label} = $label;
      $configuration_data{$uid_cpu_type}{id} = $label;
      $configuration_data{$uid_cpu_type}{TYPE} = $section;
      push(@{$configuration_data{$uid_cpu_type}{children}},$uid);

    }
    return;
  }

  foreach my $cache_type (@cache_type){
    if (!defined $configuration_data{$cache_type} || !defined $configuration_data{$cache_type}{label}){next;}
    my ($label, undef) = split("-",$configuration_data{$cache_type}{label});
    my $uid = Xormon::create_hash_key($st_serial.$section.$label);
    $configuration_data{$uid}{label} = $label;
    $configuration_data{$uid}{id} = $label;
    $configuration_data{$uid}{TYPE} = $section;
    push(@{$configuration_data{$uid}{children}},$cache_type);
    #print "$uid\n";
  }
}

sub set_data_cache2{
  my $section = "cache";
  my $section_cpu = "cpu";
  my %datas;
  $datas{"Usage_Rate"} = "cache_usage_percent";
  $datas{"Used_Size"} = "cache_allocated_size";
  $datas{"Write_Pending_Rate"} = "write_pending_percent";
  foreach my $metric (keys %datas){
    my $filename = "$performance_data_store/Cache/MPUnit/$metric.csv";
    if (-f $filename){
      set_data2($filename,$section,$datas{$metric});
    }
  }
  foreach my $cache (@cache_type){
    foreach my $uid (keys %{$performance_data{$st_type}{$section_cpu}}){
      #print "$cache $uid\n";
      #print Dumper \%configuration_data;
      if (!defined $configuration_data{$uid} || !defined $configuration_data{$uid}{label} || $configuration_data{$uid}{label} !~ m/^$cache/){next;}
      my $id = Xormon::create_hash_key($st_serial.$section.$cache);

      #cache configuration
      $configuration_data{$id}{id} = $cache;
      $configuration_data{$id}{label} = $cache;
      $configuration_data{$id}{TYPE} = "cache";
      #
      foreach my $timestmap (keys %{$performance_data{$st_type}{$section_cpu}{$uid}}){
        if (Xormon::isdigit($performance_data{$st_type}{$section_cpu}{$uid}{$timestmap}{"cpu_system"})){
          if (Xormon::isdigit($performance_data{$st_type}{$section}{$id}{$timestmap}{"cpu_system"})){
            $performance_data{$st_type}{$section}{$id}{$timestmap}{"cpu_system"} = $performance_data{$st_type}{$section}{$id}{$timestmap}{"cpu_system"} + $performance_data{$st_type}{$section_cpu}{$uid}{$timestmap}{"cpu_system"};
            $performance_data{$st_type}{$section}{$id}{$timestmap}{"count"}++;
          }
          else{
            $performance_data{$st_type}{$section}{$id}{$timestmap}{"cpu_system"} = $performance_data{$st_type}{$section_cpu}{$uid}{$timestmap}{"cpu_system"};
            $performance_data{$st_type}{$section}{$id}{$timestmap}{"count"} = 1;
          }
        }
      }
    }
  }
  foreach my $id (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$id}}){
      my $sys = $performance_data{$st_type}{$section}{$id}{$timestamp}{cpu_system};
      my $count = $performance_data{$st_type}{$section}{$id}{$timestamp}{count};
      if (Xormon::isdigit($sys) && Xormon::isdigit($count)){
        $performance_data{$st_type}{$section}{$id}{$timestamp}{cpu_system} = round($sys / $count,1);
        delete $performance_data{$st_type}{$section}{$id}{$timestamp}{count};
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
  #print Dumper \@cache_type;
}

sub set_data_raid_group2{
  my $section = "raid_group";
  my %datas;
  $datas{Random_Read_IOPS} = "rnd_read_io";
  $datas{Sequential_Read_IOPS} = "seq_read_io";
  $datas{Random_Write_IOPS} = "seq_write_io";
  $datas{Sequential_Write_IOPS} = "rnd_write_io";
  $datas{Random_Read_Transfer_Rate} = "rnd_read";
  $datas{Sequential_Read_Transfer_Rate} = "seq_read";
  $datas{Random_Write_Transfer_Rate} = "seq_write";
  $datas{Sequential_Write_Transfer_Rate} = "rnd_write";

  $datas{Read_Response_Time} = "resp_t_r";
  $datas{Write_Response_Time} = "resp_t_w";
  $datas{Total_Response_Time} = "resp_t";
  $datas{Backend_Transfer_Count_Total} = "data_rate_b";
  #$datas{PG_D2CS_Trans} = "D2CS";
  #$datas{PG_D2CR_Trans} = "D2CR";

  foreach my $element (keys %datas){
    my $file = "$performance_data_store/ParityGroup/Internal/$element.csv";
    if (! -f $file){next;}
    set_data2($file,$section,$datas{$element});
  }
  foreach my $uid (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$uid}}){
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"read_io"} = get_metrics_sum("seq_read_io","rnd_read_io","read_io",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_read_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_read_io"};

      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"write_io"} = get_metrics_sum("seq_write_io","rnd_write_io","write_io",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_write_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_write_io"};

      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"read"} = get_metrics_sum("seq_read","rnd_read","read",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_read"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_read"};

      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"write"} = get_metrics_sum("seq_write","rnd_write","write",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_write"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_write"};

      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"} = get_metrics_sum("read_io","write_io","io_rate",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      #if (!defined $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"} || (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"}) && $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"} == 0)){
      #  delete $performance_data{$st_type}{$section}{$uid}{$timestamp};
      #  next;
      #}
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"data_rate"} = get_metrics_sum("read","write","data_rate",$performance_data{$st_type}{$section}{$uid}{$timestamp});
    }
    #if (keys %{$performance_data{$st_type}{$section}{$uid} } == 0){
    #  delete $performance_data{$st_type}{$section}{$uid};
    #  next;
    #}
  }
  #if (keys %{$performance_data{$st_type}{$section} } == 0){
  #  delete $performance_data{$st_type}{$section};
  #  return;
  #}
  #print Dumper \$performance_data{$st_type}{$section};
  #print Dumper \%performance_data;
  my $cmd  = "v1/objects/parity-groups";
  my $data = cmd_api($cmd, "GET", "parity_groups", 0, "");
  #print Dumper \$data;

  if ( $data ne "" && defined $data->{data} && ref( $data->{data} ) eq "ARRAY" ) {
    foreach my $rg ( @{ $data->{data} } ) {
      my $id            = $rg->{parityGroupId};
      my $used_capacity = "";
      if (!defined $id || $id eq ""){next;}
      my $uid = Xormon::create_hash_key($st_serial.$section.$id);
      if ( !defined $performance_data{$st_type}{$section}{$uid}) { next; }
      my $capacity  = $rg->{totalCapacity};
      my $drive_type_name = $rg->{driveTypeName};
      my $raid_type = $rg->{raidType};
      my $count_disk;
      if (defined $raid_type){
        ( $count_disk, undef ) = split( m/\+/, $raid_type );
        $count_disk =~ s/D//g;
      }
      if ( !Xormon::isdigit($capacity) ) { $capacity = ""; }
      else {
        #$rg_capacity{$id}{"Total size(GB)"} = $capacity;
        #if ( isdigit($formatted_cap_tot) ) {
        #  $formatted_cap_tot = $formatted_cap_tot + $capacity;
        #}
        #else {
        #  $formatted_cap_tot = $capacity;
        #}
        #if ( Xormon::isdigit( $rg->{availableVolumeCapacity} ) ) {
          #$rg_capacity{$id}{"Used size(GB)"} = $capacity - $rg->{availableVolumeCapacity};
        #}
        $capacity = round( $capacity * (1024 * 1024 * 1024) );
      }
      my $free_capacity = $rg->{availableVolumeCapacity};
      if ( !Xormon::isdigit($free_capacity) ) { $free_capacity = ""; }
      else {
        #if ( Xormon::isdigit($formatted_cap_free) ) {
        #  $formatted_cap_free = $formatted_cap_free + $free_capacity;
        #}
        #else {
        #  $formatted_cap_free = $free_capacity;
        #}

        $free_capacity = round( $free_capacity * (1024 * 1024 * 1024) );
        if ( Xormon::isdigit($capacity) ) {
          $used_capacity = round( $capacity - $free_capacity );
        }
      }
      foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$uid}}){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_used} = $used_capacity;
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_total} = $capacity;
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_free} = $free_capacity;
        if (defined $drive_type_name){
          $performance_data{$st_type}{$section}{$uid}{$timestamp}{type} = $drive_type_name;
        }
        if (Xormon::isdigit($count_disk)){
          $performance_data{$st_type}{$section}{$uid}{$timestamp}{count_disk} = $count_disk;
        }

      }
      #if ( defined $raid_type ) {
      #  $rg_capacity{$id}{"type"} = $raid_type;
      #}

      #if ( defined $performance_data{$section}{$sub_section}{$id} ) {
      #  $performance_data{$section}{$sub_section}{$id}{"Total size(TB)"} = $capacity;
      #  $performance_data{$section}{$sub_section}{$id}{"Used size(TB)"}  = $used_capacity;
      #}

    }
  }

  ### id se muze rovnat 22-22 dvojciferne cisla osetrit i u export tool 1
  #print Dumper \$performance_data{$st_type}{$section};

}

sub set_data_cache{
  my $section = "cache";
  my %datas;
  $datas{"PHY_Write_Pending_Rate_"} = "write_pending_percent";
  $datas{"PHY_Short_Write_Pending_Rate_"} = "write_pending_percent";
  $datas{"PHY_Cache_Usage_Rate_"} = "cache_usage_percent";
  $datas{"PHY_Short_Cache_Usage_Rate_"} = "cache_usage_percent";
  $datas{"PHY_Cache_Allocate_"} = "cache_allocated_size";
  $datas{"PHY_Short_Cache_Allocate_"} = "cache_allocated_size";
  my @types = ("PhyMPU_dat", "PhyMainPK_dat", "PhyCMPK_dat", "PhyESW_dat");
  foreach my $files (keys %datas){
    foreach my $type (@types){
      my @files = <$performance_data_store/$type/$files*csv>;
      foreach my $file (@files){
        my $filename = basename($file);
        my @filename_parse = split("_",$filename);
        my $cpu_type_prefix_uid = $filename_parse[-1];
        $cpu_type_prefix_uid =~ s/\.csv//g;
        my $cpu_type_name = $cpu_type_prefix_uid;
        $cpu_type_prefix_uid = Xormon::create_hash_key($st_serial.$section.$cpu_type_prefix_uid);
        $configuration_data{$cpu_type_prefix_uid}{label} = $cpu_type_name;
        $configuration_data{$cpu_type_prefix_uid}{id} = $cpu_type_name;
        $configuration_data{$cpu_type_prefix_uid}{TYPE} = "$section";
        if (!grep( /^$cpu_type_prefix_uid$/, @cache_type )){
          push(@cache_type,$cpu_type_prefix_uid);
        }
        set_data($file,$section,$datas{$files});
      }
    }
  }
  #print Dumper \%performance_data;
  #print Dumper \@cache_type;
  foreach my $cpu_type (@cache_type){
    foreach my $id (keys %{$performance_data{$st_type}{$section}}){
      if ($id =~ m/^$cpu_type/){
        foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$id}}){
          foreach my $item (keys %{$performance_data{$st_type}{$section}{$id}{$timestamp}}){
            my $value = $performance_data{$st_type}{$section}{$id}{$timestamp}{$item};
            if (Xormon::isdigit($value)){
              if (defined $performance_data{$st_type}{$section}{$cpu_type} && defined $performance_data{$st_type}{$section}{$cpu_type}{$timestamp} && Xormon::isdigit($performance_data{$st_type}{$section}{$cpu_type}{$timestamp}{$item})){
                $performance_data{$st_type}{$section}{$cpu_type}{$timestamp}{$item} = $performance_data{$st_type}{$section}{$cpu_type}{$timestamp}{$item} + $value;
              }
              else{
                $performance_data{$st_type}{$section}{$cpu_type}{$timestamp}{$item} = $value;
              }
            }
          }
        }
      }
    }
  }
  my $section_cpu = "cpu";
  foreach my $id (keys %{$performance_data{$st_type}{$section}}){
    #print "$id\n";
    if ($id =~ m/--/){
      delete $performance_data{$st_type}{$section}{$id};
      next;
    }
    if (!defined $configuration_data{$id} || !defined $configuration_data{$id}{label}){next;}
    my $node_label = $configuration_data{$id}{label};
    foreach my $uid (keys %{$performance_data{$st_type}{$section_cpu}}){
      if (!defined $configuration_data{$uid} || !defined $configuration_data{$uid}{label}){next;}
      my $cpu_label = $configuration_data{$uid}{label};
      if ($cpu_label =~ m/^$node_label/) {
        $caches{$uid} = $id;
        foreach my $timestmap (keys %{$performance_data{$st_type}{$section_cpu}{$uid}}){
          if (Xormon::isdigit($performance_data{$st_type}{$section_cpu}{$uid}{$timestmap}{"cpu_system"})){
            if (Xormon::isdigit($performance_data{$st_type}{$section}{$id}{$timestmap}{"cpu_system"})){
              $performance_data{$st_type}{$section}{$id}{$timestmap}{"cpu_system"} = $performance_data{$st_type}{$section}{$id}{$timestmap}{"cpu_system"} + $performance_data{$st_type}{$section_cpu}{$uid}{$timestmap}{"cpu_system"};
              $performance_data{$st_type}{$section}{$id}{$timestmap}{"count"}++;
            }
            else{
              $performance_data{$st_type}{$section}{$id}{$timestmap}{"cpu_system"} = $performance_data{$st_type}{$section_cpu}{$uid}{$timestmap}{"cpu_system"};
              $performance_data{$st_type}{$section}{$id}{$timestmap}{"count"} = 1;
            }
          }
        }
      }
    }
  }
  foreach my $id (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$id}}){
      my $sys = $performance_data{$st_type}{$section}{$id}{$timestamp}{cpu_system};
      my $count = $performance_data{$st_type}{$section}{$id}{$timestamp}{count};
      if (Xormon::isdigit($sys) && Xormon::isdigit($count)){
        $performance_data{$st_type}{$section}{$id}{$timestamp}{cpu_system} = round($sys / $count,1);
        delete $performance_data{$st_type}{$section}{$id}{$timestamp}{count};
      }
    }
  }
  #print Dumper \@cache_type;
  ### add sys from cpu section ###
  #print Dumper \$performance_data{$st_type}{$section};
}

sub set_data_volume2{
  my $section     = "volume";
  my %datas;
  $datas{Random_Read_IOPS} = "rnd_read_io";
  $datas{Sequential_Read_IOPS} = "seq_read_io";
  $datas{Sequential_Write_IOPS} = "seq_write_io";
  $datas{Random_Write_IOPS} = "rnd_write_io";
  $datas{Sequential_Read_Hit_Rate} = "seq_read_hit";
  $datas{Random_Read_Hit_Rate} = "rnd_read_hit";
  $datas{Sequential_Write_Hit_Rate} = "seq_write_hit";
  $datas{Random_Write_Hit_Rate} = "rnd_write_hit";
  $datas{Random_Read_Transfer_Rate} = "rnd_read";
  $datas{Sequential_Read_Transfer_Rate} = "seq_read";
  $datas{Random_Write_Transfer_Rate} = "rnd_write";
  $datas{Sequential_Write_Transfer_Rate} = "seq_write";

  $datas{Read_Response_Time} = "resp_t_r";
  $datas{Write_Response_Time} = "resp_t_w";
  $datas{Total_Response_Time} = "resp_t";
  $datas{Backend_Transfer_Count_Total} = "io_rate_b";

  foreach my $file (keys %datas){
    my $filename = "$performance_data_store/LDEV/DP/$file.csv";
    if (-f $filename){
      set_data2($filename,$section,$datas{$file});
    }
    $filename = "$performance_data_store/LDEV/Basic/$file.csv";
    if (-f $filename){
      set_data2($filename,$section,$datas{$file});
    }
  }

  foreach my $uid (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$uid}}){
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"read_io"} = get_metrics_sum("seq_read_io","rnd_read_io","read_io",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"r_cache_hit"} = round(calculate_weighted_average($performance_data{$st_type}{$section}{$uid}{$timestamp},"seq_read_hit","seq_read_io","rnd_read_hit","rnd_read_io"),1);
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_read_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_read_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_read_hit"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_read_hit"};
      if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$timestamp}{"r_cache_hit"}) && $performance_data{$st_type}{$section}{$uid}{$timestamp}{"r_cache_hit"} > 100){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{"r_cache_hit"} = 100;
      }

      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"write_io"} = get_metrics_sum("seq_write_io","rnd_write_io","write_io",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"w_cache_hit"} = round(calculate_weighted_average($performance_data{$st_type}{$section}{$uid}{$timestamp},"seq_write_hit","seq_write_io","rnd_write_hit","rnd_write_io"),1);
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_write_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_write_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_write_hit"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_write_hit"};
      if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$timestamp}{"w_cache_hit"}) && $performance_data{$st_type}{$section}{$uid}{$timestamp}{"w_cache_hit"} > 100){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{"w_cache_hit"} = 100;
      }

      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"read"} = get_metrics_sum("seq_read","rnd_read","read",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_read"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_read"};
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"write"} = get_metrics_sum("seq_write","rnd_write","write",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_write"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_write"};
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"} = get_metrics_sum("read_io","write_io","io_rate",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"data_rate"} = get_metrics_sum("read","write","data_rate",$performance_data{$st_type}{$section}{$uid}{$timestamp});

    }
  }
  my $data    = cmd_api("v1/objects/ldevs?count=16384&ldevOption=defined", "GET", "ldevs", 0, "");
  if ( $data ne "" && defined $data->{data} && ref( $data->{data} ) eq "ARRAY" ) {
    foreach my $volume ( @{ $data->{data} } ) {
      my $type = $volume->{emulationType};
      if ( defined $type && $type eq "NOT DEFINED" ) { next; }
      my $id = $volume->{ldevId};
      if ( !defined $id ) { next; }
      my $pool_volume = $volume->{attributes};
      my $hex         = sprintf( "0x%X", $id );
      $hex =~ s/x//;
      my $len = length($hex);
      if ( $len < 6 ) {
        my $add_null = 6 - $len;
        for ( my $i = 0; $i < $add_null; $i++ ) {
          $hex = "0$hex";
        }
      }
      my @chars = split( "", $hex );
      $hex = "$chars[0]$chars[1]:$chars[2]$chars[3]:$chars[4]$chars[5]";
      $id  = $hex;
      my $uid = Xormon::create_hash_key($st_serial.$section.$id);
      if (!defined $performance_data{$st_type}{$section}{$uid}){
        next;
      }
      my $pool_id = $volume->{poolId};
      if ( !defined $pool_id || $pool_id eq "") { $pool_id = "-1"; }

      my $capacity     = $volume->{blockCapacity};
      my $raw_capacity = "";

      if ( !Xormon::isdigit($capacity) ) { $capacity = ""; }
      else {
        $raw_capacity = $capacity;
        $capacity     = ( $capacity * 512 );
      }
      my $used_capacity = $volume->{numOfUsedBlock};
      if ( !Xormon::isdigit($used_capacity) ) { $used_capacity = ""; }
      else {
        $used_capacity = ( $used_capacity * 512 );
      }
      my $ports = $volume->{ports};
      if (defined $ports && ref($ports) eq "ARRAY" ){
        foreach my $host_group (@{$ports}){
          my $name = $host_group->{hostGroupName};
          if (defined $name && $name ne ""){
            push(@{$volumes{$uid}{host_group}},$name);
          }
        }
      }
      $volumes{$uid}{pool} = $pool_id;
      foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$uid}}){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_used} = $used_capacity;
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_total} = $capacity;
        if (Xormon::isdigit($used_capacity) && Xormon::isdigit($capacity)){
          $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_free} = $capacity - $used_capacity;
          if ($performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_free} < 0){
            $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_free} = 0;
            $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_used} = $capacity;
          }
        }
      }
    }
  }
  foreach my $uid (keys %{$performance_data{$st_type}{$section}}){
    if (!defined $volumes{$uid}){
      $volumes{$uid}{pool} = -1;
    }
  }
  #print Dumper \$performance_data{$st_type}{$section}{"5665e7dcb0"};
}

sub set_data_volume {
  my $section     = "volume";
  my %datas;
  $datas{LDEV_Rnd_Read_IOPS} = "rnd_read_io";
  $datas{LDEV_Seq_Read_IOPS} = "seq_read_io";
  $datas{LDEV_Seq_Write_IOPS} = "seq_write_io";
  $datas{LDEV_Rnd_Write_IOPS} = "rnd_write_io";
  $datas{LDEV_Seq_Read_Hit} = "seq_read_hit";
  $datas{LDEV_Rnd_Read_Hit} = "rnd_read_hit";
  $datas{LDEV_Seq_Write_Hit} = "seq_write_hit";
  $datas{LDEV_Rnd_Write_Hit} = "rnd_write_hit";
  $datas{LDEV_Read_TransRate} = "read";
  $datas{LDEV_Write_TransRate} = "write";
  $datas{LDEV_Read_Response} = "resp_t_r";
  $datas{LDEV_Write_Response} = "resp_t_w";
  $datas{LDEV_Response} = "resp_t";
  $datas{LDEV_C2D_Trans} = "data_rate_b";
  $datas{LDEV_D2CS_Trans} = "D2CS";
  $datas{LDEV_D2CR_Trans} = "D2CR";

  foreach my $files (keys %datas){
    my @files = <$performance_data_store/LDEVEachOfCU_dat/$files/*>;
    foreach my $file (@files){
      set_data($file,$section,$datas{$files});
    }
    @files = <$performance_data_store/LDEV_dat/$files/*>;
    foreach my $file (@files){
      set_data($file,$section,$datas{$files});
    }
  }
  
  ### IOPS READ ###
  foreach my $uid (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$uid}}){
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"read_io"} = get_metrics_sum("seq_read_io","rnd_read_io","read_io",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"r_cache_hit"} = round(calculate_weighted_average($performance_data{$st_type}{$section}{$uid}{$timestamp},"seq_read_hit","seq_read_io","rnd_read_hit","rnd_read_io"),1);
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_read_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_read_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_read_hit"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_read_hit"};
      if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$timestamp}{"r_cache_hit"}) && $performance_data{$st_type}{$section}{$uid}{$timestamp}{"r_cache_hit"} > 100){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{"r_cache_hit"} = 100;
      }

      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"write_io"} = get_metrics_sum("seq_write_io","rnd_write_io","write_io",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"w_cache_hit"} = round(calculate_weighted_average($performance_data{$st_type}{$section}{$uid}{$timestamp},"seq_write_hit","seq_write_io","rnd_write_hit","rnd_write_io"),1);
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_write_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_write_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_write_hit"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_write_hit"};
      if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$timestamp}{"w_cache_hit"}) && $performance_data{$st_type}{$section}{$uid}{$timestamp}{"w_cache_hit"} > 100){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{"w_cache_hit"} = 100;
      }

      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"} = get_metrics_sum("read_io","write_io","io_rate",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"data_rate"} = get_metrics_sum("read","write","data_rate",$performance_data{$st_type}{$section}{$uid}{$timestamp});


      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate_b"} = get_metrics_sum("D2CR","D2CS","io_rate_b",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"D2CR"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"D2CS"};
    }
  }
  my $data    = cmd_api("v1/objects/ldevs?count=16384&ldevOption=defined", "GET", "ldevs", 0, "");
  if ( $data ne "" && defined $data->{data} && ref( $data->{data} ) eq "ARRAY" ) {
    foreach my $volume ( @{ $data->{data} } ) {
      my $type = $volume->{emulationType};
      if ( defined $type && $type eq "NOT DEFINED" ) { next; }
      my $id = $volume->{ldevId};
      if ( !defined $id ) { next; }
      my $pool_volume = $volume->{attributes};
      my $hex         = sprintf( "0x%X", $id );
      $hex =~ s/x//;
      my $len = length($hex);
      if ( $len < 6 ) {
        my $add_null = 6 - $len;
        for ( my $i = 0; $i < $add_null; $i++ ) {
          $hex = "0$hex";
        }
      }
      my @chars = split( "", $hex );
      $hex = "$chars[0]$chars[1]:$chars[2]$chars[3]:$chars[4]$chars[5]";
      $id  = $hex;
      my $uid = Xormon::create_hash_key($st_serial.$section.$id);
      if (!defined $performance_data{$st_type}{$section}{$uid}){
        next;
      }
      my $pool_id = $volume->{poolId};
      if ( !defined $pool_id || $pool_id eq "") { $pool_id = "-1"; }

      my $capacity     = $volume->{blockCapacity};
      my $raw_capacity = "";

      if ( !Xormon::isdigit($capacity) ) { $capacity = ""; }
      else {
        $raw_capacity = $capacity;
        $capacity     = ( $capacity * 512 );
      }
      my $used_capacity = $volume->{numOfUsedBlock};
      if ( !Xormon::isdigit($used_capacity) ) { $used_capacity = ""; }
      else {
        $used_capacity = ( $used_capacity * 512 );
      }
      my $ports = $volume->{ports};
      if (defined $ports && ref($ports) eq "ARRAY" ){
        foreach my $host_group (@{$ports}){
          my $name = $host_group->{hostGroupName};
          if (defined $name && $name ne ""){
            push(@{$volumes{$uid}{host_group}},$name);
          }
        }
      }
      $volumes{$uid}{pool} = $pool_id;
      foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$uid}}){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_used} = $used_capacity;
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_total} = $capacity;
        if (Xormon::isdigit($used_capacity) && Xormon::isdigit($capacity)){
          $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_free} = $capacity - $used_capacity;
          if ($performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_free} < 0){
            $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_free} = 0;
            $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_used} = $capacity;
          }
        }
      }
    }
  }
  foreach my $uid (keys %{$performance_data{$st_type}{$section}}){
    if (!defined $volumes{$uid}){
      $volumes{$uid}{pool} = -1;
    }
  }
  
  #print Dumper \$performance_data{$st_type}{$section}{"6996fd207c"};
  #print Dumper \$volumes{"873019929c"};
  #print Dumper \$performance_data{$st_type}{$section};
}

sub set_data_raid_group_conf{
  my $section = "raid_group";
  my $cmd  = "v1/objects/parity-groups";
  my $data = cmd_api($cmd, "GET", "parity_groups", 1, "");
  my @data_rg_table = ();
  #print Dumper \$data;
  if ( $data ne "" && defined $data->{data} && ref( $data->{data} ) eq "ARRAY" ) {
    foreach my $rg ( @{ $data->{data} } ) {
      my $id            = $rg->{parityGroupId};
      my $used_capacity = "";
      my $physical_capacity_used = "";
      if (!defined $id || $id eq ""){next;}
      my $uid = Xormon::create_hash_key($st_serial.$section.$id);
      #if ( !defined $performance_data{$st_type}{$section}{$uid}) { next; }
      my $capacity  = $rg->{totalCapacity};
      my $raid_type = $rg->{raidType};
      my $compression = $rg->{isAcceleratedCompressionEnabled};
      my $raid_level = $rg->{raidLevel};
      my $drive_type_name = $rg->{driveTypeName};
      my $physical_capacity_total = $rg->{physicalCapacity};
      my $physical_capacity_free = $rg->{availablePhysicalCapacity};
      if ( !Xormon::isdigit($capacity) ) { $capacity = ""; }
      else {
        $capacity = round( $capacity * (1024 * 1024 * 1024) );
      }
      my $free_capacity = $rg->{availableVolumeCapacity};
      if ( !Xormon::isdigit($free_capacity) ) { $free_capacity = ""; }
      else {
        $free_capacity = round( $free_capacity * (1024 * 1024 * 1024) );
        if ( Xormon::isdigit($capacity) ) {
          $used_capacity = round( $capacity - $free_capacity );
        }
      }
      if (!defined $compression){$compression = "";}
      if (!defined $raid_level){$raid_level = "";}
      if (!defined $drive_type_name){$drive_type_name = "";}
      if (Xormon::isdigit($physical_capacity_total)){
        $physical_capacity_total *= 1024 * 1024 * 1024;
      }
      else{
        $physical_capacity_total = "";
      }
      if (Xormon::isdigit($physical_capacity_free)){
        $physical_capacity_free *= 1024 * 1024 * 1024;
        if (Xormon::isdigit($physical_capacity_total)){
          $physical_capacity_used = $physical_capacity_total - $physical_capacity_free;
        }
      }
      else{
        $physical_capacity_free = "";
      }
      $configuration_data{$uid}{label}  = $id;
      $configuration_data{$uid}{id}  = $id;
      $configuration_data{$uid}{TYPE} = $section;
      $configuration_data{$uid}{capacity_used} = $used_capacity;
      $configuration_data{$uid}{capacity_total} = $capacity;
      $configuration_data{$uid}{capacity_free} = $free_capacity;
      $configuration_data{$uid}{"drive_type"} = $drive_type_name;
      $configuration_data{$uid}{"hw_compression"} = $compression;
      $configuration_data{$uid}{raid_level} = $raid_level;
      $configuration_data{$uid}{raid_type} = $raid_type;
      $configuration_data{$uid}{physical_capacity_total} = $physical_capacity_total;
      $configuration_data{$uid}{physical_capacity_used} = $physical_capacity_used;
      $configuration_data{$uid}{physical_capacity_free} = $physical_capacity_free;
      push(@data_rg_table,$configuration_data{$uid});

    }
  }
  $configuration_data{$cfg->{'uuid'}}{raid_group} = encode_json(\@data_rg_table);
  #print Dumper \@data_rg_table;
  #print Dumper \%configuration_data;
}

sub set_data_raid_group{
  my $section = "raid_group";
  my %datas;
  $datas{PG_Rnd_Read_IOPS} = "rnd_read_io";
  $datas{PG_Seq_Read_IOPS} = "seq_read_io";
  $datas{PG_Seq_Write_IOPS} = "seq_write_io";
  $datas{PG_Rnd_Write_IOPS} = "rnd_write_io";
  $datas{PG_Read_TransRate} = "read";
  $datas{PG_Write_TransRate} = "write";
  $datas{PG_Read_Response} = "resp_t_r";
  $datas{PG_Write_Response} = "resp_t_w";
  $datas{PG_Response} = "resp_t";
  $datas{PG_BackTrans} = "data_rate_b";
  $datas{PG_D2CS_Trans} = "D2CS";
  $datas{PG_D2CR_Trans} = "D2CR";

  foreach my $element (keys %datas){
    my $file = "$performance_data_store/PG_dat/$element.csv";
    if (! -f $file){next;}
    set_data($file,$section,$datas{$element});
  }
  ### IOPS READ ###
  foreach my $uid (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$uid}}){
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"read_io"} = get_metrics_sum("seq_read_io","rnd_read_io","read_io",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_read_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_read_io"};

      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"write_io"} = get_metrics_sum("seq_write_io","rnd_write_io","write_io",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"seq_write_io"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"rnd_write_io"};

      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"} = get_metrics_sum("read_io","write_io","io_rate",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      #if (!defined $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"} || (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"}) && $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"} == 0)){
      #  delete $performance_data{$st_type}{$section}{$uid}{$timestamp};
      #  next;
      #}
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"data_rate"} = get_metrics_sum("read","write","data_rate",$performance_data{$st_type}{$section}{$uid}{$timestamp});

      $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate_b"} = get_metrics_sum("D2CR","D2CS","io_rate_b",$performance_data{$st_type}{$section}{$uid}{$timestamp});
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"D2CR"};
      delete $performance_data{$st_type}{$section}{$uid}{$timestamp}{"D2CS"};
    }
    #if (keys %{$performance_data{$st_type}{$section}{$uid} } == 0){
    #  delete $performance_data{$st_type}{$section}{$uid};
    #  next;
    #}
  }
  #if (keys %{$performance_data{$st_type}{$section} } == 0){
  #  delete $performance_data{$st_type}{$section};
  #  return;
  #}
  #print Dumper \$performance_data{$st_type}{$section};
  my $cmd  = "v1/objects/parity-groups";
  my $data = cmd_api($cmd, "GET", "parity_groups", 0, "");

  if ( $data ne "" && defined $data->{data} && ref( $data->{data} ) eq "ARRAY" ) {
    foreach my $rg ( @{ $data->{data} } ) {
      my $id            = $rg->{parityGroupId};
      my $used_capacity = "";
      if (!defined $id || $id eq ""){next;}
      my $uid = Xormon::create_hash_key($st_serial.$section.$id);
      if ( !defined $performance_data{$st_type}{$section}{$uid}) { next; }
      my $capacity  = $rg->{totalCapacity};
      my $drive_type_name = $rg->{driveTypeName};
      my $raid_type = $rg->{raidType};
      my $count_disk;
      if (defined $raid_type){
        ( $count_disk, undef ) = split( m/\+/, $raid_type );
        $count_disk =~ s/D//g;
      }
      if ( !Xormon::isdigit($capacity) ) { $capacity = ""; }
      else {
        #$rg_capacity{$id}{"Total size(GB)"} = $capacity;
        #if ( isdigit($formatted_cap_tot) ) {
        #  $formatted_cap_tot = $formatted_cap_tot + $capacity;
        #}
        #else {
        #  $formatted_cap_tot = $capacity;
        #}
        #if ( Xormon::isdigit( $rg->{availableVolumeCapacity} ) ) {
          #$rg_capacity{$id}{"Used size(GB)"} = $capacity - $rg->{availableVolumeCapacity};
        #}
        $capacity = round( $capacity * (1024 * 1024 * 1024) );
      }
      my $free_capacity = $rg->{availableVolumeCapacity};
      if ( !Xormon::isdigit($free_capacity) ) { $free_capacity = ""; }
      else {
        #if ( Xormon::isdigit($formatted_cap_free) ) {
        #  $formatted_cap_free = $formatted_cap_free + $free_capacity;
        #}
        #else {
        #  $formatted_cap_free = $free_capacity;
        #}

        $free_capacity = round( $free_capacity * (1024 * 1024 * 1024) );
        if ( Xormon::isdigit($capacity) ) {
          $used_capacity = round( $capacity - $free_capacity );
        }
      }
      foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$uid}}){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_used} = $used_capacity;
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_total} = $capacity;
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{capacity_free} = $free_capacity;
        if (defined $drive_type_name){
          $performance_data{$st_type}{$section}{$uid}{$timestamp}{type} = $drive_type_name;
        }
        if (Xormon::isdigit($count_disk)){
          $performance_data{$st_type}{$section}{$uid}{$timestamp}{count_disk} = $count_disk;
        }
      }
      #if ( defined $raid_type ) {
      #  $rg_capacity{$id}{"type"} = $raid_type;
      #}

      #if ( defined $performance_data{$section}{$sub_section}{$id} ) {
      #  $performance_data{$section}{$sub_section}{$id}{"Total size(TB)"} = $capacity;
      #  $performance_data{$section}{$sub_section}{$id}{"Used size(TB)"}  = $used_capacity;
      #}

    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}

sub calculate_weighted_average{
  my $data_tmp = shift;
  my $numerator_metric1 = shift;
  my $denominator_metric1 = shift;
  my $numerator_metric2 = shift;
  my $denominator_metric2 = shift;


  my %data = %{$data_tmp};

  my $sum = "";
  my $denominator = "";
  if (Xormon::isdigit($data{$denominator_metric1}) && Xormon::isdigit($data{$numerator_metric1})){
    $sum = $data{$denominator_metric1} * $data{$numerator_metric1};
    $denominator = $data{$denominator_metric1};
  }
  if (Xormon::isdigit($data{$denominator_metric2}) && Xormon::isdigit($data{$numerator_metric2})){
    if (Xormon::isdigit($sum)){
      $sum = $sum + ($data{$denominator_metric2} * $data{$numerator_metric2});
      $denominator = $denominator + $data{$denominator_metric2};
    }
    else{
      $sum = $data{$denominator_metric2} * $data{$numerator_metric2};
      $denominator = $data{$denominator_metric2};
    }
  }
  if (Xormon::isdigit($sum) && Xormon::isdigit($denominator)){
    if ($denominator == 0){
      return 0;
    }
    return $sum / $denominator;
  }
  return "";
}

sub get_difference_of_values{
  my $item1 = shift;
  my $item2 = shift;
  my $data_tmp = shift;
  my %data = %{$data_tmp};
  my $value1 = $data{$item1};
  my $value2 = $data{$item2};

  if (Xormon::isdigit($value2) && Xormon::isdigit($value1)){
    return ($value1 - $value2);
  }
  if (Xormon::isdigit($value1) && !Xormon::isdigit($value2)){
    return $value1;
  }
  return "";
}


sub get_metrics_sum{
  my $item1 = shift;
  my $item2 = shift;
  my $item3 = shift;
  my $data_tmp = shift;
  my %data = %{$data_tmp};
  my $value1 = $data{$item1};
  my $value2 = $data{$item2};

  if (Xormon::isdigit($value2)){
    if (Xormon::isdigit($value1)){
      return ($value2 + $value1);
    }
    else{
      return $value2;
    }
  }
  elsif(Xormon::isdigit($value1)){
    return $value1;
  }
  return "";

}

sub set_data_port2{
  my $section = "port";
  my @postfix_files = ("IOPS", "Transfer_Rate", "Response_Time");
  my @metrics = ("io_rate", "data_rate", "resp_t");
  my $index = -1;
  foreach my $file (@postfix_files){
    $index++;
    if (-f "$performance_data_store/Port/Target/Total_$file.csv"){
      set_data2("$performance_data_store/Port/Target/Total_$file.csv",$section,$metrics[$index]);
    }
    if (-f "$performance_data_store/Port/Initiator/Total_$file.csv"){
      set_data2("$performance_data_store/Port/Initiator/Total_$file.csv",$section,$metrics[$index] . "_initiator");
    }
  }
  #print Dumper \%performance_data;

}


sub set_data_port{
  my $section = "port";
  my @postfix_files = ("IOPS", "KBPS", "Response");
  my @metrics = ("io_rate", "data_rate", "resp_t");
  my $index = -1;
  foreach my $file (@postfix_files){
    $index++;
    if (-f "$performance_data_store/Port_dat/Port_$file.csv"){
      set_data("$performance_data_store/Port_dat/Port_$file.csv",$section,$metrics[$index]);
    }
    if (-f "$performance_data_store/Port_dat/Port_Initiator_$file.csv"){
      set_data("$performance_data_store/Port_dat/Port_Initiator_$file.csv",$section,$metrics[$index] . "_initiator");
    }
  }
  #print Dumper \%performance_data;
}

sub set_data_host2{


  ### We have not data for hosts data from export tool 2 then calculate via volumes
  my $section = "host";
  my %host_groups;
  my $cmd        = "v1/objects/host-groups";
  my $data       = cmd_api($cmd,"GET","host_groups",0,"");
  if ( $data eq "" || ref( $data->{data} ) ne "ARRAY" ) {return;}
  foreach my $host_group ( @{ $data->{data} } ) {
    if ( defined $host_group->{hostGroupId} ) {
      my ( $port, $id ) = split( ",", $host_group->{hostGroupId} );
      if ( defined $port && defined $id ) {
        my $hosts = cmd_api("v1/objects/host-wwns?portId=$port&hostGroupNumber=$id", "GET", "v1/objects/host-wwns--portId-$port-hostGroupNumber-$id", 0, "" );
        if ($hosts eq "" || ref( $hosts->{data} ) ne "ARRAY"){next;}
        foreach my $host (@{$hosts->{data}}){
          my $host_group_name = $host->{hostGroupName};
          my $host_wwn = $host->{hostWwn};
          if (!defined $host_group_name || $host_group_name eq "" || !defined $host_wwn || $host_wwn eq ""){next;}
          $host_wwn = uc($host_wwn);
          if (!grep( /^$host_wwn$/, @{$host_groups{$host_group_name}})){
            push(@{$host_groups{$host_group_name}},$host_wwn);
          }
        }
        #print Dumper \$hosts;
      }
    }
  }

  foreach my $volume (keys %volumes){
    foreach my $hs_group (@{$volumes{$volume}{"host_group"}}){
      foreach my $orginal_hs_group (keys %host_groups){
        if ($orginal_hs_group =~ m/^$hs_group/){
          $volumes{$volume}{hosts} = $host_groups{$orginal_hs_group};
        }
      }
      if (defined $host_groups{$hs_group}){
        $volumes{$volume}{hosts} = $host_groups{$hs_group};
      }
    }
  }
  my $section_volume = "volume";
  my @items = ("io_rate","data_rate", "resp_t");
  foreach my $uid (keys %{$performance_data{$st_type}{$section_volume}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section_volume}{$uid}}){
      foreach my $item (@items){
        my $value = $performance_data{$st_type}{$section_volume}{$uid}{$timestamp}{$item};
        my $io_rate = $performance_data{$st_type}{$section_volume}{$uid}{$timestamp}{io_rate};
        if (Xormon::isdigit($value)){
          if (defined $volumes{$uid} && defined $volumes{$uid}{hosts}){
            foreach my $host (@{$volumes{$uid}{hosts}}){
              my $uid_host = Xormon::create_hash_key($st_serial.$section.$host);
              if ($item eq "resp_t"){
                if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$uid_host} && defined $performance_data{$st_type}{$section}{$uid_host}{$timestamp} && defined $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{numerator}){
                  if (Xormon::isdigit($io_rate)){
                    $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{numerator} = $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{numerator} + ($value * $io_rate);
                    $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{denominator} = $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{denominator} + $io_rate;
                  }
                }
                else{
                  if (Xormon::isdigit($io_rate)){
                    $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{numerator} = $value * $io_rate;
                    $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{denominator} = $io_rate;
                  }
                }
                next;
              }
              if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$uid_host} && defined $performance_data{$st_type}{$section}{$uid_host}{$timestamp} && defined $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{$item}){
                $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{$item} = $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{$item} + $value;
              }
              else{
                $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{$item} = $value;
              }
            }
          }
        }
      }
    }
  }
  foreach my $id (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$id}}){
      my $numerator = $performance_data{$st_type}{$section}{$id}{$timestamp}{numerator};
      my $denominator = $performance_data{$st_type}{$section}{$id}{$timestamp}{denominator};
      if (Xormon::isdigit($numerator) && Xormon::isdigit($denominator)){
        if ($denominator == 0){
          $performance_data{$st_type}{$section}{$id}{$timestamp}{resp_t} = 0;
        }
        else{
          $performance_data{$st_type}{$section}{$id}{$timestamp}{resp_t} = round($numerator / $denominator,3);
        }
      }
      if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{numerator}){
        delete $performance_data{$st_type}{$section}{$id}{$timestamp}{numerator};
      }
      if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{denominator}){
        delete $performance_data{$st_type}{$section}{$id}{$timestamp}{denominator};
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};

}

sub set_data_host{
  my $section = "host";
  my @files = <$performance_data_store/PortWWN_dat/*>;
  my @ports = ();
  foreach my $file (@files){
    my $filename = basename($file);
    my @filename_parse = split("_",$filename);
    my $port = $filename_parse[1];
    $port =~ s/\.csv//g;
    if (!grep( /^$port$/, @ports )){
      push(@ports,$port);
    }
    if ($file =~ m/IOPS/){
      set_data($file,$section,"io_rate");
    }
    elsif($file =~ m/KBPS/){
      set_data($file,$section,"data_rate");
    }
    elsif($file =~ m/Response/){
      set_data($file,$section,"resp_t");
    }
  }
  foreach my $port (@ports){
    foreach my $id (keys %{$performance_data{$st_type}{$section}}){
      my (undef, $wwn) = split("--",$id);
      if (!defined $wwn || $wwn eq ""){next;}
      $wwn =~ s/\(.*//g;
      if (!defined $wwn || $wwn eq ""){next;}
      my $uid = Xormon::create_hash_key($st_serial.$section.$wwn);
      if ($id =~ m/^$port/){
        foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$id}}){
          foreach my $item (keys %{$performance_data{$st_type}{$section}{$id}{$timestamp}}){
            my $value = $performance_data{$st_type}{$section}{$id}{$timestamp}{$item};
            if (Xormon::isdigit($value)){
              if ($item eq "resp_t"){
                my $io_rate = $performance_data{$st_type}{$section}{$id}{$timestamp}{"io_rate"};
                if (defined $performance_data{$st_type}{$section}{$uid} && defined $performance_data{$st_type}{$section}{$uid}{$timestamp} && Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$timestamp}{numerator})){
                  if (Xormon::isdigit($io_rate)){
                    $performance_data{$st_type}{$section}{$uid}{$timestamp}{numerator} = ($performance_data{$st_type}{$section}{$uid}{$timestamp}{numerator} + ($value * $io_rate));
                    $performance_data{$st_type}{$section}{$uid}{$timestamp}{denominator} = $performance_data{$st_type}{$section}{$uid}{$timestamp}{denominator} + $io_rate;
                  }
                }
                else{
                  if (Xormon::isdigit($io_rate)){
                    $performance_data{$st_type}{$section}{$uid}{$timestamp}{numerator} = $value * $io_rate;
                    $performance_data{$st_type}{$section}{$uid}{$timestamp}{denominator} = $io_rate;
                  }
                }
              }
              else{
                if (defined $performance_data{$st_type}{$section}{$uid} && defined $performance_data{$st_type}{$section}{$uid}{$timestamp} && Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$timestamp}{$item})){
                  $performance_data{$st_type}{$section}{$uid}{$timestamp}{$item} = $performance_data{$st_type}{$section}{$uid}{$timestamp}{$item} + $value;
                }
                else{
                  $performance_data{$st_type}{$section}{$uid}{$timestamp}{$item} = $value;
                }
              }
            }
          }
        }
      }
    }
  }
  foreach my $id (keys %{$performance_data{$st_type}{$section}}){
    if ($id =~ m/--/){
      delete $performance_data{$st_type}{$section}{$id};
      next;
    }
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$id}}){
      my $numerator = $performance_data{$st_type}{$section}{$id}{$timestamp}{numerator};
      my $denominator = $performance_data{$st_type}{$section}{$id}{$timestamp}{denominator};
      if (Xormon::isdigit($numerator) && Xormon::isdigit($denominator)){
        if ($denominator == 0){
          $performance_data{$st_type}{$section}{$id}{$timestamp}{resp_t} = 0;
        }
        else{
          $performance_data{$st_type}{$section}{$id}{$timestamp}{resp_t} = round($numerator / $denominator,3);
        }
      }
      if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{numerator}){
        delete $performance_data{$st_type}{$section}{$id}{$timestamp}{numerator};
      }
      if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{denominator}){
        delete $performance_data{$st_type}{$section}{$id}{$timestamp}{denominator};
      }
    }
  }
  ### testing
  #delete $performance_data{$st_type}{$section};
  if (keys %{$performance_data{$st_type}{$section} } > 0){
    return;
  }
  my %host_groups;
  my $cmd        = "v1/objects/host-groups";
  my $data       = cmd_api($cmd,"GET","host_groups",0,"");
  if ( $data eq "" || ref( $data->{data} ) ne "ARRAY" ) {return;}
  foreach my $host_group ( @{ $data->{data} } ) {
    if ( defined $host_group->{hostGroupId} ) {
      my ( $port, $id ) = split( ",", $host_group->{hostGroupId} );
      if ( defined $port && defined $id ) {
        my $hosts = cmd_api("v1/objects/host-wwns?portId=$port&hostGroupNumber=$id", "GET", "v1/objects/host-wwns--portId-$port-hostGroupNumber-$id", 0, "" );
        if ($hosts eq "" || ref( $hosts->{data} ) ne "ARRAY"){next;}
        foreach my $host (@{$hosts->{data}}){
          my $host_group_name = $host->{hostGroupName};
          my $host_wwn = $host->{hostWwn};
          if (!defined $host_group_name || $host_group_name eq "" || !defined $host_wwn || $host_wwn eq ""){next;}
          $host_wwn = uc($host_wwn);
          if (!grep( /^$host_wwn$/, @{$host_groups{$host_group_name}})){
            push(@{$host_groups{$host_group_name}},$host_wwn);
          }
        }
        #print Dumper \$hosts;
      }
    }
  }
  #print Dumper \%volumes;
  foreach my $volume (keys %volumes){
    foreach my $hs_group (@{$volumes{$volume}{"host_group"}}){
      if (defined $host_groups{$hs_group}){
        $volumes{$volume}{hosts} = $host_groups{$hs_group};
      }
    }
  }
  my $section_volume = "volume";
  my @items = ("io_rate","data_rate", "resp_t");
  foreach my $uid (keys %{$performance_data{$st_type}{$section_volume}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section_volume}{$uid}}){
      foreach my $item (@items){
        my $value = $performance_data{$st_type}{$section_volume}{$uid}{$timestamp}{$item};
        my $io_rate = $performance_data{$st_type}{$section_volume}{$uid}{$timestamp}{io_rate};
        if (Xormon::isdigit($value)){
          if (defined $volumes{$uid} && defined $volumes{$uid}{hosts}){
            foreach my $host (@{$volumes{$uid}{hosts}}){
              my $uid_host = Xormon::create_hash_key($st_serial.$section.$host);
              if ($item eq "resp_t"){
                if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$uid_host} && defined $performance_data{$st_type}{$section}{$uid_host}{$timestamp} && defined $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{numerator}){
                  if (Xormon::isdigit($io_rate)){
                    $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{numerator} = $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{numerator} + ($value * $io_rate);
                    $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{denominator} = $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{denominator} + $io_rate;
                  }
                }
                else{
                  if (Xormon::isdigit($io_rate)){
                    $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{numerator} = $value * $io_rate;
                    $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{denominator} = $io_rate;
                  }
                }
                next;
              }
              if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$uid_host} && defined $performance_data{$st_type}{$section}{$uid_host}{$timestamp} && defined $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{$item}){
                $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{$item} = $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{$item} + $value;
              }
              else{
                $performance_data{$st_type}{$section}{$uid_host}{$timestamp}{$item} = $value;
              }
            }
          }
        }
      }
    }
  }
  foreach my $id (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$id}}){
      my $numerator = $performance_data{$st_type}{$section}{$id}{$timestamp}{numerator};
      my $denominator = $performance_data{$st_type}{$section}{$id}{$timestamp}{denominator};
      if (Xormon::isdigit($numerator) && Xormon::isdigit($denominator)){
        if ($denominator == 0){
          $performance_data{$st_type}{$section}{$id}{$timestamp}{resp_t} = 0;
        }
        else{
          $performance_data{$st_type}{$section}{$id}{$timestamp}{resp_t} = round($numerator / $denominator,3);
        }
      }
      if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{numerator}){
        delete $performance_data{$st_type}{$section}{$id}{$timestamp}{numerator};
      }
      if (defined $performance_data{$st_type}{$section}{$id}{$timestamp}{denominator}){
        delete $performance_data{$st_type}{$section}{$id}{$timestamp}{denominator};
      }
    }
  }
  #print Dumper \%host_groups;
  ### host data from volumes

  ###

  #print Dumper \@ports;
  #
  #print Dumper \$performance_data{$st_type}{$section};
}

sub set_data2{
  my $file = shift;
  my $section = shift;
  my $metric = shift;
  my @data = get_array_data($file);
  my %position;
  my $rounded = 0;
  my $denominator = 1;
  my $cpu_type_prefix_uid;
  my $host_name;
  my %unique_data;
  my @debug_data = ("07:24", "07:29", "07:34", "07:39", "07:44", "07:49", "07:54");

  if ($metric =~ m/resp_t/){
    $rounded = 3;
    $denominator = 1000;
  }
  if ($metric =~ m/data_rate|^write$|^read$|^rnd_write$|^rnd_read$|^seq_read$|^seq_write$/){
    $denominator = 0.001;
  }
  if ($metric eq "cache_allocated_size"){
    $denominator = 0.000001;
  }
  foreach my $line (@data){
    chomp $line;
    if ($line eq "" || $line =~ m/^"Date","Time/){next;}
    $line =~ s/"//g;
    my ( $date, $time, $element, $value ) = split( ",", $line );
    if ( !defined $date || !defined $time || !defined $element || !defined $value ) { next; }
    if (!$enviroment){
      if ($debug_data[$index_data] ne "$time"){next;}
    }
    if ($element !~ m/^CL\d-[A-Z]/ && $section eq "port"){next;}
    if ($element !~ m/^[0-9]{1,2}?-[0-9]{1,2}?$/ && $section eq "raid_group"){next;}
    if ($element !~ m/^[A-Z]{3}-[A-Z0-9]{2}\/[A-Z0-9]{2}$/ && $section eq "cpu" ){next;}
    if ($element !~ m/^[A-Z]{3}-[A-Z0-9]{2}$/ && $section eq "cache" ){next;}
    if ($element !~ m/^[0-9 A-Z][0-9 A-Z]:[0-9 A-Z][0-9 A-Z]:[0-9 A-Z][0-9 A-Z]/ && $section eq "volume"){next;}
    if ($section eq "volume" && $element =~ m/^([0-9 A-Z][0-9 A-Z]):([0-9 A-Z][0-9 A-Z]):([0-9 A-Z][0-9 A-Z])/){
      $element = "$1:$2:$3";
    }
    if ($section eq "cpu"){
      ### cache type
      my ($cache_type, undef) = split("/",$element);
      if (!grep( /^$cache_type$/, @cache_type)){
        push(@cache_type,$cache_type);
      }
      $element =~ s/\//-/g;

    }
    my $t = "$date" . "T" . "$time";
    $t =~ s/\//:/g;
    $t = $t . ":00";
    #print "$t\n";
    if ($offset_time ne ""){
      $t = $t . $offset_time;
    }
    #print "$t\n";
    my $timestamp      = str2time($t);    ### utc timestamp
    if (!$enviroment){
      $timestamp = $timestamp_global;
    }
    if (Xormon::isdigit($service_data_in->{last_perf_timestamp}) && $service_data_in->{last_perf_timestamp} > $timestamp ){
      next;
    }
    #print "$section | $element\n";
    my $uid = Xormon::create_hash_key($st_serial.$section.$element);
    if ($section eq "volume"){
      if (!defined $volumes_gad{$uid}){
        $volumes_gad{$uid}{id} = $element; 
      }
    }
    if ($section eq "cpu"){
      $configuration_data{$uid}{id} = $element;
      $configuration_data{$uid}{label} = $element;
      $configuration_data{$uid}{TYPE} = $section;
    }
    if (Xormon::isdigit($timestamp) && Xormon::isdigit($value) && $value >= 0){
      #if (defined $performance_data{$st_type}{$section}{$uid}{$timestamp}){next;} ### testing
      if (defined $unique_data{$uid} && !$enviroment ){next;} #### testing
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{$metric} = round($value / $denominator,$rounded);
      #print "nastavujeme $uid | $section | $metric | $file\n";
      $unique_data{$uid} = $value; ### testing
    }
  }
}

sub set_data{
  my $file = shift;
  my $section = shift;
  my $metric = shift;
  my @data = get_array_data($file);
  my %position;
  my $rounded = 0;
  my $denominator = 1;
  my $cpu_type_prefix_uid;
  my $host_name;
  if ($metric =~ m/resp_t/){
    $rounded = 3;
    $denominator = 1000;
  }
  if ($metric =~ m/data_rate|^write$|^read$/){
    $denominator = 0.001;
  }
  if ($metric eq "cache_allocated_size"){
    $denominator = 0.000001;
  }
  if ($section eq "cache"){
    my $filename = basename($file);
    my @filename_parse = split("_",$filename);
    $cpu_type_prefix_uid = $filename_parse[-1];
    $cpu_type_prefix_uid =~ s/\.csv//g;
    $cpu_type_prefix_uid = Xormon::create_hash_key($st_serial.$section.$cpu_type_prefix_uid);
  }
  if ($section eq "host"){
    my $filename = basename($file);
    my @filename_parse = split("_",$filename);
    $host_name = $filename_parse[1];
    $host_name =~ s/\.csv//g;
    #$host_name = Xormon::create_hash_key($st_serial.$section.$cpu_type_prefix_uid);
  }
  my $size_file = scalar @data;
  $size_file--;

  if (!$enviroment && $index_data > $size_file){
    $index_data = 6;
  }
  my $index_line = -1;
  foreach my $line (@data){
    chomp $line;
    $index_line++;
    if ($line eq ""){next;}
    $line =~ s/"//g;
    if ($line =~ m/No/){
      undef %position;
      my @header = split(",",$line);
      my $index = -1;
      foreach my $header (@header){
        $index++;
        if ($header =~ m/No|time/){
          next;
        }
        if ($header !~ m/^[0-9]{1,2}?-[0-9]{1,2}?$/ && $section eq "raid_group"){next;}
        if ($header !~ m/^CL\d-[A-Z]/ && $section eq "port"){next;}
        if ($header !~ m/^[0-9 A-Z][0-9 A-Z]:[0-9 A-Z][0-9 A-Z]:[0-9 A-Z][0-9 A-Z]/ && $section eq "volume"){next;}
        if ($section eq "volume" && $header =~ m/^([0-9 A-Z][0-9 A-Z]):([0-9 A-Z][0-9 A-Z]):([0-9 A-Z][0-9 A-Z])/){
          $header = "$1:$2:$3";
        }
        if ($header !~ m/^[A-Z]{3}-[A-Z0-9]{3}\.[A-Z0-9]{5}-[A-Z0-9]{2}/ && $section eq "cpu" ){next;}
        my $uid = Xormon::create_hash_key($st_serial.$section.$header);
        if ($section eq "volume"){
          if (!defined $volumes_gad{$uid}){
            $volumes_gad{$uid}{id} = $header; 
          }
        }
        if ($section eq "cache"){
          $uid = "$cpu_type_prefix_uid--$header";
        }
        if ($section eq "host"){
          $uid = "$host_name--$header";
        }
        if ($section eq "cpu"){
          $configuration_data{$uid}{id} = $header;
          $configuration_data{$uid}{label} = $header;
          $configuration_data{$uid}{TYPE} = $section;

        }
        #$performance_data{$st_type}{$section}{$uid}{postion} = $index;
        $position{$index} = $uid;
      }
    
    }
    else{
      
      if (%position){
        if (!$enviroment && $index_line != $index_data){next;}
        my @data_line = split(",",$line);
        $data_line[1] =~ s/\///g;
        $data_line[1] =~ s/ /T/g;
        $data_line[1] =~ s/://g;
        $data_line[1] = $data_line[1] . "00";
        if ($offset_time ne ""){
          $data_line[1] = $data_line[1] . $offset_time;
        }
        #print $data_line[1] . "\n";
        my $timestamp = str2time($data_line[1]);
        if (!$enviroment){
          $timestamp = $timestamp_global;
        }
        if (Xormon::isdigit($service_data_in->{last_perf_timestamp}) && $service_data_in->{last_perf_timestamp} > $timestamp ){
          next;
        }
        #my $gtime = gmtime($timestamp);
        #$timestamp = str2time($gtime);


        my $index = -1;
        foreach my $value (@data_line){
          $index++;
          if ($index < 1){next;}
          #print "$index | $file\n";
          #print "$baseline | $timestamp | $value " . $position{$index} . "\n";
          if (Xormon::isdigit($timestamp) && Xormon::isdigit($value) && $value >= 0 && defined $position{$index}){
            #print "$metric | $section | $value | $denominator\n";
            $performance_data{$st_type}{$section}{$position{$index}}{$timestamp}{$metric} = round($value / $denominator,$rounded);
          }
        }
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section}{"6996fd207c"};
  #print Dumper \%performance_data;
}

sub set_data_port_conf {
  my $section = "port";
  my $cmd = "v1/objects/ports";
  my $data = cmd_api($cmd,"GET","ports",1,"");
  if ( $data ne "" && defined $data->{data} && ref( $data->{data} ) eq "ARRAY" ) {
    foreach my $port ( @{ $data->{data} } ) {
      my $port_id  = $port->{portId};
      my $port_wwn = $port->{wwn};
      if ( !defined $port_id )  { next; }
      if ( !defined $port_wwn ) { $port_wwn = ""; }
      else {
        $prefix_uuid_volume = $port_wwn;
      }
      my $uid = Xormon::create_hash_key($st_serial.$section.$port_id);
      $configuration_data{$uid}{id} = $port_id;
      $configuration_data{$uid}{label} = $port_id;
      $configuration_data{$uid}{wwn} = $port_wwn;
      $configuration_data{$uid}{TYPE} = $section;

    }
  }
  #print Dumper \%configuration_data;
}


sub remove_whitespace {
  my $value = shift;
  if ( !defined $value || $value eq "" ) {
    return "";
  }
  $value =~ s/^\s+|\s+$//g;
  return $value;

}



sub delete_empty_metrics{
  my $first_time = "";
  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        foreach my $item (keys %{$performance_data{$st_type}{$type}{$uid}{$time}}){
          if (!Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item}) || $performance_data{$st_type}{$type}{$uid}{$time}{$item} < 0){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
          if ($item =~ m/numerator|denominator/ || (($item eq "type" || $item eq "count") && $type eq "raid_group")){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
        }
      }
      my $time = (( sort keys %{$performance_data{$st_type}{$type}{$uid}})[-1]);
      if ($first_time eq "" && Xormon::isdigit($time)){
        $first_time = $time;
      }
      if (Xormon::isdigit($first_time) && Xormon::isdigit($time) && $time < $first_time){
        $first_time = $time;
      }
    }
  }
  if (Xormon::isdigit($first_time)){
    $service_data_out->{last_perf_timestamp} = $first_time;
  }
  else{
    if (Xormon::isdigit($service_data_in->{last_perf_timestamp})){
      $service_data_out->{last_perf_timestamp} = $service_data_in->{last_perf_timestamp};
    }
  }
  my $section = "raid_group";
  if (defined $performance_data{$st_type}{$section}){
    foreach my $uid (keys %{$performance_data{$st_type}{$section}}){
      foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$uid}}){
        if (!defined $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"} || (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"}) && $performance_data{$st_type}{$section}{$uid}{$timestamp}{"io_rate"} == 0)){
          delete $performance_data{$st_type}{$section}{$uid}{$timestamp};
          next;
        }
      }
      if (keys %{$performance_data{$st_type}{$section}{$uid} } == 0){
        delete $performance_data{$st_type}{$section}{$uid};
      }
    }
  }
  if (keys %{$performance_data{$st_type}{$section} } == 0){
    delete $performance_data{$st_type}{$section};
    return;
  }
}

sub calculate_missing_metrics{

  my $item_total = "capacity_total";
  my $item_used = "capacity_used";
  my $item_used_percent = "capacity_used_percent";

  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        #capacity_used_percent
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_used})){
          if ($performance_data{$st_type}{$type}{$uid}{$time}{$item_total} == 0){
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = 0;
          }
          else{
            my $pom = ($performance_data{$st_type}{$type}{$uid}{$time}{$item_used} / $performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) * 100;
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = round($pom,2);
          }
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{data_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && $performance_data{$st_type}{$type}{$uid}{$time}{io_rate} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{data_rate} / $performance_data{$st_type}{$type}{$uid}{$time}{io_rate});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{read_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{read} / $performance_data{$st_type}{$type}{$uid}{$time}{read_io});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{write_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{write} / $performance_data{$st_type}{$type}{$uid}{$time}{write_io});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = 0;
        }

      }
    }
  }
}

sub cmd_api{
  #cmd_api("prikaz", "GET", "file", "1", "json_body");
  my $cmd = shift;
  my $type = shift;
  my $debug_file = shift;
  my $execute = shift;
  my $hash_tmp = shift;
  my $login = shift;
  my %hash_data;
  my $json_body;
  if (defined $hash_tmp && $hash_tmp ne ""){
    %hash_data = %{$hash_tmp};
    $json_body = encode_json( \%hash_data );
  }
  if (!defined $login){
    $login = 0;
  }
  if (!defined $type || $type eq ""){$type = "GET";}
  if (!defined $execute || $execute eq ""){
    $execute = 1;
  }
  if (!$enviroment){
    $execute = 0; ### testing
    $login = 0;   ### testing
  }

  $debug_file =~ s/\//-/g;
  #print "$debug_data/$debug_file.json\n";
  if (!$execute && defined $debug_file && $debug_file ne "" && -f "$debug_data/$debug_file.json" && !$login){
    my $data;
    my $file = "$debug_data/$debug_file.json";
    eval { $data = Xormon::read_json($file); };
    if ($@){
      $logger->error("Read file json $file is failed");
      return "";
    }
    else{
      return $data;

    }
  }
  my $url = "$protocol://$api_ip:$api_port/ConfigurationManager/$cmd";

  if ($cfg->{'exportRadio'} eq "manager"){
    my $cmd_new = $cmd;
    if ($cmd ne "v1/objects/storages" && $cmd ne "v1/objects/storages/$cfg->{'hcmDeviceId'}"){
      $cmd_new =~ s/v1\/objects/v1\/objects\/storages\/$cfg->{'hcmDeviceId'}/g;
    }
    if($cmd =~ m/sessions/ && $type eq "POST"){
      $cmd_new = "v1/objects/storages/$cfg->{'hcmDeviceId'}/sessions";
    }
    if ($type eq "DELETE"){
      $cmd_new = "v1/objects/storages/$cfg->{'hcmDeviceId'}/sessions/$session_id";
    }
    $url = "$cfg->{'hcmProto'}://$cfg->{'hcmHost'}:$cfg->{'hcmPort'}/ConfigurationManager/$cmd_new";
  }


  my $ua = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( $type => $url );
  $req->header( 'Content-Type'   => 'application/json' );
  $req->header( 'Accept'         => 'application/json' );
  $req->header( 'Connection'     => 'keep-alive' );
  $req->header( 'Content-Length' => '0' );
  if ($login){
    $req->authorization_basic( $cfg->{'apiUsername'}, $cfg->{'apiPassword'} );
  }
  else {
    if (defined $token && $token ne ""){
      $req->header( 'authorization' => "Session $token" );
    }
    else{
      if ($cmd =~ m/storages\/|snmp/){
        my $user = $cfg->{'apiUsername'};
        my $passwd = $cfg->{'apiPassword'};
	      $req->authorization_basic( $cfg->{'apiUsername'}, $cfg->{'apiPassword'} );
      }
      ### storages list all devices without authorization
    }
  }
  if (defined $json_body){
    $req->content($json_body);
  }
  #print Dumper \$req;
  #return "";
  my $res = $ua->request($req);
  if ( ! $res->is_success ) {
    if (!$connection_test){
      $logger->error("API command $url failed");
    }
    else{
      push(@rows_api_test,"API command $url failed\n");
      push(@rows_api_test_data,"API command $url failed\n");
      push(@rows_api_test,$res->content);
      push(@rows_api_test_data,$res->content);

    }
    my $out = $res->content;
    $logger->error($out);
    return "";
  }
  #my $out = $res->content;
  if ($type eq "DELETE"){
    $session_id = "";
    $token = "";    
    return 1;	  
  }
  my $data;
  eval {$data = decode_json( $res->content );};
  if ($@){
    if (!$connection_test){
      $logger->error("Decode json ($url) failed");
      my $out = $res->content;
      $logger->error($out);
    }
    else{
      push(@rows_api_test,"Decode json ($url) failed\n");
      push(@rows_api_test_data,"Decode json ($url) failed\n");
      push(@rows_api_test,$res->content);
      push(@rows_api_test_data,$res->content);
    }
    return "";
  }
  if (defined $debug_file && $debug_file ne ""){
    eval {Xormon::save_as_json("$debug_data/$debug_file.json",$data);};
    if ($@){
      $logger->error("Save data to file $debug_data/$debug_file.json failed");
    }
  }
  return $data;
}

sub set_architecture{

  my %device_arch = (
    "item_id" => $cfg->{'uuid'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => $st_type,
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'uuid'}
  );
  #if (defined $cfg->{serial_number}){
  #  $device_arch{serial_number} = $cfg->{serial_number};
  #}
  #if (defined $cfg->{version}){
  #  $device_arch{version} = $cfg->{version};
  #}
  #if (defined $cfg->{model}){
  #  $device_arch{model} = $cfg->{model};
  #}
  push(@architecture, \%device_arch);


  #print Dumper \%configuration_data;
  #return;

  foreach my $uid (keys %configuration_data){
    my $name = $configuration_data{$uid}{label};
    my $type = $configuration_data{$uid}{TYPE};
    if (!defined $name || !defined $type){
      if ($uid eq $st_serial){next;}
      delete $configuration_data{$uid};	    
      $logger->error("Element $uid has not label or type!");
      next;
    }
    my @children = ();
    my @parents = ();
    if (defined $configuration_data{$uid}{children}){
      @children = @{$configuration_data{$uid}{children}};
      my @child = ();
      foreach my $child (@children){
        if (defined $configuration_data{$child}){
	        push(@child,$child);	  
	      }
      }
      ## delete children and parent from item_properties
      @children = @child;
      delete $configuration_data{$uid}{children};
    }
    if (defined $configuration_data{$uid}{parents}){
      @parents = @{$configuration_data{$uid}{parents}};
      my @par = ();
      foreach my $parent (@parents){
        if (defined $configuration_data{$parent}){
          push(@par,$parent);
        }
      }
      @parents = @par;
      delete $configuration_data{$uid}{parents};
    }
    #print Dumper \@children;
    #print Dumper \@parents;
    my %arch = (
      "item_id" => $uid,
      "label" => $name,
      "hw_type" => $st_type,
      "subsystem" => $type,
      "hostcfg_id" => $cfg->{'uuid'},
      "parents" => [$cfg->{'uuid'}]
      );
    if (@parents){
      push(@{$arch{parents}},@parents);
    }
    if (@children){
      push(@{$arch{children}},@children);
    }
    #if ($type eq "volume"){
      # WWN for mapping
    #  $arch{wwn} = Xormon::formatWWN($uid);
    #}
    if (($type eq "port" || $type eq "volume") && defined $configuration_data{$uid}{wwn}){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($configuration_data{$uid}{wwn});
      $configuration_data{$uid}{wwn} = Xormon::formatWWN($configuration_data{$uid}{wwn});
    }
    #push(@{$arch{parents}},$cfg->{'uuid'});
    push(@architecture, \%arch);

  }
  #print Dumper \@architecture;
  #foreach my $item (keys %{$data{conf}{device}}){
  #  $configuration_data{$st_serial}{$item} = $data{conf}{device}{$item};
  #}
  #print Dumper \$configuration_data{device}{$st_serial};

  #save configuration
  my @keys = ("serial", "version", "model", "ctla", "machine_name", "ctlb");
  foreach my $key (@keys){
    if (defined $cfg->{$key}){
      if ($key eq "machine_name"){
        $configuration_data{$cfg->{'uuid'}}{"node_name"} = $cfg->{$key};
      }
      else{
        $configuration_data{$cfg->{'uuid'}}{$key} = $cfg->{$key};
      }
    }
  }

  #event log
  $configuration_data{$cfg->{'uuid'}}{alerts} = encode_json(\@health_status);
  #print Dumper \$configuration_data{$cfg->{'uuid'}}{alerts};
  #print Dumper \@architecture;
  ### device information save

  #print Dumper \%configuration_data;
  my $rc = $xormon->saveConfJSON(\%configuration_data);
  my $status_configuration = get_status($rc, "saveConfJSON (configuration)");
  #$logger->info("Task for configuration ".$cfg->{hostalias}." is done. Status: $status");

  #save architecture
  #print Dumper \@architecture;

  $rc = $xormon->saveArchitecture(\@architecture);
  my $status_architecture = get_status($rc, "saveArchitecture");
  #$logger->info("Task for saveArchitecture ".$cfg->{hostalias}." is done. Status: $status");

  #print Dumper \@hs_global;
  $rc = $xormon->saveStatus(\@hs_global);
  my $status_health = get_status($rc, "health status");

  return ($status_architecture, $status_configuration, $status_health);
}


sub connection_test {
  my $client = shift;
  $cfg = shift;
  $connection_test = 1;
  my %tcp_test;
  #print Dumper \$cfg;

  my $test = Test->new($client);
  my @variables = ("hostcfg_id", "hostalias", "node1Host", "restProto", "apiUsername", "apiPassword", "svpHost", "exportRadio");
  if (defined $cfg->{'exportRadio'}  && $cfg->{'exportRadio'} eq "manager"){
    push(@variables,"hcmHost");
    push(@variables,"hcmPort");
    push(@variables,"hcmProto");
    push(@variables,"hcmDeviceId");

  }
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
  $logger = XormonLogger->new( 'vspg', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  $storage_name = $cfg->{hostalias};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $st_serial = $cfg->{'uuid'};
  $protocol = $cfg->{'restProto'};
  data_structure();

  if ($protocol eq "https"){
    $api_port = 443;
  }
  if (defined $cfg->{exportToolHome}){
    $vsp_clidir = $cfg->{exportToolHome};
  }
  else{
    $vsp_clidir = "/opt/hds";
  }
  $api_ip = $cfg->{'node1Host'};

  push(@ips,$cfg->{'node1Host'});
  if (defined $cfg->{'node2Host'} && $cfg->{'node2Host'} ne "" && !grep {/^$cfg->{'node2Host'}$/} @ips){
    push(@ips,$cfg->{'node2Host'});
  }
  if (!grep {/^$cfg->{'svpHost'}$/} @ips){
    push(@ips,$cfg->{'svpHost'});
  }
  if (!defined $vsp_clidir){
    $vsp_clidir = "/opt/hds";
  }
  if ($cfg->{'exportRadio'} eq "manager"){
    @ips = ();
    push(@ips,$cfg->{"hcmHost"});
  }
  ### Install java
  my $java_test=`java -version 2>&1 | egrep version`;
  if ($java_test eq ""){
    push(@rows_api_test,"Java not installed!");
    $test->addSection("Install Java", 0, \@rows_api_test);
    return $test->stop();
  }
  push(@rows_api_test,"Java installed.\n$java_test");
  $test->addSection("Install Java", 1, \@rows_api_test);
  ###
  
   #### tcp test
  my $tcp_text = "";
  my $tcp_test = 1;
  if ($export_tool_version == 1){
    push(@{$tcp_test{$cfg->{'svpHost'}}},1099);
    push(@{$tcp_test{$cfg->{'svpHost'}}},51099);
    push(@{$tcp_test{$cfg->{'svpHost'}}},51100);
    push(@{$tcp_test{$cfg->{'svpHost'}}},51101);
    push(@{$tcp_test{$cfg->{'svpHost'}}},11099);

    if ($cfg->{'exportRadio'} eq "manager"){
      push(@{$tcp_test{$cfg->{"hcmHost"}}},$cfg->{"hcmPort"});
    }
    else{
      push(@{$tcp_test{$api_ip}},$api_port);
    }
  }
  else{
    if ($cfg->{'exportRadio'} eq "manager"){
      push(@{$tcp_test{$cfg->{"hcmHost"}}},$cfg->{"hcmPort"});
    }
    else{
      push(@{$tcp_test{$api_ip}},$api_port);
    }
  }
  foreach my $ip (keys %tcp_test){
    foreach my $port (@{$tcp_test{$ip}}){
      my ($code, $text) = Xormon::conntest_tcp($ip,$port);
      $tcp_text = $tcp_text . "$text\n";
      if ($code == 0){
        $tcp_test = 0;
      }
    }
  }
  if ($tcp_test == 0){
    $test->addSection("API host network connection (Warning)", 1, $tcp_text);
  }
  else{
    $test->addSection("API host network connection", $tcp_test, $tcp_text);
  }


  ###########


  ### Install export tool
  (my $serial_number, my $version_tmp) = get_version_serial();
  if (!defined $serial_number || $serial_number eq "" || !defined $version_tmp || $version_tmp eq ""){
    if ($cfg->{'exportRadio'} eq "manager"){
      push(@rows_api_test, "Serial number or version export tool is not defined");
      $test->addSection("Install Export tool", 0, \@rows_api_test);
      return $test->stop();
    }
    ### check vsp ip
    $api_ip = $cfg->{'svpHost'};
    ($serial_number, $version_tmp) = get_version_serial();
    if (!defined $serial_number || $serial_number eq "" || !defined $version_tmp || $version_tmp eq ""){
      push(@rows_api_test, "Serial number or version export tool is not defined");
      $test->addSection("Install Export tool", 0, \@rows_api_test);
      return $test->stop();
    }
  }
  my ($version, undef) = split("/",$version_tmp);
  if (!-d "$vsp_clidir/$version"){
    $version = get_version_export_tool($version);
    if (!-d "$vsp_clidir/$version"){
      $test->addSection("Install Export tool", 0, "Version export tool $vsp_clidir/$version is not installed");
      return $test->stop();
    }
  }
  if (check_export_version2("$vsp_clidir/$version")){
    $export_tool_version = 2;
  }
  $test->addSection("Install Export tool", 1, "install: $vsp_clidir/$version\nversion: $export_tool_version");
  ####

  ### API test
  foreach my $ip (@ips){
    $api_ip = $ip;
    set_token_and_session_id();
    if (defined $token && $token ne "" && defined $session_id && $session_id ne ""){last;}
  }
  if (!defined $token || $token eq "" || !defined $session_id || $session_id eq ""){
    push(@rows_api_test,"Session id or token is not defined!\n");
    $test->addSection("API authorization", 0, \@rows_api_test);
    return $test->stop();
  }
  set_data_device_conf();
  my @configuration_data_header = get_configuration_header();
  foreach my $element (@configuration_data_header){
    #print Dumper \$element;
    foreach my $key (keys %{$element}){
      if (defined $cfg->{$key}){
        push(@rows_api_test,"$element->{$key}: $cfg->{$key}\n");
      }
    }
  }

  #print Dumper \$cfg;
  $test->addSection("API authorization", 1, \@rows_api_test);
  ###

  ### API TEST DATA (Export tool)
  if ($export_tool_version == 1){
    my $command = get_command_export_tool("$vsp_clidir/$version", $serial_number);
    if ($command eq "0"){
      cmd_api("v1/objects/sessions/$session_id", "DELETE", "v1/objects/sessions",1, "", 0);
      $test->addSection("Export tool data test", 0, \@rows_api_test_data);
      return $test->stop();
    }
    if (!cmd_export_tool("$command",600)){
      cmd_api("v1/objects/sessions/$session_id", "DELETE", "v1/objects/sessions",1, "", 0);
      foreach my $line (@rows_api_test_data){
        chomp $line;
        if ($line =~ m/sun.security.x509/i){
          push(@rows_api_test_data,"\n\nJava security error (SSL / X509). Install a version of java older than 16. We recommend java 11.\n\n");
          last;
        }
      }
      $test->addSection("Export tool data test", 0, \@rows_api_test_data);
      return $test->stop();
    }
    $test->addSection("Export tool data test", 1, "Performance data ok");

  }
  else{
    if (!get_performance_statistics2("$vsp_clidir/$version")){
      $test->addSection("Export tool data test", 0, \@rows_api_test_data);
      return $test->stop();
    }
    $test->addSection("Export tool data test", 1, "Performance data ok");
  }
  cmd_api("v1/objects/sessions/$session_id", "DELETE", "v1/objects/sessions",1, "", 0);
  ###

  return $test->stop();

}
sub get_configuration_header{
  my @data;
  push(@data, {'machine_name' => "Machine Name"});
  push(@data, {'model' => "Machine Type-Model"});
  push(@data, {'ctla' => "IP address controller A"});
  push(@data, {'ctlb' => "IP address controller B"});
  push(@data, {'version' => "Software Version"});
  push(@data, {'serial' => "Machine Serial"});
  return @data;

}

sub data_structure {
  if (!-d "$data_directory/data"){
    mkdir "$data_directory/data" || $logger->error( "Cannot mkdir $data_directory/data: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial"){
    mkdir "$data_directory/data/$st_serial" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial/IOSTATS"){
    mkdir "$data_directory/data/$st_serial/IOSTATS" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial/IOSTATS: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }


  my $directory = "$data_directory/data/$st_serial/IOSTATS";
  if ( !-d $directory ) {
    mkdir $directory || $logger->error( "Cannot mkdir $directory: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if ( !-d "$directory/configuration" ) {
    mkdir "$directory/configuration" || $logger->error( "Cannot mkdir $directory/configuration: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  for ( my $i = 0; $i < 60; $i = $i + 5 ) {
    if ( $i < 10 ) {
      if ( !-d "$directory/0$i" ) {
        mkdir "$directory/0$i" || $logger->error( "Cannot mkdir $directory/0$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
    else {
      if ( !-d "$directory/$i" ) {
        mkdir "$directory/$i" || $logger->error( "Cannot mkdir $directory/$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
  } ## end for ( my $i = 0; $i < 60...)
  my @entry_time = split( " ", localtime() );
  ( undef, $active_minute, undef ) = split( ":", $entry_time[3] );
  if ( !-d "$directory/$active_minute" ) {
    mkdir "$directory/$active_minute" || $logger->error( "Cannot mkdir $directory/$active_minute: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  $debug_data = "$directory/configuration";
  $wrkdir = "$data_directory/data/$st_serial";
  $performance_data_store = "$directory/$active_minute";
  if (!$enviroment){
    $performance_data_store = "$directory/00";
  }
  if ($enviroment){
    `rm -r $performance_data_store/* 2>/dev/null`;
  }


} ## end sub data_structure

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

sub set_token_and_session_id{

  my $data = cmd_api("v1/objects/sessions", "POST", "v1/objects/sessions",1, "", 1);
  if (defined $data && $data ne "" && defined $data->{token} && defined $data->{sessionId}){
    $token = $data->{token};
    $session_id = $data->{sessionId};
  }
}

sub get_performance_statistics2{
  my $dir = shift;

  my $export_tool_dir = "$dir";
  if (-f "$wrkdir/runUnix.sh"){
    $export_tool_dir = "$wrkdir";
  }

  create_export_tool_properties($dir);
  bash_file_export_tool2($dir);

  my $sample_rate = get_sample_rate2($export_tool_dir);
  if (!Xormon::isdigit($sample_rate)){
    if ($connection_test == 1){
      push(@rows_api_test_data,"Sample rate is not defined $sample_rate\n");
      return 0;
    }
    else{
      $logger->error("Sample rate is not defined $sample_rate ") && return 0;
    }
  }
  elsif($sample_rate != $sample_rate_global){
    if (!set_sample_rate($export_tool_dir)){
      if ($connection_test == 1){
        push(@rows_api_test_data,"Sample rate cannot be set\n");
        return 0;
      }
      else{
        $logger->error("Sample rate cannot be set") && return 0;
      }
    }
  }
  my ($start_time, $end_time) = get_time_range($export_tool_dir);
  if ($start_time eq "0"){return 0;}
  return cmd_export_tool("-range $start_time:$end_time -option nocompress",7200,$export_tool_dir)

}

sub set_sample_rate{
  my $dir = shift;
  my $cmd = "set interval";
  my $data = cmd_export_tool($cmd,60,$dir);
  return $data;

}

sub get_time_range{
  my $dir = shift;
  my $cmd = "show range";
  my $data = cmd_export_tool($cmd,60,$dir);
  if (!$enviroment){
    return ("202309010000", "202309020000")
  }
  if (defined $data && ref($data) eq "ARRAY"){
    my $size = scalar @{$data};
    if ( !Xormon::isdigit($size) || $size < 1 ) {
      if ($connection_test == 1){
        push(@rows_api_test_data,"Export tool 2 does not provide historical data!\n");
        return 0;
      }
      else{
        $logger->error("Export tool 2 does not provide historical data!") && return 0;
      }
    }
    pop(@{$data});
    pop(@{$data});
    $size = scalar @{$data};
    my $diff = 300;
    if (Xormon::isdigit($service_data_in->{last_perf_timestamp})){
      $diff = time() - $service_data_in->{last_perf_timestamp};
    }
    my $count_data = ceil( $diff / ($sample_rate_global * 60) );
    if ($count_data == 0){
      $count_data = 1;
    }
    if ($connection_test == 1){
      $count_data = 1;
    }
    if ($count_data > 24){
      $count_data = 6;
    }
    my $start_interval_time = 2;
    $start_interval_time = $start_interval_time + $count_data;

    if ( $count_data > 1 && !defined $data->[ $size - $start_interval_time ] ) {
      if ($connection_test == 1){
        push(@rows_api_test_data,"Export tool 2 does not provide historical data $count_data * 5 minutes! $!\n");
      }
      else{
        $logger->error("Export tool 2 does not provide historical data $count_data * 5 minutes! $!");
      }
      $start_interval_time = 3;
    }
    if ( !defined $data->[-1] || !defined $data->[ $size - $start_interval_time ] ) {
      #$logger->info($data);
      if ($connection_test == 1){
        push(@rows_api_test_data,"Export tool 2 does not provide historical data!! $!\n");
        return 0;
      }
      else{
        $logger->error("Export tool 2 does not provide historical data!! $!") && return 0;
      }
    }
    my $start_time = remove_whitespace( $data->[ $size - $start_interval_time ] );
    my $end_time   = remove_whitespace( $data->[-1] );
    my ( $prefix, $postfix )   = split( " ", $start_time );
    my ( $year, $month, $day ) = split( "/", $prefix );
    my ( $hour, $minute )      = split( ":", $postfix );
    $start_time = "$year$month$day$hour$minute";
    $end_time =~ s/\//:/g;
    $end_time =~ s/ /T/g;
    my $time = str2time($end_time);
    $time     = $time + 60;
    $end_time = get_time($time);
    $end_time =~ s/://g;
    $end_time =~ s/T//g;
    chop $end_time;
    chop $end_time;
    return ($start_time, $end_time);
    #print Dumper \@{$data};
  }
  return 0;
}


sub get_sample_rate2{
  my $dir = shift;
  my $cmd = "show interval";
  my $data = cmd_export_tool($cmd,60,$dir);
  if (!$enviroment){
    return 5;
  }
  if (defined $data && ref($data) eq "ARRAY"){
    foreach my $line (@{$data}) {
      chomp $line;
      $line = remove_whitespace($line);
      if ( $line eq "" ) { next; }
      if ( $line =~ m/^Interval/ ) {
       #Interval : 5 min
        my ( undef, $val ) = split( ":", $line );
        $val =~ s/min//g;
        $val = remove_whitespace($val);
        return $val;
      }
    }
  }
  if (defined $data && $data == 0){
    return "";
  }
  return "";
}


sub check_export_version2{
  my $dir = shift;
  if ( -f "$dir/runUnix.sh" && -f "$dir/exportTool.properties" && -f "$dir/exportTool.jar" && -f "$dir/Readme.txt") {
    my @data = get_array_data("$dir/Readme.txt");
    my $test = grep {/Export Tool 2/} @data;
    if ($test) {
      return 1;
    }

  }
  return 0;

}

sub get_run_bash_file{
  my $export_tool_dir = shift;
  my @files = ("$export_tool_dir/command.txt", "$export_tool_dir/runUnix.bat");
  my $run_file;

  foreach my $file (@files){
    if (!-f "$file" || -z "$file"){
      if ($file =~ m/runUnix.bat/ ){
        $file = "$export_tool_dir/runUnix.sh";
        if (!-f $file || -z $file){
          if ($connection_test){
            push(@rows_api_test_data,"File $file does not exist\n");
            return 0;
          }
          else{
            $logger->error("File $file does not exist") && return 0;
          }
        }
        else{$run_file = $file;last;}
      }
      if ($connection_test){
        push(@rows_api_test_data,"File $file does not exist\n");
        return 0;
      }
      else{
        $logger->error("File $file does not exist") && return 0;
      }
    }
  }
  if (!defined $run_file){
    $run_file = $files[1];
  }
  return $run_file;
}

sub get_command_export_tool{
  my $export_tool_dir = shift;
  my $serial_number = shift;
  my $run_file = get_run_bash_file($export_tool_dir);
  if (!defined $run_file || $run_file eq "0"){return 0;}

  my $command_file = create_command_file("$export_tool_dir/command.txt", $serial_number);
  if (!-f $command_file){return 0;}
  my $command = "";
  foreach my $line (get_array_data($run_file)) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^java/ ) {
      my $line_pom = $line;
      $line     =~ s/command\.txt/$command_file/g;
      $line     =~ s/\.\/lib/$export_tool_dir\/lib/g;
      $line     =~ s/=log/=$wrkdir\/log/g;
      $line     =~ s/-Del.secmode=.* //g;
      $command  = $line;
    }
  }
  if (!$enviroment){
    $command = "java -classpath $export_tool_dir/ Simple";
  }
  #my $java = "/opt/java/bin:/usr/java8_64/jre/bin:/usr/java8_32/jre/bin:/usr/bin";
  #my $java = "/etc/java/java-1.8.0-openjdk/";
  #my $java_whereis = `whereis java`;
  #$java_whereis =~ s/^java://g;
  #$java_whereis =~ s/ /:/g;
  #chomp $java_whereis;
  #$java = $java . "$java_whereis";
  #$ENV{PATH} = "$java:$path";


  #my $arg1 = "-Djava.security.egd=file:///dev/urandom -Dgraal.VectorizeSIMD=false -Djavax.net.ssl.trustStoreType=jks -Djavax.net.ssl.trustStore=/etc/pki/java/cacerts";
  #my $arg2 = "--add-exports java.base/sun.security.util=ALL-UNNAMED --add-exports java.base/sun.security.x509=ALL-UNNAMED";
  #$command =~ s/^java/java $arg2/;
  #$command =~ s/-Del.security=ignore/-Del.security=verify/g;
  #print "$command\n";
  return $command;

}

sub get_performance_statistics{

  (my $serial_number, my $version_tmp) = get_version_serial();
  if (!defined $serial_number || $serial_number eq "" || !defined $version_tmp || $version_tmp eq ""){
    $logger->error("Serial number or version export tool is not defined") && return 0;
  }
  my ($version, undef) = split("/",$version_tmp);
  if (!-d "$vsp_clidir/$version"){
    $version = get_version_export_tool($version);
    if (!-d "$vsp_clidir/$version"){
      $logger->error("Version export tool $vsp_clidir/$version is not installed") && return 0;
    }
  }

  ### export tool 2
  if (check_export_version2("$vsp_clidir/$version")){
    $export_tool_version = 2;
    return get_performance_statistics2("$vsp_clidir/$version");
    #return 0;
  }
  ###
  my $command = get_command_export_tool("$vsp_clidir/$version", $serial_number);
  if ($command eq "0"){return 0;}
  #print "$command\n";

  #print "$command\n";
  #`rm -r $performance_data_store/* 2>/dev/null`;
  return cmd_export_tool($command,7200);

}
sub create_command_file{
  my $file = shift;
  my $serial_number = shift;
  my @data = get_array_data($file);
  my $test = grep {/svpip/} @data;
  my $type_storage = "";
  my $command = ";";
  if ($test) {
    $type_storage = 0;
  }
  else {
    $type_storage = 1;
  }
  my $vspg_sample = get_time_range_export_tool1();

  my @commands = ("PhyMPU", "PhyMainPK", "PhyCMPK", "PhyESW");
  foreach my $cmd (@commands){
    if (grep {/$cmd/} @data){
      $command = "group $cmd Short";
      if ($command =~ m/PhyMPU/){
        $command = "group $cmd";
      }
      last;
    }
  }
  my @command_data = ("");
  if ($type_storage){
    push(@command_data,"ip $cfg->{svpHost}; Specifies IP adress of SVP\n");
    push(@command_data, "dkcsn $serial_number; Specifies Serial Number of SVP\n");

  }
  else{
    push(@command_data,"svpip $cfg->{svpHost}; Specifies IP adress of SVP\n");
  }
  push(@command_data, "login \"$cfg->{'apiUsername'}\" \"$cfg->{'apiPassword'}\"; Logs user into DKC\n");
  push(@command_data, "show\n");
  push(@command_data, "group Port ; Port usage\n");
  if (!$connection_test){
    push(@command_data, "group PG ; Parity Group Statistics\n");
    push(@command_data, "group LDEVEachOfCU virtual ; LDEV usage in CUs\n");
    push(@command_data, "group PortWWN ; Stats for HBAs connected to ports\n");
    push(@command_data, "group LDEV ; LDEV statistics\n");
  }
  if ($type_storage){
    if (!$connection_test){
      push(@command_data, "$command; Access Paths and Write Pending\n");
      push(@command_data, "group PhyProc ; Micro-Processor usage\n");
    }
    push(@command_data, "range $vspg_sample\n");
  }
  else{
    if (!$connection_test){
      push(@command_data,"$command; Access Paths and Write Pending\n");
      push(@command_data,"group PhyProc Short ; Micro-Processor usage\n");
    }
    push(@command_data,"shortrange $vspg_sample\n");
  }
  push(@command_data, "outpath \"$performance_data_store\"  ; specifies the sub-directory in which files will be saved\n");
  push(@command_data, "option nocompress\n");
  push(@command_data, "apply\n");

  write_to_file(\@command_data,"$data_directory/data/$st_serial/command_$active_minute.txt");
  #print Dumper \@command_data;
  return "$data_directory/data/$st_serial/command_$active_minute.txt";

}

sub write_to_file{
  my $data_tmp = shift;
  my $file = shift;
  if (ref($data_tmp) eq "ARRAY"){
    my @data = @{$data_tmp};
    open( DATA, ">$file" ) or $logger->error("Cannot open file: $file : $!") && return;
    print DATA @data;
    close DATA;
  }
  else{
    open( DATA, ">$file" ) or $logger->error("Cannot open file: $file : $!") && return;
    print DATA $data_tmp;
    close DATA;
  }
}

sub get_array_data {
  my $file = shift;
  if (-f $file) {
    open( FH, "< $file" ) || $logger->error( "Cannot read $file: $!" . __FILE__ . ":" . __LINE__ ) && return ();
    my @file_all = <FH>;
    close(FH);
    return @file_all;
  }
  else {
    $logger->error( "File not exist $file: $!" . __FILE__ . ":" . __LINE__ );
    return ();
  }
}

sub get_version_serial {
  my $cmd     = "v1/objects/storages";
  my $data    = cmd_api($cmd, "GET", $cmd, 1, "", 0);
  my $serial  = "";
  my $version = "";
  if ( $data ne "" && defined $data->{data} && ref( $data->{data} ) eq "ARRAY" ) {
    foreach my $element ( @{ $data->{data} } ) {
      $serial = $element->{serialNumber};
      if ( !defined $serial ) { $serial = ""; }
      my $device_id = $element->{storageDeviceId};
      if ( defined $device_id ) {
        if ( $cfg->{'exportRadio'} eq "manager" && $device_id ne $cfg->{'hcmDeviceId'}) { next; }
        my $cmd                 = "v1/objects/storages/$device_id";
        my $export_tool_version = cmd_api($cmd, "GET", $cmd, 1, "", 0);
        if ( $export_tool_version ne "" && defined $export_tool_version->{dkcMicroVersion} ) {
          $version = $export_tool_version->{dkcMicroVersion};
        }
      }
    }
  }
  return ($serial, $version);
}

sub get_version_export_tool {
  my $version         = shift;
  my $global_diff     = "";
  my $export_tool_new = "";

  my ( $val1, $val2, $val3 ) = split( "-", $version );
  my $short_version = "$val1-$val2";
  foreach my $dir (<$vsp_clidir/$short_version*>) {
    my ( undef, undef, $last_val ) = split( "-", $dir );
    my $diff = abs( $last_val - $val3 );
    if ( $global_diff ne "" ) {
      if ( $diff < $global_diff ) {
        $global_diff     = $diff;
        $export_tool_new = "$short_version-$last_val";
      }
    }
    else {
      $global_diff     = $diff;
      $export_tool_new = "$short_version-$last_val";
    }
  }
  if ($export_tool_new eq ""){
    return $version;
  }
  return $export_tool_new;

}

sub cmd_export_tool {

  my $cmd     = shift;
  my $timeout = shift;
  my $dir = shift;
  if ($cmd !~ m/^java/){
    my $cmd_new = "cd $dir; chmod a+x runUnix.sh; $run_unix $cmd -ip $api_ip -login ". $cfg->{'apiUsername'} . " " .  $cfg->{'apiPassword'};
    if ($cmd eq "set interval"){
      $cmd_new = $cmd_new . " -interval $sample_rate_global";
    }
    if ($cmd =~ m/-option nocompress/){
      $cmd_new = "cd $dir; chmod a+x runUnix.sh; $run_unix export data -ip $api_ip -login ". $cfg->{'apiUsername'} . " " .  $cfg->{'apiPassword'} . " -outpath $performance_data_store $cmd";
    }
    if (!$enviroment){
      $cmd_new = "java -classpath $dir GetInterval";
      if ($cmd =~ m/show range/){
        $cmd_new = "java -classpath $dir GetRange";
      }
    }
    $cmd = $cmd_new;
  }
  if (!$enviroment){
    $cmd = "ls -l";
  }
  my ( $fh, $filename ) = tempfile( UNLINK => 1 );
  my $pid = fork();
  if ( !defined $pid ) {
    $logger->error( "no fork $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  elsif ( $pid == 0 ) {
    setpgrp( 0, 0 );
    #`$cmd 2>>$filename`;
    if (defined $dir){
      `$cmd >> $filename 2>&1;echo \$? >>$filename`;
    }
    else{
      `$cmd 2>>$filename;echo \$? >>$filename`;
    }

    #if ( -f $filename){
    #  my @data = get_array_data($filename);
    #
    #}
    #if ( $? != 0 ) {
    #  write_to_file( $filename, "$?" );
    #  $logger->error( "command failed: $cmd $? " . __FILE__ . ":" . __LINE__ );
    #  my $proc = `cat $filename`;
    #  $logger->error("$proc");
    #}
    exit(0);
  }
  eval {
    local $SIG{ALRM} = sub { die "die in SIGALRM"; };
    alarm($timeout);
    waitpid( $pid, 0 );
    alarm(0);
  };
  if ( $@ =~ /die in SIGALRM/ ) {
    my $act_time = localtime();
    kill -15, $pid;
    sleep 1;
    kill -9, $pid;
    waitpid( $pid, 0 );
    if ($connection_test){
      push(@rows_api_test_data, "command $cmd timed out after : $timeout seconds $timeout seconds pid $pid proc : $$\n");
      return 0;
    }
    else{
      $logger->error(" command $cmd timed out after : $timeout seconds $timeout seconds pid $pid proc : $$") && return 0;
    }
  }
  if ( -f $filename){
    my @data = get_array_data($filename);
    #print Dumper \@data;
    if (defined $data[-1]){
      chomp $data[-1];
    }
    if ( $data[-1] ne "0" ) {
      if ($connection_test){
        push(@rows_api_test_data, "command failed: $cmd code: " . $data[-1] . " " .  __FILE__ . ":" . __LINE__ . "\n");
        push(@rows_api_test_data,@data);
      }
      else{
        $logger->error( "command failed: $cmd code: " . $data[-1] . " " .  __FILE__ . ":" . __LINE__ );
        $logger->error(@data);
      }
      return 0;
    }
    if (defined $dir ){ ### show interval and show range
      return \@data;
    }
  }
  return 1;
}

sub basename {
  my $full = shift;
  my $out  = "";

  # basename without direct function
  my @base = split( /\//, $full );
  foreach my $m (@base) {
    $out = $m;
  }

  return $out;
}

sub create_export_tool_properties {
  my $dir = shift;

  my $file = "$dir/exportTool.properties";
  if ( -f "$dir/exportTool.properties.backup" ) {
    $file = "$dir/exportTool.properties.backup";
  }
  my $file_out = "$wrkdir/exportTool.properties";

  my @data          = get_array_data($file);
  my @data_new      = ();
  my $csv_header    = 0;
  my $log_level     = 0;
  my $log_dir       = 0;
  my $http_protocol = 0;
  my $port_header   = 0;
  my $port          = 443;
  my $protocol_uc = uc($protocol);

  if ( Xormon::isdigit($export_tool_port) ) {
    $port = $export_tool_port;
  }
  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }

    if ( $line =~ m/^exportdata.protocol/ ) {
      $http_protocol = 1;
      push( @data_new, "exportdata.protocol=$protocol_uc\n" );
      next;
    }
    if ( $line =~ m/^exportdata.portNumber/ ) {
      $port_header = 1;
      push( @data_new, "exportdata.portNumber =$port\n" );
      next;
    }

    if ( $line =~ m/^exportdata.isCsvHeader/ ) {
      $csv_header = 1;
      push( @data_new, "exportdata.isCsvHeader = true\n" );
      next;
    }

    #log.level = NORMAL
    #log.directory = ./log
    if ( $line =~ m/^log.level/ ) {
      $log_level = 1;
      push( @data_new, "log.level = NORMAL\n" );
      next;
    }
    if ( $line =~ m/^log.directory/ ) {
      $log_dir = 1;
      push( @data_new, "log.directory = $wrkdir/log\n" );
      next;
    }
    push( @data_new, "$line\n" );
  }
  if ( $csv_header == 0 ) {
    push( @data_new, "exportdata.isCsvHeader = true\n" );
  }
  if ( $log_level == 0 ) {
    push( @data_new, "log.level = NORMAL\n" );
  }
  if ( $log_dir == 0 ) {
    push( @data_new, "log.directory = $wrkdir/log\n" );
  }
  if ( $http_protocol == 0 ) {
    push( @data_new, "exportdata.protocol=$protocol\n" );
  }
  if ( $port_header == 0 ) {
    push( @data_new, "exportdata.portNumber =$port\n" );
  }

  #unlink("$file");
  write_to_file(\@data_new,$file_out);

}

sub bash_file_export_tool2 {
  my $dir = shift;

  my $copy = copy_file( "$dir/runUnix.sh", "$wrkdir/runUnix_test.sh" );
  if ($copy) {
    my $file     = "$wrkdir/runUnix.sh";
    my @data_out = ();
    my @data     = get_array_data("$wrkdir/runUnix_test.sh");
    foreach my $line (@data) {
      chomp $line;
      if ( $line =~ m/\.\// ) {
        $line =~ s/\.\//$dir\//g;
        push( @data_out, "$line\n" );
        next;
      }
      push( @data_out, "$line\n" );
    }
    write_to_file(\@data_out,$file);
  }
}

sub copy_file {
  my $file1 = shift;
  my $file2 = shift;

  use File::Copy;
  eval { copy( $file1, $file2 ); };
  if ($@) {
    $logger->error("Error copy file $file1 to copy file $file2");
    return 0;
  }
  if ( !-f $file2 ) { return 0; }
  return 1;
}

sub get_time {
  my $timestamp_end = shift;
  my @time          = localtime($timestamp_end);
  my $time_human    = localtime($timestamp_end);
  my $sec           = $time[0];
  my $minute        = $time[1];
  my $hour          = $time[2];
  my $day           = $time[3];
  my $month         = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;
  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }
  my $time_end = "$year:$month:$day" . "T" . "$hour:$minute:$sec";
  return $time_end;

}

sub check_device_id{
  if ($cfg->{'exportRadio'} eq "manager"){
    my $cmd          = "v1/objects/storages";
    my $data         = cmd_api($cmd, "GET", "storages", 1, "");
    if ( $data ne "" && defined $data->{data} && ref( $data->{data} ) eq "ARRAY" ) {
      foreach my $element ( @{ $data->{data} } ) {
        my $device_id = $element->{storageDeviceId};
        if (!defined $device_id || $device_id ne $cfg->{'hcmDeviceId'}){
          return 0;
        }
        else{
          return 1;
        }
      }
    }
  }
  else{
    return 1;
  }
  return 0;
}

sub get_global_timestamp{
  my $act_timestamp = time();
  (my $sec,my $min,my $hour,my $mday,my $mon, my $year,my $wday,my $yday,my $isdst) = localtime();
  return $act_timestamp - $sec;

}

sub delete_files{
  my $dir = shift;
  my @files = <$dir/*>;
  my $time_act = time();
  my $erase_time = 3600 * 72;
  foreach my $file (@files){
    my $mtime     = ( stat($file) )[9];
    if ($time_act - $mtime > $erase_time){
      unlink($file);
    }
  }
}

sub set_offset_time {
  if ( $offset_time eq "" ) {
    my $file_port_iops = "$performance_data_store/Port_dat/Port_IOPS.csv";
    if ( -f $file_port_iops ) {
      my $mtime       = ( stat("$file_port_iops") )[9];
      my $last_update = time() - $mtime;
      if ( $last_update > 18000 ) {
        return;
      }
      my @data = get_array_data($file_port_iops);
      foreach my $line (@data) {
        chomp $line;
        if ( $line =~ m/^To/ ) {
          $line =~ s/To.*: //g;
          $line =~ s/\//:/g;
          $line =~ s/ /T/g;
          my $time = str2time($line);
          if ( !Xormon::isdigit($time) ) { return; }
          my $secs = $time - time();
          my $sign = "+";
          if ( $secs < 0 ) {
            $sign = "-";
            $secs = abs($secs);
          }
          if ( $secs < 1800 ) { return; }
          my $days = int( $secs / ( 24 * 60 * 60 ) );
          $secs -= ( $days * 24 * 60 * 60 );
          my $hours = int( $secs / 3600 );
          $secs -= ( $hours * 3600 );

          my $mins = int( $secs / 60 );
          $secs -= ( $mins * 60 );
          if ( $mins < 10 ) {
            $mins = "0$mins";
          }
          if ( $hours < 10 ) {
            $hours = "0$hours";
          }
          $offset_time = "$sign$hours$mins";
        }
      }
    }
  }
}



1;
